@@ -30,7 +30,7 @@ struct _v810_state
 	UINT8 irq_line;
 	UINT8 irq_state;
 	UINT8 nmi_line;
-	device_irq_callback irq_callback;
+	device_irq_acknowledge_callback irq_callback;
 	legacy_cpu_device *device;
 	address_space *program;
 	direct_read_data *direct;
