@@ -30,7 +30,7 @@ struct _ui_gfx_state
 
 	/* intermediate bitmaps */
 	UINT8			bitmap_dirty;		/* is the bitmap dirty? */
-	bitmap_t *		bitmap;				/* bitmap for drawing gfx and tilemaps */
+	bitmap_rgb32 *	bitmap;				/* bitmap for drawing gfx and tilemaps */
 	render_texture *texture;			/* texture for rendering the above bitmap */
 
 	/* palette-specific data */
@@ -84,7 +84,7 @@ static void palette_handler(running_machine &machine, render_container *containe
 
 /* graphics set handling */
 static void gfxset_handle_keys(running_machine &machine, ui_gfx_state *state, int xcells, int ycells);
-static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, int index, bitmap_t *bitmap, int dstx, int dsty, int color, int rotate);
+static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, int index, bitmap_rgb32 &bitmap, int dstx, int dsty, int color, int rotate);
 static void gfxset_update_bitmap(running_machine &machine, ui_gfx_state *state, int xcells, int ycells, gfx_element *gfx);
 static void gfxset_handler(running_machine &machine, render_container *container, ui_gfx_state *state);
 
@@ -154,7 +154,7 @@ UINT32 ui_gfx_ui_handler(running_machine &machine, render_container *container,
 	ui_gfx_state *state = &ui_gfx;
 
 	/* if we have nothing, implicitly cancel */
-	if (machine.total_colors() == 0 && machine.colortable == NULL && machine.gfx[0] == NULL && tilemap_count(machine) == 0)
+	if (machine.total_colors() == 0 && machine.colortable == NULL && machine.gfx[0] == NULL && machine.tilemap().count() == 0)
 		goto cancel;
 
 	/* if we're not paused, mark the bitmap dirty */
@@ -189,7 +189,7 @@ again:
 
 		case 2:
 			/* if we have tilemaps, display them */
-			if (tilemap_count(machine) > 0)
+			if (machine.tilemap().count() > 0)
 			{
 				tilemap_handler(machine, container, state);
 				break;
@@ -693,16 +693,16 @@ static void gfxset_update_bitmap(running_machine &machine, ui_gfx_state *state,
 	cellypix = 1 + ((state->gfxset.rotate[set] & ORIENTATION_SWAP_XY) ? gfx->width : gfx->height);
 
 	/* realloc the bitmap if it is too small */
-	if (state->bitmap == NULL || state->texture == NULL || state->bitmap->bpp != 32 || state->bitmap->width != cellxpix * xcells || state->bitmap->height != cellypix * ycells)
+	if (state->bitmap == NULL || state->texture == NULL || state->bitmap->bpp() != 32 || state->bitmap->width() != cellxpix * xcells || state->bitmap->height() != cellypix * ycells)
 	{
 		/* free the old stuff */
 		machine.render().texture_free(state->texture);
 		global_free(state->bitmap);
 
 		/* allocate new stuff */
-		state->bitmap = global_alloc(bitmap_t(cellxpix * xcells, cellypix * ycells, BITMAP_FORMAT_ARGB32));
+		state->bitmap = global_alloc(bitmap_rgb32(cellxpix * xcells, cellypix * ycells));
 		state->texture = machine.render().texture_alloc();
-		state->texture->set_bitmap(state->bitmap, NULL, TEXFORMAT_ARGB32);
+		state->texture->set_bitmap(*state->bitmap, state->bitmap->cliprect(), TEXFORMAT_ARGB32);
 
 		/* force a redraw */
 		state->bitmap_dirty = TRUE;
@@ -717,10 +717,7 @@ static void gfxset_update_bitmap(running_machine &machine, ui_gfx_state *state,
 			rectangle cellbounds;
 
 			/* make a rect that covers this row */
-			cellbounds.min_x = 0;
-			cellbounds.max_x = state->bitmap->width - 1;
-			cellbounds.min_y = y * cellypix;
-			cellbounds.max_y = (y + 1) * cellypix - 1;
+			cellbounds.set(0, state->bitmap->width() - 1, y * cellypix, (y + 1) * cellypix - 1);
 
 			/* only display if there is data to show */
 			if (state->gfxset.offset[set] + y * xcells < gfx->total_elements)
@@ -736,21 +733,21 @@ static void gfxset_update_bitmap(running_machine &machine, ui_gfx_state *state,
 
 					/* only render if there is data */
 					if (index < gfx->total_elements)
-						gfxset_draw_item(machine, gfx, index, state->bitmap, cellbounds.min_x, cellbounds.min_y, state->gfxset.color[set], state->gfxset.rotate[set]);
+						gfxset_draw_item(machine, gfx, index, *state->bitmap, cellbounds.min_x, cellbounds.min_y, state->gfxset.color[set], state->gfxset.rotate[set]);
 
 					/* otherwise, fill with transparency */
 					else
-						bitmap_fill(state->bitmap, &cellbounds, 0);
+						state->bitmap->fill(0, cellbounds);
 				}
 			}
 
 			/* otherwise, fill with transparency */
 			else
-				bitmap_fill(state->bitmap, &cellbounds, 0);
+				state->bitmap->fill(0, cellbounds);
 		}
 
 		/* reset the texture to force an update */
-		state->texture->set_bitmap(state->bitmap, NULL, TEXFORMAT_ARGB32);
+		state->texture->set_bitmap(*state->bitmap, state->bitmap->cliprect(), TEXFORMAT_ARGB32);
 		state->bitmap_dirty = FALSE;
 	}
 }
@@ -761,7 +758,7 @@ static void gfxset_update_bitmap(running_machine &machine, ui_gfx_state *state,
     the view
 -------------------------------------------------*/
 
-static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, int index, bitmap_t *bitmap, int dstx, int dsty, int color, int rotate)
+static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, int index, bitmap_rgb32 &bitmap, int dstx, int dsty, int color, int rotate)
 {
 	static const pen_t default_palette[] =
 	{
@@ -771,7 +768,6 @@ static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, i
 	int width = (rotate & ORIENTATION_SWAP_XY) ? gfx->height : gfx->width;
 	int height = (rotate & ORIENTATION_SWAP_XY) ? gfx->width : gfx->height;
 	const rgb_t *palette = (machine.total_colors() != 0) ? palette_entry_list_raw(machine.palette) : NULL;
-	UINT32 rowpixels = bitmap->rowpixels;
 	UINT32 palette_mask = ~0;
 	int x, y;
 
@@ -786,14 +782,13 @@ static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, i
 	/* loop over rows in the cell */
 	for (y = 0; y < height; y++)
 	{
-		UINT32 *dest = (UINT32 *)bitmap->base + (dsty + y) * rowpixels + dstx;
+		UINT32 *dest = &bitmap.pix32(dsty + y, dstx);
 		const UINT8 *src = gfx_element_get_data(gfx, index);
 
 		/* loop over columns in the cell */
 		for (x = 0; x < width; x++)
 		{
 			int effx = x, effy = y;
-			rgb_t pixel;
 			const UINT8 *s;
 
 			/* compute effective x,y values after rotation */
@@ -818,11 +813,7 @@ static void gfxset_draw_item(running_machine &machine, const gfx_element *gfx, i
 			s = src + effy * gfx->line_modulo;
 
 			/* extract the pixel */
-			if (gfx->flags & GFX_ELEMENT_PACKED)
-				pixel = (s[effx/2] >> ((effx & 1) * 4)) & 0xf;
-			else
-				pixel = s[effx];
-			*dest++ = 0xff000000 | palette[pixel & palette_mask];
+			*dest++ = 0xff000000 | palette[s[effx] & palette_mask];
 		}
 	}
 }
@@ -855,7 +846,9 @@ static void tilemap_handler(running_machine &machine, render_container *containe
 	char title[100];
 
 	/* get the size of the tilemap itself */
-	tilemap_size_by_index(machine, state->tilemap.which, &mapwidth, &mapheight);
+	tilemap_t *tilemap = machine.tilemap().find(state->tilemap.which);
+	mapwidth = tilemap->width();
+	mapheight = tilemap->height();
 	if (state->tilemap.rotate & ORIENTATION_SWAP_XY)
 		{ UINT32 temp = mapwidth; mapwidth = mapheight; mapheight = temp; }
 
@@ -907,7 +900,7 @@ static void tilemap_handler(running_machine &machine, render_container *containe
 	boxbounds.y1 = mapboxbounds.y1 + 0.5f * chheight;
 
 	/* figure out the title and expand the outer box to fit */
-	sprintf(title, "TMAP %d/%d %dx%d OFFS %d,%d", state->tilemap.which, tilemap_count(machine) - 1, mapwidth, mapheight, state->tilemap.xoffs, state->tilemap.yoffs);
+	sprintf(title, "TMAP %d/%d %dx%d OFFS %d,%d", state->tilemap.which, machine.tilemap().count() - 1, mapwidth, mapheight, state->tilemap.xoffs, state->tilemap.yoffs);
 	titlewidth = ui_font->string_width(chheight, machine.render().ui_aspect(), title);
 	if (boxbounds.x1 - boxbounds.x0 < titlewidth + chwidth)
 	{
@@ -961,11 +954,13 @@ static void tilemap_handle_keys(running_machine &machine, ui_gfx_state *state, i
 	/* clamp within range */
 	if (state->tilemap.which < 0)
 		state->tilemap.which = 0;
-	if (state->tilemap.which >= tilemap_count(machine))
-		state->tilemap.which = tilemap_count(machine) - 1;
+	if (state->tilemap.which >= machine.tilemap().count())
+		state->tilemap.which = machine.tilemap().count() - 1;
 
 	/* cache some info in locals */
-	tilemap_size_by_index(machine, state->tilemap.which, &mapwidth, &mapheight);
+	tilemap_t *tilemap = machine.tilemap().find(state->tilemap.which);
+	mapwidth = tilemap->width();
+	mapheight = tilemap->height();
 
 	/* handle zoom (minus,plus) */
 	if (ui_input_pressed(machine, IPT_UI_ZOOM_OUT))
@@ -1031,34 +1026,22 @@ static void tilemap_handle_keys(running_machine &machine, ui_gfx_state *state, i
 
 static void tilemap_update_bitmap(running_machine &machine, ui_gfx_state *state, int width, int height)
 {
-	bitmap_format screen_format = machine.primary_screen->format();
-	palette_t *palette = NULL;
-	int screen_texformat;
-
-	/* convert the screen format to a texture format */
-	switch (screen_format)
-	{
-		case BITMAP_FORMAT_INDEXED16:	screen_texformat = TEXFORMAT_PALETTE16;	palette = machine.palette;	break;
-		case BITMAP_FORMAT_RGB15:		screen_texformat = TEXFORMAT_RGB15;		palette = NULL;				break;
-		case BITMAP_FORMAT_RGB32:		screen_texformat = TEXFORMAT_RGB32;		palette = NULL;				break;
-		default:						fatalerror("Invalid bitmap format!");								break;
-	}
-
 	/* swap the coordinates back if they were talking about a rotated surface */
 	if (state->tilemap.rotate & ORIENTATION_SWAP_XY)
 		{ UINT32 temp = width; width = height; height = temp; }
 
 	/* realloc the bitmap if it is too small */
-	if (state->bitmap == NULL || state->texture == NULL || state->bitmap->format != screen_format || state->bitmap->width != width || state->bitmap->height != height)
+	if (state->bitmap == NULL || state->texture == NULL || state->bitmap->width() != width || state->bitmap->height() != height)
 	{
 		/* free the old stuff */
 		machine.render().texture_free(state->texture);
 		global_free(state->bitmap);
 
 		/* allocate new stuff */
-		state->bitmap = global_alloc(bitmap_t(width, height, screen_format));
+		state->bitmap = global_alloc(bitmap_rgb32(width, height));
+		state->bitmap->set_palette(machine.palette);
 		state->texture = machine.render().texture_alloc();
-		state->texture->set_bitmap(state->bitmap, NULL, screen_texformat, palette);
+		state->texture->set_bitmap(*state->bitmap, state->bitmap->cliprect(), TEXFORMAT_RGB32);
 
 		/* force a redraw */
 		state->bitmap_dirty = TRUE;
@@ -1067,10 +1050,11 @@ static void tilemap_update_bitmap(running_machine &machine, ui_gfx_state *state,
 	/* handle the redraw */
 	if (state->bitmap_dirty)
 	{
-		tilemap_draw_by_index(machine, state->bitmap, state->tilemap.which, state->tilemap.xoffs, state->tilemap.yoffs);
+		tilemap_t *tilemap = machine.tilemap().find(state->tilemap.which);
+		tilemap->draw_debug(*state->bitmap, state->tilemap.xoffs, state->tilemap.yoffs);
 
 		/* reset the texture to force an update */
-		state->texture->set_bitmap(state->bitmap, NULL, screen_texformat, palette);
+		state->texture->set_bitmap(*state->bitmap, state->bitmap->cliprect(), TEXFORMAT_RGB32);
 		state->bitmap_dirty = FALSE;
 	}
 }
