@@ -13,6 +13,8 @@
 ** Improved over the years by MESS and MAME teams.
 **
 ** Todo:
+** - External VDP input and sync (pin 34/35 on 9918A)
+** - Updates during mid-scanline, probably only used in some MSX1 demos
 ** - Colours are incorrect. [fixed by R Nabet ?]
 ** - Sprites 8-31 are ghosted/cloned in mode 3 when using less than
 **   three pattern tables. Exact behaviour is not known.
@@ -133,18 +135,37 @@ READ8_MEMBER( tms9928a_device::register_read )
 	UINT8 data = m_StatusReg;
 
 	m_StatusReg = m_FifthSprite;
-	if (m_INT) {
-		m_INT = 0;
-		if ( !m_irq_changed.isnull() )
-			m_irq_changed( m_INT );
-	}
+	check_interrupt();
 	m_latch = 0;
 
 	return data;
 }
 
 
-void tms9928a_device::change_register(UINT8 reg, UINT8 val) {
+void tms9928a_device::check_interrupt()
+{
+	// trigger if vblank and interrupt-enable bits are set
+	UINT8 b = (m_StatusReg & 0x80 && m_Regs[1] & 0x20) ? 1 : 0;
+
+	if (b != m_INT)
+	{
+		m_INT = b;
+		if ( !m_irq_changed.isnull() )
+			m_irq_changed( m_INT );
+	}
+}
+
+
+void tms9928a_device::update_backdrop()
+{
+	// update backdrop colour to transparent if EXTVID bit is set
+	if ((m_Regs[7] & 15) == 0)
+		palette_set_color(machine(), 0, MAKE_ARGB(m_Regs[0] & 1 ? 0 : 255,0,0,0));
+}
+
+
+void tms9928a_device::change_register(UINT8 reg, UINT8 val)
+{
 	static const UINT8 Mask[8] =
 		{ 0x03, 0xfb, 0x0f, 0xff, 0x07, 0x7f, 0x07, 0xff };
 	static const char *const modes[] =
@@ -154,8 +175,8 @@ void tms9928a_device::change_register(UINT8 reg, UINT8 val) {
 		"Mode 1+3 (BOGUS)", "Mode 2+3 (MULTICOLOR variation)",
 		"Mode 1+2+3 (BOGUS)"
 	};
-	UINT8 b;
 
+	UINT8 prev = m_Regs[reg];
 	val &= Mask[reg];
 	m_Regs[reg] = val;
 
@@ -178,17 +199,12 @@ void tms9928a_device::change_register(UINT8 reg, UINT8 val) {
 			m_pattern = (m_Regs[4] * 2048) & (m_vram_size - 1);
 		}
 		m_mode = ( (m_reva ? (m_Regs[0] & 2) : 0) | ((m_Regs[1] & 0x10)>>4) | ((m_Regs[1] & 8)>>1));
+		if ((val ^ prev) & 1)
+			update_backdrop();
 		logerror("TMS9928A('%s'): %s\n", tag(), modes[m_mode]);
 		break;
 	case 1:
-		/* check for changes in the INT line */
-		b = (val & 0x20) && (m_StatusReg & 0x80) ;
-		if (b != m_INT)
-		{
-			m_INT = b;
-			if ( !m_irq_changed.isnull() )
-				m_irq_changed( m_INT );
-		}
+		check_interrupt();
 		m_mode = ( (m_reva ? (m_Regs[0] & 2) : 0) | ((m_Regs[1] & 0x10)>>4) | ((m_Regs[1] & 8)>>1));
 		logerror("TMS9928A('%s'): %s\n", tag(), modes[m_mode]);
 		break;
@@ -225,7 +241,8 @@ void tms9928a_device::change_register(UINT8 reg, UINT8 val) {
 		m_spritepattern = (val * 2048) & (m_vram_size - 1);
 		break;
 	case 7:
-		/* The backdrop is updated at TMS9928A_refresh() */
+		if ((val ^ prev) & 15)
+			update_backdrop();
 		break;
 	}
 }
@@ -257,21 +274,10 @@ WRITE8_MEMBER( tms9928a_device::register_write )
 
 void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr)
 {
-	int vpos = m_screen->vpos();
+	int raw_vpos = m_screen->vpos();
+	int vpos = raw_vpos * m_vertical_size / m_screen->height();
 	UINT16 BackColour = m_Regs[7] & 15;
-	UINT16 *p = BITMAP_ADDR16( m_tmpbmp, vpos, 0 );
-
-	if (!BackColour)
-		BackColour=1;
-
-	if ( vpos == 0 )
-	{
-		rgb_t oldcolor = palette_get_color(machine(), 0);
-
-		/* note we preserve the alpha here; this is so that it can be controlled independently */
-		/* see cliffhgr.c for an example */
-		palette_set_color(machine(), 0, (tms9928a_palette[BackColour] & MAKE_ARGB(0,255,255,255)) | (oldcolor & MAKE_ARGB(255,0,0,0)));
-	}
+	UINT16 *p = &m_tmpbmp.pix16(vpos);
 
 	int y = vpos - m_top_border;
 
@@ -281,18 +287,11 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 		for ( int i = 0; i < TMS9928A_TOTAL_HORZ; i++ )
 			p[i] = BackColour;
 
-		/* Check for end of active display */
-		if ( y == 192 )
+		/* vblank is set 1 line after end of active display */
+		if ( y == 193 )
 		{
-			UINT8 b;
-
 			m_StatusReg |= 0x80;
-			b = (m_Regs[1] & 0x20) != 0;
-			if (b != m_INT) {
-				m_INT = b;
-				if ( !m_irq_changed.isnull() )
-					m_irq_changed( m_INT );
-			}
+			check_interrupt();
 		}
 	}
 	else
@@ -317,8 +316,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 					UINT8 charcode = m_vMem[ addr ];
 					UINT8 pattern = m_vMem[ m_pattern + ( charcode << 3 ) + ( y & 7 ) ];
 					UINT8 colour = m_vMem[ m_colour + ( charcode >> 3 ) ];
-					UINT16 fg = colour >> 4;
-					UINT16 bg = colour & 15;
+					UINT16 fg = (colour >> 4) ? (colour >> 4) : BackColour;
+					UINT16 bg = (colour & 15) ? (colour & 15) : BackColour;
 
 					for ( int i = 0; i < 8; pattern <<= 1, i++ )
 						p[x+i] = ( pattern & 0x80 ) ? fg : bg;
@@ -330,14 +329,14 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 			//if (vpos==100 ) popmessage("TMS9928A MODE 1");
 			{
 				UINT16 addr = m_nametbl + ( ( y >> 3 ) * 40 );
-				UINT16 fg = m_Regs[7] >> 4;
-				UINT16 bg = m_Regs[7] & 15;
+				UINT16 fg = (m_Regs[7] >> 4) ? (m_Regs[7] >> 4) : BackColour;
+				UINT16 bg = BackColour;
 
-				/* Extra 8 pixels left border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 8; x++ )
+				/* Extra 6 pixels left border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 6; x++ )
 					p[x] = bg;
 
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 8; x < TMS9928A_HORZ_DISPLAY_START + 248; x+= 6, addr++ )
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 6; x < TMS9928A_HORZ_DISPLAY_START + 246; x+= 6, addr++ )
 				{
 					UINT16 charcode = m_vMem[ addr ];
 					UINT8 pattern = m_vMem[ m_pattern + ( charcode << 3 ) + ( y & 7 ) ];
@@ -346,8 +345,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 						p[x+i] = ( pattern & 0x80 ) ? fg : bg;
 				}
 
-				/* Extra 8 pixels right border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 248; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
+				/* Extra 10 pixels right border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 246; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
 					p[x] = bg;
 			}
 			break;
@@ -362,8 +361,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 					UINT16 charcode = m_vMem[ addr ] + ( ( y >> 6 ) << 8 );
 					UINT8 pattern = m_vMem[ m_pattern + ( ( charcode & m_patternmask ) << 3 ) + ( y & 7 ) ];
 					UINT8 colour = m_vMem[ m_colour + ( ( charcode & m_colourmask ) << 3 ) + ( y & 7 ) ];
-					UINT16 fg = colour >> 4;
-					UINT16 bg = colour & 15;
+					UINT16 fg = (colour >> 4) ? (colour >> 4) : BackColour;
+					UINT16 bg = (colour & 15) ? (colour & 15) : BackColour;
 
 					for ( int i = 0; i < 8; pattern <<= 1, i++ )
 						p[x+i] = ( pattern & 0x80 ) ? fg : bg;
@@ -375,14 +374,14 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 			//if (vpos==100) popmessage("TMS9928A MODE1+2");
 			{
 				UINT16 addr = m_nametbl + ( ( y >> 3 ) * 40 );
-				UINT16 fg = m_Regs[7] >> 4;
-				UINT16 bg = m_Regs[7] & 15;
+				UINT16 fg = (m_Regs[7] >> 4) ? (m_Regs[7] >> 4) : BackColour;
+				UINT16 bg = BackColour;
 
-				/* Extra 8 pixels left border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 8; x++ )
+				/* Extra 6 pixels left border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 6; x++ )
 					p[x] = bg;
 
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 8; x < TMS9928A_HORZ_DISPLAY_START + 248; x+= 6, addr++ )
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 6; x < TMS9928A_HORZ_DISPLAY_START + 246; x+= 6, addr++ )
 				{
 					UINT16 charcode = ( m_vMem[ addr ] + ( ( y >> 6 ) << 8 ) ) & m_patternmask;
 					UINT8 pattern = m_vMem[ m_pattern + ( charcode << 3 ) + ( y & 7 ) ];
@@ -391,8 +390,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 						p[x+i] = ( pattern & 0x80 ) ? fg : bg;
 				}
 
-				/* Extra 8 pixels right border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 248; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
+				/* Extra 10 pixels right border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 246; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
 					p[x] = bg;
 			}
 			break;
@@ -406,8 +405,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 				{
 					UINT8 charcode = m_vMem[ addr ];
 					UINT8 colour = m_vMem[ m_pattern + ( charcode << 3 ) + ( ( y >> 2 ) & 7 ) ];
-					UINT16 fg = colour >> 4;
-					UINT16 bg = colour & 15;
+					UINT16 fg = (colour >> 4) ? (colour >> 4) : BackColour;
+					UINT16 bg = (colour & 15) ? (colour & 15) : BackColour;
 
 					p[x+0] = p[x+1] = p[x+2] = p[x+3] = fg;
 					p[x+4] = p[x+5] = p[x+6] = p[x+7] = bg;
@@ -418,21 +417,21 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 		case 5:	case 7:		/* MODE bogus */
 			//if (vpos==100 ) popmessage("TMS9928A MODE bogus");
 			{
-				UINT16 fg = m_Regs[7] >> 4;
-				UINT16 bg = m_Regs[7] & 15;
+				UINT16 fg = (m_Regs[7] >> 4) ? (m_Regs[7] >> 4) : BackColour;
+				UINT16 bg = BackColour;
 
-				/* Extra 8 pixels left border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 8; x++ )
+				/* Extra 6 pixels left border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START; x < TMS9928A_HORZ_DISPLAY_START + 6; x++ )
 					p[x] = bg;
 
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 8; x < TMS9928A_HORZ_DISPLAY_START + 248; x+= 6 )
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 6; x < TMS9928A_HORZ_DISPLAY_START + 246; x+= 6 )
 				{
 					p[x+0] = p[x+1] = p[x+2] = p[x+3] = fg;
 					p[x+4] = p[x+5] = bg;
 				}
 
-				/* Extra 8 pixels right border */
-				for ( int x = TMS9928A_HORZ_DISPLAY_START + 248; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
+				/* Extra 10 pixels right border */
+				for ( int x = TMS9928A_HORZ_DISPLAY_START + 246; x < TMS9928A_HORZ_DISPLAY_START + 256; x++ )
 					p[x] = bg;
 			}
 			break;
@@ -446,8 +445,8 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 				{
 					UINT8 charcode = m_vMem[ addr ];
 					UINT8 colour = m_vMem[ m_pattern + ( ( ( charcode + ( ( y >> 2 ) & 7 ) + ( ( y >> 6 ) << 8 ) ) & m_patternmask ) << 3 ) ];
-					UINT16 fg = colour >> 4;
-					UINT16 bg = colour & 15;
+					UINT16 fg = (colour >> 4) ? (colour >> 4) : BackColour;
+					UINT16 bg = (colour & 15) ? (colour & 15) : BackColour;
 
 					p[x+0] = p[x+1] = p[x+2] = p[x+3] = fg;
 					p[x+4] = p[x+5] = p[x+6] = p[x+7] = bg;
@@ -475,8 +474,7 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 			{
 				int spr_y = m_vMem[ m_spriteattribute + sprattr + 0 ];
 
-				if (!fifth_encountered)
-					m_FifthSprite = sprattr / 4;
+				m_FifthSprite = sprattr / 4;
 
 				/* Stop processing sprites */
 				if ( spr_y == 208 )
@@ -499,8 +497,11 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 					num_sprites++;
 
 					/* Fifth sprite encountered? */
-					if( num_sprites == 5 )
+					if ( num_sprites == 5 )
+					{
 						fifth_encountered = true;
+						break;
+					}
 
 					if ( sprite_mag )
 						pataddr += ( ( ( y - spr_y ) & 0x1F ) >> 1 );
@@ -555,7 +556,7 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 			/* Update sprite overflow bits */
 			if (~m_StatusReg & 0x40)
 			{
-				m_StatusReg |= m_FifthSprite;
+				m_StatusReg = (m_StatusReg & 0xe0) | m_FifthSprite;
 				if (fifth_encountered && ~m_StatusReg & 0x80)
 					m_StatusReg |= 0x40;
 			}
@@ -567,13 +568,14 @@ void tms9928a_device::device_timer(emu_timer &timer, device_timer_id id, int par
 	}
 
 	/* Schedule next callback */
-	m_line_timer->adjust( m_screen->time_until_pos( ( vpos + 1 ) % m_screen->height() , 0 ) );
+	m_line_timer->adjust( m_screen->time_until_pos( ( raw_vpos + 1 ) % m_screen->height() , 0 ) );
 }
 
 
-void tms9928a_device::update( bitmap_t *bitmap, const rectangle *cliprect )
+UINT32 tms9928a_device::screen_update( screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect )
 {
 	copybitmap( bitmap, m_tmpbmp, 0, 0, 0, 0, cliprect );
+	return 0;
 }
 
 
@@ -601,6 +603,7 @@ void tms9928a_device::device_start()
 	assert( m_screen != NULL );
 
 	m_top_border = m_50hz ? TMS9928A_VERT_DISPLAY_START_PAL : TMS9928A_VERT_DISPLAY_START_NTSC;
+	m_vertical_size = m_50hz ? TMS9928A_TOTAL_VERT_PAL : TMS9928A_TOTAL_VERT_NTSC;
 
 	m_irq_changed.resolve( m_out_int_line, *this );
 
@@ -608,12 +611,10 @@ void tms9928a_device::device_start()
 	m_vMem = auto_alloc_array_clear(machine(), UINT8, m_vram_size);
 
 	/* back bitmap */
-	m_tmpbmp = auto_bitmap_alloc(machine(), TMS9928A_TOTAL_HORZ, TMS9928A_TOTAL_VERT_PAL, m_screen->format());
+	m_tmpbmp.allocate(TMS9928A_TOTAL_HORZ, TMS9928A_TOTAL_VERT_PAL);
 
 	m_line_timer = timer_alloc(TIMER_LINE);
 
-	m_LimitSprites = 1;
-
 	save_item(NAME(m_Regs[0]));
 	save_item(NAME(m_Regs[1]));
 	save_item(NAME(m_Regs[2]));
