@@ -8,8 +8,8 @@ class mfi_format : public floppy_image_format_t
 public:
 	mfi_format();
 
-	virtual int identify(io_generic *io);
-	virtual bool load(io_generic *io, floppy_image *image);
+	virtual int identify(io_generic *io, UINT32 form_factor);
+	virtual bool load(io_generic *io, UINT32 form_factor, floppy_image *image);
 	virtual bool save(io_generic *io, floppy_image *image);
 
 	virtual const char *name() const;
@@ -34,10 +34,11 @@ private:
 	struct header {
 		char sign[16];
 		unsigned int cyl_count, head_count;
+		unsigned int form_factor, variant;
 	};
 
 	struct entry {
-		unsigned int offset, compressed_size, uncompressed_size;
+		unsigned int offset, compressed_size, uncompressed_size, write_splice;
 	};
 };
 
