@@ -757,6 +757,41 @@ file_error core_fload(const char *filename, void **data, UINT32 *length)
 	return FILERR_NONE;
 }
 
+file_error core_fload(const char *filename, dynamic_buffer &data)
+{
+	core_file *file = NULL;
+	file_error err;
+	UINT64 size;
+
+	/* attempt to open the file */
+	err = core_fopen(filename, OPEN_FLAG_READ, &file);
+	if (err != FILERR_NONE)
+		return err;
+
+	/* get the size */
+	size = core_fsize(file);
+	if ((UINT32)size != size)
+	{
+		core_fclose(file);
+		return FILERR_OUT_OF_MEMORY;
+	}
+
+	/* allocate memory */
+	data.resize(size);
+
+	/* read the data */
+	if (core_fread(file, data, size) != size)
+	{
+		core_fclose(file);
+		data.reset();
+		return FILERR_FAILURE;
+	}
+
+	/* close the file and return data */
+	core_fclose(file);
+	return FILERR_NONE;
+}
+
 
 
 /***************************************************************************
@@ -883,7 +918,7 @@ int CLIB_DECL core_fprintf(core_file *f, const char *fmt, ...)
     assumptions about path separators
 -------------------------------------------------*/
 
-astring *core_filename_extract_base(astring *result, const char *name, int strip_extension)
+astring &core_filename_extract_base(astring &result, const char *name, bool strip_extension)
 {
 	/* find the start of the name */
 	const char *start = name + strlen(name);
@@ -891,11 +926,11 @@ astring *core_filename_extract_base(astring *result, const char *name, int strip
 		start--;
 
 	/* copy the rest into an astring */
-	astring_cpyc(result, start);
+	result.cpy(start);
 
 	/* chop the extension if present */
 	if (strip_extension)
-		astring_substr(result, 0, astring_rchr(result, 0, '.'));
+		result.substr(0, result.rchr(0, '.'));
 	return result;
 }
 
