@@ -21,11 +21,12 @@ class berzerk_state : public driver_device
 {
 public:
 	berzerk_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_videoram(*this, "videoram"),
+		m_colorram(*this, "colorram"){ }
 
-	UINT8 *m_videoram;
-	size_t m_videoram_size;
-	UINT8 *m_colorram;
+	required_shared_ptr<UINT8> m_videoram;
+	required_shared_ptr<UINT8> m_colorram;
 	UINT8 m_magicram_control;
 	UINT8 m_last_shift_data;
 	UINT8 m_intercept;
@@ -37,6 +38,22 @@ public:
 	int m_p1_direction;
 	int m_p2_counter_74ls161;
 	int m_p2_direction;
+	DECLARE_READ8_MEMBER(led_on_r);
+	DECLARE_WRITE8_MEMBER(led_on_w);
+	DECLARE_READ8_MEMBER(led_off_r);
+	DECLARE_WRITE8_MEMBER(led_off_w);
+	DECLARE_WRITE8_MEMBER(irq_enable_w);
+	DECLARE_WRITE8_MEMBER(nmi_enable_w);
+	DECLARE_WRITE8_MEMBER(nmi_disable_w);
+	DECLARE_READ8_MEMBER(nmi_enable_r);
+	DECLARE_READ8_MEMBER(nmi_disable_r);
+	DECLARE_WRITE8_MEMBER(magicram_w);
+	DECLARE_WRITE8_MEMBER(magicram_control_w);
+	DECLARE_READ8_MEMBER(intercept_v256_r);
+	DECLARE_WRITE8_MEMBER(berzerk_audio_w);
+	DECLARE_READ8_MEMBER(berzerk_audio_r);
+	DECLARE_READ8_MEMBER(moonwarp_p1_r);
+	DECLARE_READ8_MEMBER(moonwarp_p2_r);
 };
 
 
@@ -70,31 +87,31 @@ static const UINT8 nmi_trigger_v256s [NMIS_PER_FRAME] = { 0x00, 0x00, 0x00, 0x00
  *
  *************************************/
 
-static READ8_HANDLER( led_on_r )
+READ8_MEMBER(berzerk_state::led_on_r)
 {
-	set_led_status(space->machine(), 0, 1);
+	set_led_status(machine(), 0, 1);
 
 	return 0;
 }
 
 
-static WRITE8_HANDLER( led_on_w )
+WRITE8_MEMBER(berzerk_state::led_on_w)
 {
-	set_led_status(space->machine(), 0, 1);
+	set_led_status(machine(), 0, 1);
 }
 
 
-static READ8_HANDLER( led_off_r )
+READ8_MEMBER(berzerk_state::led_off_r)
 {
-	set_led_status(space->machine(), 0, 0);
+	set_led_status(machine(), 0, 0);
 
 	return 0;
 }
 
 
-static WRITE8_HANDLER( led_off_w )
+WRITE8_MEMBER(berzerk_state::led_off_w)
 {
-	set_led_status(space->machine(), 0, 0);
+	set_led_status(machine(), 0, 0);
 }
 
 
@@ -153,10 +170,9 @@ static int vsync_chain_counter_to_vpos(UINT8 counter, UINT8 v256)
  *
  *************************************/
 
-static WRITE8_HANDLER( irq_enable_w )
+WRITE8_MEMBER(berzerk_state::irq_enable_w)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
-	state->m_irq_enabled = data & 0x01;
+	m_irq_enabled = data & 0x01;
 }
 
 
@@ -212,33 +228,29 @@ static void start_irq_timer(running_machine &machine)
  *
  *************************************/
 
-static WRITE8_HANDLER( nmi_enable_w )
+WRITE8_MEMBER(berzerk_state::nmi_enable_w)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
-	state->m_nmi_enabled = 1;
+	m_nmi_enabled = 1;
 }
 
 
-static WRITE8_HANDLER( nmi_disable_w )
+WRITE8_MEMBER(berzerk_state::nmi_disable_w)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
-	state->m_nmi_enabled = 0;
+	m_nmi_enabled = 0;
 }
 
 
-static READ8_HANDLER( nmi_enable_r )
+READ8_MEMBER(berzerk_state::nmi_enable_r)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
-	state->m_nmi_enabled = 1;
+	m_nmi_enabled = 1;
 
 	return 0;
 }
 
 
-static READ8_HANDLER( nmi_disable_r )
+READ8_MEMBER(berzerk_state::nmi_disable_r)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
-	state->m_nmi_enabled = 0;
+	m_nmi_enabled = 0;
 
 	return 0;
 }
@@ -347,65 +359,62 @@ static VIDEO_START( berzerk )
 }
 
 
-static WRITE8_HANDLER( magicram_w )
+WRITE8_MEMBER(berzerk_state::magicram_w)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
 	UINT8 alu_output;
 
-	UINT8 current_video_data = state->m_videoram[offset];
+	UINT8 current_video_data = m_videoram[offset];
 
 	/* shift data towards LSB.  MSB bits are filled by data from last_shift_data.
        The shifter consists of 5 74153 devices @ 7A, 8A, 9A, 10A and 11A,
        followed by 4 more 153's at 11B, 10B, 9B and 8B, which optionally
        reverse the order of the resulting bits */
-	UINT8 shift_flop_output = (((UINT16)state->m_last_shift_data << 8) | data) >> (state->m_magicram_control & 0x07);
+	UINT8 shift_flop_output = (((UINT16)m_last_shift_data << 8) | data) >> (m_magicram_control & 0x07);
 
-	if (state->m_magicram_control & 0x08)
+	if (m_magicram_control & 0x08)
 		shift_flop_output = BITSWAP8(shift_flop_output, 0, 1, 2, 3, 4, 5, 6, 7);
 
 	/* collision detection - AND gate output goes to the K pin of the flip-flop,
        while J is LO, therefore, it only resets, never sets */
 	if (shift_flop_output & current_video_data)
-		state->m_intercept = 0;
+		m_intercept = 0;
 
 	/* perform ALU step */
 	TTL74181_write(LS181_12C, TTL74181_INPUT_A0, 4, shift_flop_output & 0x0f);
 	TTL74181_write(LS181_10C, TTL74181_INPUT_A0, 4, shift_flop_output >> 4);
 	TTL74181_write(LS181_12C, TTL74181_INPUT_B0, 4, current_video_data & 0x0f);
 	TTL74181_write(LS181_10C, TTL74181_INPUT_B0, 4, current_video_data >> 4);
-	TTL74181_write(LS181_12C, TTL74181_INPUT_S0, 4, state->m_magicram_control >> 4);
-	TTL74181_write(LS181_10C, TTL74181_INPUT_S0, 4, state->m_magicram_control >> 4);
+	TTL74181_write(LS181_12C, TTL74181_INPUT_S0, 4, m_magicram_control >> 4);
+	TTL74181_write(LS181_10C, TTL74181_INPUT_S0, 4, m_magicram_control >> 4);
 
 	alu_output = (TTL74181_read(LS181_10C, TTL74181_OUTPUT_F0, 4) << 4) |
 				 (TTL74181_read(LS181_12C, TTL74181_OUTPUT_F0, 4) << 0);
 
-	state->m_videoram[offset] = alu_output ^ 0xff;
+	m_videoram[offset] = alu_output ^ 0xff;
 
 	/* save data for next time */
-	state->m_last_shift_data = data & 0x7f;
+	m_last_shift_data = data & 0x7f;
 }
 
 
-static WRITE8_HANDLER( magicram_control_w )
+WRITE8_MEMBER(berzerk_state::magicram_control_w)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
 	/* save the control byte, clear the shift data latch,
        and set the intercept flip-flop */
-	state->m_magicram_control = data;
-	state->m_last_shift_data = 0;
-	state->m_intercept = 1;
+	m_magicram_control = data;
+	m_last_shift_data = 0;
+	m_intercept = 1;
 }
 
 
-static READ8_HANDLER( intercept_v256_r )
+READ8_MEMBER(berzerk_state::intercept_v256_r)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
 	UINT8 counter;
 	UINT8 v256;
 
-	vpos_to_vsync_chain_counter(space->machine().primary_screen->vpos(), &counter, &v256);
+	vpos_to_vsync_chain_counter(machine().primary_screen->vpos(), &counter, &v256);
 
-	return (!state->m_intercept << 7) | v256;
+	return (!m_intercept << 7) | v256;
 }
 
 
@@ -417,7 +426,7 @@ static void get_pens(running_machine &machine, pen_t *pens)
 	int color;
 	double color_weights[2];
 
-	if (input_port_read(machine, MONITOR_TYPE_PORT_TAG) == 0)
+	if (machine.root_device().ioport(MONITOR_TYPE_PORT_TAG)->read() == 0)
 		compute_resistor_weights(0, 0xff, -1.0,
 								 2, resistances_wg, color_weights, 0, 270,
 								 2, resistances_wg, color_weights, 0, 270,
@@ -444,15 +453,15 @@ static void get_pens(running_machine &machine, pen_t *pens)
 }
 
 
-static SCREEN_UPDATE( berzerk )
+static SCREEN_UPDATE_RGB32( berzerk )
 {
-	berzerk_state *state = screen->machine().driver_data<berzerk_state>();
+	berzerk_state *state = screen.machine().driver_data<berzerk_state>();
 	pen_t pens[NUM_PENS];
 	offs_t offs;
 
-	get_pens(screen->machine(), pens);
+	get_pens(screen.machine(), pens);
 
-	for (offs = 0; offs < state->m_videoram_size; offs++)
+	for (offs = 0; offs < state->m_videoram.bytes(); offs++)
 	{
 		int i;
 
@@ -465,7 +474,7 @@ static SCREEN_UPDATE( berzerk )
 		for (i = 0; i < 4; i++)
 		{
 			pen_t pen = (data & 0x80) ? pens[color >> 4] : RGB_BLACK;
-			*BITMAP_ADDR32(bitmap, y, x) = pen;
+			bitmap.pix32(y, x) = pen;
 
 			x = x + 1;
 			data = data << 1;
@@ -474,7 +483,7 @@ static SCREEN_UPDATE( berzerk )
 		for (; i < 8; i++)
 		{
 			pen_t pen = (data & 0x80) ? pens[color & 0x0f] : RGB_BLACK;
-			*BITMAP_ADDR32(bitmap, y, x) = pen;
+			bitmap.pix32(y, x) = pen;
 
 			x = x + 1;
 			data = data << 1;
@@ -492,7 +501,7 @@ static SCREEN_UPDATE( berzerk )
  *
  *************************************/
 
-static WRITE8_HANDLER( berzerk_audio_w )
+WRITE8_MEMBER(berzerk_state::berzerk_audio_w)
 {
 	device_t *device;
 	int clock_divisor;
@@ -501,7 +510,7 @@ static WRITE8_HANDLER( berzerk_audio_w )
 	{
 	/* offset 4 writes to the S14001A */
 	case 4:
-		device = space->machine().device("speech");
+		device = machine().device("speech");
 		switch (data >> 6)
 		{
 		/* write data to the S14001 */
@@ -519,7 +528,7 @@ static WRITE8_HANDLER( berzerk_audio_w )
 			break;
 
 		case 1:
-			device = space->machine().device("speech");
+			device = machine().device("speech");
 			/* volume */
 			s14001a_set_volume(device, ((data & 0x38) >> 3) + 1);
 
@@ -537,21 +546,21 @@ static WRITE8_HANDLER( berzerk_audio_w )
 
 	/* offset 6 writes to the sfxcontrol latch */
 	case 6:
-		exidy_sfxctrl_w(space->machine().device("exidy"), data >> 6, data);
+		exidy_sfxctrl_w(machine().device("exidy"), data >> 6, data);
 		break;
 
 	/* everything else writes to the 6840 */
 	default:
-		exidy_sh6840_w(space->machine().device("exidy"), offset, data);
+		exidy_sh6840_w(machine().device("exidy"), offset, data);
 		break;
 
 	}
 }
 
 
-static READ8_HANDLER( berzerk_audio_r )
+READ8_MEMBER(berzerk_state::berzerk_audio_r)
 {
-	device_t *device = space->machine().device("speech");
+	device_t *device = machine().device("speech");
 	switch (offset)
 	{
 	/* offset 4 reads from the S14001A */
@@ -563,7 +572,7 @@ static READ8_HANDLER( berzerk_audio_r )
 		return 0;
 	/* everything else reads from the 6840 */
 	default:
-		return exidy_sh6840_r(space->machine().device("exidy"), offset);
+		return exidy_sh6840_r(machine().device("exidy"), offset);
 	}
 }
 
@@ -572,8 +581,9 @@ static READ8_HANDLER( berzerk_audio_r )
 static SOUND_RESET(berzerk)
 {
 	address_space *space = machine.device("maincpu")->memory().space(AS_IO);
+	berzerk_state *state = machine.driver_data<berzerk_state>();
 	/* clears the flip-flop controlling the volume and freq on the speech chip */
-	berzerk_audio_w(space, 4, 0x40);
+	state->berzerk_audio_w(*space, 4, 0x40);
 }
 
 
@@ -584,22 +594,22 @@ static SOUND_RESET(berzerk)
  *
  *************************************/
 
-static ADDRESS_MAP_START( berzerk_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( berzerk_map, AS_PROGRAM, 8, berzerk_state )
 	AM_RANGE(0x0000, 0x07ff) AM_ROM
 	AM_RANGE(0x0800, 0x0bff) AM_MIRROR(0x0400) AM_RAM AM_SHARE("nvram")
 	AM_RANGE(0x1000, 0x3fff) AM_ROM
-	AM_RANGE(0x4000, 0x5fff) AM_RAM AM_BASE_MEMBER(berzerk_state, m_videoram) AM_SIZE_MEMBER(berzerk_state, m_videoram_size) AM_SHARE("share1")
-	AM_RANGE(0x6000, 0x7fff) AM_RAM_WRITE(magicram_w) AM_SHARE("share1")
-	AM_RANGE(0x8000, 0x87ff) AM_MIRROR(0x3800) AM_RAM AM_BASE_MEMBER(berzerk_state, m_colorram)
+	AM_RANGE(0x4000, 0x5fff) AM_RAM AM_SHARE("videoram")
+	AM_RANGE(0x6000, 0x7fff) AM_RAM_WRITE(magicram_w) AM_SHARE("videoram")
+	AM_RANGE(0x8000, 0x87ff) AM_MIRROR(0x3800) AM_RAM AM_SHARE("colorram")
 	AM_RANGE(0xc000, 0xffff) AM_NOP
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( frenzy_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( frenzy_map, AS_PROGRAM, 8, berzerk_state )
 	AM_RANGE(0x0000, 0x3fff) AM_ROM
-	AM_RANGE(0x4000, 0x5fff) AM_RAM AM_BASE_MEMBER(berzerk_state, m_videoram) AM_SIZE_MEMBER(berzerk_state, m_videoram_size) AM_SHARE("share1")
-	AM_RANGE(0x6000, 0x7fff) AM_RAM_WRITE(magicram_w) AM_SHARE("share1")
-	AM_RANGE(0x8000, 0x87ff) AM_MIRROR(0x3800) AM_RAM AM_BASE_MEMBER(berzerk_state, m_colorram)
+	AM_RANGE(0x4000, 0x5fff) AM_RAM AM_SHARE("videoram")
+	AM_RANGE(0x6000, 0x7fff) AM_RAM_WRITE(magicram_w) AM_SHARE("videoram")
+	AM_RANGE(0x8000, 0x87ff) AM_MIRROR(0x3800) AM_RAM AM_SHARE("colorram")
 	AM_RANGE(0xc000, 0xcfff) AM_ROM
 	AM_RANGE(0xf800, 0xfbff) AM_MIRROR(0x0400) AM_RAM AM_SHARE("nvram")
 ADDRESS_MAP_END
@@ -612,7 +622,7 @@ ADDRESS_MAP_END
  *
  *************************************/
 
-static ADDRESS_MAP_START( berzerk_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( berzerk_io_map, AS_IO, 8, berzerk_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	AM_RANGE(0x00, 0x3f) AM_NOP
 	AM_RANGE(0x40, 0x47) AM_READWRITE(berzerk_audio_r, berzerk_audio_w)
@@ -885,40 +895,38 @@ static INPUT_PORTS_START( frenzy )
 	PORT_DIPSETTING(    0xff, "255" )
 INPUT_PORTS_END
 
-static READ8_HANDLER( moonwarp_p1_r )
+READ8_MEMBER(berzerk_state::moonwarp_p1_r)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
 	// This seems to be the same type of dial as the later 'moon war 2' set uses
 	// see http://www.cityofberwyn.com/schematics/stern/MoonWar_opto.tiff for schematic
 	// I.e. a 74ls161 counts from 0 to 15 which is the absolute number of bars passed on the quadrature
 	// one difference is it lacks the strobe input (does it?), which if not active causes
 	// the dial input to go open bus. This is used in moon war 2 to switch between player 1
 	// and player 2 dials, which share a single port. moonwarp uses separate ports for the dials.
-	signed char dialread = input_port_read(space->machine(),"P1_DIAL");
+	signed char dialread = ioport("P1_DIAL")->read();
 	UINT8 ret;
-	UINT8 buttons = (input_port_read(space->machine(),"P1")&0xe0);
-	if (dialread < 0) state->m_p1_direction = 0;
-	else if (dialread > 0) state->m_p1_direction = 0x10;
-	state->m_p1_counter_74ls161 += abs(dialread);
-	state->m_p1_counter_74ls161 &= 0xf;
-	ret = state->m_p1_counter_74ls161 | state->m_p1_direction | buttons;
-	//fprintf(stderr, "dialread1: %02x, p1_counter_74ls161: %02x, spinner ret is %02x\n", dialread, state->m_p1_counter_74ls161, ret);
+	UINT8 buttons = (ioport("P1")->read()&0xe0);
+	if (dialread < 0) m_p1_direction = 0;
+	else if (dialread > 0) m_p1_direction = 0x10;
+	m_p1_counter_74ls161 += abs(dialread);
+	m_p1_counter_74ls161 &= 0xf;
+	ret = m_p1_counter_74ls161 | m_p1_direction | buttons;
+	//fprintf(stderr, "dialread1: %02x, p1_counter_74ls161: %02x, spinner ret is %02x\n", dialread, m_p1_counter_74ls161, ret);
 	return ret;
 }
 
-static READ8_HANDLER( moonwarp_p2_r )
+READ8_MEMBER(berzerk_state::moonwarp_p2_r)
 {
-	berzerk_state *state = space->machine().driver_data<berzerk_state>();
 	// same as above, but for player 2 in cocktail mode
-	signed char dialread = input_port_read(space->machine(),"P2_DIAL");
+	signed char dialread = ioport("P2_DIAL")->read();
 	UINT8 ret;
-	UINT8 buttons = (input_port_read(space->machine(),"P2")&0xe0);
-	if (dialread < 0) state->m_p2_direction = 0;
-	else if (dialread > 0) state->m_p2_direction = 0x10;
-	state->m_p2_counter_74ls161 += abs(dialread);
-	state->m_p2_counter_74ls161 &= 0xf;
-	ret = state->m_p2_counter_74ls161 | state->m_p2_direction | buttons;
-	//fprintf(stderr, "dialread2: %02x, p2_counter_74ls161: %02x, spinner ret is %02x\n", dialread, state->m_p2_counter_74ls161, ret);
+	UINT8 buttons = (ioport("P2")->read()&0xe0);
+	if (dialread < 0) m_p2_direction = 0;
+	else if (dialread > 0) m_p2_direction = 0x10;
+	m_p2_counter_74ls161 += abs(dialread);
+	m_p2_counter_74ls161 &= 0xf;
+	ret = m_p2_counter_74ls161 | m_p2_direction | buttons;
+	//fprintf(stderr, "dialread2: %02x, p2_counter_74ls161: %02x, spinner ret is %02x\n", dialread, m_p2_counter_74ls161, ret);
 	return ret;
 }
 
@@ -1084,9 +1092,8 @@ static MACHINE_CONFIG_START( berzerk, berzerk_state )
 	MCFG_VIDEO_START(berzerk)
 
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_RAW_PARAMS(PIXEL_CLOCK, HTOTAL, HBEND, HBSTART, VTOTAL, VBEND, VBSTART)
-	MCFG_SCREEN_UPDATE(berzerk)
+	MCFG_SCREEN_UPDATE_STATIC(berzerk)
 
 	/* audio hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -1229,8 +1236,9 @@ ROM_END
 static DRIVER_INIT( moonwarp )
 {
 	address_space *io = machine.device("maincpu")->memory().space(AS_IO);
-	io->install_legacy_read_handler (0x48, 0x48, FUNC(moonwarp_p1_r));
-	io->install_legacy_read_handler (0x4a, 0x4a, FUNC(moonwarp_p2_r));
+	berzerk_state *state = machine.driver_data<berzerk_state>();
+	io->install_read_handler (0x48, 0x48, read8_delegate(FUNC(berzerk_state::moonwarp_p1_r), state));
+	io->install_read_handler (0x4a, 0x4a, read8_delegate(FUNC(berzerk_state::moonwarp_p2_r), state));
 }
 
 /*************************************
