@@ -170,23 +170,33 @@ class cybertnk_state : public driver_device
 {
 public:
 	cybertnk_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_spr_ram(*this, "spr_ram"),
+		m_tx_vram(*this, "tx_vram"),
+		m_bg_vram(*this, "bg_vram"),
+		m_fg_vram(*this, "fg_vram"),
+		m_io_ram(*this, "io_ram"),
+		m_roadram(*this, "roadram"){ }
 
 	tilemap_t *m_tx_tilemap;
-	UINT16 *m_tx_vram;
-	UINT16 *m_bg_vram;
-	UINT16 *m_fg_vram;
-	UINT16 *m_spr_ram;
-	UINT16 *m_io_ram;
-	UINT16 *m_roadram;
+	required_shared_ptr<UINT16> m_spr_ram;
+	required_shared_ptr<UINT16> m_tx_vram;
+	required_shared_ptr<UINT16> m_bg_vram;
+	required_shared_ptr<UINT16> m_fg_vram;
+	required_shared_ptr<UINT16> m_io_ram;
+	required_shared_ptr<UINT16> m_roadram;
 	int m_test_x;
 	int m_test_y;
 	int m_start_offs;
 	int m_color_pen;
+	DECLARE_WRITE16_MEMBER(tx_vram_w);
+	DECLARE_READ16_MEMBER(io_r);
+	DECLARE_WRITE16_MEMBER(io_w);
+	DECLARE_READ8_MEMBER(soundport_r);
 };
 
 
-#define LOG_UNKNOWN_WRITE logerror("unknown io write CPU '%s':%08x  0x%08x 0x%04x & 0x%04x\n", space->device().tag(), cpu_get_pc(&space->device()), offset*2, data, mem_mask);
+#define LOG_UNKNOWN_WRITE logerror("unknown io write CPU '%s':%08x  0x%08x 0x%04x & 0x%04x\n", space.device().tag(), cpu_get_pc(&space.device()), offset*2, data, mem_mask);
 
 static TILE_GET_INFO( get_tx_tile_info )
 {
@@ -203,44 +213,27 @@ static VIDEO_START( cybertnk )
 {
 	cybertnk_state *state = machine.driver_data<cybertnk_state>();
 	state->m_tx_tilemap = tilemap_create(machine, get_tx_tile_info,tilemap_scan_rows,8,8,128,32);
-	tilemap_set_transparent_pen(state->m_tx_tilemap,0);
+	state->m_tx_tilemap->set_transparent_pen(0);
 }
 
-static void draw_pixel( bitmap_t* bitmap, const rectangle *cliprect, int y, int x, int pen)
+static void draw_pixel( bitmap_ind16 &bitmap, const rectangle &cliprect, int y, int x, int pen)
 {
-	if (x>cliprect->max_x) return;
-	if (x<cliprect->min_x) return;
-	if (y>cliprect->max_y) return;
-	if (y<cliprect->min_y) return;
-
-	*BITMAP_ADDR16(bitmap, y, x) = pen;
+	if (cliprect.contains(x, y))
+		bitmap.pix16(y, x) = pen;
 }
 
-static SCREEN_UPDATE( cybertnk )
+static UINT32 update_screen(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect, int screen_shift)
 {
-	cybertnk_state *state = screen->machine().driver_data<cybertnk_state>();
-	device_t *left_screen  = screen->machine().device("lscreen");
-	device_t *right_screen = screen->machine().device("rscreen");
-	int screen_shift = 0;
+	cybertnk_state *state = screen.machine().driver_data<cybertnk_state>();
 
-	if (screen==left_screen)
-	{
-		screen_shift = 0;
-
-	}
-	else if (screen==right_screen)
-	{
-		screen_shift = -256;
-	}
-
-	tilemap_set_scrolldx(state->m_tx_tilemap, screen_shift, screen_shift);
+	state->m_tx_tilemap->set_scrolldx(screen_shift, screen_shift);
 
 
-	bitmap_fill(bitmap, cliprect, get_black_pen(screen->machine()));
+	bitmap.fill(get_black_pen(screen.machine()), cliprect);
 
 	{
 		int i;
-		const gfx_element *gfx = screen->machine().gfx[3];
+		const gfx_element *gfx = screen.machine().gfx[3];
 
 
 		for (i=0;i<0x1000/4;i+=4)
@@ -258,7 +251,7 @@ static SCREEN_UPDATE( cybertnk )
 
 	{
 		int count,x,y;
-		const gfx_element *gfx = screen->machine().gfx[2];
+		const gfx_element *gfx = screen.machine().gfx[2];
 
 		count = 0;
 
@@ -279,7 +272,7 @@ static SCREEN_UPDATE( cybertnk )
 
 	{
 		int count,x,y;
-		const gfx_element *gfx = screen->machine().gfx[1];
+		const gfx_element *gfx = screen.machine().gfx[1];
 
 		count = 0;
 
@@ -301,7 +294,7 @@ static SCREEN_UPDATE( cybertnk )
 	/* non-tile based spriteram (BARE-BONES, looks pretty complex) */
 	if(1)
 	{
-		const UINT8 *blit_ram = screen->machine().region("spr_gfx")->base();
+		const UINT8 *blit_ram = screen.machine().root_device().memregion("spr_gfx")->base();
 		int offs,x,y,z,xsize,ysize,yi,xi,col_bank,fx,zoom;
 		UINT32 spr_offs,spr_offs_helper;
 		int xf,yf,xz,yz;
@@ -354,11 +347,11 @@ static SCREEN_UPDATE( cybertnk )
 							dot|= col_bank<<4;
 							if(fx)
 							{
-								draw_pixel(bitmap, cliprect, y+yz, x+xsize-(xz)+screen_shift, screen->machine().pens[dot]);
+								draw_pixel(bitmap, cliprect, y+yz, x+xsize-(xz)+screen_shift, screen.machine().pens[dot]);
 							}
 							else
 							{
-								draw_pixel(bitmap, cliprect, y+yz, x+xz+screen_shift, screen->machine().pens[dot]);
+								draw_pixel(bitmap, cliprect, y+yz, x+xz+screen_shift, screen.machine().pens[dot]);
 							}
 						}
 						xf+=zoom;
@@ -386,11 +379,11 @@ static SCREEN_UPDATE( cybertnk )
 							dot|= col_bank<<4;
 							if(fx)
 							{
-								draw_pixel(bitmap, cliprect, y+yz, x+xsize-(xz)+screen_shift, screen->machine().pens[dot]);
+								draw_pixel(bitmap, cliprect, y+yz, x+xsize-(xz)+screen_shift, screen.machine().pens[dot]);
 							}
 							else
 							{
-								draw_pixel(bitmap, cliprect, y+yz, x+xz+screen_shift, screen->machine().pens[dot]);
+								draw_pixel(bitmap, cliprect, y+yz, x+xz+screen_shift, screen.machine().pens[dot]);
 							}
 						}
 						xf+=zoom;
@@ -422,58 +415,58 @@ static SCREEN_UPDATE( cybertnk )
 		}
 	}
 
-	tilemap_draw(bitmap,cliprect,state->m_tx_tilemap,0,0);
+	state->m_tx_tilemap->draw(bitmap, cliprect, 0,0);
 
 
 //0x62 0x9a 1c2d0
 //0x62 0x9a 1e1e4
 //0x20 0x9c 2011c
-	if (screen==left_screen)
+	if (screen_shift == 0)
 	{
 		if(0) //sprite gfx debug viewer
 		{
 			int x,y,count;
-			const UINT8 *blit_ram = screen->machine().region("spr_gfx")->base();
+			const UINT8 *blit_ram = screen.machine().root_device().memregion("spr_gfx")->base();
 
-			if(screen->machine().input().code_pressed(KEYCODE_Z))
+			if(screen.machine().input().code_pressed(KEYCODE_Z))
 			state->m_test_x++;
 
-			if(screen->machine().input().code_pressed(KEYCODE_X))
+			if(screen.machine().input().code_pressed(KEYCODE_X))
 			state->m_test_x--;
 
-			if(screen->machine().input().code_pressed(KEYCODE_A))
+			if(screen.machine().input().code_pressed(KEYCODE_A))
 			state->m_test_y++;
 
-			if(screen->machine().input().code_pressed(KEYCODE_S))
+			if(screen.machine().input().code_pressed(KEYCODE_S))
 			state->m_test_y--;
 
-			if(screen->machine().input().code_pressed(KEYCODE_Q))
+			if(screen.machine().input().code_pressed(KEYCODE_Q))
 			state->m_start_offs+=0x200;
 
-			if(screen->machine().input().code_pressed(KEYCODE_W))
+			if(screen.machine().input().code_pressed(KEYCODE_W))
 			state->m_start_offs-=0x200;
 
-			if(screen->machine().input().code_pressed_once(KEYCODE_T))
+			if(screen.machine().input().code_pressed_once(KEYCODE_T))
 			state->m_start_offs+=0x20000;
 
-			if(screen->machine().input().code_pressed_once(KEYCODE_Y))
+			if(screen.machine().input().code_pressed_once(KEYCODE_Y))
 			state->m_start_offs-=0x20000;
 
-			if(screen->machine().input().code_pressed(KEYCODE_E))
+			if(screen.machine().input().code_pressed(KEYCODE_E))
 			state->m_start_offs+=4;
 
-			if(screen->machine().input().code_pressed(KEYCODE_R))
+			if(screen.machine().input().code_pressed(KEYCODE_R))
 			state->m_start_offs-=4;
 
-			if(screen->machine().input().code_pressed(KEYCODE_D))
+			if(screen.machine().input().code_pressed(KEYCODE_D))
 			state->m_color_pen++;
 
-			if(screen->machine().input().code_pressed(KEYCODE_F))
+			if(screen.machine().input().code_pressed(KEYCODE_F))
 			state->m_color_pen--;
 
 			popmessage("%02x %02x %04x %02x",state->m_test_x,state->m_test_y,state->m_start_offs,state->m_color_pen);
 
-			bitmap_fill(bitmap,cliprect,get_black_pen(screen->machine()));
+			bitmap.fill(get_black_pen(screen.machine()), cliprect);
 
 			count = (state->m_start_offs);
 
@@ -490,28 +483,28 @@ static SCREEN_UPDATE( cybertnk )
 					color|= ((blit_ram[count+3] & 0xff) << 0);
 
 					dot = (color & 0xf0000000) >> 28;
-					*BITMAP_ADDR16(bitmap, y, x+0) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+0) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x0f000000) >> 24;
-					*BITMAP_ADDR16(bitmap, y, x+4) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+4) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x00f00000) >> 20;
-					*BITMAP_ADDR16(bitmap, y, x+1) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+1) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x000f0000) >> 16;
-					*BITMAP_ADDR16(bitmap, y, x+5) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+5) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x0000f000) >> 12;
-					*BITMAP_ADDR16(bitmap, y, x+2) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+2) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x00000f00) >> 8;
-					*BITMAP_ADDR16(bitmap, y, x+6) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+6) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x000000f0) >> 4;
-					*BITMAP_ADDR16(bitmap, y, x+3) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+3) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					dot = (color & 0x0000000f) >> 0;
-					*BITMAP_ADDR16(bitmap, y, x+7) = screen->machine().pens[dot+(state->m_color_pen<<4)];
+					bitmap.pix16(y, x+7) = screen.machine().pens[dot+(state->m_color_pen<<4)];
 
 					count+=4;
 				}
@@ -523,38 +516,39 @@ static SCREEN_UPDATE( cybertnk )
 	return 0;
 }
 
+static SCREEN_UPDATE_IND16( cybertnk_left ) { return update_screen(screen, bitmap, cliprect, 0); }
+static SCREEN_UPDATE_IND16( cybertnk_right ) { return update_screen(screen, bitmap, cliprect, -256); }
 
-static WRITE16_HANDLER( tx_vram_w )
+
+WRITE16_MEMBER(cybertnk_state::tx_vram_w)
 {
-	cybertnk_state *state = space->machine().driver_data<cybertnk_state>();
-	COMBINE_DATA(&state->m_tx_vram[offset]);
-	tilemap_mark_tile_dirty(state->m_tx_tilemap,offset);
+	COMBINE_DATA(&m_tx_vram[offset]);
+	m_tx_tilemap->mark_tile_dirty(offset);
 }
 
-static READ16_HANDLER( io_r )
+READ16_MEMBER(cybertnk_state::io_r)
 {
-	cybertnk_state *state = space->machine().driver_data<cybertnk_state>();
 	switch( offset )
 	{
 		case 2/2:
-			return input_port_read(space->machine(), "DSW1");
+			return ioport("DSW1")->read();
 
 		// 0x00110007 is controller device select
 		// 0x001100D5 is controller data
 		// 0x00110004 low is controller data ready
 		case 4/2:
-			switch( (state->m_io_ram[6/2]) & 0xff )
+			switch( (m_io_ram[6/2]) & 0xff )
 			{
 				case 0:
-					state->m_io_ram[0xd4/2] = input_port_read(space->machine(), "TRAVERSE");
+					m_io_ram[0xd4/2] = ioport("TRAVERSE")->read();
 					break;
 
 				case 0x20:
-					state->m_io_ram[0xd4/2] = input_port_read(space->machine(), "ELEVATE");
+					m_io_ram[0xd4/2] = ioport("ELEVATE")->read();
 					break;
 
 				case 0x40:
-					state->m_io_ram[0xd4/2] = input_port_read(space->machine(), "ACCEL");
+					m_io_ram[0xd4/2] = ioport("ACCEL")->read();
 					break;
 
 				case 0x42:
@@ -562,11 +556,11 @@ static READ16_HANDLER( io_r )
 					// controller return value is stored in $42(a6)
 					// but I don't see it referenced again.
 					//popmessage("unknown controller device 0x42");
-					state->m_io_ram[0xd4/2] = 0;
+					m_io_ram[0xd4/2] = 0;
 					break;
 
 				case 0x60:
-					state->m_io_ram[0xd4/2] = input_port_read(space->machine(), "HANDLE");
+					m_io_ram[0xd4/2] = ioport("HANDLE")->read();
 					break;
 
 				//default:
@@ -575,36 +569,35 @@ static READ16_HANDLER( io_r )
 			return 0;
 
 		case 6/2:
-			return input_port_read(space->machine(), "IN0"); // high half
+			return ioport("IN0")->read(); // high half
 
 		case 8/2:
-			return input_port_read(space->machine(), "IN0"); // low half
+			return ioport("IN0")->read(); // low half
 
 		case 0xa/2:
-			return input_port_read(space->machine(), "DSW2");
+			return ioport("DSW2")->read();
 
 		case 0xd4/2:
-			return state->m_io_ram[offset]; // controller data
+			return m_io_ram[offset]; // controller data
 
 		default:
 		{
 			//popmessage("unknown io read 0x%08x", offset);
-			return state->m_io_ram[offset];
+			return m_io_ram[offset];
 		}
 	}
 }
 
-static WRITE16_HANDLER( io_w )
+WRITE16_MEMBER(cybertnk_state::io_w)
 {
-	cybertnk_state *state = space->machine().driver_data<cybertnk_state>();
-	COMBINE_DATA(&state->m_io_ram[offset]);
+	COMBINE_DATA(&m_io_ram[offset]);
 
 	switch( offset )
 	{
 		case 0/2:
 			// sound data
 			if (ACCESSING_BITS_0_7)
-				cputag_set_input_line(space->machine(), "audiocpu", 0, HOLD_LINE);
+				cputag_set_input_line(machine(), "audiocpu", 0, HOLD_LINE);
 			else
 				LOG_UNKNOWN_WRITE
 			break;
@@ -658,7 +651,7 @@ static WRITE16_HANDLER( io_w )
 		case 0x80/2:
 		case 0x82/2:
 		case 0x84/2:
-			popmessage("%02x %02x %02x %02x %02x %02x %02x",state->m_io_ram[0x40/2],state->m_io_ram[0x42/2],state->m_io_ram[0x44/2],state->m_io_ram[0x46/2],state->m_io_ram[0x48/2],state->m_io_ram[0x4a/2],state->m_io_ram[0x4c/2]);
+			popmessage("%02x %02x %02x %02x %02x %02x %02x",m_io_ram[0x40/2],m_io_ram[0x42/2],m_io_ram[0x44/2],m_io_ram[0x46/2],m_io_ram[0x48/2],m_io_ram[0x4a/2],m_io_ram[0x4c/2]);
 			break;
 
 		default:
@@ -667,39 +660,38 @@ static WRITE16_HANDLER( io_w )
 	}
 }
 
-static READ8_HANDLER( soundport_r )
+READ8_MEMBER(cybertnk_state::soundport_r)
 {
-	cybertnk_state *state = space->machine().driver_data<cybertnk_state>();
-	return state->m_io_ram[0] & 0xff;
+	return m_io_ram[0] & 0xff;
 }
 
-static ADDRESS_MAP_START( master_mem, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( master_mem, AS_PROGRAM, 16, cybertnk_state )
 	AM_RANGE(0x000000, 0x03ffff) AM_ROM
 	AM_RANGE(0x080000, 0x087fff) AM_RAM /*Work RAM*/
-	AM_RANGE(0x0a0000, 0x0a0fff) AM_RAM AM_BASE_MEMBER(cybertnk_state, m_spr_ram) // non-tile based sprite ram
-	AM_RANGE(0x0c0000, 0x0c1fff) AM_RAM_WRITE(tx_vram_w) AM_BASE_MEMBER(cybertnk_state, m_tx_vram)
-	AM_RANGE(0x0c4000, 0x0c5fff) AM_RAM AM_BASE_MEMBER(cybertnk_state, m_bg_vram)
-	AM_RANGE(0x0c8000, 0x0c9fff) AM_RAM AM_BASE_MEMBER(cybertnk_state, m_fg_vram)
+	AM_RANGE(0x0a0000, 0x0a0fff) AM_RAM AM_SHARE("spr_ram") // non-tile based sprite ram
+	AM_RANGE(0x0c0000, 0x0c1fff) AM_RAM_WRITE(tx_vram_w) AM_SHARE("tx_vram")
+	AM_RANGE(0x0c4000, 0x0c5fff) AM_RAM AM_SHARE("bg_vram")
+	AM_RANGE(0x0c8000, 0x0c9fff) AM_RAM AM_SHARE("fg_vram")
 	AM_RANGE(0x0e0000, 0x0e0fff) AM_RAM AM_SHARE("sharedram")
-	AM_RANGE(0x100000, 0x107fff) AM_RAM_WRITE(paletteram16_xBBBBBGGGGGRRRRR_word_w) AM_BASE_GENERIC(paletteram)
-	AM_RANGE(0x110000, 0x1101ff) AM_READWRITE(io_r,io_w) AM_BASE_MEMBER(cybertnk_state, m_io_ram)
+	AM_RANGE(0x100000, 0x107fff) AM_RAM_WRITE(paletteram_xBBBBBGGGGGRRRRR_word_w) AM_SHARE("paletteram")
+	AM_RANGE(0x110000, 0x1101ff) AM_READWRITE(io_r,io_w) AM_SHARE("io_ram")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( slave_mem, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( slave_mem, AS_PROGRAM, 16, cybertnk_state )
 	AM_RANGE(0x000000, 0x01ffff) AM_ROM
 	AM_RANGE(0x080000, 0x083fff) AM_RAM /*Work RAM*/
-	AM_RANGE(0x0c0000, 0x0c0fff) AM_RAM AM_BASE_MEMBER(cybertnk_state, m_roadram)
+	AM_RANGE(0x0c0000, 0x0c0fff) AM_RAM AM_SHARE("roadram")
 	AM_RANGE(0x100000, 0x100fff) AM_RAM AM_SHARE("sharedram")
 	AM_RANGE(0x140000, 0x140003) AM_NOP /*Watchdog? Written during loops and interrupts*/
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( sound_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( sound_mem, AS_PROGRAM, 8, cybertnk_state )
 	AM_RANGE(0x0000, 0x7fff ) AM_ROM
 	AM_RANGE(0x8000, 0x9fff ) AM_RAM
 	AM_RANGE(0xa001, 0xa001 ) AM_READ(soundport_r)
 	AM_RANGE(0xa005, 0xa006 ) AM_NOP
-	AM_RANGE(0xa000, 0xa001 ) AM_DEVREADWRITE("ym1", y8950_r, y8950_w)
-	AM_RANGE(0xc000, 0xc001 ) AM_DEVREADWRITE("ym2", y8950_r, y8950_w)
+	AM_RANGE(0xa000, 0xa001 ) AM_DEVREADWRITE_LEGACY("ym1", y8950_r, y8950_w)
+	AM_RANGE(0xc000, 0xc001 ) AM_DEVREADWRITE_LEGACY("ym2", y8950_r, y8950_w)
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( cybertnk )
@@ -853,7 +845,19 @@ static const gfx_layout tile_8x8x4 =
     8*8
 };
 
-static GFXLAYOUT_RAW( roadlayout, 4, 1024, 1, 1024*4, 1024*4 ) // could be wrong.. needs to be 512 wide, might not be 8bpp
+// could be wrong.. needs to be 512 wide, might not be 8bpp
+static const UINT32 xoffsets[] = { STEP1024(0,4) };
+static const gfx_layout roadlayout =
+{
+	1024,1,
+	RGN_FRAC(1,1),
+	4,
+	{ STEP4(0,1) },
+	EXTENDED_XOFFS,
+	{ 0 },
+	1024*4,
+	xoffsets
+};
 
 static GFXDECODE_START( cybertnk )
 	GFXDECODE_ENTRY( "gfx1", 0, tile_8x8x4,     0x1400, 16 ) /*Pal offset???*/
@@ -862,25 +866,6 @@ static GFXDECODE_START( cybertnk )
 	GFXDECODE_ENTRY( "road_data", 0, roadlayout,     0,      0x400 )
 GFXDECODE_END
 
-#if 0
-static INTERRUPT_GEN( master_irq )
-{
-	switch(cpu_getiloops(device))
-	{
-		case 0: device_set_input_line(device,1,HOLD_LINE); break;
-		case 1: device_set_input_line(device,3,HOLD_LINE); break;
-	}
-}
-
-static INTERRUPT_GEN( slave_irq )
-{
-	switch(cpu_getiloops(device))
-	{
-		case 0: device_set_input_line(device,3,HOLD_LINE); break;
-		case 1: device_set_input_line(device,1,HOLD_LINE); break;
-	}
-}
-#endif
 
 static const y8950_interface y8950_config = {
 	0 /* TODO */
@@ -904,20 +889,18 @@ static MACHINE_CONFIG_START( cybertnk, cybertnk_state )
 	MCFG_DEFAULT_LAYOUT(layout_dualhsxs)
 
 	MCFG_SCREEN_ADD("lscreen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 0*8, 28*8-1)
-	MCFG_SCREEN_UPDATE(cybertnk)
+	MCFG_SCREEN_UPDATE_STATIC(cybertnk_left)
 
 	MCFG_SCREEN_ADD("rscreen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 0*8, 28*8-1)
-	MCFG_SCREEN_UPDATE(cybertnk)
+	MCFG_SCREEN_UPDATE_STATIC(cybertnk_right)
 
 	MCFG_GFXDECODE(cybertnk)
 	MCFG_PALETTE_LENGTH(0x4000)
@@ -1031,7 +1014,7 @@ DRIVER_INIT( cybertnk )
     UINT8* road_data;
     int i;
 
-    road_data = machine.region("road_data")->base();
+    road_data = machine.root_device().memregion("road_data")->base();
     for (i=0;i < 0x40000;i++)
     {
         road_data[i] = BITSWAP8(road_data[i],3,2,1,0,7,6,5,4);
