@@ -8,7 +8,7 @@
 
  TODO:
 
- - finish 8275 CRT emulation, split off as device
+ - convert driver to use i8275 CRT device emulation
  - fix gfx glitches and banking
  - correct colors
  - DIPs
@@ -297,7 +297,6 @@ uPC1352C @ N3
 */
 
 #include "emu.h"
-#include "deprecat.h"
 #include "cpu/i8085/i8085.h"
 #include "sound/ay8910.h"
 
@@ -305,7 +304,9 @@ class dwarfd_state : public driver_device
 {
 public:
 	dwarfd_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+		m_maincpu(*this,"maincpu")
+		{ }
 
 	/* video-related */
 	int m_bank;
@@ -330,6 +331,17 @@ public:
 	/* memory */
 	UINT8    m_dw_ram[0x1000];
 	UINT8    m_videobuf[0x8000];
+
+	required_device<cpu_device> m_maincpu;
+	DECLARE_WRITE8_MEMBER(i8275_preg_w);
+	DECLARE_READ8_MEMBER(i8275_preg_r);
+	DECLARE_WRITE8_MEMBER(i8275_creg_w);
+	DECLARE_READ8_MEMBER(i8275_sreg_r);
+	DECLARE_READ8_MEMBER(dwarfd_ram_r);
+	DECLARE_WRITE8_MEMBER(dwarfd_ram_w);
+	DECLARE_WRITE8_MEMBER(output1_w);
+	DECLARE_WRITE8_MEMBER(output2_w);
+	DECLARE_READ8_MEMBER(qc_b8_r);
 };
 
 
@@ -370,76 +382,75 @@ enum
 };
 
 
-static WRITE8_HANDLER (i8275_preg_w) //param reg
+WRITE8_MEMBER(dwarfd_state::i8275_preg_w)//param reg
 {
-	dwarfd_state *state = space->machine().driver_data<dwarfd_state>();
 
-	switch (state->m_i8275Command)
+	switch (m_i8275Command)
 	{
 		case I8275_COMMAND_RESET:
 		{
-			switch (state->m_i8275CommandSeqCnt)
+			switch (m_i8275CommandSeqCnt)
 			{
 				case 4:
 				{
 					//screen byte comp byte 1
-					state->m_i8275SpacedRows = data >> 7;
-					state->m_i8275HorizontalCharactersRow = (data & 0x7f) + 1;
-					if (state->m_i8275HorizontalCharactersRow > 80)
+					m_i8275SpacedRows = data >> 7;
+					m_i8275HorizontalCharactersRow = (data & 0x7f) + 1;
+					if (m_i8275HorizontalCharactersRow > 80)
 					{
-						logerror("i8275 Undefined num of characters/Row! = %d\n", state->m_i8275HorizontalCharactersRow);
-						state->m_i8275HorizontalCharactersRow = CHARACTERS_UNDEFINED;
+						logerror("i8275 Undefined num of characters/Row! = %d\n", m_i8275HorizontalCharactersRow);
+						m_i8275HorizontalCharactersRow = CHARACTERS_UNDEFINED;
 					}
 					else
 					{
-						logerror("i8275 %d characters/row\n", state->m_i8275HorizontalCharactersRow);
+						logerror("i8275 %d characters/row\n", m_i8275HorizontalCharactersRow);
 					}
-					if (state->m_i8275SpacedRows & 1)
+					if (m_i8275SpacedRows & 1)
 					{
-						logerror("i8275 spaced rows\n");
+						logerror("i8275 &spaced rows\n");
 					}
 					else
 					{
 						logerror("i8275 normal rows\n");
 					}
-					state->m_i8275CommandSeqCnt--;
+					m_i8275CommandSeqCnt--;
 				}
 				break;
 
 				case 3:
 				{
 					//screen byte comp byte 2
-					state->m_i8275VerticalRows = (data & 0x3f) + 1;
-					state->m_i8275VerticalRetraceRows = (data >> 6) + 1;
+					m_i8275VerticalRows = (data & 0x3f) + 1;
+					m_i8275VerticalRetraceRows = (data >> 6) + 1;
 
-					logerror("i8275 %d rows\n", state->m_i8275VerticalRows);
-					logerror("i8275 %d vertical retrace rows\n", state->m_i8275VerticalRetraceRows);
+					logerror("i8275 %d rows\n", m_i8275VerticalRows);
+					logerror("i8275 %d vertical retrace rows\n", m_i8275VerticalRetraceRows);
 
-					state->m_i8275CommandSeqCnt--;
+					m_i8275CommandSeqCnt--;
 				}
 				break;
 
 				case 2:
 				{
 					//screen byte comp byte 3
-					state->m_i8275Underline = (data >> 4) + 1;
-					state->m_i8275Lines = (data & 0xf) + 1;
-					logerror("i8275 underline placement: %d\n", state->m_i8275Underline);
-					logerror("i8275 %d lines/row\n", state->m_i8275Lines);
+					m_i8275Underline = (data >> 4) + 1;
+					m_i8275Lines = (data & 0xf) + 1;
+					logerror("i8275 underline placement: %d\n", m_i8275Underline);
+					logerror("i8275 %d lines/row\n", m_i8275Lines);
 
-					state->m_i8275CommandSeqCnt--;
+					m_i8275CommandSeqCnt--;
 				}
 				break;
 
 				case 1:
 				{
 					//screen byte comp byte 4
-					state->m_i8275LineCounterMode = data >> 7;
-					state->m_i8275FieldAttributeMode = (data >> 6) & 1;
-					state->m_i8275CursorFormat = (data >> 4) & 3;
-					state->m_i8275HorizontalRetrace = ((data & 0xf) + 1) << 1;
-					logerror("i8275 line counter mode: %d\n", state->m_i8275LineCounterMode);
-					if (state->m_i8275FieldAttributeMode)
+					m_i8275LineCounterMode = data >> 7;
+					m_i8275FieldAttributeMode = (data >> 6) & 1;
+					m_i8275CursorFormat = (data >> 4) & 3;
+					m_i8275HorizontalRetrace = ((data & 0xf) + 1) << 1;
+					logerror("i8275 line counter mode: %d\n", m_i8275LineCounterMode);
+					if (m_i8275FieldAttributeMode)
 					{
 						logerror("i8275 field attribute mode non-transparent\n");
 					}
@@ -448,7 +459,7 @@ static WRITE8_HANDLER (i8275_preg_w) //param reg
 						logerror("i8275 field attribute mode transparent\n");
 					}
 
-					switch (state->m_i8275CursorFormat)
+					switch (m_i8275CursorFormat)
 					{
 						case 0:	{logerror("i8275 cursor format - blinking reverse video block\n");}	break;
 						case 1:	{logerror("i8275 cursor format - blinking underline\n");}break;
@@ -456,8 +467,8 @@ static WRITE8_HANDLER (i8275_preg_w) //param reg
 						case 3:	{logerror("i8275 cursor format - nonblinking underline\n");}break;
 					}
 
-					logerror("i8275 %d chars for horizontal retrace\n",state->m_i8275HorizontalRetrace );
-					state->m_i8275CommandSeqCnt--;
+					logerror("i8275 %d chars for horizontal retrace\n",m_i8275HorizontalRetrace );
+					m_i8275CommandSeqCnt--;
 				}
 				break;
 
@@ -486,79 +497,76 @@ static WRITE8_HANDLER (i8275_preg_w) //param reg
 
 }
 
-static READ8_HANDLER (i8275_preg_r) //param reg
+READ8_MEMBER(dwarfd_state::i8275_preg_r)//param reg
 {
 	return 0;
 }
 
-static WRITE8_HANDLER (i8275_creg_w) //comand reg
+WRITE8_MEMBER(dwarfd_state::i8275_creg_w)//comand reg
 {
-	dwarfd_state *state = space->machine().driver_data<dwarfd_state>();
 
 	switch (data>>5)
 	{
 		case 0:
 		{
 			/* reset */
-			state->m_i8275Command = I8275_COMMAND_RESET;
-			state->m_i8275CommandSeqCnt = I8275_COMMAND_RESET_LENGTH;
+			m_i8275Command = I8275_COMMAND_RESET;
+			m_i8275CommandSeqCnt = I8275_COMMAND_RESET_LENGTH;
 		}
 		break;
 
 		case 5:
 		{
 			/* enable interrupt */
-			state->m_i8275Command = I8275_COMMAND_EI;
-			state->m_i8275CommandSeqCnt = I8275_COMMAND_EI_LENGTH;
+			m_i8275Command = I8275_COMMAND_EI;
+			m_i8275CommandSeqCnt = I8275_COMMAND_EI_LENGTH;
 		}
 		break;
 
 		case 6:
 		{
 			/* disable interrupt */
-			state->m_i8275Command = I8275_COMMAND_DI;
-			state->m_i8275CommandSeqCnt = I8275_COMMAND_DI_LENGTH;
+			m_i8275Command = I8275_COMMAND_DI;
+			m_i8275CommandSeqCnt = I8275_COMMAND_DI_LENGTH;
 		}
 		break;
 
 		case 7:
 		{
 			/* preset counters */
-			state->m_i8275CommandSeqCnt = I8275_COMMAND_PRESET_LENGTH;
+			m_i8275CommandSeqCnt = I8275_COMMAND_PRESET_LENGTH;
 
 		}
 		break;
 	}
 }
 
-static READ8_HANDLER (i8275_sreg_r) //status
+READ8_MEMBER(dwarfd_state::i8275_sreg_r)//status
 {
 	return 0;
 }
 
-static READ8_HANDLER(dwarfd_ram_r)
+READ8_MEMBER(dwarfd_state::dwarfd_ram_r)
 {
-	dwarfd_state *state = space->machine().driver_data<dwarfd_state>();
 
-	if (state->m_crt_access == 0)
+	if (m_crt_access == 0)
 	{
-		return state->m_dw_ram[offset];
+		return m_dw_ram[offset];
 	}
 	else
 	{
-		state->m_videobuf[state->m_line * 256 + state->m_idx] = state->m_dw_ram[offset];
-		state->m_idx++;
-		return state->m_dw_ram[offset];
+		m_videobuf[m_line * 256 + m_idx] = m_dw_ram[offset];
+		m_idx++;
+		return m_dw_ram[offset];
 	}
 }
 
-static WRITE8_HANDLER(dwarfd_ram_w)
+WRITE8_MEMBER(dwarfd_state::dwarfd_ram_w)
 {
-	dwarfd_state *state = space->machine().driver_data<dwarfd_state>();
-	state->m_dw_ram[offset] = data;
+	m_dw_ram[offset] = data;
 }
 
-static WRITE8_HANDLER(output1_w)
+WRITE8_MEMBER(dwarfd_state::output1_w)
 {
 /*
  bits:
@@ -573,7 +581,7 @@ static WRITE8_HANDLER(output1_w)
 */
 }
 
-static WRITE8_HANDLER(output2_w)
+WRITE8_MEMBER(dwarfd_state::output2_w)
 {
 /*
  bits:
@@ -589,20 +597,20 @@ static WRITE8_HANDLER(output2_w)
 }
 
 
-static READ8_HANDLER(qc_b8_r)
+READ8_MEMBER(dwarfd_state::qc_b8_r)
 {
-	return space->machine().rand();
+	return machine().rand();
 }
 
-static ADDRESS_MAP_START( mem_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( mem_map, AS_PROGRAM, 8, dwarfd_state )
 	AM_RANGE(0x0000, 0x3fff) AM_ROM
 	AM_RANGE(0x4000, 0x4fff) AM_READWRITE(dwarfd_ram_r, dwarfd_ram_w)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( io_map, AS_IO, 8, dwarfd_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x01, 0x01) AM_DEVREAD("aysnd", ay8910_r)
-	AM_RANGE(0x02, 0x03) AM_DEVWRITE("aysnd", ay8910_data_address_w)
+	AM_RANGE(0x01, 0x01) AM_DEVREAD_LEGACY("aysnd", ay8910_r)
+	AM_RANGE(0x02, 0x03) AM_DEVWRITE_LEGACY("aysnd", ay8910_data_address_w)
 
 	AM_RANGE(0x20, 0x20) AM_READWRITE(i8275_preg_r, i8275_preg_w)
 	AM_RANGE(0x21, 0x21) AM_READWRITE(i8275_sreg_r, i8275_creg_w)
@@ -613,12 +621,12 @@ static ADDRESS_MAP_START( io_map, AS_IO, 8 )
 	AM_RANGE(0xc1, 0xc1) AM_READ_PORT("DSW2")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( qc_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( qc_map, AS_PROGRAM, 8, dwarfd_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM
 	AM_RANGE(0x8000, 0x8fff) AM_READWRITE(dwarfd_ram_r, dwarfd_ram_w)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( qc_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( qc_io_map, AS_IO, 8, dwarfd_state )
 	AM_IMPORT_FROM( io_map )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	AM_RANGE(0xb8, 0xb8) AM_READ(qc_b8_r)
@@ -771,7 +779,7 @@ static VIDEO_START(dwarfd)
 {
 }
 
-static void drawCrt( running_machine &machine, bitmap_t *bitmap,const rectangle *cliprect )
+static void drawCrt( running_machine &machine, bitmap_ind16 &bitmap,const rectangle &cliprect )
 {
 	dwarfd_state *state = machine.driver_data<dwarfd_state>();
 	int x, y;
@@ -832,10 +840,10 @@ static void drawCrt( running_machine &machine, bitmap_t *bitmap,const rectangle
 }
 
 
-static SCREEN_UPDATE( dwarfd )
+static SCREEN_UPDATE_IND16( dwarfd )
 {
-	bitmap_fill(bitmap, cliprect, get_black_pen(screen->machine()));
-	drawCrt(screen->machine(), bitmap, cliprect);
+	bitmap.fill(get_black_pen(screen.machine()), cliprect);
+	drawCrt(screen.machine(), bitmap, cliprect);
 	return 0;
 }
 
@@ -855,22 +863,25 @@ static I8085_CONFIG( dwarfd_i8085_config )
 };
 
 
-#define NUM_LINES 25
-static INTERRUPT_GEN( dwarfd_interrupt )
+static TIMER_DEVICE_CALLBACK( dwarfd_interrupt )
 {
-	dwarfd_state *state = device->machine().driver_data<dwarfd_state>();
+	dwarfd_state *state = timer.machine().driver_data<dwarfd_state>();
+	int scanline = param;
 
-	if (cpu_getiloops(device) < NUM_LINES)
+	if((scanline % 8) != 0)
+		return;
+
+	if (scanline < 25*8)
 	{
-		device_set_input_line(device, I8085_RST65_LINE, HOLD_LINE); // 34 - every 8th line
-		state->m_line = cpu_getiloops(device);
+		device_set_input_line(state->m_maincpu, I8085_RST65_LINE, HOLD_LINE); // 34 - every 8th line
+		state->m_line = scanline/8;
 		state->m_idx = 0;
 	}
 	else
 	{
-		if (cpu_getiloops(device) == NUM_LINES)
+		if (scanline == 25*8)
 		{
-			device_set_input_line(device, I8085_RST55_LINE, HOLD_LINE);//2c - generated by  crt - end of frame
+			device_set_input_line(state->m_maincpu, I8085_RST55_LINE, HOLD_LINE);//2c - generated by  crt - end of frame
 		}
 	}
 }
@@ -1056,8 +1067,7 @@ static MACHINE_CONFIG_START( dwarfd, dwarfd_state )
 	MCFG_CPU_CONFIG(dwarfd_i8085_config)
 	MCFG_CPU_PROGRAM_MAP(mem_map)
 	MCFG_CPU_IO_MAP(io_map)
-
-	MCFG_CPU_VBLANK_INT_HACK(dwarfd_interrupt,NUM_LINES+4) //16 +vblank + 1 unused
+	MCFG_TIMER_ADD_SCANLINE("scantimer", dwarfd_interrupt, "screen", 0, 1)
 
 	MCFG_MACHINE_START(dwarfd)
 	MCFG_MACHINE_RESET(dwarfd)
@@ -1066,10 +1076,9 @@ static MACHINE_CONFIG_START( dwarfd, dwarfd_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
-	MCFG_SCREEN_SIZE(272*2, 200)
+	MCFG_SCREEN_SIZE(272*2, 200+4*8)
 	MCFG_SCREEN_VISIBLE_AREA(0, 272*2-1, 0, 200-1)
-	MCFG_SCREEN_UPDATE(dwarfd)
+	MCFG_SCREEN_UPDATE_STATIC(dwarfd)
 
 	MCFG_GFXDECODE(dwarfd)
 	MCFG_PALETTE_LENGTH(0x100)
@@ -1281,8 +1290,8 @@ static DRIVER_INIT(dwarfd)
 	UINT8 *src, *dst;
 
 	/* expand gfx roms */
-	src = machine.region("gfx1")->base();
-	dst = machine.region("gfx2")->base();
+	src = state->memregion("gfx1")->base();
+	dst = state->memregion("gfx2")->base();
 
 	for (i = 0; i < 0x4000; i++)
 	{
@@ -1295,7 +1304,7 @@ static DRIVER_INIT(dwarfd)
 	}
 
 	/* use low bit as 'interpolation' bit */
-	src = machine.region("gfx2")->base();
+	src = machine.root_device().memregion("gfx2")->base();
 	for (i = 0; i < 0x8000; i++)
 	{
 		if (src[i] & 0x10)
@@ -1326,12 +1335,12 @@ static DRIVER_INIT(qc)
 	DRIVER_INIT_CALL(dwarfd);
 
 	// hacks for program to proceed
-	machine.region("maincpu")->base()[0x6564] = 0x00;
-	machine.region("maincpu")->base()[0x6565] = 0x00;
+	machine.root_device().memregion("maincpu")->base()[0x6564] = 0x00;
+	machine.root_device().memregion("maincpu")->base()[0x6565] = 0x00;
 
-	machine.region("maincpu")->base()[0x59b2] = 0x00;
-	machine.region("maincpu")->base()[0x59b3] = 0x00;
-	machine.region("maincpu")->base()[0x59b4] = 0x00;
+	machine.root_device().memregion("maincpu")->base()[0x59b2] = 0x00;
+	machine.root_device().memregion("maincpu")->base()[0x59b3] = 0x00;
+	machine.root_device().memregion("maincpu")->base()[0x59b4] = 0x00;
 
 }
 
