@@ -48,7 +48,7 @@ Head Panic
 #include "sound/okim6295.h"
 #include "sound/3812intf.h"
 #include "includes/esd16.h"
-
+#include "video/decospr.h"
 
 /***************************************************************************
 
@@ -58,20 +58,18 @@ Head Panic
 
 ***************************************************************************/
 
-static WRITE16_HANDLER( esd16_spriteram_w )
+WRITE16_MEMBER(esd16_state::esd16_spriteram_w)
 {
-	esd16_state *state = space->machine().driver_data<esd16_state>();
-	COMBINE_DATA(&state->m_spriteram[offset]);
+	COMBINE_DATA(&m_spriteram[offset]);
 }
 
-static WRITE16_HANDLER( esd16_sound_command_w )
+WRITE16_MEMBER(esd16_state::esd16_sound_command_w)
 {
-	esd16_state *state = space->machine().driver_data<esd16_state>();
 	if (ACCESSING_BITS_0_7)
 	{
-		soundlatch_w(space, 0, data & 0xff);
-		device_set_input_line(state->m_audio_cpu, 0, ASSERT_LINE);		// Generate an IRQ
-		device_spin_until_time(&space->device(), attotime::from_usec(50));	// Allow the other CPU to reply
+		soundlatch_byte_w(space, 0, data & 0xff);
+		device_set_input_line(m_audio_cpu, 0, ASSERT_LINE);		// Generate an IRQ
+		device_spin_until_time(&space.device(), attotime::from_usec(50));	// Allow the other CPU to reply
 	}
 }
 
@@ -81,19 +79,20 @@ static WRITE16_HANDLER( esd16_sound_command_w )
  AFAIK)
 */
 
-static ADDRESS_MAP_START( multchmp_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( multchmp_map, AS_PROGRAM, 16, esd16_state )
 	AM_RANGE(0x000000, 0x07ffff) AM_ROM																		// ROM
 	AM_RANGE(0x100000, 0x10ffff) AM_RAM																		// RAM
-	AM_RANGE(0x200000, 0x2005ff) AM_RAM_WRITE(paletteram16_xRRRRRGGGGGBBBBB_word_w) AM_BASE_GENERIC(paletteram)	// Palette
-/**/AM_RANGE(0x300000, 0x3007ff) AM_RAM AM_BASE_SIZE_MEMBER(esd16_state, m_spriteram, m_spriteram_size)				// Sprites
+	AM_RANGE(0x200000, 0x2005ff) AM_RAM_WRITE(paletteram_xRRRRRGGGGGBBBBB_word_w) AM_SHARE("paletteram")	// Palette
+/**/AM_RANGE(0x300000, 0x3007ff) AM_RAM AM_SHARE("spriteram")				// Sprites
 	AM_RANGE(0x300800, 0x300807) AM_WRITE(esd16_spriteram_w)												// Sprites (Mirrored)
-/**/AM_RANGE(0x400000, 0x403fff) AM_RAM_WRITE(esd16_vram_0_w) AM_BASE_MEMBER(esd16_state, m_vram_0)						// Layers
-/**/AM_RANGE(0x420000, 0x423fff) AM_RAM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)						//
-/**/AM_RANGE(0x500000, 0x500003) AM_RAM AM_BASE_MEMBER(esd16_state, m_scroll_0)											// Scroll
-/**/AM_RANGE(0x500004, 0x500007) AM_RAM AM_BASE_MEMBER(esd16_state, m_scroll_1)											//
-/**/AM_RANGE(0x500008, 0x50000b) AM_RAM																		//
-/**/AM_RANGE(0x50000c, 0x50000f) AM_RAM																		//
-	AM_RANGE(0x600000, 0x600001) AM_WRITENOP															// IRQ Ack
+/**/AM_RANGE(0x400000, 0x403fff) AM_RAM_WRITE(esd16_vram_0_w) AM_SHARE("vram_0")						// Layers
+/**/AM_RANGE(0x420000, 0x423fff) AM_RAM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")						//
+/**/AM_RANGE(0x500000, 0x500003) AM_RAM AM_SHARE("scroll_0")											// Scroll
+/**/AM_RANGE(0x500004, 0x500007) AM_RAM AM_SHARE("scroll_1")											//
+	AM_RANGE(0x500008, 0x500009) AM_WRITEONLY AM_SHARE("platform_x")
+	AM_RANGE(0x50000a, 0x50000b) AM_WRITEONLY AM_SHARE("platform_y")
+	AM_RANGE(0x50000c, 0x50000d) AM_WRITENOP																// ??
+	AM_RANGE(0x50000e, 0x50000f) AM_WRITEONLY AM_SHARE("head_layersize")								// ??
 	AM_RANGE(0x600002, 0x600003) AM_READ_PORT("P1_P2")
 	AM_RANGE(0x600004, 0x600005) AM_READ_PORT("SYSTEM")
 	AM_RANGE(0x600006, 0x600007) AM_READ_PORT("DSW")
@@ -103,51 +102,49 @@ static ADDRESS_MAP_START( multchmp_map, AS_PROGRAM, 16 )
 	AM_RANGE(0x700008, 0x70000b) AM_READNOP															// ? Only read once
 ADDRESS_MAP_END
 
-static WRITE16_HANDLER(hedpanic_platform_w)
+WRITE16_MEMBER(esd16_state::hedpanic_platform_w)
 {
-	esd16_state *state = space->machine().driver_data<esd16_state>();
-	int offsets = state->m_headpanic_platform_x[0] + 0x40 * state->m_headpanic_platform_y[0];
+	int offsets = m_headpanic_platform_x[0] + 0x40 * m_headpanic_platform_y[0];
 
-	state->m_vram_1[offsets] = data;
-	tilemap_mark_tile_dirty(state->m_tilemap_1_16x16, offsets);
+	m_vram_1[offsets] = data;
+	m_tilemap_1_16x16->mark_tile_dirty(offsets);
 }
 
 
-static READ16_HANDLER( esd_eeprom_r )
+READ16_MEMBER(esd16_state::esd_eeprom_r)
 {
-	esd16_state *state = space->machine().driver_data<esd16_state>();
 	if (ACCESSING_BITS_8_15)
 	{
-		return ((state->m_eeprom->read_bit() & 0x01) << 15);
+		return ((m_eeprom->read_bit() & 0x01) << 15);
 	}
 
-//  logerror("(0x%06x) unk EEPROM read: %04x\n", cpu_get_pc(&space->device()), mem_mask);
+//  logerror("(0x%06x) unk EEPROM read: %04x\n", cpu_get_pc(&space.device()), mem_mask);
 	return 0;
 }
 
-static WRITE16_HANDLER( esd_eeprom_w )
+WRITE16_MEMBER(esd16_state::esd_eeprom_w)
 {
 	if (ACCESSING_BITS_8_15)
-		input_port_write(space->machine(), "EEPROMOUT", data, 0xffff);
+		ioport("EEPROMOUT")->write(data, 0xffff);
 
-//  logerror("(0x%06x) Unk EEPROM write: %04x %04x\n", cpu_get_pc(&space->device()), data, mem_mask);
+//  logerror("(0x%06x) Unk EEPROM write: %04x %04x\n", cpu_get_pc(&space.device()), data, mem_mask);
 }
 
-static ADDRESS_MAP_START( hedpanic_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( hedpanic_map, AS_PROGRAM, 16, esd16_state )
 	AM_RANGE(0x000000, 0x07ffff) AM_ROM																		// ROM
 	AM_RANGE(0x100000, 0x10ffff) AM_RAM																		// RAM
-	AM_RANGE(0x800000, 0x800fff) AM_RAM_WRITE(paletteram16_xRRRRRGGGGGBBBBB_word_w) AM_BASE_GENERIC(paletteram)	// Palette
-	AM_RANGE(0x900000, 0x9007ff) AM_WRITEONLY AM_BASE_SIZE_MEMBER(esd16_state, m_spriteram, m_spriteram_size)		// Sprites
+	AM_RANGE(0x800000, 0x800fff) AM_RAM_WRITE(paletteram_xRRRRRGGGGGBBBBB_word_w) AM_SHARE("paletteram")	// Palette
+	AM_RANGE(0x900000, 0x9007ff) AM_WRITEONLY AM_SHARE("spriteram")		// Sprites
 	AM_RANGE(0x900800, 0x900807) AM_WRITE(esd16_spriteram_w)												// Sprites (Mirrored)
-	AM_RANGE(0xa00000, 0xa03fff) AM_WRITE(esd16_vram_0_w) AM_BASE_MEMBER(esd16_state, m_vram_0)							// Layers
-	AM_RANGE(0xa20000, 0xa23fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							//
-	AM_RANGE(0xa24000, 0xa27fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							// mirror?
-	AM_RANGE(0xb00000, 0xb00003) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_0)									// Scroll
-	AM_RANGE(0xb00004, 0xb00007) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_1)									//
-	AM_RANGE(0xb00008, 0xb00009) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_x)
-	AM_RANGE(0xb0000a, 0xb0000b) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_y)
+	AM_RANGE(0xa00000, 0xa03fff) AM_WRITE(esd16_vram_0_w) AM_SHARE("vram_0")							// Layers
+	AM_RANGE(0xa20000, 0xa23fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							//
+	AM_RANGE(0xa24000, 0xa27fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							// mirror?
+	AM_RANGE(0xb00000, 0xb00003) AM_WRITEONLY AM_SHARE("scroll_0")									// Scroll
+	AM_RANGE(0xb00004, 0xb00007) AM_WRITEONLY AM_SHARE("scroll_1")									//
+	AM_RANGE(0xb00008, 0xb00009) AM_WRITEONLY AM_SHARE("platform_x")
+	AM_RANGE(0xb0000a, 0xb0000b) AM_WRITEONLY AM_SHARE("platform_y")
 	AM_RANGE(0xb0000c, 0xb0000d) AM_WRITENOP																// ??
-	AM_RANGE(0xb0000e, 0xb0000f) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_head_layersize)								// ??
+	AM_RANGE(0xb0000e, 0xb0000f) AM_WRITEONLY AM_SHARE("head_layersize")								// ??
 	AM_RANGE(0xc00000, 0xc00001) AM_WRITENOP // IRQ Ack
 	AM_RANGE(0xc00002, 0xc00003) AM_READ_PORT("P1_P2")
 	AM_RANGE(0xc00004, 0xc00005) AM_READ_PORT("SYSTEM")
@@ -161,13 +158,13 @@ ADDRESS_MAP_END
 
 /* Multi Champ Deluxe, like Head Panic but different addresses */
 
-static ADDRESS_MAP_START( mchampdx_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( mchampdx_map, AS_PROGRAM, 16, esd16_state )
 	AM_RANGE(0x000000, 0x07ffff) AM_ROM																		// ROM
 	AM_RANGE(0x200000, 0x20ffff) AM_RAM																		// RAM
-	AM_RANGE(0x300000, 0x303fff) AM_WRITE(esd16_vram_0_w) AM_BASE_MEMBER(esd16_state, m_vram_0)							// Layers
-	AM_RANGE(0x320000, 0x323fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							//
-	AM_RANGE(0x324000, 0x327fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							// mirror?
-	AM_RANGE(0x400000, 0x400fff) AM_RAM_WRITE(paletteram16_xRRRRRGGGGGBBBBB_word_w) AM_BASE_GENERIC(paletteram)	// Palette
+	AM_RANGE(0x300000, 0x303fff) AM_WRITE(esd16_vram_0_w) AM_SHARE("vram_0")							// Layers
+	AM_RANGE(0x320000, 0x323fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							//
+	AM_RANGE(0x324000, 0x327fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							// mirror?
+	AM_RANGE(0x400000, 0x400fff) AM_RAM_WRITE(paletteram_xRRRRRGGGGGBBBBB_word_w) AM_SHARE("paletteram")	// Palette
 	AM_RANGE(0x500000, 0x500001) AM_WRITENOP	// IRQ Ack
 	AM_RANGE(0x500002, 0x500003) AM_READ_PORT("P1_P2")
 	AM_RANGE(0x500004, 0x500005) AM_READ_PORT("SYSTEM")
@@ -176,33 +173,33 @@ static ADDRESS_MAP_START( mchampdx_map, AS_PROGRAM, 16 )
 	AM_RANGE(0x50000a, 0x50000b) AM_WRITENOP																// ? 2 not checked
 	AM_RANGE(0x50000c, 0x50000d) AM_WRITE(esd16_sound_command_w)											// To Sound CPU // ok
 	AM_RANGE(0x50000e, 0x50000f) AM_WRITE(esd_eeprom_w)
-	AM_RANGE(0x600000, 0x6007ff) AM_WRITEONLY AM_BASE_SIZE_MEMBER(esd16_state, m_spriteram, m_spriteram_size)		// Sprites
+	AM_RANGE(0x600000, 0x6007ff) AM_WRITEONLY AM_SHARE("spriteram")		// Sprites
 	AM_RANGE(0x600800, 0x600807) AM_WRITE(esd16_spriteram_w)												// Sprites (Mirrored)
-	AM_RANGE(0x700000, 0x700003) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_0)									// Scroll
-	AM_RANGE(0x700004, 0x700007) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_1)									//
-	AM_RANGE(0x700008, 0x700009) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_x)							// not used in mchampdx?
-	AM_RANGE(0x70000a, 0x70000b) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_y)							// not used in mchampdx?
+	AM_RANGE(0x700000, 0x700003) AM_WRITEONLY AM_SHARE("scroll_0")									// Scroll
+	AM_RANGE(0x700004, 0x700007) AM_WRITEONLY AM_SHARE("scroll_1")									//
+	AM_RANGE(0x700008, 0x700009) AM_WRITEONLY AM_SHARE("platform_x")							// not used in mchampdx?
+	AM_RANGE(0x70000a, 0x70000b) AM_WRITEONLY AM_SHARE("platform_y")							// not used in mchampdx?
 	AM_RANGE(0x70000c, 0x70000d) AM_WRITENOP																// ??
-	AM_RANGE(0x70000e, 0x70000f) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_head_layersize)								// ??
+	AM_RANGE(0x70000e, 0x70000f) AM_WRITEONLY AM_SHARE("head_layersize")								// ??
 	AM_RANGE(0xd00008, 0xd00009) AM_WRITE(hedpanic_platform_w)												// not used in mchampdx?
 ADDRESS_MAP_END
 
 /* Tang Tang & Deluxe 5 - like the others but again with different addresses */
 
-static ADDRESS_MAP_START( tangtang_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( tangtang_map, AS_PROGRAM, 16, esd16_state )
 	AM_RANGE(0x000000, 0x07ffff) AM_ROM																		// ROM
-	AM_RANGE(0x100000, 0x100fff) AM_RAM_WRITE(paletteram16_xRRRRRGGGGGBBBBB_word_w) AM_BASE_GENERIC(paletteram)	// RAM
-	AM_RANGE(0x200000, 0x2007ff) AM_WRITEONLY AM_BASE_SIZE_MEMBER(esd16_state, m_spriteram, m_spriteram_size)		// Sprites
+	AM_RANGE(0x100000, 0x100fff) AM_RAM_WRITE(paletteram_xRRRRRGGGGGBBBBB_word_w) AM_SHARE("paletteram")	// RAM
+	AM_RANGE(0x200000, 0x2007ff) AM_WRITEONLY AM_SHARE("spriteram")		// Sprites
 	AM_RANGE(0x200800, 0x200807) AM_WRITE(esd16_spriteram_w)												// Sprites (Mirrored)
-	AM_RANGE(0x300000, 0x303fff) AM_WRITE(esd16_vram_0_w) AM_BASE_MEMBER(esd16_state, m_vram_0)							// Layers
-	AM_RANGE(0x320000, 0x323fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							//
-	AM_RANGE(0x324000, 0x327fff) AM_WRITE(esd16_vram_1_w) AM_BASE_MEMBER(esd16_state, m_vram_1)							// mirror?
-	AM_RANGE(0x400000, 0x400003) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_0)									// Scroll
-	AM_RANGE(0x400004, 0x400007) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_scroll_1)									//
-	AM_RANGE(0x400008, 0x400009) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_x)							// not used in mchampdx?
-	AM_RANGE(0x40000a, 0x40000b) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_headpanic_platform_y)							// not used in mchampdx?
+	AM_RANGE(0x300000, 0x303fff) AM_WRITE(esd16_vram_0_w) AM_SHARE("vram_0")							// Layers
+	AM_RANGE(0x320000, 0x323fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							//
+	AM_RANGE(0x324000, 0x327fff) AM_WRITE(esd16_vram_1_w) AM_SHARE("vram_1")							// mirror?
+	AM_RANGE(0x400000, 0x400003) AM_WRITEONLY AM_SHARE("scroll_0")									// Scroll
+	AM_RANGE(0x400004, 0x400007) AM_WRITEONLY AM_SHARE("scroll_1")									//
+	AM_RANGE(0x400008, 0x400009) AM_WRITEONLY AM_SHARE("platform_x")							// not used in mchampdx?
+	AM_RANGE(0x40000a, 0x40000b) AM_WRITEONLY AM_SHARE("platform_y")							// not used in mchampdx?
 	AM_RANGE(0x40000c, 0x40000d) AM_WRITENOP																// ??
-	AM_RANGE(0x40000e, 0x40000f) AM_WRITEONLY AM_BASE_MEMBER(esd16_state, m_head_layersize)									// ??
+	AM_RANGE(0x40000e, 0x40000f) AM_WRITEONLY AM_SHARE("head_layersize")									// ??
 	AM_RANGE(0x500000, 0x500001) AM_WRITENOP	// IRQ Ack
 	AM_RANGE(0x500002, 0x500003) AM_READ_PORT("P1_P2")
 	AM_RANGE(0x500004, 0x500005) AM_READ_PORT("SYSTEM")
@@ -224,33 +221,32 @@ ADDRESS_MAP_END
 
 ***************************************************************************/
 
-static WRITE8_HANDLER( esd16_sound_rombank_w )
+WRITE8_MEMBER(esd16_state::esd16_sound_rombank_w)
 {
 	int bank = data & 0xf;
-	if (data != bank)	logerror("CPU #1 - PC %04X: unknown bank bits: %02X\n", cpu_get_pc(&space->device()), data);
+	if (data != bank)	logerror("CPU #1 - PC %04X: unknown bank bits: %02X\n", cpu_get_pc(&space.device()), data);
 	if (bank >= 3)	bank += 1;
-	memory_set_bank(space->machine(), "bank1", bank);
+	membank("bank1")->set_entry(bank);
 }
 
-static ADDRESS_MAP_START( multchmp_sound_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( multchmp_sound_map, AS_PROGRAM, 8, esd16_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM								// ROM
 	AM_RANGE(0x8000, 0xbfff) AM_ROMBANK("bank1")						// Banked ROM
 	AM_RANGE(0xf800, 0xffff) AM_RAM								// RAM
 ADDRESS_MAP_END
 
-static READ8_HANDLER( esd16_sound_command_r )
+READ8_MEMBER(esd16_state::esd16_sound_command_r)
 {
-	esd16_state *state = space->machine().driver_data<esd16_state>();
 
 	/* Clear IRQ only after reading the command, or some get lost */
-	device_set_input_line(state->m_audio_cpu, 0, CLEAR_LINE);
-	return soundlatch_r(space, 0);
+	device_set_input_line(m_audio_cpu, 0, CLEAR_LINE);
+	return soundlatch_byte_r(space, 0);
 }
 
-static ADDRESS_MAP_START( multchmp_sound_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( multchmp_sound_io_map, AS_IO, 8, esd16_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x00, 0x01) AM_DEVWRITE("ymsnd", ym3812_w)	// YM3812
-	AM_RANGE(0x02, 0x02) AM_DEVREADWRITE_MODERN("oki", okim6295_device, read, write)	// M6295
+	AM_RANGE(0x00, 0x01) AM_DEVWRITE_LEGACY("ymsnd", ym3812_w)	// YM3812
+	AM_RANGE(0x02, 0x02) AM_DEVREADWRITE("oki", okim6295_device, read, write)	// M6295
 	AM_RANGE(0x03, 0x03) AM_READ(esd16_sound_command_r)	// From Main CPU
 	AM_RANGE(0x04, 0x04) AM_WRITENOP	// ? $00, $30
 	AM_RANGE(0x05, 0x05) AM_WRITE(esd16_sound_rombank_w)	// ROM Bank
@@ -309,21 +305,21 @@ static INPUT_PORTS_START( multchmp )
 	PORT_DIPSETTING(      0x0008, DEF_STR( Off ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
 	PORT_DIPNAME( 0x0030, 0x0030, DEF_STR( Coin_A ) )		PORT_DIPLOCATION("SW1:5,6")
-	PORT_DIPSETTING(      0x0000, DEF_STR( 4C_1C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
-	PORT_DIPSETTING(      0x0010, DEF_STR( 3C_1C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
-	PORT_DIPSETTING(      0x0020, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
-	PORT_DIPSETTING(      0x0010, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0000, DEF_STR( 4C_1C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0010, DEF_STR( 3C_1C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0020, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0010, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
 	PORT_DIPSETTING(      0x0030, DEF_STR( 1C_1C ) )
-	PORT_DIPSETTING(      0x0000, DEF_STR( 2C_3C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
-	PORT_DIPSETTING(      0x0020, DEF_STR( 1C_2C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0000, DEF_STR( 2C_3C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0020, DEF_STR( 1C_2C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
 	PORT_DIPNAME( 0x00c0, 0x00c0, DEF_STR( Coin_B ) )		PORT_DIPLOCATION("SW1:7,8")
-	PORT_DIPSETTING(      0x0040, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0040, DEF_STR( 2C_1C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
 	PORT_DIPSETTING(      0x00c0, DEF_STR( 1C_1C ) )
-	PORT_DIPSETTING(      0x0000, DEF_STR( 2C_3C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
-	PORT_DIPSETTING(      0x0080, DEF_STR( 1C_2C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0002)
-	PORT_DIPSETTING(      0x0080, DEF_STR( 1C_3C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
-	PORT_DIPSETTING(      0x0040, DEF_STR( 1C_4C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
-	PORT_DIPSETTING(      0x0000, DEF_STR( 1C_6C ) )	PORT_CONDITION("DSW", 0x0002, PORTCOND_EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0000, DEF_STR( 2C_3C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0080, DEF_STR( 1C_2C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0002)
+	PORT_DIPSETTING(      0x0080, DEF_STR( 1C_3C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0040, DEF_STR( 1C_4C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
+	PORT_DIPSETTING(      0x0000, DEF_STR( 1C_6C ) )	PORT_CONDITION("DSW", 0x0002, EQUALS, 0x0000)
 
 	PORT_DIPNAME( 0x0300, 0x0300, DEF_STR( Difficulty ) )		PORT_DIPLOCATION("SW2:1,2")
 	PORT_DIPSETTING(      0x0200, DEF_STR( Easy ) )
@@ -434,11 +430,24 @@ static const gfx_layout layout_16x16x5 =
 	RGN_FRAC(1,5),
 	5,
 	{ RGN_FRAC(4,5),RGN_FRAC(3,5),RGN_FRAC(2,5),RGN_FRAC(1,5), RGN_FRAC(0,5) },
-	{ STEP8(0+7,-1), STEP8(8*16+7,-1) },
+	{ STEP8(8*16,1), STEP8(0,1) },
 	{ STEP16(0,8) },
 	16*16
 };
 
+
+static const gfx_layout hedpanic_sprite_16x16x5 =
+{
+	16,16,
+	RGN_FRAC(1,3),
+	5,
+	{   RGN_FRAC(2,3), RGN_FRAC(0,3), RGN_FRAC(0,3)+8, RGN_FRAC(1,3),RGN_FRAC(1,3)+8 },
+	{ 256+0,256+1,256+2,256+3,256+4,256+5,256+6,256+7,0,1,2,3,4,5,6,7 },
+	{ 0*16,1*16,2*16,3*16,4*16,5*16,6*16,7*16,8*16,9*16,10*16,11*16,12*16,13*16,14*16,15*16 },
+	16*32,
+};
+
+
 /* 8x8x8 */
 static const gfx_layout layout_8x8x8 =
 {
@@ -452,12 +461,6 @@ static const gfx_layout layout_8x8x8 =
 	8*8*2,
 };
 
-static GFXDECODE_START( esd16 )
-	GFXDECODE_ENTRY( "gfx1", 0, layout_16x16x5, 0x200, 8 ) // [0] Sprites
-	GFXDECODE_ENTRY( "gfx2", 0, layout_8x8x8,   0x000, 2 ) // [1] Layers
-	GFXDECODE_ENTRY( "gfx1", 0, layout_16x16x5, 0x200, 8 ) // [0] Sprites
-GFXDECODE_END
-
 static const gfx_layout hedpanic_layout_8x8x8 =
 {
 	8,8,
@@ -484,16 +487,12 @@ static const gfx_layout hedpanic_layout_16x16x8 =
 };
 
 
-static const gfx_layout hedpanic_sprite_16x16x5 =
-{
-	16,16,
-	RGN_FRAC(1,3),
-	5,
-	{   RGN_FRAC(2,3), RGN_FRAC(0,3), RGN_FRAC(0,3)+8, RGN_FRAC(1,3),RGN_FRAC(1,3)+8 },
-	{ 7,6,5,4,3,2,1,0, 256+7,256+6,256+5,256+4,256+3,256+2,256+1,256+0 },
-	{ 0*16,1*16,2*16,3*16,4*16,5*16,6*16,7*16,8*16,9*16,10*16,11*16,12*16,13*16,14*16,15*16 },
-	16*32,
-};
+
+static GFXDECODE_START( esd16 )
+	GFXDECODE_ENTRY( "gfx1", 0, layout_16x16x5, 0x200, 8 ) // [0] Sprites
+	GFXDECODE_ENTRY( "gfx2", 0, layout_8x8x8,   0x000, 2 ) // [1] Layers
+	GFXDECODE_ENTRY( "gfx2", 0, hedpanic_layout_16x16x8,   0x000, 2 ) // [1] Layers
+GFXDECODE_END
 
 
 static GFXDECODE_START( hedpanic )
@@ -519,9 +518,9 @@ GFXDECODE_END
 static MACHINE_START( esd16 )
 {
 	esd16_state *state = machine.driver_data<esd16_state>();
-	UINT8 *AUDIO = machine.region("audiocpu")->base();
+	UINT8 *AUDIO = state->memregion("audiocpu")->base();
 
-	memory_configure_bank(machine, "bank1", 0, 17, &AUDIO[0x0000], 0x4000);
+	state->membank("bank1")->configure_entries(0, 17, &AUDIO[0x0000], 0x4000);
 
 	state->m_audio_cpu = machine.device("audiocpu");
 	state->m_eeprom = machine.device<eeprom_device>("eeprom");
@@ -536,7 +535,15 @@ static MACHINE_RESET( esd16 )
 	state->m_tilemap0_color = 0;
 }
 
-static MACHINE_CONFIG_START( multchmp, esd16_state )
+static UINT16 hedpanic_pri_callback(UINT16 x)
+{
+	if (x & 0x8000)
+		return 0xfffe; // under "tilemap 1"
+	else
+		return 0; // above everything
+}
+
+static MACHINE_CONFIG_START( esd16, esd16_state )
 
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu",M68000, 16000000)
@@ -555,10 +562,15 @@ static MACHINE_CONFIG_START( multchmp, esd16_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(0x140, 0x100)
 	MCFG_SCREEN_VISIBLE_AREA(0, 0x140-1, 0+8, 0x100-8-1)
-	MCFG_SCREEN_UPDATE(esd16)
+	MCFG_SCREEN_UPDATE_STATIC(hedpanic)
+
+	MCFG_DEVICE_ADD("spritegen", DECO_SPRITE, 0)
+	decospr_device::set_gfx_region(*device, 0);
+	decospr_device::set_is_bootleg(*device, true);
+	decospr_device::set_pri_callback(*device, hedpanic_pri_callback);
+	decospr_device::set_flipallx(*device, 1);
 
 	MCFG_GFXDECODE(esd16)
 	MCFG_PALETTE_LENGTH(768)
@@ -575,7 +587,7 @@ static MACHINE_CONFIG_START( multchmp, esd16_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.60)
 MACHINE_CONFIG_END
 
-static MACHINE_CONFIG_DERIVED( hedpanic, multchmp )
+static MACHINE_CONFIG_DERIVED( hedpanic, esd16 )
 
 	/* basic machine hardware */
 
@@ -588,10 +600,13 @@ static MACHINE_CONFIG_DERIVED( hedpanic, multchmp )
 
 	MCFG_GFXDECODE(hedpanic)
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(hedpanic)
+	MCFG_SCREEN_UPDATE_STATIC(hedpanic)
 
+	MCFG_DEVICE_MODIFY("spritegen")
+	decospr_device::set_offsets(*device, -0x18,-0x100);
 MACHINE_CONFIG_END
 
+
 static MACHINE_CONFIG_DERIVED( mchampdx, hedpanic )
 
 	/* basic machine hardware */
@@ -617,11 +632,16 @@ static MACHINE_CONFIG_DERIVED( swatpolc, hedpanic )
 	MCFG_GFXDECODE(tangtang)
 MACHINE_CONFIG_END
 
-static MACHINE_CONFIG_DERIVED( hedpanio, hedpanic )
-
+static MACHINE_CONFIG_DERIVED( hedpanio, esd16 )
 	/* basic machine hardware */
-	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(hedpanio)
+	MCFG_CPU_MODIFY("maincpu")
+	MCFG_CPU_PROGRAM_MAP(hedpanic_map)
+
+	MCFG_EEPROM_93C46_ADD("eeprom")
+
+	MCFG_PALETTE_LENGTH(0x1000/2)
+
+	MCFG_GFXDECODE(hedpanic)
 MACHINE_CONFIG_END
 
 
@@ -1279,8 +1299,8 @@ ROM_END
 ***************************************************************************/
 
 /* ESD 11-09-98 */
-GAME( 1999, multchmp, 0,        multchmp, multchmp, 0, ROT0, "ESD",         "Multi Champ (World, ver. 2.5)", GAME_SUPPORTS_SAVE )
-GAME( 1998, multchmpk,multchmp, multchmp, multchmp, 0, ROT0, "ESD",         "Multi Champ (Korea)", GAME_SUPPORTS_SAVE )
+GAME( 1999, multchmp, 0,        esd16,    multchmp, 0, ROT0, "ESD",         "Multi Champ (World, ver. 2.5)", GAME_SUPPORTS_SAVE )
+GAME( 1998, multchmpk,multchmp, esd16,    multchmp, 0, ROT0, "ESD",         "Multi Champ (Korea)", GAME_SUPPORTS_SAVE )
 
 /* ESD 05-28-99 */
 GAME( 1999, hedpanico,hedpanic, hedpanio, hedpanic, 0, ROT0, "ESD",         "Head Panic (ver. 0615, 15/06/1999)", GAME_SUPPORTS_SAVE )
