@@ -19,9 +19,9 @@
 // These values were derived based on disassembly of the code. Usually, it
 // was pretty obvious what the values should be. Of course, this will have
 // to change if a different ROM set ever surfaces.
-static READ8_HANDLER( fastfred_custom_io_r )
+READ8_MEMBER(fastfred_state::fastfred_custom_io_r)
 {
-    switch (cpu_get_pc(&space->device()))
+    switch (cpu_get_pc(&space.device()))
     {
     case 0x03c0: return 0x9d;
     case 0x03e6: return 0x9f;
@@ -47,14 +47,14 @@ static READ8_HANDLER( fastfred_custom_io_r )
     case 0x7b58: return 0x20;
     }
 
-    logerror("Uncaught custom I/O read %04X at %04X\n", 0xc800+offset, cpu_get_pc(&space->device()));
+    logerror("Uncaught custom I/O read %04X at %04X\n", 0xc800+offset, cpu_get_pc(&space.device()));
     return 0x00;
 }
 
-static READ8_HANDLER( flyboy_custom1_io_r )
+READ8_MEMBER(fastfred_state::flyboy_custom1_io_r)
 {
 
-	switch (cpu_get_pc(&space->device()))
+	switch (cpu_get_pc(&space.device()))
 	{
 	 case 0x049d: return 0xad;	/* compare */
 	 case 0x04b9:			/* compare with 0x9e ??? When ??? */
@@ -75,14 +75,14 @@ static READ8_HANDLER( flyboy_custom1_io_r )
 	 return 0x00;
 	}
 
-	logerror("Uncaught custom I/O read %04X at %04X\n", 0xc085+offset, cpu_get_pc(&space->device()));
+	logerror("Uncaught custom I/O read %04X at %04X\n", 0xc085+offset, cpu_get_pc(&space.device()));
 	return 0x00;
 }
 
-static READ8_HANDLER( flyboy_custom2_io_r )
+READ8_MEMBER(fastfred_state::flyboy_custom2_io_r)
 {
 
-	switch (cpu_get_pc(&space->device()))
+	switch (cpu_get_pc(&space.device()))
 	{
 	 case 0x0395: return 0xf7;	/* $C900 compare         */
 	 case 0x03f5:			/* $c8fd                 */
@@ -100,19 +100,19 @@ static READ8_HANDLER( flyboy_custom2_io_r )
 	 return 0x00;
 	}
 
-	logerror("Uncaught custom I/O read %04X at %04X\n", 0xc8fb+offset, cpu_get_pc(&space->device()));
+	logerror("Uncaught custom I/O read %04X at %04X\n", 0xc8fb+offset, cpu_get_pc(&space.device()));
 	return 0x00;
 }
 
 
-static READ8_HANDLER( jumpcoas_custom_io_r )
+READ8_MEMBER(fastfred_state::jumpcoas_custom_io_r)
 {
 	if (offset == 0x100)  return 0x63;
 
 	return 0x00;
 }
 
-static READ8_HANDLER( boggy84_custom_io_r )
+READ8_MEMBER(fastfred_state::boggy84_custom_io_r)
 {
 	if (offset == 0x100)  return 0x6a;
 
@@ -130,126 +130,135 @@ static MACHINE_START( imago )
 	gfx_element_set_source(machine.gfx[1], state->m_imago_sprites);
 }
 
-static WRITE8_HANDLER( imago_dma_irq_w )
+WRITE8_MEMBER(fastfred_state::imago_dma_irq_w)
 {
-	cputag_set_input_line(space->machine(), "maincpu", 0, data & 1 ? ASSERT_LINE : CLEAR_LINE);
+	cputag_set_input_line(machine(), "maincpu", 0, data & 1 ? ASSERT_LINE : CLEAR_LINE);
 }
 
-static WRITE8_HANDLER( imago_sprites_bank_w )
+WRITE8_MEMBER(fastfred_state::imago_sprites_bank_w)
 {
-	fastfred_state *state = space->machine().driver_data<fastfred_state>();
-	state->m_imago_sprites_bank = (data & 2) >> 1;
+	m_imago_sprites_bank = (data & 2) >> 1;
 }
 
-static WRITE8_HANDLER( imago_sprites_dma_w )
+WRITE8_MEMBER(fastfred_state::imago_sprites_dma_w)
 {
-	fastfred_state *state = space->machine().driver_data<fastfred_state>();
-	UINT8 *rom = (UINT8 *)space->machine().region("gfx2")->base();
+	UINT8 *rom = (UINT8 *)machine().root_device().memregion("gfx2")->base();
 	UINT8 sprites_data;
 
-	sprites_data = rom[state->m_imago_sprites_address + 0x2000*0 + state->m_imago_sprites_bank * 0x1000];
-	state->m_imago_sprites[offset + 0x800*0] = sprites_data;
+	sprites_data = rom[m_imago_sprites_address + 0x2000*0 + m_imago_sprites_bank * 0x1000];
+	m_imago_sprites[offset + 0x800*0] = sprites_data;
 
-	sprites_data = rom[state->m_imago_sprites_address + 0x2000*1 + state->m_imago_sprites_bank * 0x1000];
-	state->m_imago_sprites[offset + 0x800*1] = sprites_data;
+	sprites_data = rom[m_imago_sprites_address + 0x2000*1 + m_imago_sprites_bank * 0x1000];
+	m_imago_sprites[offset + 0x800*1] = sprites_data;
 
-	sprites_data = rom[state->m_imago_sprites_address + 0x2000*2 + state->m_imago_sprites_bank * 0x1000];
-	state->m_imago_sprites[offset + 0x800*2] = sprites_data;
+	sprites_data = rom[m_imago_sprites_address + 0x2000*2 + m_imago_sprites_bank * 0x1000];
+	m_imago_sprites[offset + 0x800*2] = sprites_data;
 
-	gfx_element_mark_dirty(space->machine().gfx[1], offset/32);
+	gfx_element_mark_dirty(machine().gfx[1], offset/32);
 }
 
-static READ8_HANDLER( imago_sprites_offset_r )
+READ8_MEMBER(fastfred_state::imago_sprites_offset_r)
 {
-	fastfred_state *state = space->machine().driver_data<fastfred_state>();
-	state->m_imago_sprites_address = offset;
+	m_imago_sprites_address = offset;
 	return 0xff; //not really used
 }
 
-static ADDRESS_MAP_START( fastfred_map, AS_PROGRAM, 8 )
+WRITE8_MEMBER(fastfred_state::nmi_mask_w)
+{
+
+	m_nmi_mask = data & 1;
+}
+
+WRITE8_MEMBER(fastfred_state::sound_nmi_mask_w)
+{
+
+	m_sound_nmi_mask = data & 1;
+}
+
+static ADDRESS_MAP_START( fastfred_map, AS_PROGRAM, 8, fastfred_state )
 	AM_RANGE(0x0000, 0xbfff) AM_ROM
 	AM_RANGE(0xc000, 0xc7ff) AM_RAM
-	AM_RANGE(0xd000, 0xd3ff) AM_MIRROR(0x400) AM_RAM_WRITE(fastfred_videoram_w) AM_BASE_MEMBER(fastfred_state, m_videoram)
-	AM_RANGE(0xd800, 0xd83f) AM_RAM_WRITE(fastfred_attributes_w) AM_BASE_MEMBER(fastfred_state, m_attributesram)
-	AM_RANGE(0xd840, 0xd85f) AM_RAM AM_BASE_MEMBER(fastfred_state, m_spriteram) AM_SIZE_MEMBER(fastfred_state, m_spriteram_size)
+	AM_RANGE(0xd000, 0xd3ff) AM_MIRROR(0x400) AM_RAM_WRITE_LEGACY(fastfred_videoram_w) AM_SHARE("videoram")
+	AM_RANGE(0xd800, 0xd83f) AM_RAM_WRITE_LEGACY(fastfred_attributes_w) AM_SHARE("attributesram")
+	AM_RANGE(0xd840, 0xd85f) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0xd860, 0xdbff) AM_RAM // Unused, but initialized
-	AM_RANGE(0xe000, 0xe000) AM_READ_PORT("BUTTONS") AM_WRITEONLY AM_BASE_MEMBER(fastfred_state, m_background_color)
+	AM_RANGE(0xe000, 0xe000) AM_READ_PORT("BUTTONS") AM_WRITEONLY AM_SHARE("bgcolor")
 	AM_RANGE(0xe800, 0xe800) AM_READ_PORT("JOYS")
 	AM_RANGE(0xf000, 0xf000) AM_READ_PORT("DSW") AM_WRITENOP
-	AM_RANGE(0xf001, 0xf001) AM_WRITE(interrupt_enable_w)
-	AM_RANGE(0xf002, 0xf002) AM_WRITE(fastfred_colorbank1_w)
-	AM_RANGE(0xf003, 0xf003) AM_WRITE(fastfred_colorbank2_w)
-	AM_RANGE(0xf004, 0xf004) AM_WRITE(fastfred_charbank1_w)
-	AM_RANGE(0xf005, 0xf005) AM_WRITE(fastfred_charbank2_w)
-	AM_RANGE(0xf006, 0xf006) AM_WRITE(fastfred_flip_screen_x_w)
-	AM_RANGE(0xf007, 0xf007) AM_WRITE(fastfred_flip_screen_y_w)
-	AM_RANGE(0xf116, 0xf116) AM_WRITE(fastfred_flip_screen_x_w)
-	AM_RANGE(0xf117, 0xf117) AM_WRITE(fastfred_flip_screen_y_w)
-	AM_RANGE(0xf800, 0xf800) AM_READWRITE(watchdog_reset_r, soundlatch_w)
+	AM_RANGE(0xf001, 0xf001) AM_WRITE(nmi_mask_w)
+	AM_RANGE(0xf002, 0xf002) AM_WRITE_LEGACY(fastfred_colorbank1_w)
+	AM_RANGE(0xf003, 0xf003) AM_WRITE_LEGACY(fastfred_colorbank2_w)
+	AM_RANGE(0xf004, 0xf004) AM_WRITE_LEGACY(fastfred_charbank1_w)
+	AM_RANGE(0xf005, 0xf005) AM_WRITE_LEGACY(fastfred_charbank2_w)
+	AM_RANGE(0xf006, 0xf006) AM_WRITE_LEGACY(fastfred_flip_screen_x_w)
+	AM_RANGE(0xf007, 0xf007) AM_WRITE_LEGACY(fastfred_flip_screen_y_w)
+	AM_RANGE(0xf116, 0xf116) AM_WRITE_LEGACY(fastfred_flip_screen_x_w)
+	AM_RANGE(0xf117, 0xf117) AM_WRITE_LEGACY(fastfred_flip_screen_y_w)
+	AM_RANGE(0xf800, 0xf800) AM_READWRITE(watchdog_reset_r, soundlatch_byte_w)
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( jumpcoas_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( jumpcoas_map, AS_PROGRAM, 8, fastfred_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM
 	AM_RANGE(0xc000, 0xc7ff) AM_RAM
-	AM_RANGE(0xd000, 0xd03f) AM_RAM_WRITE(fastfred_attributes_w) AM_BASE_MEMBER(fastfred_state, m_attributesram)
-	AM_RANGE(0xd040, 0xd05f) AM_RAM AM_BASE_MEMBER(fastfred_state, m_spriteram) AM_SIZE_MEMBER(fastfred_state, m_spriteram_size)
+	AM_RANGE(0xd000, 0xd03f) AM_RAM_WRITE_LEGACY(fastfred_attributes_w) AM_SHARE("attributesram")
+	AM_RANGE(0xd040, 0xd05f) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0xd060, 0xd3ff) AM_RAM
-	AM_RANGE(0xd800, 0xdbff) AM_MIRROR(0x400) AM_RAM_WRITE(fastfred_videoram_w) AM_BASE_MEMBER(fastfred_state, m_videoram)
-	AM_RANGE(0xe000, 0xe000) AM_WRITEONLY AM_BASE_MEMBER(fastfred_state, m_background_color)
+	AM_RANGE(0xd800, 0xdbff) AM_MIRROR(0x400) AM_RAM_WRITE_LEGACY(fastfred_videoram_w) AM_SHARE("videoram")
+	AM_RANGE(0xe000, 0xe000) AM_WRITEONLY AM_SHARE("bgcolor")
 	AM_RANGE(0xe800, 0xe800) AM_READ_PORT("DSW1")
 	AM_RANGE(0xe801, 0xe801) AM_READ_PORT("DSW2")
 	AM_RANGE(0xe802, 0xe802) AM_READ_PORT("BUTTONS")
 	AM_RANGE(0xe803, 0xe803) AM_READ_PORT("JOYS")
 	AM_RANGE(0xf000, 0xf000) AM_WRITENOP // Unused, but initialized
-	AM_RANGE(0xf001, 0xf001) AM_WRITE(interrupt_enable_w)
-	AM_RANGE(0xf002, 0xf002) AM_WRITE(fastfred_colorbank1_w)
-	AM_RANGE(0xf003, 0xf003) AM_WRITE(fastfred_colorbank2_w)
-	AM_RANGE(0xf004, 0xf004) AM_WRITE(fastfred_charbank1_w)
-	AM_RANGE(0xf005, 0xf005) AM_WRITE(fastfred_charbank2_w)
-	AM_RANGE(0xf006, 0xf006) AM_WRITE(fastfred_flip_screen_x_w)
-	AM_RANGE(0xf007, 0xf007) AM_WRITE(fastfred_flip_screen_y_w)
-	AM_RANGE(0xf116, 0xf116) AM_WRITE(fastfred_flip_screen_x_w)
-	AM_RANGE(0xf117, 0xf117) AM_WRITE(fastfred_flip_screen_y_w)
+	AM_RANGE(0xf001, 0xf001) AM_WRITE(nmi_mask_w)
+	AM_RANGE(0xf002, 0xf002) AM_WRITE_LEGACY(fastfred_colorbank1_w)
+	AM_RANGE(0xf003, 0xf003) AM_WRITE_LEGACY(fastfred_colorbank2_w)
+	AM_RANGE(0xf004, 0xf004) AM_WRITE_LEGACY(fastfred_charbank1_w)
+	AM_RANGE(0xf005, 0xf005) AM_WRITE_LEGACY(fastfred_charbank2_w)
+	AM_RANGE(0xf006, 0xf006) AM_WRITE_LEGACY(fastfred_flip_screen_x_w)
+	AM_RANGE(0xf007, 0xf007) AM_WRITE_LEGACY(fastfred_flip_screen_y_w)
+	AM_RANGE(0xf116, 0xf116) AM_WRITE_LEGACY(fastfred_flip_screen_x_w)
+	AM_RANGE(0xf117, 0xf117) AM_WRITE_LEGACY(fastfred_flip_screen_y_w)
 	//AM_RANGE(0xf800, 0xf800) AM_READ(watchdog_reset_r)  // Why doesn't this work???
-	AM_RANGE(0xf800, 0xf801) AM_READNOP AM_DEVWRITE("ay8910.1", ay8910_address_data_w)
+	AM_RANGE(0xf800, 0xf801) AM_READNOP AM_DEVWRITE_LEGACY("ay8910.1", ay8910_address_data_w)
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( imago_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( imago_map, AS_PROGRAM, 8, fastfred_state )
 	AM_RANGE(0x0000, 0x0fff) AM_ROM
 	AM_RANGE(0x1000, 0x1fff) AM_READ(imago_sprites_offset_r)
 	AM_RANGE(0x2000, 0x6fff) AM_ROM
 	AM_RANGE(0xb000, 0xb3ff) AM_RAM // same fg videoram (which one of the 2 is really used?)
 	AM_RANGE(0xb800, 0xbfff) AM_RAM_WRITE(imago_sprites_dma_w)
 	AM_RANGE(0xc000, 0xc7ff) AM_RAM
-	AM_RANGE(0xc800, 0xcbff) AM_RAM_WRITE(imago_fg_videoram_w) AM_BASE_MEMBER(fastfred_state, m_imago_fg_videoram)
-	AM_RANGE(0xd000, 0xd3ff) AM_RAM_WRITE(fastfred_videoram_w) AM_BASE_MEMBER(fastfred_state, m_videoram)
-	AM_RANGE(0xd800, 0xd83f) AM_RAM_WRITE(fastfred_attributes_w) AM_BASE_MEMBER(fastfred_state, m_attributesram)
-	AM_RANGE(0xd840, 0xd85f) AM_RAM AM_BASE_MEMBER(fastfred_state, m_spriteram) AM_SIZE_MEMBER(fastfred_state, m_spriteram_size)
+	AM_RANGE(0xc800, 0xcbff) AM_RAM_WRITE_LEGACY(imago_fg_videoram_w) AM_SHARE("imago_fg_vram")
+	AM_RANGE(0xd000, 0xd3ff) AM_RAM_WRITE_LEGACY(fastfred_videoram_w) AM_SHARE("videoram")
+	AM_RANGE(0xd800, 0xd83f) AM_RAM_WRITE_LEGACY(fastfred_attributes_w) AM_SHARE("attributesram")
+	AM_RANGE(0xd840, 0xd85f) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0xd860, 0xd8ff) AM_RAM // Unused, but initialized
 	AM_RANGE(0xe000, 0xe000) AM_READ_PORT("BUTTONS")
 	AM_RANGE(0xe800, 0xe800) AM_READ_PORT("JOYS")
 	AM_RANGE(0xf000, 0xf000) AM_READ_PORT("DSW") AM_WRITENOP // writes 1 when level starts, 0 when game over
-	AM_RANGE(0xf001, 0xf001) AM_WRITE(interrupt_enable_w)
-	AM_RANGE(0xf002, 0xf002) AM_WRITE(fastfred_colorbank1_w)
-	AM_RANGE(0xf003, 0xf003) AM_WRITE(fastfred_colorbank2_w)
+	AM_RANGE(0xf001, 0xf001) AM_WRITE(nmi_mask_w)
+	AM_RANGE(0xf002, 0xf002) AM_WRITE_LEGACY(fastfred_colorbank1_w)
+	AM_RANGE(0xf003, 0xf003) AM_WRITE_LEGACY(fastfred_colorbank2_w)
 	AM_RANGE(0xf004, 0xf004) AM_WRITE(imago_dma_irq_w)
-	AM_RANGE(0xf005, 0xf005) AM_WRITE(imago_charbank_w)
-	AM_RANGE(0xf006, 0xf006) AM_WRITE(fastfred_flip_screen_x_w)
-	AM_RANGE(0xf007, 0xf007) AM_WRITE(fastfred_flip_screen_y_w)
+	AM_RANGE(0xf005, 0xf005) AM_WRITE_LEGACY(imago_charbank_w)
+	AM_RANGE(0xf006, 0xf006) AM_WRITE_LEGACY(fastfred_flip_screen_x_w)
+	AM_RANGE(0xf007, 0xf007) AM_WRITE_LEGACY(fastfred_flip_screen_y_w)
 	AM_RANGE(0xf400, 0xf400) AM_WRITENOP // writes 0 or 2
 	AM_RANGE(0xf401, 0xf401) AM_WRITE(imago_sprites_bank_w)
-	AM_RANGE(0xf800, 0xf800) AM_READNOP AM_WRITE(soundlatch_w)
+	AM_RANGE(0xf800, 0xf800) AM_READNOP AM_WRITE(soundlatch_byte_w)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( sound_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( sound_map, AS_PROGRAM, 8, fastfred_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROM
 	AM_RANGE(0x2000, 0x23ff) AM_RAM
-	AM_RANGE(0x3000, 0x3000) AM_READWRITE(soundlatch_r, interrupt_enable_w)
+	AM_RANGE(0x3000, 0x3000) AM_READ(soundlatch_byte_r) AM_WRITE(sound_nmi_mask_w)
 	AM_RANGE(0x4000, 0x4000) AM_WRITEONLY  // Reset PSG's
-	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE("ay8910.1", ay8910_address_data_w)
-	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE("ay8910.2", ay8910_address_data_w)
+	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE_LEGACY("ay8910.1", ay8910_address_data_w)
+	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE_LEGACY("ay8910.2", ay8910_address_data_w)
 	AM_RANGE(0x7000, 0x7000) AM_READNOP // only for Imago, read but not used
 ADDRESS_MAP_END
 
@@ -618,26 +627,40 @@ GFXDECODE_END
 
 #define CLOCK 18432000  /* The crystal is 18.432MHz */
 
+static INTERRUPT_GEN( vblank_irq )
+{
+	fastfred_state *state = device->machine().driver_data<fastfred_state>();
+
+	if(state->m_nmi_mask)
+		device_set_input_line(device, INPUT_LINE_NMI, PULSE_LINE);
+}
+
+static INTERRUPT_GEN( sound_timer_irq )
+{
+	fastfred_state *state = device->machine().driver_data<fastfred_state>();
+
+	if(state->m_sound_nmi_mask)
+		device_set_input_line(device, INPUT_LINE_NMI, PULSE_LINE);
+}
 
 static MACHINE_CONFIG_START( fastfred, fastfred_state )
 
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu", Z80, CLOCK/6)     /* 3.072 MHz */
 	MCFG_CPU_PROGRAM_MAP(fastfred_map)
-	MCFG_CPU_VBLANK_INT("screen", nmi_line_pulse)
+	MCFG_CPU_VBLANK_INT("screen", vblank_irq)
 
 	MCFG_CPU_ADD("audiocpu", Z80, CLOCK/12)	 /* 1.536 MHz */
 	MCFG_CPU_PROGRAM_MAP(sound_map)
-	MCFG_CPU_PERIODIC_INT(nmi_line_pulse,4*60)
+	MCFG_CPU_PERIODIC_INT(sound_timer_irq,4*60)
 
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))	//CLOCK/16/60
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(fastfred)
+	MCFG_SCREEN_UPDATE_STATIC(fastfred)
 
 	MCFG_GFXDECODE(fastfred)
 	MCFG_PALETTE_LENGTH(32*8)
@@ -684,7 +707,7 @@ static MACHINE_CONFIG_DERIVED( imago, fastfred )
 
 	MCFG_VIDEO_START(imago)
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(imago)
+	MCFG_SCREEN_UPDATE_STATIC(imago)
 MACHINE_CONFIG_END
 
 #undef CLOCK
@@ -981,8 +1004,8 @@ ROM_END
 static DRIVER_INIT( flyboy )
 {
 	fastfred_state *state = machine.driver_data<fastfred_state>();
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc085, 0xc099, FUNC(flyboy_custom1_io_r));
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc8fb, 0xc900, FUNC(flyboy_custom2_io_r));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc085, 0xc099, read8_delegate(FUNC(fastfred_state::flyboy_custom1_io_r),state));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc8fb, 0xc900, read8_delegate(FUNC(fastfred_state::flyboy_custom2_io_r),state));
 	state->m_hardware_type = 1;
 }
 
@@ -995,7 +1018,7 @@ static DRIVER_INIT( flyboyb )
 static DRIVER_INIT( fastfred )
 {
 	fastfred_state *state = machine.driver_data<fastfred_state>();
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc800, 0xcfff, FUNC(fastfred_custom_io_r));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc800, 0xcfff, read8_delegate(FUNC(fastfred_state::fastfred_custom_io_r),state));
 	machine.device("maincpu")->memory().space(AS_PROGRAM)->nop_write(0xc800, 0xcfff);
 	state->m_hardware_type = 1;
 }
@@ -1003,7 +1026,7 @@ static DRIVER_INIT( fastfred )
 static DRIVER_INIT( jumpcoas )
 {
 	fastfred_state *state = machine.driver_data<fastfred_state>();
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc800, 0xcfff, FUNC(jumpcoas_custom_io_r));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc800, 0xcfff, read8_delegate(FUNC(fastfred_state::jumpcoas_custom_io_r),state));
 	machine.device("maincpu")->memory().space(AS_PROGRAM)->nop_write(0xc800, 0xcfff);
 	state->m_hardware_type = 0;
 }
@@ -1011,7 +1034,7 @@ static DRIVER_INIT( jumpcoas )
 static DRIVER_INIT( boggy84b )
 {
 	fastfred_state *state = machine.driver_data<fastfred_state>();
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc800, 0xcfff, FUNC(jumpcoas_custom_io_r));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc800, 0xcfff, read8_delegate(FUNC(fastfred_state::jumpcoas_custom_io_r),state));
 	machine.device("maincpu")->memory().space(AS_PROGRAM)->nop_write(0xc800, 0xcfff);
 	state->m_hardware_type = 2;
 }
@@ -1019,7 +1042,7 @@ static DRIVER_INIT( boggy84b )
 static DRIVER_INIT( boggy84 )
 {
 	fastfred_state *state = machine.driver_data<fastfred_state>();
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xc800, 0xcfff, FUNC(boggy84_custom_io_r));
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xc800, 0xcfff, read8_delegate(FUNC(fastfred_state::boggy84_custom_io_r),state));
 	machine.device("maincpu")->memory().space(AS_PROGRAM)->nop_write(0xc800, 0xcfff);
 	state->m_hardware_type = 2;
 }
