@@ -94,12 +94,30 @@ class highvdeo_state : public driver_device
 {
 public:
 	highvdeo_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_blit_ram(*this, "blit_ram"){ }
 
-	UINT16 *m_blit_ram;
+	required_shared_ptr<UINT16> m_blit_ram;
 	UINT16 m_vblank_bit;
 	UINT16 m_brasil_prot_latch;
 	struct { int r,g,b,offs,offs_internal; } m_pal;
+	DECLARE_READ16_MEMBER(read1_r);
+	DECLARE_READ16_MEMBER(read2_r);
+	DECLARE_READ16_MEMBER(read3_r);
+	DECLARE_WRITE16_MEMBER(tv_vcf_paletteram_w);
+	DECLARE_WRITE16_MEMBER(tv_vcf_bankselect_w);
+	DECLARE_WRITE16_MEMBER(write1_w);
+	DECLARE_READ16_MEMBER(tv_ncf_read2_r);
+	DECLARE_WRITE16_MEMBER(tv_tcf_paletteram_w);
+	DECLARE_WRITE16_MEMBER(tv_tcf_bankselect_w);
+	DECLARE_READ16_MEMBER(newmcard_status_r);
+	DECLARE_READ16_MEMBER(newmcard_vblank_r);
+	DECLARE_WRITE16_MEMBER(newmcard_vblank_w);
+	DECLARE_WRITE16_MEMBER(write2_w);
+	DECLARE_READ16_MEMBER(brasil_status_r);
+	DECLARE_WRITE16_MEMBER(brasil_status_w);
+	DECLARE_READ16_MEMBER(ciclone_status_r);
+	DECLARE_WRITE16_MEMBER(fashion_output_w);
 };
 
 
@@ -110,28 +128,28 @@ static VIDEO_START(tourvisn)
 
 }
 
-static SCREEN_UPDATE(tourvisn)
+static SCREEN_UPDATE_RGB32(tourvisn)
 {
-	highvdeo_state *state = screen->machine().driver_data<highvdeo_state>();
+	highvdeo_state *state = screen.machine().driver_data<highvdeo_state>();
 	int x,y,count;
 
 	count = (0/2);
 
-	for(y=0;y<(screen->visible_area().max_y+1);y++)
+	for(y=0;y<(screen.visible_area().max_y+1);y++)
 	{
-		for(x=0;x<(screen->visible_area().max_x+1)/2;x++)
+		for(x=0;x<(screen.visible_area().max_x+1)/2;x++)
 		{
 			UINT32 color;
 
 			color = ((state->m_blit_ram[count]) & 0x00ff)>>0;
 
-			if((x*2)<screen->visible_area().max_x && ((y)+0)<screen->visible_area().max_y)
-				*BITMAP_ADDR32(bitmap, y, (x*2)+0) = screen->machine().pens[color];
+			if(cliprect.contains((x*2)+0, y))
+				bitmap.pix32(y, (x*2)+0) = screen.machine().pens[color];
 
 			color = ((state->m_blit_ram[count]) & 0xff00)>>8;
 
-			if(((x*2)+1)<screen->visible_area().max_x && ((y)+0)<screen->visible_area().max_y)
-				*BITMAP_ADDR32(bitmap, y, (x*2)+1) = screen->machine().pens[color];
+			if(cliprect.contains((x*2)+1, y))
+				bitmap.pix32(y, (x*2)+1) = screen.machine().pens[color];
 
 			count++;
 		}
@@ -141,9 +159,9 @@ static SCREEN_UPDATE(tourvisn)
 }
 
 /*Later HW, RGB565 instead of RAM-based pens (+ ramdac).*/
-static SCREEN_UPDATE(brasil)
+static SCREEN_UPDATE_RGB32(brasil)
 {
-	highvdeo_state *state = screen->machine().driver_data<highvdeo_state>();
+	highvdeo_state *state = screen.machine().driver_data<highvdeo_state>();
 	int x,y,count;
 
 	count = (0/2);
@@ -162,8 +180,8 @@ static SCREEN_UPDATE(brasil)
 			b = (color & 0x001f) << 3;
 			g = (color & 0x07e0) >> 3;
 			r = (color & 0xf800) >> 8;
-			if(x<screen->visible_area().max_x && y<screen->visible_area().max_y)
-				*BITMAP_ADDR32(bitmap, y, x) = b | (g<<8) | (r<<16);
+			if(cliprect.contains(x, y))
+				bitmap.pix32(y, x) = b | (g<<8) | (r<<16);
 
 			count++;
 		}
@@ -174,48 +192,47 @@ static SCREEN_UPDATE(brasil)
 
 
 
-static READ16_HANDLER( read1_r )
+READ16_MEMBER(highvdeo_state::read1_r)
 {
-	return input_port_read(space->machine(), "IN0");
+	return ioport("IN0")->read();
 }
 
-static READ16_HANDLER( read2_r )
+READ16_MEMBER(highvdeo_state::read2_r)
 {
-	return input_port_read(space->machine(), "IN1");
+	return ioport("IN1")->read();
 }
 
-static READ16_HANDLER( read3_r )
+READ16_MEMBER(highvdeo_state::read3_r)
 {
-	return input_port_read(space->machine(), "IN2");
+	return ioport("IN2")->read();
 }
 
-static WRITE16_HANDLER( tv_vcf_paletteram_w )
+WRITE16_MEMBER(highvdeo_state::tv_vcf_paletteram_w)
 {
-	highvdeo_state *state = space->machine().driver_data<highvdeo_state>();
 	switch(offset*2)
 	{
 		case 0:
-			state->m_pal.offs = data;
+			m_pal.offs = data;
 			break;
 		case 2:
-			state->m_pal.offs_internal = 0;
+			m_pal.offs_internal = 0;
 			break;
 		case 4:
-			switch(state->m_pal.offs_internal)
+			switch(m_pal.offs_internal)
 			{
 				case 0:
-					state->m_pal.r = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
-					state->m_pal.offs_internal++;
+					m_pal.r = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
+					m_pal.offs_internal++;
 					break;
 				case 1:
-					state->m_pal.g = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
-					state->m_pal.offs_internal++;
+					m_pal.g = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
+					m_pal.offs_internal++;
 					break;
 				case 2:
-					state->m_pal.b = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
-					palette_set_color(space->machine(), state->m_pal.offs, MAKE_RGB(state->m_pal.r, state->m_pal.g, state->m_pal.b));
-					state->m_pal.offs_internal = 0;
-					state->m_pal.offs++;
+					m_pal.b = ((data & 0x3f) << 2) | ((data & 0x30) >> 4);
+					palette_set_color(machine(), m_pal.offs, MAKE_RGB(m_pal.r, m_pal.g, m_pal.b));
+					m_pal.offs_internal = 0;
+					m_pal.offs++;
 					break;
 			}
 
@@ -223,15 +240,15 @@ static WRITE16_HANDLER( tv_vcf_paletteram_w )
 	}
 }
 
-static WRITE16_HANDLER( tv_vcf_bankselect_w )
+WRITE16_MEMBER(highvdeo_state::tv_vcf_bankselect_w)
 {
 	UINT32 bankaddress;
-	UINT8 *ROM = space->machine().region("user1")->base();
+	UINT8 *ROM = memregion("user1")->base();
 
 	/* bits 0, 1 select the ROM bank */
 	bankaddress = (data & 0x03) * 0x40000;
 
-	memory_set_bankptr(space->machine(), "bank1", &ROM[bankaddress]);
+	membank("bank1")->set_base(&ROM[bankaddress]);
 }
 
 
@@ -255,7 +272,7 @@ static READ16_DEVICE_HANDLER( tv_oki6376_r )
 	return 0xff;
 }
 
-static WRITE16_HANDLER( write1_w )
+WRITE16_MEMBER(highvdeo_state::write1_w)
 {
 /*
     - Lbits -
@@ -278,26 +295,26 @@ static WRITE16_HANDLER( write1_w )
 //  popmessage("%04x %04x",t1,t3);
 }
 
-static ADDRESS_MAP_START( tv_vcf_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( tv_vcf_map, AS_PROGRAM, 16, highvdeo_state )
 	AM_RANGE(0x00000, 0x003ff) AM_RAM /*irq vector area*/
 	AM_RANGE(0x00400, 0x03fff) AM_RAM AM_SHARE("nvram")
-	AM_RANGE(0x40000, 0x4ffff) AM_RAM AM_BASE_MEMBER(highvdeo_state, m_blit_ram) /*blitter ram*/
+	AM_RANGE(0x40000, 0x4ffff) AM_RAM AM_SHARE("blit_ram") /*blitter ram*/
 	AM_RANGE(0x80000, 0xbffff) AM_ROMBANK("bank1")
 	AM_RANGE(0xc0000, 0xfffff) AM_ROM AM_REGION("boot_prg",0)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( tv_vcf_io, AS_IO, 16 )
-	AM_RANGE(0x0000, 0x0001) AM_WRITE( write1_w ) // lamps
-	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE( "oki", tv_oki6376_w )
-	AM_RANGE(0x0008, 0x0009) AM_READ( read1_r )
-	AM_RANGE(0x000a, 0x000b) AM_READ( read2_r )
-	AM_RANGE(0x000c, 0x000d) AM_READ( read3_r )
-	AM_RANGE(0x0010, 0x0015) AM_WRITE( tv_vcf_paletteram_w )
-	AM_RANGE(0x0030, 0x0031) AM_WRITE( tv_vcf_bankselect_w ) AM_DEVREAD( "oki", tv_oki6376_r )
+static ADDRESS_MAP_START( tv_vcf_io, AS_IO, 16, highvdeo_state )
+	AM_RANGE(0x0000, 0x0001) AM_WRITE(write1_w ) // lamps
+	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE_LEGACY("oki", tv_oki6376_w )
+	AM_RANGE(0x0008, 0x0009) AM_READ(read1_r )
+	AM_RANGE(0x000a, 0x000b) AM_READ(read2_r )
+	AM_RANGE(0x000c, 0x000d) AM_READ(read3_r )
+	AM_RANGE(0x0010, 0x0015) AM_WRITE(tv_vcf_paletteram_w )
+	AM_RANGE(0x0030, 0x0031) AM_WRITE(tv_vcf_bankselect_w ) AM_DEVREAD_LEGACY("oki", tv_oki6376_r )
 ADDRESS_MAP_END
 
 
-static READ16_HANDLER( tv_ncf_read2_r )
+READ16_MEMBER(highvdeo_state::tv_ncf_read2_r)
 {
 	static int resetpulse = 0;
 
@@ -305,7 +322,7 @@ static READ16_HANDLER( tv_ncf_read2_r )
 	// machine resets itself.
 	resetpulse ^= 0x40;
 
-	return (input_port_read(space->machine(), "IN1") & 0xbf) | resetpulse;
+	return (ioport("IN1")->read() & 0xbf) | resetpulse;
 }
 
 static WRITE16_DEVICE_HANDLER( tv_ncf_oki6376_w )
@@ -325,65 +342,65 @@ static WRITE16_DEVICE_HANDLER( tv_ncf_oki6376_st_w )
 	}
 }
 
-static ADDRESS_MAP_START( tv_ncf_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( tv_ncf_map, AS_PROGRAM, 16, highvdeo_state )
 	AM_RANGE(0x00000, 0x003ff) AM_RAM /*irq vector area*/
 	AM_RANGE(0x00400, 0x03fff) AM_RAM AM_SHARE("nvram")
-	AM_RANGE(0x20000, 0x2ffff) AM_RAM AM_BASE_MEMBER(highvdeo_state, m_blit_ram) /*blitter ram*/
+	AM_RANGE(0x20000, 0x2ffff) AM_RAM AM_SHARE("blit_ram") /*blitter ram*/
 	AM_RANGE(0x40000, 0xbffff) AM_ROM AM_REGION("user1",0x40000)
 	AM_RANGE(0xc0000, 0xfffff) AM_ROM AM_REGION("boot_prg",0)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( tv_ncf_io, AS_IO, 16 )
-	AM_RANGE(0x0000, 0x0001) AM_WRITE( write1_w ) // lamps
-	AM_RANGE(0x0008, 0x0009) AM_DEVWRITE( "oki", tv_ncf_oki6376_w )
-	AM_RANGE(0x000a, 0x000b) AM_DEVWRITE( "oki", tv_ncf_oki6376_st_w )
-	AM_RANGE(0x000c, 0x000d) AM_READ( read1_r )
-	AM_RANGE(0x0010, 0x0011) AM_READ( tv_ncf_read2_r )
-	AM_RANGE(0x0012, 0x0013) AM_READ( read3_r )
-	AM_RANGE(0x0030, 0x0035) AM_WRITE( tv_vcf_paletteram_w )
+static ADDRESS_MAP_START( tv_ncf_io, AS_IO, 16, highvdeo_state )
+	AM_RANGE(0x0000, 0x0001) AM_WRITE(write1_w ) // lamps
+	AM_RANGE(0x0008, 0x0009) AM_DEVWRITE_LEGACY("oki", tv_ncf_oki6376_w )
+	AM_RANGE(0x000a, 0x000b) AM_DEVWRITE_LEGACY("oki", tv_ncf_oki6376_st_w )
+	AM_RANGE(0x000c, 0x000d) AM_READ(read1_r )
+	AM_RANGE(0x0010, 0x0011) AM_READ(tv_ncf_read2_r )
+	AM_RANGE(0x0012, 0x0013) AM_READ(read3_r )
+	AM_RANGE(0x0030, 0x0035) AM_WRITE(tv_vcf_paletteram_w )
 ADDRESS_MAP_END
 
 
-static WRITE16_HANDLER( tv_tcf_paletteram_w )
+WRITE16_MEMBER(highvdeo_state::tv_tcf_paletteram_w)
 {
 	int r, g, b, color;
 
-	COMBINE_DATA(&space->machine().generic.paletteram.u16[offset]);
+	COMBINE_DATA(&m_generic_paletteram_16[offset]);
 
-	color = space->machine().generic.paletteram.u16[offset];
+	color = m_generic_paletteram_16[offset];
 	r = (color >> 8) & 0xf8;
 	g = (color >> 3) & 0xf8;
 	b = (color << 3) & 0xf8;
 
-	palette_set_color_rgb(space->machine(), offset, r, g, b);
+	palette_set_color_rgb(machine(), offset, r, g, b);
 }
 
-static WRITE16_HANDLER( tv_tcf_bankselect_w )
+WRITE16_MEMBER(highvdeo_state::tv_tcf_bankselect_w)
 {
 	UINT32 bankaddress;
-	UINT8 *ROM = space->machine().region("user1")->base();
+	UINT8 *ROM = memregion("user1")->base();
 
 	/* bits 0, 1, 2 select the ROM bank */
 	bankaddress = (data & 0x07) * 0x40000;
 
-	memory_set_bankptr(space->machine(), "bank1", &ROM[bankaddress]);
+	membank("bank1")->set_base(&ROM[bankaddress]);
 }
 
-static ADDRESS_MAP_START( tv_tcf_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( tv_tcf_map, AS_PROGRAM, 16, highvdeo_state )
 	AM_RANGE(0x00000, 0x003ff) AM_RAM /*irq vector area*/
 	AM_RANGE(0x00400, 0x03fff) AM_RAM AM_SHARE("nvram")
-	AM_RANGE(0x40000, 0x5d4bf) AM_RAM AM_BASE_MEMBER(highvdeo_state, m_blit_ram) /*blitter ram*/
-	AM_RANGE(0x7fe00, 0x7ffff) AM_RAM_WRITE( tv_tcf_paletteram_w ) AM_BASE_GENERIC(paletteram)
+	AM_RANGE(0x40000, 0x5d4bf) AM_RAM AM_SHARE("blit_ram") /*blitter ram*/
+	AM_RANGE(0x7fe00, 0x7ffff) AM_RAM_WRITE(tv_tcf_paletteram_w ) AM_SHARE("paletteram")
 	AM_RANGE(0x80000, 0xbffff) AM_ROMBANK("bank1")
 	AM_RANGE(0xc0000, 0xfffff) AM_ROM AM_REGION("boot_prg",0)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( tv_tcf_io, AS_IO, 16 )
-	AM_RANGE(0x0000, 0x0001) AM_WRITE( write1_w ) // lamps
-	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE( "oki", tv_oki6376_w )
-	AM_RANGE(0x0008, 0x0009) AM_READ( read1_r )
-	AM_RANGE(0x000a, 0x000b) AM_READ( read2_r )
-	AM_RANGE(0x0030, 0x0031) AM_READ( read3_r ) AM_WRITE( tv_tcf_bankselect_w )
+static ADDRESS_MAP_START( tv_tcf_io, AS_IO, 16, highvdeo_state )
+	AM_RANGE(0x0000, 0x0001) AM_WRITE(write1_w ) // lamps
+	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE_LEGACY("oki", tv_oki6376_w )
+	AM_RANGE(0x0008, 0x0009) AM_READ(read1_r )
+	AM_RANGE(0x000a, 0x000b) AM_READ(read2_r )
+	AM_RANGE(0x0030, 0x0031) AM_READ(read3_r ) AM_WRITE(tv_tcf_bankselect_w )
 ADDRESS_MAP_END
 
 /****************************
@@ -392,7 +409,7 @@ ADDRESS_MAP_END
 *
 ****************************/
 
-static READ16_HANDLER( newmcard_status_r )
+READ16_MEMBER(highvdeo_state::newmcard_status_r)
 {
 	switch(offset*2)
 	{
@@ -403,19 +420,17 @@ static READ16_HANDLER( newmcard_status_r )
 }
 
 
-static READ16_HANDLER( newmcard_vblank_r )
+READ16_MEMBER(highvdeo_state::newmcard_vblank_r)
 {
-	highvdeo_state *state = space->machine().driver_data<highvdeo_state>();
-	return state->m_vblank_bit; //0x80
+	return m_vblank_bit; //0x80
 }
 
-static WRITE16_HANDLER( newmcard_vblank_w )
+WRITE16_MEMBER(highvdeo_state::newmcard_vblank_w)
 {
-	highvdeo_state *state = space->machine().driver_data<highvdeo_state>();
-	state->m_vblank_bit = data;
+	m_vblank_bit = data;
 }
 
-static WRITE16_HANDLER( write2_w )
+WRITE16_MEMBER(highvdeo_state::write2_w)
 {
 	int i;
 
@@ -423,31 +438,31 @@ static WRITE16_HANDLER( write2_w )
 
 	for(i=0;i<4;i++)
 	{
-		coin_counter_w(space->machine(), i,data & 0x20);
-		coin_lockout_w(space->machine(), i,~data & 0x08);
+		coin_counter_w(machine(), i,data & 0x20);
+		coin_lockout_w(machine(), i,~data & 0x08);
 	}
 }
 
-static ADDRESS_MAP_START( newmcard_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( newmcard_map, AS_PROGRAM, 16, highvdeo_state )
 	AM_RANGE(0x00000, 0x003ff) AM_RAM /*irq vector area*/
 	AM_RANGE(0x00400, 0x0ffff) AM_RAM AM_SHARE("nvram")
-	AM_RANGE(0x40000, 0x7ffff) AM_RAM AM_BASE_MEMBER(highvdeo_state, m_blit_ram) /*blitter ram*/
+	AM_RANGE(0x40000, 0x7ffff) AM_RAM AM_SHARE("blit_ram") /*blitter ram*/
 	AM_RANGE(0x80000, 0xbffff) AM_ROMBANK("bank1")
 	AM_RANGE(0xc0000, 0xfffff) AM_ROM AM_REGION("boot_prg",0)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( newmcard_io, AS_IO, 16 )
-	AM_RANGE(0x0030, 0x0033) AM_READ( newmcard_status_r )
-	AM_RANGE(0x0030, 0x0031) AM_WRITE( tv_tcf_bankselect_w )
-	AM_RANGE(0x0000, 0x0001) AM_WRITE( write1_w ) // lamps
-	AM_RANGE(0x0002, 0x0003) AM_WRITE( write2_w ) // coin counter & coin lockout
-	AM_RANGE(0x0004, 0x0005) AM_WRITE( newmcard_vblank_w )
-	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE( "oki", tv_oki6376_w )
-	AM_RANGE(0x0008, 0x0009) AM_READ( read1_r )
-	AM_RANGE(0x000a, 0x000b) AM_READ( read2_r )
-	AM_RANGE(0x000c, 0x000d) AM_READ( newmcard_vblank_r )
-	AM_RANGE(0x000e, 0x000f) AM_READ( read3_r )
-	AM_RANGE(0x0010, 0x0015) AM_WRITE( tv_vcf_paletteram_w )
+static ADDRESS_MAP_START( newmcard_io, AS_IO, 16, highvdeo_state )
+	AM_RANGE(0x0030, 0x0033) AM_READ(newmcard_status_r )
+	AM_RANGE(0x0030, 0x0031) AM_WRITE(tv_tcf_bankselect_w )
+	AM_RANGE(0x0000, 0x0001) AM_WRITE(write1_w ) // lamps
+	AM_RANGE(0x0002, 0x0003) AM_WRITE(write2_w ) // coin counter & coin lockout
+	AM_RANGE(0x0004, 0x0005) AM_WRITE(newmcard_vblank_w )
+	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE_LEGACY("oki", tv_oki6376_w )
+	AM_RANGE(0x0008, 0x0009) AM_READ(read1_r )
+	AM_RANGE(0x000a, 0x000b) AM_READ(read2_r )
+	AM_RANGE(0x000c, 0x000d) AM_READ(newmcard_vblank_r )
+	AM_RANGE(0x000e, 0x000f) AM_READ(read3_r )
+	AM_RANGE(0x0010, 0x0015) AM_WRITE(tv_vcf_paletteram_w )
 ADDRESS_MAP_END
 
 /****************************
@@ -457,9 +472,8 @@ ADDRESS_MAP_END
 ****************************/
 
 
-static READ16_HANDLER( brasil_status_r )
+READ16_MEMBER(highvdeo_state::brasil_status_r)
 {
-	highvdeo_state *state = space->machine().driver_data<highvdeo_state>();
 	static UINT16 resetpulse;
 
 	switch(offset*2)
@@ -468,7 +482,7 @@ static READ16_HANDLER( brasil_status_r )
 		resetpulse^=0x10;
 
 		return 3 | resetpulse;
-		case 2: return (state->m_brasil_prot_latch & 3); //and 0x3f
+		case 2: return (m_brasil_prot_latch & 3); //and 0x3f
 	}
 
 	return 0;
@@ -477,43 +491,42 @@ static READ16_HANDLER( brasil_status_r )
 
 
 /*bankaddress might be incorrect.*/
-static WRITE16_HANDLER( brasil_status_w )
+WRITE16_MEMBER(highvdeo_state::brasil_status_w)
 {
-	highvdeo_state *state = space->machine().driver_data<highvdeo_state>();
 	UINT32 bankaddress;
-	UINT8 *ROM = space->machine().region("user1")->base();
+	UINT8 *ROM = memregion("user1")->base();
 
 	switch(data & 3) //data & 7?
 	{
-		case 0: state->m_brasil_prot_latch = 1; break;
-		case 1: state->m_brasil_prot_latch = 0; break;
-		case 2: state->m_brasil_prot_latch = 2; break;
+		case 0: m_brasil_prot_latch = 1; break;
+		case 1: m_brasil_prot_latch = 0; break;
+		case 2: m_brasil_prot_latch = 2; break;
 	}
 
 	bankaddress = (data & 0x07) * 0x40000;
 
-	memory_set_bankptr(space->machine(), "bank1", &ROM[bankaddress]);
+	membank("bank1")->set_base(&ROM[bankaddress]);
 
 //  popmessage("%04x",data);
 }
 
-static ADDRESS_MAP_START( brasil_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( brasil_map, AS_PROGRAM, 16, highvdeo_state )
 	AM_RANGE(0x00000, 0x003ff) AM_RAM /*irq vector area*/
 	AM_RANGE(0x00400, 0x0ffff) AM_RAM AM_SHARE("nvram")
-	AM_RANGE(0x40000, 0x7ffff) AM_RAM AM_BASE_MEMBER(highvdeo_state, m_blit_ram) /*blitter ram*/
+	AM_RANGE(0x40000, 0x7ffff) AM_RAM AM_SHARE("blit_ram") /*blitter ram*/
 	AM_RANGE(0x80000, 0xbffff) AM_ROMBANK("bank1")
 	AM_RANGE(0xc0000, 0xfffff) AM_ROM AM_REGION("boot_prg",0)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( brasil_io, AS_IO, 16 )
-	AM_RANGE(0x0030, 0x0033) AM_READ( brasil_status_r )
-	AM_RANGE(0x0030, 0x0031) AM_WRITE( brasil_status_w )
-	AM_RANGE(0x0000, 0x0001) AM_WRITE( write1_w ) // lamps
-	AM_RANGE(0x0002, 0x0003) AM_WRITE( write2_w ) // coin counter & coin lockout
-	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE( "oki", tv_oki6376_w )
-	AM_RANGE(0x0008, 0x0009) AM_READ( read1_r )
-	AM_RANGE(0x000a, 0x000b) AM_READ( read2_r )
-	AM_RANGE(0x000e, 0x000f) AM_READ( read3_r )
+static ADDRESS_MAP_START( brasil_io, AS_IO, 16, highvdeo_state )
+	AM_RANGE(0x0030, 0x0033) AM_READ(brasil_status_r )
+	AM_RANGE(0x0030, 0x0031) AM_WRITE(brasil_status_w )
+	AM_RANGE(0x0000, 0x0001) AM_WRITE(write1_w ) // lamps
+	AM_RANGE(0x0002, 0x0003) AM_WRITE(write2_w ) // coin counter & coin lockout
+	AM_RANGE(0x0006, 0x0007) AM_DEVWRITE_LEGACY("oki", tv_oki6376_w )
+	AM_RANGE(0x0008, 0x0009) AM_READ(read1_r )
+	AM_RANGE(0x000a, 0x000b) AM_READ(read2_r )
+	AM_RANGE(0x000e, 0x000f) AM_READ(read3_r )
 //  AM_RANGE(0x000e, 0x000f) AM_WRITE
 //  AM_RANGE(0xffa2, 0xffa3) AM_WRITE
 ADDRESS_MAP_END
@@ -912,10 +925,9 @@ static MACHINE_CONFIG_START( tv_vcf, highvdeo_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(400, 300)
 	MCFG_SCREEN_VISIBLE_AREA(0, 320-1, 0, 200-1)
-	MCFG_SCREEN_UPDATE(tourvisn)
+	MCFG_SCREEN_UPDATE_STATIC(tourvisn)
 
 	MCFG_PALETTE_LENGTH(0x100)
 
@@ -979,10 +991,9 @@ static MACHINE_CONFIG_START( brasil, highvdeo_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(400, 300)
 	MCFG_SCREEN_VISIBLE_AREA(0, 400-1, 0, 300-1)
-	MCFG_SCREEN_UPDATE(brasil)
+	MCFG_SCREEN_UPDATE_STATIC(brasil)
 
 	MCFG_PALETTE_LENGTH(0x100)
 
@@ -1162,7 +1173,7 @@ ROM_START( ciclone )
 ROM_END
 
 /*Ciclone*/
-static READ16_HANDLER( ciclone_status_r )
+READ16_MEMBER(highvdeo_state::ciclone_status_r)
 {
 	static UINT16 resetpulse;
 	switch(offset*2)
@@ -1178,7 +1189,8 @@ static READ16_HANDLER( ciclone_status_r )
 
 static DRIVER_INIT( ciclone )
 {
-	machine.device("maincpu")->memory().space(AS_IO)->install_legacy_read_handler(0x0030, 0x0033, FUNC(ciclone_status_r) );
+	highvdeo_state *state = machine.driver_data<highvdeo_state>();
+	machine.device("maincpu")->memory().space(AS_IO)->install_read_handler(0x0030, 0x0033, read16_delegate(FUNC(highvdeo_state::ciclone_status_r), state));
 }
 
 /*
@@ -1233,7 +1245,7 @@ ROM_START( fashion )
 	ROM_LOAD( "sound-fashion-v-1-memory4m.u16", 0x00000, 0x80000, CRC(2927c799) SHA1(f11cad096a23fee10bfdff5bf944c96e30f4a8b8) )
 ROM_END
 
-static WRITE16_HANDLER( fashion_output_w )
+WRITE16_MEMBER(highvdeo_state::fashion_output_w)
 {
 	int i;
 
@@ -1241,14 +1253,15 @@ static WRITE16_HANDLER( fashion_output_w )
 
 	for(i=0;i<4;i++)
 	{
-		coin_counter_w(space->machine(), i,data & 0x20);
-		coin_lockout_w(space->machine(), i,~data & 0x01);
+		coin_counter_w(machine(), i,data & 0x20);
+		coin_lockout_w(machine(), i,~data & 0x01);
 	}
 }
 
 static DRIVER_INIT( fashion )
 {
-	machine.device("maincpu")->memory().space(AS_IO)->install_legacy_write_handler(0x0002, 0x0003, FUNC(fashion_output_w) );
+	highvdeo_state *state = machine.driver_data<highvdeo_state>();
+	machine.device("maincpu")->memory().space(AS_IO)->install_write_handler(0x0002, 0x0003, write16_delegate(FUNC(highvdeo_state::fashion_output_w), state));
 }
 
 GAMEL( 2000, tour4000,  0,      tv_vcf,   tv_vcf,   0,       ROT0,  "High Video", "Tour 4000",         0, layout_fashion )
