@@ -106,14 +106,19 @@ class jollyjgr_state : public driver_device
 {
 public:
 	jollyjgr_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_videoram(*this, "videoram"),
+		m_colorram(*this, "colorram"),
+		m_spriteram(*this, "spriteram"),
+		m_bitmap(*this, "bitmap"),
+		m_bulletram(*this, "bulletram"){ }
 
 	/* memory pointers */
-	UINT8 *  m_videoram;
-	UINT8 *  m_colorram;
-	UINT8 *  m_spriteram;
-	UINT8 *  m_bulletram;
-	UINT8 *  m_bitmap;
+	required_shared_ptr<UINT8> m_videoram;
+	required_shared_ptr<UINT8> m_colorram;
+	required_shared_ptr<UINT8> m_spriteram;
+	required_shared_ptr<UINT8> m_bitmap;
+	optional_shared_ptr<UINT8> m_bulletram;
 
 	/* video-related */
 	tilemap_t  *m_bg_tilemap;
@@ -125,6 +130,10 @@ public:
 	UINT8      m_bitmap_disable;
 	UINT8      m_tilemap_bank;
 	UINT8      m_pri;
+	DECLARE_WRITE8_MEMBER(jollyjgr_videoram_w);
+	DECLARE_WRITE8_MEMBER(jollyjgr_attrram_w);
+	DECLARE_WRITE8_MEMBER(jollyjgr_misc_w);
+	DECLARE_WRITE8_MEMBER(jollyjgr_coin_lookout_w);
 };
 
 
@@ -134,16 +143,14 @@ public:
  *
  *************************************/
 
-static WRITE8_HANDLER( jollyjgr_videoram_w )
+WRITE8_MEMBER(jollyjgr_state::jollyjgr_videoram_w)
 {
-	jollyjgr_state *state = space->machine().driver_data<jollyjgr_state>();
-	state->m_videoram[offset] = data;
-	tilemap_mark_tile_dirty(state->m_bg_tilemap, offset);
+	m_videoram[offset] = data;
+	m_bg_tilemap->mark_tile_dirty(offset);
 }
 
-static WRITE8_HANDLER( jollyjgr_attrram_w )
+WRITE8_MEMBER(jollyjgr_state::jollyjgr_attrram_w)
 {
-	jollyjgr_state *state = space->machine().driver_data<jollyjgr_state>();
 
 	if (offset & 1)
 	{
@@ -151,38 +158,37 @@ static WRITE8_HANDLER( jollyjgr_attrram_w )
 		int i;
 
 		for (i = offset >> 1; i < 0x0400; i += 32)
-			tilemap_mark_tile_dirty(state->m_bg_tilemap, i);
+			m_bg_tilemap->mark_tile_dirty(i);
 	}
 	else
 	{
-		tilemap_set_scrolly(state->m_bg_tilemap, offset >> 1, data);
+		m_bg_tilemap->set_scrolly(offset >> 1, data);
 	}
 
-	state->m_colorram[offset] = data;
+	m_colorram[offset] = data;
 }
 
-static WRITE8_HANDLER( jollyjgr_misc_w )
+WRITE8_MEMBER(jollyjgr_state::jollyjgr_misc_w)
 {
-	jollyjgr_state *state = space->machine().driver_data<jollyjgr_state>();
 
 	// they could be swapped, because it always set "data & 3"
-	state->m_flip_x = data & 1;
-	state->m_flip_y = data & 2;
+	m_flip_x = data & 1;
+	m_flip_y = data & 2;
 
 	// same for these two (used by Frog & Spiders)
-	state->m_bitmap_disable = data & 0x40;
-	state->m_tilemap_bank = data & 0x20;
+	m_bitmap_disable = data & 0x40;
+	m_tilemap_bank = data & 0x20;
 
-	state->m_pri = data & 4;
+	m_pri = data & 4;
 
-	tilemap_set_flip(state->m_bg_tilemap, (state->m_flip_x ? TILEMAP_FLIPX : 0) | (state->m_flip_y ? TILEMAP_FLIPY : 0));
+	m_bg_tilemap->set_flip((m_flip_x ? TILEMAP_FLIPX : 0) | (m_flip_y ? TILEMAP_FLIPY : 0));
 
-	state->m_nmi_enable = data & 0x80;
+	m_nmi_enable = data & 0x80;
 }
 
-static WRITE8_HANDLER( jollyjgr_coin_lookout_w )
+WRITE8_MEMBER(jollyjgr_state::jollyjgr_coin_lookout_w)
 {
-	coin_lockout_global_w(space->machine(), data & 1);
+	coin_lockout_global_w(machine(), data & 1);
 
 	/* bits 4, 5, 6 and 7 are used too */
 }
@@ -193,40 +199,40 @@ static WRITE8_HANDLER( jollyjgr_coin_lookout_w )
  *
  *************************************/
 
-static ADDRESS_MAP_START( jollyjgr_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( jollyjgr_map, AS_PROGRAM, 8, jollyjgr_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM
 	AM_RANGE(0x8000, 0x87ff) AM_RAM
 	AM_RANGE(0x8ff8, 0x8ff8) AM_READ_PORT("DSW1")
 	AM_RANGE(0x8ff9, 0x8ff9) AM_READ_PORT("INPUTS")
-	AM_RANGE(0x8ff8, 0x8ff8) AM_DEVWRITE("aysnd", ay8910_address_w)
-	AM_RANGE(0x8ffa, 0x8ffa) AM_READ_PORT("SYSTEM") AM_DEVWRITE("aysnd", ay8910_data_w)
+	AM_RANGE(0x8ff8, 0x8ff8) AM_DEVWRITE_LEGACY("aysnd", ay8910_address_w)
+	AM_RANGE(0x8ffa, 0x8ffa) AM_READ_PORT("SYSTEM") AM_DEVWRITE_LEGACY("aysnd", ay8910_data_w)
 	AM_RANGE(0x8ffc, 0x8ffc) AM_WRITE(jollyjgr_misc_w)
 	AM_RANGE(0x8ffd, 0x8ffd) AM_WRITE(jollyjgr_coin_lookout_w)
 	AM_RANGE(0x8fff, 0x8fff) AM_READ_PORT("DSW2")
-	AM_RANGE(0x9000, 0x93ff) AM_RAM_WRITE(jollyjgr_videoram_w) AM_BASE_MEMBER(jollyjgr_state, m_videoram)
-	AM_RANGE(0x9800, 0x983f) AM_RAM_WRITE(jollyjgr_attrram_w) AM_BASE_MEMBER(jollyjgr_state, m_colorram)
-	AM_RANGE(0x9840, 0x987f) AM_RAM AM_BASE_MEMBER(jollyjgr_state, m_spriteram)
+	AM_RANGE(0x9000, 0x93ff) AM_RAM_WRITE(jollyjgr_videoram_w) AM_SHARE("videoram")
+	AM_RANGE(0x9800, 0x983f) AM_RAM_WRITE(jollyjgr_attrram_w) AM_SHARE("colorram")
+	AM_RANGE(0x9840, 0x987f) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0x9880, 0x9bff) AM_RAM
-	AM_RANGE(0xa000, 0xffff) AM_RAM AM_BASE_MEMBER(jollyjgr_state, m_bitmap)
+	AM_RANGE(0xa000, 0xffff) AM_RAM AM_SHARE("bitmap")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( fspider_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( fspider_map, AS_PROGRAM, 8, jollyjgr_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM
 	AM_RANGE(0x8000, 0x87ff) AM_RAM
 	AM_RANGE(0x8ff8, 0x8ff8) AM_READ_PORT("DSW1")
 	AM_RANGE(0x8ff9, 0x8ff9) AM_READ_PORT("INPUTS")
-	AM_RANGE(0x8ff8, 0x8ff8) AM_DEVWRITE("aysnd", ay8910_address_w)
-	AM_RANGE(0x8ffa, 0x8ffa) AM_READ_PORT("SYSTEM") AM_DEVWRITE("aysnd", ay8910_data_w)
+	AM_RANGE(0x8ff8, 0x8ff8) AM_DEVWRITE_LEGACY("aysnd", ay8910_address_w)
+	AM_RANGE(0x8ffa, 0x8ffa) AM_READ_PORT("SYSTEM") AM_DEVWRITE_LEGACY("aysnd", ay8910_data_w)
 	AM_RANGE(0x8ffc, 0x8ffc) AM_WRITE(jollyjgr_misc_w)
 	AM_RANGE(0x8ffd, 0x8ffd) AM_WRITE(jollyjgr_coin_lookout_w)
 	AM_RANGE(0x8fff, 0x8fff) AM_READ_PORT("DSW2")
-	AM_RANGE(0x9000, 0x93ff) AM_RAM_WRITE(jollyjgr_videoram_w) AM_BASE_MEMBER(jollyjgr_state, m_videoram)
-	AM_RANGE(0x9800, 0x983f) AM_RAM_WRITE(jollyjgr_attrram_w) AM_BASE_MEMBER(jollyjgr_state, m_colorram)
-	AM_RANGE(0x9840, 0x987f) AM_RAM AM_BASE_MEMBER(jollyjgr_state, m_spriteram)
+	AM_RANGE(0x9000, 0x93ff) AM_RAM_WRITE(jollyjgr_videoram_w) AM_SHARE("videoram")
+	AM_RANGE(0x9800, 0x983f) AM_RAM_WRITE(jollyjgr_attrram_w) AM_SHARE("colorram")
+	AM_RANGE(0x9840, 0x987f) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0x9880, 0x989f) AM_RAM // ?
-	AM_RANGE(0x98a0, 0x98af) AM_RAM AM_BASE_MEMBER(jollyjgr_state, m_bulletram)
+	AM_RANGE(0x98a0, 0x98af) AM_RAM AM_SHARE("bulletram")
 	AM_RANGE(0x98b0, 0x9bff) AM_RAM // ?
-	AM_RANGE(0xa000, 0xffff) AM_RAM AM_BASE_MEMBER(jollyjgr_state, m_bitmap)
+	AM_RANGE(0xa000, 0xffff) AM_RAM AM_SHARE("bitmap")
 ADDRESS_MAP_END
 
 
@@ -396,6 +402,7 @@ INPUT_PORTS_END
 
 static PALETTE_INIT( jollyjgr )
 {
+	const UINT8 *color_prom = machine.root_device().memregion("proms")->base();
 	int i;
 
 	/* tilemap / sprites palette */
@@ -441,11 +448,11 @@ static VIDEO_START( jollyjgr )
 	jollyjgr_state *state = machine.driver_data<jollyjgr_state>();
 	state->m_bg_tilemap = tilemap_create(machine, get_bg_tile_info, tilemap_scan_rows, 8, 8, 32, 32);
 
-	tilemap_set_transparent_pen(state->m_bg_tilemap, 0);
-	tilemap_set_scroll_cols(state->m_bg_tilemap, 32);
+	state->m_bg_tilemap->set_transparent_pen(0);
+	state->m_bg_tilemap->set_scroll_cols(32);
 }
 
-static void draw_bitmap( running_machine &machine, bitmap_t *bitmap )
+static void draw_bitmap( running_machine &machine, bitmap_ind16 &bitmap )
 {
 	jollyjgr_state *state = machine.driver_data<jollyjgr_state>();
 	int x, y, count;
@@ -467,13 +474,13 @@ static void draw_bitmap( running_machine &machine, bitmap_t *bitmap )
 				if(color)
 				{
 					if(state->m_flip_x && state->m_flip_y)
-						*BITMAP_ADDR16(bitmap, y, x * 8 + i) = color + 32;
+						bitmap.pix16(y, x * 8 + i) = color + 32;
 					else if(state->m_flip_x && !state->m_flip_y)
-						*BITMAP_ADDR16(bitmap, 255 - y, x * 8 + i) = color + 32;
+						bitmap.pix16(255 - y, x * 8 + i) = color + 32;
 					else if(!state->m_flip_x && state->m_flip_y)
-						*BITMAP_ADDR16(bitmap, y, 255 - x * 8 - i) = color + 32;
+						bitmap.pix16(y, 255 - x * 8 - i) = color + 32;
 					else
-						*BITMAP_ADDR16(bitmap, 255 - y, 255 - x * 8 - i) = color + 32;
+						bitmap.pix16(255 - y, 255 - x * 8 - i) = color + 32;
 				}
 			}
 
@@ -482,27 +489,27 @@ static void draw_bitmap( running_machine &machine, bitmap_t *bitmap )
 	}
 }
 
-static SCREEN_UPDATE( jollyjgr )
+static SCREEN_UPDATE_IND16( jollyjgr )
 {
-	jollyjgr_state *state = screen->machine().driver_data<jollyjgr_state>();
+	jollyjgr_state *state = screen.machine().driver_data<jollyjgr_state>();
 	UINT8 *spriteram = state->m_spriteram;
 	int offs;
 
-	bitmap_fill(bitmap, cliprect, 32);
+	bitmap.fill(32, cliprect);
 
 	if(state->m_pri) //used in Frog & Spiders level 3
 	{
 		if(!(state->m_bitmap_disable))
-			draw_bitmap(screen->machine(), bitmap);
+			draw_bitmap(screen.machine(), bitmap);
 
-		tilemap_draw(bitmap, cliprect, state->m_bg_tilemap, 0, 0);
+		state->m_bg_tilemap->draw(bitmap, cliprect, 0, 0);
 	}
 	else
 	{
-		tilemap_draw(bitmap, cliprect, state->m_bg_tilemap, 0, 0);
+		state->m_bg_tilemap->draw(bitmap, cliprect, 0, 0);
 
 		if(!(state->m_bitmap_disable))
-			draw_bitmap(screen->machine(), bitmap);
+			draw_bitmap(screen.machine(), bitmap);
 	}
 
 	/* Sprites are the same as in Galaxian */
@@ -529,7 +536,7 @@ static SCREEN_UPDATE( jollyjgr )
 		if (offs < 3 * 4)
 			sy++;
 
-		drawgfx_transpen(bitmap,cliprect,screen->machine().gfx[1],
+		drawgfx_transpen(bitmap,cliprect,screen.machine().gfx[1],
 				code,color,
 				flipx,flipy,
 				sx,sy,0);
@@ -538,12 +545,12 @@ static SCREEN_UPDATE( jollyjgr )
 	return 0;
 }
 
-static SCREEN_UPDATE( fspider )
+static SCREEN_UPDATE_IND16( fspider )
 {
-	jollyjgr_state *state = screen->machine().driver_data<jollyjgr_state>();
+	jollyjgr_state *state = screen.machine().driver_data<jollyjgr_state>();
 
 	// Draw bg and sprites
-	SCREEN_UPDATE_CALL(jollyjgr);
+	SCREEN_UPDATE16_CALL(jollyjgr);
 
 	/* Draw bullets
     16 bytes, 2 bytes per bullet (y,x). 2 player bullets, 6 enemy bullets.
@@ -559,10 +566,10 @@ static SCREEN_UPDATE( fspider )
 		if (state->m_flip_y) sy^=0xff;
 		if (state->m_flip_x) sx+=8;
 
-		if (sy>=cliprect->min_y && sy<=cliprect->max_y)
+		if (sy>=cliprect.min_y && sy<=cliprect.max_y)
 			for (int x=sx-4;x<sx;x++)
-				if (x>=cliprect->min_x && x<=cliprect->max_x)
-					*BITMAP_ADDR16(bitmap,sy,x)=bc;
+				if (x>=cliprect.min_x && x<=cliprect.max_x)
+					bitmap.pix16(sy, x)=bc;
 	}
 
 	return 0;
@@ -655,10 +662,9 @@ static MACHINE_CONFIG_START( jollyjgr, jollyjgr_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(256, 256)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(jollyjgr)
+	MCFG_SCREEN_UPDATE_STATIC(jollyjgr)
 
 	MCFG_GFXDECODE(jollyjgr)
 	MCFG_PALETTE_LENGTH(32+8) /* 32 for tilemap and sprites + 8 for the bitmap */
@@ -679,7 +685,7 @@ static MACHINE_CONFIG_DERIVED( fspider, jollyjgr )
 	MCFG_CPU_PROGRAM_MAP(fspider_map)
 
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(fspider)
+	MCFG_SCREEN_UPDATE_STATIC(fspider)
 
 MACHINE_CONFIG_END
 
