@@ -101,7 +101,6 @@ DSW2 stored @ $f237
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
-#include "deprecat.h"
 #include "cpu/m6805/m6805.h"
 #include "cpu/mcs51/mcs51.h"
 #include "sound/ay8910.h"
@@ -111,10 +110,10 @@ DSW2 stored @ $f237
 
 static SAMPLES_START( pbillian_sh_start )
 {
-	superqix_state *state = device->machine().driver_data<superqix_state>();
-	running_machine &machine = device->machine();
-	UINT8 *src = machine.region("samples")->base();
-	int i, len = machine.region("samples")->bytes();
+	superqix_state *state = device.machine().driver_data<superqix_state>();
+	running_machine &machine = device.machine();
+	UINT8 *src = state->memregion("samples")->base();
+	int i, len = state->memregion("samples")->bytes();
 
 	/* convert 8-bit unsigned samples to 8-bit signed */
 	state->m_samplebuf = auto_alloc_array(machine, INT16, len);
@@ -122,12 +121,11 @@ static SAMPLES_START( pbillian_sh_start )
 		state->m_samplebuf[i] = (INT8)(src[i] ^ 0x80) * 256;
 }
 
-static WRITE8_HANDLER( pbillian_sample_trigger_w )
+WRITE8_MEMBER(superqix_state::pbillian_sample_trigger_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	device_t *samples = space->machine().device("samples");
-	UINT8 *src = space->machine().region("samples")->base();
-	int len = space->machine().region("samples")->bytes();
+	samples_device *samples = machine().device<samples_device>("samples");
+	UINT8 *src = memregion("samples")->base();
+	int len = memregion("samples")->bytes();
 	int start,end;
 
 	start = data << 7;
@@ -136,7 +134,7 @@ static WRITE8_HANDLER( pbillian_sample_trigger_w )
 	while (end < len && src[end] != 0xff)
 		end++;
 
-	sample_start_raw(samples, 0, state->m_samplebuf + start, end - start, 5000, 0); // 5khz ?
+	samples->start_raw(0, m_samplebuf + start, end - start, 5000); // 5khz ?
 }
 
 
@@ -176,7 +174,7 @@ static READ8_DEVICE_HANDLER( in4_mcu_r )
 {
 	superqix_state *state = device->machine().driver_data<superqix_state>();
 //  logerror("%04x: in4_mcu_r\n",cpu_get_pc(&space->device()));
-	return input_port_read(device->machine(), "P2") | (state->m_from_mcu_pending << 6) | (state->m_from_z80_pending << 7);
+	return state->ioport("P2")->read() | (state->m_from_mcu_pending << 6) | (state->m_from_z80_pending << 7);
 }
 
 static READ8_DEVICE_HANDLER( sqix_from_mcu_r )
@@ -194,9 +192,9 @@ static TIMER_CALLBACK( mcu_acknowledge_callback )
 //  logerror("Z80->MCU %02x\n",state->m_from_z80);
 }
 
-static READ8_HANDLER( mcu_acknowledge_r )
+READ8_MEMBER(superqix_state::mcu_acknowledge_r)
 {
-	space->machine().scheduler().synchronize(FUNC(mcu_acknowledge_callback));
+	machine().scheduler().synchronize(FUNC(mcu_acknowledge_callback));
 	return 0;
 }
 
@@ -207,138 +205,132 @@ static WRITE8_DEVICE_HANDLER( sqix_z80_mcu_w )
 	state->m_portb = data;
 }
 
-static WRITE8_HANDLER( bootleg_mcu_p1_w )
+WRITE8_MEMBER(superqix_state::bootleg_mcu_p1_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
 	switch ((data & 0x0e) >> 1)
 	{
 		case 0:
 			// ???
 			break;
 		case 1:
-			coin_counter_w(space->machine(), 0,data & 1);
+			coin_counter_w(machine(), 0,data & 1);
 			break;
 		case 2:
-			coin_counter_w(space->machine(), 1,data & 1);
+			coin_counter_w(machine(), 1,data & 1);
 			break;
 		case 3:
-			coin_lockout_global_w(space->machine(), (data & 1) ^ state->m_invert_coin_lockout);
+			coin_lockout_global_w(machine(), (data & 1) ^ m_invert_coin_lockout);
 			break;
 		case 4:
-			flip_screen_set(space->machine(), data & 1);
+			flip_screen_set(data & 1);
 			break;
 		case 5:
-			state->m_port1 = data;
-			if ((state->m_port1 & 0x80) == 0)
+			m_port1 = data;
+			if ((m_port1 & 0x80) == 0)
 			{
-				state->m_port3_latch = state->m_port3;
+				m_port3_latch = m_port3;
 			}
 			break;
 		case 6:
-			state->m_from_mcu_pending = 0;	// ????
+			m_from_mcu_pending = 0;	// ????
 			break;
 		case 7:
 			if ((data & 1) == 0)
 			{
-//              logerror("%04x: MCU -> Z80 %02x\n",cpu_get_pc(&space->device()),state->m_port3);
-				state->m_from_mcu = state->m_port3_latch;
-				state->m_from_mcu_pending = 1;
-				state->m_from_z80_pending = 0;	// ????
+//              logerror("%04x: MCU -> Z80 %02x\n",cpu_get_pc(&space.device()),m_port3);
+				m_from_mcu = m_port3_latch;
+				m_from_mcu_pending = 1;
+				m_from_z80_pending = 0;	// ????
 			}
 			break;
 	}
 }
 
-static WRITE8_HANDLER( mcu_p3_w )
+WRITE8_MEMBER(superqix_state::mcu_p3_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	state->m_port3 = data;
+	m_port3 = data;
 }
 
-static READ8_HANDLER( bootleg_mcu_p3_r )
+READ8_MEMBER(superqix_state::bootleg_mcu_p3_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	if ((state->m_port1 & 0x10) == 0)
+	if ((m_port1 & 0x10) == 0)
 	{
-		return input_port_read(space->machine(), "DSW1");
+		return ioport("DSW1")->read();
 	}
-	else if ((state->m_port1 & 0x20) == 0)
+	else if ((m_port1 & 0x20) == 0)
 	{
-		return input_port_read(space->machine(), "SYSTEM") | (state->m_from_mcu_pending << 6) | (state->m_from_z80_pending << 7);
+		return sqix_system_status_r(space, 0);
 	}
-	else if ((state->m_port1 & 0x40) == 0)
+	else if ((m_port1 & 0x40) == 0)
 	{
-//      logerror("%04x: read Z80 command %02x\n",cpu_get_pc(&space->device()),state->m_from_z80);
-		state->m_from_z80_pending = 0;
-		return state->m_from_z80;
+//      logerror("%04x: read Z80 command %02x\n",cpu_get_pc(&space.device()),m_from_z80);
+		m_from_z80_pending = 0;
+		return m_from_z80;
 	}
 	return 0;
 }
 
-static READ8_HANDLER( sqixu_mcu_p0_r )
+READ8_MEMBER(superqix_state::sqix_system_status_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	return input_port_read(space->machine(), "SYSTEM") | (state->m_from_mcu_pending << 6) | (state->m_from_z80_pending << 7);
+	return ioport("SYSTEM")->read() | (m_from_mcu_pending << 6) | (m_from_z80_pending << 7);
 }
 
-static WRITE8_HANDLER( sqixu_mcu_p2_w )
+WRITE8_MEMBER(superqix_state::sqixu_mcu_p2_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
 	// bit 0 = unknown (clocked often)
 
 	// bit 1 = coin cointer 1
-	coin_counter_w(space->machine(), 0,data & 2);
+	coin_counter_w(machine(), 0,data & 2);
 
 	// bit 2 = coin counter 2
-	coin_counter_w(space->machine(), 1,data & 4);
+	coin_counter_w(machine(), 1,data & 4);
 
 	// bit 3 = coin lockout
-	coin_lockout_global_w(space->machine(), ~data & 8);
+	coin_lockout_global_w(machine(), ~data & 8);
 
 	// bit 4 = flip screen
-	flip_screen_set(space->machine(), data & 0x10);
+	flip_screen_set(data & 0x10);
 
 	// bit 5 = unknown (set on startup)
 
 	// bit 6 = unknown
 	if ((data & 0x40) == 0)
-		state->m_from_mcu_pending = 0;	// ????
+		m_from_mcu_pending = 0;	// ????
 
 	// bit 7 = clock latch from port 3 to Z80
-	if ((state->m_port2 & 0x80) != 0 && (data & 0x80) == 0)
+	if ((m_port2 & 0x80) != 0 && (data & 0x80) == 0)
 	{
-//      logerror("%04x: MCU -> Z80 %02x\n",cpu_get_pc(&space->device()),state->m_port3);
-		state->m_from_mcu = state->m_port3;
-		state->m_from_mcu_pending = 1;
-		state->m_from_z80_pending = 0;	// ????
+//      logerror("%04x: MCU -> Z80 %02x\n",cpu_get_pc(&space.device()),m_port3);
+		m_from_mcu = m_port3;
+		m_from_mcu_pending = 1;
+		m_from_z80_pending = 0;	// ????
 	}
 
-	state->m_port2 = data;
+	m_port2 = data;
 }
 
-static READ8_HANDLER( sqixu_mcu_p3_r )
+READ8_MEMBER(superqix_state::sqixu_mcu_p3_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-//  logerror("%04x: read Z80 command %02x\n",cpu_get_pc(&space->device()),state->m_from_z80);
-	state->m_from_z80_pending = 0;
-	return state->m_from_z80;
+//  logerror("%04x: read Z80 command %02x\n",cpu_get_pc(&space.device()),m_from_z80);
+	m_from_z80_pending = 0;
+	return m_from_z80;
 }
 
 
-static READ8_HANDLER( nmi_ack_r )
+READ8_MEMBER(superqix_state::nmi_ack_r)
 {
-	cputag_set_input_line(space->machine(), "maincpu", INPUT_LINE_NMI, CLEAR_LINE);
-	return 0;
+	cputag_set_input_line(machine(), "maincpu", INPUT_LINE_NMI, CLEAR_LINE);
+	return sqix_system_status_r(space, 0);
 }
 
 static READ8_DEVICE_HANDLER( bootleg_in0_r )
 {
-	return BITSWAP8(input_port_read(device->machine(), "DSW1"), 0,1,2,3,4,5,6,7);
+	return BITSWAP8(device->machine().root_device().ioport("DSW1")->read(), 0,1,2,3,4,5,6,7);
 }
 
-static WRITE8_HANDLER( bootleg_flipscreen_w )
+WRITE8_MEMBER(superqix_state::bootleg_flipscreen_w)
 {
-	flip_screen_set(space->machine(), ~data & 1);
+	flip_screen_set(~data & 1);
 }
 
 
@@ -360,7 +352,7 @@ static int read_dial(running_machine &machine, int player)
 	int newpos;
 
 	/* get the new position and adjust the result */
-	newpos = input_port_read(machine, player ? "DIAL2" : "DIAL1");
+	newpos = state->ioport(player ? "DIAL2" : "DIAL1")->read();
 	if (newpos != state->m_oldpos[player])
 	{
 		state->m_sign[player] = ((newpos - state->m_oldpos[player]) & 0x80) >> 7;
@@ -412,86 +404,81 @@ static TIMER_CALLBACK( delayed_mcu_z80_w )
  */
 
 
-static READ8_HANDLER( hotsmash_68705_portA_r )
+READ8_MEMBER(superqix_state::hotsmash_68705_portA_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-//  logerror("%04x: 68705 reads port A = %02x\n",cpu_get_pc(&space->device()),state->m_portA_in);
-	return state->m_portA_in;
+//  logerror("%04x: 68705 reads port A = %02x\n",cpu_get_pc(&space.device()),m_portA_in);
+	return m_portA_in;
 }
 
-static WRITE8_HANDLER( hotsmash_68705_portB_w )
+WRITE8_MEMBER(superqix_state::hotsmash_68705_portB_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	state->m_portB_out = data;
+	m_portB_out = data;
 }
 
-static READ8_HANDLER( hotsmash_68705_portC_r )
+READ8_MEMBER(superqix_state::hotsmash_68705_portC_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	return state->m_portC;
+	return m_portC;
 }
 
-static WRITE8_HANDLER( hotsmash_68705_portC_w )
+WRITE8_MEMBER(superqix_state::hotsmash_68705_portC_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	state->m_portC = data;
+	m_portC = data;
 
 	if ((data & 0x08) == 0)
 	{
 		switch (data & 0x07)
 		{
 			case 0x0:	// dsw A
-				state->m_portA_in = input_port_read(space->machine(), "DSW1");
+				m_portA_in = ioport("DSW1")->read();
 				break;
 
 			case 0x1:	// dsw B
-				state->m_portA_in = input_port_read(space->machine(), "DSW2");
+				m_portA_in = ioport("DSW2")->read();
 				break;
 
 			case 0x2:
 				break;
 
 			case 0x3:	// command from Z80
-				state->m_portA_in = state->m_from_z80;
-//              logerror("%04x: z80 reads command %02x\n",cpu_get_pc(&space->device()),state->m_from_z80);
+				m_portA_in = m_from_z80;
+//              logerror("%04x: z80 reads command %02x\n",cpu_get_pc(&space.device()),m_from_z80);
 				break;
 
 			case 0x4:
 				break;
 
 			case 0x5:	// answer to Z80
-				space->machine().scheduler().synchronize(FUNC(delayed_mcu_z80_w), state->m_portB_out);
+				machine().scheduler().synchronize(FUNC(delayed_mcu_z80_w), m_portB_out);
 				break;
 
 			case 0x6:
-				state->m_portA_in = read_dial(space->machine(), 0);
+				m_portA_in = read_dial(machine(), 0);
 				break;
 
 			case 0x7:
-				state->m_portA_in = read_dial(space->machine(), 1);
+				m_portA_in = read_dial(machine(), 1);
 				break;
 		}
 	}
 }
 
-static WRITE8_HANDLER( hotsmash_z80_mcu_w )
+WRITE8_MEMBER(superqix_state::hotsmash_z80_mcu_w)
 {
-	space->machine().scheduler().synchronize(FUNC(delayed_z80_mcu_w), data);
+	machine().scheduler().synchronize(FUNC(delayed_z80_mcu_w), data);
 }
 
-static READ8_HANDLER(hotsmash_from_mcu_r)
+READ8_MEMBER(superqix_state::hotsmash_from_mcu_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-//  logerror("%04x: z80 reads answer %02x\n",cpu_get_pc(&space->device()),state->m_from_mcu);
-	state->m_from_mcu_pending = 0;
-	return state->m_from_mcu;
+//  logerror("%04x: z80 reads answer %02x\n",cpu_get_pc(&space.device()),m_from_mcu);
+	m_from_mcu_pending = 0;
+	return m_from_mcu;
 }
 
 static READ8_DEVICE_HANDLER(hotsmash_ay_port_a_r)
 {
 	superqix_state *state = device->machine().driver_data<superqix_state>();
 //  logerror("%04x: ay_port_a_r and mcu_pending is %d\n",cpu_get_pc(&space->device()),state->m_from_mcu_pending);
-	return input_port_read(device->machine(), "SYSTEM") | ((state->m_from_mcu_pending^1) << 7);
+	return state->ioport("SYSTEM")->read() | 0x40 | ((state->m_from_mcu_pending^1) << 7);
 }
 
 /**************************************************************************
@@ -500,35 +487,33 @@ pbillian MCU simulation
 
 **************************************************************************/
 
-static WRITE8_HANDLER( pbillian_z80_mcu_w )
+WRITE8_MEMBER(superqix_state::pbillian_z80_mcu_w)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
-	state->m_from_z80 = data;
+	m_from_z80 = data;
 }
 
-static READ8_HANDLER(pbillian_from_mcu_r)
+READ8_MEMBER(superqix_state::pbillian_from_mcu_r)
 {
-	superqix_state *state = space->machine().driver_data<superqix_state>();
 
-	switch (state->m_from_z80)
+	switch (m_from_z80)
 	{
 		case 0x01:
 		{
-			UINT8 p = input_port_read(space->machine(), state->m_curr_player ? "PLUNGER2" : "PLUNGER1") & 0xbf;
+			UINT8 p = ioport(m_curr_player ? "PLUNGER2" : "PLUNGER1")->read() & 0xbf;
 			if ((p & 0x3f) == 0) p |= 0x40;
 			return p;
 		}
 
-		case 0x02: return input_port_read(space->machine(), state->m_curr_player ? "DIAL2" : "DIAL1");
+		case 0x02: return ioport(m_curr_player ? "DIAL2" : "DIAL1")->read();
 
-		case 0x04: return input_port_read(space->machine(), "DSW1");
-		case 0x08: return input_port_read(space->machine(), "DSW2");
+		case 0x04: return ioport("DSW1")->read();
+		case 0x08: return ioport("DSW2")->read();
 
-		case 0x80: state->m_curr_player = 0; return 0;
-		case 0x81: state->m_curr_player = 1; return 0;
+		case 0x80: m_curr_player = 0; return 0;
+		case 0x81: m_curr_player = 1; return 0;
 	}
 
-//  logerror("408[%x] r at %x\n",state->m_from_z80,cpu_get_pc(&space->device()));
+//  logerror("408[%x] r at %x\n",m_from_z80,cpu_get_pc(&space.device()));
 	return 0;
 }
 
@@ -536,7 +521,7 @@ static READ8_DEVICE_HANDLER(pbillian_ay_port_a_r)
 {
 //  logerror("%04x: ay_port_a_r\n",cpu_get_pc(&space->device()));
 	/* bits 76------  MCU status bits */
-	return (device->machine().rand() & 0xc0) | input_port_read(device->machine(), "BUTTONS");
+	return (device->machine().rand() & 0xc0) | device->machine().root_device().ioport("BUTTONS")->read();
 }
 
 
@@ -563,7 +548,7 @@ static void machine_init_common(running_machine &machine)
 static MACHINE_START( superqix )
 {
 	/* configure the banks */
-	memory_configure_bank(machine, "bank1", 0, 4, machine.region("maincpu")->base() + 0x10000, 0x4000);
+	machine.root_device().membank("bank1")->configure_entries(0, 4, machine.root_device().memregion("maincpu")->base() + 0x10000, 0x4000);
 
 	machine_init_common(machine);
 }
@@ -571,25 +556,25 @@ static MACHINE_START( superqix )
 static MACHINE_START( pbillian )
 {
 	/* configure the banks */
-	memory_configure_bank(machine, "bank1", 0, 2, machine.region("maincpu")->base() + 0x10000, 0x4000);
+	machine.root_device().membank("bank1")->configure_entries(0, 2, machine.root_device().memregion("maincpu")->base() + 0x10000, 0x4000);
 
 	machine_init_common(machine);
 }
 
 
-static ADDRESS_MAP_START( main_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( main_map, AS_PROGRAM, 8, superqix_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROM
 	AM_RANGE(0x8000, 0xbfff) AM_ROMBANK("bank1")
-	AM_RANGE(0xe000, 0xe0ff) AM_RAM AM_BASE_SIZE_MEMBER(superqix_state, m_spriteram, m_spriteram_size)
+	AM_RANGE(0xe000, 0xe0ff) AM_RAM AM_SHARE("spriteram")
 	AM_RANGE(0xe100, 0xe7ff) AM_RAM
-	AM_RANGE(0xe800, 0xefff) AM_RAM_WRITE(superqix_videoram_w) AM_BASE_MEMBER(superqix_state, m_videoram)
+	AM_RANGE(0xe800, 0xefff) AM_RAM_WRITE(superqix_videoram_w) AM_SHARE("videoram")
 	AM_RANGE(0xf000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pbillian_port_map, AS_IO, 8 )
-	AM_RANGE(0x0000, 0x01ff) AM_RAM_WRITE(paletteram_BBGGRRII_w) AM_BASE_GENERIC(paletteram)
-	AM_RANGE(0x0401, 0x0401) AM_DEVREAD("aysnd", ay8910_r)
-	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE("aysnd", ay8910_data_address_w)
+static ADDRESS_MAP_START( pbillian_port_map, AS_IO, 8, superqix_state )
+	AM_RANGE(0x0000, 0x01ff) AM_RAM_WRITE(paletteram_BBGGRRII_byte_w) AM_SHARE("paletteram")
+	AM_RANGE(0x0401, 0x0401) AM_DEVREAD_LEGACY("aysnd", ay8910_r)
+	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE_LEGACY("aysnd", ay8910_data_address_w)
 	AM_RANGE(0x0408, 0x0408) AM_READ(pbillian_from_mcu_r)
 	AM_RANGE(0x0408, 0x0408) AM_WRITE(pbillian_z80_mcu_w)
 	AM_RANGE(0x0410, 0x0410) AM_WRITE(pbillian_0410_w)
@@ -599,10 +584,10 @@ static ADDRESS_MAP_START( pbillian_port_map, AS_IO, 8 )
 	AM_RANGE(0x041b, 0x041b) AM_READNOP  // input related? but probably not used
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hotsmash_port_map, AS_IO, 8 )
-	AM_RANGE(0x0000, 0x01ff) AM_RAM_WRITE(paletteram_BBGGRRII_w) AM_BASE_GENERIC(paletteram)
-	AM_RANGE(0x0401, 0x0401) AM_DEVREAD("aysnd", ay8910_r)
-	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE("aysnd", ay8910_data_address_w)
+static ADDRESS_MAP_START( hotsmash_port_map, AS_IO, 8, superqix_state )
+	AM_RANGE(0x0000, 0x01ff) AM_RAM_WRITE(paletteram_BBGGRRII_byte_w) AM_SHARE("paletteram")
+	AM_RANGE(0x0401, 0x0401) AM_DEVREAD_LEGACY("aysnd", ay8910_r)
+	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE_LEGACY("aysnd", ay8910_data_address_w)
 	AM_RANGE(0x0408, 0x0408) AM_READ(hotsmash_from_mcu_r)
 	AM_RANGE(0x0408, 0x0408) AM_WRITE(hotsmash_z80_mcu_w)
 	AM_RANGE(0x0410, 0x0410) AM_WRITE(pbillian_0410_w)
@@ -612,34 +597,21 @@ static ADDRESS_MAP_START( hotsmash_port_map, AS_IO, 8 )
 	AM_RANGE(0x041b, 0x041b) AM_READNOP  // input related? but probably not used
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( sqix_port_map, AS_IO, 8 )
-	AM_RANGE(0x0000, 0x00ff) AM_RAM_WRITE(paletteram_BBGGRRII_w) AM_BASE_GENERIC(paletteram)
-	AM_RANGE(0x0401, 0x0401) AM_DEVREAD("ay1", ay8910_r)
-	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE("ay1", ay8910_data_address_w)
-	AM_RANGE(0x0405, 0x0405) AM_DEVREAD("ay2", ay8910_r)
-	AM_RANGE(0x0406, 0x0407) AM_DEVWRITE("ay2", ay8910_data_address_w)
+static ADDRESS_MAP_START( sqix_port_map, AS_IO, 8, superqix_state )
+	AM_RANGE(0x0000, 0x00ff) AM_RAM_WRITE(paletteram_BBGGRRII_byte_w) AM_SHARE("paletteram")
+	AM_RANGE(0x0401, 0x0401) AM_DEVREAD_LEGACY("ay1", ay8910_r)
+	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE_LEGACY("ay1", ay8910_data_address_w)
+	AM_RANGE(0x0405, 0x0405) AM_DEVREAD_LEGACY("ay2", ay8910_r)
+	AM_RANGE(0x0406, 0x0407) AM_DEVWRITE_LEGACY("ay2", ay8910_data_address_w)
 	AM_RANGE(0x0408, 0x0408) AM_READ(mcu_acknowledge_r)
 	AM_RANGE(0x0410, 0x0410) AM_WRITE(superqix_0410_w)	/* ROM bank, NMI enable, tile bank */
 	AM_RANGE(0x0418, 0x0418) AM_READ(nmi_ack_r)
-	AM_RANGE(0x0800, 0x77ff) AM_RAM_WRITE(superqix_bitmapram_w) AM_BASE_MEMBER(superqix_state, m_bitmapram)
-	AM_RANGE(0x8800, 0xf7ff) AM_RAM_WRITE(superqix_bitmapram2_w) AM_BASE_MEMBER(superqix_state, m_bitmapram2)
-ADDRESS_MAP_END
-
-static ADDRESS_MAP_START( bootleg_port_map, AS_IO, 8 )
-	AM_RANGE(0x0000, 0x00ff) AM_RAM_WRITE(paletteram_BBGGRRII_w) AM_BASE_GENERIC(paletteram)
-	AM_RANGE(0x0401, 0x0401) AM_DEVREAD("ay1", ay8910_r)
-	AM_RANGE(0x0402, 0x0403) AM_DEVWRITE("ay1", ay8910_data_address_w)
-	AM_RANGE(0x0405, 0x0405) AM_DEVREAD("ay2", ay8910_r)
-	AM_RANGE(0x0406, 0x0407) AM_DEVWRITE("ay2", ay8910_data_address_w)
-	AM_RANGE(0x0408, 0x0408) AM_WRITE(bootleg_flipscreen_w)
-	AM_RANGE(0x0410, 0x0410) AM_WRITE(superqix_0410_w)	/* ROM bank, NMI enable, tile bank */
-	AM_RANGE(0x0418, 0x0418) AM_READ_PORT("SYSTEM")
-	AM_RANGE(0x0800, 0x77ff) AM_RAM_WRITE(superqix_bitmapram_w) AM_BASE_MEMBER(superqix_state, m_bitmapram)
-	AM_RANGE(0x8800, 0xf7ff) AM_RAM_WRITE(superqix_bitmapram2_w) AM_BASE_MEMBER(superqix_state, m_bitmapram2)
+	AM_RANGE(0x0800, 0x77ff) AM_RAM_WRITE(superqix_bitmapram_w) AM_SHARE("bitmapram")
+	AM_RANGE(0x8800, 0xf7ff) AM_RAM_WRITE(superqix_bitmapram2_w) AM_SHARE("bitmapram2")
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( m68705_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( m68705_map, AS_PROGRAM, 8, superqix_state )
 	ADDRESS_MAP_GLOBAL_MASK(0x7ff)
 	AM_RANGE(0x0000, 0x0000) AM_READ(hotsmash_68705_portA_r)
 	AM_RANGE(0x0001, 0x0001) AM_WRITE(hotsmash_68705_portB_w)
@@ -651,13 +623,13 @@ ADDRESS_MAP_END
 
 /* I8751 memory handlers */
 
-static ADDRESS_MAP_START( bootleg_mcu_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( bootleg_mcu_io_map, AS_IO, 8, superqix_state )
 	AM_RANGE(MCS51_PORT_P1, MCS51_PORT_P1) AM_WRITE(bootleg_mcu_p1_w)
 	AM_RANGE(MCS51_PORT_P3, MCS51_PORT_P3) AM_READWRITE(bootleg_mcu_p3_r, mcu_p3_w)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( sqixu_mcu_io_map, AS_IO, 8 )
-	AM_RANGE(MCS51_PORT_P0, MCS51_PORT_P0) AM_READ(sqixu_mcu_p0_r)
+static ADDRESS_MAP_START( sqixu_mcu_io_map, AS_IO, 8, superqix_state )
+	AM_RANGE(MCS51_PORT_P0, MCS51_PORT_P0) AM_READ(sqix_system_status_r)
 	AM_RANGE(MCS51_PORT_P1, MCS51_PORT_P1) AM_READ_PORT("DSW1")
 	AM_RANGE(MCS51_PORT_P2, MCS51_PORT_P2) AM_WRITE(sqixu_mcu_p2_w)
 	AM_RANGE(MCS51_PORT_P3, MCS51_PORT_P3) AM_READWRITE(sqixu_mcu_p3_r, mcu_p3_w)
@@ -722,7 +694,7 @@ static INPUT_PORTS_START( pbillian )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_START1 )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_COIN2 )
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_COIN1 )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
+	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_SPECIAL )
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_SPECIAL )
 
 	PORT_START("BUTTONS")
@@ -806,7 +778,7 @@ static INPUT_PORTS_START( hotsmash )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_START1 )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_COIN2 )//$49c
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_COIN1 )//$42d
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
+	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_SPECIAL )	// ?
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_SPECIAL )	// mcu status (0 = pending mcu->z80)
 
 	PORT_START("DIAL1")
@@ -872,7 +844,7 @@ static INPUT_PORTS_START( superqix )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_START2 )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_SERVICE1 )	// doesn't work in bootleg
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_UNKNOWN )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
+	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_SPECIAL )	// Z80 status (pending mcu->z80)
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_SPECIAL )	// Z80 status (pending z80->mcu)
 
 	PORT_START("P1")
@@ -882,7 +854,7 @@ static INPUT_PORTS_START( superqix )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT ) PORT_4WAY
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 )
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_UNKNOWN )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_VBLANK )	/* ??? */
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_CUSTOM ) PORT_VBLANK("screen")	/* ??? */
 	PORT_SERVICE( 0x80, IP_ACTIVE_LOW )
 
 	PORT_START("P2")
@@ -1014,20 +986,20 @@ static const ay8910_interface bootleg_ay8910_interface_2 =
 	DEVCB_NULL
 };
 
-
-
-static INTERRUPT_GEN( sqix_interrupt )
+static INTERRUPT_GEN( vblank_irq )
 {
-	/* highly suspicious... */
-	if (cpu_getiloops(device) <= 3)
-		nmi_line_assert(device);
+	superqix_state *state = device->machine().driver_data<superqix_state>();
+
+	if(state->m_nmi_mask)
+		device_set_input_line(device, INPUT_LINE_NMI, PULSE_LINE);
 }
 
-static INTERRUPT_GEN( bootleg_interrupt )
+static INTERRUPT_GEN( sqix_timer_irq )
 {
-	/* highly suspicious... */
-	if (cpu_getiloops(device) <= 3)
-		nmi_line_pulse(device);
+	superqix_state *state = device->machine().driver_data<superqix_state>();
+
+	if (state->m_nmi_mask)
+		device_set_input_line(device, INPUT_LINE_NMI, ASSERT_LINE);
 }
 
 
@@ -1036,7 +1008,7 @@ static MACHINE_CONFIG_START( pbillian, superqix_state )
 	MCFG_CPU_ADD("maincpu", Z80,12000000/2)		 /* 6 MHz */
 	MCFG_CPU_PROGRAM_MAP(main_map)
 	MCFG_CPU_IO_MAP(pbillian_port_map)
-	MCFG_CPU_VBLANK_INT("screen", nmi_line_pulse)
+	MCFG_CPU_VBLANK_INT("screen", vblank_irq)
 
 	MCFG_MACHINE_START(pbillian)
 
@@ -1044,10 +1016,9 @@ static MACHINE_CONFIG_START( pbillian, superqix_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(256, 256)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(pbillian)
+	MCFG_SCREEN_UPDATE_STATIC(pbillian)
 
 	MCFG_GFXDECODE(pbillian)
 	MCFG_PALETTE_LENGTH(512)
@@ -1060,8 +1031,7 @@ static MACHINE_CONFIG_START( pbillian, superqix_state )
 	MCFG_SOUND_CONFIG(pbillian_ay8910_interface)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.30)
 
-	MCFG_SOUND_ADD("samples", SAMPLES, 0)
-	MCFG_SOUND_CONFIG(pbillian_samples_interface)
+	MCFG_SAMPLES_ADD("samples", pbillian_samples_interface)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.50)
 MACHINE_CONFIG_END
 
@@ -1069,7 +1039,7 @@ static MACHINE_CONFIG_START( hotsmash, superqix_state )
 	MCFG_CPU_ADD("maincpu", Z80,12000000/2)		 /* 6 MHz */
 	MCFG_CPU_PROGRAM_MAP(main_map)
 	MCFG_CPU_IO_MAP(hotsmash_port_map)
-	MCFG_CPU_VBLANK_INT("screen", nmi_line_pulse)
+	MCFG_CPU_VBLANK_INT("screen", vblank_irq)
 
 	MCFG_CPU_ADD("mcu", M68705, 4000000) /* ???? */
 	MCFG_CPU_PROGRAM_MAP(m68705_map)
@@ -1080,10 +1050,9 @@ static MACHINE_CONFIG_START( hotsmash, superqix_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(256, 256)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(pbillian)
+	MCFG_SCREEN_UPDATE_STATIC(pbillian)
 
 	MCFG_GFXDECODE(pbillian)
 	MCFG_PALETTE_LENGTH(512)
@@ -1096,8 +1065,7 @@ static MACHINE_CONFIG_START( hotsmash, superqix_state )
 	MCFG_SOUND_CONFIG(hotsmash_ay8910_interface)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.30)
 
-	MCFG_SOUND_ADD("samples", SAMPLES, 0)
-	MCFG_SOUND_CONFIG(pbillian_samples_interface)
+	MCFG_SAMPLES_ADD("samples", pbillian_samples_interface)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.50)
 MACHINE_CONFIG_END
 
@@ -1107,12 +1075,12 @@ static MACHINE_CONFIG_START( sqix, superqix_state )
 	MCFG_CPU_ADD("maincpu", Z80, 12000000/2)	/* 6 MHz */
 	MCFG_CPU_PROGRAM_MAP(main_map)
 	MCFG_CPU_IO_MAP(sqix_port_map)
-	MCFG_CPU_VBLANK_INT_HACK(sqix_interrupt,6)	/* ??? */
+	MCFG_CPU_PERIODIC_INT(sqix_timer_irq, 4*60) /* ??? */
 
 	MCFG_CPU_ADD("mcu", I8751, 12000000/3)	/* ??? */
 	MCFG_CPU_IO_MAP(bootleg_mcu_io_map)
 
-	MCFG_QUANTUM_TIME(attotime::from_hz(30000))
+	MCFG_QUANTUM_PERFECT_CPU("maincpu")
 
 	MCFG_MACHINE_START(superqix)
 
@@ -1120,10 +1088,9 @@ static MACHINE_CONFIG_START( sqix, superqix_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500) /* not accurate */)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(superqix)
+	MCFG_SCREEN_UPDATE_STATIC(superqix)
 
 	MCFG_GFXDECODE(sqix)
 	MCFG_PALETTE_LENGTH(256)
@@ -1155,8 +1122,8 @@ static MACHINE_CONFIG_START( sqixbl, superqix_state )
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu", Z80, 12000000/2)	/* 6 MHz */
 	MCFG_CPU_PROGRAM_MAP(main_map)
-	MCFG_CPU_IO_MAP(bootleg_port_map)
-	MCFG_CPU_VBLANK_INT_HACK(bootleg_interrupt,6)	/* ??? */
+	MCFG_CPU_IO_MAP(sqix_port_map)
+	MCFG_CPU_PERIODIC_INT(sqix_timer_irq, 4*60) /* ??? */
 
 	MCFG_MACHINE_START(superqix)
 
@@ -1164,10 +1131,9 @@ static MACHINE_CONFIG_START( sqixbl, superqix_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500) /* not accurate */)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(superqix)
+	MCFG_SCREEN_UPDATE_STATIC(superqix)
 
 	MCFG_GFXDECODE(sqix)
 	MCFG_PALETTE_LENGTH(256)
@@ -1372,8 +1338,8 @@ static DRIVER_INIT( perestro )
 	int i,j;
 
 	/* decrypt program code; the address lines are shuffled around in a non-trivial way */
-	src = machine.region("maincpu")->base();
-	len = machine.region("maincpu")->bytes();
+	src = machine.root_device().memregion("maincpu")->base();
+	len = machine.root_device().memregion("maincpu")->bytes();
 	for (i = 0;i < len;i += 16)
 	{
 		memcpy(temp,&src[i],16);
@@ -1392,8 +1358,8 @@ static DRIVER_INIT( perestro )
 	}
 
 	/* decrypt gfx ROMs; simple bit swap on the address lines */
-	src = machine.region("gfx1")->base();
-	len = machine.region("gfx1")->bytes();
+	src = machine.root_device().memregion("gfx1")->base();
+	len = machine.root_device().memregion("gfx1")->bytes();
 	for (i = 0;i < len;i += 16)
 	{
 		memcpy(temp,&src[i],16);
@@ -1403,8 +1369,8 @@ static DRIVER_INIT( perestro )
 		}
 	}
 
-	src = machine.region("gfx2")->base();
-	len = machine.region("gfx2")->bytes();
+	src = machine.root_device().memregion("gfx2")->base();
+	len = machine.root_device().memregion("gfx2")->bytes();
 	for (i = 0;i < len;i += 16)
 	{
 		memcpy(temp,&src[i],16);
@@ -1414,8 +1380,8 @@ static DRIVER_INIT( perestro )
 		}
 	}
 
-	src = machine.region("gfx3")->base();
-	len = machine.region("gfx3")->bytes();
+	src = machine.root_device().memregion("gfx3")->base();
+	len = machine.root_device().memregion("gfx3")->bytes();
 	for (i = 0;i < len;i += 16)
 	{
 		memcpy(temp,&src[i],16);
