@@ -1,3 +1,4 @@
+
 #include "emu.h"
 #include "cpu/i8085/i8085.h"
 
@@ -5,18 +6,28 @@ class taito_state : public driver_device
 {
 public:
 	taito_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+		  m_maincpu(*this, "maincpu")
+	{ }
+
+protected:
+
+	// devices
+	required_device<cpu_device> m_maincpu;
+
+	// driver_device overrides
+	virtual void machine_reset();
 };
 
 
-static ADDRESS_MAP_START( taito_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( taito_map, AS_PROGRAM, 8, taito_state )
 	AM_RANGE(0x0000, 0xffff) AM_NOP
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( taito )
 INPUT_PORTS_END
 
-static MACHINE_RESET( taito )
+void taito_state::machine_reset()
 {
 }
 
@@ -28,8 +39,6 @@ static MACHINE_CONFIG_START( taito, taito_state )
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu", I8080, 19000000/9)
 	MCFG_CPU_PROGRAM_MAP(taito_map)
-
-	MCFG_MACHINE_RESET( taito )
 MACHINE_CONFIG_END
 
 /*--------------------------------
@@ -594,41 +603,40 @@ ROM_START(mrblkz80)
 	ROM_RELOAD( 0xf000, 0x1000)
 ROM_END
 
-GAME(198?,	taitest,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Taito Test Fixture",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	cavnegro,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Cavaleiro Negro",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	cavnegro1,	cavnegro,	taito,	taito,	taito,	ROT0,	"Taito",	"Cavaleiro Negro (alternate set 1)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	cavnegro2,	cavnegro,	taito,	taito,	taito,	ROT0,	"Taito",	"Cavaleiro Negro (alternate set 2)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	cosmic,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Cosmic",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1980,	drakor,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Drakor",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	fireact,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Fire Action",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(198?,	fireactd,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Fire Action Deluxe",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	gemini2k,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Gemini 2000",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	gemini2k1,	gemini2k,	taito,	taito,	taito,	ROT0,	"Taito",	"Gemini 2000 (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	gork,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Gork",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	hawkman,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Hawkman",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	hawkman1,	hawkman,	taito,	taito,	taito,	ROT0,	"Taito",	"Hawkman (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	ladylukt,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Lady Luck (Taito)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	lunelle,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Lunelle",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1980,	meteort,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Meteor (Taito)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1984,	mrblack,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Mr. Black",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1985,	mrblack1,	mrblack,	taito,	taito,	taito,	ROT0,	"Taito",	"Mr. Black (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1980,	obaoba,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Oba-Oba",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1980,	obaoba1,	obaoba,		taito,	taito,	taito,	ROT0,	"Taito",	"Oba-Oba (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(198?,	polar,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Polar Explorer",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	rally,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Rally",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	sharkt,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Shark (Taito)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1979,	shock,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Shock",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	snake,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Snake Machine",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1985,	sshuttle,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Space Shuttle (Taito)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1985,	sshuttle1,	sshuttle,	taito,	taito,	taito,	ROT0,	"Taito",	"Space Shuttle (Taito) (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	stest,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Speed Test",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1981,	sureshop,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Sure Shot (Pinball)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	titan,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Titan",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	titan1,		titan,		taito,	taito,	taito,	ROT0,	"Taito",	"Titan (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(198?,	vegast,		ladylukt,	taito,	taito,	taito,	ROT0,	"Taito",	"Vegas (Taito)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(198?,	voleybal,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Voley Ball",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	vortexp,	0,			taito,	taito,	taito,	ROT0,	"Taito",	"Vortex (Pinball)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	zarza,		0,			taito,	taito,	taito,	ROT0,	"Taito",	"Zarza",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1982,	zarza1,		zarza,		taito,	taito,	taito,	ROT0,	"Taito",	"Zarza (alternate set)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(198?,	mrblkz80,	mrblack,	taito,	taito,	taito,	ROT0,	"Taito",	"Mr. Black (Z-80 CPU)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-
+GAME(198?,	taitest,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Taito Test Fixture",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	cavnegro,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Cavaleiro Negro",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	cavnegro1,	cavnegro,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Cavaleiro Negro (alternate set 1)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	cavnegro2,	cavnegro,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Cavaleiro Negro (alternate set 2)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	cosmic,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Cosmic",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1980,	drakor,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Drakor",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	fireact,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Fire Action",				GAME_IS_SKELETON_MECHANICAL)
+GAME(198?,	fireactd,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Fire Action Deluxe",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	gemini2k,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Gemini 2000",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	gemini2k1,	gemini2k,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Gemini 2000 (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	gork,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Gork",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	hawkman,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Hawkman",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	hawkman1,	hawkman,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Hawkman (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	ladylukt,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Lady Luck (Taito)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	lunelle,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Lunelle",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1980,	meteort,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Meteor (Taito)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1984,	mrblack,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Mr. Black",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1985,	mrblack1,	mrblack,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Mr. Black (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1980,	obaoba,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Oba-Oba",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1980,	obaoba1,	obaoba,		taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Oba-Oba (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(198?,	polar,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Polar Explorer",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1980,	rally,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Rally",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	sharkt,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Shark (Taito)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1979,	shock,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Shock",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	snake,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Snake Machine",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1985,	sshuttle,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Space Shuttle (Taito)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1985,	sshuttle1,	sshuttle,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Space Shuttle (Taito) (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	stest,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Speed Test",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1981,	sureshop,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Sure Shot (Pinball)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	titan,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Titan",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	titan1,		titan,		taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Titan (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(198?,	vegast,		ladylukt,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Vegas (Taito)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(198?,	voleybal,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Voley Ball",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	vortexp,	0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Vortex (Pinball)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	zarza,		0,			taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Zarza",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1982,	zarza1,		zarza,		taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Zarza (alternate set)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(198?,	mrblkz80,	mrblack,	taito,	taito,	taito,	ROT0,	"Taito do Brasil",	"Mr. Black (Z-80 CPU)",				GAME_IS_SKELETON_MECHANICAL)
