@@ -80,7 +80,6 @@ TODO:
 
 Wiz:
 - Possible sprite/char priority issues.
-- There is unknown device (Sony CXK5808-55) on the board.
 - And the supplier of the screenshot says there still may be some wrong
   colors. Just before the break on Level 2 there is a cresent moon,
   the background should probably be black.
@@ -173,74 +172,71 @@ Stephh's notes (based on the games Z80 code and some tests) :
 #define STINGER_BOOM_EN2	NODE_04
 
 
-static WRITE8_HANDLER( sound_command_w )
+WRITE8_MEMBER(wiz_state::sound_command_w)
 {
-	wiz_state *state = space->machine().driver_data<wiz_state>();
-	device_t *discrete = space->machine().device("discrete");
+	device_t *discrete = machine().device("discrete");
 
 	switch (offset)
 	{
 		// 0x90 triggers a jump to non-existant address(development system?) and must be filtered
 		case 0x00:
-			if (data != 0x90) soundlatch_w(space, 0, data);
+			if (data != 0x90) soundlatch_byte_w(space, 0, data);
 		break;
 
 		// explosion sound trigger(analog?)
 		case 0x08:
-			discrete_sound_w(discrete, STINGER_BOOM_EN1, state->m_dsc1);
-			discrete_sound_w(discrete, STINGER_BOOM_EN2, state->m_dsc1^=1);
+			discrete_sound_w(discrete, STINGER_BOOM_EN1, m_dsc1);
+			discrete_sound_w(discrete, STINGER_BOOM_EN2, m_dsc1^=1);
 		break;
 
 		// player shot sound trigger(analog?)
 		case 0x0a:
-			discrete_sound_w(discrete, STINGER_SHOT_EN1, state->m_dsc0);
-			discrete_sound_w(discrete, STINGER_SHOT_EN2, state->m_dsc0^=1);
+			discrete_sound_w(discrete, STINGER_SHOT_EN1, m_dsc0);
+			discrete_sound_w(discrete, STINGER_SHOT_EN2, m_dsc0^=1);
 		break;
 	}
 }
 
-static READ8_HANDLER( wiz_protection_r )
+READ8_MEMBER(wiz_state::wiz_protection_r)
 {
-	wiz_state *state = space->machine().driver_data<wiz_state>();
-	switch (state->m_colorram2[0])
+	switch (m_colorram2[0])
 	{
 	case 0x35: return 0x25;	/* FIX: sudden player death + free play afterwards   */
 	case 0x8f: return 0x1f;	/* FIX: early boss appearance with corrupt graphics  */
 	case 0xa0: return 0x00;	/* FIX: executing junk code after defeating the boss */
 	}
 
-	return state->m_colorram2[0];
+	return m_colorram2[0];
 }
 
-static WRITE8_HANDLER( wiz_coin_counter_w )
+WRITE8_MEMBER(wiz_state::wiz_coin_counter_w)
 {
-	coin_counter_w(space->machine(), offset,data);
+	coin_counter_w(machine(), offset,data);
 }
 
-static WRITE8_HANDLER( wiz_nmi_mask_w )
+WRITE8_MEMBER(wiz_state::wiz_main_nmi_mask_w)
 {
-	wiz_state *state = space->machine().driver_data<wiz_state>();
 
-	state->m_nmi_mask = data & 1;
+	m_main_nmi_mask = data & 1;
 }
 
-static ADDRESS_MAP_START( main_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( main_map, AS_PROGRAM, 8, wiz_state )
 	AM_RANGE(0x0000, 0xbfff) AM_ROM
 	AM_RANGE(0xc000, 0xc7ff) AM_RAM
 	AM_RANGE(0xc800, 0xc801) AM_WRITE(wiz_coin_counter_w)
-	AM_RANGE(0xd000, 0xd3ff) AM_BASE_MEMBER(wiz_state, m_videoram2)					/* Fallthrough */
-	AM_RANGE(0xd400, 0xd7ff) AM_BASE_MEMBER(wiz_state, m_colorram2)
-	AM_RANGE(0xd800, 0xd83f) AM_BASE_MEMBER(wiz_state, m_attributesram2)
-	AM_RANGE(0xd840, 0xd85f) AM_BASE_MEMBER(wiz_state, m_spriteram2) AM_SIZE_MEMBER(wiz_state, m_spriteram_size)
+	AM_RANGE(0xd000, 0xd3ff) AM_SHARE("videoram2")					/* Fallthrough */
+	AM_RANGE(0xd400, 0xd7ff) AM_SHARE("colorram2")
+	AM_RANGE(0xd800, 0xd83f) AM_SHARE("attributesram2")
+	AM_RANGE(0xd840, 0xd85f) AM_SHARE("spriteram2")
 	AM_RANGE(0xd000, 0xd85f) AM_RAM
-	AM_RANGE(0xe000, 0xe3ff) AM_BASE_MEMBER(wiz_state, m_videoram)	/* Fallthrough */
+	AM_RANGE(0xe000, 0xe3ff) AM_SHARE("videoram")	/* Fallthrough */
 	AM_RANGE(0xe400, 0xe7ff) AM_RAM
-	AM_RANGE(0xe800, 0xe83f) AM_BASE_MEMBER(wiz_state, m_attributesram)
-	AM_RANGE(0xe840, 0xe85f) AM_BASE_MEMBER(wiz_state, m_spriteram)
+	AM_RANGE(0xe800, 0xe83f) AM_SHARE("attributesram")
+	AM_RANGE(0xe840, 0xe85f) AM_SHARE("spriteram")
 	AM_RANGE(0xe000, 0xe85f) AM_RAM
 	AM_RANGE(0xf000, 0xf000) AM_READ_PORT("DSW0")
-	AM_RANGE(0xf000, 0xf000) AM_RAM AM_BASE_MEMBER(wiz_state, m_sprite_bank)
-	AM_RANGE(0xf001, 0xf001) AM_WRITE(wiz_nmi_mask_w)
+	AM_RANGE(0xf000, 0xf000) AM_RAM AM_SHARE("sprite_bank")
+	AM_RANGE(0xf001, 0xf001) AM_WRITE(wiz_main_nmi_mask_w)
 	AM_RANGE(0xf002, 0xf003) AM_WRITE(wiz_palettebank_w)
 	AM_RANGE(0xf004, 0xf005) AM_WRITE(wiz_char_bank_select_w)
 	AM_RANGE(0xf006, 0xf006) AM_WRITE(wiz_flipx_w)
@@ -254,23 +250,31 @@ static ADDRESS_MAP_START( main_map, AS_PROGRAM, 8 )
 	AM_RANGE(0xf818, 0xf818) AM_WRITE(wiz_bgcolor_w)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( sound_map, AS_PROGRAM, 8 )
+WRITE8_MEMBER(wiz_state::wiz_sound_nmi_mask_w)
+{
+
+	m_sound_nmi_mask = data & 1;
+}
+
+
+/* TODO: clean this up! */
+static ADDRESS_MAP_START( sound_map, AS_PROGRAM, 8, wiz_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROM
 	AM_RANGE(0x2000, 0x23ff) AM_RAM
-	AM_RANGE(0x3000, 0x3000) AM_READWRITE(soundlatch_r,interrupt_enable_w)	/* Stinger/Scion */
-	AM_RANGE(0x4000, 0x4001) AM_DEVWRITE("8910.3", ay8910_address_data_w)
-	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE("8910.1", ay8910_address_data_w)
-	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE("8910.2", ay8910_address_data_w)				/* Wiz only */
-	AM_RANGE(0x7000, 0x7000) AM_READWRITE(soundlatch_r,interrupt_enable_w)	/* Wiz */
+	AM_RANGE(0x3000, 0x3000) AM_READ(soundlatch_byte_r) AM_WRITE(wiz_sound_nmi_mask_w)	/* Stinger/Scion */
+	AM_RANGE(0x4000, 0x4001) AM_DEVWRITE_LEGACY("8910.3", ay8910_address_data_w)
+	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE_LEGACY("8910.1", ay8910_address_data_w)
+	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE_LEGACY("8910.2", ay8910_address_data_w)		/* Wiz only */
+	AM_RANGE(0x7000, 0x7000) AM_READ(soundlatch_byte_r) AM_WRITE(wiz_sound_nmi_mask_w)	/* Wiz */
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( stinger_sound_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( stinger_sound_map, AS_PROGRAM, 8, wiz_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROM
 	AM_RANGE(0x2000, 0x23ff) AM_RAM
-	AM_RANGE(0x3000, 0x3000) AM_READWRITE(soundlatch_r,interrupt_enable_w)	/* Stinger/Scion */
-	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE("8910.1", ay8910_address_data_w)
-	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE("8910.2", ay8910_address_data_w)				/* Wiz only */
-	AM_RANGE(0x7000, 0x7000) AM_READWRITE(soundlatch_r,interrupt_enable_w)	/* Wiz */
+	AM_RANGE(0x3000, 0x3000) AM_READ(soundlatch_byte_r) AM_WRITE(wiz_sound_nmi_mask_w)	/* Stinger/Scion */
+	AM_RANGE(0x5000, 0x5001) AM_DEVWRITE_LEGACY("8910.1", ay8910_address_data_w)
+	AM_RANGE(0x6000, 0x6001) AM_DEVWRITE_LEGACY("8910.2", ay8910_address_data_w)		/* Wiz only */
+	AM_RANGE(0x7000, 0x7000) AM_READ(soundlatch_byte_r) AM_WRITE(wiz_sound_nmi_mask_w)	/* Wiz */
 ADDRESS_MAP_END
 
 
@@ -697,24 +701,33 @@ static MACHINE_RESET( wiz )
 	state->m_dsc0 = state->m_dsc1 = 1;
 }
 
-static INTERRUPT_GEN( wiz_vblank_irq )
+static INTERRUPT_GEN( wiz_vblank_interrupt )
 {
 	wiz_state *state = device->machine().driver_data<wiz_state>();
 
-	if(state->m_nmi_mask & 1)
-		device_set_input_line(device,INPUT_LINE_NMI,PULSE_LINE);
+	if(state->m_main_nmi_mask & 1)
+		device_set_input_line(device, INPUT_LINE_NMI, PULSE_LINE);
 }
 
+static INTERRUPT_GEN( wiz_sound_interrupt )
+{
+	wiz_state *state = device->machine().driver_data<wiz_state>();
+
+	if(state->m_sound_nmi_mask & 1)
+		device_set_input_line(device, INPUT_LINE_NMI, PULSE_LINE);
+}
+
+
 static MACHINE_CONFIG_START( wiz, wiz_state )
 
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu", Z80, 18432000/6)	/* 3.072 MHz ??? */
 	MCFG_CPU_PROGRAM_MAP(main_map)
-	MCFG_CPU_VBLANK_INT("screen", wiz_vblank_irq)
+	MCFG_CPU_VBLANK_INT("screen", wiz_vblank_interrupt)
 
 	MCFG_CPU_ADD("audiocpu", Z80, 14318000/8)	/* ? */
 	MCFG_CPU_PROGRAM_MAP(sound_map)
-	MCFG_CPU_PERIODIC_INT(nmi_line_pulse,4*60)	/* ??? */
+	MCFG_CPU_PERIODIC_INT(wiz_sound_interrupt,4*60)	/* ??? */
 
 	MCFG_MACHINE_RESET( wiz )
 
@@ -722,10 +735,9 @@ static MACHINE_CONFIG_START( wiz, wiz_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500) /* not accurate */	/* frames per second, vblank duration */)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 32*8-1, 2*8, 30*8-1)
-	MCFG_SCREEN_UPDATE(wiz)
+	MCFG_SCREEN_UPDATE_STATIC(wiz)
 
 	MCFG_GFXDECODE(wiz)
 	MCFG_PALETTE_LENGTH(256)
@@ -757,20 +769,14 @@ static MACHINE_CONFIG_DERIVED( stinger, wiz )
 	/* video hardware */
 	MCFG_GFXDECODE(stinger)
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(stinger)
+	MCFG_SCREEN_UPDATE_STATIC(stinger)
 
 	/* sound hardware */
-	MCFG_SOUND_MODIFY("8910.1")
-	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.12)
-
-	MCFG_SOUND_MODIFY("8910.2")
-	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.12)
-
 	MCFG_DEVICE_REMOVE("8910.3")
 
 	MCFG_SOUND_ADD("discrete", DISCRETE, 0)
 	MCFG_SOUND_CONFIG_DISCRETE(stinger)
-	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
+	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.5)
 MACHINE_CONFIG_END
 
 
@@ -792,7 +798,7 @@ static MACHINE_CONFIG_DERIVED( kungfut, wiz )
 	/* video hardware */
 	MCFG_GFXDECODE(stinger)
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(kungfut)
+	MCFG_SCREEN_UPDATE_STATIC(kungfut)
 
 MACHINE_CONFIG_END
 
@@ -1058,8 +1064,8 @@ static DRIVER_INIT( stinger )
 		{ 5,7,3, 0x28 }
 	};
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
-	UINT8 *rom = machine.region("maincpu")->base();
-	int size = machine.region("maincpu")->bytes();
+	UINT8 *rom = machine.root_device().memregion("maincpu")->base();
+	int size = machine.root_device().memregion("maincpu")->bytes();
 	UINT8 *decrypt = auto_alloc_array(machine, UINT8, size);
 	int A;
 	const UINT8 *tbl;
@@ -1100,7 +1106,8 @@ static DRIVER_INIT( scion )
 
 static DRIVER_INIT( wiz )
 {
-	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_legacy_read_handler(0xd400, 0xd400, FUNC(wiz_protection_r));
+	wiz_state *state = machine.driver_data<wiz_state>();
+	machine.device("maincpu")->memory().space(AS_PROGRAM)->install_read_handler(0xd400, 0xd400, read8_delegate(FUNC(wiz_state::wiz_protection_r),state));
 }
 
 
@@ -1108,8 +1115,8 @@ GAME( 1983, stinger,  0,       stinger, stinger,  stinger, ROT90,  "Seibu Denshi
 GAME( 1983, stinger2, stinger, stinger, stinger2, stinger, ROT90,  "Seibu Denshi", "Stinger (prototype?)", GAME_IMPERFECT_SOUND )
 GAME( 1984, scion,    0,       scion,   scion,    scion,   ROT0,   "Seibu Denshi", "Scion", GAME_IMPERFECT_SOUND | GAME_IMPERFECT_COLORS )
 GAME( 1984, scionc,   scion,   scion,   scion,    scion,   ROT0,   "Seibu Denshi (Cinematronics license)", "Scion (Cinematronics)", GAME_IMPERFECT_SOUND | GAME_IMPERFECT_COLORS )
-GAME( 1984, kungfut,  0,       kungfut, kungfut,  0,       ROT0,   "Seibu Kaihatsu", "Kung-Fu Taikun", 0 )
-GAME( 1984, kungfuta, kungfut, kungfut, kungfut,  0,       ROT0,   "Seibu Kaihatsu", "Kung-Fu Taikun (alt)" , 0) /* board was a bootleg but set might still be original */
+GAME( 1984, kungfut,  0,       kungfut, kungfut,  0,       ROT0,   "Seibu Kaihatsu", "Kung-Fu Taikun (set 1)", 0 )
+GAME( 1984, kungfuta, kungfut, kungfut, kungfut,  0,       ROT0,   "Seibu Kaihatsu", "Kung-Fu Taikun (set 2)" , 0) /* board was a bootleg but set might still be original */
 GAME( 1985, wiz,      0,       wiz,     wiz,      wiz,     ROT270, "Seibu Kaihatsu", "Wiz", 0 )
 GAME( 1985, wizt,     wiz,     wiz,     wiz,      wiz,     ROT270, "Seibu Kaihatsu (Taito license)", "Wiz (Taito, set 1)", 0 )
 GAME( 1985, wizta,    wiz,     wiz,     wiz,      wiz,     ROT270, "Seibu Kaihatsu (Taito license)", "Wiz (Taito, set 2)", 0 )
