@@ -1,3 +1,4 @@
+
 #include "emu.h"
 #include "cpu/m6809/m6809.h"
 
@@ -5,18 +6,28 @@ class wpc_dot_state : public driver_device
 {
 public:
 	wpc_dot_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+		  m_maincpu(*this, "maincpu")
+	{ }
+
+protected:
+
+	// devices
+	required_device<cpu_device> m_maincpu;
+
+	// driver_device overrides
+	virtual void machine_reset();
 };
 
 
-static ADDRESS_MAP_START( wpc_dot_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( wpc_dot_map, AS_PROGRAM, 8, wpc_dot_state )
 	AM_RANGE(0x0000, 0xffff) AM_NOP
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( wpc_dot )
 INPUT_PORTS_END
 
-static MACHINE_RESET( wpc_dot )
+void wpc_dot_state::machine_reset()
 {
 }
 
@@ -28,8 +39,6 @@ static MACHINE_CONFIG_START( wpc_dot, wpc_dot_state )
 	/* basic machine hardware */
 	MCFG_CPU_ADD("maincpu", M6809, 2000000)
 	MCFG_CPU_PROGRAM_MAP(wpc_dot_map)
-
-	MCFG_MACHINE_RESET( wpc_dot )
 MACHINE_CONFIG_END
 
 /*-----------------
@@ -55,6 +64,7 @@ ROM_START(gi_l9)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(gi_l3)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -75,6 +85,7 @@ ROM_START(gi_l3)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(gi_l4)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -95,6 +106,7 @@ ROM_START(gi_l4)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(gi_l6)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -178,6 +190,7 @@ ROM_START(pz_f4)
 	ROM_LOAD("pz_u14.l1", 0x100000, 0x40000, CRC(4d8897ce) SHA1(7a4ac9e849dae93078ddd60adbd34f3930e4cd46))
 	ROM_RELOAD( 0x100000 + 0x40000, 0x40000)
 ROM_END
+
 ROM_START(pz_l1)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -196,6 +209,7 @@ ROM_START(pz_l1)
 	ROM_LOAD("pz_u14.l1", 0x100000, 0x40000, CRC(4d8897ce) SHA1(7a4ac9e849dae93078ddd60adbd34f3930e4cd46))
 	ROM_RELOAD( 0x100000 + 0x40000, 0x40000)
 ROM_END
+
 ROM_START(pz_l2)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -214,6 +228,7 @@ ROM_START(pz_l2)
 	ROM_LOAD("pz_u14.l1", 0x100000, 0x40000, CRC(4d8897ce) SHA1(7a4ac9e849dae93078ddd60adbd34f3930e4cd46))
 	ROM_RELOAD( 0x100000 + 0x40000, 0x40000)
 ROM_END
+
 ROM_START(pz_l3)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -280,6 +295,7 @@ ROM_START(t2_l8)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(t2_l6)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -300,6 +316,7 @@ ROM_START(t2_l6)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(t2_p2f)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -320,6 +337,7 @@ ROM_START(t2_p2f)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(t2_l4)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -340,6 +358,7 @@ ROM_START(t2_l4)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(t2_l3)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -360,6 +379,7 @@ ROM_START(t2_l3)
 	ROM_RELOAD( 0x100000 + 0x40000, 0x20000)
 	ROM_RELOAD( 0x100000 + 0x60000, 0x20000)
 ROM_END
+
 ROM_START(t2_l2)
 	ROM_REGION(0x10000, "maincpu", ROMREGION_ERASEFF)
 	ROM_REGION(0x2000, "user1", ROMREGION_ERASEFF)
@@ -392,21 +412,21 @@ ROM_START(tfdmd_l3)
 ROM_END
 
 
-GAME(1991,	tfdmd_l3,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"WPC Test Fixture: DMD (L-3)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	gi_l9,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Gilligan's Island (L-9)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	gi_l3,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Gilligan's Island (L-3)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	gi_l4,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Gilligan's Island (L-4)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	gi_l6,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Gilligan's Island (L-6)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1992,	hshot_p8,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Midway",				"Hot Shot Basketball (P-8)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	hurr_l2,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Hurricane (L-2)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	pz_f4,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Party Zone (F-4)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	pz_l1,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Party Zone (L-1)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	pz_l2,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Party Zone (L-2)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	pz_l3,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",				"Party Zone (L-3)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	sf_l1,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Slugfest (L-1)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_l8,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (L-8)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_l6,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (L-6)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_p2f,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (P-2F) Profanity",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_l4,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (L-4)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_l3,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (L-3)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
-GAME(1991,	t2_l2,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",				"Terminator 2: Judgment Day (L-2)",				GAME_NOT_WORKING | GAME_NO_SOUND | GAME_MECHANICAL)
+GAME(1991,	tfdmd_l3,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"WPC Test Fixture: DMD (L-3)",					GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	gi_l9,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"Gilligan's Island (L-9)",						GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	gi_l3,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"Gilligan's Island (L-3)",						GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	gi_l4,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"Gilligan's Island (L-4)",						GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	gi_l6,		gi_l9,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"Gilligan's Island (L-6)",						GAME_IS_SKELETON_MECHANICAL)
+GAME(1992,	hshot_p8,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Midway",		"Hot Shot Basketball (P-8)",					GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	hurr_l2,	0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Hurricane (L-2)",								GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	pz_f4,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"The Party Zone (F-4)",							GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	pz_l1,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"The Party Zone (L-1)",							GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	pz_l2,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"The Party Zone (L-2)",							GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	pz_l3,		pz_f4,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Bally",		"The Party Zone (L-3)",							GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	sf_l1,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Slugfest (L-1)",								GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_l8,		0,		wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (L-8)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_l6,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (L-6)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_p2f,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (P-2F) Profanity",	GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_l4,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (L-4)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_l3,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (L-3)",				GAME_IS_SKELETON_MECHANICAL)
+GAME(1991,	t2_l2,		t2_l8,	wpc_dot,	wpc_dot,	wpc_dot,	ROT0,	"Williams",		"Terminator 2: Judgment Day (L-2)",				GAME_IS_SKELETON_MECHANICAL)
