@@ -9,21 +9,27 @@ class atetris_state : public driver_device
 public:
 	atetris_state(const machine_config &mconfig, device_type type, const char *tag)
 		: driver_device(mconfig, type, tag),
-		  m_nvram(*this, "nvram") { }
+		  m_nvram(*this, "nvram") ,
+		m_videoram(*this, "videoram"){ }
 
 	required_shared_ptr<UINT8>	m_nvram;
-	UINT8 *m_videoram;
+	required_shared_ptr<UINT8> m_videoram;
 	UINT8 *m_slapstic_source;
 	UINT8 *m_slapstic_base;
 	UINT8 m_current_bank;
 	UINT8 m_nvram_write_enable;
 	emu_timer *m_interrupt_timer;
 	tilemap_t *m_bg_tilemap;
+	DECLARE_WRITE8_MEMBER(irq_ack_w);
+	DECLARE_READ8_MEMBER(atetris_slapstic_r);
+	DECLARE_WRITE8_MEMBER(coincount_w);
+	DECLARE_WRITE8_MEMBER(nvram_w);
+	DECLARE_WRITE8_MEMBER(nvram_enable_w);
+	DECLARE_WRITE8_MEMBER(atetris_videoram_w);
 };
 
 /*----------- defined in video/atetris.c -----------*/
 
 VIDEO_START( atetris );
-SCREEN_UPDATE( atetris );
+SCREEN_UPDATE_IND16( atetris );
 
-WRITE8_HANDLER( atetris_videoram_w );
