@@ -8,12 +8,13 @@ class homerun_state : public driver_device
 {
 public:
 	homerun_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_videoram(*this, "videoram"),
+		m_spriteram(*this, "spriteram"){ }
 
 	/* memory pointers */
-	UINT8 *    m_videoram;
-	UINT8 *    m_spriteram;
-	size_t     m_spriteram_size;
+	required_shared_ptr<UINT8> m_videoram;
+	required_shared_ptr<UINT8> m_spriteram;
 
 	/* video-related */
 	tilemap_t    *m_tilemap;
@@ -25,14 +26,15 @@ public:
 	int        m_xpc;
 	int        m_gc_up;
 	int        m_gc_down;
+	DECLARE_WRITE8_MEMBER(homerun_videoram_w);
+	DECLARE_WRITE8_MEMBER(homerun_color_w);
+	DECLARE_CUSTOM_INPUT_MEMBER(homerun_40_r);
 };
 
 
 /*----------- defined in video/homerun.c -----------*/
 
-WRITE8_HANDLER( homerun_videoram_w );
-WRITE8_HANDLER( homerun_color_w );
 WRITE8_DEVICE_HANDLER( homerun_banking_w );
 
 VIDEO_START(homerun);
-SCREEN_UPDATE(homerun);
+SCREEN_UPDATE_IND16(homerun);
