@@ -8,14 +8,16 @@ class jack_state : public driver_device
 {
 public:
 	jack_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_spriteram(*this, "spriteram"),
+		m_videoram(*this, "videoram"),
+		m_colorram(*this, "colorram"){ }
 
 	/* memory pointers */
-	UINT8 *    m_videoram;
-	UINT8 *    m_colorram;
-	UINT8 *    m_spriteram;
+	required_shared_ptr<UINT8> m_spriteram;
+	required_shared_ptr<UINT8> m_videoram;
+	required_shared_ptr<UINT8> m_colorram;
 //  UINT8 *    paletteram;  // currently this uses generic palette handling
-	size_t     m_spriteram_size;
 
 	/* video-related */
 	tilemap_t    *m_bg_tilemap;
@@ -30,20 +32,24 @@ public:
 
 	/* devices */
 	cpu_device *m_audiocpu;
+	DECLARE_WRITE8_MEMBER(jack_sh_command_w);
+	DECLARE_WRITE8_MEMBER(joinem_misc_w);
+	DECLARE_READ8_MEMBER(striv_question_r);
+	DECLARE_WRITE8_MEMBER(jack_videoram_w);
+	DECLARE_WRITE8_MEMBER(jack_colorram_w);
+	DECLARE_WRITE8_MEMBER(jack_paletteram_w);
+	DECLARE_READ8_MEMBER(jack_flipscreen_r);
+	DECLARE_WRITE8_MEMBER(jack_flipscreen_w);
+	DECLARE_CUSTOM_INPUT_MEMBER(sound_check_r);
 };
 
 
 /*----------- defined in video/jack.c -----------*/
 
-WRITE8_HANDLER( jack_videoram_w );
-WRITE8_HANDLER( jack_colorram_w );
-WRITE8_HANDLER( jack_paletteram_w );
-READ8_HANDLER( jack_flipscreen_r );
-WRITE8_HANDLER( jack_flipscreen_w );
 
 VIDEO_START( jack );
-SCREEN_UPDATE( jack );
+SCREEN_UPDATE_IND16( jack );
 
 PALETTE_INIT( joinem );
 VIDEO_START( joinem );
-SCREEN_UPDATE( joinem );
+SCREEN_UPDATE_IND16( joinem );
