@@ -19,6 +19,7 @@ public:
 
 	/* misc */
 	int        m_nmi_enable;
+	UINT8      m_sound_irq_mask;
 
 	/* devices */
 	device_t *m_maincpu;
@@ -27,6 +28,15 @@ public:
 	device_t *m_k007232;
 	device_t *m_k052109;
 	device_t *m_k051960;
+	DECLARE_WRITE8_MEMBER(dv_nmienable_w);
+	DECLARE_WRITE8_MEMBER(mainevt_bankswitch_w);
+	DECLARE_WRITE8_MEMBER(mainevt_coin_w);
+	DECLARE_WRITE8_MEMBER(mainevt_sh_irqtrigger_w);
+	DECLARE_WRITE8_MEMBER(mainevt_sh_irqcontrol_w);
+	DECLARE_WRITE8_MEMBER(devstor_sh_irqcontrol_w);
+	DECLARE_WRITE8_MEMBER(mainevt_sh_bankswitch_w);
+	DECLARE_READ8_MEMBER(k052109_051960_r);
+	DECLARE_WRITE8_MEMBER(k052109_051960_w);
 };
 
 /*----------- defined in video/mainevt.c -----------*/
@@ -39,5 +49,5 @@ extern void dv_sprite_callback(running_machine &machine, int *code,int *color,in
 VIDEO_START( mainevt );
 VIDEO_START( dv );
 
-SCREEN_UPDATE( mainevt );
-SCREEN_UPDATE( dv );
+SCREEN_UPDATE_IND16( mainevt );
+SCREEN_UPDATE_IND16( dv );
