@@ -8,10 +8,11 @@ class parodius_state : public driver_device
 {
 public:
 	parodius_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_ram(*this, "ram"){ }
 
 	/* memory pointers */
-	UINT8 *    m_ram;
+	required_shared_ptr<UINT8> m_ram;
 //  UINT8 *    m_paletteram;    // currently this uses generic palette handling
 
 	/* video-related */
@@ -30,6 +31,14 @@ public:
 	device_t *m_k052109;
 	device_t *m_k053245;
 	device_t *m_k053251;
+	DECLARE_READ8_MEMBER(bankedram_r);
+	DECLARE_WRITE8_MEMBER(bankedram_w);
+	DECLARE_READ8_MEMBER(parodius_052109_053245_r);
+	DECLARE_WRITE8_MEMBER(parodius_052109_053245_w);
+	DECLARE_WRITE8_MEMBER(parodius_videobank_w);
+	DECLARE_WRITE8_MEMBER(parodius_3fc0_w);
+	DECLARE_WRITE8_MEMBER(parodius_sh_irqtrigger_w);
+	DECLARE_WRITE8_MEMBER(sound_arm_nmi_w);
 };
 
 /*----------- defined in video/parodius.c -----------*/
@@ -37,4 +46,4 @@ public:
 extern void parodius_tile_callback(running_machine &machine, int layer,int bank,int *code,int *color,int *flags,int *priority);
 extern void parodius_sprite_callback(running_machine &machine, int *code,int *color,int *priority_mask);
 
-SCREEN_UPDATE( parodius );
+SCREEN_UPDATE_IND16( parodius );
