@@ -8,10 +8,11 @@ class thunderx_state : public driver_device
 {
 public:
 	thunderx_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_ram(*this, "ram"){ }
 
 	/* memory pointers */
-	UINT8 *    m_ram;
+	required_shared_ptr<UINT8> m_ram;
 	UINT8      m_pmcram[0x800];
 //  UINT8 *    m_paletteram;    // currently this uses generic palette handling
 
@@ -32,6 +33,17 @@ public:
 	device_t *m_k007232;
 	device_t *m_k052109;
 	device_t *m_k051960;
+	DECLARE_READ8_MEMBER(scontra_bankedram_r);
+	DECLARE_WRITE8_MEMBER(scontra_bankedram_w);
+	DECLARE_READ8_MEMBER(thunderx_bankedram_r);
+	DECLARE_WRITE8_MEMBER(thunderx_bankedram_w);
+	DECLARE_READ8_MEMBER(thunderx_1f98_r);
+	DECLARE_WRITE8_MEMBER(thunderx_1f98_w);
+	DECLARE_WRITE8_MEMBER(scontra_bankswitch_w);
+	DECLARE_WRITE8_MEMBER(thunderx_videobank_w);
+	DECLARE_WRITE8_MEMBER(thunderx_sh_irqtrigger_w);
+	DECLARE_READ8_MEMBER(k052109_051960_r);
+	DECLARE_WRITE8_MEMBER(k052109_051960_w);
 };
 
 
@@ -41,4 +53,4 @@ extern void thunderx_tile_callback(running_machine &machine, int layer,int bank,
 extern void thunderx_sprite_callback(running_machine &machine, int *code,int *color,int *priority_mask,int *shadow);
 
 VIDEO_START( scontra );
-SCREEN_UPDATE( scontra );
+SCREEN_UPDATE_IND16( scontra );
