@@ -552,12 +552,12 @@ static WRITE8_HANDLER( key_type3_w )
 *                                                                              *
 *******************************************************************************/
 
-WRITE8_HANDLER( namcos1_sound_bankswitch_w )
+WRITE8_MEMBER(namcos1_state::namcos1_sound_bankswitch_w)
 {
-	UINT8 *rom = space->machine().region("audiocpu")->base() + 0xc000;
+	UINT8 *rom = memregion("audiocpu")->base() + 0xc000;
 
 	int bank = (data & 0x70) >> 4;
-	memory_set_bankptr(space->machine(), "bank17",rom + 0x4000 * bank);
+	membank("bank17")->set_base(rom + 0x4000 * bank);
 }
 
 
@@ -569,36 +569,34 @@ WRITE8_HANDLER( namcos1_sound_bankswitch_w )
 *******************************************************************************/
 
 
-WRITE8_HANDLER( namcos1_cpu_control_w )
+WRITE8_MEMBER(namcos1_state::namcos1_cpu_control_w)
 {
-	namcos1_state *state = space->machine().driver_data<namcos1_state>();
-//  logerror("reset control pc=%04x %02x\n",cpu_get_pc(&space->device()),data);
-	if ((data & 1) ^ state->m_reset)
+//  logerror("reset control pc=%04x %02x\n",cpu_get_pc(&space.device()),data);
+	if ((data & 1) ^ m_reset)
 	{
-		state->m_mcu_patch_data = 0;
-		state->m_reset = data & 1;
+		m_mcu_patch_data = 0;
+		m_reset = data & 1;
 	}
 
-	cputag_set_input_line(space->machine(), "sub", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
-	cputag_set_input_line(space->machine(), "audiocpu", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
-	cputag_set_input_line(space->machine(), "mcu", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
+	cputag_set_input_line(machine(), "sub", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
+	cputag_set_input_line(machine(), "audiocpu", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
+	cputag_set_input_line(machine(), "mcu", INPUT_LINE_RESET, (data & 1) ? CLEAR_LINE : ASSERT_LINE);
 }
 
 
 
-WRITE8_HANDLER( namcos1_watchdog_w )
+WRITE8_MEMBER(namcos1_state::namcos1_watchdog_w)
 {
-	namcos1_state *state = space->machine().driver_data<namcos1_state>();
-	if (&space->device() == space->machine().device("maincpu"))
-		state->m_wdog |= 1;
-	else if (&space->device() == space->machine().device("sub"))
-		state->m_wdog |= 2;
-	else if (&space->device() == space->machine().device("audiocpu"))
-		state->m_wdog |= 4;
+	if (&space.device() == machine().device("maincpu"))
+		m_wdog |= 1;
+	else if (&space.device() == machine().device("sub"))
+		m_wdog |= 2;
+	else if (&space.device() == machine().device("audiocpu"))
+		m_wdog |= 4;
 
-	if (state->m_wdog == 7 || !state->m_reset)
+	if (m_wdog == 7 || !m_reset)
 	{
-		state->m_wdog = 0;
+		m_wdog = 0;
 		watchdog_reset_w(space,0,0);
 	}
 }
@@ -679,7 +677,7 @@ static void set_bank(running_machine &machine, int banknum, const bankhandler *h
 
 	/* for BANK handlers , memory direct and OP-code base */
 	if (handler->bank_pointer)
-		memory_set_bankptr(machine, banktags[banknum], handler->bank_pointer);
+		state->membank(banktags[banknum])->set_base(handler->bank_pointer);
 
 	/* read handlers */
 	if (!handler->bank_handler_r)
@@ -741,21 +739,21 @@ static void namcos1_bankswitch(running_machine &machine, int cpu, offs_t offset,
 	}
 }
 
-WRITE8_HANDLER( namcos1_bankswitch_w )
+WRITE8_MEMBER(namcos1_state::namcos1_bankswitch_w)
 {
-//  logerror("cpu %s: namcos1_bankswitch_w offset %04x data %02x\n", space->device().tag(), offset, data);
+//  logerror("cpu %s: namcos1_bankswitch_w offset %04x data %02x\n", device().tag(), offset, data);
 
-	namcos1_bankswitch(space->machine(), (&space->device() == space->machine().device("maincpu")) ? 0 : 1, offset, data);
+	namcos1_bankswitch(machine(), (&space.device() == machine().device("maincpu")) ? 0 : 1, offset, data);
 }
 
 /* Sub cpu set start bank port */
-WRITE8_HANDLER( namcos1_subcpu_bank_w )
+WRITE8_MEMBER(namcos1_state::namcos1_subcpu_bank_w)
 {
 //  logerror("namcos1_subcpu_bank_w offset %04x data %02x\n",offset,data);
 
 	/* Prepare code for CPU 1 */
-	namcos1_bankswitch( space->machine(), 1, 0x0e00, 0x03 );
-	namcos1_bankswitch( space->machine(), 1, 0x0e01, data );
+	namcos1_bankswitch( machine(), 1, 0x0e00, 0x03 );
+	namcos1_bankswitch( machine(), 1, 0x0e01, data );
 }
 
 /*******************************************************************************
@@ -828,7 +826,7 @@ static void namcos1_build_banks(running_machine &machine,read8_space_func key_r,
 
 	/* PRG0-PRG7 */
 	{
-		UINT8 *rom = machine.region("user1")->base();
+		UINT8 *rom = machine.root_device().memregion("user1")->base();
 
 		namcos1_install_bank(state,0x200,0x3ff,0,rom_w,0,rom);
 
@@ -903,7 +901,7 @@ MACHINE_RESET( namcos1 )
 *******************************************************************************/
 
 /* mcu banked rom area select */
-WRITE8_HANDLER( namcos1_mcu_bankswitch_w )
+WRITE8_MEMBER(namcos1_state::namcos1_mcu_bankswitch_w)
 {
 	int addr;
 
@@ -922,7 +920,7 @@ WRITE8_HANDLER( namcos1_mcu_bankswitch_w )
 	/* bit 0-1 : address line A15-A16 */
 	addr += (data & 3) * 0x8000;
 
-	memory_set_bankptr(space->machine(), "bank20", space->machine().region("mcu")->base() + addr);
+	membank("bank20")->set_base(machine().root_device().memregion("mcu")->base() + addr);
 }
 
 
@@ -939,13 +937,12 @@ WRITE8_HANDLER( namcos1_mcu_bankswitch_w )
 /* I found set $A6 only initialize in MCU                       */
 /* This patch kill write this data by MCU case $A6 to xx(clear) */
 
-WRITE8_HANDLER( namcos1_mcu_patch_w )
+WRITE8_MEMBER(namcos1_state::namcos1_mcu_patch_w)
 {
-	namcos1_state *state = space->machine().driver_data<namcos1_state>();
-	//logerror("mcu C000 write pc=%04x data=%02x\n",cpu_get_pc(&space->device()),data);
-	if (state->m_mcu_patch_data == 0xa6) return;
-	state->m_mcu_patch_data = data;
-	state->m_triram[0] = data;
+	//logerror("mcu C000 write pc=%04x data=%02x\n",cpu_get_pc(&space.device()),data);
+	if (m_mcu_patch_data == 0xa6) return;
+	m_mcu_patch_data = data;
+	m_triram[0] = data;
 }
 
 
@@ -1000,8 +997,8 @@ static void namcos1_driver_init( running_machine &machine, const struct namcos1_
 	state_save_register_global_pointer(machine, state->m_paletteram, 0x8000);
 
 	/* Point mcu & sound shared RAM to destination */
-	memory_set_bankptr(machine,  "bank18", state->m_triram );
-	memory_set_bankptr(machine,  "bank19", state->m_triram );
+	state->membank("bank18")->set_base(state->m_triram );
+	state->membank("bank19")->set_base(state->m_triram );
 
 	/* build bank elements */
 	namcos1_build_banks(machine,specific->key_r,specific->key_w);
@@ -1129,7 +1126,7 @@ DRIVER_INIT( bakutotu )
 		static const UINT8 target[8] = {0x34,0x37,0x35,0x37,0x96,0x00,0x2e,0xed};
 		UINT8 *rombase, *srcptr, *endptr, *scanptr;
 
-		rombase = machine.region("user1")->base();
+		rombase = machine.root_device().memregion("user1")->base();
 		srcptr = rombase + 0x1e000;
 		endptr = srcptr + 0xa000;
 
@@ -1285,9 +1282,9 @@ static READ8_HANDLER( quester_paddle_r )
 		int ret;
 
 		if (!qnum)
-			ret = (input_port_read(space->machine(), "CONTROL0")&0x90) | qstrobe | (input_port_read(space->machine(), "PADDLE0")&0x0f);
+			ret = (space->machine().root_device().ioport("CONTROL0")->read()&0x90) | qstrobe | (space->machine().root_device().ioport("PADDLE0")->read()&0x0f);
 		else
-			ret = (input_port_read(space->machine(), "CONTROL0")&0x90) | qstrobe | (input_port_read(space->machine(), "PADDLE1")&0x0f);
+			ret = (space->machine().root_device().ioport("CONTROL0")->read()&0x90) | qstrobe | (space->machine().root_device().ioport("PADDLE1")->read()&0x0f);
 
 		qstrobe ^= 0x40;
 
@@ -1298,9 +1295,9 @@ static READ8_HANDLER( quester_paddle_r )
 		int ret;
 
 		if (!qnum)
-			ret = (input_port_read(space->machine(), "CONTROL1")&0x90) | qnum | (input_port_read(space->machine(), "PADDLE0")>>4);
+			ret = (space->machine().root_device().ioport("CONTROL1")->read()&0x90) | qnum | (space->machine().root_device().ioport("PADDLE0")->read()>>4);
 		else
-			ret = (input_port_read(space->machine(), "CONTROL1")&0x90) | qnum | (input_port_read(space->machine(), "PADDLE1")>>4);
+			ret = (space->machine().root_device().ioport("CONTROL1")->read()&0x90) | qnum | (space->machine().root_device().ioport("PADDLE1")->read()>>4);
 
 		if (!qstrobe) qnum ^= 0x20;
 
@@ -1330,7 +1327,7 @@ static READ8_HANDLER( berabohm_buttons_r )
 	{
 		int inp = input_count;
 
-		if (inp == 4) res = input_port_read(space->machine(), "CONTROL0");
+		if (inp == 4) res = space->machine().root_device().ioport("CONTROL0")->read();
 		else
 		{
 			char portname[40];
@@ -1339,7 +1336,7 @@ static READ8_HANDLER( berabohm_buttons_r )
 			static int counter[4];
 
 			sprintf(portname,"IN%d",inp);	/* IN0-IN3 */
-			res = input_port_read(space->machine(), portname);
+			res = space->machine().root_device().ioport(portname)->read();
 			if (res & 0x80)
 			{
 				if (counter[inp] >= 0)
@@ -1363,7 +1360,7 @@ static READ8_HANDLER( berabohm_buttons_r )
 				counter[inp] = -1;
 #else
 			sprintf(portname,"IN%d",inp);	/* IN0-IN3 */
-			res = input_port_read(space->machine(), portname);
+			res = space->machine().root_device().ioport(portname)->read();
 			if (res & 1) res = 0x7f;		/* weak */
 			else if (res & 2) res = 0x48;	/* medium */
 			else if (res & 4) res = 0x40;	/* strong */
@@ -1374,7 +1371,7 @@ static READ8_HANDLER( berabohm_buttons_r )
 	}
 	else
 	{
-		res = input_port_read(space->machine(), "CONTROL1") & 0x8f;
+		res = space->machine().root_device().ioport("CONTROL1")->read() & 0x8f;
 
 		/* the strobe cannot happen too often, otherwise the MCU will waste too
            much time reading the inputs and won't have enough cycles to play two
@@ -1416,13 +1413,13 @@ static READ8_HANDLER( faceoff_inputs_r )
 
 	if (offset == 0)
 	{
-		res = (input_port_read(space->machine(), "CONTROL0") & 0x80) | stored_input[0];
+		res = (space->machine().root_device().ioport("CONTROL0")->read() & 0x80) | stored_input[0];
 
 		return res;
 	}
 	else
 	{
-		res = input_port_read(space->machine(), "CONTROL1") & 0x80;
+		res = space->machine().root_device().ioport("CONTROL1")->read() & 0x80;
 
 		/* the strobe cannot happen too often, otherwise the MCU will waste too
            much time reading the inputs and won't have enough cycles to play two
@@ -1437,17 +1434,17 @@ static READ8_HANDLER( faceoff_inputs_r )
 			switch (input_count)
 			{
 				case 0:
-					stored_input[0] = input_port_read(space->machine(), "IN0") & 0x1f;
-					stored_input[1] = (input_port_read(space->machine(), "IN3") & 0x07) << 3;
+					stored_input[0] = space->machine().root_device().ioport("IN0")->read() & 0x1f;
+					stored_input[1] = (space->machine().root_device().ioport("IN3")->read() & 0x07) << 3;
 					break;
 
 				case 3:
-					stored_input[0] = input_port_read(space->machine(), "IN2") & 0x1f;
+					stored_input[0] = space->machine().root_device().ioport("IN2")->read() & 0x1f;
 					break;
 
 				case 4:
-					stored_input[0] = input_port_read(space->machine(), "IN1") & 0x1f;
-					stored_input[1] = input_port_read(space->machine(), "IN3") & 0x18;
+					stored_input[0] = space->machine().root_device().ioport("IN1")->read() & 0x1f;
+					stored_input[1] = space->machine().root_device().ioport("IN3")->read() & 0x18;
 					break;
 
 				default:
