@@ -147,7 +147,7 @@ UINT32 amiga_gethvpos(screen_device &screen)
 {
 	amiga_state *state = screen.machine().driver_data<amiga_state>();
 	UINT32 hvpos = (state->m_last_scanline << 8) | (screen.hpos() >> 2);
-	UINT32 latchedpos = input_port_read_safe(screen.machine(), "HVPOS", 0);
+	UINT32 latchedpos = state->ioport("HVPOS")->read_safe(0);
 
 	/* if there's no latched position, or if we are in the active display area */
 	/* but before the latching point, return the live HV position */
@@ -619,7 +619,7 @@ INLINE int update_ham(amiga_state *state, int newpix)
  *
  *************************************/
 
-void amiga_render_scanline(running_machine &machine, bitmap_t *bitmap, int scanline)
+void amiga_render_scanline(running_machine &machine, bitmap_ind16 &bitmap, int scanline)
 {
 	amiga_state *state = machine.driver_data<amiga_state>();
 	UINT16 save_color0 = CUSTOM_REG(REG_COLOR00);
@@ -652,8 +652,8 @@ void amiga_render_scanline(running_machine &machine, bitmap_t *bitmap, int scanl
 	update_sprite_dma(state, scanline);
 
 	/* start of a new line, signal we're not done with it and fill up vars */
-	if (bitmap != NULL)
-		dst = BITMAP_ADDR16(bitmap, scanline, 0);
+	if (bitmap.valid())
+		dst = &bitmap.pix16(scanline);
 
 	/* all sprites off at the start of the line */
 	memset(state->m_sprite_remain, 0, sizeof(state->m_sprite_remain));
@@ -999,13 +999,13 @@ void amiga_render_scanline(running_machine &machine, bitmap_t *bitmap, int scanl
  *
  *************************************/
 
-SCREEN_UPDATE( amiga )
+SCREEN_UPDATE_IND16( amiga )
 {
 	int y;
 
 	/* render each scanline in the visible region */
-	for (y = cliprect->min_y; y <= cliprect->max_y; y++)
-		amiga_render_scanline(screen->machine(), bitmap, y);
+	for (y = cliprect.min_y; y <= cliprect.max_y; y++)
+		amiga_render_scanline(screen.machine(), bitmap, y);
 
 	return 0;
 }
