@@ -44,7 +44,6 @@
 *************************************************************************/
 
 #include "emu.h"
-#include "profiler.h"
 #include "video/vector.h"
 #include "includes/vertigo.h"
 
@@ -133,7 +132,6 @@ void vertigo_vproc_init(running_machine &machine)
 	state_save_register_item(machine, "vector_proc", NULL, 0, state->m_bsp.f);
 	state_save_register_item(machine, "vector_proc", NULL, 0, state->m_bsp.y);
 
-	state->m_vgen.set_machine(machine);
 	state_save_register_item(machine, "vector_proc", NULL, 0, state->m_vgen.sreg);
 	state_save_register_item(machine, "vector_proc", NULL, 0, state->m_vgen.l1);
 	state_save_register_item(machine, "vector_proc", NULL, 0, state->m_vgen.l2);
@@ -161,8 +159,8 @@ void vertigo_vproc_reset(running_machine &machine)
 	int i;
 	UINT64 *mcode;
 
-	state->m_vectorrom = (UINT16 *)machine.region("user1")->base();
-	mcode = (UINT64 *)machine.region("proms")->base();
+	state->m_vectorrom = (UINT16 *)state->memregion("user1")->base();
+	mcode = (UINT64 *)state->memregion("proms")->base();
 
 	/* Decode microcode */
 	for (i = 0; i < MC_LENGTH; i++)
@@ -188,6 +186,7 @@ void vertigo_vproc_reset(running_machine &machine)
 	memset(&state->m_vs, 0, sizeof(state->m_vs));
 	memset(&state->m_bsp, 0, sizeof(state->m_bsp));
 	memset(&state->m_vgen, 0, sizeof(state->m_vgen));
+	state->m_vgen.set_machine(machine);
 }
 
 
