@@ -10,7 +10,6 @@
     do R13 = SR13
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/arm7/arm7.h"
@@ -18,7 +17,6 @@
 
 #define MACHINE_RESET_MEMBER(name) void name::machine_reset()
 #define MACHINE_START_MEMBER(name) void name::machine_start()
-#define SCREEN_UPDATE_MEMBER(name) bool name::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
 
 class a7000_state : public driver_device
 {
@@ -57,7 +55,8 @@ public:
 	void vidc20_dynamic_screen_change();
 	virtual void machine_reset();
 	virtual void machine_start();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+
+	UINT32 screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 };
 
 
@@ -261,14 +260,14 @@ WRITE32_MEMBER( a7000_state::a7000_vidc20_w )
 	}
 }
 
-SCREEN_UPDATE_MEMBER( a7000_state )
+UINT32 a7000_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	int x_size,y_size,x_start,y_start;
 	int x,y,xi;
 	UINT32 count;
-	UINT8 *vram = machine().region("vram")->base();
+	UINT8 *vram = memregion("vram")->base();
 
-	bitmap_fill(&bitmap, &cliprect, machine().pens[0x100]);
+	bitmap.fill(machine().pens[0x100], cliprect);
 
 	x_size = (m_vidc20_horz_reg[HDER]-m_vidc20_horz_reg[HDSR]);
 	y_size = (m_vidc20_vert_reg[VDER]-m_vidc20_vert_reg[VDSR]);
@@ -292,7 +291,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 				for(x=0;x<x_size;x+=8)
 				{
 					for(xi=0;xi<8;xi++)
-						*BITMAP_ADDR32(&bitmap, y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi))&1];
+						bitmap.pix32(y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi))&1];
 
 					count++;
 				}
@@ -306,7 +305,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 				for(x=0;x<x_size;x+=4)
 				{
 					for(xi=0;xi<4;xi++)
-						*BITMAP_ADDR32(&bitmap, y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi*2))&3];
+						bitmap.pix32(y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi*2))&3];
 
 					count++;
 				}
@@ -320,7 +319,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 				for(x=0;x<x_size;x+=2)
 				{
 					for(xi=0;xi<2;xi++)
-						*BITMAP_ADDR32(&bitmap, y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi*4))&0xf];
+						bitmap.pix32(y+y_start, x+xi+x_start) = machine().pens[(vram[count]>>(xi*4))&0xf];
 
 					count++;
 				}
@@ -333,7 +332,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 			{
 				for(x=0;x<x_size;x++)
 				{
-					*BITMAP_ADDR32(&bitmap, y+y_start, x+x_start) = machine().pens[(vram[count])&0xff];
+					bitmap.pix32(y+y_start, x+x_start) = machine().pens[(vram[count])&0xff];
 
 					count++;
 				}
@@ -356,7 +355,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 					g = (g << 4) | (g & 0xf);
 					b = (b << 4) | (b & 0xf);
 
-					*BITMAP_ADDR32(&bitmap, y+y_start, x+x_start) =  b | g << 8 | r << 16;
+					bitmap.pix32(y+y_start, x+x_start) =  b | g << 8 | r << 16;
 
 					count+=2;
 				}
@@ -376,7 +375,7 @@ SCREEN_UPDATE_MEMBER( a7000_state )
 					g = (pen & 0x00ff00) >> 8;
 					b = (pen & 0xff0000) >> 16;
 
-					*BITMAP_ADDR32(&bitmap, y+y_start, x+x_start) =  b | g << 8 | r << 16;
+					bitmap.pix32(y+y_start, x+x_start) =  b | g << 8 | r << 16;
 
 					count+=4;
 				}
@@ -577,7 +576,7 @@ static TIMER_CALLBACK( IOMD_timer0_callback )
 	state->m_IRQ_status_A|=0x20;
 	if(state->m_IRQ_mask_A&0x20)
 	{
-		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE);
+		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE,1);
 	}
 }
 
@@ -587,7 +586,7 @@ static TIMER_CALLBACK( IOMD_timer1_callback )
 	state->m_IRQ_status_A|=0x40;
 	if(state->m_IRQ_mask_A&0x40)
 	{
-		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE);
+		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE,1);
 	}
 }
 
@@ -597,7 +596,7 @@ static TIMER_CALLBACK( flyback_timer_callback )
 	state->m_IRQ_status_A|=0x08;
 	if(state->m_IRQ_mask_A&0x08)
 	{
-		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE);
+		generic_pulse_irq_line(machine.device("maincpu"), ARM7_IRQ_LINE,1);
 	}
 
 	state->m_flyback_timer->adjust(machine.primary_screen->time_until_pos(state->m_vidc20_vert_reg[VDER]));
@@ -610,7 +609,7 @@ void a7000_state::viddma_transfer_start()
 	UINT32 dst = 0;
 	UINT32 size = m_viddma_addr_end;
 	UINT32 dma_index;
-	UINT8 *vram = machine().region("vram")->base();
+	UINT8 *vram = memregion("vram")->base();
 
 	/* TODO: this should actually be a qword transfer */
 	for(dma_index = 0;dma_index < size;dma_index++)
@@ -798,9 +797,9 @@ static MACHINE_CONFIG_START( a7000, a7000_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(1900, 1080) //max available size
 	MCFG_SCREEN_VISIBLE_AREA(0, 1900-1, 0, 1080-1)
+	MCFG_SCREEN_UPDATE_DRIVER(a7000_state, screen_update)
 	MCFG_PALETTE_LENGTH(0x200)
 MACHINE_CONFIG_END
 
