@@ -58,62 +58,6 @@ Notes:
     DIPSW2  -
 
 
-Keyboard PCB Layout
--------------------
-
-KTC 65-01870-001 (dated 1978)
-PCB-002B
-
-    |---------------|
-|---|      CN1      |---------------------------------------------------|
-|                                                                       |
-|   LS06    LS04    LS04                        LS06    LS00    LS123   |
-|   LS74                    PROM        MCU             4051    900C    |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|-----------------------------------------------------------------------|
-
-Notes:
-    All IC's shown.
-
-    MCU         - General Instruments 30293B-013 20-04592-013 (?)
-    PROM        - Synertek N82S141N 512x8 bipolar PROM "053"
-    900C        - Interdesign 900C (?)
-    CN1         - keyboard data connector
-
-
-KTC A65-01870-001 (dated 1983)
-PCB-201C
-
-    |---------------|
-|---|      CN1      |---------------------------------------------------|
-|                                                                       |
-|   LS06    LS04    LS04                                LS00    LS123   |
-|                                       MCU             4051    900C    |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|                                                                       |
-|-----------------------------------------------------------------------|
-
-Notes:
-    All IC's shown.
-
-    MCU         - General Instruments 30293B-047 20-04592-047 (?)
-    900C        - Interdesign 900C (?)
-    CN1         - keyboard data connector
-
-
 DOS PCB Layout
 --------------
 
@@ -164,7 +108,6 @@ Notes:
 
     TODO:
 
-    - memory bank switching using ABC80/13 PROM
     - proper keyboard controller emulation
     - get BASIC v1 dump
     - MyAB 80-column card
@@ -173,26 +116,25 @@ Notes:
     - floppy
     - printer
     - IEC
+    - Metric ABC CAD 1000
 
 */
 
-#define ADDRESS_MAP_MODERN
-
-#include "emu.h"
-#include "cpu/z80/z80.h"
-#include "cpu/z80/z80daisy.h"
-#include "imagedev/flopdrv.h"
-#include "machine/ram.h"
-#include "imagedev/printer.h"
-#include "machine/abcbus.h"
-#include "machine/abc830.h"
-#include "machine/lux10828.h"
-#include "machine/z80pio.h"
-#include "sound/sn76477.h"
 #include "includes/abc80.h"
 
 
 
+//**************************************************************************
+//  MACROS / CONSTANTS
+//**************************************************************************
+
+#define MMU_XM		0x01
+#define MMU_ROM		0x02
+#define MMU_VRAMS	0x04
+#define MMU_RAM		0x08
+
+
+
 //**************************************************************************
 //  KEYBOARD HACK
 //**************************************************************************
@@ -332,13 +274,13 @@ static void abc80_keyboard_scan(running_machine &machine)
 	static const char *const keynames[] = { "ROW0", "ROW1", "ROW2", "ROW3", "ROW4", "ROW5", "ROW6" };
 	int table = 0, row, col;
 
-	if (input_port_read(machine, "ROW7") & 0x07)
+	if (machine.root_device().ioport("ROW7")->read() & 0x07)
 	{
 		/* shift, upper case */
 		table |= 0x01;
 	}
 
-	if (input_port_read(machine, "ROW7") & 0x08)
+	if (machine.root_device().ioport("ROW7")->read() & 0x08)
 	{
 		/* ctrl */
 		table |= 0x02;
@@ -350,7 +292,7 @@ static void abc80_keyboard_scan(running_machine &machine)
 	/* scan keyboard */
 	for (row = 0; row < 7; row++)
 	{
-		UINT8 data = input_port_read(machine, keynames[row]);
+		UINT8 data = machine.root_device().ioport(keynames[row])->read();
 
 		for (col = 0; col < 8; col++)
 		{
@@ -389,6 +331,64 @@ static TIMER_DEVICE_CALLBACK( abc80_keyboard_tick )
 
 
 
+//**************************************************************************
+//  MEMORY MANAGEMENT
+//**************************************************************************
+
+//-------------------------------------------------
+//  read -
+//-------------------------------------------------
+
+READ8_MEMBER( abc80_state::read )
+{
+	UINT8 data = 0xff;
+	UINT8 mmu = m_mmu_rom[0x40 | (offset >> 10)];
+
+	if (!(mmu & MMU_XM))
+	{
+		data = m_bus->xmemfl_r(space, offset);
+	}
+	else if (!(mmu & MMU_ROM))
+	{
+		data = memregion(Z80_TAG)->base()[offset & 0x3fff];
+	}
+	else if (mmu & MMU_VRAMS)
+	{
+		data = m_video_ram[offset & 0x3ff];
+	}
+	else if (!(mmu & MMU_RAM))
+	{
+		data = m_ram->pointer()[offset & 0x3fff];
+	}
+
+	return data;
+}
+
+
+//-------------------------------------------------
+//  write -
+//-------------------------------------------------
+
+WRITE8_MEMBER( abc80_state::write )
+{
+	UINT8 mmu = m_mmu_rom[0x40 | (offset >> 10)];
+
+	if (!(mmu & MMU_XM))
+	{
+		m_bus->xmemw_w(space, offset, data);
+	}
+	else if (mmu & MMU_VRAMS)
+	{
+		m_video_ram[offset & 0x3ff] = data;
+	}
+	else if (!(mmu & MMU_RAM))
+	{
+		m_ram->pointer()[offset & 0x3fff] = data;
+	}
+}
+
+
+
 //**************************************************************************
 //  SOUND
 //**************************************************************************
@@ -397,45 +397,12 @@ static TIMER_DEVICE_CALLBACK( abc80_keyboard_tick )
 //  vco_voltage_w - CSG VCO voltage select
 //-------------------------------------------------
 
-WRITE_LINE_DEVICE_HANDLER( vco_voltage_w )
+WRITE_LINE_MEMBER( abc80_state::vco_voltage_w )
 {
-	sn76477_vco_voltage_w(device, state ? 2.5 : 0);
+	sn76477_vco_voltage_w(m_psg, state ? 2.5 : 0);
 }
 
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_enable_w)
-{
-	sn76477_enable_w(device,state);
-}
 
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_vco_w)
-{
-	sn76477_vco_w(device,state);
-}
-
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_mixer_b_w)
-{
-	sn76477_mixer_b_w(device,state);
-}
-
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_mixer_a_w)
-{
-	sn76477_mixer_a_w(device,state);
-}
-
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_mixer_c_w)
-{
-	sn76477_mixer_c_w(device,state);
-}
-
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_envelope_2_w)
-{
-	sn76477_envelope_2_w(device,state);
-}
-
-WRITE_LINE_DEVICE_HANDLER(port_sn76477_envelope_1_w)
-{
-	sn76477_envelope_1_w(device,state);
-}
 
 //**************************************************************************
 //  ADDRESS MAPS
@@ -447,13 +414,7 @@ WRITE_LINE_DEVICE_HANDLER(port_sn76477_envelope_1_w)
 
 static ADDRESS_MAP_START( abc80_mem, AS_PROGRAM, 8, abc80_state )
 	ADDRESS_MAP_UNMAP_HIGH
-	AM_RANGE(0x0000, 0x3fff) AM_ROM
-	AM_RANGE(0x6000, 0x6fff) AM_ROM
-	AM_RANGE(0x7000, 0x73ff) AM_ROM
-	AM_RANGE(0x7400, 0x77ff) AM_RAM AM_BASE(m_video_80_ram)
-	AM_RANGE(0x7800, 0x7bff) AM_ROM
-	AM_RANGE(0x7c00, 0x7fff) AM_RAM AM_BASE(m_video_ram)
-	AM_RANGE(0x8000, 0xffff) AM_RAM
+	AM_RANGE(0x0000, 0xffff) AM_READWRITE(read, write)
 ADDRESS_MAP_END
 
 
@@ -489,14 +450,14 @@ static INPUT_PORTS_START( abc80 )
 	PORT_INCLUDE(fake_keyboard)
 
 	PORT_START("SN76477")
-	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_enable_w)
-	PORT_BIT(0x02, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, vco_voltage_w)
-	PORT_BIT(0x04, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_vco_w)
-	PORT_BIT(0x08, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_mixer_b_w)
-	PORT_BIT(0x10, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_mixer_a_w)
-	PORT_BIT(0x20, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_mixer_c_w)
-	PORT_BIT(0x40, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_envelope_2_w)
-	PORT_BIT(0x80, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, port_sn76477_envelope_1_w)
+	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_enable_w)
+	PORT_BIT(0x02, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE_MEMBER(DEVICE_SELF, abc80_state, vco_voltage_w)
+	PORT_BIT(0x04, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_vco_w)
+	PORT_BIT(0x08, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_mixer_b_w)
+	PORT_BIT(0x10, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_mixer_a_w)
+	PORT_BIT(0x20, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_mixer_c_w)
+	PORT_BIT(0x40, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_envelope_2_w)
+	PORT_BIT(0x80, IP_ACTIVE_HIGH, IPT_SPECIAL) PORT_WRITE_LINE_DEVICE(SN76477_TAG, sn76477_envelope_1_w)
 INPUT_PORTS_END
 
 
@@ -667,10 +628,11 @@ static const cassette_interface abc80_cassette_interface =
 //-------------------------------------------------
 
 static SLOT_INTERFACE_START( abc80_abcbus_cards )
+	SLOT_INTERFACE("fd2", ABC_FD2)
 	SLOT_INTERFACE("slow", LUXOR_55_10828)
+	SLOT_INTERFACE("sio", ABC_SIO)
 SLOT_INTERFACE_END
 
-
 static ABCBUS_INTERFACE( abcbus_intf )
 {
 	DEVCB_NULL,
@@ -680,6 +642,16 @@ static ABCBUS_INTERFACE( abcbus_intf )
 };
 
 
+//-------------------------------------------------
+//  ABC80_KEYBOARD_INTERFACE( kb_intf )
+//-------------------------------------------------
+
+static ABC80_KEYBOARD_INTERFACE( kb_intf )
+{
+	DEVCB_NULL
+};
+
+
 
 //**************************************************************************
 //  MACHINE INITIALIZATION
@@ -691,16 +663,13 @@ static ABCBUS_INTERFACE( abcbus_intf )
 
 void abc80_state::machine_start()
 {
-	/* configure RAM expansion */
-	if (ram_get_size(m_ram) == 16 * 1024)
-	{
-		m_maincpu->memory().space(AS_PROGRAM)->unmap_readwrite(0x8000, 0xbfff);
-	}
+	// find memory regions
+	m_mmu_rom = memregion("mmu")->base();
 
-	/* register for state saving */
-	state_save_register_global(machine(), m_key_data);
-	state_save_register_global(machine(), m_key_strobe);
-	state_save_register_global(machine(), m_pio_astb);
+	// register for state saving
+	save_item(NAME(m_key_data));
+	save_item(NAME(m_key_strobe));
+	save_item(NAME(m_pio_astb));
 }
 
 
@@ -738,6 +707,7 @@ static MACHINE_CONFIG_START( abc80, abc80_state )
 	MCFG_PRINTER_ADD("printer")
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, abc80_cassette_interface)
 	MCFG_ABC830_ADD()
+	MCFG_ABC80_KEYBOARD_ADD(kb_intf)
 
 	// ABC bus
 	MCFG_ABCBUS_SLOT_ADD(ABCBUS_TAG, abcbus_intf, abc80_abcbus_cards, "slow", NULL)
@@ -745,7 +715,6 @@ static MACHINE_CONFIG_START( abc80, abc80_state )
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("16K")
-	MCFG_RAM_EXTRA_OPTIONS("32K")
 
 	// software list
 	MCFG_SOFTWARE_LIST_ADD("flop_list", "abc80")
@@ -777,9 +746,6 @@ ROM_START( abc80 )
 	ROM_LOAD( "iec",	   0x7000, 0x0400, NO_DUMP )
 	ROM_LOAD( "printer",   0x7800, 0x0400, NO_DUMP )
 
-	ROM_REGION( 0x200, "keyboard", 0 )
-	ROM_LOAD( "053.z5", 0x0000, 0x0200, NO_DUMP ) // 82S141 512x8
-
 	ROM_REGION( 0xa00, "chargen", 0 )
 	ROM_LOAD( "sn74s263.h2", 0x0000, 0x0a00, BAD_DUMP CRC(9e064e91) SHA1(354783c8f2865f73dc55918c9810c66f3aca751f) ) // created by hand
 
