@@ -39,7 +39,6 @@ Ports:
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -71,17 +70,17 @@ READ8_MEMBER( ace_state::io_r )
 {
 	UINT8 data = 0xff;
 
-	if (!BIT(offset, 8)) data &= input_port_read(machine(), "A8");
-	if (!BIT(offset, 9)) data &= input_port_read(machine(), "A9");
-	if (!BIT(offset, 10)) data &= input_port_read(machine(), "A10");
-	if (!BIT(offset, 11)) data &= input_port_read(machine(), "A11");
-	if (!BIT(offset, 12)) data &= input_port_read(machine(), "A12");
-	if (!BIT(offset, 13)) data &= input_port_read(machine(), "A13");
-	if (!BIT(offset, 14)) data &= input_port_read(machine(), "A14");
+	if (!BIT(offset, 8)) data &= ioport("A8")->read();
+	if (!BIT(offset, 9)) data &= ioport("A9")->read();
+	if (!BIT(offset, 10)) data &= ioport("A10")->read();
+	if (!BIT(offset, 11)) data &= ioport("A11")->read();
+	if (!BIT(offset, 12)) data &= ioport("A12")->read();
+	if (!BIT(offset, 13)) data &= ioport("A13")->read();
+	if (!BIT(offset, 14)) data &= ioport("A14")->read();
 
 	if (!BIT(offset, 15))
 	{
-		data &= input_port_read(machine(), "A15");
+		data &= ioport("A15")->read();
 
 		m_cassette->output(-1);
 		speaker_level_w(m_speaker, 0);
@@ -218,8 +217,8 @@ static WRITE8_DEVICE_HANDLER( pio_bc_w )
 
 static ADDRESS_MAP_START( ace_mem, AS_PROGRAM, 8, ace_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROM
-	AM_RANGE(0x2000, 0x23ff) AM_MIRROR(0x0400) AM_RAM AM_BASE(m_video_ram)
-	AM_RANGE(0x2800, 0x2bff) AM_MIRROR(0x0400) AM_RAM AM_BASE(m_char_ram) AM_REGION(Z80_TAG, 0xfc00)
+	AM_RANGE(0x2000, 0x23ff) AM_MIRROR(0x0400) AM_RAM AM_SHARE("video_ram")
+	AM_RANGE(0x2800, 0x2bff) AM_MIRROR(0x0400) AM_RAM AM_SHARE("char_ram") AM_REGION(Z80_TAG, 0xfc00)
 	AM_RANGE(0x3000, 0x33ff) AM_MIRROR(0x0c00) AM_RAM
 	AM_RANGE(0x4000, 0xffff) AM_RAM
 ADDRESS_MAP_END
@@ -382,10 +381,10 @@ static TIMER_DEVICE_CALLBACK( clear_irq )
 
 
 //-------------------------------------------------
-//  SCREEN_UPDATE( ace )
+//  SCREEN_UPDATE_IND16( ace )
 //-------------------------------------------------
 
-bool ace_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 ace_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	UINT8 y,ra,chr,gfx;
 	UINT16 sy=56,ma=0,x;
@@ -394,7 +393,7 @@ bool ace_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rec
 	{
 		for (ra = 0; ra < 8; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(&bitmap, sy++, 40);
+			UINT16 *p = &bitmap.pix16(sy++, 40);
 
 			for (x = ma; x < ma+32; x++)
 			{
@@ -565,7 +564,7 @@ WRITE8_MEMBER( ace_state::pio_pa_w )
     */
 
 	// centronics strobe
-	centronics_strobe_w(m_centronics, !BIT(data, 6));
+	m_centronics->strobe_w(!BIT(data, 6));
 };
 
 static Z80PIO_INTERFACE( pio_intf )
@@ -575,7 +574,7 @@ static Z80PIO_INTERFACE( pio_intf )
 	DEVCB_DRIVER_MEMBER(ace_state, pio_pa_w),
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_DEVICE_HANDLER(CENTRONICS_TAG, centronics_data_w),
+	DEVCB_DEVICE_MEMBER(CENTRONICS_TAG, centronics_device, write),
 	DEVCB_NULL
 };
 
@@ -594,7 +593,7 @@ void ace_state::machine_start()
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
 
 	/* configure RAM */
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 1*1024:
 		program->unmap_readwrite(0x4000, 0xffff);
@@ -629,7 +628,7 @@ static MACHINE_CONFIG_START( ace, ace_state )
 
 	// video hardware
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(ace_state, screen_update)
 	MCFG_SCREEN_RAW_PARAMS(XTAL_6_5MHz, 416, 0, 336, 312, 0, 304)
 	MCFG_TIMER_ADD_SCANLINE("set_irq", set_irq, SCREEN_TAG, 31*8, 264)
 	MCFG_TIMER_ADD_SCANLINE("clear_irq", clear_irq, SCREEN_TAG, 32*8, 264)
@@ -659,7 +658,7 @@ static MACHINE_CONFIG_START( ace, ace_state )
 	MCFG_SNAPSHOT_ADD("snapshot", ace, "ace", 1)
 	MCFG_I8255A_ADD(I8255_TAG, ppi_intf)
 	MCFG_Z80PIO_ADD(Z80PIO_TAG, XTAL_6_5MHz/2, pio_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 
 	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
@@ -677,7 +676,7 @@ MACHINE_CONFIG_END
 //  ROM( ace )
 //-------------------------------------------------
 
-ROM_START( ace )
+ROM_START( jupace )
 	ROM_REGION( 0x10000, Z80_TAG, 0 )
 	ROM_LOAD( "rom-a.z1", 0x0000, 0x1000, CRC(dc8438a5) SHA1(8fa97eb71e5dd17c7d190c6587ee3840f839347c) )
 	ROM_LOAD( "rom-b.z2", 0x1000, 0x1000, CRC(4009f636) SHA1(98c5d4bcd74bcf014268cf4c00b2007ea5cc21f3) )
@@ -696,4 +695,4 @@ ROM_END
 //**************************************************************************
 
 //    YEAR  NAME     PARENT    COMPAT  MACHINE    INPUT     INIT     COMPANY         FULLNAME      FLAGS
-COMP( 1981, ace,     0,        0,      ace,       ace,      0,   "Jupiter Cantab", "Jupiter Ace" , 0 )
+COMP( 1981, jupace,     0,        0,      ace,       ace,      0,   "Jupiter Cantab", "Jupiter Ace" , 0 )
