@@ -188,12 +188,12 @@
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
 #include "cpu/m6800/m6800.h"
 #include "formats/basicdsk.h"
+#include "formats/adam_cas.h"
 #include "imagedev/cartslot.h"
 #include "imagedev/cassette.h"
 #include "imagedev/flopdrv.h"
@@ -250,7 +250,7 @@ enum
 void adam_state::bankswitch()
 {
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
 	switch (m_mioc & 0x03)
 	{
@@ -258,11 +258,11 @@ void adam_state::bankswitch()
 		if (BIT(m_adamnet, 1))
 		{
 			program->unmap_readwrite(0x0000, 0x5fff);
-			program->install_rom(0x6000, 0x7fff, machine().region("wp")->base() + 0x8000);
+			program->install_rom(0x6000, 0x7fff, memregion("wp")->base() + 0x8000);
 		}
 		else
 		{
-			program->install_rom(0x0000, 0x7fff, machine().region("wp")->base());
+			program->install_rom(0x0000, 0x7fff, memregion("wp")->base());
 		}
 		break;
 
@@ -271,14 +271,14 @@ void adam_state::bankswitch()
 		break;
 
 	case LO_RAM_EXPANSION:
-		if (ram_get_size(m_ram) > 64 * 1024)
+		if (m_ram->size() > 64 * 1024)
 			program->install_ram(0x0000, 0x7fff, ram + 0x10000);
 		else
 			program->unmap_readwrite(0x0000, 0x7fff);
 		break;
 
 	case LO_OS7_ROM_INTERNAL_RAM:
-		program->install_rom(0x0000, 0x1fff, machine().region("os7")->base());
+		program->install_rom(0x0000, 0x1fff, memregion("os7")->base());
 		program->install_ram(0x2000, 0x7fff, ram + 0x2000);
 		break;
 	}
@@ -290,17 +290,17 @@ void adam_state::bankswitch()
 		break;
 
 	case HI_ROM_EXPANSION:
-		program->install_rom(0x8000, 0xffff, machine().region("xrom")->base());
+		program->install_rom(0x8000, 0xffff, memregion("xrom")->base());
 		break;
 
 	case HI_RAM_EXPANSION:
 		if (m_game)
 		{
-			program->install_rom(0x8000, 0xffff, machine().region("cart")->base());
+			program->install_rom(0x8000, 0xffff, memregion("cart")->base());
 		}
 		else
 		{
-			if (ram_get_size(m_ram) > 64 * 1024)
+			if (m_ram->size() > 64 * 1024)
 				program->install_ram(0x8000, 0xffff, ram + 0x18000);
 			else
 				program->unmap_readwrite(0x8000, 0xffff);
@@ -308,7 +308,7 @@ void adam_state::bankswitch()
 		break;
 
 	case HI_CARTRIDGE_ROM:
-		program->install_rom(0x8000, 0xffff, machine().region("cart")->base());
+		program->install_rom(0x8000, 0xffff, memregion("cart")->base());
 		break;
 	}
 }
@@ -605,19 +605,19 @@ READ8_MEMBER( adam_state::kb6801_p1_r )
 
 	UINT8 data = 0xff;
 
-	if (!BIT(m_key_y, 0)) data &= input_port_read(machine(), "Y0");
-	if (!BIT(m_key_y, 1)) data &= input_port_read(machine(), "Y1");
-	if (!BIT(m_key_y, 2)) data &= input_port_read(machine(), "Y2");
-	if (!BIT(m_key_y, 3)) data &= input_port_read(machine(), "Y3");
-	if (!BIT(m_key_y, 4)) data &= input_port_read(machine(), "Y4");
-	if (!BIT(m_key_y, 5)) data &= input_port_read(machine(), "Y5");
-	if (!BIT(m_key_y, 6)) data &= input_port_read(machine(), "Y6");
-	if (!BIT(m_key_y, 7)) data &= input_port_read(machine(), "Y7");
-	if (!BIT(m_key_y, 8)) data &= input_port_read(machine(), "Y8");
-	if (!BIT(m_key_y, 9)) data &= input_port_read(machine(), "Y9");
-	if (!BIT(m_key_y, 10)) data &= input_port_read(machine(), "Y10");
-	if (!BIT(m_key_y, 11)) data &= input_port_read(machine(), "Y11");
-	if (!BIT(m_key_y, 12)) data &= input_port_read(machine(), "Y12");
+	if (!BIT(m_key_y, 0)) data &= ioport("Y0")->read();
+	if (!BIT(m_key_y, 1)) data &= ioport("Y1")->read();
+	if (!BIT(m_key_y, 2)) data &= ioport("Y2")->read();
+	if (!BIT(m_key_y, 3)) data &= ioport("Y3")->read();
+	if (!BIT(m_key_y, 4)) data &= ioport("Y4")->read();
+	if (!BIT(m_key_y, 5)) data &= ioport("Y5")->read();
+	if (!BIT(m_key_y, 6)) data &= ioport("Y6")->read();
+	if (!BIT(m_key_y, 7)) data &= ioport("Y7")->read();
+	if (!BIT(m_key_y, 8)) data &= ioport("Y8")->read();
+	if (!BIT(m_key_y, 9)) data &= ioport("Y9")->read();
+	if (!BIT(m_key_y, 10)) data &= ioport("Y10")->read();
+	if (!BIT(m_key_y, 11)) data &= ioport("Y11")->read();
+	if (!BIT(m_key_y, 12)) data &= ioport("Y12")->read();
 
 	return data;
 }
@@ -763,15 +763,21 @@ WRITE8_MEMBER( adam_state::ddp6801_p1_w )
 
     */
 
-	// speed select
-//  cassette_set_speed(m_ddp0, BIT(data, 0) ? 80 : 20);
-//  cassette_set_speed(m_ddp1, BIT(data, 0) ? 80 : 20);
+	if(m_ddp0->exists())
+	{
+		m_ddp0->set_speed(BIT(data, 0) ? (double) 80/1.875 : 20/1.875); // speed select
+		if(!(data & 0x08)) m_ddp0->go_forward();
+		if(!(data & 0x10)) m_ddp0->go_reverse();
+		m_ddp0->change_state(BIT(data, 1) ? CASSETTE_MOTOR_DISABLED : CASSETTE_MOTOR_ENABLED, CASSETTE_MASK_MOTOR); // motor control
+	}
 
-	// motor stop 0
-	m_ddp0->set_state(BIT(data, 1) ? CASSETTE_MOTOR_DISABLED : CASSETTE_MOTOR_ENABLED);
-
-	// motor stop 1
-	m_ddp1->set_state(BIT(data, 2) ? CASSETTE_MOTOR_DISABLED : CASSETTE_MOTOR_ENABLED);
+	if(m_ddp1->exists())
+	{
+		m_ddp1->set_speed(BIT(data, 0) ? (double) 80/1.875 : 20/1.875); // speed select
+		if(!(data & 0x08)) m_ddp1->go_forward();
+		if(!(data & 0x10)) m_ddp1->go_reverse();
+		m_ddp1->change_state(BIT(data, 2) ? CASSETTE_MOTOR_DISABLED : CASSETTE_MOTOR_ENABLED, CASSETTE_MASK_MOTOR); // motor control
+	}
 
 	// data write 0
 	m_wr0 = BIT(data, 6);
@@ -799,14 +805,12 @@ READ8_MEMBER( adam_state::ddp6801_p2_r )
 
     */
 
-	UINT8 data = M6801_MODE_6;
+	UINT8 data = 0;
 
-	if (!m_reset)
-	{
-		// cassette in place 1
-		data &= ~0x01;
-		//data |= dynamic_cast<device_image_interface *>(&m_ddp1)->exists() << 1;
-	}
+	if (m_reset)
+		data |= M6801_MODE_6;
+	else
+		data |= m_ddp1->exists() << 1; // Cassette in place 1
 
 	// NET RXD
 	data |= m_rxd << 3;
@@ -833,18 +837,22 @@ WRITE8_MEMBER( adam_state::ddp6801_p2_w )
 
     */
 
-	// track
-	m_track = !BIT(data, 2);
+	if(m_ddp0->exists())
+	{
+		m_ddp0->set_channel(!BIT(data, 2)); // Track select
+		if (!m_wr0) m_ddp0->output(BIT(data, 0) ? 1.0 : -1.0); // write data
+	}
 
-	// write data
-	if (!m_wr0) m_ddp0->output(BIT(data, 0) ? 1.0 : -1.0);
-	if (!m_wr1) m_ddp1->output(BIT(data, 0) ? 1.0 : -1.0);
+	if(m_ddp1->exists())
+	{
+		m_ddp1->set_channel(!BIT(data, 2));
+		if (!m_wr1) m_ddp1->output(BIT(data, 0) ? 1.0 : -1.0);
+	}
 
 	// NET TXD
 	adamnet_txd_w(ADAMNET_DDP, BIT(data, 4));
 }
 
-
 //-------------------------------------------------
 //  ddp6801_p4_r -
 //-------------------------------------------------
@@ -868,21 +876,24 @@ READ8_MEMBER( adam_state::ddp6801_p4_r )
 
 	UINT8 data = 0;
 
-	// motion sense 0
-	data |= ((m_ddp0->get_state() & CASSETTE_MASK_UISTATE) != CASSETTE_STOPPED) << 3;
+	// drive 0
+	if(m_ddp0->exists())
+	{
+		data |= ((m_ddp0->get_state() & CASSETTE_MASK_UISTATE) != CASSETTE_STOPPED) << 3; // motion sense
+		data |= 1 << 5; // cassette in place
+		data |= (m_ddp0->input() < 0) << 7; // read data
+	}
 
-	// motion sense 1
-	data |= ((m_ddp1->get_state() & CASSETTE_MASK_UISTATE) != CASSETTE_STOPPED) << 4;
+	// drive 1
+	if(m_ddp1->exists())
+	{
+		data |= ((m_ddp1->get_state() & CASSETTE_MASK_UISTATE) != CASSETTE_STOPPED) << 4; // motion sense
+		data |= (m_ddp1->input() < 0) << 7; // read data
+	}
 
-	// cassette in place 0
-//  data |= dynamic_cast<device_image_interface *>(&m_ddp0)->exists() << 5;
-
-	// read data 0
+	// read data 0 (always 1)
 	data |= 0x40;
 
-	// read data 1
-	data |= ((m_ddp0->input() < 0) || (m_ddp1->input() < 0)) << 7;
-
 	return data;
 }
 
@@ -1162,12 +1173,9 @@ static TIMER_DEVICE_CALLBACK( paddle_tick )
 {
 	adam_state *state = timer.machine().driver_data<adam_state>();
 
-	coleco_scan_paddles(timer.machine(), &state->m_joy_status0, &state->m_joy_status1);
-
-    if (state->m_joy_status0 || state->m_joy_status1)
-	{
+	// TODO: improve irq behaviour (see drivers/coleco.c)
+	if (coleco_scan_paddles(timer.machine(), &state->m_joy_status0, &state->m_joy_status1))
 		device_set_input_line(state->m_maincpu, INPUT_LINE_IRQ0, HOLD_LINE);
-	}
 }
 
 
@@ -1197,7 +1205,7 @@ WRITE8_MEMBER( adam_state::joystick_w )
 
 READ8_MEMBER( adam_state::input1_r )
 {
-	return coleco_paddle1_read(machine(), m_joy_mode, m_joy_status0);
+	return coleco_paddle_read(machine(), 0, m_joy_mode, m_joy_status0);
 }
 
 
@@ -1207,7 +1215,7 @@ READ8_MEMBER( adam_state::input1_r )
 
 READ8_MEMBER( adam_state::input2_r )
 {
-	return coleco_paddle1_read(machine(), m_joy_mode, m_joy_status1);
+	return coleco_paddle_read(machine(), 1, m_joy_mode, m_joy_status1);
 }
 
 
@@ -1514,24 +1522,22 @@ static TMS9928A_INTERFACE(adam_tms9928a_interface)
 	DEVCB_LINE(adam_vdp_interrupt)
 };
 
-static SCREEN_UPDATE( adam )
-{
-	tms9928a_device *tms9928a = screen->machine().device<tms9928a_device>( "tms9928a" );
-
-	tms9928a->update( bitmap, cliprect );
-	return 0;
-}
-
-
 //-------------------------------------------------
 //  cassette_interface adam_cassette_interface
 //-------------------------------------------------
 
+static const struct CassetteOptions adam_cassette_options =
+{
+	2,		/* channels */
+	16,		/* bits per sample */
+	44100	/* sample frequency */
+};
+
 static const cassette_interface adam_cassette_interface =
 {
-	cassette_default_formats,
-	NULL,
-	(cassette_state)(CASSETTE_STOPPED | CASSETTE_MOTOR_ENABLED | CASSETTE_SPEAKER_MUTED),
+	coleco_adam_cassette_formats,
+	&adam_cassette_options,
+	(cassette_state)(CASSETTE_PLAY | CASSETTE_MOTOR_DISABLED | CASSETTE_SPEAKER_MUTED),
 	NULL,
 	NULL
 };
@@ -1580,7 +1586,7 @@ WRITE_LINE_MEMBER( adam_state::os3_w )
 {
 	if (state && !m_dma)
 	{
-		UINT8 *ram = ram_get_ptr(m_ram);
+		UINT8 *ram = m_ram->pointer();
 
 		if (!m_bwr)
 		{
@@ -1618,25 +1624,25 @@ static M6801_INTERFACE( master6801_intf )
 void adam_state::machine_start()
 {
 	// register for state saving
-	state_save_register_global(machine(), m_mioc);
-	state_save_register_global(machine(), m_game);
-	state_save_register_global(machine(), m_adamnet);
-	state_save_register_global_array(machine(), m_txd);
-	state_save_register_global(machine(), m_rxd);
-	state_save_register_global(machine(), m_reset);
-	state_save_register_global(machine(), m_ba);
-	state_save_register_global(machine(), m_dma);
-	state_save_register_global(machine(), m_bwr);
-	state_save_register_global(machine(), m_data_in);
-	state_save_register_global(machine(), m_data_out);
-	state_save_register_global(machine(), m_key_y);
-	state_save_register_global(machine(), m_joy_mode);
-	state_save_register_global(machine(), m_joy_status0);
-	state_save_register_global(machine(), m_joy_status1);
-	state_save_register_global(machine(), m_vdp_nmi);
-	state_save_register_global(machine(), m_wr0);
-	state_save_register_global(machine(), m_wr1);
-	state_save_register_global(machine(), m_track);
+	save_item(NAME(m_mioc));
+	save_item(NAME(m_game));
+	save_item(NAME(m_adamnet));
+	save_item(NAME(m_txd));
+	save_item(NAME(m_rxd));
+	save_item(NAME(m_reset));
+	save_item(NAME(m_ba));
+	save_item(NAME(m_dma));
+	save_item(NAME(m_bwr));
+	save_item(NAME(m_data_in));
+	save_item(NAME(m_data_out));
+	save_item(NAME(m_key_y));
+	save_item(NAME(m_joy_mode));
+	save_item(NAME(m_joy_status0));
+	save_item(NAME(m_joy_status1));
+	save_item(NAME(m_vdp_nmi));
+	save_item(NAME(m_wr0));
+	save_item(NAME(m_wr1));
+	save_item(NAME(m_track));
 }
 
 
@@ -1694,7 +1700,6 @@ static MACHINE_CONFIG_START( adam, adam_state )
 	MCFG_CPU_ADD(M6801_DDP_TAG, M6801, XTAL_4MHz)
 	MCFG_CPU_PROGRAM_MAP(ddp6801_mem)
 	MCFG_CPU_IO_MAP(ddp6801_io)
-	MCFG_DEVICE_DISABLE()
 
 	MCFG_CPU_ADD(M6801_PRN_TAG, M6801, XTAL_4MHz)
 	MCFG_CPU_PROGRAM_MAP(printer6801_mem)
@@ -1712,7 +1717,7 @@ static MACHINE_CONFIG_START( adam, adam_state )
 	// video hardware
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9928A, adam_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( adam )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9928a_device, screen_update )
 
 	// sound hardware
 	MCFG_SPEAKER_STANDARD_MONO("mono")
