@@ -164,7 +164,7 @@ static DRIVER_INIT( aleste )
 /* Memory is banked in 16k blocks. However, the multiface
 pages the memory in 8k blocks! The ROM can
 be paged into bank 0 and bank 3. */
-static ADDRESS_MAP_START(amstrad_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(amstrad_mem, AS_PROGRAM, 8, amstrad_state )
 	AM_RANGE(0x00000, 0x01fff) AM_READ_BANK("bank1") AM_WRITE_BANK("bank9")
 	AM_RANGE(0x02000, 0x03fff) AM_READ_BANK("bank2") AM_WRITE_BANK("bank10")
 	AM_RANGE(0x04000, 0x05fff) AM_READ_BANK("bank3") AM_WRITE_BANK("bank11")
@@ -178,8 +178,8 @@ ADDRESS_MAP_END
 /* I've handled the I/O ports in this way, because the ports
 are not fully decoded by the CPC h/w. Doing it this way means
 I can decode it myself and a lot of  software should work */
-static ADDRESS_MAP_START(amstrad_io, AS_IO, 8)
-	AM_RANGE(0x0000, 0xffff) AM_READWRITE( amstrad_cpc_io_r, amstrad_cpc_io_w )
+static ADDRESS_MAP_START(amstrad_io, AS_IO, 8, amstrad_state )
+	AM_RANGE(0x0000, 0xffff) AM_READWRITE(amstrad_cpc_io_r, amstrad_cpc_io_w )
 ADDRESS_MAP_END
 
 
@@ -317,9 +317,8 @@ INPUT_PORTS_END
 static INPUT_CHANGED( cpc_monitor_changed )
 {
 	running_machine &machine = field.machine();
-	const UINT8	*color_prom = NULL;
 
-	if ( (input_port_read(machine, "green_display")) & 0x01 )
+	if ( (machine.root_device().ioport("green_display")->read()) & 0x01 )
 	{
 		PALETTE_INIT_CALL( amstrad_cpc_green );
 	}
@@ -774,8 +773,8 @@ static const ay8910_interface ay8912_interface =
 {
 	AY8910_LEGACY_OUTPUT,
 	AY8910_DEFAULT_LOADS,
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, amstrad_psg_porta_read),	/* portA read */
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, amstrad_psg_porta_read),	/* portB read */
+	DEVCB_DRIVER_MEMBER(amstrad_state, amstrad_psg_porta_read),	/* portA read */
+	DEVCB_DRIVER_MEMBER(amstrad_state, amstrad_psg_porta_read),	/* portB read */
 	DEVCB_NULL,					/* portA write */
 	DEVCB_NULL					/* portB write */
 };
@@ -828,7 +827,7 @@ static const cassette_interface amstrad_cassette_interface =
 	cdt_cassette_formats,
 	NULL,
 	(cassette_state) (CASSETTE_STOPPED | CASSETTE_MOTOR_DISABLED | CASSETTE_SPEAKER_ENABLED),
-	NULL,
+	"cpc_cass",
 	NULL
 };
 
@@ -841,7 +840,7 @@ static const floppy_interface cpc6128_floppy_interface =
 	DEVCB_NULL,
 	FLOPPY_STANDARD_3_SSDD,
 	LEGACY_FLOPPY_OPTIONS_NAME(default),
-	NULL,
+	"floppy_3",
 	NULL
 };
 
@@ -891,10 +890,9 @@ static MACHINE_CONFIG_START( amstrad, amstrad_state )
 
     /* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_RAW_PARAMS( XTAL_16MHz, 1024, 32, 32 + 640 + 64, 312, 56 + 15, 200 + 15 )
-	MCFG_SCREEN_UPDATE(amstrad)
-	MCFG_SCREEN_EOF(amstrad)
+	MCFG_SCREEN_UPDATE_STATIC(amstrad)
+	MCFG_SCREEN_VBLANK_STATIC(amstrad)
 
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_ALWAYS_UPDATE)
 
@@ -914,16 +912,18 @@ static MACHINE_CONFIG_START( amstrad, amstrad_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	/* snapshot */
 	MCFG_SNAPSHOT_ADD("snapshot", amstrad, "sna", 0)
 
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, amstrad_cassette_interface )
+	MCFG_SOFTWARE_LIST_ADD("cass_list","cpc_cass")
 
 	MCFG_UPD765A_ADD("upd765", amstrad_upd765_interface)
 
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(cpc6128_floppy_interface)
+	MCFG_SOFTWARE_LIST_ADD("flop_list","cpc_flop")
 
 	MCFG_CPC_EXPANSION_SLOT_ADD("exp",cpc_exp_intf,cpc_exp_cards,NULL,NULL)
 
@@ -956,10 +956,9 @@ static MACHINE_CONFIG_START( cpcplus, amstrad_state )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_RAW_PARAMS( ( XTAL_40MHz * 2 ) / 5, 1024, 32, 32 + 640 + 64, 312, 56 + 15, 200 + 15 )
-	MCFG_SCREEN_UPDATE(amstrad)
-	MCFG_SCREEN_EOF(amstrad)
+	MCFG_SCREEN_UPDATE_STATIC(amstrad)
+	MCFG_SCREEN_VBLANK_STATIC(amstrad)
 
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_ALWAYS_UPDATE)
 
@@ -979,7 +978,7 @@ static MACHINE_CONFIG_START( cpcplus, amstrad_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	/* snapshot */
 	MCFG_SNAPSHOT_ADD("snapshot", amstrad, "sna", 0)
@@ -992,7 +991,7 @@ static MACHINE_CONFIG_START( cpcplus, amstrad_state )
 
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(cpc6128_floppy_interface)
 
-	MCFG_CPC_EXPANSION_SLOT_ADD("exp",cpc_exp_intf,cpcplus_exp_cards,NULL,NULL)
+	MCFG_CPC_EXPANSION_SLOT_ADD("exp",cpc_exp_intf,cpc_exp_cards,NULL,NULL)
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
@@ -1015,10 +1014,9 @@ static MACHINE_CONFIG_START( gx4000, amstrad_state )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_RAW_PARAMS( ( XTAL_40MHz * 2 ) / 5, 1024, 32, 32 + 640 + 64, 312, 56 + 15, 200 + 15 )
-	MCFG_SCREEN_UPDATE(amstrad)
-	MCFG_SCREEN_EOF(amstrad)
+	MCFG_SCREEN_UPDATE_STATIC(amstrad)
+	MCFG_SCREEN_VBLANK_STATIC(amstrad)
 
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_ALWAYS_UPDATE)
 
