@@ -115,23 +115,17 @@ Interrupts are not supported
 The Read/Write 2650 CPU Port-Commands do not appear to be connected to
 anything in hardware. No cartridge has been found which uses them.
 
- ******************************************************************************/
+******************************************************************************/
 
-#include <assert.h>
-#include "emu.h"
-#include "cpu/s2650/s2650.h"
 #include "includes/arcadia.h"
-#include "imagedev/cartslot.h"
 
-static ADDRESS_MAP_START( arcadia_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START( arcadia_mem, AS_PROGRAM, 8, arcadia_state )
 	AM_RANGE( 0x0000, 0x0fff) AM_ROM
 	AM_RANGE( 0x1800, 0x1aff) AM_READWRITE( arcadia_video_r, arcadia_video_w )
 	AM_RANGE( 0x2000, 0x2fff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( arcadia_io, AS_IO, 8)
-//{ S2650_CTRL_PORT,S2650_CTRL_PORT, },
-//{ S2650_DATA_PORT,S2650_DATA_PORT, },
+static ADDRESS_MAP_START( arcadia_io, AS_IO, 8, arcadia_state )
 	AM_RANGE( S2650_SENSE_PORT,S2650_SENSE_PORT) AM_READ( arcadia_vsync_r)
 ADDRESS_MAP_END
 
@@ -459,7 +453,7 @@ static PALETTE_INIT( arcadia )
 
 static DEVICE_IMAGE_LOAD( arcadia_cart )
 {
-	UINT8 *rom = image.device().machine().region("maincpu")->base();
+	UINT8 *rom = image.device().machine().root_device().memregion("maincpu")->base();
 	int size;
 
 	memset(rom, 0, 0x8000);
@@ -467,8 +461,8 @@ static DEVICE_IMAGE_LOAD( arcadia_cart )
 	{
 		size = image.length();
 
-		if (size > image.device().machine().region("maincpu")->bytes())
-			size = image.device().machine().region("maincpu")->bytes();
+		if (size > image.device().machine().root_device().memregion("maincpu")->bytes())
+			size = image.device().machine().root_device().memregion("maincpu")->bytes();
 
 		if (image.fread(rom, size) != size)
 			return IMAGE_INIT_FAIL;
@@ -543,10 +537,9 @@ static MACHINE_CONFIG_START( arcadia, arcadia_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(128+2*XPOS, 262)
 	MCFG_SCREEN_VISIBLE_AREA(0, 2*XPOS+128-1, 0, 262-1)
-	MCFG_SCREEN_UPDATE( arcadia )
+	MCFG_SCREEN_UPDATE_STATIC( arcadia )
 
 	MCFG_GFXDECODE( arcadia )
 	MCFG_PALETTE_LENGTH(ARRAY_LENGTH(arcadia_palette))
@@ -556,7 +549,7 @@ static MACHINE_CONFIG_START( arcadia, arcadia_state )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
-	MCFG_SOUND_ADD("custom", ARCADIA, 0)
+	MCFG_ARCADIA_SOUND_ADD("custom")
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.00)
 
 	/* cartridge */
@@ -740,13 +733,13 @@ ROM_END
 static DRIVER_INIT( arcadia )
 {
 	int i;
-	UINT8 *gfx=machine.region("gfx1")->base();
+	UINT8 *gfx=machine.root_device().memregion("gfx1")->base();
 	for (i=0; i<256; i++) gfx[i]=i;
 #if 0
 	// this is here to allow developement of some simple testroutines
 	// for a real console
 	{
-		UINT8 *rom=machine.region("maincpu")->base();
+		UINT8 *rom=machine.root_device().memregion("maincpu")->base();
 		/* this is a simple routine to display all rom characters
            on the display for a snapshot */
 		static const UINT8 prog[]={ // address 0 of course
