@@ -76,14 +76,14 @@ void get_ram_expansion_settings(address_space *space, int &ram_expansion_install
  *
  *************************************/
 
-static ADDRESS_MAP_START( astrocade_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( astrocade_mem, AS_PROGRAM, 8, astrocde_state )
 	AM_RANGE(0x0000, 0x0fff) AM_ROM AM_WRITE(astrocade_funcgen_w)
 	AM_RANGE(0x1000, 0x3fff) AM_ROM /* Star Fortress writes in here?? */
-	AM_RANGE(0x4000, 0x4fff) AM_RAM AM_BASE_MEMBER(astrocde_state, m_videoram) /* ASG */
+	AM_RANGE(0x4000, 0x4fff) AM_RAM AM_SHARE("videoram") /* ASG */
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( astrocade_io, AS_IO, 8 )
+static ADDRESS_MAP_START( astrocade_io, AS_IO, 8, astrocde_state )
 	AM_RANGE(0x00, 0x1f) AM_MIRROR(0xff00) AM_MASK(0xffff) AM_READWRITE(astrocade_data_chip_register_r, astrocade_data_chip_register_w)
 ADDRESS_MAP_END
 
@@ -91,7 +91,7 @@ static INPUT_CHANGED( set_write_protect )  // run when RAM expansion write prote
 {
 	int ram_expansion_installed = 0, write_protect_on = 0, expansion_ram_start = 0, expansion_ram_end = 0, shadow_ram_end = 0;
 	address_space *space = field.machine().device("maincpu")->memory().space(AS_PROGRAM);
-	UINT8 *expram = ram_get_ptr(field.machine().device("ram_tag"));
+	UINT8 *expram = field.machine().device<ram_device>("ram_tag")->pointer();
 
 	get_ram_expansion_settings(space, ram_expansion_installed, write_protect_on, expansion_ram_start, expansion_ram_end, shadow_ram_end);  // passing by reference
 
@@ -248,8 +248,7 @@ static MACHINE_CONFIG_START( astrocde, astrocde_state )
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_RAW_PARAMS(ASTROCADE_CLOCK, 455, 0, 352, 262, 0, 240)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
-	MCFG_SCREEN_UPDATE(astrocde)
+	MCFG_SCREEN_UPDATE_STATIC(astrocde)
 
 	MCFG_PALETTE_LENGTH(512)
 	MCFG_PALETTE_INIT(astrocde)
@@ -315,7 +314,7 @@ MACHINE_RESET( astrocde )
 {
     int ram_expansion_installed = 0, write_protect_on = 0, expansion_ram_start = 0, expansion_ram_end = 0, shadow_ram_end = 0;
     address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
-    UINT8 *expram = ram_get_ptr(machine.device("ram_tag"));
+    UINT8 *expram = machine.device<ram_device>("ram_tag")->pointer();
     space->unmap_readwrite(0x5000, 0xffff);  // unmap any previously installed expansion RAM
 
     get_ram_expansion_settings(space, ram_expansion_installed, write_protect_on, expansion_ram_start, expansion_ram_end, shadow_ram_end);  // passing by reference
@@ -337,14 +336,14 @@ MACHINE_RESET( astrocde )
 
 void get_ram_expansion_settings(address_space *space, int &ram_expansion_installed, int &write_protect_on, int &expansion_ram_start, int &expansion_ram_end, int &shadow_ram_end)
 {
-    if (input_port_read(space->machine(), "PROTECT") == 0x01)
+    if (space->machine().root_device().ioport("PROTECT")->read() == 0x01)
         write_protect_on = 1;
     else
         write_protect_on = 0;
 
     ram_expansion_installed = 1;
 
-    switch(input_port_read(space->machine(), "CFG"))  // check RAM expansion configuration and set address ranges
+    switch(space->machine().root_device().ioport("CFG")->read())  // check RAM expansion configuration and set address ranges
     {
         case 0x00:  // No RAM Expansion
              ram_expansion_installed = 0;
