@@ -125,7 +125,7 @@ void atom_state::bankswitch()
 {
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
 
-	UINT8 *eprom = machine().region(EXTROM_TAG)->base() + (m_eprom << 12);
+	UINT8 *eprom = memregion(EXTROM_TAG)->base() + (m_eprom << 12);
 
 	program->install_rom(0xa000, 0xafff, eprom);
 }
@@ -181,7 +181,7 @@ static ADDRESS_MAP_START( atom_mem, AS_PROGRAM, 8, atom_state )
 	AM_RANGE(0x0a00, 0x0a03) AM_MIRROR(0x1f8) AM_DEVREADWRITE_LEGACY(I8271_TAG, i8271_r, i8271_w)
 	AM_RANGE(0x0a04, 0x0a04) AM_MIRROR(0x1f8) AM_DEVREADWRITE_LEGACY(I8271_TAG, i8271_data_r, i8271_data_w)
 	AM_RANGE(0x0a05, 0x7fff) AM_RAM
-	AM_RANGE(0x8000, 0x97ff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0x8000, 0x97ff) AM_RAM AM_SHARE("video_ram")
 	AM_RANGE(0x9800, 0x9fff) AM_RAM
 	AM_RANGE(0xa000, 0xafff) AM_ROM AM_REGION(EXTROM_TAG, 0)
 	AM_RANGE(0xb000, 0xb003) AM_MIRROR(0x3fc) AM_DEVREADWRITE(INS8255_TAG, i8255_device, read, write)
@@ -208,9 +208,9 @@ ADDRESS_MAP_END
     INPUT_CHANGED( trigger_reset )
 -------------------------------------------------*/
 
-static INPUT_CHANGED( trigger_reset )
+INPUT_CHANGED_MEMBER( atom_state::trigger_reset )
 {
-	cputag_set_input_line(field.machine(), SY6502_TAG, INPUT_LINE_RESET, newval ? CLEAR_LINE : ASSERT_LINE);
+	m_maincpu->set_input_line(INPUT_LINE_RESET, newval ? CLEAR_LINE : ASSERT_LINE);
 }
 
 /*-------------------------------------------------
@@ -332,25 +332,12 @@ static INPUT_PORTS_START( atom )
 	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("REPT")         PORT_CODE(KEYCODE_RCONTROL)   PORT_CHAR(UCHAR_MAMEKEY(RCONTROL))
 
 	PORT_START("BRK")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("BREAK")        PORT_CODE(KEYCODE_ESC)   PORT_CHAR(UCHAR_MAMEKEY(ESC)) PORT_CHANGED(trigger_reset, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("BREAK")        PORT_CODE(KEYCODE_ESC)   PORT_CHAR(UCHAR_MAMEKEY(ESC)) PORT_CHANGED_MEMBER(DEVICE_SELF, atom_state, trigger_reset, 0)
 
 	PORT_START("ECONET")
 	// station ID (0-255)
 INPUT_PORTS_END
 
-/***************************************************************************
-    VIDEO
-***************************************************************************/
-
-/*-------------------------------------------------
-    SCREEN_UPDATE( atom )
--------------------------------------------------*/
-
-bool atom_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	return mc6847_update(m_vdg, &bitmap, &cliprect);
-}
-
 /***************************************************************************
     DEVICE CONFIGURATION
 ***************************************************************************/
@@ -380,10 +367,10 @@ WRITE8_MEMBER( atom_state::ppi_pa_w )
 	m_keylatch = data & 0x0f;
 
 	/* MC6847 */
-	mc6847_ag_w(m_vdg, BIT(data, 4));
-	mc6847_gm0_w(m_vdg, BIT(data, 5));
-	mc6847_gm1_w(m_vdg, BIT(data, 6));
-	mc6847_gm2_w(m_vdg, BIT(data, 7));
+	m_vdg->ag_w(BIT(data, 4));
+	m_vdg->gm0_w(BIT(data, 5));
+	m_vdg->gm1_w(BIT(data, 6));
+	m_vdg->gm2_w(BIT(data, 7));
 }
 
 READ8_MEMBER( atom_state::ppi_pb_r )
@@ -407,19 +394,19 @@ READ8_MEMBER( atom_state::ppi_pb_r )
 
 	switch (m_keylatch)
 	{
-	case 0: data &= input_port_read(machine(), "KEY0"); break;
-	case 1: data &= input_port_read(machine(), "KEY1"); break;
-	case 2: data &= input_port_read(machine(), "KEY2"); break;
-	case 3: data &= input_port_read(machine(), "KEY3"); break;
-	case 4: data &= input_port_read(machine(), "KEY4"); break;
-	case 5: data &= input_port_read(machine(), "KEY5"); break;
-	case 6: data &= input_port_read(machine(), "KEY6"); break;
-	case 7: data &= input_port_read(machine(), "KEY7"); break;
-	case 8: data &= input_port_read(machine(), "KEY8"); break;
-	case 9: data &= input_port_read(machine(), "KEY9"); break;
+	case 0: data &= ioport("KEY0")->read(); break;
+	case 1: data &= ioport("KEY1")->read(); break;
+	case 2: data &= ioport("KEY2")->read(); break;
+	case 3: data &= ioport("KEY3")->read(); break;
+	case 4: data &= ioport("KEY4")->read(); break;
+	case 5: data &= ioport("KEY5")->read(); break;
+	case 6: data &= ioport("KEY6")->read(); break;
+	case 7: data &= ioport("KEY7")->read(); break;
+	case 8: data &= ioport("KEY8")->read(); break;
+	case 9: data &= ioport("KEY9")->read(); break;
 	}
 
-	data &= input_port_read(machine(), "KEY10");
+	data &= ioport("KEY10")->read();
 
 	return data;
 }
@@ -450,10 +437,10 @@ READ8_MEMBER( atom_state::ppi_pc_r )
 	data |= (m_cassette->input() > 0.0) << 5;
 
 	/* keyboard RPT */
-	data |= BIT(input_port_read(machine(), "RPT"), 0) << 6;
+	data |= BIT(ioport("RPT")->read(), 0) << 6;
 
 	/* MC6847 FS */
-	data |= mc6847_fs_r(m_vdg) << 7;
+	data |= (m_vdg->fs_r() ? 1 : 0) << 7;
 
 	return data;
 }
@@ -483,7 +470,7 @@ WRITE8_MEMBER( atom_state::ppi_pc_w )
 	speaker_level_w(m_speaker, BIT(data, 2));
 
 	/* MC6847 CSS */
-	mc6847_css_w(m_vdg, BIT(data, 3));
+	m_vdg->css_w(BIT(data, 3));
 }
 
 static I8255_INTERFACE( ppi_intf )
@@ -502,12 +489,12 @@ static I8255_INTERFACE( ppi_intf )
 
 READ8_MEMBER( atom_state::printer_busy )
 {
-	return centronics_busy_r(m_centronics) << 7;
+	return m_centronics->busy_r() << 7;
 }
 
 WRITE8_MEMBER( atom_state::printer_data )
 {
-	centronics_data_w(m_centronics, 0, data & 0x7f);
+	m_centronics->write(space, 0, data & 0x7f);
 }
 
 static const via6522_interface via_intf =
@@ -522,7 +509,7 @@ static const via6522_interface via_intf =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_DEVICE_LINE(CENTRONICS_TAG, centronics_strobe_w),
+	DEVCB_DEVICE_LINE_MEMBER(CENTRONICS_TAG, centronics_device, strobe_w),
 	DEVCB_NULL,
 	DEVCB_CPU_INPUT_LINE(SY6502_TAG, INPUT_LINE_IRQ0)
 };
@@ -566,7 +553,6 @@ static const i8271_interface fdc_intf =
 
 static const centronics_interface atom_centronics_config =
 {
-	FALSE,
 	DEVCB_DEVICE_LINE_MEMBER(R6522_TAG, via6522_device, write_ca1),
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -632,27 +618,19 @@ static const cassette_interface atom_cassette_interface =
 
 READ8_MEMBER( atom_state::vdg_videoram_r )
 {
-	mc6847_as_w(m_vdg, BIT(m_video_ram[offset], 6));
-	mc6847_intext_w(m_vdg, BIT(m_video_ram[offset], 6));
-	mc6847_inv_w(m_vdg, BIT(m_video_ram[offset], 7));
+	if (offset == ~0) return 0xff;
+
+	m_vdg->as_w(BIT(m_video_ram[offset], 6));
+	m_vdg->intext_w(BIT(m_video_ram[offset], 6));
+	m_vdg->inv_w(BIT(m_video_ram[offset], 7));
 
 	return m_video_ram[offset];
 }
 
 static const mc6847_interface vdg_intf =
 {
-	DEVCB_DRIVER_MEMBER(atom_state, vdg_videoram_r),
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL
+	SCREEN_TAG,
+	DEVCB_DRIVER_MEMBER(atom_state, vdg_videoram_r)
 };
 
 /***************************************************************************
@@ -772,7 +750,7 @@ static DEVICE_IMAGE_LOAD( atom_cart )
 
 	/* With the following, we mirror the cart in the whole memory region */
 	for (i = 0; i < mirror; i++)
-		memcpy(image.device().machine().region(this_cart->region)->base() + this_cart->offset + i * size, temp_copy, size);
+		memcpy(image.device().machine().root_device().memregion(this_cart->region)->base() + this_cart->offset + i * size, temp_copy, size);
 
 	auto_free(image.device().machine(), temp_copy);
 
@@ -797,14 +775,8 @@ static MACHINE_CONFIG_START( atom, atom_state )
 	MCFG_CPU_PROGRAM_MAP(atom_mem)
 
 	/* video hardware */
-	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
-	MCFG_SCREEN_REFRESH_RATE(M6847_PAL_FRAMES_PER_SECOND)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
-	MCFG_SCREEN_SIZE(320, 25+192+26)
-	MCFG_SCREEN_VISIBLE_AREA(0, 319, 1, 239)
-
-	MCFG_MC6847_ADD(MC6847_TAG, vdg_intf)
-	MCFG_MC6847_TYPE(M6847_VERSION_ORIGINAL_PAL)
+    MCFG_SCREEN_MC6847_PAL_ADD(SCREEN_TAG, MC6847_TAG)
+	MCFG_MC6847_ADD(MC6847_TAG, MC6847_PAL, XTAL_4_433619MHz, vdg_intf)
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -817,7 +789,7 @@ static MACHINE_CONFIG_START( atom, atom_state )
 	MCFG_I8255_ADD(INS8255_TAG, ppi_intf)
 	MCFG_I8271_ADD(I8271_TAG, fdc_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(atom_floppy_interface)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, atom_centronics_config)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, atom_centronics_config)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, atom_cassette_interface)
 	MCFG_QUICKLOAD_ADD("quickload", atom_atm, "atm", 0)
 
