@@ -66,7 +66,6 @@
 
  ******************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "includes/avigo.h"
@@ -174,7 +173,7 @@ void avigo_state::refresh_memory(UINT8 bank, UINT8 chip_select)
 
 		case 0x01: // banked RAM
 			sprintf(bank_tag,"bank%d", bank);
-			memory_set_bankptr(machine(), bank_tag, m_ram_base + (((bank == 1 ? m_bank1_l : m_bank2_l) & 0x07)<<14));
+			membank(bank_tag)->set_base(m_ram_base + (((bank == 1 ? m_bank1_l : m_bank2_l) & 0x07)<<14));
 			space->install_readwrite_bank (bank * 0x4000, bank * 0x4000 + 0x3fff, bank_tag);
 			active_flash = -1;
 			break;
@@ -223,25 +222,32 @@ WRITE_LINE_MEMBER( avigo_state::com_interrupt )
 	refresh_ints();
 }
 
-
-
 static const ins8250_interface avigo_com_interface =
 {
-	1843200,
+	DEVCB_DEVICE_LINE_MEMBER("serport", serial_port_device, tx),
+	DEVCB_DEVICE_LINE_MEMBER("serport", rs232_port_device, dtr_w),
+	DEVCB_DEVICE_LINE_MEMBER("serport", rs232_port_device, rts_w),
 	DEVCB_DRIVER_LINE_MEMBER(avigo_state, com_interrupt),
-	NULL,
-	NULL,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
+static const rs232_port_interface avigo_serport_config =
+{
+	DEVCB_DEVICE_LINE_MEMBER("ns16550", ins8250_uart_device, rx_w),
+	DEVCB_DEVICE_LINE_MEMBER("ns16550", ins8250_uart_device, dcd_w),
+	DEVCB_DEVICE_LINE_MEMBER("ns16550", ins8250_uart_device, dsr_w),
+	DEVCB_DEVICE_LINE_MEMBER("ns16550", ins8250_uart_device, ri_w),
+	DEVCB_DEVICE_LINE_MEMBER("ns16550", ins8250_uart_device, cts_w)
+};
 
 void avigo_state::machine_reset()
 {
 	/* if is a cold start initialize flash contents */
 	if (!m_warm_start)
 	{
-		memcpy(m_flashes[0]->space()->get_read_ptr(0), machine().region("bios")->base() + 0x000000, 0x100000);
-		memcpy(m_flashes[1]->space()->get_read_ptr(0), machine().region("bios")->base() + 0x100000, 0x100000);
+		memcpy(m_flashes[0]->space()->get_read_ptr(0), memregion("bios")->base() + 0x000000, 0x100000);
+		memcpy(m_flashes[1]->space()->get_read_ptr(0), memregion("bios")->base() + 0x100000, 0x100000);
 	}
 
 	m_irq = 0;
@@ -258,15 +264,48 @@ void avigo_state::machine_reset()
 
 void avigo_state::machine_start()
 {
-	m_ram_base = (UINT8*)ram_get_ptr(m_ram);
+	m_ram_base = (UINT8*)m_ram->pointer();
 
 	// bank3 always first ram bank
-	memory_set_bankptr(machine(), "bank3", m_ram_base);
+	membank("bank3")->set_base(m_ram_base);
 
 	/* keep machine pointers to flash devices */
 	m_flashes[0] = machine().device<intelfsh8_device>("flash0");
 	m_flashes[1] = machine().device<intelfsh8_device>("flash1");
 	m_flashes[2] = machine().device<intelfsh8_device>("flash2");
+
+	machine().device<nvram_device>("nvram")->set_base(m_ram_base, m_ram->size());
+	m_warm_start = 1;
+
+	// register for state saving
+	save_item(NAME(m_key_line));
+	save_item(NAME(m_irq));
+	save_item(NAME(m_port2));
+	save_item(NAME(m_bank2_l));
+	save_item(NAME(m_bank2_h));
+	save_item(NAME(m_bank1_l));
+	save_item(NAME(m_bank1_h));
+	save_item(NAME(m_ad_control_status));
+	save_item(NAME(m_flash_at_0x4000));
+	save_item(NAME(m_flash_at_0x8000));
+	save_item(NAME(m_ad_value));
+	save_item(NAME(m_screen_column));
+	save_item(NAME(m_warm_start));
+
+	// save all flash contents
+	save_pointer(NAME((UINT8*)m_flashes[0]->space()->get_read_ptr(0)), 0x100000);
+	save_pointer(NAME((UINT8*)m_flashes[1]->space()->get_read_ptr(0)), 0x100000);
+	save_pointer(NAME((UINT8*)m_flashes[2]->space()->get_read_ptr(0)), 0x100000);
+
+	// register postload callback
+	machine().save().register_postload(save_prepost_delegate(FUNC(avigo_state::postload), this));
+}
+
+void avigo_state::postload()
+{
+	// refresh the bankswitch
+	refresh_memory(1, m_bank1_h & 0x07);
+	refresh_memory(2, m_bank2_h & 0x07);
 }
 
 static ADDRESS_MAP_START( avigo_mem , AS_PROGRAM, 8, avigo_state)
@@ -283,17 +322,17 @@ READ8_MEMBER(avigo_state::key_data_read_r)
 
 	if (!(m_key_line & 0x01))
 	{
-		data &= input_port_read(machine(), "LINE0");
+		data &= ioport("LINE0")->read();
 	}
 
 	if (!(m_key_line & 0x02))
 	{
-		data &= input_port_read(machine(), "LINE1");
+		data &= ioport("LINE1")->read();
 	}
 
 	if (!(m_key_line & 0x04))
 	{
-		data &= input_port_read(machine(), "LINE2");
+		data &= ioport("LINE2")->read();
 	}
 
 	/* bit 3 is cold/warm start */
@@ -416,14 +455,14 @@ WRITE8_MEMBER(avigo_state::ad_control_status_w)
 				if ((data & 0x08)!=0)
 				{
 					LOG(("a/d select x coordinate\n"));
-					LOG(("x coord: %d\n", input_port_read(machine(), "POSX")));
+					LOG(("x coord: %d\n", ioport("POSX")->read()));
 
 					/* on screen range 0x060->0x03a0 */
-					if (input_port_read(machine(), "LINE3") & 0x01)
+					if (ioport("LINE3")->read() & 0x01)
 					{
 						/* this might not be totally accurate because hitable screen
                         area may include the border around the screen! */
-						m_ad_value = input_port_read(machine(), "POSX");
+						m_ad_value = ioport("POSX")->read();
 					}
 					else
 					{
@@ -446,11 +485,11 @@ WRITE8_MEMBER(avigo_state::ad_control_status_w)
                     0x0350->0x03a6 is panel at bottom */
 
 					LOG(("a/d select y coordinate\n"));
-					LOG(("y coord: %d\n", input_port_read(machine(), "POSY")));
+					LOG(("y coord: %d\n", ioport("POSY")->read()));
 
-					if (input_port_read(machine(), "LINE3") & 0x01)
+					if (ioport("LINE3")->read() & 0x01)
 					{
-						m_ad_value = input_port_read(machine(), "POSY");
+						m_ad_value = ioport("POSY")->read();
 					}
 					else
 					{
@@ -602,7 +641,7 @@ static ADDRESS_MAP_START( avigo_io, AS_IO, 8, avigo_state)
 	AM_RANGE(0x010, 0x01f) AM_DEVREADWRITE("rtc", rp5c01_device, read, write)
 	AM_RANGE(0x028, 0x028) AM_WRITE( speaker_w )
 	AM_RANGE(0x02d, 0x02d) AM_READ( ad_data_r )
-	AM_RANGE(0x030, 0x037) AM_DEVREADWRITE_LEGACY("ns16550", ins8250_r, ins8250_w )
+	AM_RANGE(0x030, 0x037) AM_DEVREADWRITE("ns16550", ns16550_device, ins8250_r, ins8250_w )
 ADDRESS_MAP_END
 
 
@@ -620,7 +659,7 @@ INPUT_CHANGED_MEMBER( avigo_state::pen_irq )
 INPUT_CHANGED_MEMBER( avigo_state::pen_move_irq )
 {
 	// an irq is generated when the pen is down on the screen and is being moved
-	if (input_port_read(machine(), "LINE3") & 0x01)
+	if (ioport("LINE3")->read() & 0x01)
 	{
 		LOG(("pen move interrupt\n"));
 		m_irq |= (1<<6);
@@ -788,31 +827,6 @@ static TIMER_DEVICE_CALLBACK( avigo_1hz_timer )
 	state->refresh_ints();
 }
 
-static NVRAM_HANDLER(avigo)
-{
-	avigo_state *state = machine.driver_data<avigo_state>();
-	UINT32 ram_size = ram_get_size(state->m_ram);
-
-	if (read_or_write)
-	{
-		file->write(state->m_ram_base, ram_size);
-	}
-	else
-	{
-		if (file)
-		{
-			file->read(state->m_ram_base, ram_size);
-			state->m_warm_start = 1;
-		}
-		else
-		{
-			memset(state->m_ram_base, 0, ram_size);
-			state->m_warm_start = 0;
-		}
-	}
-}
-
-
 static QUICKLOAD_LOAD(avigo)
 {
 	avigo_state *state = image.device().machine().driver_data<avigo_state>();
@@ -864,6 +878,15 @@ static QUICKLOAD_LOAD(avigo)
 	return IMAGE_INIT_FAIL;
 }
 
+void avigo_state::nvram_init(nvram_device &nvram, void *base, size_t size)
+{
+	m_warm_start = 0;
+	memset(base, 0x00, size);
+}
+
+static SLOT_INTERFACE_START( avigo_com )
+	SLOT_INTERFACE("null_modem", NULL_MODEM)
+SLOT_INTERFACE_END
 
 static MACHINE_CONFIG_START( avigo, avigo_state )
 	/* basic machine hardware */
@@ -872,13 +895,14 @@ static MACHINE_CONFIG_START( avigo, avigo_state )
 	MCFG_CPU_IO_MAP(avigo_io)
 	MCFG_QUANTUM_TIME(attotime::from_hz(60))
 
-	MCFG_NS16550_ADD( "ns16550", avigo_com_interface )
+	MCFG_NS16550_ADD( "ns16550", avigo_com_interface, XTAL_1_8432MHz )
+	MCFG_RS232_PORT_ADD( "serport", avigo_serport_config, avigo_com, NULL, NULL )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", LCD)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(avigo_state, screen_update)
 	MCFG_SCREEN_SIZE(AVIGO_SCREEN_WIDTH, AVIGO_SCREEN_HEIGHT + AVIGO_PANEL_HEIGHT)
 	MCFG_SCREEN_VISIBLE_AREA(0, AVIGO_SCREEN_WIDTH-1, 0, AVIGO_SCREEN_HEIGHT + AVIGO_PANEL_HEIGHT -1)
 	MCFG_DEFAULT_LAYOUT(layout_avigo)
@@ -904,7 +928,7 @@ static MACHINE_CONFIG_START( avigo, avigo_state )
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("128K")
 
-	MCFG_NVRAM_HANDLER(avigo)
+	MCFG_NVRAM_ADD_CUSTOM_DRIVER("nvram", avigo_state, nvram_init)
 
 	// IRQ 1 is used for scan the pen and for cursor blinking
 	MCFG_TIMER_ADD_PERIODIC("scan_timer", avigo_scan_timer, attotime::from_hz(50))
@@ -1007,9 +1031,9 @@ ROM_START(avigo_it)
 ROM_END
 
 /*    YEAR  NAME    PARENT  COMPAT  MACHINE INPUT   INIT    COMPANY   FULLNAME */
-COMP(1997,	avigo,  	0,  		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA",         	0)
-COMP(1997,	avigo_de,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (German)",	0)
-COMP(1997,	avigo_fr,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (French)",	0)
-COMP(1997,	avigo_es,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (Spanish)",	0)
-COMP(1997,	avigo_it,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (Italian)",	0)
+COMP(1997,	avigo,  	0,  		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA",         	GAME_SUPPORTS_SAVE)
+COMP(1997,	avigo_de,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (German)",	GAME_SUPPORTS_SAVE)
+COMP(1997,	avigo_fr,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (French)",	GAME_SUPPORTS_SAVE)
+COMP(1997,	avigo_es,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (Spanish)",	GAME_SUPPORTS_SAVE)
+COMP(1997,	avigo_it,	avigo,		0,		avigo,	avigo,	0,		"Texas Instruments", "TI Avigo 10 PDA (Italian)",	GAME_SUPPORTS_SAVE)
 
