@@ -12,7 +12,6 @@
     Note: The $ key makes a square symbol.
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -26,11 +25,12 @@ public:
 	bcs3_state(const machine_config &mconfig, device_type type, const char *tag)
 		: driver_device(mconfig, type, tag),
 	m_maincpu(*this, "maincpu")
-	{ }
+	,
+		m_p_videoram(*this, "p_videoram"){ }
 
 	required_device<cpu_device> m_maincpu;
 	const UINT8 *m_p_chargen;
-	const UINT8 *m_p_videoram;
+	required_shared_ptr<const UINT8> m_p_videoram;
 	DECLARE_READ8_MEMBER(bcs3_keyboard_r);
 	virtual void machine_reset();
 	virtual void video_start();
@@ -41,23 +41,23 @@ READ8_MEMBER( bcs3_state::bcs3_keyboard_r )
 	UINT8 data = 0;
 
 	if (~offset & 0x01)
-		data |= input_port_read(machine(), "LINE0");
+		data |= ioport("LINE0")->read();
 	if (~offset & 0x02)
-		data |= input_port_read(machine(), "LINE1");
+		data |= ioport("LINE1")->read();
 	if (~offset & 0x04)
-		data |= input_port_read(machine(), "LINE2");
+		data |= ioport("LINE2")->read();
 	if (~offset & 0x08)
-		data |= input_port_read(machine(), "LINE3");
+		data |= ioport("LINE3")->read();
 	if (~offset & 0x10)
-		data |= input_port_read(machine(), "LINE4");
+		data |= ioport("LINE4")->read();
 	if (~offset & 0x20)
-		data |= input_port_read(machine(), "LINE5");
+		data |= ioport("LINE5")->read();
 	if (~offset & 0x40)
-		data |= input_port_read(machine(), "LINE6");
+		data |= ioport("LINE6")->read();
 	if (~offset & 0x80)
-		data |= input_port_read(machine(), "LINE7");
+		data |= ioport("LINE7")->read();
 	if (~offset & 0x100)
-		data |= input_port_read(machine(), "LINE8");
+		data |= ioport("LINE8")->read();
 
 	return data;
 }
@@ -68,7 +68,7 @@ static ADDRESS_MAP_START(bcs3_mem, AS_PROGRAM, 8, bcs3_state)
 	AM_RANGE( 0x1000, 0x11ff ) AM_READ_PORT("LINE9")
 	AM_RANGE( 0x1200, 0x13ff ) AM_READ(bcs3_keyboard_r)
 	AM_RANGE( 0x3c00, 0xffff ) AM_RAM
-	AM_RANGE( 0x3c50, 0x3d9f ) AM_RAM AM_BASE(m_p_videoram)
+	AM_RANGE( 0x3c50, 0x3d9f ) AM_RAM AM_SHARE("p_videoram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START(bcs3a_mem, AS_PROGRAM, 8, bcs3_state)
@@ -77,7 +77,7 @@ static ADDRESS_MAP_START(bcs3a_mem, AS_PROGRAM, 8, bcs3_state)
 	AM_RANGE( 0x1000, 0x11ff ) AM_READ_PORT("LINE9")
 	AM_RANGE( 0x1200, 0x13ff ) AM_READ(bcs3_keyboard_r)
 	AM_RANGE( 0x3c00, 0xefff ) AM_RAM
-	AM_RANGE( 0x3c00, 0x5a7f ) AM_RAM AM_BASE(m_p_videoram)
+	AM_RANGE( 0x3c00, 0x5a7f ) AM_RAM AM_SHARE("p_videoram")
 	AM_RANGE( 0xf000, 0xf3ff ) AM_ROM
 ADDRESS_MAP_END
 
@@ -87,7 +87,7 @@ static ADDRESS_MAP_START(bcs3b_mem, AS_PROGRAM, 8, bcs3_state)
 	AM_RANGE( 0x1000, 0x11ff ) AM_READ_PORT("LINE9")
 	AM_RANGE( 0x1200, 0x13ff ) AM_READ(bcs3_keyboard_r)
 	AM_RANGE( 0x3c00, 0xefff ) AM_RAM
-	AM_RANGE( 0x3c00, 0x657f ) AM_RAM AM_BASE(m_p_videoram)
+	AM_RANGE( 0x3c00, 0x657f ) AM_RAM AM_SHARE("p_videoram")
 	AM_RANGE( 0xf000, 0xf3ff ) AM_ROM
 ADDRESS_MAP_END
 
@@ -97,7 +97,7 @@ static ADDRESS_MAP_START(bcs3c_mem, AS_PROGRAM, 8, bcs3_state)
 	AM_RANGE( 0x1000, 0x11ff ) AM_READ_PORT("LINE9")
 	AM_RANGE( 0x1200, 0x13ff ) AM_READ(bcs3_keyboard_r)
 	AM_RANGE( 0x3c00, 0xffff ) AM_RAM
-	AM_RANGE( 0x3c00, 0x5ab3 ) AM_RAM AM_BASE(m_p_videoram)
+	AM_RANGE( 0x3c00, 0x5ab3 ) AM_RAM AM_SHARE("p_videoram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( bcs3_io, AS_IO, 8, bcs3_state)
@@ -216,12 +216,12 @@ MACHINE_RESET_MEMBER(bcs3_state)
 
 VIDEO_START_MEMBER( bcs3_state )
 {
-	m_p_chargen = machine().region("chargen")->base();
+	m_p_chargen = memregion("chargen")->base();
 }
 
-static SCREEN_UPDATE( bcs3 )
+static SCREEN_UPDATE_IND16( bcs3 )
 {
-	bcs3_state *state = screen->machine().driver_data<bcs3_state>();
+	bcs3_state *state = screen.machine().driver_data<bcs3_state>();
 	UINT8 y,ra,chr,gfx,rat;
 	UINT16 sy=0,ma=0,x;
 
@@ -229,7 +229,7 @@ static SCREEN_UPDATE( bcs3 )
 	{
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 			rat = (ra + 1) & 7;
 
 			for (x = ma; x < ma + 28; x++)
@@ -262,9 +262,9 @@ static SCREEN_UPDATE( bcs3 )
 
 /* This has 100 lines of screen data. I'm assuming that it only shows a portion of this,
     with the cursor always in sight. */
-static SCREEN_UPDATE( bcs3a )
+static SCREEN_UPDATE_IND16( bcs3a )
 {
-	bcs3_state *state = screen->machine().driver_data<bcs3_state>();
+	bcs3_state *state = screen.machine().driver_data<bcs3_state>();
 	UINT8 y,ra,chr,gfx,rat;
 	UINT16 sy=0,ma=128,x;
 	UINT16 cursor = (state->m_p_videoram[0x7a] | (state->m_p_videoram[0x7b] << 8)) - 0x3c80;	// get cursor relative position
@@ -275,7 +275,7 @@ static SCREEN_UPDATE( bcs3a )
 	{
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 			rat = (ra + 1) & 7;
 
 			for (x = ma; x < ma + 29; x++)
@@ -306,9 +306,9 @@ static SCREEN_UPDATE( bcs3a )
 	return 0;
 }
 
-static SCREEN_UPDATE( bcs3b )
+static SCREEN_UPDATE_IND16( bcs3b )
 {
-	bcs3_state *state = screen->machine().driver_data<bcs3_state>();
+	bcs3_state *state = screen.machine().driver_data<bcs3_state>();
 	UINT8 y,ra,chr,gfx,rat;
 	UINT16 sy=0,ma=128,x;
 	UINT16 cursor = (state->m_p_videoram[0x7a] | (state->m_p_videoram[0x7b] << 8)) - 0x3c80;	// get cursor relative position
@@ -319,7 +319,7 @@ static SCREEN_UPDATE( bcs3b )
 	{
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 			rat = (ra + 1) & 7;
 
 			for (x = ma; x < ma + 40; x++)
@@ -350,9 +350,9 @@ static SCREEN_UPDATE( bcs3b )
 	return 0;
 }
 
-static SCREEN_UPDATE( bcs3c )
+static SCREEN_UPDATE_IND16( bcs3c )
 {
-	bcs3_state *state = screen->machine().driver_data<bcs3_state>();
+	bcs3_state *state = screen.machine().driver_data<bcs3_state>();
 	UINT8 y,ra,chr,gfx,rat;
 	UINT16 sy=0,ma=0xb4,x;
 	UINT16 cursor = (state->m_p_videoram[0x08] | (state->m_p_videoram[0x09] << 8)) - 0x3c80;	// get cursor relative position
@@ -363,7 +363,7 @@ static SCREEN_UPDATE( bcs3c )
 	{
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 			rat = (ra + 1) & 7;
 
 			for (x = ma; x < ma + 29; x++)
@@ -424,10 +424,9 @@ static MACHINE_CONFIG_START( bcs3, bcs3_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(28*8, 12*10)
 	MCFG_SCREEN_VISIBLE_AREA(0,28*8-1,0,12*10-1)
-	MCFG_SCREEN_UPDATE(bcs3)
+	MCFG_SCREEN_UPDATE_STATIC(bcs3)
 	MCFG_GFXDECODE(bcs3)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(black_and_white)
@@ -441,7 +440,7 @@ static MACHINE_CONFIG_DERIVED( bcs3a, bcs3 )
 	MCFG_SCREEN_MODIFY("screen")
 	MCFG_SCREEN_SIZE(29*8, 12*10)
 	MCFG_SCREEN_VISIBLE_AREA(0,29*8-1,0,12*10-1)
-	MCFG_SCREEN_UPDATE(bcs3a)
+	MCFG_SCREEN_UPDATE_STATIC(bcs3a)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( bcs3b, bcs3 )
@@ -452,7 +451,7 @@ static MACHINE_CONFIG_DERIVED( bcs3b, bcs3 )
 	MCFG_SCREEN_MODIFY("screen")
 	MCFG_SCREEN_SIZE(40*8, 24*10)
 	MCFG_SCREEN_VISIBLE_AREA(0,40*8-1,0,24*10-1)
-	MCFG_SCREEN_UPDATE(bcs3b)
+	MCFG_SCREEN_UPDATE_STATIC(bcs3b)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( bcs3c, bcs3 )
@@ -463,7 +462,7 @@ static MACHINE_CONFIG_DERIVED( bcs3c, bcs3 )
 	MCFG_SCREEN_MODIFY("screen")
 	MCFG_SCREEN_SIZE(29*8, 12*10)
 	MCFG_SCREEN_VISIBLE_AREA(0,29*8-1,0,12*10-1)
-	MCFG_SCREEN_UPDATE(bcs3c)
+	MCFG_SCREEN_UPDATE_STATIC(bcs3c)
 MACHINE_CONFIG_END
 
 /* ROM definition */
