@@ -23,7 +23,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -67,7 +66,7 @@ void bw12_state::bankswitch()
 
 	case 2: /* BK1 */
 	case 3: /* BK2 */
-		if (ram_get_size(m_ram) > 64*1024)
+		if (m_ram->size() > 64*1024)
 		{
 			program->install_readwrite_bank(0x0000, 0x7fff, "bank1");
 		}
@@ -78,7 +77,7 @@ void bw12_state::bankswitch()
 		break;
 	}
 
-	memory_set_bank(machine(), "bank1", m_bank);
+	membank("bank1")->set_entry(m_bank);
 }
 
 void bw12_state::floppy_motor_off()
@@ -194,7 +193,7 @@ READ8_MEMBER( bw12_state::ls259_r )
 static ADDRESS_MAP_START( bw12_mem, AS_PROGRAM, 8, bw12_state )
 	AM_RANGE(0x0000, 0x7fff) AM_RAMBANK("bank1")
 	AM_RANGE(0x8000, 0xf7ff) AM_RAM
-	AM_RANGE(0xf800, 0xffff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0xf800, 0xffff) AM_RAM AM_SHARE("video_ram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( bw12_io, AS_IO, 8, bw12_state )
@@ -362,6 +361,7 @@ INPUT_PORTS_END
 static MC6845_UPDATE_ROW( bw12_update_row )
 {
 	bw12_state *state = device->machine().driver_data<bw12_state>();
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
 
 	int column, bit;
 
@@ -381,7 +381,7 @@ static MC6845_UPDATE_ROW( bw12_update_row )
 			int x = (column * 8) + bit;
 			int color = BIT(data, 7);
 
-			*BITMAP_ADDR16(bitmap, y, x) = color;
+			bitmap.pix32(y, x) = palette[color];
 
 			data <<= 1;
 		}
@@ -405,14 +405,7 @@ static const mc6845_interface bw12_mc6845_interface =
 void bw12_state::video_start()
 {
 	/* find memory regions */
-	m_char_rom = machine().region("chargen")->base();
-}
-
-bool bw12_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	m_crtc->update( &bitmap, &cliprect);
-
-	return 0;
+	m_char_rom = memregion("chargen")->base();
 }
 
 /* UPD765 Interface */
@@ -469,9 +462,9 @@ READ8_MEMBER( bw12_state::pia_pa_r )
 
 	UINT8 data = 0;
 
-	data |= centronics_busy_r(m_centronics);
-	data |= (centronics_fault_r(m_centronics) << 1);
-	data |= (centronics_pe_r(m_centronics) << 2);
+	data |= m_centronics->busy_r();
+	data |= (m_centronics->fault_r() << 1);
+	data |= (m_centronics->pe_r() << 2);
 	data |= (m_motor_on << 3);
 	data |= (m_pit_out2 << 4);
 	data |= (m_key_stb << 5);
@@ -504,13 +497,13 @@ static const pia6821_interface pia_intf =
 {
 	DEVCB_DRIVER_MEMBER(bw12_state, pia_pa_r),					/* port A input */
 	DEVCB_NULL,													/* port B input */
-	DEVCB_DEVICE_LINE(CENTRONICS_TAG, centronics_ack_r),		/* CA1 input */
+	DEVCB_DEVICE_LINE_MEMBER(CENTRONICS_TAG, centronics_device, ack_r),		/* CA1 input */
 	DEVCB_DRIVER_LINE_MEMBER(bw12_state, pia_cb1_r),			/* CB1 input */
 	DEVCB_NULL,													/* CA2 input */
 	DEVCB_NULL,													/* CB2 input */
 	DEVCB_NULL, 												/* port A output */
-	DEVCB_DEVICE_HANDLER(CENTRONICS_TAG, centronics_data_w),	/* port B output */
-	DEVCB_DEVICE_LINE(CENTRONICS_TAG, centronics_strobe_w),		/* CA2 output */
+	DEVCB_DEVICE_MEMBER(CENTRONICS_TAG, centronics_device, write),	/* port B output */
+	DEVCB_DEVICE_LINE_MEMBER(CENTRONICS_TAG, centronics_device, strobe_w),		/* CA2 output */
 	DEVCB_DRIVER_LINE_MEMBER(bw12_state, pia_cb2_w),			/* CB2 output */
 	DEVCB_CPU_INPUT_LINE(Z80_TAG, INPUT_LINE_IRQ0),				/* IRQA output */
 	DEVCB_CPU_INPUT_LINE(Z80_TAG, INPUT_LINE_IRQ0)				/* IRQB output */
@@ -520,7 +513,6 @@ static const pia6821_interface pia_intf =
 
 static const centronics_interface bw12_centronics_intf =
 {
-	0,													/* is IBM PC? */
 	DEVCB_DEVICE_LINE_MEMBER(PIA6821_TAG, pia6821_device, ca1_w),		/* ACK output */
 	DEVCB_NULL,											/* BUSY output */
 	DEVCB_NULL											/* NOT BUSY output */
@@ -587,12 +579,12 @@ static const struct pit8253_config pit_intf =
 
 READ_LINE_MEMBER( bw12_state::ay3600_shift_r )
 {
-	return BIT(input_port_read(machine(), "MODIFIERS"), 0);
+	return BIT(ioport("MODIFIERS")->read(), 0);
 }
 
 READ_LINE_MEMBER( bw12_state::ay3600_control_r )
 {
-	return BIT(input_port_read(machine(), "MODIFIERS"), 1);
+	return BIT(ioport("MODIFIERS")->read(), 1);
 }
 
 WRITE_LINE_MEMBER( bw12_state::ay3600_data_ready_w )
@@ -642,21 +634,21 @@ static AY3600_INTERFACE( bw12_ay3600_intf )
 void bw12_state::machine_start()
 {
 	/* setup memory banking */
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(Z80_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, ram_get_ptr(m_ram), 0);
-	memory_configure_bank(machine(), "bank1", 2, 2, ram_get_ptr(m_ram) + 0x10000, 0x8000);
+	membank("bank1")->configure_entry(0, memregion(Z80_TAG)->base());
+	membank("bank1")->configure_entry(1, m_ram->pointer());
+	membank("bank1")->configure_entries(2, 2, m_ram->pointer() + 0x10000, 0x8000);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_pit_out2);
-	state_save_register_global_array(machine(), m_key_data);
-	state_save_register_global(machine(), m_key_sin);
-	state_save_register_global(machine(), m_key_stb);
-	state_save_register_global(machine(), m_key_shift);
-	state_save_register_global(machine(), m_fdc_int);
-	state_save_register_global(machine(), m_motor_on);
-	state_save_register_global(machine(), m_motor0);
-	state_save_register_global(machine(), m_motor1);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_pit_out2));
+	save_item(NAME(m_key_data));
+	save_item(NAME(m_key_sin));
+	save_item(NAME(m_key_stb));
+	save_item(NAME(m_key_shift));
+	save_item(NAME(m_fdc_int));
+	save_item(NAME(m_motor_on));
+	save_item(NAME(m_motor0));
+	save_item(NAME(m_motor1));
 }
 
 void bw12_state::machine_reset()
@@ -780,7 +772,7 @@ static MACHINE_CONFIG_START( common, bw12_state )
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DEVICE(MC6845_TAG, mc6845_device, screen_update)
 	MCFG_SCREEN_SIZE(640, 200)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 200-1)
 
@@ -805,7 +797,7 @@ static MACHINE_CONFIG_START( common, bw12_state )
 	MCFG_AY3600_ADD(AY3600PRO002_TAG, 0, bw12_ay3600_intf)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, bw12_centronics_intf)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, bw12_centronics_intf)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( bw12, common )
