@@ -31,7 +31,7 @@
 
 int bw2_state::get_ramdisk_size()
 {
-	return input_port_read(machine(), "RAMCARD") * 256;
+	return ioport("RAMCARD")->read() * 256;
 }
 
 /* Memory */
@@ -57,7 +57,7 @@ void bw2_state::bankswitch(UINT8 data)
 
 	m_bank = data & 0x07;
 
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 64 * 1024:
 		max_ram_bank = BANK_RAM1;
@@ -115,7 +115,7 @@ void bw2_state::bankswitch(UINT8 data)
 		break;
 	}
 
-	memory_set_bank(machine(), "bank1", m_bank);
+	membank("bank1")->set_entry(m_bank);
 }
 
 void bw2_state::ramcard_bankswitch(UINT8 data)
@@ -139,7 +139,7 @@ void bw2_state::ramcard_bankswitch(UINT8 data)
 
 	m_bank = data & 0x07;
 
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 64 * 1024:
 	case 96 * 1024:
@@ -195,7 +195,7 @@ void bw2_state::ramcard_bankswitch(UINT8 data)
 		break;
 	}
 
-	memory_set_bank(machine(), "bank1", m_bank);
+	membank("bank1")->set_entry(m_bank);
 }
 
 WRITE8_MEMBER( bw2_state::ramcard_bank_w )
@@ -214,8 +214,8 @@ WRITE8_MEMBER( bw2_state::ramcard_bank_w )
 		program->install_readwrite_bank(0x0000, 0x7fff, "bank1");
 	}
 
-	memory_configure_bank(machine(), "bank1", BANK_RAMCARD_RAM, 1, m_ramcard_ram + bank_offset, 0);
-	memory_set_bank(machine(), "bank1", m_bank);
+	membank("bank1")->configure_entry(BANK_RAMCARD_RAM, m_ramcard_ram + bank_offset);
+	membank("bank1")->set_entry(m_bank);
 }
 
 /* Floppy */
@@ -250,7 +250,7 @@ static ADDRESS_MAP_START( bw2_io, AS_IO, 8, bw2_state )
 //  AM_RANGE( 0x30, 0x3f ) SLOT
 	AM_RANGE( 0x40, 0x40 ) AM_DEVREADWRITE(I8251_TAG, i8251_device, data_r, data_w)
 	AM_RANGE( 0x41, 0x41 ) AM_DEVREADWRITE(I8251_TAG, i8251_device, status_r, control_w)
-	AM_RANGE( 0x50, 0x50 ) AM_DEVWRITE_LEGACY(CENTRONICS_TAG, centronics_data_w)
+	AM_RANGE( 0x50, 0x50 ) AM_DEVWRITE(CENTRONICS_TAG, centronics_device, write)
 	AM_RANGE( 0x60, 0x63 ) AM_DEVREADWRITE_LEGACY(WD2797_TAG, wd17xx_r, wd17xx_w)
 //  AM_RANGE( 0x70, 0x7f ) MODEMSEL
 ADDRESS_MAP_END
@@ -431,7 +431,7 @@ WRITE8_MEMBER( bw2_state::ppi_pa_w )
 		wd17xx_set_drive(m_fdc, m_drive);
 	}
 
-	centronics_strobe_w(m_centronics, BIT(data, 7));
+	m_centronics->strobe_w(BIT(data, 7));
 }
 
 READ8_MEMBER( bw2_state::ppi_pb_r )
@@ -455,7 +455,7 @@ READ8_MEMBER( bw2_state::ppi_pb_r )
 
 	if (m_kb_row <= 9)
 	{
-		data = input_port_read(machine(), rownames[m_kb_row]);
+		data = ioport(rownames[m_kb_row])->read();
 	}
 
 	return data;
@@ -495,7 +495,7 @@ READ8_MEMBER( bw2_state::ppi_pc_r )
 
 	UINT8 data = 0;
 
-	data |= centronics_busy_r(m_centronics) << 4;
+	data |= m_centronics->busy_r() << 4;
 	data |= m_mfdbk << 5;
 
 	data |= floppy_wpt_r(m_drive ? m_floppy1 : m_floppy0) << 7;
@@ -537,14 +537,14 @@ static const struct pit8253_config pit_intf =
 {
 	{
 		{
-			XTAL_4MHz,	/* 8251 USART TXC, RXC */
+			XTAL_16MHz/4,	/* 8251 USART TXC, RXC */
 			DEVCB_LINE_VCC,
 			DEVCB_DRIVER_LINE_MEMBER(bw2_state, pit_out0_w)
 		},
 		{
 			11000,		/* LCD controller */
 			DEVCB_LINE_VCC,
-			DEVCB_LINE(pit8253_clk2_w)
+			DEVCB_DEVICE_LINE(PIT8253_TAG, pit8253_clk2_w)
 		},
 		{
 			0,		/* Floppy /MTRON */
@@ -563,16 +563,9 @@ static PALETTE_INIT( bw2 )
     palette_set_color_rgb(machine, 1, 0x31, 0x39, 0x10);
 }
 
-bool bw2_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	m_lcdc->update_screen(&bitmap, &cliprect);
-
-	return 0;
-}
-
 static MSM6255_CHAR_RAM_READ( bw2_charram_r )
 {
-	bw2_state *state =  device->machine().driver_data<bw2_state>();
+	bw2_state *state = device->machine().driver_data<bw2_state>();
 
 	return state->m_video_ram[ma & 0x3fff];
 }
@@ -606,9 +599,9 @@ static const floppy_interface bw2_floppy_interface =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	FLOPPY_STANDARD_5_25_DSHD,
+	FLOPPY_STANDARD_3_5_SSDD, // Teac FD-35
 	LEGACY_FLOPPY_OPTIONS_NAME(bw2),
-	"floppy_5_25",
+	"floppy_3_5",
 	NULL
 };
 
@@ -625,24 +618,22 @@ static const wd17xx_interface fdc_intf =
 void bw2_state::machine_start()
 {
 	/* allocate memory */
-	m_work_ram = auto_alloc_array(machine(), UINT8, ram_get_size(m_ram));
 	m_video_ram = auto_alloc_array(machine(), UINT8, BW2_VIDEORAM_SIZE);
 	m_ramcard_ram = auto_alloc_array(machine(), UINT8, BW2_RAMCARD_SIZE);
 
 	/* memory banking */
-	memory_configure_bank(machine(), "bank1", BANK_RAM1, 1, m_work_ram, 0);
-	memory_configure_bank(machine(), "bank1", BANK_VRAM, 1, m_video_ram, 0);
-	memory_configure_bank(machine(), "bank1", BANK_ROM, 1, machine().region("ic1")->base(), 0);
+	membank("bank1")->configure_entry(BANK_RAM1, m_ram->pointer());
+	membank("bank1")->configure_entry(BANK_VRAM, m_video_ram);
+	membank("bank1")->configure_entry(BANK_ROM, memregion("ic1")->base());
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_kb_row);
-	state_save_register_global_pointer(machine(), m_work_ram, ram_get_size(m_ram));
-	state_save_register_global_pointer(machine(), m_ramcard_ram, BW2_RAMCARD_SIZE);
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_drive);
-	state_save_register_global(machine(), m_mtron);
-	state_save_register_global(machine(), m_mfdbk);
-	state_save_register_global_pointer(machine(), m_video_ram, BW2_VIDEORAM_SIZE);
+	save_item(NAME(m_kb_row));
+	save_pointer(NAME(m_ramcard_ram), BW2_RAMCARD_SIZE);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_drive));
+	save_item(NAME(m_mtron));
+	save_item(NAME(m_mfdbk));
+	save_pointer(NAME(m_video_ram), BW2_VIDEORAM_SIZE);
 }
 
 void bw2_state::machine_reset()
@@ -653,10 +644,10 @@ void bw2_state::machine_reset()
 	{
 		// RAMCARD installed
 
-		memory_configure_bank(machine(), "bank1", BANK_RAMCARD_ROM, 1, machine().region("ramcard")->base(), 0);
-		memory_configure_bank(machine(), "bank1", BANK_RAM3, 2, m_work_ram + 0x8000, 0x8000);
-		memory_configure_bank(machine(), "bank1", BANK_RAMCARD_RAM, 1, m_ramcard_ram, 0);
-		memory_configure_bank(machine(), "bank1", BANK_RAM6, 1, m_work_ram + 0x18000, 0);
+		membank("bank1")->configure_entry(BANK_RAMCARD_ROM, memregion("ramcard")->base());
+		membank("bank1")->configure_entries(BANK_RAM3, 2, m_ram->pointer() + 0x8000, 0x8000);
+		membank("bank1")->configure_entry(BANK_RAMCARD_RAM, m_ramcard_ram);
+		membank("bank1")->configure_entry(BANK_RAM6, m_ram->pointer() + 0x18000);
 
 		io->install_write_handler(0x30, 0x30, 0, 0x0f, write8_delegate(FUNC(bw2_state::ramcard_bank_w), this), 0);
 	}
@@ -664,12 +655,12 @@ void bw2_state::machine_reset()
 	{
 		// no RAMCARD
 
-		memory_configure_bank(machine(), "bank1", BANK_RAM2, 5, m_work_ram + 0x8000, 0x8000);
+		membank("bank1")->configure_entries(BANK_RAM2, 5, m_ram->pointer() + 0x8000, 0x8000);
 
 		io->unmap_write(0x30, 0x30, 0, 0x0f);
 	}
 
-	memory_set_bank(machine(), "bank1", BANK_ROM);
+	membank("bank1")->set_entry(BANK_ROM);
 }
 
 static MACHINE_CONFIG_START( bw2, bw2_state )
@@ -681,7 +672,7 @@ static MACHINE_CONFIG_START( bw2, bw2_state )
 	// video hardware
 	MCFG_SCREEN_ADD( SCREEN_TAG, LCD )
 	MCFG_SCREEN_REFRESH_RATE( 60 )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
+	MCFG_SCREEN_UPDATE_DEVICE( MSM6255_TAG, msm6255_device, screen_update )
 	MCFG_SCREEN_SIZE( 640, 200 )
 	MCFG_SCREEN_VISIBLE_AREA( 0, 640-1, 0, 200-1 )
 	MCFG_DEFAULT_LAYOUT( layout_lcd )
@@ -693,7 +684,7 @@ static MACHINE_CONFIG_START( bw2, bw2_state )
 	MCFG_PIT8253_ADD(PIT8253_TAG, pit_intf)
 	MCFG_I8255A_ADD(I8255A_TAG, ppi_intf)
 	MCFG_MSM6255_ADD(MSM6255_TAG, XTAL_16MHz, lcdc_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_I8251_ADD(I8251_TAG, default_i8251_interface)
 	MCFG_WD2797_ADD(WD2797_TAG, fdc_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(bw2_floppy_interface)
