@@ -178,9 +178,6 @@ to use an EEPROM reader, in order to obtain a dump of the whole content.
 #include "machine/6526cia.h"
 
 #include "machine/cbmipt.h"
-#include "machine/c1541.h"
-#include "machine/c1571.h"
-#include "machine/c1581.h"
 #include "video/vic6567.h"
 #include "video/vdc8563.h"
 
@@ -191,7 +188,7 @@ to use an EEPROM reader, in order to obtain a dump of the whole content.
 #include "machine/cbmiec.h"
 
 #include "includes/c128.h"
-#include "includes/c64.h"
+#include "includes/c64_legacy.h"
 
 
 /*************************************
@@ -214,10 +211,10 @@ to use an EEPROM reader, in order to obtain a dump of the whole content.
  * 0x0000-0xedff ram (dram bank 1?)
  * 0xe000-0xffff ram as bank 0
  */
-static ADDRESS_MAP_START(c128_z80_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START(c128_z80_mem , AS_PROGRAM, 8, c128_state )
 #if 1
-	AM_RANGE(0x0000, 0x0fff) AM_READ_BANK("bank10") AM_WRITE(c128_write_0000)
-	AM_RANGE(0x1000, 0xbfff) AM_READ_BANK("bank11") AM_WRITE(c128_write_1000)
+	AM_RANGE(0x0000, 0x0fff) AM_READ_BANK("bank10") AM_WRITE_LEGACY(c128_write_0000)
+	AM_RANGE(0x1000, 0xbfff) AM_READ_BANK("bank11") AM_WRITE_LEGACY(c128_write_1000)
 	AM_RANGE(0xc000, 0xffff) AM_RAM
 #else
 	/* best to do reuse bankswitching numbers */
@@ -229,41 +226,41 @@ static ADDRESS_MAP_START(c128_z80_mem , AS_PROGRAM, 8)
 	AM_RANGE(0xc000, 0xdfff) AM_RAMBANK("bank6")
 	AM_RANGE(0xe000, 0xefff) AM_RAMBANK("bank7")
 	AM_RANGE(0xf000, 0xfeff) AM_RAMBANK("bank8")
-	AM_RANGE(0xff00, 0xff04) AM_READWRITE(c128_mmu8722_ff00_r, c128_mmu8722_ff00_w)
+	AM_RANGE(0xff00, 0xff04) AM_READWRITE_LEGACY(c128_mmu8722_ff00_r, c128_mmu8722_ff00_w)
 	AM_RANGE(0xff05, 0xffff) AM_RAMBANK("bank9")
 #endif
 
 #if 0
 	AM_RANGE(0x10000, 0x1ffff) AM_WRITEONLY
 	AM_RANGE(0x20000, 0xfffff) AM_WRITEONLY	   /* or nothing */
-	AM_RANGE(0x100000, 0x107fff) AM_BASE_MEMBER(c128_state, m_c128_basic)	/* maps to 0x4000 */
-	AM_RANGE(0x108000, 0x109fff) AM_BASE_MEMBER(c128_state, m_basic)	/* maps to 0xa000 */
-	AM_RANGE(0x10a000, 0x10bfff) AM_BASE_MEMBER(c128_state, m_kernal)	/* maps to 0xe000 */
-	AM_RANGE(0x10c000, 0x10cfff) AM_BASE_MEMBER(c128_state, m_editor)
-	AM_RANGE(0x10d000, 0x10dfff) AM_BASE_MEMBER(c128_state, m_z80)		/* maps to z80 0 */
-	AM_RANGE(0x10e000, 0x10ffff) AM_BASE_MEMBER(c128_state, m_c128_kernal)
-	AM_RANGE(0x110000, 0x117fff) AM_BASE_MEMBER(c128_state, m_internal_function)
-	AM_RANGE(0x118000, 0x11ffff) AM_BASE_MEMBER(c128_state, m_external_function)
-	AM_RANGE(0x120000, 0x120fff) AM_BASE_MEMBER(c128_state, m_chargen)
-	AM_RANGE(0x121000, 0x121fff) AM_BASE_MEMBER(c128_state, m_c128_chargen)
-	AM_RANGE(0x122000, 0x1227ff) AM_BASE_MEMBER(c128_state, m_colorram)
-	AM_RANGE(0x122800, 0x1327ff) AM_BASE_MEMBER(c128_state, m_vdcram)
+	AM_RANGE(0x100000, 0x107fff) AM_SHARE("c128_basic")	/* maps to 0x4000 */
+	AM_RANGE(0x108000, 0x109fff) AM_SHARE("basic")	/* maps to 0xa000 */
+	AM_RANGE(0x10a000, 0x10bfff) AM_SHARE("kernal")	/* maps to 0xe000 */
+	AM_RANGE(0x10c000, 0x10cfff) AM_SHARE("editor")
+	AM_RANGE(0x10d000, 0x10dfff) AM_SHARE("z80")		/* maps to z80 0 */
+	AM_RANGE(0x10e000, 0x10ffff) AM_SHARE("c128_kernal")
+	AM_RANGE(0x110000, 0x117fff) AM_SHARE("internal_function")
+	AM_RANGE(0x118000, 0x11ffff) AM_SHARE("external_function")
+	AM_RANGE(0x120000, 0x120fff) AM_SHARE("chargen")
+	AM_RANGE(0x121000, 0x121fff) AM_SHARE("c128_chargen")
+	AM_RANGE(0x122000, 0x1227ff) AM_SHARE("colorram")
+	AM_RANGE(0x122800, 0x1327ff) AM_SHARE("vdcram")
 	/* 2 kbyte by 8 bits, only 1 kbyte by 4 bits used) */
 #endif
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( c128_z80_io , AS_IO, 8)
-	AM_RANGE(0x1000, 0x13ff) AM_READWRITE(c64_colorram_read, c64_colorram_write)
-	AM_RANGE(0xd000, 0xd3ff) AM_DEVREADWRITE("vic2e", vic2_port_r, vic2_port_w)
-	AM_RANGE(0xd400, 0xd4ff) AM_DEVREADWRITE("sid6581", sid6581_r, sid6581_w)
-	AM_RANGE(0xd500, 0xd5ff) AM_READWRITE(c128_mmu8722_port_r, c128_mmu8722_port_w)
-	AM_RANGE(0xd600, 0xd7ff) AM_DEVREADWRITE("vdc8563", vdc8563_port_r, vdc8563_port_w)
-	AM_RANGE(0xdc00, 0xdcff) AM_DEVREADWRITE("cia_0", mos6526_r, mos6526_w)
-	AM_RANGE(0xdd00, 0xddff) AM_DEVREADWRITE("cia_1", mos6526_r, mos6526_w)
-/*  AM_RANGE(0xdf00, 0xdfff) AM_READWRITE(dma_port_r, dma_port_w) */
+static ADDRESS_MAP_START( c128_z80_io , AS_IO, 8, c128_state )
+	AM_RANGE(0x1000, 0x13ff) AM_READWRITE_LEGACY(c64_colorram_read, c64_colorram_write)
+	AM_RANGE(0xd000, 0xd3ff) AM_DEVREADWRITE_LEGACY("vic2e", vic2_port_r, vic2_port_w)
+	AM_RANGE(0xd400, 0xd4ff) AM_DEVREADWRITE_LEGACY("sid6581", sid6581_r, sid6581_w)
+	AM_RANGE(0xd500, 0xd5ff) AM_READWRITE_LEGACY(c128_mmu8722_port_r, c128_mmu8722_port_w)
+	AM_RANGE(0xd600, 0xd7ff) AM_DEVREADWRITE_LEGACY("vdc8563", vdc8563_port_r, vdc8563_port_w)
+	AM_RANGE(0xdc00, 0xdcff) AM_DEVREADWRITE_LEGACY("cia_0", mos6526_r, mos6526_w)
+	AM_RANGE(0xdd00, 0xddff) AM_DEVREADWRITE_LEGACY("cia_1", mos6526_r, mos6526_w)
+/*  AM_RANGE(0xdf00, 0xdfff) AM_READWRITE_LEGACY(dma_port_r, dma_port_w) */
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( c128_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START( c128_mem, AS_PROGRAM, 8, c128_state )
 	AM_RANGE(0x0000, 0x00ff) AM_RAMBANK("bank1")
 	AM_RANGE(0x0100, 0x01ff) AM_RAMBANK("bank2")
 	AM_RANGE(0x0200, 0x03ff) AM_RAMBANK("bank3")
@@ -271,15 +268,15 @@ static ADDRESS_MAP_START( c128_mem, AS_PROGRAM, 8)
 	AM_RANGE(0x1000, 0x1fff) AM_RAMBANK("bank5")
 	AM_RANGE(0x2000, 0x3fff) AM_RAMBANK("bank6")
 
-	AM_RANGE(0x4000, 0x7fff) AM_READ_BANK( "bank7") AM_WRITE( c128_write_4000 )
-	AM_RANGE(0x8000, 0x9fff) AM_READ_BANK( "bank8") AM_WRITE( c128_write_8000 )
-	AM_RANGE(0xa000, 0xbfff) AM_READ_BANK( "bank9") AM_WRITE( c128_write_a000 )
+	AM_RANGE(0x4000, 0x7fff) AM_READ_BANK( "bank7") AM_WRITE_LEGACY(c128_write_4000 )
+	AM_RANGE(0x8000, 0x9fff) AM_READ_BANK( "bank8") AM_WRITE_LEGACY(c128_write_8000 )
+	AM_RANGE(0xa000, 0xbfff) AM_READ_BANK( "bank9") AM_WRITE_LEGACY(c128_write_a000 )
 
-	AM_RANGE(0xc000, 0xcfff) AM_READ_BANK( "bank12") AM_WRITE( c128_write_c000 )
-	AM_RANGE(0xd000, 0xdfff) AM_READ_BANK( "bank13") AM_WRITE( c128_write_d000 )
-	AM_RANGE(0xe000, 0xfeff) AM_READ_BANK( "bank14") AM_WRITE( c128_write_e000 )
-	AM_RANGE(0xff00, 0xff04) AM_READ_BANK( "bank15") AM_WRITE( c128_write_ff00 )	   /* mmu c128 modus */
-	AM_RANGE(0xff05, 0xffff) AM_READ_BANK( "bank16") AM_WRITE( c128_write_ff05 )
+	AM_RANGE(0xc000, 0xcfff) AM_READ_BANK( "bank12") AM_WRITE_LEGACY(c128_write_c000 )
+	AM_RANGE(0xd000, 0xdfff) AM_READ_BANK( "bank13") AM_WRITE_LEGACY(c128_write_d000 )
+	AM_RANGE(0xe000, 0xfeff) AM_READ_BANK( "bank14") AM_WRITE_LEGACY(c128_write_e000 )
+	AM_RANGE(0xff00, 0xff04) AM_READ_BANK( "bank15") AM_WRITE_LEGACY(c128_write_ff00 )	   /* mmu c128 modus */
+	AM_RANGE(0xff05, 0xffff) AM_READ_BANK( "bank16") AM_WRITE_LEGACY(c128_write_ff05 )
 ADDRESS_MAP_END
 
 
@@ -667,17 +664,6 @@ static SLOT_INTERFACE_START( c128d81_iec_devices )
 	SLOT_INTERFACE("c1563", C1563)
 SLOT_INTERFACE_END
 
-static SLOT_INTERFACE_START( cbm_iec_devices )
-	SLOT_INTERFACE("c1540", C1540)
-	SLOT_INTERFACE("c1541", C1541)
-	SLOT_INTERFACE("c1541c", C1541C)
-	SLOT_INTERFACE("c1541ii", C1541II)
-	SLOT_INTERFACE("oc118", OC118)
-	SLOT_INTERFACE("c1570", C1570)
-	SLOT_INTERFACE("c1571", C1571)
-	SLOT_INTERFACE("c1581", C1581)
-SLOT_INTERFACE_END
-
 static CBM_IEC_INTERFACE( cbm_iec_intf )
 {
 	DEVCB_DEVICE_LINE("cia_0", c128_iec_srq_w),
@@ -693,50 +679,50 @@ static CBM_IEC_INTERFACE( cbm_iec_intf )
  *
  *************************************/
 
-static UINT8 c128_lightpen_x_cb( running_machine &machine )
+READ8_MEMBER( c128_state::vic_lightpen_x_cb )
 {
-	return input_port_read(machine, "LIGHTX") & ~0x01;
+	return ioport("LIGHTX")->read() & ~0x01;
 }
 
-static UINT8 c128_lightpen_y_cb( running_machine &machine )
+READ8_MEMBER( c128_state::vic_lightpen_y_cb )
 {
-	return input_port_read(machine, "LIGHTY") & ~0x01;
+	return ioport("LIGHTY")->read() & ~0x01;
 }
 
-static UINT8 c128_lightpen_button_cb( running_machine &machine )
+READ8_MEMBER( c128_state::vic_lightpen_button_cb )
 {
-	return input_port_read(machine, "OTHER") & 0x04;
+	return ioport("OTHER")->read() & 0x04;
 }
 
-static UINT8 c128_rdy_cb( running_machine &machine )
+READ8_MEMBER( c128_state::vic_rdy_cb )
 {
-	return input_port_read(machine, "CTRLSEL") & 0x08;
+	return ioport("CTRLSEL")->read() & 0x08;
 }
 
 static const vic2_interface c128_vic2_ntsc_intf = {
 	"screen",
 	"maincpu",
 	VIC8564,
-	c128_lightpen_x_cb,
-	c128_lightpen_y_cb,
-	c128_lightpen_button_cb,
-	c128_dma_read,
-	c128_dma_read_color,
-	c128_vic_interrupt,
-	c128_rdy_cb
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_x_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_y_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_button_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_dma_read),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_dma_read_color),
+	DEVCB_DRIVER_LINE_MEMBER(c128_state, vic_interrupt),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_rdy_cb)
 };
 
 static const vic2_interface c128_vic2_pal_intf = {
 	"screen",
 	"maincpu",
 	VIC8566,
-	c128_lightpen_x_cb,
-	c128_lightpen_y_cb,
-	c128_lightpen_button_cb,
-	c128_dma_read,
-	c128_dma_read_color,
-	c128_vic_interrupt,
-	c128_rdy_cb
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_x_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_y_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_lightpen_button_cb),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_dma_read),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_dma_read_color),
+	DEVCB_DRIVER_LINE_MEMBER(c128_state, vic_interrupt),
+	DEVCB_DRIVER_MEMBER(c128_state, vic_rdy_cb)
 };
 
 static const vdc8563_interface c128_vdc8563_intf = {
@@ -772,10 +758,9 @@ static MACHINE_CONFIG_START( common, c128_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(VIC6567_VRETRACERATE)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(VIC6567_COLUMNS * 2, VIC6567_LINES)
 	MCFG_SCREEN_VISIBLE_AREA(0, VIC6567_VISIBLECOLUMNS - 1, 0, VIC6567_VISIBLELINES - 1)
-	MCFG_SCREEN_UPDATE( c128 )
+	MCFG_SCREEN_UPDATE_STATIC( c128 )
 
 	MCFG_GFXDECODE( c128 )
 	MCFG_PALETTE_LENGTH((ARRAY_LENGTH(vic2_palette) + ARRAY_LENGTH(vdc8563_palette)) / 3 )
@@ -808,21 +793,11 @@ static MACHINE_CONFIG_START( common, c128_state )
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( c128, common )
-	MCFG_CBM_IEC_BUS_ADD(cbm_iec_intf)
-	MCFG_CBM_IEC_SLOT_ADD("iec4", 4, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec8", 8, cbm_iec_devices, "c1571", NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec9", 9, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec10", 10, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec11", 11, cbm_iec_devices, NULL, NULL)
+	MCFG_CBM_IEC_ADD(cbm_iec_intf, "c1571")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( c128d, common )
-	MCFG_CBM_IEC_BUS_ADD(cbm_iec_intf)
-	MCFG_CBM_IEC_SLOT_ADD("iec4", 4, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec8", 8, cbm_iec_devices, "c1571", NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec9", 9, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec10", 10, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec11", 11, cbm_iec_devices, NULL, NULL)
+	MCFG_CBM_IEC_ADD(cbm_iec_intf, "c1571")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( c128dcr, common )
