@@ -120,10 +120,7 @@ printers and other devices; most expansion modules; userport; rs232/v.24 interfa
 #include "formats/cbm_snqk.h"
 #include "includes/cbm.h"
 #include "includes/c16.h"
-#include "machine/c1541.h"
 #include "machine/c1551.h"
-#include "machine/c1571.h"
-#include "machine/c1581.h"
 #include "machine/cbmiec.h"
 #include "machine/cbmipt.h"
 #include "sound/sid6581.h"
@@ -170,56 +167,56 @@ printers and other devices; most expansion modules; userport; rs232/v.24 interfa
  * at 0xfc00 till 0xfcff is ram or rom kernal readable
  */
 
-static ADDRESS_MAP_START(c16_map, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(c16_map, AS_PROGRAM, 8, c16_state )
 	AM_RANGE(0x0000, 0x3fff) AM_RAMBANK("bank9")
 	AM_RANGE(0x4000, 0x7fff) AM_READ_BANK("bank1") AM_WRITE_BANK("bank5")	   /* only ram memory configuration */
 	AM_RANGE(0x8000, 0xbfff) AM_READ_BANK("bank2") AM_WRITE_BANK("bank6")
 	AM_RANGE(0xc000, 0xfbff) AM_READ_BANK("bank3")
 	AM_RANGE(0xfc00, 0xfcff) AM_READ_BANK("bank4")
 	AM_RANGE(0xc000, 0xfcff) AM_WRITE_BANK("bank7")
-	AM_RANGE(0xfd10, 0xfd1f) AM_READ(c16_fd1x_r)
-	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE(c16_6529_port_r, c16_6529_port_w)		/* 6529 keyboard matrix */
-	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE(c16_select_roms) /* rom chips selection */
-	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE("ted7360", ted7360_port_r, ted7360_port_w)
+	AM_RANGE(0xfd10, 0xfd1f) AM_READ_LEGACY(c16_fd1x_r)
+	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE_LEGACY(c16_6529_port_r, c16_6529_port_w)		/* 6529 keyboard matrix */
+	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE_LEGACY(c16_select_roms) /* rom chips selection */
+	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE_LEGACY("ted7360", ted7360_port_r, ted7360_port_w)
 	AM_RANGE(0xff20, 0xffff) AM_READ_BANK("bank8")
-	AM_RANGE(0xff3e, 0xff3e) AM_WRITE(c16_switch_to_rom)
-	AM_RANGE(0xff3f, 0xff3f) AM_WRITE(c16_switch_to_ram)
+	AM_RANGE(0xff3e, 0xff3e) AM_WRITE_LEGACY(c16_switch_to_rom)
+	AM_RANGE(0xff3f, 0xff3f) AM_WRITE_LEGACY(c16_switch_to_ram)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(plus4_map, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(plus4_map, AS_PROGRAM, 8, c16_state )
 	AM_RANGE(0x0000, 0x7fff) AM_READ_BANK("bank9")
 	AM_RANGE(0x8000, 0xbfff) AM_READ_BANK("bank2")
 	AM_RANGE(0xc000, 0xfbff) AM_READ_BANK("bank3")
 	AM_RANGE(0xfc00, 0xfcff) AM_READ_BANK("bank4")
 	AM_RANGE(0x0000, 0xfcff) AM_WRITE_BANK("bank9")
-	AM_RANGE(0xfd00, 0xfd0f) AM_READWRITE(c16_6551_port_r, c16_6551_port_w)
-	AM_RANGE(0xfd10, 0xfd1f) AM_READWRITE(plus4_6529_port_r, plus4_6529_port_w)
-	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE(c16_6529_port_r, c16_6529_port_w) /* 6529 keyboard matrix */
-	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE( c16_select_roms) /* rom chips selection */
-	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE("ted7360", ted7360_port_r, ted7360_port_w)
+	AM_RANGE(0xfd00, 0xfd0f) AM_READWRITE_LEGACY(c16_6551_port_r, c16_6551_port_w)
+	AM_RANGE(0xfd10, 0xfd1f) AM_READWRITE_LEGACY(plus4_6529_port_r, plus4_6529_port_w)
+	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE_LEGACY(c16_6529_port_r, c16_6529_port_w) /* 6529 keyboard matrix */
+	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE_LEGACY(c16_select_roms) /* rom chips selection */
+	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE_LEGACY("ted7360", ted7360_port_r, ted7360_port_w)
 	AM_RANGE(0xff20, 0xffff) AM_READ_BANK("bank8")
 	AM_RANGE(0xff20, 0xff3d) AM_WRITEONLY
-	AM_RANGE(0xff3e, 0xff3e) AM_WRITE(c16_switch_to_rom)
-	AM_RANGE(0xff3f, 0xff3f) AM_WRITE(c16_switch_to_ram)
+	AM_RANGE(0xff3e, 0xff3e) AM_WRITE_LEGACY(c16_switch_to_rom)
+	AM_RANGE(0xff3f, 0xff3f) AM_WRITE_LEGACY(c16_switch_to_ram)
 	AM_RANGE(0xff40, 0xffff) AM_WRITEONLY
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(c364_map , AS_PROGRAM, 8)
+static ADDRESS_MAP_START(c364_map , AS_PROGRAM, 8, c16_state )
 	AM_RANGE(0x0000, 0x7fff) AM_READ_BANK("bank9")
 	AM_RANGE(0x8000, 0xbfff) AM_READ_BANK("bank2")
 	AM_RANGE(0xc000, 0xfbff) AM_READ_BANK("bank3")
 	AM_RANGE(0xfc00, 0xfcff) AM_READ_BANK("bank4")
 	AM_RANGE(0x0000, 0xfcff) AM_WRITE_BANK("bank9")
-	AM_RANGE(0xfd00, 0xfd0f) AM_READWRITE(c16_6551_port_r, c16_6551_port_w)
-	AM_RANGE(0xfd10, 0xfd1f) AM_READWRITE(plus4_6529_port_r, plus4_6529_port_w)
-	AM_RANGE(0xfd20, 0xfd2f) AM_DEVREADWRITE("t6721", t6721_speech_r, t6721_speech_w)
-	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE(c16_6529_port_r, c16_6529_port_w) /* 6529 keyboard matrix */
-	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE(c16_select_roms) /* rom chips selection */
-	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE("ted7360", ted7360_port_r, ted7360_port_w)
+	AM_RANGE(0xfd00, 0xfd0f) AM_READWRITE_LEGACY(c16_6551_port_r, c16_6551_port_w)
+	AM_RANGE(0xfd10, 0xfd1f) AM_READWRITE_LEGACY(plus4_6529_port_r, plus4_6529_port_w)
+	AM_RANGE(0xfd20, 0xfd2f) AM_DEVREADWRITE_LEGACY("t6721", t6721_speech_r, t6721_speech_w)
+	AM_RANGE(0xfd30, 0xfd3f) AM_READWRITE_LEGACY(c16_6529_port_r, c16_6529_port_w) /* 6529 keyboard matrix */
+	AM_RANGE(0xfdd0, 0xfddf) AM_WRITE_LEGACY(c16_select_roms) /* rom chips selection */
+	AM_RANGE(0xff00, 0xff1f) AM_DEVREADWRITE_LEGACY("ted7360", ted7360_port_r, ted7360_port_w)
 	AM_RANGE(0xff20, 0xffff) AM_READ_BANK("bank8")
 	AM_RANGE(0xff20, 0xff3d) AM_WRITEONLY
-	AM_RANGE(0xff3e, 0xff3e) AM_WRITE(c16_switch_to_rom)
-	AM_RANGE(0xff3f, 0xff3f) AM_WRITE(c16_switch_to_ram)
+	AM_RANGE(0xff3e, 0xff3e) AM_WRITE_LEGACY(c16_switch_to_rom)
+	AM_RANGE(0xff3f, 0xff3f) AM_WRITE_LEGACY(c16_switch_to_ram)
 	AM_RANGE(0xff40, 0xffff) AM_WRITEONLY
 ADDRESS_MAP_END
 
@@ -419,17 +416,6 @@ static const m6502_interface c16_m7501_interface =
 	DEVCB_HANDLER(c16_m7501_port_write)	/* port_write_func */
 };
 
-static SLOT_INTERFACE_START( cbm_iec_devices )
-	SLOT_INTERFACE("c1540", C1540)
-	SLOT_INTERFACE("c1541", C1541)
-	SLOT_INTERFACE("c1541c", C1541C)
-	SLOT_INTERFACE("c1541ii", C1541II)
-	SLOT_INTERFACE("oc118", OC118)
-	SLOT_INTERFACE("c1570", C1570)
-	SLOT_INTERFACE("c1571", C1571)
-	SLOT_INTERFACE("c1581", C1581)
-SLOT_INTERFACE_END
-
 static CBM_IEC_INTERFACE( cbm_iec_intf )
 {
 	DEVCB_NULL,
@@ -439,9 +425,9 @@ static CBM_IEC_INTERFACE( cbm_iec_intf )
 	DEVCB_NULL
 };
 
-static SCREEN_UPDATE( c16 )
+static SCREEN_UPDATE_IND16( c16 )
 {
-	c16_state *state = screen->machine().driver_data<c16_state>();
+	c16_state *state = screen.machine().driver_data<c16_state>();
 	ted7360_video_update(state->m_ted7360, bitmap, cliprect);
 	return 0;
 }
@@ -460,7 +446,7 @@ static MACHINE_START( c16 )
 	state->m_maincpu = machine.device<legacy_cpu_device>("maincpu");
 	state->m_ted7360 = machine.device("ted7360");
 	state->m_cassette = machine.device<cassette_image_device>(CASSETTE_TAG);
-	state->m_messram = machine.device(RAM_TAG);
+	state->m_messram = machine.device<ram_device>(RAM_TAG);
 	state->m_sid = machine.device("sid");
 
 	state->save_item(NAME(state->m_old_level));
@@ -486,10 +472,9 @@ static MACHINE_CONFIG_START( c16, c16_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(TED7360PAL_VRETRACERATE)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(336, 216)
 	MCFG_SCREEN_VISIBLE_AREA(0, 336 - 1, 0, 216 - 1)
-	MCFG_SCREEN_UPDATE( c16 )
+	MCFG_SCREEN_UPDATE_STATIC( c16 )
 
 	MCFG_PALETTE_LENGTH(ARRAY_LENGTH(ted7360_palette) / 3)
 	MCFG_PALETTE_INIT(c16)
@@ -511,12 +496,7 @@ static MACHINE_CONFIG_START( c16, c16_state )
 	MCFG_SOFTWARE_LIST_ADD("disk_list", "plus4_flop")
 
 	/* IEC serial bus */
-	MCFG_CBM_IEC_BUS_ADD(cbm_iec_intf)
-	MCFG_CBM_IEC_SLOT_ADD("iec4", 4, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec8", 8, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec9", 9, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec10", 10, cbm_iec_devices, NULL, NULL)
-	MCFG_CBM_IEC_SLOT_ADD("iec11", 11, cbm_iec_devices, NULL, NULL)
+	MCFG_CBM_IEC_ADD(cbm_iec_intf, NULL)
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
