@@ -106,10 +106,6 @@ Keyboard: Full-sized 102 key QWERTY (19 key numeric keypad!; 4 direction
 #include "includes/cbm.h"
 #include "includes/cbmb.h"
 #include "machine/ieee488.h"
-#include "machine/c2031.h"
-#include "machine/c2040.h"
-#include "machine/c8280.h"
-#include "machine/d9060.h"
 #include "machine/cbmipt.h"
 #include "video/vic6567.h"
 #include "video/mc6845.h"
@@ -124,57 +120,57 @@ Keyboard: Full-sized 102 key QWERTY (19 key numeric keypad!; 4 direction
  *************************************/
 
 
-static ADDRESS_MAP_START(cbmb_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START(cbmb_mem , AS_PROGRAM, 8, cbmb_state )
 	AM_RANGE(0x00000, 0x0ffff) AM_READONLY AM_WRITENOP
 	AM_RANGE(0x10000, 0x4ffff) AM_RAM
 	AM_RANGE(0x50002, 0x5ffff) AM_READONLY AM_WRITENOP
 	AM_RANGE(0x60000, 0xf07ff) AM_RAM
 #if 0
-	AM_RANGE(0xf0800, 0xf0fff) AM_READ(SMH_ROM)
+	AM_RANGE(0xf0800, 0xf0fff) AM_READ_LEGACY(SMH_ROM)
 #endif
 	AM_RANGE(0xf1000, 0xf1fff) AM_ROM	/* cartridges or ram */
 	AM_RANGE(0xf2000, 0xf3fff) AM_ROM	/* cartridges or ram */
 	AM_RANGE(0xf4000, 0xf5fff) AM_ROM
 	AM_RANGE(0xf6000, 0xf7fff) AM_ROM
-	AM_RANGE(0xf8000, 0xfbfff) AM_ROM AM_BASE_MEMBER(cbmb_state, m_basic)
-	AM_RANGE(0xfd000, 0xfd7ff) AM_RAM AM_BASE_MEMBER(cbmb_state, m_videoram) /* VIDEORAM */
-	AM_RANGE(0xfd800, 0xfd800) AM_MIRROR(0xfe) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0xfd801, 0xfd801) AM_MIRROR(0xfe) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE(0xf8000, 0xfbfff) AM_ROM AM_SHARE("basic")
+	AM_RANGE(0xfd000, 0xfd7ff) AM_RAM AM_SHARE("videoram") /* VIDEORAM */
+	AM_RANGE(0xfd800, 0xfd800) AM_MIRROR(0xfe) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0xfd801, 0xfd801) AM_MIRROR(0xfe) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
 	/* disk units */
-	AM_RANGE(0xfda00, 0xfdaff) AM_DEVREADWRITE("sid6581", sid6581_r, sid6581_w)
+	AM_RANGE(0xfda00, 0xfdaff) AM_DEVREADWRITE_LEGACY("sid6581", sid6581_r, sid6581_w)
 	/* db00 coprocessor */
-	AM_RANGE(0xfdc00, 0xfdcff) AM_DEVREADWRITE("cia", mos6526_r, mos6526_w)
+	AM_RANGE(0xfdc00, 0xfdcff) AM_DEVREADWRITE_LEGACY("cia", mos6526_r, mos6526_w)
 	/* dd00 acia */
-	AM_RANGE(0xfde00, 0xfdeff) AM_DEVREADWRITE("tpi6525_0", tpi6525_r, tpi6525_w)
-	AM_RANGE(0xfdf00, 0xfdfff) AM_DEVREADWRITE("tpi6525_1", tpi6525_r, tpi6525_w)
-	AM_RANGE(0xfe000, 0xfffff) AM_ROM AM_BASE_MEMBER(cbmb_state, m_kernal)
+	AM_RANGE(0xfde00, 0xfdeff) AM_DEVREADWRITE_LEGACY("tpi6525_0", tpi6525_r, tpi6525_w)
+	AM_RANGE(0xfdf00, 0xfdfff) AM_DEVREADWRITE_LEGACY("tpi6525_1", tpi6525_r, tpi6525_w)
+	AM_RANGE(0xfe000, 0xfffff) AM_ROM AM_SHARE("kernal")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(p500_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START(p500_mem , AS_PROGRAM, 8, cbmb_state )
 	AM_RANGE(0x00000, 0x1ffff) AM_RAM
 	AM_RANGE(0x20000, 0x2ffff) AM_READONLY AM_WRITENOP
 	AM_RANGE(0x30000, 0x7ffff) AM_RAM
 	AM_RANGE(0x80000, 0x8ffff) AM_READONLY AM_WRITENOP
 	AM_RANGE(0x90000, 0xf07ff) AM_RAM
 #if 0
-	AM_RANGE(0xf0800, 0xf0fff) AM_READ(SMH_ROM)
+	AM_RANGE(0xf0800, 0xf0fff) AM_READ_LEGACY(SMH_ROM)
 #endif
 	AM_RANGE(0xf1000, 0xf1fff) AM_ROM	/* cartridges or ram */
 	AM_RANGE(0xf2000, 0xf3fff) AM_ROM	/* cartridges or ram */
 	AM_RANGE(0xf4000, 0xf5fff) AM_ROM
 	AM_RANGE(0xf6000, 0xf7fff) AM_ROM
-	AM_RANGE(0xf8000, 0xfbfff) AM_ROM AM_BASE_MEMBER(cbmb_state, m_basic)
-	AM_RANGE(0xfd000, 0xfd3ff) AM_RAM AM_BASE_MEMBER(cbmb_state, m_videoram)		/* videoram */
-	AM_RANGE(0xfd400, 0xfd7ff) AM_RAM_WRITE(cbmb_colorram_w) AM_BASE_MEMBER(cbmb_state, m_colorram)		/* colorram */
-	AM_RANGE(0xfd800, 0xfd8ff) AM_DEVREADWRITE("vic6567", vic2_port_r, vic2_port_w)
+	AM_RANGE(0xf8000, 0xfbfff) AM_ROM AM_SHARE("basic")
+	AM_RANGE(0xfd000, 0xfd3ff) AM_RAM AM_SHARE("videoram")		/* videoram */
+	AM_RANGE(0xfd400, 0xfd7ff) AM_RAM_WRITE(cbmb_colorram_w) AM_SHARE("colorram")		/* colorram */
+	AM_RANGE(0xfd800, 0xfd8ff) AM_DEVREADWRITE_LEGACY("vic6567", vic2_port_r, vic2_port_w)
 	/* disk units */
-	AM_RANGE(0xfda00, 0xfdaff) AM_DEVREADWRITE("sid6581", sid6581_r, sid6581_w)
+	AM_RANGE(0xfda00, 0xfdaff) AM_DEVREADWRITE_LEGACY("sid6581", sid6581_r, sid6581_w)
 	/* db00 coprocessor */
-	AM_RANGE(0xfdc00, 0xfdcff) AM_DEVREADWRITE("cia", mos6526_r, mos6526_w)
+	AM_RANGE(0xfdc00, 0xfdcff) AM_DEVREADWRITE_LEGACY("cia", mos6526_r, mos6526_w)
 	/* dd00 acia */
-	AM_RANGE(0xfde00, 0xfdeff) AM_DEVREADWRITE("tpi6525_0", tpi6525_r, tpi6525_w)
-	AM_RANGE(0xfdf00, 0xfdfff) AM_DEVREADWRITE("tpi6525_1", tpi6525_r, tpi6525_w)
-	AM_RANGE(0xfe000, 0xfffff) AM_ROM AM_BASE_MEMBER(cbmb_state, m_kernal)
+	AM_RANGE(0xfde00, 0xfdeff) AM_DEVREADWRITE_LEGACY("tpi6525_0", tpi6525_r, tpi6525_w)
+	AM_RANGE(0xfdf00, 0xfdfff) AM_DEVREADWRITE_LEGACY("tpi6525_1", tpi6525_r, tpi6525_w)
+	AM_RANGE(0xfe000, 0xfffff) AM_ROM AM_SHARE("kernal")
 ADDRESS_MAP_END
 
 
@@ -340,32 +336,45 @@ static PALETTE_INIT( p500 )
 	}
 }
 
-static SCREEN_UPDATE( p500 )
+static SCREEN_UPDATE_IND16( p500 )
 {
-	device_t *vic2 = screen->machine().device("vic6567");
+	device_t *vic2 = screen.machine().device("vic6567");
 
 	vic2_video_update(vic2, bitmap, cliprect);
 	return 0;
 }
 
-static UINT8 cbmb_lightpen_x_cb( running_machine &machine )
+READ8_MEMBER( cbmb_state::vic_lightpen_x_cb )
 {
-	return input_port_read(machine, "LIGHTX") & ~0x01;
+	return ioport("LIGHTX")->read() & ~0x01;
 }
 
-static UINT8 cbmb_lightpen_y_cb( running_machine &machine )
+READ8_MEMBER( cbmb_state::vic_lightpen_y_cb )
 {
-	return input_port_read(machine, "LIGHTY") & ~0x01;
+	return ioport("LIGHTY")->read() & ~0x01;
 }
 
-static UINT8 cbmb_lightpen_button_cb( running_machine &machine )
+READ8_MEMBER( cbmb_state::vic_lightpen_button_cb )
 {
-	return input_port_read(machine, "OTHER") & 0x04;
+	return ioport("OTHER")->read() & 0x04;
 }
 
-static UINT8 cbmb_rdy_cb( running_machine &machine )
+READ8_MEMBER( cbmb_state::vic_dma_read )
 {
-	return input_port_read(machine, "CTRLSEL") & 0x08;
+	if (offset >= 0x1000)
+		return m_videoram[offset & 0x3ff];
+	else
+		return m_chargen[offset & 0xfff];
+}
+
+READ8_MEMBER( cbmb_state::vic_dma_read_color )
+{
+	return m_colorram[offset & 0x3ff];
+}
+
+READ8_MEMBER( cbmb_state::vic_rdy_cb )
+{
+	return ioport("CTRLSEL")->read() & 0x08;
 }
 
 
@@ -373,13 +382,13 @@ static const vic2_interface p500_vic2_intf = {
 	"screen",
 	"maincpu",
 	VIC6567,
-	cbmb_lightpen_x_cb,
-	cbmb_lightpen_y_cb,
-	cbmb_lightpen_button_cb,
-	cbmb_dma_read,
-	cbmb_dma_read_color,
-	NULL,
-	cbmb_rdy_cb
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_lightpen_x_cb),
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_lightpen_y_cb),
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_lightpen_button_cb),
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_dma_read),
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_dma_read_color),
+	DEVCB_NULL,
+	DEVCB_DRIVER_MEMBER(cbmb_state, vic_rdy_cb)
 };
 
 /*************************************
@@ -414,19 +423,6 @@ static const tpi6525_interface cbmb_tpi_1_intf =
 	DEVCB_NULL
 };
 
-static SLOT_INTERFACE_START( cbm_ieee488_devices )
-	SLOT_INTERFACE("c2040", C2040)
-	SLOT_INTERFACE("c3040", C3040)
-	SLOT_INTERFACE("c4040", C4040)
-	SLOT_INTERFACE("c8050", C8050)
-	SLOT_INTERFACE("c8250", C8250)
-	SLOT_INTERFACE("sfd1001", SFD1001)
-	SLOT_INTERFACE("c2031", C2031)
-	SLOT_INTERFACE("c8280", C8280)
-	SLOT_INTERFACE("d9060", D9060)
-	SLOT_INTERFACE("d9090", D9090)
-SLOT_INTERFACE_END
-
 static IEEE488_INTERFACE( ieee488_intf )
 {
 	DEVCB_NULL,
@@ -450,10 +446,9 @@ static MACHINE_CONFIG_START( cbm600, cbmb_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 200)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640 - 1, 0, 200 - 1)
-	MCFG_SCREEN_UPDATE( cbmb_crtc )
+	MCFG_SCREEN_UPDATE_DEVICE("crtc", mc6845_device, screen_update)
 
 	MCFG_GFXDECODE( cbm600 )
 	MCFG_PALETTE_LENGTH(ARRAY_LENGTH(cbm700_palette) / 3)
@@ -479,11 +474,7 @@ static MACHINE_CONFIG_START( cbm600, cbmb_state )
 	MCFG_TPI6525_ADD("tpi6525_1", cbmb_tpi_1_intf)
 
 	/* IEEE bus */
-	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	MCFG_IEEE488_SLOT_ADD("ieee8", 8, cbm_ieee488_devices, "c8250", NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee9", 9, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee10", 10, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee11", 11, cbm_ieee488_devices, NULL, NULL)
+	MCFG_CBM_IEEE488_ADD(ieee488_intf, "c8250")
 
 	MCFG_FRAGMENT_ADD(cbmb_cartslot)
 MACHINE_CONFIG_END
@@ -530,12 +521,11 @@ static MACHINE_CONFIG_START( p500, cbmb_state )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(VIC6567_COLUMNS, VIC6567_LINES)
 	MCFG_SCREEN_VISIBLE_AREA(0, VIC6567_VISIBLECOLUMNS - 1, 0, VIC6567_VISIBLELINES - 1)
 	MCFG_SCREEN_REFRESH_RATE(VIC6567_VRETRACERATE)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_UPDATE( p500 )
+	MCFG_SCREEN_UPDATE_STATIC( p500 )
 
 	MCFG_PALETTE_INIT( p500 )
 	MCFG_PALETTE_LENGTH(ARRAY_LENGTH(p500_palette) / 3)
@@ -558,11 +548,7 @@ static MACHINE_CONFIG_START( p500, cbmb_state )
 	MCFG_TPI6525_ADD("tpi6525_1", cbmb_tpi_1_intf)
 
 	/* IEEE bus */
-	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	MCFG_IEEE488_SLOT_ADD("ieee8", 8, cbm_ieee488_devices, "c8250", NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee9", 9, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee10", 10, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee11", 11, cbm_ieee488_devices, NULL, NULL)
+	MCFG_CBM_IEEE488_ADD(ieee488_intf, "c8250")
 
 	MCFG_FRAGMENT_ADD(cbmb_cartslot)
 MACHINE_CONFIG_END
