@@ -38,33 +38,33 @@ NMI
 #include "formats/cgen_cas.h"
 #include "machine/ram.h"
 
-static ADDRESS_MAP_START (cgenie_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START (cgenie_mem, AS_PROGRAM, 8, cgenie_state )
 	AM_RANGE(0x0000, 0x3fff) AM_ROM
 //  AM_RANGE(0x4000, 0xbfff) AM_RAM // set up in MACHINE_START
 //  AM_RANGE(0xc000, 0xdfff) AM_ROM // installed in cgenie_init_machine
 //  AM_RANGE(0xe000, 0xefff) AM_ROM // installed in cgenie_init_machine
-	AM_RANGE(0xf000, 0xf3ff) AM_READWRITE( cgenie_colorram_r, cgenie_colorram_w ) AM_BASE_MEMBER(cgenie_state, m_colorram )
-	AM_RANGE(0xf400, 0xf7ff) AM_READWRITE( cgenie_fontram_r, cgenie_fontram_w) AM_BASE_MEMBER(cgenie_state, m_fontram )
-	AM_RANGE(0xf800, 0xf8ff) AM_READ( cgenie_keyboard_r )
+	AM_RANGE(0xf000, 0xf3ff) AM_READWRITE_LEGACY(cgenie_colorram_r, cgenie_colorram_w ) AM_SHARE("colorram")
+	AM_RANGE(0xf400, 0xf7ff) AM_READWRITE_LEGACY(cgenie_fontram_r, cgenie_fontram_w) AM_SHARE("fontram")
+	AM_RANGE(0xf800, 0xf8ff) AM_READ_LEGACY(cgenie_keyboard_r )
 	AM_RANGE(0xf900, 0xffdf) AM_NOP
-	AM_RANGE(0xffe0, 0xffe3) AM_READWRITE( cgenie_irq_status_r, cgenie_motor_w )
+	AM_RANGE(0xffe0, 0xffe3) AM_READWRITE_LEGACY(cgenie_irq_status_r, cgenie_motor_w )
 	AM_RANGE(0xffe4, 0xffeb) AM_NOP
-	AM_RANGE(0xffec, 0xffec) AM_READWRITE( cgenie_status_r, cgenie_command_w )
+	AM_RANGE(0xffec, 0xffec) AM_READWRITE_LEGACY(cgenie_status_r, cgenie_command_w )
 	AM_RANGE(0xffe4, 0xffeb) AM_NOP
-	AM_RANGE(0xffec, 0xffec) AM_WRITE( cgenie_command_w )
-	AM_RANGE(0xffed, 0xffed) AM_READWRITE( cgenie_track_r, cgenie_track_w )
-	AM_RANGE(0xffee, 0xffee) AM_READWRITE( cgenie_sector_r, cgenie_sector_w )
-	AM_RANGE(0xffef, 0xffef) AM_READWRITE( cgenie_data_r, cgenie_data_w )
+	AM_RANGE(0xffec, 0xffec) AM_WRITE_LEGACY(cgenie_command_w )
+	AM_RANGE(0xffed, 0xffed) AM_READWRITE_LEGACY(cgenie_track_r, cgenie_track_w )
+	AM_RANGE(0xffee, 0xffee) AM_READWRITE_LEGACY(cgenie_sector_r, cgenie_sector_w )
+	AM_RANGE(0xffef, 0xffef) AM_READWRITE_LEGACY(cgenie_data_r, cgenie_data_w )
 	AM_RANGE(0xfff0, 0xffff) AM_NOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START (cgenie_io, AS_IO, 8)
+static ADDRESS_MAP_START (cgenie_io, AS_IO, 8, cgenie_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0xf8, 0xf8) AM_DEVREADWRITE("ay8910", cgenie_sh_control_port_r, cgenie_sh_control_port_w )
-	AM_RANGE(0xf9, 0xf9) AM_DEVREADWRITE("ay8910", ay8910_r, ay8910_data_w )
-	AM_RANGE(0xfa, 0xfa) AM_READWRITE( cgenie_index_r, cgenie_index_w )
-	AM_RANGE(0xfb, 0xfb) AM_READWRITE( cgenie_register_r, cgenie_register_w )
-	AM_RANGE(0xff, 0xff) AM_READWRITE( cgenie_port_ff_r, cgenie_port_ff_w )
+	AM_RANGE(0xf8, 0xf8) AM_DEVREADWRITE_LEGACY("ay8910", cgenie_sh_control_port_r, cgenie_sh_control_port_w )
+	AM_RANGE(0xf9, 0xf9) AM_DEVREADWRITE_LEGACY("ay8910", ay8910_r, ay8910_data_w )
+	AM_RANGE(0xfa, 0xfa) AM_READWRITE_LEGACY(cgenie_index_r, cgenie_index_w )
+	AM_RANGE(0xfb, 0xfb) AM_READWRITE_LEGACY(cgenie_register_r, cgenie_register_w )
+	AM_RANGE(0xff, 0xff) AM_READWRITE_LEGACY(cgenie_port_ff_r, cgenie_port_ff_w )
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( cgenie )
@@ -534,10 +534,9 @@ static MACHINE_CONFIG_START( cgenie_common, cgenie_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(48*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 48*8-1,0*8,32*8-1)
-	MCFG_SCREEN_UPDATE( cgenie )
+	MCFG_SCREEN_UPDATE_STATIC( cgenie )
 
 	MCFG_GFXDECODE( cgenie )
 	MCFG_PALETTE_LENGTH(108)
@@ -711,7 +710,7 @@ DEVICE_IMAGE_LOAD( cgenie_floppy )
                 spt = pd_list[i].SPT / heads;
                 dir_sector = pd_list[i].DDSL * pd_list[i].GATM * pd_list[i].GPL + pd_list[i].SPT;
                 dir_length = pd_list[i].DDGA * pd_list[i].GPL;
-                memcpy(image.device().machine().region("maincpu")->base() + 0x5A71 + floppy_get_drive(image) * sizeof(PDRIVE), &pd_list[i], sizeof(PDRIVE));
+                memcpy(image.device().machine().root_device().memregion("maincpu")->base() + 0x5A71 + floppy_get_drive(image) * sizeof(PDRIVE), &pd_list[i], sizeof(PDRIVE));
                 break;
             }
         }
