@@ -39,7 +39,6 @@
 
 
  ******************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "includes/compis.h"
 
@@ -83,27 +82,11 @@ static PALETTE_INIT( compis )
 	palette_set_color(machine, 2, MAKE_RGB(0x00, 0xff, 0x00)); // highlight
 }
 
-void compis_state::video_start()
+static SCREEN_UPDATE_IND16( compis2 ) // temporary
 {
-	// find memory regions
-
-	VIDEO_START_NAME(generic_bitmapped)(machine());
-}
-
-static SCREEN_UPDATE( compis )
-{
-	compis_state *state = screen->machine().driver_data<compis_state>();
-	/* graphics */
-	state->m_crtc->update_screen(bitmap, cliprect);
-
-	return 0;
-}
-
-static SCREEN_UPDATE( compis2 ) // temporary
-{
-	UINT8 *m_p_videoram, *m_p_chargen;
-	m_p_videoram = screen->machine().region("vram")->base();
-	m_p_chargen = screen->machine().region("maincpu")->base()+0xca70; //bios0
+	compis_state *state = screen.machine().driver_data<compis_state>();
+	UINT8 *m_p_chargen;
+	m_p_chargen = state->memregion("maincpu")->base()+0xca70; //bios0
 	if (m_p_chargen[0x214] != 0x08) m_p_chargen+= 0x10; //bios1
 	UINT8 y,ra,chr,gfx;
 	UINT16 sy=0,ma=0,x;
@@ -112,11 +95,11 @@ static SCREEN_UPDATE( compis2 ) // temporary
 	{
 		for (ra = 0; ra < 16; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 
 			for (x = ma; x < ma + 240; x+=3)
 			{
-				chr = m_p_videoram[x];
+				chr = state->m_video_ram[x & 0x1ffff];
 
 				if (chr < 0x20)
 					gfx = 0;
@@ -141,10 +124,11 @@ static SCREEN_UPDATE( compis2 ) // temporary
 
 static UPD7220_DISPLAY_PIXELS( hgdc_display_pixels )
 {
-	UINT8 i,gfx = vram[address];
+	compis_state *state = device->machine().driver_data<compis_state>();
+	UINT8 i,gfx = state->m_video_ram[address & 0x1ffff];
 
 	for(i=0; i<8; i++)
-		*BITMAP_ADDR16(bitmap, y, x + i) = BIT((gfx >> i), 0);
+		bitmap.pix16(y, x + i) = BIT((gfx >> i), 0);
 }
 
 
@@ -161,9 +145,7 @@ static UPD7220_INTERFACE( hgdc_intf )
 /* TODO: why it writes to ROM region? */
 WRITE8_MEMBER( compis_state::vram_w )
 {
-	UINT8 *vram = machine().region("vram")->base();
-
-	vram[offset] = data;
+	m_video_ram[offset] = data;
 }
 
 static ADDRESS_MAP_START( compis_mem , AS_PROGRAM, 16, compis_state )
@@ -368,14 +350,15 @@ static const floppy_interface compis_floppy_interface =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	FLOPPY_STANDARD_5_25_DSHD,
+	FLOPPY_STANDARD_5_25_DSQD,
 	LEGACY_FLOPPY_OPTIONS_NAME(compis),
-	NULL,
+	"floppy_5_25",
 	NULL
 };
 
 static ADDRESS_MAP_START( upd7220_map, AS_0, 8, compis_state )
-	AM_RANGE(0x00000, 0x3ffff) AM_DEVREADWRITE("upd7220", upd7220_device, vram_r, vram_w)
+	ADDRESS_MAP_GLOBAL_MASK(0x1ffff)
+	AM_RANGE(0x00000, 0x1ffff) AM_RAM AM_SHARE("video_ram")
 ADDRESS_MAP_END
 
 static MACHINE_CONFIG_START( compis, compis_state )
@@ -399,10 +382,9 @@ static MACHINE_CONFIG_START( compis, compis_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
-	MCFG_SCREEN_UPDATE(compis)
+	MCFG_SCREEN_UPDATE_DEVICE("upd7220", upd7220_device, screen_update)
 #if 0
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_PALETTE_INIT(compis_gdc)
@@ -417,7 +399,7 @@ static MACHINE_CONFIG_START( compis, compis_state )
 	MCFG_PIC8259_ADD( "pic8259_slave", compis_pic8259_slave_config )
 	MCFG_I8255_ADD( "ppi8255", compis_ppi_interface )
 	MCFG_UPD7220_ADD("upd7220", XTAL_4MHz, hgdc_intf, upd7220_map) //unknown clock
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 	MCFG_I8251_ADD("uart", compis_usart_interface)
 	MCFG_MM58274C_ADD("mm58274c", compis_mm58274c_interface)
 	MCFG_UPD765A_ADD("upd765", compis_fdc_interface)
@@ -445,10 +427,9 @@ static MACHINE_CONFIG_START( compis2, compis_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 400)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 400-1)
-	MCFG_SCREEN_UPDATE(compis2)
+	MCFG_SCREEN_UPDATE_STATIC(compis2)
 	MCFG_PALETTE_LENGTH(3)
 	MCFG_PALETTE_INIT(compis)
 
@@ -459,7 +440,7 @@ static MACHINE_CONFIG_START( compis2, compis_state )
 	MCFG_PIC8259_ADD( "pic8259_slave", compis_pic8259_slave_config )
 	MCFG_I8255_ADD( "ppi8255", compis_ppi_interface )
 	MCFG_UPD7220_ADD("upd7220", XTAL_4MHz, hgdc_intf, upd7220_map) //unknown clock
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 	MCFG_I8251_ADD("uart", compis_usart_interface)
 	MCFG_MM58274C_ADD("mm58274c", compis_mm58274c_interface)
 	MCFG_UPD765A_ADD("upd765", compis_fdc_interface)
@@ -483,8 +464,6 @@ ROM_START( compis )
 
 	ROM_REGION( 0x800, "i8749", 0 )
 	ROM_LOAD( "cmpkey13.u1", 0x0000, 0x0800, CRC(3f87d138) SHA1(c04e2d325b9c04818bc7c47c3bf32b13862b11ec) )
-
-	ROM_REGION( 0x10000, "vram", ROMREGION_ERASE00 )
 ROM_END
 
 ROM_START( compis2 )
@@ -501,8 +480,6 @@ ROM_START( compis2 )
 
 	ROM_REGION( 0x800, "i8749", 0 )
 	ROM_LOAD( "cmpkey13.u1", 0x0000, 0x0800, CRC(3f87d138) SHA1(c04e2d325b9c04818bc7c47c3bf32b13862b11ec) )
-
-	ROM_REGION( 0x10000, "vram", ROMREGION_ERASE00 )
 ROM_END
 
 /*   YEAR   NAME        PARENT  COMPAT MACHINE  INPUT   INIT     COMPANY     FULLNAME */
