@@ -139,7 +139,7 @@ READ8_MEMBER( crvision_state::centronics_status_r )
 {
 	UINT8 data = 0;
 
-	data |= centronics_busy_r(m_centronics) << 7;
+	data |= m_centronics->busy_r() << 7;
 
 	return 0;
 }
@@ -150,7 +150,7 @@ READ8_MEMBER( crvision_state::centronics_status_r )
 
 WRITE8_MEMBER( crvision_state::centronics_ctrl_w )
 {
-	centronics_strobe_w(m_centronics, BIT(data, 4));
+	m_centronics->strobe_w(BIT(data, 4));
 }
 
 /***************************************************************************
@@ -164,14 +164,14 @@ WRITE8_MEMBER( crvision_state::centronics_ctrl_w )
 static ADDRESS_MAP_START( crvision_map, AS_PROGRAM, 8, crvision_state )
 	AM_RANGE(0x0000, 0x03ff) AM_MIRROR(0x0c00) AM_RAM
 	AM_RANGE(0x1000, 0x1003) AM_MIRROR(0x0ffc) AM_DEVREADWRITE(PIA6821_TAG, pia6821_device, read, write)
-	AM_RANGE(0x2000, 0x2000) AM_MIRROR(0x0ffe) AM_DEVREAD("tms9928a", tms9928a_device, vram_read)
-	AM_RANGE(0x2001, 0x2001) AM_MIRROR(0x0ffe) AM_DEVREAD("tms9928a", tms9928a_device, register_read)
-	AM_RANGE(0x3000, 0x3000) AM_MIRROR(0x0ffe) AM_DEVWRITE("tms9928a", tms9928a_device, vram_write)
-	AM_RANGE(0x3001, 0x3001) AM_MIRROR(0x0ffe) AM_DEVWRITE("tms9928a", tms9928a_device, register_write)
+	AM_RANGE(0x2000, 0x2000) AM_MIRROR(0x0ffe) AM_DEVREAD(TMS9929_TAG, tms9928a_device, vram_read)
+	AM_RANGE(0x2001, 0x2001) AM_MIRROR(0x0ffe) AM_DEVREAD(TMS9929_TAG, tms9928a_device, register_read)
+	AM_RANGE(0x3000, 0x3000) AM_MIRROR(0x0ffe) AM_DEVWRITE(TMS9929_TAG, tms9928a_device, vram_write)
+	AM_RANGE(0x3001, 0x3001) AM_MIRROR(0x0ffe) AM_DEVWRITE(TMS9929_TAG, tms9928a_device, register_write)
 	AM_RANGE(0x4000, 0x7fff) AM_ROMBANK(BANK_ROM2)
 	AM_RANGE(0x8000, 0xbfff) AM_ROMBANK(BANK_ROM1)
 //  AM_RANGE(0xc000, 0xe7ff) AM_RAMBANK(3)
-	AM_RANGE(0xe800, 0xe800) AM_DEVWRITE_LEGACY(CENTRONICS_TAG, centronics_data_w)
+	AM_RANGE(0xe800, 0xe800) AM_DEVWRITE(CENTRONICS_TAG, centronics_device, write)
 	AM_RANGE(0xe801, 0xe801) AM_READWRITE(centronics_status_r, centronics_ctrl_w)
 //  AM_RANGE(0xe802, 0xf7ff) AM_RAMBANK(4)
 	AM_RANGE(0xf800, 0xffff) AM_ROM AM_REGION(M6502_TAG, 0)
@@ -184,10 +184,10 @@ ADDRESS_MAP_END
 static ADDRESS_MAP_START( lasr2001_map, AS_PROGRAM, 8, laser2001_state )
 	AM_RANGE(0x0000, 0x03ff) AM_MIRROR(0x0c00) AM_RAM
 	AM_RANGE(0x1000, 0x1003) AM_MIRROR(0x0ffc) AM_DEVREADWRITE(PIA6821_TAG, pia6821_device, read, write)
-	AM_RANGE(0x2000, 0x2000) AM_MIRROR(0x0ffe) AM_DEVREAD("tms9928a", tms9928a_device, vram_read)
-	AM_RANGE(0x2001, 0x2001) AM_MIRROR(0x0ffe) AM_DEVREAD("tms9928a", tms9928a_device, register_read)
-	AM_RANGE(0x3000, 0x3000) AM_MIRROR(0x0ffe) AM_DEVWRITE("tms9928a", tms9928a_device, vram_write)
-	AM_RANGE(0x3001, 0x3001) AM_MIRROR(0x0ffe) AM_DEVWRITE("tms9928a", tms9928a_device, register_write)
+	AM_RANGE(0x2000, 0x2000) AM_MIRROR(0x0ffe) AM_DEVREAD(TMS9929_TAG, tms9928a_device, vram_read)
+	AM_RANGE(0x2001, 0x2001) AM_MIRROR(0x0ffe) AM_DEVREAD(TMS9929_TAG, tms9928a_device, register_read)
+	AM_RANGE(0x3000, 0x3000) AM_MIRROR(0x0ffe) AM_DEVWRITE(TMS9929_TAG, tms9928a_device, vram_write)
+	AM_RANGE(0x3001, 0x3001) AM_MIRROR(0x0ffe) AM_DEVWRITE(TMS9929_TAG, tms9928a_device, register_write)
 	AM_RANGE(0x4000, 0x7fff) AM_RAMBANK(BANK_ROM2)
 	AM_RANGE(0x8000, 0xbfff) AM_RAMBANK(BANK_ROM1)
 	AM_RANGE(0xc000, 0xffff) AM_ROM AM_REGION(M6502_TAG, 0)
@@ -201,11 +201,9 @@ ADDRESS_MAP_END
     INPUT_CHANGED( trigger_nmi )
 -------------------------------------------------*/
 
-static INPUT_CHANGED( trigger_nmi )
+INPUT_CHANGED_MEMBER( crvision_state::trigger_nmi )
 {
-	crvision_state *state = field.machine().driver_data<crvision_state>();
-
-	state->m_maincpu->set_input_line(INPUT_LINE_NMI, newval ? CLEAR_LINE : ASSERT_LINE);
+	m_maincpu->set_input_line(INPUT_LINE_NMI, newval ? CLEAR_LINE : ASSERT_LINE);
 }
 
 /*-------------------------------------------------
@@ -374,7 +372,7 @@ static INPUT_PORTS_START( crvision )
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_NAME("P2 Button 1 / - =") PORT_CODE(KEYCODE_MINUS) PORT_CHAR('-') PORT_CHAR('=') PORT_PLAYER(2)
 
 	PORT_START("NMI")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_START ) PORT_NAME("Reset") PORT_CODE(KEYCODE_F10) PORT_CHANGED(trigger_nmi, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_START ) PORT_NAME("Reset") PORT_CODE(KEYCODE_F10) PORT_CHANGED_MEMBER(DEVICE_SELF, crvision_state, trigger_nmi, 0)
 INPUT_PORTS_END
 
 /*-------------------------------------------------
@@ -511,14 +509,6 @@ static TMS9928A_INTERFACE(crvision_tms9928a_interface)
 	DEVCB_LINE(crvision_vdp_interrupt)
 };
 
-static SCREEN_UPDATE( crvision )
-{
-	tms9928a_device *tms9928a = screen->machine().device<tms9928a_device>( "tms9928a" );
-
-	tms9928a->update( bitmap, cliprect );
-	return 0;
-}
-
 /*-------------------------------------------------
     pia6821_interface pia_intf
 -------------------------------------------------*/
@@ -562,7 +552,7 @@ UINT8 crvision_state::read_keyboard(int pa)
 
 	for (i = 0; i < 8; i++)
 	{
-		value = input_port_read(machine(), keynames[pa][i]);
+		value = ioport(keynames[pa][i])->read();
 
 		if (value != 0xff)
 		{
@@ -658,14 +648,14 @@ READ8_MEMBER( laser2001_state::pia_pa_r )
 
 	UINT8 data = 0xff;
 
-	if (!BIT(m_keylatch, 0)) data &= input_port_read(machine(), "ROW0");
-	if (!BIT(m_keylatch, 1)) data &= input_port_read(machine(), "ROW1");
-	if (!BIT(m_keylatch, 2)) data &= input_port_read(machine(), "ROW2");
-	if (!BIT(m_keylatch, 3)) data &= input_port_read(machine(), "ROW3");
-	if (!BIT(m_keylatch, 4)) data &= input_port_read(machine(), "ROW4");
-	if (!BIT(m_keylatch, 5)) data &= input_port_read(machine(), "ROW5");
-	if (!BIT(m_keylatch, 6)) data &= input_port_read(machine(), "ROW6");
-	if (!BIT(m_keylatch, 7)) data &= input_port_read(machine(), "ROW7");
+	if (!BIT(m_keylatch, 0)) data &= ioport("ROW0")->read();
+	if (!BIT(m_keylatch, 1)) data &= ioport("ROW1")->read();
+	if (!BIT(m_keylatch, 2)) data &= ioport("ROW2")->read();
+	if (!BIT(m_keylatch, 3)) data &= ioport("ROW3")->read();
+	if (!BIT(m_keylatch, 4)) data &= ioport("ROW4")->read();
+	if (!BIT(m_keylatch, 5)) data &= ioport("ROW5")->read();
+	if (!BIT(m_keylatch, 6)) data &= ioport("ROW6")->read();
+	if (!BIT(m_keylatch, 7)) data &= ioport("ROW7")->read();
 
 	return data;
 }
@@ -690,10 +680,10 @@ READ8_MEMBER( laser2001_state::pia_pb_r )
 {
 	UINT8 data = 0xff;
 
-	if (!BIT(m_joylatch, 0)) data &= input_port_read(machine(), "JOY0");
-	if (!BIT(m_joylatch, 1)) data &= input_port_read(machine(), "JOY1");
-	if (!BIT(m_joylatch, 2)) data &= input_port_read(machine(), "JOY2");
-	if (!BIT(m_joylatch, 3)) data &= input_port_read(machine(), "JOY3");
+	if (!BIT(m_joylatch, 0)) data &= ioport("JOY0")->read();
+	if (!BIT(m_joylatch, 1)) data &= ioport("JOY1")->read();
+	if (!BIT(m_joylatch, 2)) data &= ioport("JOY2")->read();
+	if (!BIT(m_joylatch, 3)) data &= ioport("JOY3")->read();
 
 	return data;
 }
@@ -717,7 +707,7 @@ WRITE8_MEMBER( laser2001_state::pia_pb_w )
 	m_keylatch = data;
 
 	/* centronics data */
-	centronics_data_w(m_centronics, 0, data);
+	m_centronics->write(space, 0, data);
 }
 
 READ_LINE_MEMBER( laser2001_state::pia_ca1_r )
@@ -733,7 +723,7 @@ WRITE_LINE_MEMBER( laser2001_state::pia_ca2_w )
 READ_LINE_MEMBER( laser2001_state::pia_cb1_r )
 {
 	/* actually this is a diode-AND (READY & _BUSY), but ctronics.c returns busy status if printer image is not mounted -> Manager won't boot */
-	return sn76496_ready_r(m_psg) & (centronics_not_busy_r(m_centronics) | m_pia->ca2_output_z());
+	return sn76496_ready_r(m_psg) & (m_centronics->not_busy_r() | m_pia->ca2_output_z());
 }
 
 WRITE_LINE_MEMBER( laser2001_state::pia_cb2_w )
@@ -744,7 +734,7 @@ WRITE_LINE_MEMBER( laser2001_state::pia_cb2_w )
 	}
 	else
 	{
-		centronics_strobe_w(m_centronics, state);
+		m_centronics->strobe_w(state);
 	}
 }
 
@@ -813,7 +803,6 @@ static const floppy_interface lasr2001_floppy_interface =
 
 static const centronics_interface lasr2001_centronics_intf =
 {
-	FALSE,
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_DEVICE_LINE_MEMBER(PIA6821_TAG, pia6821_device, cb1_w)
@@ -858,7 +847,8 @@ static DEVICE_IMAGE_LOAD( crvision_cart )
 	UINT32 size;
 	UINT8 *temp_copy;
 	running_machine &machine = image.device().machine();
-	UINT8 *mem = machine.region(M6502_TAG)->base();
+	crvision_state *state = machine.driver_data<crvision_state>();
+	UINT8 *mem = state->memregion(M6502_TAG)->base();
 	address_space *program = machine.device(M6502_TAG)->memory().space(AS_PROGRAM);
 
 	if (image.software_entry() == NULL)
@@ -951,11 +941,11 @@ static DEVICE_IMAGE_LOAD( crvision_cart )
 		return IMAGE_INIT_FAIL;
 	}
 
-	memory_configure_bank(machine, "bank1", 0, 1, mem + 0x8000, 0);
-	memory_set_bank(machine, "bank1", 0);
+	state->membank("bank1")->configure_entry(0, mem + 0x8000);
+	state->membank("bank1")->set_entry(0);
 
-	memory_configure_bank(machine, "bank2", 0, 1, mem + 0x4000, 0);
-	memory_set_bank(machine, "bank2", 0);
+	state->membank("bank2")->configure_entry(0, mem + 0x4000);
+	state->membank("bank2")->set_entry(0);
 
 	auto_free(machine, temp_copy);
 
@@ -978,7 +968,7 @@ static MACHINE_CONFIG_START( creativision, crvision_state )
 	// devices
 	MCFG_PIA6821_ADD(PIA6821_TAG, pia_intf)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, crvision_cassette_interface)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 
 	// sound hardware
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -1010,9 +1000,9 @@ MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( ntsc, creativision )
     // video hardware
-	MCFG_TMS9928A_ADD( "tms9928a", TMS9918, crvision_tms9928a_interface )
+	MCFG_TMS9928A_ADD( TMS9929_TAG, TMS9918, crvision_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( crvision )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9929_TAG, tms9918_device, screen_update )
 MACHINE_CONFIG_END
 
 /*-------------------------------------------------
@@ -1021,9 +1011,9 @@ MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED_CLASS( pal, creativision, crvision_pal_state )
 	// video hardware
-	MCFG_TMS9928A_ADD( "tms9928a", TMS9929, crvision_tms9928a_interface )
+	MCFG_TMS9928A_ADD( TMS9929_TAG, TMS9929, crvision_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( crvision )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9929_TAG, tms9929_device, screen_update )
 MACHINE_CONFIG_END
 
 /*-------------------------------------------------
@@ -1039,12 +1029,12 @@ static MACHINE_CONFIG_START( lasr2001, laser2001_state )
 	MCFG_PIA6821_ADD(PIA6821_TAG, lasr2001_pia_intf)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, lasr2001_cassette_interface)
 	MCFG_LEGACY_FLOPPY_DRIVE_ADD(FLOPPY_0, lasr2001_floppy_interface)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, lasr2001_centronics_intf)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, lasr2001_centronics_intf)
 
 	// video hardware
-	MCFG_TMS9928A_ADD( "tms9928a", TMS9929A, crvision_tms9928a_interface )
+	MCFG_TMS9928A_ADD( TMS9929_TAG, TMS9929A, crvision_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( crvision )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9929_TAG, tms9929a_device, screen_update )
 
 	// sound hardware
 	MCFG_SPEAKER_STANDARD_MONO("mono")
