@@ -1,4 +1,4 @@
-/*
+/*************************************************************************************
 
     Cybiko Wireless Inter-tainment System
 
@@ -8,26 +8,17 @@
     Cybiko Classic (V2)
     Cybiko Xtreme
 
-*/
+ToDo:
+- Remove the memory leak in the nvram handler.
+- Provide facility to load games via a "cart".
+- Need instructions! The black screen doesn't fill me with confidence..unable to test.
+
+**************************************************************************************/
+
 
-/* Core includes */
-#include "emu.h"
-#include "sound/speaker.h"
 #include "includes/cybiko.h"
-
-/* Components */
-#include "cpu/h83002/h8.h"
-#include "video/hd66421.h"
-#include "machine/pcf8593.h"
-#include "machine/at45dbxx.h"
-#include "machine/sst39vfx.h"
-#include "machine/ram.h"
 #include "rendlay.h"
 
-//extern NVRAM_HANDLER( cybikov1 );
-//extern NVRAM_HANDLER( cybikov2 );
-//extern NVRAM_HANDLER( cybikoxt );
-
 //  +------------------------------------------------------+
 //  | Cybiko Classic (CY6411)                         | V2 |
 //  +------------------------------------------------------+
@@ -63,7 +54,7 @@
 ///////////////////////////
 
 // 512 kbyte ram + no memory mapped flash
-static ADDRESS_MAP_START( cybikov1_mem, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( cybikov1_mem, AS_PROGRAM, 16, cybiko_state )
 	AM_RANGE( 0x000000, 0x007fff ) AM_ROM
 	AM_RANGE( 0x600000, 0x600001 ) AM_READWRITE( cybiko_lcd_r, cybiko_lcd_w )
 	AM_RANGE( 0xe00000, 0xe07fff ) AM_READ( cybikov1_key_r )
@@ -87,7 +78,7 @@ ADDRESS_MAP_END
 //  +-------------------------------------+
 
 // 256 kbyte ram + 256 kbyte memory mapped flash
-static ADDRESS_MAP_START( cybikov2_mem, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( cybikov2_mem, AS_PROGRAM, 16, cybiko_state )
 	AM_RANGE( 0x000000, 0x007fff ) AM_ROM
 	AM_RANGE( 0x100000, 0x13ffff ) AM_READ_BANK("bank2") AM_MIRROR( 0x0c0000 )
 	AM_RANGE( 0x600000, 0x600001 ) AM_READWRITE( cybiko_lcd_r, cybiko_lcd_w ) AM_MIRROR( 0x1ffffe )
@@ -95,7 +86,7 @@ static ADDRESS_MAP_START( cybikov2_mem, AS_PROGRAM, 16 )
 ADDRESS_MAP_END
 
 // 2048 kbyte ram + 512 kbyte memory mapped flash
-static ADDRESS_MAP_START( cybikoxt_mem, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( cybikoxt_mem, AS_PROGRAM, 16, cybiko_state )
 	AM_RANGE( 0x000000, 0x007fff ) AM_ROM AM_MIRROR( 0x038000 )
 	AM_RANGE( 0x100000, 0x100001 ) AM_READWRITE( cybiko_lcd_r, cybiko_lcd_w )
 	AM_RANGE( 0x200000, 0x200003 ) AM_WRITE( cybiko_usb_w )
@@ -107,15 +98,15 @@ ADDRESS_MAP_END
 // ADDRESS MAP - IO //
 //////////////////////
 
-static ADDRESS_MAP_START( cybikov1_io, AS_IO, 8 )
+static ADDRESS_MAP_START( cybikov1_io, AS_IO, 8, cybiko_state )
 	AM_RANGE( 0xfffe40, 0xffffff ) AM_READWRITE( cybikov1_io_reg_r, cybikov1_io_reg_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( cybikov2_io, AS_IO, 8 )
+static ADDRESS_MAP_START( cybikov2_io, AS_IO, 8, cybiko_state )
 	AM_RANGE( 0xfffe40, 0xffffff ) AM_READWRITE( cybikov2_io_reg_r, cybikov2_io_reg_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( cybikoxt_io, AS_IO, 8 )
+static ADDRESS_MAP_START( cybikoxt_io, AS_IO, 8, cybiko_state )
 	AM_RANGE( 0xfffe40, 0xffffff ) AM_READWRITE( cybikoxt_io_reg_r, cybikoxt_io_reg_w )
 ADDRESS_MAP_END
 
@@ -288,6 +279,34 @@ static INPUT_PORTS_START( cybikoxt )
 
 INPUT_PORTS_END
 
+////////////////////
+// PALETTE INIT   //
+////////////////////
+
+PALETTE_INIT( hd66421 )
+{
+	// init palette
+	for (int i = 0; i < 4; i++)
+	{
+		palette_set_color(machine, i, RGB_WHITE);
+#ifndef HD66421_BRIGHTNESS_DOES_NOT_WORK
+		palette_set_pen_contrast(machine, i, 1.0 * i / (4 - 1));
+#endif
+	}
+}
+
+////////////////////
+// SCREEN UPDATE  //
+////////////////////
+
+static SCREEN_UPDATE_IND16( cybiko )
+{
+	hd66421_device *hd66421 = screen.machine().device<hd66421_device>( "hd66421" );
+	hd66421->update_screen(bitmap, cliprect);
+	return 0;
+}
+
+
 ////////////////////
 // MACHINE DRIVER //
 ////////////////////
@@ -300,15 +319,14 @@ static MACHINE_CONFIG_START( cybikov1, cybiko_state )
 	// screen
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE( 60 )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
 	MCFG_SCREEN_SIZE( HD66421_WIDTH, HD66421_HEIGHT )
 	MCFG_SCREEN_VISIBLE_AREA( 0, HD66421_WIDTH - 1, 0, HD66421_HEIGHT - 1 )
-	MCFG_SCREEN_UPDATE(hd66421)
+	MCFG_SCREEN_UPDATE_STATIC(cybiko)
 
 	// video
+	MCFG_HD66421_ADD("hd66421")
 	MCFG_PALETTE_LENGTH(4)
 	MCFG_PALETTE_INIT(hd66421)
-	MCFG_VIDEO_START(hd66421)
 	MCFG_DEFAULT_LAYOUT(layout_lcd)
 	// sound
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -317,8 +335,6 @@ static MACHINE_CONFIG_START( cybikov1, cybiko_state )
 	// machine
 	MCFG_MACHINE_START(cybikov1)
 	MCFG_MACHINE_RESET(cybikov1)
-	// non-volatile ram
-//    MCFG_NVRAM_HANDLER(cybikov1)
 	/* rtc */
 	MCFG_PCF8593_ADD("rtc")
 	MCFG_AT45DB041_ADD("flash1")
@@ -337,8 +353,6 @@ static MACHINE_CONFIG_DERIVED( cybikov2, cybikov1)
 	// machine
 	MCFG_MACHINE_START(cybikov2)
 	MCFG_MACHINE_RESET(cybikov2)
-	// non-volatile ram
-//    MCFG_NVRAM_HANDLER(cybikov2)
 	MCFG_SST39VF020_ADD("flash2", 16, ENDIANNESS_BIG)
 
 	/* internal ram */
@@ -355,8 +369,6 @@ static MACHINE_CONFIG_DERIVED( cybikoxt, cybikov1)
 	// machine
 	MCFG_MACHINE_START(cybikoxt)
 	MCFG_MACHINE_RESET(cybikoxt)
-	// non-volatile ram
-//    MCFG_NVRAM_HANDLER(cybikoxt)
 	MCFG_DEVICE_REMOVE("flash1")
 	MCFG_SST39VF400A_ADD("flash2", 16, ENDIANNESS_BIG)
 
