@@ -90,16 +90,14 @@ const floppy_format_type einstein_state::floppy_formats[] = {
  * upper 8 bits define the offset in the row,
  * data bits define data to write
  */
-static WRITE8_HANDLER( einstein_80col_ram_w )
+WRITE8_MEMBER(einstein_state::einstein_80col_ram_w)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
-	einstein->m_crtc_ram[((offset & 0x07) << 8) | ((offset >> 8) & 0xff)] = data;
+	m_crtc_ram[((offset & 0x07) << 8) | ((offset >> 8) & 0xff)] = data;
 }
 
-static READ8_HANDLER( einstein_80col_ram_r )
+READ8_MEMBER(einstein_state::einstein_80col_ram_r)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
-	return einstein->m_crtc_ram[((offset & 0x07) << 8) | ((offset >> 8) & 0xff)];
+	return m_crtc_ram[((offset & 0x07) << 8) | ((offset >> 8) & 0xff)];
 }
 
 /* TODO: Verify implementation */
@@ -113,7 +111,8 @@ static READ8_HANDLER( einstein_80col_ram_r )
 static MC6845_UPDATE_ROW( einstein_6845_update_row )
 {
 	einstein_state *einstein = device->machine().driver_data<einstein_state>();
-	UINT8 *data = device->machine().region("gfx1")->base();
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
+	UINT8 *data = device->machine().root_device().memregion("gfx1")->base();
 	UINT8 char_code, data_byte;
 	int i, x;
 
@@ -122,14 +121,14 @@ static MC6845_UPDATE_ROW( einstein_6845_update_row )
 		char_code = einstein->m_crtc_ram[(ma + i) & 0x07ff];
 		data_byte = data[(char_code << 3) + (ra & 0x07) + ((ra & 0x08) << 8)];
 
-		*BITMAP_ADDR16(bitmap, y, x + 0) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 7);
-		*BITMAP_ADDR16(bitmap, y, x + 1) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 6);
-		*BITMAP_ADDR16(bitmap, y, x + 2) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 5);
-		*BITMAP_ADDR16(bitmap, y, x + 3) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 4);
-		*BITMAP_ADDR16(bitmap, y, x + 4) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 3);
-		*BITMAP_ADDR16(bitmap, y, x + 5) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 2);
-		*BITMAP_ADDR16(bitmap, y, x + 6) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 1);
-		*BITMAP_ADDR16(bitmap, y, x + 7) = TMS9928A_PALETTE_SIZE + BIT(data_byte, 0);
+		bitmap.pix32(y, x + 0) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 7)];
+		bitmap.pix32(y, x + 1) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 6)];
+		bitmap.pix32(y, x + 2) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 5)];
+		bitmap.pix32(y, x + 3) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 4)];
+		bitmap.pix32(y, x + 4) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 3)];
+		bitmap.pix32(y, x + 5) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 2)];
+		bitmap.pix32(y, x + 6) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 1)];
+		bitmap.pix32(y, x + 7) = palette[TMS9928A_PALETTE_SIZE + BIT(data_byte, 0)];
 	}
 }
 
@@ -144,13 +143,12 @@ static WRITE_LINE_DEVICE_HANDLER( einstein_6845_de_changed )
  * bit 2 - Jumper M002 0 = 50Hz, 1 = 60Hz
  * bit 3 - Jumper M001 0 = ???, 1=??? perminently wired high on both the boards I have seen - PHS.
  */
-static READ8_HANDLER( einstein_80col_state_r )
+READ8_MEMBER(einstein_state::einstein_80col_state_r)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 	UINT8 result = 0;
 
-	result |= einstein->m_de;
-	result |= input_port_read(space->machine(), "80column_dips") & 0x06;
+	result |= m_de;
+	result |= ioport("80column_dips")->read() & 0x06;
 
 	return result;
 }
@@ -178,14 +176,14 @@ static void einstein_scan_keyboard(running_machine &machine)
 	einstein_state *einstein = machine.driver_data<einstein_state>();
 	UINT8 data = 0xff;
 
-	if (!BIT(einstein->m_keyboard_line, 0)) data &= input_port_read(machine, "LINE0");
-	if (!BIT(einstein->m_keyboard_line, 1)) data &= input_port_read(machine, "LINE1");
-	if (!BIT(einstein->m_keyboard_line, 2)) data &= input_port_read(machine, "LINE2");
-	if (!BIT(einstein->m_keyboard_line, 3)) data &= input_port_read(machine, "LINE3");
-	if (!BIT(einstein->m_keyboard_line, 4)) data &= input_port_read(machine, "LINE4");
-	if (!BIT(einstein->m_keyboard_line, 5)) data &= input_port_read(machine, "LINE5");
-	if (!BIT(einstein->m_keyboard_line, 6)) data &= input_port_read(machine, "LINE6");
-	if (!BIT(einstein->m_keyboard_line, 7)) data &= input_port_read(machine, "LINE7");
+	if (!BIT(einstein->m_keyboard_line, 0)) data &= machine.root_device().ioport("LINE0")->read();
+	if (!BIT(einstein->m_keyboard_line, 1)) data &= machine.root_device().ioport("LINE1")->read();
+	if (!BIT(einstein->m_keyboard_line, 2)) data &= machine.root_device().ioport("LINE2")->read();
+	if (!BIT(einstein->m_keyboard_line, 3)) data &= machine.root_device().ioport("LINE3")->read();
+	if (!BIT(einstein->m_keyboard_line, 4)) data &= machine.root_device().ioport("LINE4")->read();
+	if (!BIT(einstein->m_keyboard_line, 5)) data &= machine.root_device().ioport("LINE5")->read();
+	if (!BIT(einstein->m_keyboard_line, 6)) data &= machine.root_device().ioport("LINE6")->read();
+	if (!BIT(einstein->m_keyboard_line, 7)) data &= machine.root_device().ioport("LINE7")->read();
 
 	einstein->m_keyboard_data = data;
 }
@@ -198,7 +196,7 @@ static TIMER_DEVICE_CALLBACK( einstein_keyboard_timer_callback )
 	einstein_scan_keyboard(timer.machine());
 
 	/* if /fire1 or /fire2 is 0, signal a fire interrupt */
-	if ((input_port_read(timer.machine(), "BUTTONS") & 0x03) != 0)
+	if ((timer.machine().root_device().ioport("BUTTONS")->read() & 0x03) != 0)
 	{
 		einstein->m_interrupt |= EINSTEIN_FIRE_INT;
 	}
@@ -211,30 +209,28 @@ static TIMER_DEVICE_CALLBACK( einstein_keyboard_timer_callback )
 	}
 }
 
-static WRITE8_HANDLER( einstein_keyboard_line_write )
+WRITE8_MEMBER(einstein_state::einstein_keyboard_line_write)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 
 	if (VERBOSE_KEYBOARD)
 		logerror("einstein_keyboard_line_write: %02x\n", data);
 
-	einstein->m_keyboard_line = data;
+	m_keyboard_line = data;
 
 	/* re-scan the keyboard */
-	einstein_scan_keyboard(space->machine());
+	einstein_scan_keyboard(machine());
 }
 
-static READ8_HANDLER( einstein_keyboard_data_read )
+READ8_MEMBER(einstein_state::einstein_keyboard_data_read)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 
 	/* re-scan the keyboard */
-	einstein_scan_keyboard(space->machine());
+	einstein_scan_keyboard(machine());
 
 	if (VERBOSE_KEYBOARD)
-		logerror("einstein_keyboard_data_read: %02x\n", einstein->m_keyboard_data);
+		logerror("einstein_keyboard_data_read: %02x\n", m_keyboard_data);
 
-	return einstein->m_keyboard_data;
+	return m_keyboard_data;
 }
 
 
@@ -260,7 +256,7 @@ static WRITE8_DEVICE_HANDLER( einstein_drsel_w )
 	}
 
 	/* double sided drive connected? */
-	if (input_port_read(device->machine(), "config") & data)
+	if (device->machine().root_device().ioport("config")->read() & data)
 	{
 		/* bit 4 selects the side then */
 		//floppy->ss_w(BIT(data, 4));
@@ -310,16 +306,15 @@ static WRITE_LINE_DEVICE_HANDLER( einstein_serial_receive_clock )
 
 static void einstein_page_rom(running_machine &machine)
 {
-	einstein_state *einstein = machine.driver_data<einstein_state>();
-	memory_set_bankptr(machine, "bank1", einstein->m_rom_enabled ? machine.region("bios")->base() : ram_get_ptr(machine.device(RAM_TAG)));
+	einstein_state *state = machine.driver_data<einstein_state>();
+	state->membank("bank1")->set_base(state->m_rom_enabled ? machine.root_device().memregion("bios")->base() : machine.device<ram_device>(RAM_TAG)->pointer());
 }
 
 /* writing to this port is a simple trigger, and switches between RAM and ROM */
-static WRITE8_HANDLER( einstein_rom_w )
+WRITE8_MEMBER(einstein_state::einstein_rom_w)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
-	einstein->m_rom_enabled ^= 1;
-	einstein_page_rom(space->machine());
+	m_rom_enabled ^= 1;
+	einstein_page_rom(machine());
 }
 
 
@@ -327,88 +322,84 @@ static WRITE8_HANDLER( einstein_rom_w )
     INTERRUPTS
 ***************************************************************************/
 
-static READ8_HANDLER( einstein_kybintmsk_r )
+READ8_MEMBER(einstein_state::einstein_kybintmsk_r)
 {
-	device_t *printer = space->machine().device("centronics");
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
+	centronics_device *centronics = machine().device<centronics_device>("centronics");
 	UINT8 data = 0;
 
 	/* clear key int. a read of this I/O port will do this or a reset */
-	einstein->m_interrupt &= ~EINSTEIN_KEY_INT;
+	m_interrupt &= ~EINSTEIN_KEY_INT;
 
 	/* bit 0 and 1: fire buttons on the joysticks */
-	data |= input_port_read(space->machine(), "BUTTONS");
+	data |= ioport("BUTTONS")->read();
 
 	/* bit 2 to 4: printer status */
-	data |= centronics_busy_r(printer) << 2;
-	data |= centronics_pe_r(printer) << 3;
-	data |= centronics_fault_r(printer) << 4;
+	data |= centronics->busy_r() << 2;
+	data |= centronics->pe_r() << 3;
+	data |= centronics->fault_r() << 4;
 
 	/* bit 5 to 7: graph, control and shift key */
-	data |= input_port_read(space->machine(), "EXTRA");
+	data |= ioport("EXTRA")->read();
 
 	if(VERBOSE_KEYBOARD)
-		logerror("%s: einstein_kybintmsk_r %02x\n", space->machine().describe_context(), data);
+		logerror("%s: einstein_kybintmsk_r %02x\n", machine().describe_context(), data);
 
 	return data;
 }
 
-static WRITE8_HANDLER( einstein_kybintmsk_w )
+WRITE8_MEMBER(einstein_state::einstein_kybintmsk_w)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 
-	logerror("%s: einstein_kybintmsk_w %02x\n", space->machine().describe_context(), data);
+	logerror("%s: einstein_kybintmsk_w %02x\n", machine().describe_context(), data);
 
 	/* set mask from bit 0 */
 	if (data & 0x01)
 	{
 		logerror("key int is disabled\n");
-		einstein->m_interrupt_mask &= ~EINSTEIN_KEY_INT;
+		m_interrupt_mask &= ~EINSTEIN_KEY_INT;
 	}
 	else
 	{
 		logerror("key int is enabled\n");
-		einstein->m_interrupt_mask |= EINSTEIN_KEY_INT;
+		m_interrupt_mask |= EINSTEIN_KEY_INT;
 	}
 }
 
 /* writing to this I/O port sets the state of the mask; D0 is used */
 /* writing 0 enables the /ADC interrupt */
-static WRITE8_HANDLER( einstein_adcintmsk_w )
+WRITE8_MEMBER(einstein_state::einstein_adcintmsk_w)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 
-	logerror("%s: einstein_adcintmsk_w %02x\n", space->machine().describe_context(), data);
+	logerror("%s: einstein_adcintmsk_w %02x\n", machine().describe_context(), data);
 
 	if (data & 0x01)
 	{
 		logerror("adc int is disabled\n");
-		einstein->m_interrupt_mask &= ~EINSTEIN_ADC_INT;
+		m_interrupt_mask &= ~EINSTEIN_ADC_INT;
 	}
 	else
 	{
 		logerror("adc int is enabled\n");
-		einstein->m_interrupt_mask |= EINSTEIN_ADC_INT;
+		m_interrupt_mask |= EINSTEIN_ADC_INT;
 	}
 }
 
 /* writing to this I/O port sets the state of the mask; D0 is used */
 /* writing 0 enables the /FIRE interrupt */
-static WRITE8_HANDLER( einstein_fire_int_w )
+WRITE8_MEMBER(einstein_state::einstein_fire_int_w)
 {
-	einstein_state *einstein = space->machine().driver_data<einstein_state>();
 
-	logerror("%s: einstein_fire_int_w %02x\n", space->machine().describe_context(), data);
+	logerror("%s: einstein_fire_int_w %02x\n", machine().describe_context(), data);
 
 	if (data & 0x01)
 	{
 		logerror("fire int is disabled\n");
-		einstein->m_interrupt_mask &= ~EINSTEIN_FIRE_INT;
+		m_interrupt_mask &= ~EINSTEIN_FIRE_INT;
 	}
 	else
 	{
 		logerror("fire int is enabled\n");
-		einstein->m_interrupt_mask |= EINSTEIN_FIRE_INT;
+		m_interrupt_mask |= EINSTEIN_FIRE_INT;
 	}
 }
 
@@ -424,40 +415,32 @@ static TMS9928A_INTERFACE(einstein_tms9929a_interface)
 	DEVCB_NULL
 };
 
-static SCREEN_UPDATE( einstein )
-{
-	tms9929a_device *tms9929a = screen->machine().device<tms9929a_device>( "tms9929a" );
-
-	tms9929a->update( bitmap, cliprect );
-	return 0;
-}
-
 static MACHINE_START( einstein )
 {
 }
 
 static MACHINE_RESET( einstein )
 {
-	einstein_state *einstein = machine.driver_data<einstein_state>();
+	einstein_state *state = machine.driver_data<einstein_state>();
 	//device_t *floppy;
-	//UINT8 config = input_port_read(machine, "config");
+	//UINT8 config = machine.root_device().ioport("config")->read();
 
 	/* save pointers to our devices */
-	einstein->m_color_screen = machine.device("screen");
-	einstein->m_ctc = machine.device(IC_I058);
+	state->m_color_screen = machine.device("screen");
+	state->m_ctc = machine.device(IC_I058);
 
 	/* initialize memory mapping */
-	memory_set_bankptr(machine, "bank2", ram_get_ptr(machine.device(RAM_TAG)));
-	memory_set_bankptr(machine, "bank3", ram_get_ptr(machine.device(RAM_TAG)) + 0x8000);
-	einstein->m_rom_enabled = 1;
+	state->membank("bank2")->set_base(machine.device<ram_device>(RAM_TAG)->pointer());
+	state->membank("bank3")->set_base(machine.device<ram_device>(RAM_TAG)->pointer() + 0x8000);
+	state->m_rom_enabled = 1;
 	einstein_page_rom(machine);
 
 	/* a reset causes the fire int, adc int, keyboard int mask
     to be set to 1, which causes all these to be DISABLED */
-	einstein->m_interrupt = 0;
-	einstein->m_interrupt_mask = 0;
+	state->m_interrupt = 0;
+	state->m_interrupt_mask = 0;
 
-	einstein->m_ctc_trigger = 0;
+	state->m_ctc_trigger = 0;
 
 	/* configure floppy drives */
 /*  floppy_type_t type_80 = FLOPPY_STANDARD_5_25_DSHD;
@@ -500,16 +483,23 @@ static MACHINE_START( einstein2 )
     VIDEO EMULATION
 ***************************************************************************/
 
-static SCREEN_UPDATE( einstein2 )
+static SCREEN_UPDATE_RGB32( einstein2 )
 {
-	einstein_state *einstein = screen->machine().driver_data<einstein_state>();
+	einstein_state *einstein = screen.machine().driver_data<einstein_state>();
 
-	if (screen == einstein->m_color_screen)
-		SCREEN_UPDATE_CALL(einstein);
-	else if (screen == einstein->m_crtc_screen)
-		einstein->m_mc6845->update( bitmap, cliprect);
+	if (&screen == einstein->m_color_screen)
+	{
+		tms9929a_device *tms9929a = screen.machine().device<tms9929a_device>( "tms9929a" );
+		const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
+		bitmap_ind16 &src = tms9929a->get_bitmap();
+		for (int y = cliprect.min_y; y <= cliprect.max_y; y++)
+			for (int x = cliprect.min_x; x <= cliprect.max_x; x++)
+				bitmap.pix32(y, x) = palette[src.pix16(y, x)];
+	}
+	else if (&screen == einstein->m_crtc_screen)
+		einstein->m_mc6845->screen_update( screen, bitmap, cliprect);
 	else
-		fatalerror("Unknown screen '%s'", screen->tag());
+		fatalerror("Unknown screen '%s'", screen.tag());
 
 	return 0;
 }
@@ -519,45 +509,45 @@ static SCREEN_UPDATE( einstein2 )
     ADDRESS MAPS
 ***************************************************************************/
 
-static ADDRESS_MAP_START( einstein_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( einstein_mem, AS_PROGRAM, 8, einstein_state )
 	AM_RANGE(0x0000, 0x07fff) AM_READ_BANK("bank1") AM_WRITE_BANK("bank2")
 	AM_RANGE(0x8000, 0x0ffff) AM_RAMBANK("bank3")
 ADDRESS_MAP_END
 
 /* The I/O ports are decoded into 8 blocks using address lines A3 to A7 */
-static ADDRESS_MAP_START( einstein_io, AS_IO, 8 )
+static ADDRESS_MAP_START( einstein_io, AS_IO, 8, einstein_state )
 	/* block 0, ay8910 psg */
-	AM_RANGE(0x02, 0x02) AM_MIRROR(0xff04) AM_DEVREADWRITE(IC_I030, ay8910_r, ay8910_address_w)
-	AM_RANGE(0x03, 0x03) AM_MIRROR(0xff04) AM_DEVWRITE(IC_I030, ay8910_data_w)
+	AM_RANGE(0x02, 0x02) AM_MIRROR(0xff04) AM_DEVREADWRITE_LEGACY(IC_I030, ay8910_r, ay8910_address_w)
+	AM_RANGE(0x03, 0x03) AM_MIRROR(0xff04) AM_DEVWRITE_LEGACY(IC_I030, ay8910_data_w)
 	/* block 1, tms9928a vdp */
-	AM_RANGE(0x08, 0x08) AM_MIRROR(0xff06) AM_DEVREADWRITE_MODERN("tms9929a", tms9929a_device, vram_read, vram_write)
-	AM_RANGE(0x09, 0x09) AM_MIRROR(0xff06) AM_DEVREADWRITE_MODERN("tms9929a", tms9929a_device, register_read, register_write)
+	AM_RANGE(0x08, 0x08) AM_MIRROR(0xff06) AM_DEVREADWRITE("tms9929a", tms9929a_device, vram_read, vram_write)
+	AM_RANGE(0x09, 0x09) AM_MIRROR(0xff06) AM_DEVREADWRITE("tms9929a", tms9929a_device, register_read, register_write)
 	/* block 2, i8251 uart */
-	AM_RANGE(0x10, 0x10) AM_MIRROR(0xff06) AM_DEVREADWRITE_MODERN(IC_I060, i8251_device, data_r, data_w)
-	AM_RANGE(0x11, 0x11) AM_MIRROR(0xff06) AM_DEVREADWRITE_MODERN(IC_I060, i8251_device, status_r, control_w)
+	AM_RANGE(0x10, 0x10) AM_MIRROR(0xff06) AM_DEVREADWRITE(IC_I060, i8251_device, data_r, data_w)
+	AM_RANGE(0x11, 0x11) AM_MIRROR(0xff06) AM_DEVREADWRITE(IC_I060, i8251_device, status_r, control_w)
 	/* block 3, wd1770 floppy controller */
-	AM_RANGE(0x18, 0x1b) AM_MIRROR(0xff04) AM_DEVREADWRITE_MODERN(IC_I042, wd1772_t, read, write)
+	AM_RANGE(0x18, 0x1b) AM_MIRROR(0xff04) AM_DEVREADWRITE(IC_I042, wd177x_t, read, write)
 	/* block 4, internal controls */
 	AM_RANGE(0x20, 0x20) AM_MIRROR(0xff00) AM_READWRITE(einstein_kybintmsk_r, einstein_kybintmsk_w)
 	AM_RANGE(0x21, 0x21) AM_MIRROR(0xff00) AM_WRITE(einstein_adcintmsk_w)
-	AM_RANGE(0x23, 0x23) AM_MIRROR(0xff00) AM_DEVWRITE(IC_I042, einstein_drsel_w)
+	AM_RANGE(0x23, 0x23) AM_MIRROR(0xff00) AM_DEVWRITE_LEGACY(IC_I042, einstein_drsel_w)
 	AM_RANGE(0x24, 0x24) AM_MIRROR(0xff00) AM_WRITE(einstein_rom_w)
 	AM_RANGE(0x25, 0x25) AM_MIRROR(0xff00) AM_WRITE(einstein_fire_int_w)
 	/* block 5, z80ctc */
-	AM_RANGE(0x28, 0x2b) AM_MIRROR(0xff04) AM_DEVREADWRITE(IC_I058, z80ctc_r, z80ctc_w)
+	AM_RANGE(0x28, 0x2b) AM_MIRROR(0xff04) AM_DEVREADWRITE_LEGACY(IC_I058, z80ctc_r, z80ctc_w)
 	/* block 6, z80pio */
-	AM_RANGE(0x30, 0x33) AM_MIRROR(0xff04) AM_DEVREADWRITE(IC_I063, z80pio_cd_ba_r, z80pio_cd_ba_w)
+	AM_RANGE(0x30, 0x33) AM_MIRROR(0xff04) AM_DEVREADWRITE_LEGACY(IC_I063, z80pio_cd_ba_r, z80pio_cd_ba_w)
 #if 0
 	/* block 7, adc */
-	AM_RANGE(0x38, 0x38) AM_MIRROR(0xff07) AM_DEVREADWRITE(IC_I050, adc0844_r, adc0844_w)
+	AM_RANGE(0x38, 0x38) AM_MIRROR(0xff07) AM_DEVREADWRITE_LEGACY(IC_I050, adc0844_r, adc0844_w)
 #endif
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( einstein2_io, AS_IO, 8 )
+static ADDRESS_MAP_START( einstein2_io, AS_IO, 8, einstein_state )
 	AM_IMPORT_FROM(einstein_io)
 	AM_RANGE(0x40, 0x47) AM_MIRROR(0xff00) AM_MASK(0xffff) AM_READWRITE(einstein_80col_ram_r, einstein_80col_ram_w)
-	AM_RANGE(0x48, 0x48) AM_MIRROR(0xff00) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0x49, 0x49) AM_MIRROR(0xff00) AM_DEVWRITE_MODERN("crtc", mc6845_device, register_w)
+	AM_RANGE(0x48, 0x48) AM_MIRROR(0xff00) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0x49, 0x49) AM_MIRROR(0xff00) AM_DEVWRITE("crtc", mc6845_device, register_w)
 	AM_RANGE(0x4c, 0x4c) AM_MIRROR(0xff00) AM_READ(einstein_80col_state_r)
 ADDRESS_MAP_END
 
@@ -722,10 +712,10 @@ static Z80PIO_INTERFACE( einstein_pio_intf )
 {
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_DEVICE_HANDLER("centronics", centronics_data_w),
+	DEVCB_DEVICE_MEMBER("centronics", centronics_device, write),
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_DEVICE_LINE("centronics", centronics_strobe_w),
+	DEVCB_DEVICE_LINE_MEMBER("centronics", centronics_device, strobe_w),
 	DEVCB_NULL
 };
 
@@ -734,14 +724,13 @@ static const ay8910_interface einstein_ay_interface =
 	AY8910_LEGACY_OUTPUT,
 	AY8910_DEFAULT_LOADS,
 	DEVCB_NULL,
-	DEVCB_MEMORY_HANDLER(IC_I001, PROGRAM, einstein_keyboard_data_read),
-	DEVCB_MEMORY_HANDLER(IC_I001, PROGRAM, einstein_keyboard_line_write),
+	DEVCB_DRIVER_MEMBER(einstein_state, einstein_keyboard_data_read),
+	DEVCB_DRIVER_MEMBER(einstein_state, einstein_keyboard_line_write),
 	DEVCB_NULL
 };
 
 static const centronics_interface einstein_centronics_config =
 {
-	FALSE,
 	DEVCB_DEVICE_LINE(IC_I063, z80pio_astb_w),
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -812,7 +801,7 @@ static MACHINE_CONFIG_START( einstein, einstein_state )
 	/* video hardware */
 	MCFG_TMS9928A_ADD( "tms9929a", TMS9929A, einstein_tms9929a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( einstein )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9929a", tms9929a_device, screen_update )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -821,12 +810,12 @@ static MACHINE_CONFIG_START( einstein, einstein_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.20)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", einstein_centronics_config)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", einstein_centronics_config)
 
 	/* uart */
 	MCFG_I8251_ADD(IC_I060, default_i8251_interface)
 
-	MCFG_WD1772x_ADD(IC_I042, XTAL_X002)
+	MCFG_WD1770x_ADD(IC_I042, XTAL_X002)
 
 	MCFG_FLOPPY_DRIVE_ADD("fd0", einstein_floppies, "525dd", 0, einstein_state::floppy_formats)
 	MCFG_FLOPPY_DRIVE_ADD("fd1", einstein_floppies, "525dd", 0, einstein_state::floppy_formats)
@@ -855,12 +844,11 @@ static MACHINE_CONFIG_DERIVED( einstei2, einstein )
 	MCFG_DEFAULT_LAYOUT(layout_dualhsxs)
 
 	MCFG_SCREEN_ADD("80column", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
 	MCFG_SCREEN_SIZE(640, 400)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 400-1)
 	MCFG_SCREEN_REFRESH_RATE(50)
-	MCFG_SCREEN_UPDATE(einstein2)
+	MCFG_SCREEN_UPDATE_STATIC(einstein2)
 	MCFG_GFXDECODE(einstei2)
 
 	/* 2 additional colors for the 80 column screen */
