@@ -572,7 +572,6 @@ expect that the software reads these once on startup only.
 
 ******************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 /* Core includes */
 #include "emu.h"
@@ -629,19 +628,19 @@ READ8_MEMBER( fidelz80_state::fidelz80_portc_r )
 
 	if (!(m_kp_matrix&0x10))
 	{
-		data &= input_port_read(machine(), "LINE1");
+		data &= ioport("LINE1")->read();
 	}
 	if (!(m_kp_matrix&0x20))
 	{
-		data &= input_port_read(machine(), "LINE2");
+		data &= ioport("LINE2")->read();
 	}
 	if (!(m_kp_matrix&0x40))
 	{
-		data &= input_port_read(machine(), "LINE3");
+		data &= ioport("LINE3")->read();
 	}
 	if (!(m_kp_matrix&0x80))
 	{
-		data &= input_port_read(machine(), "LINE4");
+		data &= ioport("LINE4")->read();
 	}
 
 	return data;
@@ -789,25 +788,25 @@ READ8_MEMBER( fidelz80_state::vsc_pio_porta_r )
 	UINT8 data = 0;
 
 	if (m_kp_matrix & 0x01)
-		data |= (input_port_read(machine(), "COL_A"));
+		data |= (ioport("COL_A")->read());
 	if (m_kp_matrix & 0x02)
-		data |= (input_port_read(machine(), "COL_B"));
+		data |= (ioport("COL_B")->read());
 	if (m_kp_matrix & 0x04)
-		data |= (input_port_read(machine(), "COL_C"));
+		data |= (ioport("COL_C")->read());
 	if (m_kp_matrix & 0x08)
-		data |= (input_port_read(machine(), "COL_D"));
+		data |= (ioport("COL_D")->read());
 	if (m_kp_matrix & 0x10)
-		data |= (input_port_read(machine(), "COL_E"));
+		data |= (ioport("COL_E")->read());
 	if (m_kp_matrix & 0x20)
-		data |= (input_port_read(machine(), "COL_F"));
+		data |= (ioport("COL_F")->read());
 	if (m_kp_matrix & 0x40)
-		data |= (input_port_read(machine(), "COL_G"));
+		data |= (ioport("COL_G")->read());
 	if (m_kp_matrix & 0x80)
-		data |= (input_port_read(machine(), "COL_H"));
+		data |= (ioport("COL_H")->read());
 	if (m_kp_matrix & 0x100)
-		data |= (input_port_read(machine(), "COL_I"));
+		data |= (ioport("COL_I")->read());
 	if (m_kp_matrix & 0x200)
-		data |= (input_port_read(machine(), "COL_L"));
+		data |= (ioport("COL_L")->read());
 
 	return data & 0xff;
 }
@@ -903,21 +902,21 @@ READ8_MEMBER(fidelz80_state::exp_i8243_p2_r)
 	UINT8 data = 0xff;
 
 	if (m_kp_matrix & 0x01)
-		data &= input_port_read(machine(), "LINE1");
+		data &= ioport("LINE1")->read();
 	if (m_kp_matrix & 0x02)
-		data &= input_port_read(machine(), "LINE2");
+		data &= ioport("LINE2")->read();
 	if (m_kp_matrix & 0x04)
-		data &= input_port_read(machine(), "LINE3");
+		data &= ioport("LINE3")->read();
 	if (m_kp_matrix & 0x08)
-		data &= input_port_read(machine(), "LINE4");
+		data &= ioport("LINE4")->read();
 	if (m_kp_matrix & 0x10)
-		data &= input_port_read(machine(), "LINE5");
+		data &= ioport("LINE5")->read();
 	if (m_kp_matrix & 0x20)
-		data &= input_port_read(machine(), "LINE6");
+		data &= ioport("LINE6")->read();
 	if (m_kp_matrix & 0x40)
-		data &= input_port_read(machine(), "LINE7");
+		data &= ioport("LINE7")->read();
 	if (m_kp_matrix & 0x80)
-		data &= input_port_read(machine(), "LINE8");
+		data &= ioport("LINE8")->read();
 
 	return (m_i8243->i8243_p2_r(offset)&0x0f) | (data&0xf0);
 }
