@@ -37,9 +37,10 @@
         03/01/2010 Update and clean prog  by yo_fr       (jj.stac@aliceadsl.fr)
                 => add the port mapping for keyboard
         20/11/2010 : synchronization between uPD765 and Z80 are now OK, CP/M runnig! JJStacino
+        11/11/2011 : add the minidisque support -3 pouces 1/2 driver-  JJStacino
 
         don't forget to keep some information about these machine see DChector project : http://dchector.free.fr/ made by DanielCoulom
-        (and thanks to Daniel!)
+        (and thanks to Daniel!) and Yves site : http://hectorvictor.free.fr/ (thank's too Yves!)
 
     TODO :  Add the cartridge function,
             Add diskette support, (done !)
@@ -60,7 +61,7 @@
   (left)<-                     (right)->
                (down)v
 
- Fire <0> on numpad
+ Fire <?> near <1>
  Pot => INS /SUPPR
 
  Cassette : wav file (1 way, 16 bits, 44100hz)
@@ -80,11 +81,11 @@
 #include "imagedev/flopdrv.h"
 #include "formats/basicdsk.h"
 #include "cpu/z80/z80.h"
-
+#include "formats/hect_dsk.h"
 #include "includes/hec2hrp.h"
 
 /*****************************************************************************/
-static ADDRESS_MAP_START(hecdisc2_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(hecdisc2_mem, AS_PROGRAM, 8, hec2hrp_state )
 /*****************************************************************************/
 	ADDRESS_MAP_UNMAP_HIGH
 
@@ -95,97 +96,106 @@ static ADDRESS_MAP_START(hecdisc2_mem, AS_PROGRAM, 8)
 
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hecdisc2_io , AS_IO, 8)
+static ADDRESS_MAP_START( hecdisc2_io , AS_IO, 8, hec2hrp_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	// ROM Page managing
-	AM_RANGE(0x000,0x00f) AM_READWRITE( hector_disc2_io00_port_r, hector_disc2_io00_port_w )
+	AM_RANGE(0x000,0x00f) AM_READWRITE_LEGACY(hector_disc2_io00_port_r, hector_disc2_io00_port_w )
 	// RS232 - 8251 managing
-	AM_RANGE(0x020,0x02f) AM_READWRITE( hector_disc2_io20_port_r, hector_disc2_io20_port_w )
+	AM_RANGE(0x020,0x02f) AM_READWRITE_LEGACY(hector_disc2_io20_port_r, hector_disc2_io20_port_w )
 	// Hector communication managing
-	AM_RANGE(0x030,0x03f) AM_READWRITE( hector_disc2_io30_port_r, hector_disc2_io30_port_w )
-	AM_RANGE(0x040,0x04f) AM_READWRITE( hector_disc2_io40_port_r, hector_disc2_io40_port_w )
-	AM_RANGE(0x050,0x05f) AM_READWRITE( hector_disc2_io50_port_r, hector_disc2_io50_port_w )
+	AM_RANGE(0x030,0x03f) AM_READWRITE_LEGACY(hector_disc2_io30_port_r, hector_disc2_io30_port_w )
+	AM_RANGE(0x040,0x04f) AM_READWRITE_LEGACY(hector_disc2_io40_port_r, hector_disc2_io40_port_w )
+	AM_RANGE(0x050,0x05f) AM_READWRITE_LEGACY(hector_disc2_io50_port_r, hector_disc2_io50_port_w )
 	// uPD765 link:
-	AM_RANGE(0x060,0x060) AM_DEVREAD	 ("upd765",upd765_status_r			  )
-//  AM_RANGE(0x061,0x061) AM_DEVREADWRITE("upd765",upd765_data_r,upd765_data_w)
-//  AM_RANGE(0x070,0x07f) AM_DEVREADWRITE("upd765",upd765_dack_r,upd765_dack_w)
-AM_RANGE(0x061,0x061) AM_READWRITE( hector_disc2_io61_port_r, hector_disc2_io61_port_w )//patched version
-AM_RANGE(0x070,0x07F) AM_READWRITE( hector_disc2_io70_port_r, hector_disc2_io70_port_w )//patched version
+	AM_RANGE(0x060,0x060) AM_DEVREAD_LEGACY("upd765",upd765_status_r			  )
+//  AM_RANGE(0x061,0x061) AM_DEVREADWRITE_LEGACY("upd765",upd765_data_r,upd765_data_w)
+//  AM_RANGE(0x070,0x07f) AM_DEVREADWRITE_LEGACY("upd765",upd765_dack_r,upd765_dack_w)
+AM_RANGE(0x061,0x061) AM_READWRITE_LEGACY(hector_disc2_io61_port_r, hector_disc2_io61_port_w )//patched version
+AM_RANGE(0x070,0x07F) AM_READWRITE_LEGACY(hector_disc2_io70_port_r, hector_disc2_io70_port_w )//patched version
 ADDRESS_MAP_END
 
 /*****************************************************************************/
-static ADDRESS_MAP_START(hec2hrp_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(hec2hrp_mem, AS_PROGRAM, 8, hec2hrp_state )
 /*****************************************************************************/
 	ADDRESS_MAP_UNMAP_HIGH
 	/* Hardware address mapping*/
-//  AM_RANGE(0x0800,0x0808) AM_WRITE( hector_switch_bank_w)/* Bank management*/
-	AM_RANGE(0x1000,0x1000) AM_WRITE( hector_color_a_w)  /* Color c0/c1*/
-	AM_RANGE(0x1800,0x1800) AM_WRITE( hector_color_b_w)  /* Color c2/c3*/
-	AM_RANGE(0x2000,0x2003) AM_WRITE( hector_sn_2000_w)  /* Sound*/
-	AM_RANGE(0x2800,0x2803) AM_WRITE( hector_sn_2800_w)  /* Sound*/
-	AM_RANGE(0x3000,0x3000) AM_READWRITE( hector_cassette_r, hector_sn_3000_w)/* Write necessary*/
-	AM_RANGE(0x3800,0x3807) AM_READWRITE( hector_keyboard_r, hector_keyboard_w)  /* Keyboard*/
+	AM_RANGE(0x0800,0x0808) AM_WRITE(hector_switch_bank_w)/* Bank management*/
+	AM_RANGE(0x1000,0x1000) AM_WRITE(hector_color_a_w)  /* Color c0/c1*/
+	AM_RANGE(0x1800,0x1800) AM_WRITE(hector_color_b_w)  /* Color c2/c3*/
+	AM_RANGE(0x2000,0x2003) AM_WRITE(hector_sn_2000_w)  /* Sound*/
+	AM_RANGE(0x2800,0x2803) AM_WRITE(hector_sn_2800_w)  /* Sound*/
+	AM_RANGE(0x3000,0x3000) AM_READWRITE(hector_cassette_r, hector_sn_3000_w)/* Write necessary*/
+	AM_RANGE(0x3800,0x3807) AM_READWRITE(hector_keyboard_r, hector_keyboard_w)  /* Keyboard*/
 
 	/* Main ROM page*/
 	AM_RANGE(0x0000,0x3fff) AM_ROM
 
 	/* Video br mapping*/
-	AM_RANGE(0x4000,0x49ff) AM_RAM AM_BASE_MEMBER(hec2hrp_state, m_videoram)
+	AM_RANGE(0x4000,0x49ff) AM_RAM AM_SHARE("videoram")
 	/* continous RAM*/
 	AM_RANGE(0x4A00,0xbfff) AM_RAM
 	/* from 0xC000 to 0xFFFF => Bank Ram for video and data !*/
-	AM_RANGE(0xc000,0xffff) AM_RAM AM_BASE_MEMBER(hec2hrp_state, m_hector_videoram)
+	AM_RANGE(0xc000,0xffff) AM_RAM AM_SHARE("hector_videoram")
 ADDRESS_MAP_END
 
 /*****************************************************************************/
-static ADDRESS_MAP_START(hec2hrx_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(hec2hrx_mem, AS_PROGRAM, 8, hec2hrp_state )
 /*****************************************************************************/
 	ADDRESS_MAP_UNMAP_HIGH
 	/* Hardware address mapping*/
-	AM_RANGE(0x0800,0x0808) AM_WRITE( hector_switch_bank_w)/* Bank management*/
-	AM_RANGE(0x1000,0x1000) AM_WRITE( hector_color_a_w)  /* Color c0/c1*/
-	AM_RANGE(0x1800,0x1800) AM_WRITE( hector_color_b_w)  /* Color c2/c3*/
-	AM_RANGE(0x2000,0x2003) AM_WRITE( hector_sn_2000_w)  /* Sound*/
-	AM_RANGE(0x2800,0x2803) AM_WRITE( hector_sn_2800_w)  /* Sound*/
-	AM_RANGE(0x3000,0x3000) AM_READWRITE( hector_cassette_r, hector_sn_3000_w)/* Write necessary*/
-	AM_RANGE(0x3800,0x3807) AM_READWRITE( hector_keyboard_r, hector_keyboard_w)  /* Keyboard*/
+	AM_RANGE(0x0800,0x0808) AM_WRITE(hector_switch_bank_w)/* Bank management*/
+	AM_RANGE(0x1000,0x1000) AM_WRITE(hector_color_a_w)  /* Color c0/c1*/
+	AM_RANGE(0x1800,0x1800) AM_WRITE(hector_color_b_w)  /* Color c2/c3*/
+	AM_RANGE(0x2000,0x2003) AM_WRITE(hector_sn_2000_w)  /* Sound*/
+	AM_RANGE(0x2800,0x2803) AM_WRITE(hector_sn_2800_w)  /* Sound*/
+	AM_RANGE(0x3000,0x3000) AM_READWRITE(hector_cassette_r, hector_sn_3000_w)/* Write necessary*/
+	AM_RANGE(0x3800,0x3807) AM_READWRITE(hector_keyboard_r, hector_keyboard_w)  /* Keyboard*/
 
 	/* Main ROM page*/
 	AM_RANGE(0x0000,0x3fff) AM_ROMBANK("bank2")
 
 	/* Video br mapping*/
-	AM_RANGE(0x4000,0x49ff) AM_RAM AM_BASE_MEMBER(hec2hrp_state, m_videoram)
+	AM_RANGE(0x4000,0x49ff) AM_RAM AM_SHARE("videoram")
 	/* continous RAM*/
 	AM_RANGE(0x4A00,0xbfff) AM_RAM
 	/* from 0xC000 to 0xFFFF => Bank Ram for video and data !*/
-	AM_RANGE(0xc000,0xffff) AM_RAMBANK("bank1") AM_BASE_MEMBER(hec2hrp_state, m_hector_videoram)
+	AM_RANGE(0xc000,0xffff) AM_RAMBANK("bank1") AM_SHARE("hector_videoram")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hec2hrp_io , AS_IO, 8)
+static ADDRESS_MAP_START( hec2hrp_io , AS_IO, 8, hec2hrp_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x000,0x0ff) AM_READWRITE( hector_io_8255_r, hector_io_8255_w )
+	AM_RANGE(0x000,0x0ff) AM_READWRITE(hector_io_8255_r, hector_io_8255_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hec2hrx_io , AS_IO, 8)
+static ADDRESS_MAP_START( hec2hrx_io , AS_IO, 8, hec2hrp_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x0f0,0x0ff) AM_READWRITE( hector_io_8255_r, hector_io_8255_w )
+	AM_RANGE(0x0f0,0x0ff) AM_READWRITE(hector_io_8255_r, hector_io_8255_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hec2mx40_io , AS_IO, 8)
+static ADDRESS_MAP_START( hec2mdhrx_io , AS_IO, 8, hec2hrp_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x000,0x0ef) AM_WRITE(     hector_mx40_io_port_w )
-	AM_RANGE(0x0f0,0x0f3) AM_READWRITE( hector_io_8255_r, hector_io_8255_w )
+
+	// Minidisc commands and changing the rom page !*/
+	AM_RANGE(0x000,0x0EF) AM_READWRITE(hector_179x_register_r,hector_179x_register_w)/* 179x registers*/
+	AM_RANGE(0x0f0,0x0ff) AM_READWRITE(hector_io_8255_r, hector_io_8255_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( hec2mx80_io , AS_IO, 8)
+static ADDRESS_MAP_START( hec2mx40_io , AS_IO, 8, hec2hrp_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x000,0x0ef) AM_WRITE(     hector_mx80_io_port_w )
-	AM_RANGE(0x0f0,0x0f3) AM_READWRITE( hector_io_8255_r, hector_io_8255_w )
+	AM_RANGE(0x000,0x0ef) AM_WRITE(hector_mx40_io_port_w )
+	AM_RANGE(0x0f0,0x0f3) AM_READWRITE(hector_io_8255_r, hector_io_8255_w )
+ADDRESS_MAP_END
+
+static ADDRESS_MAP_START( hec2mx80_io , AS_IO, 8, hec2hrp_state )
+	ADDRESS_MAP_UNMAP_HIGH
+	ADDRESS_MAP_GLOBAL_MASK(0xff)
+	AM_RANGE(0x000,0x0ef) AM_WRITE(hector_mx80_io_port_w )
+	AM_RANGE(0x0f0,0x0f3) AM_READWRITE(hector_io_8255_r, hector_io_8255_w )
 
 
 ADDRESS_MAP_END
@@ -246,7 +256,7 @@ static INPUT_PORTS_START( hec2hrp )
 		PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_P)			PORT_CHAR('P')
 		PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_O)			PORT_CHAR('O')
 		PORT_BIT(0x10, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_N)			PORT_CHAR('N')
-		PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_COLON)
+		PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_COLON)		PORT_CHAR('M')
 		PORT_BIT(0x40, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_L)			PORT_CHAR('L')
 		PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_CODE(KEYCODE_K)			PORT_CHAR('K')
 
@@ -272,7 +282,7 @@ static INPUT_PORTS_START( hec2hrp )
 
 	PORT_START("KEY8") /* [1] - port 3000 @ 8  not for the real machine, but to emulate the analog signal of the joystick */
 		PORT_BIT(0x01, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("RESET")				PORT_CODE(KEYCODE_ESC)		PORT_CHAR(27)
-		PORT_BIT(0x02, IP_ACTIVE_LOW,  IPT_KEYBOARD) PORT_NAME("Joy(0) FIRE")		PORT_CODE(KEYCODE_0_PAD)
+		PORT_BIT(0x02, IP_ACTIVE_LOW,  IPT_KEYBOARD) PORT_NAME("Joy(0) FIRE")		PORT_CODE(KEYCODE_TILDE)
 		PORT_BIT(0x04, IP_ACTIVE_LOW,  IPT_KEYBOARD) PORT_NAME("Joy(1) FIRE")		PORT_CODE(KEYCODE_PLUS_PAD)
 		PORT_BIT(0x08, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_NAME("Pot(0)+") PORT_CODE(KEYCODE_INSERT)
 		PORT_BIT(0x10, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_NAME("Pot(0)-") PORT_CODE(KEYCODE_DEL)
@@ -298,48 +308,87 @@ static MACHINE_START( hec2hrx )
 /*****************************************************************************/
 {
 	hec2hrp_state *state = machine.driver_data<hec2hrp_state>();
-	UINT8 *RAM   = machine.region("maincpu"  )->base();	// pointer to mess ram
+	UINT8 *RAM   = machine.root_device().memregion("maincpu"  )->base();	// pointer to mess ram
 	//Patch rom possible !
 	//RAMD2[0xff6b] = 0x0ff; // force verbose mode hector !
 
 	// Memory install for bank switching
-	memory_configure_bank(machine, "bank1", HECTOR_BANK_PROG , 1, &RAM[0xc000]   , 0); // Mess ram
-	memory_configure_bank(machine, "bank1", HECTOR_BANK_VIDEO, 1, state->m_hector_videoram_hrx, 0); // Video ram
+	state->membank("bank1")->configure_entry(HECTOR_BANK_PROG , &RAM[0xc000]   ); // Mess ram
+	state->membank("bank1")->configure_entry(HECTOR_BANK_VIDEO, state->m_hector_videoram_hrx); // Video ram
 
 	// Set bank HECTOR_BANK_PROG as basic bank
-	memory_set_bank(machine, "bank1", HECTOR_BANK_PROG);
+	state->membank("bank1")->set_entry(HECTOR_BANK_PROG);
 
 /******************************************************SPECIFIQUE MX ***************************/
-	memory_configure_bank(machine, "bank2", HECTORMX_BANK_PAGE0 , 1, &RAM[0x0000]                    , 0); // Mess ram
-	memory_configure_bank(machine, "bank2", HECTORMX_BANK_PAGE1 , 1, machine.region("page1")->base() , 0); // Rom page 1
-	memory_configure_bank(machine, "bank2", HECTORMX_BANK_PAGE2 , 1, machine.region("page2")->base() , 0); // Rom page 2
-	memory_set_bank(machine, "bank2", HECTORMX_BANK_PAGE0);
+	state->membank("bank2")->configure_entry(HECTORMX_BANK_PAGE0 , &RAM[0x0000]                    ); // Mess ram
+	state->membank("bank2")->configure_entry(HECTORMX_BANK_PAGE1 , machine.root_device().memregion("page1")->base() ); // Rom page 1
+	state->membank("bank2")->configure_entry(HECTORMX_BANK_PAGE2 , machine.root_device().memregion("page2")->base() ); // Rom page 2
+	state->membank("bank2")->set_entry(HECTORMX_BANK_PAGE0);
 /******************************************************SPECIFIQUE MX ***************************/
 
 /*************************************************SPECIFIQUE DISK II ***************************/
-	memory_configure_bank(machine, "bank3", DISCII_BANK_ROM , 1, machine.region("rom_disc2")->base() , 0); // ROM
-	memory_configure_bank(machine, "bank3", DISCII_BANK_RAM , 1, machine.region("disc2mem" )->base() , 0); // RAM
-	memory_set_bank(machine, "bank3", DISCII_BANK_ROM);
+	state->membank("bank3")->configure_entry(DISCII_BANK_ROM , machine.root_device().memregion("rom_disc2")->base() ); // ROM
+	state->membank("bank3")->configure_entry(DISCII_BANK_RAM , state->memregion("disc2mem" )->base() ); // RAM
+	state->membank("bank3")->set_entry(DISCII_BANK_ROM);
 /*************************************************SPECIFIQUE DISK II ***************************/
 
 	// As video HR ram is in bank, use extern memory
-	state->m_hector_videoram  = state->m_hector_videoram_hrx;
+	state->m_hector_videoram.set_target(state->m_hector_videoram_hrx,state->m_hector_videoram.bytes());
 
 	hector_init(machine);
 	hector_disc2_init(machine); // Init of the Disc II !
 }
+/*****************************************************************************/
+static MACHINE_START( hec2mdhrx )
+/*****************************************************************************/
+//minidisc
+{
+	hec2hrp_state *state = machine.driver_data<hec2hrp_state>();
+	UINT8 *RAM   = machine.root_device().memregion("maincpu"  )->base();	// pointer to mess ram
+
+	// Memory install for bank switching
+	state->membank("bank1")->configure_entry(HECTOR_BANK_PROG , &RAM[0xc000]   ); // Mess ram
+	state->membank("bank1")->configure_entry(HECTOR_BANK_VIDEO, state->m_hector_videoram_hrx); // Video ram
+
+	// Set bank HECTOR_BANK_PROG as basic bank
+	state->membank("bank1")->set_entry(HECTOR_BANK_PROG);
+	//Here the bank 5 is not used for the language switch but for the floppy ROM.....
+	/******************************************************SPECIFIQUE Mini disque ***************************/
+	state->membank("bank2")->configure_entry(HECTOR_BANK_BASE , &RAM[0x0000]                    ); // Rom base page
+	state->membank("bank2")->configure_entry(HECTOR_BANK_DISC , state->memregion("page2")->base() ); // Rom page mini disc
+	state->membank("bank2")->set_entry(HECTOR_BANK_BASE);
+	/******************************************************SPECIFIQUE Mini disque ***************************/
+
+	// As video HR ram is in bank, use extern memory
+	state->m_hector_videoram.set_target(state->m_hector_videoram_hrx,state->m_hector_videoram.bytes());
+
+	hector_init(machine);
+	hector_minidisc_init(machine);
+}
 static MACHINE_RESET(hec2hrx)
 {
+	hec2hrp_state *state = machine.driver_data<hec2hrp_state>();
 	//Hector Memory
-	memory_set_bank(machine, "bank1", HECTOR_BANK_PROG);
-	memory_set_bank(machine, "bank2", HECTORMX_BANK_PAGE0);
+	state->membank("bank1")->set_entry(HECTOR_BANK_PROG);
+	state->membank("bank2")->set_entry(HECTORMX_BANK_PAGE0);
 	//DISK II Memory
-	memory_set_bank(machine, "bank3", DISCII_BANK_ROM);
+	state->membank("bank3")->set_entry(DISCII_BANK_ROM);
 
 	// Machines init
 	hector_reset(machine, 1, 1);
 	hector_disc2_reset(machine);
 }
+//minidisc
+static MACHINE_RESET(hec2mdhrx)
+{
+	hec2hrp_state *state = machine.driver_data<hec2hrp_state>();
+	//Hector Memory
+	state->membank("bank1")->set_entry(HECTOR_BANK_PROG);
+	state->membank("bank2")->set_entry(HECTORMX_BANK_PAGE0);
+
+	// Machines init
+	hector_reset(machine, 1, 0);
+}
 
 /* Cassette definition */
 static const struct CassetteOptions hector_cassette_options = {
@@ -352,7 +401,7 @@ static const cassette_interface hector_cassette_interface =
 {
 	hector_cassette_formats,
 	&hector_cassette_options,
-	(cassette_state)(CASSETTE_STOPPED | CASSETTE_MASK_SPEAKER),
+	(cassette_state)(CASSETTE_PLAY | CASSETTE_MOTOR_DISABLED | CASSETTE_SPEAKER_ENABLED),
 	NULL,
 	NULL
 };
@@ -363,6 +412,31 @@ static DISCRETE_SOUND_START( hec2hrp )
 	DISCRETE_OUTPUT(NODE_01, 5000)
 DISCRETE_SOUND_END
 
+/***********************************************************/
+/********* mini disque interface ***************************/
+/***********************************************************/
+
+const wd17xx_interface hector_wd17xx_interface =
+{
+	DEVCB_NULL,
+	DEVCB_NULL,// treatment for intrq line
+	DEVCB_NULL,// treatment for drq line
+	{FLOPPY_0, NULL, NULL, NULL}		// Only one floppy on the minidisc
+};
+
+const floppy_interface minidisc_floppy_interface =
+{
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	FLOPPY_STANDARD_3_5_DSDD,
+	LEGACY_FLOPPY_OPTIONS_NAME(hector_minidisc),
+	NULL,
+	NULL
+};
+
 /******************************************************************************/
 static MACHINE_CONFIG_START( hec2hr, hec2hrp_state )
 /******************************************************************************/
@@ -378,10 +452,9 @@ static MACHINE_CONFIG_START( hec2hr, hec2hrp_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(512, 230)
 	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
-	MCFG_SCREEN_UPDATE(hec2hrp)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_VIDEO_START(hec2hrp)
@@ -422,10 +495,9 @@ static MACHINE_CONFIG_START( hec2hrp, hec2hrp_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(512, 230)
 	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
-	MCFG_SCREEN_UPDATE(hec2hrp)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_VIDEO_START(hec2hrp)
@@ -473,10 +545,9 @@ static MACHINE_CONFIG_START( hec2mx40, hec2hrp_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(512, 230)
 	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
-	MCFG_SCREEN_UPDATE(hec2hrp)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_VIDEO_START(hec2hrp)
@@ -523,10 +594,56 @@ static MACHINE_CONFIG_START( hec2hrx, hec2hrp_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(512, 230)
 	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
-	MCFG_SCREEN_UPDATE(hec2hrp)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
+
+	MCFG_PALETTE_LENGTH(16)
+	MCFG_VIDEO_START(hec2hrp)
+
+	/* sound hardware */
+	MCFG_SPEAKER_STANDARD_MONO("mono")
+	MCFG_SOUND_WAVE_ADD(WAVE_TAG, CASSETTE_TAG)
+	MCFG_SOUND_ROUTE(0, "mono", 0.25)// Sound level for cassette, as it is in mono => output channel=0
+
+	MCFG_SOUND_ADD("sn76477", SN76477, 0)
+	MCFG_SOUND_CONFIG(hector_sn76477_interface)
+	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.1)
+
+	MCFG_SOUND_ADD("discrete", DISCRETE, 0) // Son 1bit
+	MCFG_SOUND_CONFIG_DISCRETE( hec2hrp )
+	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
+
+	// Gestion cassette
+	MCFG_CASSETTE_ADD( CASSETTE_TAG, hector_cassette_interface )
+
+	/* printer */
+	MCFG_PRINTER_ADD("printer")
+
+MACHINE_CONFIG_END
+/*****************************************************************************/
+static MACHINE_CONFIG_START( hec2mdhrx, hec2hrp_state )
+/*****************************************************************************/
+// minidisc
+/* basic machine hardware */
+	MCFG_CPU_ADD("maincpu",Z80, XTAL_5MHz)
+	MCFG_CPU_PROGRAM_MAP(hec2hrx_mem)
+	MCFG_CPU_IO_MAP(hec2mdhrx_io)
+	MCFG_CPU_PERIODIC_INT(irq0_line_hold,50) //  put on the Z80 irq in Hz
+	MCFG_MACHINE_RESET(hec2mdhrx)
+	MCFG_MACHINE_START(hec2mdhrx)
+
+	/* Mini Disc */
+	MCFG_FD1793_ADD("wd179x", hector_wd17xx_interface )
+	MCFG_LEGACY_FLOPPY_DRIVE_ADD(FLOPPY_0, minidisc_floppy_interface)
+
+	/* video hardware */
+	MCFG_SCREEN_ADD("screen", RASTER)
+	MCFG_SCREEN_REFRESH_RATE(50)
+	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
+	MCFG_SCREEN_SIZE(512, 230)
+	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_PALETTE_INIT(black_and_white)
@@ -552,6 +669,7 @@ static MACHINE_CONFIG_START( hec2hrx, hec2hrp_state )
 	MCFG_PRINTER_ADD("printer")
 
 MACHINE_CONFIG_END
+
 /*****************************************************************************/
 static MACHINE_CONFIG_START( hec2mx80, hec2hrp_state )
 /*****************************************************************************/
@@ -574,10 +692,9 @@ static MACHINE_CONFIG_START( hec2mx80, hec2hrp_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(400)) /* 2500 not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(512, 230)
 	MCFG_SCREEN_VISIBLE_AREA(0, 243, 0, 227)
-	MCFG_SCREEN_UPDATE(hec2hrp)
+	MCFG_SCREEN_UPDATE_STATIC(hec2hrp)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_VIDEO_START(hec2hrp)
@@ -625,8 +742,7 @@ ROM_START( hec2hrp )
 	ROM_REGION( 0x4000, "page2", ROMREGION_ERASEFF )
 	ROM_REGION( 0x10000, "disc2cpu", ROMREGION_ERASEFF )
 	ROM_REGION( 0x04000, "rom_disc2", ROMREGION_ERASEFF )
-
-	ROM_END
+ROM_END
 
 ROM_START( hec2hrx )
 	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
@@ -638,7 +754,18 @@ ROM_START( hec2hrx )
 	ROM_REGION( 0x04000, "rom_disc2", ROMREGION_ERASEFF )
 	ROM_LOAD( "d800k.bin" , 0x0000,0x1000, CRC(831bd584) SHA1(9782ee58f570042608d9d568b2c3fc4c6d87d8b9))
 	ROM_REGION( 0x10000,  "disc2mem", ROMREGION_ERASE00 )
-	ROM_END
+ROM_END
+// minidisc
+ROM_START( hec2mdhrx )
+	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
+	ROM_LOAD( "mdic1.bin" , 0x0000,0x2000, CRC(ddda1065) SHA1(e7bba14a72605238d2f8299da029b8320a563254))
+	ROM_LOAD( "mdicmb.bin" , 0x2000,0x2000, CRC(d8090747) SHA1(f2925b68002307562e2ea5e36b740e5458f0f0eb))
+
+	ROM_REGION( 0x4000, "page1", ROMREGION_ERASEFF )  // Page 1 = unused page
+	ROM_REGION( 0x4000, "page2", ROMREGION_ERASEFF )  // Page 2 = minidisc page
+	ROM_LOAD( "mdic3.bin"  , 0x0000,0x2000, CRC(87801816) SHA1(ddf441f40df014b237cdf17430d1989f3a452d04))
+	ROM_LOAD( "mdicmb.bin" , 0x2000,0x2000, CRC(d8090747) SHA1(f2925b68002307562e2ea5e36b740e5458f0f0eb))
+ROM_END
 
 ROM_START( hec2mx80 )
 	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
@@ -651,7 +778,7 @@ ROM_START( hec2mx80 )
 	ROM_REGION( 0x04000, "rom_disc2", ROMREGION_ERASEFF )
 	ROM_LOAD( "d800k.bin" , 0x0000,0x1000, CRC(831bd584) SHA1(9782ee58f570042608d9d568b2c3fc4c6d87d8b9))
 	ROM_REGION( 0x10000,  "disc2mem", ROMREGION_ERASE00 )
-	ROM_END
+ROM_END
 
 ROM_START( hec2mx40 )
 	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
@@ -666,7 +793,6 @@ ROM_START( hec2mx40 )
 	ROM_LOAD( "d800k.bin" , 0x0000,0x1000, CRC(831bd584) SHA1(9782ee58f570042608d9d568b2c3fc4c6d87d8b9))
 //  ROM_LOAD( "d200k.bin" , 0x0000,0x4000, CRC(e2801377) SHA1(0926df5b417ecd8013e35c71b76780c5a25c1cbf))
 	ROM_REGION( 0x10000,  "disc2mem", ROMREGION_ERASE00 )
-
 ROM_END
 
 /* Driver */
@@ -676,5 +802,6 @@ COMP(1983,	hec2hrp,	0,		interact,	hec2hrp,	hec2hrp,0,		"Micronique",	"Hector 2HR
 COMP(1980,	victor,		hec2hrp, 0,			hec2hrp,	hec2hrp,0,		"Micronique",	"Victor",			GAME_IMPERFECT_SOUND)
 COMP(1983,	hec2hr,		hec2hrp, 0,			hec2hr,		hec2hrp,0,		 "Micronique",	"Hector 2HR",		GAME_IMPERFECT_SOUND)
 COMP(1984,	hec2hrx,	hec2hrp, 0,			hec2hrx,	hec2hrp,0,		 "Micronique",	"Hector HRX + Disc2",		GAME_IMPERFECT_SOUND)
+COMP(1985,	hec2mdhrx,	hec2hrp, 0,			hec2mdhrx,	hec2hrp,0,		 "Micronique",	"Hector HRX + mini Disc" ,	GAME_IMPERFECT_SOUND)
 COMP(1985,	hec2mx80,	hec2hrp, 0,			hec2mx80,	hec2hrp,0,		 "Micronique",	"Hector MX 80c + Disc2" ,	GAME_IMPERFECT_SOUND)
 COMP(1985,	hec2mx40,	hec2hrp, 0,			hec2mx40,	hec2hrp,0,		 "Micronique",	"Hector MX 40c + Disc2" ,	GAME_IMPERFECT_SOUND)
