@@ -260,14 +260,20 @@ XT U44 IBM.bin: IBM 5160 PC/XT Bank-selection decoding ROM (256x4 bit). Not mapp
 #include "machine/isa_adlib.h"
 #include "machine/isa_com.h"
 #include "machine/isa_fdc.h"
+#include "machine/isa_finalchs.h"
 #include "machine/isa_gblaster.h"
 #include "machine/isa_hdc.h"
 #include "machine/isa_sblaster.h"
 #include "machine/3c503.h"
+#include "video/isa_cga.h"
+#include "video/isa_ega.h"
+#include "video/isa_mda.h"
+#include "video/isa_svga_tseng.h"
 #include "machine/ne1000.h"
-
-#include "machine/kb_keytro.h"
-#include "video/pc_cga.h"
+#include "machine/isa_mpu401.h"
+#include "machine/isa_ibm_mfc.h"
+#include "machine/pc_lpt.h"
+#include "machine/pc_keyboards.h"
 #include "includes/genpc.h"
 
 class ibmpc_state : public driver_device
@@ -278,7 +284,7 @@ public:
 
 };
 
-static ADDRESS_MAP_START( pc8_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( pc8_map, AS_PROGRAM, 8, ibmpc_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE(0x00000, 0x9ffff) AM_RAMBANK("bank10")
 	AM_RANGE(0xa0000, 0xbffff) AM_NOP
@@ -288,44 +294,42 @@ static ADDRESS_MAP_START( pc8_map, AS_PROGRAM, 8 )
 	AM_RANGE(0xf0000, 0xfffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(pc8_io, AS_IO, 8)
+static ADDRESS_MAP_START(pc8_io, AS_IO, 8, ibmpc_state )
 	ADDRESS_MAP_UNMAP_HIGH
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( ibm5150 )
-	PORT_INCLUDE( kb_keytronic_pc )		/* IN4 - IN11 */
-//  PORT_INCLUDE( pc_mouse_microsoft )  /* IN12 - IN14 */
-//  PORT_INCLUDE( pc_joystick )         /* IN15 - IN19 */
-	PORT_INCLUDE( pcvideo_cga )
 INPUT_PORTS_END
 
 static const unsigned i86_address_mask = 0x000fffff;
 
-static const kb_keytronic_interface pc_keytronic_intf =
-{
-	DEVCB_DEVICE_LINE_MEMBER("mb", ibm5160_mb_device, keyboard_clock_w),
-	DEVCB_DEVICE_LINE_MEMBER("mb", ibm5160_mb_device, keyboard_data_w)
-};
-
-static const motherboard_interface pc_keytronic_keyboard_intf =
-{
-	DEVCB_DEVICE_LINE("keyboard", kb_keytronic_clock_w),
-	DEVCB_DEVICE_LINE("keyboard", kb_keytronic_data_w)
-};
-
 static DEVICE_INPUT_DEFAULTS_START(cga)
 	DEVICE_INPUT_DEFAULTS("DSW0",0x30, 0x20)
 DEVICE_INPUT_DEFAULTS_END
 
+//static DEVICE_INPUT_DEFAULTS_START(ega)
+//  DEVICE_INPUT_DEFAULTS("DSW0",0x30, 0x00)
+//DEVICE_INPUT_DEFAULTS_END
+
 static SLOT_INTERFACE_START(ibm_isa8_cards)
+	SLOT_INTERFACE("cga", ISA8_CGA)
+	SLOT_INTERFACE("ega", ISA8_EGA)
+	SLOT_INTERFACE("mda", ISA8_MDA)
+	SLOT_INTERFACE("hercules", ISA8_HERCULES)
+	SLOT_INTERFACE("svga_et4k", ISA8_SVGA_ET4K)
 	SLOT_INTERFACE("com", ISA8_COM)
 	SLOT_INTERFACE("fdc", ISA8_FDC)
+	SLOT_INTERFACE("finalchs", ISA8_FINALCHS)
 	SLOT_INTERFACE("hdc", ISA8_HDC)
 	SLOT_INTERFACE("adlib", ISA8_ADLIB)
 	SLOT_INTERFACE("gblaster", ISA8_GAME_BLASTER)
-	SLOT_INTERFACE("sblaster", ISA8_SOUND_BLASTER_1_0)
+	SLOT_INTERFACE("sblaster1_0", ISA8_SOUND_BLASTER_1_0)
+	SLOT_INTERFACE("sblaster1_5", ISA8_SOUND_BLASTER_1_5)
 	SLOT_INTERFACE("ne1000", NE1000)
 	SLOT_INTERFACE("3c503", EL2_3C503)
+	SLOT_INTERFACE("mpu401", ISA8_MPU401)
+	SLOT_INTERFACE("lpt", ISA8_LPT)
+	SLOT_INTERFACE("ibm_mfc", ISA8_IBM_MFC)
 SLOT_INTERFACE_END
 
 static MACHINE_CONFIG_START( ibm5150, ibmpc_state )
@@ -335,27 +339,31 @@ static MACHINE_CONFIG_START( ibm5150, ibmpc_state )
 	MCFG_CPU_IO_MAP(pc8_io)
 	MCFG_CPU_CONFIG(i86_address_mask)
 
-	MCFG_IBM5150_MOTHERBOARD_ADD("mb","maincpu",pc_keytronic_keyboard_intf)
+	MCFG_IBM5150_MOTHERBOARD_ADD("mb","maincpu")
 	MCFG_DEVICE_INPUT_DEFAULTS(cga)
 
-	/* video hardware */
-	MCFG_FRAGMENT_ADD( pcvideo_cga )
-
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa1", ibm_isa8_cards, "com", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa2", ibm_isa8_cards, "fdc", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa3", ibm_isa8_cards, "hdc", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa4", ibm_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa1", ibm_isa8_cards, "cga", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa2", ibm_isa8_cards, "com", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa3", ibm_isa8_cards, "fdc", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa4", ibm_isa8_cards, "hdc", NULL)
 	MCFG_ISA8_SLOT_ADD("mb:isa", "isa5", ibm_isa8_cards, NULL, NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa6", ibm_isa8_cards, NULL, NULL)
 
 	/* keyboard */
-	MCFG_KB_KEYTRONIC_ADD("keyboard", pc_keytronic_intf)
-
-	//MCFG_CASSETTE_ADD( CASSETTE_TAG, ibm5150_cassette_interface )
+	MCFG_PC_KBDC_SLOT_ADD("mb:pc_kbdc", "kbd", pc_xt_keyboards, STR_KBD_KEYTRONIC_PC3270, NULL)
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("640K")
+
+	/* software lists */
+	MCFG_SOFTWARE_LIST_ADD("disk_list","ibm5150")
+MACHINE_CONFIG_END
+
+
+static MACHINE_CONFIG_DERIVED( ibm5140, ibm5150 )
+	/* software lists */
+	MCFG_DEVICE_REMOVE( "disk_list" )
+	MCFG_SOFTWARE_LIST_ADD("disk_list","ibm5140")
 MACHINE_CONFIG_END
 
 
@@ -366,21 +374,20 @@ static MACHINE_CONFIG_START( ibm5160, ibmpc_state )
 	MCFG_CPU_IO_MAP(pc8_io)
 	MCFG_CPU_CONFIG(i86_address_mask)
 
-	MCFG_IBM5160_MOTHERBOARD_ADD("mb","maincpu",pc_keytronic_keyboard_intf)
+	MCFG_IBM5160_MOTHERBOARD_ADD("mb","maincpu")
 	MCFG_DEVICE_INPUT_DEFAULTS(cga)
 
-	/* video hardware */
-	MCFG_FRAGMENT_ADD( pcvideo_cga )
-
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa1", ibm_isa8_cards, "com", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa2", ibm_isa8_cards, "fdc", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa3", ibm_isa8_cards, "hdc", NULL)
-	MCFG_ISA8_SLOT_ADD("mb:isa", "isa4", ibm_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa1", ibm_isa8_cards, "cga", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa2", ibm_isa8_cards, "com", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa3", ibm_isa8_cards, "fdc", NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa4", ibm_isa8_cards, "hdc", NULL)
 	MCFG_ISA8_SLOT_ADD("mb:isa", "isa5", ibm_isa8_cards, NULL, NULL)
 	MCFG_ISA8_SLOT_ADD("mb:isa", "isa6", ibm_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa7", ibm_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD("mb:isa", "isa8", ibm_isa8_cards, NULL, NULL)
 
 	/* keyboard */
-	MCFG_KB_KEYTRONIC_ADD("keyboard", pc_keytronic_intf)
+	MCFG_PC_KBDC_SLOT_ADD("mb:pc_kbdc", "kbd", pc_xt_keyboards, STR_KBD_KEYTRONIC_PC3270, NULL)
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
@@ -591,7 +598,7 @@ ROM_END
 /*     YEAR     NAME        PARENT      COMPAT  MACHINE     INPUT       INIT        COMPANY     FULLNAME */
 COMP(  1981,	ibm5150,    0,			0,		ibm5150,    ibm5150,	0,    "International Business Machines",  "IBM PC 5150" , GAME_NO_SOUND)
 COMP(  1982,	ibm5155,    ibm5150,	0,		ibm5150,    ibm5150,	0,    "International Business Machines",  "IBM PC 5155" , GAME_NO_SOUND)
-COMP(  1985,	ibm5140,    ibm5150,	0,		ibm5150,    ibm5150,	0,    "International Business Machines",  "IBM PC 5140 Convertible" , GAME_NOT_WORKING | GAME_NO_SOUND)
+COMP(  1985,	ibm5140,    ibm5150,	0,		ibm5140,    ibm5150,	0,    "International Business Machines",  "IBM PC 5140 Convertible" , GAME_NOT_WORKING | GAME_NO_SOUND)
 
 // xt class (pc but 8086)
 COMP(  1982,	ibm5160,    ibm5150,	0,		ibm5160,    ibm5150,	0,    "International Business Machines",  "IBM XT 5160" , GAME_NO_SOUND)
