@@ -22,7 +22,6 @@
 #include "machine/8530scc.h"
 #include "machine/sgi.h"
 #include "machine/eeprom.h"
-#include "machine/8530scc.h"
 #include "machine/wd33c93.h"
 #include "imagedev/harddriv.h"
 #include "imagedev/chd_cd.h"
@@ -53,6 +52,10 @@ public:
 
 	HPC_t m_HPC;
 	RTC_t m_RTC;
+	DECLARE_READ32_MEMBER(hpc_r);
+	DECLARE_WRITE32_MEMBER(hpc_w);
+	DECLARE_READ32_MEMBER(int_r);
+	DECLARE_WRITE32_MEMBER(int_w);
 };
 
 
@@ -75,7 +78,7 @@ static VIDEO_START( ip204415 )
 {
 }
 
-static SCREEN_UPDATE( ip204415 )
+static SCREEN_UPDATE_IND16( ip204415 )
 {
 	return 0;
 }
@@ -103,33 +106,30 @@ static const eeprom_interface eeprom_interface_93C56 =
 #define RTC_SECOND		state->m_RTC.nRAM[0x06]
 #define RTC_HUNDREDTH	state->m_RTC.nRAM[0x05]
 
-static READ32_HANDLER( hpc_r )
+READ32_MEMBER(ip20_state::hpc_r)
 {
-	ip20_state *state = space->machine().driver_data<ip20_state>();
-	device_t *scc;
-	running_machine &machine = space->machine();
-
+	scc8530_t *scc;
 	offset <<= 2;
 	if( offset >= 0x0e00 && offset <= 0x0e7c )
 	{
-		verboselog(machine, 2, "RTC RAM[0x%02x] Read: %02x\n", ( offset - 0xe00 ) >> 2, state->m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ] );
-		return state->m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ];
+		verboselog(machine(), 2, "RTC RAM[0x%02x] Read: %02x\n", ( offset - 0xe00 ) >> 2, m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ] );
+		return m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ];
 	}
 	switch( offset )
 	{
 	case 0x05c:
-		verboselog(machine, 2, "HPC Unknown Read: %08x (%08x) (returning 0x000000a5 as kludge)\n", 0x1fb80000 + offset, mem_mask );
+		verboselog(machine(), 2, "HPC Unknown Read: %08x (%08x) (returning 0x000000a5 as kludge)\n", 0x1fb80000 + offset, mem_mask );
 		return 0x0000a500;
 	case 0x00ac:
-		verboselog(machine, 2, "HPC Parallel Buffer Pointer Read: %08x (%08x)\n", state->m_HPC.nParBufPtr, mem_mask );
-		return state->m_HPC.nParBufPtr;
+		verboselog(machine(), 2, "HPC Parallel Buffer Pointer Read: %08x (%08x)\n", m_HPC.nParBufPtr, mem_mask );
+		return m_HPC.nParBufPtr;
 	case 0x00c0:
-		verboselog(machine, 2, "HPC Endianness Read: %08x (%08x)\n", 0x0000001f, mem_mask );
+		verboselog(machine(), 2, "HPC Endianness Read: %08x (%08x)\n", 0x0000001f, mem_mask );
 		return 0x0000001f;
 	case 0x0120:
 		if (ACCESSING_BITS_8_15)
 		{
-			return ( wd33c93_r( space, 0 ) << 8 );
+			return ( wd33c93_r( &space, 0 ) << 8 );
 		}
 		else
 		{
@@ -138,116 +138,113 @@ static READ32_HANDLER( hpc_r )
 	case 0x0124:
 		if (ACCESSING_BITS_8_15)
 		{
-			return ( wd33c93_r( space, 1 ) << 8 );
+			return ( wd33c93_r( &space, 1 ) << 8 );
 		}
 		else
 		{
 			return 0;
 		}
 	case 0x01b0:
-		verboselog(machine, 2, "HPC Misc. Status Read: %08x (%08x)\n", state->m_HPC.nMiscStatus, mem_mask );
-		return state->m_HPC.nMiscStatus;
+		verboselog(machine(), 2, "HPC Misc. Status Read: %08x (%08x)\n", m_HPC.nMiscStatus, mem_mask );
+		return m_HPC.nMiscStatus;
 	case 0x01bc:
 //      verboselog(machine, 2, "HPC CPU Serial EEPROM Read\n" );
-		return ( (space->machine().device<eeprom_device>("eeprom")->read_bit() << 4 ) );
+		return ( (machine().device<eeprom_device>("eeprom")->read_bit() << 4 ) );
 	case 0x01c4:
-		verboselog(machine, 2, "HPC Local IO Register 0 Mask Read: %08x (%08x)\n", state->m_HPC.nLocalIOReg0Mask, mem_mask );
-		return state->m_HPC.nLocalIOReg0Mask;
+		verboselog(machine(), 2, "HPC Local IO Register 0 Mask Read: %08x (%08x)\n", m_HPC.nLocalIOReg0Mask, mem_mask );
+		return m_HPC.nLocalIOReg0Mask;
 	case 0x01cc:
-		verboselog(machine, 2, "HPC Local IO Register 1 Mask Read: %08x (%08x)\n", state->m_HPC.nLocalIOReg0Mask, mem_mask );
-		return state->m_HPC.nLocalIOReg1Mask;
+		verboselog(machine(), 2, "HPC Local IO Register 1 Mask Read: %08x (%08x)\n", m_HPC.nLocalIOReg0Mask, mem_mask );
+		return m_HPC.nLocalIOReg1Mask;
 	case 0x01d4:
-		verboselog(machine, 2, "HPC VME Interrupt Mask 0 Read: %08x (%08x)\n", state->m_HPC.nLocalIOReg0Mask, mem_mask );
-		return state->m_HPC.nVMEIntMask0;
+		verboselog(machine(), 2, "HPC VME Interrupt Mask 0 Read: %08x (%08x)\n", m_HPC.nLocalIOReg0Mask, mem_mask );
+		return m_HPC.nVMEIntMask0;
 	case 0x01d8:
-		verboselog(machine, 2, "HPC VME Interrupt Mask 1 Read: %08x (%08x)\n", state->m_HPC.nLocalIOReg0Mask, mem_mask );
-		return state->m_HPC.nVMEIntMask1;
+		verboselog(machine(), 2, "HPC VME Interrupt Mask 1 Read: %08x (%08x)\n", m_HPC.nLocalIOReg0Mask, mem_mask );
+		return m_HPC.nVMEIntMask1;
 	case 0x0d00:
-		verboselog(machine, 2, "HPC DUART0 Channel B Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART0 Channel B Control Read\n" );
 //      return 0x00000004;
-		return 0x7c; //scc8530_r(machine, 0);
+		return 0x7c; //scc->reg_r(space, 0);
 	case 0x0d04:
-		verboselog(machine, 2, "HPC DUART0 Channel B Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART0 Channel B Data Read\n" );
 //      return 0;
-		scc = space->machine().device("scc");
-		return scc8530_r(scc, 2);
+		scc = machine().device<scc8530_t>("scc");
+		return scc->reg_r(space, 2);
 	case 0x0d08:
-		verboselog(machine, 2, "HPC DUART0 Channel A Control Read (%08x)\n", mem_mask	 );
+		verboselog(machine(), 2, "HPC DUART0 Channel A Control Read (%08x)\n", mem_mask	 );
 //      return 0x40;
-		return 0x7c; //scc8530_r(machine, 1);
+		return 0x7c; //scc->reg_r(space, 1);
 	case 0x0d0c:
-		verboselog(machine, 2, "HPC DUART0 Channel A Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART0 Channel A Data Read\n" );
 //      return 0;
-		scc = space->machine().device("scc");
-		return scc8530_r(scc, 3);
+		scc = machine().device<scc8530_t>("scc");
+		return scc->reg_r(space, 3);
 	case 0x0d10:
 //      verboselog(machine, 2, "HPC DUART1 Channel B Control Read\n" );
 		return 0x00000004;
 	case 0x0d14:
-		verboselog(machine, 2, "HPC DUART1 Channel B Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART1 Channel B Data Read\n" );
 		return 0;
 	case 0x0d18:
-		verboselog(machine, 2, "HPC DUART1 Channel A Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART1 Channel A Control Read\n" );
 		return 0;
 	case 0x0d1c:
-		verboselog(machine, 2, "HPC DUART1 Channel A Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART1 Channel A Data Read\n" );
 		return 0;
 	case 0x0d20:
-		verboselog(machine, 2, "HPC DUART2 Channel B Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART2 Channel B Control Read\n" );
 		return 0x00000004;
 	case 0x0d24:
-		verboselog(machine, 2, "HPC DUART2 Channel B Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART2 Channel B Data Read\n" );
 		return 0;
 	case 0x0d28:
-		verboselog(machine, 2, "HPC DUART2 Channel A Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART2 Channel A Control Read\n" );
 		return 0;
 	case 0x0d2c:
-		verboselog(machine, 2, "HPC DUART2 Channel A Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART2 Channel A Data Read\n" );
 		return 0;
 	case 0x0d30:
-		verboselog(machine, 2, "HPC DUART3 Channel B Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART3 Channel B Control Read\n" );
 		return 0x00000004;
 	case 0x0d34:
-		verboselog(machine, 2, "HPC DUART3 Channel B Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART3 Channel B Data Read\n" );
 		return 0;
 	case 0x0d38:
-		verboselog(machine, 2, "HPC DUART3 Channel A Control Read\n" );
+		verboselog(machine(), 2, "HPC DUART3 Channel A Control Read\n" );
 		return 0;
 	case 0x0d3c:
-		verboselog(machine, 2, "HPC DUART3 Channel A Data Read\n" );
+		verboselog(machine(), 2, "HPC DUART3 Channel A Data Read\n" );
 		return 0;
 	}
-	verboselog(machine, 0, "Unmapped HPC read: 0x%08x (%08x)\n", 0x1fb80000 + offset, mem_mask );
+	verboselog(machine(), 0, "Unmapped HPC read: 0x%08x (%08x)\n", 0x1fb80000 + offset, mem_mask );
 	return 0;
 }
 
-static WRITE32_HANDLER( hpc_w )
+WRITE32_MEMBER(ip20_state::hpc_w)
 {
-	ip20_state *state = space->machine().driver_data<ip20_state>();
-	device_t *scc;
+	scc8530_t *scc;
 	eeprom_device *eeprom;
-	running_machine &machine = space->machine();
-
-	eeprom = space->machine().device<eeprom_device>("eeprom");
+	eeprom = machine().device<eeprom_device>("eeprom");
 	offset <<= 2;
 	if( offset >= 0x0e00 && offset <= 0x0e7c )
 	{
-		verboselog(machine, 2, "RTC RAM[0x%02x] Write: %02x\n", ( offset - 0xe00 ) >> 2, data & 0x000000ff );
-		state->m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ] = data & 0x000000ff;
+		verboselog(machine(), 2, "RTC RAM[0x%02x] Write: %02x\n", ( offset - 0xe00 ) >> 2, data & 0x000000ff );
+		m_RTC.nRAM[ ( offset - 0xe00 ) >> 2 ] = data & 0x000000ff;
 		switch( ( offset - 0xe00 ) >> 2 )
 		{
 		case 0:
 			break;
 		case 4:
-			if( !( state->m_RTC.nRAM[0x00] & 0x80 ) )
+			if( !( m_RTC.nRAM[0x00] & 0x80 ) )
 			{
 				if( data & 0x80 )
 				{
-					state->m_RTC.nRAM[0x19] = RTC_SECOND;
-					state->m_RTC.nRAM[0x1a] = RTC_MINUTE;
-					state->m_RTC.nRAM[0x1b] = RTC_HOUR;
-					state->m_RTC.nRAM[0x1c] = RTC_DAY;
-					state->m_RTC.nRAM[0x1d] = RTC_MONTH;
+					m_RTC.nRAM[0x19] = m_RTC.nRAM[0x06]; //RTC_SECOND;
+					m_RTC.nRAM[0x1a] = m_RTC.nRAM[0x07]; //RTC_MINUTE;
+					m_RTC.nRAM[0x1b] = m_RTC.nRAM[0x08]; //RTC_HOUR;
+					m_RTC.nRAM[0x1c] = m_RTC.nRAM[0x09]; //RTC_DAY;
+					m_RTC.nRAM[0x1d] = m_RTC.nRAM[0x0a]; //RTC_MONTH;
 				}
 			}
 			break;
@@ -257,11 +254,11 @@ static WRITE32_HANDLER( hpc_w )
 	switch( offset )
 	{
 	case 0x0090:	// SCSI0 next descriptor pointer
-		state->m_HPC.nSCSI0Descriptor = data;
+		m_HPC.nSCSI0Descriptor = data;
 		break;
 
 	case 0x0094:	// SCSI0 control flags
-		state->m_HPC.nSCSI0DMACtrl = data;
+		m_HPC.nSCSI0DMACtrl = data;
 		#if 0
 		if (data & 0x80)
 		{
@@ -270,10 +267,10 @@ static WRITE32_HANDLER( hpc_w )
 			mame_printf_info("DMA activated for SCSI0\n");
 			mame_printf_info("Descriptor block:\n");
 			mame_printf_info("CTL: %08x BUFPTR: %08x DESCPTR %08x\n",
-				program_read_dword(state->m_HPC.nSCSI0Descriptor), program_read_dword(state->m_HPC.nSCSI0Descriptor+4),
-				program_read_dword(state->m_HPC.nSCSI0Descriptor+8));
+				program_read_dword(m_HPC.nSCSI0Descriptor), program_read_dword(m_HPC.nSCSI0Descriptor+4),
+				program_read_dword(m_HPC.nSCSI0Descriptor+8));
 
-			next = program_read_dword(state->m_HPC.nSCSI0Descriptor+8);
+			next = program_read_dword(m_HPC.nSCSI0Descriptor+8);
 			mame_printf_info("CTL: %08x BUFPTR: %08x DESCPTR %08x\n",
 				program_read_dword(next), program_read_dword(next+4),
 				program_read_dword(next+8));
@@ -282,14 +279,14 @@ static WRITE32_HANDLER( hpc_w )
 		break;
 
 	case 0x00ac:
-		verboselog(machine, 2, "HPC Parallel Buffer Pointer Write: %08x (%08x)\n", data, mem_mask );
-		state->m_HPC.nParBufPtr = data;
+		verboselog(machine(), 2, "HPC Parallel Buffer Pointer Write: %08x (%08x)\n", data, mem_mask );
+		m_HPC.nParBufPtr = data;
 		break;
 	case 0x0120:
 		if (ACCESSING_BITS_8_15)
 		{
-			verboselog(machine, 2, "HPC SCSI Controller Register Write: %08x\n", ( data >> 8 ) & 0x000000ff );
-			wd33c93_w( space, 0, ( data >> 8 ) & 0x000000ff );
+			verboselog(machine(), 2, "HPC SCSI Controller Register Write: %08x\n", ( data >> 8 ) & 0x000000ff );
+			wd33c93_w( &space, 0, ( data >> 8 ) & 0x000000ff );
 		}
 		else
 		{
@@ -299,8 +296,8 @@ static WRITE32_HANDLER( hpc_w )
 	case 0x0124:
 		if (ACCESSING_BITS_8_15)
 		{
-			verboselog(machine, 2, "HPC SCSI Controller Data Write: %08x\n", ( data >> 8 ) & 0x000000ff );
-			wd33c93_w( space, 1, ( data >> 8 ) & 0x000000ff );
+			verboselog(machine(), 2, "HPC SCSI Controller Data Write: %08x\n", ( data >> 8 ) & 0x000000ff );
+			wd33c93_w( &space, 1, ( data >> 8 ) & 0x000000ff );
 		}
 		else
 		{
@@ -308,130 +305,130 @@ static WRITE32_HANDLER( hpc_w )
 		}
 		break;
 	case 0x01b0:
-		verboselog(machine, 2, "HPC Misc. Status Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC Misc. Status Write: %08x (%08x)\n", data, mem_mask );
 		if( data & 0x00000001 )
 		{
-			verboselog(machine, 2, "  Force DSP hard reset\n" );
+			verboselog(machine(), 2, "  Force DSP hard reset\n" );
 		}
 		if( data & 0x00000002 )
 		{
-			verboselog(machine, 2, "  Force IRQA\n" );
+			verboselog(machine(), 2, "  Force IRQA\n" );
 		}
 		if( data & 0x00000004 )
 		{
-			verboselog(machine, 2, "  Set IRQA polarity high\n" );
+			verboselog(machine(), 2, "  Set IRQA polarity high\n" );
 		}
 		else
 		{
-			verboselog(machine, 2, "  Set IRQA polarity low\n" );
+			verboselog(machine(), 2, "  Set IRQA polarity low\n" );
 		}
 		if( data & 0x00000008 )
 		{
-			verboselog(machine, 2, "  SRAM size: 32K\n" );
+			verboselog(machine(), 2, "  SRAM size: 32K\n" );
 		}
 		else
 		{
-			verboselog(machine, 2, "  SRAM size:  8K\n" );
+			verboselog(machine(), 2, "  SRAM size:  8K\n" );
 		}
-		state->m_HPC.nMiscStatus = data;
+		m_HPC.nMiscStatus = data;
 		break;
 	case 0x01bc:
 //      verboselog(machine, 2, "HPC CPU Serial EEPROM Write: %08x (%08x)\n", data, mem_mask );
 		if( data & 0x00000001 )
 		{
-			verboselog(machine, 2, "    CPU board LED on\n" );
+			verboselog(machine(), 2, "    CPU board LED on\n" );
 		}
 		eeprom->write_bit((data & 0x00000008) ? 1 : 0 );
 		eeprom->set_cs_line((data & 0x00000002) ? ASSERT_LINE : CLEAR_LINE );
 		eeprom->set_clock_line((data & 0x00000004) ? CLEAR_LINE : ASSERT_LINE );
 		break;
 	case 0x01c4:
-		verboselog(machine, 2, "HPC Local IO Register 0 Mask Write: %08x (%08x)\n", data, mem_mask );
-		state->m_HPC.nLocalIOReg0Mask = data;
+		verboselog(machine(), 2, "HPC Local IO Register 0 Mask Write: %08x (%08x)\n", data, mem_mask );
+		m_HPC.nLocalIOReg0Mask = data;
 		break;
 	case 0x01cc:
-		verboselog(machine, 2, "HPC Local IO Register 1 Mask Write: %08x (%08x)\n", data, mem_mask );
-		state->m_HPC.nLocalIOReg1Mask = data;
+		verboselog(machine(), 2, "HPC Local IO Register 1 Mask Write: %08x (%08x)\n", data, mem_mask );
+		m_HPC.nLocalIOReg1Mask = data;
 		break;
 	case 0x01d4:
-		verboselog(machine, 2, "HPC VME Interrupt Mask 0 Write: %08x (%08x)\n", data, mem_mask );
-		state->m_HPC.nVMEIntMask0 = data;
+		verboselog(machine(), 2, "HPC VME Interrupt Mask 0 Write: %08x (%08x)\n", data, mem_mask );
+		m_HPC.nVMEIntMask0 = data;
 		break;
 	case 0x01d8:
-		verboselog(machine, 2, "HPC VME Interrupt Mask 1 Write: %08x (%08x)\n", data, mem_mask );
-		state->m_HPC.nVMEIntMask1 = data;
+		verboselog(machine(), 2, "HPC VME Interrupt Mask 1 Write: %08x (%08x)\n", data, mem_mask );
+		m_HPC.nVMEIntMask1 = data;
 		break;
 	case 0x0d00:
-		verboselog(machine, 2, "HPC DUART0 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
-		scc = space->machine().device("scc");
-		scc8530_w(scc, 0, data);
+		verboselog(machine(), 2, "HPC DUART0 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
+		scc = machine().device<scc8530_t>("scc");
+		scc->reg_w(space, 0, data);
 		break;
 	case 0x0d04:
-		verboselog(machine, 2, "HPC DUART0 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
-		scc = space->machine().device("scc");
-		scc8530_w(scc, 2, data);
+		verboselog(machine(), 2, "HPC DUART0 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
+		scc = machine().device<scc8530_t>("scc");
+		scc->reg_w(space, 2, data);
 		break;
 	case 0x0d08:
-		verboselog(machine, 2, "HPC DUART0 Channel A Control Write: %08x (%08x)\n", data, mem_mask );
-		scc = space->machine().device("scc");
-		scc8530_w(scc, 1, data);
+		verboselog(machine(), 2, "HPC DUART0 Channel A Control Write: %08x (%08x)\n", data, mem_mask );
+		scc = machine().device<scc8530_t>("scc");
+		scc->reg_w(space, 1, data);
 		break;
 	case 0x0d0c:
-		verboselog(machine, 2, "HPC DUART0 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
-		scc = space->machine().device("scc");
-		scc8530_w(scc, 3, data);
+		verboselog(machine(), 2, "HPC DUART0 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
+		scc = machine().device<scc8530_t>("scc");
+		scc->reg_w(space, 3, data);
 		break;
 	case 0x0d10:
 		if( ( data & 0x000000ff ) >= 0x00000020 )
 		{
-//          verboselog(machine, 2, "HPC DUART1 Channel B Control Write: %08x (%08x) %c\n", data, mem_mask, data & 0x000000ff );
+//          verboselog(machine(), 2, "HPC DUART1 Channel B Control Write: %08x (%08x) %c\n", data, mem_mask, data & 0x000000ff );
 			//mame_printf_info( "%c", data & 0x000000ff );
 		}
 		else
 		{
-//          verboselog(machine, 2, "HPC DUART1 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
+//          verboselog(machine(), 2, "HPC DUART1 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
 		}
 		break;
 	case 0x0d14:
 		if( ( data & 0x000000ff ) >= 0x00000020 || ( data & 0x000000ff ) == 0x0d || ( data & 0x000000ff ) == 0x0a )
 		{
-			verboselog(machine, 2, "HPC DUART1 Channel B Data Write: %08x (%08x) %c\n", data, mem_mask, data & 0x000000ff );
+			verboselog(machine(), 2, "HPC DUART1 Channel B Data Write: %08x (%08x) %c\n", data, mem_mask, data & 0x000000ff );
 			mame_printf_info( "%c", data & 0x000000ff );
 		}
 		else
 		{
-			verboselog(machine, 2, "HPC DUART1 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
+			verboselog(machine(), 2, "HPC DUART1 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
 		}
 		break;
 	case 0x0d18:
 		mame_printf_info("HPC DUART1 Channel A Control Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d1c:
-		verboselog(machine, 2, "HPC DUART1 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC DUART1 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d20:
 		mame_printf_info("HPC DUART2 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d24:
-		verboselog(machine, 2, "HPC DUART2 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC DUART2 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d28:
 		mame_printf_info("HPC DUART2 Channel A Control Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d2c:
-		verboselog(machine, 2, "HPC DUART2 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC DUART2 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d30:
 		mame_printf_info("HPC DUART3 Channel B Control Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d34:
-		verboselog(machine, 2, "HPC DUART3 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC DUART3 Channel B Data Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d38:
 		mame_printf_info("HPC DUART3 Channel A Control Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	case 0x0d3c:
-		verboselog(machine, 2, "HPC DUART3 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
+		verboselog(machine(), 2, "HPC DUART3 Channel A Data Write: %08x (%08x)\n", data, mem_mask );
 		break;
 	default:
 		mame_printf_info("Unmapped HPC write: 0x%08x (%08x): %08x\n", 0x1fb80000 + offset, mem_mask, data);
@@ -440,18 +437,18 @@ static WRITE32_HANDLER( hpc_w )
 }
 
 // INT/INT2/INT3 interrupt controllers
-static READ32_HANDLER( int_r )
+READ32_MEMBER(ip20_state::int_r)
 {
-	mame_printf_info("INT: read @ ofs %x (mask %x) (PC=%x)\n", offset, mem_mask, cpu_get_pc(&space->device()));
+	mame_printf_info("INT: read @ ofs %x (mask %x) (PC=%x)\n", offset, mem_mask, cpu_get_pc(&space.device()));
 	return 0;
 }
 
-static WRITE32_HANDLER( int_w )
+WRITE32_MEMBER(ip20_state::int_w)
 {
-	mame_printf_info("INT: write %x to ofs %x (mask %x) (PC=%x)\n", data, offset, mem_mask, cpu_get_pc(&space->device()));
+	mame_printf_info("INT: write %x to ofs %x (mask %x) (PC=%x)\n", data, offset, mem_mask, cpu_get_pc(&space.device()));
 }
 
-static ADDRESS_MAP_START( ip204415_map, AS_PROGRAM, 32 )
+static ADDRESS_MAP_START( ip204415_map, AS_PROGRAM, 32, ip20_state )
 	AM_RANGE( 0x00000000, 0x001fffff ) AM_RAM AM_SHARE("share10")
 	AM_RANGE( 0x08000000, 0x08ffffff ) AM_RAM AM_SHARE("share5")
 	AM_RANGE( 0x09000000, 0x097fffff ) AM_RAM AM_SHARE("share6")
@@ -459,9 +456,9 @@ static ADDRESS_MAP_START( ip204415_map, AS_PROGRAM, 32 )
 	AM_RANGE( 0x0c000000, 0x0c7fffff ) AM_RAM AM_SHARE("share8")
 	AM_RANGE( 0x10000000, 0x107fffff ) AM_RAM AM_SHARE("share9")
 	AM_RANGE( 0x18000000, 0x187fffff ) AM_RAM AM_SHARE("share1")
-	AM_RANGE( 0x1fa00000, 0x1fa1ffff ) AM_READWRITE( sgi_mc_r, sgi_mc_w )
-	AM_RANGE( 0x1fb80000, 0x1fb8ffff ) AM_READWRITE( hpc_r, hpc_w )
-	AM_RANGE( 0x1fbd9000, 0x1fbd903f ) AM_READWRITE( int_r, int_w )
+	AM_RANGE( 0x1fa00000, 0x1fa1ffff ) AM_READWRITE_LEGACY(sgi_mc_r, sgi_mc_w )
+	AM_RANGE( 0x1fb80000, 0x1fb8ffff ) AM_READWRITE(hpc_r, hpc_w )
+	AM_RANGE( 0x1fbd9000, 0x1fbd903f ) AM_READWRITE(int_r, int_w )
 	AM_RANGE( 0x1fc00000, 0x1fc7ffff ) AM_ROM AM_SHARE("share2") AM_REGION( "user1", 0 )
 	AM_RANGE( 0x80000000, 0x801fffff ) AM_RAM AM_SHARE("share10")
 	AM_RANGE( 0x88000000, 0x88ffffff ) AM_RAM AM_SHARE("share5")
@@ -472,9 +469,9 @@ static ADDRESS_MAP_START( ip204415_map, AS_PROGRAM, 32 )
 	AM_RANGE( 0xac000000, 0xac7fffff ) AM_RAM AM_SHARE("share8")
 	AM_RANGE( 0xb0000000, 0xb07fffff ) AM_RAM AM_SHARE("share9")
 	AM_RANGE( 0xb8000000, 0xb87fffff ) AM_RAM AM_SHARE("share1")
-	AM_RANGE( 0xbfa00000, 0xbfa1ffff ) AM_READWRITE( sgi_mc_r, sgi_mc_w )
-	AM_RANGE( 0xbfb80000, 0xbfb8ffff ) AM_READWRITE( hpc_r, hpc_w )
-	AM_RANGE( 0xbfbd9000, 0xbfbd903f ) AM_READWRITE( int_r, int_w )
+	AM_RANGE( 0xbfa00000, 0xbfa1ffff ) AM_READWRITE_LEGACY(sgi_mc_r, sgi_mc_w )
+	AM_RANGE( 0xbfb80000, 0xbfb8ffff ) AM_READWRITE(hpc_r, hpc_w )
+	AM_RANGE( 0xbfbd9000, 0xbfbd903f ) AM_READWRITE(int_r, int_w )
 	AM_RANGE( 0xbfc00000, 0xbfc7ffff ) AM_ROM AM_SHARE("share2") /* BIOS Mirror */
 ADDRESS_MAP_END
 
@@ -611,10 +608,9 @@ static MACHINE_CONFIG_START( ip204415, ip20_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE( 60 )
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(800, 600)
 	MCFG_SCREEN_VISIBLE_AREA(0, 799, 0, 599)
-	MCFG_SCREEN_UPDATE( ip204415 )
+	MCFG_SCREEN_UPDATE_STATIC( ip204415 )
 
 	MCFG_PALETTE_LENGTH(65536)
 
@@ -625,7 +621,7 @@ static MACHINE_CONFIG_START( ip204415, ip20_state )
 	MCFG_SOUND_ADD( "cdda", CDDA, 0 )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
 
-	MCFG_SCC8530_ADD("scc", 7000000)
+	MCFG_SCC8530_ADD("scc", 7000000, line_cb_t())
 
 	MCFG_CDROM_ADD( "cdrom",ip20_cdrom )
 
