@@ -40,7 +40,6 @@ ToDo:
   needs to be tested with a real cassette dump.
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/i8085/i8085.h"
@@ -61,7 +60,7 @@ ToDo:
 #include "video/iq151_video64.h"
 
 #define MACHINE_RESET_MEMBER(name) void name::machine_reset()
-#define SCREEN_UPDATE_MEMBER(name) bool name::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+#define SCREEN_UPDATE16_MEMBER(name) UINT32 name::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 
 
 class iq151_state : public driver_device
@@ -91,7 +90,7 @@ public:
 	DECLARE_READ8_MEMBER(cartslot_io_r);
 	DECLARE_WRITE8_MEMBER(cartslot_io_w);
 	virtual void machine_reset();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 
 	UINT8 m_vblank_irq_state;
 	UINT8 m_cassette_clk;
@@ -107,7 +106,7 @@ READ8_MEMBER(iq151_state::keyboard_row_r)
 	for (int i = 0; i < 8; i++)
 	{
 		sprintf(kbdrow,"X%X",i);
-		data &= input_port_read(machine(), kbdrow);
+		data &= ioport(kbdrow)->read();
 	}
 
 	return data;
@@ -121,7 +120,7 @@ READ8_MEMBER(iq151_state::keyboard_column_r)
 	for (int i = 0; i < 8; i++)
 	{
 		sprintf(kbdrow,"X%X",i);
-		if (input_port_read(machine(), kbdrow) == 0xff)
+		if (ioport(kbdrow)->read() == 0xff)
 			data |= (1 << i);
 	}
 
@@ -141,7 +140,7 @@ READ8_MEMBER(iq151_state::ppi_portc_r)
 	else
 	{
 		// kb read
-		data = input_port_read(machine(), "X8");
+		data = ioport("X8")->read();
 	}
 
 	return (data & 0xf0) | (m_cassette_data & 0x0f);
@@ -157,7 +156,7 @@ WRITE8_MEMBER(iq151_state::ppi_portc_w)
 
 WRITE8_MEMBER(iq151_state::boot_bank_w)
 {
-	memory_set_bank(machine(), "boot", data & 1);
+	membank("boot")->set_entry(data & 1);
 }
 
 
@@ -349,9 +348,9 @@ DRIVER_INIT( iq151 )
 {
 	iq151_state *state = machine.driver_data<iq151_state>();
 
-	UINT8 *RAM = machine.region("maincpu")->base();
-	memory_configure_bank(machine, "boot", 0, 1, RAM + 0xf800, 0);
-	memory_configure_bank(machine, "boot", 1, 1, RAM + 0x0000, 0);
+	UINT8 *RAM = state->memregion("maincpu")->base();
+	state->membank("boot")->configure_entry(0, RAM + 0xf800);
+	state->membank("boot")->configure_entry(1, RAM + 0x0000);
 
 	device_set_irq_callback(state->m_maincpu, iq151_irq_callback);
 
@@ -365,15 +364,15 @@ DRIVER_INIT( iq151 )
 
 MACHINE_RESET_MEMBER( iq151_state )
 {
-	memory_set_bank(machine(), "boot", 0);
+	membank("boot")->set_entry(0);
 
 	m_vblank_irq_state = 0;
 }
 
 // this machine don't have a built-in video controller, but uses external cartridge
-SCREEN_UPDATE_MEMBER( iq151_state )
+SCREEN_UPDATE16_MEMBER( iq151_state )
 {
-	bitmap_fill(&bitmap, &cliprect, 0);
+	bitmap.fill(0, cliprect);
 
 	for (int i=0; i<5; i++)
 		m_carts[i]->video_update(bitmap, cliprect);
@@ -444,7 +443,7 @@ static MACHINE_CONFIG_START( iq151, iq151_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(iq151_state, screen_update)
 	MCFG_SCREEN_SIZE(64*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0, 32*8-1, 0, 32*8-1)
 	MCFG_PALETTE_LENGTH(2)
