@@ -27,7 +27,7 @@ WRITE8_MEMBER( jtc_state::p2_w )
 
     */
 
-	centronics_strobe_w(m_centronics, BIT(data, 5));
+	m_centronics->strobe_w(BIT(data, 5));
 }
 
 READ8_MEMBER( jtc_state::p3_r )
@@ -50,7 +50,7 @@ READ8_MEMBER( jtc_state::p3_r )
 	UINT8 data = 0;
 
 	data |= ((m_cassette)->input() < 0.0) ? 1 : 0;
-	data |= centronics_busy_r(m_centronics) << 3;
+	data |= m_centronics->busy_r() << 3;
 
 	return data;
 }
@@ -121,7 +121,7 @@ static ADDRESS_MAP_START( jtc_mem, AS_PROGRAM, 8, jtc_state )
 	AM_RANGE(0x700b, 0x700b) AM_MIRROR(0x0ff0) AM_READ_PORT("Y11")
 	AM_RANGE(0x700c, 0x700c) AM_MIRROR(0x0ff0) AM_READ_PORT("Y12")
 	AM_RANGE(0xe000, 0xfdff) AM_RAM
-	AM_RANGE(0xfe00, 0xffff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0xfe00, 0xffff) AM_RAM AM_SHARE("video_ram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( jtc_es1988_mem, AS_PROGRAM, 8, jtces88_state )
@@ -141,7 +141,7 @@ static ADDRESS_MAP_START( jtc_es1988_mem, AS_PROGRAM, 8, jtces88_state )
 	AM_RANGE(0x700b, 0x700b) AM_MIRROR(0x0ff0) AM_READ_PORT("Y11")
 	AM_RANGE(0x700c, 0x700c) AM_MIRROR(0x0ff0) AM_READ_PORT("Y12")
 	AM_RANGE(0xe000, 0xfdff) AM_RAM
-	AM_RANGE(0xfe00, 0xffff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0xfe00, 0xffff) AM_RAM AM_SHARE("video_ram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( jtc_es23_mem, AS_PROGRAM, 8, jtces23_state )
@@ -164,7 +164,7 @@ static ADDRESS_MAP_START( jtc_es23_mem, AS_PROGRAM, 8, jtces23_state )
 	AM_RANGE(0x700e, 0x700e) AM_MIRROR(0x0ff0) AM_READ_PORT("Y14")
 	AM_RANGE(0x700f, 0x700f) AM_MIRROR(0x0ff0) AM_READ_PORT("Y15")
 	AM_RANGE(0xe000, 0xfdff) AM_RAM
-	AM_RANGE(0xf800, 0xffff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0xf800, 0xffff) AM_RAM AM_SHARE("video_ram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( jtc_es40_mem, AS_PROGRAM, 8, jtces40_state )
@@ -489,7 +489,7 @@ void jtc_state::video_start()
 {
 }
 
-bool jtc_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 jtc_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	int x, y, sx;
 
@@ -502,7 +502,7 @@ bool jtc_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rec
 			for (x = 0; x < 8; x++)
 			{
 				int color = BIT(data, x);
-				*BITMAP_ADDR16(&bitmap, y, (sx * 8) + x) = color;
+				bitmap.pix16(y, (sx * 8) + x) = color;
 			}
 		}
 	}
@@ -514,7 +514,7 @@ void jtces23_state::video_start()
 {
 }
 
-bool jtces23_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 jtces23_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	int x, y, sx;
 
@@ -527,7 +527,7 @@ bool jtces23_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 			for (x = 0; x < 8; x++)
 			{
 				int color = BIT(data, x);
-				*BITMAP_ADDR16(&bitmap, y, (sx * 8) + x) = color;
+				bitmap.pix16(y, (sx * 8) + x) = color;
 			}
 		}
 	}
@@ -542,20 +542,20 @@ static PALETTE_INIT( jtc_es40 )
 void jtces40_state::video_start()
 {
 	/* allocate memory */
-	m_video_ram = auto_alloc_array(machine(), UINT8, JTC_ES40_VIDEORAM_SIZE);
+	m_video_ram.allocate(JTC_ES40_VIDEORAM_SIZE);
 	m_color_ram_r = auto_alloc_array(machine(), UINT8, JTC_ES40_VIDEORAM_SIZE);
 	m_color_ram_g = auto_alloc_array(machine(), UINT8, JTC_ES40_VIDEORAM_SIZE);
 	m_color_ram_b = auto_alloc_array(machine(), UINT8, JTC_ES40_VIDEORAM_SIZE);
 
 	/* register for state saving */
 	save_item(NAME(m_video_bank));
-	save_pointer(NAME(m_video_ram), JTC_ES40_VIDEORAM_SIZE);
+	save_pointer(NAME(m_video_ram.target()), JTC_ES40_VIDEORAM_SIZE);
 	save_pointer(NAME(m_color_ram_r), JTC_ES40_VIDEORAM_SIZE);
 	save_pointer(NAME(m_color_ram_g), JTC_ES40_VIDEORAM_SIZE);
 	save_pointer(NAME(m_color_ram_b), JTC_ES40_VIDEORAM_SIZE);
 }
 
-bool jtces40_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 jtces40_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	int x, y, sx;
 
@@ -572,7 +572,7 @@ bool jtces40_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 			{
 				int color = (BIT(color_r, x) << 3) | (BIT(color_g, x) << 2) | (BIT(color_b, x) << 1) | BIT(data, x);
 
-				*BITMAP_ADDR16(&bitmap, y, (sx * 8) + x) = color;
+				bitmap.pix16(y, (sx * 8) + x) = color;
 			}
 		}
 	}
@@ -651,7 +651,7 @@ static MACHINE_CONFIG_START( basic, jtc_state )
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, jtc_cassette_interface)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( jtc, basic )
@@ -659,7 +659,7 @@ static MACHINE_CONFIG_DERIVED( jtc, basic )
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(jtc_state, screen_update)
     MCFG_SCREEN_SIZE(64, 64)
     MCFG_SCREEN_VISIBLE_AREA(0, 64-1, 0, 64-1)
 
@@ -690,7 +690,7 @@ static MACHINE_CONFIG_DERIVED_CLASS( jtces23, basic, jtces23_state )
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(jtc_state, screen_update)
     MCFG_SCREEN_SIZE(128, 128)
     MCFG_SCREEN_VISIBLE_AREA(0, 128-1, 0, 128-1)
 
@@ -712,7 +712,7 @@ static MACHINE_CONFIG_DERIVED_CLASS( jtces40, basic, jtces40_state )
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(jtc_state, screen_update)
     MCFG_SCREEN_SIZE(320, 192)
     MCFG_SCREEN_VISIBLE_AREA(0, 320-1, 0, 192-1)
 
