@@ -69,15 +69,15 @@ static UINT8 read_keyboard(running_machine &machine, UINT16 keylatch)
 {
 	UINT8 data = 0xff;
 
-	if (!BIT(keylatch, 0)) data &= input_port_read(machine, "KEY0");
-	if (!BIT(keylatch, 1)) data &= input_port_read(machine, "KEY1");
-	if (!BIT(keylatch, 2)) data &= input_port_read(machine, "KEY2");
-	if (!BIT(keylatch, 3)) data &= input_port_read(machine, "KEY3");
-	if (!BIT(keylatch, 4)) data &= input_port_read(machine, "KEY4");
-	if (!BIT(keylatch, 5)) data &= input_port_read(machine, "KEY5");
-	if (!BIT(keylatch, 6)) data &= input_port_read(machine, "KEY6");
-	if (!BIT(keylatch, 7)) data &= input_port_read(machine, "KEY7");
-	if (!BIT(keylatch, 8)) data &= input_port_read(machine, "KEY8");
+	if (!BIT(keylatch, 0)) data &= machine.root_device().ioport("KEY0")->read();
+	if (!BIT(keylatch, 1)) data &= machine.root_device().ioport("KEY1")->read();
+	if (!BIT(keylatch, 2)) data &= machine.root_device().ioport("KEY2")->read();
+	if (!BIT(keylatch, 3)) data &= machine.root_device().ioport("KEY3")->read();
+	if (!BIT(keylatch, 4)) data &= machine.root_device().ioport("KEY4")->read();
+	if (!BIT(keylatch, 5)) data &= machine.root_device().ioport("KEY5")->read();
+	if (!BIT(keylatch, 6)) data &= machine.root_device().ioport("KEY6")->read();
+	if (!BIT(keylatch, 7)) data &= machine.root_device().ioport("KEY7")->read();
+	if (!BIT(keylatch, 8)) data &= machine.root_device().ioport("KEY8")->read();
 
 	return data;
 }
@@ -123,12 +123,12 @@ void pc8201_state::bankswitch(UINT8 data)
 		program->unmap_write(0x0000, 0x7fff);
 	}
 
-	memory_set_bank(machine(), "bank1", rom_bank);
+	membank("bank1")->set_entry(rom_bank);
 
 	switch (ram_bank)
 	{
 	case 0:
-		if (ram_get_size(m_ram) > 16 * 1024)
+		if (m_ram->size() > 16 * 1024)
 		{
 			program->install_readwrite_bank(0x8000, 0xffff, "bank2");
 		}
@@ -144,21 +144,21 @@ void pc8201_state::bankswitch(UINT8 data)
 		break;
 
 	case 2:
-		if (ram_get_size(m_ram) > 32 * 1024)
+		if (m_ram->size() > 32 * 1024)
 			program->install_readwrite_bank(0x8000, 0xffff, "bank2");
 		else
 			program->unmap_readwrite(0x8000, 0xffff);
 		break;
 
 	case 3:
-		if (ram_get_size(m_ram) > 64 * 1024)
+		if (m_ram->size() > 64 * 1024)
 			program->install_readwrite_bank(0x8000, 0xffff, "bank2");
 		else
 			program->unmap_readwrite(0x8000, 0xffff);
 		break;
 	}
 
-	memory_set_bank(machine(), "bank2", ram_bank);
+	membank("bank2")->set_entry(ram_bank);
 }
 
 WRITE8_MEMBER( pc8201_state::bank_w )
@@ -205,7 +205,7 @@ WRITE8_MEMBER( pc8201_state::scp_w )
 	m_rtc->stb_w(BIT(data, 4));
 
 	/* printer strobe */
-	centronics_strobe_w(m_centronics, BIT(data, 5));
+	m_centronics->strobe_w(BIT(data, 5));
 
 	/* serial interface select */
 	m_iosel = data >> 5;
@@ -227,13 +227,14 @@ WRITE8_MEMBER( kc85_state::uart_ctrl_w )
         7
 
     */
-/*
+
     m_uart->sbs_w(BIT(data, 0));
     m_uart->epe_w(BIT(data, 1));
     m_uart->pi_w(BIT(data, 2));
     m_uart->cls1_w(BIT(data, 3));
     m_uart->cls2_w(BIT(data, 4));
-*/
+
+	m_uart->crl_w(1);
 }
 
 READ8_MEMBER( kc85_state::uart_status_r )
@@ -255,26 +256,25 @@ READ8_MEMBER( kc85_state::uart_status_r )
 
 	UINT8 data = 0x40;
 
-	// carrier detect
-	//data |= m_uart->cd_r();
+	// TODO carrier detect
 
 	// overrun error
-	//data |= m_uart->oe_r() << 1;
+	data |= m_uart->oe_r() << 1;
 
 	// framing error
-	//data |= m_uart->fe_r() << 2;
+	data |= m_uart->fe_r() << 2;
 
 	// parity error
-	//data |= m_uart->pe_r() << 3;
+	data |= m_uart->pe_r() << 3;
 
 	// transmit buffer register empty
-	//data |= m_uart->tbre_r() << 4;
+	data |= m_uart->tbre_r() << 4;
 
 	// rp TODO
 	data |= 0x20;
 
 	// low power sensor
-	data |= BIT(input_port_read(machine(), "BATTERY"), 0) << 7;
+	data |= BIT(ioport("BATTERY")->read(), 0) << 7;
 
 	return data;
 }
@@ -298,26 +298,25 @@ READ8_MEMBER( pc8201_state::uart_status_r )
 
 	UINT8 data = 0x40;
 
-	// data carrier detect / ring detect
-	//data |= m_uart->cd_r();
+	// TODO data carrier detect / ring detect
 
 	// overrun error
-	//data |= m_uart->oe_r() << 1;
+	data |= m_uart->oe_r() << 1;
 
 	// framing error
-	//data |= m_uart->fe_r() << 2;
+	data |= m_uart->fe_r() << 2;
 
 	// parity error
-	//data |= m_uart->pe_r() << 3;
+	data |= m_uart->pe_r() << 3;
 
 	// transmit buffer register empty
-	//data |= m_uart->tbre_r() << 4;
+	data |= m_uart->tbre_r() << 4;
 
 	// rp TODO
 	data |= 0x20;
 
 	// low power sensor
-	data |= BIT(input_port_read(machine(), "BATTERY"), 0) << 7;
+	data |= BIT(ioport("BATTERY")->read(), 0) << 7;
 
 	return data;
 }
@@ -392,7 +391,7 @@ READ8_MEMBER( pc8201_state::romrd_r )
 
 	if (m_rom_sel)
 	{
-		data = machine().region("option")->base()[m_rom_addr & 0x1ffff];
+		data = memregion("option")->base()[m_rom_addr & 0x1ffff];
 	}
 
 	return data;
@@ -436,10 +435,10 @@ WRITE8_MEMBER( kc85_state::ctrl_w )
     */
 
 	/* ROM bank selection */
-	memory_set_bank(machine(), "bank1", BIT(data, 0));
+	membank("bank1")->set_entry(BIT(data, 0));
 
 	/* printer strobe */
-	centronics_strobe_w(m_centronics, BIT(data, 1));
+	m_centronics->strobe_w(BIT(data, 1));
 
 	/* RTC strobe */
 	m_rtc->stb_w(BIT(data, 2));
@@ -471,10 +470,10 @@ void tandy200_state::bankswitch(UINT8 data)
 	{
 		program->install_read_bank(0x0000, 0x7fff, "bank1");
 		program->unmap_write(0x0000, 0x7fff);
-		memory_set_bank(machine(), "bank1", rom_bank);
+		membank("bank1")->set_entry(rom_bank);
 	}
 
-	if (ram_get_size(m_ram) < ((ram_bank + 1) * 24 * 1024))
+	if (m_ram->size() < ((ram_bank + 1) * 24 * 1024))
 	{
 		/* invalid RAM bank */
 		program->unmap_readwrite(0xa000, 0xffff);
@@ -482,7 +481,7 @@ void tandy200_state::bankswitch(UINT8 data)
 	else
 	{
 		program->install_readwrite_bank(0xa000, 0xffff, "bank2");
-		memory_set_bank(machine(), "bank2", ram_bank);
+		membank("bank2")->set_entry(ram_bank);
 	}
 }
 
@@ -519,7 +518,7 @@ WRITE8_MEMBER( tandy200_state::stbk_w )
     */
 
 	/* printer strobe */
-	centronics_strobe_w(m_centronics, BIT(data, 0));
+	m_centronics->strobe_w(BIT(data, 0));
 
 	/* cassette motor */
 	m_cassette->change_state(BIT(data,1) ? CASSETTE_MOTOR_ENABLED : CASSETTE_MOTOR_DISABLED, CASSETTE_MASK_MOTOR);
@@ -585,7 +584,7 @@ static ADDRESS_MAP_START( kc85_io, AS_IO, 8, kc85_state )
 //  AM_RANGE(0x90, 0x90) AM_MIRROR(0x0f) optional answering telephone unit
 //  AM_RANGE(0xa0, 0xa0) AM_MIRROR(0x0f) optional modem
 	AM_RANGE(0xb0, 0xb7) AM_MIRROR(0x08) AM_DEVREADWRITE(I8155_TAG, i8155_device, io_r, io_w)
-//  AM_RANGE(0xc0, 0xc0) AM_MIRROR(0x0f) AM_DEVREADWRITE(IM6402_TAG, data_r, data_w)
+	AM_RANGE(0xc0, 0xc0) AM_MIRROR(0x0f) AM_DEVREADWRITE(IM6402_TAG, im6402_device, read, write)
 	AM_RANGE(0xd0, 0xd0) AM_MIRROR(0x0f) AM_READWRITE(uart_status_r, uart_ctrl_w)
 	AM_RANGE(0xe0, 0xe0) AM_MIRROR(0x0f) AM_READWRITE(keyboard_r, ctrl_w)
 	AM_RANGE(0xf0, 0xf1) AM_MIRROR(0x0e) AM_READWRITE(lcd_r, lcd_w)
@@ -606,10 +605,10 @@ static ADDRESS_MAP_START( pc8201_io, AS_IO, 8, pc8201_state )
 	AM_RANGE(0x90, 0x90) AM_MIRROR(0x0f) AM_WRITE(scp_w)
 	AM_RANGE(0xa0, 0xa0) AM_MIRROR(0x0f) AM_READWRITE(bank_r, bank_w)
 	AM_RANGE(0xb0, 0xb7) AM_MIRROR(0x08) AM_DEVREADWRITE(I8155_TAG, i8155_device, io_r, io_w)
-//  AM_RANGE(0xc0, 0xc0) AM_MIRROR(0x0f) AM_DEVREADWRITE(IM6402_TAG, data_r, data_w)
-	AM_RANGE(0xd0, 0xd0) AM_MIRROR(0x0f) AM_READ(uart_status_r) AM_WRITE_BASE(kc85_state, uart_ctrl_w)
-	AM_RANGE(0xe0, 0xe0) AM_MIRROR(0x0f) AM_READ_BASE(kc85_state, keyboard_r)
-	AM_RANGE(0xf0, 0xf1) AM_MIRROR(0x0e) AM_READWRITE_BASE(kc85_state, lcd_r, lcd_w)
+	AM_RANGE(0xc0, 0xc0) AM_MIRROR(0x0f) AM_DEVREADWRITE(IM6402_TAG, im6402_device, read, write)
+	AM_RANGE(0xd0, 0xd0) AM_MIRROR(0x0f) AM_READ(uart_status_r) AM_WRITE(uart_ctrl_w)
+	AM_RANGE(0xe0, 0xe0) AM_MIRROR(0x0f) AM_READ(keyboard_r)
+	AM_RANGE(0xf0, 0xf1) AM_MIRROR(0x0e) AM_READWRITE(lcd_r, lcd_w)
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( tandy200_io, AS_IO, 8, tandy200_state )
@@ -977,8 +976,8 @@ READ8_MEMBER( kc85_state::i8155_pc_r )
 	data |= m_rtc->data_out_r();
 
 	// centronics busy
-	data |= centronics_not_busy_r(m_centronics) << 1;
-	data |= centronics_busy_r(m_centronics) << 2;
+	data |= m_centronics->not_busy_r() << 1;
+	data |= m_centronics->busy_r() << 2;
 
 	return data;
 }
@@ -989,6 +988,9 @@ WRITE_LINE_MEMBER( kc85_state::i8155_to_w )
 	{
 		speaker_level_w(m_speaker, state);
 	}
+
+	m_uart->trc_w(state);
+	m_uart->rrc_w(state);
 }
 
 static I8155_INTERFACE( kc85_8155_intf )
@@ -1019,7 +1021,7 @@ WRITE8_MEMBER( tandy200_state::i8155_pa_w )
 
     */
 
-	centronics_data_w(m_centronics, 0, data);
+	m_centronics->write(space, 0, data);
 
 	m_keylatch = (m_keylatch & 0x100) | data;
 }
@@ -1068,8 +1070,8 @@ READ8_MEMBER( tandy200_state::i8155_pc_r )
 
 	UINT8 data = 0x01;
 
-	data |= centronics_not_busy_r(m_centronics) << 1;
-	data |= centronics_busy_r(m_centronics) << 2;
+	data |= m_centronics->not_busy_r() << 1;
+	data |= m_centronics->busy_r() << 2;
 
 	return data;
 }
@@ -1093,6 +1095,19 @@ static I8155_INTERFACE( tandy200_8155_intf )
 	DEVCB_DRIVER_LINE_MEMBER(tandy200_state, i8155_to_w)	/* timer output */
 };
 
+/* IM6402 Interface */
+
+static IM6402_INTERFACE( uart_intf )
+{
+	0,
+	0,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL
+};
+
 /* I8251 Interface */
 
 static const i8251_interface tandy200_uart_intf =
@@ -1121,12 +1136,12 @@ void kc85_state::machine_start()
 	/* configure ROM banking */
 	program->install_read_bank(0x0000, 0x7fff, "bank1");
 	program->unmap_write(0x0000, 0x7fff);
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(I8085_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, machine().region("option")->base(), 0);
-	memory_set_bank(machine(), "bank1", 0);
+	membank("bank1")->configure_entry(0, memregion(I8085_TAG)->base());
+	membank("bank1")->configure_entry(1, memregion("option")->base());
+	membank("bank1")->set_entry(0);
 
 	/* configure RAM banking */
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 16 * 1024:
 		program->unmap_readwrite(0x8000, 0xbfff);
@@ -1138,43 +1153,43 @@ void kc85_state::machine_start()
 		break;
 	}
 
-	memory_configure_bank(machine(), "bank2", 0, 1, ram_get_ptr(m_ram), 0);
-	memory_set_bank(machine(), "bank2", 0);
+	membank("bank2")->configure_entry(0, m_ram->pointer());
+	membank("bank2")->set_entry(0);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_buzzer);
-	state_save_register_global(machine(), m_bell);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_buzzer));
+	save_item(NAME(m_bell));
 }
 
 void pc8201_state::machine_start()
 {
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
 	/* initialize RTC */
 	m_rtc->cs_w(1);
 	m_rtc->oe_w(1);
 
 	/* configure ROM banking */
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(I8085_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, machine().region("option")->base(), 0);
-	memory_configure_bank(machine(), "bank1", 2, 2, ram + 0x8000, 0x8000);
-	memory_set_bank(machine(), "bank1", 0);
+	membank("bank1")->configure_entry(0, memregion(I8085_TAG)->base());
+	membank("bank1")->configure_entry(1, memregion("option")->base());
+	membank("bank1")->configure_entries(2, 2, ram + 0x8000, 0x8000);
+	membank("bank1")->set_entry(0);
 
 	/* configure RAM banking */
-	memory_configure_bank(machine(), "bank2", 0, 1, ram, 0);
-	memory_configure_bank(machine(), "bank2", 2, 2, ram + 0x8000, 0x8000);
-	memory_set_bank(machine(), "bank2", 0);
+	membank("bank2")->configure_entry(0, ram);
+	membank("bank2")->configure_entries(2, 2, ram + 0x8000, 0x8000);
+	membank("bank2")->set_entry(0);
 
 	bankswitch(0);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_buzzer);
-	state_save_register_global(machine(), m_bell);
-	state_save_register_global(machine(), m_iosel);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_buzzer));
+	save_item(NAME(m_bell));
+	save_item(NAME(m_iosel));
 }
 
 void trsm100_state::machine_start()
@@ -1188,12 +1203,12 @@ void trsm100_state::machine_start()
 	/* configure ROM banking */
 	program->install_read_bank(0x0000, 0x7fff, "bank1");
 	program->unmap_write(0x0000, 0x7fff);
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(I8085_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, machine().region("option")->base(), 0);
-	memory_set_bank(machine(), "bank1", 0);
+	membank("bank1")->configure_entry(0, memregion(I8085_TAG)->base());
+	membank("bank1")->configure_entry(1, memregion("option")->base());
+	membank("bank1")->set_entry(0);
 
 	/* configure RAM banking */
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 8 * 1024:
 		program->unmap_readwrite(0x8000, 0xcfff);
@@ -1215,34 +1230,34 @@ void trsm100_state::machine_start()
 		break;
 	}
 
-	memory_configure_bank(machine(), "bank2", 0, 1, ram_get_ptr(m_ram), 0);
-	memory_set_bank(machine(), "bank2", 0);
+	membank("bank2")->configure_entry(0, m_ram->pointer());
+	membank("bank2")->set_entry(0);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_buzzer);
-	state_save_register_global(machine(), m_bell);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_buzzer));
+	save_item(NAME(m_bell));
 }
 
 void tandy200_state::machine_start()
 {
 	/* configure ROM banking */
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(I8085_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, machine().region(I8085_TAG)->base() + 0x10000, 0);
-	memory_configure_bank(machine(), "bank1", 2, 1, machine().region("option")->base(), 0);
-	memory_set_bank(machine(), "bank1", 0);
+	membank("bank1")->configure_entry(0, memregion(I8085_TAG)->base());
+	membank("bank1")->configure_entry(1, memregion(I8085_TAG)->base() + 0x10000);
+	membank("bank1")->configure_entry(2, memregion("option")->base());
+	membank("bank1")->set_entry(0);
 
 	/* configure RAM banking */
-	memory_configure_bank(machine(), "bank2", 0, 3, ram_get_ptr(m_ram), 0x6000);
-	memory_set_bank(machine(), "bank2", 0);
+	membank("bank2")->configure_entries(0, 3, m_ram->pointer(), 0x6000);
+	membank("bank2")->set_entry(0);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_bank);
-	state_save_register_global(machine(), m_tp);
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_buzzer);
-	state_save_register_global(machine(), m_bell);
+	save_item(NAME(m_bank));
+	save_item(NAME(m_tp));
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_buzzer));
+	save_item(NAME(m_bell));
 }
 
 static const cassette_interface kc85_cassette_interface =
@@ -1299,7 +1314,8 @@ static MACHINE_CONFIG_START( kc85, kc85_state )
 	/* devices */
 	MCFG_I8155_ADD(I8155_TAG, XTAL_4_9152MHz/2, kc85_8155_intf)
 	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, kc85_upd1990a_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_IM6402_ADD(IM6402_TAG, uart_intf)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, kc85_cassette_interface)
 
 	/* option ROM cartridge */
@@ -1335,7 +1351,8 @@ static MACHINE_CONFIG_START( pc8201, pc8201_state )
 	/* devices */
 	MCFG_I8155_ADD(I8155_TAG, XTAL_4_9152MHz/2, kc85_8155_intf)
 	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, kc85_upd1990a_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_IM6402_ADD(IM6402_TAG, uart_intf)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, kc85_cassette_interface)
 
 	/* option ROM cartridge */
@@ -1377,7 +1394,8 @@ static MACHINE_CONFIG_START( trsm100, trsm100_state )
 	/* devices */
 	MCFG_I8155_ADD(I8155_TAG, XTAL_4_9152MHz/2, kc85_8155_intf)
 	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, kc85_upd1990a_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_IM6402_ADD(IM6402_TAG, uart_intf)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, kc85_cassette_interface)
 //  MCFG_MC14412_ADD(MC14412_TAG, XTAL_1MHz)
 
@@ -1426,7 +1444,7 @@ static MACHINE_CONFIG_START( tandy200, tandy200_state )
 	MCFG_RP5C01_ADD(RP5C01A_TAG, XTAL_32_768kHz, tandy200_rtc_intf)
 	MCFG_I8251_ADD(I8251_TAG, /*XTAL_4_9152MHz/2,*/ tandy200_uart_intf)
 //  MCFG_MC14412_ADD(MC14412_TAG, XTAL_1MHz)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, kc85_cassette_interface)
 
 	/* option ROM cartridge */
