@@ -24,11 +24,11 @@
     ADDRESS MAP
 ***************************************************************************/
 
-static ADDRESS_MAP_START(lisa_map, AS_PROGRAM, 16)
+static ADDRESS_MAP_START(lisa_map, AS_PROGRAM, 16, lisa_state )
 	AM_RANGE(0x000000, 0xffffff) AM_READWRITE(lisa_r, lisa_w)			/* no fixed map, we use an MMU */
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( lisa_cop_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( lisa_cop_io_map, AS_IO, 8, lisa_state )
 	AM_RANGE(COP400_PORT_L, COP400_PORT_L)
 	AM_RANGE(COP400_PORT_G, COP400_PORT_G)
 	AM_RANGE(COP400_PORT_D, COP400_PORT_D)
@@ -38,7 +38,7 @@ static ADDRESS_MAP_START( lisa_cop_io_map, AS_IO, 8 )
 	AM_RANGE(COP400_PORT_CKO, COP400_PORT_CKO) AM_READNOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( kb_cop_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( kb_cop_io_map, AS_IO, 8, lisa_state )
 	AM_RANGE(COP400_PORT_L, COP400_PORT_L)
 	AM_RANGE(COP400_PORT_G, COP400_PORT_G)
 	AM_RANGE(COP400_PORT_D, COP400_PORT_D)
@@ -48,22 +48,22 @@ static ADDRESS_MAP_START( kb_cop_io_map, AS_IO, 8 )
 	AM_RANGE(COP400_PORT_CKO, COP400_PORT_CKO) AM_READNOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(lisa_fdc_map, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(lisa_fdc_map, AS_PROGRAM, 8, lisa_state )
 	ADDRESS_MAP_GLOBAL_MASK(0x1fff)	// only 8k of address space
-	AM_RANGE(0x0000, 0x03ff) AM_RAM	AM_BASE_MEMBER(lisa_state, m_fdc_ram)				/* RAM (shared with 68000) */
+	AM_RANGE(0x0000, 0x03ff) AM_RAM	AM_SHARE("fdc_ram")				/* RAM (shared with 68000) */
 	AM_RANGE(0x0400, 0x07ff) AM_READWRITE(lisa_fdc_io_r, lisa_fdc_io_w)	/* disk controller (IWM and TTL logic) */
 	AM_RANGE(0x0800, 0x0fff) AM_NOP
-	AM_RANGE(0x1000, 0x1fff) AM_ROM	AM_BASE_MEMBER(lisa_state, m_fdc_rom)				/* ROM */
+	AM_RANGE(0x1000, 0x1fff) AM_ROM	AM_REGION("fdccpu", 0x1000)	AM_SHARE("fdc_rom")		/* ROM */
 	AM_RANGE(0x2000, 0xffff) AM_READWRITE(lisa_fdc_r, lisa_fdc_w)		/* handler for wrap-around */
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(lisa210_fdc_map, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(lisa210_fdc_map, AS_PROGRAM, 8, lisa_state )
 	ADDRESS_MAP_GLOBAL_MASK(0x1fff)	// only 8k of address space
-	AM_RANGE(0x0000, 0x03ff) AM_RAM	AM_BASE_MEMBER(lisa_state, m_fdc_ram)				/* RAM (shared with 68000) */
+	AM_RANGE(0x0000, 0x03ff) AM_RAM	AM_SHARE("fdc_ram")				/* RAM (shared with 68000) */
 	AM_RANGE(0x0400, 0x07ff) AM_NOP										/* nothing, or RAM wrap-around ??? */
 	AM_RANGE(0x0800, 0x0bff) AM_READWRITE(lisa_fdc_io_r, lisa_fdc_io_w)	/* disk controller (IWM and TTL logic) */
 	AM_RANGE(0x0c00, 0x0fff) AM_NOP										/* nothing, or IO port wrap-around ??? */
-	AM_RANGE(0x1000, 0x1fff) AM_ROM	AM_BASE_MEMBER(lisa_state, m_fdc_rom)				/* ROM */
+	AM_RANGE(0x1000, 0x1fff) AM_ROM	AM_REGION("fdccpu", 0x1000)	AM_SHARE("fdc_rom")			/* ROM */
 	AM_RANGE(0x2000, 0xffff) AM_READWRITE(lisa_fdc_r, lisa_fdc_w)		/* handler for wrap-around */
 ADDRESS_MAP_END
 
@@ -168,10 +168,9 @@ static MACHINE_CONFIG_START( lisa, lisa_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(880, 380)
 	MCFG_SCREEN_VISIBLE_AREA(0, 720-1, 0, 364-1)
-	MCFG_SCREEN_UPDATE(lisa)
+	MCFG_SCREEN_UPDATE_STATIC(lisa)
 
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(black_and_white)
@@ -196,6 +195,8 @@ static MACHINE_CONFIG_START( lisa, lisa_state )
 	/* via */
 	MCFG_VIA6522_ADD("via6522_0", 500000, lisa_via6522_0_intf)
 	MCFG_VIA6522_ADD("via6522_1", 500000, lisa_via6522_1_intf)
+
+	MCFG_SCC8530_ADD("scc", 7833600, line_cb_t(FUNC(lisa_state::set_scc_interrupt), static_cast<lisa_state *>(owner)))
 MACHINE_CONFIG_END
 
 
@@ -406,7 +407,7 @@ ROM_START( lisa ) /* with twiggy drives, io40 i/o rom; technically any of the bo
 
 	ROM_REGION(0x2000,"fdccpu",0)		/* 6504 RAM and ROM */
 	/* note: other ?prototype? revisions of this rom for the lisa probably exist as well */
-    ROM_LOAD( "341-0138f.bin", 0x000000, 0x001000, CRC(edd8d560) SHA1(872211d21386cd9625b3735d7682e2b2ecff05b4) )
+    ROM_LOAD( "341-0138f.bin", 0x001000, 0x001000, CRC(edd8d560) SHA1(872211d21386cd9625b3735d7682e2b2ecff05b4) )
 
 	ROM_REGION(0x4000,"profile", 0)		/* Profile/5 HDD */
 	ROM_LOAD_OPTIONAL( "341-0080-b", 0x0000, 0x800, CRC(26df0b8d) SHA1(08f6689afb517e0a2bdaa48433003e62a66ae3c7)) /* 341-0080-B z8 mcu piggyback rom */
