@@ -293,13 +293,13 @@ Timings:
 
 /* I/O ports */
 
-static ADDRESS_MAP_START(io_map, AS_IO, 8)
+static ADDRESS_MAP_START(io_map, AS_IO, 8, lviv_state )
 	AM_RANGE(0x00, 0xff) AM_READWRITE(lviv_io_r,lviv_io_w)
 ADDRESS_MAP_END
 
 /* memory w/r functions */
 
-static ADDRESS_MAP_START(lviv_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START(lviv_mem , AS_PROGRAM, 8, lviv_state )
 	AM_RANGE(0x0000, 0x3fff) AM_RAMBANK("bank1")
 	AM_RANGE(0x4000, 0x7fff) AM_RAMBANK("bank2")
 	AM_RANGE(0x8000, 0xbfff) AM_RAMBANK("bank3")
@@ -444,10 +444,9 @@ static MACHINE_CONFIG_START( lviv, lviv_state )
 	MCFG_SCREEN_VBLANK_TIME(0)
 
     /* video hardware */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(256, 256)
 	MCFG_SCREEN_VISIBLE_AREA(0, 256-1, 0, 256-1)
-	MCFG_SCREEN_UPDATE( lviv )
+	MCFG_SCREEN_UPDATE_STATIC( lviv )
 
 	MCFG_PALETTE_LENGTH(sizeof (lviv_palette) / 3)
 	MCFG_PALETTE_INIT( lviv )
