@@ -19,7 +19,6 @@
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -71,10 +70,10 @@ READ8_MEMBER( m5_state::sts_r )
 	data |= (m_cassette)->input() >= 0 ? 1 : 0;
 
 	// centronics busy
-	data |= centronics_busy_r(m_centronics) << 1;
+	data |= m_centronics->busy_r() << 1;
 
 	// RESET key
-	data |= input_port_read(machine(), "RESET");
+	data |= ioport("RESET")->read();
 
 	return data;
 }
@@ -105,7 +104,7 @@ WRITE8_MEMBER( m5_state::com_w )
 	m_cassette->output( BIT(data, 0) ? -1.0 : 1.0);
 
 	// centronics strobe
-	centronics_strobe_w(m_centronics, BIT(data, 0));
+	m_centronics->strobe_w(BIT(data, 0));
 
 	// cassette remote
 	m_cassette->change_state(BIT(data,1) ? CASSETTE_MOTOR_ENABLED : CASSETTE_MOTOR_DISABLED, CASSETTE_MASK_MOTOR);
@@ -265,7 +264,7 @@ static ADDRESS_MAP_START( m5_io, AS_IO, 8, m5_state )
 	AM_RANGE(0x35, 0x35) AM_READ_PORT("Y5")
 	AM_RANGE(0x36, 0x36) AM_READ_PORT("Y6")
 	AM_RANGE(0x37, 0x37) AM_READ_PORT("JOY")
-	AM_RANGE(0x40, 0x40) AM_MIRROR(0x0f) AM_DEVWRITE_LEGACY(CENTRONICS_TAG, centronics_data_w)
+	AM_RANGE(0x40, 0x40) AM_MIRROR(0x0f) AM_DEVWRITE(CENTRONICS_TAG, centronics_device, write)
 	AM_RANGE(0x50, 0x50) AM_MIRROR(0x0f) AM_READWRITE(sts_r, com_w)
 //  AM_RANGE(0x60, 0x63) SIO
 //  AM_RANGE(0x6c, 0x6c) EM-64/64KBI bank select
@@ -452,14 +451,6 @@ static TMS9928A_INTERFACE(m5_tms9928a_interface)
 	DEVCB_LINE(sordm5_video_interrupt_callback)
 };
 
-static SCREEN_UPDATE( m5 )
-{
-	tms9928a_device *tms9928a = screen->machine().device<tms9928a_device>( "tms9928a" );
-
-	tms9928a->update( bitmap, cliprect );
-	return 0;
-}
-
 //-------------------------------------------------
 //  I8255_INTERFACE( ppi_intf )
 //-------------------------------------------------
@@ -619,7 +610,7 @@ void m5_state::machine_start()
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
 
 	// configure RAM
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 4*1024:
 		program->unmap_readwrite(0x8000, 0xffff);
@@ -633,11 +624,11 @@ void m5_state::machine_start()
 	}
 
 	// register for state saving
-	state_save_register_global(machine(), m_fd5_data);
-	state_save_register_global(machine(), m_fd5_com);
-	state_save_register_global(machine(), m_intra);
-	state_save_register_global(machine(), m_ibfa);
-	state_save_register_global(machine(), m_obfa);
+	save_item(NAME(m_fd5_data));
+	save_item(NAME(m_fd5_com));
+	save_item(NAME(m_intra));
+	save_item(NAME(m_ibfa));
+	save_item(NAME(m_obfa));
 }
 
 
@@ -677,7 +668,7 @@ static MACHINE_CONFIG_START( m5, m5_state )
 
 	// devices
 	MCFG_Z80CTC_ADD(Z80CTC_TAG, XTAL_14_31818MHz/4, ctc_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, cassette_intf)
 	MCFG_I8255_ADD(I8255A_TAG, ppi_intf)
 	MCFG_UPD765A_ADD(UPD765_TAG, fdc_intf)
@@ -707,7 +698,7 @@ static MACHINE_CONFIG_DERIVED( ntsc, m5 )
 	// video hardware
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9928A, m5_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( m5 )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9928a_device, screen_update )
 MACHINE_CONFIG_END
 
 
@@ -719,7 +710,7 @@ static MACHINE_CONFIG_DERIVED( pal, m5 )
 	// video hardware
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9929A, m5_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( m5 )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9928a_device, screen_update )
 MACHINE_CONFIG_END
 
 
