@@ -36,7 +36,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -70,7 +69,7 @@
 #define C15M	(C7M*2)
 #define C32M	(C15M*2)
 
-// do this here - SCREEN_UPDATE is called each scanline when stepping in the
+// do this here - SCREEN_UPDATE_IND16 is called each scanline when stepping in the
 // debugger, which means you can't escape the VIA2 IRQ handler
 //
 // RBV/MDU bits in IER/IFR:
@@ -116,7 +115,7 @@ WRITE32_MEMBER( mac_state::rbv_ramdac_w )
 			if (m_model != MODEL_MAC_CLASSIC_II)
 			{
 				// Color Classic has no MONTYPE so the safe read gets us 512x384, which is right
-				if (input_port_read_safe(space.machine(), "MONTYPE", 2) == 1)
+				if (space.machine().root_device().ioport("MONTYPE")->read_safe(2) == 1)
 				{
 					palette_set_color(space.machine(), m_rbv_clutoffs, MAKE_RGB(m_rbv_colors[2], m_rbv_colors[2], m_rbv_colors[2]));
 					m_rbv_palette[m_rbv_clutoffs] = MAKE_RGB(m_rbv_colors[2], m_rbv_colors[2], m_rbv_colors[2]);
@@ -152,7 +151,7 @@ WRITE32_MEMBER( mac_state::ariel_ramdac_w )	// this is for the "Ariel" style RAM
 			if (m_model != MODEL_MAC_CLASSIC_II)
 			{
 				// Color Classic has no MONTYPE so the safe read gets us 512x384, which is right
-				if (input_port_read_safe(space.machine(), "MONTYPE", 2) == 1)
+				if (space.machine().root_device().ioport("MONTYPE")->read_safe(2) == 1)
 				{
 					palette_set_color(space.machine(), m_rbv_clutoffs, MAKE_RGB(m_rbv_colors[2], m_rbv_colors[2], m_rbv_colors[2]));
 					m_rbv_palette[m_rbv_clutoffs] = MAKE_RGB(m_rbv_colors[2], m_rbv_colors[2], m_rbv_colors[2]);
@@ -183,6 +182,7 @@ READ8_MEMBER( mac_state::mac_sonora_vctl_r )
 {
 	if (offset == 2)
 	{
+//        printf("Sonora: read monitor ID at PC=%x\n", cpu_get_pc(m_maincpu));
 		return (6 << 4);	// 640x480 RGB monitor
 	}
 
@@ -219,12 +219,12 @@ void mac_state::rbv_recalc_irqs()
 		m_rbv_ifr = ifr | 0x80;
 
 //      printf("VIA2 raise\n");
-		this->set_via2_interrupt(1);
+		set_via2_interrupt(1);
 	}
 	else
 	{
 //      printf("VIA2 lower\n");
-		this->set_via2_interrupt(0);
+		set_via2_interrupt(0);
 	}
 }
 
@@ -239,8 +239,8 @@ READ8_MEMBER ( mac_state::mac_rbv_r )
 		if (offset == 0x10)
 		{
 			data &= ~0x38;
-			data |= (input_port_read_safe(space.machine(), "MONTYPE", 2)<<3);
-//          printf("rbv_r montype: %02x (PC %x)\n", data, cpu_get_pc(space.cpu));
+			data |= (space.machine().root_device().ioport("MONTYPE")->read_safe(2)<<3);
+//            printf("rbv_r montype: %02x (PC %x)\n", data, cpu_get_pc(space.cpu));
 		}
 
 		// bit 7 of these registers always reads as 0 on RBV
@@ -392,41 +392,6 @@ WRITE8_MEMBER ( mac_state::mac_rbv_w )
 	}
 }
 
-READ32_MEMBER(mac_state::mac_read_id)
-{
-//  printf("Mac read ID reg @ PC=%x\n", cpu_get_pc(m_maincpu));
-
-	switch (m_model)
-	{
-		case MODEL_MAC_LC_III:
-			return 0xa55a0001;	// 25 MHz LC III
-
-		case MODEL_MAC_LC_III_PLUS:
-			return 0xa55a0003;	// 33 MHz LC III+
-
-		case MODEL_MAC_POWERMAC_6100:
-			return 0xa55a3011;
-
-		case MODEL_MAC_POWERMAC_7100:
-			return 0xa55a3012;
-
-		case MODEL_MAC_POWERMAC_8100:
-			return 0xa55a3013;
-
-		case MODEL_MAC_PBDUO_210:
-			return 0xa55a1004;
-
-		case MODEL_MAC_PBDUO_230:
-			return 0xa55a1005;
-
-		case MODEL_MAC_PBDUO_250:
-			return 0xa55a1006;
-
-		default:
-			return 0;
-	}
-}
-
 // Portable/PB100 video
 static VIDEO_START( macprtb )
 {
@@ -605,7 +570,7 @@ static ADDRESS_MAP_START(macprtb_map, AS_PROGRAM, 16, mac_state )
 	AM_RANGE(0xf60000, 0xf6ffff) AM_READWRITE(mac_iwm_r, mac_iwm_w)
 	AM_RANGE(0xf70000, 0xf7ffff) AM_READWRITE(mac_via_r, mac_via_w)
 	AM_RANGE(0xf90000, 0xf9ffff) AM_READWRITE(macplus_scsi_r, macplus_scsi_w)
-	AM_RANGE(0xfa8000, 0xfaffff) AM_RAM	AM_BASE(m_vram)	// VRAM
+	AM_RANGE(0xfa8000, 0xfaffff) AM_RAM	AM_SHARE("vram16")	// VRAM
 	AM_RANGE(0xfb0000, 0xfbffff) AM_DEVREADWRITE8("asc", asc_device, read, write, 0xffff)
 	AM_RANGE(0xfc0000, 0xfcffff) AM_READ(mac_config_r)
 	AM_RANGE(0xfd0000, 0xfdffff) AM_READWRITE(mac_scc_r, mac_scc_2_w)
@@ -626,7 +591,7 @@ static ADDRESS_MAP_START(maclc_map, AS_PROGRAM, 32, mac_state )
 	AM_RANGE(0xf16000, 0xf17fff) AM_READWRITE16(mac_iwm_r, mac_iwm_w, 0xffffffff)
 	AM_RANGE(0xf24000, 0xf24003) AM_READWRITE(rbv_ramdac_r, ariel_ramdac_w)
 	AM_RANGE(0xf26000, 0xf27fff) AM_READWRITE8(mac_rbv_r, mac_rbv_w, 0xffffffff)	// VIA2 (V8)
-	AM_RANGE(0xf40000, 0xfbffff) AM_RAM AM_BASE(m_vram)
+	AM_RANGE(0xf40000, 0xfbffff) AM_RAM AM_SHARE("vram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START(maclc3_map, AS_PROGRAM, 32, mac_state )
@@ -645,7 +610,7 @@ static ADDRESS_MAP_START(maclc3_map, AS_PROGRAM, 32, mac_state )
 
 	AM_RANGE(0x5ffffffc, 0x5fffffff) AM_READ(mac_read_id)
 
-	AM_RANGE(0x60000000, 0x600fffff) AM_RAM AM_MIRROR(0x0ff00000) AM_BASE(m_vram)
+	AM_RANGE(0x60000000, 0x600fffff) AM_RAM AM_MIRROR(0x0ff00000) AM_SHARE("vram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START(macii_map, AS_PROGRAM, 32, mac_state )
@@ -692,7 +657,7 @@ static ADDRESS_MAP_START(macse30_map, AS_PROGRAM, 32, mac_state )
 	AM_RANGE(0x50016000, 0x50017fff) AM_READWRITE16(mac_iwm_r, mac_iwm_w, 0xffffffff) AM_MIRROR(0x00f00000)
 	AM_RANGE(0x50040000, 0x50041fff) AM_READWRITE16(mac_via_r, mac_via_w, 0xffffffff) AM_MIRROR(0x00f00000)	// mirror
 
-	AM_RANGE(0xfe000000, 0xfe00ffff) AM_RAM	AM_BASE(m_vram)
+	AM_RANGE(0xfe000000, 0xfe00ffff) AM_RAM	AM_SHARE("vram")
 	AM_RANGE(0xfeffe000, 0xfeffffff) AM_ROM AM_REGION("se30vrom", 0x0)
 ADDRESS_MAP_END
 
@@ -727,7 +692,7 @@ static ADDRESS_MAP_START(macpb140_map, AS_PROGRAM, 32, mac_state )
 	AM_RANGE(0x50016000, 0x50017fff) AM_READWRITE16(mac_iwm_r, mac_iwm_w, 0xffffffff) AM_MIRROR(0x01f00000)
 	AM_RANGE(0x50024000, 0x50027fff) AM_READ(buserror_r) AM_MIRROR(0x01f00000)   // bus error here to make sure we aren't mistaken for another decoder
 
-	AM_RANGE(0xfee08000, 0xfeffffff) AM_RAM	AM_BASE(m_vram)
+	AM_RANGE(0xfee08000, 0xfeffffff) AM_RAM	AM_SHARE("vram")
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START(macpb160_map, AS_PROGRAM, 32, mac_state )
@@ -744,9 +709,9 @@ static ADDRESS_MAP_START(macpb160_map, AS_PROGRAM, 32, mac_state )
 	AM_RANGE(0x50f20000, 0x50f21fff) AM_READWRITE8(mac_gsc_r, mac_gsc_w, 0xffffffff)
 	AM_RANGE(0x50f24000, 0x50f27fff) AM_READ(buserror_r)   // bus error here to make sure we aren't mistaken for another decoder
 
-	AM_RANGE(0x60000000, 0x6001ffff) AM_RAM	AM_BASE(m_vram) AM_MIRROR(0x0ffe0000)
+	AM_RANGE(0x60000000, 0x6001ffff) AM_RAM	AM_SHARE("vram") AM_MIRROR(0x0ffe0000)
 ADDRESS_MAP_END
-/*
+
 static ADDRESS_MAP_START(macpb165c_map, AS_PROGRAM, 32, mac_state )
     AM_RANGE(0x40000000, 0x400fffff) AM_ROM AM_REGION("bootrom", 0) AM_MIRROR(0x0ff00000)
 
@@ -761,11 +726,33 @@ static ADDRESS_MAP_START(macpb165c_map, AS_PROGRAM, 32, mac_state )
     AM_RANGE(0x50f20000, 0x50f21fff) AM_READ(buserror_r)    // bus error here to detect we're not the grayscale 160/165/180
     AM_RANGE(0x50f24000, 0x50f27fff) AM_READ(buserror_r)   // bus error here to make sure we aren't mistaken for another decoder
 
-    AM_RANGE(0xfc040000, 0xfc07ffff) AM_RAM AM_BASE(m_vram)
-    AM_RANGE(0xfcff0000, 0xfcffffff) AM_ROM AM_REGION("bootrom", 0xf0000)
+    // on-board color video on 165c/180c
+    AM_RANGE(0xfc000000, 0xfc07ffff) AM_RAM AM_SHARE("vram") AM_MIRROR(0x00380000) // 512k of VRAM
+    AM_RANGE(0xfc400000, 0xfcefffff) AM_READWRITE(macwd_r, macwd_w)
+// fc4003c8 = DAC control, fc4003c9 = DAC data
+// fc4003da bit 3 is VBL
+    AM_RANGE(0xfcff8000, 0xfcffffff) AM_ROM AM_REGION("vrom", 0x0000)
 ADDRESS_MAP_END
-*/
+#if 0
+static ADDRESS_MAP_START(macpd210_map, AS_PROGRAM, 32, mac_state )
+	AM_RANGE(0x40000000, 0x400fffff) AM_ROM AM_REGION("bootrom", 0) AM_MIRROR(0x0ff00000)
 
+	AM_RANGE(0x50f00000, 0x50f01fff) AM_READWRITE16(mac_via_r, mac_via_w, 0xffffffff)
+	AM_RANGE(0x50f02000, 0x50f03fff) AM_READWRITE16(mac_via2_r, mac_via2_w, 0xffffffff)
+	AM_RANGE(0x50f04000, 0x50f05fff) AM_READWRITE16(mac_scc_r, mac_scc_2_w, 0xffffffff)
+	AM_RANGE(0x50f06000, 0x50f07fff) AM_READWRITE(macii_scsi_drq_r, macii_scsi_drq_w)
+	AM_RANGE(0x50f10000, 0x50f11fff) AM_READWRITE16(macplus_scsi_r, macii_scsi_w, 0xffffffff)
+	AM_RANGE(0x50f12060, 0x50f12063) AM_READ(macii_scsi_drq_r)
+	AM_RANGE(0x50f14000, 0x50f15fff) AM_DEVREADWRITE8("asc", asc_device, read, write, 0xffffffff)
+	AM_RANGE(0x50f16000, 0x50f17fff) AM_READWRITE16(mac_iwm_r, mac_iwm_w, 0xffffffff)
+	AM_RANGE(0x50f20000, 0x50f21fff) AM_READWRITE8(mac_gsc_r, mac_gsc_w, 0xffffffff)
+	AM_RANGE(0x50f24000, 0x50f27fff) AM_READ(buserror_r)   // bus error here to make sure we aren't mistaken for another decoder
+
+	AM_RANGE(0x5ffffffc, 0x5fffffff) AM_READ(mac_read_id)
+
+	AM_RANGE(0x60000000, 0x6001ffff) AM_RAM	AM_SHARE("vram") AM_MIRROR(0x0ffe0000)
+ADDRESS_MAP_END
+#endif
 static ADDRESS_MAP_START(quadra700_map, AS_PROGRAM, 32, mac_state )
 	AM_RANGE(0x40000000, 0x400fffff) AM_ROM AM_REGION("bootrom", 0) AM_MIRROR(0x0ff00000)
 
@@ -781,7 +768,7 @@ static ADDRESS_MAP_START(quadra700_map, AS_PROGRAM, 32, mac_state )
 
 	AM_RANGE(0x50040000, 0x50041fff) AM_READWRITE16(mac_via_r, mac_via_w, 0xffffffff) AM_MIRROR(0x00fc0000)
 	// f9800000 = VDAC / DAFB
-	AM_RANGE(0xf9000000, 0xf91fffff) AM_RAM	AM_BASE(m_vram)
+	AM_RANGE(0xf9000000, 0xf91fffff) AM_RAM	AM_SHARE("vram")
 	AM_RANGE(0xf9800000, 0xf98001ff) AM_READWRITE(dafb_r, dafb_w)
 	AM_RANGE(0xf9800200, 0xf980023f) AM_READWRITE(dafb_dac_r, dafb_dac_w)
 ADDRESS_MAP_END
@@ -887,10 +874,23 @@ static const floppy_interface mac_floppy_interface =
 	DEVCB_NULL,
 	FLOPPY_STANDARD_3_5_DSHD,
 	LEGACY_FLOPPY_OPTIONS_NAME(apple35_mac),
-	NULL,
+	"floppy_3_5",
 	NULL
 };
 
+static const struct harddisk_interface mac_harddisk_config =
+{
+	NULL,
+	NULL,
+	"mac_hdd",
+	NULL
+};
+
+static const cuda_interface mac_cuda_interface =
+{
+    DEVCB_DRIVER_LINE_MEMBER(mac_state, cuda_reset_w)
+};
+
 static MACHINE_CONFIG_START( mac512ke, mac_state )
 
 	/* basic machine hardware */
@@ -903,10 +903,9 @@ static MACHINE_CONFIG_START( mac512ke, mac_state )
 
     /* video hardware */
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_UPDATE_BEFORE_VBLANK)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(MAC_H_TOTAL, MAC_V_TOTAL)
 	MCFG_SCREEN_VISIBLE_AREA(0, MAC_H_VIS-1, 0, MAC_V_VIS-1)
-	MCFG_SCREEN_UPDATE(mac)
+	MCFG_SCREEN_UPDATE_STATIC(mac)
 
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(mac)
@@ -925,10 +924,11 @@ static MACHINE_CONFIG_START( mac512ke, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 	MCFG_VIA6522_ADD("via6522_0", 1000000, mac_via6522_intf)
 
+	MCFG_MACKBD_ADD()
+
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("512K")
@@ -941,14 +941,15 @@ static MACHINE_CONFIG_DERIVED( mac128k, mac512ke )
 	MCFG_RAM_DEFAULT_SIZE("128K")
 MACHINE_CONFIG_END
 
+
 static MACHINE_CONFIG_DERIVED( macplus, mac512ke )
 	MCFG_CPU_MODIFY( "maincpu" )
 	MCFG_CPU_PROGRAM_MAP(macplus_map)
 
 	MCFG_NCR5380_ADD("ncr5380", C7M, macplus_5380intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_MODIFY(mac_floppy_interface)
 
@@ -956,6 +957,10 @@ static MACHINE_CONFIG_DERIVED( macplus, mac512ke )
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("1M,2M,2560K,4M")
+
+	// software list
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( macse, macplus )
@@ -981,10 +986,9 @@ static MACHINE_CONFIG_START( macprtb, mac_state )
 
     /* video hardware */
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_UPDATE_BEFORE_VBLANK)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(700, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 639, 0, 399)
-	MCFG_SCREEN_UPDATE(macprtb)
+	MCFG_SCREEN_UPDATE_STATIC(macprtb)
 
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(mac)
@@ -1006,17 +1010,19 @@ static MACHINE_CONFIG_START( macprtb, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 	MCFG_VIA6522_ADD("via6522_0", 783360, mac_via6522_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("1M")
 	MCFG_RAM_EXTRA_OPTIONS("1M,3M,5M,7M,9M")
+
+	// software list
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_START( macii, mac_state )
@@ -1038,7 +1044,7 @@ static MACHINE_CONFIG_START( macii, mac_state )
 
 	/* devices */
 	MCFG_NUBUS_BUS_ADD("nubus", "maincpu", nubus_intf)
-	MCFG_NUBUS_SLOT_ADD("nubus","nb9", mac_nubus_cards, "cb264", NULL)
+	MCFG_NUBUS_SLOT_ADD("nubus","nb9", mac_nubus_cards, "48gc", NULL)
 	MCFG_NUBUS_SLOT_ADD("nubus","nba", mac_nubus_cards, NULL, NULL)
 	MCFG_NUBUS_SLOT_ADD("nubus","nbb", mac_nubus_cards, NULL, NULL)
 	MCFG_NUBUS_SLOT_ADD("nubus","nbc", mac_nubus_cards, NULL, NULL)
@@ -1050,19 +1056,22 @@ static MACHINE_CONFIG_START( macii, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", C7M/10, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", C7M/10, mac_via6522_2_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("2M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,32M,64M,96M,128M")
+
+	// software list
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maciihmu, macii )
@@ -1099,18 +1108,21 @@ static MACHINE_CONFIG_START( maciifx, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", C7M/10, mac_via6522_adb_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,16M,32M,64M,96M,128M")
+
+    // software list
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maclc, macii )
@@ -1125,11 +1137,10 @@ static MACHINE_CONFIG_DERIVED( maclc, macii )
 	MCFG_VIDEO_RESET(macrbv)
 
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_RAW_PARAMS(25175000, 800, 0, 640, 525, 0, 480)
 	MCFG_SCREEN_SIZE(1024,768)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
-	MCFG_SCREEN_UPDATE(macrbvvram)
+	MCFG_SCREEN_UPDATE_STATIC(macrbvvram)
 
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("2M")
@@ -1147,7 +1158,7 @@ static MACHINE_CONFIG_DERIVED( maclc, macii )
 	MCFG_SOUND_ROUTE(0, "lspeaker", 1.0)
 	MCFG_SOUND_ROUTE(1, "rspeaker", 1.0)
 
-	MCFG_EGRET_ADD()
+	MCFG_EGRET_ADD(EGRET_341S0850)
 	MCFG_QUANTUM_PERFECT_CPU("maincpu")
 MACHINE_CONFIG_END
 
@@ -1162,6 +1173,12 @@ static MACHINE_CONFIG_DERIVED( maclc2, maclc )
 	MCFG_RAM_EXTRA_OPTIONS("6M,8M,10M")
 MACHINE_CONFIG_END
 
+static MACHINE_CONFIG_DERIVED( maccclas, maclc2 )
+
+    MCFG_EGRET_REMOVE()
+    MCFG_CUDA_ADD(CUDA_341S0788, mac_cuda_interface)    // should be 341s0417, but only the color classic used that rev and those are "collectable" ($$$$)
+MACHINE_CONFIG_END
+
 static MACHINE_CONFIG_DERIVED( maclc3, maclc )
 
 	MCFG_CPU_REPLACE("maincpu", M68030, 25000000)
@@ -1172,7 +1189,7 @@ static MACHINE_CONFIG_DERIVED( maclc3, maclc )
 	MCFG_VIDEO_RESET(macsonora)
 
 	MCFG_SCREEN_MODIFY(MAC_SCREEN_NAME)
-	MCFG_SCREEN_UPDATE(macrbvvram)
+	MCFG_SCREEN_UPDATE_STATIC(macrbvvram)
 
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
@@ -1181,6 +1198,14 @@ static MACHINE_CONFIG_DERIVED( maclc3, maclc )
 	MCFG_ASC_REPLACE("asc", C15M, ASC_TYPE_SONORA, mac_asc_irq)
 	MCFG_SOUND_ROUTE(0, "lspeaker", 1.0)
 	MCFG_SOUND_ROUTE(1, "rspeaker", 1.0)
+
+	MCFG_EGRET_REPLACE(EGRET_341S0851)
+MACHINE_CONFIG_END
+
+static MACHINE_CONFIG_DERIVED( maclc520, maclc3 )
+
+    MCFG_EGRET_REMOVE()
+    MCFG_CUDA_ADD(CUDA_341S0060, mac_cuda_interface)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maciivx, maclc )
@@ -1193,7 +1218,7 @@ static MACHINE_CONFIG_DERIVED( maciivx, maclc )
 	MCFG_VIDEO_RESET(macrbv)
 
 	MCFG_SCREEN_MODIFY(MAC_SCREEN_NAME)
-	MCFG_SCREEN_UPDATE(macrbvvram)
+	MCFG_SCREEN_UPDATE_STATIC(macrbvvram)
 
 	MCFG_NUBUS_BUS_ADD("nubus", "maincpu", nubus_intf)
 	MCFG_NUBUS_SLOT_ADD("nubus","nbc", mac_nubus_cards, NULL, NULL)
@@ -1203,6 +1228,8 @@ static MACHINE_CONFIG_DERIVED( maciivx, maclc )
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M,20M,24M,28M,32M,36M,40M,44M,48M,52M,56M,60M,64M")
+
+	MCFG_EGRET_REPLACE(EGRET_341S0851)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maciivi, maclc )
@@ -1215,7 +1242,7 @@ static MACHINE_CONFIG_DERIVED( maciivi, maclc )
 	MCFG_VIDEO_RESET(macrbv)
 
 	MCFG_SCREEN_MODIFY(MAC_SCREEN_NAME)
-	MCFG_SCREEN_UPDATE(macrbvvram)
+	MCFG_SCREEN_UPDATE_STATIC(macrbvvram)
 
 	MCFG_NUBUS_BUS_ADD("nubus", "maincpu", nubus_intf)
 	MCFG_NUBUS_SLOT_ADD("nubus","nbc", mac_nubus_cards, NULL, NULL)
@@ -1225,6 +1252,8 @@ static MACHINE_CONFIG_DERIVED( maciivi, maclc )
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M,20M,24M,28M,32M,36M,40M,44M,48M,52M,56M,60M,64M")
+
+	MCFG_EGRET_REPLACE(EGRET_341S0851)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maciix, macii )
@@ -1253,10 +1282,9 @@ static MACHINE_CONFIG_START( macse30, mac_state )
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60.15)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(1260))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(MAC_H_TOTAL, MAC_V_TOTAL)
 	MCFG_SCREEN_VISIBLE_AREA(0, MAC_H_VIS-1, 0, MAC_V_VIS-1)
-	MCFG_SCREEN_UPDATE(macse30)
+	MCFG_SCREEN_UPDATE_STATIC(macse30)
 
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(mac)
@@ -1278,19 +1306,21 @@ static MACHINE_CONFIG_START( macse30, mac_state )
 	MCFG_SWIM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", 783360, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", 783360, mac_via6522_2_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("2M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,16M,32M,48M,64M,96M,128M")
+
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_START( macpb140, mac_state )
@@ -1303,10 +1333,9 @@ static MACHINE_CONFIG_START( macpb140, mac_state )
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60.15)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(1260))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(700, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 639, 0, 399)
-	MCFG_SCREEN_UPDATE(macpb140)
+	MCFG_SCREEN_UPDATE_STATIC(macpb140)
 
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(mac)
@@ -1328,19 +1357,21 @@ static MACHINE_CONFIG_START( macpb140, mac_state )
 	MCFG_SWIM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", 783360, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", 783360, mac_via6522_2_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("2M")
 	MCFG_RAM_EXTRA_OPTIONS("4M,6M,8M")
+
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 // PowerBook 145 = 140 @ 25 MHz (still 2MB RAM - the 145B upped that to 4MB)
@@ -1373,10 +1404,9 @@ static MACHINE_CONFIG_START( macpb160, mac_state )
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60.15)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(1260))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(700, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 639, 0, 399)
-	MCFG_SCREEN_UPDATE(macpb160)
+	MCFG_SCREEN_UPDATE_STATIC(macpb160)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_PALETTE_INIT(macgsc)
@@ -1398,19 +1428,21 @@ static MACHINE_CONFIG_START( macpb160, mac_state )
 	MCFG_SWIM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", 783360, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", 783360, mac_via6522_2_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M")
+
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( macpb180, macpb160 )
@@ -1421,16 +1453,30 @@ static MACHINE_CONFIG_DERIVED( macpb180, macpb160 )
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M")
 MACHINE_CONFIG_END
-/*
+
 static MACHINE_CONFIG_DERIVED( macpb180c, macpb160 )
     MCFG_CPU_REPLACE("maincpu", M68030, 33000000)
     MCFG_CPU_PROGRAM_MAP(macpb165c_map)
 
+	MCFG_SCREEN_MODIFY(MAC_SCREEN_NAME)
+	MCFG_SCREEN_SIZE(800, 525)
+	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
+	MCFG_SCREEN_UPDATE_STATIC(macpbwd)
+
     MCFG_RAM_MODIFY(RAM_TAG)
     MCFG_RAM_DEFAULT_SIZE("4M")
     MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M")
 MACHINE_CONFIG_END
-*/
+#if 0
+static MACHINE_CONFIG_DERIVED( macpd210, macpb160 )
+	MCFG_CPU_REPLACE("maincpu", M68030, 25000000)
+	MCFG_CPU_PROGRAM_MAP(macpd210_map)
+
+	MCFG_RAM_MODIFY(RAM_TAG)
+	MCFG_RAM_DEFAULT_SIZE("4M")
+	MCFG_RAM_EXTRA_OPTIONS("8M,12M,16M,20M,24M")
+MACHINE_CONFIG_END
+#endif
 static MACHINE_CONFIG_DERIVED( macclas2, maclc )
 
 	MCFG_CPU_REPLACE("maincpu", M68030, C15M)
@@ -1442,7 +1488,7 @@ static MACHINE_CONFIG_DERIVED( macclas2, maclc )
 	MCFG_SCREEN_MODIFY(MAC_SCREEN_NAME)
 	MCFG_SCREEN_SIZE(MAC_H_TOTAL, MAC_V_TOTAL)
 	MCFG_SCREEN_VISIBLE_AREA(0, MAC_H_VIS-1, 0, MAC_V_VIS-1)
-	MCFG_SCREEN_UPDATE(macrbv)
+	MCFG_SCREEN_UPDATE_STATIC(macrbv)
 
 	MCFG_ASC_REPLACE("asc", C15M, ASC_TYPE_EAGLE, mac_asc_irq)
 	MCFG_SOUND_ROUTE(0, "lspeaker", 1.0)
@@ -1451,6 +1497,8 @@ static MACHINE_CONFIG_DERIVED( macclas2, maclc )
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("10M")
 	MCFG_RAM_EXTRA_OPTIONS("2M,4M,6M,8M,10M")
+
+	MCFG_EGRET_REPLACE(EGRET_341S0851)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( maciici, macii )
@@ -1470,11 +1518,10 @@ static MACHINE_CONFIG_DERIVED( maciici, macii )
 	MCFG_VIDEO_RESET(macrbv)
 
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_RAW_PARAMS(25175000, 800, 0, 640, 525, 0, 480)
 	MCFG_SCREEN_SIZE(640, 870)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
-	MCFG_SCREEN_UPDATE(macrbv)
+	MCFG_SCREEN_UPDATE_STATIC(macrbv)
 	MCFG_DEFAULT_LAYOUT(layout_mac)
 
 	/* internal ram */
@@ -1503,17 +1550,17 @@ static MACHINE_CONFIG_DERIVED( maciisi, macii )
 	MCFG_VIDEO_RESET(macrbv)
 
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_RAW_PARAMS(25175000, 800, 0, 640, 525, 0, 480)
 	MCFG_SCREEN_SIZE(640, 870)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
-	MCFG_SCREEN_UPDATE(macrbv)
+	MCFG_SCREEN_UPDATE_STATIC(macrbv)
 
 	/* internal ram */
 	MCFG_RAM_MODIFY(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("2M")
 	MCFG_RAM_EXTRA_OPTIONS("4M,8M,16M,32M,48M,64M,128M")
-    MCFG_EGRET_ADD()
+
+    MCFG_EGRET_ADD(EGRET_344S0100)
 	MCFG_QUANTUM_PERFECT_CPU("maincpu")
 MACHINE_CONFIG_END
 
@@ -1528,10 +1575,9 @@ static MACHINE_CONFIG_START( pwrmac, mac_state )
 	// dot clock, htotal, hstart, hend, vtotal, vstart, vend
 	MCFG_SCREEN_RAW_PARAMS(25175000, 800, 0, 640, 525, 0, 480)
 	MCFG_VIDEO_ATTRIBUTES(VIDEO_UPDATE_BEFORE_VBLANK)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(1024, 768)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
-	MCFG_SCREEN_UPDATE(macrbv)
+	MCFG_SCREEN_UPDATE_STATIC(macrbv)
 
 	MCFG_PALETTE_LENGTH(256)
 
@@ -1553,19 +1599,20 @@ static MACHINE_CONFIG_START( pwrmac, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", 783360, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", 783360, mac_via6522_2_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("8M")
 	MCFG_RAM_EXTRA_OPTIONS("16M,32M,64M,128M")
+
+    MCFG_CUDA_ADD(CUDA_341S0060, mac_cuda_interface)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_START( macqd700, mac_state )
@@ -1574,12 +1621,11 @@ static MACHINE_CONFIG_START( macqd700, mac_state )
 	MCFG_CPU_PROGRAM_MAP(quadra700_map)
 
 	MCFG_SCREEN_ADD(MAC_SCREEN_NAME, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_REFRESH_RATE(75.08)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(1260))
 	MCFG_SCREEN_SIZE(1152, 870)
 	MCFG_SCREEN_VISIBLE_AREA(0, 1152-1, 0, 870-1)
-	MCFG_SCREEN_UPDATE(macdafb)
+	MCFG_SCREEN_UPDATE_STATIC(macdafb)
 
 	MCFG_VIDEO_START(macdafb)
 	MCFG_VIDEO_RESET(macdafb)
@@ -1603,21 +1649,24 @@ static MACHINE_CONFIG_START( macqd700, mac_state )
 	MCFG_IWM_ADD("fdc", mac_iwm_interface)
 	MCFG_LEGACY_FLOPPY_SONY_2_DRIVES_ADD(mac_floppy_interface)
 
-	MCFG_SCC8530_ADD("scc", C7M)
-	MCFG_SCC8530_IRQ(mac_scc_irq)
+	MCFG_SCC8530_ADD("scc", C7M, line_cb_t(FUNC(mac_state::set_scc_interrupt), static_cast<mac_state *>(owner)))
 
 	MCFG_VIA6522_ADD("via6522_0", C7M/10, mac_via6522_adb_intf)
 	MCFG_VIA6522_ADD("via6522_1", C7M/10, mac_via6522_2_intf)
 
 	MCFG_NCR539X_ADD(MAC_539X_1_TAG, C7M, mac_539x_intf)
 
-	MCFG_HARDDISK_ADD( "harddisk1" )
-	MCFG_HARDDISK_ADD( "harddisk2" )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk1", mac_harddisk_config )
+	MCFG_HARDDISK_CONFIG_ADD( "harddisk2", mac_harddisk_config )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("4M")
 	MCFG_RAM_EXTRA_OPTIONS("8M,16M,32M,64M,68M,72M,80M,96M,128M")
+
+	// software list
+	MCFG_SOFTWARE_LIST_ADD("flop35_list","mac_flop")
+	MCFG_SOFTWARE_LIST_ADD("hdd_list", "mac_hdd")
 MACHINE_CONFIG_END
 
 static INPUT_PORTS_START( macplus )
@@ -2064,11 +2113,29 @@ ROM_START( macpb180 )
 	ROM_LOAD( "e33b2724.rom", 0x000000, 0x100000, CRC(536c60f4) SHA1(c0510682ae6d973652d7e17f3c3b27629c47afac) )
 ROM_END
 
+ROM_START( macpb180c )
+	ROM_REGION32_BE(0x100000, "bootrom", 0)
+	ROM_LOAD( "e33b2724.rom", 0x000000, 0x100000, CRC(536c60f4) SHA1(c0510682ae6d973652d7e17f3c3b27629c47afac) )
+
+    ROM_REGION32_BE(0x8000, "vrom", 0)
+    ROM_LOAD( "pb180cvrom.bin", 0x0000, 0x8000, CRC(810c75ad) SHA1(3a936e97dee5ceeb25e50197ef504e514ae689a4))
+ROM_END
+
 ROM_START( maccclas )
 	ROM_REGION32_BE(0x100000, "bootrom", 0)
 	ROM_LOAD( "ecd99dc0.rom", 0x000000, 0x100000, CRC(c84c3aa5) SHA1(fd9e852e2d77fe17287ba678709b9334d4d74f1e) )
 ROM_END
 
+/*ROM_START( macpd210 )
+    ROM_REGION32_BE(0x100000, "bootrom", 0)
+    ROM_LOAD( "ecfa989b.rom", 0x000000, 0x100000, CRC(b86ed854) SHA1(ed1371c97117a5884da4a6605ecfc5abed48ae5a) )
+ROM_END*/
+
+ROM_START( maclc520 )
+	ROM_REGION32_BE(0x100000, "bootrom", 0)
+    ROM_LOAD( "ede66cbd.rom", 0x000000, 0x100000, CRC(a893cb0f) SHA1(c54ee2f45020a4adeb7451adce04cd6e5fb69790) )
+ROM_END
+
 /*    YEAR  NAME      PARENT    COMPAT  MACHINE   INPUT     INIT     COMPANY          FULLNAME */
 COMP( 1984, mac128k,  0,		0,	mac128k,  macplus,  mac128k512k,  "Apple Computer", "Macintosh 128k",  GAME_NOT_WORKING )
 COMP( 1984, mac512k,  mac128k,  0,	mac512ke, macplus,  mac128k512k,  "Apple Computer", "Macintosh 512k",  GAME_NOT_WORKING )
@@ -2097,10 +2164,12 @@ COMP( 1991, maclc2,   0,		0,	maclc2,   maciici,  maclc2,	      "Apple Computer",
 COMP( 1992, macpb145, macpb140, 0,  macpb145, macadb,   macpb140,	  "Apple Computer", "Macintosh PowerBook 145", GAME_NOT_WORKING )
 COMP( 1992, macpb160, 0,        0,  macpb160, macadb,   macpb160,	  "Apple Computer", "Macintosh PowerBook 160", GAME_NOT_WORKING )
 COMP( 1992, macpb180, macpb160, 0,  macpb180, macadb,   macpb160,	  "Apple Computer", "Macintosh PowerBook 180", GAME_NOT_WORKING )
-//COMP( 1992, macp180c, macpb160, 0,  macpb180c,macadb,   macpb160,       "Apple Computer", "Macintosh PowerBook 180c", GAME_NOT_WORKING )
-COMP( 1993, maccclas, 0,        0,  maclc2,   macadb,   maclrcclassic,"Apple Computer", "Macintosh Color Classic", GAME_NOT_WORKING )
+COMP( 1992, macpb180c,macpb160, 0,  macpb180c,macadb,   macpb160,     "Apple Computer", "Macintosh PowerBook 180c", GAME_NOT_WORKING )
+//COMP( 1992, macpd210, 0,        0,  macpd210, macadb,   macpd210,     "Apple Computer", "Macintosh PowerBook Duo 210", GAME_NOT_WORKING )
+COMP( 1993, maccclas, 0,        0,  maccclas, macadb,   maclrcclassic,"Apple Computer", "Macintosh Color Classic", GAME_NOT_WORKING )
 COMP( 1992, macpb145b,macpb140, 0,  macpb170, macadb,   macpb140,	  "Apple Computer", "Macintosh PowerBook 145B", GAME_NOT_WORKING )
 COMP( 1993, maclc3,   0,		0,	maclc3,   maciici,  maclc3,	      "Apple Computer", "Macintosh LC III",  GAME_NOT_WORKING )
 COMP( 1993, maciivx,  0,		0,	maciivx,  maciici,  maciivx,	  "Apple Computer", "Macintosh IIvx",  GAME_NOT_WORKING )
 COMP( 1993, maciivi,  maciivx,	0,	maciivi,  maciici,  maciivx,	  "Apple Computer", "Macintosh IIvi",  GAME_NOT_WORKING )
+COMP( 1993, maclc520, 0,		0,	maclc520, maciici,  maclc520,     "Apple Computer", "Macintosh LC 520",  GAME_NOT_WORKING )
 COMP( 1994, pmac6100, 0,		0,	pwrmac,   macadb,   macpm6100,	  "Apple Computer", "Power Macintosh 6100/60",  GAME_NOT_WORKING | GAME_NO_SOUND )
