@@ -88,10 +88,10 @@
 
 
 ***************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "includes/mbee.h"
+#include "formats/z80bin.h"
 
 #define XTAL_13_5MHz 13500000
 
@@ -680,10 +680,9 @@ static MACHINE_CONFIG_START( mbee, mbee_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(250)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(64*8, 19*16)			/* need at least 17 lines for NET */
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 64*8-1, 0, 19*16-1)
-	MCFG_SCREEN_UPDATE(mbee)
+	MCFG_SCREEN_UPDATE_STATIC(mbee)
 
 	MCFG_GFXDECODE(mbee)
 	MCFG_PALETTE_LENGTH(2)
@@ -701,8 +700,8 @@ static MACHINE_CONFIG_START( mbee, mbee_state )
 	/* devices */
 	MCFG_MC6845_ADD("crtc", SY6545_1, XTAL_12MHz / 8, mbee_crtc)
 	MCFG_QUICKLOAD_ADD("quickload", mbee, "mwb,com", 2)
-	MCFG_Z80BIN_QUICKLOAD_ADD("quickload2", mbee, 2)
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_QUICKLOAD_ADD("quickload2", mbee_z80bin, "bin", 2)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, default_cassette_interface )
 MACHINE_CONFIG_END
 
@@ -722,10 +721,9 @@ static MACHINE_CONFIG_START( mbeeic, mbee_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(250)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(80*8, 310)
 	MCFG_SCREEN_VISIBLE_AREA(0, 80*8-1, 0, 19*16-1)
-	MCFG_SCREEN_UPDATE(mbee)
+	MCFG_SCREEN_UPDATE_STATIC(mbee)
 
 	MCFG_GFXDECODE(mbeeic)
 	MCFG_PALETTE_LENGTH(96)
@@ -743,8 +741,8 @@ static MACHINE_CONFIG_START( mbeeic, mbee_state )
 	/* devices */
 	MCFG_MC6845_ADD("crtc", SY6545_1, XTAL_13_5MHz / 8, mbeeic_crtc)
 	MCFG_QUICKLOAD_ADD("quickload", mbee, "mwb,com", 2)
-	MCFG_Z80BIN_QUICKLOAD_ADD("quickload2", mbee, 2)
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_QUICKLOAD_ADD("quickload2", mbee_z80bin, "bin", 2)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, default_cassette_interface )
 MACHINE_CONFIG_END
 
@@ -831,6 +829,7 @@ MACHINE_CONFIG_END
     Telcom 1.1 for the mbeeic (It could have 1.0, 1.1 or 1.2)
 
     ROM_LOAD("bn54.bin",              0x0000,  0x2000, CRC(995c53db) SHA1(46e1a5cfd5795b8cf528bacf9dc79398ff7d64af) )
+    ROM_LOAD("bn55.bin",              0x0000,  0x2000, CRC(ca2c1073) SHA1(355d90d181de899cc7af892df96305fead9c81b4) )
     ROM_LOAD("bn56.bin",              0x0000,  0x2000, CRC(3f76769d) SHA1(cfae2069d739c26fe39f734d9f705a3c965d1e6f) )
     These are alternate boot roms for the 128k. They have no menu, and look just like the 64k/56k bootup.
 
@@ -1077,7 +1076,7 @@ ROM_START( mbee56 )
 	ROM_REGION( 0x0800, "colorram", ROMREGION_ERASE00 )
 ROM_END
 
-ROM_START( mbee64 )
+ROM_START( mbee64 ) // CIAB (Computer-In-A-Book)
 	ROM_REGION(0x10000,"maincpu", ROMREGION_ERASEFF)
 
 	ROM_REGION(0x7000,"bootrom", ROMREGION_ERASEFF)
@@ -1095,7 +1094,7 @@ ROM_START( mbee64 )
 	ROM_REGION( 0x0800, "colorram", ROMREGION_ERASE00 )
 ROM_END
 
-ROM_START( mbee128 )
+ROM_START( mbee128 ) // 128K
 	ROM_REGION(0x20000,"maincpu", ROMREGION_ERASEFF)
 
 	ROM_REGION(0x7000,"bootrom", ROMREGION_ERASEFF)
@@ -1110,11 +1109,12 @@ ROM_START( mbee128 )
 	ROM_REGION( 0x0800, "attrib", ROMREGION_ERASE00 )
 ROM_END
 
-ROM_START( mbee256 )
+ROM_START( mbee256 ) // 256tc
 	ROM_REGION(0x40000,"maincpu", ROMREGION_ERASEFF)
 
 	ROM_REGION(0x7000,"bootrom", ROMREGION_ERASEFF)
-	ROM_LOAD("mbee256.bin",           0x0000,  0x4000, CRC(fe8d6a84) SHA1(a037a1b90b18a2180e9f5f216b829fcd480449a4) )
+	ROM_LOAD("256tc_boot_1.20.rom", 0x0000,  0x4000, CRC(fe8d6a84) SHA1(a037a1b90b18a2180e9f5f216b829fcd480449a4) )
+	//ROM_LOAD("256tc_boot_1.15.rom", 0x0000,  0x4000, CRC(1902062d) SHA1(e4a1c0b3f4996e313da0bac0edb6d34e3270723e) )
 
 	ROM_REGION(0x9800, "gfx", 0)
 	ROM_LOAD("char256.bin",           0x1000,  0x1000, CRC(9372af3c) SHA1(a63591822c0504de2fed52e88d64e1dbd6124b74) )
