@@ -8,12 +8,11 @@
 
 ***************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m6800/m6800.h"
 #include "sound/dac.h"
-#include "video/m6847.h"
+#include "video/mc6847.h"
 #include "video/ef9345.h"
 #include "imagedev/cassette.h"
 #include "imagedev/printer.h"
@@ -49,10 +48,10 @@ public:
 	{ }
 
 	required_device<cpu_device> m_maincpu;
-	optional_device<device_t> m_mc6847;
+	optional_device<mc6847_base_device> m_mc6847;
 	optional_device<ef9345_device> m_ef9345;
 	required_device<device_t> m_dac;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 	required_device<cassette_image_device> m_cassette;
 	required_device<printer_image_device> m_printer;
 
@@ -66,7 +65,7 @@ public:
 	UINT8 m_pr_counter;
 	UINT8 m_pr_state;
 
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
 	DECLARE_READ8_MEMBER( mc10_bfff_r );
 	DECLARE_WRITE8_MEMBER( mc10_bfff_w );
@@ -88,14 +87,14 @@ READ8_MEMBER( mc10_state::mc10_bfff_r )
 {
 	UINT8 result = 0xff;
 
-	if (!BIT(m_keyboard_strobe, 0)) result &= input_port_read(machine(), "pb0");
-	if (!BIT(m_keyboard_strobe, 1)) result &= input_port_read(machine(), "pb1");
-	if (!BIT(m_keyboard_strobe, 2)) result &= input_port_read(machine(), "pb2");
-	if (!BIT(m_keyboard_strobe, 3)) result &= input_port_read(machine(), "pb3");
-	if (!BIT(m_keyboard_strobe, 4)) result &= input_port_read(machine(), "pb4");
-	if (!BIT(m_keyboard_strobe, 5)) result &= input_port_read(machine(), "pb5");
-	if (!BIT(m_keyboard_strobe, 6)) result &= input_port_read(machine(), "pb6");
-	if (!BIT(m_keyboard_strobe, 7)) result &= input_port_read(machine(), "pb7");
+	if (!BIT(m_keyboard_strobe, 0)) result &= ioport("pb0")->read();
+	if (!BIT(m_keyboard_strobe, 1)) result &= ioport("pb1")->read();
+	if (!BIT(m_keyboard_strobe, 2)) result &= ioport("pb2")->read();
+	if (!BIT(m_keyboard_strobe, 3)) result &= ioport("pb3")->read();
+	if (!BIT(m_keyboard_strobe, 4)) result &= ioport("pb4")->read();
+	if (!BIT(m_keyboard_strobe, 5)) result &= ioport("pb5")->read();
+	if (!BIT(m_keyboard_strobe, 6)) result &= ioport("pb6")->read();
+	if (!BIT(m_keyboard_strobe, 7)) result &= ioport("pb7")->read();
 
 	return result;
 }
@@ -104,21 +103,21 @@ READ8_MEMBER( mc10_state::alice90_bfff_r )
 {
 	UINT8 result = 0xff;
 
-	if (!BIT(m_keyboard_strobe, 7)) result &= input_port_read(machine(), "pb7");
+	if (!BIT(m_keyboard_strobe, 7)) result &= ioport("pb7")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 6)) result &= input_port_read(machine(), "pb6");
+	if (!BIT(m_keyboard_strobe, 6)) result &= ioport("pb6")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 5)) result &= input_port_read(machine(), "pb5");
+	if (!BIT(m_keyboard_strobe, 5)) result &= ioport("pb5")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 4)) result &= input_port_read(machine(), "pb4");
+	if (!BIT(m_keyboard_strobe, 4)) result &= ioport("pb4")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 3)) result &= input_port_read(machine(), "pb3");
+	if (!BIT(m_keyboard_strobe, 3)) result &= ioport("pb3")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 2)) result &= input_port_read(machine(), "pb2");
+	if (!BIT(m_keyboard_strobe, 2)) result &= ioport("pb2")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 1)) result &= input_port_read(machine(), "pb1");
+	if (!BIT(m_keyboard_strobe, 1)) result &= ioport("pb1")->read();
 	else
-	if (!BIT(m_keyboard_strobe, 0)) result &= input_port_read(machine(), "pb0");
+	if (!BIT(m_keyboard_strobe, 0)) result &= ioport("pb0")->read();
 
 	return result;
 }
@@ -126,12 +125,12 @@ READ8_MEMBER( mc10_state::alice90_bfff_r )
 WRITE8_MEMBER( mc10_state::mc10_bfff_w )
 {
 	/* bit 2 to 6, mc6847 mode lines */
-	mc6847_gm2_w(m_mc6847, BIT(data, 2));
-	mc6847_intext_w(m_mc6847, BIT(data, 2));
-	mc6847_gm1_w(m_mc6847, BIT(data, 3));
-	mc6847_gm0_w(m_mc6847, BIT(data, 4));
-	mc6847_ag_w(m_mc6847, BIT(data, 5));
-	mc6847_css_w(m_mc6847, BIT(data, 6));
+	m_mc6847->gm2_w(BIT(data, 2));
+	m_mc6847->intext_w(BIT(data, 2));
+	m_mc6847->gm1_w(BIT(data, 3));
+	m_mc6847->gm0_w(BIT(data, 4));
+	m_mc6847->ag_w(BIT(data, 5));
+	m_mc6847->css_w(BIT(data, 6));
 
 	/* bit 7, dac output */
 	dac_data_w(m_dac, BIT(data, 7));
@@ -165,9 +164,9 @@ READ8_MEMBER( mc10_state::mc10_port2_r )
 	UINT8 result = 0xeb;
 
 	/* bit 1, keyboard line pa6 */
-	if (!BIT(m_keyboard_strobe, 0)) result &= input_port_read(machine(), "pb0") >> 5;
-	if (!BIT(m_keyboard_strobe, 2)) result &= input_port_read(machine(), "pb2") >> 5;
-	if (!BIT(m_keyboard_strobe, 7)) result &= input_port_read(machine(), "pb7") >> 5;
+	if (!BIT(m_keyboard_strobe, 0)) result &= ioport("pb0")->read() >> 5;
+	if (!BIT(m_keyboard_strobe, 2)) result &= ioport("pb2")->read() >> 5;
+	if (!BIT(m_keyboard_strobe, 7)) result &= ioport("pb7")->read() >> 5;
 
 	/* bit 2, printer ots input */
 	result |= (m_printer->is_ready() ? 0 : 4);
@@ -218,18 +217,19 @@ WRITE8_MEMBER( mc10_state::mc10_port2_w )
 
 READ8_MEMBER( mc10_state::mc10_mc6847_videoram_r )
 {
-	mc6847_inv_w(m_mc6847, BIT(m_ram_base[offset], 6));
-	mc6847_as_w(m_mc6847, BIT(m_ram_base[offset], 7));
+	if (offset == ~0) return 0xff;
+	m_mc6847->inv_w(BIT(m_ram_base[offset], 6));
+	m_mc6847->as_w(BIT(m_ram_base[offset], 7));
 
 	return m_ram_base[offset];
 }
 
-bool mc10_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 mc10_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	if (m_mc6847)		//mc10, alice
-		return mc6847_update(m_mc6847, &bitmap, &cliprect);
+		return m_mc6847->screen_update(screen, bitmap, cliprect);
 	else if (m_ef9345)	//alice32
-		m_ef9345->video_update(&bitmap, &cliprect);
+		return m_ef9345->screen_update(screen, bitmap, cliprect);
 
 	return 0;
 }
@@ -247,29 +247,29 @@ static TIMER_DEVICE_CALLBACK( alice32_scanline )
 
 static DRIVER_INIT( mc10 )
 {
-	mc10_state *mc10 = machine.driver_data<mc10_state>();
+	mc10_state *state = machine.driver_data<mc10_state>();
 	address_space *prg = machine.device("maincpu")->memory().space(AS_PROGRAM);
 
 	/* initialize keyboard strobe */
-	mc10->m_keyboard_strobe = 0x00;
+	state->m_keyboard_strobe = 0x00;
 
 	/* initialize memory */
-	mc10->m_ram_base = ram_get_ptr(mc10->m_ram);
-	mc10->m_ram_size = ram_get_size(mc10->m_ram);
-	mc10->m_pr_state = PRINTER_WAIT;
+	state->m_ram_base = state->m_ram->pointer();
+	state->m_ram_size = state->m_ram->size();
+	state->m_pr_state = PRINTER_WAIT;
 
-	memory_set_bankptr(machine, "bank1", mc10->m_ram_base);
+	state->membank("bank1")->set_base(state->m_ram_base);
 
 	/* initialize memory expansion */
-	if (mc10->m_ram_size == 20*1024)
-		memory_set_bankptr(machine, "bank2", mc10->m_ram_base + 0x1000);
-	else if (mc10->m_ram_size == 24*1024)
-		memory_set_bankptr(machine, "bank2", mc10->m_ram_base + 0x2000);
-	else  if (mc10->m_ram_size != 32*1024)		//ensure that is not alice90
+	if (state->m_ram_size == 20*1024)
+		state->membank("bank2")->set_base(state->m_ram_base + 0x1000);
+	else if (state->m_ram_size == 24*1024)
+		state->membank("bank2")->set_base(state->m_ram_base + 0x2000);
+	else  if (state->m_ram_size != 32*1024)		//ensure that is not alice90
 		prg->nop_readwrite(0x5000, 0x8fff);
 
 	/* register for state saving */
-	state_save_register_global(machine, mc10->m_keyboard_strobe);
+	state_save_register_global(machine, state->m_keyboard_strobe);
 
 	//for alice32 force port4 DDR to 0xff at startup
 	if (!strcmp(machine.system().name, "alice32") || !strcmp(machine.system().name, "alice90"))
@@ -405,8 +405,6 @@ static INPUT_PORTS_START( mc10 )
 	PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("/  ?  SQR")          PORT_CODE(KEYCODE_SLASH)      PORT_CHAR('/')  PORT_CHAR('?')
 	PORT_BIT(0x40, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("SHIFT")              PORT_CODE(KEYCODE_RSHIFT)     PORT_CHAR(UCHAR_SHIFT_1)
 	PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_UNUSED)
-
-	PORT_INCLUDE( m6847_artifacting )
 INPUT_PORTS_END
 
 /* Alice uses an AZERTY keyboard */
@@ -486,8 +484,6 @@ static INPUT_PORTS_START( alice )
 	PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME(";  +  POKE")         PORT_CODE(KEYCODE_COLON)      PORT_CHAR(';')  PORT_CHAR('+')
 	PORT_BIT(0x40, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("SHIFT")              PORT_CODE(KEYCODE_RSHIFT)     PORT_CHAR(UCHAR_SHIFT_1)
 	PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_UNUSED)
-
-	PORT_INCLUDE( m6847_artifacting )
 INPUT_PORTS_END
 
 
@@ -515,18 +511,8 @@ static const cassette_interface alice32_cassette_interface =
 
 static const mc6847_interface mc10_mc6847_intf =
 {
-	DEVCB_DRIVER_MEMBER(mc10_state, mc10_mc6847_videoram_r),
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL
+	"screen",
+	DEVCB_DRIVER_MEMBER(mc10_state, mc10_mc6847_videoram_r)
 };
 
 static MACHINE_CONFIG_START( mc10, mc10_state )
@@ -535,16 +521,10 @@ static MACHINE_CONFIG_START( mc10, mc10_state )
 	MCFG_CPU_ADD("maincpu", M6803, XTAL_3_579545MHz)  /* 0,894886 MHz */
 	MCFG_CPU_PROGRAM_MAP(mc10_mem)
 	MCFG_CPU_IO_MAP(mc10_io)
-	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_REFRESH_RATE(60)
 
 	/* video hardware */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
-	MCFG_SCREEN_SIZE(320, 25+192+26)
-	MCFG_SCREEN_VISIBLE_AREA(0, 319, 1, 239)
-
-	MCFG_MC6847_ADD("mc6847", mc10_mc6847_intf)
-	MCFG_MC6847_TYPE(M6847_VERSION_ORIGINAL_NTSC)
+	MCFG_SCREEN_MC6847_NTSC_ADD("screen", "mc6847")
+	MCFG_MC6847_ADD("mc6847", MC6847_NTSC, XTAL_3_579545MHz, mc10_mc6847_intf)
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -577,7 +557,7 @@ static MACHINE_CONFIG_START( alice32, mc10_state )
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(mc10_state, screen_update)
 	MCFG_SCREEN_SIZE(336, 270)
 	MCFG_SCREEN_VISIBLE_AREA(00, 336-1, 00, 270-1)
 	MCFG_PALETTE_LENGTH(8)
