@@ -24,7 +24,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -87,7 +86,7 @@ public:
 
 	virtual void machine_start();
 	virtual void machine_reset();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 };
 
 READ8_MEMBER( mstation_state::flash_0x0000_read_handler )
@@ -149,7 +148,7 @@ WRITE8_MEMBER( mstation_state::lcd_left_w )  {	lcd_w(offset, m_screen_column, da
 READ8_MEMBER ( mstation_state::lcd_right_r ) {	return lcd_r(offset, m_screen_column + 20);	}
 WRITE8_MEMBER( mstation_state::lcd_right_w ) {	lcd_w(offset, m_screen_column + 20, data);	}
 
-bool mstation_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 mstation_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	for (int x=0; x<40; x++)
 		for (int y=0; y<128; y++)
@@ -161,7 +160,7 @@ bool mstation_state::screen_update(screen_device &screen, bitmap_t &bitmap, cons
 				// columns are inverted (right to left)
 				int col = ((x < 20) ? 19 : 59) - x;
 
-				*BITMAP_ADDR16(&bitmap, y, col*8 + b)= BIT(data, 0);
+				bitmap.pix16(y, col*8 + b)= BIT(data, 0);
 				data >>= 1;
 			}
 		}
@@ -194,7 +193,7 @@ void mstation_state::refresh_memory(UINT8 bank, UINT8 chip_select)
 			break;
 		case 1: // banked RAM
 			sprintf(bank_tag,"bank%d", bank);
-			memory_set_bankptr(machine(), bank_tag, m_ram_base + (((bank == 1 ? m_bank1[0] : m_bank2[0]) & 0x07)<<14));
+			membank(bank_tag)->set_base(m_ram_base + (((bank == 1 ? m_bank1[0] : m_bank2[0]) & 0x07)<<14));
 			program->install_readwrite_bank (bank * 0x4000, bank * 0x4000 + 0x3fff, bank_tag);
 			active_flash = -1;
 			break;
@@ -303,7 +302,7 @@ READ8_MEMBER( mstation_state::kb_r )
 	for (int i=0; i<10; i++)
 	{
 		if (!(m_kb_matrix & (1<<i)))
-			data &= input_port_read(machine(), bitnames[i]);
+			data &= ioport(bitnames[i])->read();
 	}
 
 	return data;
@@ -438,11 +437,11 @@ void mstation_state::machine_start()
 	m_flashes[1] = machine().device<intelfsh8_device>("flash1");
 
 	// allocate the videoram
-	m_vram = (UINT8*)machine().region_alloc( "vram", 9600, 1, ENDIANNESS_LITTLE )->base();
-	m_ram_base = (UINT8*)ram_get_ptr(machine().device(RAM_TAG));
+	m_vram = (UINT8*)machine().memory().region_alloc( "vram", 9600, 1, ENDIANNESS_LITTLE )->base();
+	m_ram_base = (UINT8*)machine().device<ram_device>(RAM_TAG)->pointer();
 
 	// map firsh RAM bank at 0xc000-0xffff
-	memory_set_bankptr(machine(), "sysram", m_ram_base);
+	membank("sysram")->set_base(m_ram_base);
 }
 
 
@@ -508,7 +507,7 @@ static MACHINE_CONFIG_START( mstation, mstation_state )
     MCFG_SCREEN_ADD("screen", RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(mstation_state, screen_update)
     MCFG_SCREEN_SIZE(320, 128)
     MCFG_SCREEN_VISIBLE_AREA(0, 320-1, 0, 128-1)
 
