@@ -42,7 +42,7 @@
     ADDRESS_MAP( mtx_mem )
 -------------------------------------------------*/
 
-static ADDRESS_MAP_START( mtx_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( mtx_mem, AS_PROGRAM, 8, mtx_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROMBANK("bank1")
 	AM_RANGE(0x2000, 0x3fff) AM_ROMBANK("bank2")
 	AM_RANGE(0x4000, 0x7fff) AM_RAMBANK("bank3")
@@ -54,36 +54,37 @@ ADDRESS_MAP_END
     ADDRESS_MAP( mtx_io )
 -------------------------------------------------*/
 
-static ADDRESS_MAP_START( mtx_io, AS_IO, 8 )
+static ADDRESS_MAP_START( mtx_io, AS_IO, 8, mtx_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0x00, 0x00) AM_DEVREAD(CENTRONICS_TAG, mtx_strobe_r) AM_WRITE(mtx_bankswitch_w)
-	AM_RANGE(0x01, 0x01) AM_DEVREADWRITE_MODERN("tms9929a", tms9929a_device, vram_read, vram_write)
-	AM_RANGE(0x02, 0x02) AM_DEVREADWRITE_MODERN("tms9929a", tms9929a_device, register_read, register_write)
-	AM_RANGE(0x03, 0x03) AM_DEVREAD(SN76489A_TAG, mtx_sound_strobe_r) AM_DEVWRITE(CASSETTE_TAG, mtx_cst_w)
-	AM_RANGE(0x04, 0x04) AM_DEVREADWRITE(CENTRONICS_TAG, mtx_prt_r, centronics_data_w)
+	AM_RANGE(0x00, 0x00) AM_DEVREAD_LEGACY(CENTRONICS_TAG, mtx_strobe_r) AM_WRITE(mtx_bankswitch_w)
+	AM_RANGE(0x01, 0x01) AM_DEVREADWRITE("tms9929a", tms9929a_device, vram_read, vram_write)
+	AM_RANGE(0x02, 0x02) AM_DEVREADWRITE("tms9929a", tms9929a_device, register_read, register_write)
+	AM_RANGE(0x03, 0x03) AM_DEVREAD_LEGACY(SN76489A_TAG, mtx_sound_strobe_r) AM_DEVWRITE_LEGACY(CASSETTE_TAG, mtx_cst_w)
+	AM_RANGE(0x04, 0x04) AM_DEVREAD_LEGACY(CENTRONICS_TAG, mtx_prt_r)
+	AM_RANGE(0x04, 0x04) AM_DEVWRITE(CENTRONICS_TAG, centronics_device, write)
 	AM_RANGE(0x05, 0x05) AM_READWRITE(mtx_key_lo_r, mtx_sense_w)
 	AM_RANGE(0x06, 0x06) AM_READWRITE(mtx_key_hi_r, mtx_sound_latch_w)
 //  AM_RANGE(0x07, 0x07) PIO
-	AM_RANGE(0x08, 0x0b) AM_DEVREADWRITE(Z80CTC_TAG, z80ctc_r, z80ctc_w)
+	AM_RANGE(0x08, 0x0b) AM_DEVREADWRITE_LEGACY(Z80CTC_TAG, z80ctc_r, z80ctc_w)
 	AM_RANGE(0x30, 0x31) AM_WRITE(hrx_address_w)
 	AM_RANGE(0x32, 0x32) AM_READWRITE(hrx_data_r, hrx_data_w)
 	AM_RANGE(0x33, 0x33) AM_READWRITE(hrx_attr_r, hrx_attr_w)
-//  AM_RANGE(0x38, 0x38) AM_DEVWRITE(MC6845_TAG, mc6845_address_w)
-//  AM_RANGE(0x39, 0x39) AM_DEVWRITE(MC6845_TAG, mc6845_register_r, mc6845_register_w)
-/*  AM_RANGE(0x40, 0x43) AM_DEVREADWRITE(FD1791_TAG, wd17xx_r, wd17xx_w)
-    AM_RANGE(0x44, 0x44) AM_READWRITE(fdx_status_r, fdx_control_w)
-    AM_RANGE(0x45, 0x45) AM_WRITE(fdx_drv_sel_w)
-    AM_RANGE(0x46, 0x46) AM_WRITE(fdx_dma_lo_w)
-    AM_RANGE(0x47, 0x47) AM_WRITE(fdx_dma_hi_w)*/
+//  AM_RANGE(0x38, 0x38) AM_DEVWRITE_LEGACY(MC6845_TAG, mc6845_address_w)
+//  AM_RANGE(0x39, 0x39) AM_DEVWRITE_LEGACY(MC6845_TAG, mc6845_register_r, mc6845_register_w)
+/*  AM_RANGE(0x40, 0x43) AM_DEVREADWRITE_LEGACY(FD1791_TAG, wd17xx_r, wd17xx_w)
+    AM_RANGE(0x44, 0x44) AM_READWRITE_LEGACY(fdx_status_r, fdx_control_w)
+    AM_RANGE(0x45, 0x45) AM_WRITE_LEGACY(fdx_drv_sel_w)
+    AM_RANGE(0x46, 0x46) AM_WRITE_LEGACY(fdx_dma_lo_w)
+    AM_RANGE(0x47, 0x47) AM_WRITE_LEGACY(fdx_dma_hi_w)*/
 ADDRESS_MAP_END
 
 /*-------------------------------------------------
     ADDRESS_MAP( rs128_io )
 -------------------------------------------------*/
 
-static ADDRESS_MAP_START( rs128_io, AS_IO, 8 )
+static ADDRESS_MAP_START( rs128_io, AS_IO, 8, mtx_state )
 	AM_IMPORT_FROM(mtx_io)
-	AM_RANGE(0x0c, 0x0f) AM_DEVREADWRITE(Z80DART_TAG, z80dart_cd_ba_r, z80dart_cd_ba_w)
+	AM_RANGE(0x0c, 0x0f) AM_DEVREADWRITE_LEGACY(Z80DART_TAG, z80dart_cd_ba_r, z80dart_cd_ba_w)
 ADDRESS_MAP_END
 
 /***************************************************************************
@@ -337,14 +338,6 @@ static TMS9928A_INTERFACE(mtx_tms9928a_interface)
     DEVCB_LINE(mtx_tms9929a_interrupt)
 };
 
-static SCREEN_UPDATE( mtx )
-{
-	tms9929a_device *tms9929a = screen->machine().device<tms9929a_device>( "tms9929a" );
-
-	tms9929a->update( bitmap, cliprect );
-	return 0;
-}
-
 /***************************************************************************
     MACHINE DRIVERS
 ***************************************************************************/
@@ -367,7 +360,7 @@ static MACHINE_CONFIG_START( mtx512, mtx_state )
 	/* video hardware */
 	MCFG_TMS9928A_ADD( "tms9929a", TMS9929A, mtx_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( mtx )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9929a", tms9929a_device, screen_update )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -377,7 +370,7 @@ static MACHINE_CONFIG_START( mtx512, mtx_state )
 	/* devices */
 	MCFG_Z80CTC_ADD(Z80CTC_TAG, XTAL_4MHz, ctc_intf )
 	MCFG_TIMER_ADD_PERIODIC("z80ctc_timer", ctc_tick, attotime::from_hz(XTAL_4MHz/13))
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_SNAPSHOT_ADD("snapshot", mtx, "mtb", 0.5)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, mtx_cassette_interface)
 	MCFG_TIMER_ADD_PERIODIC("cassette_timer", cassette_tick, attotime::from_hz(44100))
