@@ -24,7 +24,6 @@ MZ80B
 - D000-FFFF is banked ram
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "includes/mz80.h"
 
@@ -240,8 +239,8 @@ INPUT_PORTS_END
 static ADDRESS_MAP_START( mz80k_mem, AS_PROGRAM, 8, mz80_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE(0x0000, 0x0fff) AM_ROM
-	AM_RANGE(0x1000, 0xcfff) AM_RAM AM_BASE(m_p_ram) // 48 KB of RAM
-	AM_RANGE(0xd000, 0xd7ff) AM_RAM AM_BASE(m_p_videoram) // Video RAM
+	AM_RANGE(0x1000, 0xcfff) AM_RAM AM_SHARE("p_ram") // 48 KB of RAM
+	AM_RANGE(0xd000, 0xd7ff) AM_RAM AM_SHARE("p_videoram") // Video RAM
 	AM_RANGE(0xe000, 0xe003) AM_DEVREADWRITE("ppi8255", i8255_device, read, write) /* PPIA 8255 */
 	AM_RANGE(0xe004, 0xe007) AM_DEVREADWRITE_LEGACY("pit8253", pit8253_r,pit8253_w)  /* PIT 8253  */
 	AM_RANGE(0xe008, 0xe00b) AM_READWRITE( mz80k_strobe_r, mz80k_strobe_w)
@@ -289,11 +288,10 @@ static MACHINE_CONFIG_START( mz80k, mz80_state )
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(320, 200)
 	MCFG_SCREEN_VISIBLE_AREA(0, 319, 0, 199)
 	MCFG_VIDEO_START(mz80k)
-	MCFG_SCREEN_UPDATE(mz80k)
+	MCFG_SCREEN_UPDATE_STATIC(mz80k)
 	MCFG_GFXDECODE(mz80k)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(black_and_white)
@@ -314,13 +312,13 @@ MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( mz80kj, mz80k )
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(mz80kj)
+	MCFG_SCREEN_UPDATE_STATIC(mz80kj)
 	MCFG_GFXDECODE(mz80kj)
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( mz80a, mz80k )
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE(mz80a)
+	MCFG_SCREEN_UPDATE_STATIC(mz80a)
 MACHINE_CONFIG_END
 
 
