@@ -5,7 +5,6 @@
         12/05/2009 Skeleton driver.
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -21,7 +20,7 @@
 #define MACHINE_RESET_MEMBER(name) void name::machine_reset()
 #define MACHINE_START_MEMBER(name) void name::machine_start()
 #define VIDEO_START_MEMBER(name) void name::video_start()
-#define SCREEN_UPDATE_MEMBER(name) bool name::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+#define SCREEN_UPDATE16_MEMBER(name) UINT32 name::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 
 class nanos_state : public driver_device
 {
@@ -61,7 +60,7 @@ public:
 	virtual void machine_reset();
 	virtual void machine_start();
 	virtual void video_start();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 };
 
 
@@ -273,10 +272,10 @@ INPUT_PORTS_END
 
 VIDEO_START_MEMBER( nanos_state )
 {
-	m_p_chargen = machine().region("chargen")->base();
+	m_p_chargen = memregion("chargen")->base();
 }
 
-SCREEN_UPDATE_MEMBER( nanos_state )
+SCREEN_UPDATE16_MEMBER( nanos_state )
 {
 //  static UINT8 framecnt=0;
 	UINT8 y,ra,chr,gfx;
@@ -288,13 +287,13 @@ SCREEN_UPDATE_MEMBER( nanos_state )
 	{
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(&bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 
 			for (x = ma; x < ma + 80; x++)
 			{
 				if (ra < 8)
 				{
-					chr = ram_get_ptr(machine().device(RAM_TAG))[0xf800+ x];
+					chr = machine().device<ram_device>(RAM_TAG)->pointer()[0xf800+ x];
 
 					/* get pattern of pixels for that character scanline */
 					gfx = m_p_chargen[(chr<<3) | ra ];
@@ -342,9 +341,9 @@ static WRITE8_DEVICE_HANDLER (nanos_port_b_w)
 	nanos_state *state = device->machine().driver_data<nanos_state>();
 	state->m_key_command = BIT(data,1);
 	if (BIT(data,7)) {
-		memory_set_bankptr(device->machine(), "bank1", device->machine().region("maincpu")->base());
+		state->membank("bank1")->set_base(state->memregion("maincpu")->base());
 	} else {
-		memory_set_bankptr(device->machine(), "bank1", ram_get_ptr(device->machine().device(RAM_TAG)));
+		state->membank("bank1")->set_base(device->machine().device<ram_device>(RAM_TAG)->pointer());
 	}
 }
 
@@ -368,13 +367,13 @@ static TIMER_DEVICE_CALLBACK(keyboard_callback)
 	int i;
 	UINT8 code;
 	UINT8 key_code = 0;
-	UINT8 shift = input_port_read(timer.machine(), "LINEC") & 0x02 ? 1 : 0;
-	UINT8 ctrl =  input_port_read(timer.machine(), "LINEC") & 0x01 ? 1 : 0;
+	UINT8 shift = timer.machine().root_device().ioport("LINEC")->read() & 0x02 ? 1 : 0;
+	UINT8 ctrl =  timer.machine().root_device().ioport("LINEC")->read() & 0x01 ? 1 : 0;
 	state->m_key_pressed = 0xff;
 	for(i = 0; i < 7; i++)
 	{
 
-		code =	input_port_read(timer.machine(), keynames[i]);
+		code =	timer.machine().root_device().ioport(keynames[i])->read();
 		if (code != 0)
 		{
 			if (i==0 && shift==0) {
@@ -456,9 +455,9 @@ MACHINE_RESET_MEMBER(nanos_state)
 	space->install_write_bank(0x0000, 0x0fff, "bank3");
 	space->install_write_bank(0x1000, 0xffff, "bank2");
 
-	memory_set_bankptr(machine(), "bank1", machine().region("maincpu")->base());
-	memory_set_bankptr(machine(), "bank2", ram_get_ptr(machine().device(RAM_TAG)) + 0x1000);
-	memory_set_bankptr(machine(), "bank3", ram_get_ptr(machine().device(RAM_TAG)));
+	membank("bank1")->set_base(machine().root_device().memregion("maincpu")->base());
+	membank("bank2")->set_base(machine().device<ram_device>(RAM_TAG)->pointer() + 0x1000);
+	membank("bank3")->set_base(machine().device<ram_device>(RAM_TAG)->pointer());
 
 	floppy_mon_w(floppy_get_device(space->machine(), 0), CLEAR_LINE);
 	floppy_drive_set_ready_state(floppy_get_device(space->machine(), 0), 1,1);
@@ -536,7 +535,7 @@ static MACHINE_CONFIG_START( nanos, nanos_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(nanos_state, screen_update)
 	MCFG_SCREEN_SIZE(80*8, 25*10)
 	MCFG_SCREEN_VISIBLE_AREA(0,80*8-1,0,25*10-1)
 	MCFG_GFXDECODE(nanos)
