@@ -344,9 +344,9 @@ static void nc_refresh_memory_bank_config(running_machine &machine, int bank)
 
 			mem_bank = mem_bank & state->m_membank_rom_mask;
 
-			addr = (machine.region("maincpu")->base()+0x010000) + (mem_bank<<14);
+			addr = (state->memregion("maincpu")->base()+0x010000) + (mem_bank<<14);
 
-			memory_set_bankptr(machine, bank1, addr);
+			state->membank(bank1)->set_base(addr);
 
 			space->nop_write((bank * 0x4000), (bank * 0x4000) + 0x3fff);
 			LOG(("BANK %d: ROM %d\n",bank,mem_bank));
@@ -360,10 +360,10 @@ static void nc_refresh_memory_bank_config(running_machine &machine, int bank)
 
 			mem_bank = mem_bank & state->m_membank_internal_ram_mask;
 
-			addr = ram_get_ptr(machine.device(RAM_TAG)) + (mem_bank<<14);
+			addr = machine.device<ram_device>(RAM_TAG)->pointer() + (mem_bank<<14);
 
-			memory_set_bankptr(machine, bank1, addr);
-			memory_set_bankptr(machine, bank5, addr);
+			state->membank(bank1)->set_base(addr);
+			state->membank(bank5)->set_base(addr);
 
 			space->install_write_bank((bank * 0x4000), (bank * 0x4000) + 0x3fff, nc_bankhandler_w[bank]);
 			LOG(("BANK %d: RAM\n",bank));
@@ -381,13 +381,13 @@ static void nc_refresh_memory_bank_config(running_machine &machine, int bank)
 				mem_bank = mem_bank & state->m_membank_card_ram_mask;
 				addr = state->m_card_ram + (mem_bank<<14);
 
-				memory_set_bankptr(machine, bank1, addr);
+				state->membank(bank1)->set_base(addr);
 
 				/* write enabled? */
-				if (input_port_read(machine, "EXTRA") & 0x02)
+				if (machine.root_device().ioport("EXTRA")->read() & 0x02)
 				{
 					/* yes */
-					memory_set_bankptr(machine, bank5, addr);
+					state->membank(bank5)->set_base(addr);
 
 					space->install_write_bank((bank * 0x4000), (bank * 0x4000) + 0x3fff, nc_bankhandler_w[bank]);
 				}
@@ -433,13 +433,13 @@ static void nc_common_restore_memory_from_stream(running_machine &machine)
 	/* get size of memory data stored */
 	state->m_file->read(&stored_size, sizeof(unsigned long));
 
-	if (stored_size > ram_get_size(machine.device(RAM_TAG)))
-		restore_size = ram_get_size(machine.device(RAM_TAG));
+	if (stored_size > machine.device<ram_device>(RAM_TAG)->size())
+		restore_size = machine.device<ram_device>(RAM_TAG)->size();
 	else
 		restore_size = stored_size;
 
 	/* read as much as will fit into memory */
-	state->m_file->read(ram_get_ptr(machine.device(RAM_TAG)), restore_size);
+	state->m_file->read(machine.device<ram_device>(RAM_TAG)->pointer(), restore_size);
 	/* seek over remaining data */
 	state->m_file->seek(SEEK_CUR,stored_size - restore_size);
 }
@@ -448,7 +448,7 @@ static void nc_common_restore_memory_from_stream(running_machine &machine)
 static void nc_common_store_memory_to_stream(running_machine &machine)
 {
 	nc_state *state = machine.driver_data<nc_state>();
-	UINT32 size = ram_get_size(machine.device(RAM_TAG));
+	UINT32 size = machine.device<ram_device>(RAM_TAG)->size();
 	if (!state->m_file)
 		return;
 
@@ -457,7 +457,7 @@ static void nc_common_store_memory_to_stream(running_machine &machine)
 	state->m_file->write(&size, sizeof(UINT32));
 
 	/* write data block */
-	state->m_file->write(ram_get_ptr(machine.device(RAM_TAG)), size);
+	state->m_file->write(machine.device<ram_device>(RAM_TAG)->pointer(), size);
 }
 
 static void nc_common_open_stream_for_reading(running_machine &machine)
@@ -499,7 +499,7 @@ static TIMER_DEVICE_CALLBACK(dummy_timer_callback)
 	int inputport_10_state;
 	int changed_bits;
 
-	inputport_10_state = input_port_read(timer.machine(), "EXTRA");
+	inputport_10_state = timer.machine().root_device().ioport("EXTRA")->read();
 
 	changed_bits = inputport_10_state^state->m_previous_inputport_10_state;
 
@@ -553,7 +553,7 @@ static void nc_common_init_machine(running_machine &machine)
 	state->m_memory_config[2] = 0;
 	state->m_memory_config[3] = 0;
 
-	state->m_previous_inputport_10_state = input_port_read(machine, "EXTRA");
+	state->m_previous_inputport_10_state = machine.root_device().ioport("EXTRA")->read();
 
 	/* setup reset state ints are masked */
 	state->m_irq_mask = 0;
@@ -577,7 +577,7 @@ static void nc_common_init_machine(running_machine &machine)
 	state->m_uart_control = 0x0ff;
 }
 
-static ADDRESS_MAP_START(nc_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START(nc_map, AS_PROGRAM, 8, nc_state )
 	AM_RANGE(0x0000, 0x3fff) AM_READ_BANK("bank1") AM_WRITE_BANK("bank5")
 	AM_RANGE(0x4000, 0x7fff) AM_READ_BANK("bank2") AM_WRITE_BANK("bank6")
 	AM_RANGE(0x8000, 0xbfff) AM_READ_BANK("bank3") AM_WRITE_BANK("bank7")
@@ -585,49 +585,45 @@ static ADDRESS_MAP_START(nc_map, AS_PROGRAM, 8 )
 ADDRESS_MAP_END
 
 
-static  READ8_HANDLER(nc_memory_management_r)
+READ8_MEMBER(nc_state::nc_memory_management_r)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
-	return state->m_memory_config[offset];
+	return m_memory_config[offset];
 }
 
-static WRITE8_HANDLER(nc_memory_management_w)
+WRITE8_MEMBER(nc_state::nc_memory_management_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	LOG(("Memory management W: %02x %02x\n",offset,data));
-        state->m_memory_config[offset] = data;
+        m_memory_config[offset] = data;
 
-        nc_refresh_memory_config(space->machine());
+        nc_refresh_memory_config(machine());
 }
 
-static WRITE8_HANDLER(nc_irq_mask_w)
+WRITE8_MEMBER(nc_state::nc_irq_mask_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	LOG(("irq mask w: %02x\n", data));
 	LOG_DEBUG(("irq mask nc200 w: %02x\n",data & ((1<<4) | (1<<5) | (1<<6) | (1<<7))));
 
 	/* writing mask clears ints that are to be masked? */
-	state->m_irq_mask = data;
+	m_irq_mask = data;
 
-	nc_update_interrupts(space->machine());
+	nc_update_interrupts(machine());
 }
 
-static WRITE8_HANDLER(nc_irq_status_w)
+WRITE8_MEMBER(nc_state::nc_irq_status_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	LOG(("irq status w: %02x\n", data));
 	data = data^0x0ff;
 
-	if (state->m_type == NC_TYPE_200)
+	if (m_type == NC_TYPE_200)
 	{
 		/* Russell Marks confirms that on the NC200, the key scan interrupt must be explicitly
         cleared. It is not automatically cleared when reading 0x0b9 */
 		if ((data & (1<<3))!=0)
 		{
 			/* set timer to occur again */
-			state->m_keyboard_timer->reset(attotime::from_msec(10));
+			m_keyboard_timer->reset(attotime::from_msec(10));
 
-			nc_update_interrupts(space->machine());
+			nc_update_interrupts(machine());
 		}
 	}
 
@@ -638,28 +634,26 @@ static WRITE8_HANDLER(nc_irq_status_w)
                 /* clearing keyboard int? */
                 ((data & (1<<3))!=0) &&
                 /* keyboard int request? */
-                ((state->m_irq_status & (1<<3))!=0)
+                ((m_irq_status & (1<<3))!=0)
            )
         {
 			/* set timer to occur again */
-			state->m_keyboard_timer->reset(attotime::from_msec(10));
+			m_keyboard_timer->reset(attotime::from_msec(10));
         }
 #endif
-        state->m_irq_status &=~data;
+        m_irq_status &=~data;
 
-        nc_update_interrupts(space->machine());
+        nc_update_interrupts(machine());
 }
 
-static READ8_HANDLER(nc_irq_status_r)
+READ8_MEMBER(nc_state::nc_irq_status_r)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
-        return ~((state->m_irq_status & (~state->m_irq_latch_mask)) | state->m_irq_latch);
+        return ~((m_irq_status & (~m_irq_latch_mask)) | m_irq_latch);
 }
 
 
-static READ8_HANDLER(nc_key_data_in_r)
+READ8_MEMBER(nc_state::nc_key_data_in_r)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	static const char *const keynames[] = {
 		"LINE0", "LINE1", "LINE2", "LINE3", "LINE4",
 		"LINE5", "LINE6", "LINE7", "LINE8", "LINE9"
@@ -668,15 +662,15 @@ static READ8_HANDLER(nc_key_data_in_r)
 	if (offset==9)
 	{
 		/* reading 0x0b9 will clear int and re-start scan procedure! */
-		state->m_irq_status &= ~(1<<3);
+		m_irq_status &= ~(1<<3);
 
 		/* set timer to occur again */
-		state->m_keyboard_timer->reset(attotime::from_msec(10));
+		m_keyboard_timer->reset(attotime::from_msec(10));
 
-		nc_update_interrupts(space->machine());
+		nc_update_interrupts(machine());
 	}
 
-	return input_port_read(space->machine(), keynames[offset]);
+	return ioport(keynames[offset])->read();
 }
 
 
@@ -712,9 +706,8 @@ static void nc_sound_update(running_machine &machine, int channel)
 	beep_set_frequency(machine.device(beep_device), frequency);
 }
 
-static WRITE8_HANDLER(nc_sound_w)
+WRITE8_MEMBER(nc_state::nc_sound_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	LOG(("sound w: %04x %02x\n", offset, data));
 
 	switch (offset)
@@ -722,38 +715,38 @@ static WRITE8_HANDLER(nc_sound_w)
 		case 0x0:
 		{
 		   /* update period value */
-		   state->m_sound_channel_periods[0]  =
-				(state->m_sound_channel_periods[0] & 0x0ff00) | (data & 0x0ff);
+		   m_sound_channel_periods[0]  =
+				(m_sound_channel_periods[0] & 0x0ff00) | (data & 0x0ff);
 
-		   nc_sound_update(space->machine(), 0);
+		   nc_sound_update(machine(), 0);
 		}
 		break;
 
 		case 0x01:
 		{
-		   state->m_sound_channel_periods[0] =
-				(state->m_sound_channel_periods[0] & 0x0ff) | ((data & 0x0ff)<<8);
+		   m_sound_channel_periods[0] =
+				(m_sound_channel_periods[0] & 0x0ff) | ((data & 0x0ff)<<8);
 
-		   nc_sound_update(space->machine(), 0);
+		   nc_sound_update(machine(), 0);
 		}
 		break;
 
 		case 0x02:
 		{
 		   /* update period value */
-		   state->m_sound_channel_periods[1]  =
-				(state->m_sound_channel_periods[1] & 0x0ff00) | (data & 0x0ff);
+		   m_sound_channel_periods[1]  =
+				(m_sound_channel_periods[1] & 0x0ff00) | (data & 0x0ff);
 
-		   nc_sound_update(space->machine(), 1);
+		   nc_sound_update(machine(), 1);
 		}
 		break;
 
 		case 0x03:
 		{
-		   state->m_sound_channel_periods[1] =
-				(state->m_sound_channel_periods[1] & 0x0ff) | ((data & 0x0ff)<<8);
+		   m_sound_channel_periods[1] =
+				(m_sound_channel_periods[1] & 0x0ff) | ((data & 0x0ff)<<8);
 
-		   nc_sound_update(space->machine(), 1);
+		   nc_sound_update(machine(), 1);
 		}
 		break;
 
@@ -782,25 +775,24 @@ static TIMER_CALLBACK(nc_serial_timer_callback)
 	uart->receive_clock();
 }
 
-static WRITE8_HANDLER(nc_uart_control_w)
+WRITE8_MEMBER(nc_state::nc_uart_control_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	/* update printer state */
-	nc_printer_update(space->machine(), data);
+	nc_printer_update(machine(), data);
 
 	/* on/off changed state? */
-	if (((state->m_uart_control ^ data) & (1<<3))!=0)
+	if (((m_uart_control ^ data) & (1<<3))!=0)
 	{
 		/* changed uart from off to on */
 		if ((data & (1<<3))==0)
 		{
-			devtag_reset(space->machine(), "uart");
+			devtag_reset(machine(), "uart");
 		}
 	}
 
-	state->m_serial_timer->adjust(attotime::zero, 0, attotime::from_hz(baud_rate_table[(data & 0x07)]));
+	m_serial_timer->adjust(attotime::zero, 0, attotime::from_hz(baud_rate_table[(data & 0x07)]));
 
-	state->m_uart_control = data;
+	m_uart_control = data;
 }
 
 /* NC100 printer emulation */
@@ -811,8 +803,8 @@ static WRITE8_HANDLER(nc_uart_control_w)
 /* same for nc100 and nc200 */
 static void nc_printer_update(running_machine &machine, UINT8 data)
 {
-	device_t *printer = machine.device("centronics");
-	centronics_strobe_w(printer, BIT(data, 6));
+	centronics_device *printer = machine.device<centronics_device>("centronics");
+	printer->strobe_w(BIT(data, 6));
 }
 
 /********************************************************************************************************/
@@ -820,30 +812,28 @@ static void nc_printer_update(running_machine &machine, UINT8 data)
 
 
 
-static WRITE8_HANDLER(nc100_display_memory_start_w)
+WRITE8_MEMBER(nc_state::nc100_display_memory_start_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	/* bit 7: A15 */
 	/* bit 6: A14 */
 	/* bit 5: A13 */
 	/* bit 4: A12 */
 	/* bit 3-0: not used */
-	state->m_display_memory_start = (data & 0x0f0)<<(12-4);
+	m_display_memory_start = (data & 0x0f0)<<(12-4);
 
-	LOG(("disp memory w: %04x\n", (int) state->m_display_memory_start));
+	LOG(("disp memory w: %04x\n", (int) m_display_memory_start));
 }
 
 
-static WRITE8_HANDLER(nc100_uart_control_w)
+WRITE8_MEMBER(nc_state::nc100_uart_control_w)
 {
-	//nc_state *state = space->machine().driver_data<nc_state>();
 	nc_uart_control_w(space, offset,data);
 
 //  /* is this correct?? */
 //  if (data & (1<<3))
 //  {
 //      /* clear latched irq's */
-//      state->m_irq_latch &= ~3;
+//      m_irq_latch &= ~3;
 //      nc_update_interrupts(machine);
 //  }
 }
@@ -923,7 +913,6 @@ static WRITE_LINE_DEVICE_HANDLER( nc100_centronics_ack_w )
 
 static const centronics_interface nc100_centronics_config =
 {
-	FALSE,
 	DEVCB_LINE(nc100_centronics_ack_w),
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -973,34 +962,32 @@ static MACHINE_START( nc100 )
 }
 
 
-static WRITE8_HANDLER(nc100_poweroff_control_w)
+WRITE8_MEMBER(nc_state::nc100_poweroff_control_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	/* bits 7-1: not used */
 	/* bit 0: 1 = no effect, 0 = power off */
-	state->m_poweroff_control = data;
+	m_poweroff_control = data;
 	LOG(("nc poweroff control: %02x\n",data));
 }
 
 
 /* nc100 version of card/battery status */
-static  READ8_HANDLER(nc100_card_battery_status_r)
+READ8_MEMBER(nc_state::nc100_card_battery_status_r)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
-	device_t *printer = space->machine().device("centronics");
+	centronics_device *printer = machine().device<centronics_device>("centronics");
 	int nc_card_battery_status = 0x0fc;
 
 	/* printer */
-	nc_card_battery_status |= centronics_ack_r(printer);
-	nc_card_battery_status |= centronics_busy_r(printer) << 1;
+	nc_card_battery_status |= printer->ack_r();
+	nc_card_battery_status |= printer->busy_r() << 1;
 
-	if (state->m_card_status)
+	if (m_card_status)
 	{
 		/* card present */
 		nc_card_battery_status &=~(1<<7);
 	}
 
-	if (input_port_read(space->machine(), "EXTRA") & 0x02)
+	if (ioport("EXTRA")->read() & 0x02)
 	{
 		/* card write enable */
 		nc_card_battery_status &=~(1<<6);
@@ -1014,21 +1001,21 @@ static  READ8_HANDLER(nc100_card_battery_status_r)
 	return nc_card_battery_status;
 }
 
-static WRITE8_HANDLER(nc100_memory_card_wait_state_w)
+WRITE8_MEMBER(nc_state::nc100_memory_card_wait_state_w)
 {
 	LOG(("nc100 memory card wait state: %02x\n",data));
 }
 
 
 
-static ADDRESS_MAP_START(nc100_io, AS_IO, 8)
+static ADDRESS_MAP_START(nc100_io, AS_IO, 8, nc_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE(0x00, 0x0f) AM_WRITE(nc100_display_memory_start_w)
 	AM_RANGE(0x10, 0x13) AM_READWRITE(nc_memory_management_r, nc_memory_management_w)
 	AM_RANGE(0x20, 0x20) AM_WRITE(nc100_memory_card_wait_state_w)
 	AM_RANGE(0x30, 0x30) AM_WRITE(nc100_uart_control_w)
-	AM_RANGE(0x40, 0x40) AM_DEVWRITE("centronics", centronics_data_w)
+	AM_RANGE(0x40, 0x40) AM_DEVWRITE("centronics", centronics_device, write)
 	AM_RANGE(0x50, 0x53) AM_WRITE(nc_sound_w)
 	AM_RANGE(0x60, 0x60) AM_WRITE(nc_irq_mask_w)
 	AM_RANGE(0x70, 0x70) AM_WRITE(nc100_poweroff_control_w)
@@ -1036,9 +1023,9 @@ static ADDRESS_MAP_START(nc100_io, AS_IO, 8)
 	AM_RANGE(0x91, 0x9f) AM_READ(nc_irq_status_r)
 	AM_RANGE(0xa0, 0xaf) AM_READ(nc100_card_battery_status_r)
 	AM_RANGE(0xb0, 0xb9) AM_READ(nc_key_data_in_r)
-	AM_RANGE(0xc0, 0xc0) AM_DEVREADWRITE_MODERN("uart",i8251_device, data_r, data_w)
-	AM_RANGE(0xc1, 0xc1) AM_DEVREADWRITE_MODERN("uart", i8251_device, status_r, control_w)
-	AM_RANGE(0xd0, 0xdf) AM_DEVREADWRITE_MODERN("rtc", rp5c01_device, read, write)
+	AM_RANGE(0xc0, 0xc0) AM_DEVREADWRITE("uart",i8251_device, data_r, data_w)
+	AM_RANGE(0xc1, 0xc1) AM_DEVREADWRITE("uart", i8251_device, status_r, control_w)
+	AM_RANGE(0xd0, 0xdf) AM_DEVREADWRITE("rtc", rp5c01_device, read, write)
 ADDRESS_MAP_END
 
 
@@ -1187,16 +1174,15 @@ void nc150_init_machine(running_machine &machine)
 /* NC200 hardware */
 
 #ifdef UNUSED_FUNCTION
-static WRITE8_HANDLER(nc200_display_memory_start_w)
+WRITE8_MEMBER(nc_state::nc200_display_memory_start_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	/* bit 7: A15 */
 	/* bit 6: A14 */
 	/* bit 5: A13 */
 	/* bit 4-0: not used */
-	state->m_display_memory_start = (data & 0x0e0)<<(12-4);
+	m_display_memory_start = (data & 0x0e0)<<(12-4);
 
-	LOG(("disp memory w: %04x\n", (int) state->m_display_memory_start));
+	LOG(("disp memory w: %04x\n", (int) m_display_memory_start));
 }
 #endif
 
@@ -1215,7 +1201,6 @@ static WRITE_LINE_DEVICE_HANDLER( nc200_centronics_ack_w )
 
 static const centronics_interface nc200_centronics_config =
 {
-	FALSE,
 	DEVCB_LINE(nc200_centronics_ack_w),
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -1383,9 +1368,8 @@ NC200:
 
 
 /* nc200 version of card/battery status */
-static  READ8_HANDLER(nc200_card_battery_status_r)
+READ8_MEMBER(nc_state::nc200_card_battery_status_r)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
 	int nc_card_battery_status = 0x0ff;
 
 	/* enough power */
@@ -1396,13 +1380,13 @@ static  READ8_HANDLER(nc200_card_battery_status_r)
     and there is enough power for disk usage */
 	nc_card_battery_status &=~((1<<5) | (1<<2) | (1<<0));
 
-	if (state->m_card_status)
+	if (m_card_status)
 	{
 		/* card present */
 		nc_card_battery_status&=~(1<<7);
 	}
 
-	if (input_port_read(space->machine(), "EXTRA") & 0x02)
+	if (ioport("EXTRA")->read() & 0x02)
 	{
 		/* card write enable */
 		nc_card_battery_status &=~(1<<6);
@@ -1417,33 +1401,32 @@ static  READ8_HANDLER(nc200_card_battery_status_r)
   bit 0: Parallel interface BUSY
  */
 
-static READ8_HANDLER(nc200_printer_status_r)
+READ8_MEMBER(nc_state::nc200_printer_status_r)
 {
-	device_t *printer = space->machine().device("centronics");
+	centronics_device *printer = machine().device<centronics_device>("centronics");
 	UINT8 result = 0;
 
-	result |= centronics_busy_r(printer);
+	result |= printer->busy_r();
 
 	return result;
 }
 
 
-static WRITE8_HANDLER(nc200_uart_control_w)
+WRITE8_MEMBER(nc_state::nc200_uart_control_w)
 {
-	nc_state *state = space->machine().driver_data<nc_state>();
-	/* int reset_fdc = (state->m_uart_control^data) & (1<<5); */
+	/* int reset_fdc = (m_uart_control^data) & (1<<5); */
 
 	nc_uart_control_w(space, offset,data);
 
 	if (data & (1<<3))
 	{
-		state->m_nc200_uart_interrupt_irq &=~3;
+		m_nc200_uart_interrupt_irq &=~3;
 
-		nc200_refresh_uart_interrupt(space->machine());
+		nc200_refresh_uart_interrupt(machine());
 	}
 
 	/* bit 5 is used in disk interface */
-	LOG_DEBUG(("bit 5: PC: %04x %02x\n", cpu_get_pc(space->machine().device("maincpu")), data & (1 << 5)));
+	LOG_DEBUG(("bit 5: PC: %04x %02x\n", cpu_get_pc(machine().device("maincpu")), data & (1 << 5)));
 }
 
 
@@ -1459,10 +1442,10 @@ static WRITE8_HANDLER(nc200_uart_control_w)
 /* bit 1: disk motor??  */
 /* bit 0: UPD765 Terminal Count input */
 
-static WRITE8_HANDLER(nc200_memory_card_wait_state_w)
+WRITE8_MEMBER(nc_state::nc200_memory_card_wait_state_w)
 {
-	device_t *fdc = space->machine().device("upd765");
-	LOG_DEBUG(("nc200 memory card wait state: PC: %04x %02x\n", cpu_get_pc(space->machine().device("maincpu")), data));
+	device_t *fdc = machine().device("upd765");
+	LOG_DEBUG(("nc200 memory card wait state: PC: %04x %02x\n", cpu_get_pc(machine().device("maincpu")), data));
 #if 0
 	floppy_drive_set_motor_state(0, 1);
 	floppy_drive_set_ready_state(0, 1, 1);
@@ -1473,20 +1456,20 @@ static WRITE8_HANDLER(nc200_memory_card_wait_state_w)
 /* bit 2: backlight: 1=off, 0=on */
 /* bit 1 cleared to zero in disk code */
 /* bit 0 seems to be the same as nc100 */
-static WRITE8_HANDLER(nc200_poweroff_control_w)
+WRITE8_MEMBER(nc_state::nc200_poweroff_control_w)
 {
-	LOG_DEBUG(("nc200 power off: PC: %04x %02x\n", cpu_get_pc(space->machine().device("maincpu")), data));
+	LOG_DEBUG(("nc200 power off: PC: %04x %02x\n", cpu_get_pc(machine().device("maincpu")), data));
 
-	nc200_video_set_backlight(space->machine(), ((data ^ (1 << 2)) >> 2) & 0x01);
+	nc200_video_set_backlight(machine(), ((data ^ (1 << 2)) >> 2) & 0x01);
 }
 
-static ADDRESS_MAP_START(nc200_io, AS_IO, 8)
+static ADDRESS_MAP_START(nc200_io, AS_IO, 8, nc_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	AM_RANGE(0x00, 0x0f) AM_WRITE(nc100_display_memory_start_w)
 	AM_RANGE(0x10, 0x13) AM_READWRITE(nc_memory_management_r, nc_memory_management_w)
 	AM_RANGE(0x20, 0x20) AM_WRITE(nc200_memory_card_wait_state_w)
 	AM_RANGE(0x30, 0x30) AM_WRITE(nc200_uart_control_w)
-	AM_RANGE(0x40, 0x40) AM_DEVWRITE("centronics", centronics_data_w)
+	AM_RANGE(0x40, 0x40) AM_DEVWRITE("centronics", centronics_device, write)
 	AM_RANGE(0x50, 0x53) AM_WRITE(nc_sound_w)
 	AM_RANGE(0x60, 0x60) AM_WRITE(nc_irq_mask_w)
 	AM_RANGE(0x70, 0x70) AM_WRITE(nc200_poweroff_control_w)
@@ -1494,11 +1477,11 @@ static ADDRESS_MAP_START(nc200_io, AS_IO, 8)
 	AM_RANGE(0x90, 0x90) AM_READWRITE(nc_irq_status_r, nc_irq_status_w)
 	AM_RANGE(0xa0, 0xa0) AM_READ(nc200_card_battery_status_r)
 	AM_RANGE(0xb0, 0xb9) AM_READ(nc_key_data_in_r)
-	AM_RANGE(0xc0, 0xc0) AM_DEVREADWRITE_MODERN("uart",i8251_device, data_r, data_w)
-	AM_RANGE(0xc1, 0xc1) AM_DEVREADWRITE_MODERN("uart", i8251_device, status_r, control_w)
-	AM_RANGE(0xd0, 0xd1) AM_DEVREADWRITE_MODERN("mc", mc146818_device, read, write)
-	AM_RANGE(0xe0, 0xe0) AM_DEVREAD("upd765", upd765_status_r)
-	AM_RANGE(0xe1, 0xe1) AM_DEVREADWRITE("upd765",upd765_data_r, upd765_data_w)
+	AM_RANGE(0xc0, 0xc0) AM_DEVREADWRITE("uart",i8251_device, data_r, data_w)
+	AM_RANGE(0xc1, 0xc1) AM_DEVREADWRITE("uart", i8251_device, status_r, control_w)
+	AM_RANGE(0xd0, 0xd1) AM_DEVREADWRITE("mc", mc146818_device, read, write)
+	AM_RANGE(0xe0, 0xe0) AM_DEVREAD_LEGACY("upd765", upd765_status_r)
+	AM_RANGE(0xe1, 0xe1) AM_DEVREADWRITE_LEGACY("upd765",upd765_data_r, upd765_data_w)
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START(nc200)
@@ -1627,10 +1610,9 @@ static MACHINE_CONFIG_START( nc100, nc_state )
 	MCFG_SCREEN_ADD("screen", LCD)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(480, 64)
 	MCFG_SCREEN_VISIBLE_AREA(0, 480-1, 0, 64-1)
-	MCFG_SCREEN_UPDATE( nc )
+	MCFG_SCREEN_UPDATE_STATIC( nc )
 
 	MCFG_PALETTE_LENGTH(NC_NUM_COLOURS)
 	MCFG_PALETTE_INIT( nc )
@@ -1646,7 +1628,7 @@ static MACHINE_CONFIG_START( nc100, nc_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.50)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", nc100_centronics_config)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", nc100_centronics_config)
 
 	/* uart */
 	MCFG_I8251_ADD("uart", nc100_uart_interface)
@@ -1699,7 +1681,7 @@ static MACHINE_CONFIG_DERIVED( nc200, nc100 )
 
 	/* printer */
 	MCFG_DEVICE_REMOVE("centronics")
-	MCFG_CENTRONICS_ADD("centronics", nc200_centronics_config)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", nc200_centronics_config)
 
 	/* uart */
 	MCFG_DEVICE_REMOVE("uart")
@@ -1731,8 +1713,12 @@ ROM_START(nc100)
 	ROM_REGION(((64*1024)+(256*1024)), "maincpu",0)
 	ROM_SYSTEM_BIOS(0, "106", "ROM v1.06")
     ROMX_LOAD("nc100a.rom", 0x010000, 0x040000, CRC(849884f9) SHA1(ff030dd334ca867d620ee4a94b142ef0d93b69b6), ROM_BIOS(1))
-	ROM_SYSTEM_BIOS(1, "100", "ROM v1.00")
-	ROMX_LOAD("nc100.rom",  0x010000, 0x040000, CRC(a699eca3) SHA1(ce217d5a298b959ccc3d7bc5c93b1dba043f1339), ROM_BIOS(2))
+	ROM_SYSTEM_BIOS(1, "102dk", "ROM v1.02 Danish")
+	ROMX_LOAD("nc100_dk_a1.rom", 0x010000, 0x040000, CRC(ebb54923) SHA1(30321011384c5e10204b9a837430c36fc63580d2), ROM_BIOS(2))
+	ROM_SYSTEM_BIOS(2, "102de", "ROM v1.02 German")
+	ROMX_LOAD("nc100_de_a1.rom", 0x010000, 0x040000, CRC(bd9ce223) SHA1(2efb26911832bf1456d76d2508e24c0733dc216d), ROM_BIOS(3))
+	ROM_SYSTEM_BIOS(3, "100", "ROM v1.00")
+	ROMX_LOAD("nc100.rom",  0x010000, 0x040000, CRC(a699eca3) SHA1(ce217d5a298b959ccc3d7bc5c93b1dba043f1339), ROM_BIOS(4))
 ROM_END
 
 
