@@ -36,12 +36,18 @@ public:
 
 	required_device<cpu_device>	m_maincpu;
 	required_device<device_t>	m_mb8877;
-	required_device<device_t>	m_messram;
+	required_device<ram_device>	m_messram;
 	required_device<pia6821_device>	m_pia_0;
 	required_device<pia6821_device>	m_pia_1;
 	required_device<z80dart_device>	m_sio;
 	required_device<device_t>	m_speaker;
 
+	virtual void video_start();
+
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
+
+	bitmap_ind16 m_bitmap;
+
 	memory_region	*m_fontram_region;
 	memory_region *m_vram_region;
 	UINT8	*m_fontram;
@@ -65,24 +71,24 @@ public:
 
 	void set_banks(running_machine &machine)
 	{
-		UINT8 *ram_ptr = ram_get_ptr( m_messram );
+		UINT8 *ram_ptr = m_messram->pointer();
 
 		m_ram_0000 = ram_ptr;
 
 		if ( m_pia0_porta & 0x01 )
 			m_ram_0000 += 0x10000;
 
-		memory_set_bankptr( machine, "0000", m_ram_0000 + 0x0000 );
-		memory_set_bankptr( machine, "2000", m_ram_0000 + 0x2000 );
-		memory_set_bankptr( machine, "4000", m_ram_0000 + 0x4000 );
+		membank( "0000" )->set_base( m_ram_0000 + 0x0000 );
+		membank( "2000" )->set_base( m_ram_0000 + 0x2000 );
+		membank( "4000" )->set_base( m_ram_0000 + 0x4000 );
 		m_ram_c000 = m_ram_0000 + 0xc000;
-		memory_set_bankptr( machine, "e000", m_ram_0000 + 0xe000 );
+		membank( "e000" )->set_base( m_ram_0000 + 0xe000 );
 
 		if ( m_pia0_porta & 0x80 )
 		{
-			memory_set_bankptr( machine, "0000", machine.region("maincpu")->base());
+			membank( "0000" )->set_base( memregion("maincpu")->base());
 			/* When BIOS is enabled 2000-3FFF is set to the "ROM RAM" */
-			memory_set_bankptr( machine, "2000", ram_ptr + 0x20000 );
+			membank( "2000" )->set_base( ram_ptr + 0x20000 );
 		}
 
 		if ( m_pia0_porta & 0x40 )
@@ -96,112 +102,113 @@ public:
 		else
 			device_set_input_line( m_maincpu, 0, CLEAR_LINE );
 	}
+	DECLARE_WRITE8_MEMBER(osbexec_0000_w);
+	DECLARE_READ8_MEMBER(osbexec_c000_r);
+	DECLARE_WRITE8_MEMBER(osbexec_c000_w);
+	DECLARE_READ8_MEMBER(osbexec_kbd_r);
+	DECLARE_READ8_MEMBER(osbexec_rtc_r);
 };
 
 
-static WRITE8_HANDLER( osbexec_0000_w )
+WRITE8_MEMBER(osbexec_state::osbexec_0000_w)
 {
-	osbexec_state *state = space->machine().driver_data<osbexec_state>();
 
 	/* Font RAM writing is enabled when ROM bank is enabled */
-	if ( state->m_pia0_porta & 0x80 )
+	if ( m_pia0_porta & 0x80 )
 	{
 		if ( offset < 0x1000 )
-			state->m_fontram[ offset ] = data;
+			m_fontram[ offset ] = data;
 	}
 	else
 	{
-		state->m_ram_0000[ offset ] = data;
+		m_ram_0000[ offset ] = data;
 	}
 }
 
 
-static READ8_HANDLER( osbexec_c000_r )
+READ8_MEMBER(osbexec_state::osbexec_c000_r)
 {
-	osbexec_state *state = space->machine().driver_data<osbexec_state>();
-	UINT8	data = state->m_ram_c000[offset];
+	UINT8	data = m_ram_c000[offset];
 
-	if ( ( state->m_pia0_porta & 0x40 ) && offset < 0x1000 )
+	if ( ( m_pia0_porta & 0x40 ) && offset < 0x1000 )
 	{
-		state->m_temp_attr = state->m_ram_c000[ 0x1000 + offset ];
+		m_temp_attr = m_ram_c000[ 0x1000 + offset ];
 	}
 
 	return data;
 }
 
 
-static WRITE8_HANDLER( osbexec_c000_w )
+WRITE8_MEMBER(osbexec_state::osbexec_c000_w)
 {
-	osbexec_state *state = space->machine().driver_data<osbexec_state>();
 
-	state->m_ram_c000[offset] = data;
+	m_ram_c000[offset] = data;
 
-	if ( ( state->m_pia0_porta & 0x40 ) && offset < 0x1000 )
+	if ( ( m_pia0_porta & 0x40 ) && offset < 0x1000 )
 	{
-		state->m_ram_c000[ 0x1000 + offset ] = state->m_temp_attr;
+		m_ram_c000[ 0x1000 + offset ] = m_temp_attr;
 	}
 }
 
 
-static READ8_HANDLER( osbexec_kbd_r )
+READ8_MEMBER(osbexec_state::osbexec_kbd_r)
 {
 	UINT8 data = 0xFF;
 
 	if ( offset & 0x0100 )
-		data &= input_port_read( space->machine(), "ROW0" );
+		data &= ioport( "ROW0" )->read();
 
 	if ( offset & 0x0200 )
-		data &= input_port_read( space->machine(), "ROW1" );
+		data &= ioport( "ROW1" )->read();
 
 	if ( offset & 0x0400 )
-		data &= input_port_read( space->machine(), "ROW2" );
+		data &= ioport( "ROW2" )->read();
 
 	if ( offset & 0x0800 )
-		data &= input_port_read( space->machine(), "ROW3" );
+		data &= ioport( "ROW3" )->read();
 
 	if ( offset & 0x1000 )
-		data &= input_port_read( space->machine(), "ROW4" );
+		data &= ioport( "ROW4" )->read();
 
 	if ( offset & 0x2000 )
-		data &= input_port_read( space->machine(), "ROW5" );
+		data &= ioport( "ROW5" )->read();
 
 	if ( offset & 0x4000 )
-		data &= input_port_read( space->machine(), "ROW6" );
+		data &= ioport( "ROW6" )->read();
 
 	if ( offset & 0x8000 )
-		data &= input_port_read( space->machine(), "ROW7" );
+		data &= ioport( "ROW7" )->read();
 
 	return data;
 }
 
 
-static READ8_HANDLER( osbexec_rtc_r )
+READ8_MEMBER(osbexec_state::osbexec_rtc_r)
 {
-	osbexec_state *state = space->machine().driver_data<osbexec_state>();
 
-	return state->m_rtc;
+	return m_rtc;
 }
 
 
-static ADDRESS_MAP_START( osbexec_mem, AS_PROGRAM, 8 )
-	AM_RANGE( 0x0000, 0x1FFF ) AM_READ_BANK("0000") AM_WRITE( osbexec_0000_w )	/* ROM and maybe also banked ram */
+static ADDRESS_MAP_START( osbexec_mem, AS_PROGRAM, 8, osbexec_state )
+	AM_RANGE( 0x0000, 0x1FFF ) AM_READ_BANK("0000") AM_WRITE(osbexec_0000_w )	/* ROM and maybe also banked ram */
 	AM_RANGE( 0x2000, 0x3FFF ) AM_RAMBANK("2000")								/* Banked RAM */
 	AM_RANGE( 0x4000, 0xBFFF ) AM_RAMBANK("4000")								/* Banked RAM */
-	AM_RANGE( 0xC000, 0xDFFF ) AM_READWRITE( osbexec_c000_r, osbexec_c000_w )	/* Video ram / Banked RAM */
+	AM_RANGE( 0xC000, 0xDFFF ) AM_READWRITE(osbexec_c000_r, osbexec_c000_w )	/* Video ram / Banked RAM */
 	AM_RANGE( 0xE000, 0xEFFF ) AM_RAMBANK("e000")								/* Banked RAM */
 	AM_RANGE( 0xF000, 0xFFFF ) AM_RAM											/* 4KB of non-banked RAM for system stack etc */
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( osbexec_io, AS_IO, 8 )
+static ADDRESS_MAP_START( osbexec_io, AS_IO, 8, osbexec_state )
 	ADDRESS_MAP_UNMAP_HIGH
-	AM_RANGE( 0x00, 0x03 ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE_MODERN( "pia_0", pia6821_device, read, write)				/* 6821 PIA @ UD12 */
+	AM_RANGE( 0x00, 0x03 ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE( "pia_0", pia6821_device, read, write)				/* 6821 PIA @ UD12 */
 	/* 0x04 - 0x07 - 8253 @UD1 */
-	AM_RANGE( 0x08, 0x0B ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE( "mb8877", wd17xx_r, wd17xx_w )				/* MB8877 @ UB17 input clock = 1MHz */
-	AM_RANGE( 0x0C, 0x0F ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE( "sio", z80dart_ba_cd_r, z80dart_ba_cd_w )	/* SIO @ UD4 */
-	AM_RANGE( 0x10, 0x13 ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE_MODERN( "pia_1", pia6821_device, read, write)				/* 6821 PIA @ UD8 */
-	AM_RANGE( 0x14, 0x17 ) AM_MIRROR( 0xff00 ) AM_MASK( 0xff00 ) AM_READ( osbexec_kbd_r )					/* KBD */
-	AM_RANGE( 0x18, 0x1b ) AM_MIRROR( 0xff00 ) AM_READ( osbexec_rtc_r )										/* "RTC" @ UE13/UF13 */
+	AM_RANGE( 0x08, 0x0B ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE_LEGACY("mb8877", wd17xx_r, wd17xx_w )				/* MB8877 @ UB17 input clock = 1MHz */
+	AM_RANGE( 0x0C, 0x0F ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE_LEGACY("sio", z80dart_ba_cd_r, z80dart_ba_cd_w )	/* SIO @ UD4 */
+	AM_RANGE( 0x10, 0x13 ) AM_MIRROR( 0xff00 ) AM_DEVREADWRITE( "pia_1", pia6821_device, read, write)				/* 6821 PIA @ UD8 */
+	AM_RANGE( 0x14, 0x17 ) AM_MIRROR( 0xff00 ) AM_MASK( 0xff00 ) AM_READ(osbexec_kbd_r )					/* KBD */
+	AM_RANGE( 0x18, 0x1b ) AM_MIRROR( 0xff00 ) AM_READ(osbexec_rtc_r )										/* "RTC" @ UE13/UF13 */
 	/* ?? - vid ? */
 ADDRESS_MAP_END
 
@@ -298,6 +305,16 @@ static PALETTE_INIT( osbexec )
 	palette_set_color_rgb( machine, 2, 0, 128, 0 );	/* Dimmed */
 }
 
+void osbexec_state::video_start()
+{
+	machine().primary_screen->register_screen_bitmap(m_bitmap);
+}
+
+UINT32 osbexec_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
+{
+	copybitmap(bitmap, m_bitmap, 0, 0, 0, 0, cliprect);
+	return 0;
+}
 
 /*
   UD12 - 6821 PIA
@@ -551,7 +568,7 @@ static TIMER_CALLBACK( osbexec_video_callback )
 	if ( y < 240 )
 	{
 		UINT16 row_addr = ( y / 10 ) * 128;
-		UINT16 *p = BITMAP_ADDR16( machine.generic.tmpbitmap, y, 0 );
+		UINT16 *p = &state->m_bitmap.pix16(y);
 		UINT8 char_line = y % 10;
 
 		for ( int x = 0; x < 80; x++ )
@@ -589,8 +606,8 @@ static DRIVER_INIT( osbexec )
 {
 	osbexec_state *state = machine.driver_data<osbexec_state>();
 
-	state->m_fontram_region = machine.region_alloc( "fontram", 0x1000, 1, ENDIANNESS_LITTLE);
-	state->m_vram_region = machine.region_alloc( "vram", 0x2000, 1, ENDIANNESS_LITTLE );
+	state->m_fontram_region = machine.memory().region_alloc( "fontram", 0x1000, 1, ENDIANNESS_LITTLE);
+	state->m_vram_region = machine.memory().region_alloc( "vram", 0x2000, 1, ENDIANNESS_LITTLE );
 	state->m_vram = state->m_vram_region->base();
 	state->m_fontram = state->m_fontram_region->base();
 
@@ -632,15 +649,12 @@ static MACHINE_CONFIG_START( osbexec, osbexec_state )
 	MCFG_MACHINE_RESET( osbexec )
 
 	MCFG_SCREEN_ADD("screen", RASTER)
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
+	MCFG_SCREEN_UPDATE_DRIVER(osbexec_state, screen_update)
 	MCFG_SCREEN_RAW_PARAMS( MAIN_CLOCK/2, 768, 0, 640, 260, 0, 240 )	/* May not be correct */
-	MCFG_SCREEN_UPDATE( generic_bitmapped )
 
 	MCFG_PALETTE_LENGTH( 3 )
 	MCFG_PALETTE_INIT( osbexec )
 
-	MCFG_VIDEO_START( generic_bitmapped )
-
 	MCFG_SPEAKER_STANDARD_MONO( "mono" )
 	MCFG_SOUND_ADD(SPEAKER_TAG, SPEAKER_SOUND, 0)
 	MCFG_SOUND_ROUTE( ALL_OUTPUTS, "mono", 1.00 )
