@@ -8,7 +8,6 @@
     Additional bug fixing by R. Belmont
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -32,7 +31,7 @@ public:
 	required_device<cpu_device> m_maincpu;
 	required_device<device_t> m_lsi;
 	required_device<device_t> m_dac;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 	// mc68328 needs modernising
 	//DECLARE_WRITE8_MEMBER(palm_dac_transition);
 	//DECLARE_WRITE8_MEMBER(palm_port_f_out);
@@ -53,7 +52,7 @@ static offs_t palm_dasm_override(device_t &device, char *buffer, offs_t pc, cons
 
 static INPUT_CHANGED( pen_check )
 {
-	UINT8 button = input_port_read(field.machine(), "PENB");
+	UINT8 button = field.machine().root_device().ioport("PENB")->read();
 	palm_state *state = field.machine().driver_data<palm_state>();
 
 	if(button)
@@ -64,7 +63,7 @@ static INPUT_CHANGED( pen_check )
 
 static INPUT_CHANGED( button_check )
 {
-	UINT8 button_state = input_port_read(field.machine(), "PORTD");
+	UINT8 button_state = field.machine().root_device().ioport("PORTD")->read();
 	palm_state *state = field.machine().driver_data<palm_state>();
 
 	mc68328_set_port_d_lines(state->m_lsi, button_state, (int)(FPTR)param);
@@ -102,8 +101,8 @@ static READ16_DEVICE_HANDLER( palm_spim_in )
 static void palm_spim_exchange( device_t *device )
 {
 	palm_state *state = device->machine().driver_data<palm_state>();
-	UINT8 x = input_port_read(device->machine(), "PENX");
-	UINT8 y = input_port_read(device->machine(), "PENY");
+	UINT8 x = device->machine().root_device().ioport("PENX")->read();
+	UINT8 y = state->ioport("PENY")->read();
 
 	switch( state->m_port_f_latch & 0x0f )
 	{
@@ -121,9 +120,9 @@ static MACHINE_START( palm )
 {
 	palm_state *state = machine.driver_data<palm_state>();
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
-	space->install_read_bank (0x000000, ram_get_size(machine.device(RAM_TAG)) - 1, ram_get_size(machine.device(RAM_TAG)) - 1, 0, "bank1");
-	space->install_write_bank(0x000000, ram_get_size(machine.device(RAM_TAG)) - 1, ram_get_size(machine.device(RAM_TAG)) - 1, 0, "bank1");
-	memory_set_bankptr(machine, "bank1", ram_get_ptr(machine.device(RAM_TAG)));
+	space->install_read_bank (0x000000, machine.device<ram_device>(RAM_TAG)->size() - 1, machine.device<ram_device>(RAM_TAG)->size() - 1, 0, "bank1");
+	space->install_write_bank(0x000000, machine.device<ram_device>(RAM_TAG)->size() - 1, machine.device<ram_device>(RAM_TAG)->size() - 1, 0, "bank1");
+	state->membank("bank1")->set_base(machine.device<ram_device>(RAM_TAG)->pointer());
 
 	state->save_item(NAME(state->m_port_f_latch));
 	state->save_item(NAME(state->m_spim_data));
@@ -135,9 +134,9 @@ static MACHINE_START( palm )
 static MACHINE_RESET( palm )
 {
     // Copy boot ROM
-	UINT8* bios = machine.region("bios")->base();
-	memset(ram_get_ptr(machine.device(RAM_TAG)), 0, ram_get_size(machine.device(RAM_TAG)));
-	memcpy(ram_get_ptr(machine.device(RAM_TAG)), bios, 0x20000);
+	UINT8* bios = machine.root_device().memregion("bios")->base();
+	memset(machine.device<ram_device>(RAM_TAG)->pointer(), 0, machine.device<ram_device>(RAM_TAG)->size());
+	memcpy(machine.device<ram_device>(RAM_TAG)->pointer(), bios, 0x20000);
 
 	palm_state *state = machine.driver_data<palm_state>();
 	state->m_maincpu->reset();
@@ -216,11 +215,10 @@ static MACHINE_CONFIG_START( palm, palm_state )
 
 	/* video hardware */
 	MCFG_VIDEO_ATTRIBUTES( VIDEO_UPDATE_BEFORE_VBLANK )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
 	MCFG_SCREEN_SIZE( 160, 220 )
 	MCFG_SCREEN_VISIBLE_AREA( 0, 159, 0, 219 )
 	MCFG_VIDEO_START( mc68328 )
-	MCFG_SCREEN_UPDATE( mc68328 )
+	MCFG_SCREEN_UPDATE_STATIC( mc68328 )
 	MCFG_PALETTE_LENGTH( 2 )
 	MCFG_PALETTE_INIT( mc68328 )
 	MCFG_DEFAULT_LAYOUT(layout_lcd)
