@@ -16,7 +16,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/hd61700/hd61700.h"
@@ -45,7 +44,7 @@ public:
 	UINT8 m_gatearray[2];
 
 	virtual void machine_start();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	DECLARE_WRITE16_MEMBER( gatearray_w );
 	UINT16 pb2000c_kb_r(running_machine &machine);
 	UINT16 pb1000_kb_r(running_machine &machine);
@@ -304,16 +303,11 @@ WRITE16_MEMBER( pb1000_state::gatearray_w )
 	m_gatearray[offset] = data&0xff;
 
 	if (m_gatearray[0])
-		memory_set_bankptr(machine(), "bank1", machine().region("card1")->base());
+		membank("bank1")->set_base(machine().root_device().memregion("card1")->base());
 	else if (m_gatearray[1])
-		memory_set_bankptr(machine(), "bank1", machine().region("card2")->base());
+		membank("bank1")->set_base(machine().root_device().memregion("card2")->base());
 	else
-		memory_set_bankptr(machine(), "bank1", machine().region("rom")->base());
-}
-
-bool pb1000_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	return m_hd44352->video_update(bitmap, cliprect);
+		membank("bank1")->set_base(machine().root_device().memregion("rom")->base());
 }
 
 static void lcd_control(hd61700_cpu_device &device, UINT8 data)
@@ -342,10 +336,10 @@ static void lcd_data_w(hd61700_cpu_device &device, UINT8 data)
 
 UINT16 pb1000_state::read_touchscreen(running_machine &machine, UINT8 line)
 {
-	UINT8 x = input_port_read(machine, "POSX")/0x40;
-	UINT8 y = input_port_read(machine, "POSY")/0x40;
+	UINT8 x = machine.root_device().ioport("POSX")->read()/0x40;
+	UINT8 y = machine.root_device().ioport("POSY")->read()/0x40;
 
-	if (input_port_read(machine, "TOUCH"))
+	if (machine.root_device().ioport("TOUCH")->read())
 	{
 		if (x == line-7)
 			return (0x1000<<y);
@@ -365,14 +359,14 @@ UINT16 pb1000_state::pb1000_kb_r(running_machine &machine)
 		//Read all the input lines
 		for (int line = 1; line <= 12; line++)
 		{
-			data |= input_port_read(machine, bitnames[line]);
+			data |= machine.root_device().ioport(bitnames[line])->read();
 			data |= read_touchscreen(machine, line);
 		}
 
 	}
 	else
 	{
-		data = input_port_read(machine, bitnames[m_kb_matrix & 0x0f]);
+		data = machine.root_device().ioport(bitnames[m_kb_matrix & 0x0f])->read();
 		data |= read_touchscreen(machine, m_kb_matrix & 0x0f);
 	}
 
@@ -389,13 +383,13 @@ UINT16 pb1000_state::pb2000c_kb_r(running_machine &machine)
 		//Read all the input lines
 		for (int line = 1; line <= 12; line++)
 		{
-			data |= input_port_read(machine, bitnames[line]);
+			data |= machine.root_device().ioport(bitnames[line])->read();
 		}
 
 	}
 	else
 	{
-		data = input_port_read(machine, bitnames[m_kb_matrix & 0x0f]);
+		data = machine.root_device().ioport(bitnames[m_kb_matrix & 0x0f])->read();
 	}
 
 	return data;
@@ -495,7 +489,7 @@ static TIMER_CALLBACK( keyboard_timer )
 
 void pb1000_state::machine_start()
 {
-	memory_set_bankptr(machine(), "bank1", machine().region("rom")->base());
+	membank("bank1")->set_base(machine().root_device().memregion("rom")->base());
 
 	m_kb_timer = machine().scheduler().timer_alloc(FUNC(keyboard_timer));
 	m_kb_timer->adjust(attotime::from_hz(192), 0, attotime::from_hz(192));
@@ -516,7 +510,7 @@ static MACHINE_CONFIG_START( pb1000, pb1000_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DEVICE("hd44352", hd44352_device, screen_update)
 	MCFG_SCREEN_SIZE(192, 32)
 	MCFG_SCREEN_VISIBLE_AREA(0, 192-1, 0, 32-1)
 	MCFG_DEFAULT_LAYOUT(layout_lcd)
@@ -545,7 +539,7 @@ static MACHINE_CONFIG_START( pb2000c, pb1000_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DEVICE("hd44352", hd44352_device, screen_update)
 	MCFG_SCREEN_SIZE(192, 32)
 	MCFG_SCREEN_VISIBLE_AREA(0, 192-1, 0, 32-1)
 	MCFG_DEFAULT_LAYOUT(layout_lcd)
