@@ -319,13 +319,13 @@ READ8_MEMBER( pc1512_state::printer_r )
 
         */
 
-		data |= input_port_read(machine(), "LK") & 0x07;
+		data |= ioport("LK")->read() & 0x07;
 
-		data |= centronics_fault_r(m_centronics) << 3;
-		data |= centronics_vcc_r(m_centronics) << 4;
-		data |= centronics_pe_r(m_centronics) << 5;
-		data |= centronics_ack_r(m_centronics) << 6;
-		data |= centronics_busy_r(m_centronics) << 7;
+		data |= m_centronics->fault_r() << 3;
+		data |= m_centronics->vcc_r() << 4;
+		data |= m_centronics->pe_r() << 5;
+		data |= m_centronics->ack_r() << 6;
+		data |= m_centronics->busy_r() << 7;
 		break;
 
 	case 2:
@@ -404,7 +404,7 @@ READ8_MEMBER( pc1640_state::printer_r )
         */
 		data = m_printer_control;
 		data |= m_opt << 5;
-		data |= (input_port_read(machine(), "SW") & 0x60) << 1;
+		data |= (ioport("SW")->read() & 0x60) << 1;
 		break;
 
 	default:
@@ -426,7 +426,7 @@ WRITE8_MEMBER( pc1512_state::printer_w )
 	{
 	case 0:
 		m_printer_data = data;
-		centronics_data_w(m_centronics, 0, data);
+		m_centronics->write(space, 0, data);
 		break;
 
 	case 2:
@@ -447,9 +447,9 @@ WRITE8_MEMBER( pc1512_state::printer_w )
 
 		m_printer_control = data & 0x1f;
 
-		centronics_strobe_w(m_centronics, BIT(data, 0));
-		centronics_autofeed_w(m_centronics, BIT(data, 1));
-		centronics_init_w(m_centronics, BIT(data, 2));
+		m_centronics->strobe_w(BIT(data, 0));
+		m_centronics->autofeed_w(BIT(data, 1));
+		m_centronics->init_prime_w(BIT(data, 2));
 
 		m_ack_int_enable = BIT(data, 4);
 		update_ack();
@@ -550,7 +550,7 @@ READ8_MEMBER( pc1640_state::io_r )
 	offs_t addr = offset & 0x3ff;
 	bool decoded = false;
 
-	if		(addr >= 0x000 && addr <= 0x00f) { data = i8237_r(m_dmac, offset & 0x0f); decoded = true; }
+	if		(                 addr <= 0x00f) { data = i8237_r(m_dmac, offset & 0x0f); decoded = true; }
 	else if (addr >= 0x020 && addr <= 0x021) { data = pic8259_r(m_pic, offset & 0x01); decoded = true; }
 	else if (addr >= 0x040 && addr <= 0x043) { data = pit8253_r(m_pit, offset & 0x03); decoded = true; }
 	else if (addr >= 0x060 && addr <= 0x06f) { data = system_r(space, offset & 0x0f); decoded = true; }
@@ -559,7 +559,7 @@ READ8_MEMBER( pc1640_state::io_r )
 	else if (addr >= 0x378 && addr <= 0x37b) { data = printer_r(space, offset & 0x03); decoded = true; }
 	else if (addr >= 0x3b0 && addr <= 0x3df) { data = iga_r(space, addr - 0x3b0); decoded = true; }
 	else if (addr >= 0x3f0 && addr <= 0x3f7) { data = fdc_r(space, offset & 0x07); decoded = true; }
-	else if (addr >= 0x3f8 && addr <= 0x3ff) { data = ins8250_r(m_uart, offset & 0x07); decoded = true; }
+	else if (addr >= 0x3f8 && addr <= 0x3ff) { data = m_uart->ins8250_r(space, offset & 0x07); decoded = true; }
 
 	if (decoded)
 	{
@@ -571,7 +571,7 @@ READ8_MEMBER( pc1640_state::io_r )
 	}
 	else if (!BIT(offset, 7))
 	{
-		UINT16 sw = input_port_read(machine(), "SW");
+		UINT16 sw = ioport("SW")->read();
 
 		if (!BIT(offset, 14))
 		{
@@ -622,7 +622,7 @@ static ADDRESS_MAP_START( pc1512_io, AS_IO, 16, pc1512_state )
 	AM_RANGE(0x378, 0x37b) AM_READWRITE8(printer_r, printer_w, 0xffff)
 	AM_RANGE(0x3d0, 0x3df) AM_READWRITE8(vdu_r, vdu_w, 0xffff)
 	AM_RANGE(0x3f0, 0x3f7) AM_READWRITE8(fdc_r, fdc_w, 0xffff)
-	AM_RANGE(0x3f8, 0x3ff) AM_DEVREADWRITE8_LEGACY(INS8250_TAG, ins8250_r, ins8250_w, 0xffff)
+	AM_RANGE(0x3f8, 0x3ff) AM_DEVREADWRITE8(INS8250_TAG, ins8250_device, ins8250_r, ins8250_w, 0xffff)
 ADDRESS_MAP_END
 
 
@@ -656,7 +656,7 @@ static ADDRESS_MAP_START( pc1640_io, AS_IO, 16, pc1640_state )
 	AM_RANGE(0x378, 0x37b) AM_WRITE8(printer_w, 0xffff)
 	AM_RANGE(0x3b0, 0x3df) AM_WRITE8(iga_w, 0xffff)
 	AM_RANGE(0x3f0, 0x3f7) AM_WRITE8(fdc_w, 0xffff)
-	AM_RANGE(0x3f8, 0x3ff) AM_DEVWRITE8_LEGACY(INS8250_TAG, ins8250_w, 0xffff)
+	AM_RANGE(0x3f8, 0x3ff) AM_DEVWRITE8(INS8250_TAG, ins8250_device, ins8250_w, 0xffff)
 ADDRESS_MAP_END
 
 
@@ -669,11 +669,9 @@ ADDRESS_MAP_END
 //  INPUT_CHANGED( mouse_button_1_changed )
 //-------------------------------------------------
 
-static INPUT_CHANGED( mouse_button_1_changed )
+INPUT_CHANGED_MEMBER( pc1512_state::mouse_button_1_changed )
 {
-	pc1512_state *state = field.machine().driver_data<pc1512_state>();
-
-	state->m_kb->m1_w(newval);
+	m_kb->m1_w(newval);
 }
 
 
@@ -681,11 +679,9 @@ static INPUT_CHANGED( mouse_button_1_changed )
 //  INPUT_CHANGED( mouse_button_2_changed )
 //-------------------------------------------------
 
-static INPUT_CHANGED( mouse_button_2_changed )
+INPUT_CHANGED_MEMBER( pc1512_state::mouse_button_2_changed )
 {
-	pc1512_state *state = field.machine().driver_data<pc1512_state>();
-
-	state->m_kb->m2_w(newval);
+	m_kb->m2_w(newval);
 }
 
 
@@ -693,14 +689,12 @@ static INPUT_CHANGED( mouse_button_2_changed )
 //  INPUT_CHANGED( mouse_x_changed )
 //-------------------------------------------------
 
-static INPUT_CHANGED( mouse_x_changed )
+INPUT_CHANGED_MEMBER( pc1512_state::mouse_x_changed )
 {
-	pc1512_state *state = field.machine().driver_data<pc1512_state>();
-
 	if (newval > oldval)
-		state->m_mouse_x++;
+		m_mouse_x++;
 	else
-		state->m_mouse_x--;
+		m_mouse_x--;
 }
 
 
@@ -708,14 +702,12 @@ static INPUT_CHANGED( mouse_x_changed )
 //  INPUT_CHANGED( mouse_y_changed )
 //-------------------------------------------------
 
-static INPUT_CHANGED( mouse_y_changed )
+INPUT_CHANGED_MEMBER( pc1512_state::mouse_y_changed )
 {
-	pc1512_state *state = field.machine().driver_data<pc1512_state>();
-
 	if (newval > oldval)
-		state->m_mouse_y--;
+		m_mouse_y--;
 	else
-		state->m_mouse_y++;
+		m_mouse_y++;
 }
 
 
@@ -725,14 +717,14 @@ static INPUT_CHANGED( mouse_y_changed )
 
 static INPUT_PORTS_START( mouse )
 	PORT_START("MOUSEB")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Left Mouse Button") PORT_CODE(MOUSECODE_BUTTON1) PORT_CHANGED(mouse_button_1_changed, 0)
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Right Mouse Button") PORT_CODE(MOUSECODE_BUTTON2) PORT_CHANGED(mouse_button_2_changed, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Left Mouse Button") PORT_CODE(MOUSECODE_BUTTON1) PORT_CHANGED_MEMBER(DEVICE_SELF, pc1512_state, mouse_button_1_changed, 0)
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Right Mouse Button") PORT_CODE(MOUSECODE_BUTTON2) PORT_CHANGED_MEMBER(DEVICE_SELF, pc1512_state, mouse_button_2_changed, 0)
 
 	PORT_START("MOUSEX")
-	PORT_BIT( 0xff, 0x00, IPT_MOUSE_X ) PORT_SENSITIVITY(100) PORT_KEYDELTA(5) PORT_MINMAX(0, 255) PORT_PLAYER(1) PORT_CHANGED(mouse_x_changed, 0)
+	PORT_BIT( 0xff, 0x00, IPT_MOUSE_X ) PORT_SENSITIVITY(100) PORT_KEYDELTA(5) PORT_MINMAX(0, 255) PORT_PLAYER(1) PORT_CHANGED_MEMBER(DEVICE_SELF, pc1512_state, mouse_x_changed, 0)
 
 	PORT_START("MOUSEY")
-	PORT_BIT( 0xff, 0x00, IPT_MOUSE_Y ) PORT_SENSITIVITY(100) PORT_KEYDELTA(5) PORT_MINMAX(0, 255) PORT_PLAYER(1) PORT_CHANGED(mouse_y_changed, 0)
+	PORT_BIT( 0xff, 0x00, IPT_MOUSE_Y ) PORT_SENSITIVITY(100) PORT_KEYDELTA(5) PORT_MINMAX(0, 255) PORT_PLAYER(1) PORT_CHANGED_MEMBER(DEVICE_SELF, pc1512_state, mouse_y_changed, 0)
 INPUT_PORTS_END
 
 
@@ -789,7 +781,7 @@ static INPUT_PORTS_START( pc1640 )
 	PORT_DIPSETTING(	0x00, "Diagnostic Mode" )
 
 	PORT_START("SW")
-	PORT_DIPNAME( 0x0f, 0x09, "Initial Display Mode" ) PORT_DIPLOCATION("SW:1,2,3,4") PORT_CONDITION("SW", 0x200, PORTCOND_EQUALS, 0x200)
+	PORT_DIPNAME( 0x0f, 0x09, "Initial Display Mode" ) PORT_DIPLOCATION("SW:1,2,3,4") PORT_CONDITION("SW", 0x200, EQUALS, 0x200)
 	PORT_DIPSETTING(	0x0b, "Internal MD, External CGA80" )
 	PORT_DIPSETTING(	0x0a, "Internal MD, External CGA40" )
 	PORT_DIPSETTING(	0x09, "Internal ECD350, External MDA/HERC" )
@@ -802,23 +794,23 @@ static INPUT_PORTS_START( pc1640 )
 	PORT_DIPSETTING(	0x02, "External MDA/HERC, Internal ECD200" )
 	PORT_DIPSETTING(	0x01, "External MDA/HERC, Internal CD80" )
 	PORT_DIPSETTING(	0x00, "External MDA/HERC, Internal CD40" )
-	PORT_DIPNAME( 0x10, 0x10, "MC6845 Mode" ) PORT_DIPLOCATION("SW:5") PORT_CONDITION("SW", 0x200, PORTCOND_EQUALS, 0x200)
+	PORT_DIPNAME( 0x10, 0x10, "MC6845 Mode" ) PORT_DIPLOCATION("SW:5") PORT_CONDITION("SW", 0x200, EQUALS, 0x200)
 	PORT_DIPSETTING(	0x10, "EGA" )
 	PORT_DIPSETTING(	0x00, "CGA/MDA/HERC" )
-	PORT_DIPNAME( 0x60, 0x00, "Font" ) PORT_DIPLOCATION("SW:6,7") PORT_CONDITION("SW", 0x300, PORTCOND_EQUALS, 0x300)
+	PORT_DIPNAME( 0x60, 0x00, "Font" ) PORT_DIPLOCATION("SW:6,7") PORT_CONDITION("SW", 0x300, EQUALS, 0x300)
 	PORT_DIPSETTING(	0x00, DEF_STR( English ) )
 	PORT_DIPSETTING(	0x60, "Danish" )
 	PORT_DIPSETTING(	0x40, "Portuguese" )
 	PORT_DIPSETTING(	0x20, "Greek" )
-	PORT_DIPNAME( 0x60, 0x60, "Default Display Mode" ) PORT_DIPLOCATION("SW:6,7") PORT_CONDITION("SW", 0x200, PORTCOND_EQUALS, 0x000)
+	PORT_DIPNAME( 0x60, 0x60, "Default Display Mode" ) PORT_DIPLOCATION("SW:6,7") PORT_CONDITION("SW", 0x200, EQUALS, 0x000)
 	PORT_DIPSETTING(	0x60, "External EGA" )
 	PORT_DIPSETTING(	0x40, "External CGA in 40 Column Mode" )
 	PORT_DIPSETTING(	0x20, "External CGA in 80 Column Mode" )
 	PORT_DIPSETTING(	0x00, "External Monochrome Adapter" )
-	PORT_DIPNAME( 0x80, 0x00, "Monitor" ) PORT_DIPLOCATION("SW:8") PORT_CONDITION("SW", 0x200, PORTCOND_EQUALS, 0x200)
+	PORT_DIPNAME( 0x80, 0x00, "Monitor" ) PORT_DIPLOCATION("SW:8") PORT_CONDITION("SW", 0x200, EQUALS, 0x200)
 	PORT_DIPSETTING(	0x80, "CD (Standard RGB)" )
 	PORT_DIPSETTING(	0x00, "ECD (Enhanced RGB)" )
-	PORT_DIPNAME( 0x100, 0x100, "Foreign Fonts" ) PORT_DIPLOCATION("SW:9") PORT_CONDITION("SW", 0x200, PORTCOND_EQUALS, 0x200)
+	PORT_DIPNAME( 0x100, 0x100, "Foreign Fonts" ) PORT_DIPLOCATION("SW:9") PORT_CONDITION("SW", 0x200, EQUALS, 0x200)
 	PORT_DIPSETTING(	 0x100, "Enabled" )
 	PORT_DIPSETTING(	 0x000, "Disabled" )
 	PORT_DIPNAME( 0x200, 0x200, "Internal Graphics Adapter" ) PORT_DIPLOCATION("SW:10")
@@ -907,24 +899,46 @@ WRITE8_MEMBER( pc1512_state::memw_w )
 	program->write_byte(page_offset + offset, data);
 }
 
-WRITE8_MEMBER( pc1512_state::dma0_w )
+READ8_MEMBER( pc1512_state::ior1_r )
+{
+	return m_bus->dack_r(1);
+}
+
+READ8_MEMBER( pc1512_state::ior2_r )
+{
+	if (m_nden)
+		return upd765_dack_r(m_fdc, 0);
+	else
+		return m_bus->dack_r(2);
+}
+
+READ8_MEMBER( pc1512_state::ior3_r )
+{
+	return m_bus->dack_r(3);
+}
+
+WRITE8_MEMBER( pc1512_state::iow0_w )
 {
 	m_dreq0 = 0;
 	i8237_dreq0_w(m_dmac, m_dreq0);
 }
 
-READ8_MEMBER( pc1512_state::fdc_dack_r )
+WRITE8_MEMBER( pc1512_state::iow1_w )
 {
-	UINT8 data = 0;
-
-	if (m_nden) data = upd765_dack_r(m_fdc, 0);
-
-	return data;
+	m_bus->dack_w(1, data);
 }
 
-WRITE8_MEMBER( pc1512_state::fdc_dack_w )
+WRITE8_MEMBER( pc1512_state::iow2_w )
 {
-	if (m_nden) upd765_dack_w(m_fdc, 0, data);
+	if (m_nden)
+		upd765_dack_w(m_fdc, 0, data);
+	else
+		m_bus->dack_w(2, data);
+}
+
+WRITE8_MEMBER( pc1512_state::iow3_w )
+{
+	m_bus->dack_w(3, data);
 }
 
 WRITE_LINE_MEMBER( pc1512_state::dack0_w )
@@ -953,10 +967,18 @@ static I8237_INTERFACE( dmac_intf )
 	DEVCB_DRIVER_LINE_MEMBER(pc1512_state, eop_w),
 	DEVCB_DRIVER_MEMBER(pc1512_state, memr_r),
 	DEVCB_DRIVER_MEMBER(pc1512_state, memw_w),
-	{ DEVCB_NULL, DEVCB_NULL, DEVCB_DRIVER_MEMBER(pc1512_state, fdc_dack_r), DEVCB_NULL },
-	{ DEVCB_DRIVER_MEMBER(pc1512_state, dma0_w), DEVCB_NULL, DEVCB_DRIVER_MEMBER(pc1512_state, fdc_dack_w), DEVCB_NULL },
-	{ DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack0_w), DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack1_w),
-	  DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack2_w), DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack3_w) }
+	{ DEVCB_NULL,
+	  DEVCB_DRIVER_MEMBER(pc1512_state, ior1_r),
+	  DEVCB_DRIVER_MEMBER(pc1512_state, ior2_r),
+	  DEVCB_DRIVER_MEMBER(pc1512_state, ior3_r) },
+	{ DEVCB_DRIVER_MEMBER(pc1512_state, iow0_w),
+	  DEVCB_DRIVER_MEMBER(pc1512_state, iow1_w),
+	  DEVCB_DRIVER_MEMBER(pc1512_state, iow2_w),
+	  DEVCB_DRIVER_MEMBER(pc1512_state, iow3_w) },
+	{ DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack0_w),
+	  DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack1_w),
+	  DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack2_w),
+	  DEVCB_DRIVER_LINE_MEMBER(pc1512_state, dack3_w) }
 };
 
 
@@ -1106,11 +1128,12 @@ static const upd765_interface fdc_intf =
 
 static const ins8250_interface uart_intf =
 {
-	XTAL_1_8432MHz,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
 	DEVCB_DEVICE_LINE(I8259A2_TAG, pic8259_ir4_w),
-	NULL,
-	NULL,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
 
@@ -1134,7 +1157,6 @@ WRITE_LINE_MEMBER( pc1512_state::ack_w )
 
 static const centronics_interface centronics_intf =
 {
-	1,
 	DEVCB_DRIVER_LINE_MEMBER(pc1512_state, ack_w),
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -1181,7 +1203,7 @@ void pc1512_state::machine_start()
 	device_set_irq_callback(m_maincpu, pc1512_irq_callback);
 
 	// set RAM size
-	size_t ram_size = ram_get_size(m_ram);
+	size_t ram_size = m_ram->size();
 
 	if (ram_size < 640 * 1024)
 	{
@@ -1336,15 +1358,15 @@ static MACHINE_CONFIG_START( pc1512, pc1512_state )
 	MCFG_PIT8253_ADD(I8253_TAG, pit_intf)
 	MCFG_MC146818_IRQ_ADD(MC146818_TAG, MC146818_STANDARD, rtc_intf)
 	MCFG_UPD765A_ADD(UPD765AC2_TAG, fdc_intf)
-	MCFG_INS8250_ADD(INS8250_TAG, uart_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, centronics_intf)
+	MCFG_INS8250_ADD(INS8250_TAG, uart_intf, XTAL_1_8432MHz)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, centronics_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(floppy_intf)
 
 	// ISA8 bus
-	MCFG_ISA8_BUS_ADD("isa", I8086_TAG, isabus_intf)
-	MCFG_ISA8_SLOT_ADD("isa", "isa1", pc1512_isa8_cards, NULL, NULL)
-	MCFG_ISA8_SLOT_ADD("isa", "isa2", pc1512_isa8_cards, NULL, NULL)
-	MCFG_ISA8_SLOT_ADD("isa", "isa3", pc1512_isa8_cards, NULL, NULL)
+	MCFG_ISA8_BUS_ADD(ISA_BUS_TAG, I8086_TAG, isabus_intf)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa1", pc1512_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa2", pc1512_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa3", pc1512_isa8_cards, NULL, NULL)
 
 	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
@@ -1380,15 +1402,15 @@ static MACHINE_CONFIG_START( pc1640, pc1640_state )
 	MCFG_PIT8253_ADD(I8253_TAG, pit_intf)
 	MCFG_MC146818_IRQ_ADD(MC146818_TAG, MC146818_STANDARD, rtc_intf)
 	MCFG_UPD765A_ADD(UPD765AC2_TAG, fdc_intf)
-	MCFG_INS8250_ADD(INS8250_TAG, uart_intf)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, centronics_intf)
+	MCFG_INS8250_ADD(INS8250_TAG, uart_intf, XTAL_1_8432MHz)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, centronics_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(floppy_intf)
 
 	// ISA8 bus
-	MCFG_ISA8_BUS_ADD("isa", I8086_TAG, isabus_intf)
-	MCFG_ISA8_SLOT_ADD("isa", "isa1", pc1512_isa8_cards, "wdxt_gen", NULL)
-	MCFG_ISA8_SLOT_ADD("isa", "isa2", pc1512_isa8_cards, NULL, NULL)
-	MCFG_ISA8_SLOT_ADD("isa", "isa3", pc1512_isa8_cards, NULL, NULL)
+	MCFG_ISA8_BUS_ADD(ISA_BUS_TAG, I8086_TAG, isabus_intf)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa1", pc1512_isa8_cards, "wdxt_gen", NULL)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa2", pc1512_isa8_cards, NULL, NULL)
+	MCFG_ISA8_SLOT_ADD(ISA_BUS_TAG, "isa3", pc1512_isa8_cards, NULL, NULL)
 
 	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
