@@ -49,7 +49,7 @@ WRITE8_MEMBER( pc8001_state::port10_w )
 	m_rtc->data_in_w(BIT(data, 3));
 
 	// centronics
-	centronics_data_w(m_centronics, 0, data);
+	m_centronics->write(space, 0, data);
 }
 
 WRITE8_MEMBER( pc8001_state::port30_w )
@@ -97,6 +97,58 @@ WRITE8_MEMBER( pc8001mk2_state::port31_w )
     */
 }
 
+READ8_MEMBER( pc8001_state::port40_r )
+{
+	/*
+
+        bit     description
+
+        0       BUSY
+        1       ACK
+        2       CMT CDIN
+        3       EXP /EXTON
+        4       RTC DATA OUT
+        5       VRTC
+        6
+        7
+
+    */
+
+	UINT8 data = 0x08;
+
+	data |= m_centronics->busy_r();
+	data |= m_centronics->ack_r() << 1;
+	data |= m_rtc->data_out_r() << 4;
+	data |= m_crtc->vrtc_r() << 5;
+
+	return data;
+}
+
+WRITE8_MEMBER( pc8001_state::port40_w )
+{
+	/*
+
+        bit     description
+
+        0       STROBE
+        1       RTC STB
+        2       RTC CLK
+        3       CRT /CLDS CLK
+        4
+        5       SPEAKER
+        6
+        7
+
+    */
+
+	m_centronics->strobe_w(BIT(data, 0));
+
+	m_rtc->clk_w(BIT(data, 2));
+	m_rtc->stb_w(BIT(data, 1));
+
+	speaker_level_w(m_speaker, BIT(data, 5));
+}
+
 /* Memory Maps */
 
 static ADDRESS_MAP_START( pc8001_mem, AS_PROGRAM, 8, pc8001_state )
@@ -122,7 +174,7 @@ static ADDRESS_MAP_START( pc8001_io, AS_IO, 8, pc8001_state )
 	AM_RANGE(0x20, 0x20) AM_MIRROR(0x0e) AM_DEVREADWRITE(I8251_TAG, i8251_device, data_r, data_w)
 	AM_RANGE(0x21, 0x21) AM_MIRROR(0x0e) AM_DEVREADWRITE(I8251_TAG, i8251_device, status_r, control_w)
 	AM_RANGE(0x30, 0x30) AM_MIRROR(0x0f) AM_WRITE(port30_w)
-	AM_RANGE(0x40, 0x40) AM_MIRROR(0x0f) AM_READ_PORT("R40") AM_WRITE_PORT("W40")
+	AM_RANGE(0x40, 0x40) AM_MIRROR(0x0f) AM_READWRITE(port40_r, port40_w)
 	AM_RANGE(0x50, 0x51) AM_DEVREADWRITE(UPD3301_TAG, upd3301_device, read, write)
 	AM_RANGE(0x60, 0x68) AM_DEVREADWRITE_LEGACY(I8257_TAG, i8257_r, i8257_w)
 //  AM_RANGE(0x70, 0x7f) unused
@@ -161,7 +213,7 @@ ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( pc8001mk2_io, AS_IO, 8, pc8001mk2_state )
 	AM_IMPORT_FROM(pc8001_io)
-	AM_RANGE(0x30, 0x30) AM_WRITE_BASE(pc8001_state, port30_w)
+	AM_RANGE(0x30, 0x30) AM_WRITE(port30_w)
 	AM_RANGE(0x31, 0x31) AM_WRITE(port31_w)
 //  AM_RANGE(0x5c, 0x5c) AM_WRITE(gram_on_w)
 //  AM_RANGE(0x5f, 0x5f) AM_WRITE(gram_off_w)
@@ -284,26 +336,6 @@ static INPUT_PORTS_START( pc8001 )
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_CODE(KEYCODE_ESC)							PORT_CHAR(UCHAR_MAMEKEY(ESC))
 
 	PORT_START("DSW1")
-
-	PORT_START("R40")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_SPECIAL ) PORT_READ_LINE_DEVICE(CENTRONICS_TAG, centronics_busy_r)
-	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_SPECIAL ) PORT_READ_LINE_DEVICE(CENTRONICS_TAG, centronics_ack_r)
-	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_SPECIAL ) // CMT CDIN
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_SPECIAL ) // EXP /EXTON
-	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_SPECIAL ) PORT_READ_LINE_DEVICE_MEMBER(UPD1990A_TAG, upd1990a_device, data_out_r)
-	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_SPECIAL ) PORT_READ_LINE_DEVICE_MEMBER(UPD3301_TAG, upd3301_device, vrtc_r)
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNUSED )
-
-	PORT_START("W40")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_OUTPUT ) PORT_WRITE_LINE_DEVICE(CENTRONICS_TAG, centronics_strobe_w)
-	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_OUTPUT ) PORT_WRITE_LINE_DEVICE_MEMBER(UPD1990A_TAG, upd1990a_device, stb_w)
-	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_OUTPUT ) PORT_WRITE_LINE_DEVICE_MEMBER(UPD1990A_TAG, upd1990a_device, clk_w)
-	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_OUTPUT ) // CRT /CLDS CLK
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_OUTPUT ) PORT_WRITE_LINE_DEVICE(SPEAKER_TAG, speaker_level_w)
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNUSED )
 INPUT_PORTS_END
 
 /* Video */
@@ -315,14 +347,7 @@ static PALETTE_INIT( pc8001 )
 void pc8001_state::video_start()
 {
 	// find memory regions
-	m_char_rom = machine().region("chargen")->base();
-}
-
-bool pc8001_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	m_crtc->update_screen(&bitmap, &cliprect);
-
-	return 0;
+	m_char_rom = memregion("chargen")->base();
 }
 
 /* uPD3301 Interface */
@@ -343,7 +368,7 @@ static UPD3301_DISPLAY_PIXELS( pc8001_display_pixels )
 		{
 			int color = BIT(data, 7) ^ rvv;
 
-			*BITMAP_ADDR16(bitmap, y, (sx * 8) + i) = color ? 7 : 0;
+			bitmap.pix16(y, (sx * 8) + i) = color ? 7 : 0;
 
 			data <<= 1;
 		}
@@ -356,8 +381,8 @@ static UPD3301_DISPLAY_PIXELS( pc8001_display_pixels )
 		{
 			int color = BIT(data, 7) ^ rvv;
 
-			*BITMAP_ADDR16(bitmap, y, (sx/2 * 16) + (i * 2)) = color ? 7 : 0;
-			*BITMAP_ADDR16(bitmap, y, (sx/2 * 16) + (i * 2) + 1) = color ? 7 : 0;
+			bitmap.pix16(y, (sx/2 * 16) + (i * 2)) = color ? 7 : 0;
+			bitmap.pix16(y, (sx/2 * 16) + (i * 2) + 1) = color ? 7 : 0;
 
 			data <<= 1;
 		}
@@ -448,7 +473,7 @@ static I8257_INTERFACE( dmac_intf )
 
 /* uPD1990A Interface */
 
-static UPD1990A_INTERFACE( pc8001_upd1990a_intf )
+static UPD1990A_INTERFACE( rtc_intf )
 {
 	DEVCB_NULL,
 	DEVCB_NULL
@@ -468,40 +493,40 @@ void pc8001_state::machine_start()
 	i8257_ready_w(m_dma, 1);
 
 	/* setup memory banking */
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
-	memory_configure_bank(machine(), "bank1", 1, 1, machine().region("n80")->base(), 0);
+	membank("bank1")->configure_entry(1, memregion("n80")->base());
 	program->install_read_bank(0x0000, 0x5fff, "bank1");
 	program->unmap_write(0x0000, 0x5fff);
 
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 16*1024:
-		memory_configure_bank(machine(), "bank3", 0, 1, ram, 0);
+		membank("bank3")->configure_entry(0, ram);
 		program->unmap_readwrite(0x6000, 0xbfff);
 		program->unmap_readwrite(0x8000, 0xbfff);
 		program->install_readwrite_bank(0xc000, 0xffff, "bank3");
 		break;
 
 	case 32*1024:
-		memory_configure_bank(machine(), "bank3", 0, 1, ram, 0);
+		membank("bank3")->configure_entry(0, ram);
 		program->unmap_readwrite(0x6000, 0xbfff);
 		program->install_readwrite_bank(0x8000, 0xffff, "bank3");
 		break;
 
 	case 64*1024:
-		memory_configure_bank(machine(), "bank1", 0, 1, ram, 0);
-		memory_configure_bank(machine(), "bank2", 0, 1, ram + 0x6000, 0);
-		memory_configure_bank(machine(), "bank3", 0, 1, ram + 0x8000, 0);
+		membank("bank1")->configure_entry(0, ram);
+		membank("bank2")->configure_entry(0, ram + 0x6000);
+		membank("bank3")->configure_entry(0, ram + 0x8000);
 		program->install_readwrite_bank(0x0000, 0x5fff, "bank1");
 		program->install_readwrite_bank(0x6000, 0xbfff, "bank2");
 		program->install_readwrite_bank(0x8000, 0xffff, "bank3");
-		memory_set_bank(machine(), "bank2", 0);
+		membank("bank2")->set_entry(0);
 		break;
 	}
 
-	memory_set_bank(machine(), "bank1", 1);
-	memory_set_bank(machine(), "bank3", 0);
+	membank("bank1")->set_entry(1);
+	membank("bank3")->set_entry(0);
 
 	/* register for state saving */
 	save_item(NAME(m_width80));
@@ -531,7 +556,7 @@ static MACHINE_CONFIG_START( pc8001, pc8001_state )
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DEVICE(UPD3301_TAG, upd3301_device, screen_update)
 	MCFG_SCREEN_SIZE(640, 220)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 200-1)
 
@@ -547,10 +572,10 @@ static MACHINE_CONFIG_START( pc8001, pc8001_state )
 	MCFG_I8251_ADD(I8251_TAG, uart_intf)
 	MCFG_I8255A_ADD(I8255A_TAG, ppi_intf)
 	MCFG_I8257_ADD(I8257_TAG, 4000000, dmac_intf)
-	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, pc8001_upd1990a_intf)
+	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, rtc_intf)
 	MCFG_UPD3301_ADD(UPD3301_TAG, 14318180, pc8001_upd3301_intf)
 
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, pc8001_cassette_interface)
 
 	MCFG_RAM_ADD(RAM_TAG)
@@ -568,7 +593,7 @@ static MACHINE_CONFIG_START( pc8001mk2, pc8001mk2_state )
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DEVICE(UPD3301_TAG, upd3301_device, screen_update)
 	MCFG_SCREEN_SIZE(640, 220)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 200-1)
 
@@ -584,10 +609,10 @@ static MACHINE_CONFIG_START( pc8001mk2, pc8001mk2_state )
 	MCFG_I8251_ADD(I8251_TAG, uart_intf)
 	MCFG_I8255A_ADD(I8255A_TAG, ppi_intf)
 	MCFG_I8257_ADD(I8257_TAG, 4000000, dmac_intf)
-	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, pc8001_upd1990a_intf)
+	MCFG_UPD1990A_ADD(UPD1990A_TAG, XTAL_32_768kHz, rtc_intf)
 	MCFG_UPD3301_ADD(UPD3301_TAG, 14318180, pc8001_upd3301_intf)
 
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, pc8001_cassette_interface)
 
 	MCFG_RAM_ADD(RAM_TAG)
