@@ -27,6 +27,7 @@
 #include "machine/pit8253.h"
 #include "machine/upd765.h"
 #include "sound/2203intf.h"
+#include "formats/basicdsk.h"
 
 typedef struct
 {
@@ -49,9 +50,10 @@ class pc88va_state : public driver_device
 {
 public:
 	pc88va_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_palram(*this, "palram"){ }
 
-	UINT16 *m_palram;
+	required_shared_ptr<UINT16> m_palram;
 	UINT16 m_bank_reg;
 	UINT16 m_screen_ctrl_reg;
 	UINT8 m_timer3_io_reg;
@@ -70,6 +72,32 @@ public:
 	UINT8 m_i8255_1_pc;
 	UINT8 m_fdc_mode;
 	UINT8 m_fdc_irq_opcode;
+	DECLARE_READ16_MEMBER(sys_mem_r);
+	DECLARE_WRITE16_MEMBER(sys_mem_w);
+	DECLARE_READ8_MEMBER(idp_status_r);
+	DECLARE_WRITE8_MEMBER(idp_command_w);
+	DECLARE_WRITE8_MEMBER(idp_param_w);
+	DECLARE_WRITE16_MEMBER(palette_ram_w);
+	DECLARE_READ16_MEMBER(sys_port4_r);
+	DECLARE_READ16_MEMBER(bios_bank_r);
+	DECLARE_WRITE16_MEMBER(bios_bank_w);
+	DECLARE_READ8_MEMBER(rom_bank_r);
+	DECLARE_READ8_MEMBER(key_r);
+	DECLARE_WRITE16_MEMBER(backupram_wp_1_w);
+	DECLARE_WRITE16_MEMBER(backupram_wp_0_w);
+	DECLARE_READ8_MEMBER(hdd_status_r);
+	DECLARE_WRITE8_MEMBER(upd765_mc_w);
+	DECLARE_READ8_MEMBER(upd765_tc_r);
+	DECLARE_READ8_MEMBER(pc88va_fdc_r);
+	DECLARE_WRITE8_MEMBER(pc88va_fdc_w);
+	DECLARE_READ16_MEMBER(sysop_r);
+	DECLARE_READ16_MEMBER(screen_ctrl_r);
+	DECLARE_WRITE16_MEMBER(screen_ctrl_w);
+	DECLARE_WRITE8_MEMBER(timer3_ctrl_reg_w);
+	DECLARE_WRITE16_MEMBER(video_pri_w);
+	DECLARE_READ8_MEMBER(backupram_dsw_r);
+	DECLARE_WRITE8_MEMBER(sys_port1_w);
+	DECLARE_WRITE8_MEMBER(fdc_irq_vector_w);
 };
 
 
@@ -79,10 +107,10 @@ static VIDEO_START( pc88va )
 
 }
 
-static void draw_sprites(running_machine &machine, bitmap_t *bitmap, const rectangle *cliprect)
+static void draw_sprites(running_machine &machine, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	pc88va_state *state = machine.driver_data<pc88va_state>();
-	UINT16 *tvram = (UINT16 *)machine.region("tvram")->base();
+	UINT16 *tvram = (UINT16 *)(*state->memregion("tvram"));
 	int offs,i;
 
 	offs = state->m_tsp.spr_offset;
@@ -140,7 +168,7 @@ static void draw_sprites(running_machine &machine, bitmap_t *bitmap, const recta
 						pen = pen & 1 ? fg_col : (bc) ? 8 : -1;
 
 						if(pen != -1) //transparent pen
-							*BITMAP_ADDR32(bitmap, yp+y_i, xp+x_i+(x_s)) = machine.pens[pen];
+							bitmap.pix32(yp+y_i, xp+x_i+(x_s)) = machine.pens[pen];
 					}
 					spr_count+=2;
 				}
@@ -165,7 +193,7 @@ static void draw_sprites(running_machine &machine, bitmap_t *bitmap, const recta
 						pen = (BITSWAP16(tvram[(spda+spr_count) / 2],7,6,5,4,3,2,1,0,15,14,13,12,11,10,9,8)) >> (16-(x_s*8)) & 0xf;
 
 						//if(bc != -1) //transparent pen
-						*BITMAP_ADDR32(bitmap, yp+y_i, xp+x_i+(x_s)) = machine.pens[pen];
+						bitmap.pix32(yp+y_i, xp+x_i+(x_s)) = machine.pens[pen];
 					}
 					spr_count+=2;
 				}
@@ -189,11 +217,11 @@ static UINT32 calc_kanji_rom_addr(UINT8 jis1,UINT8 jis2,int x,int y)
 	return 0;
 }
 
-static void draw_text(running_machine &machine, bitmap_t *bitmap, const rectangle *cliprect)
+static void draw_text(running_machine &machine, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	pc88va_state *state = machine.driver_data<pc88va_state>();
-	UINT8 *tvram = machine.region("tvram")->base();
-	UINT8 *kanji = machine.region("kanji")->base();
+	UINT8 *tvram = machine.root_device().memregion("tvram")->base();
+	UINT8 *kanji = state->memregion("kanji")->base();
 	int xi,yi;
 	int x,y;
 	int res_x,res_y;
@@ -347,7 +375,7 @@ static void draw_text(running_machine &machine, bitmap_t *bitmap, const rectangl
 					res_x = x*8+xi;
 					res_y = y*16+yi;
 
-					if((res_x)>machine.primary_screen->visible_area().max_x || (res_y)>machine.primary_screen->visible_area().max_y)
+					if(!cliprect.contains(res_x, res_y))
 						continue;
 
 					pen = kanji[((yi*2)+lr_half_gfx)+tile_num] >> (7-xi) & 1;
@@ -360,7 +388,7 @@ static void draw_text(running_machine &machine, bitmap_t *bitmap, const rectangl
 					if(secret) { pen = 0; } //hide text
 
 					if(pen != -1) //transparent
-						*BITMAP_ADDR32(bitmap, res_y, res_x) = machine.pens[pen];
+						bitmap.pix32(res_y, res_x) = machine.pens[pen];
 				}
 			}
 
@@ -370,12 +398,12 @@ static void draw_text(running_machine &machine, bitmap_t *bitmap, const rectangl
 	}
 }
 
-static SCREEN_UPDATE( pc88va )
+static SCREEN_UPDATE_RGB32( pc88va )
 {
-	pc88va_state *state = screen->machine().driver_data<pc88va_state>();
+	pc88va_state *state = screen.machine().driver_data<pc88va_state>();
 	UINT8 pri,cur_pri_lv;
 	UINT32 screen_pri;
-	bitmap_fill(bitmap, cliprect, 0);
+	bitmap.fill(0, cliprect);
 
 	if(state->m_tsp.disp_on == 0) // don't bother if we are under DSPOFF command
 		return 0;
@@ -415,8 +443,8 @@ static SCREEN_UPDATE( pc88va )
 			{
 				switch(cur_pri_lv & 3) // (palette color mode)
 				{
-					case 0: draw_text(screen->machine(),bitmap,cliprect); break;
-					case 1: if(state->m_tsp.spr_on) { draw_sprites(screen->machine(),bitmap,cliprect); } break;
+					case 0: draw_text(screen.machine(),bitmap,cliprect); break;
+					case 1: if(state->m_tsp.spr_on) { draw_sprites(screen.machine(),bitmap,cliprect); } break;
 					case 2: /* A = graphic 0 */ break;
 					case 3: /* B = graphic 1 */ break;
 				}
@@ -427,16 +455,15 @@ static SCREEN_UPDATE( pc88va )
 	return 0;
 }
 
-static READ16_HANDLER( sys_mem_r )
+READ16_MEMBER(pc88va_state::sys_mem_r)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	switch((state->m_bank_reg & 0xf00) >> 8)
+	switch((m_bank_reg & 0xf00) >> 8)
 	{
 		case 0: // select bus slot
 			return 0xffff;
 		case 1: // TVRAM
 		{
-			UINT16 *tvram = (UINT16 *)space->machine().region("tvram")->base();
+			UINT16 *tvram = (UINT16 *)(*machine().root_device().memregion("tvram"));
 
 			if(((offset*2) & 0x30000) == 0)
 				return tvram[offset];
@@ -445,17 +472,17 @@ static READ16_HANDLER( sys_mem_r )
 		}
 		case 4:
 		{
-			UINT16 *gvram = (UINT16 *)space->machine().region("gvram")->base();
+			UINT16 *gvram = (UINT16 *)(*machine().root_device().memregion("gvram"));
 
 			return gvram[offset];
 		}
 		case 8: // kanji ROM
 		case 9:
 		{
-			UINT16 *knj_ram = (UINT16 *)space->machine().region("kanji")->base();
+			UINT16 *knj_ram = (UINT16 *)(*machine().root_device().memregion("kanji"));
 			UINT32 knj_offset;
 
-			knj_offset = (offset + (((state->m_bank_reg & 0x100) >> 8)*0x20000));
+			knj_offset = (offset + (((m_bank_reg & 0x100) >> 8)*0x20000));
 
 			/* 0x00000 - 0x3ffff Kanji ROM 1*/
 			/* 0x40000 - 0x4ffff Kanji ROM 2*/
@@ -468,10 +495,10 @@ static READ16_HANDLER( sys_mem_r )
 		case 0xc: // Dictionary ROM
 		case 0xd:
 		{
-			UINT16 *dic_rom = (UINT16 *)space->machine().region("dictionary")->base();
+			UINT16 *dic_rom = (UINT16 *)(*machine().root_device().memregion("dictionary"));
 			UINT32 dic_offset;
 
-			dic_offset = (offset + (((state->m_bank_reg & 0x100) >> 8)*0x20000));
+			dic_offset = (offset + (((m_bank_reg & 0x100) >> 8)*0x20000));
 
 			return dic_rom[dic_offset];
 		}
@@ -480,16 +507,15 @@ static READ16_HANDLER( sys_mem_r )
 	return 0xffff;
 }
 
-static WRITE16_HANDLER( sys_mem_w )
+WRITE16_MEMBER(pc88va_state::sys_mem_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	switch((state->m_bank_reg & 0xf00) >> 8)
+	switch((m_bank_reg & 0xf00) >> 8)
 	{
 		case 0: // select bus slot
 			break;
 		case 1: // TVRAM
 		{
-			UINT16 *tvram = (UINT16 *)space->machine().region("tvram")->base();
+			UINT16 *tvram = (UINT16 *)(*machine().root_device().memregion("tvram"));
 
 			if(((offset*2) & 0x30000) == 0)
 				COMBINE_DATA(&tvram[offset]);
@@ -497,7 +523,7 @@ static WRITE16_HANDLER( sys_mem_w )
 		break;
 		case 4: // TVRAM
 		{
-			UINT16 *gvram = (UINT16 *)space->machine().region("gvram")->base();
+			UINT16 *gvram = (UINT16 *)(*machine().root_device().memregion("gvram"));
 
 			COMBINE_DATA(&gvram[offset]);
 		}
@@ -505,16 +531,16 @@ static WRITE16_HANDLER( sys_mem_w )
 		case 8: // kanji ROM, backup RAM at 0xb0000 - 0xb3fff
 		case 9:
 		{
-			UINT16 *knj_ram = (UINT16 *)space->machine().region("kanji")->base();
+			UINT16 *knj_ram = (UINT16 *)(*machine().root_device().memregion("kanji"));
 			UINT32 knj_offset;
 
-			knj_offset = ((offset) + (((state->m_bank_reg & 0x100) >> 8)*0x20000));
+			knj_offset = ((offset) + (((m_bank_reg & 0x100) >> 8)*0x20000));
 
 			if(knj_offset >= 0x50000/2 && knj_offset <= 0x53fff/2) // TODO: there's an area that can be write protected
 			{
 				COMBINE_DATA(&knj_ram[knj_offset]);
-				gfx_element_mark_dirty(space->machine().gfx[0], (knj_offset * 2) / 8);
-				gfx_element_mark_dirty(space->machine().gfx[1], (knj_offset * 2) / 32);
+				gfx_element_mark_dirty(machine().gfx[0], (knj_offset * 2) / 8);
+				gfx_element_mark_dirty(machine().gfx[1], (knj_offset * 2) / 32);
 			}
 		}
 		break;
@@ -527,7 +553,7 @@ static WRITE16_HANDLER( sys_mem_w )
 	}
 }
 
-static ADDRESS_MAP_START( pc88va_map, AS_PROGRAM, 16 )
+static ADDRESS_MAP_START( pc88va_map, AS_PROGRAM, 16, pc88va_state )
 	AM_RANGE(0x00000, 0x7ffff) AM_RAM
 //  AM_RANGE(0x80000, 0x9ffff) AM_RAM // EMM
 	AM_RANGE(0xa0000, 0xdffff) AM_READWRITE(sys_mem_r,sys_mem_w)
@@ -535,7 +561,7 @@ static ADDRESS_MAP_START( pc88va_map, AS_PROGRAM, 16 )
 	AM_RANGE(0xf0000, 0xfffff) AM_ROMBANK("rom10_bank")
 ADDRESS_MAP_END
 
-static READ8_HANDLER( idp_status_r )
+READ8_MEMBER(pc88va_state::idp_status_r)
 {
 /*
     x--- ---- LP   Light-pen signal detection (with VA use failure)
@@ -565,46 +591,45 @@ static READ8_HANDLER( idp_status_r )
 #define SPRSW  0x85
 #define SPROV  0x81
 
-static WRITE8_HANDLER( idp_command_w )
+WRITE8_MEMBER(pc88va_state::idp_command_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
 	switch(data)
 	{
 		/* 0x10 - SYNC: sets CRTC values */
-		case SYNC:   state->m_cmd = SYNC;  state->m_buf_size = 14; state->m_buf_index = 0; break;
+		case SYNC:   m_cmd = SYNC;  m_buf_size = 14; m_buf_index = 0; break;
 
 		/* 0x12 - DSPON: set DiSPlay ON and set up tvram table vreg */
-		case DSPON:  state->m_cmd = DSPON; state->m_buf_size = 3;  state->m_buf_index = 0; break;
+		case DSPON:  m_cmd = DSPON; m_buf_size = 3;  m_buf_index = 0; break;
 
 		/* 0x13 - DSPOFF: set DiSPlay OFF */
-		case DSPOFF: state->m_cmd = DSPOFF; state->m_tsp.disp_on = 0; break;
+		case DSPOFF: m_cmd = DSPOFF; m_tsp.disp_on = 0; break;
 
 		/* 0x14 - DSPDEF: set DiSPlay DEFinitions */
-		case DSPDEF: state->m_cmd = DSPDEF; state->m_buf_size = 6; state->m_buf_index = 0; break;
+		case DSPDEF: m_cmd = DSPDEF; m_buf_size = 6; m_buf_index = 0; break;
 
 		/* 0x15 - CURDEF: set CURsor DEFinition */
-		case CURDEF: state->m_cmd = CURDEF; state->m_buf_size = 1; state->m_buf_index = 0; break;
+		case CURDEF: m_cmd = CURDEF; m_buf_size = 1; m_buf_index = 0; break;
 
 		/* 0x16 - ACTSCR: ??? */
-		case ACTSCR: state->m_cmd = ACTSCR; state->m_buf_size = 1; state->m_buf_index = 0; break;
+		case ACTSCR: m_cmd = ACTSCR; m_buf_size = 1; m_buf_index = 0; break;
 
 		/* 0x15 - CURS: set CURSor position */
-		case CURS:   state->m_cmd = CURS;   state->m_buf_size = 4; state->m_buf_index = 0; break;
+		case CURS:   m_cmd = CURS;   m_buf_size = 4; m_buf_index = 0; break;
 
 		/* 0x8c - EMUL: set 3301 EMULation */
-		case EMUL:   state->m_cmd = EMUL;   state->m_buf_size = 4; state->m_buf_index = 0; break;
+		case EMUL:   m_cmd = EMUL;   m_buf_size = 4; m_buf_index = 0; break;
 
 		/* 0x88 - EXIT: ??? */
-		case EXIT:   state->m_cmd = EXIT; break;
+		case EXIT:   m_cmd = EXIT; break;
 
 		/* 0x82 - SPRON: set SPRite ON */
-		case SPRON:  state->m_cmd = SPRON;  state->m_buf_size = 3; state->m_buf_index = 0; break;
+		case SPRON:  m_cmd = SPRON;  m_buf_size = 3; m_buf_index = 0; break;
 
 		/* 0x83 - SPROFF: set SPRite OFF */
-		case SPROFF: state->m_cmd = SPROFF; state->m_tsp.spr_on = 0; break;
+		case SPROFF: m_cmd = SPROFF; m_tsp.spr_on = 0; break;
 
 		/* 0x85 - SPRSW: ??? */
-		case SPRSW:  state->m_cmd = SPRSW;  state->m_buf_size = 1; state->m_buf_index = 0; break;
+		case SPRSW:  m_cmd = SPRSW;  m_buf_size = 1; m_buf_index = 0; break;
 
 		/* 0x81 - SPROV: set SPRite OVerflow information */
 		/*
@@ -612,10 +637,10 @@ static WRITE8_HANDLER( idp_command_w )
         --x- ---- Sprite Collision flag
         ---x xxxx First sprite that caused Sprite Over event
         */
-		case SPROV:  state->m_cmd = SPROV; /* TODO: where it returns the info? */ break;
+		case SPROV:  m_cmd = SPROV; /* TODO: where it returns the info? */ break;
 
 		/* TODO: 0x89 shouldn't trigger, should be one of the above commands */
-		default:   state->m_cmd = 0x00; printf("PC=%05x: Unknown IDP %02x cmd set\n",cpu_get_pc(&space->device()),data); break;
+		default:   m_cmd = 0x00; printf("PC=%05x: Unknown IDP %02x cmd set\n",cpu_get_pc(&space.device()),data); break;
 	}
 }
 
@@ -661,10 +686,7 @@ static void execute_sync_cmd(running_machine &machine)
 	x_vis_area = state->m_buf_ram[4] * 4;
 	y_vis_area = (state->m_buf_ram[0xa])|((state->m_buf_ram[0xb] & 0x40)<<2);
 
-	visarea.min_x = 0;
-	visarea.min_y = 0;
-	visarea.max_x = x_vis_area - 1;
-	visarea.max_y = y_vis_area - 1;
+	visarea.set(0, x_vis_area - 1, 0, y_vis_area - 1);
 
 	//if(y_vis_area == 400)
 	//  refresh = HZ_TO_ATTOSECONDS(24800) * x_vis_area * y_vis_area; //24.8 KHz
@@ -788,29 +810,28 @@ static void execute_sprsw_cmd(running_machine &machine)
 	tsp_sprite_enable(machine, 0xa0000 + state->m_tsp.spr_offset + (state->m_buf_ram[0] & 0xf8), (state->m_buf_ram[0] & 2) << 8);
 }
 
-static WRITE8_HANDLER( idp_param_w )
+WRITE8_MEMBER(pc88va_state::idp_param_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	if(state->m_cmd == DSPOFF || state->m_cmd == EXIT || state->m_cmd == SPROFF || state->m_cmd == SPROV) // no param commands
+	if(m_cmd == DSPOFF || m_cmd == EXIT || m_cmd == SPROFF || m_cmd == SPROV) // no param commands
 		return;
 
-	state->m_buf_ram[state->m_buf_index] = data;
-	state->m_buf_index++;
+	m_buf_ram[m_buf_index] = data;
+	m_buf_index++;
 
-	if(state->m_buf_index >= state->m_buf_size)
+	if(m_buf_index >= m_buf_size)
 	{
-		state->m_buf_index = 0;
-		switch(state->m_cmd)
+		m_buf_index = 0;
+		switch(m_cmd)
 		{
-			case SYNC:		execute_sync_cmd(space->machine());	break;
-			case DSPON: 	execute_dspon_cmd(space->machine());	break;
-			case DSPDEF:	execute_dspdef_cmd(space->machine()); break;
-			case CURDEF:	execute_curdef_cmd(space->machine()); break;
-			case ACTSCR:	execute_actscr_cmd(space->machine()); break;
-			case CURS:		execute_curs_cmd(space->machine());   break;
-			case EMUL:		execute_emul_cmd(space->machine());   break;
-			case SPRON:		execute_spron_cmd(space->machine());  break;
-			case SPRSW:		execute_sprsw_cmd(space->machine());	break;
+			case SYNC:		execute_sync_cmd(machine());	break;
+			case DSPON: 	execute_dspon_cmd(machine());	break;
+			case DSPDEF:	execute_dspdef_cmd(machine()); break;
+			case CURDEF:	execute_curdef_cmd(machine()); break;
+			case ACTSCR:	execute_actscr_cmd(machine()); break;
+			case CURS:		execute_curs_cmd(machine());   break;
+			case EMUL:		execute_emul_cmd(machine());   break;
+			case SPRON:		execute_spron_cmd(machine());  break;
+			case SPRSW:		execute_sprsw_cmd(machine());	break;
 
 			default:
 				//printf("%02x\n",data);
@@ -819,38 +840,35 @@ static WRITE8_HANDLER( idp_param_w )
 	}
 }
 
-static WRITE16_HANDLER( palette_ram_w )
+WRITE16_MEMBER(pc88va_state::palette_ram_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
 	int r,g,b;
-	COMBINE_DATA(&state->m_palram[offset]);
+	COMBINE_DATA(&m_palram[offset]);
 
-	b = (state->m_palram[offset] & 0x001e) >> 1;
-	r = (state->m_palram[offset] & 0x03c0) >> 6;
-	g = (state->m_palram[offset] & 0x7800) >> 11;
+	b = (m_palram[offset] & 0x001e) >> 1;
+	r = (m_palram[offset] & 0x03c0) >> 6;
+	g = (m_palram[offset] & 0x7800) >> 11;
 
-	palette_set_color_rgb(space->machine(),offset,pal4bit(r),pal4bit(g),pal4bit(b));
+	palette_set_color_rgb(machine(),offset,pal4bit(r),pal4bit(g),pal4bit(b));
 }
 
-static READ16_HANDLER( sys_port4_r )
+READ16_MEMBER(pc88va_state::sys_port4_r)
 {
 	UINT8 vrtc,sw1;
-	vrtc = (space->machine().primary_screen->vpos() < 200) ? 0x20 : 0x00; // vblank
+	vrtc = (machine().primary_screen->vpos() < 200) ? 0x20 : 0x00; // vblank
 
-	sw1 = (input_port_read(space->machine(), "DSW") & 1) ? 2 : 0;
+	sw1 = (ioport("DSW")->read() & 1) ? 2 : 0;
 
 	return vrtc | sw1 | 0xc0;
 }
 
-static READ16_HANDLER( bios_bank_r )
+READ16_MEMBER(pc88va_state::bios_bank_r)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	return state->m_bank_reg;
+	return m_bank_reg;
 }
 
-static WRITE16_HANDLER( bios_bank_w )
+WRITE16_MEMBER(pc88va_state::bios_bank_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
 	/*
     -x-- ---- ---- ---- SMM (compatibility mode)
     ---x ---- ---- ---- GMSP (VRAM drawing Mode)
@@ -859,35 +877,35 @@ static WRITE16_HANDLER( bios_bank_w )
     ---- ---- ---- xxxx RBC0 (0xe0000 - 0xeffff ROM bank)
     */
 	if ((mem_mask&0xffff) == 0xffff)
-		state->m_bank_reg = data;
+		m_bank_reg = data;
 	else if ((mem_mask & 0xffff) == 0xff00)
-		state->m_bank_reg = (data & 0xff00) | (state->m_bank_reg & 0x00ff);
+		m_bank_reg = (data & 0xff00) | (m_bank_reg & 0x00ff);
 	else if ((mem_mask & 0xffff) == 0x00ff)
-		state->m_bank_reg = (data & 0x00ff) | (state->m_bank_reg & 0xff00);
+		m_bank_reg = (data & 0x00ff) | (m_bank_reg & 0xff00);
 
 
 	/* RBC1 */
 	{
-		UINT8 *ROM10 = space->machine().region("rom10")->base();
+		UINT8 *ROM10 = memregion("rom10")->base();
 
-		if((state->m_bank_reg & 0xe0) == 0x00)
-			memory_set_bankptr(space->machine(), "rom10_bank", &ROM10[(state->m_bank_reg & 0x10) ? 0x10000 : 0x00000]);
+		if((m_bank_reg & 0xe0) == 0x00)
+			membank("rom10_bank")->set_base(&ROM10[(m_bank_reg & 0x10) ? 0x10000 : 0x00000]);
 	}
 
 	/* RBC0 */
 	{
-		UINT8 *ROM00 = space->machine().region("rom00")->base();
+		UINT8 *ROM00 = memregion("rom00")->base();
 
-		memory_set_bankptr(space->machine(), "rom00_bank", &ROM00[(state->m_bank_reg & 0xf)*0x10000]); // TODO: docs says that only 0 - 5 are used, dunno why ...
+		membank("rom00_bank")->set_base(&ROM00[(m_bank_reg & 0xf)*0x10000]); // TODO: docs says that only 0 - 5 are used, dunno why ...
 	}
 }
 
-static READ8_HANDLER( rom_bank_r )
+READ8_MEMBER(pc88va_state::rom_bank_r)
 {
 	return 0xff; // bit 7 low is va91 rom bank status
 }
 
-static READ8_HANDLER( key_r )
+READ8_MEMBER(pc88va_state::key_r)
 {
 	// note row C bit 2 does something at POST ... some kind of test mode?
 	static const char *const keynames[] = { "KEY0", "KEY1", "KEY2", "KEY3",
@@ -895,32 +913,30 @@ static READ8_HANDLER( key_r )
 	                                        "KEY8", "KEY9", "KEYA", "KEYB",
 	                                        "KEYC", "KEYD", "KEYE", "KEYF" };
 
-	return input_port_read(space->machine(), keynames[offset]);
+	return ioport(keynames[offset])->read();
 }
 
-static WRITE16_HANDLER( backupram_wp_1_w )
+WRITE16_MEMBER(pc88va_state::backupram_wp_1_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	state->m_backupram_wp = 1;
+	m_backupram_wp = 1;
 }
 
-static WRITE16_HANDLER( backupram_wp_0_w )
+WRITE16_MEMBER(pc88va_state::backupram_wp_0_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	state->m_backupram_wp = 0;
+	m_backupram_wp = 0;
 }
 
-static READ8_HANDLER( hdd_status_r )
+READ8_MEMBER(pc88va_state::hdd_status_r)
 {
 	return 0x20;
 }
 
-static WRITE8_HANDLER( upd765_mc_w )
+WRITE8_MEMBER(pc88va_state::upd765_mc_w)
 {
-	floppy_mon_w(floppy_get_device(space->machine(), 0), (data & 1) ? CLEAR_LINE : ASSERT_LINE);
-	floppy_mon_w(floppy_get_device(space->machine(), 1), (data & 2) ? CLEAR_LINE : ASSERT_LINE);
-	floppy_drive_set_ready_state(floppy_get_device(space->machine(), 0), (data & 1), 0);
-	floppy_drive_set_ready_state(floppy_get_device(space->machine(), 1), (data & 2), 0);
+	floppy_mon_w(floppy_get_device(machine(), 0), (data & 1) ? CLEAR_LINE : ASSERT_LINE);
+	floppy_mon_w(floppy_get_device(machine(), 1), (data & 2) ? CLEAR_LINE : ASSERT_LINE);
+	floppy_drive_set_ready_state(floppy_get_device(machine(), 0), (data & 1), 0);
+	floppy_drive_set_ready_state(floppy_get_device(machine(), 1), (data & 2), 0);
 }
 
 static TIMER_CALLBACK( pc8801fd_upd765_tc_to_zero )
@@ -930,16 +946,15 @@ static TIMER_CALLBACK( pc8801fd_upd765_tc_to_zero )
 	upd765_tc_w(machine.device("upd765"), 0);
 }
 
-static READ8_HANDLER( upd765_tc_r )
+READ8_MEMBER(pc88va_state::upd765_tc_r)
 {
-	//pc88va_state *state = space->machine().driver_data<pc88va_state>();
 
-	upd765_tc_w(space->machine().device("upd765"), 1);
-	space->machine().scheduler().timer_set(attotime::from_usec(500), FUNC(pc8801fd_upd765_tc_to_zero));
+	upd765_tc_w(machine().device("upd765"), 1);
+	machine().scheduler().timer_set(attotime::from_usec(500), FUNC(pc8801fd_upd765_tc_to_zero));
 	return 0;
 }
 
-static READ8_HANDLER( pc88va_fdc_r )
+READ8_MEMBER(pc88va_state::pc88va_fdc_r)
 {
 	switch(offset*2)
 	{
@@ -949,24 +964,23 @@ static READ8_HANDLER( pc88va_fdc_r )
 		/* ---x ---- RDY: (0) Busy (1) Ready */
 		case 0x06: // FDC control port 2
 			return 0;
-		case 0x08: return upd765_status_r(space->machine().device("upd765"), 0);
-		case 0x0a: return upd765_data_r(space->machine().device("upd765"), 0);
+		case 0x08: return upd765_status_r(machine().device("upd765"), 0);
+		case 0x0a: return upd765_data_r(machine().device("upd765"), 0);
 	}
 
 	return 0xff;
 }
 
-static WRITE8_HANDLER( pc88va_fdc_w )
+WRITE8_MEMBER(pc88va_state::pc88va_fdc_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
 	switch(offset*2)
 	{
 		/*
         ---- ---x MODE: FDC op mode (0) Intelligent (1) DMA
         */
 		case 0x00: // FDC mode register
-			state->m_fdc_mode = data & 1;
-			cputag_set_input_line(space->machine(), "fdccpu", INPUT_LINE_HALT, (state->m_fdc_mode) ? ASSERT_LINE : CLEAR_LINE);
+			m_fdc_mode = data & 1;
+			cputag_set_input_line(machine(), "fdccpu", INPUT_LINE_HALT, (m_fdc_mode) ? ASSERT_LINE : CLEAR_LINE);
 			break;
 		/*
         --x- ---- CLK: FDC clock selection (0) 4.8MHz (1) 8 MHz
@@ -992,30 +1006,28 @@ static WRITE8_HANDLER( pc88va_fdc_w )
 			printf("%02x\n",data);
 			break; // FDC control port 2
 		case 0x08: break; // UPD765 status
-		case 0x0a: upd765_data_w(space->machine().device("upd765"), 0,data); break;
+		case 0x0a: upd765_data_w(machine().device("upd765"), 0,data); break;
 	}
 }
 
 
-static READ16_HANDLER( sysop_r )
+READ16_MEMBER(pc88va_state::sysop_r)
 {
 	UINT8 sys_op;
 
-	sys_op = input_port_read(space->machine(), "SYSOP_SW") & 3;
+	sys_op = ioport("SYSOP_SW")->read() & 3;
 
 	return 0xfffc | sys_op; // docs says all the other bits are high
 }
 
-static READ16_HANDLER( screen_ctrl_r )
+READ16_MEMBER(pc88va_state::screen_ctrl_r)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	return state->m_screen_ctrl_reg;
+	return m_screen_ctrl_reg;
 }
 
-static WRITE16_HANDLER( screen_ctrl_w )
+WRITE16_MEMBER(pc88va_state::screen_ctrl_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	state->m_screen_ctrl_reg = data;
+	m_screen_ctrl_reg = data;
 }
 
 static TIMER_CALLBACK( t3_mouse_callback )
@@ -1028,33 +1040,31 @@ static TIMER_CALLBACK( t3_mouse_callback )
 	}
 }
 
-static WRITE8_HANDLER( timer3_ctrl_reg_w )
+WRITE8_MEMBER(pc88va_state::timer3_ctrl_reg_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
 	/*
     x--- ---- MINTEN (TCU irq enable)
     ---- --xx general purpose timer 3 interval (120, 60, 30, 15)
     */
-	state->m_timer3_io_reg = data;
+	m_timer3_io_reg = data;
 
 	if(data & 0x80)
-		state->m_t3_mouse_timer->adjust(attotime::from_hz(120 >> (state->m_timer3_io_reg & 3)));
+		m_t3_mouse_timer->adjust(attotime::from_hz(120 >> (m_timer3_io_reg & 3)));
 	else
 	{
-		pic8259_ir5_w(space->machine().device("pic8259_slave"), 0);
-		state->m_t3_mouse_timer->adjust(attotime::never);
+		pic8259_ir5_w(machine().device("pic8259_slave"), 0);
+		m_t3_mouse_timer->adjust(attotime::never);
 	}
 }
 
-static WRITE16_HANDLER( video_pri_w )
+WRITE16_MEMBER(pc88va_state::video_pri_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	COMBINE_DATA(&state->m_video_pri_reg[offset]);
+	COMBINE_DATA(&m_video_pri_reg[offset]);
 }
 
-static READ8_HANDLER( backupram_dsw_r )
+READ8_MEMBER(pc88va_state::backupram_dsw_r)
 {
-	UINT16 *knj_ram = (UINT16 *)space->machine().region("kanji")->base();
+	UINT16 *knj_ram = (UINT16 *)(*machine().root_device().memregion("kanji"));
 
 	if(offset == 0)
 		return knj_ram[(0x50000 + 0x1fc2) / 2] & 0xff;
@@ -1062,12 +1072,12 @@ static READ8_HANDLER( backupram_dsw_r )
 	return knj_ram[(0x50000 + 0x1fc6) / 2] & 0xff;
 }
 
-static WRITE8_HANDLER( sys_port1_w )
+WRITE8_MEMBER(pc88va_state::sys_port1_w)
 {
 	// ...
 }
 
-static ADDRESS_MAP_START( pc88va_io_map, AS_IO, 16 )
+static ADDRESS_MAP_START( pc88va_io_map, AS_IO, 16, pc88va_state )
 	AM_RANGE(0x0000, 0x000f) AM_READ8(key_r,0xffff) // Keyboard ROW reading
 //  AM_RANGE(0x0010, 0x0010) Printer / Calendar Clock Interface
 	AM_RANGE(0x0020, 0x0021) AM_NOP // RS-232C
@@ -1076,7 +1086,7 @@ static ADDRESS_MAP_START( pc88va_io_map, AS_IO, 16 )
 //  AM_RANGE(0x0034, 0x0034) GVRAM Control Port 1
 //  AM_RANGE(0x0035, 0x0035) GVRAM Control Port 2
 	AM_RANGE(0x0040, 0x0041) AM_READ(sys_port4_r) // (R) System Port 4 (W) System port 3 (strobe port)
-	AM_RANGE(0x0044, 0x0045) AM_MIRROR(0x0002) AM_DEVREADWRITE8("ym", ym2203_r,ym2203_w,0xffff)
+	AM_RANGE(0x0044, 0x0045) AM_MIRROR(0x0002) AM_DEVREADWRITE8_LEGACY("ym", ym2203_r,ym2203_w,0xffff)
 //  AM_RANGE(0x005c, 0x005c) (R) GVRAM status
 //  AM_RANGE(0x005c, 0x005f) (W) GVRAM selection
 //  AM_RANGE(0x0070, 0x0070) ? (*)
@@ -1090,7 +1100,7 @@ static ADDRESS_MAP_START( pc88va_io_map, AS_IO, 16 )
 //  AM_RANGE(0x00e6, 0x00e6) 8214 IRQ mask (*)
 //  AM_RANGE(0x00e8, 0x00e9) ? (*)
 //  AM_RANGE(0x00ec, 0x00ed) ? (*)
-	AM_RANGE(0x00fc, 0x00ff) AM_DEVREADWRITE8_MODERN("d8255_2", i8255_device, read, write, 0xffff) // d8255 2, FDD
+	AM_RANGE(0x00fc, 0x00ff) AM_DEVREADWRITE8("d8255_2", i8255_device, read, write, 0xffff) // d8255 2, FDD
 
 	AM_RANGE(0x0100, 0x0101) AM_READWRITE(screen_ctrl_r,screen_ctrl_w) // Screen Control Register
 //  AM_RANGE(0x0102, 0x0103) Graphic Screen Control Register
@@ -1114,24 +1124,24 @@ static ADDRESS_MAP_START( pc88va_io_map, AS_IO, 16 )
 //  AM_RANGE(0x0158, 0x0159) Interruption Mode Modification
 //  AM_RANGE(0x015c, 0x015f) NMI mask port (strobe port)
 //  AM_RANGE(0x0160, 0x016f) DMA Controller
-	AM_RANGE(0x0184, 0x0187) AM_DEVREADWRITE8("pic8259_slave", pic8259_r, pic8259_w, 0x00ff)
-	AM_RANGE(0x0188, 0x018b) AM_DEVREADWRITE8("pic8259_master", pic8259_r, pic8259_w, 0x00ff) // ICU, also controls 8214 emulation
+	AM_RANGE(0x0184, 0x0187) AM_DEVREADWRITE8_LEGACY("pic8259_slave", pic8259_r, pic8259_w, 0x00ff)
+	AM_RANGE(0x0188, 0x018b) AM_DEVREADWRITE8_LEGACY("pic8259_master", pic8259_r, pic8259_w, 0x00ff) // ICU, also controls 8214 emulation
 //  AM_RANGE(0x0190, 0x0191) System Port 5
 //  AM_RANGE(0x0196, 0x0197) Keyboard sub CPU command port
 	AM_RANGE(0x0198, 0x0199) AM_WRITE(backupram_wp_1_w) //Backup RAM write inhibit
 	AM_RANGE(0x019a, 0x019b) AM_WRITE(backupram_wp_0_w) //Backup RAM write permission
-	AM_RANGE(0x01a0, 0x01a7) AM_DEVREADWRITE8("pit8253", pit8253_r, pit8253_w, 0x00ff)// vTCU (timer counter unit)
+	AM_RANGE(0x01a0, 0x01a7) AM_DEVREADWRITE8_LEGACY("pit8253", pit8253_r, pit8253_w, 0x00ff)// vTCU (timer counter unit)
 	AM_RANGE(0x01a8, 0x01a9) AM_WRITE8(timer3_ctrl_reg_w,0x00ff) // General-purpose timer 3 control port
 	AM_RANGE(0x01b0, 0x01bb) AM_READWRITE8(pc88va_fdc_r,pc88va_fdc_w,0x00ff)// FDC related (765)
 //  AM_RANGE(0x01c0, 0x01c1) ?
 	AM_RANGE(0x01c6, 0x01c7) AM_WRITENOP // ???
-	AM_RANGE(0x01c8, 0x01cf) AM_DEVREADWRITE8_MODERN("d8255_3", i8255_device, read, write,0xff00) //i8255 3 (byte access)
+	AM_RANGE(0x01c8, 0x01cf) AM_DEVREADWRITE8("d8255_3", i8255_device, read, write,0xff00) //i8255 3 (byte access)
 //  AM_RANGE(0x01d0, 0x01d1) Expansion RAM bank selection
 	AM_RANGE(0x0200, 0x021f) AM_RAM // Frame buffer 0 control parameter
 	AM_RANGE(0x0220, 0x023f) AM_RAM // Frame buffer 1 control parameter
 	AM_RANGE(0x0240, 0x025f) AM_RAM // Frame buffer 2 control parameter
 	AM_RANGE(0x0260, 0x027f) AM_RAM // Frame buffer 3 control parameter
-	AM_RANGE(0x0300, 0x033f) AM_RAM_WRITE(palette_ram_w) AM_BASE_MEMBER(pc88va_state, m_palram) // Palette RAM (xBBBBxRRRRxGGGG format)
+	AM_RANGE(0x0300, 0x033f) AM_RAM_WRITE(palette_ram_w) AM_SHARE("palram") // Palette RAM (xBBBBxRRRRxGGGG format)
 
 //  AM_RANGE(0x0500, 0x05ff) GVRAM
 //  AM_RANGE(0x1000, 0xfeff) user area (???)
@@ -1140,25 +1150,24 @@ ADDRESS_MAP_END
 // (*) are specific N88 V1 / V2 ports
 
 /* FDC subsytem CPU */
-static ADDRESS_MAP_START( pc88va_z80_map, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( pc88va_z80_map, AS_PROGRAM, 8, pc88va_state )
 	AM_RANGE(0x0000, 0x1fff) AM_ROM
 	AM_RANGE(0x4000, 0x7fff) AM_RAM
 ADDRESS_MAP_END
 
-static WRITE8_HANDLER( fdc_irq_vector_w )
+WRITE8_MEMBER(pc88va_state::fdc_irq_vector_w)
 {
-	pc88va_state *state = space->machine().driver_data<pc88va_state>();
-	state->m_fdc_irq_opcode = data;
+	m_fdc_irq_opcode = data;
 }
 
-static ADDRESS_MAP_START( pc88va_z80_io_map, AS_IO, 8 )
+static ADDRESS_MAP_START( pc88va_z80_io_map, AS_IO, 8, pc88va_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	AM_RANGE(0xf0, 0xf0) AM_WRITE(fdc_irq_vector_w) // Interrupt Opcode Port
 //  AM_RANGE(0xf4, 0xf4) // Drive Control Port
 	AM_RANGE(0xf8, 0xf8) AM_READWRITE(upd765_tc_r,upd765_mc_w) // (R) Terminal Count Port (W) Motor Control Port
-	AM_RANGE(0xfa, 0xfa) AM_DEVREAD("upd765", upd765_status_r )
-	AM_RANGE(0xfb, 0xfb) AM_DEVREADWRITE("upd765", upd765_data_r, upd765_data_w )
-	AM_RANGE(0xfc, 0xff) AM_DEVREADWRITE_MODERN("d8255_2s", i8255_device, read, write)
+	AM_RANGE(0xfa, 0xfa) AM_DEVREAD_LEGACY("upd765", upd765_status_r )
+	AM_RANGE(0xfb, 0xfb) AM_DEVREADWRITE_LEGACY("upd765", upd765_data_r, upd765_data_w )
+	AM_RANGE(0xfc, 0xff) AM_DEVREADWRITE("d8255_2s", i8255_device, read, write)
 ADDRESS_MAP_END
 
 
@@ -1431,11 +1440,11 @@ static READ8_DEVICE_HANDLER( r232_ctrl_porta_r )
 {
 	UINT8 sw5, sw4, sw3, sw2,speed_sw;
 
-	speed_sw = (input_port_read(device->machine(), "SPEED_SW") & 1) ? 0x20 : 0x00;
-	sw5 = (input_port_read(device->machine(), "DSW") & 0x10);
-	sw4 = (input_port_read(device->machine(), "DSW") & 0x08);
-	sw3 = (input_port_read(device->machine(), "DSW") & 0x04);
-	sw2 = (input_port_read(device->machine(), "DSW") & 0x02);
+	speed_sw = (device->machine().root_device().ioport("SPEED_SW")->read() & 1) ? 0x20 : 0x00;
+	sw5 = (device->machine().root_device().ioport("DSW")->read() & 0x10);
+	sw4 = (device->machine().root_device().ioport("DSW")->read() & 0x08);
+	sw3 = (device->machine().root_device().ioport("DSW")->read() & 0x04);
+	sw2 = (device->machine().root_device().ioport("DSW")->read() & 0x02);
 
 	return 0xc1 | sw5 | sw4 | sw3 | sw2 | speed_sw;
 }
@@ -1444,7 +1453,7 @@ static READ8_DEVICE_HANDLER( r232_ctrl_portb_r )
 {
 	UINT8 xsw1;
 
-	xsw1 = (input_port_read(device->machine(), "DSW") & 1) ? 0 : 8;
+	xsw1 = (device->machine().root_device().ioport("DSW")->read() & 1) ? 0 : 8;
 
 	return 0xf7 | xsw1;
 }
@@ -1524,11 +1533,11 @@ static MACHINE_START( pc88va )
 static MACHINE_RESET( pc88va )
 {
 	pc88va_state *state = machine.driver_data<pc88va_state>();
-	UINT8 *ROM00 = machine.region("rom00")->base();
-	UINT8 *ROM10 = machine.region("rom10")->base();
+	UINT8 *ROM00 = machine.root_device().memregion("rom00")->base();
+	UINT8 *ROM10 = state->memregion("rom10")->base();
 
-	memory_set_bankptr(machine, "rom10_bank", &ROM10[0x00000]);
-	memory_set_bankptr(machine, "rom00_bank", &ROM00[0x00000]);
+	state->membank("rom10_bank")->set_base(&ROM10[0x00000]);
+	state->membank("rom00_bank")->set_base(&ROM00[0x00000]);
 
 	state->m_bank_reg = 0x4100;
 	state->m_backupram_wp = 1;
@@ -1553,7 +1562,16 @@ static INTERRUPT_GEN( pc88va_vrtc_irq )
 	pic8259_ir2_w(device->machine().device("pic8259_master"), 1);
 }
 
-/* TODO */
+/* Not sure if parameters are correct for pc88va (copied from x68k) */
+static LEGACY_FLOPPY_OPTIONS_START( pc88va )
+LEGACY_FLOPPY_OPTION( img2d, "xdf,hdm,2hd", "XDF disk image", basicdsk_identify_default, basicdsk_construct_default, NULL,
+					 HEADS([2])
+					 TRACKS([77])
+					 SECTORS([8])
+					 SECTOR_LENGTH([1024])
+					 FIRST_SECTOR_ID([1]))
+LEGACY_FLOPPY_OPTIONS_END
+
 static const floppy_interface pc88va_floppy_interface =
 {
 	DEVCB_NULL,
@@ -1562,8 +1580,8 @@ static const floppy_interface pc88va_floppy_interface =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	FLOPPY_STANDARD_5_25_DSHD,
-	LEGACY_FLOPPY_OPTIONS_NAME(default),
-	NULL,
+	LEGACY_FLOPPY_OPTIONS_NAME(pc88va),
+	"floppy_5_25",
 	NULL
 };
 
@@ -1642,10 +1660,9 @@ static MACHINE_CONFIG_START( pc88va, pc88va_state )
 
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(640, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 200-1)
-	MCFG_SCREEN_UPDATE( pc88va )
+	MCFG_SCREEN_UPDATE_STATIC( pc88va )
 
 	MCFG_PALETTE_LENGTH(32)
 //  MCFG_PALETTE_INIT( pc8801 )
@@ -1666,6 +1683,7 @@ static MACHINE_CONFIG_START( pc88va, pc88va_state )
 
 	MCFG_UPD765A_ADD("upd765", pc88va_upd765_interface)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(pc88va_floppy_interface)
+	MCFG_SOFTWARE_LIST_ADD("disk_list","pc88va")
 
     MCFG_PIT8253_ADD("pit8253",pc88va_pit8253_config)
 
@@ -1725,7 +1743,8 @@ ROM_START( pc88va )
 	ROM_LOAD( "soundbios.rom", 0x0000, 0x2000, NO_DUMP )
 
 	ROM_REGION( 0x80000, "kanji", ROMREGION_ERASEFF )
-	ROM_LOAD( "vafont.rom", 0x0000, 0x50000, CRC(faf7c466) SHA1(196b3d5b7407cb4f286ffe5c1e34ebb1f6905a8c)) // should be splitted
+	ROM_LOAD( "vafont.rom", 0x0000, 0x50000, BAD_DUMP CRC(faf7c466) SHA1(196b3d5b7407cb4f286ffe5c1e34ebb1f6905a8c)) // should be splitted
+
 	ROM_REGION( 0x80000, "dictionary", 0 )
 	ROM_LOAD( "vadic.rom",  0x0000, 0x80000, CRC(f913c605) SHA1(5ba1f3578d0aaacdaf7194a80e6d520c81ae55fb))
 
