@@ -21,13 +21,13 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
 #include "machine/ram.h"
 #include "sound/beep.h"
 #include "machine/pce220_ser.h"
+#include "machine/nvram.h"
 #include "rendlay.h"
 
 // Interrupt flags
@@ -47,7 +47,7 @@ public:
 		{ }
 
 	required_device<cpu_device> m_maincpu;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 	required_device<device_t> m_beep;
 	required_device<pce220_serial_device> m_serial;
 
@@ -70,7 +70,7 @@ public:
 
 	virtual void machine_start();
 	virtual void machine_reset();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	DECLARE_READ8_MEMBER( lcd_status_r );
 	DECLARE_WRITE8_MEMBER( lcd_control_w );
 	DECLARE_READ8_MEMBER( lcd_data_r );
@@ -107,7 +107,7 @@ public:
 
 	virtual void machine_start();
 	virtual void machine_reset();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	DECLARE_READ8_MEMBER( g850v_bank_r );
 	DECLARE_WRITE8_MEMBER( g850v_bank_w );
 	DECLARE_READ8_MEMBER( g850v_lcd_status_r );
@@ -116,8 +116,10 @@ public:
 	DECLARE_WRITE8_MEMBER( g850v_lcd_data_w );
 };
 
-bool pce220_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 pce220_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
+	UINT8 lcd_symbols[4];
+
 	if (m_lcd_on)
 	{
 		for (int y = 0; y < 4; y++)
@@ -132,34 +134,54 @@ bool pce220_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 					{
 						//first 12 columns
 						int panel1_addr = ((m_lcd_start_line>>3) + y)*0x40 + row_pos;
-						*BITMAP_ADDR16(&bitmap, y*8 + yi, x*6 + xi) = (m_vram[panel1_addr & 0x1ff] >> yi) & 1;
+						bitmap.pix16(y*8 + yi, x*6 + xi) = (m_vram[panel1_addr & 0x1ff] >> yi) & 1;
 
 						//last 12 columns
 						int panel2_addr = ((m_lcd_start_line>>3) + y + 4)*0x40 + (59-row_pos);
-						*BITMAP_ADDR16(&bitmap, y*8 + yi, (x+12)*6 + xi) = (m_vram[panel2_addr & 0x1ff] >> yi) & 1;
+						bitmap.pix16(y*8 + yi, (x+12)*6 + xi) = (m_vram[panel2_addr & 0x1ff] >> yi) & 1;
 					}
 
 					row_pos++;
 				}
 			}
 		}
+
+		lcd_symbols[0] = m_vram[((m_lcd_start_line>>3)*0x40 + 0x03c) & 0x1ff];
+		lcd_symbols[1] = m_vram[((m_lcd_start_line>>3)*0x40 + 0x0fc) & 0x1ff];
+		lcd_symbols[2] = m_vram[((m_lcd_start_line>>3)*0x40 + 0x13c) & 0x1ff];
+		lcd_symbols[3] = m_vram[((m_lcd_start_line>>3)*0x40 + 0x1fc) & 0x1ff];
 	}
 	else
 	{
-		bitmap_fill(&bitmap, &cliprect, 0);
+		bitmap.fill(0, cliprect);
+		memset(lcd_symbols, 0, sizeof(lcd_symbols));
 	}
-/*
-    // TODO: LCD Symbols
-    popmessage("%x %x %x %x",   m_vram[((m_lcd_start_line>>3)*0x40 + 0x03c) & 0x1ff],
-                                m_vram[((m_lcd_start_line>>3)*0x40 + 0x0fc) & 0x1ff],
-                                m_vram[((m_lcd_start_line>>3)*0x40 + 0x13c) & 0x1ff],
-                                m_vram[((m_lcd_start_line>>3)*0x40 + 0x1fc) & 0x1ff]);
-*/
+
+	output_set_value("BUSY" , (lcd_symbols[0] & 0x01) ? 1 : 0);
+	output_set_value("CAPS" , (lcd_symbols[0] & 0x02) ? 1 : 0);
+	output_set_value("KANA" , (lcd_symbols[0] & 0x04) ? 1 : 0);
+	output_set_value("SYO"  , (lcd_symbols[0] & 0x08) ? 1 : 0);
+	output_set_value("2ndF" , (lcd_symbols[0] & 0x10) ? 1 : 0);
+	output_set_value("TEXT" , (lcd_symbols[1] & 0x08) ? 1 : 0);
+	output_set_value("CASL" , (lcd_symbols[1] & 0x10) ? 1 : 0);
+	output_set_value("PRO"  , (lcd_symbols[1] & 0x20) ? 1 : 0);
+	output_set_value("RUN"  , (lcd_symbols[1] & 0x40) ? 1 : 0);
+	output_set_value("BATT" , (lcd_symbols[2] & 0x01) ? 1 : 0);
+	output_set_value("E"    , (lcd_symbols[2] & 0x02) ? 1 : 0);
+	output_set_value("M"    , (lcd_symbols[2] & 0x04) ? 1 : 0);
+	output_set_value("CONST", (lcd_symbols[2] & 0x08) ? 1 : 0);
+	output_set_value("RAD"  , (lcd_symbols[2] & 0x10) ? 1 : 0);
+	output_set_value("G"    , (lcd_symbols[2] & 0x20) ? 1 : 0);
+	output_set_value("DE"   , (lcd_symbols[2] & 0x40) ? 1 : 0);
+	output_set_value("STAT" , (lcd_symbols[3] & 0x20) ? 1 : 0);
+	output_set_value("PRINT", (lcd_symbols[3] & 0x40) ? 1 : 0);
+
     return 0;
 }
 
-bool pcg850v_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 pcg850v_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
+	UINT8 lcd_symbols[6];
 	int color0 = 0;
 	int color1 = 1;
 
@@ -196,19 +218,45 @@ bool pcg850v_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 					for (int yi = 0; yi < 8; yi++)
 					{
 						int addr = ((m_lcd_start_line>>3) + y)*0x100 + row_pos;
-						*BITMAP_ADDR16(&bitmap, y*8 + yi, x*6 + xi) = ((m_vram[addr & 0x7ff] >> yi) & 1 ) ? color1 : color0;
+						bitmap.pix16(y*8 + yi, x*6 + xi) = ((m_vram[addr & 0x7ff] >> yi) & 1 ) ? color1 : color0;
 					}
 
 					row_pos++;
 				}
 			}
 		}
+
+		lcd_symbols[0] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x090) & 0x7ff];
+		lcd_symbols[1] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x190) & 0x7ff];
+		lcd_symbols[2] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x290) & 0x7ff];
+		lcd_symbols[3] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x390) & 0x7ff];
+		lcd_symbols[4] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x490) & 0x7ff];
+		lcd_symbols[5] = m_vram[((m_lcd_start_line>>3)*0x100 + 0x590) & 0x7ff];
 	}
 	else
 	{
-		bitmap_fill(&bitmap, &cliprect, 0);
+		bitmap.fill(0, cliprect);
+		memset(lcd_symbols, 0, sizeof(lcd_symbols));
 	}
 
+	output_set_value("RUN"  , (lcd_symbols[0] & 0x02) ? 1 : 0);
+	output_set_value("PRO"  , (lcd_symbols[0] & 0x08) ? 1 : 0);
+	output_set_value("TEXT" , (lcd_symbols[0] & 0x40) ? 1 : 0);
+	output_set_value("CASL" , (lcd_symbols[1] & 0x08) ? 1 : 0);
+	output_set_value("STAT" , (lcd_symbols[2] & 0x01) ? 1 : 0);
+	output_set_value("2ndF" , (lcd_symbols[2] & 0x20) ? 1 : 0);
+	output_set_value("M"    , (lcd_symbols[2] & 0x80) ? 1 : 0);
+	output_set_value("CAPS" , (lcd_symbols[3] & 0x04) ? 1 : 0);
+	output_set_value("KANA" , (lcd_symbols[3] & 0x80) ? 1 : 0);
+	output_set_value("SYO"  , (lcd_symbols[4] & 0x02) ? 1 : 0);
+	output_set_value("DE"   , (lcd_symbols[4] & 0x10) ? 1 : 0);
+	output_set_value("G"    , (lcd_symbols[4] & 0x40) ? 1 : 0);
+	output_set_value("RAD"  , (lcd_symbols[5] & 0x01) ? 1 : 0);
+	output_set_value("CONST", (lcd_symbols[5] & 0x04) ? 1 : 0);
+	output_set_value("PRINT", (lcd_symbols[5] & 0x10) ? 1 : 0);
+	output_set_value("BUSY" , (lcd_symbols[5] & 0x40) ? 1 : 0);
+	output_set_value("BATT" , (lcd_symbols[5] & 0x80) ? 1 : 0);
+
     return 0;
 }
 
@@ -262,16 +310,16 @@ WRITE8_MEMBER( pce220_state::rom_bank_w )
 
 	m_bank_num = data;
 
-	memory_set_bank(machine(), "bank3", bank3);
-	memory_set_bank(machine(), "bank4", bank4);
+	membank("bank3")->set_entry(bank3);
+	membank("bank4")->set_entry(bank4);
 }
 
 WRITE8_MEMBER( pce220_state::ram_bank_w )
 {
 	UINT8 bank = BIT(data,2);
 
-	memory_set_bank(machine(), "bank1", bank);
-	memory_set_bank(machine(), "bank2", bank);
+	membank("bank1")->set_entry(bank);
+	membank("bank2")->set_entry(bank);
 }
 
 READ8_MEMBER( pce220_state::timer_r )
@@ -291,7 +339,7 @@ WRITE8_MEMBER( pce220_state::boot_bank_w )
 	{
 		address_space *space_prg = m_maincpu->memory().space(AS_PROGRAM);
 		space_prg->install_write_bank(0x0000, 0x3fff, "bank1");
-		memory_set_bank(machine(), "bank1", 0);
+		membank("bank1")->set_entry(0);
 	}
 }
 
@@ -348,7 +396,7 @@ READ8_MEMBER( pce220_state::port1f_r )
 	data |= m_serial->in_ack()<<1;
 	data |= m_serial->in_xin()<<2;
 
-	data |= input_port_read(machine(), "ON")<<7;
+	data |= ioport("ON")->read()<<7;
 
 	return data;
 }
@@ -371,25 +419,25 @@ READ8_MEMBER( pce220_state::kb_r )
 	UINT8 data = 0x00;
 
 	if (m_kb_matrix & 0x01)
-		data |= input_port_read(machine(), "LINE0");
+		data |= ioport("LINE0")->read();
 	if (m_kb_matrix & 0x02)
-		data |= input_port_read(machine(), "LINE1");
+		data |= ioport("LINE1")->read();
 	if (m_kb_matrix & 0x04)
-		data |= input_port_read(machine(), "LINE2");
+		data |= ioport("LINE2")->read();
 	if (m_kb_matrix & 0x08)
-		data |= input_port_read(machine(), "LINE3");
+		data |= ioport("LINE3")->read();
 	if (m_kb_matrix & 0x10)
-		data |= input_port_read(machine(), "LINE4");
+		data |= ioport("LINE4")->read();
 	if (m_kb_matrix & 0x20)
-		data |= input_port_read(machine(), "LINE5");
+		data |= ioport("LINE5")->read();
 	if (m_kb_matrix & 0x40)
-		data |= input_port_read(machine(), "LINE6");
+		data |= ioport("LINE6")->read();
 	if (m_kb_matrix & 0x80)
-		data |= input_port_read(machine(), "LINE7");
+		data |= ioport("LINE7")->read();
 	if (m_kb_matrix & 0x100)
-		data |= input_port_read(machine(), "LINE8");
+		data |= ioport("LINE8")->read();
 	if (m_kb_matrix & 0x200)
-		data |= input_port_read(machine(), "LINE9");
+		data |= ioport("LINE9")->read();
 
 	return data;
 }
@@ -426,7 +474,7 @@ WRITE8_MEMBER( pcg850v_state::g850v_bank_w )
 	if (data < 0x16)
 	{
 		space_prg->install_read_bank(0xc000, 0xffff, "bank4");
-		memory_set_bank(machine(), "bank4", data);
+		membank("bank4")->set_entry(data);
 	}
 	else
 	{
@@ -795,28 +843,31 @@ INPUT_PORTS_END
 
 void pce220_state::machine_start()
 {
-	UINT8 *rom = machine().region("user1")->base();
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *rom = memregion("user1")->base();
+	UINT8 *ram = m_ram->pointer();
 
-	memory_configure_bank(machine(), "bank1", 0, 2, ram + 0x0000, 0x8000);
-	memory_configure_bank(machine(), "bank2", 0, 2, ram + 0x4000, 0x8000);
-	memory_configure_bank(machine(), "bank3", 0, 8, rom, 0x4000);
-	memory_configure_bank(machine(), "bank4", 0, 8, rom, 0x4000);
+	membank("bank1")->configure_entries(0, 2, ram + 0x0000, 0x8000);
+	membank("bank2")->configure_entries(0, 2, ram + 0x4000, 0x8000);
+	membank("bank3")->configure_entries(0, 8, rom, 0x4000);
+	membank("bank4")->configure_entries(0, 8, rom, 0x4000);
 
-	m_vram = (UINT8*)machine().region("lcd_vram")->base();
+	m_vram = (UINT8*)machine().root_device().memregion("lcd_vram")->base();
+
+	machine().device<nvram_device>("nvram")->set_base(ram, m_ram->size());
 }
 
 void pcg850v_state::machine_start()
 {
-	UINT8 *rom = machine().region("user1")->base();
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *rom = memregion("user1")->base();
+	UINT8 *ram = m_ram->pointer();
 
-	memory_configure_bank(machine(), "bank1", 0, 2, ram + 0x0000, 0x8000);
-	memory_configure_bank(machine(), "bank2", 0, 2, ram + 0x4000, 0x8000);
-	memory_configure_bank(machine(), "bank3", 0, 22, rom, 0x4000);
-	memory_configure_bank(machine(), "bank4", 0, 22, rom, 0x4000);
+	membank("bank1")->configure_entries(0, 2, ram + 0x0000, 0x8000);
+	membank("bank2")->configure_entries(0, 2, ram + 0x4000, 0x8000);
+	membank("bank3")->configure_entries(0, 22, rom, 0x4000);
+	membank("bank4")->configure_entries(0, 22, rom, 0x4000);
 
-	m_vram = (UINT8*)machine().region("lcd_vram")->base();
+	m_vram = (UINT8*)machine().root_device().memregion("lcd_vram")->base();
+	machine().device<nvram_device>("nvram")->set_base(ram, m_ram->size());
 }
 
 void pce220_state::machine_reset()
@@ -825,7 +876,7 @@ void pce220_state::machine_reset()
 	space->unmap_write(0x0000, 0x3fff);
 
 	// install the boot code into the first bank
-	memory_set_bankptr(machine(), "bank1", machine().region("user1")->base() + 0x0000);
+	membank("bank1")->set_base(machine().root_device().memregion("user1")->base() + 0x0000);
 
 	m_lcd_index_row = 0;
 	m_lcd_index_col = 0;
@@ -862,29 +913,6 @@ static TIMER_DEVICE_CALLBACK(pce220_timer_callback)
 	}
 }
 
-static NVRAM_HANDLER(pce220)
-{
-	pce220_state *state = machine.driver_data<pce220_state>();
-	UINT8 *ram_base = (UINT8*)ram_get_ptr(state->m_ram);
-	UINT32 ram_size = ram_get_size(state->m_ram);
-
-	if (read_or_write)
-	{
-		file->write(ram_base, ram_size);
-	}
-	else
-	{
-		if (file)
-		{
-			file->read(ram_base, ram_size);
-		}
-		else
-		{
-			memset(ram_base, 0, ram_size);
-		}
-	}
-}
-
 static PALETTE_INIT(pce220)
 {
 	palette_set_color(machine, 0, MAKE_RGB(138, 146, 148));
@@ -903,7 +931,7 @@ static MACHINE_CONFIG_START( pce220, pce220_state )
     MCFG_SCREEN_ADD("screen", RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(pce220_state, screen_update)
     MCFG_SCREEN_SIZE(24*6, 4*8)
     MCFG_SCREEN_VISIBLE_AREA(0, 24*6-1, 0, 4*8-1)
 
@@ -918,7 +946,7 @@ static MACHINE_CONFIG_START( pce220, pce220_state )
 
 	MCFG_TIMER_ADD_PERIODIC("pce220_timer", pce220_timer_callback, attotime::from_msec(468))
 
-	MCFG_NVRAM_HANDLER(pce220)
+	MCFG_NVRAM_ADD_0FILL("nvram")
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
@@ -938,7 +966,7 @@ static MACHINE_CONFIG_START( pcg850v, pcg850v_state )
     MCFG_SCREEN_ADD("screen", RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_UPDATE_DRIVER(pcg850v_state, screen_update)
     MCFG_SCREEN_SIZE(144, 48)
     MCFG_SCREEN_VISIBLE_AREA(0, 144-1, 0, 48-1)
 
@@ -953,7 +981,7 @@ static MACHINE_CONFIG_START( pcg850v, pcg850v_state )
 
 	MCFG_TIMER_ADD_PERIODIC("pce220_timer", pce220_timer_callback, attotime::from_msec(468))
 
-	MCFG_NVRAM_HANDLER(pce220)
+	MCFG_NVRAM_ADD_0FILL("nvram")
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
