@@ -57,7 +57,7 @@ and the java source).
  */
 
 
-static ADDRESS_MAP_START(pdp1_map, AS_PROGRAM, 32)
+static ADDRESS_MAP_START(pdp1_map, AS_PROGRAM, 32, pdp1_state )
 	AM_RANGE(0x00000, 0x3ffff) AM_RAM
 ADDRESS_MAP_END
 
@@ -495,7 +495,7 @@ static MACHINE_RESET( pdp1 )
 	pdp1_state *state = machine.driver_data<pdp1_state>();
 	int cfg;
 
-	cfg = input_port_read(machine, "CFG");
+	cfg = machine.root_device().ioport("CFG")->read();
 	pdp1_reset_param.extend_support = (cfg >> pdp1_config_extend_bit) & pdp1_config_extend_mask;
 	pdp1_reset_param.hw_mul_div = (cfg >> pdp1_config_hw_mul_div_bit) & pdp1_config_hw_mul_div_mask;
 	pdp1_reset_param.type_20_sbs = (cfg >> pdp1_config_type_20_sbs_bit) & pdp1_config_type_20_sbs_mask;
@@ -655,7 +655,7 @@ static MACHINE_START( pdp1 )
 	};
 
 	/* set up our font */
-	dst = machine.region("gfx1")->base();
+	dst = machine.root_device().memregion("gfx1")->base();
 	memcpy(dst, fontdata6x8, pdp1_fontdata_size);
 
 	machine.add_notifier(MACHINE_NOTIFY_EXIT, machine_notify_delegate(FUNC(pdp1_machine_stop),&machine));
@@ -673,16 +673,156 @@ static MACHINE_START( pdp1 )
     perforated tape handling
 */
 
+class pdp1_readtape_image_device :	public device_t,
+									public device_image_interface
+{
+public:
+	// construction/destruction
+	pdp1_readtape_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock);
+
+	// image-level overrides
+	virtual iodevice_t image_type() const { return IO_PUNCHTAPE; }
+
+	virtual bool is_readable()  const { return 1; }
+	virtual bool is_writeable() const { return 0; }
+	virtual bool is_creatable() const { return 0; }
+	virtual bool must_be_loaded() const { return 0; }
+	virtual bool is_reset_on_load() const { return 0; }
+	virtual const char *image_interface() const { return NULL; }
+	virtual const char *file_extensions() const { return "tap,rim"; }
+	virtual const option_guide *create_option_guide() const { return NULL; }
+
+	virtual bool call_load();
+	virtual void call_unload();
+protected:
+	// device-level overrides
+    virtual void device_config_complete() { update_names(); }
+	virtual void device_start() { }
+};
+
+const device_type PDP1_READTAPE = &device_creator<pdp1_readtape_image_device>;
+
+pdp1_readtape_image_device::pdp1_readtape_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+    : device_t(mconfig, PDP1_READTAPE, "PDP1 Tape Reader", tag, owner, clock),
+	  device_image_interface(mconfig, *this)
+{
+}
+
+class pdp1_punchtape_image_device :	public device_t,
+									public device_image_interface
+{
+public:
+	// construction/destruction
+	pdp1_punchtape_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock);
+
+	// image-level overrides
+	virtual iodevice_t image_type() const { return IO_PUNCHTAPE; }
+
+	virtual bool is_readable()  const { return 0; }
+	virtual bool is_writeable() const { return 1; }
+	virtual bool is_creatable() const { return 1; }
+	virtual bool must_be_loaded() const { return 0; }
+	virtual bool is_reset_on_load() const { return 0; }
+	virtual const char *image_interface() const { return NULL; }
+	virtual const char *file_extensions() const { return "tap,rim"; }
+	virtual const option_guide *create_option_guide() const { return NULL; }
+
+	virtual bool call_load();
+	virtual void call_unload();
+protected:
+	// device-level overrides
+    virtual void device_config_complete() { update_names(); }
+	virtual void device_start() { }
+};
+
+const device_type PDP1_PUNCHTAPE = &device_creator<pdp1_punchtape_image_device>;
+
+pdp1_punchtape_image_device::pdp1_punchtape_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+    : device_t(mconfig, PDP1_PUNCHTAPE, "PDP1 Tape Puncher", tag, owner, clock),
+	  device_image_interface(mconfig, *this)
+{
+}
+
+
+class pdp1_printer_image_device :	public device_t,
+									public device_image_interface
+{
+public:
+	// construction/destruction
+	pdp1_printer_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock);
+
+	// image-level overrides
+	virtual iodevice_t image_type() const { return IO_PRINTER; }
+
+	virtual bool is_readable()  const { return 0; }
+	virtual bool is_writeable() const { return 1; }
+	virtual bool is_creatable() const { return 1; }
+	virtual bool must_be_loaded() const { return 0; }
+	virtual bool is_reset_on_load() const { return 0; }
+	virtual const char *image_interface() const { return NULL; }
+	virtual const char *file_extensions() const { return "typ"; }
+	virtual const option_guide *create_option_guide() const { return NULL; }
+
+	virtual bool call_load();
+	virtual void call_unload();
+protected:
+	// device-level overrides
+    virtual void device_config_complete() { update_names(); }
+	virtual void device_start() { }
+};
+
+const device_type PDP1_PRINTER = &device_creator<pdp1_printer_image_device>;
+
+pdp1_printer_image_device::pdp1_printer_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+    : device_t(mconfig, PDP1_PRINTER, "PDP1 Typewriter", tag, owner, clock),
+	  device_image_interface(mconfig, *this)
+{
+}
+
+class pdp1_cylinder_image_device :	public device_t,
+									public device_image_interface
+{
+public:
+	// construction/destruction
+	pdp1_cylinder_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock);
+
+	// image-level overrides
+	virtual iodevice_t image_type() const { return IO_CYLINDER; }
+
+	virtual bool is_readable()  const { return 1; }
+	virtual bool is_writeable() const { return 1; }
+	virtual bool is_creatable() const { return 1; }
+	virtual bool must_be_loaded() const { return 0; }
+	virtual bool is_reset_on_load() const { return 0; }
+	virtual const char *image_interface() const { return NULL; }
+	virtual const char *file_extensions() const { return "drm"; }
+	virtual const option_guide *create_option_guide() const { return NULL; }
+
+	virtual bool call_load();
+	virtual void call_unload();
+protected:
+	// device-level overrides
+    virtual void device_config_complete() { update_names(); }
+	virtual void device_start() { }
+};
+
+const device_type PDP1_CYLINDER = &device_creator<pdp1_cylinder_image_device>;
+
+pdp1_cylinder_image_device::pdp1_cylinder_image_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+    : device_t(mconfig, PDP1_CYLINDER, "PDP1 Cylinder", tag, owner, clock),
+	  device_image_interface(mconfig, *this)
+{
+}
 
 /*
     Open a perforated tape image
 */
-static DEVICE_IMAGE_LOAD( pdp1_readtape )
+bool pdp1_readtape_image_device::call_load()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 
 	/* reader unit */
-	state->m_tape_reader.fd = &image;
+	state->m_tape_reader.fd = this;
 
 	/* start motor */
 	state->m_tape_reader.motor_on = 1;
@@ -707,9 +847,9 @@ static DEVICE_IMAGE_LOAD( pdp1_readtape )
 	return IMAGE_INIT_PASS;
 }
 
-static DEVICE_IMAGE_UNLOAD( pdp1_readtape )
+void pdp1_readtape_image_device::call_unload()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 
 	/* reader unit */
 	state->m_tape_reader.fd = NULL;
@@ -912,19 +1052,19 @@ static void iot_rrb(device_t *device, int op2, int nac, int mb, int *io, int ac)
 }
 
 
-static DEVICE_IMAGE_LOAD( pdp1_punchtape )
+bool pdp1_punchtape_image_device::call_load()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 
 	/* punch unit */
-	state->m_tape_puncher.fd = &image;
+	state->m_tape_puncher.fd = this;
 
 	return IMAGE_INIT_PASS;
 }
 
-static DEVICE_IMAGE_UNLOAD( pdp1_punchtape )
+void pdp1_punchtape_image_device::call_unload()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 
 	/* punch unit */
 	state->m_tape_puncher.fd = NULL;
@@ -1024,20 +1164,20 @@ static void iot_ppb(device_t *device, int op2, int nac, int mb, int *io, int ac)
 /*
     Open a file for typewriter output
 */
-static DEVICE_IMAGE_LOAD(pdp1_typewriter)
+bool pdp1_printer_image_device::call_load()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 	/* open file */
-	state->m_typewriter.fd = &image;
+	state->m_typewriter.fd = this;
 
 	state->m_io_status |= io_st_tyo;
 
 	return IMAGE_INIT_PASS;
 }
 
-static DEVICE_IMAGE_UNLOAD(pdp1_typewriter)
+void pdp1_printer_image_device::call_unload()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 	state->m_typewriter.fd = NULL;
 }
 
@@ -1363,18 +1503,18 @@ static void parallel_drum_init(pdp1_state *state)
 /*
     Open a file for drum
 */
-static DEVICE_IMAGE_LOAD(pdp1_drum)
+bool pdp1_cylinder_image_device::call_load()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 	/* open file */
-	state->m_parallel_drum.fd = &image;
+	state->m_parallel_drum.fd = this;
 
 	return IMAGE_INIT_PASS;
 }
 
-static DEVICE_IMAGE_UNLOAD(pdp1_drum)
+void pdp1_cylinder_image_device::call_unload()
 {
-	pdp1_state *state = image.device().machine().driver_data<pdp1_state>();
+	pdp1_state *state = machine().driver_data<pdp1_state>();
 	state->m_parallel_drum.fd = NULL;
 }
 
@@ -1493,7 +1633,7 @@ static void iot_dra(device_t *device, int op2, int nac, int mb, int *io, int ac)
 */
 static void iot_011(device_t *device, int op2, int nac, int mb, int *io, int ac)
 {
-	int key_state = input_port_read(device->machine(), "SPACEWAR");
+	int key_state = device->machine().root_device().ioport("SPACEWAR")->read();
 	int reply;
 
 
@@ -1582,7 +1722,7 @@ static void pdp1_keyboard(running_machine &machine)
 
 	for (i=0; i<4; i++)
 	{
-		typewriter_keys[i] = input_port_read(machine, twrnames[i]);
+		typewriter_keys[i] = machine.root_device().ioport(twrnames[i])->read();
 	}
 
 	for (i=0; i<4; i++)
@@ -1613,9 +1753,9 @@ static void pdp1_lightpen(running_machine &machine)
 	int x_delta, y_delta;
 	int current_state;
 
-	state->m_lightpen.active = (input_port_read(machine, "CFG") >> pdp1_config_lightpen_bit) & pdp1_config_lightpen_mask;
+	state->m_lightpen.active = (machine.root_device().ioport("CFG")->read() >> pdp1_config_lightpen_bit) & pdp1_config_lightpen_mask;
 
-	current_state = input_port_read(machine, "LIGHTPEN");
+	current_state = machine.root_device().ioport("LIGHTPEN")->read();
 
 	/* update pen down state */
 	state->m_lightpen.down = state->m_lightpen.active && (current_state & pdp1_lightpen_down);
@@ -1637,8 +1777,8 @@ static void pdp1_lightpen(running_machine &machine)
 	state->m_old_lightpen = current_state;
 
 	/* update pen position */
-	x_delta = input_port_read(machine, "LIGHTX");
-	y_delta = input_port_read(machine, "LIGHTY");
+	x_delta = machine.root_device().ioport("LIGHTX")->read();
+	y_delta = machine.root_device().ioport("LIGHTY")->read();
 
 	if (x_delta >= 0x80)
 		x_delta -= 0x100;
@@ -1676,10 +1816,10 @@ INTERRUPT_GEN( pdp1_interrupt )
 	int ta_transitions;
 
 
-	cpu_set_reg(device, PDP1_SS, input_port_read(device->machine(), "SENSE"));
+	cpu_set_reg(device, PDP1_SS, device->machine().root_device().ioport("SENSE")->read());
 
 	/* read new state of control keys */
-	control_keys = input_port_read(device->machine(), "CSW");
+	control_keys = state->ioport("CSW")->read();
 
 	if (control_keys & pdp1_control)
 	{
@@ -1771,7 +1911,7 @@ INTERRUPT_GEN( pdp1_interrupt )
 
 
 		/* handle test word keys */
-		tw_keys = (input_port_read(machine, "TWDMSB") << 16) | input_port_read(machine, "TWDLSB");
+		tw_keys = (machine.root_device().ioport("TWDMSB")->read() << 16) | machine.root_device().ioport("TWDLSB")->read();
 
 		/* compute transitions */
 		tw_transitions = tw_keys & (~ state->m_old_tw_keys);
@@ -1784,7 +1924,7 @@ INTERRUPT_GEN( pdp1_interrupt )
 
 
 		/* handle address keys */
-		ta_keys = input_port_read(machine, "TSTADD");
+		ta_keys = machine.root_device().ioport("TSTADD")->read();
 
 		/* compute transitions */
 		ta_transitions = ta_keys & (~ state->m_old_ta_keys);
@@ -1808,101 +1948,6 @@ INTERRUPT_GEN( pdp1_interrupt )
 	pdp1_lightpen(machine);
 }
 
-static DEVICE_START(pdp1_readtape) {}
-static DEVICE_START(pdp1_punchtape) {}
-static DEVICE_START(pdp1_typewriter) {}
-static DEVICE_START(pdp1_drum) {}
-
-
-static DEVICE_GET_INFO(pdp1_readtape)
-{
-	switch(state)
-	{
-		case DEVINFO_INT_IMAGE_TYPE:			info->i = IO_PUNCHTAPE; break;
-		case DEVINFO_INT_IMAGE_READABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:		info->i = 0; break;
-		case DEVINFO_INT_IMAGE_CREATABLE:		info->i = 0; break;
-		case DEVINFO_INT_IMAGE_CREATE_OPTCOUNT:		info->i = 1; break;
-
-		case DEVINFO_FCT_START:				info->start = DEVICE_START_NAME(pdp1_readtape); break;
-		case DEVINFO_FCT_IMAGE_LOAD:			info->f = (genf *) DEVICE_IMAGE_LOAD_NAME(pdp1_readtape); break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:			info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME(pdp1_readtape); break;
-
-		case DEVINFO_STR_NAME:		                strcpy(info->s, "PDP1 Tape Reader"); break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:		strcpy(info->s, "tap,rim"); break;
-	}
-}
-
-DECLARE_LEGACY_IMAGE_DEVICE(PDP1_READTAPE, pdp1_readtape);
-DEFINE_LEGACY_IMAGE_DEVICE(PDP1_READTAPE, pdp1_readtape);
-
-static DEVICE_GET_INFO(pdp1_punchtape)
-{
-	switch(state)
-	{
-		case DEVINFO_INT_IMAGE_TYPE:			info->i = IO_PUNCHTAPE; break;
-		case DEVINFO_INT_IMAGE_READABLE:		info->i = 0; break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATE_OPTCOUNT:		info->i = 1; break;
-
-		case DEVINFO_FCT_START:				info->start = DEVICE_START_NAME(pdp1_punchtape); break;
-		case DEVINFO_FCT_IMAGE_LOAD:			info->f = (genf *) DEVICE_IMAGE_LOAD_NAME(pdp1_punchtape); break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:			info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME(pdp1_punchtape); break;
-
-		case DEVINFO_STR_NAME:		                strcpy(info->s, "PDP1 Tape Puncher"); break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:		strcpy(info->s, "tap,rim"); break;
-	}
-}
-
-DECLARE_LEGACY_IMAGE_DEVICE(PDP1_PUNCHTAPE, pdp1_punchtape);
-DEFINE_LEGACY_IMAGE_DEVICE(PDP1_PUNCHTAPE, pdp1_punchtape);
-
-static DEVICE_GET_INFO(pdp1_printer)
-{
-	switch(state)
-	{
-		case DEVINFO_INT_IMAGE_TYPE:			info->i = IO_PRINTER; break;
-		case DEVINFO_INT_IMAGE_READABLE:		info->i = 0; break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATE_OPTCOUNT:		info->i = 1; break;
-
-		case DEVINFO_FCT_START:				info->start = DEVICE_START_NAME(pdp1_typewriter); break;
-		case DEVINFO_FCT_IMAGE_LOAD:			info->f = (genf *) DEVICE_IMAGE_LOAD_NAME(pdp1_typewriter); break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:			info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME(pdp1_typewriter); break;
-
-		case DEVINFO_STR_NAME:		                strcpy(info->s, "PDP1 Typewriter"); break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:		strcpy(info->s, "typ"); break;
-	}
-}
-
-DECLARE_LEGACY_IMAGE_DEVICE(PDP1_PRINTER, pdp1_printer);
-DEFINE_LEGACY_IMAGE_DEVICE(PDP1_PRINTER, pdp1_printer);
-
-static DEVICE_GET_INFO(pdp1_cylinder)
-{
-	switch(state)
-	{
-		case DEVINFO_INT_IMAGE_TYPE:			info->i = IO_CYLINDER; break;
-		case DEVINFO_INT_IMAGE_READABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATABLE:		info->i = 1; break;
-		case DEVINFO_INT_IMAGE_CREATE_OPTCOUNT:		info->i = 1; break;
-
-		case DEVINFO_FCT_START:				info->start = DEVICE_START_NAME(pdp1_drum); break;
-		case DEVINFO_FCT_IMAGE_LOAD:			info->f = (genf *) DEVICE_IMAGE_LOAD_NAME(pdp1_drum); break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:			info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME(pdp1_drum); break;
-
-		case DEVINFO_STR_NAME:		                strcpy(info->s, "PDP1 Cylinder"); break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:		strcpy(info->s, "drm"); break;
-	}
-}
-
-DECLARE_LEGACY_IMAGE_DEVICE(PDP1_CYLINDER, pdp1_cylinder);
-DEFINE_LEGACY_IMAGE_DEVICE(PDP1_CYLINDER, pdp1_cylinder);
-
-
 
 static MACHINE_CONFIG_START( pdp1, pdp1_state )
 
@@ -1920,11 +1965,10 @@ static MACHINE_CONFIG_START( pdp1, pdp1_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(refresh_rate)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(virtual_width, virtual_height)
 	MCFG_SCREEN_VISIBLE_AREA(0, virtual_width-1, 0, virtual_height-1)
-	MCFG_SCREEN_UPDATE(pdp1)
-	MCFG_SCREEN_EOF(pdp1)
+	MCFG_SCREEN_UPDATE_STATIC(pdp1)
+	MCFG_SCREEN_VBLANK_STATIC(pdp1)
 
 	MCFG_CRT_ADD( "crt", pdp1_crt_interface )
 	MCFG_DEVICE_ADD("readt", PDP1_READTAPE, 0)
