@@ -65,7 +65,6 @@
         23/02/2009 Skeleton driver.
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/t11/t11.h"
@@ -81,7 +80,7 @@ public:
 	{ }
 
 	required_device<cpu_device> m_maincpu;
-	required_device<device_t> m_terminal;
+	required_device<generic_terminal_device> m_terminal;
 	DECLARE_READ16_MEMBER( term_r );
 	DECLARE_READ16_MEMBER( term_tx_status_r );
 	DECLARE_READ16_MEMBER( term_rx_status_r );
@@ -93,7 +92,7 @@ public:
 
 WRITE16_MEMBER(pdp11_state::term_w)
 {
-	terminal_write(m_terminal, 0, data);
+	m_terminal->write(space, 0, data);
 }
 
 READ16_MEMBER(pdp11_state::term_r)
@@ -220,8 +219,8 @@ INPUT_PORTS_END
 static MACHINE_RESET(pdp11)
 {
 	// Load M9301-YA
-	UINT8* user1 = machine.region("user1")->base();
-	UINT8* maincpu = machine.region("maincpu")->base();
+	UINT8* user1 = machine.root_device().memregion("user1")->base();
+	UINT8* maincpu = machine.root_device().memregion("maincpu")->base();
 	int i;
 
 	for(i=0;i<0x100;i++) {
@@ -264,31 +263,31 @@ static void load9312prom(UINT8 *desc, UINT8 *src, int size)
 static MACHINE_RESET(pdp11ub2)
 {
 	// Load M9312
-	UINT8* user1 = machine.region("consproms")->base() + input_port_read(machine,"CONSPROM") * 0x0400;
-	UINT8* maincpu = machine.region("maincpu")->base();
+	UINT8* user1 = machine.root_device().memregion("consproms")->base() + machine.root_device().ioport("CONSPROM")->read() * 0x0400;
+	UINT8* maincpu = machine.root_device().memregion("maincpu")->base();
 
 	//0165000
 	load9312prom(maincpu + 0165000,user1,0x100);
 
-	UINT8 s1 = input_port_read(machine,"S1");
+	UINT8 s1 = machine.root_device().ioport("S1")->read();
 
 	if (s1 & 0x02) { // if boot enabled
 		UINT16 addr = 0173000;
 		if (s1 & 1) {
 			addr = 0165000;
 		}
-		addr += input_port_read(machine,"S1_2") * 2;
+		addr += machine.root_device().ioport("S1_2")->read() * 2;
 		cpu_set_reg(machine.device("maincpu"), T11_PC, addr);
 	}
 
 	//0173000
-	load9312prom(maincpu + 0173000,machine.region("devproms")->base() + input_port_read(machine,"DEVPROM1") * 0x0200,0x080);
+	load9312prom(maincpu + 0173000,machine.root_device().memregion("devproms")->base() + machine.root_device().ioport("DEVPROM1")->read() * 0x0200,0x080);
 	//0173200
-	load9312prom(maincpu + 0173200,machine.region("devproms")->base() + input_port_read(machine,"DEVPROM2") * 0x0200,0x080);
+	load9312prom(maincpu + 0173200,machine.root_device().memregion("devproms")->base() + machine.root_device().ioport("DEVPROM2")->read() * 0x0200,0x080);
 	//0173400
-	load9312prom(maincpu + 0173400,machine.region("devproms")->base() + input_port_read(machine,"DEVPROM3") * 0x0200,0x080);
+	load9312prom(maincpu + 0173400,machine.root_device().memregion("devproms")->base() + machine.root_device().ioport("DEVPROM3")->read() * 0x0200,0x080);
 	//0173600
-	load9312prom(maincpu + 0173600,machine.region("devproms")->base() + input_port_read(machine,"DEVPROM4") * 0x0200,0x080);
+	load9312prom(maincpu + 0173600,machine.root_device().memregion("devproms")->base() + machine.root_device().ioport("DEVPROM4")->read() * 0x0200,0x080);
 
 }
 
@@ -327,7 +326,6 @@ static MACHINE_CONFIG_START( pdp11, pdp11_state )
 	MCFG_MACHINE_RESET(pdp11)
 
 	/* video hardware */
-	MCFG_FRAGMENT_ADD( generic_terminal )
 	MCFG_GENERIC_TERMINAL_ADD(TERMINAL_TAG, terminal_intf)
 MACHINE_CONFIG_END
 
