@@ -388,10 +388,6 @@ normal keyboards?
 #include "machine/cbmipt.h"
 #include "video/mc6845.h"
 #include "machine/ram.h"
-#include "machine/c2031.h"
-#include "machine/c2040.h"
-#include "machine/c8280.h"
-#include "machine/d9060.h"
 
 /* devices config */
 #include "includes/cbm.h"
@@ -404,28 +400,28 @@ normal keyboards?
  *
  *************************************/
 
-static ADDRESS_MAP_START(pet_mem , AS_PROGRAM, 8)
-	AM_RANGE(0x8000, 0x83ff) AM_MIRROR(0x0c00) AM_RAM AM_BASE_MEMBER(pet_state, m_videoram)
+static ADDRESS_MAP_START(pet_mem , AS_PROGRAM, 8, pet_state )
+	AM_RANGE(0x8000, 0x83ff) AM_MIRROR(0x0c00) AM_RAM AM_SHARE("videoram")
 	AM_RANGE(0xa000, 0xe7ff) AM_ROM
-	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE_MODERN("pia_0", pia6821_device, read, write)
-	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE_MODERN("pia_1", pia6821_device, read, write)
-	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE_MODERN("via6522_0", via6522_device, read, write)
-/*  AM_RANGE(0xe900, 0xe91f) AM_DEVREAD("ieee_bus", cbm_ieee_state)    // for debugging */
+	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE("pia_0", pia6821_device, read, write)
+	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE("pia_1", pia6821_device, read, write)
+	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE("via6522_0", via6522_device, read, write)
+/*  AM_RANGE(0xe900, 0xe91f) AM_DEVREAD_LEGACY("ieee_bus", cbm_ieee_state)    // for debugging */
 	AM_RANGE(0xf000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pet40_mem , AS_PROGRAM, 8)
-	AM_RANGE(0x8000, 0x83ff) AM_MIRROR(0x0c00) AM_RAM AM_BASE_MEMBER(pet_state, m_videoram)
+static ADDRESS_MAP_START( pet40_mem , AS_PROGRAM, 8, pet_state )
+	AM_RANGE(0x8000, 0x83ff) AM_MIRROR(0x0c00) AM_RAM AM_SHARE("videoram")
 	AM_RANGE(0xa000, 0xe7ff) AM_ROM
-	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE_MODERN("pia_0", pia6821_device, read, write)
-	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE_MODERN("pia_1", pia6821_device, read, write)
-	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE_MODERN("via6522_0", via6522_device, read, write)
-	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE("pia_0", pia6821_device, read, write)
+	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE("pia_1", pia6821_device, read, write)
+	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE("via6522_0", via6522_device, read, write)
+	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
 	AM_RANGE(0xf000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pet80_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pet80_mem , AS_PROGRAM, 8, pet_state )
 	AM_RANGE(0x8000, 0x8fff) AM_RAMBANK("bank1")
 	AM_RANGE(0x9000, 0x9fff) AM_RAMBANK("bank2")
 	AM_RANGE(0xa000, 0xafff) AM_RAMBANK("bank3")
@@ -434,11 +430,11 @@ static ADDRESS_MAP_START( pet80_mem , AS_PROGRAM, 8)
 #if 1
 	AM_RANGE(0xe800, 0xefff) AM_RAMBANK("bank7")
 #else
-	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE("pia_0", pia6821_r, pia6821_w)
-	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE("pia_1", pia6821_r, pia6821_w)
-	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE_MODERN("via6522_0", via6522_device, read, write)
-	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE_LEGACY("pia_0", pia6821_r, pia6821_w)
+	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE_LEGACY("pia_1", pia6821_r, pia6821_w)
+	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE("via6522_0", via6522_device, read, write)
+	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
 #endif
 	AM_RANGE(0xf000, 0xffff) AM_READ_BANK("bank8")
 	AM_RANGE(0xf000, 0xffef) AM_WRITE_BANK("bank8")
@@ -461,32 +457,32 @@ ADDRESS_MAP_END
         bit 7    1=enable system latch
 
 */
-static ADDRESS_MAP_START( superpet_mem , AS_PROGRAM, 8)
-	AM_RANGE(0x0000, 0x7fff) AM_RAM AM_SHARE("share1") AM_BASE_MEMBER(pet_state, m_memory)
-	AM_RANGE(0x8000, 0x87ff) AM_RAM AM_SHARE("share2") AM_BASE_MEMBER(pet_state, m_videoram)
+static ADDRESS_MAP_START( superpet_mem , AS_PROGRAM, 8, pet_state )
+	AM_RANGE(0x0000, 0x7fff) AM_RAM AM_SHARE("memory")
+	AM_RANGE(0x8000, 0x87ff) AM_RAM AM_SHARE("videoram")
 	AM_RANGE(0xa000, 0xe7ff) AM_ROM
-	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE_MODERN("pia_0", pia6821_device, read, write)
-	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE_MODERN("pia_1", pia6821_device, read, write)
-	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE_MODERN("via6522_0", via6522_device, read, write)
-	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE("pia_0", pia6821_device, read, write)
+	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE("pia_1", pia6821_device, read, write)
+	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE("via6522_0", via6522_device, read, write)
+	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
 	/* 0xefe0, 0xefe3, mos 6702 */
 	/* 0xeff0, 0xeff3, acia6551 */
-	AM_RANGE(0xeff8, 0xefff) AM_READWRITE(superpet_r, superpet_w)
+	AM_RANGE(0xeff8, 0xefff) AM_READWRITE_LEGACY(superpet_r, superpet_w)
 	AM_RANGE(0xf000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( superpet_m6809_mem, AS_PROGRAM, 8)
-	AM_RANGE(0x0000, 0x7fff) AM_RAM AM_SHARE("share1")	/* same memory as m6502 */
-	AM_RANGE(0x8000, 0x87ff) AM_RAM AM_SHARE("share2")	/* same memory as m6502 */
+static ADDRESS_MAP_START( superpet_m6809_mem, AS_PROGRAM, 8, pet_state )
+	AM_RANGE(0x0000, 0x7fff) AM_RAM AM_SHARE("memory")	/* same memory as m6502 */
+	AM_RANGE(0x8000, 0x87ff) AM_RAM AM_SHARE("videoram")	/* same memory as m6502 */
     AM_RANGE(0x9000, 0x9fff) AM_RAMBANK("bank1")	/* 64 kbyte ram turned in */
 	AM_RANGE(0xa000, 0xe7ff) AM_ROM
-	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE_MODERN("pia_0", pia6821_device, read, write)
-	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE_MODERN("pia_1", pia6821_device, read, write)
-	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE_MODERN("via6522_0", via6522_device, read, write)
-	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
-	AM_RANGE(0xeff8, 0xefff) AM_READWRITE(superpet_r, superpet_w)
+	AM_RANGE(0xe810, 0xe813) AM_DEVREADWRITE("pia_0", pia6821_device, read, write)
+	AM_RANGE(0xe820, 0xe823) AM_DEVREADWRITE("pia_1", pia6821_device, read, write)
+	AM_RANGE(0xe840, 0xe84f) AM_DEVREADWRITE("via6522_0", via6522_device, read, write)
+	AM_RANGE(0xe880, 0xe880) AM_DEVWRITE("crtc", mc6845_device, address_w)
+	AM_RANGE(0xe881, 0xe881) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE(0xeff8, 0xefff) AM_READWRITE_LEGACY(superpet_r, superpet_w)
 	AM_RANGE(0xf000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
@@ -640,26 +636,6 @@ static VIDEO_START( pet_crtc )
 {
 }
 
-static SCREEN_UPDATE( pet_crtc )
-{
-	mc6845_device *mc6845 = screen->machine().device<mc6845_device>("crtc");
-	mc6845->update( bitmap, cliprect);
-	return 0;
-}
-
-static SLOT_INTERFACE_START( cbm_ieee488_devices )
-	SLOT_INTERFACE("c2040", C2040)
-	SLOT_INTERFACE("c3040", C3040)
-	SLOT_INTERFACE("c4040", C4040)
-	SLOT_INTERFACE("c8050", C8050)
-	SLOT_INTERFACE("c8250", C8250)
-	SLOT_INTERFACE("sfd1001", SFD1001)
-	SLOT_INTERFACE("c2031", C2031)
-	SLOT_INTERFACE("c8280", C8280)
-	SLOT_INTERFACE("d9060", D9060)
-	SLOT_INTERFACE("d9090", D9090)
-SLOT_INTERFACE_END
-
 static IEEE488_INTERFACE( ieee488_intf )
 {
 	DEVCB_NULL,
@@ -691,10 +667,9 @@ static MACHINE_CONFIG_START( pet_general, pet_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(320, 200)
 	MCFG_SCREEN_VISIBLE_AREA(0, 320 - 1, 0, 200 - 1)
-	MCFG_SCREEN_UPDATE( pet )
+	MCFG_SCREEN_UPDATE_STATIC( pet )
 
 	MCFG_GFXDECODE( pet )
 	MCFG_PALETTE_LENGTH(ARRAY_LENGTH(pet_palette) / 3)
@@ -723,11 +698,7 @@ static MACHINE_CONFIG_DERIVED( pet, pet_general )
 	MCFG_RAM_EXTRA_OPTIONS("8K,16K")
 
 	/* IEEE bus */
-	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	MCFG_IEEE488_SLOT_ADD("ieee8", 8, cbm_ieee488_devices, "c4040", NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee9", 9, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee10", 10, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee11", 11, cbm_ieee488_devices, NULL, NULL)
+	MCFG_CBM_IEEE488_ADD(ieee488_intf, "c4040")
 MACHINE_CONFIG_END
 
 
@@ -745,11 +716,7 @@ static MACHINE_CONFIG_DERIVED( pet2001, pet_general )
 	MCFG_RAM_EXTRA_OPTIONS("4K")
 
 	/* IEEE bus */
-	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	MCFG_IEEE488_SLOT_ADD("ieee8", 8, cbm_ieee488_devices, "c4040", NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee9", 9, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee10", 10, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee11", 11, cbm_ieee488_devices, NULL, NULL)
+	MCFG_CBM_IEEE488_ADD(ieee488_intf, "c4040")
 MACHINE_CONFIG_END
 
 
@@ -761,7 +728,7 @@ static MACHINE_CONFIG_DERIVED( pet40, pet )
 
 	MCFG_VIDEO_START( pet_crtc )
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE( pet_crtc )
+	MCFG_SCREEN_UPDATE_DEVICE( "crtc", mc6845_device, screen_update )
 
 	MCFG_FRAGMENT_ADD(pet4_cartslot)
 MACHINE_CONFIG_END
@@ -783,10 +750,9 @@ static MACHINE_CONFIG_DERIVED( pet80, pet_general )
 
     /* video hardware */
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 250)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640 - 1, 0, 250 - 1)
-	MCFG_SCREEN_UPDATE( pet_crtc )
+	MCFG_SCREEN_UPDATE_DEVICE( "crtc", mc6845_device, screen_update )
 
 	MCFG_MC6845_ADD("crtc", MC6845, XTAL_12MHz / 2	/* This is a wild guess and mostly likely incorrect */, crtc_pet80)
 
@@ -802,11 +768,7 @@ static MACHINE_CONFIG_DERIVED( pet80, pet_general )
 	MCFG_RAM_DEFAULT_SIZE("32K")
 
 	/* IEEE bus */
-	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	MCFG_IEEE488_SLOT_ADD("ieee8", 8, cbm_ieee488_devices, "c8050", NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee9", 9, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee10", 10, cbm_ieee488_devices, NULL, NULL)
-	MCFG_IEEE488_SLOT_ADD("ieee11", 11, cbm_ieee488_devices, NULL, NULL)
+	MCFG_CBM_IEEE488_ADD(ieee488_intf, "c8050")
 MACHINE_CONFIG_END
 
 
