@@ -80,93 +80,93 @@
 /* special keys
    red c-ce and reset; warm boot, program NOT lost*/
 
-static ADDRESS_MAP_START( pc1401_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1401_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
 	AM_RANGE( 0x3800, 0x47ff) AM_RAM
-	AM_RANGE( 0x6000, 0x67ff) AM_READWRITE( pc1401_lcd_read, pc1401_lcd_write ) AM_MIRROR(0x1000)
+	AM_RANGE( 0x6000, 0x67ff) AM_READWRITE_LEGACY(pc1401_lcd_read, pc1401_lcd_write ) AM_MIRROR(0x1000)
 	AM_RANGE( 0x8000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1402_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1402_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
 	AM_RANGE( 0x2000, 0x47ff) AM_RAM
-	AM_RANGE( 0x6000, 0x67ff) AM_READWRITE( pc1401_lcd_read, pc1401_lcd_write ) AM_MIRROR(0x1000)
+	AM_RANGE( 0x6000, 0x67ff) AM_READWRITE_LEGACY(pc1401_lcd_read, pc1401_lcd_write ) AM_MIRROR(0x1000)
 	AM_RANGE( 0x8000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1250_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1250_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
 	AM_RANGE( 0x4000, 0x7fff) AM_ROM
 	AM_RANGE( 0xc000, 0xc7ff) AM_RAM // 2KB RAM
-	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1251_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1251_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
 	AM_RANGE( 0x4000, 0x7fff) AM_ROM
 	AM_RANGE( 0xb800, 0xc7ff) AM_RAM // 4KB RAM
-	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1255_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1255_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
 	AM_RANGE( 0x4000, 0x7fff) AM_ROM
 	AM_RANGE( 0xa000, 0xc7ff) AM_RAM // 10KB RAM
-	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+	AM_RANGE( 0xf800, 0xf8ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1260_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1260_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
-	AM_RANGE( 0x2000, 0x20ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
-	//AM_RANGE( 0x2800, 0x28ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+	AM_RANGE( 0x2000, 0x20ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
+	//AM_RANGE( 0x2800, 0x28ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
 	AM_RANGE( 0x5800, 0x67ff) AM_RAM // 4KB RAM
 	AM_RANGE( 0x8000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1261_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1261_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
-	AM_RANGE( 0x2000, 0x20ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
-	//AM_RANGE( 0x2800, 0x28ff) AM_READWRITE( pc1251_lcd_read, pc1251_lcd_write)
+	AM_RANGE( 0x2000, 0x20ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
+	//AM_RANGE( 0x2800, 0x28ff) AM_READWRITE_LEGACY(pc1251_lcd_read, pc1251_lcd_write)
 	AM_RANGE( 0x4000, 0x67ff) AM_RAM // 10KB RAM
 	AM_RANGE( 0x8000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
 
-static ADDRESS_MAP_START( pc1350_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1350_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
-	AM_RANGE( 0x7000, 0x7eff) AM_READWRITE( pc1350_lcd_read, pc1350_lcd_write )
+	AM_RANGE( 0x7000, 0x7eff) AM_READWRITE_LEGACY(pc1350_lcd_read, pc1350_lcd_write )
 	AM_RANGE( 0x8000, 0xffff) AM_ROM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1403_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1403_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
-	AM_RANGE( 0x3000, 0x30bf) AM_READWRITE( pc1403_lcd_read, pc1403_lcd_write )
-	AM_RANGE( 0x3800, 0x3fff) AM_READWRITE( pc1403_asic_read, pc1403_asic_write )
+	AM_RANGE( 0x3000, 0x30bf) AM_READWRITE_LEGACY(pc1403_lcd_read, pc1403_lcd_write )
+	AM_RANGE( 0x3800, 0x3fff) AM_READWRITE_LEGACY(pc1403_asic_read, pc1403_asic_write )
 	AM_RANGE( 0x4000, 0x7fff) AM_ROMBANK("bank1")
 	AM_RANGE( 0xe000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1403h_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( pc1403h_mem , AS_PROGRAM, 8, pc1403_state )
 	AM_RANGE( 0x0000, 0x1fff) AM_ROM
-	AM_RANGE( 0x3000, 0x30bf) AM_READWRITE( pc1403_lcd_read, pc1403_lcd_write )
-	AM_RANGE( 0x3800, 0x3fff) AM_READWRITE( pc1403_asic_read, pc1403_asic_write )
+	AM_RANGE( 0x3000, 0x30bf) AM_READWRITE_LEGACY(pc1403_lcd_read, pc1403_lcd_write )
+	AM_RANGE( 0x3800, 0x3fff) AM_READWRITE_LEGACY(pc1403_asic_read, pc1403_asic_write )
 	AM_RANGE( 0x4000, 0x7fff) AM_ROMBANK("bank1")
 	AM_RANGE( 0x8000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
 
 #if 0
-static ADDRESS_MAP_START( pc1421_readmem , AS_PROGRAM, 8)
-	AM_RANGE( 0x0000, 0x1fff) AM_READ( SMH_ROM )
-	AM_RANGE( 0x3800, 0x47ff) AM_READ( SMH_RAM )
-	AM_RANGE( 0x8000, 0xffff) AM_READ( SMH_ROM )
+static ADDRESS_MAP_START( pc1421_readmem , AS_PROGRAM, 8, pc1403_state )
+	AM_RANGE( 0x0000, 0x1fff) AM_READ_LEGACY(SMH_ROM )
+	AM_RANGE( 0x3800, 0x47ff) AM_READ_LEGACY(SMH_RAM )
+	AM_RANGE( 0x8000, 0xffff) AM_READ_LEGACY(SMH_ROM )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( pc1421_writemem , AS_PROGRAM, 8)
-	AM_RANGE( 0x0000, 0x1fff) AM_WRITE( SMH_ROM )
-	AM_RANGE( 0x2000, 0x37ff) AM_WRITE( SMH_RAM )
-	AM_RANGE( 0x3800, 0x47ff) AM_WRITE( SMH_RAM )
-	AM_RANGE( 0x8000, 0xffff) AM_WRITE( SMH_ROM )
+static ADDRESS_MAP_START( pc1421_writemem , AS_PROGRAM, 8, pc1403_state )
+	AM_RANGE( 0x0000, 0x1fff) AM_WRITE_LEGACY(SMH_ROM )
+	AM_RANGE( 0x2000, 0x37ff) AM_WRITE_LEGACY(SMH_RAM )
+	AM_RANGE( 0x3800, 0x47ff) AM_WRITE_LEGACY(SMH_RAM )
+	AM_RANGE( 0x8000, 0xffff) AM_WRITE_LEGACY(SMH_ROM )
 ADDRESS_MAP_END
 #endif
 
@@ -718,7 +718,8 @@ static const sc61860_cpu_core pc1401_config =
 static MACHINE_CONFIG_FRAGMENT( pocketc )
 	MCFG_QUANTUM_TIME(attotime::from_hz(60))
 
-	MCFG_NVRAM_HANDLER( pc1401 )
+	MCFG_NVRAM_ADD_0FILL("cpu_nvram")
+	MCFG_NVRAM_ADD_0FILL("ram_nvram")
 
 	/*
        aim: show sharp with keyboard
@@ -729,7 +730,6 @@ static MACHINE_CONFIG_FRAGMENT( pocketc )
 	MCFG_SCREEN_ADD("screen", LCD)
 	MCFG_SCREEN_REFRESH_RATE(20)	/* very early and slow lcd */
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(594, 273)
 	MCFG_SCREEN_VISIBLE_AREA(0, 594-1, 0, 273-1)
 //  MCFG_SCREEN_SIZE(640, 273)
@@ -747,10 +747,12 @@ static MACHINE_CONFIG_START( pc1401, pc1401_state )
 	MCFG_CPU_PROGRAM_MAP(pc1401_mem)
 	MCFG_CPU_CONFIG(pc1401_config)
 
+	MCFG_MACHINE_START( pc1401 )
+
 	MCFG_FRAGMENT_ADD(pocketc)
 
 	MCFG_SCREEN_MODIFY("screen")
-	MCFG_SCREEN_UPDATE( pc1401 )
+	MCFG_SCREEN_UPDATE_STATIC( pc1401 )
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( pc1402, pc1401 )
@@ -773,13 +775,13 @@ static MACHINE_CONFIG_START( pc1250, pc1251_state )
 
 	MCFG_FRAGMENT_ADD(pocketc)
 
-	MCFG_NVRAM_HANDLER( pc1251 )
+	MCFG_MACHINE_START( pc1251 )
 
 	/* video hardware */
 	MCFG_SCREEN_MODIFY("screen")
 	MCFG_SCREEN_SIZE(608, 300)
 	MCFG_SCREEN_VISIBLE_AREA(0, 608-1, 0, 300-1)
-	MCFG_SCREEN_UPDATE( pc1251 )
+	MCFG_SCREEN_UPDATE_STATIC( pc1251 )
 	MCFG_GFXDECODE( pc1251 )
 MACHINE_CONFIG_END
 
@@ -793,34 +795,21 @@ static MACHINE_CONFIG_DERIVED( pc1255, pc1250 )
 	MCFG_CPU_PROGRAM_MAP( pc1255_mem)
 MACHINE_CONFIG_END
 
-static NVRAM_HANDLER( pc1260 )
+static MACHINE_START( pc1260 )
 {
 	device_t *main_cpu = machine.device("maincpu");
-	UINT8 *ram = machine.region("maincpu")->base() + 0x4000;
+	UINT8 *ram = machine.root_device().memregion("maincpu")->base() + 0x4000;
 	UINT8 *cpu = sc61860_internal_ram(main_cpu);
 
-	if (read_or_write)
-	{
-		file->write(cpu, 96);
-		file->write(ram, 0x2800);
-	}
-	else if (file)
-	{
-		file->read(cpu, 96);
-		file->read(ram, 0x2800);
-	}
-	else
-	{
-		memset(cpu, 0, 96);
-		memset(ram, 0, 0x2800);
-	}
+	machine.device<nvram_device>("cpu_nvram")->set_base(cpu, 96);
+	machine.device<nvram_device>("ram_nvram")->set_base(ram, 0x2800);
 }
 
 static MACHINE_CONFIG_DERIVED( pc1260, pc1250 )
 	MCFG_CPU_MODIFY( "maincpu" )
 	MCFG_CPU_PROGRAM_MAP( pc1260_mem)
 
-	MCFG_NVRAM_HANDLER( pc1260 )
+	MCFG_MACHINE_START( pc1260 )
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( pc1261, pc1260 )
@@ -844,7 +833,6 @@ static MACHINE_CONFIG_START( pc1350, pc1350_state )
 	MCFG_FRAGMENT_ADD( pocketc )
 
 	MCFG_MACHINE_START( pc1350 )
-	MCFG_NVRAM_HANDLER( pc1350 )
 
 	/*
        aim: show sharp with keyboard
@@ -854,7 +842,7 @@ static MACHINE_CONFIG_START( pc1350, pc1350_state )
 	MCFG_SCREEN_MODIFY("screen")
 	MCFG_SCREEN_SIZE(640, 252)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 252-1)
-	MCFG_SCREEN_UPDATE( pc1350 )
+	MCFG_SCREEN_UPDATE_STATIC( pc1350 )
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
@@ -881,7 +869,7 @@ static MACHINE_CONFIG_START( pc1403, pc1403_state )
 	MCFG_CPU_PROGRAM_MAP( pc1403_mem)
 	MCFG_CPU_CONFIG( pc1403_config )
 
-	MCFG_NVRAM_HANDLER( pc1403 )
+	MCFG_MACHINE_START( pc1403 )
 
 	/*
        aim: show sharp with keyboard
@@ -893,7 +881,7 @@ static MACHINE_CONFIG_START( pc1403, pc1403_state )
 	MCFG_SCREEN_VISIBLE_AREA(0, 848-1, 0, 320-1)
 //  MCFG_SCREEN_SIZE(848, 361)
 //  MCFG_SCREEN_VISIBLE_AREA(0, 848-1, 0, 361-1)
-	MCFG_SCREEN_UPDATE( pc1403 )
+	MCFG_SCREEN_UPDATE_STATIC( pc1403 )
 
 	MCFG_VIDEO_START( pc1403 )
 MACHINE_CONFIG_END
