@@ -74,7 +74,6 @@ Notes:
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -93,7 +92,7 @@ public:
 		{ }
 
 	required_device<cpu_device> m_maincpu;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 
 	UINT8 m_bank[7];
 	UINT8 m_kb_matrix;
@@ -103,7 +102,7 @@ public:
 	UINT8 *m_vram;
 
 	virtual void machine_start();
-	bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 
 	DECLARE_READ8_MEMBER( bankswitch_r );
 	DECLARE_WRITE8_MEMBER( bankswitch_w );
@@ -126,36 +125,36 @@ WRITE8_MEMBER( prestige_state::bankswitch_w )
 	switch (offset)
 	{
 	case 0:
-		memory_set_bank(machine(), "bank1", data & 0x3f);
+		membank("bank1")->set_entry(data & 0x3f);
 		break;
 
 	case 1:
 		if (m_bank[5] & 0x08)
-			memory_set_bank(machine(), "bank2", 0x40 + (data & 1));
+			membank("bank2")->set_entry(0x40 + (data & 1));
 		else
-			memory_set_bank(machine(), "bank2", data & 0x3f);
+			membank("bank2")->set_entry(data & 0x3f);
 		break;
 
 	case 2:
 		if (m_bank[5] & 0x04)
-			memory_set_bank(machine(), "bank3", 0x40 + (data & 1));
+			membank("bank3")->set_entry(0x40 + (data & 1));
 		else
-			memory_set_bank(machine(), "bank3", data & 0x3f);
+			membank("bank3")->set_entry(data & 0x3f);
 		break;
 
 	case 3:
 		if (m_bank[5] & 0x02)
-			memory_set_bank(machine(), "bank4", 0x04 + (data & 0x03));
+			membank("bank4")->set_entry(0x04 + (data & 0x03));
 		else
-			memory_set_bank(machine(), "bank4", data & 0x03);
+			membank("bank4")->set_entry(data & 0x03);
 		break;
 
 	case 4:
-		memory_set_bank(machine(), "bank5", data & 0x03);
+		membank("bank5")->set_entry(data & 0x03);
 		break;
 
 	case 5:
-		if (input_port_read(machine(), "CART_TYPE") == 0x01)
+		if (ioport("CART_TYPE")->read() == 0x01)
 		{
 			//cartridge memory is writable
 			if (data & 0x08)
@@ -200,7 +199,7 @@ READ8_MEMBER( prestige_state::kb_r )
 
 	for (int line=0; line<8; line++)
 		if (!(m_kb_matrix & (1<<line)))
-			data &= input_port_read(machine(), bitnames[offset][line]);
+			data &= ioport(bitnames[offset][line])->read();
 
 	return data;
 }
@@ -217,10 +216,10 @@ READ8_MEMBER( prestige_state::mouse_r )
 	switch( offset )
 	{
 		case 0:		//x-axis
-			data = (input_port_read(machine(), "MOUSEX") - m_mousex);
+			data = (ioport("MOUSEX")->read() - m_mousex);
 			break;
 		case 1:		//y-axis
-			data = (input_port_read(machine(), "MOUSEY") - m_mousey);
+			data = (ioport("MOUSEY")->read() - m_mousey);
 			break;
 	}
 
@@ -232,10 +231,10 @@ WRITE8_MEMBER( prestige_state::mouse_w )
 	switch( offset )
 	{
 		case 0:		//x-axis
-			m_mousex = input_port_read(machine(), "MOUSEX");
+			m_mousex = ioport("MOUSEX")->read();
 			break;
 		case 1:		//y-axis
-			m_mousey = input_port_read(machine(), "MOUSEY");
+			m_mousey = ioport("MOUSEY")->read();
 			break;
 	}
 }
@@ -415,25 +414,25 @@ static IRQ_CALLBACK( prestige_int_ack )
 
 void prestige_state::machine_start()
 {
-	UINT8 *ram = ram_get_ptr(m_ram);
-	memset(ram, 0x00, ram_get_size(m_ram));
+	UINT8 *ram = m_ram->pointer();
+	memset(ram, 0x00, m_ram->size());
 
 	device_set_irq_callback(m_maincpu, prestige_int_ack);
 
-	memory_configure_bank(machine(), "bank1", 0, 64, machine().region("maincpu")->base(), 0x4000);
-	memory_configure_bank(machine(), "bank2", 0, 64, machine().region("maincpu")->base(), 0x4000);
-	memory_configure_bank(machine(), "bank2", 64, 2, machine().region("cart")->base(), 0x4000);
-	memory_configure_bank(machine(), "bank3", 0, 64, machine().region("maincpu")->base(), 0x4000);
-	memory_configure_bank(machine(), "bank3", 64, 2, machine().region("cart")->base(), 0x4000);
-	memory_configure_bank(machine(), "bank4", 0, 4, ram, 0x2000);
-	memory_configure_bank(machine(), "bank4", 4, 4, machine().region("cart")->base(), 0x2000);
-	memory_configure_bank(machine(), "bank5", 0, 4, ram, 0x2000);
+	membank("bank1")->configure_entries(0, 64, memregion("maincpu")->base(), 0x4000);
+	membank("bank2")->configure_entries(0, 64, memregion("maincpu")->base(), 0x4000);
+	membank("bank2")->configure_entries(64, 2, memregion("cart")->base(), 0x4000);
+	membank("bank3")->configure_entries(0, 64, memregion("maincpu")->base(), 0x4000);
+	membank("bank3")->configure_entries(64, 2, memregion("cart")->base(), 0x4000);
+	membank("bank4")->configure_entries(0, 4, ram, 0x2000);
+	membank("bank4")->configure_entries(4, 4, memregion("cart")->base(), 0x2000);
+	membank("bank5")->configure_entries(0, 4, ram, 0x2000);
 
-	memory_set_bank(machine(), "bank1", 0);
-	memory_set_bank(machine(), "bank2", 0);
-	memory_set_bank(machine(), "bank3", 0);
-	memory_set_bank(machine(), "bank4", 0);
-	memory_set_bank(machine(), "bank5", 0);
+	membank("bank1")->set_entry(0);
+	membank("bank2")->set_entry(0);
+	membank("bank3")->set_entry(0);
+	membank("bank4")->set_entry(0);
+	membank("bank5")->set_entry(0);
 
 	//pointer to the videoram
 	m_vram = ram;
@@ -445,7 +444,7 @@ static PALETTE_INIT( prestige )
 	palette_set_color(machine, 1, MAKE_RGB(16, 37, 84));
 }
 
-bool prestige_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 prestige_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	UINT16 addr = 0;
 
@@ -457,7 +456,7 @@ bool prestige_state::screen_update(screen_device &screen, bitmap_t &bitmap, cons
 
 			for (int x = 0; x < 8; x++)
 			{
-				*BITMAP_ADDR16(&bitmap, y, (sx * 8) + x) = BIT(data, 7);
+				bitmap.pix16(y, (sx * 8) + x) = BIT(data, 7);
 
 				data <<= 1;
 			}
@@ -486,7 +485,7 @@ static MACHINE_CONFIG_START( prestige, prestige_state )
     MCFG_SCREEN_ADD("screen", LCD)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(prestige_state, screen_update)
 	MCFG_SCREEN_SIZE( 240, 100 )
 	MCFG_SCREEN_VISIBLE_AREA( 0, 240-1, 0, 100-1 )
 
