@@ -15,6 +15,7 @@
 #include "imagedev/cassette.h"
 #include "machine/tf20.h"
 #include "machine/ram.h"
+#include "machine/nvram.h"
 #include "px4.lh"
 
 
@@ -55,7 +56,7 @@ static const int receive_rate[] = { 2112, 1536, 768, 384, 192, 96, 48, 24, 3072,
     MACROS
 ***************************************************************************/
 
-#define ART_TX_ENABLED	(BIT(px4->m_artcr, 0))
+#define ART_TX_ENABLED	(BIT(m_artcr, 0))
 #define ART_RX_ENABLED	(BIT(px4->m_artcr, 2))
 
 #define ART_DATA		(BIT(px4->m_artmr, 2))	/* number of data bits, 7 or 8 */
@@ -75,7 +76,7 @@ public:
 		: driver_device(mconfig, type, tag) { }
 
 	/* internal ram */
-	device_t *m_ram;
+	ram_device *m_ram;
 
 	/* gapnit register */
 	UINT8 m_ctrl1;
@@ -113,7 +114,7 @@ public:
 	UINT8 m_interrupt_status;
 
 	/* centronics printer */
-	device_t *m_printer;
+	centronics_device *m_centronics;
 
 	/* external ramdisk */
 	offs_t m_ramdisk_address;
@@ -127,6 +128,37 @@ public:
 	/* external devices */
 	device_t *m_sio_device;
 	device_t *m_rs232c_device;
+	DECLARE_READ8_MEMBER(px4_icrlc_r);
+	DECLARE_WRITE8_MEMBER(px4_ctrl1_w);
+	DECLARE_READ8_MEMBER(px4_icrhc_r);
+	DECLARE_WRITE8_MEMBER(px4_cmdr_w);
+	DECLARE_READ8_MEMBER(px4_icrlb_r);
+	DECLARE_WRITE8_MEMBER(px4_ctrl2_w);
+	DECLARE_READ8_MEMBER(px4_icrhb_r);
+	DECLARE_READ8_MEMBER(px4_isr_r);
+	DECLARE_WRITE8_MEMBER(px4_ier_w);
+	DECLARE_READ8_MEMBER(px4_str_r);
+	DECLARE_WRITE8_MEMBER(px4_bankr_w);
+	DECLARE_READ8_MEMBER(px4_sior_r);
+	DECLARE_WRITE8_MEMBER(px4_sior_w);
+	DECLARE_WRITE8_MEMBER(px4_vadr_w);
+	DECLARE_WRITE8_MEMBER(px4_yoff_w);
+	DECLARE_WRITE8_MEMBER(px4_fr_w);
+	DECLARE_WRITE8_MEMBER(px4_spur_w);
+	DECLARE_READ8_MEMBER(px4_ctgif_r);
+	DECLARE_WRITE8_MEMBER(px4_ctgif_w);
+	DECLARE_READ8_MEMBER(px4_artdir_r);
+	DECLARE_WRITE8_MEMBER(px4_artdor_w);
+	DECLARE_READ8_MEMBER(px4_artsr_r);
+	DECLARE_WRITE8_MEMBER(px4_artmr_w);
+	DECLARE_READ8_MEMBER(px4_iostr_r);
+	DECLARE_WRITE8_MEMBER(px4_artcr_w);
+	DECLARE_WRITE8_MEMBER(px4_swr_w);
+	DECLARE_WRITE8_MEMBER(px4_ioctlr_w);
+	DECLARE_WRITE8_MEMBER(px4_ramdisk_address_w);
+	DECLARE_READ8_MEMBER(px4_ramdisk_data_r);
+	DECLARE_WRITE8_MEMBER(px4_ramdisk_data_w);
+	DECLARE_READ8_MEMBER(px4_ramdisk_control_r);
 };
 
 
@@ -316,154 +348,144 @@ static TIMER_DEVICE_CALLBACK( frc_tick )
 }
 
 /* input capture register low command trigger */
-static READ8_HANDLER( px4_icrlc_r )
+READ8_MEMBER(px4_state::px4_icrlc_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_icrlc_r\n", space->machine().describe_context());
+		logerror("%s: px4_icrlc_r\n", machine().describe_context());
 
 	/* latch value */
-	px4->m_frc_latch = px4->m_frc_value;
+	m_frc_latch = m_frc_value;
 
-	return px4->m_frc_latch & 0xff;
+	return m_frc_latch & 0xff;
 }
 
 /* control register 1 */
-static WRITE8_HANDLER( px4_ctrl1_w )
+WRITE8_MEMBER(px4_state::px4_ctrl1_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 	int baud;
 
 	if (VERBOSE)
-		logerror("%s: px4_ctrl1_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_ctrl1_w (0x%02x)\n", machine().describe_context(), data);
 
 	/* baudrate generator */
 	baud = data >> 4;
 
 	if (baud <= 12)
 	{
-		px4->m_transmit_timer->adjust(attotime::zero, 0, attotime::from_hz(XTAL_7_3728MHz/2/transmit_rate[baud]));
-		px4->m_receive_timer->adjust(attotime::zero, 0, attotime::from_hz(XTAL_7_3728MHz/2/receive_rate[baud]));
+		m_transmit_timer->adjust(attotime::zero, 0, attotime::from_hz(XTAL_7_3728MHz/2/transmit_rate[baud]));
+		m_receive_timer->adjust(attotime::zero, 0, attotime::from_hz(XTAL_7_3728MHz/2/receive_rate[baud]));
 	}
 
-	px4->m_ctrl1 = data;
+	m_ctrl1 = data;
 }
 
 /* input capture register high command trigger */
-static READ8_HANDLER( px4_icrhc_r )
+READ8_MEMBER(px4_state::px4_icrhc_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_icrhc_r\n", space->machine().describe_context());
+		logerror("%s: px4_icrhc_r\n", machine().describe_context());
 
-	return (px4->m_frc_latch >> 8) & 0xff;
+	return (m_frc_latch >> 8) & 0xff;
 }
 
 /* command register */
-static WRITE8_HANDLER( px4_cmdr_w )
+WRITE8_MEMBER(px4_state::px4_cmdr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_cmdr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_cmdr_w (0x%02x)\n", machine().describe_context(), data);
 
 	/* clear overflow interrupt? */
 	if (BIT(data, 2))
 	{
-		px4->m_isr &= ~INT3_OVF;
-		gapnit_interrupt(space->machine());
+		m_isr &= ~INT3_OVF;
+		gapnit_interrupt(machine());
 	}
 }
 
 /* input capture register low barcode trigger */
-static READ8_HANDLER( px4_icrlb_r )
+READ8_MEMBER(px4_state::px4_icrlb_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_icrlb_r\n", space->machine().describe_context());
+		logerror("%s: px4_icrlb_r\n", machine().describe_context());
 
-	return px4->m_icrb & 0xff;
+	return m_icrb & 0xff;
 }
 
 /* control register 2 */
-static WRITE8_HANDLER( px4_ctrl2_w )
+WRITE8_MEMBER(px4_state::px4_ctrl2_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_ctrl2_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_ctrl2_w (0x%02x)\n", machine().describe_context(), data);
 
 	/* bit 0, MIC, cassette output */
-	px4->m_ext_cas->output( BIT(data, 0) ? -1.0 : +1.0);
+	m_ext_cas->output( BIT(data, 0) ? -1.0 : +1.0);
 
 	/* bit 1, RMT, cassette motor */
 	if (BIT(data, 1))
 	{
-		px4->m_ext_cas->change_state(CASSETTE_MOTOR_ENABLED,CASSETTE_MASK_MOTOR);
-		px4->m_ext_cas_timer->adjust(attotime::zero, 0, attotime::from_hz(44100));
+		m_ext_cas->change_state(CASSETTE_MOTOR_ENABLED,CASSETTE_MASK_MOTOR);
+		m_ext_cas_timer->adjust(attotime::zero, 0, attotime::from_hz(44100));
 	}
 	else
 	{
-		px4->m_ext_cas->change_state(CASSETTE_MOTOR_DISABLED,CASSETTE_MASK_MOTOR);
-		px4->m_ext_cas_timer->adjust(attotime::zero);
+		m_ext_cas->change_state(CASSETTE_MOTOR_DISABLED,CASSETTE_MASK_MOTOR);
+		m_ext_cas_timer->adjust(attotime::zero);
 	}
 }
 
 /* input capture register high barcode trigger */
-static READ8_HANDLER( px4_icrhb_r )
+READ8_MEMBER(px4_state::px4_icrhb_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_icrhb_r\n", space->machine().describe_context());
+		logerror("%s: px4_icrhb_r\n", machine().describe_context());
 
 	/* clear icf interrupt */
-	px4->m_isr &= ~INT2_ICF;
-	gapnit_interrupt(space->machine());
+	m_isr &= ~INT2_ICF;
+	gapnit_interrupt(machine());
 
-	return (px4->m_icrb >> 8) & 0xff;
+	return (m_icrb >> 8) & 0xff;
 }
 
 /* interrupt status register */
-static READ8_HANDLER( px4_isr_r )
+READ8_MEMBER(px4_state::px4_isr_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_isr_r\n", space->machine().describe_context());
+		logerror("%s: px4_isr_r\n", machine().describe_context());
 
-	return px4->m_isr;
+	return m_isr;
 }
 
 /* interrupt enable register */
-static WRITE8_HANDLER( px4_ier_w )
+WRITE8_MEMBER(px4_state::px4_ier_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_ier_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_ier_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_ier = data;
-	gapnit_interrupt(space->machine());
+	m_ier = data;
+	gapnit_interrupt(machine());
 }
 
 /* status register */
-static READ8_HANDLER( px4_str_r )
+READ8_MEMBER(px4_state::px4_str_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 	UINT8 result = 0;
 
 	if (VERBOSE)
-		logerror("%s: px4_str_r\n", space->machine().describe_context());
+		logerror("%s: px4_str_r\n", machine().describe_context());
 
-	result |= (px4->m_ext_cas)->input() > 0 ? 1 : 0;
+	result |= (m_ext_cas)->input() > 0 ? 1 : 0;
 	result |= 1 << 1;	/* BCRD, barcode reader input */
 	result |= 1 << 2;	/* RDY signal from 7805 */
 	result |= 1 << 3;	/* RDYSIO, enable access to the 7805 */
-	result |= px4->m_bankr & 0xf0;	/* bit 4-7, BANK - memory bank */
+	result |= m_bankr & 0xf0;	/* bit 4-7, BANK - memory bank */
 
 	return result;
 }
@@ -471,40 +493,39 @@ static READ8_HANDLER( px4_str_r )
 /* helper function to map rom capsules */
 static void install_rom_capsule(address_space *space, int size, const char *region)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
+	px4_state *state = space->machine().driver_data<px4_state>();
 
 	/* ram, part 1 */
 	space->install_readwrite_bank(0x0000, 0xdfff - size, "bank1");
-	memory_set_bankptr(space->machine(), "bank1", ram_get_ptr(px4->m_ram));
+	state->membank("bank1")->set_base(state->m_ram->pointer());
 
 	/* actual rom data, part 1 */
 	space->install_read_bank(0xe000 - size, 0xffff - size, "bank2");
 	space->nop_write(0xe000 - size, 0xffff - size);
-	memory_set_bankptr(space->machine(), "bank2", space->machine().region(region)->base() + (size - 0x2000));
+	state->membank("bank2")->set_base(space->machine().root_device().memregion(region)->base() + (size - 0x2000));
 
 	/* rom data, part 2 */
 	if (size != 0x2000)
 	{
 		space->install_read_bank(0x10000 - size, 0xdfff, "bank3");
 		space->nop_write(0x10000 - size, 0xdfff);
-		memory_set_bankptr(space->machine(), "bank3", space->machine().region(region)->base());
+		state->membank("bank3")->set_base(state->memregion(region)->base());
 	}
 
 	/* ram, continued */
 	space->install_readwrite_bank(0xe000, 0xffff, "bank4");
-	memory_set_bankptr(space->machine(), "bank4", ram_get_ptr(px4->m_ram) + 0xe000);
+	state->membank("bank4")->set_base(state->m_ram->pointer() + 0xe000);
 }
 
 /* bank register */
-static WRITE8_HANDLER( px4_bankr_w )
+WRITE8_MEMBER(px4_state::px4_bankr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
-	address_space *space_program = space->machine().device("maincpu")->memory().space(AS_PROGRAM);
+	address_space *space_program = machine().device("maincpu")->memory().space(AS_PROGRAM);
 
 	if (VERBOSE)
-		logerror("%s: px4_bankr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_bankr_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_bankr = data;
+	m_bankr = data;
 
 	/* bank switch */
 	switch (data >> 4)
@@ -513,15 +534,15 @@ static WRITE8_HANDLER( px4_bankr_w )
 		/* system bank */
 		space_program->install_read_bank(0x0000, 0x7fff, "bank1");
 		space_program->nop_write(0x0000, 0x7fff);
-		memory_set_bankptr(space->machine(), "bank1", space->machine().region("os")->base());
+		membank("bank1")->set_base(machine().root_device().memregion("os")->base());
 		space_program->install_readwrite_bank(0x8000, 0xffff, "bank2");
-		memory_set_bankptr(space->machine(), "bank2", ram_get_ptr(px4->m_ram) + 0x8000);
+		membank("bank2")->set_base(m_ram->pointer() + 0x8000);
 		break;
 
 	case 0x04:
 		/* memory */
 		space_program->install_readwrite_bank(0x0000, 0xffff, "bank1");
-		memory_set_bankptr(space->machine(), "bank1", ram_get_ptr(px4->m_ram));
+		membank("bank1")->set_base(m_ram->pointer());
 		break;
 
 	case 0x08: install_rom_capsule(space_program, 0x2000, "capsule1"); break;
@@ -539,25 +560,23 @@ static WRITE8_HANDLER( px4_bankr_w )
 }
 
 /* serial io register */
-static READ8_HANDLER( px4_sior_r )
+READ8_MEMBER(px4_state::px4_sior_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_sior_r 0x%02x\n", space->machine().describe_context(), px4->m_sior);
+		logerror("%s: px4_sior_r 0x%02x\n", machine().describe_context(), m_sior);
 
-	return px4->m_sior;
+	return m_sior;
 }
 
 /* serial io register */
-static WRITE8_HANDLER( px4_sior_w )
+WRITE8_MEMBER(px4_state::px4_sior_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_sior_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_sior_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_sior = data;
+	m_sior = data;
 
 	switch (data)
 	{
@@ -573,24 +592,24 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: Read Status\n");
 
-		if (px4->m_interrupt_status != 0)
+		if (m_interrupt_status != 0)
 		{
 			if (VERBOSE)
-				logerror("> 7508 has interrupts pending: 0x%02x\n", px4->m_interrupt_status);
+				logerror("> 7508 has interrupts pending: 0x%02x\n", m_interrupt_status);
 
 			/* signal the interrupt(s) */
-			px4->m_sior = 0xc1 | px4->m_interrupt_status;
-			px4->m_interrupt_status = 0x00;
+			m_sior = 0xc1 | m_interrupt_status;
+			m_interrupt_status = 0x00;
 		}
-		else if (px4->m_key_status != 0xff)
+		else if (m_key_status != 0xff)
 		{
-			px4->m_sior = px4->m_key_status;
-			px4->m_key_status = 0xff;
+			m_sior = m_key_status;
+			m_key_status = 0xff;
 		}
 		else
 		{
 			/* nothing happenend */
-			px4->m_sior = 0xbf;
+			m_sior = 0xbf;
 		}
 
 		break;
@@ -608,7 +627,7 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: KB Interrupt OFF\n");
 
-		px4->m_key_int_enabled = FALSE;
+		m_key_int_enabled = FALSE;
 		break;
 
 	case 0x16:
@@ -616,7 +635,7 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: KB Interrupt ON\n");
 
-		px4->m_key_int_enabled = TRUE;
+		m_key_int_enabled = TRUE;
 		break;
 
 	case 0x07: if (VERBOSE) logerror("7508 cmd: Clock Read\n"); break;
@@ -628,7 +647,7 @@ static WRITE8_HANDLER( px4_sior_w )
 			logerror("7508 cmd: Power Switch Read\n");
 
 		/* indicate that the power switch is in the "ON" position */
-		px4->m_sior = 0x01;
+		m_sior = 0x01;
 		break;
 
 	case 0x09: if (VERBOSE) logerror("7508 cmd: Alarm Read\n"); break;
@@ -640,7 +659,7 @@ static WRITE8_HANDLER( px4_sior_w )
 
 		if (VERBOSE)
 			logerror("7508 cmd: DIP Switch Read\n");
-		px4->m_sior = input_port_read(space->machine(), "dips");
+		m_sior = ioport("dips")->read();
 		break;
 
 	case 0x0b: if (VERBOSE) logerror("7508 cmd: Stop Key Interrupt disable\n"); break;
@@ -653,7 +672,7 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: 1 sec. Interrupt OFF\n");
 
-		px4->m_one_sec_int_enabled = FALSE;
+		m_one_sec_int_enabled = FALSE;
 		break;
 
 	case 0x1d:
@@ -661,7 +680,7 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: 1 sec. Interrupt ON\n");
 
-		px4->m_one_sec_int_enabled = TRUE;
+		m_one_sec_int_enabled = TRUE;
 		break;
 
 	case 0x0e:
@@ -669,7 +688,7 @@ static WRITE8_HANDLER( px4_sior_w )
 		if (VERBOSE)
 			logerror("7508 cmd: KB Clear\n");
 
-		px4->m_sior = 0xbf;
+		m_sior = 0xbf;
 		break;
 
 	case 0x0f: if (VERBOSE) logerror("7508 cmd: System Reset\n"); break;
@@ -682,39 +701,37 @@ static WRITE8_HANDLER( px4_sior_w )
 ***************************************************************************/
 
 /* vram start address register */
-static WRITE8_HANDLER( px4_vadr_w )
+WRITE8_MEMBER(px4_state::px4_vadr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_vadr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_vadr_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_vadr = data;
+	m_vadr = data;
 }
 
 /* y offset register */
-static WRITE8_HANDLER( px4_yoff_w )
+WRITE8_MEMBER(px4_state::px4_yoff_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_yoff_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_yoff_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_yoff = data;
+	m_yoff = data;
 }
 
 /* frame register */
-static WRITE8_HANDLER( px4_fr_w )
+WRITE8_MEMBER(px4_state::px4_fr_w)
 {
 	if (VERBOSE)
-		logerror("%s: px4_fr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_fr_w (0x%02x)\n", machine().describe_context(), data);
 }
 
 /* speed-up register */
-static WRITE8_HANDLER( px4_spur_w )
+WRITE8_MEMBER(px4_state::px4_spur_w)
 {
 	if (VERBOSE)
-		logerror("%s: px4_spur_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_spur_w (0x%02x)\n", machine().describe_context(), data);
 }
 
 
@@ -726,7 +743,7 @@ static TIMER_CALLBACK( transmit_data )
 {
 	px4_state *px4 = machine.driver_data<px4_state>();
 
-	if (ART_TX_ENABLED)
+	if (BIT(px4->m_artcr, 0))// ART_TX_ENABLED
 	{
 
 	}
@@ -743,86 +760,81 @@ static TIMER_CALLBACK( receive_data )
 }
 
 /* cartridge interface */
-static READ8_HANDLER( px4_ctgif_r )
+READ8_MEMBER(px4_state::px4_ctgif_r)
 {
 	if (VERBOSE)
-		logerror("%s: px4_ctgif_r @ 0x%02x\n", space->machine().describe_context(), offset);
+		logerror("%s: px4_ctgif_r @ 0x%02x\n", machine().describe_context(), offset);
 
 	return 0xff;
 }
 
 /* cartridge interface */
-static WRITE8_HANDLER( px4_ctgif_w )
+WRITE8_MEMBER(px4_state::px4_ctgif_w)
 {
 	if (VERBOSE)
-		logerror("%s: px4_ctgif_w (0x%02x @ 0x%02x)\n", space->machine().describe_context(), data, offset);
+		logerror("%s: px4_ctgif_w (0x%02x @ 0x%02x)\n", machine().describe_context(), data, offset);
 }
 
 /* art data input register */
-static READ8_HANDLER( px4_artdir_r )
+READ8_MEMBER(px4_state::px4_artdir_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_artdir_r\n", space->machine().describe_context());
+		logerror("%s: px4_artdir_r\n", machine().describe_context());
 
-	return px4->m_artdir;
+	return m_artdir;
 }
 
 /* art data output register */
-static WRITE8_HANDLER( px4_artdor_w )
+WRITE8_MEMBER(px4_state::px4_artdor_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_artdor_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_artdor_w (0x%02x)\n", machine().describe_context(), data);
 
 	/* clear ready */
-	px4->m_artsr &= ~ART_TXRDY;
+	m_artsr &= ~ART_TXRDY;
 
-	px4->m_artdor = data;
+	m_artdor = data;
 }
 
 /* art status register */
-static READ8_HANDLER( px4_artsr_r )
+READ8_MEMBER(px4_state::px4_artsr_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 	UINT8 result = 0;
 
 	if (VERBOSE)
-		logerror("%s: px4_artsr_r\n", space->machine().describe_context());
+		logerror("%s: px4_artsr_r\n", machine().describe_context());
 
-	result |= px4_rs232c_dsr(px4->m_rs232c_device) << 7;
+	result |= px4_rs232c_dsr(m_rs232c_device) << 7;
 
-	return result | px4->m_artsr;
+	return result | m_artsr;
 }
 
 /* art mode register */
-static WRITE8_HANDLER( px4_artmr_w )
+WRITE8_MEMBER(px4_state::px4_artmr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_artmr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_artmr_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_artmr = data;
+	m_artmr = data;
 }
 
 /* io status register */
-static READ8_HANDLER( px4_iostr_r )
+READ8_MEMBER(px4_state::px4_iostr_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 	UINT8 result = 0;
 
 	if (VERBOSE)
-		logerror("%s: px4_iostr_r\n", space->machine().describe_context());
+		logerror("%s: px4_iostr_r\n", machine().describe_context());
 
-	result |= centronics_busy_r(px4->m_printer) << 0;
-	result |= !centronics_pe_r(px4->m_printer) << 1;
-	result |= px4_sio_pin(px4->m_sio_device) << 2;
-	result |= px4_sio_rxd(px4->m_sio_device) << 3;
-	result |= px4_rs232c_dcd(px4->m_rs232c_device) << 4;
-	result |= px4_rs232c_cts(px4->m_rs232c_device) << 5;
+	result |= m_centronics->busy_r() << 0;
+	result |= !m_centronics->pe_r() << 1;
+	result |= px4_sio_pin(m_sio_device) << 2;
+	result |= px4_sio_rxd(m_sio_device) << 3;
+	result |= px4_rs232c_dcd(m_rs232c_device) << 4;
+	result |= px4_rs232c_cts(m_rs232c_device) << 5;
 	result |= 1 << 6;	/* bit 6, csel, cartridge option select signal, set to 'other mode' */
 	result |= 0 << 7;	/* bit 7, caud - audio input from cartridge */
 
@@ -830,62 +842,59 @@ static READ8_HANDLER( px4_iostr_r )
 }
 
 /* art command register */
-static WRITE8_HANDLER( px4_artcr_w )
+WRITE8_MEMBER(px4_state::px4_artcr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_artcr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_artcr_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_artcr = data;
+	m_artcr = data;
 
 	/* bit 0, txe - transmit enable */
 	if (!ART_TX_ENABLED)
 	{
 		/* force high when disabled */
-		px4_sio_txd(px4->m_sio_device, ASSERT_LINE);
-		px4_rs232c_txd(px4->m_rs232c_device, ASSERT_LINE);
+		px4_sio_txd(m_sio_device, ASSERT_LINE);
+		px4_rs232c_txd(m_rs232c_device, ASSERT_LINE);
 	}
 
 	/* bit 3, sbrk - break output */
 	if (ART_TX_ENABLED && BIT(data, 3))
 	{
 		/* force low when enabled and transmit enabled */
-		px4_sio_txd(px4->m_sio_device, CLEAR_LINE);
-		px4_rs232c_txd(px4->m_rs232c_device, CLEAR_LINE);
+		px4_sio_txd(m_sio_device, CLEAR_LINE);
+		px4_rs232c_txd(m_rs232c_device, CLEAR_LINE);
 	}
 
 	/* error reset */
 	if (BIT(data, 4))
-		px4->m_artsr &= ~(ART_PE | ART_OE | ART_FE);
+		m_artsr &= ~(ART_PE | ART_OE | ART_FE);
 
-	px4_rs232c_dtr(px4->m_rs232c_device, BIT(data, 1));
-	px4_rs232c_rts(px4->m_rs232c_device, BIT(data, 5));
+	px4_rs232c_dtr(m_rs232c_device, BIT(data, 1));
+	px4_rs232c_rts(m_rs232c_device, BIT(data, 5));
 }
 
 /* switch register */
-static WRITE8_HANDLER( px4_swr_w )
+WRITE8_MEMBER(px4_state::px4_swr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_swr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_swr_w (0x%02x)\n", machine().describe_context(), data);
 
-	px4->m_swr = data;
+	m_swr = data;
 }
 
 /* io control register */
-static WRITE8_HANDLER( px4_ioctlr_w )
+WRITE8_MEMBER(px4_state::px4_ioctlr_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	if (VERBOSE)
-		logerror("%s: px4_ioctlr_w (0x%02x)\n", space->machine().describe_context(), data);
+		logerror("%s: px4_ioctlr_w (0x%02x)\n", machine().describe_context(), data);
 
-	centronics_strobe_w(px4->m_printer, !BIT(data, 0));
-	centronics_prime_w(px4->m_printer, BIT(data, 1));
+	m_centronics->strobe_w(!BIT(data, 0));
+	m_centronics->init_prime_w(BIT(data, 1));
 
-	px4_sio_pout(px4->m_sio_device, BIT(data, 2));
+	px4_sio_pout(m_sio_device, BIT(data, 2));
 
 	/* bit 3, cartridge reset */
 
@@ -919,7 +928,7 @@ static TIMER_DEVICE_CALLBACK( upd7508_1sec_callback )
 static INPUT_CHANGED( key_callback )
 {
 	px4_state *px4 = field.machine().driver_data<px4_state>();
-	UINT32 oldvalue = oldval * field.mask, newvalue = newval * field.mask;
+	UINT32 oldvalue = oldval * field.mask(), newvalue = newval * field.mask();
 	UINT32 delta = oldvalue ^ newvalue;
 	int i, scancode = 0xff, down = 0;
 
@@ -961,74 +970,59 @@ static INPUT_CHANGED( key_callback )
     EXTERNAL RAM-DISK
 ***************************************************************************/
 
-static WRITE8_HANDLER( px4_ramdisk_address_w )
+WRITE8_MEMBER(px4_state::px4_ramdisk_address_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
 	switch (offset)
 	{
-	case 0x00: px4->m_ramdisk_address = (px4->m_ramdisk_address & 0xffff00) | data; break;
-	case 0x01: px4->m_ramdisk_address = (px4->m_ramdisk_address & 0xff00ff) | (data << 8); break;
-	case 0x02: px4->m_ramdisk_address = (px4->m_ramdisk_address & 0x00ffff) | ((data & 0x07) << 16); break;
+	case 0x00: m_ramdisk_address = (m_ramdisk_address & 0xffff00) | data; break;
+	case 0x01: m_ramdisk_address = (m_ramdisk_address & 0xff00ff) | (data << 8); break;
+	case 0x02: m_ramdisk_address = (m_ramdisk_address & 0x00ffff) | ((data & 0x07) << 16); break;
 	}
 }
 
-static READ8_HANDLER( px4_ramdisk_data_r )
+READ8_MEMBER(px4_state::px4_ramdisk_data_r)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 	UINT8 ret = 0xff;
 
-	if (px4->m_ramdisk_address < 0x20000)
+	if (m_ramdisk_address < 0x20000)
 	{
 		/* read from ram */
-		ret = px4->m_ramdisk[px4->m_ramdisk_address];
+		ret = m_ramdisk[m_ramdisk_address];
 	}
-	else if (px4->m_ramdisk_address < 0x40000)
+	else if (m_ramdisk_address < 0x40000)
 	{
 		/* read from rom */
-		ret = space->machine().region("ramdisk")->base()[px4->m_ramdisk_address];
+		ret = memregion("ramdisk")->base()[m_ramdisk_address];
 	}
 
-	px4->m_ramdisk_address = (px4->m_ramdisk_address & 0xffff00) | ((px4->m_ramdisk_address & 0xff) + 1);
+	m_ramdisk_address = (m_ramdisk_address & 0xffff00) | ((m_ramdisk_address & 0xff) + 1);
 
 	return ret;
 }
 
-static WRITE8_HANDLER( px4_ramdisk_data_w )
+WRITE8_MEMBER(px4_state::px4_ramdisk_data_w)
 {
-	px4_state *px4 = space->machine().driver_data<px4_state>();
 
-	if (px4->m_ramdisk_address < 0x20000)
-		px4->m_ramdisk[px4->m_ramdisk_address] = data;
+	if (m_ramdisk_address < 0x20000)
+		m_ramdisk[m_ramdisk_address] = data;
 
-	px4->m_ramdisk_address = (px4->m_ramdisk_address & 0xffff00) | ((px4->m_ramdisk_address & 0xff) + 1);
+	m_ramdisk_address = (m_ramdisk_address & 0xffff00) | ((m_ramdisk_address & 0xff) + 1);
 }
 
-static READ8_HANDLER( px4_ramdisk_control_r )
+READ8_MEMBER(px4_state::px4_ramdisk_control_r)
 {
 	/* bit 7 determines the presence of a ram-disk */
 	return 0x7f;
 }
 
-static NVRAM_HANDLER( px4_ramdisk )
-{
-	px4_state *px4 = machine.driver_data<px4_state>();
-
-	if (read_or_write)
-		file->write(px4->m_ramdisk, 0x20000);
-	else
-		if (file)
-			file->read(px4->m_ramdisk, 0x20000);
-}
-
-
 /***************************************************************************
     VIDEO EMULATION
 ***************************************************************************/
 
-static SCREEN_UPDATE( px4 )
+static SCREEN_UPDATE_IND16( px4 )
 {
-	px4_state *px4 = screen->machine().driver_data<px4_state>();
+	px4_state *px4 = screen.machine().driver_data<px4_state>();
 
 	/* display enabled? */
 	if (BIT(px4->m_yoff, 7))
@@ -1036,7 +1030,7 @@ static SCREEN_UPDATE( px4 )
 		int y, x;
 
 		/* get vram start address */
-		UINT8 *vram = &ram_get_ptr(px4->m_ram)[(px4->m_vadr & 0xf8) << 8];
+		UINT8 *vram = &px4->m_ram->pointer()[(px4->m_vadr & 0xf8) << 8];
 
 		for (y = 0; y < 64; y++)
 		{
@@ -1045,14 +1039,14 @@ static SCREEN_UPDATE( px4 )
 
 			for (x = 0; x < 240/8; x++)
 			{
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 0) = BIT(*vram, 7);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 1) = BIT(*vram, 6);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 2) = BIT(*vram, 5);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 3) = BIT(*vram, 4);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 4) = BIT(*vram, 3);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 5) = BIT(*vram, 2);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 6) = BIT(*vram, 1);
-				*BITMAP_ADDR16(bitmap, row, x * 8 + 7) = BIT(*vram, 0);
+				bitmap.pix16(row, x * 8 + 0) = BIT(*vram, 7);
+				bitmap.pix16(row, x * 8 + 1) = BIT(*vram, 6);
+				bitmap.pix16(row, x * 8 + 2) = BIT(*vram, 5);
+				bitmap.pix16(row, x * 8 + 3) = BIT(*vram, 4);
+				bitmap.pix16(row, x * 8 + 4) = BIT(*vram, 3);
+				bitmap.pix16(row, x * 8 + 5) = BIT(*vram, 2);
+				bitmap.pix16(row, x * 8 + 6) = BIT(*vram, 1);
+				bitmap.pix16(row, x * 8 + 7) = BIT(*vram, 0);
 
 				vram++;
 			}
@@ -1064,7 +1058,7 @@ static SCREEN_UPDATE( px4 )
 	else
 	{
 		/* display is disabled, draw an empty screen */
-		bitmap_fill(bitmap, cliprect, 0);
+		bitmap.fill(0, cliprect);
 	}
 
 	return 0;
@@ -1077,35 +1071,35 @@ static SCREEN_UPDATE( px4 )
 
 static DRIVER_INIT( px4 )
 {
-	px4_state *px4 = machine.driver_data<px4_state>();
+	px4_state *state = machine.driver_data<px4_state>();
 
 	/* find devices */
-	px4->m_ram = machine.device(RAM_TAG);
+	state->m_ram = machine.device<ram_device>(RAM_TAG);
 
 	/* init 7508 */
-	px4->m_one_sec_int_enabled = TRUE;
-	px4->m_key_int_enabled = TRUE;
-	px4->m_alarm_int_enabled = TRUE;
+	state->m_one_sec_int_enabled = TRUE;
+	state->m_key_int_enabled = TRUE;
+	state->m_alarm_int_enabled = TRUE;
 
 	/* art */
-	px4->m_receive_timer = machine.scheduler().timer_alloc(FUNC(receive_data));
-	px4->m_transmit_timer = machine.scheduler().timer_alloc(FUNC(transmit_data));
+	state->m_receive_timer = machine.scheduler().timer_alloc(FUNC(receive_data));
+	state->m_transmit_timer = machine.scheduler().timer_alloc(FUNC(transmit_data));
 
 	/* printer */
-	px4->m_printer = machine.device("centronics");
+	state->m_centronics = machine.device<centronics_device>("centronics");
 
 	/* external cassette or barcode reader */
-	px4->m_ext_cas_timer = machine.scheduler().timer_alloc(FUNC(ext_cassette_read));
-	px4->m_ext_cas = machine.device<cassette_image_device>("extcas");
-	px4->m_ear_last_state = 0;
+	state->m_ext_cas_timer = machine.scheduler().timer_alloc(FUNC(ext_cassette_read));
+	state->m_ext_cas = machine.device<cassette_image_device>("extcas");
+	state->m_ear_last_state = 0;
 
 	/* external devices */
-	px4->m_sio_device = machine.device("floppy");
-	px4->m_rs232c_device = NULL;
+	state->m_sio_device = machine.device("floppy");
+	state->m_rs232c_device = NULL;
 
 	/* map os rom and last half of memory */
-	memory_set_bankptr(machine, "bank1", machine.region("os")->base());
-	memory_set_bankptr(machine, "bank2", ram_get_ptr(px4->m_ram) + 0x8000);
+	state->membank("bank1")->set_base(machine.root_device().memregion("os")->base());
+	state->membank("bank2")->set_base(state->m_ram->pointer() + 0x8000);
 }
 
 static DRIVER_INIT( px4p )
@@ -1125,17 +1119,22 @@ static MACHINE_RESET( px4 )
 	px4->m_artsr = ART_TXRDY | ART_TXEMPTY;
 }
 
+static MACHINE_START( px4_ramdisk )
+{
+	px4_state *px4 = machine.driver_data<px4_state>();
+	machine.device<nvram_device>("nvram")->set_base(px4->m_ramdisk, 0x20000);
+}
 
 /***************************************************************************
     ADDRESS MAPS
 ***************************************************************************/
 
-static ADDRESS_MAP_START( px4_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( px4_mem, AS_PROGRAM, 8, px4_state )
 	AM_RANGE(0x0000, 0x7fff) AM_ROMBANK("bank1")
 	AM_RANGE(0x8000, 0xffff) AM_RAMBANK("bank2")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( px4_io, AS_IO, 8 )
+static ADDRESS_MAP_START( px4_io, AS_IO, 8, px4_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	/* gapnit, 0x00-0x07 */
@@ -1158,13 +1157,13 @@ static ADDRESS_MAP_START( px4_io, AS_IO, 8 )
 	AM_RANGE(0x14, 0x14) AM_READWRITE(px4_artdir_r, px4_artdor_w)
 	AM_RANGE(0x15, 0x15) AM_READWRITE(px4_artsr_r, px4_artmr_w)
 	AM_RANGE(0x16, 0x16) AM_READWRITE(px4_iostr_r, px4_artcr_w)
-	AM_RANGE(0x17, 0x17) AM_DEVWRITE("centronics", centronics_data_w)
+	AM_RANGE(0x17, 0x17) AM_DEVWRITE("centronics", centronics_device, write)
 	AM_RANGE(0x18, 0x18) AM_WRITE(px4_swr_w)
 	AM_RANGE(0x19, 0x19) AM_WRITE(px4_ioctlr_w)
 	AM_RANGE(0x1a, 0x1f) AM_NOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( px4p_io, AS_IO, 8 )
+static ADDRESS_MAP_START( px4p_io, AS_IO, 8, px4_state )
 	AM_IMPORT_FROM(px4_io)
 	AM_RANGE(0x90, 0x92) AM_WRITE(px4_ramdisk_address_w)
 	AM_RANGE(0x93, 0x93) AM_READWRITE(px4_ramdisk_data_r, px4_ramdisk_data_w)
@@ -1368,10 +1367,9 @@ static MACHINE_CONFIG_START( px4, px4_state )
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", LCD)
 	MCFG_SCREEN_REFRESH_RATE(72)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(240, 64)
 	MCFG_SCREEN_VISIBLE_AREA(0, 239, 0, 63)
-	MCFG_SCREEN_UPDATE(px4)
+	MCFG_SCREEN_UPDATE_STATIC(px4)
 
 	MCFG_DEFAULT_LAYOUT(layout_px4)
 
@@ -1386,7 +1384,7 @@ static MACHINE_CONFIG_START( px4, px4_state )
 	MCFG_RAM_DEFAULT_SIZE("64k")
 
 	/* centronics printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	/* external cassette */
 	MCFG_CASSETTE_ADD("extcas", px4_cassette_interface)
@@ -1406,7 +1404,8 @@ static MACHINE_CONFIG_DERIVED( px4p, px4 )
 	MCFG_CPU_MODIFY("maincpu")
 	MCFG_CPU_IO_MAP(px4p_io)
 
-	MCFG_NVRAM_HANDLER(px4_ramdisk)
+	MCFG_MACHINE_START(px4_ramdisk)
+	MCFG_NVRAM_ADD_0FILL("nvram")
 
 	MCFG_PALETTE_INIT(px4p)
 
