@@ -3,12 +3,39 @@
         Pyldin-601
 
         12/05/2009 Skeleton driver.
+        22/04/2012 Added sound, fixed keyboard, marked as working [Robbbert]
+
+        ToDo?
+        - PYL601 - command 'MODE80' does nothing
+
+        - PYL601a - most software looks odd (unplayable) because of the
+          different design of the screen.
+        - PYL601A - command 'MODE40' doesn't go to 40-columns, instead
+          there is a space between each letter.
+
+
+        The BASIC
+        - to get back to dos, enter SYSTEM
+        - It has its own internal monitor: MON to enter, Q to exit.
+
+
+
+':maincpu' (F013): unmapped program memory write to E628 = 00 & FF
+':maincpu' (FCBC): unmapped program memory write to E636 = 00 & FF
+':maincpu' (FCBC): unmapped program memory write to E637 = 00 & FF
+':maincpu' (FCC2): unmapped program memory write to E634 = 07 & FF
+':maincpu' (FCC2): unmapped program memory write to E635 = FF & FF
+':maincpu' (FCC7): unmapped program memory write to E637 = 34 & FF
+':maincpu' (FCCC): unmapped program memory write to E637 = 3C & FF
+':maincpu' (FCCF): unmapped program memory write to E636 = 3C & FF
+':maincpu' (FCD3): unmapped program memory read from E634 & FF
 
 ****************************************************************************/
 
 #include "emu.h"
 #include "cpu/m6800/m6800.h"
 #include "video/mc6845.h"
+#include "sound/speaker.h"
 #include "imagedev/flopdrv.h"
 #include "formats/basicdsk.h"
 #include "machine/upd765.h"
@@ -19,7 +46,11 @@ class pyl601_state : public driver_device
 {
 public:
 	pyl601_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+	m_speaker(*this, SPEAKER_TAG),
+	m_fdc(*this, "upd765"),
+	m_ram(*this, RAM_TAG)
+	{ }
 
 	UINT8 m_rom_page;
 	UINT32 m_vdisk_addr;
@@ -28,100 +59,110 @@ public:
 	UINT8 m_video_mode;
 	UINT8 m_tick50_mark;
 	UINT8 m_floppy_ctrl;
+	DECLARE_READ8_MEMBER(rom_page_r);
+	DECLARE_WRITE8_MEMBER(rom_page_w);
+	DECLARE_WRITE8_MEMBER(vdisk_page_w);
+	DECLARE_WRITE8_MEMBER(vdisk_h_w);
+	DECLARE_WRITE8_MEMBER(vdisk_l_w);
+	DECLARE_WRITE8_MEMBER(vdisk_data_w);
+	DECLARE_READ8_MEMBER(vdisk_data_r);
+	DECLARE_READ8_MEMBER(keyboard_r);
+	DECLARE_READ8_MEMBER(keycheck_r);
+	DECLARE_WRITE8_MEMBER(video_mode_w);
+	DECLARE_READ8_MEMBER(video_mode_r);
+	DECLARE_READ8_MEMBER(timer_r);
+	DECLARE_WRITE8_MEMBER(speaker_w);
+	DECLARE_WRITE8_MEMBER(led_w);
+	DECLARE_WRITE8_MEMBER(floppy_w);
+	DECLARE_READ8_MEMBER(floppy_r);
+	UINT8 selectedline(UINT16 data);
+	required_device<device_t> m_speaker;
+	required_device<device_t> m_fdc;
+	required_device<ram_device> m_ram;
 };
 
 
 
-static READ8_HANDLER (rom_page_r)
+READ8_MEMBER(pyl601_state::rom_page_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	return state->m_rom_page;
+	return m_rom_page;
 }
 
-static WRITE8_HANDLER (rom_page_w)
+WRITE8_MEMBER(pyl601_state::rom_page_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	state->m_rom_page =data;
+	m_rom_page = data;
 	if (data & 8)
 	{
 		int chip = (data >> 4) % 5;
 		int page = data & 7;
-		memory_set_bankptr(space->machine(), "bank2", space->machine().region("romdisk")->base() + chip*0x10000 + page * 0x2000);
+		membank("bank2")->set_base(machine().root_device().memregion("romdisk")->base() + chip*0x10000 + page * 0x2000);
 	}
 	else
 	{
-		memory_set_bankptr(space->machine(), "bank2", ram_get_ptr(space->machine().device(RAM_TAG)) + 0xc000);
+		membank("bank2")->set_base(m_ram->pointer() + 0xc000);
 	}
 }
 
 
-static WRITE8_HANDLER (vdisk_page_w)
+WRITE8_MEMBER(pyl601_state::vdisk_page_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	state->m_vdisk_addr = (state->m_vdisk_addr & 0x0ffff) | ((data & 0x0f)<<16);
+	m_vdisk_addr = (m_vdisk_addr & 0x0ffff) | ((data & 0x0f)<<16);
 }
 
-static WRITE8_HANDLER (vdisk_h_w)
+WRITE8_MEMBER(pyl601_state::vdisk_h_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	state->m_vdisk_addr = (state->m_vdisk_addr & 0xf00ff) | (data<<8);
+	m_vdisk_addr = (m_vdisk_addr & 0xf00ff) | (data<<8);
 }
 
-static WRITE8_HANDLER (vdisk_l_w)
+WRITE8_MEMBER(pyl601_state::vdisk_l_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	state->m_vdisk_addr = (state->m_vdisk_addr & 0xfff00) | data;
+	m_vdisk_addr = (m_vdisk_addr & 0xfff00) | data;
 }
 
-static WRITE8_HANDLER (vdisk_data_w)
+WRITE8_MEMBER(pyl601_state::vdisk_data_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	ram_get_ptr(space->machine().device(RAM_TAG))[0x10000 + (state->m_vdisk_addr & 0x7ffff)] = data;
-	state->m_vdisk_addr++;
-	state->m_vdisk_addr&=0x7ffff;
+	m_ram->pointer()[0x10000 + (m_vdisk_addr & 0x7ffff)] = data;
+	m_vdisk_addr++;
+	m_vdisk_addr&=0x7ffff;
 }
 
-static READ8_HANDLER (vdisk_data_r)
+READ8_MEMBER(pyl601_state::vdisk_data_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	UINT8 retVal = ram_get_ptr(space->machine().device(RAM_TAG))[0x10000 + (state->m_vdisk_addr & 0x7ffff)];
-	state->m_vdisk_addr++;
-	state->m_vdisk_addr &= 0x7ffff;
+	UINT8 retVal = m_ram->pointer()[0x10000 + (m_vdisk_addr & 0x7ffff)];
+	m_vdisk_addr++;
+	m_vdisk_addr &= 0x7ffff;
 	return retVal;
 }
 
-static UINT8 selectedline(UINT16 data)
+UINT8 pyl601_state::selectedline(UINT16 data)
 {
 	UINT8 i;
 	for(i = 0; i < 16; i++)
-	{
 		if (BIT(data, i))
-		{
 			return i;
-		}
-	}
+
 	return 0;
 }
 
-static READ8_HANDLER ( keyboard_r )
+READ8_MEMBER(pyl601_state::keyboard_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	return state->m_key_code;
+	UINT8 ret = m_key_code;
+	m_key_code = 0xff;
+	return ret;
 }
 
-static READ8_HANDLER ( keycheck_r )
+READ8_MEMBER(pyl601_state::keycheck_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
 	UINT8 retVal = 0x3f;
-	UINT8 *keyboard = space->machine().region("keyboard")->base();
-	UINT16 row1 = input_port_read(space->machine(), "ROW1");
-	UINT16 row2 = input_port_read(space->machine(), "ROW2");
-	UINT16 row3 = input_port_read(space->machine(), "ROW3");
-	UINT16 row4 = input_port_read(space->machine(), "ROW4");
-	UINT16 row5 = input_port_read(space->machine(), "ROW5");
+	UINT8 *keyboard = memregion("keyboard")->base();
+	UINT16 row1 = ioport("ROW1")->read();
+	UINT16 row2 = ioport("ROW2")->read();
+	UINT16 row3 = ioport("ROW3")->read();
+	UINT16 row4 = ioport("ROW4")->read();
+	UINT16 row5 = ioport("ROW5")->read();
 	UINT16 all = row1 | row2 | row3 | row4 | row5;
-	UINT16 addr = (input_port_read(space->machine(), "SHIFT") & 1) | (input_port_read(space->machine(), "CTRL") & 1) << 1;
-	if (all != 0xff)
+	UINT16 addr = ioport("MODIFIERS")->read();
+	if (all)
 	{
 		addr |= selectedline(all) << 2;
 
@@ -131,40 +172,39 @@ static READ8_HANDLER ( keycheck_r )
 		addr |=  ((row2 == 0x00) ? 1 : 0) << 9;
 		addr |=  ((row1 == 0x00) ? 1 : 0) << 10;
 
-		state->m_key_code = keyboard[addr];
-		state->m_keyboard_clk = ~state->m_keyboard_clk;
+		m_key_code = keyboard[addr];
+		m_keyboard_clk = ~m_keyboard_clk;
 
-		if (state->m_keyboard_clk)
+		if (m_keyboard_clk)
 			retVal |= 0x80;
 	}
 	return retVal;
 }
 
 
-static WRITE8_HANDLER (video_mode_w)
+WRITE8_MEMBER(pyl601_state::video_mode_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	state->m_video_mode = data;
-}
-static READ8_HANDLER (video_mode_r)
-{
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	return state->m_video_mode;
+	m_video_mode = data;
 }
 
-static READ8_HANDLER (timer_r)
+READ8_MEMBER(pyl601_state::video_mode_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	UINT8 retVal= state->m_tick50_mark | 0x37;
-	state->m_tick50_mark = 0;
+	return m_video_mode;
+}
+
+READ8_MEMBER(pyl601_state::timer_r)
+{
+	UINT8 retVal= m_tick50_mark | 0x37;
+	m_tick50_mark = 0;
 	return retVal;
 }
 
-static WRITE8_HANDLER (speaker_w)
+WRITE8_MEMBER(pyl601_state::speaker_w)
 {
+	speaker_level_w(m_speaker, BIT(data, 3));
 }
 
-static WRITE8_HANDLER (led_w)
+WRITE8_MEMBER(pyl601_state::led_w)
 {
 //  UINT8 caps_led = BIT(data,4);
 }
@@ -178,30 +218,30 @@ static UPD765_GET_IMAGE( pyldin_upd765_get_image )
 {
 	return get_floppy_image(device->machine(), (floppy_index & 1)^1);
 }
-static WRITE8_HANDLER( floppy_w )
+
+WRITE8_MEMBER(pyl601_state::floppy_w)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
 	// bit 0 is reset (if zero)
 	// bit 1 is TC state
 	// bit 2 is drive selected
 	// bit 3 is motor state
-	device_t *floppy = space->machine().device("upd765");
-	if (BIT(data,0)==0) {
+
+	if (BIT(data,0)==0)
 		//reset
-		upd765_reset(floppy,0);
-	}
-	floppy_mon_w(get_floppy_image(space->machine(), BIT(data,2)), !BIT(data, 3));
+		upd765_reset(m_fdc, 0);
 
-	floppy_drive_set_ready_state(get_floppy_image(space->machine(), 0), BIT(data,2), 0);
+	floppy_mon_w(get_floppy_image(machine(), BIT(data,2)), !BIT(data, 3));
 
-	upd765_tc_w(floppy, BIT(data,1));
+	floppy_drive_set_ready_state(get_floppy_image(machine(), 0), BIT(data,2), 0);
 
-	state->m_floppy_ctrl = data;
+	upd765_tc_w(m_fdc, BIT(data, 1));
+
+	m_floppy_ctrl = data;
 }
-static READ8_HANDLER (floppy_r)
+
+READ8_MEMBER(pyl601_state::floppy_r)
 {
-	pyl601_state *state = space->machine().driver_data<pyl601_state>();
-	return state->m_floppy_ctrl;
+	return m_floppy_ctrl;
 }
 
 static const struct upd765_interface pyldin_upd765_interface =
@@ -213,15 +253,13 @@ static const struct upd765_interface pyldin_upd765_interface =
 	{FLOPPY_0,FLOPPY_1, NULL, NULL}
 };
 
-static ADDRESS_MAP_START(pyl601_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START(pyl601_mem, AS_PROGRAM, 8, pyl601_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE( 0x0000, 0xbfff ) AM_RAMBANK("bank1")
 	AM_RANGE( 0xc000, 0xdfff ) AM_RAMBANK("bank2")
 	AM_RANGE( 0xe000, 0xe5ff ) AM_RAMBANK("bank3")
-	AM_RANGE( 0xe600, 0xe600 ) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE( 0xe601, 0xe601 ) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
-	AM_RANGE( 0xe604, 0xe604 ) AM_DEVWRITE_MODERN("crtc", mc6845_device, address_w)
-	AM_RANGE( 0xe605, 0xe605 ) AM_DEVREADWRITE_MODERN("crtc", mc6845_device, register_r, register_w)
+	AM_RANGE( 0xe600, 0xe600 ) AM_MIRROR(4) AM_DEVREADWRITE("crtc", mc6845_device, status_r, address_w)
+	AM_RANGE( 0xe601, 0xe601 ) AM_MIRROR(4) AM_DEVREADWRITE("crtc", mc6845_device, register_r, register_w)
 	AM_RANGE( 0xe628, 0xe628 ) AM_READ(keyboard_r)
 	AM_RANGE( 0xe629, 0xe629 ) AM_READWRITE(video_mode_r,video_mode_w)
 	AM_RANGE( 0xe62a, 0xe62a ) AM_READWRITE(keycheck_r,led_w)
@@ -233,8 +271,8 @@ static ADDRESS_MAP_START(pyl601_mem, AS_PROGRAM, 8)
 	AM_RANGE( 0xe682, 0xe682 ) AM_WRITE(vdisk_l_w)
 	AM_RANGE( 0xe683, 0xe683 ) AM_READWRITE(vdisk_data_r,vdisk_data_w)
 	AM_RANGE( 0xe6c0, 0xe6c0 ) AM_READWRITE(floppy_r, floppy_w)
-	AM_RANGE( 0xe6d0, 0xe6d0 ) AM_DEVREAD("upd765", upd765_status_r)
-	AM_RANGE( 0xe6d1, 0xe6d1 ) AM_DEVREADWRITE("upd765", upd765_data_r, upd765_data_w)
+	AM_RANGE( 0xe6d0, 0xe6d0 ) AM_DEVREAD_LEGACY("upd765", upd765_status_r)
+	AM_RANGE( 0xe6d1, 0xe6d1 ) AM_DEVREADWRITE_LEGACY("upd765", upd765_data_r, upd765_data_w)
 	AM_RANGE( 0xe6f0, 0xe6f0 ) AM_READWRITE(rom_page_r, rom_page_w)
 	AM_RANGE( 0xe700, 0xefff ) AM_RAMBANK("bank4")
 	AM_RANGE( 0xf000, 0xffff ) AM_READ_BANK("bank5") AM_WRITE_BANK("bank6")
@@ -333,24 +371,22 @@ static INPUT_PORTS_START( pyl601 )
 	PORT_BIT( 0x1000, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_F1) PORT_CHAR(UCHAR_MAMEKEY(F1))
 	PORT_BIT( 0x0800, IP_ACTIVE_HIGH, IPT_KEYBOARD ) PORT_CODE(KEYCODE_ESC) PORT_CHAR(UCHAR_MAMEKEY(ESC))
 
-	PORT_START("SHIFT")
+	PORT_START("MODIFIERS")
 	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("Shift") PORT_CODE(KEYCODE_LSHIFT) PORT_CODE(KEYCODE_RSHIFT) PORT_CHAR(UCHAR_SHIFT_1)
-
-	PORT_START("CTRL")
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("Ctrl") PORT_CODE(KEYCODE_LCONTROL) PORT_CODE(KEYCODE_RCONTROL) PORT_CHAR(UCHAR_MAMEKEY(PGUP))
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("Ctrl") PORT_CODE(KEYCODE_LCONTROL) PORT_CODE(KEYCODE_RCONTROL) PORT_CHAR(UCHAR_MAMEKEY(PGUP))
 INPUT_PORTS_END
 
 static MACHINE_RESET(pyl601)
 {
 	pyl601_state *state = machine.driver_data<pyl601_state>();
-	UINT8 *ram = ram_get_ptr(machine.device(RAM_TAG));
+	UINT8 *ram = state->m_ram->pointer();
 	state->m_key_code = 0xff;
-	memory_set_bankptr(machine, "bank1", ram + 0x0000);
-	memory_set_bankptr(machine, "bank2", ram + 0xc000);
-	memory_set_bankptr(machine, "bank3", ram + 0xe000);
-	memory_set_bankptr(machine, "bank4", ram + 0xe700);
-	memory_set_bankptr(machine, "bank5", machine.region("maincpu")->base() + 0xf000);
-	memory_set_bankptr(machine, "bank6", ram + 0xf000);
+	state->membank("bank1")->set_base(ram + 0x0000);
+	state->membank("bank2")->set_base(ram + 0xc000);
+	state->membank("bank3")->set_base(ram + 0xe000);
+	state->membank("bank4")->set_base(ram + 0xe700);
+	state->membank("bank5")->set_base(state->memregion("maincpu")->base() + 0xf000);
+	state->membank("bank6")->set_base(ram + 0xf000);
 
 	machine.device("maincpu")->reset();
 }
@@ -359,17 +395,11 @@ static VIDEO_START( pyl601 )
 {
 }
 
-static SCREEN_UPDATE( pyl601 )
-{
-	mc6845_device *mc6845 = screen->machine().device<mc6845_device>("crtc");
-	mc6845->update( bitmap, cliprect);
-	return 0;
-}
-
 static MC6845_UPDATE_ROW( pyl601_update_row )
 {
 	pyl601_state *state = device->machine().driver_data<pyl601_state>();
-	UINT8 *charrom = device->machine().region("gfx1")->base();
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
+	UINT8 *charrom = state->memregion("chargen")->base();
 
 	int column, bit, i;
 	UINT8 data;
@@ -377,22 +407,18 @@ static MC6845_UPDATE_ROW( pyl601_update_row )
 	{
 		for (column = 0; column < x_count; column++)
 		{
-			UINT8 code = ram_get_ptr(device->machine().device(RAM_TAG))[(((ma + column) & 0x0fff) + 0xf000)];
+			UINT8 code = state->m_ram->pointer()[(((ma + column) & 0x0fff) + 0xf000)];
 			code = ((code << 1) | (code >> 7)) & 0xff;
 			if (column == cursor_x-2)
-			{
 				data = 0xff;
-			}
 			else
-			{
 				data = charrom[((code << 3) | (ra & 0x07)) & 0x7ff];
-			}
+
 			for (bit = 0; bit < 8; bit++)
 			{
 				int x = (column * 8) + bit;
-				int color = BIT(data, 7) ? 1 : 0;
 
-				*BITMAP_ADDR16(bitmap, y, x) = color;
+				bitmap.pix32(y, x) = palette[BIT(data, 7)];
 
 				data <<= 1;
 			}
@@ -402,10 +428,10 @@ static MC6845_UPDATE_ROW( pyl601_update_row )
 	{
 		for (i = 0; i < x_count; i++)
 		{
-			data = ram_get_ptr(device->machine().device(RAM_TAG))[(((ma + i) << 3) | (ra & 0x07)) & 0xffff];
+			data = state->m_ram->pointer()[(((ma + i) << 3) | (ra & 0x07)) & 0xffff];
 			for (bit = 0; bit < 8; bit++)
 			{
-				*BITMAP_ADDR16(bitmap, y, (i * 8) + bit) = BIT(data, 7) ? 1 : 0;
+				bitmap.pix32(y, (i * 8) + bit) = palette[BIT(data, 7)];
 				data <<= 1;
 			}
 		}
@@ -415,7 +441,8 @@ static MC6845_UPDATE_ROW( pyl601_update_row )
 static MC6845_UPDATE_ROW( pyl601a_update_row )
 {
 	pyl601_state *state = device->machine().driver_data<pyl601_state>();
-	UINT8 *charrom = device->machine().region("gfx1")->base();
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
+	UINT8 *charrom = state->memregion("chargen")->base();
 
 	int column, bit, i;
 	UINT8 data;
@@ -423,19 +450,16 @@ static MC6845_UPDATE_ROW( pyl601a_update_row )
 	{
 		for (column = 0; column < x_count; column++)
 		{
-			UINT8 code = ram_get_ptr(device->machine().device(RAM_TAG))[(((ma + column) & 0x0fff) + 0xf000)];
+			UINT8 code = state->m_ram->pointer()[(((ma + column) & 0x0fff) + 0xf000)];
 			data = charrom[((code << 4) | (ra & 0x07)) & 0xfff];
 			if (column == cursor_x)
-			{
 				data = 0xff;
-			}
 
 			for (bit = 0; bit < 8; bit++)
 			{
 				int x = (column * 8) + bit;
-				int color = BIT(data, 7) ? 1 : 0;
 
-				*BITMAP_ADDR16(bitmap, y, x) = color;
+				bitmap.pix32(y, x) = palette[BIT(data, 7)];
 
 				data <<= 1;
 			}
@@ -445,10 +469,10 @@ static MC6845_UPDATE_ROW( pyl601a_update_row )
 	{
 		for (i = 0; i < x_count; i++)
 		{
-			data = ram_get_ptr(device->machine().device(RAM_TAG))[(((ma + i) << 3) | (ra & 0x07)) & 0xffff];
+			data = state->m_ram->pointer()[(((ma + i) << 3) | (ra & 0x07)) & 0xffff];
 			for (bit = 0; bit < 8; bit++)
 			{
-				*BITMAP_ADDR16(bitmap, y, (i * 8) + bit) = BIT(data, 7) ? 1 : 0;
+				bitmap.pix32(y, (i * 8) + bit) = palette[BIT(data, 7)];
 				data <<= 1;
 			}
 		}
@@ -486,7 +510,7 @@ static const mc6845_interface pyl601a_crtc6845_interface =
 
 static DRIVER_INIT(pyl601)
 {
-	memset(ram_get_ptr(machine.device(RAM_TAG)), 0, 64 * 1024);
+	memset(machine.device<ram_device>(RAM_TAG)->pointer(), 0, 64 * 1024);
 }
 
 static INTERRUPT_GEN( pyl601_interrupt )
@@ -546,11 +570,11 @@ static const gfx_layout pyl601a_charlayout =
 };
 
 static GFXDECODE_START( pyl601 )
-	GFXDECODE_ENTRY( "gfx1", 0x0000, pyl601_charlayout, 0, 1 )
+	GFXDECODE_ENTRY( "chargen", 0x0000, pyl601_charlayout, 0, 1 )
 GFXDECODE_END
 
 static GFXDECODE_START( pyl601a )
-	GFXDECODE_ENTRY( "gfx1", 0x0000, pyl601a_charlayout, 0, 1 )
+	GFXDECODE_ENTRY( "chargen", 0x0000, pyl601a_charlayout, 0, 1 )
 GFXDECODE_END
 
 static MACHINE_CONFIG_START( pyl601, pyl601_state )
@@ -565,21 +589,22 @@ static MACHINE_CONFIG_START( pyl601, pyl601_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 200)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640 - 1, 0, 200 - 1)
-	MCFG_SCREEN_UPDATE( pyl601 )
-
+	MCFG_VIDEO_START( pyl601 )
+	MCFG_SCREEN_UPDATE_DEVICE("crtc", mc6845_device, screen_update)
 	MCFG_GFXDECODE(pyl601)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(monochrome_green)
 
+	/* sound hardware */
+	MCFG_SPEAKER_STANDARD_MONO("mono")
+	MCFG_SOUND_ADD(SPEAKER_TAG, SPEAKER_SOUND, 0)
+	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.50)
+
+	/* Devices */
 	MCFG_MC6845_ADD("crtc", MC6845, XTAL_2MHz, pyl601_crtc6845_interface)
-
-	MCFG_VIDEO_START( pyl601 )
-
 	MCFG_UPD765A_ADD("upd765", pyldin_upd765_interface)
-
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(pyldin_floppy_interface)
 
 	/* internal ram */
@@ -601,10 +626,10 @@ ROM_START( pyl601 )
 	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
 	ROM_LOAD( "bios.rom",   0xf000, 0x1000, CRC(41fe4c4b) SHA1(d8ca92aea0eb283e8d7779cb976bcdfa03e81aea))
 
-	ROM_REGION(0x0800, "gfx1",0)
+	ROM_REGION(0x0800, "chargen", 0)
 	ROM_LOAD( "video.rom",  0x0000, 0x0800, CRC(1c23ba43) SHA1(eb1cfc139858abd0aedbbf3d523f8ba55d27a11d))
 
-	ROM_REGION(0x50000, "romdisk",ROMREGION_ERASEFF)
+	ROM_REGION(0x50000, "romdisk", ROMREGION_ERASEFF)
 	ROM_LOAD( "rom0.rom", 0x00000, 0x10000, CRC(60103920) SHA1(ee5b4ee5b513c4a0204da751e53d63b8c6c0aab9))
 	ROM_LOAD( "rom1.rom", 0x10000, 0x10000, CRC(cb4a9b22) SHA1(dd09e4ba35b8d1a6f60e6e262aaf2f156367e385))
 	ROM_LOAD( "rom2.rom", 0x20000, 0x08000, CRC(0b7684bf) SHA1(c02ad1f2a6f484cd9d178d8b060c21c0d4e53442))
@@ -614,7 +639,7 @@ ROM_START( pyl601 )
 	ROM_LOAD( "rom4.rom", 0x40000, 0x08000, CRC(d88ac21d) SHA1(022db11fdcf8db81ce9efd9cd9fa50ebca88e79e))
 	ROM_COPY("romdisk", 0x40000, 0x48000, 0x08000)
 
-	ROM_REGION(0x0800, "keyboard",0)
+	ROM_REGION(0x0800, "keyboard", 0)
 	ROM_LOAD( "keyboard.rom", 0x0000, 0x0800, CRC(41fbe5ca) SHA1(875adaef53bc37e92ad0b6b6ee3d8fd28344d358))
 ROM_END
 
@@ -622,10 +647,10 @@ ROM_START( pyl601a )
 	ROM_REGION( 0x10000, "maincpu", ROMREGION_ERASEFF )
 	ROM_LOAD( "bios_a.rom", 0xf000, 0x1000, CRC(e018b11e) SHA1(884d59abd5fa5af1295d1b5a53693facc7945b63))
 
-	ROM_REGION(0x1000, "gfx1",0)
+	ROM_REGION(0x1000, "chargen", 0)
 	ROM_LOAD( "video_a.rom", 0x0000,0x1000, CRC(00fa4077) SHA1(d39d15969a08bdb768d08bea4ec9a9cb498232fd))
 
-	ROM_REGION(0x50000, "romdisk",ROMREGION_ERASEFF)
+	ROM_REGION(0x50000, "romdisk", ROMREGION_ERASEFF)
 	ROM_LOAD( "rom0.rom", 0x00000, 0x10000, CRC(60103920) SHA1(ee5b4ee5b513c4a0204da751e53d63b8c6c0aab9))
 	ROM_LOAD( "rom1.rom", 0x10000, 0x10000, CRC(cb4a9b22) SHA1(dd09e4ba35b8d1a6f60e6e262aaf2f156367e385))
 	ROM_LOAD( "rom2.rom", 0x20000, 0x08000, CRC(0b7684bf) SHA1(c02ad1f2a6f484cd9d178d8b060c21c0d4e53442))
@@ -635,11 +660,11 @@ ROM_START( pyl601a )
 	ROM_LOAD( "rom4.rom", 0x40000, 0x08000, CRC(d88ac21d) SHA1(022db11fdcf8db81ce9efd9cd9fa50ebca88e79e))
 	ROM_COPY("romdisk", 0x40000, 0x48000, 0x08000)
 
-	ROM_REGION(0x0800, "keyboard",0)
+	ROM_REGION(0x0800, "keyboard", 0)
 	ROM_LOAD( "keyboard.rom", 0x0000, 0x0800, CRC(41fbe5ca) SHA1(875adaef53bc37e92ad0b6b6ee3d8fd28344d358))
 ROM_END
 /* Driver */
 
-/*    YEAR  NAME    PARENT  COMPAT   MACHINE    INPUT    INIT   COMPANY   FULLNAME       FLAGS */
-COMP( 1989, pyl601,  0,       0,	pyl601, 	pyl601, pyl601, "Mikroelektronika",   "Pyldin-601",		GAME_NOT_WORKING | GAME_NO_SOUND)
-COMP( 1989, pyl601a, pyl601,  0,	pyl601a,	pyl601, pyl601, "Mikroelektronika",   "Pyldin-601A",		GAME_NOT_WORKING | GAME_NO_SOUND)
+/*    YEAR  NAME     PARENT   COMPAT   MACHINE    INPUT    INIT        COMPANY           FULLNAME    FLAGS */
+COMP( 1989, pyl601,  0,       0,       pyl601,    pyl601,  pyl601, "Mikroelektronika", "Pyldin-601", 0 )
+COMP( 1989, pyl601a, pyl601,  0,       pyl601a,   pyl601,  pyl601, "Mikroelektronika", "Pyldin-601A", 0 )
