@@ -71,7 +71,6 @@
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -248,14 +247,14 @@ READ8_MEMBER( ql_state::ipc_bus_r )
 
 	UINT8 data = 0;
 
-	if (BIT(m_keylatch, 0)) data |= input_port_read(machine(), "ROW0") | input_port_read(machine(), "JOY0");
-	if (BIT(m_keylatch, 1)) data |= input_port_read(machine(), "ROW1") | input_port_read(machine(), "JOY1");
-	if (BIT(m_keylatch, 2)) data |= input_port_read(machine(), "ROW2");
-	if (BIT(m_keylatch, 3)) data |= input_port_read(machine(), "ROW3");
-	if (BIT(m_keylatch, 4)) data |= input_port_read(machine(), "ROW4");
-	if (BIT(m_keylatch, 5)) data |= input_port_read(machine(), "ROW5");
-	if (BIT(m_keylatch, 6)) data |= input_port_read(machine(), "ROW6");
-	if (BIT(m_keylatch, 7)) data |= input_port_read(machine(), "ROW7");
+	if (BIT(m_keylatch, 0)) data |= ioport("ROW0")->read() | ioport("JOY0")->read();
+	if (BIT(m_keylatch, 1)) data |= ioport("ROW1")->read() | ioport("JOY1")->read();
+	if (BIT(m_keylatch, 2)) data |= ioport("ROW2")->read();
+	if (BIT(m_keylatch, 3)) data |= ioport("ROW3")->read();
+	if (BIT(m_keylatch, 4)) data |= ioport("ROW4")->read();
+	if (BIT(m_keylatch, 5)) data |= ioport("ROW5")->read();
+	if (BIT(m_keylatch, 6)) data |= ioport("ROW6")->read();
+	if (BIT(m_keylatch, 7)) data |= ioport("ROW7")->read();
 
 	return data;
 }
@@ -304,26 +303,26 @@ READ8_MEMBER( ql_state::trump_card_rom_r )
 {
 	// If we have more than 640K them map extra ram into top 256K
 	// else just map to unmap.
-	if (ram_get_size(m_ram)>(640*1024))
+	if (m_ram->size()>(640*1024))
 		space.install_ram(0x0c0000, 0x0fffff, NULL);
 	else
 		space.unmap_readwrite(0x0c0000, 0x0fffff);
 
 	// Setup trumcard rom mapped to rom so unlink us
-	space.install_rom(0x010000, 0x018000, &machine().region(M68008_TAG)->base()[TRUMP_ROM_BASE]);
+	space.install_rom(0x010000, 0x018000, &machine().root_device().memregion(M68008_TAG)->base()[TRUMP_ROM_BASE]);
 
-	return machine().region(M68008_TAG)->base()[TRUMP_ROM_BASE+offset];
+	return memregion(M68008_TAG)->base()[TRUMP_ROM_BASE+offset];
 }
 
 READ8_MEMBER( ql_state::cart_rom_r )
 {
 	// Setup trumcard rom mapped in at $c0000
-	space.install_rom(0x0c0000, 0x0c8000, &machine().region(M68008_TAG)->base()[TRUMP_ROM_BASE]);
+	space.install_rom(0x0c0000, 0x0c8000, &machine().root_device().memregion(M68008_TAG)->base()[TRUMP_ROM_BASE]);
 
 	// Setup cart rom to rom handler, so unlink us
-	space.install_rom(0x0c000, 0x0ffff, &machine().region(M68008_TAG)->base()[CART_ROM_BASE]);
+	space.install_rom(0x0c000, 0x0ffff, &machine().root_device().memregion(M68008_TAG)->base()[CART_ROM_BASE]);
 
-	return machine().region(M68008_TAG)->base()[CART_ROM_BASE+offset];
+	return memregion(M68008_TAG)->base()[CART_ROM_BASE+offset];
 }
 
 void ql_state::trump_card_set_control(UINT8 data)
@@ -724,12 +723,12 @@ static PALETTE_INIT( ql )
 
 
 //-------------------------------------------------
-//  SCREEN_UPDATE( ql )
+//  SCREEN_UPDATE_IND16( ql )
 //-------------------------------------------------
 
-bool ql_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 ql_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
-	m_zx8301->update_screen(&bitmap, &cliprect);
+	m_zx8301->update_screen(bitmap, cliprect);
 
 	return 0;
 }
@@ -768,42 +767,42 @@ WRITE_LINE_MEMBER( ql_state::ql_comdata_w )
 
 WRITE_LINE_MEMBER( ql_state::zx8302_mdselck_w )
 {
-	microdrive_clk_w(m_mdv2, state);
-	microdrive_clk_w(m_mdv1, state);
+	m_mdv2->clk_w(state);
+	m_mdv1->clk_w(state);
 }
 
 WRITE_LINE_MEMBER( ql_state::zx8302_mdrdw_w )
 {
-	microdrive_read_write_w(m_mdv1, state);
-	microdrive_read_write_w(m_mdv2, state);
+	m_mdv1->read_write_w(state);
+	m_mdv2->read_write_w(state);
 }
 
 WRITE_LINE_MEMBER( ql_state::zx8302_erase_w )
 {
-	microdrive_erase_w(m_mdv1, state);
-	microdrive_erase_w(m_mdv2, state);
+	m_mdv1->erase_w(state);
+	m_mdv2->erase_w(state);
 }
 
 WRITE_LINE_MEMBER( ql_state::zx8302_raw1_w )
 {
-	microdrive_data1_w(m_mdv1, state);
-	microdrive_data1_w(m_mdv2, state);
+	m_mdv1->data1_w(state);
+	m_mdv2->data1_w(state);
 }
 
 READ_LINE_MEMBER( ql_state::zx8302_raw1_r )
 {
-	return microdrive_data1_r(m_mdv1) | microdrive_data1_r(m_mdv2);
+	return m_mdv1->data1_r() | m_mdv2->data1_r();
 }
 
 WRITE_LINE_MEMBER( ql_state::zx8302_raw2_w )
 {
-	microdrive_data2_w(m_mdv1, state);
-	microdrive_data2_w(m_mdv2, state);
+	m_mdv1->data2_w(state);
+	m_mdv2->data2_w(state);
 }
 
 READ_LINE_MEMBER( ql_state::zx8302_raw2_r )
 {
-	return microdrive_data2_r(m_mdv1) | microdrive_data2_r(m_mdv2);
+	return m_mdv1->data2_r() | m_mdv2->data2_r();
 }
 
 static ZX8302_INTERFACE( ql_zx8302_intf )
@@ -819,7 +818,7 @@ static ZX8302_INTERFACE( ql_zx8302_intf )
 	DEVCB_NULL, // NETOUT
 	DEVCB_NULL, // NETIN
 	DEVCB_DRIVER_LINE_MEMBER(ql_state, zx8302_mdselck_w),
-	DEVCB_DEVICE_LINE(MDV_1, microdrive_comms_in_w),
+	DEVCB_DEVICE_LINE_MEMBER(MDV_1, microdrive_image_device, comms_in_w),
 	DEVCB_DRIVER_LINE_MEMBER(ql_state, zx8302_mdrdw_w),
 	DEVCB_DRIVER_LINE_MEMBER(ql_state, zx8302_erase_w),
 	DEVCB_DRIVER_LINE_MEMBER(ql_state, zx8302_raw1_w),
@@ -894,7 +893,9 @@ wd17xx_interface ql_wd17xx_interface =
 
 static MICRODRIVE_CONFIG( mdv1_config )
 {
-	DEVCB_DEVICE_LINE(MDV_2, microdrive_comms_in_w)
+	DEVCB_DEVICE_LINE_MEMBER(MDV_2, microdrive_image_device, comms_in_w),
+	NULL,
+	NULL,
 };
 
 
@@ -904,7 +905,9 @@ static MICRODRIVE_CONFIG( mdv1_config )
 
 static MICRODRIVE_CONFIG( mdv2_config )
 {
-	DEVCB_NULL
+	DEVCB_NULL,
+	NULL,
+	NULL
 };
 
 
@@ -928,19 +931,19 @@ static MICRODRIVE_CONFIG( mdv2_config )
 void ql_state::machine_start()
 {
 	// register for state saving
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_ipl);
-	state_save_register_global(machine(), m_comdata);
-	state_save_register_global(machine(), m_baudx4);
-	state_save_register_global(machine(), m_printer_char);
-	state_save_register_global(machine(), m_disk_io_byte);
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_ipl));
+	save_item(NAME(m_comdata));
+	save_item(NAME(m_baudx4));
+	save_item(NAME(m_printer_char));
+	save_item(NAME(m_disk_io_byte));
 }
 
 void ql_state::machine_reset()
 {
 	address_space	*program	= m_maincpu->memory().space(AS_PROGRAM);
 
-	m_disk_type=input_port_read(machine(),QL_CONFIG_PORT) & DISK_TYPE_MASK;
+	m_disk_type=ioport(QL_CONFIG_PORT)->read() & DISK_TYPE_MASK;
 	logerror("disktype=%d\n",m_disk_type);
 
 	m_printer_char=0;
@@ -948,7 +951,7 @@ void ql_state::machine_reset()
 
 	logerror("Configuring RAM\n");
 	// configure RAM
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 		case 128*1024:
 			program->unmap_readwrite(0x040000, 0x0fffff);
@@ -975,7 +978,7 @@ void ql_state::machine_reset()
 	{
 		case DISK_TYPE_SANDY :
 			logerror("Configuring SandySuperDisk\n");
-			program->install_rom(0x0c0000, 0x0c3fff, &machine().region(M68008_TAG)->base()[SANDY_ROM_BASE]);
+			program->install_rom(0x0c0000, 0x0c3fff, &machine().root_device().memregion(M68008_TAG)->base()[SANDY_ROM_BASE]);
 			program->install_read_handler(SANDY_IO_BASE, SANDY_IO_END, 0, 0, read8_delegate(FUNC(ql_state::disk_io_r), this));
 			program->install_write_handler(SANDY_IO_BASE, SANDY_IO_END, 0, 0, write8_delegate(FUNC(ql_state::disk_io_w), this));
 			m_disk_io_base=SANDY_IO_BASE;
@@ -1011,7 +1014,7 @@ static MACHINE_CONFIG_START( ql, ql_state )
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50.08)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) // not accurate
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(ql_state, screen_update)
 	MCFG_SCREEN_SIZE(960, 312)
 	MCFG_SCREEN_VISIBLE_AREA(0, 512-1, 0, 256-1)
 
