@@ -23,7 +23,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -66,7 +65,7 @@ public:
 		{ }
 
 	required_device<cpu_device> m_maincpu;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 	required_device<device_t> m_beep;
 	fujitsu_29dl16x_device *m_flash[4];
 
@@ -91,7 +90,7 @@ public:
 
 	virtual void machine_start();
 	virtual void machine_reset();
-	bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 
 	DECLARE_READ8_MEMBER( bankswitch_r );
 	DECLARE_WRITE8_MEMBER( bankswitch_w );
@@ -323,14 +322,14 @@ WRITE8_MEMBER( rex6000_state::irq_w )
 
 READ8_MEMBER( rex6000_state::touchscreen_r )
 {
-	UINT16 x = input_port_read(machine(), "PENX");
-	UINT16 y = input_port_read(machine(), "PENY");
-	UINT16 battery = input_port_read(machine(), "BATTERY");
+	UINT16 x = ioport("PENX")->read();
+	UINT16 y = ioport("PENY")->read();
+	UINT16 battery = ioport("BATTERY")->read();
 
 	switch (offset)
 	{
 		case 0x08:
-			return ((input_port_read(machine(), "INPUT") & 0x40) ? 0x20 : 0x00) | 0X10;
+			return ((ioport("INPUT")->read() & 0x40) ? 0x20 : 0x00) | 0X10;
 		case 0x09:
 			if (m_touchscreen[4] & 0x80)
 				return (battery>>0) & 0xff;
@@ -450,8 +449,8 @@ void rex6000_state::machine_start()
 	m_flash[2] = machine().device<fujitsu_29dl16x_device>("flash1a");
 	m_flash[3] = machine().device<fujitsu_29dl16x_device>("flash1b");
 
-	m_ram_base = ram_get_ptr(m_ram);
-	memory_set_bankptr(machine(), "ram", m_ram_base + 0x4000);
+	m_ram_base = m_ram->pointer();
+	membank("ram")->set_base(m_ram_base + 0x4000);
 }
 void rex6000_state::machine_reset()
 {
@@ -465,7 +464,7 @@ void rex6000_state::machine_reset()
 	m_banks[1].type = 0x00;
 	m_banks[1].type = 0;
 
-	memset(m_ram_base, 0, ram_get_size(m_ram));
+	memset(m_ram_base, 0, m_ram->size());
 	memset(m_bank, 0, sizeof(m_bank));
 	memset(m_beep_io, 0, sizeof(m_beep_io));
 	memset(m_lcd_base, 0, sizeof(m_lcd_base));
@@ -478,7 +477,7 @@ void rex6000_state::machine_reset()
 	m_beep_mode = 0;
 }
 
-bool rex6000_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 rex6000_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	UINT16 lcd_bank = MAKE_BANK(m_lcd_base[0], m_lcd_base[1]&0x1f);
 	UINT8 mem_type = identify_bank_type(lcd_bank);
@@ -502,14 +501,14 @@ bool rex6000_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 
 				for (int b=0; b<8; b++)
 				{
-					*BITMAP_ADDR16(&bitmap, y, (x * 8) + b) = BIT(data, 7);
+					bitmap.pix16(y, (x * 8) + b) = BIT(data, 7);
 					data <<= 1;
 				}
 			}
 	}
 	else
 	{
-		bitmap_fill(&bitmap, &cliprect, 0);
+		bitmap.fill(0, cliprect);
 	}
 
 	return 0;
@@ -661,7 +660,7 @@ static MACHINE_CONFIG_START( rex6000, rex6000_state )
     MCFG_SCREEN_ADD("screen", LCD)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(rex6000_state, screen_update)
     MCFG_SCREEN_SIZE(240, 120)
     MCFG_SCREEN_VISIBLE_AREA(0, 240-1, 0, 120-1)
 	MCFG_DEFAULT_LAYOUT(layout_lcd)
