@@ -41,7 +41,6 @@
                 memory and it appears that the operator is not provided any information of what happened.
 
 *************************************************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -53,7 +52,7 @@
 
 #define MACHINE_RESET_MEMBER(name) void name::machine_reset()
 #define VIDEO_START_MEMBER(name) void name::video_start()
-#define SCREEN_UPDATE_MEMBER(name) bool name::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+#define SCREEN_UPDATE16_MEMBER(name) UINT32 name::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 
 class rx78_state : public driver_device
 {
@@ -86,7 +85,7 @@ public:
 	UINT8 m_key_mux;
 	virtual void machine_reset();
 	virtual void video_start();
-	virtual bool screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect);
+	UINT32 screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 };
 
 
@@ -113,12 +112,12 @@ VIDEO_START_MEMBER( rx78_state )
 {
 }
 
-SCREEN_UPDATE_MEMBER( rx78_state )
+SCREEN_UPDATE16_MEMBER( rx78_state )
 {
-	UINT8 *vram = machine().region("vram")->base();
+	UINT8 *vram = memregion("vram")->base();
 	int x,y,count;
 
-	bitmap_fill(&bitmap, &cliprect, 16);
+	bitmap.fill(16, cliprect);
 
 	count = 0x2c0; //first 0x2bf bytes aren't used for bitmap drawing apparently
 
@@ -140,7 +139,7 @@ SCREEN_UPDATE_MEMBER( rx78_state )
 				color |= ((pen[2] & 1) << 2);
 
 				if(color)
-					*BITMAP_ADDR16(&bitmap, y, x+i) = color;
+					bitmap.pix16(y, x+i) = color;
 
 				/* fg color */
 				pen[0] = (m_pri_mask & 0x01) ? (vram[count + 0x0000] >> (i)) : 0x00;
@@ -152,7 +151,7 @@ SCREEN_UPDATE_MEMBER( rx78_state )
 				color |= ((pen[2] & 1) << 2);
 
 				if(color)
-					*BITMAP_ADDR16(&bitmap, y, x+i) = color;
+					bitmap.pix16(y, x+i) = color;
 			}
 			count++;
 		}
@@ -175,13 +174,13 @@ READ8_MEMBER( rx78_state::key_r )
 
 		res = 0;
 		for(i=0;i<15;i++)
-			res |= input_port_read(machine(), keynames[i]);
+			res |= ioport(keynames[i])->read();
 
 		return res;
 	}
 
 	if(m_key_mux >= 1 && m_key_mux <= 15)
-		return input_port_read(machine(), keynames[m_key_mux - 1]);
+		return ioport(keynames[m_key_mux - 1])->read();
 
 	return 0;
 }
@@ -193,7 +192,7 @@ WRITE8_MEMBER( rx78_state::key_w )
 
 READ8_MEMBER( rx78_state::rx78_vram_r )
 {
-	UINT8 *vram = machine().region("vram")->base();
+	UINT8 *vram = memregion("vram")->base();
 
 	if(m_vram_read_bank == 0 || m_vram_read_bank > 6)
 		return 0xff;
@@ -203,7 +202,7 @@ READ8_MEMBER( rx78_state::rx78_vram_r )
 
 WRITE8_MEMBER( rx78_state::rx78_vram_w )
 {
-	UINT8 *vram = machine().region("vram")->base();
+	UINT8 *vram = memregion("vram")->base();
 
 	if(m_vram_write_bank & 0x01) { vram[offset + 0 * 0x2000] = data; }
 	if(m_vram_write_bank & 0x02) { vram[offset + 1 * 0x2000] = data; }
@@ -417,7 +416,7 @@ MACHINE_RESET_MEMBER(rx78_state)
 
 static DEVICE_IMAGE_LOAD( rx78_cart )
 {
-	UINT8 *cart = image.device().machine().region("cart_img")->base();
+	UINT8 *cart = image.device().machine().root_device().memregion("cart_img")->base();
 	UINT32 size;
 
 	if (image.software_entry() == NULL)
@@ -474,7 +473,7 @@ static MACHINE_CONFIG_START( rx78, rx78_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(rx78_state, screen_update)
 	MCFG_SCREEN_SIZE(192, 184)
 	MCFG_SCREEN_VISIBLE_AREA(0, 192-1, 0, 184-1)
 	MCFG_PALETTE_LENGTH(16+1) //+1 for the background color
@@ -515,7 +514,7 @@ ROM_END
 
 static DRIVER_INIT( rx78 )
 {
-	UINT32 ram_size = ram_get_size(machine.device(RAM_TAG));
+	UINT32 ram_size = machine.device<ram_device>(RAM_TAG)->size();
 	address_space *prg = machine.device("maincpu")->memory().space(AS_PROGRAM);
 
 	if(ram_size == 0x4000)
