@@ -4,10 +4,8 @@
 
 
 ***************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
-#include "devcb.h"
 #include "cpu/upd7810/upd7810.h"
 #include "imagedev/cartslot.h"
 #include "audio/upd1771.h"
@@ -17,7 +15,8 @@ class scv_state : public driver_device
 {
 public:
 	scv_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+		m_videoram(*this,"videoram")		{ }
 
 	DECLARE_WRITE8_MEMBER(scv_porta_w);
 	DECLARE_READ8_MEMBER(scv_portb_r);
@@ -26,7 +25,7 @@ public:
 	DECLARE_WRITE8_MEMBER(scv_cart_ram_w);
 	DECLARE_WRITE8_MEMBER(scv_cart_ram2_w);
 	DECLARE_WRITE_LINE_MEMBER(scv_upd1771_ack_w);
-	UINT8 *m_p_videoram;
+	required_shared_ptr<UINT8> m_videoram;
 	UINT8 m_porta;
 	UINT8 m_portc;
 	emu_timer *m_vb_timer;
@@ -43,9 +42,9 @@ public:
 static ADDRESS_MAP_START( scv_mem, AS_PROGRAM, 8, scv_state )
 	AM_RANGE( 0x0000, 0x0fff ) AM_ROM		/* BIOS */
 
-	AM_RANGE( 0x2000, 0x3403 ) AM_RAM AM_BASE(m_p_videoram )	/* VRAM + 4 registers */
+	AM_RANGE( 0x2000, 0x3403 ) AM_RAM AM_SHARE("videoram")	/* VRAM + 4 registers */
 
-	AM_RANGE( 0x3600, 0x3600 ) AM_DEVWRITE_LEGACY( "upd1771c", upd1771_w )
+	AM_RANGE( 0x3600, 0x3600 ) AM_DEVWRITE_LEGACY("upd1771c", upd1771_w )
 
 	AM_RANGE( 0x8000, 0x9fff ) AM_ROMBANK("bank0")
 	AM_RANGE( 0xa000, 0xbfff ) AM_ROMBANK("bank1")
@@ -181,28 +180,28 @@ READ8_MEMBER( scv_state::scv_portb_r )
 	UINT8 data = 0xff;
 
 	if ( ! ( m_porta & 0x01 ) )
-		data &= input_port_read( machine(), "PA0" );
+		data &= ioport( "PA0" )->read();
 
 	if ( ! ( m_porta & 0x02 ) )
-		data &= input_port_read( machine(), "PA1" );
+		data &= ioport( "PA1" )->read();
 
 	if ( ! ( m_porta & 0x04 ) )
-		data &= input_port_read( machine(), "PA2" );
+		data &= ioport( "PA2" )->read();
 
 	if ( ! ( m_porta & 0x08 ) )
-		data &= input_port_read( machine(), "PA3" );
+		data &= ioport( "PA3" )->read();
 
 	if ( ! ( m_porta & 0x10 ) )
-		data &= input_port_read( machine(), "PA4" );
+		data &= ioport( "PA4" )->read();
 
 	if ( ! ( m_porta & 0x20 ) )
-		data &= input_port_read( machine(), "PA5" );
+		data &= ioport( "PA5" )->read();
 
 	if ( ! ( m_porta & 0x40 ) )
-		data &= input_port_read( machine(), "PA6" );
+		data &= ioport( "PA6" )->read();
 
 	if ( ! ( m_porta & 0x80 ) )
-		data &= input_port_read( machine(), "PA7" );
+		data &= ioport( "PA7" )->read();
 
 	return data;
 }
@@ -212,7 +211,7 @@ READ8_MEMBER( scv_state::scv_portc_r )
 {
 	UINT8 data = m_portc;
 
-	data = ( data & 0xfe ) | ( input_port_read( machine(), "PC0" ) & 0x01 );
+	data = ( data & 0xfe ) | ( ioport( "PC0" )->read() & 0x01 );
 
 	return data;
 }
@@ -228,45 +227,45 @@ static void scv_set_banks( running_machine &machine )
 	{
 	case 0:
 	case 0x2000:
-		memory_set_bankptr( machine, "bank0", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank1", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank2", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank3", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank4", state->m_cart_rom + 0x1000 );
+		state->membank( "bank0" )->set_base( state->m_cart_rom );
+		state->membank( "bank1" )->set_base( state->m_cart_rom );
+		state->membank( "bank2" )->set_base( state->m_cart_rom );
+		state->membank( "bank3" )->set_base( state->m_cart_rom );
+		state->membank( "bank4" )->set_base( state->m_cart_rom + 0x1000 );
 		break;
 	case 0x4000:
-		memory_set_bankptr( machine, "bank0", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank1", state->m_cart_rom + 0x2000 );
-		memory_set_bankptr( machine, "bank2", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank3", state->m_cart_rom + 0x2000 );
-		memory_set_bankptr( machine, "bank4", state->m_cart_rom + 0x3000 );
+		state->membank( "bank0" )->set_base( state->m_cart_rom );
+		state->membank( "bank1" )->set_base( state->m_cart_rom + 0x2000 );
+		state->membank( "bank2" )->set_base( state->m_cart_rom );
+		state->membank( "bank3" )->set_base( state->m_cart_rom + 0x2000 );
+		state->membank( "bank4" )->set_base( state->m_cart_rom + 0x3000 );
 		break;
 	case 0x8000:
-		memory_set_bankptr( machine, "bank0", state->m_cart_rom );
-		memory_set_bankptr( machine, "bank1", state->m_cart_rom + 0x2000 );
-		memory_set_bankptr( machine, "bank2", state->m_cart_rom + 0x4000 );
-		memory_set_bankptr( machine, "bank3", state->m_cart_rom + 0x6000 );
-		memory_set_bankptr( machine, "bank4", state->m_cart_rom + 0x7000 );
+		state->membank( "bank0" )->set_base( state->m_cart_rom );
+		state->membank( "bank1" )->set_base( state->m_cart_rom + 0x2000 );
+		state->membank( "bank2" )->set_base( state->m_cart_rom + 0x4000 );
+		state->membank( "bank3" )->set_base( state->m_cart_rom + 0x6000 );
+		state->membank( "bank4" )->set_base( state->m_cart_rom + 0x7000 );
 		break;
 	case 0x10000:
-		memory_set_bankptr( machine, "bank0", state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0x8000 : 0 ) );
-		memory_set_bankptr( machine, "bank1", state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xa000 : 0x2000 ) );
-		memory_set_bankptr( machine, "bank2", state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xc000 : 0x4000 ) );
-		memory_set_bankptr( machine, "bank3", state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xe000 : 0x6000 ) );
-		memory_set_bankptr( machine, "bank4", state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xf000 : 0x7000 ) );
+		state->membank( "bank0" )->set_base( state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0x8000 : 0 ) );
+		state->membank( "bank1" )->set_base( state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xa000 : 0x2000 ) );
+		state->membank( "bank2" )->set_base( state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xc000 : 0x4000 ) );
+		state->membank( "bank3" )->set_base( state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xe000 : 0x6000 ) );
+		state->membank( "bank4" )->set_base( state->m_cart_rom + ( ( state->m_portc & 0x20 ) ? 0xf000 : 0x7000 ) );
 		break;
 	case 0x20000:	/* Pole Position 2 */
 		int base = ( ( state->m_portc >> 5 ) & 0x03 ) * 0x8000 ;
-		memory_set_bankptr( machine, "bank0", state->m_cart_rom + base + 0 );
-		memory_set_bankptr( machine, "bank1", state->m_cart_rom + base + 0x2000 );
-		memory_set_bankptr( machine, "bank2", state->m_cart_rom + base + 0x4000 );
-		memory_set_bankptr( machine, "bank3", state->m_cart_rom + base + 0x6000 );
-		memory_set_bankptr( machine, "bank4", state->m_cart_rom + base + 0x7000 );
+		state->membank( "bank0" )->set_base( state->m_cart_rom + base + 0 );
+		state->membank( "bank1" )->set_base( state->m_cart_rom + base + 0x2000 );
+		state->membank( "bank2" )->set_base( state->m_cart_rom + base + 0x4000 );
+		state->membank( "bank3" )->set_base( state->m_cart_rom + base + 0x6000 );
+		state->membank( "bank4" )->set_base( state->m_cart_rom + base + 0x7000 );
 		/* On-cart RAM is enabled when PC6 is high */
 		if ( state->m_cart_ram && state->m_portc & 0x40 )
 		{
 			state->m_cart_ram_enabled = true;
-			memory_set_bankptr( machine, "bank4", state->m_cart_ram );
+			state->membank( "bank4" )->set_base( state->m_cart_ram );
 		}
 		break;
 	}
@@ -276,12 +275,12 @@ static void scv_set_banks( running_machine &machine )
 	{
 		if ( state->m_cart_ram_size == 0x1000 )
 		{
-			memory_set_bankptr( machine, "bank4", state->m_cart_ram );
+			state->membank( "bank4" )->set_base( state->m_cart_ram );
 		}
 		else
 		{
-			memory_set_bankptr( machine, "bank3", state->m_cart_ram );
-			memory_set_bankptr( machine, "bank4", state->m_cart_ram + 0x1000 );
+			state->membank( "bank3" )->set_base( state->m_cart_ram );
+			state->membank( "bank4" )->set_base( state->m_cart_ram + 0x1000 );
 		}
 		state->m_cart_ram_enabled = true;
 	}
@@ -303,7 +302,7 @@ static DEVICE_START( scv_cart )
 {
 	scv_state *state = device->machine().driver_data<scv_state>();
 
-	state->m_cart_rom = device->machine().region( "cart" )->base();
+	state->m_cart_rom = state->memregion( "cart" )->base();
 	state->m_cart_rom_size = 0;
 	state->m_cart_ram = NULL;
 	state->m_cart_ram_size = 0;
@@ -318,10 +317,10 @@ static DEVICE_IMAGE_LOAD( scv_cart )
 
 	if ( image.software_entry() == NULL )
 	{
-		UINT8 *cart = image.device().machine().region( "cart" )->base();
+		UINT8 *cart = image.device().machine().root_device().memregion( "cart" )->base();
 		int size = image.length();
 
-		if ( size > image.device().machine().region( "cart" )->bytes() )
+		if ( size > state->memregion( "cart" )->bytes() )
 		{
 			image.seterror( IMAGE_ERROR_UNSPECIFIED, "Unsupported cartridge size" );
 			return IMAGE_INIT_FAIL;
@@ -423,30 +422,30 @@ static TIMER_CALLBACK( scv_vb_callback )
 }
 
 
-INLINE void plot_sprite_part( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 pat, UINT8 col )
+INLINE void plot_sprite_part( bitmap_ind16 &bitmap, UINT8 x, UINT8 y, UINT8 pat, UINT8 col )
 {
 	if ( pat & 0x08 )
-		*BITMAP_ADDR16( bitmap, y, x ) = col;
+		bitmap.pix16(y, x ) = col;
 	if ( pat & 0x04 && x < 255 )
-		*BITMAP_ADDR16( bitmap, y, x + 1 ) = col;
+		bitmap.pix16(y, x + 1 ) = col;
 	if ( pat & 0x02 && x < 254 )
-		*BITMAP_ADDR16( bitmap, y, x + 2 ) = col;
+		bitmap.pix16(y, x + 2 ) = col;
 	if ( pat & 0x01 && x < 253 )
-		*BITMAP_ADDR16( bitmap, y, x + 3 ) = col;
+		bitmap.pix16(y, x + 3 ) = col;
 }
 
 
-INLINE void draw_sprite( scv_state *state, bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 tile_idx, UINT8 col, UINT8 left, UINT8 right, UINT8 top, UINT8 bottom, UINT8 clip_y )
+INLINE void draw_sprite( scv_state *state, bitmap_ind16 &bitmap, UINT8 x, UINT8 y, UINT8 tile_idx, UINT8 col, UINT8 left, UINT8 right, UINT8 top, UINT8 bottom, UINT8 clip_y )
 {
 	int j;
 
 	y += clip_y * 2;
 	for ( j = clip_y * 4; j < 32; j += 4 )
 	{
-		UINT8 pat0 = state->m_p_videoram[ tile_idx * 32 + j + 0 ];
-		UINT8 pat1 = state->m_p_videoram[ tile_idx * 32 + j + 1 ];
-		UINT8 pat2 = state->m_p_videoram[ tile_idx * 32 + j + 2 ];
-		UINT8 pat3 = state->m_p_videoram[ tile_idx * 32 + j + 3 ];
+		UINT8 pat0 = state->m_videoram[ tile_idx * 32 + j + 0 ];
+		UINT8 pat1 = state->m_videoram[ tile_idx * 32 + j + 1 ];
+		UINT8 pat2 = state->m_videoram[ tile_idx * 32 + j + 2 ];
+		UINT8 pat3 = state->m_videoram[ tile_idx * 32 + j + 3 ];
 
 		if ( ( top && j < 16 ) || ( bottom && j >= 16 ) )
 		{
@@ -478,7 +477,7 @@ INLINE void draw_sprite( scv_state *state, bitmap_t *bitmap, UINT8 x, UINT8 y, U
 }
 
 
-INLINE void draw_text( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 *char_data, UINT8 fg, UINT8 bg )
+INLINE void draw_text( bitmap_ind16 &bitmap, UINT8 x, UINT8 y, UINT8 *char_data, UINT8 fg, UINT8 bg )
 {
 	int i;
 
@@ -486,32 +485,32 @@ INLINE void draw_text( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 *char_data, UIN
 	{
 		UINT8 d = char_data[i];
 
-		*BITMAP_ADDR16( bitmap, y + i, x + 0 ) = ( d & 0x80 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 1 ) = ( d & 0x40 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 2 ) = ( d & 0x20 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 3 ) = ( d & 0x10 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 4 ) = ( d & 0x08 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 5 ) = ( d & 0x04 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 6 ) = ( d & 0x02 ) ? fg : bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 7 ) = ( d & 0x01 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 0 ) = ( d & 0x80 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 1 ) = ( d & 0x40 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 2 ) = ( d & 0x20 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 3 ) = ( d & 0x10 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 4 ) = ( d & 0x08 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 5 ) = ( d & 0x04 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 6 ) = ( d & 0x02 ) ? fg : bg;
+		bitmap.pix16(y + i, x + 7 ) = ( d & 0x01 ) ? fg : bg;
 	}
 
 	for ( i = 8; i < 16; i++ )
 	{
-		*BITMAP_ADDR16( bitmap, y + i, x + 0 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 1 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 2 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 3 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 4 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 5 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 6 ) = bg;
-		*BITMAP_ADDR16( bitmap, y + i, x + 7 ) = bg;
+		bitmap.pix16(y + i, x + 0 ) = bg;
+		bitmap.pix16(y + i, x + 1 ) = bg;
+		bitmap.pix16(y + i, x + 2 ) = bg;
+		bitmap.pix16(y + i, x + 3 ) = bg;
+		bitmap.pix16(y + i, x + 4 ) = bg;
+		bitmap.pix16(y + i, x + 5 ) = bg;
+		bitmap.pix16(y + i, x + 6 ) = bg;
+		bitmap.pix16(y + i, x + 7 ) = bg;
 	}
 
 }
 
 
-INLINE void draw_semi_graph( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 data, UINT8 fg )
+INLINE void draw_semi_graph( bitmap_ind16 &bitmap, UINT8 x, UINT8 y, UINT8 data, UINT8 fg )
 {
 	int i;
 
@@ -520,45 +519,45 @@ INLINE void draw_semi_graph( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 data, UIN
 
 	for ( i = 0; i < 4; i++ )
 	{
-		*BITMAP_ADDR16(bitmap, y + i, x + 0) = fg;
-		*BITMAP_ADDR16(bitmap, y + i, x + 1) = fg;
-		*BITMAP_ADDR16(bitmap, y + i, x + 2) = fg;
-		*BITMAP_ADDR16(bitmap, y + i, x + 3) = fg;
+		bitmap.pix16(y + i, x + 0) = fg;
+		bitmap.pix16(y + i, x + 1) = fg;
+		bitmap.pix16(y + i, x + 2) = fg;
+		bitmap.pix16(y + i, x + 3) = fg;
 	}
 }
 
 
-INLINE void draw_block_graph( bitmap_t *bitmap, UINT8 x, UINT8 y, UINT8 col )
+INLINE void draw_block_graph( bitmap_ind16 &bitmap, UINT8 x, UINT8 y, UINT8 col )
 {
 	int i;
 
 	for ( i = 0; i < 8; i++ )
 	{
-		*BITMAP_ADDR16(bitmap, y + i, x + 0) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 1) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 2) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 3) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 4) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 5) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 6) = col;
-		*BITMAP_ADDR16(bitmap, y + i, x + 7) = col;
+		bitmap.pix16(y + i, x + 0) = col;
+		bitmap.pix16(y + i, x + 1) = col;
+		bitmap.pix16(y + i, x + 2) = col;
+		bitmap.pix16(y + i, x + 3) = col;
+		bitmap.pix16(y + i, x + 4) = col;
+		bitmap.pix16(y + i, x + 5) = col;
+		bitmap.pix16(y + i, x + 6) = col;
+		bitmap.pix16(y + i, x + 7) = col;
 	}
 }
 
 
-static SCREEN_UPDATE( scv )
+static SCREEN_UPDATE_IND16( scv )
 {
-	scv_state *state = screen->machine().driver_data<scv_state>();
+	scv_state *state = screen.machine().driver_data<scv_state>();
 	int x, y;
-	UINT8 fg = state->m_p_videoram[0x1403] >> 4;
-	UINT8 bg = state->m_p_videoram[0x1403] & 0x0f;
-	UINT8 gr_fg = state->m_p_videoram[0x1401] >> 4;
-	UINT8 gr_bg = state->m_p_videoram[0x1401] & 0x0f;
-	int clip_x = ( state->m_p_videoram[0x1402] & 0x0f ) * 2;
-	int clip_y = state->m_p_videoram[0x1402] >> 4;
+	UINT8 fg = state->m_videoram[0x1403] >> 4;
+	UINT8 bg = state->m_videoram[0x1403] & 0x0f;
+	UINT8 gr_fg = state->m_videoram[0x1401] >> 4;
+	UINT8 gr_bg = state->m_videoram[0x1401] & 0x0f;
+	int clip_x = ( state->m_videoram[0x1402] & 0x0f ) * 2;
+	int clip_y = state->m_videoram[0x1402] >> 4;
 
 	/* Clear the screen */
-	bitmap_fill( bitmap, cliprect, gr_bg );
+	bitmap.fill(gr_bg , cliprect);
 
 	/* Draw background */
 	for ( y = 0; y < 16; y++ )
@@ -566,29 +565,29 @@ static SCREEN_UPDATE( scv )
 		int text_y = 0;
 
 		if ( y < clip_y )
-			text_y = ( state->m_p_videoram[0x1400] & 0x80 ) ? 0 : 1;
+			text_y = ( state->m_videoram[0x1400] & 0x80 ) ? 0 : 1;
 		else
-			text_y = ( state->m_p_videoram[0x1400] & 0x80 ) ? 1 : 0;
+			text_y = ( state->m_videoram[0x1400] & 0x80 ) ? 1 : 0;
 
 		for ( x = 0; x < 32; x++ )
 		{
 			int text_x = 0;
-			UINT8 d = state->m_p_videoram[ 0x1000 + y * 32 + x ];
+			UINT8 d = state->m_videoram[ 0x1000 + y * 32 + x ];
 
 			if ( x < clip_x )
-				text_x = ( state->m_p_videoram[0x1400] & 0x40 ) ? 0 : 1;
+				text_x = ( state->m_videoram[0x1400] & 0x40 ) ? 0 : 1;
 			else
-				text_x = ( state->m_p_videoram[0x1400] & 0x40 ) ? 1 : 0;
+				text_x = ( state->m_videoram[0x1400] & 0x40 ) ? 1 : 0;
 
 			if ( text_x && text_y )
 			{
 				/* Text mode */
-				UINT8 *char_data = screen->machine().region( "charrom" )->base() + ( d & 0x7f ) * 8;
+				UINT8 *char_data = state->memregion( "charrom" )->base() + ( d & 0x7f ) * 8;
 				draw_text( bitmap, x * 8, y * 16, char_data, fg, bg );
 			}
 			else
 			{
-				switch ( state->m_p_videoram[0x1400] & 0x03 )
+				switch ( state->m_videoram[0x1400] & 0x03 )
 				{
 				case 0x01:		/* Semi graphics mode */
 					draw_semi_graph( bitmap, x * 8    , y * 16     , d & 0x80, gr_fg );
@@ -612,20 +611,20 @@ static SCREEN_UPDATE( scv )
 	}
 
 	/* Draw sprites if enabled */
-	if ( state->m_p_videoram[0x1400] & 0x10 )
+	if ( state->m_videoram[0x1400] & 0x10 )
 	{
 		int i;
 
 		for ( i = 0; i < 128; i++ )
 		{
-			UINT8 spr_y = state->m_p_videoram[ 0x1200 + i * 4 ] & 0xfe;
-			UINT8 y_32 = state->m_p_videoram[ 0x1200 + i * 4 ] & 0x01;		/* Xx32 sprite */
-			UINT8 clip = state->m_p_videoram[ 0x1201 + i * 4 ] >> 4;
-			UINT8 col = state->m_p_videoram[ 0x1201 + i * 4 ] & 0x0f;
-			UINT8 spr_x = state->m_p_videoram[ 0x1202 + i * 4 ] & 0xfe;
-			UINT8 x_32 = state->m_p_videoram[ 0x1202 + i * 4 ] & 0x01;		/* 32xX sprite */
-			UINT8 tile_idx = state->m_p_videoram[ 0x1203 + i * 4 ] & 0x7f;
-			UINT8 half = state->m_p_videoram[ 0x1203 + i * 4] & 0x80;
+			UINT8 spr_y = state->m_videoram[ 0x1200 + i * 4 ] & 0xfe;
+			UINT8 y_32 = state->m_videoram[ 0x1200 + i * 4 ] & 0x01;		/* Xx32 sprite */
+			UINT8 clip = state->m_videoram[ 0x1201 + i * 4 ] >> 4;
+			UINT8 col = state->m_videoram[ 0x1201 + i * 4 ] & 0x0f;
+			UINT8 spr_x = state->m_videoram[ 0x1202 + i * 4 ] & 0xfe;
+			UINT8 x_32 = state->m_videoram[ 0x1202 + i * 4 ] & 0x01;		/* 32xX sprite */
+			UINT8 tile_idx = state->m_videoram[ 0x1203 + i * 4 ] & 0x7f;
+			UINT8 half = state->m_videoram[ 0x1203 + i * 4] & 0x80;
 			UINT8 left = 1;
 			UINT8 right = 1;
 			UINT8 top = 1;
@@ -669,7 +668,7 @@ static SCREEN_UPDATE( scv )
 			}
 
 			/* Check if 2 color sprites are enabled */
-			if ( ( state->m_p_videoram[0x1400] & 0x20 ) && ( i & 0x20 ) )
+			if ( ( state->m_videoram[0x1400] & 0x20 ) && ( i & 0x20 ) )
 			{
 				/* 2 color sprite handling */
 				draw_sprite( state, bitmap, spr_x, spr_y, tile_idx, col, left, right, top, bottom, clip );
@@ -760,9 +759,8 @@ static MACHINE_CONFIG_START( scv, scv_state )
 
 	/* Video chip is EPOCH TV-1 */
 	MCFG_SCREEN_ADD( "screen", RASTER )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
 	MCFG_SCREEN_RAW_PARAMS( XTAL_14_31818MHz/2, 456, 17, 209, 262, 0, 240 )	/* TODO: Verify */
-	MCFG_SCREEN_UPDATE( scv )
+	MCFG_SCREEN_UPDATE_STATIC( scv )
 
 	MCFG_GFXDECODE(scv)
 	MCFG_PALETTE_LENGTH( 16 )
@@ -798,9 +796,8 @@ static MACHINE_CONFIG_START( scv_pal, scv_state )
 
 	/* Video chip is EPOCH TV-1A */
 	MCFG_SCREEN_ADD( "screen", RASTER )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
 	MCFG_SCREEN_RAW_PARAMS( XTAL_13_4MHz/2, 456, 17, 209, 342, 0, 256 )		/* TODO: Verify */
-	MCFG_SCREEN_UPDATE( scv )
+	MCFG_SCREEN_UPDATE_STATIC( scv )
 
 	MCFG_GFXDECODE(scv)
 	MCFG_PALETTE_LENGTH( 16 )
