@@ -62,7 +62,6 @@ Notes:
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -112,14 +111,14 @@ WRITE8_MEMBER( sg1000_state::tvdraw_axis_w )
 {
 	if (data & 0x01)
 	{
-		m_tvdraw_data = input_port_read(machine(), "TVDRAW_X");
+		m_tvdraw_data = ioport("TVDRAW_X")->read();
 
 		if (m_tvdraw_data < 4) m_tvdraw_data = 4;
 		if (m_tvdraw_data > 251) m_tvdraw_data = 251;
 	}
 	else
 	{
-		m_tvdraw_data = input_port_read(machine(), "TVDRAW_Y") + 32;
+		m_tvdraw_data = ioport("TVDRAW_Y")->read() + 32;
 	}
 }
 
@@ -129,7 +128,7 @@ WRITE8_MEMBER( sg1000_state::tvdraw_axis_w )
 
 READ8_MEMBER( sg1000_state::tvdraw_status_r )
 {
-	return input_port_read(machine(), "TVDRAW_PEN");
+	return ioport("TVDRAW_PEN")->read();
 }
 
 /*-------------------------------------------------
@@ -269,9 +268,9 @@ ADDRESS_MAP_END
     INPUT_CHANGED( trigger_nmi )
 -------------------------------------------------*/
 
-static INPUT_CHANGED( trigger_nmi )
+INPUT_CHANGED_MEMBER( sg1000_state::trigger_nmi )
 {
-	cputag_set_input_line(field.machine(), Z80_TAG, INPUT_LINE_NMI, (input_port_read(field.machine(), "NMI") ? CLEAR_LINE : ASSERT_LINE));
+	m_maincpu->set_input_line(INPUT_LINE_NMI, newval ? CLEAR_LINE : ASSERT_LINE);
 }
 
 /*-------------------------------------------------
@@ -312,7 +311,7 @@ static INPUT_PORTS_START( sg1000 )
 	PORT_BIT( 0xf0, IP_ACTIVE_LOW, IPT_UNUSED )
 
 	PORT_START("NMI")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_START ) PORT_NAME("PAUSE") PORT_CODE(KEYCODE_P) PORT_CHANGED(trigger_nmi, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_START ) PORT_NAME("PAUSE") PORT_CODE(KEYCODE_P) PORT_CHANGED_MEMBER(DEVICE_SELF, sg1000_state, trigger_nmi, 0)
 
 	PORT_INCLUDE( tvdraw )
 INPUT_PORTS_END
@@ -495,7 +494,7 @@ INPUT_PORTS_START( sk1100 )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_PLAYER(2)
 
 	PORT_START("NMI")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("RESET") PORT_CODE(KEYCODE_F10) PORT_CHANGED(trigger_nmi, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD ) PORT_NAME("RESET") PORT_CODE(KEYCODE_F10) PORT_CHANGED_MEMBER(DEVICE_SELF, sg1000_state, trigger_nmi, 0)
 INPUT_PORTS_END
 
 /*-------------------------------------------------
@@ -544,14 +543,6 @@ static TMS9928A_INTERFACE(sg1000_tms9918a_interface)
 	DEVCB_LINE(sg1000_vdp_interrupt)
 };
 
-static SCREEN_UPDATE( sg1000 )
-{
-	tms9918a_device *tms9918a = screen->machine().device<tms9918a_device>( TMS9918A_TAG );
-
-	tms9918a->update( bitmap, cliprect );
-	return 0;
-}
-
 /*-------------------------------------------------
     I8255_INTERFACE( sc3000_ppi_intf )
 -------------------------------------------------*/
@@ -573,7 +564,7 @@ READ8_MEMBER( sc3000_state::ppi_pa_r )
 
 	static const char *const keynames[] = { "PA0", "PA1", "PA2", "PA3", "PA4", "PA5", "PA6", "PA7" };
 
-	return input_port_read(machine(), keynames[m_keylatch]);
+	return ioport(keynames[m_keylatch])->read();
 }
 
 READ8_MEMBER( sc3000_state::ppi_pb_r )
@@ -594,7 +585,7 @@ READ8_MEMBER( sc3000_state::ppi_pb_r )
 	static const char *const keynames[] = { "PB0", "PB1", "PB2", "PB3", "PB4", "PB5", "PB6", "PB7" };
 
 	/* keyboard */
-	UINT8 data = input_port_read(machine(), keynames[m_keylatch]);
+	UINT8 data = ioport(keynames[m_keylatch])->read();
 
 	/* cartridge contact */
 	data |= 0x10;
@@ -672,15 +663,15 @@ void sg1000_state::install_cartridge(UINT8 *ptr, int size)
 	case 40 * 1024:
 		program->install_read_bank(0x8000, 0x9fff, "bank1");
 		program->unmap_write(0x8000, 0x9fff);
-		memory_configure_bank(machine(), "bank1", 0, 1, machine().region(Z80_TAG)->base() + 0x8000, 0);
-		memory_set_bank(machine(), "bank1", 0);
+		membank("bank1")->configure_entry(0, memregion(Z80_TAG)->base() + 0x8000);
+		membank("bank1")->set_entry(0);
 		break;
 
 	case 48 * 1024:
 		program->install_read_bank(0x8000, 0xbfff, "bank1");
 		program->unmap_write(0x8000, 0xbfff);
-		memory_configure_bank(machine(), "bank1", 0, 1, machine().region(Z80_TAG)->base() + 0x8000, 0);
-		memory_set_bank(machine(), "bank1", 0);
+		membank("bank1")->configure_entry(0, memregion(Z80_TAG)->base() + 0x8000);
+		membank("bank1")->set_entry(0);
 		break;
 
 	default:
@@ -708,7 +699,9 @@ static DEVICE_IMAGE_LOAD( sg1000_cart )
 	running_machine &machine = image.device().machine();
 	sg1000_state *state = machine.driver_data<sg1000_state>();
 	address_space *program = machine.device(Z80_TAG)->memory().space(AS_PROGRAM);
-	UINT8 *ptr = machine.region(Z80_TAG)->base();
+	UINT8 *ptr = state->memregion(Z80_TAG)->base();
+	UINT32 ram_size = 0x400;
+	bool install_2000_ram = false;
 	UINT32 size;
 
 	if (image.software_entry() == NULL)
@@ -721,13 +714,92 @@ static DEVICE_IMAGE_LOAD( sg1000_cart )
 	{
 		size = image.get_software_region_length("rom");
 		memcpy(ptr, image.get_software_region("rom"), size);
+
+		const char *needs_addon = image.get_feature("needs_addon");
+
+		//
+		// The Dahjee (Type A) RAM cartridge had 9KB of RAM. 1KB replaces
+		// the main unit's system ram (0xC000-0xC3FF) and 8K which appears
+		// at 0x2000-0x3FFF in the memory map.
+		//
+		if ( needs_addon && ! strcmp( needs_addon, "dahjee_type_a" ) )
+		{
+			install_2000_ram = true;
+		}
+
+		//
+		// The Dahjee (Type B) RAM cartridge had 8KB of RAM which
+		// replaces the main unit RAM in the memory map. (0xC000-0xDFFF area)
+		//
+		if ( needs_addon && ! strcmp( needs_addon, "dahjee_type_b" ) )
+		{
+			ram_size = 0x2000;
+		}
+	}
+
+	// Try to auto-detect special features
+	if ( ! install_2000_ram && ram_size == 0x400 )
+	{
+		if ( size >= 0x8000 )
+		{
+			int x2000_3000 = 0, xd000_e000_f000 = 0, x2000_ff = 0;
+
+			for ( int i = 0; i < 0x8000; i++ )
+			{
+				if ( ptr[i] == 0x32 )
+				{
+					UINT16 addr = ptr[i+1] | ( ptr[i+2] << 8 );
+
+					switch ( addr & 0xF000 )
+					{
+					case 0x2000:
+					case 0x3000:
+						i += 2;
+						x2000_3000++;
+						break;
+
+					case 0xD000:
+					case 0xE000:
+					case 0xF000:
+						i += 2;
+						xd000_e000_f000++;
+						break;
+					}
+				}
+			}
+			for ( int i = 0x2000; i < 0x4000; i++ )
+			{
+				if ( ptr[i] == 0xFF )
+				{
+					x2000_ff++;
+				}
+			}
+			if ( x2000_ff == 0x2000 && ( xd000_e000_f000 > 10 || x2000_3000 > 10 ) )
+			{
+				if ( xd000_e000_f000 > x2000_3000 )
+				{
+					// Type B
+					ram_size = 0x2000;
+				}
+				else
+				{
+					// Type A
+					install_2000_ram = true;
+				}
+			}
+		}
 	}
 
 	/* cartridge ROM banking */
 	state->install_cartridge(ptr, size);
 
+	if ( install_2000_ram )
+	{
+		program->install_ram(0x2000, 0x3FFF);
+	}
+
 	/* work RAM banking */
-	program->install_readwrite_bank(0xc000, 0xc3ff, 0, 0x3c00, "bank2");
+	program->install_readwrite_bank(0xc000, 0xc000 + ram_size - 1, 0, 0x4000 - ram_size, "bank2");
 
 	return IMAGE_INIT_PASS;
 }
@@ -741,7 +813,7 @@ static DEVICE_IMAGE_LOAD( omv_cart )
 	running_machine &machine = image.device().machine();
 	sg1000_state *state = machine.driver_data<sg1000_state>();
 	UINT32 size;
-	UINT8 *ptr = machine.region(Z80_TAG)->base();
+	UINT8 *ptr = state->memregion(Z80_TAG)->base();
 
 	if (image.software_entry() == NULL)
 	{
@@ -797,7 +869,7 @@ static DEVICE_IMAGE_LOAD( sc3000_cart )
 {
 	running_machine &machine = image.device().machine();
 	sc3000_state *state = machine.driver_data<sc3000_state>();
-	UINT8 *ptr = machine.region(Z80_TAG)->base();
+	UINT8 *ptr = state->memregion(Z80_TAG)->base();
 	UINT32 size;
 
 	if (image.software_entry() == NULL)
@@ -840,7 +912,7 @@ READ8_MEMBER( sf7000_state::ppi_pa_r )
 	UINT8 data = 0;
 
 	data |= m_fdc_irq;
-	data |= centronics_busy_r(m_centronics) << 1;
+	data |= m_centronics->busy_r() << 1;
 	data |= m_fdc_index << 2;
 
 	return data;
@@ -875,10 +947,10 @@ WRITE8_MEMBER( sf7000_state::ppi_pc_w )
 	}
 
 	/* ROM selection */
-	memory_set_bank(machine(), "bank1", BIT(data, 6));
+	membank("bank1")->set_entry(BIT(data, 6));
 
 	/* printer strobe */
-	centronics_strobe_w(m_centronics, BIT(data, 7));
+	m_centronics->strobe_w(BIT(data, 7));
 }
 
 static I8255_INTERFACE( sf7000_ppi_intf )
@@ -886,7 +958,7 @@ static I8255_INTERFACE( sf7000_ppi_intf )
 	DEVCB_DRIVER_MEMBER(sf7000_state, ppi_pa_r),				// Port A read
 	DEVCB_NULL,													// Port A write
 	DEVCB_NULL,													// Port B read
-	DEVCB_DEVICE_HANDLER(CENTRONICS_TAG, centronics_data_w),	// Port B write
+	DEVCB_DEVICE_MEMBER(CENTRONICS_TAG, centronics_device, write),	// Port B write
 	DEVCB_NULL,													// Port C read
 	DEVCB_DRIVER_MEMBER(sf7000_state, ppi_pc_w)					// Port C write
 };
@@ -960,7 +1032,7 @@ static const floppy_interface sf7000_floppy_interface =
 
 static TIMER_CALLBACK( lightgun_tick )
 {
-	UINT8 *rom = machine.region(Z80_TAG)->base();
+	UINT8 *rom = machine.root_device().memregion(Z80_TAG)->base();
 
 	if (IS_CARTRIDGE_TV_DRAW(rom))
 	{
@@ -984,7 +1056,7 @@ void sg1000_state::machine_start()
 	machine().scheduler().timer_set(attotime::zero, FUNC(lightgun_tick));
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_tvdraw_data);
+	save_item(NAME(m_tvdraw_data));
 }
 
 /*-------------------------------------------------
@@ -997,8 +1069,8 @@ void sc3000_state::machine_start()
 	machine().scheduler().timer_set(attotime::zero, FUNC(lightgun_tick));
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_tvdraw_data);
-	state_save_register_global(machine(), m_keylatch);
+	save_item(NAME(m_tvdraw_data));
+	save_item(NAME(m_keylatch));
 }
 
 
@@ -1009,14 +1081,14 @@ void sc3000_state::machine_start()
 void sf7000_state::machine_start()
 {
 	/* configure memory banking */
-	memory_configure_bank(machine(), "bank1", 0, 1, machine().region(Z80_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", 1, 1, ram_get_ptr(m_ram), 0);
-	memory_configure_bank(machine(), "bank2", 0, 1, ram_get_ptr(m_ram), 0);
+	membank("bank1")->configure_entry(0, memregion(Z80_TAG)->base());
+	membank("bank1")->configure_entry(1, m_ram->pointer());
+	membank("bank2")->configure_entry(0, m_ram->pointer());
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_fdc_irq);
-	state_save_register_global(machine(), m_fdc_index);
+	save_item(NAME(m_keylatch));
+	save_item(NAME(m_fdc_irq));
+	save_item(NAME(m_fdc_index));
 }
 
 /*-------------------------------------------------
@@ -1025,8 +1097,8 @@ void sf7000_state::machine_start()
 
 void sf7000_state::machine_reset()
 {
-	memory_set_bank(machine(), "bank1", 0);
-	memory_set_bank(machine(), "bank2", 0);
+	membank("bank1")->set_entry(0);
+	membank("bank2")->set_entry(0);
 }
 
 /***************************************************************************
@@ -1046,7 +1118,7 @@ static MACHINE_CONFIG_START( sg1000, sg1000_state )
     /* video hardware */
 	MCFG_TMS9928A_ADD( TMS9918A_TAG, TMS9918A, sg1000_tms9918a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( sg1000 )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9918A_TAG, tms9918a_device, screen_update )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -1099,7 +1171,7 @@ static MACHINE_CONFIG_START( sc3000, sc3000_state )
     /* video hardware */
 	MCFG_TMS9928A_ADD( TMS9918A_TAG, TMS9918A, sg1000_tms9918a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( sg1000 )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9918A_TAG, tms9918a_device, screen_update )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -1138,7 +1210,7 @@ static MACHINE_CONFIG_START( sf7000, sf7000_state )
     /* video hardware */
 	MCFG_TMS9928A_ADD( TMS9918A_TAG, TMS9918A, sg1000_tms9918a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( sg1000 )
+	MCFG_SCREEN_UPDATE_DEVICE( TMS9918A_TAG, tms9918a_device, screen_update )
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -1152,7 +1224,7 @@ static MACHINE_CONFIG_START( sf7000, sf7000_state )
 	MCFG_UPD765A_ADD(UPD765_TAG, sf7000_upd765_interface)
 	MCFG_LEGACY_FLOPPY_DRIVE_ADD(FLOPPY_0, sf7000_floppy_interface)
 //  MCFG_PRINTER_ADD("sp400") /* serial printer */
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
 	MCFG_CASSETTE_ADD(CASSETTE_TAG, sc3000_cassette_interface)
 
 	/* internal ram */
