@@ -154,10 +154,9 @@ NOTES (2011-08-08)
                          goes wrong.
 
 ********************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "includes/sorcerer.h"
-
+#include "formats/z80bin.h"
 
 static ADDRESS_MAP_START( sorcerer_mem, AS_PROGRAM, 8, sorcerer_state)
 	ADDRESS_MAP_UNMAP_HIGH
@@ -193,7 +192,7 @@ ADDRESS_MAP_END
 static INPUT_PORTS_START(sorcerer)
 	PORT_START("VS")
 	/* vblank */
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_VBLANK)
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_CUSTOM) PORT_VBLANK("screen")
 
 	/* line 0 */
 	PORT_START("LINE0")
@@ -343,17 +342,17 @@ static GFXDECODE_START( sorcerer )
 	GFXDECODE_ENTRY( "maincpu", 0xf800, sorcerer_charlayout, 0, 1 )
 GFXDECODE_END
 
-static SCREEN_UPDATE( sorcerer )
+static SCREEN_UPDATE_IND16( sorcerer )
 {
 	UINT8 y,ra,chr,gfx;
 	UINT16 sy=0,ma=0xf080,x;
-	UINT8 *RAM = screen->machine().region("maincpu")->base();
+	UINT8 *RAM = screen.machine().root_device().memregion("maincpu")->base();
 
 	for (y = 0; y < 30; y++)
 	{
 		for (ra = 0; ra < 8; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 
 			for (x = ma; x < ma+64; x++)
 			{
@@ -428,10 +427,9 @@ static MACHINE_CONFIG_START( sorcerer, sorcerer_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(200))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(64*8, 30*8)
 	MCFG_SCREEN_VISIBLE_AREA(0, 64*8-1, 0, 30*8-1)
-	MCFG_SCREEN_UPDATE( sorcerer )
+	MCFG_SCREEN_UPDATE_STATIC( sorcerer )
 
 	MCFG_GFXDECODE(sorcerer)
 	MCFG_PALETTE_LENGTH(2)
@@ -449,11 +447,11 @@ static MACHINE_CONFIG_START( sorcerer, sorcerer_state )
 	MCFG_AY31015_ADD( "uart", sorcerer_ay31015_config )
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	/* quickload */
 	MCFG_SNAPSHOT_ADD("snapshot", sorcerer, "snp", 2)
-	MCFG_Z80BIN_QUICKLOAD_ADD("quickload", sorcerer, 3)
+	MCFG_QUICKLOAD_ADD("quickload", sorcerer, "bin", 3)
 
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, sorcerer_cassette_interface )
 	MCFG_CASSETTE_ADD( CASSETTE2_TAG, sorcerer_cassette_interface )
@@ -485,8 +483,9 @@ MACHINE_CONFIG_END
 
 static DRIVER_INIT( sorcerer )
 {
-	UINT8 *RAM = machine.region("maincpu")->base();
-	memory_configure_bank(machine, "boot", 0, 2, &RAM[0x0000], 0xe000);
+	sorcerer_state *state = machine.driver_data<sorcerer_state>();
+	UINT8 *RAM = state->memregion("maincpu")->base();
+	state->membank("boot")->configure_entries(0, 2, &RAM[0x0000], 0xe000);
 }
 
 /***************************************************************************
