@@ -292,19 +292,18 @@ SamRam
  bit 2-0: border colour
 */
 
-WRITE8_HANDLER(spectrum_port_fe_w)
+WRITE8_MEMBER(spectrum_state::spectrum_port_fe_w)
 {
-	spectrum_state *state = space->machine().driver_data<spectrum_state>();
-	device_t *speaker = space->machine().device(SPEAKER_TAG);
+	device_t *speaker = machine().device(SPEAKER_TAG);
 	unsigned char Changed;
 
-	Changed = state->m_port_fe_data^data;
+	Changed = m_port_fe_data^data;
 
 	/* border colour changed? */
 	if ((Changed & 0x07)!=0)
 	{
 		/* yes - send event */
-		spectrum_EventList_AddItemOffset(space->machine(), 0x0fe, data & 0x07, space->machine().device<cpu_device>("maincpu")->attotime_to_cycles(space->machine().primary_screen->scan_period() * space->machine().primary_screen->vpos()));
+		spectrum_EventList_AddItemOffset(machine(), 0x0fe, data & 0x07, machine().device<cpu_device>("maincpu")->attotime_to_cycles(machine().primary_screen->scan_period() * machine().primary_screen->vpos()));
 	}
 
 	if ((Changed & (1<<4))!=0)
@@ -316,19 +315,19 @@ WRITE8_HANDLER(spectrum_port_fe_w)
 	if ((Changed & (1<<3))!=0)
 	{
 		/* write cassette data */
-		space->machine().device<cassette_image_device>(CASSETTE_TAG)->output((data & (1<<3)) ? -1.0 : +1.0);
+		machine().device<cassette_image_device>(CASSETTE_TAG)->output((data & (1<<3)) ? -1.0 : +1.0);
 	}
 
-	state->m_port_fe_data = data;
+	m_port_fe_data = data;
 }
 
-DIRECT_UPDATE_HANDLER(spectrum_direct)
+DIRECT_UPDATE_MEMBER(spectrum_state::spectrum_direct)
 {
     /* Hack for correct handling 0xffff interrupt vector */
     if (address == 0x0001)
-        if (cpu_get_reg(machine.device("maincpu"), STATE_GENPCBASE)==0xffff)
+        if (cpu_get_reg(machine().device("maincpu"), STATE_GENPCBASE)==0xffff)
         {
-            cpu_set_reg(machine.device("maincpu"), Z80_PC, 0xfff4);
+            cpu_set_reg(machine().device("maincpu"), Z80_PC, 0xfff4);
             return 0xfff4;
         }
     return address;
@@ -336,21 +335,21 @@ DIRECT_UPDATE_HANDLER(spectrum_direct)
 
 /* KT: more accurate keyboard reading */
 /* DJR: Spectrum+ keys added */
-READ8_HANDLER(spectrum_port_fe_r)
+READ8_MEMBER(spectrum_state::spectrum_port_fe_r)
 {
 	int lines = offset >> 8;
 	int data = 0xff;
 
-	int cs_extra1 = input_port_read_safe(space->machine(), "PLUS0", 0xff) & 0x1f;
-	int cs_extra2 = input_port_read_safe(space->machine(), "PLUS1", 0xff) & 0x1f;
-	int cs_extra3 = input_port_read_safe(space->machine(), "PLUS2", 0xff) & 0x1f;
-	int ss_extra1 = input_port_read_safe(space->machine(), "PLUS3", 0xff) & 0x1f;
-	int ss_extra2 = input_port_read_safe(space->machine(), "PLUS4", 0xff) & 0x1f;
+	int cs_extra1 = ioport("PLUS0")->read_safe(0xff) & 0x1f;
+	int cs_extra2 = ioport("PLUS1")->read_safe(0xff) & 0x1f;
+	int cs_extra3 = ioport("PLUS2")->read_safe(0xff) & 0x1f;
+	int ss_extra1 = ioport("PLUS3")->read_safe(0xff) & 0x1f;
+	int ss_extra2 = ioport("PLUS4")->read_safe(0xff) & 0x1f;
 
 	/* Caps - V */
 	if ((lines & 1) == 0)
 	{
-		data &= input_port_read(space->machine(), "LINE0");
+		data &= ioport("LINE0")->read();
 		/* CAPS for extra keys */
 		if (cs_extra1 != 0x1f || cs_extra2 != 0x1f || cs_extra3 != 0x1f)
 			data &= ~0x01;
@@ -358,32 +357,32 @@ READ8_HANDLER(spectrum_port_fe_r)
 
 	/* A - G */
 	if ((lines & 2) == 0)
-		data &= input_port_read(space->machine(), "LINE1");
+		data &= ioport("LINE1")->read();
 
 	/* Q - T */
 	if ((lines & 4) == 0)
-		data &= input_port_read(space->machine(), "LINE2");
+		data &= ioport("LINE2")->read();
 
 	/* 1 - 5 */
 	if ((lines & 8) == 0)
-		data &= input_port_read(space->machine(), "LINE3") & cs_extra1;
+		data &= ioport("LINE3")->read() & cs_extra1;
 
 	/* 6 - 0 */
 	if ((lines & 16) == 0)
-		data &= input_port_read(space->machine(), "LINE4") & cs_extra2;
+		data &= ioport("LINE4")->read() & cs_extra2;
 
 	/* Y - P */
 	if ((lines & 32) == 0)
-		data &= input_port_read(space->machine(), "LINE5") & ss_extra1;
+		data &= ioport("LINE5")->read() & ss_extra1;
 
 	/* H - Enter */
 	if ((lines & 64) == 0)
-		data &= input_port_read(space->machine(), "LINE6");
+		data &= ioport("LINE6")->read();
 
 		/* B - Space */
 	if ((lines & 128) == 0)
 	{
-		data &= input_port_read(space->machine(), "LINE7") & cs_extra3 & ss_extra2;
+		data &= ioport("LINE7")->read() & cs_extra3 & ss_extra2;
 		/* SYMBOL SHIFT for extra keys */
 		if (ss_extra1 != 0x1f || ss_extra2 != 0x1f)
 			data &= ~0x02;
@@ -392,57 +391,56 @@ READ8_HANDLER(spectrum_port_fe_r)
 	data |= (0xe0); /* Set bits 5-7 - as reset above */
 
 	/* cassette input from wav */
-	if ((space->machine().device<cassette_image_device>(CASSETTE_TAG))->input() > 0.0038 )
+	if ((machine().device<cassette_image_device>(CASSETTE_TAG))->input() > 0.0038 )
 	{
 		data &= ~0x40;
 	}
 
 	/* Issue 2 Spectrums default to having bits 5, 6 & 7 set.
     Issue 3 Spectrums default to having bits 5 & 7 set and bit 6 reset. */
-	if (input_port_read(space->machine(), "CONFIG") & 0x80)
+	if (ioport("CONFIG")->read() & 0x80)
 		data ^= (0x40);
 
 	return data;
 }
 
 /* kempston joystick interface */
-READ8_HANDLER(spectrum_port_1f_r)
+READ8_MEMBER(spectrum_state::spectrum_port_1f_r)
 {
-	return input_port_read(space->machine(), "KEMPSTON") & 0x1f;
+	return ioport("KEMPSTON")->read() & 0x1f;
 }
 
 /* fuller joystick interface */
-READ8_HANDLER(spectrum_port_7f_r)
+READ8_MEMBER(spectrum_state::spectrum_port_7f_r)
 {
-	return input_port_read(space->machine(), "FULLER") | (0xff^0x8f);
+	return ioport("FULLER")->read() | (0xff^0x8f);
 }
 
 /* mikrogen joystick interface */
-READ8_HANDLER(spectrum_port_df_r)
+READ8_MEMBER(spectrum_state::spectrum_port_df_r)
 {
-	return input_port_read(space->machine(), "MIKROGEN") | (0xff^0x1f);
+	return ioport("MIKROGEN")->read() | (0xff^0x1f);
 }
 
-static READ8_HANDLER ( spectrum_port_ula_r )
+READ8_MEMBER(spectrum_state::spectrum_port_ula_r)
 {
-	spectrum_state *state = space->machine().driver_data<spectrum_state>();
-	int vpos = space->machine().primary_screen->vpos();
+	int vpos = machine().primary_screen->vpos();
 
-	return vpos<193 ? state->m_video_ram[(vpos&0xf8)<<2]:0xff;
+	return vpos<193 ? m_video_ram[(vpos&0xf8)<<2]:0xff;
 }
 
 /* Memory Maps */
 
-static ADDRESS_MAP_START (spectrum_mem, AS_PROGRAM, 8)
+static ADDRESS_MAP_START (spectrum_mem, AS_PROGRAM, 8, spectrum_state )
 	AM_RANGE(0x0000, 0x3fff) AM_ROM
-	AM_RANGE(0x4000, 0x5aff) AM_RAM AM_BASE_MEMBER(spectrum_state,m_video_ram)
+	AM_RANGE(0x4000, 0x5aff) AM_RAM AM_SHARE("video_ram")
 //  AM_RANGE(0x5b00, 0x7fff) AM_RAM
 //  AM_RANGE(0x8000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
 /* ports are not decoded full.
 The function decodes the ports appropriately */
-static ADDRESS_MAP_START (spectrum_io, AS_IO, 8)
+static ADDRESS_MAP_START (spectrum_io, AS_IO, 8, spectrum_state )
 	AM_RANGE(0x00, 0x00) AM_READWRITE(spectrum_port_fe_r,spectrum_port_fe_w) AM_MIRROR(0xfffe) AM_MASK(0xffff)
 	AM_RANGE(0x1f, 0x1f) AM_READ(spectrum_port_1f_r) AM_MIRROR(0xff00)
 	AM_RANGE(0x7f, 0x7f) AM_READ(spectrum_port_7f_r) AM_MIRROR(0xff00)
@@ -463,25 +461,25 @@ static INPUT_PORTS_START( spec_joys )
 	PORT_CONFSETTING(  0x03, "Mikrogen" )
 
 	PORT_START("KEMPSTON") /* Kempston joystick interface */
-	PORT_BIT(0x01, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Kempston)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x01)
-	PORT_BIT(0x02, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Kempston)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x01)
-	PORT_BIT(0x04, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Kempston)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x01)
-	PORT_BIT(0x08, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Kempston)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x01)
-	PORT_BIT(0x10, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Kempston)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x01)
+	PORT_BIT(0x01, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Kempston)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x01)
+	PORT_BIT(0x02, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Kempston)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x01)
+	PORT_BIT(0x04, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Kempston)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x01)
+	PORT_BIT(0x08, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Kempston)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x01)
+	PORT_BIT(0x10, IP_ACTIVE_HIGH, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Kempston)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x01)
 
 	PORT_START("FULLER") /* Fuller joystick interface */
-	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Fuller)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x02)
-	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Fuller)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x02)
-	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Fuller)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x02)
-	PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Fuller)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x02)
-	PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Fuller)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x02)
+	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Fuller)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x02)
+	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Fuller)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x02)
+	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Fuller)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x02)
+	PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Fuller)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x02)
+	PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Fuller)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x02)
 
 	PORT_START("MIKROGEN") /* Mikrogen joystick interface */
-	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Mikrogen)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x03)
-	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Mikrogen)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x03)
-	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Mikrogen)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x03)
-	PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Mikrogen)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x03)
-	PORT_BIT(0x10, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Mikrogen)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, PORTCOND_EQUALS, 0x03)
+	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Up (Mikrogen)") PORT_CODE(JOYCODE_Y_UP_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x03)
+	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Down (Mikrogen)") PORT_CODE(JOYCODE_Y_DOWN_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x03)
+	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Right (Mikrogen)") PORT_CODE(JOYCODE_X_RIGHT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x03)
+	PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Left (Mikrogen)") PORT_CODE(JOYCODE_X_LEFT_SWITCH) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x03)
+	PORT_BIT(0x10, IP_ACTIVE_LOW, IPT_KEYBOARD) PORT_NAME("P1 Joystick Fire (Mikrogen)") PORT_CODE(JOYCODE_BUTTON1) PORT_CONDITION("JOY_INTF", 0x0f, EQUALS, 0x03)
 INPUT_PORTS_END
 
 /*
@@ -624,7 +622,7 @@ DRIVER_INIT( spectrum )
 {
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
 
-	switch (ram_get_size(machine.device(RAM_TAG)))
+	switch (machine.device<ram_device>(RAM_TAG)->size())
 	{
 	    case 48*1024:
 		space->install_ram(0x8000, 0xffff, NULL); // Fall through
@@ -638,7 +636,7 @@ MACHINE_RESET( spectrum )
 	spectrum_state *state = machine.driver_data<spectrum_state>();
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
 
-	space->set_direct_update_handler(direct_update_delegate(FUNC(spectrum_direct), &machine));
+	space->set_direct_update_handler(direct_update_delegate(FUNC(spectrum_state::spectrum_direct), state));
 
 	state->m_port_7ffd_data = -1;
 	state->m_port_1ffd_data = -1;
@@ -691,7 +689,7 @@ static DEVICE_IMAGE_LOAD( spectrum_cart )
 			return IMAGE_INIT_FAIL;
 		}
 
-		if (image.fread(image.device().machine().region("maincpu")->base(), filesize) != filesize)
+		if (image.fread(image.device().machine().root_device().memregion("maincpu")->base(), filesize) != filesize)
 		{
 			image.seterror(IMAGE_ERROR_UNSPECIFIED, "Error loading file");
 			return IMAGE_INIT_FAIL;
@@ -700,7 +698,7 @@ static DEVICE_IMAGE_LOAD( spectrum_cart )
 	else
 	{
 		filesize = image.get_software_region_length("rom");
-		memcpy(image.device().machine().region("maincpu")->base(), image.get_software_region("rom"), filesize);
+		memcpy(image.device().machine().root_device().memregion("maincpu")->base(), image.get_software_region("rom"), filesize);
 	}
 	return IMAGE_INIT_PASS;
 }
@@ -720,11 +718,10 @@ MACHINE_CONFIG_START( spectrum_common, spectrum_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50.08)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(SPEC_SCREEN_WIDTH, SPEC_SCREEN_HEIGHT)
 	MCFG_SCREEN_VISIBLE_AREA(0, SPEC_SCREEN_WIDTH-1, 0, SPEC_SCREEN_HEIGHT-1)
-	MCFG_SCREEN_UPDATE( spectrum )
-	MCFG_SCREEN_EOF( spectrum )
+	MCFG_SCREEN_UPDATE_STATIC( spectrum )
+	MCFG_SCREEN_VBLANK_STATIC( spectrum )
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_PALETTE_INIT( spectrum )
@@ -944,6 +941,12 @@ ROM_START(didakm91)
 	ROM_CART_LOAD("cart", 0x0000, 0x4000, ROM_NOCLEAR | ROM_NOMIRROR | ROM_OPTIONAL)
 ROM_END
 
+ROM_START(didakm92)
+	ROM_REGION(0x10000,"maincpu",0)
+	ROM_LOAD("didakm92.bin",0x0000,0x4000, CRC(57264d4f) SHA1(23644fe949cbf527747959d07b72db01de378c4c))
+	ROM_CART_LOAD("cart", 0x0000, 0x4000, ROM_NOCLEAR | ROM_NOMIRROR | ROM_OPTIONAL)
+ROM_END
+
 ROM_START(didaktk)
 	ROM_REGION(0x10000,"maincpu",0)
 	ROM_LOAD("didaktk.rom",0x0000,0x4000, CRC(8ec8a625) SHA1(cba35517d33a5c97e3d9110f12a417c6c5cdeca8))
@@ -1024,8 +1027,8 @@ COMP( 1985, hc85,     spectrum, 0,		spectrum,		spectrum,	spectrum,	"ICE-Felix",
 COMP( 1988, hc88,     spectrum, 0,		spectrum,		spectrum,	spectrum,	"ICE-Felix",	"HC-88" , GAME_NOT_WORKING)
 COMP( 1990, hc90,     spectrum, 0,		spectrum,		spectrum,	spectrum,	"ICE-Felix",	"HC-90" , 0)
 COMP( 1991, hc91,     spectrum, 0,		spectrum,		spec_plus,	spectrum,	"ICE-Felix",	"HC-91" , 0)
-COMP( 1988, cobrasp,   spectrum, 0,		spectrum,		spectrum,	spectrum,	"ITCI",	"Cobra" , GAME_NOT_WORKING)
-COMP( 1988, cobra80,  spectrum, 0,		spectrum,		spectrum,	spectrum,	"ITCI",	"Cobra 80K" , GAME_NOT_WORKING)
+COMP( 1988, cobrasp,   spectrum, 0,		spectrum,		spectrum,	spectrum,	"ITCI",	"Cobra (ITCI)" , GAME_NOT_WORKING)
+COMP( 1988, cobra80,  spectrum, 0,		spectrum,		spectrum,	spectrum,	"ITCI",	"Cobra 80K (ITCI)" , GAME_NOT_WORKING)
 COMP( 1987, cip01,    spectrum, 0,		spectrum,		spectrum,	spectrum,	"Electronica",	"CIP-01" , 0)	// keyboard should be spectrum, but image was not clear
 COMP( 1988, cip03,    spectrum, 0,		spectrum,		spectrum,	spectrum,	"Electronica",	"CIP-03" , 0)	// keyboard should be spectrum, but image was not clear
 COMP( 1990, jet,      spectrum, 0,		spectrum,		spectrum,	spectrum,	"Electromagnetica",	"JET" , 0)	// keyboard should be spectrum, but image was not clear
@@ -1034,6 +1037,7 @@ COMP( 1988, dgama88,  spectrum, 0,		spectrum,		spectrum,	spectrum,	"Didaktik Ska
 COMP( 1989, dgama89,  spectrum, 0,		spectrum,		spectrum,	spectrum,	"Didaktik Skalica",	"Didaktik Gama 89" , 0)
 COMP( 1990, didakt90, spectrum, 0,		spectrum,		spectrum,	spectrum,	"Didaktik Skalica",	"Didaktik 90" , 0)
 COMP( 1991, didakm91, spectrum, 0,		spectrum,		spec_plus,	spectrum,	"Didaktik Skalica",	"Didaktik M 91" , 0)
+COMP( 1992, didakm92, spectrum, 0,		spectrum,		spec_plus,	spectrum,	"Didaktik Skalica",	"Didaktik M 92" , 0)
 COMP( 1992, didaktk,  spectrum, 0,		spectrum,		spec_plus,	spectrum,	"Didaktik Skalica",	"Didaktik Kompakt" , 0)
 COMP( 1993, didakm93, spectrum, 0,		spectrum,		spec_plus,	spectrum,	"Didaktik Skalica",	"Didaktik M 93" , 0)
 COMP( 1988, mistrum,  spectrum, 0,		spectrum,		spectrum,	spectrum,	"Amaterske RADIO",	"Mistrum" , 0)	// keyboard could be spectrum in some models (since it was a build-yourself design)
