@@ -4,7 +4,6 @@
     Driver by MooglyGuy
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/ssem/ssem.h"
@@ -16,9 +15,10 @@ class ssem_state : public driver_device
 {
 public:
 	ssem_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_store(*this, "store"){ }
 
-	UINT8 *m_store;
+	required_shared_ptr<UINT8> m_store;
 	UINT8 m_store_line;
 };
 
@@ -54,7 +54,7 @@ INLINE UINT32 reverse(UINT32 v)
 \****************************************************/
 
 static ADDRESS_MAP_START( ssem_map, AS_PROGRAM, 8, ssem_state )
-	AM_RANGE(0x00, 0x7f) AM_RAM AM_BASE(m_store)// Primary store
+	AM_RANGE(0x00, 0x7f) AM_RAM AM_SHARE("store")// Primary store
 ADDRESS_MAP_END
 
 /****************************************************\
@@ -82,11 +82,11 @@ enum
 static INPUT_CHANGED( panel_check )
 {
 	ssem_state *state = field.machine().driver_data<ssem_state>();
-	UINT8 edit0_state = input_port_read(field.machine(), "EDIT0");
-	UINT8 edit1_state = input_port_read(field.machine(), "EDIT1");
-	UINT8 edit2_state = input_port_read(field.machine(), "EDIT2");
-	UINT8 edit3_state = input_port_read(field.machine(), "EDIT3");
-	UINT8 misc_state = input_port_read(field.machine(), "MISC");
+	UINT8 edit0_state = field.machine().root_device().ioport("EDIT0")->read();
+	UINT8 edit1_state = field.machine().root_device().ioport("EDIT1")->read();
+	UINT8 edit2_state = field.machine().root_device().ioport("EDIT2")->read();
+	UINT8 edit3_state = field.machine().root_device().ioport("EDIT3")->read();
+	UINT8 misc_state = field.machine().root_device().ioport("MISC")->read();
 	device_t *ssem_cpu = field.machine().device("maincpu");
 
 	switch( (int)(FPTR)param )
@@ -392,7 +392,7 @@ static const UINT8 char_glyphs[0x80][8] =
 	{ 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff },
 };
 
-static void glyph_print(running_machine &machine, bitmap_t *bitmap, INT32 x, INT32 y, const char *msg, ...)
+static void glyph_print(running_machine &machine, bitmap_rgb32 &bitmap, INT32 x, INT32 y, const char *msg, ...)
 {
 	va_list arg_list;
 	char buf[32768];
@@ -412,7 +412,7 @@ static void glyph_print(running_machine &machine, bitmap_t *bitmap, INT32 x, INT
 			INT32 line = 0;
 			for(line = 0; line < 8; line++)
 			{
-				UINT32 *d = BITMAP_ADDR32(bitmap, y + line, 0);
+				UINT32 *d = &bitmap.pix32(y + line);
 				INT32 bit = 0;
 				for(bit = 0; bit < 8; bit++)
 				{
@@ -441,11 +441,11 @@ static void glyph_print(running_machine &machine, bitmap_t *bitmap, INT32 x, INT
 	}
 }
 
-static SCREEN_UPDATE( ssem )
+static SCREEN_UPDATE_RGB32( ssem )
 {
-	ssem_state *state = screen->machine().driver_data<ssem_state>();
+	ssem_state *state = screen.machine().driver_data<ssem_state>();
 	UINT32 line = 0;
-	device_t *ssem_cpu = screen->machine().device("maincpu");
+	device_t *ssem_cpu = screen.machine().device("maincpu");
 	UINT32 accum = cpu_get_reg(ssem_cpu, SSEM_A);
 	UINT32 bit = 0;
 	UINT32 word = 0;
@@ -460,11 +460,11 @@ static SCREEN_UPDATE( ssem )
 		{
 			if(word & (1 << (31 - bit)))
 			{
-				glyph_print(screen->machine(), bitmap, bit << 3, line << 3, "%c", line == state->m_store_line ? 4 : 2);
+				glyph_print(screen.machine(), bitmap, bit << 3, line << 3, "%c", line == state->m_store_line ? 4 : 2);
 			}
 			else
 			{
-				glyph_print(screen->machine(), bitmap, bit << 3, line << 3, "%c", line == state->m_store_line ? 3 : 1);
+				glyph_print(screen.machine(), bitmap, bit << 3, line << 3, "%c", line == state->m_store_line ? 3 : 1);
 			}
 		}
 	}
@@ -473,11 +473,11 @@ static SCREEN_UPDATE( ssem )
 	{
 		if(accum & (1 << bit))
 		{
-			glyph_print(screen->machine(), bitmap, bit << 3, 264, "%c", 2);
+			glyph_print(screen.machine(), bitmap, bit << 3, 264, "%c", 2);
 		}
 		else
 		{
-			glyph_print(screen->machine(), bitmap, bit << 3, 264, "%c", 1);
+			glyph_print(screen.machine(), bitmap, bit << 3, 264, "%c", 1);
 		}
 	}
 
@@ -485,7 +485,7 @@ static SCREEN_UPDATE( ssem )
 				   (state->m_store[(state->m_store_line << 2) | 1] << 16) |
 				   (state->m_store[(state->m_store_line << 2) | 2] <<  8) |
 				   (state->m_store[(state->m_store_line << 2) | 3] <<  0));
-	glyph_print(screen->machine(), bitmap, 0, 272, "LINE:%02d  VALUE:%08x  HALT:%d", state->m_store_line, word, cpu_get_reg(ssem_cpu, SSEM_HALT));
+	glyph_print(screen.machine(), bitmap, 0, 272, "LINE:%02d  VALUE:%08x  HALT:%d", state->m_store_line, word, cpu_get_reg(ssem_cpu, SSEM_HALT));
 	return 0;
 }
 
@@ -641,11 +641,9 @@ static MACHINE_CONFIG_START( ssem, ssem_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(0))
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
 	MCFG_SCREEN_SIZE(256, 280)
 	MCFG_SCREEN_VISIBLE_AREA(0, 255, 0, 279)
-	MCFG_VIDEO_START(generic_bitmapped)
-	MCFG_SCREEN_UPDATE(ssem)
+	MCFG_SCREEN_UPDATE_STATIC(ssem)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(black_and_white)
 
