@@ -208,7 +208,6 @@
         25/08/2009 Skeleton driver.
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -221,20 +220,26 @@ public:
 	sun3_state(const machine_config &mconfig, device_type type, const char *tag)
 		: driver_device(mconfig, type, tag),
 	m_maincpu(*this, "maincpu")
-	{ }
+	,
+		m_p_ram(*this, "p_ram"){ }
 
 	required_device<cpu_device> m_maincpu;
 	virtual void machine_reset();
 
-	UINT16* m_p_ram;
+	required_shared_ptr<UINT32> m_p_ram;
 };
 
 static ADDRESS_MAP_START(sun3_mem, AS_PROGRAM, 32, sun3_state)
 	ADDRESS_MAP_UNMAP_HIGH
-	AM_RANGE(0x00000000, 0x00ffffff) AM_RAM AM_BASE(m_p_ram) // 16MB
+	AM_RANGE(0x00000000, 0x00ffffff) AM_RAM AM_SHARE("p_ram") // 16MB
 	AM_RANGE(0x0fef0000, 0x0fefffff) AM_ROM AM_REGION("user1",0)
 ADDRESS_MAP_END
 
+static ADDRESS_MAP_START(sun3x_mem, AS_PROGRAM, 32, sun3_state)
+	AM_RANGE(0x00000000, 0x00ffffff) AM_RAM AM_SHARE("p_ram") // 16MB
+	AM_RANGE(0xfefe0000, 0xfefeffff) AM_ROM AM_REGION("user1",0)
+ADDRESS_MAP_END
+
 /* Input ports */
 static INPUT_PORTS_START( sun3 )
 INPUT_PORTS_END
@@ -242,9 +247,9 @@ INPUT_PORTS_END
 
 MACHINE_RESET_MEMBER(sun3_state)
 {
-	UINT8* user1 = machine().region("user1")->base();
+	UINT8* user1 = memregion("user1")->base();
 
-	memcpy((UINT8*)m_p_ram,user1,0x10000);
+	memcpy((UINT8*)m_p_ram.target(),user1,0x10000);
 
 	machine().device("maincpu")->reset();
 }
@@ -256,6 +261,12 @@ static MACHINE_CONFIG_START( sun3, sun3_state )
 	MCFG_CPU_PROGRAM_MAP(sun3_mem)
 MACHINE_CONFIG_END
 
+static MACHINE_CONFIG_START( sun3x, sun3_state )
+	/* basic machine hardware */
+	MCFG_CPU_ADD("maincpu", M68030, 16670000)
+	MCFG_CPU_PROGRAM_MAP(sun3x_mem)
+MACHINE_CONFIG_END
+
 /* ROM definition */
 
 ROM_START( sun3_50 )
@@ -421,5 +432,5 @@ COMP( 198?, sun3_150,  0,       0,       sun3,      sun3,     0,  "Sun Microsyst
 COMP( 198?, sun3_260,  0,       0,       sun3,      sun3,     0,  "Sun Microsystems", "Sun 3/260/280", GAME_NOT_WORKING | GAME_NO_SOUND) // Prism
 COMP( 198?, sun3_e,    0,       0,       sun3,      sun3,     0,  "Sun Microsystems", "Sun 3/E", GAME_NOT_WORKING | GAME_NO_SOUND) // Polaris
 
-COMP( 198?, sun3_80,   0,       0,       sun3,      sun3,     0,  "Sun Microsystems", "Sun 3x/80", GAME_NOT_WORKING | GAME_NO_SOUND) // Hydra
-COMP( 198?, sun3_460,  0,       0,       sun3,      sun3,     0,  "Sun Microsystems", "Sun 3x/460/470/480", GAME_NOT_WORKING | GAME_NO_SOUND) // Pegasus
+COMP( 198?, sun3_80,   0,       0,       sun3x,     sun3,     0,  "Sun Microsystems", "Sun 3x/80", GAME_NOT_WORKING | GAME_NO_SOUND) // Hydra
+COMP( 198?, sun3_460,  0,       0,       sun3x,     sun3,     0,  "Sun Microsystems", "Sun 3x/460/470/480", GAME_NOT_WORKING | GAME_NO_SOUND) // Pegasus
