@@ -72,7 +72,6 @@ DEBUG TRICKS:
     bpclear
 
 ***************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -118,7 +117,8 @@ class supracan_state : public driver_device
 public:
 	supracan_state(const machine_config &mconfig, device_type type, const char *tag)
 		: driver_device(mconfig, type, tag),
-	m_maincpu(*this, "maincpu")
+		  m_maincpu(*this, "maincpu"),
+		  m_soundram(*this, "soundram")
 	{
 		m_m6502_reset = 0;
 	}
@@ -140,7 +140,7 @@ public:
 	acan_sprdma_regs_t m_acan_sprdma_regs;
 
 	UINT16 m_m6502_reset;
-	UINT8 *m_soundram;
+	required_shared_ptr<UINT8> m_soundram;
 	UINT8 m_soundlatch;
 	UINT8 m_soundcpu_irq_src;
 	UINT8 m_sound_irq_enable_reg;
@@ -192,7 +192,7 @@ public:
 	bool m_hack_68k_to_6502_access;
 
 	tilemap_t *m_tilemap_sizes[4][4];
-	bitmap_t *m_sprite_final_bitmap;
+	bitmap_ind16 m_sprite_final_bitmap;
 	void write_swapped_byte(int offset, UINT8 byte);
 };
 
@@ -260,7 +260,7 @@ static int supracan_tilemap_get_region(running_machine &machine, int layer)
 
 }
 
-static void supracan_tilemap_get_info_common(running_machine &machine, int layer, tile_data *tileinfo, int count)
+static void supracan_tilemap_get_info_common(running_machine &machine, int layer, tile_data &tileinfo, int count)
 {
 	supracan_state *state = machine.driver_data<supracan_state>();
 
@@ -315,7 +315,7 @@ static void supracan_tilemap_get_info_common(running_machine &machine, int layer
 }
 
 // I wonder how different this really is.. my guess, not at all.
-static void supracan_tilemap_get_info_roz(running_machine &machine, int layer, tile_data *tileinfo, int count)
+static void supracan_tilemap_get_info_roz(running_machine &machine, int layer, tile_data &tileinfo, int count)
 {
 	supracan_state *state = machine.driver_data<supracan_state>();
 
@@ -398,11 +398,11 @@ static TILE_GET_INFO( get_supracan_roz_tile_info )
 static VIDEO_START( supracan )
 {
 	supracan_state *state = machine.driver_data<supracan_state>();
-	state->m_sprite_final_bitmap = auto_bitmap_alloc(machine, 1024, 1024, BITMAP_FORMAT_INDEXED16);
+	state->m_sprite_final_bitmap.allocate(1024, 1024, BITMAP_FORMAT_IND16);
 
-	state->m_vram = (UINT16*)machine.region("ram_gfx")->base();
-	state->m_vram_swapped = (UINT16*)machine.region("ram_gfx2")->base();
-	state->m_vram_addr_swapped = (UINT8*)machine.region("ram_gfx3")->base(); // hack for 1bpp layer at startup
+	state->m_vram = (UINT16*)(*machine.root_device().memregion("ram_gfx"));
+	state->m_vram_swapped = (UINT16*)(*machine.root_device().memregion("ram_gfx2"));
+	state->m_vram_addr_swapped = (UINT8*)(*state->memregion("ram_gfx3")); // hack for 1bpp layer at startup
 
 	state->m_tilemap_sizes[0][0] = tilemap_create(machine, get_supracan_tilemap0_tile_info, tilemap_scan_rows, 8, 8, 32, 32);
 	state->m_tilemap_sizes[0][1] = tilemap_create(machine, get_supracan_tilemap0_tile_info, tilemap_scan_rows, 8, 8, 64, 32);
@@ -462,7 +462,7 @@ static int get_tilemap_dimensions(running_machine &machine, int &xsize, int &ysi
 
 
 
-static void draw_sprites(running_machine &machine, bitmap_t *bitmap, const rectangle *cliprect)
+static void draw_sprites(running_machine &machine, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	supracan_state *state = machine.driver_data<supracan_state>();
 	UINT16 *supracan_vram = state->m_vram;
@@ -600,8 +600,8 @@ static void draw_sprites(running_machine &machine, bitmap_t *bitmap, const recta
 			UINT32 delta = (1 << 17) / xscale;
 			for(int sy = 0; sy < ysize*8; sy++)
 			{
-				UINT16 *src = BITMAP_ADDR16(sprite_bitmap, sy, 0);
-				UINT16 *dst = BITMAP_ADDR16(bitmap, y + sy, 0);
+				UINT16 *src = &sprite_bitmap->pix16(sy);
+				UINT16 *dst = &bitmap.pix16(y + sy);
 				UINT32 dx = x << 16;
 				for(int sx = 0; sx < xsize*8; sx++)
 				{
@@ -628,21 +628,21 @@ static void mark_active_tilemap_all_dirty(running_machine &machine, int layer)
 	which_tilemap_size = get_tilemap_dimensions(machine, xsize, ysize, layer);
 //  for (int i=0;i<4;i++)
 //    tilemap_mark_all_tiles_dirty(state->m_tilemap_sizes[layer][i]);
-	tilemap_mark_all_tiles_dirty(state->m_tilemap_sizes[layer][which_tilemap_size]);
+	state->m_tilemap_sizes[layer][which_tilemap_size]->mark_all_dirty();
 }
 
 
 
 /* draws ROZ with linescroll OR columnscroll to 16-bit indexed bitmap */
-static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_t* bitmap, const rectangle *cliprect, tilemap_t *tmap, UINT32 startx, UINT32 starty, int incxx, int incxy, int incyx, int incyy, int wraparound/*, int columnscroll, UINT32* scrollram*/, int transmask)
+static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_ind16 &bitmap, const rectangle &cliprect, tilemap_t *tmap, UINT32 startx, UINT32 starty, int incxx, int incxy, int incyx, int incyy, int wraparound/*, int columnscroll, UINT32* scrollram*/, int transmask)
 {
-	//bitmap_t *destbitmap = bitmap;
-	bitmap_t *srcbitmap = tilemap_get_pixmap(tmap);
-	//bitmap_t *srcbitmapflags = tilemap_get_flagsmap(tmap);
-	const int xmask = srcbitmap->width-1;
-	const int ymask = srcbitmap->height-1;
-	const int widthshifted = srcbitmap->width << 16;
-	const int heightshifted = srcbitmap->height << 16;
+	//bitmap_ind16 *destbitmap = bitmap;
+	bitmap_ind16 &srcbitmap = tmap->pixmap();
+	//bitmap_ind16 &srcbitmapflags = tmap->flagsmap();
+	const int xmask = srcbitmap.width()-1;
+	const int ymask = srcbitmap.height()-1;
+	const int widthshifted = srcbitmap.width() << 16;
+	const int heightshifted = srcbitmap.height() << 16;
 	UINT32 cx;
 	UINT32 cy;
 	int x;
@@ -658,14 +658,14 @@ static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_t* bitma
 	//int destadvance = destbitmap->bpp / 8;
 
 	/* pre-advance based on the cliprect */
-	startx += cliprect->min_x * incxx + cliprect->min_y * incyx;
-	starty += cliprect->min_x * incxy + cliprect->min_y * incyy;
+	startx += cliprect.min_x * incxx + cliprect.min_y * incyx;
+	starty += cliprect.min_x * incxy + cliprect.min_y * incyy;
 
 	/* extract start/end points */
-	sx = cliprect->min_x;
-	sy = cliprect->min_y;
-	ex = cliprect->max_x;
-	ey = cliprect->max_y;
+	sx = cliprect.min_x;
+	sy = cliprect.min_y;
+	ex = cliprect.max_x;
+	ey = cliprect.max_y;
 
 	{
 		/* loop over rows */
@@ -678,8 +678,8 @@ static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_t* bitma
 			cy = starty;
 
 			/* get dest and priority pointers */
-			dest = BITMAP_ADDR16( bitmap, sy, sx);
-			//destflags = BITMAP_ADDR8( bitmapflags, sy, sx);
+			dest = &bitmap.pix16(sy, sx);
+			//destflags = &bitmapflags->pix8(sy, sx);
 
 			/* loop over columns */
 			while (x <= ex)
@@ -692,28 +692,28 @@ static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_t* bitma
 						int scroll = 0;//scrollram[(cx>>16)&0x3ff]);
 
 
-						UINT16 data = BITMAP_ADDR16(srcbitmap,
+						UINT16 data = &srcbitmap.pix16(
 												((cy >> 16) - scroll) & ymask,
 												(cx >> 16) & xmask)[0];
 
 						if ((data & transmask)!=0)
 							dest[0] = data;
 
-						//destflags[0] = BITMAP_ADDR8(srcbitmapflags, ((cy >> 16) - scrollram[(cx>>16)&0x3ff]) & ymask, (cx >> 16) & xmask)[0];
+						//destflags[0] = &srcbitmapflags.pix8(((cy >> 16) - scrollram[(cx>>16)&0x3ff]) & ymask, (cx >> 16) & xmask)[0];
 					}
 					else
 					#endif
 					{
 						int scroll = 0;//scrollram[(cy>>16)&0x3ff]);
-						UINT16 data =  BITMAP_ADDR16(srcbitmap,
+						UINT16 data =  srcbitmap.pix16(
 											   (cy >> 16) & ymask,
-											   ((cx >> 16) - scroll) & xmask)[0];
+											   ((cx >> 16) - scroll) & xmask);
 
 
 						if ((data & transmask)!=0)
 							dest[0] = data;
 
-						//destflags[0] = BITMAP_ADDR8(srcbitmapflags, (cy >> 16) & ymask, ((cx >> 16) - scrollram[(cy>>16)&0x3ff]) & xmask)[0];
+						//destflags[0] = &srcbitmapflags.pix8((cy >> 16) & ymask, ((cx >> 16) - scrollram[(cy>>16)&0x3ff]) & xmask)[0];
 					}
 				}
 
@@ -758,9 +758,9 @@ static void supracan_suprnova_draw_roz(running_machine &machine, bitmap_t* bitma
 // Sango Fighter Intro: 03c8: 0000 0011 1100 1000   ----: ---- ---- ---- ----   6c20        4620        ----        0x01
 // Sango Fighter Game:  03ce: 0000 0011 1100 1110   0622: 0000 0110 0010 0010   2620        4620        ----        0x01
 
-static SCREEN_UPDATE( supracan )
+static SCREEN_UPDATE_IND16( supracan )
 {
-	supracan_state *state = (supracan_state *)screen->machine().driver_data<supracan_state>();
+	supracan_state *state = (supracan_state *)screen.machine().driver_data<supracan_state>();
 
 
 
@@ -768,30 +768,29 @@ static SCREEN_UPDATE( supracan )
 
 	if (0)
 	{
-		if (cliprect->min_y == 0x00)
+		if (cliprect.min_y == 0x00)
 		{
-			const rectangle &visarea = screen->visible_area();
+			const rectangle &visarea = screen.visible_area();
 
-			bitmap_fill(state->m_sprite_final_bitmap, &visarea, 0x00);
-			bitmap_fill(bitmap, &visarea, 0x80);
+			state->m_sprite_final_bitmap.fill(0x00, visarea);
+			bitmap.fill(0x80, visarea);
 
-			draw_sprites(screen->machine(), state->m_sprite_final_bitmap, &visarea);
+			draw_sprites(screen.machine(), state->m_sprite_final_bitmap, visarea);
 		}
 	}
 	else
 	{
 
-		bitmap_fill(state->m_sprite_final_bitmap, cliprect, 0x00);
-		bitmap_fill(bitmap, cliprect, 0x80);
+		state->m_sprite_final_bitmap.fill(0x00, cliprect);
+		bitmap.fill(0x80, cliprect);
 
-		draw_sprites(screen->machine(), state->m_sprite_final_bitmap, cliprect);
+		draw_sprites(screen.machine(), state->m_sprite_final_bitmap, cliprect);
 	}
 
 
 
 	// mix screen
 	int xsize = 0, ysize = 0;
-	bitmap_t *src_bitmap = 0;
 //  int tilemap_num;
 	int which_tilemap_size;
 	int priority = 0;
@@ -825,9 +824,9 @@ static SCREEN_UPDATE( supracan )
 			if (priority==pri)
 			{
 //            tilemap_num = layer;
-				which_tilemap_size = get_tilemap_dimensions(screen->machine(), xsize, ysize, layer);
-				src_bitmap = tilemap_get_pixmap(state->m_tilemap_sizes[layer][which_tilemap_size]);
-				int gfx_region = supracan_tilemap_get_region(screen->machine(), layer);
+				which_tilemap_size = get_tilemap_dimensions(screen.machine(), xsize, ysize, layer);
+				bitmap_ind16 &src_bitmap = state->m_tilemap_sizes[layer][which_tilemap_size]->pixmap();
+				int gfx_region = supracan_tilemap_get_region(screen.machine(), layer);
 				int transmask = 0xff;
 
 				switch (gfx_region)
@@ -858,10 +857,10 @@ static SCREEN_UPDATE( supracan )
 						int y,x;
 						// yes, it will draw a single line if you specify a cliprect as such (partial updates...)
 
-						for (y=cliprect->min_y;y<=cliprect->max_y;y++)
+						for (y=cliprect.min_y;y<=cliprect.max_y;y++)
 						{
 							// these will have to change to ADDR32 etc. once alpha blending is supported
-							UINT16* screen = BITMAP_ADDR16(bitmap, y, 0);
+							UINT16* screen = &bitmap.pix16(y);
 
 							int actualy = y&mosaic_mask;
 
@@ -872,9 +871,9 @@ static SCREEN_UPDATE( supracan )
 									continue;
 
 
-							UINT16* src = BITMAP_ADDR16(src_bitmap, (realy)&((ysize*8)-1), 0);
+							UINT16* src = &src_bitmap.pix16((realy)&((ysize*8)-1));
 
-							for (x=cliprect->min_x;x<=cliprect->max_x;x++)
+							for (x=cliprect.min_x;x<=cliprect.max_x;x++)
 							{
 								int actualx = x & mosaic_mask;
 								int realx = actualx+scrollx;
@@ -929,13 +928,9 @@ static SCREEN_UPDATE( supracan )
 						if (!(state->m_roz_mode & 0x0200) && (state->m_roz_mode&0xf000) ) // HACK - Not Trusted, Acan Logo, Speedy Dragon Intro ,Speed Dragon Bonus stage need it.  Monopoly and JTT *don't* causes graphical issues
 						{
 							// NOT accurate, causes issues when the attract mode loops and the logo is shown the 2nd time in some games - investigate
-							for (int y=cliprect->min_y;y<=cliprect->max_y;y++)
+							for (int y=cliprect.min_y;y<=cliprect.max_y;y++)
 							{
-								rectangle clip;
-								clip.min_x = cliprect->min_x;
-								clip.max_x = cliprect->max_x;
-
-								clip.min_y = clip.max_y = y;
+								rectangle clip(cliprect.min_x, cliprect.max_x, y, y);
 
 								scrollx = (state->m_roz_scrollx);
 								scrolly = (state->m_roz_scrolly);
@@ -953,12 +948,12 @@ static SCREEN_UPDATE( supracan )
 
 
 								if (state->m_vram[state->m_roz_unk_base0/2 + y]) // incxx = 0, no draw?
-									supracan_suprnova_draw_roz(screen->machine(), bitmap, &clip, state->m_tilemap_sizes[layer][which_tilemap_size], scrollx<<8, scrolly<<8, incxx<<8, incxy<<8, incyx<<8, incyy<<8, wrap, transmask);
+									supracan_suprnova_draw_roz(screen.machine(), bitmap, clip, state->m_tilemap_sizes[layer][which_tilemap_size], scrollx<<8, scrolly<<8, incxx<<8, incxy<<8, incyx<<8, incyy<<8, wrap, transmask);
 							}
 						}
 						else
 						{
-							supracan_suprnova_draw_roz(screen->machine(), bitmap, cliprect, state->m_tilemap_sizes[layer][which_tilemap_size], scrollx<<8, scrolly<<8, incxx<<8, incxy<<8, incyx<<8, incyy<<8, wrap, transmask);
+							supracan_suprnova_draw_roz(screen.machine(), bitmap, cliprect, state->m_tilemap_sizes[layer][which_tilemap_size], scrollx<<8, scrolly<<8, incxx<<8, incxy<<8, incyx<<8, incyy<<8, wrap, transmask);
 						}
 					}
 				}
@@ -970,12 +965,12 @@ static SCREEN_UPDATE( supracan )
 	// just draw the sprites on top for now
 	if(state->m_video_flags & 0x08)
 	{
-		for (int y=cliprect->min_y;y<=cliprect->max_y;y++)
+		for (int y=cliprect.min_y;y<=cliprect.max_y;y++)
 		{
-			UINT16* src = BITMAP_ADDR16( state->m_sprite_final_bitmap, y, 0);
-			UINT16* dst = BITMAP_ADDR16( bitmap, y, 0);
+			UINT16* src = &state->m_sprite_final_bitmap.pix16(y);
+			UINT16* dst = &bitmap.pix16(y);
 
-			for (int x=cliprect->min_x;x<=cliprect->max_x;x++)
+			for (int x=cliprect.min_x;x<=cliprect.max_x;x++)
 			{
 				UINT16 dat = src[x];
 				if (dat) dst[x] = dat;
@@ -1119,7 +1114,7 @@ static ADDRESS_MAP_START( supracan_mem, AS_PROGRAM, 16, supracan_state )
 	AM_RANGE( 0xe90000, 0xe9001f ) AM_READWRITE( supracan_sound_r, supracan_sound_w )
 	AM_RANGE( 0xe90020, 0xe9003f ) AM_WRITE( supracan_dma_w )
 	AM_RANGE( 0xf00000, 0xf001ff ) AM_READWRITE( supracan_video_r, supracan_video_w )
-	AM_RANGE( 0xf00200, 0xf003ff ) AM_RAM AM_WRITE_LEGACY(paletteram16_xBBBBBGGGGGRRRRR_word_w) AM_BASE_GENERIC(paletteram)
+	AM_RANGE( 0xf00200, 0xf003ff ) AM_RAM AM_WRITE(paletteram_xBBBBBGGGGGRRRRR_word_w) AM_SHARE("paletteram")
 	AM_RANGE( 0xf40000, 0xf5ffff ) AM_READWRITE(supracan_vram_r, supracan_vram_w)
 	AM_RANGE( 0xfc0000, 0xfdffff ) AM_MIRROR(0x30000) AM_RAM /* System work ram */
 ADDRESS_MAP_END
@@ -1220,7 +1215,7 @@ WRITE8_MEMBER( supracan_state::supracan_6502_soundmem_w )
 }
 
 static ADDRESS_MAP_START( supracan_sound_mem, AS_PROGRAM, 8, supracan_state )
-	AM_RANGE( 0x0000, 0xffff ) AM_READWRITE(supracan_6502_soundmem_r, supracan_6502_soundmem_w) AM_BASE(m_soundram)
+	AM_RANGE( 0x0000, 0xffff ) AM_READWRITE(supracan_6502_soundmem_r, supracan_6502_soundmem_w) AM_SHARE("soundram")
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( supracan )
@@ -1640,10 +1635,7 @@ WRITE16_MEMBER( supracan_state::supracan_video_w )
 
 				rectangle visarea = machine().primary_screen->visible_area();
 
-				visarea.min_x = 0;
-				visarea.min_y = 8;
-				visarea.max_y = 232 - 1;
-				visarea.max_x = ((m_video_flags & 0x100) ? 320 : 256) - 1;
+				visarea.set(0, ((m_video_flags & 0x100) ? 320 : 256) - 1, 8, 232 - 1);
 				machine().primary_screen->configure(348, 256, visarea, machine().primary_screen->frame_period().attoseconds);
 			}
 			break;
@@ -1743,7 +1735,7 @@ WRITE16_MEMBER( supracan_state::supracan_video_w )
 
 static DEVICE_IMAGE_LOAD( supracan_cart )
 {
-	UINT8 *cart = image.device().machine().region("cart")->base();
+	UINT8 *cart = image.device().machine().root_device().memregion("cart")->base();
 	UINT32 size = 0;
 
 	if (image.software_entry() == NULL)
@@ -1922,10 +1914,8 @@ static MACHINE_CONFIG_START( supracan, supracan_state )
 	MCFG_MACHINE_RESET( supracan )
 
 	MCFG_SCREEN_ADD( "screen", RASTER )
-	MCFG_SCREEN_FORMAT( BITMAP_FORMAT_INDEXED16 )
-	//MCFG_SCREEN_FORMAT( BITMAP_FORMAT_RGB32 )
 	MCFG_SCREEN_RAW_PARAMS(XTAL_10_738635MHz/2, 348, 0, 256, 256, 0, 240 )	/* No idea if this is correct */
-	MCFG_SCREEN_UPDATE( supracan )
+	MCFG_SCREEN_UPDATE_STATIC( supracan )
 	MCFG_VIDEO_START( supracan )
 	MCFG_PALETTE_LENGTH( 32768 )
 	MCFG_PALETTE_INIT( supracan )
