@@ -28,47 +28,47 @@
 #include "machine/ram.h"
 #include "rendlay.h"
 
-static ADDRESS_MAP_START( svi318_mem, AS_PROGRAM, 8 )
-	AM_RANGE( 0x0000, 0x7fff) AM_READ_BANK("bank1") AM_WRITE( svi318_writemem1 )
-	AM_RANGE( 0x8000, 0xbfff) AM_READ_BANK("bank2") AM_WRITE( svi318_writemem2 )
-	AM_RANGE( 0xc000, 0xffff) AM_READ_BANK("bank3") AM_WRITE( svi318_writemem3 )
+static ADDRESS_MAP_START( svi318_mem, AS_PROGRAM, 8, svi318_state )
+	AM_RANGE( 0x0000, 0x7fff) AM_READ_BANK("bank1") AM_WRITE(svi318_writemem1 )
+	AM_RANGE( 0x8000, 0xbfff) AM_READ_BANK("bank2") AM_WRITE(svi318_writemem2 )
+	AM_RANGE( 0xc000, 0xffff) AM_READ_BANK("bank3") AM_WRITE(svi318_writemem3 )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( svi328_806_mem, AS_PROGRAM, 8 )
-	AM_RANGE( 0x0000, 0x7fff) AM_READ_BANK("bank1") AM_WRITE( svi318_writemem1 )
-	AM_RANGE( 0x8000, 0xbfff) AM_READ_BANK("bank2") AM_WRITE( svi318_writemem2 )
-	AM_RANGE( 0xc000, 0xefff) AM_READ_BANK("bank3") AM_WRITE( svi318_writemem3 )
-	AM_RANGE( 0xf000, 0xffff) AM_READ_BANK("bank4") AM_WRITE( svi318_writemem4 )
+static ADDRESS_MAP_START( svi328_806_mem, AS_PROGRAM, 8, svi318_state )
+	AM_RANGE( 0x0000, 0x7fff) AM_READ_BANK("bank1") AM_WRITE(svi318_writemem1 )
+	AM_RANGE( 0x8000, 0xbfff) AM_READ_BANK("bank2") AM_WRITE(svi318_writemem2 )
+	AM_RANGE( 0xc000, 0xefff) AM_READ_BANK("bank3") AM_WRITE(svi318_writemem3 )
+	AM_RANGE( 0xf000, 0xffff) AM_READ_BANK("bank4") AM_WRITE(svi318_writemem4 )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( svi318_io, AS_IO, 8 )
+static ADDRESS_MAP_START( svi318_io, AS_IO, 8, svi318_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE( 0x00, 0x38) AM_READWRITE( svi318_io_ext_r, svi318_io_ext_w )
-	AM_RANGE( 0x80, 0x80) AM_DEVWRITE_MODERN( "tms9928a", tms9928a_device, vram_write )
-	AM_RANGE( 0x81, 0x81) AM_DEVWRITE_MODERN( "tms9928a", tms9928a_device, register_write )
-	AM_RANGE( 0x84, 0x84) AM_DEVREAD_MODERN( "tms9928a", tms9928a_device, vram_read )
-	AM_RANGE( 0x85, 0x85) AM_DEVREAD_MODERN( "tms9928a", tms9928a_device, register_read )
-	AM_RANGE( 0x88, 0x88) AM_DEVWRITE("ay8910", ay8910_address_w )
-	AM_RANGE( 0x8c, 0x8c) AM_DEVWRITE("ay8910", ay8910_data_w )
-	AM_RANGE( 0x90, 0x90) AM_DEVREAD("ay8910", ay8910_r )
+	AM_RANGE( 0x00, 0x38) AM_READWRITE(svi318_io_ext_r, svi318_io_ext_w )
+	AM_RANGE( 0x80, 0x80) AM_DEVWRITE( "tms9928a", tms9928a_device, vram_write )
+	AM_RANGE( 0x81, 0x81) AM_DEVWRITE( "tms9928a", tms9928a_device, register_write )
+	AM_RANGE( 0x84, 0x84) AM_DEVREAD( "tms9928a", tms9928a_device, vram_read )
+	AM_RANGE( 0x85, 0x85) AM_DEVREAD( "tms9928a", tms9928a_device, register_read )
+	AM_RANGE( 0x88, 0x88) AM_DEVWRITE_LEGACY("ay8910", ay8910_address_w )
+	AM_RANGE( 0x8c, 0x8c) AM_DEVWRITE_LEGACY("ay8910", ay8910_data_w )
+	AM_RANGE( 0x90, 0x90) AM_DEVREAD_LEGACY("ay8910", ay8910_r )
 	AM_RANGE( 0x96, 0x97) AM_WRITE(svi318_ppi_w)
-	AM_RANGE( 0x98, 0x9a) AM_DEVREAD_MODERN("ppi8255", i8255_device, read)
+	AM_RANGE( 0x98, 0x9a) AM_DEVREAD("ppi8255", i8255_device, read)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( svi328_806_io, AS_IO, 8 )
+static ADDRESS_MAP_START( svi328_806_io, AS_IO, 8, svi318_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE( 0x00, 0x58) AM_READWRITE( svi318_io_ext_r, svi318_io_ext_w )
-	AM_RANGE( 0x80, 0x80) AM_DEVWRITE_MODERN( "tms9928a", tms9928a_device, vram_write )
-	AM_RANGE( 0x81, 0x81) AM_DEVWRITE_MODERN( "tms9928a", tms9928a_device, register_write )
-	AM_RANGE( 0x84, 0x84) AM_DEVREAD_MODERN( "tms9928a", tms9928a_device, vram_read )
-	AM_RANGE( 0x85, 0x85) AM_DEVREAD_MODERN( "tms9928a", tms9928a_device, register_read )
-	AM_RANGE( 0x88, 0x88) AM_DEVWRITE("ay8910", ay8910_address_w )
-	AM_RANGE( 0x8c, 0x8c) AM_DEVWRITE("ay8910", ay8910_data_w )
-	AM_RANGE( 0x90, 0x90) AM_DEVREAD("ay8910", ay8910_r )
+	AM_RANGE( 0x00, 0x58) AM_READWRITE(svi318_io_ext_r, svi318_io_ext_w )
+	AM_RANGE( 0x80, 0x80) AM_DEVWRITE( "tms9928a", tms9928a_device, vram_write )
+	AM_RANGE( 0x81, 0x81) AM_DEVWRITE( "tms9928a", tms9928a_device, register_write )
+	AM_RANGE( 0x84, 0x84) AM_DEVREAD( "tms9928a", tms9928a_device, vram_read )
+	AM_RANGE( 0x85, 0x85) AM_DEVREAD( "tms9928a", tms9928a_device, register_read )
+	AM_RANGE( 0x88, 0x88) AM_DEVWRITE_LEGACY("ay8910", ay8910_address_w )
+	AM_RANGE( 0x8c, 0x8c) AM_DEVWRITE_LEGACY("ay8910", ay8910_data_w )
+	AM_RANGE( 0x90, 0x90) AM_DEVREAD_LEGACY("ay8910", ay8910_r )
 	AM_RANGE( 0x96, 0x97) AM_WRITE(svi318_ppi_w)
-	AM_RANGE( 0x98, 0x9a) AM_DEVREAD_MODERN("ppi8255", i8255_device, read)
+	AM_RANGE( 0x98, 0x9a) AM_DEVREAD("ppi8255", i8255_device, read)
 ADDRESS_MAP_END
 
 /*
@@ -254,10 +254,10 @@ static const ay8910_interface svi318_ay8910_interface =
 {
 	AY8910_LEGACY_OUTPUT,
 	AY8910_DEFAULT_LOADS,
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, svi318_psg_port_a_r),
+	DEVCB_DRIVER_MEMBER(svi318_state, svi318_psg_port_a_r),
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, svi318_psg_port_b_w)
+	DEVCB_DRIVER_MEMBER(svi318_state, svi318_psg_port_b_w)
 };
 
 static WRITE_LINE_DEVICE_HANDLER(vdp_interrupt)
@@ -272,14 +272,6 @@ static TMS9928A_INTERFACE(svi318_tms9928a_interface)
     DEVCB_LINE(vdp_interrupt)
 };
 
-static SCREEN_UPDATE( svi318 )
-{
-	tms9928a_device *tms9928a = screen->machine().device<tms9928a_device>( "tms9928a" );
-
-	tms9928a->update( bitmap, cliprect );
-	return 0;
-}
-
 static const cassette_interface svi318_cassette_interface =
 {
 	svi_cassette_formats,
@@ -327,13 +319,13 @@ static MACHINE_CONFIG_START( svi318, svi318_state )
 
 	MCFG_I8255_ADD( "ppi8255", svi318_ppi8255_interface )
 
-	MCFG_INS8250_ADD( "ins8250_0", svi318_ins8250_interface[0] )
-	MCFG_INS8250_ADD( "ins8250_1", svi318_ins8250_interface[1] )
+	MCFG_INS8250_ADD( "ins8250_0", svi318_ins8250_interface[0], 1000000 )
+	MCFG_INS8250_ADD( "ins8250_1", svi318_ins8250_interface[1], 3072000 )
 
 	/* Video hardware */
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9929A, svi318_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( svi318 )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9929a_device, screen_update )
 
 	/* Sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -346,7 +338,7 @@ static MACHINE_CONFIG_START( svi318, svi318_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.75)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, svi318_cassette_interface )
 
@@ -372,7 +364,7 @@ static MACHINE_CONFIG_DERIVED( svi318n, svi318 )
 	MCFG_DEVICE_REMOVE("screen")
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9928A, svi318_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( svi318 )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9928a_device, screen_update )
 
 	MCFG_MACHINE_START( svi318_ntsc )
 	MCFG_MACHINE_RESET( svi318 )
@@ -438,24 +430,23 @@ static MACHINE_CONFIG_START( svi328_806, svi318_state )
 
 	MCFG_I8255A_ADD( "ppi8255", svi318_ppi8255_interface )
 
-	MCFG_INS8250_ADD( "ins8250_0", svi318_ins8250_interface[0] )
-	MCFG_INS8250_ADD( "ins8250_1", svi318_ins8250_interface[1] )
+	MCFG_INS8250_ADD( "ins8250_0", svi318_ins8250_interface[0], 1000000 )
+	MCFG_INS8250_ADD( "ins8250_1", svi318_ins8250_interface[1], 3072000 )
 
 	/* Video hardware */
 	MCFG_DEFAULT_LAYOUT( layout_dualhsxs )
 
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9929A, svi318_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_PAL( "screen" )
-	MCFG_SCREEN_UPDATE( svi318 )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9929a_device, screen_update )
 	MCFG_PALETTE_LENGTH(TMS9928A_PALETTE_SIZE + 2)	/* 2 additional entries for monochrome svi806 output */
 
 	MCFG_SCREEN_ADD("svi806", RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500))
 	MCFG_SCREEN_SIZE(640, 400)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 400-1)
-	MCFG_SCREEN_UPDATE( svi328_806 )
+	MCFG_SCREEN_UPDATE_DEVICE("crtc", mc6845_device, screen_update)
 
 	MCFG_GFXDECODE(svi328)
 
@@ -474,7 +465,7 @@ static MACHINE_CONFIG_START( svi328_806, svi318_state )
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.75)
 
 	/* printer */
-	MCFG_CENTRONICS_ADD("centronics", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, svi318_cassette_interface )
 
