@@ -24,26 +24,16 @@
 
 */
 
-#define ADDRESS_MAP_MODERN
-
-#include "emu.h"
-#include "cpu/i86/i86.h"
-#include "cpu/mcs48/mcs48.h"
-#include "imagedev/flopdrv.h"
-#include "machine/ram.h"
-#include "machine/ctronics.h"
-#include "machine/i8255.h"
-#include "machine/i8251.h"
-#include "machine/pit8253.h"
-#include "machine/pic8259.h"
-#include "machine/upd765.h"
-#include "sound/speaker.h"
-#include "video/crt9007.h"
-#include "video/crt9021.h"
-#include "video/crt9212.h"
 #include "includes/tandy2k.h"
 
-/* Read/Write Handlers */
+enum
+{
+	LPINEN = 0,
+	KBDINEN,
+	PORTINEN
+};
+
+// Read/Write Handlers
 
 void tandy2k_state::dma_request(int line, int state)
 {
@@ -104,36 +94,36 @@ WRITE8_MEMBER( tandy2k_state::enable_w )
         2       SPKRGATE    enable periodic m_speaker output
         3       SPKRDATA    direct output to m_speaker
         4       RFRQGATE    enable refresh and baud rate clocks
-        5       _FDCRESET   reset 8272
+        5       FDCRESET*   reset 8272
         6       TMRIN0      enable 80186 timer 0
         7       TMRIN1      enable 80186 timer 1
 
     */
 
-	/* keyboard enable */
-	m_kben = BIT(data, 0);
+	// keyboard enable
+	m_kb->power_w(BIT(data, 0));
 
-	/* external baud rate clock */
+	// external baud rate clock
 	m_extclk = BIT(data, 1);
 
-	/* m_speaker gate */
+	// m_speaker gate
 	pit8253_gate0_w(m_pit, BIT(data, 2));
 
-	/* m_speaker data */
+	// m_speaker data
 	m_spkrdata = BIT(data, 3);
 	speaker_update();
 
-	/* refresh and baud rate clocks */
+	// refresh and baud rate clocks
 	pit8253_gate1_w(m_pit, BIT(data, 4));
 	pit8253_gate2_w(m_pit, BIT(data, 4));
 
-	/* FDC reset */
+	// FDC reset
 	upd765_reset_w(m_fdc, BIT(data, 5));
 
-	/* timer 0 enable */
+	// timer 0 enable
 	device_set_input_line(m_maincpu, INPUT_LINE_TMRIN0, BIT(data, 6));
 
-	/* timer 1 enable */
+	// timer 1 enable
 	device_set_input_line(m_maincpu, INPUT_LINE_TMRIN1, BIT(data, 7));
 }
 
@@ -156,7 +146,7 @@ WRITE8_MEMBER( tandy2k_state::dma_mux_w )
 
 	m_dma_mux = data;
 
-	/* check for DMA error */
+	// check for DMA error
 	int drq0 = 0;
 	int drq1 = 0;
 
@@ -170,6 +160,17 @@ WRITE8_MEMBER( tandy2k_state::dma_mux_w )
 	pic8259_ir6_w(m_pic1, dme);
 }
 
+READ8_MEMBER( tandy2k_state::kbint_clr_r )
+{
+	if (m_pb_sel == KBDINEN)
+	{
+		m_kb->busy_w(1);
+		pic8259_ir0_w(m_pic1, CLEAR_LINE);
+	}
+
+	return 0xff;
+}
+
 READ16_MEMBER( tandy2k_state::vpac_r )
 {
 	if (ACCESSING_BITS_0_7)
@@ -225,10 +226,10 @@ WRITE8_MEMBER( tandy2k_state::addr_ctrl_w )
 
     */
 
-	/* video access */
+	// video access
 	m_vram_base = data & 0x1f;
 
-	/* dots per char */
+	// dots per char
 	int character_width = BIT(data, 6) ? 8 : 10;
 
 	if (m_clkcnt != BIT(data, 6))
@@ -237,7 +238,7 @@ WRITE8_MEMBER( tandy2k_state::addr_ctrl_w )
 		m_clkcnt = BIT(data, 6);
 	}
 
-	/* video clock speed */
+	// video clock speed
 	if (m_clkspd != BIT(data, 5))
 	{
 		float pixel_clock = BIT(data, 5) ? XTAL_16MHz*28/16 : XTAL_16MHz*28/20;
@@ -248,96 +249,19 @@ WRITE8_MEMBER( tandy2k_state::addr_ctrl_w )
 		m_clkspd = BIT(data, 5);
 	}
 
-	/* video source select */
+	// video source select
 	m_vidouts = BIT(data, 7);
 
 	logerror("Address Control %02x\n", data);
 }
 
-READ8_MEMBER( tandy2k_state::keyboard_x0_r )
-{
-  /*
-
-        bit     description
-
-        0       X0
-        1       X1
-        2       X2
-        3       X3
-        4       X4
-        5       X5
-        6       X6
-        7       X7
-
-    */
-
-	UINT8 data = 0xff;
-
-	if (!BIT(m_keylatch, 0)) data &= input_port_read(machine(), "Y0");
-	if (!BIT(m_keylatch, 1)) data &= input_port_read(machine(), "Y1");
-	if (!BIT(m_keylatch, 2)) data &= input_port_read(machine(), "Y2");
-	if (!BIT(m_keylatch, 3)) data &= input_port_read(machine(), "Y3");
-	if (!BIT(m_keylatch, 4)) data &= input_port_read(machine(), "Y4");
-	if (!BIT(m_keylatch, 5)) data &= input_port_read(machine(), "Y5");
-	if (!BIT(m_keylatch, 6)) data &= input_port_read(machine(), "Y6");
-	if (!BIT(m_keylatch, 7)) data &= input_port_read(machine(), "Y7");
-	if (!BIT(m_keylatch, 8)) data &= input_port_read(machine(), "Y8");
-	if (!BIT(m_keylatch, 9)) data &= input_port_read(machine(), "Y9");
-	if (!BIT(m_keylatch, 10)) data &= input_port_read(machine(), "Y10");
-	if (!BIT(m_keylatch, 11)) data &= input_port_read(machine(), "Y11");
-
-	return ~data;
-}
-
-WRITE8_MEMBER( tandy2k_state::keyboard_y0_w )
-{
-	/*
-
-        bit     description
-
-        0       Y0
-        1       Y1
-        2       Y2
-        3       Y3
-        4       Y4
-        5       Y5
-        6       Y6
-        7       Y7
-
-    */
-
-	/* keyboard row select */
-	m_keylatch = (m_keylatch & 0xff00) | data;
-}
-
-WRITE8_MEMBER( tandy2k_state::keyboard_y8_w )
-{
-	/*
-
-        bit     description
-
-        0       Y8
-        1       Y9
-        2       Y10
-        3       Y11
-        4       LED 2
-        5       LED 1
-        6
-        7
-
-    */
-
-	/* keyboard row select */
-	m_keylatch = ((data & 0x0f) << 8) | (m_keylatch & 0xff);
-}
-
-/* Memory Maps */
+// Memory Maps
 
 static ADDRESS_MAP_START( tandy2k_mem, AS_PROGRAM, 16, tandy2k_state )
 	ADDRESS_MAP_UNMAP_HIGH
 //  AM_RANGE(0x00000, 0xdffff) AM_RAM
-	AM_RANGE(0xe0000, 0xf7fff) AM_RAM AM_BASE(m_hires_ram)
-	AM_RANGE(0xf8000, 0xfbfff) AM_RAM AM_BASE(m_char_ram)
+	AM_RANGE(0xe0000, 0xf7fff) AM_RAM AM_SHARE("hires_ram")
+	AM_RANGE(0xf8000, 0xfbfff) AM_RAM AM_SHARE("char_ram")
 	AM_RANGE(0xfc000, 0xfdfff) AM_MIRROR(0x2000) AM_ROM AM_REGION(I80186_TAG, 0)
 ADDRESS_MAP_END
 
@@ -350,6 +274,7 @@ static ADDRESS_MAP_START( tandy2k_io, AS_IO, 16, tandy2k_state )
 	AM_RANGE(0x00030, 0x00031) AM_DEVREAD8_LEGACY(I8272A_TAG, upd765_status_r, 0x00ff)
 	AM_RANGE(0x00032, 0x00033) AM_DEVREADWRITE8_LEGACY(I8272A_TAG, upd765_data_r, upd765_data_w, 0x00ff)
 	AM_RANGE(0x00040, 0x00047) AM_DEVREADWRITE8_LEGACY(I8253_TAG, pit8253_r, pit8253_w, 0x00ff)
+	AM_RANGE(0x00052, 0x00053) AM_READ8(kbint_clr_r, 0x00ff)
 	AM_RANGE(0x00050, 0x00057) AM_DEVREADWRITE8(I8255A_TAG, i8255_device, read, write, 0x00ff)
 	AM_RANGE(0x00060, 0x00063) AM_DEVREADWRITE8_LEGACY(I8259A_0_TAG, pic8259_r, pic8259_w, 0x00ff)
 	AM_RANGE(0x00070, 0x00073) AM_DEVREADWRITE8_LEGACY(I8259A_1_TAG, pic8259_r, pic8259_w, 0x00ff)
@@ -373,143 +298,19 @@ static ADDRESS_MAP_START( vpac_mem, AS_0, 8, tandy2k_state )
 	AM_RANGE(0x0000, 0x3fff) AM_READ(videoram_r)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( keyboard_io, AS_IO, 8, tandy2k_state )
-  AM_RANGE(MCS48_PORT_P1, MCS48_PORT_P1) AM_WRITE(keyboard_y0_w)
-  AM_RANGE(MCS48_PORT_P2, MCS48_PORT_P2) AM_WRITE(keyboard_y8_w)
-  AM_RANGE(MCS48_PORT_BUS, MCS48_PORT_BUS) AM_READ(keyboard_x0_r)
-ADDRESS_MAP_END
-
-/* Input Ports */
+// Input Ports
 
 static INPUT_PORTS_START( tandy2k )
-	PORT_START("ROW0")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW1")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW2")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW3")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW4")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW5")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW6")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW7")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW8")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW9")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW10")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
-
-	PORT_START("ROW11")
-	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_KEYBOARD )
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_KEYBOARD )
+	// defined in machine/tandy2kb.c
 INPUT_PORTS_END
 
-/* Video */
+// Video
 
-bool tandy2k_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 tandy2k_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	if (m_vidouts)
 	{
-		m_vac->update_screen(&bitmap, &cliprect);
+		m_vac->screen_update(screen, bitmap, cliprect);
 	}
 
 	return 0;
@@ -531,7 +332,7 @@ static CRT9007_DRAW_SCANLINE( tandy2k_crt9007_display_pixels )
         {
             if (BIT(charram_data, 7))
             {
-                *BITMAP_ADDR16(bitmap, y, x + (sx * 10) + bit) = 1;
+                bitmap.pix16(y, x + (sx * 10) + bit) = 1;
             }
 
             charram_data <<= 1;
@@ -594,7 +395,7 @@ static CRT9021_INTERFACE( vac_intf )
 	DEVCB_LINE_VCC // ATTEN
 };
 
-/* Intel 8251A Interface */
+// Intel 8251A Interface
 
 WRITE_LINE_MEMBER( tandy2k_state::rxrdy_w )
 {
@@ -608,7 +409,7 @@ WRITE_LINE_MEMBER( tandy2k_state::txrdy_w )
 	pic8259_ir2_w(m_pic0, m_rxrdy | m_txrdy);
 }
 
-static const i8251_interface i8251_intf =
+static const i8251_interface usart_intf =
 {
 	DEVCB_NULL,
 	DEVCB_NULL,
@@ -621,7 +422,7 @@ static const i8251_interface i8251_intf =
 	DEVCB_NULL
 };
 
-/* Intel 8253 Interface */
+// Intel 8253 Interface
 
 WRITE_LINE_MEMBER( tandy2k_state::outspkr_w )
 {
@@ -642,7 +443,7 @@ WRITE_LINE_MEMBER( tandy2k_state::rfrqpulse_w )
 {
 }
 
-static const struct pit8253_config i8253_intf =
+static const struct pit8253_config pit_intf =
 {
 	{
 		{
@@ -661,14 +462,7 @@ static const struct pit8253_config i8253_intf =
 	}
 };
 
-/* Intel 8255A Interface */
-
-enum
-{
-	LPINEN = 0,
-	KBDINEN,
-	PORTINEN
-};
+// Intel 8255A Interface
 
 READ8_MEMBER( tandy2k_state::ppi_pb_r )
 {
@@ -692,28 +486,29 @@ READ8_MEMBER( tandy2k_state::ppi_pb_r )
 	switch (m_pb_sel)
 	{
 	case LPINEN:
-		/* printer acknowledge */
-		data |= centronics_ack_r(m_centronics) << 3;
+		// printer acknowledge
+		data |= m_centronics->ack_r() << 3;
 
-		/* printer fault */
-		data |= centronics_fault_r(m_centronics) << 4;
+		// printer fault
+		data |= m_centronics->fault_r() << 4;
 
-		/* printer select */
-		data |= centronics_vcc_r(m_centronics) << 5;
+		// printer select
+		data |= m_centronics->vcc_r() << 5;
 
-		/* paper empty */
-		data |= centronics_pe_r(m_centronics) << 6;
+		// paper empty
+		data |= m_centronics->pe_r() << 6;
 
-		/* printer busy */
-		data |= centronics_busy_r(m_centronics) << 7;
+		// printer busy
+		data |= m_centronics->busy_r() << 7;
 		break;
 
 	case KBDINEN:
-		/* keyboard data */
+		// keyboard data
+		data = m_kbdin;
 		break;
 
 	case PORTINEN:
-		/* PCB revision */
+		// PCB revision
 		data = 0x03;
 		break;
 	}
@@ -738,27 +533,27 @@ WRITE8_MEMBER( tandy2k_state::ppi_pc_w )
 
     */
 
-	/* input select */
+	// input select
 	m_pb_sel = (data >> 1) & 0x03;
 
-	/* interrupt */
+	// interrupt
 	pic8259_ir3_w(m_pic1, BIT(data, 3));
 
-	/* printer strobe */
-	centronics_strobe_w(m_centronics, BIT(data, 7));
+	// printer strobe
+	m_centronics->strobe_w(BIT(data, 7));
 }
 
-static I8255A_INTERFACE( i8255_intf )
+static I8255A_INTERFACE( ppi_intf )
 {
 	DEVCB_NULL,													// Port A read
-	DEVCB_DEVICE_HANDLER(CENTRONICS_TAG, centronics_data_w),	// Port A write
+	DEVCB_DEVICE_MEMBER(CENTRONICS_TAG, centronics_device, write),	// Port A write
 	DEVCB_DRIVER_MEMBER(tandy2k_state, ppi_pb_r),				// Port B write
 	DEVCB_NULL,													// Port B write
 	DEVCB_NULL,													// Port C read
 	DEVCB_DRIVER_MEMBER(tandy2k_state, ppi_pc_w)				// Port C write
 };
 
-/* Intel 8259 Interfaces */
+// Intel 8259 Interfaces
 
 /*
 
@@ -773,7 +568,7 @@ static I8255A_INTERFACE( i8255_intf )
 
 */
 
-static const struct pic8259_interface i8259_0_intf =
+static const struct pic8259_interface pic0_intf =
 {
 	DEVCB_CPU_INPUT_LINE(I80186_TAG, INPUT_LINE_INT0),
 	DEVCB_LINE_VCC,
@@ -793,14 +588,14 @@ static const struct pic8259_interface i8259_0_intf =
 
 */
 
-static const struct pic8259_interface i8259_1_intf =
+static const struct pic8259_interface pic1_intf =
 {
 	DEVCB_CPU_INPUT_LINE(I80186_TAG, INPUT_LINE_INT1),
 	DEVCB_LINE_VCC,
 	DEVCB_NULL
 };
 
-/* Floppy Configuration */
+// Floppy Configuration
 
 static const floppy_interface tandy2k_floppy_interface =
 {
@@ -809,20 +604,20 @@ static const floppy_interface tandy2k_floppy_interface =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	FLOPPY_STANDARD_5_25_DSQD,
+	FLOPPY_STANDARD_5_25_DSQD, // Mitsubishi M4853
 	LEGACY_FLOPPY_OPTIONS_NAME(default),
-	NULL,
+	"floppy_5_25",
 	NULL
 };
 
-/* Intel 8272 Interface */
+// Intel 8272 Interface
 
 WRITE_LINE_MEMBER( tandy2k_state::busdmarq0_w )
 {
 	dma_request(0, state);
 }
 
-static const struct upd765_interface i8272_intf =
+static const struct upd765_interface fdc_intf =
 {
 	DEVCB_DEVICE_LINE(I8259A_0_TAG, pic8259_ir4_w),
 	DEVCB_DRIVER_LINE_MEMBER(tandy2k_state, busdmarq0_w),
@@ -831,64 +626,90 @@ static const struct upd765_interface i8272_intf =
 	{ FLOPPY_0, FLOPPY_1, NULL, NULL }
 };
 
-/* Centronics Interface */
+// Centronics Interface
 
 static const centronics_interface centronics_intf =
 {
-	0,												/* is IBM PC? */
-	DEVCB_DEVICE_LINE_MEMBER(I8255A_TAG, i8255_device, pc6_w),	/* ACK output */
-	DEVCB_NULL,										/* BUSY output */
-	DEVCB_NULL										/* NOT BUSY output */
+	DEVCB_DEVICE_LINE_MEMBER(I8255A_TAG, i8255_device, pc6_w),	// ACK output
+	DEVCB_NULL,										// BUSY output
+	DEVCB_NULL										// NOT BUSY output
 };
 
-/* Machine Initialization */
+// Keyboard
+
+WRITE_LINE_MEMBER( tandy2k_state::kbdclk_w )
+{
+	if (!m_kbdclk && state)
+	{
+		m_kbdin >>= 1;
+		m_kbdin |= m_kb->data_r() << 7;
+	}
+
+	m_kbdclk = state;
+}
+
+WRITE_LINE_MEMBER( tandy2k_state::kbddat_w )
+{
+	if (!m_kbddat && state)
+	{
+		m_kb->busy_w(m_kbdclk);
+		pic8259_ir0_w(m_pic1, !m_kbdclk);
+	}
+
+	m_kbddat = state;
+}
+
+static TANDY2K_KEYBOARD_INTERFACE( kb_intf )
+{
+	DEVCB_DRIVER_LINE_MEMBER(tandy2k_state, kbdclk_w),
+	DEVCB_DRIVER_LINE_MEMBER(tandy2k_state, kbddat_w)
+};
+
+// Machine Initialization
 
 void tandy2k_state::machine_start()
 {
-	/* memory banking */
+	// memory banking
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
-	int ram_size = ram_get_size(m_ram);
+	UINT8 *ram = m_ram->pointer();
+	int ram_size = m_ram->size();
 
 	program->install_ram(0x00000, ram_size - 1, ram);
 
-	/* patch out i186 relocation register check */
-	UINT8 *rom = machine().region(I80186_TAG)->base();
+	// patch out i186 relocation register check
+	UINT8 *rom = memregion(I80186_TAG)->base();
 	rom[0x1f16] = 0x90;
 	rom[0x1f17] = 0x90;
 
-	/* register for state saving */
-	state_save_register_global(machine(), m_dma_mux);
-	state_save_register_global(machine(), m_kben);
-	state_save_register_global(machine(), m_keylatch);
-	state_save_register_global(machine(), m_extclk);
-	state_save_register_global(machine(), m_rxrdy);
-	state_save_register_global(machine(), m_txrdy);
-	state_save_register_global(machine(), m_pb_sel);
-	state_save_register_global(machine(), m_vidouts);
-	state_save_register_global(machine(), m_clkspd);
-	state_save_register_global(machine(), m_clkcnt);
-	state_save_register_global(machine(), m_outspkr);
-	state_save_register_global(machine(), m_spkrdata);
+	// register for state saving
+	save_item(NAME(m_dma_mux));
+	save_item(NAME(m_kbdclk));
+	save_item(NAME(m_kbddat));
+	save_item(NAME(m_kbdin));
+	save_item(NAME(m_extclk));
+	save_item(NAME(m_rxrdy));
+	save_item(NAME(m_txrdy));
+	save_item(NAME(m_pb_sel));
+	save_item(NAME(m_vidouts));
+	save_item(NAME(m_clkspd));
+	save_item(NAME(m_clkcnt));
+	save_item(NAME(m_outspkr));
+	save_item(NAME(m_spkrdata));
 }
 
-/* Machine Driver */
+// Machine Driver
 
 static MACHINE_CONFIG_START( tandy2k, tandy2k_state )
-    /* basic machine hardware */
+    // basic machine hardware
 	MCFG_CPU_ADD(I80186_TAG, I80186, XTAL_16MHz)
     MCFG_CPU_PROGRAM_MAP(tandy2k_mem)
     MCFG_CPU_IO_MAP(tandy2k_io)
 
-	MCFG_CPU_ADD(I8048_TAG, I8048, 1000000) // ?
-	MCFG_CPU_IO_MAP(keyboard_io)
-	MCFG_DEVICE_DISABLE()
-
-    /* video hardware */
+    // video hardware
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
-    MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) // not accurate
+	MCFG_SCREEN_UPDATE_DRIVER(tandy2k_state, screen_update)
     MCFG_SCREEN_SIZE(640, 480)
     MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
 
@@ -900,51 +721,61 @@ static MACHINE_CONFIG_START( tandy2k, tandy2k_state )
 	MCFG_CRT9212_ADD(CRT9212_1_TAG, drb1_intf)
 	MCFG_CRT9021_ADD(CRT9021B_TAG, XTAL_16MHz*28/16/8, vac_intf)
 
-	/* sound hardware */
+	// sound hardware
 	MCFG_SPEAKER_STANDARD_MONO("mono")
 	MCFG_SOUND_ADD(SPEAKER_TAG, SPEAKER_SOUND, 0)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
-	/* devices */
-	MCFG_I8255A_ADD(I8255A_TAG, i8255_intf)
-	MCFG_I8251_ADD(I8251A_TAG, i8251_intf)
-	MCFG_PIT8253_ADD(I8253_TAG, i8253_intf)
-	MCFG_PIC8259_ADD(I8259A_0_TAG, i8259_0_intf)
-	MCFG_PIC8259_ADD(I8259A_1_TAG, i8259_1_intf)
-	MCFG_UPD765A_ADD(I8272A_TAG, i8272_intf)
+	// devices
+	MCFG_I8255A_ADD(I8255A_TAG, ppi_intf)
+	MCFG_I8251_ADD(I8251A_TAG, usart_intf)
+	MCFG_PIT8253_ADD(I8253_TAG, pit_intf)
+	MCFG_PIC8259_ADD(I8259A_0_TAG, pic0_intf)
+	MCFG_PIC8259_ADD(I8259A_1_TAG, pic1_intf)
+	MCFG_UPD765A_ADD(I8272A_TAG, fdc_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(tandy2k_floppy_interface)
-	MCFG_CENTRONICS_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD(CENTRONICS_TAG, standard_centronics)
+	MCFG_TANDY2K_KEYBOARD_ADD(kb_intf)
 
-	/* internal ram */
+	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("128K")
 	MCFG_RAM_EXTRA_OPTIONS("256K,384K,512K,640K,768K,896K")
 MACHINE_CONFIG_END
 
 static MACHINE_CONFIG_DERIVED( tandy2k_hd, tandy2k )
-    /* basic machine hardware */
+    // basic machine hardware
 	MCFG_CPU_MODIFY(I80186_TAG)
     MCFG_CPU_IO_MAP(tandy2k_hd_io)
 
-	/* Tandon TM502 hard disk */
+	// Tandon TM502 hard disk
+	MCFG_HARDDISK_ADD("harddisk0")
 	//MCFG_WD1010_ADD(WD1010_TAG, wd1010_intf)
+	//MCFG_WD1100_11_ADD(WD1100_11_TAG, wd1100_11_intf)
 MACHINE_CONFIG_END
 
-/* ROMs */
+// ROMs
 
 ROM_START( tandy2k )
 	ROM_REGION( 0x2000, I80186_TAG, 0 )
 	ROM_LOAD16_BYTE( "484a00.u48", 0x0000, 0x1000, CRC(a5ee3e90) SHA1(4b1f404a4337c67065dd272d62ff88dcdee5e34b) )
 	ROM_LOAD16_BYTE( "474600.u47", 0x0001, 0x1000, CRC(345701c5) SHA1(a775cbfa110b7a88f32834aaa2a9b868cbeed25b) )
 
-	ROM_REGION( 0x400, I8048_TAG, 0 )
-	ROM_LOAD( "keyboard.m1", 0x0000, 0x0400, NO_DUMP )
+	ROM_REGION( 0x100, "plds", 0 )
+	ROM_LOAD( "82s153.u62", 0x000, 0x100, NO_DUMP ) // interrupt/DMA
+	ROM_LOAD( "82s153.u68", 0x000, 0x100, NO_DUMP ) // video
+	ROM_LOAD( "82s153.u95", 0x000, 0x100, NO_DUMP ) // memory timing
+	ROM_LOAD( "pal10l8.u82", 0x000, 0x100, NO_DUMP ) // video
+	ROM_LOAD( "pal16l8a.u102", 0x000, 0x100, NO_DUMP ) // bus interface
+	ROM_LOAD( "pal16l8a.u103", 0x000, 0x100, NO_DUMP ) // bus interface
+	ROM_LOAD( "pal20l8.u103", 0x000, 0x100, NO_DUMP ) // bus interface, alternate
+	ROM_LOAD( "pal16r6a.u16", 0x000, 0x100, NO_DUMP ) // HDC
 ROM_END
 
 #define rom_tandy2khd rom_tandy2k
 
-/* System Drivers */
+// System Drivers
 
-/*    YEAR  NAME        PARENT      COMPAT  MACHINE     INPUT       INIT    COMPANY                 FULLNAME        FLAGS */
+//    YEAR  NAME        PARENT      COMPAT  MACHINE     INPUT       INIT    COMPANY                 FULLNAME        FLAGS
 COMP( 1983, tandy2k,	0,			0,		tandy2k,	tandy2k,	0,		"Tandy Radio Shack",	"Tandy 2000",	GAME_NOT_WORKING)
 COMP( 1983, tandy2khd,	tandy2k,	0,		tandy2k_hd,	tandy2k,	0,		"Tandy Radio Shack",	"Tandy 2000HD",	GAME_NOT_WORKING)
