@@ -22,7 +22,6 @@
 
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m6800/m6800.h"
@@ -30,7 +29,6 @@
 #include "machine/6821pia.h"
 #include "machine/6850acia.h"
 #include "machine/ieee488.h"
-#include "machine/rs232.h"
 #include "sound/speaker.h"
 #include "video/vector.h"
 #include "includes/tek405x.h"
@@ -127,15 +125,15 @@ void tek4051_state::bankswitch(UINT8 data)
 	switch (lbs)
 	{
 	case LBS_RBC:
-		program->install_rom(0x8800, 0xa7ff, machine().region(MC6800_TAG)->base() + 0x800);
+		program->install_rom(0x8800, 0xa7ff, memregion(MC6800_TAG)->base() + 0x800);
 		break;
 
 	case LBS_BSOFL:
-		program->install_rom(0x8800, 0xa7ff, machine().region("020_0147_00")->base());
+		program->install_rom(0x8800, 0xa7ff, memregion("020_0147_00")->base());
 		break;
 
 	case LBS_BSCOM:
-		program->install_rom(0x8800, 0xa7ff, machine().region("672_0799_08")->base());
+		program->install_rom(0x8800, 0xa7ff, memregion("672_0799_08")->base());
 		break;
 
 	default:
@@ -643,7 +641,7 @@ READ8_MEMBER( tek4051_state::kb_pia_pa_r )
     */
 
 	UINT8 data = 0;
-	UINT8 special = input_port_read(machine(), "SPECIAL");
+	UINT8 special = ioport("SPECIAL")->read();
 
 	// keyboard column
 	data = m_kc;
@@ -672,7 +670,7 @@ READ8_MEMBER( tek4051_state::kb_pia_pb_r )
     */
 
 	UINT8 data = 0;
-	UINT8 special = input_port_read(machine(), "SPECIAL");
+	UINT8 special = ioport("SPECIAL")->read();
 
 	// shift
 	data |= (BIT(special, 0) & BIT(special, 1));
@@ -1047,10 +1045,8 @@ READ8_MEMBER( tek4051_state::com_pia_pb_r )
 	UINT8 data = 0;
 
 	// data terminal ready
-	//data |= rs232_dtr_r(m_rs232) << 1;
 
 	// request to send
-	//data |= rs232_rts_r(m_rs232) << 2;
 
 	return data;
 }
@@ -1073,7 +1069,6 @@ WRITE8_MEMBER( tek4051_state::com_pia_pb_w )
     */
 
 	// clear to send
-	//rs232_cts_w(m_rs232, BIT(data, 3));
 
 	// baud rate select
 	int osc = BIT(data, 7) ? 28160 : 38400;
@@ -1108,9 +1103,9 @@ static const pia6821_interface com_pia_intf =
 	DEVCB_NULL,
 	DEVCB_DRIVER_MEMBER(tek4051_state, com_pia_pb_r),
 	DEVCB_NULL, // SRX (RS-232 pin 12)
-	DEVCB_NULL,//DEVCB_DEVICE_LINE(RS232_TAG, rs232_dtr_r),
 	DEVCB_NULL,
-	DEVCB_NULL,//DEVCB_DEVICE_LINE(RS232_TAG, rs232_rts_r),
+	DEVCB_NULL,
+	DEVCB_NULL,
 	DEVCB_DRIVER_MEMBER(tek4051_state, com_pia_pa_w),
 	DEVCB_DRIVER_MEMBER(tek4051_state, com_pia_pb_w),
 	DEVCB_NULL,
@@ -1134,8 +1129,8 @@ static ACIA6850_INTERFACE( acia_intf )
 {
 	0,
 	0,
-	DEVCB_NULL,//DEVCB_DEVICE_LINE(RS232_TAG, rs232_td_r),
-	DEVCB_NULL,//DEVCB_DEVICE_LINE(RS232_TAG, rs232_rd_w),
+	DEVCB_NULL,
+	DEVCB_NULL,
 	DEVCB_LINE_GND,
 	DEVCB_NULL,
 	DEVCB_LINE_GND,
@@ -1175,7 +1170,7 @@ void tek4051_state::machine_start()
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
 
 	// configure RAM
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 8*1024:
 		program->unmap_readwrite(0x2000, 0x7fff);
@@ -1222,10 +1217,9 @@ static MACHINE_CONFIG_START( tek4051, tek4051_state )
     MCFG_SCREEN_ADD(SCREEN_TAG, VECTOR)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) // not accurate
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
     MCFG_SCREEN_SIZE(1024, 780)
     MCFG_SCREEN_VISIBLE_AREA(0, 1024-1, 0, 780-1)
-	MCFG_SCREEN_UPDATE(vector)
+	MCFG_SCREEN_UPDATE_STATIC(vector)
 
 	MCFG_PALETTE_INIT(monochrome_green)
 	MCFG_PALETTE_LENGTH(2)
@@ -1245,7 +1239,6 @@ static MACHINE_CONFIG_START( tek4051, tek4051_state )
 	MCFG_PIA6821_ADD(MC6820_COM_TAG, com_pia_intf)
 	MCFG_ACIA6850_ADD(MC6850_TAG, acia_intf)
 	MCFG_IEEE488_BUS_ADD(ieee488_intf)
-	//MCFG_RS232_ADD(RS232_TAG, rs232_intf)
 
 	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
@@ -1267,10 +1260,9 @@ static MACHINE_CONFIG_START( tek4052, tek4052_state )
     MCFG_SCREEN_ADD(SCREEN_TAG, VECTOR)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) // not accurate
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
     MCFG_SCREEN_SIZE(1024, 780)
     MCFG_SCREEN_VISIBLE_AREA(0, 1024-1, 0, 780-1)
-	MCFG_SCREEN_UPDATE(vector)
+	MCFG_SCREEN_UPDATE_STATIC(vector)
 
 	MCFG_PALETTE_INIT(monochrome_green)
 	MCFG_PALETTE_LENGTH(2)
