@@ -15,7 +15,6 @@
 
 ****************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/m68000/m68000.h"
@@ -32,7 +31,7 @@ UINT8 ti68k_state::keypad_r (running_machine &machine)
 	for (bit = 0; bit < 10; bit++)
 		if (~m_kb_mask & (0x01 << bit))
 			for (port = 0; port < 8; port++)
-				data ^= input_port_read(machine, bitnames[port]) & (0x01 << bit) ? 0x01 << port : 0x00;
+				data ^= machine.root_device().ioport(bitnames[port])->read() & (0x01 << bit) ? 0x01 << port : 0x00;
 	return data;
 }
 
@@ -146,7 +145,7 @@ READ16_MEMBER ( ti68k_state::flash_r )
 	}
 	else
 	{
-		UINT16 *rom_base = (UINT16 *)space.machine().region("flash")->base();
+		UINT16 *rom_base = (UINT16 *)(*space.machine().root_device().memregion("flash"));
 
 		return rom_base[offset];
 	}
@@ -421,7 +420,7 @@ INPUT_PORTS_END
 
 void ti68k_state::machine_start()
 {
-	UINT16 *rom = (UINT16 *)machine().region("flash")->base();
+	UINT16 *rom = (UINT16 *)(*machine().root_device().memregion("flash"));
 	int i;
 
 	m_flash_mem = !((rom[0x32] & 0x0f) != 0);
@@ -482,7 +481,7 @@ void ti68k_state::machine_reset()
 }
 
 /* video */
-bool ti68k_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 ti68k_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	/* preliminary implementation, doesn't use the contrast value */
 	UINT8 width = screen.width();
@@ -490,14 +489,14 @@ bool ti68k_state::screen_update(screen_device &screen, bitmap_t &bitmap, const r
 	UINT8 x, y, b;
 
 	if (!m_lcd_on || !m_lcd_base)
-		bitmap_fill(&bitmap, NULL, 0);
+		bitmap.fill(0);
 	else
 		for (y = 0; y < height; y++)
 			for (x = 0; x < width / 8; x++)
 			{
 				UINT8 s_byte= m_maincpu->space(AS_PROGRAM)->read_byte(m_lcd_base + y * (width/8) + x);
 				for (b = 0; b<8; b++)
-					*BITMAP_ADDR16(&bitmap, y, x * 8 + (7 - b)) = BIT(s_byte, b);
+					bitmap.pix16(y, x * 8 + (7 - b)) = BIT(s_byte, b);
 			}
 
 	return 0;
@@ -520,7 +519,7 @@ static MACHINE_CONFIG_START( ti89, ti68k_state )
     MCFG_SCREEN_ADD("screen", RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(ti68k_state, screen_update)
     MCFG_SCREEN_SIZE(240, 128)
     MCFG_SCREEN_VISIBLE_AREA(0, 160-1, 0, 100-1)
     MCFG_PALETTE_LENGTH(2)
