@@ -12,6 +12,7 @@
 
     TODO:
 
+    - floppy broken
     - palette RAM should be written during HBLANK
     - double sided disks have t0s0,t0s1,t1s0,t1s1... format
     - DART clocks
@@ -58,9 +59,9 @@ void tiki100_state::bankswitch()
 			program->install_readwrite_handler(0x0000, 0x7fff, read8_delegate(FUNC(tiki100_state::gfxram_r), this), write8_delegate(FUNC(tiki100_state::gfxram_w), this));
 			program->install_readwrite_bank(0x8000, 0xffff, "bank3");
 
-			memory_set_bank(machine(), "bank1", BANK_VIDEO_RAM);
-			memory_set_bank(machine(), "bank2", BANK_VIDEO_RAM);
-			memory_set_bank(machine(), "bank3", BANK_RAM);
+			membank("bank1")->set_entry(BANK_VIDEO_RAM);
+			membank("bank2")->set_entry(BANK_VIDEO_RAM);
+			membank("bank3")->set_entry(BANK_RAM);
 		}
 	}
 	else
@@ -73,9 +74,9 @@ void tiki100_state::bankswitch()
 			program->install_readwrite_bank(0x4000, 0x7fff, "bank2");
 			program->install_readwrite_bank(0x8000, 0xffff, "bank3");
 
-			memory_set_bank(machine(), "bank1", BANK_ROM);
-			memory_set_bank(machine(), "bank2", BANK_RAM);
-			memory_set_bank(machine(), "bank3", BANK_RAM);
+			membank("bank1")->set_entry(BANK_ROM);
+			membank("bank2")->set_entry(BANK_RAM);
+			membank("bank3")->set_entry(BANK_RAM);
 		}
 		else
 		{
@@ -84,9 +85,9 @@ void tiki100_state::bankswitch()
 			program->install_readwrite_bank(0x4000, 0x7fff, "bank2");
 			program->install_readwrite_bank(0x8000, 0xffff, "bank3");
 
-			memory_set_bank(machine(), "bank1", BANK_RAM);
-			memory_set_bank(machine(), "bank2", BANK_RAM);
-			memory_set_bank(machine(), "bank3", BANK_RAM);
+			membank("bank1")->set_entry(BANK_RAM);
+			membank("bank2")->set_entry(BANK_RAM);
+			membank("bank3")->set_entry(BANK_RAM);
 		}
 	}
 }
@@ -96,7 +97,7 @@ void tiki100_state::bankswitch()
 READ8_MEMBER( tiki100_state::keyboard_r )
 {
 	static const char *const keynames[] = { "ROW1", "ROW2", "ROW3", "ROW4", "ROW5", "ROW6", "ROW7", "ROW8", "ROW9", "ROW10", "ROW11", "ROW12" };
-	UINT8 data = input_port_read(machine(), keynames[m_keylatch]);
+	UINT8 data = ioport(keynames[m_keylatch])->read();
 
 	m_keylatch++;
 
@@ -374,7 +375,7 @@ INPUT_PORTS_END
 
 /* Video */
 
-bool tiki100_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 tiki100_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	UINT16 addr = (m_scroll << 7);
 	int sx, y, pixel, mode = (m_mode >> 4) & 0x03;
@@ -392,7 +393,7 @@ bool tiki100_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 				{
 					int x = (sx * 8) + pixel;
 
-					*BITMAP_ADDR16(&bitmap, y, x) = 0;
+					bitmap.pix16(y, x) = 0;
 				}
 				break;
 
@@ -402,7 +403,7 @@ bool tiki100_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 					int x = (sx * 8) + pixel;
 					int color = BIT(data, 0);
 
-					*BITMAP_ADDR16(&bitmap, y, x) = color;
+					bitmap.pix16(y, x) = color;
 
 					data >>= 1;
 				}
@@ -414,8 +415,8 @@ bool tiki100_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 					int x = (sx * 8) + (pixel * 2);
 					int color = data & 0x03;
 
-					*BITMAP_ADDR16(&bitmap, y, x) = color;
-					*BITMAP_ADDR16(&bitmap, y, x + 1) = color;
+					bitmap.pix16(y, x) = color;
+					bitmap.pix16(y, x + 1) = color;
 
 					data >>= 2;
 				}
@@ -427,10 +428,10 @@ bool tiki100_state::screen_update(screen_device &screen, bitmap_t &bitmap, const
 					int x = (sx * 8) + (pixel * 4);
 					int color = data & 0x0f;
 
-					*BITMAP_ADDR16(&bitmap, y, x) = color;
-					*BITMAP_ADDR16(&bitmap, y, x + 1) = color;
-					*BITMAP_ADDR16(&bitmap, y, x + 2) = color;
-					*BITMAP_ADDR16(&bitmap, y, x + 3) = color;
+					bitmap.pix16(y, x) = color;
+					bitmap.pix16(y, x + 1) = color;
+					bitmap.pix16(y, x + 2) = color;
+					bitmap.pix16(y, x + 3) = color;
 
 					data >>= 4;
 				}
@@ -551,16 +552,16 @@ void tiki100_state::machine_start()
 	m_video_ram = auto_alloc_array(machine(), UINT8, TIKI100_VIDEORAM_SIZE);
 
 	/* setup memory banking */
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
-	memory_configure_bank(machine(), "bank1", BANK_ROM, 1, machine().region(Z80_TAG)->base(), 0);
-	memory_configure_bank(machine(), "bank1", BANK_RAM, 1, ram, 0);
-	memory_configure_bank(machine(), "bank1", BANK_VIDEO_RAM, 1, m_video_ram, 0);
+	membank("bank1")->configure_entry(BANK_ROM, memregion(Z80_TAG)->base());
+	membank("bank1")->configure_entry(BANK_RAM, ram);
+	membank("bank1")->configure_entry(BANK_VIDEO_RAM, m_video_ram);
 
-	memory_configure_bank(machine(), "bank2", BANK_RAM, 1, ram + 0x4000, 0);
-	memory_configure_bank(machine(), "bank2", BANK_VIDEO_RAM, 1, m_video_ram + 0x4000, 0);
+	membank("bank2")->configure_entry(BANK_RAM, ram + 0x4000);
+	membank("bank2")->configure_entry(BANK_VIDEO_RAM, m_video_ram + 0x4000);
 
-	memory_configure_bank(machine(), "bank3", BANK_RAM, 1, ram + 0x8000, 0);
+	membank("bank3")->configure_entry(BANK_RAM, ram + 0x8000);
 
 	bankswitch();
 
@@ -574,6 +575,7 @@ void tiki100_state::machine_start()
 	save_item(NAME(m_keylatch));
 }
 
+// physical disks have 2:1 sector interleave
 static LEGACY_FLOPPY_OPTIONS_START(tiki100)
 	LEGACY_FLOPPY_OPTION(tiki100, "dsk", "SSSD disk image", basicdsk_identify_default, basicdsk_construct_default, NULL,
 		HEADS([1])
@@ -610,7 +612,7 @@ static const floppy_interface tiki100_floppy_interface =
 	DEVCB_NULL,
 	FLOPPY_STANDARD_5_25_DSHD,
 	LEGACY_FLOPPY_OPTIONS_NAME(tiki100),
-	NULL,
+	"floppy_5_25",
 	NULL
 };
 
@@ -618,7 +620,7 @@ static const floppy_interface tiki100_floppy_interface =
 
 static MACHINE_CONFIG_START( tiki100, tiki100_state )
 	/* basic machine hardware */
-    MCFG_CPU_ADD(Z80_TAG, Z80, 2000000)
+    MCFG_CPU_ADD(Z80_TAG, Z80, XTAL_8MHz/4)
     MCFG_CPU_PROGRAM_MAP(tiki100_mem)
     MCFG_CPU_IO_MAP(tiki100_io)
 	MCFG_CPU_CONFIG(tiki100_daisy_chain)
@@ -627,29 +629,33 @@ static MACHINE_CONFIG_START( tiki100, tiki100_state )
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
     MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(tiki100_state, screen_update)
     MCFG_SCREEN_SIZE(1024, 256)
     MCFG_SCREEN_VISIBLE_AREA(0, 1024-1, 0, 256-1)
 
 	MCFG_PALETTE_LENGTH(16)
+	// pixel clock 20.01782 MHz
 
 	/* devices */
-	MCFG_Z80DART_ADD(Z80DART_TAG, 2000000, dart_intf)
-	MCFG_Z80PIO_ADD(Z80PIO_TAG, 2000000, pio_intf)
-	MCFG_Z80CTC_ADD(Z80CTC_TAG, 2000000, ctc_intf)
-	MCFG_TIMER_ADD_PERIODIC("ctc", ctc_tick, attotime::from_hz(2000000))
+	MCFG_Z80DART_ADD(Z80DART_TAG, XTAL_8MHz/4, dart_intf)
+	MCFG_Z80PIO_ADD(Z80PIO_TAG, XTAL_8MHz/4, pio_intf)
+	MCFG_Z80CTC_ADD(Z80CTC_TAG, XTAL_8MHz/4, ctc_intf)
+	MCFG_TIMER_ADD_PERIODIC("ctc", ctc_tick, attotime::from_hz(XTAL_8MHz/4))
 	MCFG_FD1797_ADD(FD1797_TAG, fdc_intf) // FD1767PL-02 or FD1797-PL
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(tiki100_floppy_interface)
 
 	/* sound hardware */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
-	MCFG_SOUND_ADD(AY8912_TAG, AY8912, 2000000)
+	MCFG_SOUND_ADD(AY8912_TAG, AY8912, XTAL_8MHz/4)
 	MCFG_SOUND_CONFIG(ay8910_intf)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("64K")
+
+	// software list
+	MCFG_SOFTWARE_LIST_ADD("flop_list", "tiki100")
 MACHINE_CONFIG_END
 
 /* ROMs */
@@ -657,6 +663,9 @@ MACHINE_CONFIG_END
 ROM_START( kontiki )
 	ROM_REGION( 0x10000, Z80_TAG, ROMREGION_ERASEFF )
 	ROM_LOAD( "tikirom-1.30.u10",  0x0000, 0x2000, CRC(c482dcaf) SHA1(d140706bb7fc8b1fbb37180d98921f5bdda73cf9) )
+
+	ROM_REGION( 0x100, "proms", 0 )
+	ROM_LOAD( "63ls140.u4", 0x000, 0x100, NO_DUMP )
 ROM_END
 
 ROM_START( tiki100 )
@@ -667,6 +676,9 @@ ROM_START( tiki100 )
 	ROMX_LOAD( "tikirom-1.35.u10",  0x0000, 0x2000, CRC(7dac5ee7) SHA1(14d622fd843833faec346bf5357d7576061f5a3d), ROM_BIOS(1) )
 	ROM_SYSTEM_BIOS( 1, "v203w", "TIKI ROM v2.03 W" )
 	ROMX_LOAD( "tikirom-2.03w.u10", 0x0000, 0x2000, CRC(79662476) SHA1(96336633ecaf1b2190c36c43295ac9f785d1f83a), ROM_BIOS(2) )
+
+	ROM_REGION( 0x100, "proms", 0 )
+	ROM_LOAD( "63ls140.u4", 0x000, 0x100, NO_DUMP )
 ROM_END
 
 /* System Drivers */
