@@ -166,7 +166,6 @@ A=AMA, P=PRO, these keys don't exist, and so the games cannot be played.
 
 *********************************************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/tms9900/tms9900.h"
@@ -185,12 +184,12 @@ public:
 		: driver_device(mconfig, type, tag),
 	m_maincpu(*this, "maincpu"),
 	m_cass(*this, CASSETTE_TAG),
-	m_printer(*this, "printer")
+	m_centronics(*this, "centronics")
 	{ }
 
 	required_device<cpu_device> m_maincpu;
 	optional_device<cassette_image_device> m_cass;
-	optional_device<device_t> m_printer;
+	optional_device<centronics_device> m_centronics;
 	DECLARE_READ8_MEMBER(key_r);
 	DECLARE_READ8_MEMBER(tutor_mapper_r);
 	DECLARE_WRITE8_MEMBER(tutor_mapper_w);
@@ -226,16 +225,17 @@ static DRIVER_INIT(tutor)
 	tutor_state *state = machine.driver_data<tutor_state>();
 	state->m_tape_interrupt_timer = machine.scheduler().timer_alloc(FUNC(tape_interrupt_handler));
 
-	memory_configure_bank(machine, "bank1", 0, 1, machine.region("maincpu")->base() + basic_base, 0);
-	memory_configure_bank(machine, "bank1", 1, 1, machine.region("maincpu")->base() + cartridge_base, 0);
-	memory_set_bank(machine, "bank1", 0);
+	state->membank("bank1")->configure_entry(0, machine.root_device().memregion("maincpu")->base() + basic_base);
+	state->membank("bank1")->configure_entry(1, state->memregion("maincpu")->base() + cartridge_base);
+	state->membank("bank1")->set_entry(0);
 }
 
 
 static DRIVER_INIT(pyuuta)
 {
 	DRIVER_INIT_CALL(tutor);
-	memory_set_bank(machine, "bank1", 1);
+	tutor_state *state = machine.driver_data<tutor_state>();
+	state->membank("bank1")->set_entry(1);
 }
 
 
@@ -246,13 +246,6 @@ static TMS9928A_INTERFACE(tutor_tms9928a_interface)
 	DEVCB_NULL
 };
 
-static SCREEN_UPDATE( tutor )
-{
-	tms9928a_device *tms9928a = screen->machine().device<tms9928a_device>( "tms9928a" );
-	tms9928a->update( bitmap, cliprect );
-	return 0;
-}
-
 static MACHINE_START(tutor)
 {
 }
@@ -288,13 +281,13 @@ READ8_MEMBER( tutor_state::key_r )
 	UINT8 value;
 
 	snprintf(port, ARRAY_LENGTH(port), "LINE%d", offset);
-	value = input_port_read(machine(), port);
+	value = ioport(port)->read();
 
 	/* hack for ports overlapping with joystick */
 	if (offset == 4 || offset == 5)
 	{
 		snprintf(port, ARRAY_LENGTH(port), "LINE%d_alt", offset);
-		value |= input_port_read(machine(), port);
+		value |= ioport(port)->read();
 	}
 
 	return value;
@@ -304,7 +297,7 @@ READ8_MEMBER( tutor_state::key_r )
 static DEVICE_IMAGE_LOAD( tutor_cart )
 {
 	UINT32 size;
-	UINT8 *ptr = image.device().machine().region("maincpu")->base();
+	UINT8 *ptr = image.device().machine().root_device().memregion("maincpu")->base();
 
 	if (image.software_entry() == NULL)
 	{
@@ -323,7 +316,7 @@ static DEVICE_IMAGE_LOAD( tutor_cart )
 
 static DEVICE_IMAGE_UNLOAD( tutor_cart )
 {
-	memset(image.device().machine().region("maincpu")->base() + cartridge_base, 0, 0x6000);
+	memset(image.device().machine().root_device().memregion("maincpu")->base() + cartridge_base, 0, 0x6000);
 }
 
 /*
@@ -371,13 +364,13 @@ WRITE8_MEMBER( tutor_state::tutor_mapper_w )
 	case 0x08:
 		/* disable cartridge ROM, enable BASIC ROM at base >8000 */
 		m_cartridge_enable = 0;
-		memory_set_bank(machine(), "bank1", 0);
+		membank("bank1")->set_entry(0);
 		break;
 
 	case 0x0c:
 		/* enable cartridge ROM, disable BASIC ROM at base >8000 */
 		m_cartridge_enable = 1;
-		memory_set_bank(machine(), "bank1", 1);
+		membank("bank1")->set_entry(1);
 		break;
 
 	default:
@@ -471,7 +464,7 @@ READ8_MEMBER( tutor_state::tutor_printer_r )
 	{
 	case 0x20:
 		/* busy */
-		reply = centronics_busy_r(m_printer) ? 0x00 : 0xff;
+		reply = m_centronics->busy_r() ? 0x00 : 0xff;
 		break;
 
 	default:
@@ -490,12 +483,12 @@ WRITE8_MEMBER( tutor_state::tutor_printer_w )
 	{
 	case 0x10:
 		/* data */
-		centronics_data_w(m_printer, 0, data);
+		m_centronics->write(space, 0, data);
 		break;
 
 	case 0x40:
 		/* strobe */
-		centronics_strobe_w(m_printer, BIT(data, 7));
+		m_centronics->strobe_w(BIT(data, 7));
 		break;
 
 	default:
@@ -761,7 +754,7 @@ static MACHINE_CONFIG_START( tutor, tutor_state )
 	/* video hardware */
 	MCFG_TMS9928A_ADD( "tms9928a", TMS9928A, tutor_tms9928a_interface )
 	MCFG_TMS9928A_SCREEN_ADD_NTSC( "screen" )
-	MCFG_SCREEN_UPDATE( tutor )
+	MCFG_SCREEN_UPDATE_DEVICE( "tms9928a", tms9928a_device, screen_update )
 
 	/* sound */
 	MCFG_SPEAKER_STANDARD_MONO("mono")
@@ -770,7 +763,7 @@ static MACHINE_CONFIG_START( tutor, tutor_state )
 	MCFG_SOUND_WAVE_ADD(WAVE_TAG, CASSETTE_TAG)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
-	MCFG_CENTRONICS_ADD("printer", standard_centronics)
+	MCFG_CENTRONICS_PRINTER_ADD("centronics", standard_centronics)
 
 	MCFG_CASSETTE_ADD( CASSETTE_TAG, default_cassette_interface )
 
@@ -788,7 +781,7 @@ MACHINE_CONFIG_END
 static MACHINE_CONFIG_DERIVED( pyuutajr, tutor )
 	MCFG_CPU_MODIFY("maincpu")
 	MCFG_CPU_PROGRAM_MAP(pyuutajr_mem)
-	//MCFG_DEVICE_REMOVE("printer")
+	//MCFG_DEVICE_REMOVE("centronics")
 	//MCFG_DEVICE_REMOVE(CASSETTE_TAG)
 MACHINE_CONFIG_END
 
