@@ -46,60 +46,56 @@ elektor TV Game Computer which is a kind of developer machine for the VC4000.
 Go to the bottom to see the game list and emulation status of each.
 ******************************************************************************/
 
-#include "emu.h"
-#include "cpu/s2650/s2650.h"
-
 #include "includes/vc4000.h"
-#include "imagedev/cartslot.h"
-#include "imagedev/snapquik.h"
 
 static QUICKLOAD_LOAD( vc4000 );
 
-static READ8_HANDLER(vc4000_key_r)
+READ8_MEMBER( vc4000_state::vc4000_key_r )
 {
 	UINT8 data=0;
-	switch(offset & 0x0f) {
+	switch(offset & 0x0f)
+	{
 	case 0x08:
-		data = input_port_read(space->machine(), "KEYPAD1_1");
+		data = ioport("KEYPAD1_1")->read();
 		break;
 	case 0x09:
-		data = input_port_read(space->machine(), "KEYPAD1_2");
+		data = ioport("KEYPAD1_2")->read();
 		break;
 	case 0x0a:
-		data = input_port_read(space->machine(), "KEYPAD1_3");
+		data = ioport("KEYPAD1_3")->read();
 		break;
 	case 0x0b:
-		data = input_port_read(space->machine(), "PANEL");
+		data = ioport("PANEL")->read();
 		break;
 	case 0x0c:
-		data = input_port_read(space->machine(), "KEYPAD2_1");
+		data = ioport("KEYPAD2_1")->read();
 		break;
 	case 0x0d:
-		data = input_port_read(space->machine(), "KEYPAD2_2");
+		data = ioport("KEYPAD2_2")->read();
 		break;
 	case 0x0e:
-		data = input_port_read(space->machine(), "KEYPAD2_3");
+		data = ioport("KEYPAD2_3")->read();
 		break;
 	}
 	return data;
 }
 
-static WRITE8_HANDLER(vc4000_sound_ctl)
+WRITE8_MEMBER( vc4000_state::vc4000_sound_ctl )
 {
 	logerror("Write to sound control register offset= %d value= %d\n", offset, data);
 }
 
 
-static ADDRESS_MAP_START( vc4000_mem , AS_PROGRAM, 8)
+static ADDRESS_MAP_START( vc4000_mem, AS_PROGRAM, 8, vc4000_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	ADDRESS_MAP_GLOBAL_MASK(0x1fff)
 	AM_RANGE(0x0000, 0x07ff) AM_ROM
-	AM_RANGE(0x1680, 0x16ff) AM_READWRITE( vc4000_key_r, vc4000_sound_ctl ) AM_MIRROR(0x0800)
-	AM_RANGE(0x1700, 0x17ff) AM_READWRITE( vc4000_video_r, vc4000_video_w ) AM_MIRROR(0x0800)
+	AM_RANGE(0x1680, 0x16ff) AM_READWRITE(vc4000_key_r, vc4000_sound_ctl) AM_MIRROR(0x0800)
+	AM_RANGE(0x1700, 0x17ff) AM_READWRITE(vc4000_video_r, vc4000_video_w) AM_MIRROR(0x0800)
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( vc4000_io , AS_IO, 8)
-	AM_RANGE( S2650_SENSE_PORT,S2650_SENSE_PORT) AM_READ( vc4000_vsync_r)
+static ADDRESS_MAP_START( vc4000_io, AS_IO, 8, vc4000_state )
+	AM_RANGE( S2650_SENSE_PORT,S2650_SENSE_PORT) AM_READ(vc4000_vsync_r)
 ADDRESS_MAP_END
 
 static INPUT_PORTS_START( vc4000 )
@@ -194,6 +190,7 @@ static PALETTE_INIT( vc4000 )
 static DEVICE_IMAGE_LOAD( vc4000_cart )
 {
 	running_machine &machine = image.device().machine();
+	vc4000_state *state = machine.driver_data<vc4000_state>();
 	address_space *memspace = machine.device("maincpu")->memory().space(AS_PROGRAM);
 	UINT32 size;
 
@@ -208,34 +205,34 @@ static DEVICE_IMAGE_LOAD( vc4000_cart )
 	if (size > 0x1000)	/* 6k rom + 1k ram - Chess2 only */
 	{
 		memspace->install_read_bank(0x0800, 0x15ff, "bank1");	/* extra rom */
-		memory_set_bankptr(machine, "bank1", machine.region("maincpu")->base() + 0x1000);
+		state->membank("bank1")->set_base(machine.root_device().memregion("maincpu")->base() + 0x1000);
 
 		memspace->install_readwrite_bank(0x1800, 0x1bff, "bank2");	/* ram */
-		memory_set_bankptr(machine, "bank2", machine.region("maincpu")->base() + 0x1800);
+		state->membank("bank2")->set_base(machine.root_device().memregion("maincpu")->base() + 0x1800);
 	}
 	else if (size > 0x0800)	/* some 4k roms have 1k of mirrored ram */
 	{
 		memspace->install_read_bank(0x0800, 0x0fff, "bank1");	/* extra rom */
-		memory_set_bankptr(machine, "bank1", machine.region("maincpu")->base() + 0x0800);
+		state->membank("bank1")->set_base(machine.root_device().memregion("maincpu")->base() + 0x0800);
 
 		memspace->install_readwrite_bank(0x1000, 0x15ff, 0, 0x800, "bank2"); /* ram */
-		memory_set_bankptr(machine, "bank2", machine.region("maincpu")->base() + 0x1000);
+		state->membank("bank2")->set_base(machine.root_device().memregion("maincpu")->base() + 0x1000);
 	}
 	else if (size == 0x0800)	/* 2k roms + 2k ram - Hobby Module(Radofin) and elektor TVGC*/
 	{
 		memspace->install_readwrite_bank(0x0800, 0x0fff, "bank1"); /* ram */
-		memory_set_bankptr(machine, "bank1", machine.region("maincpu")->base() + 0x0800);
+		state->membank("bank1")->set_base(machine.root_device().memregion("maincpu")->base() + 0x0800);
 	}
 
 	if (size > 0)
 	{
 		if (image.software_entry() == NULL)
 		{
-			if (image.fread( machine.region("maincpu")->base(), size) != size)
+			if (image.fread( machine.root_device().memregion("maincpu")->base(), size) != size)
 				return IMAGE_INIT_FAIL;
 		}
 		else
-			memcpy(machine.region("maincpu")->base(), image.get_software_region("rom"), size);
+			memcpy(machine.root_device().memregion("maincpu")->base(), image.get_software_region("rom"), size);
 	}
 
 	return IMAGE_INIT_PASS;
@@ -252,10 +249,9 @@ static MACHINE_CONFIG_START( vc4000, vc4000_state )
 	/* video hardware */
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(226, 312)
 	MCFG_SCREEN_VISIBLE_AREA(8, 184, 0, 269)
-	MCFG_SCREEN_UPDATE( vc4000 )
+	MCFG_SCREEN_UPDATE_STATIC( vc4000 )
 
 	MCFG_PALETTE_LENGTH(8)
 	MCFG_PALETTE_INIT( vc4000 )
