@@ -20,12 +20,19 @@
     - brightness/contrast
     - MC6852
     - codec sound
+    - hard disk (Tandon TM502, TM603SE)
 
 */
 
+enum
+{
+	LED_A = 0,
+	LED_B
+};
+
 #include "includes/victor9k.h"
 
-/* Memory Maps */
+// Memory Maps
 
 static ADDRESS_MAP_START( victor9k_mem, AS_PROGRAM, 8, victor9k_state )
 //  AM_RANGE(0x00000, 0xdffff) AM_RAM
@@ -41,30 +48,24 @@ static ADDRESS_MAP_START( victor9k_mem, AS_PROGRAM, 8, victor9k_state )
 	AM_RANGE(0xe80a0, 0xe80af) AM_DEVREADWRITE(M6522_4_TAG, via6522_device, read, write)
 	AM_RANGE(0xe80c0, 0xe80cf) AM_DEVREADWRITE(M6522_6_TAG, via6522_device, read, write)
 	AM_RANGE(0xe80e0, 0xe80ef) AM_DEVREADWRITE(M6522_5_TAG, via6522_device, read, write)
-	AM_RANGE(0xf0000, 0xf0fff) AM_MIRROR(0x1000) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0xf0000, 0xf0fff) AM_MIRROR(0x1000) AM_RAM AM_SHARE("video_ram")
 	AM_RANGE(0xfe000, 0xfffff) AM_ROM AM_REGION(I8088_TAG, 0)
 ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( floppy_io, AS_IO, 8, victor9k_state )
-//  AM_RANGE(MCS48_PORT_P1, MCS48_PORT_P1)
-//  AM_RANGE(MCS48_PORT_P2, MCS48_PORT_P2)
-//  AM_RANGE(MCS48_PORT_T1, MCS48_PORT_T1)
-//  AM_RANGE(MCS48_PORT_BUS, MCS48_PORT_BUS)
+	AM_RANGE(MCS48_PORT_P1, MCS48_PORT_P1) AM_NOP
+	AM_RANGE(MCS48_PORT_P2, MCS48_PORT_P2) AM_NOP
+	AM_RANGE(MCS48_PORT_T1, MCS48_PORT_T1) AM_NOP
+	AM_RANGE(MCS48_PORT_BUS, MCS48_PORT_BUS) AM_NOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( keyboard_io, AS_IO, 8, victor9k_state )
-//  AM_RANGE(MCS48_PORT_P1, MCS48_PORT_P1)
-//  AM_RANGE(MCS48_PORT_P2, MCS48_PORT_P2)
-//  AM_RANGE(MCS48_PORT_T1, MCS48_PORT_T1)
-//  AM_RANGE(MCS48_PORT_BUS, MCS48_PORT_BUS)
-ADDRESS_MAP_END
-
-/* Input Ports */
+// Input Ports
 
 static INPUT_PORTS_START( victor9k )
+	// defined in machine/victor9kb.c
 INPUT_PORTS_END
 
-/* Video */
+// Video
 
 #define CODE_NON_DISPLAY	0x1000
 #define CODE_UNDERLINE		0x2000
@@ -75,6 +76,7 @@ static MC6845_UPDATE_ROW( victor9k_update_row )
 {
 	victor9k_state *state = device->machine().driver_data<victor9k_state>();
 	address_space *program = state->m_maincpu->memory().space(AS_PROGRAM);
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
 
 	if (BIT(ma, 13))
 	{
@@ -96,7 +98,7 @@ static MC6845_UPDATE_ROW( victor9k_update_row )
 
 				if (sx == cursor_x) color = 1;
 
-				*BITMAP_ADDR16(bitmap, y, x + sx*10) = color;
+				bitmap.pix32(y, x + sx*10) = palette[color];
 			}
 
 			video_ram_addr += 2;
@@ -105,13 +107,6 @@ static MC6845_UPDATE_ROW( victor9k_update_row )
 	}
 }
 
-WRITE_LINE_MEMBER( victor9k_state::vsync_w )
-{
-	pic8259_ir7_w(m_pic, state);
-
-	m_vert = state;
-}
-
 static const mc6845_interface hd46505s_intf =
 {
 	SCREEN_TAG,
@@ -122,18 +117,11 @@ static const mc6845_interface hd46505s_intf =
 	DEVCB_NULL,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_DRIVER_LINE_MEMBER(victor9k_state, vsync_w),
+	DEVCB_DEVICE_LINE(I8259A_TAG, pic8259_ir7_w),
 	NULL
 };
 
-bool victor9k_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
-{
-	m_crtc->update(&bitmap, &cliprect);
-
-	return 0;
-}
-
-/* Intel 8253 Interface */
+// Intel 8253 Interface
 
 static WRITE_LINE_DEVICE_HANDLER( mux_serial_b_w )
 {
@@ -162,7 +150,7 @@ static const struct pit8253_config pit_intf =
 	}
 };
 
-/* Intel 8259 Interfaces */
+// Intel 8259 Interfaces
 
 /*
 
@@ -186,43 +174,43 @@ static const struct pic8259_interface pic_intf =
 	DEVCB_NULL
 };
 
-/* NEC uPD7201 Interface */
+// NEC uPD7201 Interface
 
 static UPD7201_INTERFACE( mpsc_intf )
 {
-	DEVCB_DEVICE_LINE(I8259A_TAG, pic8259_ir1_w), /* interrupt */
+	DEVCB_DEVICE_LINE(I8259A_TAG, pic8259_ir1_w), // interrupt
 	{
 		{
-			0,					/* receive clock */
-			0,					/* transmit clock */
-			DEVCB_NULL,			/* receive DRQ */
-			DEVCB_NULL,			/* transmit DRQ */
-			DEVCB_NULL,			/* receive data */
-			DEVCB_NULL,			/* transmit data */
-			DEVCB_NULL,			/* clear to send */
-			DEVCB_NULL,			/* data carrier detect */
-			DEVCB_NULL,			/* ready to send */
-			DEVCB_NULL,			/* data terminal ready */
-			DEVCB_NULL,			/* wait */
-			DEVCB_NULL			/* sync output */
+			0,					// receive clock
+			0,					// transmit clock
+			DEVCB_NULL,			// receive DRQ
+			DEVCB_NULL,			// transmit DRQ
+			DEVCB_NULL,			// receive data
+			DEVCB_NULL,			// transmit data
+			DEVCB_NULL,			// clear to send
+			DEVCB_NULL,			// data carrier detect
+			DEVCB_NULL,			// ready to send
+			DEVCB_NULL,			// data terminal ready
+			DEVCB_NULL,			// wait
+			DEVCB_NULL			// sync output
 		}, {
-			0,					/* receive clock */
-			0,					/* transmit clock */
-			DEVCB_NULL,			/* receive DRQ */
-			DEVCB_NULL,			/* transmit DRQ */
-			DEVCB_NULL,			/* receive data */
-			DEVCB_NULL,			/* transmit data */
-			DEVCB_NULL,			/* clear to send */
-			DEVCB_NULL,			/* data carrier detect */
-			DEVCB_NULL,			/* ready to send */
-			DEVCB_NULL,			/* data terminal ready */
-			DEVCB_NULL,			/* wait */
-			DEVCB_NULL			/* sync output */
+			0,					// receive clock
+			0,					// transmit clock
+			DEVCB_NULL,			// receive DRQ
+			DEVCB_NULL,			// transmit DRQ
+			DEVCB_NULL,			// receive data
+			DEVCB_NULL,			// transmit data
+			DEVCB_NULL,			// clear to send
+			DEVCB_NULL,			// data carrier detect
+			DEVCB_NULL,			// ready to send
+			DEVCB_NULL,			// data terminal ready
+			DEVCB_NULL,			// wait
+			DEVCB_NULL			// sync output
 		}
 	}
 };
 
-/* MC6852 Interface */
+// MC6852 Interface
 
 WRITE_LINE_MEMBER( victor9k_state::ssda_irq_w )
 {
@@ -244,7 +232,7 @@ static MC6852_INTERFACE( ssda_intf )
 	DEVCB_NULL
 };
 
-/* M6522 Interface */
+// M6522 Interface
 
 WRITE8_MEMBER( victor9k_state::via1_pa_w )
 {
@@ -285,28 +273,28 @@ READ8_MEMBER( victor9k_state::via1_pb_r )
 
 	UINT8 data = 0;
 
-	/* data valid */
+	// data valid
 	data |= m_ieee488->dav_r();
 
-	/* end or identify */
+	// end or identify
 	data |= m_ieee488->eoi_r() << 1;
 
-	/* remote enable */
+	// remote enable
 	data |= m_ieee488->ren_r() << 2;
 
-	/* attention */
+	// attention
 	data |= m_ieee488->atn_r() << 3;
 
-	/* interface clear */
+	// interface clear
 	data |= m_ieee488->ifc_r() << 4;
 
-	/* service request */
+	// service request
 	data |= m_ieee488->srq_r() << 5;
 
-	/* not ready for data */
+	// not ready for data
 	data |= m_ieee488->nrfd_r() << 6;
 
-	/* data not accepted */
+	// data not accepted
 	data |= m_ieee488->ndac_r() << 7;
 
 	return data;
@@ -329,28 +317,28 @@ WRITE8_MEMBER( victor9k_state::via1_pb_w )
 
     */
 
-	/* data valid */
+	// data valid
 	m_ieee488->dav_w(BIT(data, 0));
 
-	/* end or identify */
+	// end or identify
 	m_ieee488->eoi_w(BIT(data, 1));
 
-	/* remote enable */
+	// remote enable
 	m_ieee488->ren_w(BIT(data, 2));
 
-	/* attention */
+	// attention
 	m_ieee488->atn_w(BIT(data, 3));
 
-	/* interface clear */
+	// interface clear
 	m_ieee488->ifc_w(BIT(data, 4));
 
-	/* service request */
+	// service request
 	m_ieee488->srq_w(BIT(data, 5));
 
-	/* not ready for data */
+	// not ready for data
 	m_ieee488->nrfd_w(BIT(data, 6));
 
-	/* data not accepted */
+	// data not accepted
 	m_ieee488->ndac_w(BIT(data, 7));
 }
 
@@ -390,8 +378,8 @@ READ8_MEMBER( victor9k_state::via2_pa_r )
 
         bit     description
 
-        PA0     _INT/EXTA
-        PA1     _INT/EXTB
+        PA0
+        PA1
         PA2     RIA
         PA3     DSRA
         PA4     RIB
@@ -403,8 +391,11 @@ READ8_MEMBER( victor9k_state::via2_pa_r )
 
 	UINT8 data = 0;
 
-	/* vertical sync */
-	data |= m_vert << 7;
+	// keyboard data
+	data |= m_kb->kbdata_r() << 6;
+
+	// vertical sync
+	data |= m_crtc->vsync_r() << 7;
 
 	return data;
 }
@@ -417,12 +408,12 @@ WRITE8_MEMBER( victor9k_state::via2_pa_w )
 
         PA0     _INT/EXTA
         PA1     _INT/EXTB
-        PA2     RIA
-        PA3     DSRA
-        PA4     RIB
-        PA5     DSRB
-        PA6     KBDATA
-        PA7     VERT
+        PA2
+        PA3
+        PA4
+        PA5
+        PA6
+        PA7
 
     */
 }
@@ -444,10 +435,13 @@ WRITE8_MEMBER( victor9k_state::via2_pb_w )
 
     */
 
-	/* brightness */
+	// keyboard acknowledge
+	m_kb->kback_w(BIT(data, 1));
+
+	// brightness
 	m_brt = (data >> 2) & 0x07;
 
-	/* contrast */
+	// contrast
 	m_cont = data >> 5;
 }
 
@@ -463,9 +457,9 @@ static const via6522_interface via2_intf =
 	DEVCB_DRIVER_MEMBER(victor9k_state, via2_pa_r),
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_NULL, // KBRDY
+	DEVCB_DEVICE_LINE_MEMBER(VICTOR9K_KEYBOARD_TAG, victor9k_keyboard_device, kbrdy_r),
 	DEVCB_NULL, // SRQ/BUSY
-	DEVCB_NULL, // KBDATA
+	DEVCB_DEVICE_LINE_MEMBER(VICTOR9K_KEYBOARD_TAG, victor9k_keyboard_device, kbdata_r),
 
 	DEVCB_DRIVER_MEMBER(victor9k_state, via2_pa_w),
 	DEVCB_DRIVER_MEMBER(victor9k_state, via2_pb_w),
@@ -552,7 +546,7 @@ WRITE8_MEMBER( victor9k_state::via3_pb_w )
 
     */
 
-	/* codec clock output */
+	// codec clock output
 	m_ssda->rx_clk_w(BIT(data, 7));
 	m_ssda->tx_clk_w(BIT(data, 7));
 }
@@ -723,12 +717,12 @@ READ8_MEMBER( victor9k_state::via6_pa_r )
 
         bit     description
 
-        PA0     LED0A
+        PA0
         PA1     TRK0D0
-        PA2     LED1A
+        PA2
         PA3     TRK0D1
-        PA4     SIDE SELECT
-        PA5     DRIVE SELECT
+        PA4
+        PA5
         PA6     WPS
         PA7     SYNC
 
@@ -736,15 +730,17 @@ READ8_MEMBER( victor9k_state::via6_pa_r )
 
 	UINT8 data = 0;
 
-	/* drive 0 track 0 sense */
+	// track 0 drive A sense
 	data |= floppy_tk00_r(m_floppy0) << 1;
 
-	/* drive 1 track 0 sense */
+	// track 0 drive B sense
 	data |= floppy_tk00_r(m_floppy1) << 3;
 
-	/* write protect sense */
+	// write protect sense
 	data |= floppy_wpt_r(m_drive ? m_floppy1 : m_floppy0) << 6;
 
+	// disk sync detect
+
 	return data;
 }
 
@@ -755,20 +751,26 @@ WRITE8_MEMBER( victor9k_state::via6_pa_w )
         bit     description
 
         PA0     LED0A
-        PA1     TRK0D0
+        PA1
         PA2     LED1A
-        PA3     TRK0D1
+        PA3
         PA4     SIDE SELECT
         PA5     DRIVE SELECT
-        PA6     WPS
-        PA7     SYNC
+        PA6
+        PA7
 
     */
 
-	/* side select */
+	// LED, drive A
+	output_set_led_value(LED_A, BIT(data, 0));
+
+	// LED, drive B
+	output_set_led_value(LED_B, BIT(data, 2));
+
+	// dual side select
 	m_side = BIT(data, 4);
 
-	/* drive select */
+	// select drive A/B
 	m_drive = BIT(data, 5);
 }
 
@@ -780,24 +782,26 @@ READ8_MEMBER( victor9k_state::via6_pb_r )
 
         PB0     RDY0
         PB1     RDY1
-        PB2     SCRESET
+        PB2
         PB3     _DS1
         PB4     _DS0
         PB5     SINGLE/_DOUBLE SIDED
-        PB6     stepper enable A
-        PB7     stepper enable B
+        PB6
+        PB7
 
     */
 
 	UINT8 data = 0;
 
-	/* drive 0 ready */
-	data |= !(floppy_drive_get_flag_state(m_floppy0, FLOPPY_DRIVE_READY) == FLOPPY_DRIVE_READY) << 4;
+	// motor speed status, drive A
 
-	/* drive 1 ready */
-	data |= !(floppy_drive_get_flag_state(m_floppy1, FLOPPY_DRIVE_READY) == FLOPPY_DRIVE_READY) << 3;
+	// motor speed status, drive B
 
-	/* single/double sided */
+	// door B sense
+
+	// door A sense
+
+	// single/double sided
 	data |= floppy_twosid_r(m_drive ? m_floppy1 : m_floppy0) << 5;
 
 	return data;
@@ -809,24 +813,24 @@ WRITE8_MEMBER( victor9k_state::via6_pb_w )
 
         bit     description
 
-        PB0     RDY0
-        PB1     RDY1
+        PB0
+        PB1
         PB2     SCRESET
-        PB3     _DS1
-        PB4     _DS0
-        PB5     SINGLE/_DOUBLE SIDED
+        PB3
+        PB4
+        PB5
         PB6     stepper enable A
         PB7     stepper enable B
 
     */
 
-	/* motor speed controller reset */
+	// motor speed controller reset
 	device_set_input_line(m_fdc_cpu, INPUT_LINE_RESET, BIT(data, 2));
 
-	/* stepper enable A */
+	// stepper enable A
 	m_se[0] = BIT(data, 6);
 
-	/* stepper enable B */
+	// stepper enable B
 	m_se[1] = BIT(data, 7);
 }
 
@@ -864,7 +868,21 @@ static const via6522_interface via6_intf =
 	DEVCB_DRIVER_LINE_MEMBER(victor9k_state, via6_irq_w)
 };
 
-/* Floppy Configuration */
+// Keyboard
+
+WRITE_LINE_MEMBER( victor9k_state::kbrdy_w )
+{
+	m_via2->write_cb1(state);
+
+	pic8259_ir6_w(m_pic, state ? CLEAR_LINE : ASSERT_LINE);
+}
+
+static VICTOR9K_KEYBOARD_INTERFACE( kb_intf )
+{
+	DEVCB_DRIVER_LINE_MEMBER(victor9k_state, kbrdy_w)
+};
+
+// Floppy Configuration
 
 static const floppy_interface victor9k_floppy_interface =
 {
@@ -875,11 +893,11 @@ static const floppy_interface victor9k_floppy_interface =
 	DEVCB_NULL,
 	FLOPPY_STANDARD_5_25_DSQD,
 	LEGACY_FLOPPY_OPTIONS_NAME(default),
-	NULL,
+	"floppy_5_25",
 	NULL
 };
 
-/* IEEE-488 Interface */
+// IEEE-488 Interface
 
 static IEEE488_INTERFACE( ieee488_intf )
 {
@@ -893,7 +911,7 @@ static IEEE488_INTERFACE( ieee488_intf )
 	DEVCB_NULL
 };
 
-/* Machine Initialization */
+// Machine Initialization
 
 static IRQ_CALLBACK( victor9k_irq_callback )
 {
@@ -904,36 +922,29 @@ static IRQ_CALLBACK( victor9k_irq_callback )
 
 void victor9k_state::machine_start()
 {
-	/* set interrupt callback */
+	// set interrupt callback
 	device_set_irq_callback(m_maincpu, victor9k_irq_callback);
 
-	/* memory banking */
+	// memory banking
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
-	int ram_size = ram_get_size(m_ram);
-
-	program->install_ram(0x00000, ram_size - 1, ram);
+	program->install_ram(0x00000, m_ram->size() - 1, m_ram->pointer());
 }
 
-/* Machine Driver */
+// Machine Driver
 
 static MACHINE_CONFIG_START( victor9k, victor9k_state )
-	/* basic machine hardware */
+	// basic machine hardware
 	MCFG_CPU_ADD(I8088_TAG, I8088, XTAL_30MHz/6)
 	MCFG_CPU_PROGRAM_MAP(victor9k_mem)
 
 	MCFG_CPU_ADD(I8048_TAG, I8048, XTAL_30MHz/6)
 	MCFG_CPU_IO_MAP(floppy_io)
 
-	MCFG_CPU_ADD(I8022_TAG, I8022, 100000)
-	MCFG_CPU_IO_MAP(keyboard_io)
-	MCFG_DEVICE_DISABLE()
-
-    /* video hardware */
+    // video hardware
     MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
     MCFG_SCREEN_REFRESH_RATE(50)
-    MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-    MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+    MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) // not accurate
+    MCFG_SCREEN_UPDATE_DEVICE(HD46505S_TAG, hd6845_device, screen_update)
     MCFG_SCREEN_SIZE(640, 480)
     MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
 
@@ -942,12 +953,12 @@ static MACHINE_CONFIG_START( victor9k, victor9k_state )
 
 	MCFG_MC6845_ADD(HD46505S_TAG, HD6845, 1000000, hd46505s_intf) // HD6845 == HD46505S
 
-	/* sound hardware */
+	// sound hardware
 	MCFG_SPEAKER_STANDARD_MONO("mono")
 	MCFG_SOUND_ADD(HC55516_TAG, HC55516, 100000)
 	MCFG_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.25)
 
-	/* devices */
+	// devices
 	MCFG_IEEE488_BUS_ADD(ieee488_intf)
 	MCFG_PIC8259_ADD(I8259A_TAG, pic_intf)
 	MCFG_PIT8253_ADD(I8253_TAG, pit_intf)
@@ -960,14 +971,15 @@ static MACHINE_CONFIG_START( victor9k, victor9k_state )
 	MCFG_VIA6522_ADD(M6522_5_TAG, XTAL_30MHz/30, via5_intf)
 	MCFG_VIA6522_ADD(M6522_6_TAG, XTAL_30MHz/30, via6_intf)
 	MCFG_LEGACY_FLOPPY_2_DRIVES_ADD(victor9k_floppy_interface)
+	MCFG_VICTOR9K_KEYBOARD_ADD(kb_intf)
 
-	/* internal ram */
+	// internal ram
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("128K")
 	MCFG_RAM_EXTRA_OPTIONS("256K,384K,512K,640K,768K,896K")
 MACHINE_CONFIG_END
 
-/* ROMs */
+// ROMs
 
 ROM_START( victor9k )
 	ROM_REGION( 0x2000, I8088_TAG, 0 )
@@ -979,17 +991,14 @@ ROM_START( victor9k )
 	ROMX_LOAD( "v9000 univ. fe f3f7 13db.7j", 0x0000, 0x1000, CRC(25c7a59f) SHA1(8784e9aa7eb9439f81e18b8e223c94714e033911), ROM_BIOS(2) )
 	ROMX_LOAD( "v9000 univ. ff f3f7 39fe.8j", 0x1000, 0x1000, CRC(496c7467) SHA1(eccf428f62ef94ab85f4a43ba59ae6a066244a66), ROM_BIOS(2) )
 
-	ROM_REGION( 0x800, "gcr", 0 )
-	ROM_LOAD( "100836-001.4k", 0x000, 0x800, CRC(adc601bd) SHA1(6eeff3d2063ae2d97452101aa61e27ef83a467e5) )
-
 	ROM_REGION( 0x400, I8048_TAG, 0)
 	ROM_LOAD( "36080.5d", 0x000, 0x400, CRC(9bf49f7d) SHA1(b3a11bb65105db66ae1985b6f482aab6ea1da38b) )
 
-	ROM_REGION( 0x800, I8022_TAG, 0)
-	ROM_LOAD( "keyboard controller i8022", 0x000, 0x800, NO_DUMP )
+	ROM_REGION( 0x800, "gcr", 0 )
+	ROM_LOAD( "100836-001.4k", 0x000, 0x800, CRC(adc601bd) SHA1(6eeff3d2063ae2d97452101aa61e27ef83a467e5) )
 ROM_END
 
-/* System Drivers */
+// System Drivers
 
-/*    YEAR  NAME      PARENT  COMPAT  MACHINE   INPUT     INIT  COMPANY                     FULLNAME        FLAGS */
+//    YEAR  NAME      PARENT  COMPAT  MACHINE   INPUT     INIT  COMPANY                     FULLNAME        FLAGS
 COMP( 1982, victor9k, 0,      0,      victor9k, victor9k, 0,    "Victor Business Products", "Victor 9000",	GAME_NOT_WORKING )
