@@ -247,9 +247,9 @@ WRITE8_MEMBER( vip_state::bankswitch_w )
 {
 	/* enable RAM */
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
-	switch (ram_get_size(m_ram))
+	switch (m_ram->size())
 	{
 	case 1 * 1024:
 		program->install_ram(0x0000, 0x03ff, 0, 0x7c00, ram);
@@ -301,13 +301,11 @@ ADDRESS_MAP_END
 
 /* Input Ports */
 
-static INPUT_CHANGED( reset_w )
+INPUT_CHANGED_MEMBER( vip_state::reset_w )
 {
-	vip_state *state = field.machine().driver_data<vip_state>();
-
 	if (oldval && !newval)
 	{
-		state->machine_reset();
+		machine_reset();
 	}
 }
 
@@ -349,7 +347,7 @@ static INPUT_PORTS_START( vip )
 	PORT_BIT( 0x8000, IP_ACTIVE_HIGH, IPT_KEYPAD ) PORT_NAME("VP-580 F") PORT_CODE(KEYCODE_L)
 
 	PORT_START("RUN")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_KEYPAD ) PORT_NAME("Run/Reset") PORT_CODE(KEYCODE_R) PORT_TOGGLE PORT_CHANGED(reset_w, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_KEYPAD ) PORT_NAME("Run/Reset") PORT_CODE(KEYCODE_R) PORT_TOGGLE PORT_CHANGED_MEMBER(DEVICE_SELF, vip_state, reset_w, 0)
 
 	PORT_START("KEYBOARD")
 	PORT_CONFNAME( 0x07, KEYBOARD_KEYPAD, "Keyboard")
@@ -438,16 +436,16 @@ WRITE8_MEMBER( vip_state::colorram_w )
 	m_cgc->con_w(0);
 }
 
-bool vip_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 vip_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
-	switch (input_port_read(machine(), "VIDEO"))
+	switch (ioport("VIDEO")->read())
 	{
 	case VIDEO_CDP1861:
-		m_vdc->update_screen(&bitmap, &cliprect);
+		m_vdc->screen_update(screen, bitmap, cliprect);
 		break;
 
 	case VIDEO_CDP1862:
-		m_cgc->update_screen(&bitmap, &cliprect);
+		m_cgc->screen_update(screen, bitmap, cliprect);
 		break;
 	}
 
@@ -458,7 +456,7 @@ bool vip_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rec
 
 READ_LINE_MEMBER( vip_state::clear_r )
 {
-	return BIT(input_port_read(machine(), "RUN"), 0);
+	return BIT(ioport("RUN")->read(), 0);
 }
 
 READ_LINE_MEMBER( vip_state::ef2_r )
@@ -470,15 +468,15 @@ READ_LINE_MEMBER( vip_state::ef2_r )
 
 READ_LINE_MEMBER( vip_state::ef3_r )
 {
-	return BIT(input_port_read(machine(), "KEYPAD"), m_keylatch);
+	return BIT(ioport("KEYPAD")->read(), m_keylatch);
 }
 
 READ_LINE_MEMBER( vip_state::ef4_r )
 {
-	switch (input_port_read(machine(), "KEYBOARD"))
+	switch (ioport("KEYBOARD")->read())
 	{
 	case KEYBOARD_VP580:
-		return BIT(input_port_read(machine(), "VP-580"), m_keylatch);
+		return BIT(ioport("VP-580")->read(), m_keylatch);
 	}
 
 	return 0;
@@ -488,7 +486,7 @@ static COSMAC_SC_WRITE( vip_sc_w )
 {
 	vip_state *state = device->machine().driver_data<vip_state>();
 
-	switch (input_port_read(device->machine(), "SOUND"))
+	switch (state->ioport("SOUND")->read())
 	{
 	case SOUND_VP550:
 		state->m_vp550->sc1_w(BIT(sc, 1));
@@ -503,7 +501,7 @@ static COSMAC_SC_WRITE( vip_sc_w )
 WRITE_LINE_MEMBER( vip_state::q_w )
 {
 	// sound output
-	switch (input_port_read(machine(), "SOUND"))
+	switch (ioport("SOUND")->read())
 	{
 	case SOUND_SPEAKER:
 		discrete_sound_w(m_beeper, NODE_01, state);
@@ -534,7 +532,7 @@ WRITE_LINE_MEMBER( vip_state::q_w )
 
 WRITE8_MEMBER( vip_state::dma_w )
 {
-	switch (input_port_read(machine(), "VIDEO"))
+	switch (ioport("VIDEO")->read())
 	{
 	case VIDEO_CDP1861:
 		m_vdc->dma_w(space, offset, data);
@@ -578,10 +576,10 @@ static COSMAC_INTERFACE( cosmac_intf )
 
 void vip_state::machine_start()
 {
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
 	/* randomize RAM contents */
-	for (UINT16 addr = 0; addr < ram_get_size(m_ram); addr++)
+	for (UINT16 addr = 0; addr < m_ram->size(); addr++)
 	{
 		ram[addr] = machine().rand() & 0xff;
 	}
@@ -599,9 +597,9 @@ void vip_state::machine_start()
 	m_vp551->q_w(0);
 
 	/* register for state saving */
-	state_save_register_global_pointer(machine(), m_colorram, VP590_COLOR_RAM_SIZE);
-	state_save_register_global(machine(), m_color);
-	state_save_register_global(machine(), m_keylatch);
+	save_pointer(NAME(m_colorram), VP590_COLOR_RAM_SIZE);
+	save_item(NAME(m_color));
+	save_item(NAME(m_keylatch));
 }
 
 void vip_state::machine_reset()
@@ -619,7 +617,7 @@ void vip_state::machine_reset()
 	m_vp551->reset();
 
 	/* configure video */
-	switch (input_port_read(machine(), "VIDEO"))
+	switch (ioport("VIDEO")->read())
 	{
 	case VIDEO_CDP1861:
 		io->unmap_write(0x05, 0x05);
@@ -633,7 +631,7 @@ void vip_state::machine_reset()
 	}
 
 	/* configure audio */
-	switch (input_port_read(machine(), "SOUND"))
+	switch (ioport("SOUND")->read())
 	{
 	case SOUND_SPEAKER:
 		m_vp595->install_write_handlers(io, false);
@@ -661,7 +659,7 @@ void vip_state::machine_reset()
 	}
 
 	/* enable ROM all through address space */
-	program->install_rom(0x0000, 0x01ff, 0, 0xfe00, machine().region(CDP1802_TAG)->base());
+	program->install_rom(0x0000, 0x01ff, 0, 0xfe00, memregion(CDP1802_TAG)->base());
 }
 
 /* Machine Drivers */
@@ -683,7 +681,8 @@ static MACHINE_CONFIG_START( vip, vip_state )
 	MCFG_CPU_CONFIG(cosmac_intf)
 
     /* video hardware */
-	MCFG_CDP1861_SCREEN_ADD(SCREEN_TAG, XTAL_3_52128MHz/2)
+	MCFG_CDP1861_SCREEN_ADD(CDP1861_TAG, SCREEN_TAG, XTAL_3_52128MHz/2)
+	MCFG_SCREEN_UPDATE_DRIVER(vip_state, screen_update)
 
 	MCFG_PALETTE_LENGTH(16)
 	MCFG_PALETTE_INIT(black_and_white)
@@ -741,7 +740,7 @@ ROM_END
 
 static QUICKLOAD_LOAD( vip )
 {
-	UINT8 *ptr = image.device().machine().region(CDP1802_TAG)->base();
+	UINT8 *ptr = image.device().machine().root_device().memregion(CDP1802_TAG)->base();
 	UINT8 *chip8_ptr = NULL;
 	int chip8_size = 0;
 	int size = image.length();
@@ -749,17 +748,17 @@ static QUICKLOAD_LOAD( vip )
 	if (strcmp(image.filetype(), "c8") == 0)
 	{
 		/* CHIP-8 program */
-		chip8_ptr = image.device().machine().region("chip8")->base();
-		chip8_size = image.device().machine().region("chip8")->bytes();
+		chip8_ptr = image.device().machine().root_device().memregion("chip8")->base();
+		chip8_size = image.device().machine().root_device().memregion("chip8")->bytes();
 	}
 	else if (strcmp(image.filename(), "c8x") == 0)
 	{
 		/* CHIP-8X program */
-		chip8_ptr = image.device().machine().region("chip8x")->base();
-		chip8_size = image.device().machine().region("chip8x")->bytes();
+		chip8_ptr = image.device().machine().root_device().memregion("chip8x")->base();
+		chip8_size = image.device().machine().root_device().memregion("chip8x")->bytes();
 	}
 
-	if ((size + chip8_size) > ram_get_size(image.device().machine().device(RAM_TAG)))
+	if ((size + chip8_size) > image.device().machine().device<ram_device>(RAM_TAG)->size())
 	{
 		return IMAGE_INIT_FAIL;
 	}
