@@ -11,7 +11,6 @@
         TODO: finish support for the on-cpu board alternate character set rom
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/i8085/i8085.h"
@@ -29,7 +28,8 @@ public:
 	m_maincpu(*this, "maincpu"),
 	m_crtc(*this, "vt100_video"),
 	m_speaker(*this, SPEAKER_TAG)
-	{ }
+	,
+		m_p_ram(*this, "p_ram"){ }
 
 	required_device<cpu_device> m_maincpu;
 	required_device<device_t> m_crtc;
@@ -41,7 +41,7 @@ public:
 	DECLARE_WRITE8_MEMBER(vt100_nvr_latch_w);
 	DECLARE_READ8_MEMBER(vt100_read_video_ram_r);
 	DECLARE_WRITE8_MEMBER(vt100_clear_video_interrupt);
-	UINT8 *m_p_ram;
+	required_shared_ptr<UINT8> m_p_ram;
 	bool m_keyboard_int;
 	bool m_receiver_int;
 	bool m_vertical_int;
@@ -57,7 +57,7 @@ public:
 static ADDRESS_MAP_START(vt100_mem, AS_PROGRAM, 8, vt100_state)
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE( 0x0000, 0x1fff ) AM_ROM  // ROM ( 4 * 2K)
-	AM_RANGE( 0x2000, 0x2bff ) AM_RAM AM_BASE(m_p_ram) // Screen and scratch RAM
+	AM_RANGE( 0x2000, 0x2bff ) AM_RAM AM_SHARE("p_ram") // Screen and scratch RAM
 	AM_RANGE( 0x2c00, 0x2fff ) AM_RAM  // AVO Screen RAM
 	AM_RANGE( 0x3000, 0x3fff ) AM_RAM  // AVO Attribute RAM (4 bits wide)
 	// 0x4000, 0x7fff is unassigned
@@ -116,7 +116,7 @@ static TIMER_DEVICE_CALLBACK(keyboard_callback)
 		for(i = 0; i < 16; i++)
 		{
 			sprintf(kbdrow,"LINE%X", i);
-			code =	input_port_read(timer.machine(), kbdrow);
+			code =	timer.machine().root_device().ioport(kbdrow)->read();
 			if (code < 0xff)
 			{
 				state->m_keyboard_int = 1;
@@ -308,9 +308,9 @@ static INPUT_PORTS_START( vt100 )
 		PORT_BIT(0x80, IP_ACTIVE_HIGH, IPT_UNUSED) // Always return 0x7f on last scan line
 INPUT_PORTS_END
 
-static SCREEN_UPDATE( vt100 )
+static SCREEN_UPDATE_IND16( vt100 )
 {
-	device_t *devconf = screen->machine().device("vt100_video");
+	device_t *devconf = screen.machine().device("vt100_video");
 	vt_video_update( devconf, bitmap, cliprect);
 	return 0;
 }
@@ -412,17 +412,15 @@ static MACHINE_CONFIG_START( vt100, vt100_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(60)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(80*10, 25*10)
 	MCFG_SCREEN_VISIBLE_AREA(0, 80*10-1, 0, 25*10-1)
-	MCFG_SCREEN_UPDATE(vt100)
+	MCFG_SCREEN_UPDATE_STATIC(vt100)
 
 	MCFG_GFXDECODE(vt100)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(monochrome_green)
 
 	MCFG_DEFAULT_LAYOUT( layout_vt100 )
-	MCFG_VIDEO_START(generic_bitmapped)
 
 	MCFG_VT100_VIDEO_ADD("vt100_video", vt100_video_interface)
 
