@@ -73,7 +73,7 @@
 #include "imagedev/flopdrv.h"
 #include "formats/vt_cas.h"
 
-static ADDRESS_MAP_START(vtech2_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START(vtech2_mem, AS_PROGRAM, 8, vtech2_state )
 	ADDRESS_MAP_UNMAP_HIGH
 	AM_RANGE(0x0000, 0x3fff) AM_RAMBANK("bank1")
 	AM_RANGE(0x4000, 0x7fff) AM_RAMBANK("bank2")
@@ -81,7 +81,7 @@ static ADDRESS_MAP_START(vtech2_mem, AS_PROGRAM, 8 )
 	AM_RANGE(0xc000, 0xffff) AM_RAMBANK("bank4")
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START(vtech2_io, AS_IO, 8)
+static ADDRESS_MAP_START(vtech2_io, AS_IO, 8, vtech2_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
 	AM_RANGE(0x10, 0x1f) AM_READWRITE(laser_fdc_r, laser_fdc_w)
 	AM_RANGE(0x40, 0x43) AM_WRITE(laser_bank_select_w)
@@ -439,10 +439,9 @@ static MACHINE_CONFIG_START( laser350, vtech2_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(0)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(88*8, 24*8+32)
 	MCFG_SCREEN_VISIBLE_AREA(0*8, 88*8-1, 0*8, 24*8+32-1)
-	MCFG_SCREEN_UPDATE(laser)
+	MCFG_SCREEN_UPDATE_STATIC(laser)
 
 	MCFG_GFXDECODE( vtech2 )
 	MCFG_PALETTE_LENGTH(528)
