@@ -22,9 +22,11 @@
     http://www.classiccmp.org/dunfield/img41867/system.htm
     http://www.microcodeconsulting.com/z80/plus2.htm
 
+    Note:
+    - MK-82 have same roms as original Big Board
+    - MK-83 have 256K of RAM
 */
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -90,13 +92,13 @@ void xerox820_state::scan_keyboard()
 	int table = 0, row, col;
 	int keydata = -1;
 
-	if (input_port_read(machine(), "ROW9") & 0x07)
+	if (ioport("ROW9")->read() & 0x07)
 	{
 		/* shift, upper case */
 		table = 1;
 	}
 
-	if (input_port_read(machine(), "ROW9") & 0x18)
+	if (ioport("ROW9")->read() & 0x18)
 	{
 		/* ctrl */
 		table = 2;
@@ -105,7 +107,7 @@ void xerox820_state::scan_keyboard()
 	/* scan keyboard */
 	for (row = 0; row < 9; row++)
 	{
-		UINT8 data = input_port_read(machine(), keynames[row]);
+		UINT8 data = ioport(keynames[row])->read();
 
 		for (col = 0; col < 8; col++)
 		{
@@ -140,12 +142,12 @@ static TIMER_DEVICE_CALLBACK( xerox820_keyboard_tick )
 void xerox820_state::bankswitch(int bank)
 {
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
 	if (bank)
 	{
 		/* ROM */
-		program->install_rom(0x0000, 0x0fff, machine().region("monitor")->base());
+		program->install_rom(0x0000, 0x0fff, memregion("monitor")->base());
 		program->unmap_readwrite(0x1000, 0x1fff);
 		program->install_ram(0x3000, 0x3fff, m_video_ram);
 	}
@@ -159,12 +161,12 @@ void xerox820_state::bankswitch(int bank)
 void xerox820ii_state::bankswitch(int bank)
 {
 	address_space *program = m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *ram = ram_get_ptr(m_ram);
+	UINT8 *ram = m_ram->pointer();
 
 	if (bank)
 	{
 		/* ROM */
-		program->install_rom(0x0000, 0x17ff, machine().region("monitor")->base());
+		program->install_rom(0x0000, 0x17ff, memregion("monitor")->base());
 		program->unmap_readwrite(0x1800, 0x2fff);
 		program->install_ram(0x3000, 0x3fff, m_video_ram);
 		program->unmap_readwrite(0x4000, 0xbfff);
@@ -236,7 +238,7 @@ WRITE8_MEMBER( xerox820ii_state::sync_w )
 
 static ADDRESS_MAP_START( xerox820_mem, AS_PROGRAM, 8, xerox820_state )
 	ADDRESS_MAP_UNMAP_HIGH
-	AM_RANGE(0x3000, 0x3fff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0x3000, 0x3fff) AM_RAM AM_SHARE("video_ram")
 	AM_RANGE(0x4000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
@@ -254,7 +256,7 @@ ADDRESS_MAP_END
 
 static ADDRESS_MAP_START( xerox820ii_mem, AS_PROGRAM, 8, xerox820ii_state )
 	ADDRESS_MAP_UNMAP_HIGH
-	AM_RANGE(0x3000, 0x3fff) AM_RAM AM_BASE(m_video_ram)
+	AM_RANGE(0x3000, 0x3fff) AM_RAM AM_SHARE("video_ram")
 	AM_RANGE(0xc000, 0xffff) AM_RAM
 ADDRESS_MAP_END
 
@@ -272,6 +274,14 @@ static ADDRESS_MAP_START( xerox168_mem, AS_PROGRAM, 16, xerox820ii_state )
 	AM_RANGE(0xff000, 0xfffff) AM_ROM AM_REGION(I8086_TAG, 0)
 ADDRESS_MAP_END
 
+static ADDRESS_MAP_START( mk83_mem, AS_PROGRAM, 8, xerox820_state )
+	ADDRESS_MAP_UNMAP_HIGH
+	AM_RANGE(0x3000, 0x6fff) AM_RAM
+	AM_RANGE(0x7000, 0x7fff) AM_RAM AM_SHARE("video_ram")
+	AM_RANGE(0x8000, 0xffff) AM_RAM
+ADDRESS_MAP_END
+
+
 /* Input Ports */
 
 static INPUT_PORTS_START( xerox820 )
@@ -548,9 +558,9 @@ static Z80CTC_INTERFACE( ctc_intf )
 {
 	0,              			/* timer disables */
 	DEVCB_CPU_INPUT_LINE(Z80_TAG, INPUT_LINE_IRQ0),	/* interrupt handler */
-	DEVCB_LINE(ctc_z0_w),		/* ZC/TO0 callback */
-	DEVCB_LINE(z80ctc_trg2_w),	/* ZC/TO1 callback */
-	DEVCB_LINE(ctc_z2_w)		/* ZC/TO2 callback */
+	DEVCB_DEVICE_LINE(Z80CTC_TAG, ctc_z0_w),		/* ZC/TO0 callback */
+	DEVCB_DEVICE_LINE(Z80CTC_TAG, z80ctc_trg2_w),	/* ZC/TO1 callback */
+	DEVCB_DEVICE_LINE(Z80CTC_TAG, ctc_z2_w)		/* ZC/TO2 callback */
 };
 
 /* Z80 Daisy Chain */
@@ -614,11 +624,11 @@ static COM8116_INTERFACE( com8116_intf )
 void xerox820_state::video_start()
 {
 	/* find memory regions */
-	m_char_rom = machine().region("chargen")->base();
+	m_char_rom = memregion("chargen")->base();
 }
 
 
-bool xerox820_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 xerox820_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	UINT8 y,ra,chr,gfx;
 	UINT16 sy=0,ma=(m_scroll + 1) * 0x80,x;
@@ -631,7 +641,7 @@ bool xerox820_state::screen_update(screen_device &screen, bitmap_t &bitmap, cons
 
 		for (ra = 0; ra < 10; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(&bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 
 			for (x = ma; x < ma + 80; x++)
 			{
@@ -706,14 +716,14 @@ void xerox820_state::machine_start()
 	floppy_install_load_proc(m_floppy1, xerox820_load_proc);
 
 	/* register for state saving */
-	state_save_register_global(machine(), m_keydata);
-	state_save_register_global(machine(), m_scroll);
-	state_save_register_global(machine(), m_ncset2);
-	state_save_register_global(machine(), m_vatt);
-	state_save_register_global(machine(), m_fdc_irq);
-	state_save_register_global(machine(), m_fdc_drq);
-	state_save_register_global(machine(), m_8n5);
-	state_save_register_global(machine(), m_dsdd);
+	save_item(NAME(m_keydata));
+	save_item(NAME(m_scroll));
+	save_item(NAME(m_ncset2));
+	save_item(NAME(m_vatt));
+	save_item(NAME(m_fdc_irq));
+	save_item(NAME(m_fdc_drq));
+	save_item(NAME(m_8n5));
+	save_item(NAME(m_dsdd));
 }
 
 void xerox820_state::machine_reset()
@@ -813,7 +823,7 @@ static MACHINE_CONFIG_START( xerox820, xerox820_state )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(xerox820_state, screen_update)
 	MCFG_SCREEN_RAW_PARAMS(XTAL_10_69425MHz, 700, 0, 560, 260, 0, 240)
 
 	MCFG_GFXDECODE(xerox820)
@@ -847,7 +857,7 @@ static MACHINE_CONFIG_START( xerox820ii, xerox820ii_state )
 
 	/* video hardware */
 	MCFG_SCREEN_ADD(SCREEN_TAG, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(xerox820ii_state, screen_update)
 	MCFG_SCREEN_RAW_PARAMS(XTAL_10_69425MHz, 700, 0, 560, 260, 0, 240)
 
 	MCFG_GFXDECODE(xerox820ii)
@@ -882,6 +892,11 @@ static MACHINE_CONFIG_DERIVED( xerox168, xerox820ii )
 	MCFG_RAM_EXTRA_OPTIONS("320K")
 MACHINE_CONFIG_END
 
+static MACHINE_CONFIG_DERIVED( mk83, xerox820 )
+	MCFG_CPU_MODIFY(Z80_TAG)
+	MCFG_CPU_PROGRAM_MAP(mk83_mem)
+MACHINE_CONFIG_END
+
 /* ROMs */
 
 ROM_START( xerox820 )
@@ -942,10 +957,17 @@ ROM_START( bigboard )
 	ROM_LOAD( "bigboard.u73", 0x0000, 0x0800, CRC(10bf0d81) SHA1(7ec73670a4d9d6421a5d6a4c4edc8b7c87923f6c) )
 ROM_END
 
+ROM_START( mk83 )
+	ROM_REGION( 0x1000, "monitor", 0 )
+	ROM_LOAD( "2732mk83.bin", 0x0000, 0x1000, CRC(a845c7e1) SHA1(3ccf629c5cd384953794ac4a1d2b45678bd40e92))
+	ROM_REGION( 0x800, "chargen", 0 )
+	ROM_LOAD( "2716mk83.bin", 0x0000, 0x0800, CRC(10bf0d81) SHA1(7ec73670a4d9d6421a5d6a4c4edc8b7c87923f6c))
+ROM_END
 /* System Drivers */
 
 /*    YEAR  NAME        PARENT      COMPAT  MACHINE     INPUT       INIT    COMPANY                         FULLNAME        FLAGS */
-COMP( 1981, xerox820,	0,			0,		xerox820,	xerox820,	0,		"Xerox",						"Xerox 820",	GAME_NO_SOUND)
+COMP( 1981, xerox820,   0,          0,      xerox820,   xerox820,   0,      "Xerox",                        "Xerox 820",    GAME_NO_SOUND)
 COMP( 1983, xerox820ii, xerox820,   0,      xerox820ii, xerox820,   0,      "Xerox",                        "Xerox 820-II", GAME_NOT_WORKING | GAME_NO_SOUND)
 COMP( 1983, xerox168,   xerox820,   0,      xerox168,   xerox820,   0,      "Xerox",                        "Xerox 16/8",   GAME_NOT_WORKING | GAME_NO_SOUND)
 COMP( 1980, bigboard,   0,          0,      xerox820,   xerox820,   0,      "Digital Research Computers",   "Big Board",    GAME_NOT_WORKING | GAME_NO_SOUND)
+COMP( 198?, mk83,       0,          0,      mk83,       xerox820,   0,      "Scomar",                       "MK-83",        GAME_NOT_WORKING | GAME_NO_SOUND)
