@@ -140,7 +140,6 @@ ZDIPSW      EQU 0FFH    ; Configuration dip switches
   ZDIPSWHZ    EQU 10000000B   ; 1=50Hz(0=60HZ)
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/i86/i86.h"
@@ -227,14 +226,14 @@ static VIDEO_START( z100 )
 	state->m_gvram = auto_alloc_array_clear(machine, UINT8, 0x30000);
 }
 
-static SCREEN_UPDATE( z100 )
+static SCREEN_UPDATE_IND16( z100 )
 {
-	z100_state *state = screen->machine().driver_data<z100_state>();
+	z100_state *state = screen.machine().driver_data<z100_state>();
 	int x,y,xi,yi;
 	int dot;
 	int i;
 
-	bitmap_fill(bitmap, cliprect, 0);
+	bitmap.fill(0, cliprect);
 
 	for(y=0;y<mc6845_v_display;y++)
 	{
@@ -259,7 +258,7 @@ static SCREEN_UPDATE( z100 )
 						dot = state->m_display_mask;
 
 					if(y*mc6845_tile_height+yi < 216 && x*8+xi < 640) /* TODO: safety check */
-						*BITMAP_ADDR16(bitmap, y*mc6845_tile_height+yi, x*8+xi) = screen->machine().pens[dot];
+						bitmap.pix16(y*mc6845_tile_height+yi, x*8+xi) = screen.machine().pens[dot];
 				}
 			}
 		}
@@ -759,7 +758,7 @@ static MACHINE_RESET(z100)
 	//z100_state *state = machine.driver_data<z100_state>();
 	int i;
 
-	if(input_port_read(machine,"CONFIG") & 1)
+	if(machine.root_device().ioport("CONFIG")->read() & 1)
 	{
 		for(i=0;i<8;i++)
 			palette_set_color_rgb(machine, i,pal1bit(i >> 1),pal1bit(i >> 2),pal1bit(i >> 0));
@@ -784,11 +783,10 @@ static MACHINE_CONFIG_START( z100, z100_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(640, 480)
 	MCFG_SCREEN_VISIBLE_AREA(0, 640-1, 0, 480-1)
 	MCFG_VIDEO_START(z100)
-	MCFG_SCREEN_UPDATE(z100)
+	MCFG_SCREEN_UPDATE_STATIC(z100)
 	MCFG_PALETTE_LENGTH(8)
 	MCFG_PALETTE_INIT(z100)
 
@@ -816,7 +814,7 @@ ROM_END
 
 static DRIVER_INIT( z100 )
 {
-	UINT8 *ROM = machine.region("ipl")->base();
+	UINT8 *ROM = machine.root_device().memregion("ipl")->base();
 
 	ROM[0xfc116 & 0x3fff] = 0x90; // patch parity IRQ check
 	ROM[0xfc117 & 0x3fff] = 0x90;
