@@ -17,7 +17,6 @@ There are others which just hang or reboot the system, due to no input checking.
 
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -34,7 +33,8 @@ public:
 		: driver_device(mconfig, type, tag),
 	m_maincpu(*this, "maincpu"),
 	m_cass(*this, CASSETTE_TAG)
-	{ }
+	,
+		m_p_videoram(*this, "p_videoram"){ }
 
 	required_device<cpu_device> m_maincpu;
 	required_device<cassette_image_device> m_cass;
@@ -42,7 +42,7 @@ public:
 	DECLARE_READ8_MEMBER(z1013_port_b_r);
 	DECLARE_WRITE8_MEMBER(z1013_port_b_w);
 	DECLARE_READ8_MEMBER(z1013k7659_port_b_r);
-	UINT8 *m_p_videoram;
+	required_shared_ptr<UINT8> m_p_videoram;
 	const UINT8 *m_p_chargen;
 	UINT8 m_keyboard_line;
 	bool m_keyboard_part;
@@ -52,7 +52,7 @@ public:
 /* Address maps */
 static ADDRESS_MAP_START(z1013_mem, AS_PROGRAM, 8, z1013_state)
 	AM_RANGE( 0x0000, 0xebff ) AM_RAM
-	AM_RANGE( 0xec00, 0xefff ) AM_RAM AM_BASE(m_p_videoram)
+	AM_RANGE( 0xec00, 0xefff ) AM_RAM AM_SHARE("p_videoram")
 	AM_RANGE( 0xf000, 0xffff ) AM_ROM //  ROM
 ADDRESS_MAP_END
 
@@ -187,12 +187,12 @@ INPUT_PORTS_EXTERN( k7659 );
 VIDEO_START( z1013 )
 {
 	z1013_state *state = machine.driver_data<z1013_state>();
-	state->m_p_chargen = machine.region("chargen")->base();
+	state->m_p_chargen = state->memregion("chargen")->base();
 }
 
-SCREEN_UPDATE( z1013 )
+SCREEN_UPDATE_IND16( z1013 )
 {
-	z1013_state *state = screen->machine().driver_data<z1013_state>();
+	z1013_state *state = screen.machine().driver_data<z1013_state>();
 	UINT8 y,ra,chr,gfx;
 	UINT16 sy=0,ma=0,x;
 
@@ -200,7 +200,7 @@ SCREEN_UPDATE( z1013 )
 	{
 		for (ra = 0; ra < 8; ra++)
 		{
-			UINT16 *p = BITMAP_ADDR16(bitmap, sy++, 0);
+			UINT16 *p = &bitmap.pix16(sy++);
 
 			for (x = ma; x < ma+32; x++)
 			{
@@ -242,7 +242,7 @@ READ8_MEMBER( z1013_state::z1013_port_b_r )
 {
 	char kbdrow[6];
 	sprintf(kbdrow,"X%d", m_keyboard_line & 7);
-	UINT8 data = input_port_read(machine(), kbdrow);
+	UINT8 data = ioport(kbdrow)->read();
 
 	if (m_keyboard_part)
 		data >>= 4;
@@ -374,11 +374,10 @@ static MACHINE_CONFIG_START( z1013, z1013_state )
 	MCFG_SCREEN_ADD("screen", RASTER)
 	MCFG_SCREEN_REFRESH_RATE(50)
 	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_SIZE(32*8, 32*8)
 	MCFG_SCREEN_VISIBLE_AREA(0, 32*8-1, 0, 32*8-1)
 	MCFG_VIDEO_START(z1013)
-	MCFG_SCREEN_UPDATE(z1013)
+	MCFG_SCREEN_UPDATE_STATIC(z1013)
 	MCFG_GFXDECODE(z1013)
 	MCFG_PALETTE_LENGTH(2)
 	MCFG_PALETTE_INIT(black_and_white)
