@@ -100,7 +100,7 @@
 /* peripheral chips */
 #include "machine/ay31015.h"
 #include "machine/kr2376.h"
-#include "video/m6847.h"
+#include "video/mc6847.h"
 #include "machine/wd17xx.h"
 
 /* Layout */
@@ -117,66 +117,66 @@
 
 /* LX.382 CPU Board RAM */
 /* LX.382 CPU Board EPROM */
-static ADDRESS_MAP_START( z80ne_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( z80ne_mem, AS_PROGRAM, 8, z80ne_state )
 	AM_RANGE( 0x0000, 0x03ff ) AM_RAMBANK("bank1")
 	AM_RANGE( 0x0400, 0x7fff ) AM_RAM
 	AM_RANGE( 0x8000, 0x83ff ) AM_ROMBANK("bank2")
 	AM_RANGE( 0x8400, 0xffff ) AM_READNOP AM_WRITENOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80net_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( z80net_mem, AS_PROGRAM, 8, z80ne_state )
 	AM_RANGE( 0x0000, 0x03ff ) AM_RAMBANK("bank1")
 	AM_RANGE( 0x0400, 0x7fff ) AM_RAM
 	AM_RANGE( 0x8000, 0x83ff ) AM_ROMBANK("bank2")
 	AM_RANGE( 0x8400, 0xebff ) AM_RAM
-	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_BASE_MEMBER(z80ne_state, m_videoram) /* (6847) */
+	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_SHARE("videoram") /* (6847) */
 	AM_RANGE( 0xee00, 0xffff ) AM_RAM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80netb_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( z80netb_mem, AS_PROGRAM, 8, z80ne_state )
 	AM_RANGE( 0x0000, 0x3fff ) AM_ROM
 	AM_RANGE( 0x4000, 0xebff ) AM_RAM
-	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_BASE_MEMBER(z80ne_state, m_videoram) /* (6847) */
+	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_SHARE("videoram") /* (6847) */
 	AM_RANGE( 0xee00, 0xffff ) AM_RAM
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80ne_io, AS_IO, 8 )
+static ADDRESS_MAP_START( z80ne_io, AS_IO, 8, z80ne_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0xee, 0xee) AM_READWRITE( lx385_data_r, lx385_data_w )
-	AM_RANGE(0xef, 0xef) AM_READWRITE( lx385_ctrl_r, lx385_ctrl_w )
-	AM_RANGE(0xf0, 0xff) AM_READWRITE( lx383_r, lx383_w )
+	AM_RANGE(0xee, 0xee) AM_READWRITE(lx385_data_r, lx385_data_w )
+	AM_RANGE(0xef, 0xef) AM_READWRITE(lx385_ctrl_r, lx385_ctrl_w )
+	AM_RANGE(0xf0, 0xff) AM_READWRITE(lx383_r, lx383_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80net_io, AS_IO, 8 )
+static ADDRESS_MAP_START( z80net_io, AS_IO, 8, z80ne_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0xea, 0xea) AM_READ( lx388_data_r )
-	AM_RANGE(0xeb, 0xeb) AM_READ( lx388_read_field_sync )
-	AM_RANGE(0xee, 0xee) AM_READWRITE( lx385_data_r, lx385_data_w )
-	AM_RANGE(0xef, 0xef) AM_READWRITE( lx385_ctrl_r, lx385_ctrl_w )
-	AM_RANGE(0xf0, 0xff) AM_READWRITE( lx383_r, lx383_w )
+	AM_RANGE(0xea, 0xea) AM_READ(lx388_data_r )
+	AM_RANGE(0xeb, 0xeb) AM_READ(lx388_read_field_sync )
+	AM_RANGE(0xee, 0xee) AM_READWRITE(lx385_data_r, lx385_data_w )
+	AM_RANGE(0xef, 0xef) AM_READWRITE(lx385_ctrl_r, lx385_ctrl_w )
+	AM_RANGE(0xf0, 0xff) AM_READWRITE(lx383_r, lx383_w )
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80netf_mem, AS_PROGRAM, 8 )
+static ADDRESS_MAP_START( z80netf_mem, AS_PROGRAM, 8, z80ne_state )
 	AM_RANGE( 0x0000, 0x03ff ) AM_RAMBANK("bank1")
 	AM_RANGE( 0x0400, 0x3fff ) AM_RAMBANK("bank2")
 	AM_RANGE( 0x4000, 0x7fff ) AM_RAM
 	AM_RANGE( 0x8000, 0x83ff ) AM_RAMBANK("bank3")
 	AM_RANGE( 0x8400, 0xdfff ) AM_RAM
 	AM_RANGE( 0xe000, 0xebff ) AM_READNOP AM_WRITENOP
-	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_BASE_MEMBER(z80ne_state, m_videoram) /* (6847) */
+	AM_RANGE( 0xec00, 0xedff ) AM_RAM AM_SHARE("videoram") /* (6847) */
 	AM_RANGE( 0xee00, 0xefff ) AM_READNOP AM_WRITENOP
 	AM_RANGE( 0xf000, 0xf3ff ) AM_RAMBANK("bank4")
 	AM_RANGE( 0xf400, 0xffff ) AM_READNOP AM_WRITENOP
 ADDRESS_MAP_END
 
-static ADDRESS_MAP_START( z80netf_io, AS_IO, 8 )
+static ADDRESS_MAP_START( z80netf_io, AS_IO, 8, z80ne_state )
 	ADDRESS_MAP_GLOBAL_MASK(0xff)
-	AM_RANGE(0xd0, 0xd7) AM_DEVREADWRITE("wd1771", lx390_fdc_r, lx390_fdc_w)
-	AM_RANGE(0xea, 0xea) AM_READ( lx388_data_r )
-	AM_RANGE(0xeb, 0xeb) AM_READ( lx388_read_field_sync )
-	AM_RANGE(0xee, 0xee) AM_READWRITE( lx385_data_r, lx385_data_w )
-	AM_RANGE(0xef, 0xef) AM_READWRITE( lx385_ctrl_r, lx385_ctrl_w )
-	AM_RANGE(0xf0, 0xff) AM_READWRITE( lx383_r, lx383_w )
+	AM_RANGE(0xd0, 0xd7) AM_DEVREADWRITE_LEGACY("wd1771", lx390_fdc_r, lx390_fdc_w)
+	AM_RANGE(0xea, 0xea) AM_READ(lx388_data_r )
+	AM_RANGE(0xeb, 0xeb) AM_READ(lx388_read_field_sync )
+	AM_RANGE(0xee, 0xee) AM_READWRITE(lx385_data_r, lx385_data_w )
+	AM_RANGE(0xef, 0xef) AM_READWRITE(lx385_ctrl_r, lx385_ctrl_w )
+	AM_RANGE(0xf0, 0xff) AM_READWRITE(lx383_r, lx383_w )
 ADDRESS_MAP_END
 
 
@@ -454,18 +454,19 @@ static const floppy_interface z80netf_floppy_interface =
 
 static const mc6847_interface z80net_mc6847_intf =
 {
+	"lx388",
 	DEVCB_HANDLER(lx388_mc6847_videoram_r),
-	DEVCB_LINE_GND,
-	DEVCB_LINE_GND,
-	DEVCB_LINE_GND,
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_LINE_GND,
-	DEVCB_LINE_GND,
-	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL
+
+	DEVCB_LINE_GND,				/* AG */
+	DEVCB_LINE_GND,				/* GM2 */
+	DEVCB_LINE_GND,				/* GM1 */
+	DEVCB_LINE_GND,				/* GM0 */
+	DEVCB_LINE_GND,				/* CSS */
+	DEVCB_NULL,					/* AS */
+	DEVCB_NULL,					/* INTEXT */
+	DEVCB_NULL,					/* INV */
 };
 
 static MACHINE_CONFIG_START( z80ne, z80ne_state )
@@ -501,16 +502,8 @@ static MACHINE_CONFIG_DERIVED( z80net, z80ne )
 	MCFG_KR2376_ADD( "lx388_kr2376", lx388_kr2376_interface)
 
 	/* video hardware */
-	MCFG_SCREEN_ADD("lx388", RASTER)
-	MCFG_SCREEN_REFRESH_RATE(M6847_PAL_FRAMES_PER_SECOND)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
-	MCFG_SCREEN_SIZE(320, 25+192+26)
-	MCFG_SCREEN_VISIBLE_AREA(0, 319, 1, 239)
-	MCFG_SCREEN_UPDATE(lx388)
-
-	MCFG_MC6847_ADD("mc6847", z80net_mc6847_intf)
-	MCFG_MC6847_TYPE(M6847_VERSION_ORIGINAL_PAL)
-	MCFG_MC6847_PALETTE(lx388palette)
+    MCFG_SCREEN_MC6847_PAL_ADD("lx388", "mc6847")
+	MCFG_MC6847_ADD("mc6847", MC6847_PAL, XTAL_4_433619MHz, z80net_mc6847_intf)
 
 	MCFG_DEFAULT_LAYOUT(layout_z80net)
 
@@ -537,19 +530,11 @@ static MACHINE_CONFIG_START( z80netb, z80ne_state )
 	MCFG_KR2376_ADD( "lx388_kr2376", lx388_kr2376_interface)
 
 	/* video hardware */
-	MCFG_SCREEN_ADD("lx388", RASTER)
-	MCFG_SCREEN_REFRESH_RATE(M6847_PAL_FRAMES_PER_SECOND)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
-	MCFG_SCREEN_SIZE(320, 25+192+26)
-	MCFG_SCREEN_VISIBLE_AREA(0, 319, 1, 239)
-	MCFG_SCREEN_UPDATE(lx388)
+    MCFG_SCREEN_MC6847_PAL_ADD("lx388", "mc6847")
+	MCFG_MC6847_ADD("mc6847", MC6847_PAL, XTAL_4_433619MHz, z80net_mc6847_intf)
 
 	MCFG_DEFAULT_LAYOUT(layout_z80netb)
 
-	MCFG_MC6847_ADD("mc6847", z80net_mc6847_intf)
-	MCFG_MC6847_TYPE(M6847_VERSION_ORIGINAL_PAL)
-	MCFG_MC6847_PALETTE(lx388palette)
-
 	/* internal ram */
 	MCFG_RAM_ADD(RAM_TAG)
 	MCFG_RAM_DEFAULT_SIZE("32K")
@@ -573,16 +558,8 @@ static MACHINE_CONFIG_START( z80netf, z80ne_state )
 	MCFG_KR2376_ADD( "lx388_kr2376", lx388_kr2376_interface)
 
 	/* video hardware */
-	MCFG_SCREEN_ADD("lx388", RASTER)
-	MCFG_SCREEN_REFRESH_RATE(M6847_PAL_FRAMES_PER_SECOND)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_RGB32)
-	MCFG_SCREEN_SIZE(320, 25+192+26)
-	MCFG_SCREEN_VISIBLE_AREA(0, 319, 1, 239)
-	MCFG_SCREEN_UPDATE(lx388)
-
-	MCFG_MC6847_ADD("mc6847", z80net_mc6847_intf)
-	MCFG_MC6847_TYPE(M6847_VERSION_ORIGINAL_PAL)
-	MCFG_MC6847_PALETTE(lx388palette)
+    MCFG_SCREEN_MC6847_PAL_ADD("lx388", "mc6847")
+	MCFG_MC6847_ADD("mc6847", MC6847_PAL, XTAL_4_433619MHz, z80net_mc6847_intf)
 
 	MCFG_FD1771_ADD("wd1771", default_wd17xx_interface)
 	MCFG_LEGACY_FLOPPY_4_DRIVES_ADD(z80netf_floppy_interface)
