@@ -39,7 +39,6 @@
 
 
 ****************************************************************************/
-#define ADDRESS_MAP_MODERN
 
 #include "includes/zx.h"
 
@@ -392,11 +391,10 @@ static MACHINE_CONFIG_START( zx80, zx_state )
 	MCFG_MACHINE_RESET(zx80)
 
 	/* video hardware */
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
+	MCFG_SCREEN_UPDATE_DRIVER(zx_state, screen_update)
 	MCFG_SCREEN_SIZE(ZX81_PIXELS_PER_SCANLINE, ZX81_PAL_SCANLINES)
 	MCFG_SCREEN_VISIBLE_AREA(0, ZX81_PIXELS_PER_SCANLINE-1, 0, ZX81_PAL_SCANLINES-1)
-	MCFG_SCREEN_UPDATE(generic_bitmapped)
-	MCFG_SCREEN_EOF(zx)
+	MCFG_SCREEN_VBLANK_STATIC(zx)
 
 	MCFG_GFXDECODE(zx80)
 	MCFG_PALETTE_LENGTH(4)
