@@ -32,8 +32,8 @@ extern const z80pio_interface ac1_z80pio_intf;
 extern const gfx_layout ac1_charlayout;
 
 VIDEO_START( ac1 );
-SCREEN_UPDATE( ac1 );
-SCREEN_UPDATE( ac1_32 );
+SCREEN_UPDATE_IND16( ac1 );
+SCREEN_UPDATE_IND16( ac1_32 );
 
 
 #endif /* AC1_h_ */
