@@ -34,9 +34,10 @@ class concept_state : public driver_device
 {
 public:
 	concept_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag),
+		m_videoram(*this,"videoram") { }
 
-	UINT16 *m_videoram;
+	required_shared_ptr<UINT16> m_videoram;
 	UINT8 m_pending_interrupts;
 	char m_clock_enable;
 	char m_clock_address;
@@ -47,6 +48,11 @@ public:
 	UINT8 m_fdc_local_status;
 	UINT8 m_fdc_local_command;
 	expansion_slot_t m_expansion_slots[4];
+	DECLARE_READ16_MEMBER(concept_io_r);
+	DECLARE_WRITE16_MEMBER(concept_io_w);
+	DECLARE_WRITE8_MEMBER(concept_fdc_reg_w);
+	DECLARE_READ8_MEMBER(concept_hdc_reg_r);
+	DECLARE_WRITE8_MEMBER(concept_hdc_reg_w);
 };
 
 
@@ -57,10 +63,8 @@ extern const wd17xx_interface concept_wd17xx_interface;
 
 MACHINE_START(concept);
 VIDEO_START(concept);
-SCREEN_UPDATE(concept);
+SCREEN_UPDATE_IND16(concept);
 INTERRUPT_GEN( concept_interrupt );
-READ16_HANDLER(concept_io_r);
-WRITE16_HANDLER(concept_io_w);
 
 
 #endif /* CONCEPT_H_ */
