@@ -1,7 +1,6 @@
 #ifndef __LC80__
 #define __LC80__
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "cpu/z80/z80.h"
@@ -35,7 +34,7 @@ public:
 	required_device<z80pio_device> m_pio2;
 	required_device<cassette_image_device> m_cassette;
 	required_device<device_t> m_speaker;
-	required_device<device_t> m_ram;
+	required_device<ram_device> m_ram;
 
 	virtual void machine_start();
 
@@ -46,6 +45,8 @@ public:
 	DECLARE_READ8_MEMBER( pio1_pb_r );
 	DECLARE_WRITE8_MEMBER( pio1_pb_w );
 	DECLARE_READ8_MEMBER( pio2_pb_r );
+	DECLARE_INPUT_CHANGED_MEMBER( trigger_reset );
+	DECLARE_INPUT_CHANGED_MEMBER( trigger_nmi );
 
 	void update_display();
 
