@@ -9,8 +9,9 @@
 #ifndef PC1251_H_
 #define PC1251_H_
 
+#include "machine/nvram.h"
 
-#define PC1251_CONTRAST (input_port_read(machine, "DSW0") & 0x07)
+#define PC1251_CONTRAST (machine.root_device().ioport("DSW0")->read() & 0x07)
 
 
 class pc1251_state : public driver_device
@@ -38,14 +39,14 @@ int pc1251_ina(device_t *device);
 int pc1251_inb(device_t *device);
 
 DRIVER_INIT( pc1251 );
-NVRAM_HANDLER( pc1251 );
+MACHINE_START( pc1251 );
 
 
 /*----------- defined in video/pc1251.c -----------*/
 
 READ8_HANDLER(pc1251_lcd_read);
 WRITE8_HANDLER(pc1251_lcd_write);
-SCREEN_UPDATE( pc1251 );
+SCREEN_UPDATE_IND16( pc1251 );
 
 
 #endif /* PC1251_H_ */
