@@ -9,7 +9,9 @@
 #ifndef PC1401_H_
 #define PC1401_H_
 
-#define CONTRAST (input_port_read(machine, "DSW0") & 0x07)
+#include "machine/nvram.h"
+
+#define CONTRAST (machine.root_device().ioport("DSW0")->read() & 0x07)
 
 
 class pc1401_state : public driver_device
@@ -37,14 +39,14 @@ int pc1401_ina(device_t *device);
 int pc1401_inb(device_t *device);
 
 DRIVER_INIT( pc1401 );
-NVRAM_HANDLER( pc1401 );
+MACHINE_START( pc1401 );
 
 
 /*----------- defined in video/pc1401.c -----------*/
 
 READ8_HANDLER(pc1401_lcd_read);
 WRITE8_HANDLER(pc1401_lcd_write);
-SCREEN_UPDATE( pc1401 );
+SCREEN_UPDATE_IND16( pc1401 );
 
 
 #endif /* PC1401_H_ */
