@@ -25,6 +25,11 @@ public:
 	UINT8 m_model;
 	emu_timer * m_cassette_timer;
 	void (*update_memory)(running_machine &);
+	DECLARE_READ8_MEMBER(pmd85_io_r);
+	DECLARE_WRITE8_MEMBER(pmd85_io_w);
+	DECLARE_READ8_MEMBER(mato_io_r);
+	DECLARE_WRITE8_MEMBER(mato_io_w);
+	DECLARE_DIRECT_UPDATE_MEMBER(pmd85_opbaseoverride);
 };
 
 
@@ -35,10 +40,6 @@ extern const i8255_interface pmd85_ppi8255_interface[4];
 extern const i8255_interface alfa_ppi8255_interface[3];
 extern const i8255_interface mato_ppi8255_interface;
 
- READ8_HANDLER ( pmd85_io_r );
-WRITE8_HANDLER ( pmd85_io_w );
- READ8_HANDLER ( mato_io_r );
-WRITE8_HANDLER ( mato_io_w );
 DRIVER_INIT ( pmd851 );
 DRIVER_INIT ( pmd852a );
 DRIVER_INIT ( pmd853 );
@@ -51,7 +52,7 @@ extern MACHINE_RESET( pmd85 );
 /*----------- defined in video/pmd85.c -----------*/
 
 extern VIDEO_START( pmd85 );
-extern SCREEN_UPDATE( pmd85 );
+extern SCREEN_UPDATE_IND16( pmd85 );
 extern const unsigned char pmd85_palette[3*3];
 extern PALETTE_INIT( pmd85 );
 
