@@ -14,9 +14,10 @@ class poly88_state : public driver_device
 {
 public:
 	poly88_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag) { }
+		: driver_device(mconfig, type, tag) ,
+		m_video_ram(*this, "video_ram"){ }
 
-	UINT8 *m_video_ram;
+	required_shared_ptr<UINT8> m_video_ram;
 	UINT8 *m_FNT;
 	UINT8 m_intr;
 	UINT8 m_last_code;
@@ -26,6 +27,9 @@ public:
 	int m_previous_level;
 	int m_clk_level;
 	int m_clk_level_tape;
+	DECLARE_WRITE8_MEMBER(poly88_baud_rate_w);
+	DECLARE_READ8_MEMBER(poly88_keyboard_r);
+	DECLARE_WRITE8_MEMBER(poly88_intr_w);
 };
 
 
@@ -34,9 +38,6 @@ public:
 DRIVER_INIT(poly88);
 MACHINE_RESET(poly88);
 INTERRUPT_GEN( poly88_interrupt );
-READ8_HANDLER(poly88_keyboard_r);
-WRITE8_HANDLER(poly88_intr_w);
-WRITE8_HANDLER(poly88_baud_rate_w);
 
 extern const i8251_interface poly88_usart_interface;
 
@@ -46,7 +47,7 @@ extern SNAPSHOT_LOAD( poly88 );
 /*----------- defined in video/poly88.c -----------*/
 
 extern VIDEO_START( poly88 );
-extern SCREEN_UPDATE( poly88 );
+extern SCREEN_UPDATE_IND16( poly88 );
 
 #endif /* POLY88_H_ */
 
