@@ -171,19 +171,19 @@ static int get_id_from_device( device_t *device )
 {
 	int id = -1;
 
-	if ( ! strcmp( "harddisk1", device->tag() ) )
+	if ( ! strcmp( ":harddisk1", device->tag() ) )
 	{
 		id = 0;
 	}
-	else if ( ! strcmp( "harddisk2", device->tag() ) )
+	else if ( ! strcmp( ":harddisk2", device->tag() ) )
 	{
 		id = 1;
 	}
-	else if ( ! strcmp( "harddisk3", device->tag() ) )
+	else if ( ! strcmp( ":harddisk3", device->tag() ) )
 	{
 		id = 2;
 	}
-	else if ( ! strcmp( "harddisk4", device->tag() ) )
+	else if ( ! strcmp( ":harddisk4", device->tag() ) )
 	{
 		id = 3;
 	}
@@ -1039,7 +1039,9 @@ WRITE16_HANDLER(ti990_hdc_w)
 static const struct harddisk_interface ti990_harddisk_config =
 {
 	DEVICE_IMAGE_LOAD_NAME( ti990_hd ),
-	DEVICE_IMAGE_UNLOAD_NAME( ti990_hd )
+	DEVICE_IMAGE_UNLOAD_NAME( ti990_hd ),
+	NULL,
+	NULL
 };
 
 MACHINE_CONFIG_FRAGMENT( ti990_hdc )
