@@ -1,9 +1,16 @@
+/**********************************************************************
+
+    Luxor ABC-55/77 keyboard emulation
+
+    Copyright MESS Team.
+    Visit http://mamedev.org for licensing and usage restrictions.
+
+*********************************************************************/
+
 /*
 
-Luxor ABC55/77
-
-Keyboard PCB Layout
--------------------
+PCB Layout
+----------
 
 KTC A65-02486-232
 
@@ -179,13 +186,11 @@ machine_config_constructor abc77_device::device_mconfig_additions() const
 //  INPUT_CHANGED( keyboard_reset )
 //-------------------------------------------------
 
-INPUT_CHANGED( abc77_device::keyboard_reset )
+INPUT_CHANGED_MEMBER( abc77_device::keyboard_reset )
 {
-    abc77_device *keyboard = static_cast<abc77_device *>(field.machine().device(ABC77_TAG));
-
 	if (oldval && !newval)
 	{
-		keyboard->device_reset();
+		device_reset();
 	}
 }
 
@@ -325,7 +330,7 @@ INPUT_PORTS_START( abc77 )
 	PORT_DIPNAME( 0x04, 0x04, "External Encoding PROM" ) // @ Z14
 	PORT_DIPSETTING(    0x04, DEF_STR( Off ) )
 	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
-	PORT_DIPNAME( 0x18, 0x18, "Keyboard Language" ) PORT_CONDITION("DSW", 0x04, PORTCOND_EQUALS, 0x00)
+	PORT_DIPNAME( 0x18, 0x18, "Keyboard Language" ) PORT_CONDITION("DSW", 0x04, EQUALS, 0x00)
 	PORT_DIPSETTING(    0x00, "Danish" )
 	PORT_DIPSETTING(    0x10, DEF_STR( French ) )
 	PORT_DIPSETTING(    0x08, DEF_STR( German ) )
@@ -333,7 +338,7 @@ INPUT_PORTS_START( abc77 )
 	PORT_BIT( 0xe0, IP_ACTIVE_LOW, IPT_UNUSED)
 
 	PORT_START("SW1")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Keyboard Reset") PORT_CHANGED(abc77_device::keyboard_reset, 0)
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_SPECIAL ) PORT_NAME("Keyboard Reset") PORT_CHANGED_MEMBER(DEVICE_SELF, abc77_device, keyboard_reset, 0)
 INPUT_PORTS_END
 
 
@@ -440,7 +445,7 @@ void abc77_device::device_start()
 void abc77_device::device_reset()
 {
 	int t = 1.1 * RES_K(100) * CAP_N(100) * 1000; // t = 1.1 * R1 * C1
-	int ea = BIT(input_port_read(this, "DSW"), 7);
+	int ea = BIT(ioport("DSW")->read(), 7);
 
 	// trigger reset
 	device_set_input_line(m_maincpu, INPUT_LINE_RESET, ASSERT_LINE);
@@ -495,7 +500,7 @@ READ8_MEMBER( abc77_device::p1_r )
 
 	if (m_stb && m_keylatch < 12)
 	{
-		data = input_port_read(this, keynames[m_keylatch]);
+		data = ioport(keynames[m_keylatch])->read();
 	}
 
 	return data;
@@ -529,7 +534,7 @@ WRITE8_MEMBER( abc77_device::p2_w )
 
 		if (m_keylatch == 1)
 		{
-			watchdog_reset(machine());
+			machine().watchdog_reset();
 		}
 	}
 
