@@ -357,54 +357,44 @@ static void bebox_set_irq_bit(running_machine &machine, unsigned int interrupt_b
  *
  *************************************/
 
-static INS8250_TRANSMIT( bebox_uart_transmit )
-{
-	if (LOG_UART)
-		logerror("bebox_uart_transmit(): data=0x%02X\n", data);
-}
-
-
-static INS8250_HANDSHAKE_OUT( bebox_uart_handshake )
-{
-	if (LOG_UART)
-		logerror("bebox_uart_handshake(): data=0x%02X\n", data);
-}
-
-
 const ins8250_interface bebox_uart_inteface_0 =
 {
-	0,
 	DEVCB_NULL,
-	NULL,
-	NULL,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
 const ins8250_interface bebox_uart_inteface_1 =
 {
-	0,
 	DEVCB_NULL,
-	NULL,
-	NULL,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
 const ins8250_interface bebox_uart_inteface_2 =
 {
-	0,
 	DEVCB_NULL,
-	NULL,
-	NULL,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
 const ins8250_interface bebox_uart_inteface_3 =
 {
-	0,
 	DEVCB_NULL,
-	bebox_uart_transmit,
-	bebox_uart_handshake,
-	NULL
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL,
+	DEVCB_NULL
 };
 
 
@@ -871,6 +861,7 @@ static const struct kbdc8042_interface bebox_8042_interface =
 	KBDC8042_STANDARD,
 	NULL,
 	bebox_keyboard_interrupt,
+	NULL,
 	bebox_get_out2
 };
 
@@ -1080,7 +1071,7 @@ MACHINE_RESET( bebox )
 	cputag_set_input_line(machine, "ppc1", INPUT_LINE_RESET, CLEAR_LINE);
 	cputag_set_input_line(machine, "ppc2", INPUT_LINE_RESET, ASSERT_LINE);
 
-	memcpy(machine.device<fujitsu_29f016a_device>("flash")->space()->get_read_ptr(0),machine.region("user1")->base(),0x200000);
+	memcpy(machine.device<fujitsu_29f016a_device>("flash")->space()->get_read_ptr(0),state->memregion("user1")->base(),0x200000);
 }
 
 static void bebox_exit(running_machine &machine)
@@ -1098,18 +1089,19 @@ MACHINE_START( bebox )
 
 DRIVER_INIT( bebox )
 {
+	bebox_state *state = machine.driver_data<bebox_state>();
 	address_space *space_0 = machine.device("ppc1")->memory().space(AS_PROGRAM);
 	address_space *space_1 = machine.device("ppc2")->memory().space(AS_PROGRAM);
 	offs_t vram_begin;
 	offs_t vram_end;
 
 	/* set up boot and flash ROM */
-	memory_set_bankptr(machine, "bank2", machine.region("user2")->base());
+	state->membank("bank2")->set_base(machine.root_device().memregion("user2")->base());
 
 	/* install MESS managed RAM */
-	space_0->install_readwrite_bank(0, ram_get_size(machine.device(RAM_TAG)) - 1, 0, 0x02000000, "bank3");
-	space_1->install_readwrite_bank(0, ram_get_size(machine.device(RAM_TAG)) - 1, 0, 0x02000000, "bank3");
-	memory_set_bankptr(machine, "bank3", ram_get_ptr(machine.device(RAM_TAG)));
+	space_0->install_readwrite_bank(0, machine.device<ram_device>(RAM_TAG)->size() - 1, 0, 0x02000000, "bank3");
+	space_1->install_readwrite_bank(0, machine.device<ram_device>(RAM_TAG)->size() - 1, 0, 0x02000000, "bank3");
+	state->membank("bank3")->set_base(machine.device<ram_device>(RAM_TAG)->pointer());
 
 	kbdc8042_init(machine, &bebox_8042_interface);
 
@@ -1138,6 +1130,6 @@ DRIVER_INIT( bebox )
 			U64(0x4E80042000000000)
 		};
 		space_1->install_read_bank(0x9421FFF0, 0x9421FFFF, "bank1");
-		memory_set_bankptr(machine, "bank1", ops);
+		state->membank("bank1")->set_base(ops);
 	}
 }
