@@ -11,7 +11,7 @@
 
 #include "emu.h"
 #include "includes/c128.h"
-#include "includes/c64.h"
+#include "includes/c64_legacy.h"
 #include "includes/cbm.h"
 #include "machine/cbmiec.h"
 #include "cpu/m6502/m6502.h"
@@ -94,19 +94,19 @@ static void c128_nmi( running_machine &machine )
 	device_t *cia_1 = machine.device("cia_1");
 	int cia1irq = mos6526_irq_r(cia_1);
 
-	if (state->m_nmilevel != (input_port_read(machine, "SPECIAL") & 0x80) || cia1irq)	/* KEY_RESTORE */
+	if (state->m_nmilevel != (machine.root_device().ioport("SPECIAL")->read() & 0x80) || cia1irq)	/* KEY_RESTORE */
 	{
 		if (1) // this was never valid, there is no active CPU during a timer firing!  cpu_getactivecpu() == 0)
 		{
 			/* z80 */
-			cputag_set_input_line(machine, "maincpu", INPUT_LINE_NMI, (input_port_read(machine, "SPECIAL") & 0x80) || cia1irq);
+			cputag_set_input_line(machine, "maincpu", INPUT_LINE_NMI, (machine.root_device().ioport("SPECIAL")->read() & 0x80) || cia1irq);
 		}
 		else
 		{
-			cputag_set_input_line(machine, "m8502", INPUT_LINE_NMI, (input_port_read(machine, "SPECIAL") & 0x80) || cia1irq);
+			cputag_set_input_line(machine, "m8502", INPUT_LINE_NMI, (machine.root_device().ioport("SPECIAL")->read() & 0x80) || cia1irq);
 		}
 
-		state->m_nmilevel = (input_port_read(machine, "SPECIAL") & 0x80) || cia1irq;
+		state->m_nmilevel = (machine.root_device().ioport("SPECIAL")->read() & 0x80) || cia1irq;
 	}
 }
 
@@ -184,15 +184,14 @@ static void c128_cia0_interrupt( device_t *device, int level )
 	c128_irq(device->machine(), level || state->m_vicirq);
 }
 
-void c128_vic_interrupt( running_machine &machine, int level )
+WRITE_LINE_MEMBER( c128_state::vic_interrupt )
 {
-	c128_state *state = machine.driver_data<c128_state>();
-	device_t *cia_0 = machine.device("cia_0");
+	device_t *cia_0 = machine().device("cia_0");
 #if 1
-	if (level != state->m_vicirq)
+	if (state  != m_vicirq)
 	{
-		c128_irq (machine, level || mos6526_irq_r(cia_0));
-		state->m_vicirq = level;
+		c128_irq (machine(), state || mos6526_irq_r(cia_0));
+		m_vicirq = state;
 	}
 #endif
 }
@@ -461,12 +460,12 @@ static READ8_HANDLER( c128_read_io )
 		return state->m_colorram[offset & 0x3ff];
 	else if (offset == 0xc00)
 		{
-			cia_set_port_mask_value(cia_0, 0, input_port_read(space->machine(), "CTRLSEL") & 0x80 ? c64_keyline[8] : c64_keyline[9] );
+			cia_set_port_mask_value(cia_0, 0, state->ioport("CTRLSEL")->read() & 0x80 ? c64_keyline[8] : c64_keyline[9] );
 			return mos6526_r(cia_0, offset);
 		}
 	else if (offset == 0xc01)
 		{
-			cia_set_port_mask_value(cia_0, 1, input_port_read(space->machine(), "CTRLSEL") & 0x80 ? c64_keyline[9] : c64_keyline[8] );
+			cia_set_port_mask_value(cia_0, 1, space->machine().root_device().ioport("CTRLSEL")->read() & 0x80 ? c64_keyline[9] : c64_keyline[8] );
 			return mos6526_r(cia_0, offset);
 		}
 	else if (offset < 0xd00)
@@ -497,16 +496,16 @@ void c128_bankswitch_64( running_machine &machine, int reset )
 	charen = (data & 4) ? 1 : 0;
 
 	if ((!state->m_game && state->m_exrom) || (loram && hiram && !state->m_exrom))
-		memory_set_bankptr(machine, "bank8", state->m_roml);
+		state->membank("bank8")->set_base(state->m_roml);
 	else
-		memory_set_bankptr(machine, "bank8", state->m_memory + 0x8000);
+		state->membank("bank8")->set_base(state->m_memory + 0x8000);
 
 	if ((!state->m_game && state->m_exrom && hiram) || (!state->m_exrom))
-		memory_set_bankptr(machine, "bank9", state->m_romh);
+		state->membank("bank9")->set_base(state->m_romh);
 	else if (loram && hiram)
-		memory_set_bankptr(machine, "bank9", state->m_basic);
+		state->membank("bank9")->set_base(state->m_basic);
 	else
-		memory_set_bankptr(machine, "bank9", state->m_memory + 0xa000);
+		state->membank("bank9")->set_base(state->m_memory + 0xa000);
 
 	if ((!state->m_game && state->m_exrom) || (charen && (loram || hiram)))
 	{
@@ -518,30 +517,30 @@ void c128_bankswitch_64( running_machine &machine, int reset )
 		machine.device("m8502")->memory().space(AS_PROGRAM)->install_read_bank(0xd000, 0xdfff, "bank5");
 		state->m_write_io = 0;
 		if ((!charen && (loram || hiram)))
-			memory_set_bankptr(machine, "bank13", state->m_chargen);
+			state->membank("bank13")->set_base(state->m_chargen);
 		else
-			memory_set_bankptr(machine, "bank13", state->m_memory + 0xd000);
+			state->membank("bank13")->set_base(state->m_memory + 0xd000);
 	}
 
 	if (!state->m_game && state->m_exrom)
 	{
-		memory_set_bankptr(machine, "bank14", state->m_romh);
-		memory_set_bankptr(machine, "bank15", state->m_romh+0x1f00);
-		memory_set_bankptr(machine, "bank16", state->m_romh+0x1f05);
+		state->membank("bank14")->set_base(state->m_romh);
+		state->membank("bank15")->set_base(state->m_romh+0x1f00);
+		state->membank("bank16")->set_base(state->m_romh+0x1f05);
 	}
 	else
 	{
 		if (hiram)
 		{
-			memory_set_bankptr(machine, "bank14", state->m_kernal);
-			memory_set_bankptr(machine, "bank15", state->m_kernal+0x1f00);
-			memory_set_bankptr(machine, "bank16", state->m_kernal+0x1f05);
+			state->membank("bank14")->set_base(state->m_kernal);
+			state->membank("bank15")->set_base(state->m_kernal+0x1f00);
+			state->membank("bank16")->set_base(state->m_kernal+0x1f05);
 		}
 		else
 		{
-			memory_set_bankptr(machine, "bank14", state->m_memory + 0xe000);
-			memory_set_bankptr(machine, "bank15", state->m_memory + 0xff00);
-			memory_set_bankptr(machine, "bank16", state->m_memory + 0xff05);
+			state->membank("bank14")->set_base(state->m_memory + 0xe000);
+			state->membank("bank15")->set_base(state->m_memory + 0xff00);
+			state->membank("bank16")->set_base(state->m_memory + 0xff05);
 		}
 	}
 	state->m_old_data = data;
@@ -557,10 +556,10 @@ static void c128_bankswitch_z80( running_machine &machine )
 	 state->m_ram = state->m_memory + MMU_RAM_ADDR;
 	 state->m_va1617 = MMU_VIC_ADDR;
 #if 1
-	 memory_set_bankptr(machine, "bank10", state->m_z80);
-	 memory_set_bankptr(machine, "bank11", state->m_ram + 0x1000);
-	 if ( (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
-		  || (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
+	 state->membank("bank10")->set_base(state->m_z80);
+	 state->membank("bank11")->set_base(state->m_ram + 0x1000);
+	 if ( (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
+		  || (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
 		 state->m_ram = NULL;
 #else
 	 if (MMU_BOTTOM)
@@ -569,21 +568,21 @@ static void c128_bankswitch_z80( running_machine &machine )
 		 state->m_ram_bottom = 0;
 
 	 if (MMU_RAM_ADDR==0) { /* this is used in z80 mode for rom on/off switching !*/
-		 memory_set_bankptr(machine, "bank10", state->m_z80);
-		 memory_set_bankptr(machine, "bank11", state->m_z80 + 0x400);
+		 state->membank("bank10")->set_base(state->m_z80);
+		 state->membank("bank11")->set_base(state->m_z80 + 0x400);
 	 }
 	 else
 	 {
-		 memory_set_bankptr(machine, "bank10", (state->m_ram_bottom > 0 ? state->m_memory : state->m_ram));
-		 memory_set_bankptr(machine, "bank11", (state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
+		 state->membank("bank10")->set_base((state->m_ram_bottom > 0 ? state->m_memory : state->m_ram));
+		 state->membank("bank11")->set_base((state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
 	 }
 
-	 memory_set_bankptr(machine, "bank1", (state->m_ram_bottom > 0 ? state->m_memory : state->m_ram));
-	 memory_set_bankptr(machine, "bank2", (state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
+	 state->membank("bank1")->set_base((state->m_ram_bottom > 0 ? state->m_memory : state->m_ram));
+	 state->membank("bank2")->set_base((state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
 
-	 memory_set_bankptr(machine, "bank3", (state->m_ram_bottom > 0x1000 ? state->m_memory : state->m_ram) + 0x1000);
-	 memory_set_bankptr(machine, "bank4", (state->m_ram_bottom > 0x2000 ? state->m_memory : state->m_ram) + 0x2000);
-	 memory_set_bankptr(machine, "bank5", state->m_ram + 0x4000);
+	 state->membank("bank3")->set_base((state->m_ram_bottom > 0x1000 ? state->m_memory : state->m_ram) + 0x1000);
+	 state->membank("bank4")->set_base((state->m_ram_bottom > 0x2000 ? state->m_memory : state->m_ram) + 0x2000);
+	 state->membank("bank5")->set_base(state->m_ram + 0x4000);
 
 	 if (MMU_TOP)
 		 state->m_ram_top = 0x10000 - MMU_SIZE;
@@ -591,27 +590,27 @@ static void c128_bankswitch_z80( running_machine &machine )
 		 state->m_ram_top = 0x10000;
 
 	 if (state->m_ram_top > 0xc000)
-		memory_set_bankptr(machine, "bank6", state->m_ram + 0xc000);
+		state->membank("bank6")->set_base(state->m_ram + 0xc000);
 	 else
-		memory_set_bankptr(machine, "bank6", state->m_memory + 0xc000);
+		state->membank("bank6")->set_base(state->m_memory + 0xc000);
 
 	 if (state->m_ram_top > 0xe000)
-		memory_set_bankptr(machine, "bank7", state->m_ram + 0xe000);
+		state->membank("bank7")->set_base(state->m_ram + 0xe000);
 	 else
-		memory_set_bankptr(machine, "bank7", state->m_memory + 0xd000);
+		state->membank("bank7")->set_base(state->m_memory + 0xd000);
 
 	 if (state->m_ram_top > 0xf000)
-		memory_set_bankptr(machine, "bank8", state->m_ram + 0xf000);
+		state->membank("bank8")->set_base(state->m_ram + 0xf000);
 	 else
-		memory_set_bankptr(machine, "bank8", state->m_memory + 0xe000);
+		state->membank("bank8")->set_base(state->m_memory + 0xe000);
 
 	 if (state->m_ram_top > 0xff05)
-		memory_set_bankptr(machine, "bank9", state->m_ram + 0xff05);
+		state->membank("bank9")->set_base(state->m_ram + 0xff05);
 	 else
-		memory_set_bankptr(machine, "bank9", state->m_memory + 0xff05);
+		state->membank("bank9")->set_base(state->m_memory + 0xff05);
 
-	 if ( (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
-		  || (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
+	 if ( (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
+		  || (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
 		 state->m_ram = NULL;
 #endif
 }
@@ -628,17 +627,17 @@ static void c128_bankswitch_128( running_machine &machine, int reset )
 		state->m_ram_bottom = 0;
 		state->m_ram_top = 0x10000;
 
-		memory_set_bankptr(machine, "bank1", state->m_memory);
-		memory_set_bankptr(machine, "bank2", state->m_memory + 0x100);
+		state->membank("bank1")->set_base(state->m_memory);
+		state->membank("bank2")->set_base(state->m_memory + 0x100);
 
-		memory_set_bankptr(machine, "bank3", state->m_memory + 0x200);
-		memory_set_bankptr(machine, "bank4", state->m_memory + 0x400);
-		memory_set_bankptr(machine, "bank5", state->m_memory + 0x1000);
-		memory_set_bankptr(machine, "bank6", state->m_memory + 0x2000);
+		state->membank("bank3")->set_base(state->m_memory + 0x200);
+		state->membank("bank4")->set_base(state->m_memory + 0x400);
+		state->membank("bank5")->set_base(state->m_memory + 0x1000);
+		state->membank("bank6")->set_base(state->m_memory + 0x2000);
 
-		memory_set_bankptr(machine, "bank7", state->m_memory + 0x4000);
+		state->membank("bank7")->set_base(state->m_memory + 0x4000);
 
-		memory_set_bankptr(machine, "bank12", state->m_memory + 0xc000);
+		state->membank("bank12")->set_base(state->m_memory + 0xc000);
 
 		c128_bankswitch_64(machine, reset);
 	}
@@ -646,47 +645,47 @@ static void c128_bankswitch_128( running_machine &machine, int reset )
 	{
 		state->m_ram = state->m_memory + MMU_RAM_ADDR;
 		state->m_va1617 = MMU_VIC_ADDR;
-		memory_set_bankptr(machine, "bank1", state->m_memory + state->m_mmu_page0);
-		memory_set_bankptr(machine, "bank2", state->m_memory + state->m_mmu_page1);
+		state->membank("bank1")->set_base(state->m_memory + state->m_mmu_page0);
+		state->membank("bank2")->set_base(state->m_memory + state->m_mmu_page1);
 		if (MMU_BOTTOM)
 			{
 				state->m_ram_bottom = MMU_SIZE;
 			}
 		else
 			state->m_ram_bottom = 0;
-		memory_set_bankptr(machine, "bank3", (state->m_ram_bottom > 0x200 ? state->m_memory : state->m_ram) + 0x200);
-		memory_set_bankptr(machine, "bank4", (state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
-		memory_set_bankptr(machine, "bank5", (state->m_ram_bottom > 0x1000 ? state->m_memory : state->m_ram) + 0x1000);
-		memory_set_bankptr(machine, "bank6", (state->m_ram_bottom > 0x2000 ? state->m_memory : state->m_ram) + 0x2000);
+		state->membank("bank3")->set_base((state->m_ram_bottom > 0x200 ? state->m_memory : state->m_ram) + 0x200);
+		state->membank("bank4")->set_base((state->m_ram_bottom > 0x400 ? state->m_memory : state->m_ram) + 0x400);
+		state->membank("bank5")->set_base((state->m_ram_bottom > 0x1000 ? state->m_memory : state->m_ram) + 0x1000);
+		state->membank("bank6")->set_base((state->m_ram_bottom > 0x2000 ? state->m_memory : state->m_ram) + 0x2000);
 
 		if (MMU_RAM_LO)
 		{
-			memory_set_bankptr(machine, "bank7", state->m_ram + 0x4000);
+			state->membank("bank7")->set_base(state->m_ram + 0x4000);
 		}
 		else
 		{
-			memory_set_bankptr(machine, "bank7", state->m_c128_basic);
+			state->membank("bank7")->set_base(state->m_c128_basic);
 		}
 
 		if (MMU_RAM_MID)
 		{
-			memory_set_bankptr(machine, "bank8", state->m_ram + 0x8000);
-			memory_set_bankptr(machine, "bank9", state->m_ram + 0xa000);
+			state->membank("bank8")->set_base(state->m_ram + 0x8000);
+			state->membank("bank9")->set_base(state->m_ram + 0xa000);
 		}
 		else if (MMU_ROM_MID)
 		{
-			memory_set_bankptr(machine, "bank8", state->m_c128_basic + 0x4000);
-			memory_set_bankptr(machine, "bank9", state->m_c128_basic + 0x6000);
+			state->membank("bank8")->set_base(state->m_c128_basic + 0x4000);
+			state->membank("bank9")->set_base(state->m_c128_basic + 0x6000);
 		}
 		else if (MMU_INTERNAL_ROM_MID)
 		{
-			memory_set_bankptr(machine, "bank8", state->m_internal_function);
-			memory_set_bankptr(machine, "bank9", state->m_internal_function + 0x2000);
+			state->membank("bank8")->set_base(state->m_internal_function);
+			state->membank("bank9")->set_base(state->m_internal_function + 0x2000);
 		}
 		else
 		{
-			memory_set_bankptr(machine, "bank8", state->m_external_function);
-			memory_set_bankptr(machine, "bank9", state->m_external_function + 0x2000);
+			state->membank("bank8")->set_base(state->m_external_function);
+			state->membank("bank9")->set_base(state->m_external_function + 0x2000);
 		}
 
 		if (MMU_TOP)
@@ -714,70 +713,70 @@ static void c128_bankswitch_128( running_machine &machine, int reset )
 		{
 			if (state->m_ram_top > 0xc000)
 			{
-				memory_set_bankptr(machine, "bank12", state->m_ram + 0xc000);
+				state->membank("bank12")->set_base(state->m_ram + 0xc000);
 			}
 			else
 			{
-				memory_set_bankptr(machine, "bank12", state->m_memory + 0xc000);
+				state->membank("bank12")->set_base(state->m_memory + 0xc000);
 			}
 			if (!MMU_IO_ON)
 			{
 				if (state->m_ram_top > 0xd000)
 				{
-					memory_set_bankptr(machine, "bank13", state->m_ram + 0xd000);
+					state->membank("bank13")->set_base(state->m_ram + 0xd000);
 				}
 				else
 				{
-					memory_set_bankptr(machine, "bank13", state->m_memory + 0xd000);
+					state->membank("bank13")->set_base(state->m_memory + 0xd000);
 				}
 			}
 			if (state->m_ram_top > 0xe000)
 			{
-				memory_set_bankptr(machine, "bank14", state->m_ram + 0xe000);
+				state->membank("bank14")->set_base(state->m_ram + 0xe000);
 			}
 			else
 			{
-				memory_set_bankptr(machine, "bank14", state->m_memory + 0xe000);
+				state->membank("bank14")->set_base(state->m_memory + 0xe000);
 			}
 			if (state->m_ram_top > 0xff05)
 			{
-				memory_set_bankptr(machine, "bank16", state->m_ram + 0xff05);
+				state->membank("bank16")->set_base(state->m_ram + 0xff05);
 			}
 			else
 			{
-				memory_set_bankptr(machine, "bank16", state->m_memory + 0xff05);
+				state->membank("bank16")->set_base(state->m_memory + 0xff05);
 			}
 		}
 		else if (MMU_ROM_HI)
 		{
-			memory_set_bankptr(machine, "bank12", state->m_editor);
+			state->membank("bank12")->set_base(state->m_editor);
 			if (!MMU_IO_ON) {
-				memory_set_bankptr(machine, "bank13", state->m_c128_chargen);
+				state->membank("bank13")->set_base(state->m_c128_chargen);
 			}
-			memory_set_bankptr(machine, "bank14", state->m_c128_kernal);
-			memory_set_bankptr(machine, "bank16", state->m_c128_kernal + 0x1f05);
+			state->membank("bank14")->set_base(state->m_c128_kernal);
+			state->membank("bank16")->set_base(state->m_c128_kernal + 0x1f05);
 		}
 		else if (MMU_INTERNAL_ROM_HI)
 		{
-			memory_set_bankptr(machine, "bank12", state->m_internal_function);
+			state->membank("bank12")->set_base(state->m_internal_function);
 			if (!MMU_IO_ON) {
-				memory_set_bankptr(machine, "bank13", state->m_internal_function + 0x1000);
+				state->membank("bank13")->set_base(state->m_internal_function + 0x1000);
 			}
-			memory_set_bankptr(machine, "bank14", state->m_internal_function + 0x2000);
-			memory_set_bankptr(machine, "bank16", state->m_internal_function + 0x3f05);
+			state->membank("bank14")->set_base(state->m_internal_function + 0x2000);
+			state->membank("bank16")->set_base(state->m_internal_function + 0x3f05);
 		}
 		else					   /*if (MMU_EXTERNAL_ROM_HI) */
 		{
-			memory_set_bankptr(machine, "bank12", state->m_external_function);
+			state->membank("bank12")->set_base(state->m_external_function);
 			if (!MMU_IO_ON) {
-				memory_set_bankptr(machine, "bank13", state->m_external_function + 0x1000);
+				state->membank("bank13")->set_base(state->m_external_function + 0x1000);
 			}
-			memory_set_bankptr(machine, "bank14", state->m_external_function + 0x2000);
-			memory_set_bankptr(machine, "bank16", state->m_external_function + 0x3f05);
+			state->membank("bank14")->set_base(state->m_external_function + 0x2000);
+			state->membank("bank16")->set_base(state->m_external_function + 0x3f05);
 		}
 
-		if ( (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
-				|| (( (input_port_read(machine, "SPECIAL") & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
+		if ( (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x02 ) && (MMU_RAM_ADDR >= 0x40000))
+				|| (( (machine.root_device().ioport("SPECIAL")->read() & 0x06) == 0x00) && (MMU_RAM_ADDR >= 0x20000)) )
 			state->m_ram = NULL;
 	}
 }
@@ -903,14 +902,14 @@ READ8_HANDLER( c128_mmu8722_port_r )
 			data &= ~0x10;
 		if (!state->m_exrom)
 			data &= ~0x20;
-		if (input_port_read(space->machine(), "SPECIAL") & 0x10)
+		if (space->machine().root_device().ioport("SPECIAL")->read() & 0x10)
 			data &= ~0x80;
 		break;
 	case 0xb:
 		/* hinybble number of 64 kb memory blocks */
-		if ((input_port_read(space->machine(), "SPECIAL") & 0x06) == 0x02)		// 256KB RAM
+		if ((space->machine().root_device().ioport("SPECIAL")->read() & 0x06) == 0x02)		// 256KB RAM
 			data = 0x4f;
-		else if ((input_port_read(space->machine(), "SPECIAL") & 0x06) == 0x04)	//  1MB
+		else if ((state->ioport("SPECIAL")->read() & 0x06) == 0x04)	//  1MB
 			data = 0xf;
 		else
 			data = 0x2f;
@@ -1030,38 +1029,36 @@ WRITE8_HANDLER( c128_write_ff05 )
  * a15 and a14 portlines
  * 0x1000-0x1fff, 0x9000-0x9fff char rom
  */
-int c128_dma_read(running_machine &machine, int offset)
+READ8_MEMBER( c128_state::vic_dma_read )
 {
-	c128_state *state = machine.driver_data<c128_state>();
-	UINT8 c64_port6510 = m6510_get_port(machine.device<legacy_cpu_device>("m8502"));
+	UINT8 c64_port6510 = m6510_get_port(machine().device<legacy_cpu_device>("m8502"));
 
 	/* main memory configuration to include */
-	if (state->m_c64mode)
+	if (m_c64mode)
 	{
-		if (!state->m_game && state->m_exrom)
+		if (!m_game && m_exrom)
 		{
 			if (offset < 0x3000)
-				return state->m_memory[offset];
-			return state->m_romh[offset & 0x1fff];
+				return m_memory[offset];
+			return m_romh[offset & 0x1fff];
 		}
-		if (((state->m_vicaddr - state->m_memory + offset) & 0x7000) == 0x1000)
-			return state->m_chargen[offset & 0xfff];
-		return state->m_vicaddr[offset];
+		if (((m_vicaddr - m_memory + offset) & 0x7000) == 0x1000)
+			return m_chargen[offset & 0xfff];
+		return m_vicaddr[offset];
 	}
-	if (!(c64_port6510 & 4) && (((state->m_c128_vicaddr - state->m_memory + offset) & 0x7000) == 0x1000))
-		return state->m_c128_chargen[offset & 0xfff];
-	return state->m_c128_vicaddr[offset];
+	if (!(c64_port6510 & 4) && (((m_c128_vicaddr - m_memory + offset) & 0x7000) == 0x1000))
+		return m_c128_chargen[offset & 0xfff];
+	return m_c128_vicaddr[offset];
 }
 
-int c128_dma_read_color(running_machine &machine, int offset)
+READ8_MEMBER( c128_state::vic_dma_read_color )
 {
-	c128_state *state = machine.driver_data<c128_state>();
-	UINT8 c64_port6510 = m6510_get_port(machine.device<legacy_cpu_device>("m8502"));
+	UINT8 c64_port6510 = m6510_get_port(machine().device<legacy_cpu_device>("m8502"));
 
-	if (state->m_c64mode)
-		return state->m_colorram[offset & 0x3ff] & 0xf;
+	if (m_c64mode)
+		return m_colorram[offset & 0x3ff] & 0xf;
 	else
-		return state->m_colorram[(offset & 0x3ff)|((c64_port6510 & 0x3) << 10)] & 0xf;
+		return m_colorram[(offset & 0x3ff)|((c64_port6510 & 0x3) << 10)] & 0xf;
 }
 
 /* 2008-09-01
@@ -1131,7 +1128,7 @@ READ8_DEVICE_HANDLER(c128_m6510_port_read)
 	else
 		data |=  0x10;
 
-	if (input_port_read(device->machine(), "SPECIAL") & 0x20)		/* Check Caps Lock */
+	if (state->ioport("SPECIAL")->read() & 0x20)		/* Check Caps Lock */
 		data &= ~0x40;
 	else
 		data |=  0x40;
@@ -1142,8 +1139,8 @@ READ8_DEVICE_HANDLER(c128_m6510_port_read)
 static void c128_common_driver_init( running_machine &machine )
 {
 	c128_state *state = machine.driver_data<c128_state>();
-	UINT8 *gfx=machine.region("gfx1")->base();
-	UINT8 *ram = machine.region("maincpu")->base();
+	UINT8 *gfx=machine.root_device().memregion("gfx1")->base();
+	UINT8 *ram = state->memregion("maincpu")->base();
 	int i;
 
 	state->m_memory = ram;
@@ -1262,9 +1259,9 @@ INTERRUPT_GEN( c128_frame_interrupt )
 
 	c128_nmi(device->machine());
 
-	if ((input_port_read(device->machine(), "SPECIAL") & 0x08) != state->m_monitor)
+	if ((device->machine().root_device().ioport("SPECIAL")->read() & 0x08) != state->m_monitor)
 	{
-		if (input_port_read(device->machine(), "SPECIAL") & 0x08)
+		if (device->machine().root_device().ioport("SPECIAL")->read() & 0x08)
 		{
 			vic2_set_rastering(vic2e, 0);
 			vdc8563_set_rastering(vdc8563, 1);
@@ -1279,7 +1276,7 @@ INTERRUPT_GEN( c128_frame_interrupt )
 			else
 				device->machine().primary_screen->set_visible_area(0, VIC6567_VISIBLECOLUMNS - 1, 0, VIC6567_VISIBLELINES - 1);
 		}
-		state->m_monitor = input_port_read(device->machine(), "SPECIAL") & 0x08;
+		state->m_monitor = device->machine().root_device().ioport("SPECIAL")->read() & 0x08;
 	}
 
 
@@ -1288,22 +1285,22 @@ INTERRUPT_GEN( c128_frame_interrupt )
 
 	/* Fix Me! Currently, neither left Shift nor Shift Lock work in c128, but reading the correspondent input produces a bug!
     Hence, we overwrite the actual reading as it never happens */
-	if ((input_port_read(device->machine(), "SPECIAL") & 0x40))	//
+	if ((device->machine().root_device().ioport("SPECIAL")->read() & 0x40))	//
 		c64_keyline[1] |= 0x80;
 
 	/* c128 specific: keypad input ports */
 	for (i = 0; i < 3; i++)
 	{
 		value = 0xff;
-		value &= ~input_port_read(device->machine(), c128ports[i]);
+		value &= ~device->machine().root_device().ioport(c128ports[i])->read();
 		state->m_keyline[i] = value;
 	}
 }
 
-SCREEN_UPDATE( c128 )
+SCREEN_UPDATE_IND16( c128 )
 {
-	device_t *vic2e = screen->machine().device("vic2e");
-	device_t *vdc8563 = screen->machine().device("vdc8563");
+	device_t *vic2e = screen.machine().device("vic2e");
+	device_t *vdc8563 = screen.machine().device("vdc8563");
 
 	vdc8563_video_update(vdc8563, bitmap, cliprect);
 	vic2_video_update(vic2e, bitmap, cliprect);
