@@ -48,6 +48,7 @@ enum
 const device_type C1570 = &device_creator<c1570_device>;
 const device_type C1571 = &device_creator<c1571_device>;
 const device_type C1571CR = &device_creator<c1571cr_device>;
+const device_type MINI_CHIEF = &device_creator<mini_chief_device>;
 
 
 //-------------------------------------------------
@@ -72,6 +73,10 @@ void base_c1571_device::device_config_complete()
 	case TYPE_1571CR:
 		m_shortname = "c1571cr";
 		break;
+
+	case TYPE_MINI_CHIEF:
+		m_shortname = "minichif";
+		break;
 	}
 }
 
@@ -109,6 +114,16 @@ ROM_START( c1571cr )
 ROM_END
 
 
+//-------------------------------------------------
+//  ROM( minichief )
+//-------------------------------------------------
+
+ROM_START( minichief )
+	ROM_REGION( 0x8000, M6502_TAG, 0 )
+	ROM_LOAD( "ictdos710.u2", 0x0000, 0x8000, CRC(aaacf7e9) SHA1(c1296995238ef23f18e7fec70a144a0566a25a27) )
+ROM_END
+
+
 //-------------------------------------------------
 //  rom_region - device-specific ROM region
 //-------------------------------------------------
@@ -126,6 +141,9 @@ const rom_entry *base_c1571_device::device_rom_region() const
 
 	case TYPE_1571CR:
 		return ROM_NAME( c1571cr );
+
+	case TYPE_MINI_CHIEF:
+		return ROM_NAME( minichief );
 	}
 }
 
@@ -441,6 +459,15 @@ WRITE_LINE_MEMBER( base_c1571_device::cia_irq_w )
 }
 
 
+WRITE_LINE_MEMBER( base_c1571_device::cia_pc_w )
+{
+	if (m_other != NULL)
+    {
+    	m_other->parallel_strobe_w(state);
+    }
+}
+
+
 WRITE_LINE_MEMBER( base_c1571_device::cia_cnt_w )
 {
 	// fast serial clock out
@@ -457,17 +484,32 @@ WRITE_LINE_MEMBER( base_c1571_device::cia_sp_w )
 }
 
 
+READ8_MEMBER( base_c1571_device::cia_pb_r )
+{
+	return m_parallel_data;
+}
+
+
+WRITE8_MEMBER( base_c1571_device::cia_pb_w )
+{
+	if (m_other != NULL)
+    {
+		m_other->parallel_data_w(data);
+	}
+}
+
+
 static MOS6526_INTERFACE( cia_intf )
 {
 	0,
 	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_irq_w),
-	DEVCB_NULL,
+	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_pc_w),
 	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_cnt_w),
 	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_sp_w),
 	DEVCB_NULL,
 	DEVCB_NULL,
-	DEVCB_NULL,
-	DEVCB_NULL
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_pb_r),
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, base_c1571_device, cia_pb_w)
 };
 
 
@@ -567,7 +609,7 @@ static const floppy_interface c1570_floppy_interface =
 	DEVCB_NULL,
 	FLOPPY_STANDARD_5_25_SSDD,
 	LEGACY_FLOPPY_OPTIONS_NAME(c1570),
-	NULL,
+	"floppy_5_25",
 	NULL
 };
 
@@ -673,6 +715,7 @@ inline void base_c1571_device::set_iec_srq()
 base_c1571_device::base_c1571_device(const machine_config &mconfig, device_type type, const char *name, const char *tag, device_t *owner, UINT32 clock, UINT32 variant)
     : device_t(mconfig, type, name, tag, owner, clock),
 	  device_cbm_iec_interface(mconfig, *this),
+      device_c64_floppy_parallel_interface(mconfig, *this),
 	  m_maincpu(*this, M6502_TAG),
 	  m_via0(*this, M6522_0_TAG),
 	  m_via1(*this, M6522_1_TAG),
@@ -717,6 +760,14 @@ c1571cr_device::c1571cr_device(const machine_config &mconfig, const char *tag, d
 	: base_c1571_device(mconfig, C1571CR, "C1571CR", tag, owner, clock, TYPE_1571CR) { }
 
 
+//-------------------------------------------------
+//  mini_chief_device - constructor
+//-------------------------------------------------
+
+mini_chief_device::mini_chief_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+	: base_c1571_device(mconfig, MINI_CHIEF, "ICT Mini Chief", tag, owner, clock, TYPE_MINI_CHIEF) { }
+
+
 //-------------------------------------------------
 //  device_start - device-specific startup
 //-------------------------------------------------
@@ -761,7 +812,7 @@ void base_c1571_device::cbm_iec_srq(int state)
 {
 	if (!m_ser_dir)
 	{
-		//m_cia->cnt_w(state);
+		m_cia->cnt_w(state);
 	}
 }
 
@@ -787,7 +838,7 @@ void base_c1571_device::cbm_iec_data(int state)
 {
 	if (!m_ser_dir)
 	{
-		//m_cia->sp_w(state);
+		m_cia->sp_w(state);
 	}
 }
 
@@ -805,6 +856,26 @@ void base_c1571_device::cbm_iec_reset(int state)
 }
 
 
+//-------------------------------------------------
+//  parallel_data_w -
+//-------------------------------------------------
+
+void base_c1571_device::parallel_data_w(UINT8 data)
+{
+    m_parallel_data = data;
+}
+
+
+//-------------------------------------------------
+//  parallel_strobe_w -
+//-------------------------------------------------
+
+void base_c1571_device::parallel_strobe_w(int state)
+{
+    m_cia->flag_w(state);
+}
+
+
 //-------------------------------------------------
 //  on_disk_change -
 //-------------------------------------------------
