@@ -9,7 +9,6 @@
 ***************************************************************************/
 
 #include "emu.h"
-#include "image.h"
 #include "audio/ted7360.h"
 #include "cpu/m6502/m6502.h"
 #include "imagedev/cassette.h"
@@ -107,45 +106,45 @@ READ8_DEVICE_HANDLER(c16_m7501_port_read)
 static void c16_bankswitch( running_machine &machine )
 {
 	c16_state *state = machine.driver_data<c16_state>();
-	UINT8 *rom = machine.region("maincpu")->base();
-	memory_set_bankptr(machine, "bank9", ram_get_ptr(state->m_messram));
+	UINT8 *rom = state->memregion("maincpu")->base();
+	state->membank("bank9")->set_base(state->m_messram->pointer());
 
 	switch (state->m_lowrom)
 	{
 	case 0:
-		memory_set_bankptr(machine, "bank2", rom + 0x10000);
+		state->membank("bank2")->set_base(rom + 0x10000);
 		break;
 	case 1:
-		memory_set_bankptr(machine, "bank2", rom + 0x18000);
+		state->membank("bank2")->set_base(rom + 0x18000);
 		break;
 	case 2:
-		memory_set_bankptr(machine, "bank2", rom + 0x20000);
+		state->membank("bank2")->set_base(rom + 0x20000);
 		break;
 	case 3:
-		memory_set_bankptr(machine, "bank2", rom + 0x28000);
+		state->membank("bank2")->set_base(rom + 0x28000);
 		break;
 	}
 
 	switch (state->m_highrom)
 	{
 	case 0:
-		memory_set_bankptr(machine, "bank3", rom + 0x14000);
-		memory_set_bankptr(machine, "bank8", rom + 0x17f20);
+		state->membank("bank3")->set_base(rom + 0x14000);
+		state->membank("bank8")->set_base(rom + 0x17f20);
 		break;
 	case 1:
-		memory_set_bankptr(machine, "bank3", rom + 0x1c000);
-		memory_set_bankptr(machine, "bank8", rom + 0x1ff20);
+		state->membank("bank3")->set_base(rom + 0x1c000);
+		state->membank("bank8")->set_base(rom + 0x1ff20);
 		break;
 	case 2:
-		memory_set_bankptr(machine, "bank3", rom + 0x24000);
-		memory_set_bankptr(machine, "bank8", rom + 0x27f20);
+		state->membank("bank3")->set_base(rom + 0x24000);
+		state->membank("bank8")->set_base(rom + 0x27f20);
 		break;
 	case 3:
-		memory_set_bankptr(machine, "bank3", rom + 0x2c000);
-		memory_set_bankptr(machine, "bank8", rom + 0x2ff20);
+		state->membank("bank3")->set_base(rom + 0x2c000);
+		state->membank("bank8")->set_base(rom + 0x2ff20);
 		break;
 	}
-	memory_set_bankptr(machine, "bank4", rom + 0x17c00);
+	state->membank("bank4")->set_base(rom + 0x17c00);
 }
 
 WRITE8_HANDLER( c16_switch_to_rom )
@@ -182,15 +181,15 @@ WRITE8_HANDLER( c16_select_roms )
 WRITE8_HANDLER( c16_switch_to_ram )
 {
 	c16_state *state = space->machine().driver_data<c16_state>();
-	UINT8 *ram = ram_get_ptr(state->m_messram);
-	UINT32 ram_size = ram_get_size(state->m_messram);
+	UINT8 *ram = state->m_messram->pointer();
+	UINT32 ram_size = state->m_messram->size();
 
 	ted7360_rom_switch_w(state->m_ted7360, 0);
 
-	memory_set_bankptr(space->machine(), "bank2", ram + (0x8000 % ram_size));
-	memory_set_bankptr(space->machine(), "bank3", ram + (0xc000 % ram_size));
-	memory_set_bankptr(space->machine(), "bank4", ram + (0xfc00 % ram_size));
-	memory_set_bankptr(space->machine(), "bank8", ram + (0xff20 % ram_size));
+	state->membank("bank2")->set_base(ram + (0x8000 % ram_size));
+	state->membank("bank3")->set_base(ram + (0xc000 % ram_size));
+	state->membank("bank4")->set_base(ram + (0xfc00 % ram_size));
+	state->membank("bank8")->set_base(ram + (0xff20 % ram_size));
 }
 
 UINT8 c16_read_keyboard( running_machine &machine, int databus )
@@ -338,7 +337,7 @@ READ8_HANDLER( c16_6551_port_r )
 int c16_dma_read( running_machine &machine, int offset )
 {
 	c16_state *state = machine.driver_data<c16_state>();
-	return ram_get_ptr(state->m_messram)[offset % ram_get_size(state->m_messram)];
+	return state->m_messram->pointer()[offset % state->m_messram->size()];
 }
 
 int c16_dma_read_rom( running_machine &machine, int offset )
@@ -379,7 +378,7 @@ int c16_dma_read_rom( running_machine &machine, int offset )
 		}
 	}
 
-	return ram_get_ptr(state->m_messram)[offset % ram_get_size(state->m_messram)];
+	return state->m_messram->pointer()[offset % state->m_messram->size()];
 }
 
 void c16_interrupt( running_machine &machine, int level )
@@ -397,13 +396,13 @@ void c16_interrupt( running_machine &machine, int level )
 static void c16_common_driver_init( running_machine &machine )
 {
 	c16_state *state = machine.driver_data<c16_state>();
-	UINT8 *rom = machine.region("maincpu")->base();
+	UINT8 *rom = state->memregion("maincpu")->base();
 
 	/* initial bankswitch (notice that TED7360 is init to ROM) */
-	memory_set_bankptr(machine, "bank2", rom + 0x10000);
-	memory_set_bankptr(machine, "bank3", rom + 0x14000);
-	memory_set_bankptr(machine, "bank4", rom + 0x17c00);
-	memory_set_bankptr(machine, "bank8", rom + 0x17f20);
+	state->membank("bank2")->set_base(rom + 0x10000);
+	state->membank("bank3")->set_base(rom + 0x14000);
+	state->membank("bank4")->set_base(rom + 0x17c00);
+	state->membank("bank8")->set_base(rom + 0x17f20);
 
 	state->m_mem10000 = rom + 0x10000;
 	state->m_mem14000 = rom + 0x14000;
@@ -455,8 +454,8 @@ MACHINE_RESET( c16 )
 {
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
 	c16_state *state = machine.driver_data<c16_state>();
-	UINT8 *ram = ram_get_ptr(state->m_messram);
-	UINT32 ram_size = ram_get_size(state->m_messram);
+	UINT8 *ram = state->m_messram->pointer();
+	UINT32 ram_size = state->m_messram->size();
 
 	memset(state->m_keyline, 0xff, ARRAY_LENGTH(state->m_keyline));
 
@@ -467,21 +466,21 @@ MACHINE_RESET( c16 )
 
 	if (state->m_pal)
 	{
-		memory_set_bankptr(machine, "bank1", ram + (0x4000 % ram_size));
+		state->membank("bank1")->set_base(ram + (0x4000 % ram_size));
 
-		memory_set_bankptr(machine, "bank5", ram + (0x4000 % ram_size));
-		memory_set_bankptr(machine, "bank6", ram + (0x8000 % ram_size));
-		memory_set_bankptr(machine, "bank7", ram + (0xc000 % ram_size));
+		state->membank("bank5")->set_base(ram + (0x4000 % ram_size));
+		state->membank("bank6")->set_base(ram + (0x8000 % ram_size));
+		state->membank("bank7")->set_base(ram + (0xc000 % ram_size));
 
 		space->install_write_bank(0xff20, 0xff3d,"bank10");
 		space->install_write_bank(0xff40, 0xffff, "bank11");
-		memory_set_bankptr(machine, "bank10", ram + (0xff20 % ram_size));
-		memory_set_bankptr(machine, "bank11", ram + (0xff40 % ram_size));
+		state->membank("bank10")->set_base(ram + (0xff20 % ram_size));
+		state->membank("bank11")->set_base(ram + (0xff40 % ram_size));
 	}
 	else
 	{
 		space->install_write_bank(0x4000, 0xfcff, "bank10");
-		memory_set_bankptr(machine, "bank10", ram + (0x4000 % ram_size));
+		state->membank("bank10")->set_base(ram + (0x4000 % ram_size));
 	}
 }
 
@@ -492,7 +491,7 @@ MACHINE_RESET( c16 )
 static WRITE8_HANDLER( c16_sidcart_16k )
 {
 	c16_state *state = space->machine().driver_data<c16_state>();
-	UINT8 *ram = ram_get_ptr(state->m_messram);
+	UINT8 *ram = state->m_messram->pointer();
 
 	ram[0x1400 + offset] = data;
 	ram[0x5400 + offset] = data;
@@ -506,7 +505,7 @@ static WRITE8_HANDLER( c16_sidcart_64k )
 {
 	c16_state *state = space->machine().driver_data<c16_state>();
 
-	ram_get_ptr(state->m_messram)[0xd400 + offset] = data;
+	state->m_messram->pointer()[0xd400 + offset] = data;
 
 	sid6581_w(state->m_sid, offset, data);
 }
@@ -516,7 +515,7 @@ static TIMER_CALLBACK( c16_sidhack_tick )
 	address_space *space = machine.device("maincpu")->memory().space(AS_PROGRAM);
 	c16_state *state = space->machine().driver_data<c16_state>();
 
-	if (input_port_read_safe(machine, "SID", 0x00) & 0x02)
+	if (machine.root_device().ioport("SID")->read_safe(0x00) & 0x02)
 	{
 		if (state->m_pal)
 			space->install_legacy_write_handler(0xd400, 0xd41f, FUNC(c16_sidcart_16k));
@@ -535,7 +534,7 @@ static TIMER_CALLBACK( c16_sidcard_tick )
 	c16_state *state = machine.driver_data<c16_state>();
 	address_space *space = state->m_maincpu->memory().space(AS_PROGRAM);
 
-	if (input_port_read_safe(machine, "SID", 0x00) & 0x01)
+	if (machine.root_device().ioport("SID")->read_safe(0x00) & 0x01)
 		space->install_legacy_readwrite_handler(*state->m_sid, 0xfe80, 0xfe9f, FUNC(sid6581_r), FUNC(sid6581_w));
 	else
 		space->install_legacy_readwrite_handler(*state->m_sid, 0xfd40, 0xfd5f, FUNC(sid6581_r), FUNC(sid6581_w));
@@ -551,48 +550,48 @@ INTERRUPT_GEN( c16_frame_interrupt )
 	for (i = 0; i < 8; i++)
 	{
 		value = 0xff;
-		value &= ~input_port_read(device->machine(), c16ports[i]);
+		value &= ~device->machine().root_device().ioport(c16ports[i])->read();
 
 		/* Shift Lock is mapped on Left/Right Shift */
-		if ((i == 1) && (input_port_read(device->machine(), "SPECIAL") & 0x80))
+		if ((i == 1) && (device->machine().root_device().ioport("SPECIAL")->read() & 0x80))
 			value &= ~0x80;
 
 		state->m_keyline[i] = value;
 	}
 
-	if (input_port_read(device->machine(), "CTRLSEL") & 0x01)
+	if (device->machine().root_device().ioport("CTRLSEL")->read() & 0x01)
 	{
 		value = 0xff;
-		if (input_port_read(device->machine(), "JOY0") & 0x10)			/* Joypad1_Button */
+		if (device->machine().root_device().ioport("JOY0")->read() & 0x10)			/* Joypad1_Button */
 			{
-				if (input_port_read(device->machine(), "SPECIAL") & 0x40)
+				if (device->machine().root_device().ioport("SPECIAL")->read() & 0x40)
 					value &= ~0x80;
 				else
 					value &= ~0x40;
 			}
 
-		value &= ~(input_port_read(device->machine(), "JOY0") & 0x0f);	/* Other Inputs Joypad1 */
+		value &= ~(device->machine().root_device().ioport("JOY0")->read() & 0x0f);	/* Other Inputs Joypad1 */
 
-		if (input_port_read(device->machine(), "SPECIAL") & 0x40)
+		if (device->machine().root_device().ioport("SPECIAL")->read() & 0x40)
 			state->m_keyline[9] = value;
 		else
 			state->m_keyline[8] = value;
 	}
 
-	if (input_port_read(device->machine(), "CTRLSEL") & 0x10)
+	if (device->machine().root_device().ioport("CTRLSEL")->read() & 0x10)
 	{
 		value = 0xff;
-		if (input_port_read(device->machine(), "JOY1") & 0x10)			/* Joypad2_Button */
+		if (device->machine().root_device().ioport("JOY1")->read() & 0x10)			/* Joypad2_Button */
 			{
-				if (input_port_read(device->machine(), "SPECIAL") & 0x40)
+				if (device->machine().root_device().ioport("SPECIAL")->read() & 0x40)
 					value &= ~0x40;
 				else
 					value &= ~0x80;
 			}
 
-		value &= ~(input_port_read(device->machine(), "JOY1") & 0x0f);	/* Other Inputs Joypad2 */
+		value &= ~(device->machine().root_device().ioport("JOY1")->read() & 0x0f);	/* Other Inputs Joypad2 */
 
-		if (input_port_read(device->machine(), "SPECIAL") & 0x40)
+		if (device->machine().root_device().ioport("SPECIAL")->read() & 0x40)
 			state->m_keyline[8] = value;
 		else
 			state->m_keyline[9] = value;
@@ -610,8 +609,8 @@ INTERRUPT_GEN( c16_frame_interrupt )
 #endif
 	}
 
-	set_led_status(device->machine(), 1, input_port_read(device->machine(), "SPECIAL") & 0x80 ? 1 : 0);		/* Shift Lock */
-	set_led_status(device->machine(), 0, input_port_read(device->machine(), "SPECIAL") & 0x40 ? 1 : 0);		/* Joystick Swap */
+	set_led_status(device->machine(), 1, device->machine().root_device().ioport("SPECIAL")->read() & 0x80 ? 1 : 0);		/* Shift Lock */
+	set_led_status(device->machine(), 0, device->machine().root_device().ioport("SPECIAL")->read() & 0x40 ? 1 : 0);		/* Joystick Swap */
 }
 
 
@@ -623,7 +622,7 @@ INTERRUPT_GEN( c16_frame_interrupt )
 
 static void plus4_software_list_cartridge_load(device_image_interface &image)
 {
-	UINT8 *mem = image.device().machine().region("maincpu")->base();
+	UINT8 *mem = image.device().machine().root_device().memregion("maincpu")->base();
 
 	size_t size = image.get_software_region_length("c1l");
 	if (size)
@@ -644,7 +643,7 @@ static void plus4_software_list_cartridge_load(device_image_interface &image)
 
 static int plus4_crt_load( device_image_interface &image )
 {
-	UINT8 *mem = image.device().machine().region("maincpu")->base();
+	UINT8 *mem = image.device().machine().root_device().memregion("maincpu")->base();
 	int size = image.length(), test;
 	const char *filetype;
 	int address = 0;
