@@ -131,7 +131,7 @@ void isa8_device::device_config_complete()
     	memset(&m_out_drq2_cb, 0, sizeof(m_out_drq2_cb));
     	memset(&m_out_drq3_cb, 0, sizeof(m_out_drq3_cb));
 	}
-	m_maincpu = mconfig().devicelist().find(m_cputag);
+	m_maincpu = mconfig().device(m_cputag);
 }
 
 //**************************************************************************
@@ -145,12 +145,31 @@ void isa8_device::device_config_complete()
 isa8_device::isa8_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) :
         device_t(mconfig, ISA8, "ISA8", tag, owner, clock)
 {
+	for(int i=0;i<8;i++)
+	{
+		m_dma_device[i] = NULL;
+		m_dma_eop[i] = false;
+	}
+	m_nmi_enabled = false;
 }
 
 isa8_device::isa8_device(const machine_config &mconfig, device_type type, const char *name, const char *tag, device_t *owner, UINT32 clock) :
         device_t(mconfig, type, name, tag, owner, clock)
 {
+	for(int i=0;i<8;i++)
+	{
+		m_dma_device[i] = NULL;
+		m_dma_eop[i] = false;
+	}
+	m_nmi_enabled = false;
 }
+
+void isa8_device::set_dma_channel(UINT8 channel, device_isa8_card_interface *dev, bool do_eop)
+{
+	m_dma_device[channel] = dev;
+	m_dma_eop[channel] = do_eop;
+}
+
 //-------------------------------------------------
 //  device_start - device-specific startup
 //-------------------------------------------------
@@ -177,9 +196,62 @@ void isa8_device::device_reset()
 {
 }
 
-void isa8_device::add_isa_card(device_isa8_card_interface *card)
+void isa8_device::install_memory(offs_t start, offs_t end, offs_t mask, offs_t mirror, read8_space_func rhandler, const char* rhandler_name, write8_space_func whandler, const char *whandler_name)
 {
-	m_device_list.append(*card);
+	int buswidth = m_maincpu->memory().space_config(AS_PROGRAM)->m_databus_width;
+	switch(buswidth)
+	{
+		case 8:
+			m_maincpu->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0);
+			break;
+		case 16:
+			m_maincpu->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff);
+			break;
+		case 32:
+			if ((start % 4) == 0) {
+				if ((end-start)==1) {
+					m_maincpu->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(start, end+2, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0x0000ffff);
+				} else {
+					m_maincpu->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(start, end,   mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffffffff);
+				}
+			} else {
+				// we handle just misalligned by 2
+				m_maincpu->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(start-2, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff0000);
+			}
+			break;
+		default:
+			fatalerror("ISA8: Bus width %d not supported", buswidth);
+			break;
+	}
+}
+
+void isa8_device::install_device(offs_t start, offs_t end, offs_t mask, offs_t mirror, read8_space_func rhandler, const char* rhandler_name, write8_space_func whandler, const char *whandler_name)
+{
+	int buswidth = m_maincpu->memory().space_config(AS_PROGRAM)->m_databus_width;
+	switch(buswidth)
+	{
+		case 8:
+			m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0);
+			break;
+		case 16:
+			m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff);
+			break;
+		case 32:
+			if ((start % 4) == 0) {
+				if ((end-start)==1) {
+					m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(start, end+2, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0x0000ffff);
+				} else {
+					m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(start, end,   mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffffffff);
+				}
+			} else {
+				// we handle just misalligned by 2
+				m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(start-2, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff0000);
+			}
+			break;
+		default:
+			fatalerror("ISA8: Bus width %d not supported", buswidth);
+			break;
+	}
 }
 
 void isa8_device::install_device(device_t *dev, offs_t start, offs_t end, offs_t mask, offs_t mirror, read8_device_func rhandler, const char* rhandler_name, write8_device_func whandler, const char *whandler_name)
@@ -194,7 +266,16 @@ void isa8_device::install_device(device_t *dev, offs_t start, offs_t end, offs_t
 			m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(*dev, start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff);
 			break;
 		case 32:
-			m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(*dev, start, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffffffff);
+			if ((start % 4) == 0) {
+				if ((end-start)==1) {
+					m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(*dev, start, end+2, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0x0000ffff);
+				} else {
+					m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(*dev, start, end,   mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffffffff);
+				}
+			} else {
+				// we handle just misalligned by 2
+				m_maincpu->memory().space(AS_IO)->install_legacy_readwrite_handler(*dev, start-2, end, mask, mirror, rhandler, rhandler_name, whandler, whandler_name,0xffff0000);
+			}
 			break;
 		default:
 			fatalerror("ISA8: Bus width %d not supported", buswidth);
@@ -214,7 +295,16 @@ void isa8_device::install_device(offs_t start, offs_t end, offs_t mask, offs_t m
 			m_maincpu->memory().space(AS_IO)->install_readwrite_handler(start, end, mask, mirror, rhandler, whandler, 0xffff);
 			break;
 		case 32:
-			m_maincpu->memory().space(AS_IO)->install_readwrite_handler(start, end, mask, mirror, rhandler, whandler, 0xffffffff);
+			if ((start % 4) == 0) {
+				if ((end-start)==1) {
+					m_maincpu->memory().space(AS_IO)->install_readwrite_handler(start, end+2, mask, mirror, rhandler, whandler, 0x0000ffff);
+				} else {
+					m_maincpu->memory().space(AS_IO)->install_readwrite_handler(start, end,   mask, mirror, rhandler, whandler, 0xffffffff);
+				}
+			} else {
+				// we handle just misalligned by 2
+				m_maincpu->memory().space(AS_IO)->install_readwrite_handler(start-2, end, mask, mirror, rhandler, whandler, 0xffff0000);
+			}
 			break;
 		default:
 			fatalerror("ISA8: Bus width %d not supported", buswidth);
@@ -226,7 +316,7 @@ void isa8_device::install_bank(offs_t start, offs_t end, offs_t mask, offs_t mir
 {
 	address_space *space = m_maincpu->memory().space(AS_PROGRAM);
 	space->install_readwrite_bank(start, end, mask, mirror, tag );
-	memory_set_bankptr(machine(), tag, data);
+	machine().root_device().membank(tag)->set_base(data);
 }
 
 void isa8_device::unmap_bank(offs_t start, offs_t end, offs_t mask, offs_t mirror)
@@ -238,10 +328,16 @@ void isa8_device::unmap_bank(offs_t start, offs_t end, offs_t mask, offs_t mirro
 void isa8_device::install_rom(device_t *dev, offs_t start, offs_t end, offs_t mask, offs_t mirror, const char *tag, const char *region)
 {
 	astring tempstring;
-	address_space *space = m_maincpu->memory().space(AS_PROGRAM);
-	space->install_read_bank(start, end, mask, mirror, tag);
-	space->unmap_write(start, end, mask, mirror);
-	memory_set_bankptr(machine(), tag, machine().region(dev->subtag(tempstring, region))->base());
+	if (machine().root_device().memregion("isa")) {
+		UINT8 *src = dev->memregion(region)->base();
+		UINT8 *dest = machine().root_device().memregion("isa")->base() + start - 0xc0000;
+		memcpy(dest,src, end - start + 1);
+	} else {
+		address_space *space = m_maincpu->memory().space(AS_PROGRAM);
+		space->install_read_bank(start, end, mask, mirror, tag);
+		space->unmap_write(start, end, mask, mirror);
+		machine().root_device().membank(tag)->set_base(machine().root_device().memregion(dev->subtag(tempstring, region))->base());
+	}
 }
 
 void isa8_device::unmap_rom(offs_t start, offs_t end, offs_t mask, offs_t mirror)
@@ -250,6 +346,15 @@ void isa8_device::unmap_rom(offs_t start, offs_t end, offs_t mask, offs_t mirror
 	space->unmap_read(start, end, mask, mirror);
 }
 
+bool isa8_device::is_option_rom_space_available(offs_t start, int size)
+{
+	m_maincpu = machine().device(m_cputag);
+	address_space *space = m_maincpu->memory().space(AS_PROGRAM);
+	for(int i = 0; i < size; i += 4096) // 4KB granularity should be enough
+		if(space->get_read_ptr(start + i)) return false;
+	return true;
+}
+
 // interrupt request from isa card
 WRITE_LINE_MEMBER( isa8_device::irq2_w ) { m_out_irq2_func(state); }
 WRITE_LINE_MEMBER( isa8_device::irq3_w ) { m_out_irq3_func(state); }
@@ -265,39 +370,32 @@ WRITE_LINE_MEMBER( isa8_device::drq3_w ) { m_out_drq3_func(state); }
 
 UINT8 isa8_device::dack_r(int line)
 {
-	UINT8 retVal = 0xff;
-	device_isa8_card_interface *entry = m_device_list.first();
-	while(entry) {
-		if (entry->have_dack(line)) {
-			retVal = entry->dack_r(line);
-			break;
-		}
-		entry = entry->next();
-	}
-	return retVal;
+	if (m_dma_device[line])
+		return m_dma_device[line]->dack_r(line);
+	return 0xff;
 }
 
 void isa8_device::dack_w(int line,UINT8 data)
 {
-	device_isa8_card_interface *entry = m_device_list.first();
-	while(entry) {
-		if (entry->have_dack(line)) {
-			entry->dack_w(line,data);
-		}
-		entry = entry->next();
-	}
+	if (m_dma_device[line])
+		return m_dma_device[line]->dack_w(line,data);
 }
 
 void isa8_device::eop_w(int state)
 {
-	device_isa8_card_interface *entry = m_device_list.first();
-	while(entry) {
-		entry->eop_w(state);
-		entry = entry->next();
+	for (int i=0;i<8;i++) {
+		if (m_dma_eop[i]==TRUE && m_dma_device[i])
+			m_dma_device[i]->eop_w(state);
 	}
 }
 
-
+void isa8_device::nmi()
+{
+	if (m_nmi_enabled)
+	{
+		device_set_input_line( m_maincpu, INPUT_LINE_NMI, PULSE_LINE );
+	}
+}
 //**************************************************************************
 //  DEVICE CONFIG ISA8 CARD INTERFACE
 //**************************************************************************
@@ -338,11 +436,6 @@ void device_isa8_card_interface::eop_w(int state)
 {
 }
 
-bool device_isa8_card_interface::have_dack(int line)
-{
-	return FALSE;
-}
-
 void device_isa8_card_interface::static_set_isabus_tag(device_t &device, const char *tag)
 {
 	device_isa8_card_interface &isa_card = dynamic_cast<device_isa8_card_interface &>(device);
@@ -352,7 +445,6 @@ void device_isa8_card_interface::static_set_isabus_tag(device_t &device, const c
 void device_isa8_card_interface::set_isa_device()
 {
 	m_isa = dynamic_cast<isa8_device *>(device().machine().device(m_isa_tag));
-	m_isa->add_isa_card(this);
 }
 
 
@@ -416,7 +508,7 @@ void isa16_device::device_config_complete()
     	memset(&m_out_drq6_cb, 0, sizeof(m_out_drq6_cb));
     	memset(&m_out_drq7_cb, 0, sizeof(m_out_drq7_cb));
 	}
-	m_maincpu = mconfig().devicelist().find(m_cputag);
+	m_maincpu = mconfig().device(m_cputag);
 }
 
 //-------------------------------------------------
@@ -508,5 +600,4 @@ device_isa16_card_interface::~device_isa16_card_interface()
 void device_isa16_card_interface::set_isa_device()
 {
 	m_isa = dynamic_cast<isa16_device *>(device().machine().device(m_isa_tag));
-	m_isa->add_isa_card(this);
 }
