@@ -22,13 +22,17 @@ public:
 
 		// optional information overrides
 		virtual machine_config_constructor device_mconfig_additions() const;
+		virtual ioport_constructor device_input_ports() const;
+
+		bool is_primary() { return m_is_primary; }
 protected:
         // device-level overrides
         virtual void device_start();
         virtual void device_reset();
-
+		virtual void device_config_complete() { m_shortname = "isa_ide"; }
 private:
         // internal state
+		bool m_is_primary;
 };
 
 
