@@ -86,29 +86,6 @@ QUICKLOAD_LOAD(kc)
 
 // The KC85/4 and KC85/3 are "modular systems". These computers can be expanded with modules.
 
-/*
-    Module ID       Module Name         Module Description
-
-
-                    D001                Basis Device
-                    D002                Bus Driver device
-    a7              D004                Floppy Disk Interface Device
-
-
-    ef              M001                Digital IN/OUT
-    ee              M003                V24
-                    M005                User (empty)
-                    M007                Adapter (empty)
-    e7              M010                ADU1
-    f6              M011                64k RAM
-    fb              M012                Texor
-    f4              M022                Expander RAM (16k)
-    f7              M025                User PROM (8k)
-    fb              M026                Forth
-    fb              M027                Development
-    e3              M029                DAU1
-*/
-
 READ8_MEMBER( kc_state::expansion_read )
 {
 	UINT8 result = 0xff;
@@ -216,22 +193,48 @@ WRITE8_MEMBER( kc_state::expansion_e000_w ){ expansion_write(space, offset + 0xe
     active when the cassette motor is on.
 */
 
+void kc_state::update_cassette(int state)
+{
+	int astb = (state & m_ardy) ? 0 : 1;
 
-#define KC_CASSETTE_TIMER_FREQUENCY attotime::from_hz(4800)
+	// if state is changed updates the /ASTB line
+	if (m_astb ^ astb)
+	{
+		z80pio_astb_w(m_z80pio, astb);
 
-/* this timer is used to update the cassette */
-/* this is the current state of the cassette motor */
-/* ardy output from pio */
+		m_astb = astb;
 
+		// FIXME: temporary for allow kc85_2-3 to load cassette
+		if ((m_cassette->get_state() & 0x03) == CASSETTE_PLAY)
+			z80pio_d_w(m_z80pio, 0, m_pio_data[0]);
+	}
+}
+
+static TIMER_CALLBACK(kc_cassette_oneshot_timer)
+{
+	kc_state *state = machine.driver_data<kc_state>();
+
+	state->update_cassette(0);
+
+	state->m_cassette_oneshot_timer->reset();
+}
+
+// timer used for polling data from cassette input
+// enabled only when cassette motor is on
 static TIMER_CALLBACK(kc_cassette_timer_callback)
 {
 	kc_state *state = machine.driver_data<kc_state>();
 
-	/* the cassette data is linked to /astb input of the pio. */
+	// read cassette data
 	int bit = (state->m_cassette->input() > 0.0038) ? 1 : 0;
 
-	/* update astb with bit */
-	z80pio_astb_w(state->m_z80pio, bit & state->m_ardy);
+	// generates a pulse when the cassette input changes state
+	if (bit ^ state->m_cassette_in)
+	{
+		state->update_cassette(1);
+		state->m_cassette_in = bit;
+		state->m_cassette_oneshot_timer->adjust(attotime::from_double(TIME_OF_74LS123(RES_K(10), CAP_N(1))));
+	}
 }
 
 void kc_state::cassette_set_motor(int motor_state)
@@ -296,7 +299,7 @@ void kc_state::update_0x00000()
 
 		/* yes; set address of bank */
 		space->install_read_bank(0x0000, 0x3fff, "bank1");
-		memory_set_bankptr(machine(), "bank1", m_ram_base);
+		membank("bank1")->set_base(m_ram_base);
 
 		/* write protect ram? */
 		if ((m_pio_data[0] & (1<<3)) == 0)
@@ -342,12 +345,12 @@ void kc_state::update_0x0c000()
 {
 	address_space *space = m_maincpu->memory().space( AS_PROGRAM );
 
-	if ((m_pio_data[0] & (1<<7)) && machine().region("basic")->base() != NULL)
+	if ((m_pio_data[0] & (1<<7)) && memregion("basic")->base() != NULL)
 	{
 		/* BASIC takes next priority */
         	LOG(("BASIC rom 0x0c000\n"));
 
-        memory_set_bankptr(machine(), "bank4", machine().region("basic")->base());
+        membank("bank4")->set_base(machine().root_device().memregion("basic")->base());
 		space->install_read_bank(0xc000, 0xdfff, "bank4");
 		space->unmap_write(0xc000, 0xdfff);
 	}
@@ -370,7 +373,7 @@ void kc_state::update_0x0e000()
 		/* enable CAOS rom in memory range 0x0e000-0x0ffff */
 		LOG(("CAOS rom 0x0e000\n"));
 		/* read will access the rom */
-		memory_set_bankptr(machine(), "bank5", machine().region("caos")->base() + 0x2000);
+		membank("bank5")->set_base(machine().root_device().memregion("caos")->base() + 0x2000);
 		space->install_read_bank(0xe000, 0xffff, "bank5");
 		space->unmap_write(0xe000, 0xffff);
 	}
@@ -394,7 +397,7 @@ void kc_state::update_0x08000()
         /* IRM enabled */
         LOG(("IRM enabled\n"));
 
-		memory_set_bankptr(machine(), "bank3", m_ram_base + 0x4000);
+		membank("bank3")->set_base(m_ram_base + 0x4000);
 		space->install_readwrite_bank(0x8000, 0xbfff, "bank3");
     }
     else
@@ -420,7 +423,7 @@ void kc85_4_state::update_0x04000()
 		/* yes */
 		space->install_read_bank(0x4000, 0x7fff, "bank2");
 		/* set address of bank */
-		memory_set_bankptr(machine(), "bank2", m_ram_base + 0x4000);
+		membank("bank2")->set_base(m_ram_base + 0x4000);
 
 		/* write protect ram? */
 		if ((m_port_86_data & (1<<1)) == 0)
@@ -459,13 +462,30 @@ void kc85_4_state::update_0x0c000()
 		/* CAOS rom takes priority */
 		LOG(("CAOS rom 0x0c000\n"));
 
-		memory_set_bankptr(machine(), "bank4", machine().region("caos")->base());
+		membank("bank4")->set_base(machine().root_device().memregion("caos")->base());
 		space->install_read_bank(0xc000, 0xdfff, "bank4");
 		space->unmap_write(0xc000, 0xdfff);
 	}
 	else
 	{
-		kc_state::update_0x0c000();
+		if (m_pio_data[0] & (1<<7))
+		{
+			/* BASIC takes next priority */
+			LOG(("BASIC rom 0x0c000\n"));
+
+			int bank = memregion("basic")->bytes() == 0x8000 ? (m_port_86_data>>5) & 0x03 : 0;
+
+			membank("bank4")->set_base(machine().root_device().memregion("basic")->base() + (bank << 13));
+			space->install_read_bank(0xc000, 0xdfff, "bank4");
+			space->unmap_write(0xc000, 0xdfff);
+		}
+		else
+		{
+			LOG(("Module at 0x0c000\n"));
+
+			space->install_read_handler (0xc000, 0xdfff, 0, 0, read8_delegate(FUNC(kc_state::expansion_c000_r), this), 0);
+			space->install_write_handler(0xc000, 0xdfff, 0, 0, write8_delegate(FUNC(kc_state::expansion_c000_w), this), 0);
+		}
 	}
 }
 
@@ -480,12 +500,12 @@ void kc85_4_state::update_0x08000()
 
 		UINT8* ram_page = (UINT8*)kc85_4_get_video_ram_base(machine(), (m_port_84_data & 0x04), (m_port_84_data & 0x02));
 
-		memory_set_bankptr(machine(), "bank3", ram_page);
+		membank("bank3")->set_base(ram_page);
 		space->install_readwrite_bank(0x8000, 0xa7ff, "bank3");
 
 		ram_page = (UINT8*)kc85_4_get_video_ram_base(machine(), 0, 0);
 
-		memory_set_bankptr(machine(), "bank6", ram_page + 0x2800);
+		membank("bank6")->set_base(ram_page + 0x2800);
 		space->install_readwrite_bank(0xa800, 0xbfff, "bank6");
 	}
     else if (m_pio_data[1] & (1<<5))
@@ -497,7 +517,7 @@ void kc85_4_state::update_0x08000()
 
 		/* ram8 block chosen */
 
-		if (ram_get_size(m_ram) == 64 * 1024)
+		if (m_ram->size() == 64 * 1024)
 		{
 			// kc85_4 64K RAM
 			ram8_block = ((m_port_84_data)>>4) & 0x01;
@@ -505,21 +525,13 @@ void kc85_4_state::update_0x08000()
 		}
 		else
 		{
-			// kc85_5 224K RAM
-			if ((m_port_84_data & 0x0e) == 0)
-			{
-				ram8_block = ((m_port_84_data)>>4) & 0x01;
-				mem_ptr = m_ram_base + (ram8_block<<14);
-			}
-			else
-			{
-				ram8_block = (((m_port_84_data)>>4) & 0x0f) - 2;
-				mem_ptr = m_ram_base + (ram8_block<<14);
-			}
+			// kc85_5 256K RAM
+			ram8_block = ((m_port_84_data)>>4) & 0x0f;
+			mem_ptr = m_ram_base + (ram8_block<<14);
 		}
 
-		memory_set_bankptr(machine(), "bank3", mem_ptr);
-		memory_set_bankptr(machine(), "bank6", mem_ptr + 0x2800);
+		membank("bank3")->set_base(mem_ptr);
+		membank("bank6")->set_base(mem_ptr + 0x2800);
 		space->install_read_bank(0x8000, 0xa7ff, "bank3");
 		space->install_read_bank(0xa800, 0xbfff, "bank6");
 
@@ -767,6 +779,7 @@ WRITE_LINE_MEMBER( kc_state::keyboard_cb )
 void kc_state::machine_start()
 {
 	m_cassette_timer = machine().scheduler().timer_alloc(FUNC(kc_cassette_timer_callback));
+	m_cassette_oneshot_timer = machine().scheduler().timer_alloc(FUNC(kc_cassette_oneshot_timer));
 
 	// kc85 has a 50 Hz input to the ctc channel 2 and 3
 	// channel 2 this controls the video colour flash
@@ -775,10 +788,10 @@ void kc_state::machine_start()
 	machine().scheduler().timer_pulse(attotime::from_hz(50), FUNC(kc85_50hz_timer_callback), 0, NULL);
 	machine().scheduler().timer_pulse(attotime::from_hz(15625), FUNC(kc85_15khz_timer_callback), 0, NULL);
 
-	m_ram_base = ram_get_ptr(m_ram);
+	m_ram_base = m_ram->pointer();
 
-	m_expansions[0] = machine().device<kcexp_slot_device>("m1");
-	m_expansions[1] = machine().device<kcexp_slot_device>("m2");
+	m_expansions[0] = machine().device<kcexp_slot_device>("m8");
+	m_expansions[1] = machine().device<kcexp_slot_device>("mc");
 	m_expansions[2] = machine().device<kcexp_slot_device>("exp");
 }
 
