@@ -1,6 +1,5 @@
 #include "emu.h"
 #include "crsshair.h"
-#include "hash.h"
 #include "cpu/m6502/m6502.h"
 #include "video/ppu2c0x.h"
 #include "includes/nes.h"
@@ -26,8 +25,8 @@
 
 static void init_nes_core(running_machine &machine);
 static void nes_machine_stop(running_machine &machine);
-static READ8_HANDLER(nes_fds_r);
-static WRITE8_HANDLER(nes_fds_w);
+
+
 static void fds_irq(device_t *device, int scanline, int vblank, int blanked);
 
 /***************************************************************************
@@ -42,11 +41,9 @@ static void init_nes_core( running_machine &machine )
 	int prg_banks = (state->m_prg_chunks == 1) ? (2 * 2) : (state->m_prg_chunks * 2);
 	int i;
 
-	/* We set these here in case they weren't set in the cart loader */
-	state->m_rom = machine.region("maincpu")->base();
-	state->m_vrom = machine.region("gfx1")->base();
-	state->m_vram = machine.region("gfx2")->base();
-	state->m_ciram = machine.region("gfx3")->base();
+	state->m_rom = machine.root_device().memregion("maincpu")->base();
+	state->m_ciram = machine.root_device().memregion("ciram")->base();
+	// other pointers got set in the loading routine
 
 	/* Brutal hack put in as a consequence of the new memory system; we really need to fix the NES code */
 	space->install_readwrite_bank(0x0000, 0x07ff, 0, 0x1800, "bank10");
@@ -54,7 +51,7 @@ static void init_nes_core( running_machine &machine )
 	machine.device("ppu")->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(0, 0x1fff, FUNC(nes_chr_r), FUNC(nes_chr_w));
 	machine.device("ppu")->memory().space(AS_PROGRAM)->install_legacy_readwrite_handler(0x2000, 0x3eff, FUNC(nes_nt_r), FUNC(nes_nt_w));
 
-	memory_set_bankptr(machine, "bank10", state->m_rom);
+	state->membank("bank10")->set_base(state->m_rom);
 
 	/* If there is Disk Expansion and no cart has been loaded, setup memory accordingly */
 	if (state->m_disk_expansion && state->m_pcb_id == NO_BOARD)
@@ -64,22 +61,22 @@ static void init_nes_core( running_machine &machine )
          memory for use in nes_fds_r/nes_fds_w. Same goes for allocation of fds_ram (used for bank2)  */
 		if (state->m_fds_data == NULL)
 		{
-			UINT32 size = machine.region("maincpu")->bytes() - 0x10000;
+			UINT32 size = (state->m_prg_chunks == 1) ? 2 * 0x4000 : state->m_prg_chunks * 0x4000;
 			state->m_fds_data = auto_alloc_array_clear(machine, UINT8, size);
-			memcpy(state->m_fds_data, state->m_rom, size);	// copy in fds_data the cart PRG
+			memcpy(state->m_fds_data, state->m_prg, size);	// copy in fds_data the cart PRG
 		}
 		if (state->m_fds_ram == NULL)
 			state->m_fds_ram = auto_alloc_array(machine, UINT8, 0x8000);
 
-		space->install_legacy_read_handler(0x4030, 0x403f, FUNC(nes_fds_r));
+		space->install_read_handler(0x4030, 0x403f, read8_delegate(FUNC(nes_state::nes_fds_r),state));
 		space->install_read_bank(0x6000, 0xdfff, "bank2");
 		space->install_read_bank(0xe000, 0xffff, "bank1");
 
-		space->install_legacy_write_handler(0x4020, 0x402f, FUNC(nes_fds_w));
+		space->install_write_handler(0x4020, 0x402f, write8_delegate(FUNC(nes_state::nes_fds_w),state));
 		space->install_write_bank(0x6000, 0xdfff, "bank2");
 
-		memory_set_bankptr(machine, "bank1", &state->m_rom[0xe000]);
-		memory_set_bankptr(machine, "bank2", state->m_fds_ram);
+		state->membank("bank1")->set_base(&state->m_rom[0xe000]);
+		state->membank("bank2")->set_base(state->m_fds_ram);
 		return;
 	}
 
@@ -96,18 +93,18 @@ static void init_nes_core( running_machine &machine )
 	/* configure banks 1-4 */
 	for (i = 0; i < 4; i++)
 	{
-		memory_configure_bank(machine, bank_names[i], 0, prg_banks, machine.region("maincpu")->base() + 0x10000, 0x2000);
+		state->membank(bank_names[i])->configure_entries(0, prg_banks, state->m_prg, 0x2000);
 		// some mappers (e.g. MMC5) can map PRG RAM in  0x8000-0xffff as well
 		if (state->m_prg_ram)
-			memory_configure_bank(machine, bank_names[i], prg_banks, state->m_wram_size / 0x2000, state->m_wram, 0x2000);
+			state->membank(bank_names[i])->configure_entries(prg_banks, state->m_wram_size / 0x2000, state->m_wram, 0x2000);
 		// however, at start we point to PRG ROM
-		memory_set_bank(machine, bank_names[i], i);
+		state->membank(bank_names[i])->set_entry(i);
 		state->m_prg_bank[i] = i;
 	}
 
 	/* bank 5 configuration is more delicate, since it can have PRG RAM, PRG ROM or SRAM mapped to it */
 	/* we first map PRG ROM banks, then the battery bank (if a battery is present), and finally PRG RAM (state->m_wram) */
-	memory_configure_bank(machine, "bank5", 0, prg_banks, machine.region("maincpu")->base() + 0x10000, 0x2000);
+	state->membank("bank5")->configure_entries(0, prg_banks, state->m_prg, 0x2000);
 	state->m_battery_bank5_start = prg_banks;
 	state->m_prgram_bank5_start = prg_banks;
 	state->m_empty_bank5_start = prg_banks;
@@ -117,18 +114,18 @@ static void init_nes_core( running_machine &machine )
 	{
 		UINT32 bank_size = (state->m_battery_size > 0x2000) ? 0x2000 : state->m_battery_size;
 		int bank_num = (state->m_battery_size > 0x2000) ? state->m_battery_size / 0x2000 : 1;
-		memory_configure_bank(machine, "bank5", prg_banks, bank_num, state->m_battery_ram, bank_size);
+		state->membank("bank5")->configure_entries(prg_banks, bank_num, state->m_battery_ram, bank_size);
 		state->m_prgram_bank5_start += bank_num;
 		state->m_empty_bank5_start += bank_num;
 	}
 	/* add prg ram. */
 	if (state->m_prg_ram)
 	{
-		memory_configure_bank(machine, "bank5", state->m_prgram_bank5_start, state->m_wram_size / 0x2000, state->m_wram, 0x2000);
+		state->membank("bank5")->configure_entries(state->m_prgram_bank5_start, state->m_wram_size / 0x2000, state->m_wram, 0x2000);
 		state->m_empty_bank5_start += state->m_wram_size / 0x2000;
 	}
 
-	memory_configure_bank(machine, "bank5", state->m_empty_bank5_start, 1, state->m_rom + 0x6000, 0x2000);
+	state->membank("bank5")->configure_entry(state->m_empty_bank5_start, state->m_rom + 0x6000);
 
 	/* if we have any additional PRG RAM, point bank5 to its first bank */
 	if (state->m_battery || state->m_prg_ram)
@@ -136,14 +133,32 @@ static void init_nes_core( running_machine &machine )
 	else
 		state->m_prg_bank[4] = state->m_empty_bank5_start; // or shall we point to "maincpu" region at 0x6000? point is that we should never access this region if no sram or wram is present!
 
-	memory_set_bank(machine, "bank5", state->m_prg_bank[4]);
+	state->membank("bank5")->set_entry(state->m_prg_bank[4]);
 
 	if (state->m_four_screen_vram)
 	{
-		state->m_extended_ntram = auto_alloc_array(machine, UINT8, 0x2000);
+		state->m_extended_ntram = auto_alloc_array_clear(machine, UINT8, 0x2000);
 		state->save_pointer(NAME(state->m_extended_ntram), 0x2000);
 	}
 
+	if (state->m_four_screen_vram)
+		set_nt_mirroring(machine, PPU_MIRROR_4SCREEN);
+	else
+	{
+		switch (state->m_hard_mirroring)
+		{
+			case PPU_MIRROR_HORZ:
+			case PPU_MIRROR_VERT:
+			case PPU_MIRROR_HIGH:
+			case PPU_MIRROR_LOW:
+				set_nt_mirroring(machine, state->m_hard_mirroring);
+				break;
+			default:
+				set_nt_mirroring(machine, PPU_MIRROR_NONE);
+				break;
+		}
+	}
+
 	// there are still some quirk about writes to bank5... I hope to fix them soon. (mappers 34,45,52,246 have both mid_w and WRAM-->check)
 	if (state->m_mmc_write_mid)
 		space->install_legacy_write_handler(0x6000, 0x7fff, state->m_mmc_write_mid,state->m_mmc_write_mid_name);
@@ -193,7 +208,7 @@ MACHINE_RESET( nes )
 
 	/* Reset the mapper variables. Will also mark the char-gen ram as dirty */
 	if (state->m_disk_expansion && state->m_pcb_id == NO_BOARD)
-		ppu2c0x_set_hblank_callback(state->m_ppu, fds_irq);
+		state->m_ppu->set_hblank_callback(fds_irq);
 	else
 		nes_pcb_reset(machine);
 
@@ -213,11 +228,11 @@ static TIMER_CALLBACK( nes_irq_callback )
 
 static void nes_banks_restore(nes_state *state)
 {
-	memory_set_bank(state->machine(), "bank1", state->m_prg_bank[0]);
-	memory_set_bank(state->machine(), "bank2", state->m_prg_bank[1]);
-	memory_set_bank(state->machine(), "bank3", state->m_prg_bank[2]);
-	memory_set_bank(state->machine(), "bank4", state->m_prg_bank[3]);
-	memory_set_bank(state->machine(), "bank5", state->m_prg_bank[4]);
+	state->membank("bank1")->set_entry(state->m_prg_bank[0]);
+	state->membank("bank2")->set_entry(state->m_prg_bank[1]);
+	state->membank("bank3")->set_entry(state->m_prg_bank[2]);
+	state->membank("bank4")->set_entry(state->m_prg_bank[3]);
+	state->membank("bank5")->set_entry(state->m_prg_bank[4]);
 }
 
 static void nes_state_register( running_machine &machine )
@@ -277,14 +292,27 @@ MACHINE_START( nes )
 {
 	nes_state *state = machine.driver_data<nes_state>();
 
+	state->m_ppu = machine.device<ppu2c0x_device>("ppu");
 	init_nes_core(machine);
 	machine.add_notifier(MACHINE_NOTIFY_EXIT, machine_notify_delegate(FUNC(nes_machine_stop),&machine));
 
 	state->m_maincpu = machine.device("maincpu");
-	state->m_ppu = machine.device("ppu");
 	state->m_sound = machine.device("nessound");
 	state->m_cart = machine.device("cart");
 
+	// If we're starting famicom with no disk inserted, we still haven't initialized the VRAM needed for
+	// video emulation, so we need to take care of it now
+	if (!state->m_vram)
+	{
+		state->m_vram = auto_alloc_array(machine, UINT8, 0x4000);
+		for (int i = 0; i < 8; i++)
+		{
+			state->m_chr_map[i].source = CHRRAM;
+			state->m_chr_map[i].origin = i * 0x400; // for save state uses!
+			state->m_chr_map[i].access = &state->m_vram[state->m_chr_map[i].origin];
+		}
+	}
+
 	state->m_irq_timer = machine.scheduler().timer_alloc(FUNC(nes_irq_callback));
 	nes_state_register(machine);
 }
@@ -303,10 +331,9 @@ static void nes_machine_stop( running_machine &machine )
 
 
 
-READ8_HANDLER( nes_IN0_r )
+READ8_MEMBER(nes_state::nes_IN0_r)
 {
-	nes_state *state = space->machine().driver_data<nes_state>();
-	int cfg = input_port_read(space->machine(), "CTRLSEL");
+	int cfg = ioport("CTRLSEL")->read();
 	int ret;
 
 	if ((cfg & 0x000f) >= 0x07)	// for now we treat the FC keyboard separately from other inputs!
@@ -320,20 +347,20 @@ READ8_HANDLER( nes_IN0_r )
 		/* in the unused upper 3 bits, so typically a read from $4016 leaves 0x40 there. */
 		ret = 0x40;
 
-		ret |= ((state->m_in_0.i0 >> state->m_in_0.shift) & 0x01);
+		ret |= ((m_in_0.i0 >> m_in_0.shift) & 0x01);
 
 		/* zapper */
 		if ((cfg & 0x000f) == 0x0002)
 		{
-			int x = state->m_in_0.i1;	/* read Zapper x-position */
-			int y = state->m_in_0.i2;	/* read Zapper y-position */
+			int x = m_in_0.i1;	/* read Zapper x-position */
+			int y = m_in_0.i2;	/* read Zapper y-position */
 			UINT32 pix, color_base;
 
 			/* get the pixel at the gun position */
-			pix = ppu2c0x_get_pixel(state->m_ppu, x, y);
+			pix = m_ppu->get_pixel(x, y);
 
 			/* get the color base from the ppu */
-			color_base = ppu2c0x_get_colorbase(state->m_ppu);
+			color_base = m_ppu->get_colorbase();
 
 			/* look at the screen and see if the cursor is over a bright pixel */
 			if ((pix == color_base + 0x20) || (pix == color_base + 0x30) ||
@@ -345,13 +372,13 @@ READ8_HANDLER( nes_IN0_r )
 				ret |= 0x08;  /* no sprite hit */
 
 			/* If button 1 is pressed, indicate the light gun trigger is pressed */
-			ret |= ((state->m_in_0.i0 & 0x01) << 4);
+			ret |= ((m_in_0.i0 & 0x01) << 4);
 		}
 
 		if (LOG_JOY)
-			logerror("joy 0 read, val: %02x, pc: %04x, bits read: %d, chan0: %08x\n", ret, cpu_get_pc(&space->device()), state->m_in_0.shift, state->m_in_0.i0);
+			logerror("joy 0 read, val: %02x, pc: %04x, bits read: %d, chan0: %08x\n", ret, cpu_get_pc(&space.device()), m_in_0.shift, m_in_0.i0);
 
-		state->m_in_0.shift++;
+		m_in_0.shift++;
 	}
 
 	return ret;
@@ -361,33 +388,32 @@ READ8_HANDLER( nes_IN0_r )
 static UINT8 nes_read_fc_keyboard_line( running_machine &machine, UINT8 scan, UINT8 mode )
 {
 	static const char *const fc_keyport_names[] = { "FCKEY0", "FCKEY1", "FCKEY2", "FCKEY3", "FCKEY4", "FCKEY5", "FCKEY6", "FCKEY7", "FCKEY8" };
-	return ((input_port_read(machine, fc_keyport_names[scan]) >> (mode * 4)) & 0x0f) << 1;
+	return ((machine.root_device().ioport(fc_keyport_names[scan])->read() >> (mode * 4)) & 0x0f) << 1;
 }
 
 static UINT8 nes_read_subor_keyboard_line( running_machine &machine, UINT8 scan, UINT8 mode )
 {
 	static const char *const sub_keyport_names[] = { "SUBKEY0", "SUBKEY1", "SUBKEY2", "SUBKEY3", "SUBKEY4",
 		"SUBKEY5", "SUBKEY6", "SUBKEY7", "SUBKEY8", "SUBKEY9", "SUBKEY10", "SUBKEY11", "SUBKEY12" };
-	return ((input_port_read(machine, sub_keyport_names[scan]) >> (mode * 4)) & 0x0f) << 1;
+	return ((machine.root_device().ioport(sub_keyport_names[scan])->read() >> (mode * 4)) & 0x0f) << 1;
 }
 
-READ8_HANDLER( nes_IN1_r )
+READ8_MEMBER(nes_state::nes_IN1_r)
 {
-	nes_state *state = space->machine().driver_data<nes_state>();
-	int cfg = input_port_read(space->machine(), "CTRLSEL");
+	int cfg = ioport("CTRLSEL")->read();
 	int ret;
 
 	if ((cfg & 0x000f) == 0x07)	// for now we treat the FC keyboard separately from other inputs!
 	{
-		if (state->m_fck_scan < 9)
-			ret = ~nes_read_fc_keyboard_line(space->machine(), state->m_fck_scan, state->m_fck_mode) & 0x1e;
+		if (m_fck_scan < 9)
+			ret = ~nes_read_fc_keyboard_line(machine(), m_fck_scan, m_fck_mode) & 0x1e;
 		else
 			ret = 0x1e;
 	}
 	else if ((cfg & 0x000f) == 0x08)	// for now we treat the Subor keyboard separately from other inputs!
 	{
-		if (state->m_fck_scan < 12)
-			ret = ~nes_read_subor_keyboard_line(space->machine(), state->m_fck_scan, state->m_fck_mode) & 0x1e;
+		if (m_fck_scan < 12)
+			ret = ~nes_read_subor_keyboard_line(machine(), m_fck_scan, m_fck_mode) & 0x1e;
 		else
 			ret = 0x1e;
 	}
@@ -398,20 +424,20 @@ READ8_HANDLER( nes_IN1_r )
 		ret = 0x40;
 
 		/* Handle data line 0's serial output */
-		ret |= ((state->m_in_1.i0 >> state->m_in_1.shift) & 0x01);
+		ret |= ((m_in_1.i0 >> m_in_1.shift) & 0x01);
 
 		/* zapper */
 		if ((cfg & 0x00f0) == 0x0030)
 		{
-			int x = state->m_in_1.i1;	/* read Zapper x-position */
-			int y = state->m_in_1.i2;	/* read Zapper y-position */
+			int x = m_in_1.i1;	/* read Zapper x-position */
+			int y = m_in_1.i2;	/* read Zapper y-position */
 			UINT32 pix, color_base;
 
 			/* get the pixel at the gun position */
-			pix = ppu2c0x_get_pixel(state->m_ppu, x, y);
+			pix = m_ppu->get_pixel(x, y);
 
 			/* get the color base from the ppu */
-			color_base = ppu2c0x_get_colorbase(state->m_ppu);
+			color_base = m_ppu->get_colorbase();
 
 			/* look at the screen and see if the cursor is over a bright pixel */
 			if ((pix == color_base + 0x20) || (pix == color_base + 0x30) ||
@@ -423,25 +449,25 @@ READ8_HANDLER( nes_IN1_r )
 				ret |= 0x08;  /* no sprite hit */
 
 			/* If button 1 is pressed, indicate the light gun trigger is pressed */
-			ret |= ((state->m_in_1.i0 & 0x01) << 4);
+			ret |= ((m_in_1.i0 & 0x01) << 4);
 		}
 
 		/* arkanoid dial */
 		else if ((cfg & 0x00f0) == 0x0040)
 		{
 			/* Handle data line 2's serial output */
-			ret |= ((state->m_in_1.i2 >> state->m_in_1.shift) & 0x01) << 3;
+			ret |= ((m_in_1.i2 >> m_in_1.shift) & 0x01) << 3;
 
 			/* Handle data line 3's serial output - bits are reversed */
 			/* NPW 27-Nov-2007 - there is no third subscript! commenting out */
-			/* ret |= ((state->m_in_1[3] >> state->m_in_1.shift) & 0x01) << 4; */
-			/* ret |= ((state->m_in_1[3] << state->m_in_1.shift) & 0x80) >> 3; */
+			/* ret |= ((m_in_1[3] >> m_in_1.shift) & 0x01) << 4; */
+			/* ret |= ((m_in_1[3] << m_in_1.shift) & 0x80) >> 3; */
 		}
 
 		if (LOG_JOY)
-			logerror("joy 1 read, val: %02x, pc: %04x, bits read: %d, chan0: %08x\n", ret, cpu_get_pc(&space->device()), state->m_in_1.shift, state->m_in_1.i0);
+			logerror("joy 1 read, val: %02x, pc: %04x, bits read: %d, chan0: %08x\n", ret, cpu_get_pc(&space.device()), m_in_1.shift, m_in_1.i0);
 
-		state->m_in_1.shift++;
+		m_in_1.shift++;
 	}
 
 	return ret;
@@ -463,37 +489,37 @@ static void nes_read_input_device( running_machine &machine, int cfg, nes_input
 	{
 		case 0x01:	/* gamepad */
 			if (pad_port >= 0)
-				vals->i0 = input_port_read(machine, padnames[pad_port]);
+				vals->i0 = machine.root_device().ioport(padnames[pad_port])->read();
 			break;
 
 		case 0x02:	/* zapper 1 */
 			if (supports_zapper)
 			{
-				vals->i0 = input_port_read(machine, "ZAPPER1_T");
-				vals->i1 = input_port_read(machine, "ZAPPER1_X");
-				vals->i2 = input_port_read(machine, "ZAPPER1_Y");
+				vals->i0 = machine.root_device().ioport("ZAPPER1_T")->read();
+				vals->i1 = machine.root_device().ioport("ZAPPER1_X")->read();
+				vals->i2 = machine.root_device().ioport("ZAPPER1_Y")->read();
 			}
 			break;
 
 		case 0x03:	/* zapper 2 */
 			if (supports_zapper)
 			{
-				vals->i0 = input_port_read(machine, "ZAPPER2_T");
-				vals->i1 = input_port_read(machine, "ZAPPER2_X");
-				vals->i2 = input_port_read(machine, "ZAPPER2_Y");
+				vals->i0 = machine.root_device().ioport("ZAPPER2_T")->read();
+				vals->i1 = machine.root_device().ioport("ZAPPER2_X")->read();
+				vals->i2 = machine.root_device().ioport("ZAPPER2_Y")->read();
 			}
 			break;
 
 		case 0x04:	/* arkanoid paddle */
 			if (pad_port == 1)
-				vals->i0 = (UINT8) ((UINT8) input_port_read(machine, "PADDLE") + (UINT8)0x52) ^ 0xff;
+				vals->i0 = (UINT8) ((UINT8) machine.root_device().ioport("PADDLE")->read() + (UINT8)0x52) ^ 0xff;
 			break;
 
 		case 0x06:	/* crazy climber controller */
 			if (pad_port == 0)
 			{
-				state->m_in_0.i0 = input_port_read(machine, padnames[4]);
-				state->m_in_1.i0 = input_port_read(machine, padnames[5]);
+				state->m_in_0.i0 = machine.root_device().ioport(padnames[4])->read();
+				state->m_in_1.i0 = machine.root_device().ioport(padnames[5])->read();
 			}
 			break;
 	}
@@ -502,7 +528,7 @@ static void nes_read_input_device( running_machine &machine, int cfg, nes_input
 
 static TIMER_CALLBACK( lightgun_tick )
 {
-	if ((input_port_read(machine, "CTRLSEL") & 0x000f) == 0x0002)
+	if ((machine.root_device().ioport("CTRLSEL")->read() & 0x000f) == 0x0002)
 	{
 		/* enable lightpen crosshair */
 		crosshair_set_screen(machine, 0, CROSSHAIR_SCREEN_ALL);
@@ -513,7 +539,7 @@ static TIMER_CALLBACK( lightgun_tick )
 		crosshair_set_screen(machine, 0, CROSSHAIR_SCREEN_NONE);
 	}
 
-	if ((input_port_read(machine, "CTRLSEL") & 0x00f0) == 0x0030)
+	if ((machine.root_device().ioport("CTRLSEL")->read() & 0x00f0) == 0x0030)
 	{
 		/* enable lightpen crosshair */
 		crosshair_set_screen(machine, 1, CROSSHAIR_SCREEN_ALL);
@@ -525,13 +551,12 @@ static TIMER_CALLBACK( lightgun_tick )
 	}
 }
 
-WRITE8_HANDLER( nes_IN0_w )
+WRITE8_MEMBER(nes_state::nes_IN0_w)
 {
-	nes_state *state = space->machine().driver_data<nes_state>();
-	int cfg = input_port_read(space->machine(), "CTRLSEL");
+	int cfg = ioport("CTRLSEL")->read();
 
 	/* Check if lightgun has been chosen as input: if so, enable crosshair */
-	space->machine().scheduler().timer_set(attotime::zero, FUNC(lightgun_tick));
+	machine().scheduler().timer_set(attotime::zero, FUNC(lightgun_tick));
 
 	if ((cfg & 0x000f) >= 0x07)	// for now we treat the FC keyboard separately from other inputs!
 	{
@@ -542,13 +567,13 @@ WRITE8_HANDLER( nes_IN0_w )
 			int lines = ((cfg & 0x000f) == 0x04) ? 9 : 12;
 			UINT8 out = BIT(data, 1);	// scan
 
-			if (state->m_fck_mode && !out && ++state->m_fck_scan > lines)
-				state->m_fck_scan = 0;
+			if (m_fck_mode && !out && ++m_fck_scan > lines)
+				m_fck_scan = 0;
 
-			state->m_fck_mode = out;	// access lower or upper 4 bits
+			m_fck_mode = out;	// access lower or upper 4 bits
 
 			if (BIT(data, 0))	// reset
-				state->m_fck_scan = 0;
+				m_fck_scan = 0;
 		}
 	}
 	else
@@ -557,38 +582,38 @@ WRITE8_HANDLER( nes_IN0_w )
 			return;
 
 		if (LOG_JOY)
-			logerror("joy 0 bits read: %d\n", state->m_in_0.shift);
+			logerror("joy 0 bits read: %d\n", m_in_0.shift);
 
 		/* Toggling bit 0 high then low resets both controllers */
-		state->m_in_0.shift = 0;
-		state->m_in_1.shift = 0;
+		m_in_0.shift = 0;
+		m_in_1.shift = 0;
 
 		/* Read the input devices */
 		if ((cfg & 0x000f) != 0x06)
 		{
-			nes_read_input_device(space->machine(), cfg >>  0, &state->m_in_0, 0,  TRUE);
-			nes_read_input_device(space->machine(), cfg >>  4, &state->m_in_1, 1,  TRUE);
-			nes_read_input_device(space->machine(), cfg >>  8, &state->m_in_2, 2, FALSE);
-			nes_read_input_device(space->machine(), cfg >> 12, &state->m_in_3, 3, FALSE);
+			nes_read_input_device(machine(), cfg >>  0, &m_in_0, 0,  TRUE);
+			nes_read_input_device(machine(), cfg >>  4, &m_in_1, 1,  TRUE);
+			nes_read_input_device(machine(), cfg >>  8, &m_in_2, 2, FALSE);
+			nes_read_input_device(machine(), cfg >> 12, &m_in_3, 3, FALSE);
 		}
 		else // crazy climber pad
 		{
-			nes_read_input_device(space->machine(), 0, &state->m_in_1, 1,  TRUE);
-			nes_read_input_device(space->machine(), 0, &state->m_in_2, 2, FALSE);
-			nes_read_input_device(space->machine(), 0, &state->m_in_3, 3, FALSE);
-			nes_read_input_device(space->machine(), cfg >>  0, &state->m_in_0, 0,  TRUE);
+			nes_read_input_device(machine(), 0, &m_in_1, 1,  TRUE);
+			nes_read_input_device(machine(), 0, &m_in_2, 2, FALSE);
+			nes_read_input_device(machine(), 0, &m_in_3, 3, FALSE);
+			nes_read_input_device(machine(), cfg >>  0, &m_in_0, 0,  TRUE);
 		}
 
 		if (cfg & 0x0f00)
-			state->m_in_0.i0 |= (state->m_in_2.i0 << 8) | (0x08 << 16);
+			m_in_0.i0 |= (m_in_2.i0 << 8) | (0x08 << 16);
 
 		if (cfg & 0xf000)
-			state->m_in_1.i0 |= (state->m_in_3.i0 << 8) | (0x04 << 16);
+			m_in_1.i0 |= (m_in_3.i0 << 8) | (0x04 << 16);
 	}
 }
 
 
-WRITE8_HANDLER( nes_IN1_w )
+WRITE8_MEMBER(nes_state::nes_IN1_w)
 {
 }
 
@@ -755,21 +780,15 @@ DEVICE_IMAGE_LOAD( nes_cart )
 				logerror("-- TV System: %d\n", extend[4] & 3);
 			}
 
-			/* Free the regions that were allocated by the ROM loader */
-			image.device().machine().region_free("maincpu");
-			image.device().machine().region_free("gfx1");
-
-			/* Allocate them again with the proper size */
+			// Allocate class pointers for PRG/VROM/VRAM/WRAM
 			prg_size = (state->m_prg_chunks == 1) ? 2 * 0x4000 : state->m_prg_chunks * 0x4000;
-			image.device().machine().region_alloc("maincpu", 0x10000 + prg_size, 1, ENDIANNESS_LITTLE);
+			state->m_prg = auto_alloc_array(image.device().machine(), UINT8, prg_size);
 			if (state->m_chr_chunks)
-				image.device().machine().region_alloc("gfx1", state->m_chr_chunks * 0x2000, 1, ENDIANNESS_LITTLE);
+				state->m_vrom = auto_alloc_array(image.device().machine(), UINT8, state->m_chr_chunks * 0x2000);
 
-			state->m_rom = image.device().machine().region("maincpu")->base();
-			state->m_vrom = image.device().machine().region("gfx1")->base();
+			state->m_vram_chunks = 2;
+			state->m_vram = auto_alloc_array(image.device().machine(), UINT8, 0x4000);
 
-			state->m_vram_chunks = image.device().machine().region("gfx2")->bytes() / 0x2000;
-			state->m_vram = image.device().machine().region("gfx2")->base();
 			// FIXME: this should only be allocated if there is actual wram in the cart (i.e. if state->m_prg_ram = 1)!
 			// or if there is a trainer, I think
 			state->m_wram_size = 0x10000;
@@ -914,9 +933,9 @@ DEVICE_IMAGE_LOAD( nes_cart )
 				image.fread(&state->m_wram[0x1000], 0x200);
 
 			/* Read in the program chunks */
-			image.fread(&state->m_rom[0x10000], 0x4000 * state->m_prg_chunks);
+			image.fread(&state->m_prg[0], 0x4000 * state->m_prg_chunks);
 			if (state->m_prg_chunks == 1)
-				memcpy(&state->m_rom[0x14000], &state->m_rom[0x10000], 0x4000);
+				memcpy(&state->m_prg[0x4000], &state->m_prg[0], 0x4000);
 
 #if SPLIT_PRG
 			{
@@ -927,7 +946,7 @@ DEVICE_IMAGE_LOAD( nes_cart )
 				prgout = fopen(outname, "wb");
 				if (prgout)
 				{
-					fwrite(&state->m_rom[0x10000], 1, 0x4000 * state->m_prg_chunks, prgout);
+					fwrite(&state->m_prg[0], 1, 0x4000 * state->m_prg_chunks, prgout);
 					mame_printf_error("Created PRG chunk\n");
 				}
 
@@ -1239,35 +1258,27 @@ DEVICE_IMAGE_LOAD( nes_cart )
 				fatalerror("Unsupported UNIF chunk or corrupted header. Please report the problem at MESS Board.\n");
 			}
 
-			/* Free the regions that were allocated by the ROM loader */
-			image.device().machine().region_free("maincpu");
-			image.device().machine().region_free("gfx1");
-			image.device().machine().region_free("gfx2");
+			// Allocate class pointers for PRG/VROM/VRAM/WRAM and copy data there from the temp copies
 
-			/* Allocate them again, and copy PRG/CHR from temp buffers */
 			/* Take care of PRG */
 			prg_size = (state->m_prg_chunks == 1) ? 2 * 0x4000 : state->m_prg_chunks * 0x4000;
-			image.device().machine().region_alloc("maincpu", 0x10000 + prg_size, 1, ENDIANNESS_LITTLE);
-			state->m_rom = image.device().machine().region("maincpu")->base();
-			memcpy(&state->m_rom[0x10000], &temp_prg[0x00000], state->m_prg_chunks * 0x4000);
+			state->m_prg = auto_alloc_array(image.device().machine(), UINT8, prg_size);
+
+			memcpy(&state->m_prg[0], &temp_prg[0], state->m_prg_chunks * 0x4000);
 			/* If only a single 16K PRG chunk is present, mirror it! */
 			if (state->m_prg_chunks == 1)
-				memcpy(&state->m_rom[0x14000], &state->m_rom[0x10000], 0x4000);
+				memcpy(&state->m_prg[0x4000], &state->m_prg[0], 0x4000);
 
 			/* Take care of CHR ROM */
 			if (state->m_chr_chunks)
 			{
-				image.device().machine().region_alloc("gfx1", state->m_chr_chunks * 0x2000, 1, ENDIANNESS_LITTLE);
-				state->m_vrom = image.device().machine().region("gfx1")->base();
+				state->m_vrom = auto_alloc_array(image.device().machine(), UINT8, state->m_chr_chunks * 0x2000);
 				memcpy(&state->m_vrom[0x00000], &temp_chr[0x00000], state->m_chr_chunks * 0x2000);
 			}
 
 			/* Take care of CHR RAM */
 			if (state->m_vram_chunks)
-			{
-				image.device().machine().region_alloc("gfx2", state->m_vram_chunks * 0x2000, 1, ENDIANNESS_LITTLE);
-				state->m_vram = image.device().machine().region("gfx2")->base();
-			}
+				state->m_vram = auto_alloc_array(image.device().machine(), UINT8, state->m_vram_chunks * 0x2000);
 
 			// FIXME: this should only be allocated if there is actual wram in the cart (i.e. if state->m_prg_ram = 1)!
 			state->m_wram_size = 0x10000;
@@ -1282,7 +1293,7 @@ DEVICE_IMAGE_LOAD( nes_cart )
 				prgout = fopen(outname, "wb");
 				if (prgout)
 				{
-					fwrite(&state->m_rom[0x10000], 1, 0x4000 * state->m_prg_chunks, prgout);
+					fwrite(&state->m_prg[0], 1, 0x4000 * state->m_prg_chunks, prgout);
 					mame_printf_error("Created PRG chunk\n");
 				}
 
@@ -1324,31 +1335,22 @@ DEVICE_IMAGE_LOAD( nes_cart )
 		UINT32 vram_size = image.get_software_region_length("vram");
 		vram_size += image.get_software_region_length("vram2");
 
-		/* Free the regions that were allocated by the ROM loader */
-		image.device().machine().region_free("maincpu");
-		image.device().machine().region_free("gfx1");
-		image.device().machine().region_free("gfx2");
-
-		/* Allocate them again with the proper size */
-		image.device().machine().region_alloc("maincpu", 0x10000 + prg_size, 1, ENDIANNESS_LITTLE);
-
 		// validate the xml fields
 		if (!prg_size)
 			fatalerror("No PRG entry for this software! Please check if the xml list got corrupted");
 		if (prg_size < 0x8000)
 			fatalerror("PRG entry is too small! Please check if the xml list got corrupted");
 
+		// Allocate class pointers for PRG/VROM/VRAM/WRAM and copy data there from the temp copies
+		state->m_prg = auto_alloc_array(image.device().machine(), UINT8, prg_size);
+
 		if (chr_size)
-			image.device().machine().region_alloc("gfx1", chr_size, 1, ENDIANNESS_LITTLE);
+			state->m_vrom = auto_alloc_array(image.device().machine(), UINT8, chr_size);
 
 		if (vram_size)
-			image.device().machine().region_alloc("gfx2", vram_size, 1, ENDIANNESS_LITTLE);
+			state->m_vram = auto_alloc_array(image.device().machine(), UINT8, vram_size);
 
-		state->m_rom = image.device().machine().region("maincpu")->base();
-		state->m_vrom = image.device().machine().region("gfx1")->base();
-		state->m_vram = image.device().machine().region("gfx2")->base();
-
-		memcpy(state->m_rom + 0x10000, image.get_software_region("prg"), prg_size);
+		memcpy(state->m_prg, image.get_software_region("prg"), prg_size);
 
 		if (chr_size)
 			memcpy(state->m_vrom, image.get_software_region("chr"), chr_size);
@@ -1547,44 +1549,49 @@ static void fds_irq( device_t *device, int scanline, int vblank, int blanked )
 	}
 }
 
-static READ8_HANDLER( nes_fds_r )
+READ8_MEMBER(nes_state::nes_fds_r)
 {
-	nes_state *state = space->machine().driver_data<nes_state>();
 	UINT8 ret = 0x00;
 
 	switch (offset)
 	{
 		case 0x00: /* $4030 - disk status 0 */
-			ret = state->m_fds_status0;
+			ret = m_fds_status0;
 			/* clear the disk IRQ detect flag */
-			state->m_fds_status0 &= ~0x01;
+			m_fds_status0 &= ~0x01;
 			break;
 		case 0x01: /* $4031 - data latch */
 			/* don't read data if disk is unloaded */
-			if (state->m_fds_data == NULL)
+			if (m_fds_data == NULL)
 				ret = 0;
-			else if (state->m_fds_current_side)
-				ret = state->m_fds_data[(state->m_fds_current_side - 1) * 65500 + state->m_fds_head_position++];
+			else if (m_fds_current_side)
+			{
+				// a bunch of games (e.g. bshashsc and fairypin) keep reading beyond the last track
+				// what is the expected behavior?
+				if (m_fds_head_position > 65500)
+					m_fds_head_position = 0;
+				ret = m_fds_data[(m_fds_current_side - 1) * 65500 + m_fds_head_position++];
+			}
 			else
 				ret = 0;
 			break;
 		case 0x02: /* $4032 - disk status 1 */
 			/* return "no disk" status if disk is unloaded */
-			if (state->m_fds_data == NULL)
+			if (m_fds_data == NULL)
 				ret = 1;
-			else if (state->m_fds_last_side != state->m_fds_current_side)
+			else if (m_fds_last_side != m_fds_current_side)
 			{
 				/* If we've switched disks, report "no disk" for a few reads */
 				ret = 1;
-				state->m_fds_count++;
-				if (state->m_fds_count == 50)
+				m_fds_count++;
+				if (m_fds_count == 50)
 				{
-					state->m_fds_last_side = state->m_fds_current_side;
-					state->m_fds_count = 0;
+					m_fds_last_side = m_fds_current_side;
+					m_fds_count = 0;
 				}
 			}
 			else
-				ret = (state->m_fds_current_side == 0); /* 0 if a disk is inserted */
+				ret = (m_fds_current_side == 0); /* 0 if a disk is inserted */
 			break;
 		case 0x03: /* $4033 */
 			ret = 0x80;
@@ -1597,21 +1604,20 @@ static READ8_HANDLER( nes_fds_r )
 	return ret;
 }
 
-static WRITE8_HANDLER( nes_fds_w )
+WRITE8_MEMBER(nes_state::nes_fds_w)
 {
-	nes_state *state = space->machine().driver_data<nes_state>();
 
 	switch (offset)
 	{
 		case 0x00:
-			state->m_IRQ_count_latch = (state->m_IRQ_count_latch & 0xff00) | data;
+			m_IRQ_count_latch = (m_IRQ_count_latch & 0xff00) | data;
 			break;
 		case 0x01:
-			state->m_IRQ_count_latch = (state->m_IRQ_count_latch & 0x00ff) | (data << 8);
+			m_IRQ_count_latch = (m_IRQ_count_latch & 0x00ff) | (data << 8);
 			break;
 		case 0x02:
-			state->m_IRQ_count = state->m_IRQ_count_latch;
-			state->m_IRQ_enable = data;
+			m_IRQ_count = m_IRQ_count_latch;
+			m_IRQ_enable = data;
 			break;
 		case 0x03:
 			// d0 = sound io (1 = enable)
@@ -1621,19 +1627,19 @@ static WRITE8_HANDLER( nes_fds_w )
 			/* write data out to disk */
 			break;
 		case 0x05:
-			state->m_fds_motor_on = BIT(data, 0);
+			m_fds_motor_on = BIT(data, 0);
 
 			if (BIT(data, 1))
-				state->m_fds_head_position = 0;
+				m_fds_head_position = 0;
 
-			state->m_fds_read_mode = BIT(data, 2);
-			set_nt_mirroring(space->machine(), BIT(data, 3) ? PPU_MIRROR_HORZ : PPU_MIRROR_VERT);
+			m_fds_read_mode = BIT(data, 2);
+			set_nt_mirroring(machine(), BIT(data, 3) ? PPU_MIRROR_HORZ : PPU_MIRROR_VERT);
 
-			if ((!(data & 0x40)) && (state->m_fds_write_reg & 0x40))
-				state->m_fds_head_position -= 2; // ???
+			if ((!(data & 0x40)) && (m_fds_write_reg & 0x40))
+				m_fds_head_position -= 2; // ???
 
-			state->m_IRQ_enable_latch = BIT(data, 7);
-			state->m_fds_write_reg = data;
+			m_IRQ_enable_latch = BIT(data, 7);
+			m_fds_write_reg = data;
 			break;
 	}
 }
@@ -1670,7 +1676,6 @@ static void nes_unload_proc( device_image_interface &image )
 DRIVER_INIT( famicom )
 {
 	nes_state *state = machine.driver_data<nes_state>();
-	int i;
 
 	/* clear some of the variables we don't use */
 	state->m_trainer = 0;
@@ -1698,15 +1703,6 @@ DRIVER_INIT( famicom )
 	state->m_fds_ram = auto_alloc_array_clear(machine, UINT8, 0x8000);
 	state->save_pointer(NAME(state->m_fds_ram), 0x8000);
 
-	// setup CHR accesses to 8k VRAM
-	state->m_vram = machine.region("gfx2")->base();
-	for (i = 0; i < 8; i++)
-	{
-		state->m_chr_map[i].source = CHRRAM;
-		state->m_chr_map[i].origin = i * 0x400; // for save state uses!
-		state->m_chr_map[i].access = &state->m_vram[state->m_chr_map[i].origin];
-	}
-
 	floppy_install_load_proc(floppy_get_device(machine, 0), nes_load_proc);
 	floppy_install_unload_proc(floppy_get_device(machine, 0), nes_unload_proc);
 }
