@@ -26,7 +26,6 @@
 
 #include "machine/pit8253.h"
 
-#include "machine/pc_mouse.h"
 #include "machine/pckeybrd.h"
 #include "machine/pc_lpt.h"
 #include "machine/pc_fdc.h"
@@ -39,7 +38,6 @@
 
 #include "machine/8237dma.h"
 
-#include "machine/kb_keytro.h"
 #include "machine/ram.h"
 
 #include "memconv.h"
@@ -66,25 +64,24 @@
  *
  *************************************************************************/
 
-READ8_HANDLER(pc_page_r)
+READ8_MEMBER(pc_state::pc_page_r)
 {
 	return 0xFF;
 }
 
 
-WRITE8_HANDLER(pc_page_w)
+WRITE8_MEMBER(pc_state::pc_page_w)
 {
-	pc_state *st = space->machine().driver_data<pc_state>();
 	switch(offset % 4)
 	{
 	case 1:
-		st->m_dma_offset[0][2] = data;
+		m_dma_offset[0][2] = data;
 		break;
 	case 2:
-		st->m_dma_offset[0][3] = data;
+		m_dma_offset[0][3] = data;
 		break;
 	case 3:
-		st->m_dma_offset[0][0] = st->m_dma_offset[0][1] = data;
+		m_dma_offset[0][0] = m_dma_offset[0][1] = data;
 		break;
 	}
 }
@@ -100,25 +97,23 @@ static WRITE_LINE_DEVICE_HANDLER( pc_dma_hrq_changed )
 }
 
 
-static READ8_HANDLER( pc_dma_read_byte )
+READ8_MEMBER(pc_state::pc_dma_read_byte)
 {
 	UINT8 result;
-	pc_state *st = space->machine().driver_data<pc_state>();
-	offs_t page_offset = (((offs_t) st->m_dma_offset[0][st->m_dma_channel]) << 16)
+	offs_t page_offset = (((offs_t) m_dma_offset[0][m_dma_channel]) << 16)
 		& 0x0F0000;
 
-	result = space->read_byte( page_offset + offset);
+	result = space.read_byte( page_offset + offset);
 	return result;
 }
 
 
-static WRITE8_HANDLER( pc_dma_write_byte )
+WRITE8_MEMBER(pc_state::pc_dma_write_byte)
 {
-	pc_state *st = space->machine().driver_data<pc_state>();
-	offs_t page_offset = (((offs_t) st->m_dma_offset[0][st->m_dma_channel]) << 16)
+	offs_t page_offset = (((offs_t) m_dma_offset[0][m_dma_channel]) << 16)
 		& 0x0F0000;
 
-	space->write_byte( page_offset + offset, data);
+	space.write_byte( page_offset + offset, data);
 }
 
 
@@ -172,13 +167,13 @@ static WRITE_LINE_DEVICE_HANDLER( pc_dack3_w ) { set_dma_channel(device, 3, stat
 
 I8237_INTERFACE( ibm5150_dma8237_config )
 {
-	DEVCB_LINE(pc_dma_hrq_changed),
-	DEVCB_LINE(pc_dma8237_out_eop),
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, pc_dma_read_byte),
-	DEVCB_MEMORY_HANDLER("maincpu", PROGRAM, pc_dma_write_byte),
-	{ DEVCB_NULL, DEVCB_NULL, DEVCB_HANDLER(pc_dma8237_fdc_dack_r), DEVCB_HANDLER(pc_dma8237_hdc_dack_r) },
-	{ DEVCB_HANDLER(pc_dma8237_0_dack_w), DEVCB_NULL, DEVCB_HANDLER(pc_dma8237_fdc_dack_w), DEVCB_HANDLER(pc_dma8237_hdc_dack_w) },
-	{ DEVCB_LINE(pc_dack0_w), DEVCB_LINE(pc_dack1_w), DEVCB_LINE(pc_dack2_w), DEVCB_LINE(pc_dack3_w) }
+	DEVCB_DEVICE_LINE("dma8237", pc_dma_hrq_changed),
+	DEVCB_DEVICE_LINE("dma8237", pc_dma8237_out_eop),
+	DEVCB_DRIVER_MEMBER(pc_state, pc_dma_read_byte),
+	DEVCB_DRIVER_MEMBER(pc_state, pc_dma_write_byte),
+	{ DEVCB_NULL, DEVCB_NULL, DEVCB_DEVICE_HANDLER("dma8237", pc_dma8237_fdc_dack_r), DEVCB_DEVICE_HANDLER("dma8237", pc_dma8237_hdc_dack_r) },
+	{ DEVCB_DEVICE_HANDLER("dma8237", pc_dma8237_0_dack_w), DEVCB_NULL, DEVCB_DEVICE_HANDLER("dma8237", pc_dma8237_fdc_dack_w), DEVCB_DEVICE_HANDLER("dma8237", pc_dma8237_hdc_dack_w) },
+	{ DEVCB_DEVICE_LINE("dma8237", pc_dack0_w), DEVCB_DEVICE_LINE("dma8237", pc_dack1_w), DEVCB_DEVICE_LINE("dma8237", pc_dack2_w), DEVCB_DEVICE_LINE("dma8237", pc_dack3_w) }
 };
 
 
@@ -357,54 +352,73 @@ static WRITE_LINE_DEVICE_HANDLER( pc_com_interrupt_2 )
 	pic8259_ir3_w(st->m_pic8259, state);
 }
 
-/* called when com registers read/written - used to update peripherals that
-are connected */
-static void pc_com_refresh_connected_common(device_t *device, int n, int data)
-{
-	/* mouse connected to this port? */
-	//if (input_port_read(device->machine(), "DSW2") & (0x80>>n))
-//      pc_mouse_handshake_in(device,data);
-}
-
-static INS8250_HANDSHAKE_OUT( pc_com_handshake_out_0 ) { pc_com_refresh_connected_common( device, 0, data ); }
-static INS8250_HANDSHAKE_OUT( pc_com_handshake_out_1 ) { pc_com_refresh_connected_common( device, 1, data ); }
-static INS8250_HANDSHAKE_OUT( pc_com_handshake_out_2 ) { pc_com_refresh_connected_common( device, 2, data ); }
-static INS8250_HANDSHAKE_OUT( pc_com_handshake_out_3 ) { pc_com_refresh_connected_common( device, 3, data ); }
-
-/* PC interface to PC-com hardware. Done this way because PCW16 also
-uses PC-com hardware and doesn't have the same setup! */
 const ins8250_interface ibm5150_com_interface[4]=
 {
 	{
-		1843200,
+		DEVCB_DEVICE_LINE_MEMBER("serport0", serial_port_device, tx),
+		DEVCB_DEVICE_LINE_MEMBER("serport0", rs232_port_device, dtr_w),
+		DEVCB_DEVICE_LINE_MEMBER("serport0", rs232_port_device, rts_w),
 		DEVCB_LINE(pc_com_interrupt_1),
-		NULL,
-		pc_com_handshake_out_0,
-		NULL
+		DEVCB_NULL,
+		DEVCB_NULL
 	},
 	{
-		1843200,
+		DEVCB_DEVICE_LINE_MEMBER("serport1", serial_port_device, tx),
+		DEVCB_DEVICE_LINE_MEMBER("serport1", rs232_port_device, dtr_w),
+		DEVCB_DEVICE_LINE_MEMBER("serport1", rs232_port_device, rts_w),
 		DEVCB_LINE(pc_com_interrupt_2),
-		NULL,
-		pc_com_handshake_out_1,
-		NULL
+		DEVCB_NULL,
+		DEVCB_NULL
 	},
 	{
-		1843200,
+		DEVCB_DEVICE_LINE_MEMBER("serport2", serial_port_device, tx),
+		DEVCB_DEVICE_LINE_MEMBER("serport2", rs232_port_device, dtr_w),
+		DEVCB_DEVICE_LINE_MEMBER("serport2", rs232_port_device, rts_w),
 		DEVCB_LINE(pc_com_interrupt_1),
-		NULL,
-		pc_com_handshake_out_2,
-		NULL
+		DEVCB_NULL,
+		DEVCB_NULL
 	},
 	{
-		1843200,
+		DEVCB_DEVICE_LINE_MEMBER("serport3", serial_port_device, tx),
+		DEVCB_DEVICE_LINE_MEMBER("serport3", rs232_port_device, dtr_w),
+		DEVCB_DEVICE_LINE_MEMBER("serport3", rs232_port_device, rts_w),
 		DEVCB_LINE(pc_com_interrupt_2),
-		NULL,
-		pc_com_handshake_out_3,
-		NULL
+		DEVCB_NULL,
+		DEVCB_NULL
 	}
 };
 
+const rs232_port_interface ibm5150_serport_config[4] =
+{
+	{
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_0", ins8250_uart_device, rx_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_0", ins8250_uart_device, dcd_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_0", ins8250_uart_device, dsr_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_0", ins8250_uart_device, ri_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_0", ins8250_uart_device, cts_w)
+	},
+	{
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_1", ins8250_uart_device, rx_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_1", ins8250_uart_device, dcd_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_1", ins8250_uart_device, dsr_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_1", ins8250_uart_device, ri_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_1", ins8250_uart_device, cts_w)
+	},
+	{
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_2", ins8250_uart_device, rx_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_2", ins8250_uart_device, dcd_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_2", ins8250_uart_device, dsr_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_2", ins8250_uart_device, ri_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_2", ins8250_uart_device, cts_w)
+	},
+	{
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_3", ins8250_uart_device, rx_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_3", ins8250_uart_device, dcd_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_3", ins8250_uart_device, dsr_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_3", ins8250_uart_device, ri_w),
+		DEVCB_DEVICE_LINE_MEMBER("ins8250_3", ins8250_uart_device, cts_w)
+	}
+};
 
 /**********************************************************
  *
@@ -414,9 +428,9 @@ const ins8250_interface ibm5150_com_interface[4]=
 
 static UINT8	nmi_enabled;
 
-WRITE8_HANDLER( pc_nmi_enable_w )
+WRITE8_MEMBER(pc_state::pc_nmi_enable_w)
 {
-	logerror( "%08X: changing NMI state to %s\n", cpu_get_pc(&space->device()), data & 0x80 ? "enabled" : "disabled" );
+	logerror( "%08X: changing NMI state to %s\n", cpu_get_pc(&space.device()), data & 0x80 ? "enabled" : "disabled" );
 
 	nmi_enabled = data & 0x80;
 }
@@ -468,7 +482,7 @@ static struct {
 } pcjr_keyb;
 
 
-READ8_HANDLER( pcjr_nmi_enable_r )
+READ8_MEMBER(pc_state::pcjr_nmi_enable_r)
 {
 	pcjr_keyb.latch = 0;
 
@@ -603,51 +617,41 @@ static void pcjr_keyb_init(running_machine &machine)
  *
  **********************************************************/
 
-
-WRITE8_HANDLER( ibm5150_kb_set_clock_signal )
+WRITE_LINE_MEMBER( pc_state::keyboard_clock_w )
 {
-	pc_state *st = space->machine().driver_data<pc_state>();
-	device_t *keyboard = space->machine().device("keyboard");
-
-	if ( st->m_ppi_clock_signal != data )
+	if ( m_ppi_clock_signal != state )
 	{
-		if ( st->m_ppi_keyb_clock && st->m_ppi_shift_enable )
+		if ( m_ppi_keyb_clock && m_ppi_shift_enable )
 		{
-			st->m_ppi_clock_signal = data;
-			if ( ! st->m_ppi_keyboard_clear )
+			m_ppi_clock_signal = state;
+			if ( ! m_ppi_keyboard_clear )
 			{
 				/* Data is clocked in on a high->low transition */
-				if ( ! data )
+				if ( ! state )
 				{
-					UINT8	trigger_irq = st->m_ppi_shift_register & 0x01;
+					UINT8   trigger_irq = m_ppi_shift_register & 0x01;
 
-					st->m_ppi_shift_register = ( st->m_ppi_shift_register >> 1 ) | ( st->m_ppi_data_signal << 7 );
+					m_ppi_shift_register = ( m_ppi_shift_register >> 1 ) | ( m_ppi_data_signal << 7 );
 					if ( trigger_irq )
 					{
-						pic8259_ir1_w(st->m_pic8259, 1);
-						st->m_ppi_shift_enable = 0;
-						st->m_ppi_clock_signal = 0;
-						kb_keytronic_clock_w(keyboard, st->m_ppi_clock_signal);
+						pic8259_ir1_w(m_pic8259, 1);
+						m_ppi_shift_enable = 0;
+						m_ppi_clock_signal = 0;
+						m_pc_kbdc->clock_write_from_mb(m_ppi_clock_signal);
 					}
 				}
 			}
 		}
 	}
-
-	kb_keytronic_clock_w(keyboard, st->m_ppi_clock_signal);
 }
 
 
-WRITE8_HANDLER( ibm5150_kb_set_data_signal )
+WRITE_LINE_MEMBER( pc_state::keyboard_data_w )
 {
-	pc_state *st = space->machine().driver_data<pc_state>();
-	device_t *keyboard = space->machine().device("keyboard");
-
-	st->m_ppi_data_signal = data;
-
-	kb_keytronic_data_w(keyboard, st->m_ppi_data_signal);
+	m_ppi_data_signal = state;
 }
 
+
 static READ8_DEVICE_HANDLER (ibm5160_ppi_porta_r)
 {
 	int data = 0xFF;
@@ -666,7 +670,7 @@ static READ8_DEVICE_HANDLER (ibm5160_ppi_porta_r)
          *      01 - color 40x25
          * 6-7  The number of floppy disk drives
          */
-		data = input_port_read(device->machine(), "DSW0");
+		data = device->machine().root_device().ioport("DSW0")->read();
 	}
 	else
 	{
@@ -691,13 +695,13 @@ static READ8_DEVICE_HANDLER ( ibm5160_ppi_portc_r )
 	if (st->m_ppi_portc_switch_high)
 	{
 		/* read hi nibble of S2 */
-		data = (data & 0xf0) | ((input_port_read(device->machine(), "DSW0") >> 4) & 0x0f);
+		data = (data & 0xf0) | ((device->machine().root_device().ioport("DSW0")->read() >> 4) & 0x0f);
 		PIO_LOG(1,"PIO_C_r (hi)",("$%02x\n", data));
 	}
 	else
 	{
 		/* read lo nibble of S2 */
-		data = (data & 0xf0) | (input_port_read(device->machine(), "DSW0") & 0x0f);
+		data = (data & 0xf0) | (device->machine().root_device().ioport("DSW0")->read() & 0x0f);
 		PIO_LOG(1,"PIO_C_r (lo)",("$%02x\n", data));
 	}
 
@@ -714,7 +718,6 @@ static READ8_DEVICE_HANDLER ( ibm5160_ppi_portc_r )
 static WRITE8_DEVICE_HANDLER( ibm5160_ppi_portb_w )
 {
 	pc_state *st = device->machine().driver_data<pc_state>();
-	device_t *keyboard = device->machine().device("keyboard");
 
 	/* PPI controller port B*/
 	st->m_ppi_portb = data;
@@ -725,7 +728,7 @@ static WRITE8_DEVICE_HANDLER( ibm5160_ppi_portb_w )
 	pc_speaker_set_spkrdata( device->machine(), data & 0x02 );
 
 	st->m_ppi_clock_signal = ( st->m_ppi_keyb_clock ) ? 1 : 0;
-	kb_keytronic_clock_w(keyboard, st->m_ppi_clock_signal);
+	st->m_pc_kbdc->clock_write_from_mb(st->m_ppi_clock_signal);
 
 	/* If PB7 is set clear the shift register and reset the IRQ line */
 	if ( st->m_ppi_keyboard_clear )
@@ -766,7 +769,7 @@ static READ8_DEVICE_HANDLER (pc_ppi_porta_r)
          *      01 - color 40x25
          * 6-7  The number of floppy disk drives
          */
-		data = input_port_read(device->machine(), "DSW0");
+		data = device->machine().root_device().ioport("DSW0")->read();
 	}
 	else
 	{
@@ -851,7 +854,7 @@ static READ8_DEVICE_HANDLER ( pcjr_ppi_portc_r )
 
 	data&=~0x80;
 	data &= ~0x04;		/* floppy drive installed */
-	if ( ram_get_size(device->machine().device(RAM_TAG)) > 64 * 1024 )	/* more than 64KB ram installed */
+	if ( device->machine().device<ram_device>(RAM_TAG)->size() > 64 * 1024 )	/* more than 64KB ram installed */
 		data &= ~0x08;
 	data = ( data & ~0x01 ) | ( pcjr_keyb.latch ? 0x01: 0x00 );
 	if ( ! ( st->m_ppi_portb & 0x08 ) )
@@ -967,15 +970,13 @@ static void pc_set_keyb_int(running_machine &machine, int state)
 
 void mess_init_pc_common(running_machine &machine, UINT32 flags, void (*set_keyb_int_func)(running_machine &, int), void (*set_hdc_int_func)(running_machine &,int,int))
 {
+	pc_state *state = machine.driver_data<pc_state>();
 	if ( set_keyb_int_func != NULL )
 		init_pc_common(machine, flags, set_keyb_int_func);
 
 	/* MESS managed RAM */
-	if ( ram_get_ptr(machine.device(RAM_TAG)) )
-		memory_set_bankptr( machine, "bank10", ram_get_ptr(machine.device(RAM_TAG)) );
-
-	/* serial mouse */
-	//pc_mouse_initialise(machine);
+	if ( machine.device<ram_device>(RAM_TAG)->pointer() )
+		state->membank( "bank10" )->set_base( machine.device<ram_device>(RAM_TAG)->pointer() );
 }
 
 
@@ -1006,8 +1007,8 @@ DRIVER_INIT( pcmda )
 
 DRIVER_INIT( europc )
 {
-	UINT8 *gfx = &machine.region("gfx1")->base()[0x8000];
-	UINT8 *rom = &machine.region("maincpu")->base()[0];
+	UINT8 *gfx = &machine.root_device().memregion("gfx1")->base()[0x8000];
+	UINT8 *rom = &machine.root_device().memregion("maincpu")->base()[0];
 	int i;
 
     /* just a plain bit pattern for graphics data generation */
@@ -1039,7 +1040,7 @@ DRIVER_INIT( t1000hx )
 
 DRIVER_INIT( pc200 )
 {
-	UINT8 *gfx = &machine.region("gfx1")->base()[0x8000];
+	UINT8 *gfx = &machine.root_device().memregion("gfx1")->base()[0x8000];
 	int i;
 
     /* just a plain bit pattern for graphics data generation */
@@ -1051,7 +1052,7 @@ DRIVER_INIT( pc200 )
 
 DRIVER_INIT( ppc512 )
 {
-	UINT8 *gfx = &machine.region("gfx1")->base()[0x8000];
+	UINT8 *gfx = &machine.root_device().memregion("gfx1")->base()[0x8000];
 	int i;
 
     /* just a plain bit pattern for graphics data generation */
@@ -1062,7 +1063,7 @@ DRIVER_INIT( ppc512 )
 }
 DRIVER_INIT( pc1512 )
 {
-	UINT8 *gfx = &machine.region("gfx1")->base()[0x8000];
+	UINT8 *gfx = &machine.root_device().memregion("gfx1")->base()[0x8000];
 	int i;
 
     /* just a plain bit pattern for graphics data generation */
@@ -1078,20 +1079,7 @@ DRIVER_INIT( pcjr )
 	mess_init_pc_common(machine, PCCOMMON_KEYBOARD_PC, pcjr_set_keyb_int, pc_set_irq_line);
 }
 
-static READ8_HANDLER( input_port_0_r ) { return input_port_read(space->machine(), "IN0"); }
-
-static const struct pc_vga_interface vga_interface =
-{
-	NULL,
-	NULL,
-
-	input_port_0_r,
-
-	AS_IO,
-	0x0000
-};
-
-
+static READ8_HANDLER( input_port_0_r ) { return space->machine().root_device().ioport("IN0")->read(); }
 
 DRIVER_INIT( pc1640 )
 {
@@ -1107,7 +1095,8 @@ DRIVER_INIT( pc_vga )
 {
 	mess_init_pc_common(machine, PCCOMMON_KEYBOARD_PC, pc_set_keyb_int, pc_set_irq_line);
 
-	pc_vga_init(machine, &vga_interface, NULL);
+	pc_vga_init(machine, input_port_0_r, NULL);
+	pc_vga_io_init(machine, machine.device("maincpu")->memory().space(AS_PROGRAM), 0xa0000, machine.device("maincpu")->memory().space(AS_IO), 0x0000);
 }
 
 static IRQ_CALLBACK(pc_irq_callback)
@@ -1151,7 +1140,6 @@ MACHINE_RESET( pc )
 	st->m_ppi_shift_register = 0;
 	st->m_ppi_shift_enable = 0;
 
-	//pc_mouse_set_serial_port( machine.device("ins8250_0") );
 	speaker_level_w( speaker, 0 );
 }
 
@@ -1190,7 +1178,6 @@ MACHINE_RESET( pcjr )
 	st->m_ppi_data_signal = 0;
 	st->m_ppi_shift_register = 0;
 	st->m_ppi_shift_enable = 0;
-	//pc_mouse_set_serial_port( machine.device("ins8250_0") );
 	speaker_level_w( speaker, 0 );
 
 	pcjr_keyb_init(machine);
@@ -1202,7 +1189,7 @@ DEVICE_IMAGE_LOAD( pcjr_cartridge )
 	UINT32	address;
 	UINT32	size;
 
-	address = ( ! strcmp( "cart2", image.device().tag() ) ) ? 0xd0000 : 0xe0000;
+	address = ( ! strcmp( ":cart2", image.device().tag() ) ) ? 0xd0000 : 0xe0000;
 
 	if ( image.software_entry() )
 	{
@@ -1210,7 +1197,7 @@ DEVICE_IMAGE_LOAD( pcjr_cartridge )
 
 		size = image.get_software_region_length("rom" );
 
-		memcpy( image.device().machine().region("maincpu")->base() + address, cart, size );
+		memcpy( image.device().machine().root_device().memregion("maincpu")->base() + address, cart, size );
 	}
 	else
 	{
@@ -1239,7 +1226,7 @@ DEVICE_IMAGE_LOAD( pcjr_cartridge )
 		}
 
 		/* Read the cartridge contents */
-		if ( ( image_size - 0x200 ) != image.fread(image.device().machine().region("maincpu")->base() + address, image_size - 0x200 ) )
+		if ( ( image_size - 0x200 ) != image.fread(image.device().machine().root_device().memregion("maincpu")->base() + address, image_size - 0x200 ) )
 		{
 			image.seterror(IMAGE_ERROR_UNSUPPORTED, "Unable to read cartridge contents" );
 			return IMAGE_INIT_FAIL;
@@ -1256,25 +1243,33 @@ DEVICE_IMAGE_LOAD( pcjr_cartridge )
  *
  **************************************************************************/
 
-INTERRUPT_GEN( pc_frame_interrupt )
+TIMER_DEVICE_CALLBACK( pc_frame_interrupt )
 {
-	pc_keyboard();
+	int scanline = param;
+
+	if((scanline % 64) == 0)
+		pc_keyboard();
 }
 
-INTERRUPT_GEN( pc_vga_frame_interrupt )
+TIMER_DEVICE_CALLBACK( pc_vga_frame_interrupt )
 {
-	//vga_timer();
-	pc_keyboard();
-}
+	int scanline = param;
 
-INTERRUPT_GEN( pcjr_frame_interrupt )
-{
-	if ( pcjr_keyb.transferring == 0 )
+	if((scanline % 64) == 0)
 	{
+		//vga_timer();
 		pc_keyboard();
 	}
 }
 
+TIMER_DEVICE_CALLBACK( pcjr_frame_interrupt )
+{
+	int scanline = param;
+
+	if((scanline % 64) == 0 &&  pcjr_keyb.transferring == 0 )
+		pc_keyboard();
+}
+
 
 #define VERBOSE_DBG 0       /* general debug messages */
 
@@ -1381,7 +1376,7 @@ void pc_rtc_init(running_machine &machine)
 	pc_rtc.timer->adjust(attotime::zero, 0, attotime(1,0));
 }
 
-READ8_HANDLER( pc_rtc_r )
+READ8_MEMBER(pc_state::pc_rtc_r)
 {
 	int data;
 	switch (offset) {
@@ -1392,7 +1387,7 @@ READ8_HANDLER( pc_rtc_r )
 	return data;
 }
 
-WRITE8_HANDLER( pc_rtc_w )
+WRITE8_MEMBER(pc_state::pc_rtc_w)
 {
 	logerror( "rtc write %.2x %.2x\n", offset, data);
 	switch(offset) {
@@ -1401,9 +1396,6 @@ WRITE8_HANDLER( pc_rtc_w )
 	}
 }
 
-READ16_HANDLER( pc16le_rtc_r ) { return read16le_with_read8_handler(pc_rtc_r, space, offset, mem_mask); }
-WRITE16_HANDLER( pc16le_rtc_w ) { write16le_with_write8_handler(pc_rtc_w, space, offset, data, mem_mask); }
-
 /*************************************************************************
  *
  *      EXP
@@ -1423,11 +1415,9 @@ static struct {
 	UINT8 reg[8];
 } pc_expansion={ { 0,0,0,0,0,0,1 } };
 
-WRITE8_HANDLER ( pc_EXP_w )
+WRITE8_MEMBER(pc_state::pc_EXP_w)
 {
-	running_machine &machine = space->machine();
-
-	DBG_LOG(1,"EXP_unit_w",("%.2x $%02x\n", offset, data));
+	//DBG_LOG(1,"EXP_unit_w",("%.2x $%02x\n", offset, data));
 	switch (offset) {
 	case 4:
 		pc_expansion.reg[4]=pc_expansion.reg[5]=pc_expansion.reg[6]=data;
@@ -1437,12 +1427,10 @@ WRITE8_HANDLER ( pc_EXP_w )
 	}
 }
 
-READ8_HANDLER ( pc_EXP_r )
+READ8_MEMBER(pc_state::pc_EXP_r)
 {
     int data;
 	UINT16 a;
-	running_machine &machine = space->machine();
-
 	switch (offset) {
 	case 6:
 		data = pc_expansion.reg[offset];
@@ -1454,7 +1442,7 @@ READ8_HANDLER ( pc_EXP_r )
 	default:
 		data = pc_expansion.reg[offset];
 	}
-    DBG_LOG(1,"EXP_unit_r",("%.2x $%02x\n", offset, data));
+    //DBG_LOG(1,"EXP_unit_r",("%.2x $%02x\n", offset, data));
 	return data;
 }
 
