@@ -175,8 +175,8 @@ static void pokemini_update_sound( running_machine &machine )
 	}
 	else
 	{
-		static const int levels[4] = { 0, 1, 1, 2 };
-		int level = levels[ state->m_pm_reg[0x71] & 0x03 ];
+		///static const int levels[4] = { 0, 1, 1, 2 };
+		int level; /// silence clang warning/// = levels[ state->m_pm_reg[0x71] & 0x03 ];
 
 //      if ( ( ( state->m_pm_reg[0x48] & 0x80 ) && ( state->m_pm_reg[0x4E] | ( state->m_pm_reg[0x4F] << 8 ) ) > ( state->m_pm_reg[0x4C] | ( state->m_pm_reg[0x4D] << 8 ) ) )
 //        || ( ( state->m_pm_reg[0x48] & 0x80 ) && state->m_pm_reg[0x4F] > state->m_pm_reg[0x4D] ) )
@@ -1354,7 +1354,7 @@ READ8_DEVICE_HANDLER( pokemini_hwreg_r )
 
 	switch( offset )
 	{
-	case 0x52:	return input_port_read(device->machine(), "INPUTS");
+	case 0x52:	return state->ioport("INPUTS")->read();
 	case 0x61:
 		if ( ! ( state->m_pm_reg[0x60] & 0x04 ) )
 		{
@@ -1396,7 +1396,7 @@ DEVICE_IMAGE_LOAD( pokemini_cart )
 		image.fseek(0x2100, SEEK_SET);
 		size -= 0x2100;
 
-		if (size != image.fread( image.device().machine().region("maincpu")->base() + 0x2100, size))
+		if (size != image.fread( image.device().machine().root_device().memregion("maincpu")->base() + 0x2100, size))
 		{
 			image.seterror(IMAGE_ERROR_UNSPECIFIED, "Error occurred while reading ROM image");
 			return IMAGE_INIT_FAIL;
@@ -1406,7 +1406,7 @@ DEVICE_IMAGE_LOAD( pokemini_cart )
 	{
 		UINT8 *cart_rom = image.get_software_region("rom");
 		UINT32 cart_rom_size = image.get_software_region_length("rom");
-		memcpy(image.device().machine().region("maincpu")->base() + 0x2100, cart_rom + 0x2100, cart_rom_size - 0x2100);
+		memcpy(image.device().machine().root_device().memregion("maincpu")->base() + 0x2100, cart_rom + 0x2100, cart_rom_size - 0x2100);
 	}
 
 	return IMAGE_INIT_PASS;
@@ -1533,14 +1533,14 @@ static TIMER_CALLBACK( pokemini_prc_counter_callback )
 					for( x = 0; x < 96; x++ ) {
 						UINT8 data = state->m_p_ram[ ( y * 12 ) + x ];
 
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 0, x) = ( data & 0x01 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 1, x) = ( data & 0x02 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 2, x) = ( data & 0x04 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 3, x) = ( data & 0x08 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 4, x) = ( data & 0x10 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 5, x) = ( data & 0x20 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 6, x) = ( data & 0x40 ) ? 3 : 0;
-						*BITMAP_ADDR16(machine.generic.tmpbitmap, y + 7, x) = ( data & 0x80 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 0, x) = ( data & 0x01 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 1, x) = ( data & 0x02 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 2, x) = ( data & 0x04 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 3, x) = ( data & 0x08 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 4, x) = ( data & 0x10 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 5, x) = ( data & 0x20 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 6, x) = ( data & 0x40 ) ? 3 : 0;
+						state->m_bitmap.pix16(y + 7, x) = ( data & 0x80 ) ? 3 : 0;
 					}
 				}
 
@@ -1551,7 +1551,7 @@ static TIMER_CALLBACK( pokemini_prc_counter_callback )
 		}
 
 		/* Set possible input irqs */
-		state->m_pm_reg[0x29] |= ~ input_port_read( machine, "INPUTS" );
+		state->m_pm_reg[0x29] |= ~ machine.root_device().ioport( "INPUTS" )->read();
 	}
 }
 
