@@ -42,28 +42,30 @@ static WRITE_LINE_DEVICE_HANDLER( svi318_ins8250_interrupt )
 		cputag_set_input_line(device->machine(), "maincpu", 0, (state ? HOLD_LINE : CLEAR_LINE));
 	}
 }
-
+#if 0
 static INS8250_REFRESH_CONNECT( svi318_com_refresh_connected )
 {
 	/* Motorola MC14412 modem */
 	ins8250_handshake_in(device, UART8250_HANDSHAKE_IN_CTS|UART8250_HANDSHAKE_IN_DSR|UART8250_INPUTS_RING_INDICATOR|UART8250_INPUTS_DATA_CARRIER_DETECT);
 }
-
+#endif
 const ins8250_interface svi318_ins8250_interface[2]=
 {
 	{
-		1000000,
+		DEVCB_NULL,
+		DEVCB_NULL,
+		DEVCB_NULL,
 		DEVCB_LINE(svi318_ins8250_interrupt),
-		NULL,
-		NULL,
-		svi318_com_refresh_connected
+		DEVCB_NULL,
+		DEVCB_NULL
 	},
 	{
-		3072000,
+		DEVCB_NULL,
+		DEVCB_NULL,
+		DEVCB_NULL,
 		DEVCB_LINE(svi318_ins8250_interrupt),
-		NULL,
-		NULL,
-		NULL
+		DEVCB_NULL,
+		DEVCB_NULL
 	}
 };
 
@@ -89,7 +91,7 @@ DEVICE_START( svi318_cart )
 DEVICE_IMAGE_LOAD( svi318_cart )
 {
 	svi318_state *state = image.device().machine().driver_data<svi318_state>();
-	UINT8 *p = image.device().machine().region("user1")->base();
+	UINT8 *p = state->memregion("user1")->base();
 	UINT32 size;
 
 	if (image.software_entry() == NULL)
@@ -150,7 +152,7 @@ static READ8_DEVICE_HANDLER ( svi318_ppi_port_a_r )
 		data |= 0x80;
 	if (!svi318_cassette_present(device->machine(), 0))
 		data |= 0x40;
-	data |= input_port_read(device->machine(), "BUTTONS") & 0x30;
+	data |= device->machine().root_device().ioport("BUTTONS")->read() & 0x30;
 
 	return data;
 }
@@ -179,7 +181,7 @@ static READ8_DEVICE_HANDLER ( svi318_ppi_port_b_r )
 
 	row = state->m_svi.keyboard_row;
 	if (row <= 10)
-		return input_port_read(device->machine(), keynames[row]);
+		return device->machine().root_device().ioport(keynames[row])->read();
 
 	return 0xff;
 }
@@ -232,10 +234,10 @@ I8255_INTERFACE( svi318_ppi8255_interface )
 	DEVCB_HANDLER(svi318_ppi_port_c_w)
 };
 
-WRITE8_HANDLER( svi318_ppi_w )
+WRITE8_MEMBER(svi318_state::svi318_ppi_w)
 {
-	i8255_device *ppi = space->machine().device<i8255_device>("ppi8255");
-	ppi->write(*space, offset + 2, data);
+	i8255_device *ppi = machine().device<i8255_device>("ppi8255");
+	ppi->write(space, offset + 2, data);
 }
 
 
@@ -254,9 +256,9 @@ WRITE8_HANDLER( svi318_ppi_w )
   8  RIGHT2 Joystick 2, Right
 */
 
-READ8_HANDLER( svi318_psg_port_a_r )
+READ8_MEMBER(svi318_state::svi318_psg_port_a_r)
 {
-	return input_port_read(space->machine(), "JOYSTICKS");
+	return ioport("JOYSTICKS")->read();
 }
 
 /*
@@ -275,14 +277,13 @@ READ8_HANDLER( svi318_psg_port_a_r )
  with RAM are disabled.
 */
 
-WRITE8_HANDLER( svi318_psg_port_b_w )
+WRITE8_MEMBER(svi318_state::svi318_psg_port_b_w)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	if ( (state->m_svi.bank_switch ^ data) & 0x20)
-		set_led_status (space->machine(), 0, !(data & 0x20) );
+	if ( (m_svi.bank_switch ^ data) & 0x20)
+		set_led_status (machine(), 0, !(data & 0x20) );
 
-	state->m_svi.bank_switch = data;
-	svi318_set_banks(space->machine());
+	m_svi.bank_switch = data;
+	svi318_set_banks(machine());
 }
 
 /* Disk drives  */
@@ -307,38 +308,36 @@ const wd17xx_interface svi_wd17xx_interface =
 	{FLOPPY_0, FLOPPY_1, NULL, NULL}
 };
 
-static WRITE8_HANDLER( svi318_fdc_drive_motor_w )
+WRITE8_MEMBER(svi318_state::svi318_fdc_drive_motor_w)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	device_t *fdc = space->machine().device("wd179x");
+	device_t *fdc = machine().device("wd179x");
 	switch (data & 3)
 	{
 	case 1:
 		wd17xx_set_drive(fdc,0);
-		state->m_fdc.driveselect = 0;
+		m_fdc.driveselect = 0;
 		break;
 	case 2:
 		wd17xx_set_drive(fdc,1);
-		state->m_fdc.driveselect = 1;
+		m_fdc.driveselect = 1;
 		break;
 	}
 }
 
-static WRITE8_HANDLER( svi318_fdc_density_side_w )
+WRITE8_MEMBER(svi318_state::svi318_fdc_density_side_w)
 {
-	device_t *fdc = space->machine().device("wd179x");
+	device_t *fdc = machine().device("wd179x");
 
 	wd17xx_dden_w(fdc, BIT(data, 0));
 	wd17xx_set_side(fdc, BIT(data, 1));
 }
 
-static READ8_HANDLER( svi318_fdc_irqdrq_r )
+READ8_MEMBER(svi318_state::svi318_fdc_irqdrq_r)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
 	UINT8 result = 0;
 
-	result |= state->m_fdc.drq << 6;
-	result |= state->m_fdc.irq << 7;
+	result |= m_fdc.drq << 6;
+	result |= m_fdc.irq << 7;
 
 	return result;
 }
@@ -346,6 +345,7 @@ static READ8_HANDLER( svi318_fdc_irqdrq_r )
 MC6845_UPDATE_ROW( svi806_crtc6845_update_row )
 {
 	svi318_state *state = device->machine().driver_data<svi318_state>();
+	const rgb_t *palette = palette_entry_list_raw(bitmap.palette());
 	int i;
 
 	for( i = 0; i < x_count; i++ )
@@ -360,7 +360,7 @@ MC6845_UPDATE_ROW( svi806_crtc6845_update_row )
 
 		for( j=0; j < 8; j++ )
 		{
-			*BITMAP_ADDR16(bitmap, y, i * 8 + j ) = TMS9928A_PALETTE_SIZE + ( ( data & 0x80 ) ? 1 : 0 );
+			bitmap.pix32(y, i * 8 + j ) = palette[TMS9928A_PALETTE_SIZE + ( ( data & 0x80 ) ? 1 : 0 )];
 			data = data << 1;
 		}
 	}
@@ -373,38 +373,23 @@ static void svi318_80col_init(running_machine &machine)
 	svi318_state *state = machine.driver_data<svi318_state>();
 	/* 2K RAM, but allocating 4KB to make banking easier */
 	/* The upper 2KB will be set to FFs and will never be written to */
-	state->m_svi.svi806_ram = machine.region_alloc("gfx2", 0x1000, 1, ENDIANNESS_LITTLE );
+	state->m_svi.svi806_ram = machine.memory().region_alloc("gfx2", 0x1000, 1, ENDIANNESS_LITTLE );
 	memset( state->m_svi.svi806_ram->base(), 0x00, 0x800 );
 	memset( state->m_svi.svi806_ram->base() + 0x800, 0xFF, 0x800 );
-	state->m_svi.svi806_gfx = machine.region("gfx1")->base();
+	state->m_svi.svi806_gfx = state->memregion("gfx1")->base();
 }
 
 
-static WRITE8_HANDLER( svi806_ram_enable_w )
+WRITE8_MEMBER(svi318_state::svi806_ram_enable_w)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	state->m_svi.svi806_ram_enabled = ( data & 0x01 );
-	svi318_set_banks(space->machine());
+	m_svi.svi806_ram_enabled = ( data & 0x01 );
+	svi318_set_banks(machine());
 }
 
 VIDEO_START( svi328_806 )
 {
 }
 
-SCREEN_UPDATE( svi328_806 )
-{
-	if (!strcmp(screen->tag(), "svi806"))
-	{
-		mc6845_device *mc6845 = screen->machine().device<mc6845_device>("crtc");
-		mc6845->update(bitmap, cliprect);
-	}
-	else
-	{
-		fatalerror("Unknown screen '%s'", screen->tag());
-	}
-	return 0;
-}
-
 MACHINE_RESET( svi328_806 )
 {
 	svi318_state *state = machine.driver_data<svi318_state>();
@@ -608,49 +593,45 @@ MACHINE_RESET( svi318 )
 
 /* Memory */
 
-WRITE8_HANDLER( svi318_writemem1 )
+WRITE8_MEMBER(svi318_state::svi318_writemem1)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	if ( state->m_svi.bankLow_read_only )
+	if ( m_svi.bankLow_read_only )
 		return;
 
-	state->m_svi.bankLow_ptr[offset] = data;
+	m_svi.bankLow_ptr[offset] = data;
 }
 
-WRITE8_HANDLER( svi318_writemem2 )
+WRITE8_MEMBER(svi318_state::svi318_writemem2)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	if ( state->m_svi.bankHigh1_read_only)
+	if ( m_svi.bankHigh1_read_only)
 		return;
 
-	state->m_svi.bankHigh1_ptr[offset] = data;
+	m_svi.bankHigh1_ptr[offset] = data;
 }
 
-WRITE8_HANDLER( svi318_writemem3 )
+WRITE8_MEMBER(svi318_state::svi318_writemem3)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	if ( state->m_svi.bankHigh2_read_only)
+	if ( m_svi.bankHigh2_read_only)
 		return;
 
-	state->m_svi.bankHigh2_ptr[offset] = data;
+	m_svi.bankHigh2_ptr[offset] = data;
 }
 
-WRITE8_HANDLER( svi318_writemem4 )
+WRITE8_MEMBER(svi318_state::svi318_writemem4)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
-	if ( state->m_svi.svi806_ram_enabled )
+	if ( m_svi.svi806_ram_enabled )
 	{
 		if ( offset < 0x800 )
 		{
-			state->m_svi.svi806_ram->u8(offset) = data;
+			m_svi.svi806_ram->u8(offset) = data;
 		}
 	}
 	else
 	{
-		if ( state->m_svi.bankHigh2_read_only )
+		if ( m_svi.bankHigh2_read_only )
 			return;
 
-		state->m_svi.bankHigh2_ptr[ 0x3000 + offset] = data;
+		m_svi.bankHigh2_ptr[ 0x3000 + offset] = data;
 	}
 }
 
@@ -658,8 +639,8 @@ static void svi318_set_banks(running_machine &machine)
 {
 	svi318_state *state = machine.driver_data<svi318_state>();
 	const UINT8 v = state->m_svi.bank_switch;
-	UINT8 *ram = ram_get_ptr(machine.device(RAM_TAG));
-	UINT32 ram_size = ram_get_size(machine.device(RAM_TAG));
+	UINT8 *ram = machine.device<ram_device>(RAM_TAG)->pointer();
+	UINT32 ram_size = machine.device<ram_device>(RAM_TAG)->size();
 
 	state->m_svi.bankLow = ( v & 1 ) ? ( ( v & 2 ) ? ( ( v & 8 ) ? SVI_INTERNAL : SVI_EXPRAM3 ) : SVI_EXPRAM2 ) : SVI_CART;
 	state->m_svi.bankHigh1 = ( v & 4 ) ? ( ( v & 16 ) ? SVI_INTERNAL : SVI_EXPRAM3 ) : SVI_EXPRAM2;
@@ -670,7 +651,7 @@ static void svi318_set_banks(running_machine &machine)
 	switch( state->m_svi.bankLow )
 	{
 	case SVI_INTERNAL:
-		state->m_svi.bankLow_ptr = machine.region("maincpu")->base();
+		state->m_svi.bankLow_ptr = state->memregion("maincpu")->base();
 		break;
 	case SVI_CART:
 		if ( state->m_pcart )
@@ -750,20 +731,20 @@ static void svi318_set_banks(running_machine &machine)
 		}
 	}
 
-	memory_set_bankptr(machine, "bank1", state->m_svi.bankLow_ptr );
-	memory_set_bankptr(machine, "bank2", state->m_svi.bankHigh1_ptr );
-	memory_set_bankptr(machine, "bank3", state->m_svi.bankHigh2_ptr );
+	state->membank("bank1")->set_base(state->m_svi.bankLow_ptr );
+	state->membank("bank2")->set_base(state->m_svi.bankHigh1_ptr );
+	state->membank("bank3")->set_base(state->m_svi.bankHigh2_ptr );
 
 	/* SVI-806 80 column card specific banking */
 	if ( state->m_svi.svi806_present )
 	{
 		if ( state->m_svi.svi806_ram_enabled )
 		{
-			memory_set_bankptr(machine, "bank4", state->m_svi.svi806_ram );
+			state->membank("bank4")->set_base(state->m_svi.svi806_ram );
 		}
 		else
 		{
-			memory_set_bankptr(machine, "bank4", state->m_svi.bankHigh2_ptr + 0x3000 );
+			state->membank("bank4")->set_base(state->m_svi.bankHigh2_ptr + 0x3000 );
 		}
 	}
 }
@@ -781,13 +762,14 @@ int svi318_cassette_present(running_machine &machine, int id)
 
 /* External I/O */
 
-READ8_HANDLER( svi318_io_ext_r )
+READ8_MEMBER(svi318_state::svi318_io_ext_r)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
 	UINT8 data = 0xff;
 	device_t *device;
+	ins8250_device *uart;
+	centronics_device *centronics = machine().device<centronics_device>("centronics");
 
-	if (state->m_svi.bankLow == SVI_CART)
+	if (m_svi.bankLow == SVI_CART)
 	{
 		return 0xff;
 	}
@@ -795,8 +777,7 @@ READ8_HANDLER( svi318_io_ext_r )
 	switch( offset )
 	{
 	case 0x12:
-		device = space->machine().device("centronics");
-		data = 0xfe | centronics_busy_r(device);
+		data = 0xfe | centronics->busy_r();
 		break;
 
 	case 0x20:
@@ -807,8 +788,8 @@ READ8_HANDLER( svi318_io_ext_r )
 	case 0x25:
 	case 0x26:
 	case 0x27:
-		device = space->machine().device("ins8250_0");
-		data = ins8250_r(device, offset & 7);
+		uart = machine().device<ins8250_device>("ins8250_0");
+		data = uart->ins8250_r(space, offset & 7);
 		break;
 
 	case 0x28:
@@ -819,44 +800,45 @@ READ8_HANDLER( svi318_io_ext_r )
 	case 0x2D:
 	case 0x2E:
 	case 0x2F:
-		device = space->machine().device("ins8250_1");
-		data = ins8250_r(device, offset & 7);
+		uart = machine().device<ins8250_device>("ins8250_1");
+		data = uart->ins8250_r(space, offset & 7);
 		break;
 
 	case 0x30:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		data = wd17xx_status_r(device, 0);
 		break;
 	case 0x31:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		data = wd17xx_track_r(device, 0);
 		break;
 	case 0x32:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		data = wd17xx_sector_r(device, 0);
 		break;
 	case 0x33:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		data = wd17xx_data_r(device, 0);
 		break;
 	case 0x34:
 		data = svi318_fdc_irqdrq_r(space, 0);
 		break;
 	case 0x51:
-		device = space->machine().device("crtc");
-		data = downcast<mc6845_device *>(device)->register_r( *space, offset );
+		device = machine().device("crtc");
+		data = downcast<mc6845_device *>(device)->register_r( space, offset );
 		break;
 	}
 
 	return data;
 }
 
-WRITE8_HANDLER( svi318_io_ext_w )
+WRITE8_MEMBER(svi318_state::svi318_io_ext_w)
 {
-	svi318_state *state = space->machine().driver_data<svi318_state>();
 	device_t *device;
+	ins8250_device *uart;
+	centronics_device *centronics = machine().device<centronics_device>("centronics");
 
-	if (state->m_svi.bankLow == SVI_CART)
+	if (m_svi.bankLow == SVI_CART)
 	{
 		return;
 	}
@@ -864,13 +846,11 @@ WRITE8_HANDLER( svi318_io_ext_w )
 	switch( offset )
 	{
 	case 0x10:
-		device = space->machine().device("centronics");
-		centronics_data_w(device, 0, data);
+		centronics->write(space, 0, data);
 		break;
 
 	case 0x11:
-		device = space->machine().device("centronics");
-		centronics_strobe_w(device, BIT(data, 0));
+		centronics->strobe_w(BIT(data, 0));
 		break;
 
 	case 0x20:
@@ -881,8 +861,8 @@ WRITE8_HANDLER( svi318_io_ext_w )
 	case 0x25:
 	case 0x26:
 	case 0x27:
-		device = space->machine().device("ins8250_0");
-		ins8250_w(device, offset & 7, data);
+		uart = machine().device<ins8250_device>("ins8250_0");
+		uart->ins8250_w(space, offset & 7, data);
 		break;
 
 	case 0x28:
@@ -893,24 +873,24 @@ WRITE8_HANDLER( svi318_io_ext_w )
 	case 0x2D:
 	case 0x2E:
 	case 0x2F:
-		device = space->machine().device("ins8250_1");
-		ins8250_w(device, offset & 7, data);
+		uart = machine().device<ins8250_device>("ins8250_1");
+		uart->ins8250_w(space, offset & 7, data);
 		break;
 
 	case 0x30:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		wd17xx_command_w(device, 0, data);
 		break;
 	case 0x31:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		wd17xx_track_w(device, 0, data);
 		break;
 	case 0x32:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		wd17xx_sector_w(device, 0, data);
 		break;
 	case 0x33:
-		device = space->machine().device("wd179x");
+		device = machine().device("wd179x");
 		wd17xx_data_w(device, 0, data);
 		break;
 	case 0x34:
@@ -921,12 +901,12 @@ WRITE8_HANDLER( svi318_io_ext_w )
 		break;
 
 	case 0x50:
-		device = space->machine().device("crtc");
-		downcast<mc6845_device *>(device)->address_w(*space, offset, data);
+		device = machine().device("crtc");
+		downcast<mc6845_device *>(device)->address_w(space, offset, data);
 		break;
 	case 0x51:
-		device = space->machine().device("crtc");
-		downcast<mc6845_device *>(device)->register_w(*space, offset, data);
+		device = machine().device("crtc");
+		downcast<mc6845_device *>(device)->register_w(space, offset, data);
 		break;
 
 	case 0x58:
