@@ -9,7 +9,6 @@
 #include <stdarg.h>
 #include "emu.h"
 #include "cpu/z80/z80.h"
-#include "machine/ti85_ser.h"
 #include "includes/ti85.h"
 
 #define TI85_SNAPSHOT_SIZE	 32976
@@ -19,7 +18,7 @@
 static TIMER_CALLBACK(ti85_timer_callback)
 {
 	ti85_state *state = machine.driver_data<ti85_state>();
-	if (input_port_read(machine, "ON") & 0x01)
+	if (machine.root_device().ioport("ON")->read() & 0x01)
 	{
 		if (state->m_ON_interrupt_mask && !state->m_ON_pressed)
 		{
@@ -39,10 +38,23 @@ static TIMER_CALLBACK(ti85_timer_callback)
 	}
 }
 
+inline void ti8x_update_bank(address_space *space, UINT8 bank, UINT8 *base, UINT8 page, bool is_ram)
+{
+	ti85_state *state = space->machine().driver_data<ti85_state>();
+	static const char *const tag[] = {"bank1", "bank2", "bank3", "bank4"};
+
+	state->membank(tag[bank&3])->set_base(base + (0x4000 * page));
+
+	if (is_ram)
+		space->install_write_bank(bank * 0x4000, bank * 0x4000 + 0x3fff, tag[bank&3]);
+	else
+		space->nop_write(bank * 0x4000, bank * 0x4000 + 0x3fff);
+}
+
 static void update_ti85_memory (running_machine &machine)
 {
 	ti85_state *state = machine.driver_data<ti85_state>();
-	memory_set_bankptr(machine, "bank2",machine.region("maincpu")->base() + 0x010000 + 0x004000*state->m_ti8x_memory_page_1);
+	state->membank("bank2")->set_base(state->m_bios + 0x004000*state->m_ti8x_memory_page_1);
 }
 
 static void update_ti83p_memory (running_machine &machine)
@@ -52,58 +64,20 @@ static void update_ti83p_memory (running_machine &machine)
 
 	if (state->m_ti8x_memory_page_1 & 0x40)
 	{
-		if (state->m_ti83p_port4 & 1)
-		{
-
-			memory_set_bankptr(machine, "bank3", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_1&0x01));
-			space->install_write_bank(0x8000, 0xbfff, "bank3");
-		}
-		else
-		{
-			memory_set_bankptr(machine, "bank2", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_1&0x01));
-			space->install_write_bank(0x4000, 0x7fff, "bank2");
-		}
+		ti8x_update_bank(space, (state->m_ti83p_port4 & 1) ? 2 : 1, state->m_ti8x_ram, state->m_ti8x_memory_page_1 & 0x01, true);
 	}
 	else
 	{
-		if (state->m_ti83p_port4 & 1)
-		{
-			memory_set_bankptr(machine, "bank3", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_1&0x1f));
-			space->unmap_write(0x8000, 0xbfff);
-		}
-		else
-		{
-			memory_set_bankptr(machine, "bank2", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_1&0x1f));
-			space->unmap_write(0x4000, 0x7fff);
-		}
+		ti8x_update_bank(space, (state->m_ti83p_port4 & 1) ? 2 : 1, state->m_bios, state->m_ti8x_memory_page_1 & 0x1f, false);
 	}
 
 	if (state->m_ti8x_memory_page_2 & 0x40)
 	{
-		if (state->m_ti83p_port4 & 1)
-		{
-			memory_set_bankptr(machine, "bank4", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_2&0x01));
-			space->install_write_bank(0xc000, 0xffff, "bank4");
-		}
-		else
-		{
-			memory_set_bankptr(machine, "bank3", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_2&0x01));
-			space->install_write_bank(0x8000, 0xbfff, "bank3");
-		}
-
+		ti8x_update_bank(space, (state->m_ti83p_port4 & 1) ? 3 : 2, state->m_ti8x_ram, state->m_ti8x_memory_page_2 & 0x01, true);
 	}
 	else
 	{
-		if (state->m_ti83p_port4 & 1)
-		{
-			memory_set_bankptr(machine, "bank4", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_2&0x1f));
-			space->unmap_write(0xc000, 0xffff);
-		}
-		else
-		{
-			memory_set_bankptr(machine, "bank3", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_2&0x1f));
-			space->unmap_write(0x8000, 0xbfff);
-		}
+		ti8x_update_bank(space, (state->m_ti83p_port4 & 1) ? 3 : 2, state->m_bios, state->m_ti8x_memory_page_2 & 0x1f, false);
 	}
 }
 
@@ -114,24 +88,20 @@ static void update_ti86_memory (running_machine &machine)
 
 	if (state->m_ti8x_memory_page_1 & 0x40)
 	{
-		memory_set_bankptr(machine, "bank2", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_1&0x07));
-		space->install_write_bank(0x4000, 0x7fff, "bank2");
+		ti8x_update_bank(space, 1, state->m_ti8x_ram, state->m_ti8x_memory_page_1 & 0x07, true);
 	}
 	else
 	{
-		memory_set_bankptr(machine, "bank2", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_1&0x0f));
-		space->unmap_write(0x4000, 0x7fff);
+		ti8x_update_bank(space, 1, state->m_bios, state->m_ti8x_memory_page_1 & 0x0f, false);
 	}
 
 	if (state->m_ti8x_memory_page_2 & 0x40)
 	{
-		memory_set_bankptr(machine, "bank3", state->m_ti8x_ram + 0x004000*(state->m_ti8x_memory_page_2&0x07));
-		space->install_write_bank(0x8000, 0xbfff, "bank3");
+		ti8x_update_bank(space, 2, state->m_ti8x_ram, state->m_ti8x_memory_page_2 & 0x07, true);
 	}
 	else
 	{
-		memory_set_bankptr(machine, "bank3", machine.region("maincpu")->base() + 0x010000 + 0x004000*(state->m_ti8x_memory_page_2&0x0f));
-		space->unmap_write(0x8000, 0xbfff);
+		ti8x_update_bank(space, 2, state->m_bios, state->m_ti8x_memory_page_2 & 0x0f, false);
 	}
 
 }
@@ -144,7 +114,7 @@ MACHINE_START( ti81 )
 {
 	ti85_state *state = machine.driver_data<ti85_state>();
 	address_space *space = state->m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *mem = machine.region("maincpu")->base();
+	state->m_bios = state->memregion("bios")->base();
 
 	state->m_timer_interrupt_mask = 0;
 	state->m_timer_interrupt_status = 0;
@@ -166,8 +136,8 @@ MACHINE_START( ti81 )
 
 	space->unmap_write(0x0000, 0x3fff);
 	space->unmap_write(0x4000, 0x7fff);
-	memory_set_bankptr(machine, "bank1", mem + 0x010000);
-	memory_set_bankptr(machine, "bank2", mem + 0x014000);
+	state->membank("bank1")->set_base(state->m_bios);
+	state->membank("bank2")->set_base(state->m_bios + 0x04000);
 }
 
 MACHINE_RESET( ti85 )
@@ -183,7 +153,7 @@ MACHINE_START( ti83p )
 {
 	ti85_state *state = machine.driver_data<ti85_state>();
 	address_space *space = state->m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *mem = machine.region("maincpu")->base();
+	state->m_bios = state->memregion("bios")->base();
 
 	state->m_timer_interrupt_mask = 0;
 	state->m_timer_interrupt_status = 0;
@@ -208,10 +178,10 @@ MACHINE_START( ti83p )
 	space->unmap_write(0x4000, 0x7fff);
 	space->unmap_write(0x8000, 0xbfff);
 
-	memory_set_bankptr(machine, "bank1", mem + 0x010000);
-	memory_set_bankptr(machine, "bank2", mem + 0x010000);
-	memory_set_bankptr(machine, "bank3", mem + 0x010000);
-	memory_set_bankptr(machine, "bank4", state->m_ti8x_ram);
+	state->membank("bank1")->set_base(state->m_bios);
+	state->membank("bank2")->set_base(state->m_bios);
+	state->membank("bank3")->set_base(state->m_bios);
+	state->membank("bank4")->set_base(state->m_ti8x_ram);
 
 	machine.scheduler().timer_pulse(attotime::from_hz(200), FUNC(ti85_timer_callback));
 
@@ -222,7 +192,7 @@ MACHINE_START( ti86 )
 {
 	ti85_state *state = machine.driver_data<ti85_state>();
 	address_space *space = state->m_maincpu->memory().space(AS_PROGRAM);
-	UINT8 *mem = machine.region("maincpu")->base();
+	state->m_bios = state->memregion("bios")->base();
 
 	state->m_timer_interrupt_mask = 0;
 	state->m_timer_interrupt_status = 0;
@@ -245,10 +215,10 @@ MACHINE_START( ti86 )
 
 	space->unmap_write(0x0000, 0x3fff);
 
-	memory_set_bankptr(machine, "bank1", mem + 0x010000);
-	memory_set_bankptr(machine, "bank2", mem + 0x014000);
+	state->membank("bank1")->set_base(state->m_bios);
+	state->membank("bank2")->set_base(state->m_bios + 0x04000);
 
-	memory_set_bankptr(machine, "bank4", state->m_ti8x_ram);
+	state->membank("bank4")->set_base(state->m_ti8x_ram);
 
 	machine.scheduler().timer_pulse(attotime::from_hz(200), FUNC(ti85_timer_callback));
 }
@@ -256,320 +226,288 @@ MACHINE_START( ti86 )
 
 /* I/O ports handlers */
 
-READ8_HANDLER ( ti85_port_0000_r )
+READ8_MEMBER(ti85_state::ti85_port_0000_r)
 {
 	return 0xff;
 }
 
-READ8_HANDLER ( ti8x_keypad_r )
+READ8_MEMBER(ti85_state::ti8x_keypad_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 	int data = 0xff;
 	int port;
 	int bit;
 	static const char *const bitnames[] = { "BIT0", "BIT1", "BIT2", "BIT3", "BIT4", "BIT5", "BIT6", "BIT7" };
 
-	if (state->m_keypad_mask == 0x7f) return data;
+	if (m_keypad_mask == 0x7f) return data;
 
 	for (bit = 0; bit < 7; bit++)
 	{
-		if (~state->m_keypad_mask&(0x01<<bit))
+		if (~m_keypad_mask&(0x01<<bit))
 		{
 			for (port = 0; port < 8; port++)
 			{
-				data ^= input_port_read(space->machine(), bitnames[port]) & (0x01<<bit) ? 0x01<<port : 0x00;
+				data ^= ioport(bitnames[port])->read() & (0x01<<bit) ? 0x01<<port : 0x00;
 			}
 		}
 	}
 	return data;
 }
 
- READ8_HANDLER ( ti85_port_0002_r )
+ READ8_MEMBER(ti85_state::ti85_port_0002_r )
 {
 	return 0xff;
 }
 
- READ8_HANDLER ( ti85_port_0003_r )
+ READ8_MEMBER(ti85_state::ti85_port_0003_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 	int data = 0;
 
-	if (state->m_LCD_status)
-		data |= state->m_LCD_mask;
-	if (state->m_ON_interrupt_status)
+	if (m_LCD_status)
+		data |= m_LCD_mask;
+	if (m_ON_interrupt_status)
 		data |= 0x01;
-	if (state->m_timer_interrupt_status)
+	if (m_timer_interrupt_status)
 		data |= 0x04;
-	if (!state->m_ON_pressed)
+	if (!m_ON_pressed)
 		data |= 0x08;
-	state->m_ON_interrupt_status = 0;
-	state->m_timer_interrupt_status = 0;
+	m_ON_interrupt_status = 0;
+	m_timer_interrupt_status = 0;
 	return data;
 }
 
- READ8_HANDLER ( ti85_port_0004_r )
+ READ8_MEMBER(ti85_state::ti85_port_0004_r )
 {
 	return 0xff;
 }
 
- READ8_HANDLER ( ti85_port_0005_r )
+ READ8_MEMBER(ti85_state::ti85_port_0005_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_memory_page_1;
+	return m_ti8x_memory_page_1;
 }
 
-READ8_HANDLER ( ti85_port_0006_r )
+READ8_MEMBER(ti85_state::ti85_port_0006_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_power_mode;
+	return m_power_mode;
 }
 
-READ8_HANDLER ( ti8x_serial_r )
+READ8_MEMBER(ti85_state::ti8x_serial_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 
-	ti85_update_serial(state->m_serial);
-	return (state->m_white_out<<3)
-		| (state->m_red_out<<2)
-		| ((ti85serial_white_in(state->m_serial,0)&(1-state->m_white_out))<<1)
-		| (ti85serial_red_in(state->m_serial,0)&(1-state->m_red_out))
-		| state->m_PCR;
+	//ti85_update_serial(m_serial);
+	return (m_white_out<<3)
+		| (m_red_out<<2)
+		//| ((ti85serial_white_in(m_serial,0)&(1-m_white_out))<<1)
+		//| (ti85serial_red_in(m_serial,0)&(1-m_red_out))
+		| 0x03	// no link cable
+		| m_PCR;
 }
 
- READ8_HANDLER ( ti82_port_0002_r )
+READ8_MEMBER(ti85_state::ti82_port_0002_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_port2;
+	return m_ti8x_port2;
 }
 
-READ8_HANDLER ( ti86_port_0005_r )
+READ8_MEMBER(ti85_state::ti86_port_0005_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_memory_page_1;
+	return m_ti8x_memory_page_1;
 }
 
- READ8_HANDLER ( ti86_port_0006_r )
+READ8_MEMBER(ti85_state::ti86_port_0006_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_memory_page_2;
+	return m_ti8x_memory_page_2;
 }
 
-READ8_HANDLER ( ti83_port_0000_r )
+READ8_MEMBER(ti85_state::ti83_port_0000_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return ((state->m_ti8x_memory_page_1 & 0x08) << 1) | 0x0C;
+	return ((m_ti8x_memory_page_1 & 0x08) << 1) | 0x0C;
 }
 
- READ8_HANDLER ( ti83_port_0002_r )
+READ8_MEMBER(ti85_state::ti83_port_0002_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_port2;
+	return m_ti8x_port2;
 }
 
- READ8_HANDLER ( ti83_port_0003_r )
+READ8_MEMBER(ti85_state::ti83_port_0003_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 	int data = 0;
 
-	data |= state->m_LCD_mask;
+	data |= m_LCD_mask;
 
-	if (state->m_ON_interrupt_status)
+	if (m_ON_interrupt_status)
 		data |= 0x01;
-	if (!state->m_ON_pressed)
+	if (!m_ON_pressed)
 		data |= 0x08;
-	state->m_ON_interrupt_status = 0;
-	state->m_timer_interrupt_status = 0;
+	m_ON_interrupt_status = 0;
+	m_timer_interrupt_status = 0;
 	return data;
 }
 
-READ8_HANDLER ( ti8x_plus_serial_r )
+READ8_MEMBER(ti85_state::ti8x_plus_serial_r)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 
-	ti85_update_serial(state->m_serial);
-	return (state->m_white_out<<3)
-		| (state->m_red_out<<2)
-		| ((ti85serial_white_in(state->m_serial,0)&(1-state->m_white_out))<<1)
-		| (ti85serial_red_in(state->m_serial,0)&(1-state->m_red_out))
-		| state->m_PCR;
+	//ti85_update_serial(m_serial);
+	return (m_white_out<<3)
+		| (m_red_out<<2)
+		//| ((ti85serial_white_in(m_serial,0)&(1-m_white_out))<<1)
+		//| (ti85serial_red_in(m_serial,0)&(1-m_red_out))
+		| 0x03	// no link cable
+		| m_PCR;
 }
 
- READ8_HANDLER ( ti83p_port_0002_r )
+READ8_MEMBER(ti85_state::ti83p_port_0002_r )
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	return state->m_ti8x_port2|3;
+	return m_ti8x_port2|3;
 }
 
-WRITE8_HANDLER ( ti81_port_0007_w )
+WRITE8_MEMBER(ti85_state::ti81_port_0007_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti81_port_7_data = data;
+	m_ti81_port_7_data = data;
 }
 
-WRITE8_HANDLER ( ti85_port_0000_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0000_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_LCD_memory_base = data;
+	m_LCD_memory_base = data;
 }
 
-WRITE8_HANDLER ( ti8x_keypad_w )
+WRITE8_MEMBER(ti85_state::ti8x_keypad_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_keypad_mask = data&0x7f;
+	m_keypad_mask = data&0x7f;
 }
 
-WRITE8_HANDLER ( ti85_port_0002_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0002_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_LCD_contrast = data&0x1f;
+	m_LCD_contrast = data&0x1f;
 }
 
-WRITE8_HANDLER ( ti85_port_0003_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0003_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	if (state->m_LCD_status && !(data&0x08))	state->m_timer_interrupt_mask = 0;
-	state->m_ON_interrupt_mask = data&0x01;
-//  state->m_timer_interrupt_mask = data&0x04;
-	state->m_LCD_mask = data&0x02;
-	state->m_LCD_status = data&0x08;
+	if (m_LCD_status && !(data&0x08))	m_timer_interrupt_mask = 0;
+	m_ON_interrupt_mask = data&0x01;
+//  m_timer_interrupt_mask = data&0x04;
+	m_LCD_mask = data&0x02;
+	m_LCD_status = data&0x08;
 }
 
-WRITE8_HANDLER ( ti85_port_0004_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0004_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_video_buffer_width = (data>>3)&0x03;
-	state->m_interrupt_speed = (data>>1)&0x03;
-	state->m_port4_bit0 = data&0x01;
+	m_video_buffer_width = (data>>3)&0x03;
+	m_interrupt_speed = (data>>1)&0x03;
+	m_port4_bit0 = data&0x01;
 }
 
-WRITE8_HANDLER ( ti85_port_0005_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0005_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = data;
-	update_ti85_memory(space->machine());
+	m_ti8x_memory_page_1 = data;
+	update_ti85_memory(machine());
 }
 
-WRITE8_HANDLER ( ti85_port_0006_w )
+WRITE8_MEMBER(ti85_state::ti85_port_0006_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_power_mode = data;
+	m_power_mode = data;
 }
 
-WRITE8_HANDLER ( ti8x_serial_w )
+WRITE8_MEMBER(ti85_state::ti8x_serial_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 
-	speaker_level_w(state->m_speaker, ( (data>>2)|(data>>3) ) & 0x01);
-	state->m_red_out=(data>>2)&0x01;
-	state->m_white_out=(data>>3)&0x01;
-	ti85serial_red_out( state->m_serial, 0, state->m_red_out );
-	ti85serial_white_out( state->m_serial, 0, state->m_white_out );
-	ti85_update_serial(state->m_serial);
-	state->m_PCR = data&0xf0;
+	speaker_level_w(m_speaker, ( (data>>2)|(data>>3) ) & 0x01);
+	m_red_out=(data>>2)&0x01;
+	m_white_out=(data>>3)&0x01;
+	//ti85serial_red_out( m_serial, 0, m_red_out );
+	//ti85serial_white_out( m_serial, 0, m_white_out );
+	//ti85_update_serial(m_serial);
+	m_PCR = data&0xf0;
 }
 
-WRITE8_HANDLER ( ti86_port_0005_w )
+WRITE8_MEMBER(ti85_state::ti86_port_0005_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = data&((data&0x40)?0x47:0x4f);
-	update_ti86_memory(space->machine());
+	m_ti8x_memory_page_1 = data&((data&0x40)?0x47:0x4f);
+	update_ti86_memory(machine());
 }
 
-WRITE8_HANDLER ( ti86_port_0006_w )
+WRITE8_MEMBER(ti85_state::ti86_port_0006_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_2 = data&((data&0x40)?0x47:0x4f);
-	update_ti86_memory(space->machine());
+	m_ti8x_memory_page_2 = data&((data&0x40)?0x47:0x4f);
+	update_ti86_memory(machine());
 }
 
-WRITE8_HANDLER ( ti82_port_0002_w )
+WRITE8_MEMBER(ti85_state::ti82_port_0002_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = (data & 0x07);
-	update_ti85_memory(space->machine());
-	state->m_ti8x_port2 = data;
+	m_ti8x_memory_page_1 = (data & 0x07);
+	update_ti85_memory(machine());
+	m_ti8x_port2 = data;
 }
 
-WRITE8_HANDLER ( ti83_port_0000_w )
+WRITE8_MEMBER(ti85_state::ti83_port_0000_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = (state->m_ti8x_memory_page_1 & 7) | ((data & 16) >> 1);
-	update_ti85_memory(space->machine());
+	m_ti8x_memory_page_1 = (m_ti8x_memory_page_1 & 7) | ((data & 16) >> 1);
+	update_ti85_memory(machine());
 }
 
-WRITE8_HANDLER ( ti83_port_0002_w )
+WRITE8_MEMBER(ti85_state::ti83_port_0002_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = (state->m_ti8x_memory_page_1 & 8) | (data & 7);
-	update_ti85_memory(space->machine());
-	state->m_ti8x_port2 = data;
+	m_ti8x_memory_page_1 = (m_ti8x_memory_page_1 & 8) | (data & 7);
+	update_ti85_memory(machine());
+	m_ti8x_port2 = data;
 }
 
-WRITE8_HANDLER ( ti83_port_0003_w )
+WRITE8_MEMBER(ti85_state::ti83_port_0003_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-		if (state->m_LCD_status && !(data&0x08))	state->m_timer_interrupt_mask = 0;
-		state->m_ON_interrupt_mask = data&0x01;
-		//state->m_timer_interrupt_mask = data&0x04;
-		state->m_LCD_mask = data&0x02;
-		state->m_LCD_status = data&0x08;
+		if (m_LCD_status && !(data&0x08))	m_timer_interrupt_mask = 0;
+		m_ON_interrupt_mask = data&0x01;
+		//m_timer_interrupt_mask = data&0x04;
+		m_LCD_mask = data&0x02;
+		m_LCD_status = data&0x08;
 }
 
-WRITE8_HANDLER ( ti8x_plus_serial_w )
+WRITE8_MEMBER(ti85_state::ti8x_plus_serial_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
 
-	speaker_level_w(state->m_speaker,( (data>>0)|(data>>1) )&0x01 );
-	state->m_red_out=(data>>0)&0x01;
-	state->m_white_out=(data>>1)&0x01;
-	ti85serial_red_out( state->m_serial, 0, state->m_red_out );
-	ti85serial_white_out( state->m_serial, 0, state->m_white_out );
-	ti85_update_serial(state->m_serial);
-	state->m_PCR = data&0xf0;
+	speaker_level_w(m_speaker,( (data>>0)|(data>>1) )&0x01 );
+	m_red_out=(data>>0)&0x01;
+	m_white_out=(data>>1)&0x01;
+	//ti85serial_red_out( m_serial, 0, m_red_out );
+	//ti85serial_white_out( m_serial, 0, m_white_out );
+	//ti85_update_serial(m_serial);
+	m_PCR = data&0xf0;
 }
 
-WRITE8_HANDLER ( ti83p_port_0002_w )
+WRITE8_MEMBER(ti85_state::ti83p_port_0002_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_port2 = data;
+	m_ti8x_port2 = data;
 }
 
-WRITE8_HANDLER ( ti83p_port_0003_w )
+WRITE8_MEMBER(ti85_state::ti83p_port_0003_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_LCD_mask = (data&0x08) >> 2;
-	state->m_ON_interrupt_mask = data & 0x01;
+	m_LCD_mask = (data&0x08) >> 2;
+	m_ON_interrupt_mask = data & 0x01;
 }
 
-WRITE8_HANDLER ( ti83p_port_0004_w )
+WRITE8_MEMBER(ti85_state::ti83p_port_0004_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	if ((data & 1) && !(state->m_ti83p_port4 & 1))
+	if ((data & 1) && !(m_ti83p_port4 & 1))
 	{
-		state->m_ti8x_memory_page_1 = 0x1f;
-		state->m_ti8x_memory_page_2 = 0x1f;
+		m_ti8x_memory_page_1 = 0x1f;
+		m_ti8x_memory_page_2 = 0x1f;
 	}
-	else if (!(data & 1) && (state->m_ti83p_port4 & 1))
+	else if (!(data & 1) && (m_ti83p_port4 & 1))
 	{
-		state->m_ti8x_memory_page_1 = 0x1f;
-		state->m_ti8x_memory_page_2 = 0x40;
+		m_ti8x_memory_page_1 = 0x1f;
+		m_ti8x_memory_page_2 = 0x40;
 	}
-	update_ti83p_memory(space->machine());
-	state->m_ti83p_port4 = data;
+	update_ti83p_memory(machine());
+	m_ti83p_port4 = data;
 }
 
-WRITE8_HANDLER ( ti83p_port_0006_w )
+WRITE8_MEMBER(ti85_state::ti83p_port_0006_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_1 = data & ((data&0x40) ? 0x41 : 0x5f);
-	update_ti83p_memory(space->machine());
+	m_ti8x_memory_page_1 = data & ((data&0x40) ? 0x41 : 0x5f);
+	update_ti83p_memory(machine());
 }
 
-WRITE8_HANDLER ( ti83p_port_0007_w )
+WRITE8_MEMBER(ti85_state::ti83p_port_0007_w)
 {
-	ti85_state *state = space->machine().driver_data<ti85_state>();
-	state->m_ti8x_memory_page_2 = data & ((data&0x40) ? 0x41 : 0x5f);
-	update_ti83p_memory(space->machine());
+	m_ti8x_memory_page_2 = data & ((data&0x40) ? 0x41 : 0x5f);
+	update_ti83p_memory(machine());
 }
 
 /* NVRAM functions */
