@@ -1,4 +1,5 @@
-/*
+/****************************************************************************
+
     TI-99 RS232 and Parallel interface card
 
     TI99 RS232 card ('rs232')
@@ -93,19 +94,17 @@
 
     The protocol changes back to normal mode after transmitting the control
     sequence.
-*/
 
-#include "emu.h"
-#include "peribox.h"
-#include "machine/tms9902.h"
+    Michael Zapf
+    February 2012: Rewritten as class
+
+*****************************************************************************/
+
 #include "ti_rs232.h"
 
-#define CRU_BASE 0x1300
 #define SENILA_0_BIT 0x80
 #define SENILA_1_BIT 0x40
 
-#define ser_region "ser_region"
-
 #define RECV_MODE_NORMAL 1
 #define RECV_MODE_ESC 2
 #define RECV_MODE_ESC_LINES 3
@@ -113,222 +112,180 @@
 #define VERBOSE 1
 #define LOG logerror
 
-// Second card: Change CRU base to 0x1500
-// #define CRU_BASE 0x1500
-
-typedef ti99_pebcard_config ti_rs232_config;
-
-typedef struct _ti_rs232_state
-{
-	device_t *uart[2];
-	device_t *serdev[2], *piodev;
-
-	int 	selected;
-	int		select_mask;
-	int		select_value;
-
-	UINT8	signals[2]; // Latches the state of the output lines for UART0/UART1
-	int		recv_mode[2]; // May be NORMAL or ESC
-
-	// Baud rate management
-	// not part of the real device, but required for the connection to the
-	// real UART
-	double time_hold[2];
-
-	// PIO flags
-	int pio_direction;		// a.k.a. PIOOC pio in output mode if 0
-	int pio_handshakeout;
-	int pio_handshakein;
-	int pio_spareout;
-	int pio_sparein;
-	int flag0;				// spare
-	int led;				// a.k.a. flag3
-	int pio_out_buffer;
-	int pio_in_buffer;
-	int pio_readable;
-	int pio_writable;
-	int pio_write;			// 1 if image is to be written to
-
-	UINT8	*rom;
-
-	/* Keeps the state of the SENILA line. */
-	UINT8	senila;
-
-	/* Keeps the value put on the bus when SENILA becomes active. */
-	UINT8	ila;
-
-	/* Callback lines to the main system. */
-	ti99_peb_connect	lines;
-
-} ti_rs232_state;
-
 #define ESC 0x1b
 
-INLINE ti_rs232_state *get_safe_token(device_t *device)
+ti_rs232_pio_device::ti_rs232_pio_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+: ti_expansion_card_device(mconfig, TI99_RS232, "TI-99 RS232/PIO interface", tag, owner, clock)
 {
-	assert(device != NULL);
-	assert(device->type() == TIRS232);
-
-	return (ti_rs232_state *)downcast<legacy_device_base *>(device)->token();
+	m_shortname = "ti99_rs232";
 }
 
-static void incoming_dtr(device_t *device, line_state value);
-static void output_line_state(device_t *tms9902, int mask, UINT8 value);
-
 
 /**************************************************************************/
 /* Ports */
 
-static DEVICE_START( ti99_rs232_serdev )
+ti_rs232_attached_device::ti_rs232_attached_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+: device_t(mconfig, TI99_RS232_DEV, "Serial attached device", tag, owner, clock),
+  device_image_interface(mconfig, *this)
 {
 }
 
-static DEVICE_START( ti99_piodev )
+ti_pio_attached_device::ti_pio_attached_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
+: device_t(mconfig, TI99_PIO_DEV, "Parallel attached device", tag, owner, clock),
+  device_image_interface(mconfig, *this)
 {
 }
 
+void ti_rs232_attached_device::device_start()
+{
+}
+
+void ti_pio_attached_device::device_start()
+{
+}
+
+void ti_rs232_attached_device::device_config_complete()
+{
+	update_names();
+}
+
+void ti_pio_attached_device::device_config_complete()
+{
+	update_names();
+}
+
 /*
     Find the index of the image name. We assume the format
     <name><number>, i.e. the number is the longest string from the right
     which can be interpreted as a number.
 */
-static int get_index_from_tagname(device_t *image)
+int ti_rs232_attached_device::get_index_from_tagname()
 {
-	const char *tag = image->tag();
-	int maxlen = strlen(tag);
+	const char *mytag = tag();
+	int maxlen = strlen(mytag);
 	int i;
 	for (i=maxlen-1; i >=0; i--)
-		if (tag[i] < 48 || tag[i] > 57) break;
+		if (mytag[i] < 48 || mytag[i] > 57) break;
 
-	return atoi(tag+i+1);
+	return atoi(mytag+i+1);
 }
 
 /*
     Initialize rs232 unit and open image
 */
-static DEVICE_IMAGE_LOAD( ti99_rs232_serdev )
+bool ti_rs232_attached_device::call_load()
 {
-	device_t *tms9902 = NULL;
+	tms9902_device* tms9902 = NULL;
+//  ti_rs232_pio_device* card = static_cast<ti_rs232_pio_device*>(owner());
 
-	// TODO: strcmp does not work this way (wrong constant or wrong function)
-	int devnumber = get_index_from_tagname(&image.device());
+	int devnumber = get_index_from_tagname();
 	if (devnumber==0)
 	{
-		tms9902 = image.device().siblingdevice("tms9902_0");
+		tms9902 = siblingdevice<tms9902_device>("tms9902_0");
 		// Turn on polling
-		tms9902_clock(tms9902, true);
+		tms9902->set_clock(true);
 	}
 	else if (devnumber==1)
 	{
-		tms9902 = image.device().siblingdevice("tms9902_1");
+		tms9902 = siblingdevice<tms9902_device>("tms9902_1");
 		// Turn on polling
-		tms9902_clock(tms9902, true);
+		tms9902->set_clock(true);
 	}
 	else
 	{
 		LOG("ti99/rs232: Could not find device tag number\n");
-		return IMAGE_INIT_FAIL;
+		return true;
 	}
 
-	incoming_dtr(tms9902, image? ASSERT_LINE : CLEAR_LINE);
+	// The following line may cause trouble in the init phase
+	// card->incoming_dtr(devnumber, (m_file!=NULL)? ASSERT_LINE : CLEAR_LINE);
 
-	return IMAGE_INIT_PASS;
+	return false;  // OK
 }
 
-static DEVICE_IMAGE_UNLOAD( ti99_rs232_serdev )
+void ti_rs232_attached_device::call_unload()
 {
-	device_t *tms9902 = NULL;
-	int devnumber = get_index_from_tagname(&image.device());
+	tms9902_device* tms9902 = NULL;
+
+	int devnumber = get_index_from_tagname();
 	if (devnumber==0)
 	{
-		tms9902 = image.device().siblingdevice("tms9902_0");
+		tms9902 = siblingdevice<tms9902_device>("tms9902_0");
 		// Turn off polling
-		tms9902_clock(tms9902, false);
+		tms9902->set_clock(false);
 	}
 	else if (devnumber==1)
 	{
-		tms9902 = image.device().siblingdevice("tms9902_1");
+		tms9902 = siblingdevice<tms9902_device>("tms9902_1");
 		// Turn off polling
-		tms9902_clock(tms9902, false);
+		tms9902->set_clock(false);
 	}
 }
 
 /*
     Initialize pio unit and open image
 */
-static DEVICE_IMAGE_LOAD( ti99_piodev )
+bool ti_pio_attached_device::call_load()
 {
-	device_t *carddev = image.device().owner();
-	ti_rs232_state *card = get_safe_token(carddev);
+	ti_rs232_pio_device* card = static_cast<ti_rs232_pio_device*>(owner());
 
+	/* tell whether the image is readable */
+	card->m_pio_readable = !has_been_created();
 	/* tell whether the image is writable */
-	card->pio_readable = !image.has_been_created();
-	/* tell whether the image is writable */
-	card->pio_writable = !image.is_readonly();
+	card->m_pio_writable = !is_readonly();
 
-	if (card->pio_write && card->pio_writable)
-		card->pio_handshakein = 0;	/* receiver ready */
+	if (card->m_pio_write && card->m_pio_writable)
+		card->m_pio_handshakein = false;	/* receiver ready */
 	else
-		card->pio_handshakein = 1;
+		card->m_pio_handshakein = true;
 
-	return IMAGE_INIT_PASS;
+	return false;  // OK
 }
 
 /*
     close a pio image
 */
-static DEVICE_IMAGE_UNLOAD( ti99_piodev )
+void ti_pio_attached_device::call_unload()
 {
-	device_t *carddev = image.device().owner();
-	ti_rs232_state *card = get_safe_token(carddev);
+	ti_rs232_pio_device* card = static_cast<ti_rs232_pio_device*>(owner());
 
-	card->pio_writable = 0;
-	card->pio_handshakein = 1;	/* receiver not ready */
-	card->pio_sparein = 0;
+	card->m_pio_writable = false;
+	card->m_pio_handshakein = true;	/* receiver not ready */
+	card->m_pio_sparein = false;
 }
 
 /****************************************************************************/
 
 /*
     CRU read
-    TODO: Check read operations
 */
-static READ8Z_DEVICE_HANDLER( cru_rz )
+void ti_rs232_pio_device::crureadz(offs_t offset, UINT8 *value)
 {
-	ti_rs232_state *card = get_safe_token(device);
-
-	if ((offset & 0xff00)==CRU_BASE)
+	if ((offset & 0xff00)==m_cru_base)
 	{
 		if ((offset & 0x00c0)==0x0000)
 		{
 			UINT8 reply = 0x00;
-			if (card->pio_direction)
-				reply |= 0x02;
-			if (card->pio_handshakein)
-				reply |= 0x04;
-			if (card->pio_sparein)
-				reply |= 0x08;
-			if (card->flag0)
-				reply |= 0x10;
+			if (m_pio_direction_in) 		reply |= 0x02;
+			if (m_pio_handshakein)			reply |= 0x04;
+			if (m_pio_sparein)				reply |= 0x08;
+			if (m_flag0)					reply |= 0x10;
 			// The CTS line is realized as CRU bits
 			// Mind that this line is handled as an output going to the remote CTS
-			if (card->signals[0] & CTS)
-				reply |= 0x20;
-			if (card->signals[1] & CTS)
-				reply |= 0x40;
-			if (card->led)
-				reply |= 0x80;
+			if ((m_signals[0] & CTS)!=0)	reply |= 0x20;
+			if ((m_signals[1] & CTS)!=0)	reply |= 0x40;
+			if (m_led)						reply |= 0x80;
 			*value = reply;
 			return;
 		}
 		if ((offset & 0x00c0)==0x0040)
 		{
-			*value = tms9902_cru_r(card->uart[0], offset>>4);
+			*value = m_uart[0]->cruread(*m_space, offset>>4, 0xff);
 			return;
 		}
 		if ((offset & 0x00c0)==0x0080)
 		{
-			*value = tms9902_cru_r(card->uart[1], offset>>1);
+			*value = m_uart[1]->cruread(*m_space, offset>>1, 0xff);
 			return;
 		}
 	}
@@ -337,98 +294,97 @@ static READ8Z_DEVICE_HANDLER( cru_rz )
 /*
     CRU write
 */
-static WRITE8_DEVICE_HANDLER( cru_w )
+void ti_rs232_pio_device::cruwrite(offs_t offset, UINT8 data)
 {
-	ti_rs232_state *card = get_safe_token(device);
-
-	if ((offset & 0xff00)==CRU_BASE)
+	if ((offset & 0xff00)==m_cru_base)
 	{
 		if ((offset & 0x00c0)==0x0040)
 		{
-			tms9902_cru_w(card->uart[0], offset>>1, data);
+			m_uart[0]->cruwrite(*m_space, offset>>1, data, 0xff);
 			return;
 		}
 		if ((offset & 0x00c0)==0x0080)
 		{
-			tms9902_cru_w(card->uart[1], offset>>1, data);
+			m_uart[1]->cruwrite(*m_space, offset>>1, data, 0xff);
 			return;
 		}
 
-		device_image_interface *image = dynamic_cast<device_image_interface *>(card->piodev);
+		device_image_interface *image = dynamic_cast<device_image_interface *>(m_piodev);
+
 		int bit = (offset & 0x00ff)>>1;
 		switch (bit)
 		{
 		case 0:
-			card->selected = data;
+			m_selected = (data!=0);
 			break;
 
 		case 1:
-			card->pio_direction = data;
+			m_pio_direction_in = (data!=0);
 			break;
 
 		case 2:
-			if (data != card->pio_handshakeout)
+			if ((data!=0) != m_pio_handshakeout)
 			{
-				card->pio_handshakeout = data;
-				if (card->pio_write && card->pio_writable && (!card->pio_direction))
+				m_pio_handshakeout = (data!=0);
+				if (m_pio_write && m_pio_writable && (!m_pio_direction_in))
 				{	/* PIO in output mode */
-					if (!card->pio_handshakeout)
+					if (!m_pio_handshakeout)
 					{	/* write data strobe */
 						/* write data and acknowledge */
-						UINT8 buf = card->pio_out_buffer;
+						UINT8 buf = m_pio_out_buffer;
 						int ret = image->fwrite(&buf, 1);
 						if (ret)
-							card->pio_handshakein = 1;
+							m_pio_handshakein = 1;
 					}
 					else
 					{
 						/* end strobe */
 						/* we can write some data: set receiver ready */
-						card->pio_handshakein = 0;
+						m_pio_handshakein = 0;
 					}
 				}
-				if ((!card->pio_write) && card->pio_readable /*&& pio_direction*/)
+				if ((!m_pio_write) && m_pio_readable /*&& pio_direction_in*/)
 				{	/* PIO in input mode */
-					if (!card->pio_handshakeout)
+					if (!m_pio_handshakeout)
 					{	/* receiver ready */
 						/* send data and strobe */
 						UINT8 buf;
 						if (image->fread(&buf, 1))
-							card->pio_in_buffer = buf;
-						card->pio_handshakein = 0;
+							m_pio_in_buffer = buf;
+						m_pio_handshakein = 0;
 					}
 					else
 					{
 						/* data acknowledge */
 						/* we can send some data: set transmitter ready */
-						card->pio_handshakein = 1;
+						m_pio_handshakein = 1;
 					}
 				}
 			}
 			break;
 
 		case 3:
-			card->pio_spareout = data;
+			m_pio_spareout = (data!=0);
 			break;
 
 		case 4:
-			card->flag0 = data;
+			m_flag0 = (data!=0);
 			break;
 
 		case 5:
 			// Set the CTS line for RS232/1
-			LOG("TI-RS232/1: Setting CTS* via CRU to %d\n", data);
-			output_line_state(card->uart[0], CTS, (data==0)? CTS : 0);
+			if (VERBOSE>5) LOG("TI-RS232/1/3: Setting CTS* via CRU to %d\n", data);
+			output_line_state(0, CTS, (data==0)? CTS : 0);
 			break;
 
 		case 6:
 			// Set the CTS line for RS232/2
-			LOG("TI-RS232/2: Setting CTS* via CRU to %d\n", data);
-			output_line_state(card->uart[1], CTS, (data==0)? CTS : 0);
+			if (VERBOSE>5) LOG("TI-RS232/2/4: Setting CTS* via CRU to %d\n", data);
+			output_line_state(1, CTS, (data==0)? CTS : 0);
 			break;
 
 		case 7:
-			card->led = data;
+			m_led = (data!=0);
 			break;
 		}
 		return;
@@ -438,30 +394,28 @@ static WRITE8_DEVICE_HANDLER( cru_w )
 /*
     Memory read
 */
-static READ8Z_DEVICE_HANDLER( data_rz )
+READ8Z_MEMBER( ti_rs232_pio_device::readz )
 {
-	ti_rs232_state *card = get_safe_token(device);
-
-	if (card->senila)
+	if (m_senila==ASSERT_LINE)
 	{
 		if (VERBOSE>3) LOG("ti99/rs232: Sensing ILA\n");
-		*value = card->ila;
+		*value = m_ila;
 		// The card ROM must be unselected, or we get two values
 		// on the data bus
 
 		// Not sure whether this is correct; there is no software that makes
 		// use of it
-		card->ila = 0;
+		m_ila = 0;
 	}
-	if (((offset & card->select_mask)==card->select_value) && card->selected)
+	if (((offset & m_select_mask)==m_select_value) && m_selected)
 	{
 		if ((offset & 0x1000)==0x0000)
 		{
-			*value = card->rom[offset&0x0fff];
+			*value = m_dsrrom[offset&0x0fff];
 		}
 		else
 		{
-			*value = card->pio_direction ? card->pio_in_buffer : card->pio_out_buffer;
+			*value = m_pio_direction_in ? m_pio_in_buffer : m_pio_out_buffer;
 		}
 	}
 }
@@ -469,37 +423,19 @@ static READ8Z_DEVICE_HANDLER( data_rz )
 /*
     Memory write
 */
-static WRITE8_DEVICE_HANDLER( data_w )
+WRITE8_MEMBER( ti_rs232_pio_device::write )
 {
-	ti_rs232_state *card = get_safe_token(device);
-
-	if (((offset & card->select_mask)==card->select_value) && card->selected)
+	if (((offset & m_select_mask)==m_select_value) && m_selected)
 	{
 		if ((offset & 0x1001)==0x1000)
 		{
-			card->pio_out_buffer = data;
+			m_pio_out_buffer = data;
 		}
 	}
 }
 
 /**************************************************************************/
 
-/*
-    Propagates the /INT signal of the UARTs to the /INT line of the pbox.
-*/
-static TMS9902_INT_CALLBACK( int_callback )
-{
-	device_t *carddev = device->owner();
-	ti_rs232_state *card = get_safe_token(carddev);
-	int senila_bit = (device==card->uart[0])? SENILA_0_BIT : SENILA_1_BIT;
-
-	if (state==FALSE)
-		card->ila |= senila_bit;
-	else
-		card->ila &= ~senila_bit;
-
-	card->lines.inta(state);
-}
 
 // ==========================================================
 
@@ -507,28 +443,23 @@ static TMS9902_INT_CALLBACK( int_callback )
     The DTR line of the interface card is wired to the CTS and DSR
     of the UART.
 */
-static void incoming_dtr(device_t *tms9902, line_state value)
+void ti_rs232_pio_device::incoming_dtr(int uartind, line_state value)
 {
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
-
 	if (VERBOSE>2) LOG("TI-RS232/%d: incoming DTR = %d\n", uartind+1, (value==ASSERT_LINE)? 1:0);
 
-	tms9902_rcv_cts(tms9902, value);
-	tms9902_rcv_dsr(tms9902, value);
+	m_uart[uartind]->rcv_cts(value);
+	m_uart[uartind]->rcv_dsr(value);
 }
 
 /*
     Data transmission
 */
-static void transmit_data(device_t *tms9902, UINT8 value)
+void ti_rs232_pio_device::transmit_data(int uartind, UINT8 value)
 {
 	UINT8 buf = value;
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
 
 	device_image_interface *serial;
-	serial = dynamic_cast<device_image_interface *>(card->serdev[uartind]);
+	serial = dynamic_cast<device_image_interface *>(m_serdev[uartind]);
 	if (!serial->exists())
 	{
 		if (VERBOSE>1) LOG("TI-RS232/%d: No serial output attached\n", uartind+1);
@@ -583,10 +514,10 @@ static void transmit_data(device_t *tms9902, UINT8 value)
       +12V 6 -|       |-( 6) <--- DSR
         nc 4 -----------( 8) <--- DCD
 */
-static UINT8 map_lines_out(device_t *carddev, int uartind, UINT8 value)
+UINT8 ti_rs232_pio_device::map_lines_out(int uartind, UINT8 value)
 {
 	UINT8 ret = 0;
-	int mapping = input_port_read(carddev->machine(), "SERIALMAP");
+	int mapping = ioport("SERIALMAP")->read();
 
 	//    00ab cdef = setting line RTS=a, CTS=b, DSR=c, DCD=d, DTR=e, RI=f
 
@@ -655,10 +586,10 @@ static UINT8 map_lines_out(device_t *carddev, int uartind, UINT8 value)
 	return ret;
 }
 
-static UINT8 map_lines_in(device_t *carddev, int uartind, UINT8 value)
+UINT8 ti_rs232_pio_device::map_lines_in(int uartind, UINT8 value)
 {
 	UINT8 ret = 0;
-	int mapping = input_port_read(carddev->machine(), "SERIALMAP");
+	int mapping = ioport("SERIALMAP")->read();
 
 	//    00ab cdef = setting line RTS=a, CTS=b, DSR=c, DCD=d, DTR=e, RI=f
 
@@ -750,15 +681,13 @@ static UINT8 map_lines_in(device_t *carddev, int uartind, UINT8 value)
     serial bridge must stop delivering data bytes until the handshake opens the
     channel again.
 */
-static void receive_data_or_line_state(device_t *tms9902, double baudpoll)
+void ti_rs232_pio_device::receive_data_or_line_state(int uartind)
 {
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
 	device_image_interface *serial;
 	UINT8 buffer;
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
 	int len = 0;
 
-	serial = dynamic_cast<device_image_interface *>(card->serdev[uartind]);
+	serial = dynamic_cast<device_image_interface *>(m_serdev[uartind]);
 
 	if (!serial->exists())
 	{
@@ -766,9 +695,11 @@ static void receive_data_or_line_state(device_t *tms9902, double baudpoll)
 		return;
 	}
 
+	double baudpoll = m_uart[uartind]->get_baudpoll();
+
 	// If more than the minimum waiting time since the last data byte has
 	// elapsed, we can get a new value.
-	if (card->time_hold[uartind] > 1.0)
+	if (m_time_hold[uartind] > 1.0)
 	{
 		len = serial->fread(&buffer, 1);
 		if (len==0) return;
@@ -776,33 +707,33 @@ static void receive_data_or_line_state(device_t *tms9902, double baudpoll)
 	else
 	{
 		// number of polls has not yet elapsed; we have to wait.
-		card->time_hold[uartind] += baudpoll;
+		m_time_hold[uartind] += baudpoll;
 		return;
 	}
 
 	// No config parameters here, only data or line setting
-	switch (card->recv_mode[uartind])
+	switch (m_recv_mode[uartind])
 	{
 	case RECV_MODE_NORMAL:
 		if (buffer==0x1b)
 		{
 			if (VERBOSE>2) LOG("TI-RS232/%d: received: %c <%02x>, switch to ESC mode\n", uartind+1, buffer, buffer);
-			card->recv_mode[uartind] = RECV_MODE_ESC;
+			m_recv_mode[uartind] = RECV_MODE_ESC;
 		}
 		else
 		{
 			if (VERBOSE>3) LOG("TI-RS232/%d: received: %c <%02x>, pass to UART\n", uartind+1, buffer, buffer);
-			tms9902_rcv_data(tms9902, buffer);
-			card->time_hold[uartind] = 0.0;
+			m_uart[uartind]->rcv_data(buffer);
+			m_time_hold[uartind] = 0.0;
 		}
 		break;
 	case RECV_MODE_ESC:
 		if (buffer==0x1b)
 		{
-			card->recv_mode[uartind] = RECV_MODE_NORMAL;
+			m_recv_mode[uartind] = RECV_MODE_NORMAL;
 			if (VERBOSE>2) LOG("TI-RS232/%d: leaving ESC mode, received: %c <%02x>, pass to UART\n", uartind+1, buffer, buffer);
-			tms9902_rcv_data(tms9902, buffer);
-			card->time_hold[uartind] = 0.0;
+			m_uart[uartind]->rcv_data(buffer);
+			m_time_hold[uartind] = 0.0;
 		}
 		else
 		{
@@ -811,10 +742,10 @@ static void receive_data_or_line_state(device_t *tms9902, double baudpoll)
 			if (buffer != 1)
 			{
 				LOG("TI-RS232/%d: expected length 1 but got %02x, leaving ESC mode.\n", uartind+1, buffer);
-				card->recv_mode[uartind] = RECV_MODE_NORMAL;
+				m_recv_mode[uartind] = RECV_MODE_NORMAL;
 			}
 			else
-				card->recv_mode[uartind] = RECV_MODE_ESC_LINES;
+				m_recv_mode[uartind] = RECV_MODE_ESC_LINES;
 		}
 		break;
 
@@ -824,58 +755,40 @@ static void receive_data_or_line_state(device_t *tms9902, double baudpoll)
 		if (buffer & EXCEPT)
 		{
 			// Exception states: BRK, FRMERR, PARERR
-			tms9902_rcv_break(tms9902, ((buffer & BRK)!=0));
+			m_uart[uartind]->rcv_break(((buffer & BRK)!=0));
 
-			if (buffer & FRMERR)
-				tms9902_rcv_framing_error(tms9902);
-			if (buffer & PARERR)
-				tms9902_rcv_parity_error(tms9902);
+			if (buffer & FRMERR)	m_uart[uartind]->rcv_framing_error();
+			if (buffer & PARERR)	m_uart[uartind]->rcv_parity_error();
 		}
 		else
 		{
-			buffer = map_lines_in(tms9902->owner(), uartind, buffer);
+			buffer = map_lines_in(uartind, buffer);
 			if (VERBOSE>2) LOG("TI-RS232/%d: received (remapped) <%02x> in ESC mode\n", uartind+1, buffer);
 
 			// The DTR line on the RS232 connector of the board is wired to both the
 			// CTS and the DSR pin of the TMS9902
 			// Apart from the data line, DTR is the only input line
-			incoming_dtr(tms9902,  (buffer & DTR)? ASSERT_LINE : CLEAR_LINE);
+			incoming_dtr(uartind,  (buffer & DTR)? ASSERT_LINE : CLEAR_LINE);
 		}
 
-		card->recv_mode[uartind] = RECV_MODE_NORMAL;
+		m_recv_mode[uartind] = RECV_MODE_NORMAL;
 		break;
 
 	default:
-		LOG("TI-RS232/%d: unknown mode: %d\n", uartind+1, card->recv_mode[uartind]);
+		if (VERBOSE>1) LOG("TI-RS232/%d: unknown mode: %d\n", uartind+1, m_recv_mode[uartind]);
 	}
 }
 
-static TMS9902_RCV_CALLBACK( rcv_callback )
-{
-	// Called from the UART when it wants to receive a character
-	// However, characters are not passed to it at this point
-	// Instead, we check for signal line change or data transmission
-	// and call the respective function
-	receive_data_or_line_state(device, baudpoll);
-}
-
-static TMS9902_XMIT_CALLBACK( xmit_callback )
-{
-	transmit_data(device, (UINT8)data);
-}
-
 /*
     Control operations like configuration or line changes
 */
-static void configure_interface(device_t *tms9902, int type, int value)
+void ti_rs232_pio_device::configure_interface(int uartind, int type, int value)
 {
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
 	UINT8 bufctrl[4];
 	device_image_interface *serial;
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
 	UINT8 esc = ESC;
 
-	serial = dynamic_cast<device_image_interface *>(card->serdev[uartind]);
+	serial = dynamic_cast<device_image_interface *>(m_serdev[uartind]);
 
 	if (!serial->exists())
 	{
@@ -928,38 +841,32 @@ static void configure_interface(device_t *tms9902, int type, int value)
 	serial->fwrite(bufctrl, bufctrl[0]+1);
 }
 
-static void set_bit(device_t *device, int uartind, int line, int value)
+void ti_rs232_pio_device::set_bit(int uartind, int line, int value)
 {
-	ti_rs232_state *card = get_safe_token(device);
 	if (VERBOSE>5)
 	{
 		switch (line)
 		{
-		case CTS: LOG("TI-RS232/%d: set CTS(out)=%s\n", uartind+1, (value)? "asserted" : "cleared"); break;
-		case DCD: LOG("TI-RS232/%d: set DCD(out)=%s\n", uartind+1, (value)? "asserted" : "cleared"); break;
-		case BRK: LOG("TI-RS232/%d: set BRK(out)=%s\n", uartind+1, (value)? "asserted" : "cleared"); break;
+		case CTS: LOG("TI-RS232/%d: set CTS(out)=%s\n", uartind+1, (value!=0)? "asserted" : "cleared"); break;
+		case DCD: LOG("TI-RS232/%d: set DCD(out)=%s\n", uartind+1, (value!=0)? "asserted" : "cleared"); break;
+		case BRK: LOG("TI-RS232/%d: set BRK(out)=%s\n", uartind+1, (value!=0)? "asserted" : "cleared"); break;
 		}
 	}
 
-	if (value)
-		card->signals[uartind] |= line;
-	else
-		card->signals[uartind] &= ~line;
+	if (value!=0)	m_signals[uartind] |= line;
+	else			m_signals[uartind] &= ~line;
 }
 
 /*
    Line changes
 */
-static void output_exception(device_t *tms9902, int param, UINT8 value)
+void ti_rs232_pio_device::output_exception(int uartind, int param, UINT8 value)
 {
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
 	device_image_interface *serial;
 	UINT8 bufctrl[2];
 	UINT8 esc = ESC;
 
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
-
-	serial = dynamic_cast<device_image_interface *>(card->serdev[uartind]);
+	serial = dynamic_cast<device_image_interface *>(m_serdev[uartind]);
 
 	if (!serial->exists())
 	{
@@ -979,16 +886,13 @@ static void output_exception(device_t *tms9902, int param, UINT8 value)
 /*
    Line changes
 */
-static void output_line_state(device_t *tms9902, int mask, UINT8 value)
+void ti_rs232_pio_device::output_line_state(int uartind, int mask, UINT8 value)
 {
-	ti_rs232_state *card = get_safe_token(tms9902->owner());
 	device_image_interface *serial;
 	UINT8 bufctrl[2];
 	UINT8 esc = ESC;
 
-	int uartind = (tms9902==card->uart[0])? 0 : 1;
-
-	serial = dynamic_cast<device_image_interface *>(card->serdev[uartind]);
+	serial = dynamic_cast<device_image_interface *>(m_serdev[uartind]);
 
 	if (!serial->exists())
 	{
@@ -1002,13 +906,13 @@ static void output_line_state(device_t *tms9902, int mask, UINT8 value)
 	bufctrl[0] = 1;
 
 	// The CTS line (coming from a CRU bit) is connected to the CTS pin
-	if (mask & CTS) set_bit(tms9902->owner(), uartind, CTS, value & CTS);
+	if (mask & CTS) set_bit(uartind, CTS, value & CTS);
 
 	// The RTS line (from 9902) is connected to the DCD pin
-	if (mask & RTS) set_bit(tms9902->owner(), uartind, DCD, value & RTS);
+	if (mask & RTS) set_bit(uartind, DCD, value & RTS);
 
 	// The DSR pin is hardwired to +5V
-	set_bit(tms9902->owner(), uartind, DSR, 1);
+	set_bit(uartind, DSR, 1);
 
 	// As of here, the lines are set according to the schematics of the
 	// serial interface.
@@ -1016,189 +920,206 @@ static void output_line_state(device_t *tms9902, int mask, UINT8 value)
 	// Now translate the signals of the board to those of a DTE-like device
 	// so that we can pass the signal to the real PC serial interface
 	// (can be imagined as if we emulated the cable)
-	bufctrl[1] = map_lines_out(tms9902->owner(), uartind, card->signals[uartind]);
+	bufctrl[1] = map_lines_out(uartind, m_signals[uartind]);
 	serial->fwrite(bufctrl, 2);
 }
 
-static TMS9902_CTRL_CALLBACK( ctrl_callback )
-{
-	switch (type)
-	{
-	case CONFIG:
-		configure_interface(device, param, value); break;
-	case EXCEPT:
-		output_exception(device, param, value); break;
-	default:
-		output_line_state(device, param, value);
-	}
-}
-
-
-static WRITE_LINE_DEVICE_HANDLER( senila )
-{
-	// put the value on the data bus. We store it in a state variable.
-	ti_rs232_state *card = get_safe_token(device);
-	if (VERBOSE>3) LOG("TI-RS232/1-2: Setting SENILA=%d\n", state);
-	card->senila = state;
-}
-
-static const tms9902_interface tms9902_params =
-{
-	3000000.,				/* clock rate (3MHz) */
-	int_callback,			/* called when interrupt pin state changes */
-	rcv_callback,			/* called when a character is received */
-	xmit_callback,			/* called when a character is transmitted */
-	ctrl_callback
-};
-
-static const ti99_peb_card tirs232_card =
-{
-	data_rz, data_w,				// memory access read/write
-	cru_rz, cru_w,					// CRU access
-	senila, NULL,					// SENILA/B access
-	NULL, NULL						// 16 bit access (none here)
-};
-
+/***********************************************************************
+    callbacks
+************************************************************************/
 /*
-    Defines the serial serdev.
+    Propagates the /INT signal of the UARTs to the /INT line of the pbox.
 */
-DEVICE_GET_INFO( ti99_rs232_serdev )
+WRITE_LINE_MEMBER( ti_rs232_pio_device::int0_callback )
 {
-	switch ( state )
-	{
-		case DEVINFO_INT_INLINE_CONFIG_BYTES:		info->i = 0;								break;
-		case DEVINFO_INT_IMAGE_TYPE:	            info->i = IO_SERIAL;                                      break;
-		case DEVINFO_INT_IMAGE_READABLE:            info->i = 1;                                               break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:			info->i = 1;                                               break;
-		case DEVINFO_INT_IMAGE_CREATABLE:	    	info->i = 1;                                               break;
+	int senila_bit = SENILA_0_BIT;
 
-		case DEVINFO_FCT_START:		                info->start = DEVICE_START_NAME( ti99_rs232_serdev );              break;
-//      case DEVINFO_FCT_RESET:                     info->reset = DEVICE_RESET_NAME( ti99_rs232_serdev );           break;
-		case DEVINFO_FCT_IMAGE_LOAD:		        info->f = (genf *) DEVICE_IMAGE_LOAD_NAME( ti99_rs232_serdev );    break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:		        info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME( ti99_rs232_serdev );    break;
-		case DEVINFO_STR_NAME:		                strcpy( info->s, "TI99 RS232 attached serial device");	                         break;
-		case DEVINFO_STR_FAMILY:                    strcpy(info->s, "Peripheral expansion");	                         break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:	    strcpy(info->s, "");                                           break;
-		case DEVINFO_STR_VERSION:						strcpy(info->s, "1.0");										break;
-		case DEVINFO_STR_SOURCE_FILE:					strcpy(info->s, __FILE__);									break;
-		case DEVINFO_STR_CREDITS:						strcpy(info->s, "Copyright the MESS Team"); 		break;
-	}
+	if (state==ASSERT_LINE)	m_ila |= senila_bit;
+	else m_ila &= ~senila_bit;
+
+	m_slot->set_inta(state);
+}
+
+WRITE_LINE_MEMBER( ti_rs232_pio_device::int1_callback )
+{
+	int senila_bit = SENILA_1_BIT;
+
+	if (state==ASSERT_LINE)	m_ila |= senila_bit;
+	else m_ila &= ~senila_bit;
+
+	m_slot->set_inta(state);
 }
 
 /*
-    Defines the PIO (parallel IO) serdev
+    Called from the UART when it wants to receive a character
+    However, characters are not passed to it at this point
+    Instead, we check for signal line change or data transmission
+    and call the respective function
 */
-DEVICE_GET_INFO( ti99_piodev )
+WRITE_LINE_MEMBER( ti_rs232_pio_device::rcv0_callback )
 {
-	switch ( state )
-	{
-		case DEVINFO_INT_INLINE_CONFIG_BYTES:		info->i = 0;								break;
-		case DEVINFO_INT_IMAGE_TYPE:	            info->i = IO_PARALLEL;                                      break;
-		case DEVINFO_INT_IMAGE_READABLE:            info->i = 1;                                               break;
-		case DEVINFO_INT_IMAGE_WRITEABLE:			info->i = 1;                                               break;
-		case DEVINFO_INT_IMAGE_CREATABLE:	    	info->i = 1;                                               break;
+	receive_data_or_line_state(0);
+}
 
-		case DEVINFO_FCT_START:		                info->start = DEVICE_START_NAME( ti99_piodev );              break;
-		case DEVINFO_FCT_IMAGE_LOAD:		        info->f = (genf *) DEVICE_IMAGE_LOAD_NAME( ti99_piodev );    break;
-		case DEVINFO_FCT_IMAGE_UNLOAD:		        info->f = (genf *) DEVICE_IMAGE_UNLOAD_NAME(ti99_piodev );  break;
-		case DEVINFO_STR_NAME:		                strcpy( info->s, "TI99 PIO attached device");	                         break;
-		case DEVINFO_STR_FAMILY:                    strcpy(info->s, "Peripheral expansion");	                         break;
-		case DEVINFO_STR_IMAGE_FILE_EXTENSIONS:	    strcpy(info->s, "");                                           break;
-		case DEVINFO_STR_VERSION:						strcpy(info->s, "1.0");										break;
-		case DEVINFO_STR_SOURCE_FILE:					strcpy(info->s, __FILE__);									break;
-		case DEVINFO_STR_CREDITS:						strcpy(info->s, "Copyright the MESS Team"); 		break;
+WRITE_LINE_MEMBER( ti_rs232_pio_device::rcv1_callback )
+{
+	receive_data_or_line_state(1);
+}
+
+WRITE8_MEMBER( ti_rs232_pio_device::xmit0_callback )
+{
+	transmit_data(0, data);
+}
+
+WRITE8_MEMBER( ti_rs232_pio_device::xmit1_callback )
+{
+	transmit_data(1, data);
+}
+
+void ti_rs232_pio_device::ctrl_callback(int uartind, int offset, UINT8 data)
+{
+	if ((offset & CONFIG)!=0)
+	{
+		// We cannot pass the configuration data as they need more than 8 bits.
+		// Could be done by a write16 function as well.
+		configure_interface(uartind, data, m_uart[uartind]->get_config_value());
+	}
+	else
+	{
+		if ((offset & EXCEPT)!=0)
+		{
+			output_exception(uartind, offset & ~EXCEPT, data);
+		}
+		else
+		{
+			output_line_state(uartind, offset, data);
+		}
 	}
 }
 
-DECLARE_LEGACY_IMAGE_DEVICE(TI99_RS232, ti99_rs232_serdev);
-DEFINE_LEGACY_IMAGE_DEVICE(TI99_RS232, ti99_rs232_serdev);
-DECLARE_LEGACY_IMAGE_DEVICE(TI99_PIO, ti99_piodev);
-DEFINE_LEGACY_IMAGE_DEVICE(TI99_PIO, ti99_piodev);
-
-static DEVICE_START( ti_rs232 )
+WRITE8_MEMBER( ti_rs232_pio_device::ctrl0_callback )
 {
-	ti_rs232_state *card = get_safe_token(device);
-	peb_callback_if *topeb = (peb_callback_if *)device->static_config();
-
-	astring region;
-	astring_assemble_3(&region, device->tag(), ":", ser_region);
-	card->rom = device->machine().region(region.cstr())->base();
-	card->lines.inta.resolve(topeb->inta, *device);
-	// READY and INTB are not used
-	card->uart[0] = device->subdevice("tms9902_0");
-	card->uart[1] = device->subdevice("tms9902_1");
-	card->serdev[0] = device->subdevice("serdev0");
-	card->serdev[1] = device->subdevice("serdev1");
-	card->piodev = device->subdevice("piodev");
+	ctrl_callback(0, offset, data);
 }
 
-static DEVICE_STOP( ti_rs232 )
+WRITE8_MEMBER( ti_rs232_pio_device::ctrl1_callback )
 {
+	ctrl_callback(1, offset, data);
 }
 
-static DEVICE_RESET( ti_rs232 )
+static const tms9902_interface tms9902_params0 =
 {
-	ti_rs232_state *card = get_safe_token(device);
-	/* Register the card */
-	device_t *peb = device->owner();
+	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, int0_callback),			/* called when interrupt pin state changes */
+	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, rcv0_callback),			/* called when a character is received */
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, xmit0_callback),			/* called when a character is transmitted */
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, ctrl0_callback)
+};
 
-	if (input_port_read(device->machine(), "SERIAL")==SERIAL_TI)
+static const tms9902_interface tms9902_params1 =
+{
+	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, int1_callback),			/* called when interrupt pin state changes */
+	DEVCB_DEVICE_LINE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, rcv1_callback),			/* called when a character is received */
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, xmit1_callback),			/* called when a character is transmitted */
+	DEVCB_DEVICE_MEMBER(DEVICE_SELF_OWNER, ti_rs232_pio_device, ctrl1_callback)
+};
+
+void ti_rs232_pio_device::device_start()
+{
+	m_dsrrom = memregion(DSRROM)->base();
+	m_uart[0] = subdevice<tms9902_device>("tms9902_0");
+	m_uart[1] = subdevice<tms9902_device>("tms9902_1");
+	m_serdev[0] = subdevice<ti_rs232_attached_device>("serdev0");
+	m_serdev[1] = subdevice<ti_rs232_attached_device>("serdev1");
+	m_piodev = subdevice<ti_pio_attached_device>("piodev");
+}
+
+void ti_rs232_pio_device::device_reset()
+{
+	m_pio_direction_in = false;
+	m_pio_handshakeout = false;
+	m_pio_spareout = false;
+	m_flag0 = false;
+
+	set_bit(0, CTS, 0);
+	set_bit(1, CTS, 0);
+
+	m_led = false;
+	m_pio_write = true;
+	m_recv_mode[0] = RECV_MODE_NORMAL;
+	m_recv_mode[1] = RECV_MODE_NORMAL;
+
+	if (m_genmod)
 	{
-		int success = mount_card(peb, device, &tirs232_card, get_pebcard_config(device)->slot);
-		if (!success) return;
+		m_select_mask = 0x1fe000;
+		m_select_value = 0x174000;
+	}
+	else
+	{
+		m_select_mask = 0x7e000;
+		m_select_value = 0x74000;
+	}
 
-		card->pio_direction = 0;
-		card->pio_handshakeout = 0;
-		card->pio_spareout = 0;
-		card->flag0 = 0;
+	m_space = machine().device("maincpu")->memory().space(AS_IO);
+	m_selected = false;
 
-		set_bit(device, 0, CTS, 0);
-		set_bit(device, 1, CTS, 0);
+	m_cru_base = (ioport("CRURS232")->read()==0)? 0x1300 : 0x1500;
 
-		card->led = 0;
-		card->pio_write = 1;
-		card->recv_mode[0] = RECV_MODE_NORMAL;
-		card->recv_mode[1] = RECV_MODE_NORMAL;
-
-		card->select_mask = 0x7e000;
-		card->select_value = 0x74000;
-
-		card->time_hold[0] = card->time_hold[1] = 0.0;
+	m_time_hold[0] = m_time_hold[1] = 0.0;
 
 		// The GenMod modification changes the address bus width of the Geneve.
 		// All peripheral cards need to be manually modified to properly decode
 		// the wider address. The next lines perform this soldering job
 		// automagically.
-		if (input_port_read(device->machine(), "MODE")==GENMOD)
-		{
-			// GenMod card modification
-			card->select_mask = 0x1fe000;
-			card->select_value = 0x174000;
-		}
-	}
+		/* if (device->machine().root_device().ioport("MODE")->read()==GENMOD)
+        {
+            // GenMod card modification
+            card->select_mask = 0x1fe000;
+            card->select_value = 0x174000;
+        }*/
 }
 
 static MACHINE_CONFIG_FRAGMENT( ti_rs232 )
-	MCFG_TMS9902_ADD("tms9902_0", tms9902_params)
-	MCFG_TMS9902_ADD("tms9902_1", tms9902_params)
-	MCFG_DEVICE_ADD("serdev0", TI99_RS232, 0)
-	MCFG_DEVICE_ADD("serdev1", TI99_RS232, 0)
-	MCFG_DEVICE_ADD("piodev", TI99_PIO, 0)
+	MCFG_TMS9902_ADD("tms9902_0", tms9902_params0, 3000000)
+	MCFG_TMS9902_ADD("tms9902_1", tms9902_params1, 3000000)
+	MCFG_DEVICE_ADD("serdev0", TI99_RS232_DEV, 0)
+	MCFG_DEVICE_ADD("serdev1", TI99_RS232_DEV, 0)
+	MCFG_DEVICE_ADD("piodev", TI99_PIO_DEV, 0)
 MACHINE_CONFIG_END
 
 ROM_START( ti_rs232 )
-	ROM_REGION(0x1000, ser_region, 0)
-	ROM_LOAD_OPTIONAL("rs232.bin", 0x0000, 0x1000, CRC(eab382fb) SHA1(ee609a18a21f1a3ddab334e8798d5f2a0fcefa91)) /* TI rs232 DSR ROM */
+	ROM_REGION(0x1000, DSRROM, 0)
+	ROM_LOAD("rs232.bin", 0x0000, 0x1000, CRC(eab382fb) SHA1(ee609a18a21f1a3ddab334e8798d5f2a0fcefa91)) /* TI rs232 DSR ROM */
 ROM_END
 
-static const char DEVTEMPLATE_SOURCE[] = __FILE__;
+INPUT_PORTS_START( ti_rs232 )
+	PORT_START(	"CRURS232"	)
+	PORT_DIPNAME( 0x01, 0x00, "TI-RS232 CRU base" )
+		PORT_DIPSETTING(	0x00, "1300" )
+		PORT_DIPSETTING(	0x00, "1500" )
 
-#define DEVTEMPLATE_ID(p,s)             p##ti_rs232##s
-#define DEVTEMPLATE_FEATURES            DT_HAS_START | DT_HAS_STOP | DT_HAS_RESET | DT_HAS_INLINE_CONFIG | DT_HAS_ROM_REGION | DT_HAS_MACHINE_CONFIG
-#define DEVTEMPLATE_NAME                "TI99 RS232/PIO interface card"
-#define DEVTEMPLATE_SHORTNAME           "ti99rs232"
-#define DEVTEMPLATE_FAMILY              "Peripheral expansion"
-#include "devtempl.h"
+	PORT_START( "SERIALMAP" )
+	PORT_CONFNAME( 0x03, 0x00, "Serial cable pin mapping" )
+		PORT_CONFSETTING(    0x00, "6-20" )
+		PORT_CONFSETTING(    0x01, "8-20" )
+		PORT_CONFSETTING(    0x02, "5-20" )
+INPUT_PORTS_END
 
-DEFINE_LEGACY_DEVICE( TIRS232, ti_rs232 );
+machine_config_constructor ti_rs232_pio_device::device_mconfig_additions() const
+{
+	return MACHINE_CONFIG_NAME( ti_rs232 );
+}
+
+const rom_entry *ti_rs232_pio_device::device_rom_region() const
+{
+	return ROM_NAME( ti_rs232 );
+}
+
+ioport_constructor ti_rs232_pio_device::device_input_ports() const
+{
+	return INPUT_PORTS_NAME(ti_rs232);
+}
+
+const device_type TI99_RS232 = &device_creator<ti_rs232_pio_device>;
+const device_type TI99_RS232_DEV = &device_creator<ti_rs232_attached_device>;
+const device_type TI99_PIO_DEV = &device_creator<ti_pio_attached_device>;
 
