@@ -78,6 +78,7 @@ typedef enum
 #define UPD765_FDD_READY 0x040
 
 #define UPD765_MF	0x40
+#define UPD765_MT	0x80
 
 #define UPD765_RESET 0x080
 
@@ -689,6 +690,19 @@ static void upd765_set_dma_drq(device_t *device, int state)
 	upd765_change_flags(device, state ? UPD765_DMA_DRQ : 0, UPD765_DMA_DRQ);
 }
 
+READ_LINE_DEVICE_HANDLER( upd765_int_r )
+{
+	upd765_t *fdc = get_safe_token(device);
+
+	return (fdc->upd765_flags & UPD765_INT) ? 1 : 0;
+}
+
+READ_LINE_DEVICE_HANDLER( upd765_drq_r )
+{
+	upd765_t *fdc = get_safe_token(device);
+
+	return (fdc->upd765_flags & UPD765_DMA_DRQ) ? 1 : 0;
+}
 
 
 /* Drive ready */
@@ -1234,7 +1248,7 @@ static int upd765_sector_count_complete(device_t *device)
 
 
 	/* multi-track? */
-	if (fdc->upd765_command_bytes[0] & 0x080)
+	if (fdc->upd765_command_bytes[0] & UPD765_MT)
 	{
 		/* it appears that in multi-track mode,
         the EOT parameter of the command is ignored!? -
@@ -1256,6 +1270,7 @@ static int upd765_sector_count_complete(device_t *device)
 			} else {
 				if (fdc->side==1)
 					return 2;
+				return 1; // do not advance to next cylinder
 			}
 		}
 
@@ -1363,46 +1378,38 @@ static int upd765_sector_count_complete(device_t *device)
 static void	upd765_increment_sector(device_t *device)
 {
 	upd765_t *fdc = get_safe_token(device);
+
 	/* multi-track? */
-	if (fdc->upd765_command_bytes[0] & 0x080)
+	if (fdc->upd765_command_bytes[0] & UPD765_MT)
 	{
 		/* reached EOT? */
-		/* if (fdc->upd765_command_bytes[4]==fdc->upd765_command_bytes[6])*/
-		if (upd765_just_read_last_sector_on_track(device))
+		if (fdc->upd765_command_bytes[4] == fdc->upd765_command_bytes[6])
 		{
-			/* reset sector id to 1 */
-			fdc->upd765_command_bytes[4] = 1;
-
-			/* yes */
-			if(fdc->side == 1) {
-				fdc->upd765_command_bytes[3] = 0;
+			if (fdc->side == 1)
+			{
 				fdc->upd765_command_bytes[2]++;
-				return;
 			}
 
-			/* reached EOT */
-			/* change side to 1 */
-			fdc->side = 1;
-			/* set head to 1 for get next sector test */
-			fdc->upd765_command_bytes[3] = 1;
+			fdc->upd765_command_bytes[3] ^= 0x01;
+			fdc->upd765_command_bytes[4] = 1;
+			fdc->side = fdc->upd765_command_bytes[3] & 0x01;
 		}
 		else
 		{
-			/* increment */
 			fdc->upd765_command_bytes[4]++;
 		}
-
 	}
 	else
 	{
-		if (upd765_just_read_last_sector_on_track(device))
+		if (fdc->upd765_command_bytes[4] == fdc->upd765_command_bytes[6])
 		{
-			fdc->upd765_command_bytes[4] = 1;
 			fdc->upd765_command_bytes[2]++;
-			return;
+			fdc->upd765_command_bytes[4] = 1;
+		}
+		else
+		{
+			fdc->upd765_command_bytes[4]++;
 		}
-
-		fdc->upd765_command_bytes[4]++;
 	}
 }
 
@@ -1528,15 +1535,6 @@ static TIMER_CALLBACK(upd765_continue_command)
 				/* sector id == EOT */
 				UINT8 ddam;
 
-				/* nothing to write */
-				if ((fdc->upd765_transfer_bytes_remaining==0) && (fdc->upd765_flags & UPD765_DMA_MODE))
-				{
-					if (fdc->upd765_flags & UPD765_TC)
-						upd765_write_complete(device);
-					// if TC not set, what?
-					break;
-				}
-
 				ddam = 0;
 				if (fdc->command == 0x09)
 				{
@@ -1544,8 +1542,19 @@ static TIMER_CALLBACK(upd765_continue_command)
 				}
 
 				/* write data to disc */
+				// logerror("floppy_drive_write_sector_data side=%d sector=%d tc=%x\n", fdc->side, fdc->sector_id, fdc->upd765_flags & UPD765_TC);
 				floppy_drive_write_sector_data(current_image(device), fdc->side, fdc->sector_id,fdc->data_buffer,upd765_n_to_bytes(fdc->upd765_command_bytes[5]),ddam);
 
+				/* nothing to write */
+				if ((fdc->upd765_transfer_bytes_remaining==0) && (fdc->upd765_flags & UPD765_DMA_MODE))
+				{
+					if (fdc->upd765_flags & UPD765_TC)
+					{
+						upd765_write_complete(device);
+						break;
+					}
+				}
+
 				if (upd765_sector_count_complete(device))
 				{
 					upd765_increment_sector(device);
