@@ -1,16 +1,50 @@
+/***************************************************************************
+
+    Copyright Olivier Galibert
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+        * Redistributions in binary form must reproduce the above copyright
+          notice, this list of conditions and the following disclaimer in
+          the documentation and/or other materials provided with the
+          distribution.
+        * Neither the name 'MAME' nor the names of its contributors may be
+          used to endorse or promote products derived from this software
+          without specific prior written permission.
+
+    THIS SOFTWARE IS PROVIDED BY AARON GILES ''AS IS'' AND ANY EXPRESS OR
+    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+    DISCLAIMED. IN NO EVENT SHALL AARON GILES BE LIABLE FOR ANY DIRECT,
+    INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+    STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
+    IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+    POSSIBILITY OF SUCH DAMAGE.
+
+****************************************************************************/
+
 #include "wd1772.h"
 
 #include "debugger.h"
 
+const device_type WD1770x = &device_creator<wd1770_t>;
 const device_type WD1772x = &device_creator<wd1772_t>;
+const device_type WD1773x = &device_creator<wd1773_t>;
 
-const int wd1772_t::step_times[4] = { 48000, 96000, 16000, 24000 };
-
-wd1772_t::wd1772_t(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) : device_t(mconfig, WD1772x, "WD1772", tag, owner, clock)
+wd177x_t::wd177x_t(const machine_config &mconfig, device_type type, const char *name, const char *tag, device_t *owner, UINT32 clock) :
+	device_t(mconfig, type, name, tag, owner, clock)
 {
 }
 
-void wd1772_t::device_start()
+void wd177x_t::device_start()
 {
 	t_gen = timer_alloc(TM_GEN);
 	t_cmd = timer_alloc(TM_CMD);
@@ -33,7 +67,7 @@ void wd1772_t::device_start()
 	save_item(NAME(last_dir));
 }
 
-void wd1772_t::device_reset()
+void wd177x_t::device_reset()
 {
 	command = 0x00;
 	main_state = IDLE;
@@ -47,12 +81,12 @@ void wd1772_t::device_reset()
 	counter = 0;
 	status_type_1 = true;
 	last_dir = 1;
-	cur_live.tm = attotime::never;
 	intrq = false;
 	drq = false;
+	live_abort();
 }
 
-void wd1772_t::set_floppy(floppy_image_device *_floppy)
+void wd177x_t::set_floppy(floppy_image_device *_floppy)
 {
 	if(floppy == _floppy)
 		return;
@@ -65,27 +99,28 @@ void wd1772_t::set_floppy(floppy_image_device *_floppy)
 	floppy = _floppy;
 
 	if(floppy) {
-		floppy->mon_w(status & S_MON ? 0 : 1);
-		floppy->setup_index_pulse_cb(floppy_image_device::index_pulse_cb(FUNC(wd1772_t::index_callback), this));
+		if(has_motor())
+			floppy->mon_w(status & S_MON ? 0 : 1);
+		floppy->setup_index_pulse_cb(floppy_image_device::index_pulse_cb(FUNC(wd177x_t::index_callback), this));
 	}
 }
 
-void wd1772_t::setup_intrq_cb(line_cb cb)
+void wd177x_t::setup_intrq_cb(line_cb cb)
 {
 	intrq_cb = cb;
 }
 
-void wd1772_t::setup_drq_cb(line_cb cb)
+void wd177x_t::setup_drq_cb(line_cb cb)
 {
 	drq_cb = cb;
 }
 
-void wd1772_t::dden_w(bool _dden)
+void wd177x_t::dden_w(bool _dden)
 {
 	dden = _dden;
 }
 
-astring wd1772_t::tts(attotime t)
+astring wd177x_t::tts(attotime t)
 {
 	char buf[256];
 	int nsec = t.attoseconds / ATTOSECONDS_PER_NANOSECOND;
@@ -93,12 +128,12 @@ astring wd1772_t::tts(attotime t)
 	return buf;
 }
 
-astring wd1772_t::ttsn()
+astring wd177x_t::ttsn()
 {
 	return tts(machine().time());
 }
 
-void wd1772_t::device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr)
+void wd177x_t::device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr)
 {
 	live_sync();
 
@@ -112,7 +147,7 @@ void wd1772_t::device_timer(emu_timer &timer, device_timer_id id, int param, voi
 	general_continue();
 }
 
-void wd1772_t::command_end()
+void wd177x_t::command_end()
 {
 	main_state = sub_state = IDLE;
 	status &= ~S_BUSY;
@@ -122,16 +157,16 @@ void wd1772_t::command_end()
 		intrq_cb(intrq);
 }
 
-void wd1772_t::seek_start(int state)
+void wd177x_t::seek_start(int state)
 {
 	main_state = state;
 	status = (status & ~(S_CRC|S_RNF|S_SPIN)) | S_BUSY;
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = true;
 	seek_continue();
 }
 
-void wd1772_t::seek_continue()
+void wd177x_t::seek_continue()
 {
 	for(;;) {
 		switch(sub_state) {
@@ -152,7 +187,7 @@ void wd1772_t::seek_continue()
 			if(main_state == RESTORE && floppy && !floppy->trk00_r())
 				sub_state = SEEK_DONE;
 
-			if(main_state == SEEK && floppy && track == data)
+			if(main_state == SEEK && track == data)
 				sub_state = SEEK_DONE;
 
 			if(sub_state == SPINUP_DONE) {
@@ -174,7 +209,7 @@ void wd1772_t::seek_continue()
 			}
 			counter++;
 			sub_state = SEEK_WAIT_STEP_TIME;
-			delay_cycles(t_gen, step_times[command & 3]);
+			delay_cycles(t_gen, step_time(command & 3));
 			return;
 
 		case SEEK_WAIT_STEP_TIME:
@@ -223,7 +258,7 @@ void wd1772_t::seek_continue()
 			if(command & 0x04) {
 				sub_state = SCAN_ID;
 				counter = 0;
-				live_start(SEARCH_ADDRESS_MARK);
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
 			command_end();
@@ -231,12 +266,12 @@ void wd1772_t::seek_continue()
 
 		case SCAN_ID:
 			if(cur_live.idbuf[0] != track) {
-				live_start(SEARCH_ADDRESS_MARK);
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
 			if(cur_live.crc) {
 				status |= S_CRC;
-				live_start(SEARCH_ADDRESS_MARK);
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
 			command_end();
@@ -254,20 +289,30 @@ void wd1772_t::seek_continue()
 	}
 }
 
-void wd1772_t::read_sector_start()
+bool wd177x_t::sector_matches() const
+{
+	if(cur_live.idbuf[0] != track || cur_live.idbuf[2] != sector)
+		return false;
+	if(!has_side_check())
+		return true;
+	if(command & 8)
+		return cur_live.idbuf[1] & 1;
+	else
+		return !(cur_live.idbuf[1] & 1);
+}
+
+void wd177x_t::read_sector_start()
 {
 	main_state = READ_SECTOR;
 	status = (status & ~(S_CRC|S_LOST|S_RNF|S_WP|S_DDM)) | S_BUSY;
 	drop_drq();
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = false;
 	read_sector_continue();
 }
 
-void wd1772_t::read_sector_continue()
+void wd177x_t::read_sector_continue()
 {
-	static int size_codes[4] = { 128, 256, 512, 1024 };
-
 	for(;;) {
 		switch(sub_state) {
 		case SPINUP:
@@ -284,7 +329,7 @@ void wd1772_t::read_sector_continue()
 		case SPINUP_DONE:
 			if(command & 4) {
 				sub_state = SETTLE_WAIT;
-				delay_cycles(t_gen, 120000);
+				delay_cycles(t_gen, settle_time());
 				return;
 			} else {
 				sub_state = SETTLE_DONE;
@@ -297,22 +342,22 @@ void wd1772_t::read_sector_continue()
 		case SETTLE_DONE:
 			sub_state = SCAN_ID;
 			counter = 0;
-			live_start(SEARCH_ADDRESS_MARK);
+			live_start(SEARCH_ADDRESS_MARK_HEADER);
 			return;
 
 		case SCAN_ID:
-			if(cur_live.idbuf[0] != track || cur_live.idbuf[2] != sector) {
-				live_start(SEARCH_ADDRESS_MARK);
+			if(!sector_matches()) {
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
 			if(cur_live.crc) {
 				status |= S_CRC;
-				live_start(SEARCH_ADDRESS_MARK);
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
-			sector_size = size_codes[cur_live.idbuf[3] & 3];
+			sector_size = 128 << (cur_live.idbuf[3] & 3);
 			sub_state = SECTOR_READ;
-			live_start(SEARCH_ADDRESS_MARK);
+			live_start(SEARCH_ADDRESS_MARK_DATA);
 			return;
 
 		case SCAN_ID_FAILED:
@@ -324,9 +369,9 @@ void wd1772_t::read_sector_continue()
 			if(cur_live.crc)
 				status |= S_CRC;
 
-			if(command & 0x10) {
+			if(command & 0x10 && !(status & S_RNF)) {
 				sector++;
-				sub_state = SPINUP_DONE;
+				sub_state = SETTLE_DONE;
 			} else {
 				command_end();
 				return;
@@ -340,17 +385,17 @@ void wd1772_t::read_sector_continue()
 	}
 }
 
-void wd1772_t::read_track_start()
+void wd177x_t::read_track_start()
 {
 	main_state = READ_TRACK;
 	status = (status & ~(S_LOST|S_RNF)) | S_BUSY;
 	drop_drq();
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = false;
 	read_track_continue();
 }
 
-void wd1772_t::read_track_continue()
+void wd177x_t::read_track_continue()
 {
 	for(;;) {
 		switch(sub_state) {
@@ -368,7 +413,7 @@ void wd1772_t::read_track_continue()
 		case SPINUP_DONE:
 			if(command & 4) {
 				sub_state = SETTLE_WAIT;
-				delay_cycles(t_gen, 120000);
+				delay_cycles(t_gen, settle_time());
 				return;
 
 			} else {
@@ -389,7 +434,6 @@ void wd1772_t::read_track_continue()
 		case WAIT_INDEX_DONE:
 			sub_state = TRACK_DONE;
 			live_start(READ_TRACK_DATA);
-			cur_live.pll.start_writing(machine().time());
 			return;
 
 		case TRACK_DONE:
@@ -403,17 +447,17 @@ void wd1772_t::read_track_continue()
 	}
 }
 
-void wd1772_t::read_id_start()
+void wd177x_t::read_id_start()
 {
 	main_state = READ_ID;
 	status = (status & ~(S_WP|S_DDM|S_LOST|S_RNF)) | S_BUSY;
 	drop_drq();
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = false;
 	read_id_continue();
 }
 
-void wd1772_t::read_id_continue()
+void wd177x_t::read_id_continue()
 {
 	for(;;) {
 		switch(sub_state) {
@@ -431,7 +475,7 @@ void wd1772_t::read_id_continue()
 		case SPINUP_DONE:
 			if(command & 4) {
 				sub_state = SETTLE_WAIT;
-				delay_cycles(t_gen, 120000);
+				delay_cycles(t_gen, settle_time());
 				return;
 			} else {
 				sub_state = SETTLE_DONE;
@@ -444,7 +488,7 @@ void wd1772_t::read_id_continue()
 		case SETTLE_DONE:
 			sub_state = SCAN_ID;
 			counter = 0;
-			live_start(SEARCH_ADDRESS_MARK);
+			live_start(SEARCH_ADDRESS_MARK_HEADER);
 			return;
 
 		case SCAN_ID:
@@ -463,17 +507,17 @@ void wd1772_t::read_id_continue()
 	}
 }
 
-void wd1772_t::write_track_start()
+void wd177x_t::write_track_start()
 {
 	main_state = WRITE_TRACK;
 	status = (status & ~(S_WP|S_DDM|S_LOST|S_RNF)) | S_BUSY;
 	drop_drq();
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = false;
 	write_track_continue();
 }
 
-void wd1772_t::write_track_continue()
+void wd177x_t::write_track_continue()
 {
 	for(;;) {
 		switch(sub_state) {
@@ -491,7 +535,7 @@ void wd1772_t::write_track_continue()
 		case SPINUP_DONE:
 			if(command & 4) {
 				sub_state = SETTLE_WAIT;
-				delay_cycles(t_gen, 120000);
+				delay_cycles(t_gen, settle_time());
 				return;
 			} else {
 				sub_state = SETTLE_DONE;
@@ -525,6 +569,7 @@ void wd1772_t::write_track_continue()
 		case WAIT_INDEX_DONE:
 			sub_state = TRACK_DONE;
 			live_start(WRITE_TRACK_DATA);
+			cur_live.pll.start_writing(machine().time());
 			return;
 
 		case TRACK_DONE:
@@ -539,20 +584,18 @@ void wd1772_t::write_track_continue()
 }
 
 
-void wd1772_t::write_sector_start()
+void wd177x_t::write_sector_start()
 {
 	main_state = WRITE_SECTOR;
 	status = (status & ~(S_CRC|S_LOST|S_RNF|S_WP|S_DDM)) | S_BUSY;
 	drop_drq();
-	sub_state = SPINUP;
+	sub_state = has_motor() ? SPINUP : SPINUP_DONE;
 	status_type_1 = false;
 	write_sector_continue();
 }
 
-void wd1772_t::write_sector_continue()
+void wd177x_t::write_sector_continue()
 {
-	static int size_codes[4] = { 128, 256, 512, 1024 };
-
 	for(;;) {
 		switch(sub_state) {
 		case SPINUP:
@@ -569,7 +612,7 @@ void wd1772_t::write_sector_continue()
 		case SPINUP_DONE:
 			if(command & 4) {
 				sub_state = SETTLE_WAIT;
-				delay_cycles(t_gen, 120000);
+				delay_cycles(t_gen, settle_time());
 				return;
 			} else {
 				sub_state = SETTLE_DONE;
@@ -582,20 +625,20 @@ void wd1772_t::write_sector_continue()
 		case SETTLE_DONE:
 			sub_state = SCAN_ID;
 			counter = 0;
-			live_start(SEARCH_ADDRESS_MARK);
+			live_start(SEARCH_ADDRESS_MARK_HEADER);
 			return;
 
 		case SCAN_ID:
-			if(cur_live.idbuf[0] != track || cur_live.idbuf[2] != sector) {
-				live_start(SEARCH_ADDRESS_MARK);
+			if(!sector_matches()) {
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
 			if(cur_live.crc) {
 				status |= S_CRC;
-				live_start(SEARCH_ADDRESS_MARK);
+				live_start(SEARCH_ADDRESS_MARK_HEADER);
 				return;
 			}
-			sector_size = size_codes[cur_live.idbuf[3] & 3];
+			sector_size = 128 << (cur_live.idbuf[3] & 3);
 			sub_state = SECTOR_WRITE;
 			live_start(WRITE_SECTOR_PRE);
 			return;
@@ -622,7 +665,7 @@ void wd1772_t::write_sector_continue()
 	}
 }
 
-void wd1772_t::interrupt_start()
+void wd177x_t::interrupt_start()
 {
 	if(status & S_BUSY) {
 		main_state = sub_state = cur_live.state = IDLE;
@@ -636,7 +679,7 @@ void wd1772_t::interrupt_start()
 	}
 }
 
-void wd1772_t::general_continue()
+void wd177x_t::general_continue()
 {
 	if(cur_live.state != IDLE) {
 		live_run();
@@ -671,7 +714,7 @@ void wd1772_t::general_continue()
 	}
 }
 
-void wd1772_t::do_generic()
+void wd177x_t::do_generic()
 {
 	switch(sub_state) {
 	case IDLE:
@@ -702,7 +745,7 @@ void wd1772_t::do_generic()
 	}
 }
 
-void wd1772_t::do_cmd_w()
+void wd177x_t::do_cmd_w()
 {
 	//  fprintf(stderr, "%s: command %02x\n", ttsn().cstr(), cmd_buffer);
 
@@ -729,7 +772,7 @@ void wd1772_t::do_cmd_w()
 	}
 }
 
-void wd1772_t::cmd_w(UINT8 val)
+void wd177x_t::cmd_w(UINT8 val)
 {
 	if(intrq && ((command & 0xf8) != 0xd8)) {
 		intrq = false;
@@ -746,7 +789,7 @@ void wd1772_t::cmd_w(UINT8 val)
 	delay_cycles(t_cmd, dden ? 384 : 184);
 }
 
-UINT8 wd1772_t::status_r()
+UINT8 wd177x_t::status_r()
 {
 	if(intrq && ((command & 0xf8) != 0xd8)) {
 		intrq = false;
@@ -779,13 +822,13 @@ UINT8 wd1772_t::status_r()
 	return status;
 }
 
-void wd1772_t::do_track_w()
+void wd177x_t::do_track_w()
 {
 	track = track_buffer;
 	track_buffer = -1;
 }
 
-void wd1772_t::track_w(UINT8 val)
+void wd177x_t::track_w(UINT8 val)
 {
 	// No more than one write in flight
 	if(track_buffer != -1)
@@ -795,18 +838,18 @@ void wd1772_t::track_w(UINT8 val)
 	delay_cycles(t_track, dden ? 256 : 128);
 }
 
-UINT8 wd1772_t::track_r()
+UINT8 wd177x_t::track_r()
 {
 	return track;
 }
 
-void wd1772_t::do_sector_w()
+void wd177x_t::do_sector_w()
 {
 	sector = sector_buffer;
 	sector_buffer = -1;
 }
 
-void wd1772_t::sector_w(UINT8 val)
+void wd177x_t::sector_w(UINT8 val)
 {
 	// No more than one write in flight
 	if(sector_buffer != -1)
@@ -816,24 +859,24 @@ void wd1772_t::sector_w(UINT8 val)
 	delay_cycles(t_sector, dden ? 256 : 128);
 }
 
-UINT8 wd1772_t::sector_r()
+UINT8 wd177x_t::sector_r()
 {
 	return sector;
 }
 
-void wd1772_t::data_w(UINT8 val)
+void wd177x_t::data_w(UINT8 val)
 {
 	data = val;
 	drop_drq();
 }
 
-UINT8 wd1772_t::data_r()
+UINT8 wd177x_t::data_r()
 {
 	drop_drq();
 	return data;
 }
 
-void wd1772_t::gen_w(int reg, UINT8 val)
+void wd177x_t::gen_w(int reg, UINT8 val)
 {
 	switch(reg) {
 	case 0: cmd_w(val); break;
@@ -843,7 +886,7 @@ void wd1772_t::gen_w(int reg, UINT8 val)
 	}
 }
 
-UINT8 wd1772_t::gen_r(int reg)
+UINT8 wd177x_t::gen_r(int reg)
 {
 	switch(reg) {
 	case 0: return status_r(); break;
@@ -854,12 +897,12 @@ UINT8 wd1772_t::gen_r(int reg)
 	return 0xff;
 }
 
-void wd1772_t::delay_cycles(emu_timer *tm, int cycles)
+void wd177x_t::delay_cycles(emu_timer *tm, int cycles)
 {
 	tm->adjust(clocks_to_attotime(cycles));
 }
 
-void wd1772_t::spinup()
+void wd177x_t::spinup()
 {
 	if(command & 0x08)
 		sub_state = SPINUP_DONE;
@@ -874,7 +917,7 @@ void wd1772_t::spinup()
 
 }
 
-void wd1772_t::index_callback(floppy_image_device *floppy, int state)
+void wd177x_t::index_callback(floppy_image_device *floppy, int state)
 {
 	live_sync();
 
@@ -885,11 +928,13 @@ void wd1772_t::index_callback(floppy_image_device *floppy, int state)
 
 	switch(sub_state) {
 	case IDLE:
-		motor_timeout ++;
-		if(motor_timeout >= 5) {
-			status &= ~S_MON;
-			if(floppy)
-				floppy->mon_w(1);
+		if(has_motor()) {
+			motor_timeout ++;
+			if(motor_timeout >= 5) {
+				status &= ~S_MON;
+				if(floppy)
+					floppy->mon_w(1);
+			}
 		}
 		break;
 
@@ -924,8 +969,10 @@ void wd1772_t::index_callback(floppy_image_device *floppy, int state)
 
 	case SCAN_ID:
 		counter++;
-		if(counter == 5)
+		if(counter == 5) {
 			sub_state = SCAN_ID_FAILED;
+			live_abort();
+		}
 		break;
 
 	case WAIT_INDEX:
@@ -944,17 +991,17 @@ void wd1772_t::index_callback(floppy_image_device *floppy, int state)
 	general_continue();
 }
 
-bool wd1772_t::intrq_r()
+bool wd177x_t::intrq_r()
 {
 	return intrq;
 }
 
-bool wd1772_t::drq_r()
+bool wd177x_t::drq_r()
 {
 	return drq;
 }
 
-void wd1772_t::live_start(int state)
+void wd177x_t::live_start(int state)
 {
 	cur_live.tm = machine().time();
 	cur_live.state = state;
@@ -974,24 +1021,24 @@ void wd1772_t::live_start(int state)
 	live_run();
 }
 
-void wd1772_t::checkpoint()
+void wd177x_t::checkpoint()
 {
 	cur_live.pll.commit(floppy, cur_live.tm);
 	checkpoint_live = cur_live;
 }
 
-void wd1772_t::rollback()
+void wd177x_t::rollback()
 {
 	cur_live = checkpoint_live;
 }
 
-void wd1772_t::live_delay(int state)
+void wd177x_t::live_delay(int state)
 {
 	cur_live.next_state = state;
 	t_gen->adjust(cur_live.tm - machine().time());
 }
 
-void wd1772_t::live_sync()
+void wd177x_t::live_sync()
 {
 	if(!cur_live.tm.is_never()) {
 		if(cur_live.tm > machine().time()) {
@@ -1002,8 +1049,10 @@ void wd1772_t::live_sync()
 		} else {
 			//          fprintf(stderr, "%s: Committing (%s)\n", ttsn().cstr(), tts(cur_live.tm).cstr());
 			cur_live.pll.commit(floppy, cur_live.tm);
-			if(cur_live.next_state != -1)
+			if(cur_live.next_state != -1) {
 				cur_live.state = cur_live.next_state;
+				cur_live.next_state = -1;
+			}
 			if(cur_live.state == IDLE) {
 				cur_live.pll.stop_writing(floppy, cur_live.tm);
 				cur_live.tm = attotime::never;
@@ -1014,9 +1063,9 @@ void wd1772_t::live_sync()
 	}
 }
 
-void wd1772_t::live_abort()
+void wd177x_t::live_abort()
 {
-	if(cur_live.tm > machine().time()) {
+	if(!cur_live.tm.is_never() && cur_live.tm > machine().time()) {
 		rollback();
 		live_run(machine().time());
 	}
@@ -1027,7 +1076,7 @@ void wd1772_t::live_abort()
 	cur_live.next_state = -1;
 }
 
-bool wd1772_t::read_one_bit(attotime limit)
+bool wd177x_t::read_one_bit(attotime limit)
 {
 	int bit = cur_live.pll.get_next_bit(cur_live.tm, floppy, limit);
 	if(bit < 0)
@@ -1045,7 +1094,7 @@ bool wd1772_t::read_one_bit(attotime limit)
 	return false;
 }
 
-bool wd1772_t::write_one_bit(attotime limit)
+bool wd177x_t::write_one_bit(attotime limit)
 {
 	bool bit = cur_live.shift_reg & 0x8000;
 	if(cur_live.pll.write_next_bit(bit, cur_live.tm, floppy, limit))
@@ -1061,14 +1110,14 @@ bool wd1772_t::write_one_bit(attotime limit)
 	return false;
 }
 
-void wd1772_t::live_write_raw(UINT16 raw)
+void wd177x_t::live_write_raw(UINT16 raw)
 {
 	//  logerror("write %04x %04x\n", raw, cur_live.crc);
 	cur_live.shift_reg = raw;
 	cur_live.data_bit_context = raw & 1;
 }
 
-void wd1772_t::live_write_mfm(UINT8 mfm)
+void wd177x_t::live_write_mfm(UINT8 mfm)
 {
 	bool context = cur_live.data_bit_context;
 	UINT16 raw = 0;
@@ -1085,7 +1134,7 @@ void wd1772_t::live_write_mfm(UINT8 mfm)
 	//  logerror("write %02x   %04x %04x\n", mfm, cur_live.crc, raw);
 }
 
-void wd1772_t::live_run(attotime limit)
+void wd177x_t::live_run(attotime limit)
 {
 	if(cur_live.state == IDLE || cur_live.next_state != -1)
 		return;
@@ -1109,7 +1158,7 @@ void wd1772_t::live_run(attotime limit)
 
 	for(;;) {
 		switch(cur_live.state) {
-		case SEARCH_ADDRESS_MARK:
+		case SEARCH_ADDRESS_MARK_HEADER:
 			if(read_one_bit(limit))
 				return;
 #if 0
@@ -1125,16 +1174,15 @@ void wd1772_t::live_run(attotime limit)
 					cur_live.bit_counter);
 #endif
 
-			if(cur_live.shift_reg == 0x4489 || cur_live.shift_reg == 0x5224) {
-				if(cur_live.shift_reg == 0x4489)
-					cur_live.crc = 0x443b;
+			if(cur_live.shift_reg == 0x4489) {
+				cur_live.crc = 0x443b;
 				cur_live.data_separator_phase = false;
 				cur_live.bit_counter = 0;
-				cur_live.state = READ_BLOCK_HEADER;
+				cur_live.state = READ_HEADER_BLOCK_HEADER;
 			}
 			break;
 
-		case READ_BLOCK_HEADER:
+		case READ_HEADER_BLOCK_HEADER: {
 			if(read_one_bit(limit))
 				return;
 #if 0
@@ -1149,46 +1197,30 @@ void wd1772_t::live_run(attotime limit)
 					(cur_live.shift_reg & 0x0001 ? 0x01 : 0x00),
 					cur_live.bit_counter);
 #endif
-			if(cur_live.bit_counter != 16) {
-				if(cur_live.shift_reg == 0x4489 || cur_live.shift_reg == 0x5224) {
-					if(cur_live.shift_reg == 0x4489)
-						cur_live.crc = 0x443b;
-					cur_live.data_separator_phase = false;
-					cur_live.bit_counter = 0;
-				}
+			if(cur_live.bit_counter & 15)
+				break;
+
+			int slot = cur_live.bit_counter >> 4;
+
+			if(slot < 3) {
+				if(cur_live.shift_reg != 0x4489)
+					cur_live.state = SEARCH_ADDRESS_MARK_HEADER;
+				break;
+			}
+			if(cur_live.data_reg != 0xfe && cur_live.data_reg != 0xff) {
+				cur_live.state = SEARCH_ADDRESS_MARK_HEADER;
 				break;
 			}
 
 			cur_live.bit_counter = 0;
 
-			if(cur_live.shift_reg == 0x4489 || cur_live.shift_reg == 0x5224)
-				break;
-
-			switch(cur_live.data_reg & 0xfe) {
-			case 0xfa: // Sector data
-				if(sub_state == SECTOR_READ) {
-					cur_live.state = READ_SECTOR_DATA;
-					break;
-				}
-				cur_live.state = SEARCH_ADDRESS_MARK;
-				break;
-
-			case 0xfe: // ID
-				if(main_state == READ_ID) {
-					cur_live.state = READ_ID_BLOCK_TO_DMA;
-					break;
-				}
-				if(sub_state == SCAN_ID) {
-					cur_live.state = READ_ID_BLOCK_TO_LOCAL;
-					break;
-				}
-
-			default:
-				cur_live.state = SEARCH_ADDRESS_MARK;
-				break;
-			}
+			if(main_state == READ_ID)
+				cur_live.state = READ_ID_BLOCK_TO_DMA;
+			else
+				cur_live.state = READ_ID_BLOCK_TO_LOCAL;
 
 			break;
+		}
 
 		case READ_ID_BLOCK_TO_LOCAL: {
 			if(read_one_bit(limit))
@@ -1229,6 +1261,76 @@ void wd1772_t::live_run(attotime limit)
 			checkpoint();
 			break;
 
+		case SEARCH_ADDRESS_MARK_DATA:
+			if(read_one_bit(limit))
+				return;
+#if 0
+			fprintf(stderr, "%s: shift = %04x data=%02x c=%d.%x\n", tts(cur_live.tm).cstr(), cur_live.shift_reg,
+					(cur_live.shift_reg & 0x4000 ? 0x80 : 0x00) |
+					(cur_live.shift_reg & 0x1000 ? 0x40 : 0x00) |
+					(cur_live.shift_reg & 0x0400 ? 0x20 : 0x00) |
+					(cur_live.shift_reg & 0x0100 ? 0x10 : 0x00) |
+					(cur_live.shift_reg & 0x0040 ? 0x08 : 0x00) |
+					(cur_live.shift_reg & 0x0010 ? 0x04 : 0x00) |
+					(cur_live.shift_reg & 0x0004 ? 0x02 : 0x00) |
+					(cur_live.shift_reg & 0x0001 ? 0x01 : 0x00),
+					cur_live.bit_counter >> 4, cur_live.bit_counter & 15);
+#endif
+			if(cur_live.bit_counter > 43*16) {
+				live_delay(SEARCH_ADDRESS_MARK_DATA_FAILED);
+				return;
+			}
+
+			if(cur_live.bit_counter >= 28*16 && cur_live.shift_reg == 0x4489) {
+				cur_live.crc = 0x443b;
+				cur_live.data_separator_phase = false;
+				cur_live.bit_counter = 0;
+				cur_live.state = READ_DATA_BLOCK_HEADER;
+			}
+			break;
+
+		case READ_DATA_BLOCK_HEADER: {
+			if(read_one_bit(limit))
+				return;
+#if 0
+			fprintf(stderr, "%s: shift = %04x data=%02x counter=%d\n", tts(cur_live.tm).cstr(), cur_live.shift_reg,
+					(cur_live.shift_reg & 0x4000 ? 0x80 : 0x00) |
+					(cur_live.shift_reg & 0x1000 ? 0x40 : 0x00) |
+					(cur_live.shift_reg & 0x0400 ? 0x20 : 0x00) |
+					(cur_live.shift_reg & 0x0100 ? 0x10 : 0x00) |
+					(cur_live.shift_reg & 0x0040 ? 0x08 : 0x00) |
+					(cur_live.shift_reg & 0x0010 ? 0x04 : 0x00) |
+					(cur_live.shift_reg & 0x0004 ? 0x02 : 0x00) |
+					(cur_live.shift_reg & 0x0001 ? 0x01 : 0x00),
+					cur_live.bit_counter);
+#endif
+			if(cur_live.bit_counter & 15)
+				break;
+
+			int slot = cur_live.bit_counter >> 4;
+
+			if(slot < 3) {
+				if(cur_live.shift_reg != 0x4489) {
+					live_delay(SEARCH_ADDRESS_MARK_DATA_FAILED);
+					return;
+				}
+				break;
+			}
+			if((cur_live.data_reg & 0xfe) != 0xfa && (cur_live.data_reg & 0xfe) != 0xfc) {
+				live_delay(SEARCH_ADDRESS_MARK_DATA_FAILED);
+				return;
+			}
+
+			cur_live.bit_counter = 0;
+			cur_live.state = READ_SECTOR_DATA;
+			break;
+		}
+
+		case SEARCH_ADDRESS_MARK_DATA_FAILED:
+			status |= S_RNF;
+			cur_live.state = IDLE;
+			return;
+
 		case READ_SECTOR_DATA: {
 			if(read_one_bit(limit))
 				return;
@@ -1268,6 +1370,13 @@ void wd1772_t::live_run(attotime limit)
 
 			// Incorrect, hmmm
 			// Probably >2 + not just after a sync if <16
+
+			// Transitions 00..00 -> 4489.4489.4489 at varied syncs:
+			//  0: 00.00.14.a1   1: ff.fe.c2.a1   2: 00.01.14.a1   3: ff.fc.c2.a1
+			//  4: 00.02.14.a1   5: ff.f8.c2.a1   6: 00.05.14.a1   7: ff.f0.c2.a1
+			//  8: 00.00.0a.a1   9: ff.ff.e1.a1  10: 00.00.14.a1  11: ff.ff.ce.a1
+			// 12: 00.00.14.a1  13: ff.ff.c2.a1  14: 00.00.14.a1  15: ff.ff.c2.a1
+
 			bool output_byte = cur_live.bit_counter > 5;
 
 			cur_live.data_separator_phase = false;
@@ -1296,10 +1405,9 @@ void wd1772_t::live_run(attotime limit)
 
 			switch(data) {
 			case 0xf5:
-				if(cur_live.previous_type != live_info::PT_SYNC)
-					cur_live.crc = 0xffff;
 				live_write_raw(0x4489);
-				cur_live.previous_type = live_info::PT_SYNC;
+				cur_live.crc = 0x968b; // Ensures that the crc is cdb4 after writing the byte
+				cur_live.previous_type = live_info::PT_NONE;
 				break;
 			case 0xf6:
 				cur_live.previous_type = live_info::PT_NONE;
@@ -1436,7 +1544,7 @@ void wd1772_t::live_run(attotime limit)
 	}
 }
 
-void wd1772_t::set_drq()
+void wd177x_t::set_drq()
 {
 	if(drq)
 		status |= S_LOST;
@@ -1447,7 +1555,7 @@ void wd1772_t::set_drq()
 	}
 }
 
-void wd1772_t::drop_drq()
+void wd177x_t::drop_drq()
 {
 	if(drq) {
 		drq = false;
@@ -1456,13 +1564,13 @@ void wd1772_t::drop_drq()
 	}
 }
 
-void wd1772_t::pll_t::set_clock(attotime period)
+void wd177x_t::pll_t::set_clock(attotime period)
 {
 	for(int i=0; i<42; i++)
 		delays[i] = period*(i+1);
 }
 
-void wd1772_t::pll_t::reset(attotime when)
+void wd177x_t::pll_t::reset(attotime when)
 {
 	counter = 0;
 	increment = 128;
@@ -1478,7 +1586,7 @@ void wd1772_t::pll_t::reset(attotime when)
 	write_start_time = attotime::never;
 }
 
-int wd1772_t::pll_t::get_next_bit(attotime &tm, floppy_image_device *floppy, attotime limit)
+int wd177x_t::pll_t::get_next_bit(attotime &tm, floppy_image_device *floppy, attotime limit)
 {
 	attotime when = floppy ? floppy->get_next_transition(ctime) : attotime::never;
 #if 0
@@ -1558,21 +1666,21 @@ int wd1772_t::pll_t::get_next_bit(attotime &tm, floppy_image_device *floppy, att
 	return bit;
 }
 
-void wd1772_t::pll_t::start_writing(attotime tm)
+void wd177x_t::pll_t::start_writing(attotime tm)
 {
 	write_start_time = tm;
 	write_position = 0;
 }
 
-void wd1772_t::pll_t::stop_writing(floppy_image_device *floppy, attotime tm)
+void wd177x_t::pll_t::stop_writing(floppy_image_device *floppy, attotime tm)
 {
 	commit(floppy, tm);
 	write_start_time = attotime::never;
 }
 
-bool wd1772_t::pll_t::write_next_bit(bool bit, attotime &tm, floppy_image_device *floppy, attotime limit)
+bool wd177x_t::pll_t::write_next_bit(bool bit, attotime &tm, floppy_image_device *floppy, attotime limit)
 {
-	if(write_start_time == attotime::never) {
+	if(write_start_time.is_never()) {
 		write_start_time = ctime;
 		write_position = 0;
 	}
@@ -1600,9 +1708,9 @@ bool wd1772_t::pll_t::write_next_bit(bool bit, attotime &tm, floppy_image_device
 	return false;
 }
 
-void wd1772_t::pll_t::commit(floppy_image_device *floppy, attotime tm)
+void wd177x_t::pll_t::commit(floppy_image_device *floppy, attotime tm)
 {
-	if(write_start_time == attotime::never || tm == write_start_time)
+	if(write_start_time.is_never() || tm == write_start_time)
 		return;
 
 	if(floppy)
@@ -1610,3 +1718,62 @@ void wd1772_t::pll_t::commit(floppy_image_device *floppy, attotime tm)
 	write_start_time = tm;
 	write_position = 0;
 }
+
+int wd177x_t::step_time(int mode) const
+{
+	const static int step_times[4] = { 48000, 96000, 160000, 240000 };
+	return step_times[mode];
+}
+
+int wd177x_t::settle_time() const
+{
+	return 240000;
+}
+
+bool wd177x_t::has_side_check() const
+{
+	return false;
+}
+
+wd1770_t::wd1770_t(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) : wd177x_t(mconfig, WD1770x, "WD1770", tag, owner, clock)
+{
+}
+
+bool wd1770_t::has_motor() const
+{
+	return true;
+}
+
+wd1772_t::wd1772_t(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) : wd177x_t(mconfig, WD1772x, "WD1772", tag, owner, clock)
+{
+}
+
+bool wd1772_t::has_motor() const
+{
+	return true;
+}
+
+int wd1772_t::step_time(int mode) const
+{
+	const static int step_times[4] = { 48000, 96000, 16000, 24000 };
+	return step_times[mode];
+}
+
+int wd1772_t::settle_time() const
+{
+	return 120000;
+}
+
+wd1773_t::wd1773_t(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) : wd177x_t(mconfig, WD1773x, "WD1773", tag, owner, clock)
+{
+}
+
+bool wd1773_t::has_motor() const
+{
+	return false;
+}
+
+bool wd1773_t::has_side_check() const
+{
+	return false;
+}
