@@ -36,7 +36,7 @@ public:
 protected:
 	// device-level overrides
 	virtual void device_start();
-	virtual void device_reset() {};
+	virtual void device_reset();
     virtual void device_config_complete() { m_shortname = "x68k_neptunex"; }
 
 private:
