@@ -9,7 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "imgtoolx.h"
+#include "imgtool.h"
 #include "formats/imageutl.h"
 #include "library.h"
 #include "modules.h"
@@ -85,6 +85,13 @@ static int global_omit_untested;
 static void (*global_warn)(const char *message);
 
 
+void CLIB_DECL logerror(const char *format, ...)
+{
+	va_list arg;
+	va_start(arg, format);
+	printf(format, arg);
+	va_end(arg);
+}
 
 /***************************************************************************
 
