@@ -178,7 +178,7 @@ static void apply_char_overrides(int nb_char_overrides, const char_override_t ch
 void vdt911_init(running_machine &machine)
 {
 	UINT8 *base;
-	UINT8 *chr = machine.region(vdt911_chr_region)->base();
+	UINT8 *chr = machine.root_device().memregion(vdt911_chr_region)->base();
 
 	/* set up US character definitions */
 	base = chr+vdt911_US_chr_offset;
@@ -494,7 +494,7 @@ WRITE8_DEVICE_HANDLER( vdt911_cru_w )
 /*
     Video refresh
 */
-void vdt911_refresh(device_t *device, bitmap_t *bitmap, int x, int y)
+void vdt911_refresh(device_t *device, bitmap_ind16 &bitmap, const rectangle &cliprect, int x, int y)
 {
 	vdt_t *vdt = get_safe_token(device);
 	const gfx_element *gfx = device->machine().gfx[vdt->model];
@@ -510,15 +510,9 @@ void vdt911_refresh(device_t *device, bitmap_t *bitmap, int x, int y)
 
 	if (! vdt->display_enable)
 	{
-		rectangle my_rect;
+		rectangle my_rect(x, x + 80*7 - 1, y, y + height*10 - 1);
 
-		my_rect.min_x = x;
-		my_rect.max_x = x + 80*7 - 1;
-
-		my_rect.min_y = y;
-		my_rect.max_y = y + height*10 - 1;
-
-		bitmap_fill(bitmap, &my_rect, 0);
+		bitmap.fill(0, my_rect);
 	}
 	else
 		for (i=0; i<height; i++)
@@ -538,7 +532,7 @@ void vdt911_refresh(device_t *device, bitmap_t *bitmap, int x, int y)
 
 				address++;
 
-				drawgfx_opaque(bitmap, NULL, gfx, cur_char, color, 0, 0,
+				drawgfx_opaque(bitmap, cliprect, gfx, cur_char, color, 0, 0,
 						x+j*7, y+i*10);
 			}
 		}
@@ -597,7 +591,7 @@ void vdt911_keyboard(device_t *device)
 	/* read current key state */
 	for (i = 0; i < 6; i++)
 	{
-		key_buf[i] = input_port_read(device->machine(), keynames[i]);
+		key_buf[i] = device->machine().root_device().ioport(keynames[i])->read();
 	}
 
 	/* parse modifier keys */
