@@ -26,33 +26,33 @@ VIDEO_START( ac1 )
 {
 }
 
-SCREEN_UPDATE( ac1 )
+SCREEN_UPDATE_IND16( ac1 )
 {
 	int x,y;
-	address_space *space = screen->machine().device("maincpu")->memory().space(AS_PROGRAM);
+	address_space *space = screen.machine().device("maincpu")->memory().space(AS_PROGRAM);
 
 	for(y = 0; y < 16; y++ )
 	{
 		for(x = 0; x < 64; x++ )
 		{
 			int code = space->read_byte(AC1_VIDEO_MEMORY + x + y*64);
-			drawgfx_opaque(bitmap, NULL, screen->machine().gfx[0],  code , 0, 0,0, 63*6-x*6,15*8-y*8);
+			drawgfx_opaque(bitmap, cliprect, screen.machine().gfx[0],  code , 0, 0,0, 63*6-x*6,15*8-y*8);
 		}
 	}
 	return 0;
 }
 
-SCREEN_UPDATE( ac1_32 )
+SCREEN_UPDATE_IND16( ac1_32 )
 {
 	int x,y;
-	address_space *space = screen->machine().device("maincpu")->memory().space(AS_PROGRAM);
+	address_space *space = screen.machine().device("maincpu")->memory().space(AS_PROGRAM);
 
 	for(y = 0; y < 32; y++ )
 	{
 		for(x = 0; x < 64; x++ )
 		{
 			int code = space->read_byte(AC1_VIDEO_MEMORY + x + y*64);
-			drawgfx_opaque(bitmap, NULL, screen->machine().gfx[0],  code , 0, 0,0, 63*6-x*6,31*8-y*8);
+			drawgfx_opaque(bitmap, cliprect, screen.machine().gfx[0],  code , 0, 0,0, 63*6-x*6,31*8-y*8);
 		}
 	}
 	return 0;
