@@ -158,7 +158,7 @@ void st_state::shifter_tick()
 		break;
 	}
 
-	*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x) = pen;
+	m_bitmap.pix32(y, x) = pen;
 }
 
 
@@ -251,36 +251,36 @@ void st_state::glue_tick()
 	{
 	case 0:
 		pen = shift_mode_0();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x) = pen;
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+1) = pen;
+		m_bitmap.pix32(y, x) = pen;
+		m_bitmap.pix32(y, x+1) = pen;
 		pen = shift_mode_0();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+2) = pen;
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+3) = pen;
+		m_bitmap.pix32(y, x+2) = pen;
+		m_bitmap.pix32(y, x+3) = pen;
 		pen = shift_mode_0();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+4) = pen;
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+5) = pen;
+		m_bitmap.pix32(y, x+4) = pen;
+		m_bitmap.pix32(y, x+5) = pen;
 		pen = shift_mode_0();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+6) = pen;
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+7) = pen;
+		m_bitmap.pix32(y, x+6) = pen;
+		m_bitmap.pix32(y, x+7) = pen;
 		break;
 
 	case 1:
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x) = pen;
+		m_bitmap.pix32(y, x) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+1) = pen;
+		m_bitmap.pix32(y, x+1) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+2) = pen;
+		m_bitmap.pix32(y, x+2) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+3) = pen;
+		m_bitmap.pix32(y, x+3) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+4) = pen;
+		m_bitmap.pix32(y, x+4) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+5) = pen;
+		m_bitmap.pix32(y, x+5) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+6) = pen;
+		m_bitmap.pix32(y, x+6) = pen;
 		pen = shift_mode_1();
-		*BITMAP_ADDR32(machine().generic.tmpbitmap, y, x+7) = pen;
+		m_bitmap.pix32(y, x+7) = pen;
 		break;
 
 	case 2:
@@ -1114,41 +1114,41 @@ void st_state::video_start()
 //  m_shifter_timer->adjust(machine().primary_screen->time_until_pos(0), 0, attotime::from_hz(Y2/4)); // 125 ns
 	m_glue_timer->adjust(machine().primary_screen->time_until_pos(0), 0, attotime::from_hz(Y2/16)); // 500 ns
 
-	/* register for state saving */
-	state_save_register_global(machine(), m_shifter_base);
-	state_save_register_global(machine(), m_shifter_ofs);
-	state_save_register_global(machine(), m_shifter_sync);
-	state_save_register_global(machine(), m_shifter_mode);
-	state_save_register_global_array(machine(), m_shifter_palette);
-	state_save_register_global_array(machine(), m_shifter_rr);
-	state_save_register_global_array(machine(), m_shifter_ir);
-	state_save_register_global(machine(), m_shifter_bitplane);
-	state_save_register_global(machine(), m_shifter_shift);
-	state_save_register_global(machine(), m_shifter_h);
-	state_save_register_global(machine(), m_shifter_v);
-	state_save_register_global(machine(), m_shifter_de);
+	machine().primary_screen->register_screen_bitmap(m_bitmap);
 
-	state_save_register_global_array(machine(), m_blitter_halftone);
-	state_save_register_global(machine(), m_blitter_src_inc_x);
-	state_save_register_global(machine(), m_blitter_src_inc_y);
-	state_save_register_global(machine(), m_blitter_dst_inc_x);
-	state_save_register_global(machine(), m_blitter_dst_inc_y);
-	state_save_register_global(machine(), m_blitter_src);
-	state_save_register_global(machine(), m_blitter_dst);
-	state_save_register_global(machine(), m_blitter_endmask1);
-	state_save_register_global(machine(), m_blitter_endmask2);
-	state_save_register_global(machine(), m_blitter_endmask3);
-	state_save_register_global(machine(), m_blitter_xcount);
-	state_save_register_global(machine(), m_blitter_ycount);
-	state_save_register_global(machine(), m_blitter_xcountl);
-	state_save_register_global(machine(), m_blitter_hop);
-	state_save_register_global(machine(), m_blitter_op);
-	state_save_register_global(machine(), m_blitter_ctrl);
-	state_save_register_global(machine(), m_blitter_skew);
+	/* register for state saving */
+	save_item(NAME(m_shifter_base));
+	save_item(NAME(m_shifter_ofs));
+	save_item(NAME(m_shifter_sync));
+	save_item(NAME(m_shifter_mode));
+	save_item(NAME(m_shifter_palette));
+	save_item(NAME(m_shifter_rr));
+	save_item(NAME(m_shifter_ir));
+	save_item(NAME(m_shifter_bitplane));
+	save_item(NAME(m_shifter_shift));
+	save_item(NAME(m_shifter_h));
+	save_item(NAME(m_shifter_v));
+	save_item(NAME(m_shifter_de));
+
+	save_item(NAME(m_blitter_halftone));
+	save_item(NAME(m_blitter_src_inc_x));
+	save_item(NAME(m_blitter_src_inc_y));
+	save_item(NAME(m_blitter_dst_inc_x));
+	save_item(NAME(m_blitter_dst_inc_y));
+	save_item(NAME(m_blitter_src));
+	save_item(NAME(m_blitter_dst));
+	save_item(NAME(m_blitter_endmask1));
+	save_item(NAME(m_blitter_endmask2));
+	save_item(NAME(m_blitter_endmask3));
+	save_item(NAME(m_blitter_xcount));
+	save_item(NAME(m_blitter_ycount));
+	save_item(NAME(m_blitter_xcountl));
+	save_item(NAME(m_blitter_hop));
+	save_item(NAME(m_blitter_op));
+	save_item(NAME(m_blitter_ctrl));
+	save_item(NAME(m_blitter_skew));
 
 	set_screen_parameters();
-
-	VIDEO_START_NAME(generic_bitmapped)(machine());
 }
 
 
@@ -1161,8 +1161,8 @@ void ste_state::video_start()
 	st_state::video_start();
 
 	// register for state saving
-	state_save_register_global(machine(), m_shifter_lineofs);
-	state_save_register_global(machine(), m_shifter_pixelofs);
+	save_item(NAME(m_shifter_lineofs));
+	save_item(NAME(m_shifter_pixelofs));
 }
 
 //-------------------------------------------------
@@ -1171,5 +1171,12 @@ void ste_state::video_start()
 
 void stbook_state::video_start()
 {
-	VIDEO_START_NAME(generic_bitmapped)(machine());
 }
+
+
+UINT32 st_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
+{
+	copybitmap(bitmap, m_bitmap, 0, 0, 0, 0, cliprect);
+	return 0;
+}
+
