@@ -15,9 +15,9 @@ VIDEO_START( bk0010 )
 {
 }
 
-SCREEN_UPDATE( bk0010 )
+SCREEN_UPDATE_IND16( bk0010 )
 {
-	bk_state *state = screen->machine().driver_data<bk_state>();
+	bk_state *state = screen.machine().driver_data<bk_state>();
 	UINT16 code;
 	int y, x, b;
 	int nOfs;
@@ -31,7 +31,7 @@ SCREEN_UPDATE( bk0010 )
 			code = state->m_bk0010_video_ram[((y+nOfs) %256)*32 + x];
 			for (b = 0; b < 16; b++)
 			{
-				*BITMAP_ADDR16(bitmap, y, x*16 + b) =  (code >> b) & 0x01;
+				bitmap.pix16(y, x*16 + b) =  (code >> b) & 0x01;
 			}
 		}
 	}
