@@ -779,7 +779,7 @@ static void Nick_DoDisplay(NICK_STATE *nick)
 		/* get display mode */
 		DisplayMode = NICK_GET_DISPLAY_MODE(pLPT->MB);
 
-		if ((nick->ScanLineCount == 0))	// ||
+		if (nick->ScanLineCount == 0)	// ||
 			//((pLPT->MB & NICK_MB_VRES)==0))
 		{
 			/* doing first line */
@@ -1001,7 +1001,7 @@ WRITE8_HANDLER( epnick_reg_w )
 	}
 }
 
-static void Nick_DoScreen(NICK_STATE *nick, bitmap_t *bm)
+static void Nick_DoScreen(NICK_STATE *nick, bitmap_ind16 &bm)
 {
 	int line = 0;
 
@@ -1009,9 +1009,9 @@ static void Nick_DoScreen(NICK_STATE *nick, bitmap_t *bm)
 	{
 
 		/* set write address for line */
-		nick->dest = BITMAP_ADDR16(bm, line, 0);
+		nick->dest = &bm.pix16(line);
 		nick->dest_pos = 0;
-		nick->dest_max_pos = bm->width;
+		nick->dest_max_pos = bm.width();
 
 		/* write line */
 		Nick_DoLine(nick);
@@ -1029,15 +1029,16 @@ VIDEO_START( epnick )
 	ep_state *state = machine.driver_data<ep_state>();
 	state->nick = auto_alloc_clear(machine, NICK_STATE);
 
-	state->nick->videoram = ram_get_ptr(machine.device(RAM_TAG));
+	state->nick->videoram = machine.device<ram_device>(RAM_TAG)->pointer();
 	Nick_Init(state->nick);
-	VIDEO_START_CALL(generic_bitmapped);
+	machine.primary_screen->register_screen_bitmap(state->m_bitmap);
 }
 
 
-SCREEN_UPDATE( epnick )
+SCREEN_UPDATE_IND16( epnick )
 {
-	ep_state *state = screen->machine().driver_data<ep_state>();
-	Nick_DoScreen(state->nick,screen->machine().generic.tmpbitmap);
-	return SCREEN_UPDATE_CALL(generic_bitmapped);
+	ep_state *state = screen.machine().driver_data<ep_state>();
+	Nick_DoScreen(state->nick,state->m_bitmap);
+	copybitmap(bitmap, state->m_bitmap, 0, 0, 0, 0, cliprect);
+	return 0;
 }
