@@ -20,12 +20,11 @@
  *   bit 6: Sub-CPU cancel IRQ
  */
 
-READ8_HANDLER( fm7_subintf_r )
+READ8_MEMBER(fm7_state::fm7_subintf_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT8 ret = 0x00;
 
-	if(state->m_video.sub_busy != 0 || state->m_video.sub_halt != 0)
+	if(m_video.sub_busy != 0 || m_video.sub_halt != 0)
 		ret |= 0x80;
 
 	ret |= 0x7e;
@@ -34,51 +33,47 @@ READ8_HANDLER( fm7_subintf_r )
 	return ret;
 }
 
-WRITE8_HANDLER( fm7_subintf_w )
+WRITE8_MEMBER(fm7_state::fm7_subintf_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.sub_halt = data & 0x80;
+	m_video.sub_halt = data & 0x80;
 	if(data & 0x80)
-		state->m_video.sub_busy = data & 0x80;
+		m_video.sub_busy = data & 0x80;
 
-	cputag_set_input_line(space->machine(),"sub",INPUT_LINE_HALT,(data & 0x80) ? ASSERT_LINE : CLEAR_LINE);
+	cputag_set_input_line(machine(),"sub",INPUT_LINE_HALT,(data & 0x80) ? ASSERT_LINE : CLEAR_LINE);
 	if(data & 0x40)
-		cputag_set_input_line(space->machine(),"sub",M6809_IRQ_LINE,ASSERT_LINE);
+		cputag_set_input_line(machine(),"sub",M6809_IRQ_LINE,ASSERT_LINE);
 	//popmessage("Sub CPU Interface write: %02x\n",data);
 }
 
-READ8_HANDLER( fm7_sub_busyflag_r )
+READ8_MEMBER(fm7_state::fm7_sub_busyflag_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(state->m_video.sub_halt == 0)
-		state->m_video.sub_busy = 0x00;
+	if(m_video.sub_halt == 0)
+		m_video.sub_busy = 0x00;
 	return 0x00;
 }
 
-WRITE8_HANDLER( fm7_sub_busyflag_w )
+WRITE8_MEMBER(fm7_state::fm7_sub_busyflag_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.sub_busy = 0x80;
+	m_video.sub_busy = 0x80;
 }
 
 /*
  * Sub-CPU port 0xd402
  *   Read-only: Acknowledge Cancel IRQ
  */
-READ8_HANDLER( fm7_cancel_ack )
+READ8_MEMBER(fm7_state::fm7_cancel_ack)
 {
-	cputag_set_input_line(space->machine(),"sub",M6809_IRQ_LINE,CLEAR_LINE);
+	cputag_set_input_line(machine(),"sub",M6809_IRQ_LINE,CLEAR_LINE);
 	return 0x00;
 }
 
 /*
  * Reading from 0xd404 (sub-CPU) causes an "Attention" FIRQ on the main CPU
  */
-READ8_HANDLER( fm7_attn_irq_r )
+READ8_MEMBER(fm7_state::fm7_attn_irq_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.attn_irq = 1;
-	cputag_set_input_line(space->machine(),"maincpu",M6809_FIRQ_LINE,ASSERT_LINE);
+	m_video.attn_irq = 1;
+	cputag_set_input_line(machine(),"maincpu",M6809_FIRQ_LINE,ASSERT_LINE);
 	return 0xff;
 }
 
@@ -88,17 +83,15 @@ READ8_HANDLER( fm7_attn_irq_r )
  *  On read, enables VRAM access
  *  On write, disables VRAM access
  */
-READ8_HANDLER( fm7_vram_access_r )
+READ8_MEMBER(fm7_state::fm7_vram_access_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.vram_access = 1;
+	m_video.vram_access = 1;
 	return 0xff;
 }
 
-WRITE8_HANDLER( fm7_vram_access_w )
+WRITE8_MEMBER(fm7_state::fm7_vram_access_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.vram_access = 0;
+	m_video.vram_access = 0;
 }
 
 static TIMER_CALLBACK( fm77av_alu_task_end )
@@ -638,279 +631,267 @@ static void fm77av_line_draw(running_machine &machine)
 	machine.scheduler().timer_set(attotime::from_usec(byte_count/16), FUNC(fm77av_alu_task_end));
 }
 
-READ8_HANDLER( fm7_vram_r )
+READ8_MEMBER(fm7_state::fm7_vram_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	int offs;
 	UINT16 page = 0x0000;
 
-	if(state->m_video.active_video_page != 0)
+	if(m_video.active_video_page != 0)
 		page = 0xc000;
 
-	if(offset < 0x4000 && (state->m_video.multi_page & 0x01))
+	if(offset < 0x4000 && (m_video.multi_page & 0x01))
 		return 0xff;
-	if((offset < 0x8000 && offset >=0x4000) && (state->m_video.multi_page & 0x02))
+	if((offset < 0x8000 && offset >=0x4000) && (m_video.multi_page & 0x02))
 		return 0xff;
-	if((offset < 0xc000 && offset >=0x8000) && (state->m_video.multi_page & 0x04))
+	if((offset < 0xc000 && offset >=0x8000) && (m_video.multi_page & 0x04))
 		return 0xff;
 
-	if(state->m_alu.command & 0x80) // ALU active, writes to VRAM even when reading it (go figure)
+	if(m_alu.command & 0x80) // ALU active, writes to VRAM even when reading it (go figure)
 	{
+		fm7_state *state = machine().driver_data<fm7_state>();
 		fm7_alu_function(state, offset+page);
 	}
 
-	if(state->m_video.modestatus & 0x40)
+	if(m_video.modestatus & 0x40)
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset2) & 0x1fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset2) & 0x1fff);
 		else
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset) & 0x1fff);
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset) & 0x1fff);
 	}
 	else
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset2) & 0x3fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset2) & 0x3fff);
 		else
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset) & 0x3fff);
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset) & 0x3fff);
 	}
-	return state->m_video_ram[offs + page];
+	return m_video_ram[offs + page];
 }
 
-WRITE8_HANDLER( fm7_vram_w )
+WRITE8_MEMBER(fm7_state::fm7_vram_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	int offs;
 	UINT16 page = 0x0000;
 
-	if(state->m_video.active_video_page != 0)
+	if(m_video.active_video_page != 0)
 		page = 0xc000;
 
-	if(offset < 0x4000 && (state->m_video.multi_page & 0x01))
+	if(offset < 0x4000 && (m_video.multi_page & 0x01))
 		return;
-	if((offset < 0x8000 && offset >=0x4000) && (state->m_video.multi_page & 0x02))
+	if((offset < 0x8000 && offset >=0x4000) && (m_video.multi_page & 0x02))
 		return;
-	if((offset < 0xc000 && offset >=0x8000) && (state->m_video.multi_page & 0x04))
+	if((offset < 0xc000 && offset >=0x8000) && (m_video.multi_page & 0x04))
 		return;
 
-	if(state->m_alu.command & 0x80) // ALU active
+	if(m_alu.command & 0x80) // ALU active
 	{
+		fm7_state *state = machine().driver_data<fm7_state>();
 		fm7_alu_function(state, offset+page);
 		return;
 	}
 
-	if(state->m_video.modestatus & 0x40)
+	if(m_video.modestatus & 0x40)
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset2) & 0x1fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset2) & 0x1fff);
 		else
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset) & 0x1fff);
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset) & 0x1fff);
 	}
 	else
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset2) & 0x3fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset2) & 0x3fff);
 		else
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset) & 0x3fff);
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset) & 0x3fff);
 	}
-//  if(state->m_video.vram_access != 0)
-		state->m_video_ram[offs+page] = data;
+//  if(m_video.vram_access != 0)
+		m_video_ram[offs+page] = data;
 }
 
 // not pretty, but it should work.
-static WRITE8_HANDLER( fm7_vram_banked_w )
+WRITE8_MEMBER(fm7_state::fm7_vram_banked_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	int offs;
 	UINT16 page = 0x0000;
 
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return;
 
-	if(state->m_video.active_video_page != 0)
+	if(m_video.active_video_page != 0)
 		page = 0xc000;
 
-	if(offset < 0x4000 && (state->m_video.multi_page & 0x01))
+	if(offset < 0x4000 && (m_video.multi_page & 0x01))
 		return;
-	if((offset < 0x8000 && offset >=0x4000) && (state->m_video.multi_page & 0x02))
+	if((offset < 0x8000 && offset >=0x4000) && (m_video.multi_page & 0x02))
 		return;
-	if((offset < 0xc000 && offset >=0x8000) && (state->m_video.multi_page & 0x04))
+	if((offset < 0xc000 && offset >=0x8000) && (m_video.multi_page & 0x04))
 		return;
 
-	if(state->m_alu.command & 0x80) // ALU active
+	if(m_alu.command & 0x80) // ALU active
 	{
+		fm7_state *state = machine().driver_data<fm7_state>();
 		fm7_alu_function(state, offset+page);
 		return;
 	}
 
-	if(state->m_video.modestatus & 0x40)
+	if(m_video.modestatus & 0x40)
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset2) & 0x1fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset2) & 0x1fff);
 		else
-			offs = (offset & 0xe000) | ((offset + state->m_video.vram_offset) & 0x1fff);
+			offs = (offset & 0xe000) | ((offset + m_video.vram_offset) & 0x1fff);
 	}
 	else
 	{
-		if(state->m_video.active_video_page != 0)
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset2) & 0x3fff);
+		if(m_video.active_video_page != 0)
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset2) & 0x3fff);
 		else
-			offs = (offset & 0xc000) | ((offset + state->m_video.vram_offset) & 0x3fff);
+			offs = (offset & 0xc000) | ((offset + m_video.vram_offset) & 0x3fff);
 	}
-	state->m_video_ram[offs+page] = data;
+	m_video_ram[offs+page] = data;
 }
 
-READ8_HANDLER( fm7_vram0_r )
+READ8_MEMBER(fm7_state::fm7_vram0_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset);
 }
 
-READ8_HANDLER( fm7_vram1_r )
+READ8_MEMBER(fm7_state::fm7_vram1_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x1000);
 }
 
-READ8_HANDLER( fm7_vram2_r )
+READ8_MEMBER(fm7_state::fm7_vram2_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x2000);
 }
 
-READ8_HANDLER( fm7_vram3_r )
+READ8_MEMBER(fm7_state::fm7_vram3_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x3000);
 }
 
-READ8_HANDLER( fm7_vram4_r )
+READ8_MEMBER(fm7_state::fm7_vram4_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x4000);
 }
 
-READ8_HANDLER( fm7_vram5_r )
+READ8_MEMBER(fm7_state::fm7_vram5_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x5000);
 }
 
-READ8_HANDLER( fm7_vram6_r )
+READ8_MEMBER(fm7_state::fm7_vram6_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x6000);
 }
 
-READ8_HANDLER( fm7_vram7_r )
+READ8_MEMBER(fm7_state::fm7_vram7_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x7000);
 }
 
-READ8_HANDLER( fm7_vram8_r )
+READ8_MEMBER(fm7_state::fm7_vram8_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x8000);
 }
 
-READ8_HANDLER( fm7_vram9_r )
+READ8_MEMBER(fm7_state::fm7_vram9_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0x9000);
 }
 
-READ8_HANDLER( fm7_vramA_r )
+READ8_MEMBER(fm7_state::fm7_vramA_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0xa000);
 }
 
-READ8_HANDLER( fm7_vramB_r )
+READ8_MEMBER(fm7_state::fm7_vramB_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	if(!state->m_video.sub_halt)  // no access if sub CPU is not halted.
+	if(!m_video.sub_halt)  // no access if sub CPU is not halted.
 		return 0xff;
 	return fm7_vram_r(space,offset+0xb000);
 }
 
-WRITE8_HANDLER( fm7_vram0_w )
+WRITE8_MEMBER(fm7_state::fm7_vram0_w)
 {
 	fm7_vram_banked_w(space,offset,data);
 }
 
-WRITE8_HANDLER( fm7_vram1_w )
+WRITE8_MEMBER(fm7_state::fm7_vram1_w)
 {
 	fm7_vram_banked_w(space,offset+0x1000,data);
 }
 
-WRITE8_HANDLER( fm7_vram2_w )
+WRITE8_MEMBER(fm7_state::fm7_vram2_w)
 {
 	fm7_vram_banked_w(space,offset+0x2000,data);
 }
 
-WRITE8_HANDLER( fm7_vram3_w )
+WRITE8_MEMBER(fm7_state::fm7_vram3_w)
 {
 	fm7_vram_banked_w(space,offset+0x3000,data);
 }
 
-WRITE8_HANDLER( fm7_vram4_w )
+WRITE8_MEMBER(fm7_state::fm7_vram4_w)
 {
 	fm7_vram_banked_w(space,offset+0x4000,data);
 }
 
-WRITE8_HANDLER( fm7_vram5_w )
+WRITE8_MEMBER(fm7_state::fm7_vram5_w)
 {
 	fm7_vram_banked_w(space,offset+0x5000,data);
 }
 
-WRITE8_HANDLER( fm7_vram6_w )
+WRITE8_MEMBER(fm7_state::fm7_vram6_w)
 {
 	fm7_vram_banked_w(space,offset+0x6000,data);
 }
 
-WRITE8_HANDLER( fm7_vram7_w )
+WRITE8_MEMBER(fm7_state::fm7_vram7_w)
 {
 	fm7_vram_banked_w(space,offset+0x7000,data);
 }
 
-WRITE8_HANDLER( fm7_vram8_w )
+WRITE8_MEMBER(fm7_state::fm7_vram8_w)
 {
 	fm7_vram_banked_w(space,offset+0x8000,data);
 }
 
-WRITE8_HANDLER( fm7_vram9_w )
+WRITE8_MEMBER(fm7_state::fm7_vram9_w)
 {
 	fm7_vram_banked_w(space,offset+0x9000,data);
 }
 
-WRITE8_HANDLER( fm7_vramA_w )
+WRITE8_MEMBER(fm7_state::fm7_vramA_w)
 {
 	fm7_vram_banked_w(space,offset+0xa000,data);
 }
 
-WRITE8_HANDLER( fm7_vramB_w )
+WRITE8_MEMBER(fm7_state::fm7_vramB_w)
 {
 	fm7_vram_banked_w(space,offset+0xb000,data);
 }
@@ -921,17 +902,15 @@ WRITE8_HANDLER( fm7_vramB_w )
  *  On read, enables the CRT display
  *  On write, disables the CRT display
  */
-READ8_HANDLER( fm7_crt_r )
+READ8_MEMBER(fm7_state::fm7_crt_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.crt_enable = 1;
+	m_video.crt_enable = 1;
 	return 0xff;
 }
 
-WRITE8_HANDLER( fm7_crt_w )
+WRITE8_MEMBER(fm7_state::fm7_crt_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.crt_enable = 0;
+	m_video.crt_enable = 0;
 }
 
 /*
@@ -940,45 +919,44 @@ WRITE8_HANDLER( fm7_crt_w )
  *  0xd40e: bits 0-6 - offset in bytes (high byte) (bit 6 is used for 400 line video only)
  *  0xd40f: bits 0-7 - offset in bytes (low byte)
  */
-WRITE8_HANDLER( fm7_vram_offset_w )
+WRITE8_MEMBER(fm7_state::fm7_vram_offset_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT16 new_offset = 0;
 
 	switch(offset)
 	{
 		case 0:
-			if(state->m_video.active_video_page != 0)
-				new_offset = ((data & 0x3f) << 8) | (state->m_video.vram_offset2 & 0x00ff);
+			if(m_video.active_video_page != 0)
+				new_offset = ((data & 0x3f) << 8) | (m_video.vram_offset2 & 0x00ff);
 			else
-				new_offset = ((data & 0x3f) << 8) | (state->m_video.vram_offset & 0x00ff);
+				new_offset = ((data & 0x3f) << 8) | (m_video.vram_offset & 0x00ff);
 			break;
 		case 1:  // low 5 bits are used on FM-77AV and later only
-			if(state->m_type == SYS_FM7)
-				new_offset = (state->m_video.vram_offset & 0xff00) | (data & 0xe0);
+			if(m_type == SYS_FM7)
+				new_offset = (m_video.vram_offset & 0xff00) | (data & 0xe0);
 			else
 			{
-				if(state->m_video.active_video_page != 0)
+				if(m_video.active_video_page != 0)
 				{
-					if(state->m_video.fine_offset != 0)
-						new_offset = (state->m_video.vram_offset2 & 0xff00) | (data & 0xff);
+					if(m_video.fine_offset != 0)
+						new_offset = (m_video.vram_offset2 & 0xff00) | (data & 0xff);
 					else
-						new_offset = (state->m_video.vram_offset2 & 0xff00) | (data & 0xe0);
+						new_offset = (m_video.vram_offset2 & 0xff00) | (data & 0xe0);
 				}
 				else
 				{
-					if(state->m_video.fine_offset != 0)
-						new_offset = (state->m_video.vram_offset & 0xff00) | (data & 0xff);
+					if(m_video.fine_offset != 0)
+						new_offset = (m_video.vram_offset & 0xff00) | (data & 0xff);
 					else
-						new_offset = (state->m_video.vram_offset & 0xff00) | (data & 0xe0);
+						new_offset = (m_video.vram_offset & 0xff00) | (data & 0xe0);
 				}
 			}
 			break;
 	}
-	if(state->m_video.active_video_page != 0)
-		state->m_video.vram_offset2 = new_offset;
+	if(m_video.active_video_page != 0)
+		m_video.vram_offset2 = new_offset;
 	else
-		state->m_video.vram_offset = new_offset;
+		m_video.vram_offset = new_offset;
 }
 
 /*
@@ -989,10 +967,9 @@ WRITE8_HANDLER( fm7_vram_offset_w )
  *  bits 6-4: VRAM planes to display (G,R,B) (1=disable)
  *  bits 2-0: VRAM CPU access (G,R,B) (1=disable)
  */
-WRITE8_HANDLER( fm7_multipage_w )
+WRITE8_MEMBER(fm7_state::fm7_multipage_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	state->m_video.multi_page = data & 0x77;
+	m_video.multi_page = data & 0x77;
 }
 
 /*
@@ -1003,15 +980,13 @@ WRITE8_HANDLER( fm7_multipage_w )
  *  bit 1 = Red
  *  bit 0 = Blue
  */
-READ8_HANDLER( fm7_palette_r )
+READ8_MEMBER(fm7_state::fm7_palette_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	return state->m_video.fm7_pal[offset];
+	return m_video.fm7_pal[offset];
 }
 
-WRITE8_HANDLER( fm7_palette_w )
+WRITE8_MEMBER(fm7_state::fm7_palette_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT8 r = 0,g = 0,b = 0;
 
 	if(data & 0x04)
@@ -1021,8 +996,8 @@ WRITE8_HANDLER( fm7_palette_w )
 	if(data & 0x01)
 		b = 0xff;
 
-	palette_set_color(space->machine(),offset,MAKE_RGB(r,g,b));
-	state->m_video.fm7_pal[offset] = data & 0x07;
+	palette_set_color(machine(),offset,MAKE_RGB(r,g,b));
+	m_video.fm7_pal[offset] = data & 0x07;
 }
 
 /*
@@ -1036,41 +1011,40 @@ WRITE8_HANDLER( fm7_palette_w )
  *  fd33: red level (4 bits)
  *  fd34: green level (4 bits)
  */
-WRITE8_HANDLER( fm77av_analog_palette_w )
+WRITE8_MEMBER(fm7_state::fm77av_analog_palette_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	int val;
 
 	switch(offset)
 	{
 		case 0:
-			val = ((data & 0x0f) << 8) | (state->m_video.fm77av_pal_selected & 0x00ff);
-			state->m_video.fm77av_pal_selected = val;
+			val = ((data & 0x0f) << 8) | (m_video.fm77av_pal_selected & 0x00ff);
+			m_video.fm77av_pal_selected = val;
 			break;
 		case 1:
-			val = data | (state->m_video.fm77av_pal_selected & 0x0f00);
-			state->m_video.fm77av_pal_selected = val;
+			val = data | (m_video.fm77av_pal_selected & 0x0f00);
+			m_video.fm77av_pal_selected = val;
 			break;
 		case 2:
-			state->m_video.fm77av_pal_b[state->m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
-			palette_set_color(space->machine(),state->m_video.fm77av_pal_selected+8,
-				MAKE_RGB(state->m_video.fm77av_pal_r[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_g[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_b[state->m_video.fm77av_pal_selected]));
+			m_video.fm77av_pal_b[m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
+			palette_set_color(machine(),m_video.fm77av_pal_selected+8,
+				MAKE_RGB(m_video.fm77av_pal_r[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_g[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_b[m_video.fm77av_pal_selected]));
 			break;
 		case 3:
-			state->m_video.fm77av_pal_r[state->m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
-			palette_set_color(space->machine(),state->m_video.fm77av_pal_selected+8,
-				MAKE_RGB(state->m_video.fm77av_pal_r[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_g[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_b[state->m_video.fm77av_pal_selected]));
+			m_video.fm77av_pal_r[m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
+			palette_set_color(machine(),m_video.fm77av_pal_selected+8,
+				MAKE_RGB(m_video.fm77av_pal_r[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_g[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_b[m_video.fm77av_pal_selected]));
 			break;
 		case 4:
-			state->m_video.fm77av_pal_g[state->m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
-			palette_set_color(space->machine(),state->m_video.fm77av_pal_selected+8,
-				MAKE_RGB(state->m_video.fm77av_pal_r[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_g[state->m_video.fm77av_pal_selected],
-				state->m_video.fm77av_pal_b[state->m_video.fm77av_pal_selected]));
+			m_video.fm77av_pal_g[m_video.fm77av_pal_selected] = (data & 0x0f) << 4;
+			palette_set_color(machine(),m_video.fm77av_pal_selected+8,
+				MAKE_RGB(m_video.fm77av_pal_r[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_g[m_video.fm77av_pal_selected],
+				m_video.fm77av_pal_b[m_video.fm77av_pal_selected]));
 			break;
 	}
 }
@@ -1090,37 +1064,35 @@ WRITE8_HANDLER( fm77av_analog_palette_w )
  *             bit 6 - display VRAM page
  *             bit 7 - NMI mask register (1=mask)
  */
-READ8_HANDLER( fm77av_video_flags_r )
+READ8_MEMBER(fm7_state::fm77av_video_flags_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT8 ret = 0xff;
 
-	if(space->machine().primary_screen->vblank())
+	if(machine().primary_screen->vblank())
 		ret &= ~0x80;
 
-	if(state->m_alu.busy != 0)
+	if(m_alu.busy != 0)
 		ret &= ~0x10;
 
-	if(state->m_video.vsync_flag == 0)
+	if(m_video.vsync_flag == 0)
 		ret &= ~0x04;
 
-	if(!state->m_video.sub_reset)
+	if(!m_video.sub_reset)
 		ret &= ~0x01;
 
 	return ret;
 }
 
-WRITE8_HANDLER( fm77av_video_flags_w )
+WRITE8_MEMBER(fm7_state::fm77av_video_flags_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	UINT8* RAM = space->machine().region("subsyscg")->base();
+	UINT8* RAM = memregion("subsyscg")->base();
 
-	state->m_video.cgrom = data & 0x03;
-	memory_set_bankptr(space->machine(),"bank20",RAM+(state->m_video.cgrom*0x800));
-	state->m_video.fine_offset = data & 0x04;
-	state->m_video.active_video_page = data & 0x20;
-	state->m_video.display_video_page = data & 0x40;
-	state->m_video.nmi_mask = data & 0x80;
+	m_video.cgrom = data & 0x03;
+	membank("bank20")->set_base(RAM+(m_video.cgrom*0x800));
+	m_video.fine_offset = data & 0x04;
+	m_video.active_video_page = data & 0x20;
+	m_video.display_video_page = data & 0x40;
+	m_video.nmi_mask = data & 0x80;
 }
 
 /*
@@ -1130,41 +1102,34 @@ WRITE8_HANDLER( fm77av_video_flags_w )
  *  bit 1 (R/O) - DISPTMG status (0=blank)
  *  bit 0 (R/O) - VSync status (1=sync?)
  */
-READ8_HANDLER( fm77av_sub_modestatus_r )
+READ8_MEMBER(fm7_state::fm77av_sub_modestatus_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT8 ret = 0x00;
 
 	ret |= 0xbc;
-	ret |= (state->m_video.modestatus & 0x40);
+	ret |= (m_video.modestatus & 0x40);
 
-	if(!space->machine().primary_screen->vblank())
+	if(!machine().primary_screen->vblank())
 		ret |= 0x02;
 
-	if(state->m_video.vsync_flag != 0)
+	if(m_video.vsync_flag != 0)
 		ret |= 0x01;
 
 	return ret;
 }
 
-WRITE8_HANDLER( fm77av_sub_modestatus_w )
+WRITE8_MEMBER(fm7_state::fm77av_sub_modestatus_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	rectangle rect;
-	state->m_video.modestatus = data & 0x40;
+	m_video.modestatus = data & 0x40;
 	if(data & 0x40)
 	{
-		rect.min_x = rect.min_y = 0;
-		rect.max_x = 320-1;
-		rect.max_y = 200-1;
-		space->machine().primary_screen->configure(320,200,rect,HZ_TO_ATTOSECONDS(60));
+		rectangle rect(0, 320-1, 0, 200-1);
+		machine().primary_screen->configure(320,200,rect,HZ_TO_ATTOSECONDS(60));
 	}
 	else
 	{
-		rect.min_x = rect.min_y = 0;
-		rect.max_x = 640-1;
-		rect.max_y = 200-1;
-		space->machine().primary_screen->configure(640,200,rect,HZ_TO_ATTOSECONDS(60));
+		rectangle rect(0, 640-1, 0, 200-1);
+		machine().primary_screen->configure(640,200,rect,HZ_TO_ATTOSECONDS(60));
 	}
 }
 
@@ -1175,49 +1140,48 @@ WRITE8_HANDLER( fm77av_sub_modestatus_w )
  *  bits 1 and 0 select which subsys ROM to be banked into sub CPU space
  *  on the FM-77AV40 and later, bit 2 can also selected to bank in sub monitor RAM.
  */
-WRITE8_HANDLER( fm77av_sub_bank_w )
+WRITE8_MEMBER(fm7_state::fm77av_sub_bank_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-//  UINT8* RAM = space->machine().region("sub")->base();
+//  UINT8* RAM = memregion("sub")->base();
 	UINT8* ROM;
 
-	if((data & 0x03) == (state->m_sb_prev & 0x03))
+	if((data & 0x03) == (m_sb_prev & 0x03))
 		return;
 
-	state->m_video.subrom = data & 0x03;
+	m_video.subrom = data & 0x03;
 	switch (data & 0x03)
 	{
 		case 0x00:  // Type C, 640x200 (as used on the FM-7)
-			ROM = space->machine().region("subsys_c")->base();
-		//  memory_set_bankptr(space->machine(),20,ROM);
-			memory_set_bankptr(space->machine(),"bank21",ROM+0x800);
+			ROM = memregion("subsys_c")->base();
+		//  membank(20)->set_base(ROM);
+			membank("bank21")->set_base(ROM+0x800);
 			logerror("VID: Sub ROM Type C selected\n");
 			break;
 		case 0x01:  // Type A, 640x200
-			ROM = space->machine().region("subsys_a")->base();
-		//  memory_set_bankptr(space->machine(),20,RAM+0xd800);
-			memory_set_bankptr(space->machine(),"bank21",ROM);
+			ROM = memregion("subsys_a")->base();
+		//  membank(20)->set_base(RAM+0xd800);
+			membank("bank21")->set_base(ROM);
 			logerror("VID: Sub ROM Type A selected\n");
 			break;
 		case 0x02:  // Type B, 320x200
-			ROM = space->machine().region("subsys_b")->base();
-		//  memory_set_bankptr(space->machine(),20,RAM+0xd800);
-			memory_set_bankptr(space->machine(),"bank21",ROM);
+			ROM = memregion("subsys_b")->base();
+		//  membank(20)->set_base(RAM+0xd800);
+			membank("bank21")->set_base(ROM);
 			logerror("VID: Sub ROM Type B selected\n");
 			break;
 		case 0x03:  // CG Font?
-			ROM = space->machine().region("subsyscg")->base();
-		//  memory_set_bankptr(space->machine(),20,RAM+0xd800);
-			memory_set_bankptr(space->machine(),"bank21",ROM);
+			ROM = memregion("subsyscg")->base();
+		//  membank(20)->set_base(RAM+0xd800);
+			membank("bank21")->set_base(ROM);
 			logerror("VID: Sub ROM CG selected\n");
 			break;
 	}
 	// reset sub CPU, set busy flag, set reset flag
-	cputag_set_input_line(space->machine(),"sub",INPUT_LINE_RESET,PULSE_LINE);
-	state->m_video.sub_busy = 0x80;
-	state->m_video.sub_halt = 0;
-	state->m_video.sub_reset = 1;
-	state->m_sb_prev = data;
+	cputag_set_input_line(machine(),"sub",INPUT_LINE_RESET,PULSE_LINE);
+	m_video.sub_busy = 0x80;
+	m_video.sub_halt = 0;
+	m_video.sub_reset = 1;
+	m_sb_prev = data;
 }
 
 /*
@@ -1248,26 +1212,25 @@ WRITE8_HANDLER( fm77av_sub_bank_w )
  *  0xd428-29(W): Line X1 (High-Low, X9-X0)
  *  0xd42a-2b(W): Line Y1 (High-Low, Y8-Y0)
  */
-READ8_HANDLER( fm77av_alu_r )
+READ8_MEMBER(fm7_state::fm77av_alu_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	switch(offset)
 	{
 		case 0x00:
-			logerror("ALU: read from command register (%02x)\n",state->m_alu.command);
-			return state->m_alu.command;
+			logerror("ALU: read from command register (%02x)\n",m_alu.command);
+			return m_alu.command;
 		case 0x01:
 			logerror("ALU: read from logical colour\n");
-			return state->m_alu.lcolour;
+			return m_alu.lcolour;
 		case 0x02:
 			logerror("ALU: read from mask register\n");
-			return state->m_alu.mask;
+			return m_alu.mask;
 		case 0x03:
-			logerror("ALU: read from compare data register (%02x)\n",state->m_alu.compare_data);
-			return state->m_alu.compare_data;
+			logerror("ALU: read from compare data register (%02x)\n",m_alu.compare_data);
+			return m_alu.compare_data;
 		case 0x0b:
 			logerror("ALU: read from bank disable register\n");
-			return 0xf7 | state->m_alu.bank_disable;
+			return 0xf7 | m_alu.bank_disable;
 		default:
 			logerror("ALU: read from invalid register 0x%02x\n",offset);
 			return 0xff;
@@ -1275,23 +1238,22 @@ READ8_HANDLER( fm77av_alu_r )
 	return 0xff;
 }
 
-WRITE8_HANDLER( fm77av_alu_w )
+WRITE8_MEMBER(fm7_state::fm77av_alu_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
 	UINT16 dat;
 
 	switch(offset)
 	{
 		case 0x00:
-			state->m_alu.command = data;
+			m_alu.command = data;
 			logerror("ALU: write to command register - %02x\n",data);
 			break;
 		case 0x01:
-			state->m_alu.lcolour = data & 0x07;
+			m_alu.lcolour = data & 0x07;
 			logerror("ALU: write to logical colour - %02x\n",data);
 			break;
 		case 0x02:
-			state->m_alu.mask = data;
+			m_alu.mask = data;
 			logerror("ALU: write to mask register - %02x\n",data);
 			break;
 		case 0x03:
@@ -1302,86 +1264,86 @@ WRITE8_HANDLER( fm77av_alu_w )
 		case 0x08:
 		case 0x09:
 		case 0x0a:
-			state->m_alu.compare[offset-3] = data;
+			m_alu.compare[offset-3] = data;
 			logerror("ALU: write to compare register %i - %02x\n",offset-3,data);
 			break;
 		case 0x0b:
-			state->m_alu.bank_disable = data & 0x03;
+			m_alu.bank_disable = data & 0x03;
 //          logerror("ALU: write to bank disable register - %02x\n",data);
 			break;
 		case 0x0c:
-			state->m_alu.tilepaint_b = data;
+			m_alu.tilepaint_b = data;
 //          logerror("ALU: write to tilepaint (blue) register - %02x\n",data);
 			break;
 		case 0x0d:
-			state->m_alu.tilepaint_r = data;
+			m_alu.tilepaint_r = data;
 //          logerror("ALU: write to tilepaint (red) register - %02x\n",data);
 			break;
 		case 0x0e:
-			state->m_alu.tilepaint_g = data;
+			m_alu.tilepaint_g = data;
 //          logerror("ALU: write to tilepaint (green) register - %02x\n",data);
 			break;
 		case 0x10:
-			dat = ((data & 0x1f) << 8) | (state->m_alu.addr_offset & 0x00ff);
-			state->m_alu.addr_offset = dat;
-//          logerror("ALU: write to address offset (high) register - %02x (%04x)\n",data,state->m_alu.addr_offset);
+			dat = ((data & 0x1f) << 8) | (m_alu.addr_offset & 0x00ff);
+			m_alu.addr_offset = dat;
+//          logerror("ALU: write to address offset (high) register - %02x (%04x)\n",data,m_alu.addr_offset);
 			break;
 		case 0x11:
-			dat = (state->m_alu.addr_offset & 0xff00) | data;
-			state->m_alu.addr_offset = dat;
-//          logerror("ALU: write to address offset (low) register - %02x (%04x)\n",data,state->m_alu.addr_offset);
+			dat = (m_alu.addr_offset & 0xff00) | data;
+			m_alu.addr_offset = dat;
+//          logerror("ALU: write to address offset (low) register - %02x (%04x)\n",data,m_alu.addr_offset);
 			break;
 		case 0x12:
-			dat = (data << 8) | (state->m_alu.line_style & 0x00ff);
-			state->m_alu.line_style = dat;
-//          logerror("ALU: write to line style (high) register - %02x (%04x)\n",data,state->m_alu.line_style);
+			dat = (data << 8) | (m_alu.line_style & 0x00ff);
+			m_alu.line_style = dat;
+//          logerror("ALU: write to line style (high) register - %02x (%04x)\n",data,m_alu.line_style);
 			break;
 		case 0x13:
-			dat = (state->m_alu.line_style & 0xff00) | data;
-			state->m_alu.line_style = dat;
-//          logerror("ALU: write to line style (low) register - %02x (%04x)\n",data,state->m_alu.line_style);
+			dat = (m_alu.line_style & 0xff00) | data;
+			m_alu.line_style = dat;
+//          logerror("ALU: write to line style (low) register - %02x (%04x)\n",data,m_alu.line_style);
 			break;
 		case 0x14:
-			dat = ((data & 0x03) << 8) | (state->m_alu.x0 & 0x00ff);
-			state->m_alu.x0 = dat;
-//          logerror("ALU: write to X0 (high) register - %02x (%04x)\n",data,state->m_alu.x0);
+			dat = ((data & 0x03) << 8) | (m_alu.x0 & 0x00ff);
+			m_alu.x0 = dat;
+//          logerror("ALU: write to X0 (high) register - %02x (%04x)\n",data,m_alu.x0);
 			break;
 		case 0x15:
-			dat = (state->m_alu.x0 & 0xff00) | data;
-			state->m_alu.x0 = dat;
-//          logerror("ALU: write to X0 (low) register - %02x (%04x)\n",data,state->m_alu.x0);
+			dat = (m_alu.x0 & 0xff00) | data;
+			m_alu.x0 = dat;
+//          logerror("ALU: write to X0 (low) register - %02x (%04x)\n",data,m_alu.x0);
 			break;
 		case 0x16:
-			dat = ((data & 0x01) << 8) | (state->m_alu.y0 & 0x00ff);
-			state->m_alu.y0 = dat;
-//          logerror("ALU: write to Y0 (high) register - %02x (%04x)\n",data,state->m_alu.y0);
+			dat = ((data & 0x01) << 8) | (m_alu.y0 & 0x00ff);
+			m_alu.y0 = dat;
+//          logerror("ALU: write to Y0 (high) register - %02x (%04x)\n",data,m_alu.y0);
 			break;
 		case 0x17:
-			dat = (state->m_alu.y0 & 0xff00) | data;
-			state->m_alu.y0 = dat;
-//          logerror("ALU: write to Y0 (low) register - %02x (%04x)\n",data,state->m_alu.y0);
+			dat = (m_alu.y0 & 0xff00) | data;
+			m_alu.y0 = dat;
+//          logerror("ALU: write to Y0 (low) register - %02x (%04x)\n",data,m_alu.y0);
 			break;
 		case 0x18:
-			dat = ((data & 0x03) << 8) | (state->m_alu.x1 & 0x00ff);
-			state->m_alu.x1 = dat;
-//          logerror("ALU: write to X1 (high) register - %02x (%04x)\n",data,state->m_alu.x1);
+			dat = ((data & 0x03) << 8) | (m_alu.x1 & 0x00ff);
+			m_alu.x1 = dat;
+//          logerror("ALU: write to X1 (high) register - %02x (%04x)\n",data,m_alu.x1);
 			break;
 		case 0x19:
-			dat = (state->m_alu.x1 & 0xff00) | data;
-			state->m_alu.x1 = dat;
-//          logerror("ALU: write to X1 (low) register - %02x (%04x)\n",data,state->m_alu.x1);
+			dat = (m_alu.x1 & 0xff00) | data;
+			m_alu.x1 = dat;
+//          logerror("ALU: write to X1 (low) register - %02x (%04x)\n",data,m_alu.x1);
 			break;
 		case 0x1a:
-			dat = ((data & 0x01) << 8) | (state->m_alu.y1 & 0x00ff);
-			state->m_alu.y1 = dat;
-//          logerror("ALU: write to Y1 (high) register - %02x (%04x)\n",data,state->m_alu.y1);
+			dat = ((data & 0x01) << 8) | (m_alu.y1 & 0x00ff);
+			m_alu.y1 = dat;
+//          logerror("ALU: write to Y1 (high) register - %02x (%04x)\n",data,m_alu.y1);
 			break;
 		case 0x1b:
-			dat = (state->m_alu.y1 & 0xff00) | data;
-			state->m_alu.y1 = dat;
+			dat = (m_alu.y1 & 0xff00) | data;
+			m_alu.y1 = dat;
 			// draw line
-			fm77av_line_draw(space->machine());
-//          logerror("ALU: write to Y1 (low) register - %02x (%04x)\n",data,state->m_alu.y1);
+			fm77av_line_draw(machine());
+//          logerror("ALU: write to Y1 (low) register - %02x (%04x)\n",data,m_alu.y1);
 			break;
 		default:
 			logerror("ALU: write 0x%02x to invalid register 0x%02x\n",data,offset);
@@ -1404,25 +1366,24 @@ TIMER_CALLBACK( fm77av_vsync )
 }
 
 // called when banked into main CPU space by the MMR, available only if sub CPU is halted
-READ8_HANDLER( fm7_sub_ram_ports_banked_r )
+READ8_MEMBER(fm7_state::fm7_sub_ram_ports_banked_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	UINT8* RAM = space->machine().region("maincpu")->base();
+	UINT8* RAM = memregion("maincpu")->base();
 	UINT8* ROM;
 
-	if(!state->m_video.sub_halt)
+	if(!m_video.sub_halt)
 		return 0xff;
 
 	if(offset < 0x380)  // work RAM
 		return RAM[0x1d000+offset];
 	if(offset >= 0x380 && offset < 0x400) // shared RAM
-		return state->m_shared_ram[offset-0x380];
+		return m_shared_ram[offset-0x380];
 	if(offset >= 0x500 && offset < 0x800) // work RAM
 		return RAM[0x1d000+offset];
 	if(offset > 0x800) // CGROM
 	{
-		ROM = space->machine().region("subsyscg")->base();
-		return ROM[(state->m_video.cgrom*0x800)+(offset-0x800)];
+		ROM = memregion("subsyscg")->base();
+		return ROM[(m_video.cgrom*0x800)+(offset-0x800)];
 	}
 
 	if(offset >= 0x410 && offset <= 0x42b)
@@ -1457,12 +1418,11 @@ READ8_HANDLER( fm7_sub_ram_ports_banked_r )
 	return 0xff;
 }
 
-WRITE8_HANDLER( fm7_sub_ram_ports_banked_w )
+WRITE8_MEMBER(fm7_state::fm7_sub_ram_ports_banked_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	UINT8* RAM = space->machine().region("maincpu")->base();
+	UINT8* RAM = memregion("maincpu")->base();
 
-	if(!state->m_video.sub_halt)
+	if(!m_video.sub_halt)
 		return;
 
 	if(offset < 0x380)  // work RAM
@@ -1472,7 +1432,7 @@ WRITE8_HANDLER( fm7_sub_ram_ports_banked_w )
 	}
 	if(offset >= 0x380 && offset < 0x400) // shared RAM
 	{
-		state->m_shared_ram[offset-0x380] = data;
+		m_shared_ram[offset-0x380] = data;
 		return;
 	}
 	if(offset >= 0x500 && offset < 0x800) // work RAM
@@ -1514,23 +1474,21 @@ WRITE8_HANDLER( fm7_sub_ram_ports_banked_w )
 	}
 }
 
-READ8_HANDLER( fm7_console_ram_banked_r )
+READ8_MEMBER(fm7_state::fm7_console_ram_banked_r)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	UINT8* RAM = space->machine().region("maincpu")->base();
+	UINT8* RAM = memregion("maincpu")->base();
 
-	if(!state->m_video.sub_halt)
+	if(!m_video.sub_halt)
 		return 0xff;
 
 	return RAM[0x1c000+offset];
 }
 
-WRITE8_HANDLER( fm7_console_ram_banked_w )
+WRITE8_MEMBER(fm7_state::fm7_console_ram_banked_w)
 {
-	fm7_state *state = space->machine().driver_data<fm7_state>();
-	UINT8* RAM = space->machine().region("maincpu")->base();
+	UINT8* RAM = memregion("maincpu")->base();
 
-	if(!state->m_video.sub_halt)
+	if(!m_video.sub_halt)
 		return;
 
 	RAM[0x1c000+offset] = data;
@@ -1554,9 +1512,9 @@ VIDEO_START( fm7 )
 	state->m_video.vsync_flag = 0;
 }
 
-SCREEN_UPDATE( fm7 )
+SCREEN_UPDATE_IND16( fm7 )
 {
-	fm7_state *state = screen->machine().driver_data<fm7_state>();
+	fm7_state *state = screen.machine().driver_data<fm7_state>();
 	UINT8 code_r = 0,code_g = 0,code_b = 0;
 	UINT8 code_r2 = 0,code_g2 = 0,code_b2 = 0;
 	UINT8 code_r3 = 0,code_g3 = 0,code_b3 = 0;
@@ -1604,7 +1562,7 @@ SCREEN_UPDATE( fm7 )
 					col |= (((code_g >> b) & 0x01) ? 128 : 0) | (((code_g2 >> b) & 0x01) ? 64 : 0) | (((code_g3 >> b) & 0x01) ? 32 : 0) | (((code_g4 >> b) & 0x01) ? 16 : 0);
 					col |= (((code_r >> b) & 0x01) ? 2048 : 0) | (((code_r2 >> b) & 0x01) ? 1024 : 0) | (((code_r3 >> b) & 0x01) ? 512 : 0) | (((code_r4 >> b) & 0x01) ? 256 : 0);
 					col += 8;  // use analog palette
-					*BITMAP_ADDR16(bitmap, y,  x*8+(7-b)) =  col;
+					bitmap.pix16(y, x*8+(7-b)) =  col;
 				}
 			}
 		}
@@ -1624,7 +1582,7 @@ SCREEN_UPDATE( fm7 )
 				for (b = 0; b < 8; b++)
 				{
 					col = (((code_r >> b) & 0x01) ? 4 : 0) + (((code_g >> b) & 0x01) ? 2 : 0) + (((code_b >> b) & 0x01) ? 1 : 0);
-					*BITMAP_ADDR16(bitmap, y,  x*8+(7-b)) =  col;
+					bitmap.pix16(y, x*8+(7-b)) =  col;
 				}
 			}
 		}
