@@ -99,28 +99,25 @@ static void draw_sprites(running_machine &machine, const rectangle* rect);
 static void towns_crtc_refresh_mode(running_machine &machine)
 {
 	towns_state* state = machine.driver_data<towns_state>();
-	rectangle scr;
 	unsigned int width,height;
 
-	scr.min_x = scr.min_y = 0;
-	scr.max_x = state->m_video.towns_crtc_reg[4];
-	scr.max_y = state->m_video.towns_crtc_reg[8] / 2;
+	rectangle scr(0, state->m_video.towns_crtc_reg[4], 0, state->m_video.towns_crtc_reg[8] / 2);
 
 	// layer 0
 	width = state->m_video.towns_crtc_reg[10] - state->m_video.towns_crtc_reg[9];
 	height = (state->m_video.towns_crtc_reg[14] - state->m_video.towns_crtc_reg[13]) / 2;
-	state->m_video.towns_crtc_layerscr[0].min_x = (scr.max_x / 2) - (width / 2);
-	state->m_video.towns_crtc_layerscr[0].min_y = (scr.max_y / 2) - (height / 2);
-	state->m_video.towns_crtc_layerscr[0].max_x = (scr.max_x / 2) + (width / 2);
-	state->m_video.towns_crtc_layerscr[0].max_y = (scr.max_y / 2) + (height / 2);
+	state->m_video.towns_crtc_layerscr[0].min_x = scr.xcenter() - (width / 2);
+	state->m_video.towns_crtc_layerscr[0].min_y = scr.ycenter() - (height / 2);
+	state->m_video.towns_crtc_layerscr[0].max_x = scr.xcenter() + (width / 2);
+	state->m_video.towns_crtc_layerscr[0].max_y = scr.ycenter() + (height / 2);
 
 	// layer 1
 	width = state->m_video.towns_crtc_reg[12] - state->m_video.towns_crtc_reg[11];
 	height = (state->m_video.towns_crtc_reg[16] - state->m_video.towns_crtc_reg[15]) / 2;
-	state->m_video.towns_crtc_layerscr[1].min_x = (scr.max_x / 2) - (width / 2);
-	state->m_video.towns_crtc_layerscr[1].min_y = (scr.max_y / 2) - (height / 2);
-	state->m_video.towns_crtc_layerscr[1].max_x = (scr.max_x / 2) + (width / 2);
-	state->m_video.towns_crtc_layerscr[1].max_y = (scr.max_y / 2) + (height / 2);
+	state->m_video.towns_crtc_layerscr[1].min_x = scr.xcenter() - (width / 2);
+	state->m_video.towns_crtc_layerscr[1].min_y = scr.ycenter() - (height / 2);
+	state->m_video.towns_crtc_layerscr[1].max_x = scr.xcenter() + (width / 2);
+	state->m_video.towns_crtc_layerscr[1].max_y = scr.ycenter() + (height / 2);
 
 	// sanity checks
 	if(scr.max_x == 0 || scr.max_y == 0)
@@ -146,7 +143,7 @@ READ8_MEMBER( towns_state::towns_gfx_r )
 	UINT8 ret = 0;
 
 	if(m_towns_mainmem_enable != 0)
-		return ram_get_ptr(m_messram)[offset+0xc0000];
+		return m_messram->pointer()[offset+0xc0000];
 
 	offset = offset << 2;
 
@@ -169,7 +166,7 @@ WRITE8_MEMBER( towns_state::towns_gfx_w )
 {
 	if(m_towns_mainmem_enable != 0)
 	{
-		ram_get_ptr(m_messram)[offset+0xc0000] = data;
+		m_messram->pointer()[offset+0xc0000] = data;
 		return;
 	}
 	offset = offset << 2;
@@ -252,7 +249,7 @@ static void towns_update_kanji_offset(running_machine &machine)
 
 READ8_MEMBER( towns_state::towns_video_cff80_r )
 {
-	UINT8* ROM = space.machine().region("user")->base();
+	UINT8* ROM = space.machine().root_device().memregion("user")->base();
 
 	switch(offset)
 	{
@@ -268,10 +265,10 @@ READ8_MEMBER( towns_state::towns_video_cff80_r )
 			else
 				return 0x00;
 		case 0x06:
-			if(m_video.towns_vblank_flag != 0)
-				return 0x10;
-			else
-				return 0x00;
+          if(m_video.towns_vblank_flag != 0)
+              return 0x10;
+          else
+              return 0x00;
 		case 0x16:  // Kanji character data
 			return ROM[(m_video.towns_kanji_offset << 1) + 0x180000];
 		case 0x17:  // Kanji character data
@@ -330,7 +327,7 @@ WRITE8_MEMBER( towns_state::towns_video_cff80_w )
 READ8_MEMBER( towns_state::towns_video_cff80_mem_r )
 {
 	if(m_towns_mainmem_enable != 0)
-		return ram_get_ptr(m_messram)[offset+0xcff80];
+		return m_messram->pointer()[offset+0xcff80];
 
 	return towns_video_cff80_r(space,offset);
 }
@@ -339,7 +336,7 @@ WRITE8_MEMBER( towns_state::towns_video_cff80_mem_w )
 {
 	if(m_towns_mainmem_enable != 0)
 	{
-		ram_get_ptr(m_messram)[offset+0xcff80] = data;
+		m_messram->pointer()[offset+0xcff80] = data;
 		return;
 	}
 	towns_video_cff80_w(space,offset,data);
@@ -577,6 +574,11 @@ WRITE8_MEMBER(towns_state::towns_video_ff81_w)
 	logerror("VGA: VRAM wplane select (I/O) = 0x%02x\n",m_video.towns_vram_wplane);
 }
 
+READ32_MEMBER(towns_state::towns_video_unknown_r)
+{
+	return 0x00000000;
+}
+
 /*
  *  Sprite RAM, low memory
  *  Writing to 0xc8xxx or 0xcaxxx activates TVRAM
@@ -589,8 +591,8 @@ WRITE8_MEMBER(towns_state::towns_video_ff81_w)
  */
 READ8_MEMBER(towns_state::towns_spriteram_low_r)
 {
-	UINT8* RAM = ram_get_ptr(m_messram);
-	UINT8* ROM = space.machine().region("user")->base();
+	UINT8* RAM = m_messram->pointer();
+	UINT8* ROM = space.machine().root_device().memregion("user")->base();
 
 	if(offset < 0x1000)
 	{  // 0xc8000-0xc8fff
@@ -627,7 +629,7 @@ READ8_MEMBER(towns_state::towns_spriteram_low_r)
 
 WRITE8_MEMBER(towns_state::towns_spriteram_low_w)
 {
-	UINT8* RAM = ram_get_ptr(m_messram);
+	UINT8* RAM = m_messram->pointer();
 
 	if(offset < 0x1000)
 	{  // 0xc8000-0xc8fff
@@ -689,7 +691,7 @@ static void render_sprite_4(running_machine &machine, UINT32 poffset, UINT32 cof
 	UINT16 xstart,xend,ystart,yend;
 	int xdir,ydir;
 	int width = (state->m_video.towns_crtc_reg[12] - state->m_video.towns_crtc_reg[11]) / (((state->m_video.towns_crtc_reg[27] & 0x0f00) >> 8)+1);
-	int height = (state->m_video.towns_crtc_reg[16] - state->m_video.towns_crtc_reg[15]) / (((state->m_video.towns_crtc_reg[27] & 0xf000) >> 12)+1);
+	int height = (state->m_video.towns_crtc_reg[16] - state->m_video.towns_crtc_reg[15]) / (((state->m_video.towns_crtc_reg[27] & 0xf000) >> 12)+2);
 
 	if(xflip)
 	{
@@ -733,11 +735,14 @@ static void render_sprite_4(running_machine &machine, UINT32 poffset, UINT32 cof
 			col = state->m_towns_txtvram[coffset+(pixel*2)] | (state->m_towns_txtvram[coffset+(pixel*2)+1] << 8);
 			voffset += (state->m_video.towns_crtc_reg[24] * 4) * (ypos & 0x1ff);  // scanline size in bytes * y pos
 			voffset += (xpos & 0x1ff) * 2;
-			voffset &= 0x3ffff;
-			if(xpos < width && ypos < height && pixel != 0)
+			if((state->m_video.towns_sprite_page != 0 && voffset > 0x1ffff && voffset < 0x40000)
+					|| (state->m_video.towns_sprite_page == 0 && voffset < 0x20000))
 			{
-				state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
-				state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				if(xpos < width && ypos < height && pixel != 0)
+				{
+					state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
+					state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				}
 			}
 			if(xflip)
 				voffset+=2;
@@ -745,11 +750,14 @@ static void render_sprite_4(running_machine &machine, UINT32 poffset, UINT32 cof
 				voffset-=2;
 			pixel = state->m_towns_txtvram[poffset] & 0x0f;
 			col = state->m_towns_txtvram[coffset+(pixel*2)] | (state->m_towns_txtvram[coffset+(pixel*2)+1] << 8);
-			voffset &= 0x3ffff;
-			if(xpos < width && ypos < height && pixel != 0)
+			if((state->m_video.towns_sprite_page != 0 && voffset > 0x1ffff && voffset < 0x40000)
+					|| (state->m_video.towns_sprite_page == 0 && voffset < 0x20000))
 			{
-				state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
-				state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				if(xpos < width && ypos < height && pixel != 0)
+				{
+					state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
+					state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				}
 			}
 			poffset++;
 			poffset &= 0x1ffff;
@@ -766,7 +774,7 @@ static void render_sprite_16(running_machine &machine, UINT32 poffset, UINT16 x,
 	UINT16 xstart,ystart,xend,yend;
 	int xdir,ydir;
 	int width = (state->m_video.towns_crtc_reg[12] - state->m_video.towns_crtc_reg[11]) / (((state->m_video.towns_crtc_reg[27] & 0x0f00) >> 8)+1);
-	int height = (state->m_video.towns_crtc_reg[16] - state->m_video.towns_crtc_reg[15]) / (((state->m_video.towns_crtc_reg[27] & 0xf000) >> 12)+1);
+	int height = (state->m_video.towns_crtc_reg[16] - state->m_video.towns_crtc_reg[15]) / (((state->m_video.towns_crtc_reg[27] & 0xf000) >> 12)+2);
 
 	if(xflip)
 	{
@@ -809,11 +817,14 @@ static void render_sprite_16(running_machine &machine, UINT32 poffset, UINT16 x,
 			col = state->m_towns_txtvram[poffset] | (state->m_towns_txtvram[poffset+1] << 8);
 			voffset += (state->m_video.towns_crtc_reg[24] * 4) * (ypos & 0x1ff);  // scanline size in bytes * y pos
 			voffset += (xpos & 0x1ff) * 2;
-			voffset &= 0x3ffff;
-			if(xpos < width && ypos < height && col < 0x8000)
+			if((state->m_video.towns_sprite_page != 0 && voffset > 0x1ffff && voffset < 0x40000)
+					|| (state->m_video.towns_sprite_page == 0 && voffset < 0x20000))
 			{
-				state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
-				state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				if(xpos < width && ypos < height && col < 0x8000)
+				{
+					state->m_towns_gfxvram[0x40000+voffset+1] = (col & 0xff00) >> 8;
+					state->m_towns_gfxvram[0x40000+voffset] = col & 0x00ff;
+				}
 			}
 			poffset+=2;
 			poffset &= 0x1ffff;
@@ -887,7 +898,7 @@ static void draw_sprites(running_machine &machine, const rectangle* rect)
 	state->m_video.sprite_timer->adjust(machine.device<cpu_device>("maincpu")->cycles_to_attotime(128 * (1025-sprite_limit)));
 }
 
-static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap_t* bitmap,const rectangle* rect,int layer,int line,int scanline)
+static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap_rgb32 &bitmap,const rectangle* rect,int layer,int line,int scanline)
 {
 	towns_state* state = machine.driver_data<towns_state>();
 	UINT32 off = 0;
@@ -950,7 +961,7 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 			colour = (state->m_towns_gfxvram[off+(layer*0x40000)+1] << 8) | state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour < 0x8000)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
@@ -970,11 +981,11 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 			colour = (state->m_towns_gfxvram[off+(layer*0x40000)+1] << 8) | state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour < 0x8000)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
@@ -994,15 +1005,15 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 			colour = (state->m_towns_gfxvram[off+(layer*0x40000)+1] << 8) | state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour < 0x8000)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
@@ -1022,19 +1033,19 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 			colour = (state->m_towns_gfxvram[off+(layer*0x40000)+1] << 8) | state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour < 0x8000)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
@@ -1054,23 +1065,23 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 			colour = (state->m_towns_gfxvram[off+(layer*0x40000)+1] << 8) | state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour < 0x8000)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
-				*BITMAP_ADDR32(bitmap,scanline,x+4) =
+				bitmap.pix32(scanline, x+4) =
 					((colour & 0x001f) << 3)
 					| ((colour & 0x7c00) << 1)
 					| ((colour & 0x03e0) << 14);
@@ -1080,7 +1091,7 @@ static void towns_crtc_draw_scan_layer_hicolour(running_machine &machine, bitmap
 	}
 }
 
-static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* bitmap,const rectangle* rect,int layer,int line,int scanline)
+static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_rgb32 &bitmap,const rectangle* rect,int layer,int line,int scanline)
 {
 	towns_state* state = machine.driver_data<towns_state>();
 	int off = 0;
@@ -1141,7 +1152,7 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)];
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1161,11 +1172,11 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)+1];
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1185,15 +1196,15 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)+1];
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1213,19 +1224,19 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)+1];
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1245,23 +1256,23 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)+1];
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+4) =
+				bitmap.pix32(scanline, x+4) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1271,7 +1282,7 @@ static void towns_crtc_draw_scan_layer_256(running_machine &machine, bitmap_t* b
 	}
 }
 
-static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bitmap,const rectangle* rect,int layer,int line,int scanline)
+static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_rgb32 &bitmap,const rectangle* rect,int layer,int line,int scanline)
 {
 	towns_state* state = machine.driver_data<towns_state>();
 	int off = 0;
@@ -1332,7 +1343,7 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] >> 4;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1340,7 +1351,7 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] & 0x0f;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1360,11 +1371,11 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] >> 4;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1372,11 +1383,11 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] & 0x0f;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1396,15 +1407,15 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] >> 4;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+4) =
+				bitmap.pix32(scanline, x+4) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+5) =
+				bitmap.pix32(scanline, x+5) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1412,15 +1423,15 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] & 0x0f;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1440,19 +1451,19 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] >> 4;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x+4) =
+				bitmap.pix32(scanline, x+4) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+5) =
+				bitmap.pix32(scanline, x+5) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+6) =
+				bitmap.pix32(scanline, x+6) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+7) =
+				bitmap.pix32(scanline, x+7) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1460,19 +1471,19 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] & 0x0f;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1492,23 +1503,23 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] >> 4;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x+5) =
+				bitmap.pix32(scanline, x+5) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+6) =
+				bitmap.pix32(scanline, x+6) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+7) =
+				bitmap.pix32(scanline, x+7) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+8) =
+				bitmap.pix32(scanline, x+8) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+9) =
+				bitmap.pix32(scanline, x+9) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1516,23 +1527,23 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 			colour = state->m_towns_gfxvram[off+(layer*0x40000)] & 0x0f;
 			if(colour != 0)
 			{
-				*BITMAP_ADDR32(bitmap,scanline,x) =
+				bitmap.pix32(scanline, x) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+1) =
+				bitmap.pix32(scanline, x+1) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+2) =
+				bitmap.pix32(scanline, x+2) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+3) =
+				bitmap.pix32(scanline, x+3) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
-				*BITMAP_ADDR32(bitmap,scanline,x+4) =
+				bitmap.pix32(scanline, x+4) =
 					(state->m_video.towns_palette_r[colour] << 16)
 					| (state->m_video.towns_palette_g[colour] << 8)
 					| (state->m_video.towns_palette_b[colour]);
@@ -1542,7 +1553,7 @@ static void towns_crtc_draw_scan_layer_16(running_machine &machine, bitmap_t* bi
 	}
 }
 
-static void towns_crtc_draw_layer(running_machine &machine,bitmap_t* bitmap,const rectangle* rect,int layer)
+static void towns_crtc_draw_layer(running_machine &machine,bitmap_rgb32 &bitmap,const rectangle* rect,int layer)
 {
 	towns_state* state = machine.driver_data<towns_state>();
 	int line;
@@ -1653,7 +1664,7 @@ static void render_text_char(running_machine &machine, UINT8 x, UINT8 y, UINT8 a
 	UINT8 colour;
 	UINT8 data;
 	UINT8 temp;
-	UINT8* font_rom = machine.region("user")->base();
+	UINT8* font_rom = state->memregion("user")->base();
 	int a,b;
 
 	// all characters are 16 pixels high
@@ -1798,21 +1809,21 @@ void towns_state::video_start()
 	m_video.sprite_timer = machine().scheduler().timer_alloc(FUNC(towns_sprite_done));
 }
 
-bool towns_state::screen_update(screen_device &screen, bitmap_t &bitmap, const rectangle &cliprect)
+UINT32 towns_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
-	bitmap_fill(&bitmap,&cliprect,0x00000000);
+	bitmap.fill(0x00000000, cliprect);
 
 	if(!(m_video.towns_video_reg[1] & 0x01))
 	{
 		if(!screen.machine().input().code_pressed(KEYCODE_Q))
 		{
 			if((m_video.towns_layer_ctrl & 0x03) != 0)
-				towns_crtc_draw_layer(screen.machine(),&bitmap,&m_video.towns_crtc_layerscr[1],1);
+				towns_crtc_draw_layer(screen.machine(),bitmap,&m_video.towns_crtc_layerscr[1],1);
 		}
 		if(!screen.machine().input().code_pressed(KEYCODE_W))
 		{
 			if((m_video.towns_layer_ctrl & 0x0c) != 0)
-				towns_crtc_draw_layer(screen.machine(),&bitmap,&m_video.towns_crtc_layerscr[0],0);
+				towns_crtc_draw_layer(screen.machine(),bitmap,&m_video.towns_crtc_layerscr[0],0);
 		}
 	}
 	else
@@ -1820,20 +1831,20 @@ bool towns_state::screen_update(screen_device &screen, bitmap_t &bitmap, const r
 		if(!screen.machine().input().code_pressed(KEYCODE_Q))
 		{
 			if((m_video.towns_layer_ctrl & 0x0c) != 0)
-				towns_crtc_draw_layer(screen.machine(),&bitmap,&m_video.towns_crtc_layerscr[0],0);
+				towns_crtc_draw_layer(screen.machine(),bitmap,&m_video.towns_crtc_layerscr[0],0);
 		}
 		if(!screen.machine().input().code_pressed(KEYCODE_W))
 		{
 			if((m_video.towns_layer_ctrl & 0x03) != 0)
-				towns_crtc_draw_layer(screen.machine(),&bitmap,&m_video.towns_crtc_layerscr[1],1);
+				towns_crtc_draw_layer(screen.machine(),bitmap,&m_video.towns_crtc_layerscr[1],1);
 		}
 	}
 
 #if 0
 #ifdef SPR_DEBUG
-	if(screen->machine().input().code_pressed(KEYCODE_O))
+	if(screen.machine().input().code_pressed(KEYCODE_O))
 		pshift+=0x80;
-	if(screen->machine().input().code_pressed(KEYCODE_I))
+	if(screen.machine().input().code_pressed(KEYCODE_I))
 		pshift-=0x80;
 	popmessage("Pixel shift = %08x",pshift);
 #endif
@@ -1841,15 +1852,15 @@ bool towns_state::screen_update(screen_device &screen, bitmap_t &bitmap, const r
 
 #ifdef CRTC_REG_DISP
 	popmessage("CRTC: %i %i %i %i %i %i %i %i %i\n%i %i %i %i | %i %i %i %i\n%04x %i %i %i | %04x %i %i %i\nZOOM: %04x\nVideo: %02x %02x\nText=%i Spr=%02x\nReg28=%04x",
-		state->m_video.towns_crtc_reg[0],state->m_video.towns_crtc_reg[1],state->m_video.towns_crtc_reg[2],state->m_video.towns_crtc_reg[3],
-		state->m_video.towns_crtc_reg[4],state->m_video.towns_crtc_reg[5],state->m_video.towns_crtc_reg[6],state->m_video.towns_crtc_reg[7],
-		state->m_video.towns_crtc_reg[8],
-		state->m_video.towns_crtc_reg[9],state->m_video.towns_crtc_reg[10],state->m_video.towns_crtc_reg[11],state->m_video.towns_crtc_reg[12],
-		state->m_video.towns_crtc_reg[13],state->m_video.towns_crtc_reg[14],state->m_video.towns_crtc_reg[15],state->m_video.towns_crtc_reg[16],
-		state->m_video.towns_crtc_reg[17],state->m_video.towns_crtc_reg[18],state->m_video.towns_crtc_reg[19],state->m_video.towns_crtc_reg[20],
-		state->m_video.towns_crtc_reg[21],state->m_video.towns_crtc_reg[22],state->m_video.towns_crtc_reg[23],state->m_video.towns_crtc_reg[24],
-		state->m_video.towns_crtc_reg[27],state->m_video.towns_video_reg[0],state->m_video.towns_video_reg[1],state->m_video.towns_tvram_enable,state->m_video.towns_sprite_reg[1] & 0x80,
-		state->m_video.towns_crtc_reg[28]);
+		m_video.towns_crtc_reg[0],m_video.towns_crtc_reg[1],m_video.towns_crtc_reg[2],m_video.towns_crtc_reg[3],
+		m_video.towns_crtc_reg[4],m_video.towns_crtc_reg[5],m_video.towns_crtc_reg[6],m_video.towns_crtc_reg[7],
+		m_video.towns_crtc_reg[8],
+		m_video.towns_crtc_reg[9],m_video.towns_crtc_reg[10],m_video.towns_crtc_reg[11],m_video.towns_crtc_reg[12],
+		m_video.towns_crtc_reg[13],m_video.towns_crtc_reg[14],m_video.towns_crtc_reg[15],m_video.towns_crtc_reg[16],
+		m_video.towns_crtc_reg[17],m_video.towns_crtc_reg[18],m_video.towns_crtc_reg[19],m_video.towns_crtc_reg[20],
+		m_video.towns_crtc_reg[21],m_video.towns_crtc_reg[22],m_video.towns_crtc_reg[23],m_video.towns_crtc_reg[24],
+		m_video.towns_crtc_reg[27],m_video.towns_video_reg[0],m_video.towns_video_reg[1],m_video.towns_tvram_enable,m_video.towns_sprite_reg[1] & 0x80,
+		m_video.towns_crtc_reg[28]);
 #endif
 
 	return 0;
