@@ -452,7 +452,7 @@ located at I/O port 0x3CE, and a data register located at I/O port 0x3CF.
 /*
     Prototypes
 */
-static SCREEN_UPDATE( pc_ega );
+static SCREEN_UPDATE_IND16( pc_ega );
 static CRTC_EGA_UPDATE_ROW( ega_update_row );
 static CRTC_EGA_ON_DE_CHANGED( ega_de_changed );
 static CRTC_EGA_ON_VSYNC_CHANGED( ega_vsync_changed );
@@ -474,9 +474,8 @@ static const crtc_ega_interface crtc_ega_ega_intf =
 
 MACHINE_CONFIG_FRAGMENT( pcvideo_ega )
 	MCFG_SCREEN_ADD(EGA_SCREEN_NAME, RASTER)
-	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
 	MCFG_SCREEN_RAW_PARAMS(16257000,912,0,640,262,0,200)
-	MCFG_SCREEN_UPDATE( pc_ega )
+	MCFG_SCREEN_UPDATE_STATIC( pc_ega )
 
 	MCFG_PALETTE_LENGTH( 64 )
 	MCFG_CRTC_EGA_ADD(EGA_CRTC_NAME, 16257000/8, crtc_ega_ega_intf)
@@ -488,6 +487,48 @@ ROM_START( ega )
 	ROM_REGION(0x4000, "user2", ROMREGION_ERASE00)
 ROM_END
 
+/*
+0000 - MONOC PRIMARY, EGA COLOR, 40x25
+0001 - MONOC PRIMARY, EGA COLOR, 80x25
+0010 - MONOC PRIMARY, EGA HI RES EMULATE (SAME AS 0001)
+0011 - MONOC PRIMARY, EGA HI RES ENHANCED
+0100 - COLOR 40 PRIMARY, EGA MONOCHROME
+0101 - COLOR 80 PRIMARY, EGA MONOCHROME
+
+0110 - MONOC SECONDARY, EGA COLOR, 40x24
+0111 - MONOC SECONDARY, EGA COLOR, 80x25
+1000 - MONOC SECONDARY, EGA HI RES EMULATE (SAME AS 0111)
+1001 - MONOC SECONDARY, EGA HI RES ENHANCED
+1010 - COLOR 40 SECONDARY, EGA
+1011 - COLOR 80 SECONDARY, EGA
+
+1100 - RESERVED
+1101 - RESERVED
+1110 - RESERVED
+1111 - RESERVED
+*/
+
+INPUT_PORTS_START( ega )
+	PORT_START( "config" )
+	PORT_CONFNAME( 0x0f, 0x0e, "dipswitches" )
+	PORT_CONFSETTING( 0x00, "0000 - MDA PRIMARY, EGA COLOR, 40x25" )
+	PORT_CONFSETTING( 0x08, "0001 - MDA PRIMARY, EGA COLOR, 80x25" )
+	PORT_CONFSETTING( 0x04, "0010 - MDA PRIMARY, EGA HI RES EMULATE (SAME AS 0001)" )
+	PORT_CONFSETTING( 0x0c, "0011 - MDA PRIMARY, EGA HI RES ENHANCED" )
+	PORT_CONFSETTING( 0x02, "0100 - CGA 40 PRIMARY, EGA MONOCHROME" )
+	PORT_CONFSETTING( 0x0a, "0101 - CGA 80 PRIMARY, EGA MONOCHROME" )
+	PORT_CONFSETTING( 0x06, "0110 - MDA SECONDARY, EGA COLOR, 40x25" )
+	PORT_CONFSETTING( 0x0e, "0111 - MDA SECONDARY, EGA COLOR, 80x25" )
+	PORT_CONFSETTING( 0x01, "1000 - MDA SECONDARY, EGA HI RES EMULATE (SAME AS 0111)" )
+	PORT_CONFSETTING( 0x09, "1001 - MDA SECONDARY, EGA HI RES ENHANCED" )
+	PORT_CONFSETTING( 0x05, "1010 - COLOR 40 SECONDARY, EGA" )
+	PORT_CONFSETTING( 0x0d, "1011 - COLOR 80 SECONDARY, EGA" )
+	PORT_CONFSETTING( 0x03, "1100 - RESERVED" )
+	PORT_CONFSETTING( 0x0b, "1101 - RESERVED" )
+	PORT_CONFSETTING( 0x07, "1110 - RESERVED" )
+	PORT_CONFSETTING( 0x0f, "1111 - RESERVED" )
+INPUT_PORTS_END
+
 //**************************************************************************
 //  GLOBAL VARIABLES
 //**************************************************************************
@@ -514,6 +555,11 @@ const rom_entry *isa8_ega_device::device_rom_region() const
 	return ROM_NAME( ega );
 }
 
+ioport_constructor isa8_ega_device::device_input_ports() const
+{
+	return INPUT_PORTS_NAME( ega );
+}
+
 //**************************************************************************
 //  LIVE DEVICE
 //**************************************************************************
@@ -523,7 +569,7 @@ const rom_entry *isa8_ega_device::device_rom_region() const
 //-------------------------------------------------
 
 isa8_ega_device::isa8_ega_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock) :
-        device_t(mconfig, ISA8_EGA, "ISA8_EGA", tag, owner, clock),
+        device_t(mconfig, ISA8_EGA, "IBM Enhanced Graphics Adapter", tag, owner, clock),
 		device_isa8_card_interface(mconfig, *this)
 {
 	m_shortname = "ega";
@@ -552,8 +598,8 @@ void isa8_ega_device::device_start()
 		palette_set_color_rgb( machine(), i, r, g, b );
 	}
 	astring tempstring;
-	UINT8	*dst = machine().region(subtag(tempstring, "user2" ))->base() + 0x0000;
-	UINT8	*src = machine().region(subtag(tempstring, "user1" ))->base() + 0x3fff;
+	UINT8	*dst = memregion(subtag(tempstring, "user2" ))->base() + 0x0000;
+	UINT8	*src = memregion(subtag(tempstring, "user1" ))->base() + 0x3fff;
 	int		i;
 
 	/* Perform the EGA bios address line swaps */
@@ -657,9 +703,9 @@ void isa8_ega_device::install_banks()
 	}
 }
 
-static SCREEN_UPDATE( pc_ega )
+static SCREEN_UPDATE_IND16( pc_ega )
 {
-	isa8_ega_device *ega = dynamic_cast<isa8_ega_device*>(screen->owner());
+	isa8_ega_device *ega = dynamic_cast<isa8_ega_device*>(screen.owner());
 	crtc_ega_update( ega->m_crtc_ega, bitmap, cliprect);
 	return 0;
 }
@@ -702,14 +748,33 @@ static CRTC_EGA_ON_VBLANK_CHANGED( ega_vblank_changed )
 
 static CRTC_EGA_UPDATE_ROW( pc_ega_graphics )
 {
-	UINT16	*p = BITMAP_ADDR16(bitmap, y, 0);
+	isa8_ega_device *ega = dynamic_cast<isa8_ega_device*>(device->owner());
+	UINT16	*p = &bitmap.pix16(y);
 	int	i;
 
 //  logerror( "pc_ega_graphics: y = %d, x_count = %d, ma = %d, ra = %d\n", y, x_count, ma, ra );
 
-	for ( i = 0; i < x_count; i++ )
+	switch ( ega->m_attribute.data[ 0x12 ] & 0x0f )
 	{
-		*p = 0;
+	case 0x03:
+		for ( i = 0; i < x_count; i++ )
+		{
+			UINT16 offset = ( ( ( ma + i ) << 1 ) & 0x1fff ) | ( ( y & 1 ) << 13 );
+			UINT8 data = ega->m_videoram[ offset ];
+
+			*p = ega->m_attribute.data[ ( data >> 6 )        ]; p++;
+			*p = ega->m_attribute.data[ ( data >> 4 ) & 0x03 ]; p++;
+			*p = ega->m_attribute.data[ ( data >> 2 ) & 0x03 ]; p++;
+			*p = ega->m_attribute.data[   data        & 0x03 ]; p++;
+
+			data = ega->m_videoram[ offset + 1 ];
+
+			*p = ega->m_attribute.data[ ( data >> 6 )        ]; p++;
+			*p = ega->m_attribute.data[ ( data >> 4 ) & 0x03 ]; p++;
+			*p = ega->m_attribute.data[ ( data >> 2 ) & 0x03 ]; p++;
+			*p = ega->m_attribute.data[   data        & 0x03 ]; p++;
+		}
+		break;
 	}
 }
 
@@ -717,7 +782,7 @@ static CRTC_EGA_UPDATE_ROW( pc_ega_graphics )
 static CRTC_EGA_UPDATE_ROW( pc_ega_text )
 {
 	isa8_ega_device *ega = dynamic_cast<isa8_ega_device*>(device->owner());
-	UINT16	*p = BITMAP_ADDR16(bitmap, y, 0);
+	UINT16	*p = &bitmap.pix16(y);
 	int	i;
 
 //  logerror( "pc_ega_text: y = %d, x_count = %d, ma = %d, ra = %d\n", y, x_count, ma, ra );
@@ -727,9 +792,38 @@ static CRTC_EGA_UPDATE_ROW( pc_ega_text )
 		UINT16	offset = ( ma + i ) << 1;
 		UINT8	chr = ega->m_videoram[ offset ];
 		UINT8	attr = ega->m_videoram[ offset + 1 ];
-		UINT8	data = ( attr & 0x08 ) ? ega->m_charA[ chr * 32 + ra ] : ega->m_charB[ chr * 32 + ra ];
-		UINT16	fg = ega->m_attribute.data[ attr & 0x0F ];
-		UINT16	bg = ega->m_attribute.data[ attr >> 4 ];
+		UINT8	data = 0;
+		UINT16	fg = ega->m_attribute.data[ attr & 0x07 ];
+		UINT16	bg = ega->m_attribute.data[ ( attr >> 4 ) & 0x07 ];
+
+		/* Is attribute bit 3 used as intensity or as character set selector? */
+		if ( ega->m_attribute.data[ 0x12 ] & 0x08 )
+		{
+			/* intensity selector */
+			data = ega->m_charB[ chr * 32 + ra ];
+			fg += ( attr & 0x08 ) ? 0x38 : 0x00;
+		}
+		else
+		{
+			/* character set selector */
+			data = ( attr & 0x08 ) ? ega->m_charA[ chr * 32 + ra ] : ega->m_charB[ chr * 32 + ra ];
+		}
+
+		if ( i == cursor_x )
+		{
+			if ( ega->m_frame_cnt & 0x08 )
+			{
+				data = 0xFF;
+			}
+		}
+		else
+		{
+			/* Check for blinking */
+			if ( ( ega->m_attribute.data[0x10] & 0x08 ) && ( attr & 0x80 ) && ( ega->m_frame_cnt & 0x10 ) )
+			{
+				data = 0x00;
+			}
+		}
 
 		*p = ( data & 0x80 ) ? fg : bg; p++;
 		*p = ( data & 0x40 ) ? fg : bg; p++;
@@ -913,27 +1007,6 @@ WRITE8_MEMBER(isa8_ega_device::pc_ega8_3d0_w )
 
 READ8_MEMBER(isa8_ega_device::pc_ega8_3c0_r )
 {
-	UINT8	dips = 0x0e; // bits in reverse order 0111 == > 1110
-/*
-0000 - MONOC PRIMARY, EGA COLOR, 40x25
-0001 - MONOC PRIMARY, EGA COLOR, 80x25
-0010 - MONOC PRIMARY, EGA HI RES EMULATE (SAME AS 0001)
-0011 - MONOC PRIMARY, EGA HI RES ENHANCED
-0100 - COLOR 40 PRIMARY, EGA MONOCHROME
-0101 - COLOR 80 PRIMARY, EGA MONOCHROME
-
-0110 - MONOC SECONDARY, EGA COLOR, 40x24
-0111 - MONOC SECONDARY, EGA COLOR, 80x25
-1000 - MONOC SECONDARY, EGA HI RES EMULATE (SAME AS 0111)
-1001 - MONOC SECONDARY, EGA HI RES ENHANCED
-1010 - COLOR 40 SECONDARY, EGA
-1011 - COLOR 80 SECONDARY, EGA
-
-1100 - RESERVED
-1101 - RESERVED
-1110 - RESERVED
-1111 - RESERVED
-*/
 	int data = 0xff;
 
 	if ( VERBOSE_EGA )
@@ -949,10 +1022,14 @@ READ8_MEMBER(isa8_ega_device::pc_ega8_3c0_r )
 
 	/* Feature Read */
 	case 2:
-		data = ( data & 0x0f );
-		data |= ( ( m_feature_control & 0x03 ) << 5 );
-		data |= ( m_vsync ? 0x00 : 0x80 );
-		data |= ( ( ( dips >> ( ( ( m_misc_output & 0x0c ) >> 2 ) ) ) & 0x01 ) << 4 );
+		{
+			UINT8 dips = ioport("config")->read();
+
+			data = ( data & 0x0f );
+			data |= ( ( m_feature_control & 0x03 ) << 5 );
+			data |= ( m_vsync ? 0x00 : 0x80 );
+			data |= ( ( ( dips >> ( ( ( m_misc_output & 0x0c ) >> 2 ) ) ) & 0x01 ) << 4 );
+		}
 		break;
 
 	/* Sequencer */
