@@ -38,7 +38,7 @@ PALETTE_INIT( k2ge );
 WRITE8_DEVICE_HANDLER( k1ge_w );
 READ8_DEVICE_HANDLER( k1ge_r );
 
-void k1ge_update( device_t *device, bitmap_t *bitmap, const rectangle *cliprect );
+void k1ge_update( device_t *device, bitmap_ind16 &bitmap, const rectangle &cliprect );
 
 #endif
 
