@@ -114,7 +114,7 @@ void kc85_video_set_blink_state(running_machine &machine, int data)
 
 
 /* draw 8 pixels */
-static void kc85_draw_8_pixels(kc_state *state, bitmap_t *bitmap,int x,int y, unsigned char colour_byte, unsigned char gfx_byte)
+static void kc85_draw_8_pixels(kc_state *state, bitmap_ind16 &bitmap,int x,int y, unsigned char colour_byte, unsigned char gfx_byte)
 {
 	int background_pen;
 	int foreground_pen;
@@ -136,9 +136,9 @@ static void kc85_draw_8_pixels(kc_state *state, bitmap_t *bitmap,int x,int y, un
 		{
 			int pen = pens[((gfx_byte>>7) & 0x07) | ((colour_byte>>6) & 0x02)];
 
-			if ((px >= 0) && (px < bitmap->width) && (y >= 0) && (y < bitmap->height))
+			if ((px >= 0) && (px < bitmap.width()) && (y >= 0) && (y < bitmap.height()))
 			{
-				*BITMAP_ADDR16(bitmap, y, px) = pen;
+				bitmap.pix16(y, px) = pen;
 			}
 
 			px++;
@@ -179,9 +179,9 @@ static void kc85_draw_8_pixels(kc_state *state, bitmap_t *bitmap,int x,int y, un
 		{
 			int pen = pens[(gfx_byte>>7) & 0x01];
 
-			if ((px >= 0) && (px < bitmap->width) && (y >= 0) && (y < bitmap->height))
+			if ((px >= 0) && (px < bitmap.width()) && (y >= 0) && (y < bitmap.height()))
 			{
-				*BITMAP_ADDR16(bitmap, y, px) = pen;
+				bitmap.pix16(y, px) = pen;
 			}
 			px++;
 			gfx_byte = gfx_byte<<1;
@@ -294,7 +294,7 @@ struct grab_info
 struct video_update_state
 {
 	/* bitmap to render to */
-	bitmap_t *bitmap;
+	bitmap_ind16 *bitmap;
 	/* grab colour and pixel information for 8-pixels referenced by x,y coordinate */
 	void (*pixel_grab_callback)(struct grab_info *,int x,int y, unsigned char *colour_ptr, unsigned char *pixel_ptr);
 	/* current coords */
@@ -344,7 +344,7 @@ static void kc85_common_process_cycles(kc_state *state, struct video_update_stat
 					/* grab colour and pixel information */
 					video_update->pixel_grab_callback(&video_update->grab_data,video_update->x,video_update->y,&colour_byte, &gfx_byte);
 					/* draw to screen */
-					kc85_draw_8_pixels(state, video_update->bitmap, video_update->render_x, video_update->render_y,colour_byte, gfx_byte);
+					kc85_draw_8_pixels(state, *video_update->bitmap, video_update->render_x, video_update->render_y,colour_byte, gfx_byte);
 					/* update render coordinate */
 					video_update->render_x+=8;
 					video_update->x++;
@@ -514,7 +514,7 @@ static void kc85_common_vh_process_lines(kc_state *state, struct video_update_st
 
 /* the kc85 screen is 320 pixels wide and 256 pixels tall */
 /* if we assume a 50Hz display, there are 312 lines for the complete frame, leaving 56 lines not visible */
-static void kc85_common_process_frame(running_machine &machine, bitmap_t *bitmap, void (*pixel_grab_callback)(struct grab_info *,int x,int y,unsigned char *, unsigned char *),struct grab_info *grab_data)
+static void kc85_common_process_frame(running_machine &machine, bitmap_ind16 &bitmap, void (*pixel_grab_callback)(struct grab_info *,int x,int y,unsigned char *, unsigned char *),struct grab_info *grab_data)
 {
 	kc_state *state = machine.driver_data<kc_state>();
 	int cycles_remaining_in_frame = KC85_CYCLES_PER_FRAME;
@@ -530,7 +530,7 @@ static void kc85_common_process_frame(running_machine &machine, bitmap_t *bitmap
 	video_update.x = 0;
 	video_update.y = 0;
 	memcpy(&video_update.grab_data, grab_data, sizeof(struct grab_info));
-	video_update.bitmap = bitmap;
+	video_update.bitmap = &bitmap;
 	video_update.pixel_grab_callback = pixel_grab_callback;
 	video_update.horizontal.state = 0;
 	video_update.horizontal.cycles_remaining_in_state = horizontal_graphics_state_cycles[video_update.horizontal.state];
@@ -653,13 +653,13 @@ static void kc85_4_pixel_grab_callback(struct grab_info *grab_data,int x,int y,
 
 
 /***************************************************************************
-  Draw the game screen in the given bitmap_t.
+  Draw the game screen in the given bitmap_ind16.
   Do NOT call osd_update_display() from this function,
   it will be called by the main emulation engine.
 ***************************************************************************/
-SCREEN_UPDATE( kc85_4 )
+SCREEN_UPDATE_IND16( kc85_4 )
 {
-	kc85_4_state *state = screen->machine().driver_data<kc85_4_state>();
+	kc85_4_state *state = screen.machine().driver_data<kc85_4_state>();
 #if 0
 	unsigned char *pixel_ram = state->m_display_video_ram;
 	unsigned char *colour_ram = pixel_ram + 0x04000;
@@ -688,7 +688,7 @@ SCREEN_UPDATE( kc85_4 )
 	grab_data.pixel_ram = state->m_display_video_ram;
 	grab_data.colour_ram = state->m_display_video_ram + 0x04000;
 
-	kc85_common_process_frame(screen->machine(), bitmap, kc85_4_pixel_grab_callback,&grab_data);
+	kc85_common_process_frame(screen.machine(), bitmap, kc85_4_pixel_grab_callback,&grab_data);
 
 	return 0;
 }
@@ -733,17 +733,17 @@ static void kc85_3_pixel_grab_callback(struct grab_info *grab_data,int x,int y,
 }
 
 /***************************************************************************
-  Draw the game screen in the given bitmap_t.
+  Draw the game screen in the given bitmap_ind16.
   Do NOT call osd_update_display() from this function,
   it will be called by the main emulation engine.
 ***************************************************************************/
-SCREEN_UPDATE( kc85_3 )
+SCREEN_UPDATE_IND16( kc85_3 )
 {
-	kc_state *state = screen->machine().driver_data<kc_state>();
+	kc_state *state = screen.machine().driver_data<kc_state>();
 
 #if 0
 	/* colour ram takes up 0x02800 bytes */
-	   unsigned char *pixel_ram = ram_get_ptr(machine.device(RAM_TAG))+0x08000;
+	   unsigned char *pixel_ram = machine.device<ram_device>(RAM_TAG)->pointer()+0x08000;
 	unsigned char *colour_ram = pixel_ram + 0x02800;
 
 	int x,y;
@@ -787,7 +787,7 @@ SCREEN_UPDATE( kc85_3 )
 	grab_data.pixel_ram = state->m_ram_base + 0x04000;
 	grab_data.colour_ram = state->m_ram_base + 0x04000 + 0x02800;
 
-	kc85_common_process_frame(screen->machine(), bitmap, kc85_3_pixel_grab_callback,&grab_data);
+	kc85_common_process_frame(screen.machine(), bitmap, kc85_3_pixel_grab_callback,&grab_data);
 
 	return 0;
 }
