@@ -50,7 +50,6 @@ Apple color FPD      01           11           10   (FPD = Full Page Display)
 
 ***************************************************************************/
 
-#define ADDRESS_MAP_MODERN
 
 #include "emu.h"
 #include "sound/asc.h"
@@ -63,15 +62,13 @@ PALETTE_INIT( mac )
 	palette_set_color_rgb(machine, 1, 0x00, 0x00, 0x00);
 }
 
-// 16-level grayscale
+// 4-level grayscale
 PALETTE_INIT( macgsc )
 {
-	for (int i = 15; i >= 0; i--)
-	{
-		UINT8 component = i | (i<<4);
-
-		palette_set_color_rgb(machine, 15-i, component, component, component);
-	}
+	palette_set_color_rgb(machine, 0, 0xff, 0xff, 0xff);
+	palette_set_color_rgb(machine, 1, 0x7f, 0x7f, 0x7f);
+	palette_set_color_rgb(machine, 2, 0x3f, 0x3f, 0x3f);
+	palette_set_color_rgb(machine, 3, 0x00, 0x00, 0x00);
 }
 
 VIDEO_START( mac )
@@ -81,21 +78,21 @@ VIDEO_START( mac )
 #define MAC_MAIN_SCREEN_BUF_OFFSET	0x5900
 #define MAC_ALT_SCREEN_BUF_OFFSET	0xD900
 
-SCREEN_UPDATE( mac )
+SCREEN_UPDATE_IND16( mac )
 {
 	UINT32 video_base;
 	const UINT16 *video_ram;
 	UINT16 word;
 	UINT16 *line;
 	int y, x, b;
-	mac_state *state = screen->machine().driver_data<mac_state>();
+	mac_state *state = screen.machine().driver_data<mac_state>();
 
-	video_base = ram_get_size(screen->machine().device(RAM_TAG)) - (state->m_screen_buffer ? MAC_MAIN_SCREEN_BUF_OFFSET : MAC_ALT_SCREEN_BUF_OFFSET);
-	video_ram = (const UINT16 *) (ram_get_ptr(screen->machine().device(RAM_TAG)) + video_base);
+	video_base = screen.machine().device<ram_device>(RAM_TAG)->size() - (state->m_screen_buffer ? MAC_MAIN_SCREEN_BUF_OFFSET : MAC_ALT_SCREEN_BUF_OFFSET);
+	video_ram = (const UINT16 *) (screen.machine().device<ram_device>(RAM_TAG)->pointer() + video_base);
 
 	for (y = 0; y < MAC_V_VIS; y++)
 	{
-		line = BITMAP_ADDR16(bitmap, y, 0);
+		line = &bitmap.pix16(y);
 
 		for (x = 0; x < MAC_H_VIS; x += 16)
 		{
@@ -109,21 +106,21 @@ SCREEN_UPDATE( mac )
 	return 0;
 }
 
-SCREEN_UPDATE( macse30 )
+SCREEN_UPDATE_IND16( macse30 )
 {
 	UINT32 video_base;
 	const UINT16 *video_ram;
 	UINT16 word;
 	UINT16 *line;
 	int y, x, b;
-	mac_state *state = screen->machine().driver_data<mac_state>();
+	mac_state *state = screen.machine().driver_data<mac_state>();
 
 	video_base = state->m_screen_buffer ? 0x8000 : 0;
 	video_ram = (const UINT16 *) &state->m_vram[video_base/4];
 
 	for (y = 0; y < MAC_V_VIS; y++)
 	{
-		line = BITMAP_ADDR16(bitmap, y, 0);
+		line = &bitmap.pix16(y);
 
 		for (x = 0; x < MAC_H_VIS; x += 16)
 		{
@@ -137,19 +134,19 @@ SCREEN_UPDATE( macse30 )
 	return 0;
 }
 
-SCREEN_UPDATE( macprtb )
+SCREEN_UPDATE_IND16( macprtb )
 {
 	const UINT16 *video_ram;
 	UINT16 word;
 	UINT16 *line;
 	int y, x, b;
-	mac_state *state = screen->machine().driver_data<mac_state>();
+	mac_state *state = screen.machine().driver_data<mac_state>();
 
-	video_ram = (const UINT16 *) state->m_vram;
+	video_ram = (const UINT16 *) state->m_vram16.target();
 
 	for (y = 0; y < 400; y++)
 	{
-		line = BITMAP_ADDR16(bitmap, y, 0);
+		line = &bitmap.pix16(y);
 
 		for (x = 0; x < 640; x += 16)
 		{
@@ -163,19 +160,19 @@ SCREEN_UPDATE( macprtb )
 	return 0;
 }
 
-SCREEN_UPDATE( macpb140 )
+SCREEN_UPDATE_IND16( macpb140 )
 {
 	const UINT16 *video_ram;
 	UINT16 word;
 	UINT16 *line;
 	int y, x, b;
-	mac_state *state = screen->machine().driver_data<mac_state>();
+	mac_state *state = screen.machine().driver_data<mac_state>();
 
-	video_ram = (const UINT16 *) state->m_vram;
+	video_ram = (const UINT16 *) state->m_vram.target();
 
 	for (y = 0; y < 400; y++)
 	{
-		line = BITMAP_ADDR16(bitmap, y, 0);
+		line = &bitmap.pix16(y);
 
 		for (x = 0; x < 640; x += 16)
 		{
@@ -189,24 +186,29 @@ SCREEN_UPDATE( macpb140 )
 	return 0;
 }
 
-SCREEN_UPDATE( macpb160 )
+SCREEN_UPDATE_IND16( macpb160 )
 {
 	UINT16 *line;
 	int y, x;
 	UINT8 pixels;
-	mac_state *state = screen->machine().driver_data<mac_state>();
-	UINT8 *vram8 = (UINT8 *)state->m_vram;
+	mac_state *state = screen.machine().driver_data<mac_state>();
+	UINT8 *vram8 = (UINT8 *)state->m_vram.target();
 
 	for (y = 0; y < 400; y++)
 	{
-		line = BITMAP_ADDR16(bitmap, y, 0);
+		line = &bitmap.pix16(y);
 
-		for (x = 0; x < 640/2; x++)
+		for (x = 0; x < 640/4; x++)
 		{
-			pixels = vram8[(y * 320) + (BYTE4_XOR_BE(x))];
+			pixels = vram8[(y * 160) + (BYTE4_XOR_BE(x))];
+
+//          *line++ = (pixels>>4)&0xf;
+//          *line++ = pixels&0xf;
+            *line++ = ((pixels>>6)&3);
+            *line++ = ((pixels>>4)&3);
+            *line++ = ((pixels>>2)&3);
+            *line++ = (pixels&3);
 
-			*line++ = (pixels>>4)&0xf;
-			*line++ = pixels&0xf;
 		}
 	}
 	return 0;
@@ -249,7 +251,7 @@ VIDEO_RESET(macrbv)
 	visarea.min_x = 0;
 	visarea.min_y = 0;
 	view = 0;
-	mac->m_rbv_montype = input_port_read_safe(machine, "MONTYPE", 2);
+	mac->m_rbv_montype = machine.root_device().ioport("MONTYPE")->read_safe(2);
 	switch (mac->m_rbv_montype)
 	{
 		case 1:	// 15" portrait display
@@ -306,7 +308,7 @@ VIDEO_RESET(macsonora)
 	visarea.min_x = 0;
 	visarea.min_y = 0;
 
-	mac->m_rbv_montype = input_port_read_safe(machine, "MONTYPE", 2);
+	mac->m_rbv_montype = machine.root_device().ioport("MONTYPE")->read_safe(2);
 	switch (mac->m_rbv_montype)
 	{
 		case 1:	// 15" portrait display
@@ -378,12 +380,12 @@ VIDEO_START( macv8 )
 	mac->m_rbv_type = RBV_TYPE_V8;
 }
 
-SCREEN_UPDATE( macrbv )
+SCREEN_UPDATE_RGB32( macrbv )
 {
 	UINT32 *scanline;
 	int x, y, hres, vres;
-	mac_state *mac = screen->machine().driver_data<mac_state>();
-	UINT8 *vram8 = (UINT8 *)ram_get_ptr(screen->machine().device(RAM_TAG));
+	mac_state *mac = screen.machine().driver_data<mac_state>();
+	UINT8 *vram8 = (UINT8 *)screen.machine().device<ram_device>(RAM_TAG)->pointer();
 
 	switch (mac->m_rbv_montype)
 	{
@@ -418,7 +420,7 @@ SCREEN_UPDATE( macrbv )
 
 			for (y = 0; y < vres; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				for (x = 0; x < hres; x+=8)
 				{
 					pixels = vram8[(y * (hres/8)) + ((x/8)^3)];
@@ -442,7 +444,7 @@ SCREEN_UPDATE( macrbv )
 
 			for (y = 0; y < vres; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				for (x = 0; x < hres/4; x++)
 				{
 					pixels = vram8[(y * (hres/4)) + (BYTE4_XOR_BE(x))];
@@ -462,7 +464,7 @@ SCREEN_UPDATE( macrbv )
 
 			for (y = 0; y < vres; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 
 				for (x = 0; x < hres/2; x++)
 				{
@@ -481,7 +483,7 @@ SCREEN_UPDATE( macrbv )
 
 			for (y = 0; y < vres; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 
 				for (x = 0; x < hres; x++)
 				{
@@ -495,11 +497,11 @@ SCREEN_UPDATE( macrbv )
 	return 0;
 }
 
-SCREEN_UPDATE( macrbvvram )
+SCREEN_UPDATE_RGB32( macrbvvram )
 {
 	UINT32 *scanline;
 	int x, y;
-	mac_state *mac = screen->machine().driver_data<mac_state>();
+	mac_state *mac = screen.machine().driver_data<mac_state>();
 	UINT8 mode = 0;
 
 	switch (mac->m_rbv_type)
@@ -524,14 +526,14 @@ SCREEN_UPDATE( macrbvvram )
 	{
 		case 0:	// 1bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 
 			if (mac->m_rbv_type == RBV_TYPE_SONORA)
 			{
 				for (y = 0; y < 480; y++)
 				{
-					scanline = BITMAP_ADDR32(bitmap, y, 0);
+					scanline = &bitmap.pix32(y);
 					for (x = 0; x < 640; x+=8)
 					{
 						pixels = vram8[(y * 80) + ((x/8)^3)];
@@ -551,7 +553,7 @@ SCREEN_UPDATE( macrbvvram )
 			{
 				for (y = 0; y < 480; y++)
 				{
-					scanline = BITMAP_ADDR32(bitmap, y, 0);
+					scanline = &bitmap.pix32(y);
 					for (x = 0; x < 640; x+=8)
 					{
 						pixels = vram8[(y * 0x400) + ((x/8)^3)];
@@ -572,12 +574,12 @@ SCREEN_UPDATE( macrbvvram )
 
 		case 1:	// 2bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 
 			for (y = 0; y < 480; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				for (x = 0; x < 640/4; x++)
 				{
 					pixels = vram8[(y * 160) + (BYTE4_XOR_BE(x))];
@@ -593,12 +595,12 @@ SCREEN_UPDATE( macrbvvram )
 
 		case 2: // 4bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 
 			for (y = 0; y < 480; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 
 				for (x = 0; x < 640/2; x++)
 				{
@@ -613,14 +615,14 @@ SCREEN_UPDATE( macrbvvram )
 
 		case 3: // 8bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 
 			if (mac->m_rbv_type == RBV_TYPE_SONORA)
 			{
 				for (y = 0; y < 480; y++)
 				{
-					scanline = BITMAP_ADDR32(bitmap, y, 0);
+					scanline = &bitmap.pix32(y);
 
 					for (x = 0; x < 640; x++)
 					{
@@ -633,7 +635,7 @@ SCREEN_UPDATE( macrbvvram )
 			{
 				for (y = 0; y < 480; y++)
 				{
-					scanline = BITMAP_ADDR32(bitmap, y, 0);
+					scanline = &bitmap.pix32(y);
 
 					for (x = 0; x < 640; x++)
 					{
@@ -867,23 +869,23 @@ WRITE32_MEMBER(mac_state::dafb_dac_w)
 	}
 }
 
-SCREEN_UPDATE( macdafb )
+SCREEN_UPDATE_RGB32( macdafb )
 {
 	UINT32 *scanline;
 	int x, y;
-	mac_state *mac = screen->machine().driver_data<mac_state>();
+	mac_state *mac = screen.machine().driver_data<mac_state>();
 
 	switch (mac->m_dafb_mode)
 	{
 		case 0:	// 1bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 			vram8 += mac->m_dafb_base;
 
 			for (y = 0; y < 870; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				for (x = 0; x < 1152; x+=8)
 				{
 					pixels = vram8[(y * mac->m_dafb_stride) + ((x/8)^3)];
@@ -903,13 +905,13 @@ SCREEN_UPDATE( macdafb )
 
 		case 1:	// 2bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 			vram8 += mac->m_dafb_base;
 
 			for (y = 0; y < 870; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				for (x = 0; x < 1152/4; x++)
 				{
 					pixels = vram8[(y * mac->m_dafb_stride) + (BYTE4_XOR_BE(x))];
@@ -925,13 +927,13 @@ SCREEN_UPDATE( macdafb )
 
 		case 2: // 4bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 			vram8 += mac->m_dafb_base;
 
 			for (y = 0; y < 870; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 
 				for (x = 0; x < 1152/2; x++)
 				{
@@ -946,13 +948,13 @@ SCREEN_UPDATE( macdafb )
 
 		case 3: // 8bpp
 		{
-			UINT8 *vram8 = (UINT8 *)mac->m_vram;
+			UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
 			UINT8 pixels;
 			vram8 += mac->m_dafb_base;
 
 			for (y = 0; y < 870; y++)
 			{
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 
 				for (x = 0; x < 1152; x++)
 				{
@@ -968,7 +970,7 @@ SCREEN_UPDATE( macdafb )
 			{
 				UINT32 *base;
 
-				scanline = BITMAP_ADDR32(bitmap, y, 0);
+				scanline = &bitmap.pix32(y);
 				base = (UINT32 *)&mac->m_vram[(y * (mac->m_dafb_stride/4)) + (mac->m_dafb_base/4)];
 				for (x = 0; x < 640; x++)
 				{
@@ -981,3 +983,81 @@ SCREEN_UPDATE( macdafb )
 	return 0;
 }
 
+SCREEN_UPDATE_RGB32( macpbwd )    /* Color PowerBooks using an off-the-shelf WD video chipset */
+{
+	UINT32 *scanline;
+	int x, y;
+	mac_state *mac = screen.machine().driver_data<mac_state>();
+    UINT8 *vram8 = (UINT8 *)mac->m_vram.target();
+    UINT8 pixels;
+
+//    vram8 += 0x40000;
+
+    for (y = 0; y < 480; y++)
+    {
+        scanline = &bitmap.pix32(y);
+        for (x = 0; x < 640; x++)
+        {
+            pixels = vram8[(y * 640) + (BYTE4_XOR_BE(x))];
+            *scanline++ = mac->m_rbv_palette[pixels];
+        }
+    }
+
+    return 0;
+}
+
+READ32_MEMBER(mac_state::macwd_r)
+{
+    switch (offset)
+    {
+        case 0xf6:
+            if (m_screen->vblank())
+            {
+                return 0xffffffff;
+            }
+            else
+            {
+                return 0;
+            }
+            break;
+
+        default:
+//            printf("macwd_r: @ %x, mask %08x (PC=%x)\n", offset, mem_mask, cpu_get_pc(m_maincpu));
+            break;
+    }
+    return 0;
+}
+
+WRITE32_MEMBER(mac_state::macwd_w)
+{
+    switch (offset)
+    {
+        case 0xf2:
+            if (mem_mask == 0xff000000) // DAC control
+            {
+                m_rbv_clutoffs = data>>24;
+                m_rbv_count = 0;
+            }
+            else if (mem_mask == 0x00ff0000)    // DAC data
+            {
+                m_rbv_colors[m_rbv_count++] = (data>>16)&0xff;
+                if (m_rbv_count == 3)
+                {
+//                    printf("RAMDAC: color %d = %02x %02x %02x\n", m_rbv_clutoffs, m_rbv_colors[0], m_rbv_colors[1], m_rbv_colors[2]);
+                    m_rbv_palette[m_rbv_clutoffs] = MAKE_RGB(m_rbv_colors[0], m_rbv_colors[1], m_rbv_colors[2]);
+                    m_rbv_clutoffs++;
+                    m_rbv_count = 0;
+                }
+            }
+            else
+            {
+                printf("macwd: Unknown DAC write, data %08x, mask %08x\n", data, mem_mask);
+            }
+            break;
+
+        default:
+//            printf("macwd_w: %x @ %x, mask %08x (PC=%x)\n", data, offset, mem_mask, cpu_get_pc(m_maincpu));
+            break;
+    }
+}
+
