@@ -15,10 +15,10 @@ VIDEO_START( mikro80 )
 {
 }
 
-SCREEN_UPDATE( mikro80 )
+SCREEN_UPDATE_IND16( mikro80 )
 {
-	mikro80_state *state = screen->machine().driver_data<mikro80_state>();
-	UINT8 *gfx = screen->machine().region("gfx1")->base();
+	mikro80_state *state = screen.machine().driver_data<mikro80_state>();
+	UINT8 *gfx = state->memregion("gfx1")->base();
 	int x,y,b;
 
 	for(y = 0; y < 32*8; y++ )
@@ -31,7 +31,7 @@ SCREEN_UPDATE( mikro80 )
 			for (b = 7; b >= 0; b--)
 			{
 				UINT8 col = (code >> b) & 0x01;
-				*BITMAP_ADDR16(bitmap, y, x*8+(7-b)) =  attr ? col ^ 1 : col;
+				bitmap.pix16(y, x*8+(7-b)) =  attr ? col ^ 1 : col;
 			}
 		}
 	}
