@@ -2,6 +2,12 @@
 
   video/odyssey2.c
 
+  2012-02-04 DanBoris
+    - Changed color of background grid color 0 to match sprite color 0 (Fixes KTAA title screen)
+    - Fixed Odyssey2_video_w so that m_o2_vdc.reg[] is always updated (Fixes Blockout)
+    - Changed quad character generation so character height is always taken from 4th character (KTAA level 2)
+
+
 ***************************************************************************/
 
 #include "emu.h"
@@ -47,7 +53,7 @@ const UINT8 odyssey2_colors[] =
 	0xFF,0xFF,0xFF,
 
 	/* Character,Sprite colors */
-	0x71,0x71,0x71,   /* Dark Grey */
+	0x80,0x80,0x80,   /* Dark Grey */
 	0xFF,0x80,0x80,   /* Red */
 	0x00,0xC0,0x00,   /* Green */
 	0xff,0x9b,0x60,   /* Orange */
@@ -136,19 +142,18 @@ PALETTE_INIT( odyssey2 )
 	}
 }
 
-READ8_HANDLER( odyssey2_video_r )
+READ8_MEMBER(odyssey2_state::odyssey2_video_r)
 {
-	odyssey2_state *state = space->machine().driver_data<odyssey2_state>();
     UINT8 data = 0;
 
     switch (offset)
     {
         case 0xa1:
-			data = state->m_control_status;
-			state->m_iff = 0;
-			cputag_set_input_line(space->machine(), "maincpu", 0, CLEAR_LINE);
-			state->m_control_status &= ~ 0x08;
-			if ( space->machine().primary_screen->hpos() < I824X_START_ACTIVE_SCAN || space->machine().primary_screen->hpos() > I824X_END_ACTIVE_SCAN )
+			data = m_control_status;
+			m_iff = 0;
+			cputag_set_input_line(machine(), "maincpu", 0, CLEAR_LINE);
+			m_control_status &= ~ 0x08;
+			if ( machine().primary_screen->hpos() < I824X_START_ACTIVE_SCAN || machine().primary_screen->hpos() > I824X_END_ACTIVE_SCAN )
 			{
 				data |= 1;
 			}
@@ -156,97 +161,87 @@ READ8_HANDLER( odyssey2_video_r )
             break;
 
         case 0xa2:
-			data = state->m_collision_status;
-			state->m_collision_status = 0;
+			data = m_collision_status;
+			m_collision_status = 0;
 
             break;
 
         case 0xa4:
 
-            if ((state->m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY))
-                state->m_y_beam_pos = space->machine().primary_screen->vpos() - state->m_start_vpos;
+            if ((m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY))
+                m_y_beam_pos = machine().primary_screen->vpos() - m_start_vpos;
 
-            data = state->m_y_beam_pos;
+            data = m_y_beam_pos;
 
             break;
 
 
         case 0xa5:
 
-            if ((state->m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY))
+            if ((m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY))
 			{
-                state->m_x_beam_pos = space->machine().primary_screen->hpos();
-				if ( state->m_x_beam_pos < I824X_START_ACTIVE_SCAN )
+                m_x_beam_pos = machine().primary_screen->hpos();
+				if ( m_x_beam_pos < I824X_START_ACTIVE_SCAN )
 				{
-					state->m_x_beam_pos = state->m_x_beam_pos - I824X_START_ACTIVE_SCAN + I824X_LINE_CLOCKS;
+					m_x_beam_pos = m_x_beam_pos - I824X_START_ACTIVE_SCAN + I824X_LINE_CLOCKS;
 				}
 				else
 				{
-					state->m_x_beam_pos = state->m_x_beam_pos - I824X_START_ACTIVE_SCAN;
+					m_x_beam_pos = m_x_beam_pos - I824X_START_ACTIVE_SCAN;
 				}
 			}
 
-            data = state->m_x_beam_pos;
+            data = m_x_beam_pos;
 
             break;
 
         default:
-            data = state->m_o2_vdc.reg[offset];
+            data = m_o2_vdc.reg[offset];
     }
 
     return data;
 }
 
-WRITE8_HANDLER( odyssey2_video_w )
+WRITE8_MEMBER(odyssey2_state::odyssey2_video_w)
 {
-	odyssey2_state *state = space->machine().driver_data<odyssey2_state>();
 	/* Update the sound */
 	if( offset >= 0xa7 && offset <= 0xaa )
-		state->m_sh_channel->update();
+		m_sh_channel->update();
 
-    if (offset == 0xa0)
-    {
-        if (    state->m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY
+    if (offset == 0xa0) {
+
+
+        if (    m_o2_vdc.s.control & VDC_CONTROL_REG_STROBE_XY
              && !(data & VDC_CONTROL_REG_STROBE_XY))
         {
             /* Toggling strobe bit, tuck away values */
-            state->m_x_beam_pos = space->machine().primary_screen->hpos();
-			if ( state->m_x_beam_pos < I824X_START_ACTIVE_SCAN )
+            m_x_beam_pos = machine().primary_screen->hpos();
+			if ( m_x_beam_pos < I824X_START_ACTIVE_SCAN )
 			{
-				state->m_x_beam_pos = state->m_x_beam_pos - I824X_START_ACTIVE_SCAN + 228;
+				m_x_beam_pos = m_x_beam_pos - I824X_START_ACTIVE_SCAN + 228;
 			}
 			else
 			{
-				state->m_x_beam_pos = state->m_x_beam_pos - I824X_START_ACTIVE_SCAN;
+				m_x_beam_pos = m_x_beam_pos - I824X_START_ACTIVE_SCAN;
 			}
 
-            state->m_y_beam_pos = space->machine().primary_screen->vpos() - state->m_start_vpos;
-
-            /* This is wrong but more games work with it, TODO: Figure
-             * out correct change.  Maybe update the screen here??
-             * It seems what happens is 0x0 is written to $A0 just before
-             * VLBANK (video update) so no screen updates happen.
-             */
-
-            return;
+            m_y_beam_pos = machine().primary_screen->vpos() - m_start_vpos;
         }
     }
 
-    state->m_o2_vdc.reg[offset] = data;
+    m_o2_vdc.reg[offset] = data;
 }
 
-WRITE8_HANDLER ( odyssey2_lum_w )
+WRITE8_MEMBER(odyssey2_state::odyssey2_lum_w)
 {
-	odyssey2_state *state = space->machine().driver_data<odyssey2_state>();
-	state->m_lum = data;
+	m_lum = data;
 }
 
-READ8_HANDLER( odyssey2_t1_r )
+READ8_MEMBER(odyssey2_state::odyssey2_t1_r)
 {
-	odyssey2_state *state = space->machine().driver_data<odyssey2_state>();
-	if ( space->machine().primary_screen->vpos() > state->m_start_vpos && space->machine().primary_screen->vpos() < state->m_start_vblank )
+	if ( machine().primary_screen->vpos() > m_start_vpos && machine().primary_screen->vpos() < m_start_vblank )
 	{
-		if ( space->machine().primary_screen->hpos() >= I824X_START_ACTIVE_SCAN && space->machine().primary_screen->hpos() < I824X_END_ACTIVE_SCAN )
+		if ( machine().primary_screen->hpos() >= I824X_START_ACTIVE_SCAN && machine().primary_screen->hpos() < I824X_END_ACTIVE_SCAN )
 		{
 			return 1;
 		}
@@ -277,10 +272,8 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 		state->m_control_status &= ~ 0x01;
 
 		/* Draw a line */
-		rect.min_y = rect.max_y = vpos;
-		rect.min_x = I824X_START_ACTIVE_SCAN;
-		rect.max_x = I824X_END_ACTIVE_SCAN - 1;
-		bitmap_fill( state->m_tmp_bitmap, &rect , ( (state->m_o2_vdc.s.color >> 3) & 0x7 ) | ( ( state->m_lum << 3 ) ^ 0x08 ));
+		rect.set(I824X_START_ACTIVE_SCAN, I824X_END_ACTIVE_SCAN - 1, vpos, vpos);
+		state->m_tmp_bitmap.fill(( (state->m_o2_vdc.s.color >> 3) & 0x7 ) | ( ( state->m_lum << 3 ) ^ 0x08 ), rect );
 
 		/* Clear collision map */
 		memset( collision_map, 0, sizeof( collision_map ) );
@@ -309,7 +302,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 							{
 								int px = x_grid_offset + i * width + k;
 								collision_map[ px ] |= COLLISION_HORIZ_GRID_DOTS;
-								*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + px ) = color;
+								state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + px ) = color;
 							}
 						}
 					}
@@ -340,7 +333,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 									state->m_collision_status |= collision_map[ px ];
 								}
 								collision_map[ px ] |= COLLISION_VERTICAL_GRID;
-								*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + px ) = color;
+								state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + px ) = color;
 							}
 						}
 					}
@@ -382,7 +375,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 									state->m_collision_status |= collision_map[ x ];
 								}
 								collision_map[ x ] |= COLLISION_CHARACTERS;
-								*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + x ) = color;
+								state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + x ) = color;
 							}
 						}
 					}
@@ -400,14 +393,23 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 					int	x = state->m_o2_vdc.s.quad[i].single[0].x;
 					int j;
 
+					// Charaecter height is always determined by the height of the 4th character
+					int char_height = 8 - ( ( ( y >> 1 ) + state->m_o2_vdc.s.quad[i].single[3].ptr ) & 7 );
+
 					for ( j = 0; j < ARRAY_LENGTH( state->m_o2_vdc.s.quad[0].single ); j++, x += 8 )
 					{
-						int		char_height = 8 - ( ( ( y >> 1 ) + state->m_o2_vdc.s.quad[i].single[j].ptr ) & 7 );
+
+
 						if ( y <= ( vpos - state->m_start_vpos ) && ( vpos - state->m_start_vpos ) < y + char_height * 2 )
 						{
-							UINT16 color = 16 + ( ( state->m_o2_vdc.s.quad[i].single[j].color & 0x0E ) >> 1 );
+
+						UINT16 color = 16 + ( ( state->m_o2_vdc.s.quad[i].single[j].color & 0x0E ) >> 1 );
+
+
 							int	offset = ( state->m_o2_vdc.s.quad[i].single[j].ptr | ( ( state->m_o2_vdc.s.quad[i].single[j].color & 0x01 ) << 8 ) ) + ( y >> 1 ) + ( ( vpos - state->m_start_vpos - y ) >> 1 );
+
 							UINT8	chr = ((char*)o2_shape)[ offset & 0x1FF ];
+
 							UINT8	m;
 							for ( m = 0x80; m > 0; m >>= 1, x++ )
 							{
@@ -426,7 +428,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 											state->m_collision_status |= collision_map[ x ];
 										}
 										collision_map[ x ] |= COLLISION_CHARACTERS;
-										*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + x ) = color;
+										state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + x ) = color;
 									}
 								}
 							}
@@ -472,7 +474,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 										state->m_collision_status |= collision_map[ x ];
 									}
 									collision_map[ x ] |= ( 1 << i );
-									*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + x ) = color;
+									state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + x ) = color;
 								}
 								if ( x >= -1 && x < 159 )
 								{
@@ -487,7 +489,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 										state->m_collision_status |= collision_map[ x ];
 									}
 									collision_map[ x ] |= ( 1 << i );
-									*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + x + 1 ) = color;
+									state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + x + 1 ) = color;
 								}
 							}
 						}
@@ -520,7 +522,7 @@ static TIMER_CALLBACK( i824x_scanline_callback )
 										state->m_collision_status |= collision_map[ x ];
 									}
 									collision_map[ x ] |= ( 1 << i );
-									*BITMAP_ADDR16( state->m_tmp_bitmap, vpos, I824X_START_ACTIVE_SCAN + x ) = color;
+									state->m_tmp_bitmap.pix16(vpos, I824X_START_ACTIVE_SCAN + x ) = color;
 								}
 							}
 						}
@@ -566,8 +568,6 @@ VIDEO_START( odyssey2 )
 {
 	odyssey2_state *state = machine.driver_data<odyssey2_state>();
 	screen_device *screen = machine.first_screen();
-	int width = screen->width();
-	int height = screen->height();
 
 	memset(state->m_o2_vdc.reg, 0, 0x100);
 
@@ -580,7 +580,6 @@ VIDEO_START( odyssey2 )
 	state->m_start_vpos = 0;
 	state->m_start_vblank = 0;
 	state->m_lum = 0;
-	state->m_lfsr = 0;
 
 	state->m_o2_snd_shift[0] = machine.sample_rate() / 983;
 	state->m_o2_snd_shift[1] = machine.sample_rate() / 3933;
@@ -590,7 +589,7 @@ VIDEO_START( odyssey2 )
 	state->m_control_status = 0;
 	state->m_iff = 0;
 
-	state->m_tmp_bitmap = auto_bitmap_alloc( machine, width, height, screen->format() );
+	screen->register_screen_bitmap(state->m_tmp_bitmap);
 
 	state->m_i824x_line_timer = machine.scheduler().timer_alloc(FUNC(i824x_scanline_callback));
 	state->m_i824x_line_timer->adjust( machine.primary_screen->time_until_pos(1, I824X_START_ACTIVE_SCAN ), 0,  machine.primary_screen->scan_period() );
@@ -605,9 +604,9 @@ VIDEO_START( odyssey2 )
 
 ***************************************************************************/
 
-SCREEN_UPDATE( odyssey2 )
+SCREEN_UPDATE_IND16( odyssey2 )
 {
-	odyssey2_state *state = screen->machine().driver_data<odyssey2_state>();
+	odyssey2_state *state = screen.machine().driver_data<odyssey2_state>();
 	copybitmap( bitmap, state->m_tmp_bitmap, 0, 0, 0, 0, cliprect );
 
 	return 0;
@@ -636,13 +635,13 @@ DEVICE_GET_INFO( odyssey2_sound )
 STREAM_UPDATE( odyssey2_sh_update )
 {
 	odyssey2_state *state = device->machine().driver_data<odyssey2_state>();
-	UINT32 signal;
+	UINT32 old_signal, signal;
 	int ii;
 	int period;
 	stream_sample_t *buffer = outputs[0];
 
 	/* Generate the signal */
-	signal = state->m_o2_vdc.s.shift3 | (state->m_o2_vdc.s.shift2 << 8) | (state->m_o2_vdc.s.shift1 << 16);
+	old_signal = signal = state->m_o2_vdc.s.shift3 | (state->m_o2_vdc.s.shift2 << 8) | (state->m_o2_vdc.s.shift1 << 16);
 
 	if( state->m_o2_vdc.s.sound & 0x80 )	/* Sound is enabled */
 	{
@@ -660,16 +659,12 @@ STREAM_UPDATE( odyssey2_sh_update )
 				{
 					signal |= *buffer << 23;
 				}
-				/* Noise poly is : L=16 W=2 10000000000100001 */
-				state->m_lfsr = ( state->m_lfsr >> 1 ) | ( ( ( state->m_lfsr & 0x01 ) ^ ( ( state->m_lfsr & 0x800 ) >> 11 ) ) << 15 );
-				if ( ! state->m_lfsr )
-				{
-					state->m_lfsr = 0xFFFF;
-				}
 				/* Check if noise should be applied */
 				if ( state->m_o2_vdc.s.sound & 0x10 )
 				{
-					*buffer |= ( state->m_lfsr & 0x01 );
+					/* Noise tap is on bits 0 and 5 and fed back to bits 15 (and 23!) */
+					UINT32 new_bit = ( ( old_signal ) ^ ( old_signal >> 5 ) ) & 0x01;
+					signal = ( old_signal & 0xFF0000 ) | ( ( old_signal & 0xFFFF ) >> 1 ) | ( new_bit << 15 ) | ( new_bit << 23 );
 				}
 				state->m_o2_vdc.s.shift3 = signal & 0xFF;
 				state->m_o2_vdc.s.shift2 = ( signal >> 8 ) & 0xFF;
