@@ -19,9 +19,9 @@ VIDEO_START( ondra )
 	state->m_video_enable = 0;
 }
 
-SCREEN_UPDATE( ondra )
+SCREEN_UPDATE_IND16( ondra )
 {
-	ondra_state *state = screen->machine().driver_data<ondra_state>();
+	ondra_state *state = screen.machine().driver_data<ondra_state>();
 	UINT8 code1,code2;
 	int y, x, b;
 	int Vaddr = 0x2800;
@@ -31,12 +31,12 @@ SCREEN_UPDATE( ondra )
 		{
 			for (y = 127; y >=0; y--)
 			{
-				code1 = ram_get_ptr(screen->machine().device(RAM_TAG))[0xd700 + Vaddr + 0x80];
-				code2 = ram_get_ptr(screen->machine().device(RAM_TAG))[0xd700 + Vaddr + 0x00];
+				code1 = screen.machine().device<ram_device>(RAM_TAG)->pointer()[0xd700 + Vaddr + 0x80];
+				code2 = screen.machine().device<ram_device>(RAM_TAG)->pointer()[0xd700 + Vaddr + 0x00];
 				for (b = 0; b < 8; b++)
 				{
-					*BITMAP_ADDR16(bitmap, 2*y, x*8+b) =  ((code1 << b) & 0x80) ? 1 : 0;
-					*BITMAP_ADDR16(bitmap, 2*y+1,   x*8+b) =  ((code2 << b) & 0x80) ? 1 : 0;
+					bitmap.pix16(2*y, x*8+b) =  ((code1 << b) & 0x80) ? 1 : 0;
+					bitmap.pix16(2*y+1, x*8+b) =  ((code2 << b) & 0x80) ? 1 : 0;
 				}
 				Vaddr++;
 			}
