@@ -15,9 +15,9 @@ VIDEO_START( orao )
 {
 }
 
-SCREEN_UPDATE( orao )
+SCREEN_UPDATE_IND16( orao )
 {
-	orao_state *state = screen->machine().driver_data<orao_state>();
+	orao_state *state = screen.machine().driver_data<orao_state>();
 	UINT8 code;
 	int y, x, b;
 
@@ -30,7 +30,7 @@ SCREEN_UPDATE( orao )
 			code = state->m_video_ram[addr++];
 			for (b = 0; b < 8; b++)
 			{
-				*BITMAP_ADDR16(bitmap, y, horpos++) =  (code >> b) & 0x01;
+				bitmap.pix16(y, horpos++) =  (code >> b) & 0x01;
 			}
 		}
 	}
