@@ -18,9 +18,9 @@ VIDEO_START( p2000m )
 }
 
 
-SCREEN_UPDATE( p2000m )
+SCREEN_UPDATE_IND16( p2000m )
 {
-	p2000t_state *state = screen->machine().driver_data<p2000t_state>();
+	p2000t_state *state = screen.machine().driver_data<p2000t_state>();
 	UINT8 *videoram = state->m_p_videoram;
 	int offs, sx, sy, code, loop;
 
@@ -43,15 +43,15 @@ SCREEN_UPDATE( p2000m )
 			if (code < 32) code = 32;
 		}
 
-		drawgfxzoom_opaque (bitmap, NULL, screen->machine().gfx[0], code,
+		drawgfxzoom_opaque (bitmap, cliprect, screen.machine().gfx[0], code,
 			videoram[offs + 2048] & 0x08 ? 0 : 1, 0, 0, sx, sy, 0x20000, 0x20000);
 
 		if (videoram[offs] & 0x80)
 		{
 			for (loop = 0; loop < 12; loop++)
 			{
-				*BITMAP_ADDR16(bitmap, sy + 18, sx + loop) = 0;	/* cursor */
-				*BITMAP_ADDR16(bitmap, sy + 19, sx + loop) = 0;	/* cursor */
+				bitmap.pix16(sy + 18, sx + loop) = 0;	/* cursor */
+				bitmap.pix16(sy + 19, sx + loop) = 0;	/* cursor */
 			}
 		}
 	}
