@@ -25,27 +25,17 @@
 
 
 
-static const rectangle typewriter_bitmap_bounds =
+
+
+static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_ind16 &bitmap);
+static void pdp1_draw_panel(running_machine &machine, bitmap_ind16 &bitmap);
+
+static void pdp1_erase_lightpen(pdp1_state *state, bitmap_ind16 &bitmap);
+static void pdp1_draw_lightpen(pdp1_state *state, bitmap_ind16 &bitmap);
+
+INLINE void pdp1_plot_pixel(bitmap_ind16 &bitmap, int x, int y, UINT32 color)
 {
-	0,	typewriter_window_width-1,	/* min_x, max_x */
-	0,	typewriter_window_height-1,	/* min_y, max_y */
-};
-
-static const rectangle panel_bitmap_bounds =
-{
-	0,	panel_window_width-1,	/* min_x, max_x */
-	0,	panel_window_height-1,	/* min_y, max_y */
-};
-
-static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_t *bitmap);
-static void pdp1_draw_panel(running_machine &machine, bitmap_t *bitmap);
-
-static void pdp1_erase_lightpen(pdp1_state *state, bitmap_t *bitmap);
-static void pdp1_draw_lightpen(pdp1_state *state, bitmap_t *bitmap);
-
-INLINE void pdp1_plot_pixel(bitmap_t *bitmap, int x, int y, UINT32 color)
-{
-	*BITMAP_ADDR16(bitmap, y, x) = color;
+	bitmap.pix16(y, x) = color;
 }
 
 /*
@@ -57,23 +47,28 @@ VIDEO_START( pdp1 )
 	state->m_typewriter_color = color_typewriter_black;
 
 	/* alloc bitmaps for our private fun */
-	state->m_panel_bitmap = auto_bitmap_alloc(machine, panel_window_width, panel_window_height, BITMAP_FORMAT_INDEXED16);
-	state->m_typewriter_bitmap = auto_bitmap_alloc(machine, typewriter_window_width, typewriter_window_height, BITMAP_FORMAT_INDEXED16);
+	state->m_panel_bitmap.allocate(panel_window_width, panel_window_height, BITMAP_FORMAT_IND16);
+	state->m_typewriter_bitmap.allocate(typewriter_window_width, typewriter_window_height, BITMAP_FORMAT_IND16);
 
 	/* set up out bitmaps */
 	pdp1_draw_panel_backdrop(machine, state->m_panel_bitmap);
 
-	bitmap_fill(state->m_typewriter_bitmap, &typewriter_bitmap_bounds, pen_typewriter_bg);
+	const rectangle typewriter_bitmap_bounds(0, typewriter_window_width-1, 0, typewriter_window_height-1);
+	state->m_typewriter_bitmap.fill(pen_typewriter_bg, typewriter_bitmap_bounds);
 
 	state->m_crt = machine.device("crt");
 }
 
 
-SCREEN_EOF( pdp1 )
+SCREEN_VBLANK( pdp1 )
 {
-	pdp1_state *state = machine.driver_data<pdp1_state>();
+	// rising edge
+	if (vblank_on)
+	{
+		pdp1_state *state = screen.machine().driver_data<pdp1_state>();
 
-	crt_eof(state->m_crt);
+		crt_eof(state->m_crt);
+	}
 }
 
 /*
@@ -92,14 +87,14 @@ void pdp1_plot(running_machine &machine, int x, int y)
 /*
     video_update_pdp1: effectively redraw the screen
 */
-SCREEN_UPDATE( pdp1 )
+SCREEN_UPDATE_IND16( pdp1 )
 {
-	pdp1_state *state = screen->machine().driver_data<pdp1_state>();
+	pdp1_state *state = screen.machine().driver_data<pdp1_state>();
 	pdp1_erase_lightpen(state, bitmap);
 	crt_update(state->m_crt, bitmap);
 	pdp1_draw_lightpen(state, bitmap);
 
-	pdp1_draw_panel(screen->machine(), state->m_panel_bitmap);
+	pdp1_draw_panel(screen.machine(), state->m_panel_bitmap);
 	copybitmap(bitmap, state->m_panel_bitmap, 0, 0, panel_window_offset_x, panel_window_offset_y, cliprect);
 
 	copybitmap(bitmap, state->m_typewriter_bitmap, 0, 0, typewriter_window_offset_x, typewriter_window_offset_y, cliprect);
@@ -157,7 +152,7 @@ enum
 };
 
 /* draw a small 8*8 LED (or is this a lamp? ) */
-static void pdp1_draw_led(running_machine &machine, bitmap_t *bitmap, int x, int y, int state)
+static void pdp1_draw_led(running_machine &machine, bitmap_ind16 &bitmap, int x, int y, int state)
 {
 	int xx, yy;
 
@@ -167,7 +162,7 @@ static void pdp1_draw_led(running_machine &machine, bitmap_t *bitmap, int x, int
 }
 
 /* draw nb_bits leds which represent nb_bits bits in value */
-static void pdp1_draw_multipleled(running_machine &machine, bitmap_t *bitmap, int x, int y, int value, int nb_bits)
+static void pdp1_draw_multipleled(running_machine &machine, bitmap_ind16 &bitmap, int x, int y, int value, int nb_bits)
 {
 	while (nb_bits)
 	{
@@ -181,7 +176,7 @@ static void pdp1_draw_multipleled(running_machine &machine, bitmap_t *bitmap, in
 
 
 /* draw a small 8*8 switch */
-static void pdp1_draw_switch(running_machine &machine, bitmap_t *bitmap, int x, int y, int state)
+static void pdp1_draw_switch(running_machine &machine, bitmap_ind16 &bitmap, int x, int y, int state)
 {
 	int xx, yy;
 	int i;
@@ -222,7 +217,7 @@ static void pdp1_draw_switch(running_machine &machine, bitmap_t *bitmap, int x,
 
 
 /* draw nb_bits switches which represent nb_bits bits in value */
-static void pdp1_draw_multipleswitch(running_machine &machine, bitmap_t *bitmap, int x, int y, int value, int nb_bits)
+static void pdp1_draw_multipleswitch(running_machine &machine, bitmap_ind16 &bitmap, int x, int y, int value, int nb_bits)
 {
 	while (nb_bits)
 	{
@@ -236,14 +231,14 @@ static void pdp1_draw_multipleswitch(running_machine &machine, bitmap_t *bitmap,
 
 
 /* write a single char on screen */
-static void pdp1_draw_char(running_machine &machine, bitmap_t *bitmap, char character, int x, int y, int color)
+static void pdp1_draw_char(running_machine &machine, bitmap_ind16 &bitmap, char character, int x, int y, int color)
 {
-	drawgfx_transpen(bitmap, NULL, machine.gfx[0], character-32, color, 0, 0,
+	drawgfx_transpen(bitmap, bitmap.cliprect(), machine.gfx[0], character-32, color, 0, 0,
 				x+1, y, 0);
 }
 
 /* write a string on screen */
-static void pdp1_draw_string(running_machine &machine, bitmap_t *bitmap, const char *buf, int x, int y, int color)
+static void pdp1_draw_string(running_machine &machine, bitmap_ind16 &bitmap, const char *buf, int x, int y, int color)
 {
 	while (* buf)
 	{
@@ -258,11 +253,12 @@ static void pdp1_draw_string(running_machine &machine, bitmap_t *bitmap, const c
 /*
     draw the operator control panel (fixed backdrop)
 */
-static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_t *bitmap)
+static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_ind16 &bitmap)
 {
 	pdp1_state *state = machine.driver_data<pdp1_state>();
 	/* fill with black */
-	bitmap_fill(state->m_panel_bitmap, &panel_bitmap_bounds, pen_panel_bg);
+	const rectangle panel_bitmap_bounds(0, panel_window_width-1,	0, panel_window_height-1);
+	state->m_panel_bitmap.fill(pen_panel_bg, panel_bitmap_bounds);
 
 	/* column 1: registers, test word, test address */
 	pdp1_draw_string(machine, bitmap, "program counter", x_panel_col1_offset, y_panel_pc_offset, color_panel_caption);
@@ -274,7 +270,7 @@ static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_t *bitmap)
 	pdp1_draw_string(machine, bitmap, "test word", x_panel_col1_offset, y_panel_tw_offset, color_panel_caption);
 
 	/* column separator */
-	plot_box(bitmap, x_panel_col2_offset-4, panel_window_offset_y+8, 1, 96, pen_panel_caption);
+	bitmap.plot_box(x_panel_col2_offset-4, panel_window_offset_y+8, 1, 96, pen_panel_caption);
 
 	/* column 2: 1-bit indicators */
 	pdp1_draw_string(machine, bitmap, "run", x_panel_col2_offset+8, y_panel_run_offset, color_panel_caption);
@@ -292,17 +288,17 @@ static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_t *bitmap)
 	pdp1_draw_string(machine, bitmap, "i-o sync", x_panel_col2_offset+8, y_panel_ios_offset, color_panel_caption);
 
 	/* column separator */
-	plot_box(bitmap, x_panel_col3_offset-4, panel_window_offset_y+8, 1, 96, pen_panel_caption);
+	bitmap.plot_box(x_panel_col3_offset-4, panel_window_offset_y+8, 1, 96, pen_panel_caption);
 
 	/* column 3: power, single step, single inst, sense, flags, instr... */
 	pdp1_draw_string(machine, bitmap, "power", x_panel_col3_offset+16, y_panel_power_offset, color_panel_caption);
 	pdp1_draw_string(machine, bitmap, "single step", x_panel_col3_offset+16, y_panel_sngl_step_offset, color_panel_caption);
 	pdp1_draw_string(machine, bitmap, "single inst.", x_panel_col3_offset+16, y_panel_sngl_inst_offset, color_panel_caption);
 	/* separator */
-	plot_box(bitmap, x_panel_col3_offset+8, y_panel_sep1_offset+4, 96, 1, pen_panel_caption);
+	bitmap.plot_box(x_panel_col3_offset+8, y_panel_sep1_offset+4, 96, 1, pen_panel_caption);
 	pdp1_draw_string(machine, bitmap, "sense switches", x_panel_col3_offset, y_panel_ss_offset, color_panel_caption);
 	/* separator */
-	plot_box(bitmap, x_panel_col3_offset+8, y_panel_sep2_offset+4, 96, 1, pen_panel_caption);
+	bitmap.plot_box(x_panel_col3_offset+8, y_panel_sep2_offset+4, 96, 1, pen_panel_caption);
 	pdp1_draw_string(machine, bitmap, "program flags", x_panel_col3_offset, y_panel_pf_offset, color_panel_caption);
 	pdp1_draw_string(machine, bitmap, "instruction", x_panel_col3_offset, y_panel_ir_offset, color_panel_caption);
 }
@@ -310,7 +306,7 @@ static void pdp1_draw_panel_backdrop(running_machine &machine, bitmap_t *bitmap)
 /*
     draw the operator control panel (dynamic elements)
 */
-static void pdp1_draw_panel(running_machine &machine, bitmap_t *bitmap)
+static void pdp1_draw_panel(running_machine &machine, bitmap_ind16 &bitmap)
 {
 	/* column 1: registers, test word, test address */
 	pdp1_draw_multipleled(machine, bitmap, x_panel_col1_offset+16, y_panel_pc_offset+8, cpu_get_reg(machine.device("maincpu"), PDP1_PC), 16);
@@ -364,11 +360,6 @@ enum
 	typewriter_write_offset_y = typewriter_window_height-typewriter_line_height,
 	typewriter_scroll_step = typewriter_line_height
 };
-static const rectangle typewriter_scroll_clear_window =
-{
-	0, typewriter_window_width-1,	/* min_x, max_x */
-	typewriter_window_height-typewriter_scroll_step, typewriter_window_height-1,	/* min_y, max_y */
-};
 
 enum
 {
@@ -388,7 +379,8 @@ static void pdp1_typewriter_linefeed(running_machine &machine)
 		draw_scanline8(state->m_typewriter_bitmap, 0, y, typewriter_window_width, buf, machine.pens);
 	}
 
-	bitmap_fill(state->m_typewriter_bitmap, &typewriter_scroll_clear_window, pen_typewriter_bg);
+	const rectangle typewriter_scroll_clear_window(0, typewriter_window_width-1,	typewriter_window_height-typewriter_scroll_step, typewriter_window_height-1);
+	state->m_typewriter_bitmap.fill(pen_typewriter_bg, typewriter_scroll_clear_window);
 }
 
 void pdp1_typewriter_drawchar(running_machine &machine, int character)
@@ -511,7 +503,7 @@ void pdp1_update_lightpen_state(running_machine &machine, const lightpen_t *new_
 }
 
 #if 1
-static void pdp1_draw_circle(bitmap_t *bitmap, int x, int y, int radius, int color_)
+static void pdp1_draw_circle(bitmap_ind16 &bitmap, int x, int y, int radius, int color_)
 {
 	int interval;
 	int a;
@@ -546,7 +538,7 @@ static void pdp1_draw_circle(bitmap_t *bitmap, int x, int y, int radius, int col
 	}
 }
 #else
-static void pdp1_draw_circle(bitmap_t *bitmap, int x, int y, int radius, int color)
+static void pdp1_draw_circle(bitmap_ind16 &bitmap, int x, int y, int radius, int color)
 {
 	float fx, fy;
 	float interval;
@@ -578,7 +570,7 @@ static void pdp1_draw_circle(bitmap_t *bitmap, int x, int y, int radius, int col
 }
 #endif
 
-static void pdp1_erase_lightpen(pdp1_state *state, bitmap_t *bitmap)
+static void pdp1_erase_lightpen(pdp1_state *state, bitmap_ind16 &bitmap)
 {
 	if (state->m_previous_lightpen_state.active)
 	{
@@ -596,7 +588,7 @@ static void pdp1_erase_lightpen(pdp1_state *state, bitmap_t *bitmap)
 	}
 }
 
-static void pdp1_draw_lightpen(pdp1_state *state, bitmap_t *bitmap)
+static void pdp1_draw_lightpen(pdp1_state *state, bitmap_ind16 &bitmap)
 {
 	if (state->m_lightpen_state.active)
 	{
