@@ -54,7 +54,7 @@ static int thom_update_screen_size( running_machine &machine )
 {
 	screen_device *screen = machine.first_screen();
 	const rectangle &visarea = screen->visible_area();
-	UINT8 p = input_port_read(machine, "vconfig");
+	UINT8 p = machine.root_device().ioport("vconfig")->read();
 	int new_w, new_h, changed = 0;
 
 	switch ( p & 3 )
@@ -135,8 +135,8 @@ struct thom_vsignal thom_get_vsignal ( running_machine &machine )
 
 static void thom_get_lightpen_pos( running_machine &machine, int*x, int* y )
 {
-	*x = input_port_read(machine, "lightpen_x");
-	*y = input_port_read(machine, "lightpen_y");
+	*x = machine.root_device().ioport("lightpen_x")->read();
+	*y = machine.root_device().ioport("lightpen_y")->read();
 
 	if ( *x < 0 )
 		*x = 0;
@@ -260,7 +260,7 @@ static  INT16 thom_border_r[THOM_TOTAL_HEIGHT+1];
 
 
 /* active area, updated one scan-line at a time every 64us,
-   then blitted in SCREEN_UPDATE
+   then blitted in SCREEN_UPDATE_IND16
 */
 static UINT16 thom_vbody[640*200];
 
@@ -870,7 +870,7 @@ void thom_set_mode_point ( running_machine &machine, int point )
 {
 	assert( point >= 0 && point <= 1 );
 	thom_mode_point = ( ! point ) * 0x2000;
-	memory_set_bank( machine, THOM_VRAM_BANK, ! point );
+	machine.root_device().membank( THOM_VRAM_BANK )->set_entry( ! point );
 }
 
 
@@ -897,7 +897,7 @@ void thom_floppy_active ( running_machine &machine, int write )
 
 
 
-SCREEN_UPDATE ( thom )
+SCREEN_UPDATE_IND16 ( thom )
 {
 	int y, ypos;
 	const int scale = thom_hires ? 2 : 1;
@@ -909,9 +909,9 @@ SCREEN_UPDATE ( thom )
 	const int ybot = THOM_BORDER_HEIGHT + thom_bheight + 200;
 	UINT16* v = thom_vbody;
 	pen_t border = 0;
-	rectangle wrect = { 0, xright - 1, 0, 0 };
-	rectangle lrect = { 0, xbleft - 1, 0, 0 };
-	rectangle rrect = { xbright, xright - 1, 0, 0 };
+	rectangle wrect(0, xright - 1, 0, 0);
+	rectangle lrect(0, xbleft - 1, 0, 0);
+	rectangle rrect(xbright, xright - 1, 0, 0);
 
 	//LOG (( "%f thom: video update called\n", machine.time().as_double()));
 
@@ -934,7 +934,7 @@ SCREEN_UPDATE ( thom )
 		}
 		while ( y < THOM_BORDER_HEIGHT && thom_border_l[ y ] == -1 );
 		wrect.max_y = ypos - 1;
-		bitmap_fill( bitmap,  &wrect , border);
+		bitmap.fill(border, wrect );
 	}
 
 	/* left border */
@@ -950,7 +950,7 @@ SCREEN_UPDATE ( thom )
 		}
 		while ( y < yup && thom_border_l[ y ] == -1 );
 		lrect.max_y = ypos - 1;
-		bitmap_fill( bitmap, &lrect , border);
+		bitmap.fill(border, lrect );
 	}
 
 	/* lower border */
@@ -965,7 +965,7 @@ SCREEN_UPDATE ( thom )
 			ypos ++ /* += scale */;
 		} while ( y < ybot && thom_border_l[ y ] == -1 );
 		wrect.max_y = ypos - 1;
-		bitmap_fill( bitmap, &wrect , border);
+		bitmap.fill(border, wrect );
 	}
 
 	/* right border */
@@ -986,7 +986,7 @@ SCREEN_UPDATE ( thom )
 		}
 		while ( y < yup && thom_border_r[ y ] == -1 );
 		rrect.max_y = ypos - 1;
-		bitmap_fill( bitmap, &rrect , border);
+		bitmap.fill(border, rrect );
 	}
 
 	/* active area */
@@ -1034,59 +1034,63 @@ static TIMER_CALLBACK( thom_set_init )
 }
 
 /* call this at the very beginning of each new frame */
-SCREEN_EOF ( thom )
+SCREEN_VBLANK ( thom )
 {
-	int fnew, fold = FLOP_STATE;
-	int i;
-	UINT16 b = 0;
-	struct thom_vsignal l = thom_get_lightpen_vsignal( machine, 0, -1, 0 );
-
-	LOG (( "%f thom: video eof called\n", machine.time().as_double() ));
-
-	/* floppy indicator count */
-	if ( thom_floppy_wcount )
-		thom_floppy_wcount--;
-	if ( thom_floppy_rcount )
-		thom_floppy_rcount--;
-	fnew = FLOP_STATE;
-	if ( fnew != fold )
-		output_set_value( "floppy", fnew );
-
-	/* prepare state for next frame */
-	for ( i = 0; i <= THOM_TOTAL_HEIGHT; i++ )
+	// rising edge
+	if (vblank_on)
 	{
-		if ( thom_border_l[ i ] != -1 )
-			b = thom_border_l[ i ];
-		if ( thom_border_r[ i ] != -1 )
-			b = thom_border_r[ i ];
+		int fnew, fold = FLOP_STATE;
+		int i;
+		UINT16 b = 0;
+		struct thom_vsignal l = thom_get_lightpen_vsignal( screen.machine(), 0, -1, 0 );
+
+		LOG (( "%f thom: video eof called\n", screen.machine().time().as_double() ));
+
+		/* floppy indicator count */
+		if ( thom_floppy_wcount )
+			thom_floppy_wcount--;
+		if ( thom_floppy_rcount )
+			thom_floppy_rcount--;
+		fnew = FLOP_STATE;
+		if ( fnew != fold )
+			output_set_value( "floppy", fnew );
+
+		/* prepare state for next frame */
+		for ( i = 0; i <= THOM_TOTAL_HEIGHT; i++ )
+		{
+			if ( thom_border_l[ i ] != -1 )
+				b = thom_border_l[ i ];
+			if ( thom_border_r[ i ] != -1 )
+				b = thom_border_r[ i ];
+		}
+		memset( thom_border_l, 0xff, sizeof( thom_border_l ) );
+		memset( thom_border_r, 0xff, sizeof( thom_border_r ) );
+		thom_border_l[ 0 ] = b;
+		thom_border_r[ 0 ] = b;
+		thom_vstate_last_dirty = thom_vstate_dirty;
+		thom_vstate_dirty = 0;
+
+		/* schedule first init signal */
+		thom_init_timer->adjust(attotime::from_usec( 64 * THOM_BORDER_HEIGHT + 7 ));
+
+		/* schedule first lightpen signal */
+		l.line &= ~1; /* hack (avoid lock in MO6 palette selection) */
+		thom_lightpen_timer->adjust(
+				   attotime::from_usec( 64 * ( THOM_BORDER_HEIGHT + l.line - 2 ) + 16 ), 0);
+
+		/* schedule first active-area scanline call-back */
+		thom_scanline_timer->adjust(attotime::from_usec( 64 * THOM_BORDER_HEIGHT + 7), -1);
+
+		/* reset video frame time */
+		thom_video_timer->adjust(attotime::zero);
+
+		/* update screen size according to user options */
+		if ( thom_update_screen_size( screen.machine() ) )
+			thom_vstate_dirty = 1;
+
+		/* hi-res automatic */
+		thom_hires_better = thom_mode_is_hires( thom_vmode );
 	}
-	memset( thom_border_l, 0xff, sizeof( thom_border_l ) );
-	memset( thom_border_r, 0xff, sizeof( thom_border_r ) );
-	thom_border_l[ 0 ] = b;
-	thom_border_r[ 0 ] = b;
-	thom_vstate_last_dirty = thom_vstate_dirty;
-	thom_vstate_dirty = 0;
-
-	/* schedule first init signal */
-	thom_init_timer->adjust(attotime::from_usec( 64 * THOM_BORDER_HEIGHT + 7 ));
-
-	/* schedule first lightpen signal */
-	l.line &= ~1; /* hack (avoid lock in MO6 palette selection) */
-	thom_lightpen_timer->adjust(
-			   attotime::from_usec( 64 * ( THOM_BORDER_HEIGHT + l.line - 2 ) + 16 ), 0);
-
-	/* schedule first active-area scanline call-back */
-	thom_scanline_timer->adjust(attotime::from_usec( 64 * THOM_BORDER_HEIGHT + 7), -1);
-
-	/* reset video frame time */
-	thom_video_timer->adjust(attotime::zero);
-
-	/* update screen size according to user options */
-	if ( thom_update_screen_size( machine ) )
-		thom_vstate_dirty = 1;
-
-	/* hi-res automatic */
-	thom_hires_better = thom_mode_is_hires( thom_vmode );
 }
 
 
@@ -1145,7 +1149,7 @@ VIDEO_START ( thom )
 
 	thom_mode_point = 0;
 	state_save_register_global(machine,  thom_mode_point );
-	memory_set_bank( machine, THOM_VRAM_BANK, 0 );
+	machine.root_device().membank( THOM_VRAM_BANK )->set_entry( 0 );
 
 	thom_floppy_rcount = 0;
 	thom_floppy_wcount = 0;
