@@ -80,7 +80,7 @@ struct _vic2_state
 	vic2_type  type;
 
 	screen_device *screen;			// screen which sets bitmap properties
-	device_t *cpu;
+	cpu_device *cpu;
 
 	UINT8 rdy_cycles;
 	UINT8 reg[0x80];
@@ -89,7 +89,7 @@ struct _vic2_state
 
 	UINT16 chargenaddr, videoaddr, bitmapaddr;
 
-	bitmap_t *bitmap;
+	bitmap_ind16 *bitmap;
 
 	UINT16 colors[4], spritemulti[4];
 
@@ -149,19 +149,19 @@ struct _vic2_state
 	UINT8 device_suspended;
 
 	/* DMA */
-	vic2_dma_read          dma_read;
-	vic2_dma_read_color    dma_read_color;
+	devcb_resolved_read8          in_dma_read_func;
+	devcb_resolved_read8    in_dma_read_color_func;
 
 	/* IRQ */
-	vic2_irq               interrupt;
+	devcb_resolved_write_line               out_interrupt_func;
 
 	/* RDY */
-	vic2_rdy_callback      rdy_workaround_cb;
+	devcb_resolved_read8      in_rdy_workaround_func;
 
 	/* lightpen */
-	vic2_lightpen_button_callback lightpen_button_cb;
-	vic2_lightpen_x_callback lightpen_x_cb;
-	vic2_lightpen_y_callback lightpen_y_cb;
+	devcb_resolved_read8 in_lightpen_button_func;
+	devcb_resolved_read8 in_lightpen_x_func;
+	devcb_resolved_read8 in_lightpen_y_func;
 };
 
 
@@ -193,9 +193,9 @@ struct _vic2_state
 
 /* 2008-05 FP: lightpen code needs to read input port from c64.c and cbmb.c */
 
-#define LIGHTPEN_BUTTON		(vic2->lightpen_button_cb(machine))
-#define LIGHTPEN_X_VALUE	(vic2->lightpen_x_cb(machine))
-#define LIGHTPEN_Y_VALUE	(vic2->lightpen_y_cb(machine))
+#define LIGHTPEN_BUTTON		(vic2->in_lightpen_button_func(0))
+#define LIGHTPEN_X_VALUE	(vic2->in_lightpen_x_func(0))
+#define LIGHTPEN_Y_VALUE	(vic2->in_lightpen_y_func(0))
 
 /* lightpen delivers values from internal counters; they do not start with the visual area or frame area */
 #define VIC2_MAME_XPOS			0
@@ -291,7 +291,7 @@ static void vic2_set_interrupt( running_machine &machine, int mask, vic2_state *
 		{
 			DBG_LOG(2, "vic2", ("irq start %.2x\n", mask));
 			vic2->reg[0x19] |= 0x80;
-			vic2->interrupt(machine, 1);
+			vic2->out_interrupt_func(1);
 		}
 	}
 	vic2->reg[0x19] |= mask;
@@ -304,7 +304,7 @@ static void vic2_clear_interrupt( running_machine &machine, int mask, vic2_state
 	{
 		DBG_LOG(2, "vic2", ("irq end %.2x\n", mask));
 		vic2->reg[0x19] &= ~0x80;
-		vic2->interrupt(machine, 0);
+		vic2->out_interrupt_func(0);
 	}
 }
 
@@ -340,13 +340,13 @@ static TIMER_CALLBACK( vic2_timer_timeout )
 // Idle access
 INLINE void vic2_idle_access( running_machine &machine, vic2_state *vic2 )
 {
-	vic2->dma_read(machine, 0x3fff);
+	vic2->in_dma_read_func(0x3fff);
 }
 
 // Fetch sprite data pointer
 INLINE void vic2_spr_ptr_access( running_machine &machine, vic2_state *vic2, int num )
 {
-	vic2->spr_ptr[num] = vic2->dma_read(machine, SPRITE_ADDR(num)) << 6;
+	vic2->spr_ptr[num] = vic2->in_dma_read_func(SPRITE_ADDR(num)) << 6;
 }
 
 // Fetch sprite data, increment data counter
@@ -354,7 +354,7 @@ INLINE void vic2_spr_data_access( running_machine &machine, vic2_state *vic2, in
 {
 	if (vic2->spr_dma_on & (1 << num))
 	{
-		vic2->spr_data[num][bytenum] = vic2->dma_read(machine, (vic2->mc[num] & 0x3f) | vic2->spr_ptr[num]);
+		vic2->spr_data[num][bytenum] = vic2->in_dma_read_func((vic2->mc[num] & 0x3f) | vic2->spr_ptr[num]);
 		vic2->mc[num]++;
 	}
 	else
@@ -375,7 +375,7 @@ INLINE void vic2_suspend_cpu( running_machine &machine, vic2_state *vic2 )
 	if (vic2->device_suspended == 0)
 	{
 		vic2->first_ba_cycle = vic2->cycles_counter;
-		if ((vic2->rdy_workaround_cb != NULL) && (vic2->rdy_workaround_cb(machine) != 7 ))
+		if (vic2->in_rdy_workaround_func(0) != 7 )
 		{
 //          device_suspend(machine.firstcpu, SUSPEND_REASON_SPIN, 0);
 		}
@@ -388,10 +388,7 @@ INLINE void vic2_resume_cpu( running_machine &machine, vic2_state *vic2 )
 {
 	if (vic2->device_suspended == 1)
 	{
-		if ((vic2->rdy_workaround_cb != NULL))
-		{
-//  device_resume(machine.firstcpu, SUSPEND_REASON_SPIN);
-		}
+	//  device_resume(machine.firstcpu, SUSPEND_REASON_SPIN);
 		vic2->device_suspended = 0;
 	}
 }
@@ -399,7 +396,7 @@ INLINE void vic2_resume_cpu( running_machine &machine, vic2_state *vic2 )
 // Refresh access
 INLINE void vic2_refresh_access( running_machine &machine, vic2_state *vic2 )
 {
-	vic2->dma_read(machine, 0x3f00 | vic2->ref_cnt--);
+	vic2->in_dma_read_func(0x3f00 | vic2->ref_cnt--);
 }
 
 
@@ -453,13 +450,13 @@ INLINE void vic2_matrix_access( running_machine &machine, vic2_state *vic2 )
 {
 //  if (vic2->device_suspended == 1)
 	{
-		if ((vic2->cycles_counter - vic2->first_ba_cycle) < 0)
+		if (vic2->cycles_counter < vic2->first_ba_cycle)
 			vic2->matrix_line[vic2->ml_index] = vic2->color_line[vic2->ml_index] = 0xff;
 		else
 		{
 			UINT16 adr = (vic2->vc & 0x03ff) | VIDEOADDR;
-			vic2->matrix_line[vic2->ml_index] = vic2->dma_read(machine, adr); \
-			vic2->color_line[vic2->ml_index] = vic2->dma_read_color(machine, (adr & 0x03ff)); \
+			vic2->matrix_line[vic2->ml_index] = vic2->in_dma_read_func(adr); \
+			vic2->color_line[vic2->ml_index] = vic2->in_dma_read_color_func((adr & 0x03ff)); \
 		}
 	}
 }
@@ -476,7 +473,7 @@ INLINE void vic2_graphics_access( running_machine &machine, vic2_state *vic2 )
 			adr = (vic2->matrix_line[vic2->ml_index] << 3) | vic2->chargenaddr | vic2->rc;
 		if (ECMON)
 			adr &= 0xf9ff;
-		vic2->gfx_data = vic2->dma_read(machine, adr);
+		vic2->gfx_data = vic2->in_dma_read_func(adr);
 		vic2->char_data = vic2->matrix_line[vic2->ml_index];
 		vic2->color_data = vic2->color_line[vic2->ml_index];
 		vic2->ml_index++;
@@ -484,7 +481,7 @@ INLINE void vic2_graphics_access( running_machine &machine, vic2_state *vic2 )
 	}
 	else
 	{
-		vic2->gfx_data = vic2->dma_read(machine, (ECMON ? 0x39ff : 0x3fff));
+		vic2->gfx_data = vic2->in_dma_read_func((ECMON ? 0x39ff : 0x3fff));
 		vic2->char_data = 0;
 	}
 }
@@ -521,7 +518,7 @@ INLINE void vic2_draw_background( vic2_state *vic2 )
 				c = 0;
 				break;
 		}
-		plot_box(vic2->bitmap, vic2->graphic_x, VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, c);
+		vic2->bitmap->plot_box(vic2->graphic_x, VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, c);
 	}
 }
 
@@ -533,21 +530,21 @@ INLINE void vic2_draw_mono( vic2_state *vic2, UINT16 p, UINT8 c0, UINT8 c1 )
 	c[0] = c0;
 	c[1] = c1;
 
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = c[data & 1];
 	vic2->fore_coll_buf[p + 7] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = c[data & 1];
 	vic2->fore_coll_buf[p + 6] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = c[data & 1];
 	vic2->fore_coll_buf[p + 5] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = c[data & 1];
 	vic2->fore_coll_buf[p + 4] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = c[data & 1];
 	vic2->fore_coll_buf[p + 3] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = c[data & 1];
 	vic2->fore_coll_buf[p + 2] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = c[data & 1];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = c[data & 1];
 	vic2->fore_coll_buf[p + 1] = data & 1; data >>= 1;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = c[data];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = c[data];
 	vic2->fore_coll_buf[p + 0] = data & 1;
 }
 
@@ -561,21 +558,21 @@ INLINE void vic2_draw_multi( vic2_state *vic2, UINT16 p, UINT8 c0, UINT8 c1, UIN
 	c[2] = c2;
 	c[3] = c3;
 
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = c[data & 3];
 	vic2->fore_coll_buf[p + 7] = data & 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = c[data & 3];
 	vic2->fore_coll_buf[p + 6] = data & 2; data >>= 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = c[data & 3];
 	vic2->fore_coll_buf[p + 5] = data & 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = c[data & 3];
 	vic2->fore_coll_buf[p + 4] = data & 2; data >>= 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = c[data & 3];
 	vic2->fore_coll_buf[p + 3] = data & 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = c[data & 3];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = c[data & 3];
 	vic2->fore_coll_buf[p + 2] = data & 2; data >>= 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = c[data];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = c[data];
 	vic2->fore_coll_buf[p + 1] = data & 2;
-	*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = c[data];
+	vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = c[data];
 	vic2->fore_coll_buf[p + 0] = data & 2;
 }
 
@@ -642,57 +639,57 @@ static void vic2_draw_graphics( vic2_state *vic2 )
 				vic2_draw_mono(vic2, p, tmp_col, vic2->color_data & 0x0f);
 				break;
 			case 5:
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
 				vic2->fore_coll_buf[p + 7] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
 				vic2->fore_coll_buf[p + 6] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
 				vic2->fore_coll_buf[p + 5] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
 				vic2->fore_coll_buf[p + 4] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
 				vic2->fore_coll_buf[p + 3] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
 				vic2->fore_coll_buf[p + 2] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
 				vic2->fore_coll_buf[p + 1] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
 				vic2->fore_coll_buf[p + 0] = 0;
 				break;
 			case 6:
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
 				vic2->fore_coll_buf[p + 7] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
 				vic2->fore_coll_buf[p + 6] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
 				vic2->fore_coll_buf[p + 5] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
 				vic2->fore_coll_buf[p + 4] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
 				vic2->fore_coll_buf[p + 3] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
 				vic2->fore_coll_buf[p + 2] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
 				vic2->fore_coll_buf[p + 1] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
 				vic2->fore_coll_buf[p + 0] = 0;
 				break;
 			case 7:
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 7) = 0;
 				vic2->fore_coll_buf[p + 7] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 6) = 0;
 				vic2->fore_coll_buf[p + 6] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 5) = 0;
 				vic2->fore_coll_buf[p + 5] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 4) = 0;
 				vic2->fore_coll_buf[p + 4] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 3) = 0;
 				vic2->fore_coll_buf[p + 3] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 2) = 0;
 				vic2->fore_coll_buf[p + 2] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 1) = 0;
 				vic2->fore_coll_buf[p + 1] = 0;
-				*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
+				vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + 0) = 0;
 				vic2->fore_coll_buf[p + 0] = 0;
 				break;
 		}
@@ -767,12 +764,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 							if (SPRITE_PRIORITY(snum))
 							{
 								if (vic2->fore_coll_buf[p + i] == 0)
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 							else
 							{
-								*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+								vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 						}
@@ -815,12 +812,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 							if (SPRITE_PRIORITY(snum))
 							{
 								if (vic2->fore_coll_buf[p + i] == 0)
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 							else
 							{
-								*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+								vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 						}
@@ -846,12 +843,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 								if (SPRITE_PRIORITY(snum))
 								{
 									if (vic2->fore_coll_buf[p + i] == 0)
-										*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+										vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 								else
 								{
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 							}
@@ -872,12 +869,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 								if (SPRITE_PRIORITY(snum))
 								{
 									if (vic2->fore_coll_buf[p + i] == 0)
-										*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+										vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 								else
 								{
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 							}
@@ -927,12 +924,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 							if (SPRITE_PRIORITY(snum))
 							{
 								if (vic2->fore_coll_buf[p + i] == 0)
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 							else
 							{
-								*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
+								vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = col;
 								vic2->spr_coll_buf[p + i] = sbit;
 							}
 						}
@@ -957,12 +954,12 @@ static void vic2_draw_sprites( running_machine &machine, vic2_state *vic2 )
 								if (SPRITE_PRIORITY(snum))
 								{
 									if (vic2->fore_coll_buf[p + i] == 0)
-										*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+										vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 								else
 								{
-									*BITMAP_ADDR16(vic2->bitmap, VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
+									vic2->bitmap->pix16(VIC2_RASTER_2_EMU(vic2->rasterline), p + i) = color;
 									vic2->spr_coll_buf[p + i] = sbit;
 								}
 							}
@@ -1000,7 +997,7 @@ static TIMER_CALLBACK( pal_timer_callback )
 	UINT8 mask;
 	//static int adjust[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
 
-	UINT8 cpu_cycles = machine.device<cpu_device>("maincpu")->total_cycles() & 0xff;
+	UINT8 cpu_cycles = vic2->cpu->total_cycles() & 0xff;
 	UINT8 vic_cycles = (vic2->cycles_counter + 1) & 0xff;
 	vic2->cycles_counter++;
 
@@ -1235,7 +1232,7 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 
 		vic2->raster_x = 0xfffc;
 
-		if ((vic2->rdy_workaround_cb(machine) == 0 ) && (vic2->is_bad_line))
+		if ((vic2->in_rdy_workaround_func(0) == 0 ) && (vic2->is_bad_line))
 			vic2->rdy_cycles += (43+adjust(0));
 
 		vic2->cycle++;
@@ -1250,7 +1247,7 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 
 		vic2->vc = vic2->vc_base;
 
-		if ((vic2->rdy_workaround_cb(machine) == 1 ) && (vic2->is_bad_line))
+		if ((vic2->in_rdy_workaround_func(0) == 1 ) && (vic2->is_bad_line))
 			vic2->rdy_cycles += (42+adjust(0));
 
 		vic2->cycle++;
@@ -1270,7 +1267,7 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 		vic2->ml_index = 0;
 		vic2_matrix_access(machine, vic2);
 
-		if ((vic2->rdy_workaround_cb(machine) == 2 ) && (vic2->is_bad_line))
+		if ((vic2->in_rdy_workaround_func(0) == 2 ) && (vic2->is_bad_line))
 			vic2->rdy_cycles += (41+adjust(0));
 
 		vic2->cycle++;
@@ -1294,7 +1291,7 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 
 		vic2_matrix_access(machine, vic2);
 
-		if ((vic2->rdy_workaround_cb(machine) == 3 ) && (vic2->is_bad_line))
+		if ((vic2->in_rdy_workaround_func(0) == 3 ) && (vic2->is_bad_line))
 			vic2->rdy_cycles += (40+adjust(0));
 
 		vic2->cycle++;
@@ -1331,7 +1328,7 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 		vic2_fetch_if_bad_line(vic2);
 		vic2_matrix_access(machine, vic2);
 
-		if ((vic2->rdy_workaround_cb(machine) == 4 ) && (vic2->is_bad_line))
+		if ((vic2->in_rdy_workaround_func(0) == 4 ) && (vic2->is_bad_line))
 			vic2->rdy_cycles += (40+adjust(0));
 
 		vic2->cycle++;
@@ -1532,24 +1529,24 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 
 			if (vic2->border_on_sample[0])
 				for (i = 0; i < 4; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 
 			if (vic2->border_on_sample[1])
-				plot_box(vic2->bitmap, VIC2_X_2_EMU(4 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[4]);
+				vic2->bitmap->plot_box(VIC2_X_2_EMU(4 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[4]);
 
 			if (vic2->border_on_sample[2])
 				for (i = 5; i < 43; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 
 			if (vic2->border_on_sample[3])
-				plot_box(vic2->bitmap, VIC2_X_2_EMU(43 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[43]);
+				vic2->bitmap->plot_box(VIC2_X_2_EMU(43 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[43]);
 
 			if (vic2->border_on_sample[4])
 			{
 				for (i = 44; i < 48; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 				for (i = 48; i < 51; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[47]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[47]);
 			}
 		}
 
@@ -1610,12 +1607,12 @@ if (machine.input().code_pressed_once(KEYCODE_Z)) printf("b:%02x 1:%02x 2:%02x 3
 
 	if ((cpu_cycles == vic_cycles) && (vic2->rdy_cycles > 0))
 	{
-		device_spin_until_time (machine.firstcpu, machine.device<cpu_device>("maincpu")->cycles_to_attotime(vic2->rdy_cycles));
+		device_spin_until_time (machine.firstcpu, vic2->cpu->cycles_to_attotime(vic2->rdy_cycles));
 		vic2->rdy_cycles = 0;
 	}
 
 	vic2->raster_x += 8;
-	machine.scheduler().timer_set(machine.device<cpu_device>("maincpu")->cycles_to_attotime(1), FUNC(pal_timer_callback), 0, vic2);
+	machine.scheduler().timer_set(vic2->cpu->cycles_to_attotime(1), FUNC(pal_timer_callback), 0, vic2);
 }
 
 static TIMER_CALLBACK( ntsc_timer_callback )
@@ -2121,24 +2118,24 @@ static TIMER_CALLBACK( ntsc_timer_callback )
 
 			if (vic2->border_on_sample[0])
 				for (i = 0; i < 4; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 
 			if (vic2->border_on_sample[1])
-				plot_box(vic2->bitmap, VIC2_X_2_EMU(4 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[4]);
+				vic2->bitmap->plot_box(VIC2_X_2_EMU(4 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[4]);
 
 			if (vic2->border_on_sample[2])
 				for (i = 5; i < 43; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 
 			if (vic2->border_on_sample[3])
-				plot_box(vic2->bitmap, VIC2_X_2_EMU(43 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[43]);
+				vic2->bitmap->plot_box(VIC2_X_2_EMU(43 * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[43]);
 
 			if (vic2->border_on_sample[4])
 			{
 				for (i = 44; i < 48; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[i]);
 				for (i = 48; i < 53; i++)
-					plot_box(vic2->bitmap, VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[47]);
+					vic2->bitmap->plot_box(VIC2_X_2_EMU(i * 8), VIC2_RASTER_2_EMU(vic2->rasterline), 8, 1, vic2->border_color_sample[47]);
 			}
 		}
 
@@ -2194,7 +2191,7 @@ static TIMER_CALLBACK( ntsc_timer_callback )
 	}
 
 	vic2->raster_x += 8;
-	machine.scheduler().timer_set(machine.device<cpu_device>("maincpu")->cycles_to_attotime(1), FUNC(ntsc_timer_callback), 0, vic2);
+	machine.scheduler().timer_set(vic2->cpu->cycles_to_attotime(1), FUNC(ntsc_timer_callback), 0, vic2);
 }
 
 
@@ -2581,12 +2578,12 @@ READ8_DEVICE_HANDLER( vic2_port_r )
 	return val;
 }
 
-UINT32 vic2_video_update( device_t *device, bitmap_t *bitmap, const rectangle *cliprect )
+UINT32 vic2_video_update( device_t *device, bitmap_ind16 &bitmap, const rectangle &cliprect )
 {
 	vic2_state *vic2 = get_safe_token(device);
 
 	if (vic2->on)
-		copybitmap(bitmap, vic2->bitmap, 0, 0, 0, 0, cliprect);
+		copybitmap(bitmap, *vic2->bitmap, 0, 0, 0, 0, cliprect);
 	return 0;
 }
 
@@ -2601,26 +2598,25 @@ static DEVICE_START( vic2 )
 	int width, height;
 	int i;
 
-	vic2->cpu = device->machine().device(intf->cpu);
+	// resolve callbacks
+	vic2->in_lightpen_x_func.resolve(intf->in_x_cb, *device);
+	vic2->in_lightpen_y_func.resolve(intf->in_y_cb, *device);
+	vic2->in_lightpen_button_func.resolve(intf->in_button_cb, *device);
+	vic2->in_dma_read_func.resolve(intf->in_dma_read_cb, *device);
+	vic2->in_dma_read_color_func.resolve(intf->in_dma_read_color_cb, *device);
+	vic2->out_interrupt_func.resolve(intf->out_irq_cb, *device);
+	vic2->in_rdy_workaround_func.resolve(intf->in_rdy_cb, *device);
+
+	vic2->cpu = device->machine().device<cpu_device>(intf->cpu);
 
 	vic2->screen = device->machine().device<screen_device>(intf->screen);
 	width = vic2->screen->width();
 	height = vic2->screen->height();
 
-	vic2->bitmap = auto_bitmap_alloc(device->machine(), width, height, BITMAP_FORMAT_INDEXED16);
+	vic2->bitmap = auto_bitmap_ind16_alloc(device->machine(), width, height);
 
 	vic2->type = intf->type;
 
-	vic2->dma_read = intf->dma_read;
-	vic2->dma_read_color = intf->dma_read_color;
-	vic2->interrupt = intf->irq;
-
-	vic2->rdy_workaround_cb = intf->rdy_cb;
-
-	vic2->lightpen_button_cb = intf->button_cb;
-	vic2->lightpen_x_cb = intf->x_cb;
-	vic2->lightpen_y_cb = intf->y_cb;
-
 	// immediately call the timer to handle the first line
 	if (vic2->type == VIC6569 || vic2->type == VIC8566)
 		device->machine().scheduler().timer_set(downcast<cpu_device *>(vic2->cpu)->cycles_to_attotime(0), FUNC(pal_timer_callback), 0, vic2);
@@ -2673,7 +2669,7 @@ static DEVICE_START( vic2 )
 
 	device->save_item(NAME(vic2->on));
 
-	device->save_item(NAME(*vic2->bitmap));
+	//device->save_item(NAME(vic2->bitmap));
 
 	device->save_item(NAME(vic2->chargenaddr));
 	device->save_item(NAME(vic2->videoaddr));
