@@ -2,20 +2,21 @@
 //
 //  sdlmain.c - main file for SDLMAME.
 //
-//  Copyright (c) 1996-2011, Nicola Salmoria and the MAME Team.
+//  Copyright (c) 1996-2012, Nicola Salmoria and the MAME Team.
 //  Visit http://mamedev.org for licensing and usage restrictions.
 //
 //  SDLMAME by Olivier Galibert and R. Belmont
 //
 //============================================================
 
-// standard sdl header
-#include <SDL/SDL.h>
-#include <SDL/SDL_version.h>
 
 #ifdef SDLMAME_UNIX
 #ifndef SDLMAME_MACOSX
+#if (SDLMAME_SDL2)
+#include <SDL2/SDL_ttf.h>
+#else
 #include <SDL/SDL_ttf.h>
+#endif
 #include <fontconfig/fontconfig.h>
 #endif
 #ifdef SDLMAME_MACOSX
@@ -24,14 +25,17 @@
 #endif
 
 // standard includes
-#ifdef MESS
+#if !defined(SDLMAME_WIN32) && !defined(SDLMAME_OS2)
 #include <unistd.h>
 #endif
+
 #ifdef SDLMAME_OS2
 #define INCL_DOS
 #include <os2.h>
 #endif
 
+#include "sdlinc.h"
+
 // MAME headers
 #include "osdepend.h"
 #include "emu.h"
@@ -64,11 +68,7 @@
 #if defined(SDLMAME_WIN32) || defined(SDLMAME_MACOSX) || defined(SDLMAME_OS2)
 	#define INI_PATH ".;ini"
 #else
-#ifdef MESS
-	#define INI_PATH "$HOME/.mess;.;ini"
-#else
-	#define INI_PATH "$HOME/.mame;.;ini"
-#endif // MESS
+	#define INI_PATH "$HOME/.APP_NAME;.;ini"
 #endif // MACOSX
 #endif // INI_PATH
 
@@ -112,10 +112,10 @@ const options_entry sdl_options::s_option_entries[] =
 	{ SDLOPTION_CENTERH,                      "1",        OPTION_BOOLEAN,    "center horizontally within the view area" },
 	{ SDLOPTION_CENTERV,                      "1",        OPTION_BOOLEAN,    "center vertically within the view area" },
 	#if (SDL_VERSION_ATLEAST(1,2,10))
-	{ SDLOPTION_WAITVSYNC,                    "0",        OPTION_BOOLEAN,    "enable waiting for the start of VBLANK before flipping screens; reduces tearing effects" },
-	{ SDLOPTION_SYNCREFRESH,                  "0",        OPTION_BOOLEAN,    "enable using the start of VBLANK for throttling instead of the game time" },
+	{ SDLOPTION_WAITVSYNC ";vs",              "0",        OPTION_BOOLEAN,    "enable waiting for the start of VBLANK before flipping screens; reduces tearing effects" },
+	{ SDLOPTION_SYNCREFRESH ";sr",            "0",        OPTION_BOOLEAN,    "enable using the start of VBLANK for throttling instead of the game time" },
 	#endif
-#if (SDL_VERSION_ATLEAST(1,3,0))
+#if (SDLMAME_SDL2)
 	{ SDLOPTION_SCALEMODE ";sm",         SDLOPTVAL_NONE,  OPTION_STRING,     "Scale mode: none, hwblit, hwbest, yv12, yuy2, yv12x2, yuy2x2 (-video soft only)" },
 #else
 	{ SDLOPTION_SCALEMODE ";sm",         SDLOPTVAL_NONE,  OPTION_STRING,     "Scale mode: none, async, yv12, yuy2, yv12x2, yuy2x2 (-video soft only)" },
@@ -196,13 +196,11 @@ const options_entry sdl_options::s_option_entries[] =
 	{ NULL, 		                          NULL,  OPTION_HEADER,     "SDL KEYBOARD MAPPING" },
 	{ SDLOPTION_KEYMAP,                      "0",    OPTION_BOOLEAN,    "enable keymap" },
 	{ SDLOPTION_KEYMAP_FILE,                 "keymap.dat", OPTION_STRING, "keymap filename" },
-#ifdef MESS
 #ifdef SDLMAME_MACOSX
-	{ SDLOPTION_UIMODEKEY,					 "DELETE", OPTION_STRING,   "Key to toggle MESS keyboard mode" },
+	{ SDLOPTION_UIMODEKEY,					 "DELETE", OPTION_STRING,   "Key to toggle keyboard mode" },
 #else
-	{ SDLOPTION_UIMODEKEY,			         "SCRLOCK", OPTION_STRING,  "Key to toggle MESS keyboard mode" },
+	{ SDLOPTION_UIMODEKEY,			         "SCRLOCK", OPTION_STRING,  "Key to toggle keyboard mode" },
 #endif	// SDLMAME_MACOSX
-#endif	// MESS
 
 	// joystick mapping
 	{ NULL, 		                         NULL,   OPTION_HEADER,     "SDL JOYSTICK MAPPING" },
@@ -216,7 +214,7 @@ const options_entry sdl_options::s_option_entries[] =
 	{ SDLOPTION_JOYINDEX "8",                SDLOPTVAL_AUTO, OPTION_STRING,         "name of joystick mapped to joystick #8" },
 	{ SDLOPTION_SIXAXIS,			         "0",	 OPTION_BOOLEAN,    "Use special handling for PS3 Sixaxis controllers" },
 
-#if (SDL_VERSION_ATLEAST(1,3,0))
+#if (SDLMAME_SDL2)
 	{ NULL, 		                         NULL,   OPTION_HEADER,     "SDL MOUSE MAPPING" },
 	{ SDLOPTION_MOUSEINDEX "1",              SDLOPTVAL_AUTO, OPTION_STRING,         "name of mouse mapped to mouse #1" },
 	{ SDLOPTION_MOUSEINDEX "2",              SDLOPTVAL_AUTO, OPTION_STRING,         "name of mouse mapped to mouse #2" },
@@ -240,7 +238,7 @@ const options_entry sdl_options::s_option_entries[] =
 	// SDL low level driver options
 	{ NULL, 		                         NULL,   OPTION_HEADER,     "SDL LOWLEVEL DRIVER OPTIONS" },
 	{ SDLOPTION_VIDEODRIVER ";vd",           SDLOPTVAL_AUTO,  OPTION_STRING,        "sdl video driver to use ('x11', 'directfb', ... or 'auto' for SDL default" },
-#if (SDL_VERSION_ATLEAST(1,3,0))
+#if (SDLMAME_SDL2)
 	{ SDLOPTION_RENDERDRIVER ";rd",          SDLOPTVAL_AUTO,  OPTION_STRING,        "sdl render driver to use ('software', 'opengl', 'directfb' ... or 'auto' for SDL default" },
 #endif
 	{ SDLOPTION_AUDIODRIVER ";ad",           SDLOPTVAL_AUTO,  OPTION_STRING,        "sdl audio driver to use ('alsa', 'arts', ... or 'auto' for SDL default" },
@@ -275,7 +273,10 @@ void MorphToPM()
 
 sdl_options::sdl_options()
 {
+	astring ini_path(INI_PATH);
 	add_entries(s_option_entries);
+	ini_path.replace(0, "APP_NAME", emulator_info::get_appname_lower());
+	set_default_value(SDLOPTION_INIPATH, ini_path.cstr());
 }
 
 
@@ -300,7 +301,7 @@ int main(int argc, char *argv[])
 {
 	int res = 0;
 
-#if	!(SDL_VERSION_ATLEAST(1,3,0))
+#if	!(SDLMAME_SDL2)
 	/* Load SDL dynamic link library */
 	if ( SDL_Init(SDL_INIT_NOPARACHUTE) < 0 ) {
 		fprintf(stderr, "WinMain() error: %s", SDL_GetError());
@@ -413,9 +414,19 @@ sdl_osd_interface::~sdl_osd_interface()
 
 void sdl_osd_interface::osd_exit(running_machine &machine)
 {
+	#ifdef SDLMAME_NETWORK
+		sdlnetdev_deinit(machine);
+	#endif
 
 	if (!SDLMAME_INIT_IN_WORKER_THREAD)
+	{
+		/* FixMe: Bug in SDL2.0, Quitting joystick will cause SIGSEGV */
+#if SDLMAME_SDL2
+		SDL_QuitSubSystem(SDL_INIT_TIMER|SDL_INIT_AUDIO| SDL_INIT_VIDEO /*| SDL_INIT_JOYSTICK */);
+#else
 		SDL_Quit();
+#endif
+	}
 }
 
 //============================================================
@@ -491,7 +502,7 @@ static void defines_verbose(void)
 
 static void osd_sdl_info(void)
 {
-#if SDL_VERSION_ATLEAST(1,3,0)
+#if SDLMAME_SDL2
 	int i, num = SDL_GetNumVideoDrivers();
 
 	mame_printf_verbose("Available videodrivers: ");
@@ -571,15 +582,15 @@ void sdl_osd_interface::init(running_machine &machine)
 		osd_setenv(SDLENV_VIDEODRIVER, stemp, 1);
 	}
 
-	if (SDL_VERSION_ATLEAST(1,3,0))
-	{
+#if (SDLMAME_SDL2)
 		stemp = options.render_driver();
 		if (stemp != NULL && strcmp(stemp, SDLOPTVAL_AUTO) != 0)
 		{
 			mame_printf_verbose("Setting SDL renderdriver '%s' ...\n", stemp);
-			osd_setenv(SDLENV_RENDERDRIVER, stemp, 1);
+			//osd_setenv(SDLENV_RENDERDRIVER, stemp, 1);
+			SDL_SetHint(SDL_HINT_RENDER_DRIVER, stemp);
 		}
-	}
+#endif
 
 	/* Set the SDL environment variable for drivers wanting to load the
      * lib at startup.
@@ -598,15 +609,15 @@ void sdl_osd_interface::init(running_machine &machine)
 	/* get number of processors */
 	stemp = options.numprocessors();
 
-	sdl_num_processors = 0;
+    osd_num_processors = 0;
 
 	if (strcmp(stemp, "auto") != 0)
 	{
-		sdl_num_processors = atoi(stemp);
-		if (sdl_num_processors < 1)
+		osd_num_processors = atoi(stemp);
+		if (osd_num_processors < 1)
 		{
 			mame_printf_warning("Warning: numprocessors < 1 doesn't make much sense. Assuming auto ...\n");
-			sdl_num_processors = 0;
+			osd_num_processors = 0;
 		}
 	}
 
@@ -614,7 +625,7 @@ void sdl_osd_interface::init(running_machine &machine)
 
 	if (!SDLMAME_INIT_IN_WORKER_THREAD)
 	{
-#if (SDL_VERSION_ATLEAST(1,3,0))
+#if (SDLMAME_SDL2)
 		if (SDL_InitSubSystem(SDL_INIT_TIMER|SDL_INIT_AUDIO| SDL_INIT_VIDEO| SDL_INIT_JOYSTICK|SDL_INIT_NOPARACHUTE)) {
 #else
 		if (SDL_Init(SDL_INIT_TIMER|SDL_INIT_AUDIO| SDL_INIT_VIDEO| SDL_INIT_JOYSTICK|SDL_INIT_NOPARACHUTE)) {
@@ -669,7 +680,7 @@ void sdl_osd_interface::init(running_machine &machine)
 		m_watchdog->setTimeout(watchdog_timeout);
 	}
 
-#if (SDL_VERSION_ATLEAST(1,3,0))
+#if (SDLMAME_SDL2)
 	SDL_EventState(SDL_TEXTINPUT, SDL_TRUE);
 #else
 	SDL_EnableUNICODE(SDL_TRUE);
@@ -705,7 +716,7 @@ osd_font sdl_osd_interface::font_open(const char *_name, int &height)
 
 	/* handle bdf fonts in the core */
 	if (name.len() > 4)
-		if (name.toupper().substr(name.len()-4,4) == ".BDF" )
+		if (name.makeupper().substr(name.len()-4,4) == ".BDF" )
 			return NULL;
 
 	font_name = CFStringCreateWithCString( NULL, _name, kCFStringEncodingUTF8 );
@@ -768,11 +779,10 @@ void sdl_osd_interface::font_close(osd_font font)
 //  pixel of a black & white font
 //-------------------------------------------------
 
-bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, INT32 &width, INT32 &xoffs, INT32 &yoffs)
+bool sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, bitmap_argb32 &bitmap, INT32 &width, INT32 &xoffs, INT32 &yoffs)
 {
    UniChar uni_char;
    CGGlyph glyph;
-   bitmap_t *bitmap = (bitmap_t *)NULL;
    CTFontRef ct_font = (CTFontRef)font;
    const CFIndex count = 1;
    CGRect bounding_rect, success_rect;
@@ -813,9 +823,9 @@ bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum,
       color_space = CGColorSpaceCreateDeviceRGB();
       bits_per_component = 8;
 
-      bitmap = auto_alloc(machine(), bitmap_t(bitmap_width, bitmap_height, BITMAP_FORMAT_ARGB32));
+      bitmap.allocate(bitmap_width, bitmap_height);
 
-      context_ref = CGBitmapContextCreate( bitmap->base, bitmap_width, bitmap_height, bits_per_component, bitmap->rowpixels*4, color_space, bitmap_info );
+      context_ref = CGBitmapContextCreate( bitmap.raw_pixptr(0), bitmap_width, bitmap_height, bits_per_component, bitmap.rowpixels()*4, color_space, bitmap_info );
 
       if( context_ref != NULL )
       {
@@ -833,7 +843,7 @@ bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum,
       CGColorSpaceRelease( color_space );
    }
 
-   return bitmap;
+   return bitmap.valid();
 }
 #else // UNIX but not OSX
 
@@ -851,6 +861,22 @@ static TTF_Font * TTF_OpenFont_Magic(astring name, int fsize)
 	return TTF_OpenFont(name.cstr(), POINT_SIZE);
 }
 
+static bool BDF_Check_Magic(astring name)
+{
+    emu_file file(OPEN_FLAG_READ);
+    if (file.open(name) == FILERR_NONE)
+    {
+		unsigned char buffer[9];
+		unsigned char magic[9] = { 'S', 'T', 'A', 'R', 'T', 'F', 'O', 'N', 'T' };
+		file.read(buffer, 9);
+        file.close();
+		if (!memcmp(buffer, magic, 9))
+			return true;
+    }
+
+    return false;
+}
+
 static TTF_Font *search_font_config(astring name, bool bold, bool italic, bool underline, bool &bakedstyles)
 {
 	TTF_Font *font = (TTF_Font *)NULL;
@@ -1009,7 +1035,10 @@ osd_font sdl_osd_interface::font_open(const char *_name, int &height)
 
 	if (!font)
 	{
-		printf("WARNING: Couldn't find/open TrueType font %s, using MAME default\n", name.cstr());
+        if (!BDF_Check_Magic(name))
+        {
+            printf("WARNING: font %s, is not TrueType or BDF, using MAME default\n", name.cstr());
+        }
 		return NULL;
 	}
 
@@ -1056,10 +1085,9 @@ void sdl_osd_interface::font_close(osd_font font)
 //  pixel of a black & white font
 //-------------------------------------------------
 
-bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, INT32 &width, INT32 &xoffs, INT32 &yoffs)
+bool sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, bitmap_argb32 &bitmap, INT32 &width, INT32 &xoffs, INT32 &yoffs)
 {
 	TTF_Font *ttffont;
-	bitmap_t *bitmap = (bitmap_t *)NULL;
 	SDL_Surface *drawsurf;
 	SDL_Color fcol = { 0xff, 0xff, 0xff };
 	UINT16 ustr[16];
@@ -1074,12 +1102,12 @@ bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum,
 	if (drawsurf)
 	{
 		// allocate a MAME destination bitmap
-		bitmap = auto_alloc(machine(), bitmap_t(drawsurf->w, drawsurf->h, BITMAP_FORMAT_ARGB32));
+		bitmap.allocate(drawsurf->w, drawsurf->h);
 
 		// copy the rendered character image into it
-		for (int y = 0; y < bitmap->height; y++)
+		for (int y = 0; y < bitmap.height(); y++)
 		{
-			UINT32 *dstrow = BITMAP_ADDR32(bitmap, y, 0);
+			UINT32 *dstrow = &bitmap.pix32(y);
 			UINT8 *srcrow = (UINT8 *)drawsurf->pixels;
 
 			srcrow += (y * drawsurf->pitch);
@@ -1097,7 +1125,7 @@ bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum,
 		SDL_FreeSurface(drawsurf);
 	}
 
-	return bitmap;
+	return bitmap.valid();
 }
 #endif	// not OSX
 #else	// not UNIX
@@ -1128,8 +1156,8 @@ void sdl_osd_interface::font_close(osd_font font)
 //  pixel of a black & white font
 //-------------------------------------------------
 
-bitmap_t *sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, INT32 &width, INT32 &xoffs, INT32 &yoffs)
+bool sdl_osd_interface::font_get_bitmap(osd_font font, unicode_char chnum, bitmap_argb32 &bitmap, INT32 &width, INT32 &xoffs, INT32 &yoffs)
 {
-	return (bitmap_t *)NULL;
+	return false;
 }
 #endif
